# generated format python
#file name  ../data/classified/stopword0.7.csv
STOPWORDS = {}
STOPWORDS[u'بيد'] = {
    'word': u'بَيْدَ',
    'procletic': u'',
    'tags': u'أداة:استثناء',
    'vocalized': u'بَيْدَ',
    'stem': u'بَيْدَ',
    'type': u'STOPWORD',
    'original': u'بَيْدَ',
    'encletic': u''
}
STOPWORDS[u'وبيد'] = {
    'word': u'وبيد',
    'procletic': u'-وَ',
    'tags': u'أداة:استثناء:معطوف',
    'vocalized': u'وَبَيْدَ',
    'stem': u'بَيْدَ',
    'type': u'STOPWORD',
    'original': u'بَيْدَ',
    'encletic': u''
}
STOPWORDS[u'فبيد'] = {
    'word': u'فبيد',
    'procletic': u'-فَ',
    'tags': u'أداة:استثناء:معطوف',
    'vocalized': u'فَبَيْدَ',
    'stem': u'بَيْدَ',
    'type': u'STOPWORD',
    'original': u'بَيْدَ',
    'encletic': u''
}
STOPWORDS[u'سوى'] = {
    'word': u'سِوَى',
    'procletic': u'',
    'tags': u'أداة:استثناء',
    'vocalized': u'سِوَى',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u''
}
STOPWORDS[u'وسوى'] = {
    'word': u'وسوى',
    'procletic': u'-وَ',
    'tags': u'أداة:استثناء:معطوف',
    'vocalized': u'وَسِوَى',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u''
}
STOPWORDS[u'فسوى'] = {
    'word': u'فسوى',
    'procletic': u'-فَ',
    'tags': u'أداة:استثناء:معطوف',
    'vocalized': u'فَسِوَى',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u''
}
STOPWORDS[u'غير'] = {
    'word': u'غَيْرَ',
    'procletic': u'',
    'tags': u'أداة:استثناء',
    'vocalized': u'غَيْرَ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u''
}
STOPWORDS[u'بغير'] = {
    'word': u'بغير',
    'procletic': u'-بِ',
    'tags': u'أداة:استثناء:مجرور',
    'vocalized': u'بِغَيْرَ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u''
}
STOPWORDS[u'كغير'] = {
    'word': u'كغير',
    'procletic': u'-كَ',
    'tags': u'أداة:استثناء:مجرور',
    'vocalized': u'كَغَيْرَ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u''
}
STOPWORDS[u'لغير'] = {
    'word': u'لغير',
    'procletic': u'-لِ',
    'tags': u'أداة:استثناء:مجرور',
    'vocalized': u'لِغَيْرَ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u''
}
STOPWORDS[u'لغير'] = {
    'word': u'لغير',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور',
    'vocalized': u'لَغَيْرَ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u''
}
STOPWORDS[u'وغير'] = {
    'word': u'وغير',
    'procletic': u'-وَ',
    'tags': u'أداة:استثناء:معطوف',
    'vocalized': u'وَغَيْرَ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u''
}
STOPWORDS[u'فغير'] = {
    'word': u'فغير',
    'procletic': u'-فَ',
    'tags': u'أداة:استثناء:معطوف',
    'vocalized': u'فَغَيْرَ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u''
}
STOPWORDS[u'وبغير'] = {
    'word': u'وبغير',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:استثناء:مجرور:معطوف',
    'vocalized': u'وَبِغَيْرَ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u''
}
STOPWORDS[u'فبغير'] = {
    'word': u'فبغير',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:استثناء:مجرور:معطوف',
    'vocalized': u'فَبِغَيْرَ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u''
}
STOPWORDS[u'وكغير'] = {
    'word': u'وكغير',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:استثناء:مجرور:معطوف',
    'vocalized': u'وَكَغَيْرَ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u''
}
STOPWORDS[u'فكغير'] = {
    'word': u'فكغير',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:استثناء:مجرور:معطوف',
    'vocalized': u'فَكَغَيْرَ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u''
}
STOPWORDS[u'ولغير'] = {
    'word': u'ولغير',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:استثناء:مجرور:معطوف',
    'vocalized': u'وَلِغَيْرَ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u''
}
STOPWORDS[u'فلغير'] = {
    'word': u'فلغير',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:استثناء:مجرور:معطوف',
    'vocalized': u'فَلِغَيْرَ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u''
}
STOPWORDS[u'ولغير'] = {
    'word': u'ولغير',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'وَلَغَيْرَ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u''
}
STOPWORDS[u'فلغير'] = {
    'word': u'فلغير',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'فَلَغَيْرَ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u''
}
STOPWORDS[u'أغير'] = {
    'word': u'أغير',
    'procletic': u'-أَ',
    'tags': u'أداة:استثناء:مجرور',
    'vocalized': u'أَغَيْرَ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u''
}
STOPWORDS[u'أبغير'] = {
    'word': u'أبغير',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:استثناء:مجرور:مجرور',
    'vocalized': u'أَبِغَيْرَ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u''
}
STOPWORDS[u'أكغير'] = {
    'word': u'أكغير',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:استثناء:مجرور:مجرور',
    'vocalized': u'أَكَغَيْرَ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u''
}
STOPWORDS[u'ألغير'] = {
    'word': u'ألغير',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:استثناء:مجرور:مجرور',
    'vocalized': u'أَلِغَيْرَ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u''
}
STOPWORDS[u'ألغير'] = {
    'word': u'ألغير',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور',
    'vocalized': u'أَلَغَيْرَ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u''
}
STOPWORDS[u'أوغير'] = {
    'word': u'أوغير',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:استثناء:معطوف:مجرور',
    'vocalized': u'أَوَغَيْرَ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u''
}
STOPWORDS[u'أفغير'] = {
    'word': u'أفغير',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:استثناء:معطوف:مجرور',
    'vocalized': u'أَفَغَيْرَ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u''
}
STOPWORDS[u'أوبغير'] = {
    'word': u'أوبغير',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَبِغَيْرَ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u''
}
STOPWORDS[u'أفبغير'] = {
    'word': u'أفبغير',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَبِغَيْرَ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u''
}
STOPWORDS[u'أوكغير'] = {
    'word': u'أوكغير',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَكَغَيْرَ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u''
}
STOPWORDS[u'أفكغير'] = {
    'word': u'أفكغير',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَكَغَيْرَ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u''
}
STOPWORDS[u'أولغير'] = {
    'word': u'أولغير',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَلِغَيْرَ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u''
}
STOPWORDS[u'أفلغير'] = {
    'word': u'أفلغير',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَلِغَيْرَ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u''
}
STOPWORDS[u'أولغير'] = {
    'word': u'أولغير',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَلَغَيْرَ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u''
}
STOPWORDS[u'أفلغير'] = {
    'word': u'أفلغير',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَلَغَيْرَ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u''
}
STOPWORDS[u'غيري'] = {
    'word': u'غيري',
    'procletic': u'',
    'tags': u'أداة:استثناء:مضاف',
    'vocalized': u'غَيْرَِي',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'ِي'
}
STOPWORDS[u'غيرك'] = {
    'word': u'غيرك',
    'procletic': u'',
    'tags': u'أداة:استثناء:مضاف',
    'vocalized': u'غَيْرَكَ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كَ'
}
STOPWORDS[u'غيرك'] = {
    'word': u'غيرك',
    'procletic': u'',
    'tags': u'أداة:استثناء:مضاف',
    'vocalized': u'غَيْرَكِ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كِ'
}
STOPWORDS[u'غيره'] = {
    'word': u'غيره',
    'procletic': u'',
    'tags': u'أداة:استثناء:مضاف',
    'vocalized': u'غَيْرَه',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'ه'
}
STOPWORDS[u'غيركم'] = {
    'word': u'غيركم',
    'procletic': u'',
    'tags': u'أداة:استثناء:مضاف',
    'vocalized': u'غَيْرَكُمْ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'غيركن'] = {
    'word': u'غيركن',
    'procletic': u'',
    'tags': u'أداة:استثناء:مضاف',
    'vocalized': u'غَيْرَكُنَّ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'غيرها'] = {
    'word': u'غيرها',
    'procletic': u'',
    'tags': u'أداة:استثناء:مضاف',
    'vocalized': u'غَيْرَهَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هَا'
}
STOPWORDS[u'غيرهم'] = {
    'word': u'غيرهم',
    'procletic': u'',
    'tags': u'أداة:استثناء:مضاف',
    'vocalized': u'غَيْرَهُمْ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'غيرهن'] = {
    'word': u'غيرهن',
    'procletic': u'',
    'tags': u'أداة:استثناء:مضاف',
    'vocalized': u'غَيْرَهُنَّ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'غيرنا'] = {
    'word': u'غيرنا',
    'procletic': u'',
    'tags': u'أداة:استثناء:مضاف',
    'vocalized': u'غَيْرَنَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'نَا'
}
STOPWORDS[u'غيركما'] = {
    'word': u'غيركما',
    'procletic': u'',
    'tags': u'أداة:استثناء:مضاف',
    'vocalized': u'غَيْرَكُمَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'غيرهما'] = {
    'word': u'غيرهما',
    'procletic': u'',
    'tags': u'أداة:استثناء:مضاف',
    'vocalized': u'غَيْرَهُمَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'بغيري'] = {
    'word': u'بغيري',
    'procletic': u'-بِ',
    'tags': u'أداة:استثناء:مجرور:مضاف',
    'vocalized': u'بِغَيْرَِي',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'ِي'
}
STOPWORDS[u'بغيرك'] = {
    'word': u'بغيرك',
    'procletic': u'-بِ',
    'tags': u'أداة:استثناء:مجرور:مضاف',
    'vocalized': u'بِغَيْرَكَ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كَ'
}
STOPWORDS[u'بغيرك'] = {
    'word': u'بغيرك',
    'procletic': u'-بِ',
    'tags': u'أداة:استثناء:مجرور:مضاف',
    'vocalized': u'بِغَيْرَكِ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كِ'
}
STOPWORDS[u'بغيره'] = {
    'word': u'بغيره',
    'procletic': u'-بِ',
    'tags': u'أداة:استثناء:مجرور:مضاف',
    'vocalized': u'بِغَيْرَه',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'ه'
}
STOPWORDS[u'بغيركم'] = {
    'word': u'بغيركم',
    'procletic': u'-بِ',
    'tags': u'أداة:استثناء:مجرور:مضاف',
    'vocalized': u'بِغَيْرَكُمْ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'بغيركن'] = {
    'word': u'بغيركن',
    'procletic': u'-بِ',
    'tags': u'أداة:استثناء:مجرور:مضاف',
    'vocalized': u'بِغَيْرَكُنَّ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'بغيرها'] = {
    'word': u'بغيرها',
    'procletic': u'-بِ',
    'tags': u'أداة:استثناء:مجرور:مضاف',
    'vocalized': u'بِغَيْرَهَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هَا'
}
STOPWORDS[u'بغيرهم'] = {
    'word': u'بغيرهم',
    'procletic': u'-بِ',
    'tags': u'أداة:استثناء:مجرور:مضاف',
    'vocalized': u'بِغَيْرَهُمْ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'بغيرهن'] = {
    'word': u'بغيرهن',
    'procletic': u'-بِ',
    'tags': u'أداة:استثناء:مجرور:مضاف',
    'vocalized': u'بِغَيْرَهُنَّ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'بغيرنا'] = {
    'word': u'بغيرنا',
    'procletic': u'-بِ',
    'tags': u'أداة:استثناء:مجرور:مضاف',
    'vocalized': u'بِغَيْرَنَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'نَا'
}
STOPWORDS[u'بغيركما'] = {
    'word': u'بغيركما',
    'procletic': u'-بِ',
    'tags': u'أداة:استثناء:مجرور:مضاف',
    'vocalized': u'بِغَيْرَكُمَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'بغيرهما'] = {
    'word': u'بغيرهما',
    'procletic': u'-بِ',
    'tags': u'أداة:استثناء:مجرور:مضاف',
    'vocalized': u'بِغَيْرَهُمَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'كغيري'] = {
    'word': u'كغيري',
    'procletic': u'-كَ',
    'tags': u'أداة:استثناء:مجرور:مضاف',
    'vocalized': u'كَغَيْرَِي',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'ِي'
}
STOPWORDS[u'كغيرك'] = {
    'word': u'كغيرك',
    'procletic': u'-كَ',
    'tags': u'أداة:استثناء:مجرور:مضاف',
    'vocalized': u'كَغَيْرَكَ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كَ'
}
STOPWORDS[u'كغيرك'] = {
    'word': u'كغيرك',
    'procletic': u'-كَ',
    'tags': u'أداة:استثناء:مجرور:مضاف',
    'vocalized': u'كَغَيْرَكِ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كِ'
}
STOPWORDS[u'كغيره'] = {
    'word': u'كغيره',
    'procletic': u'-كَ',
    'tags': u'أداة:استثناء:مجرور:مضاف',
    'vocalized': u'كَغَيْرَه',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'ه'
}
STOPWORDS[u'كغيركم'] = {
    'word': u'كغيركم',
    'procletic': u'-كَ',
    'tags': u'أداة:استثناء:مجرور:مضاف',
    'vocalized': u'كَغَيْرَكُمْ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'كغيركن'] = {
    'word': u'كغيركن',
    'procletic': u'-كَ',
    'tags': u'أداة:استثناء:مجرور:مضاف',
    'vocalized': u'كَغَيْرَكُنَّ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'كغيرها'] = {
    'word': u'كغيرها',
    'procletic': u'-كَ',
    'tags': u'أداة:استثناء:مجرور:مضاف',
    'vocalized': u'كَغَيْرَهَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هَا'
}
STOPWORDS[u'كغيرهم'] = {
    'word': u'كغيرهم',
    'procletic': u'-كَ',
    'tags': u'أداة:استثناء:مجرور:مضاف',
    'vocalized': u'كَغَيْرَهُمْ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'كغيرهن'] = {
    'word': u'كغيرهن',
    'procletic': u'-كَ',
    'tags': u'أداة:استثناء:مجرور:مضاف',
    'vocalized': u'كَغَيْرَهُنَّ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'كغيرنا'] = {
    'word': u'كغيرنا',
    'procletic': u'-كَ',
    'tags': u'أداة:استثناء:مجرور:مضاف',
    'vocalized': u'كَغَيْرَنَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'نَا'
}
STOPWORDS[u'كغيركما'] = {
    'word': u'كغيركما',
    'procletic': u'-كَ',
    'tags': u'أداة:استثناء:مجرور:مضاف',
    'vocalized': u'كَغَيْرَكُمَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'كغيرهما'] = {
    'word': u'كغيرهما',
    'procletic': u'-كَ',
    'tags': u'أداة:استثناء:مجرور:مضاف',
    'vocalized': u'كَغَيْرَهُمَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'لغيري'] = {
    'word': u'لغيري',
    'procletic': u'-لِ',
    'tags': u'أداة:استثناء:مجرور:مضاف',
    'vocalized': u'لِغَيْرَِي',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'ِي'
}
STOPWORDS[u'لغيرك'] = {
    'word': u'لغيرك',
    'procletic': u'-لِ',
    'tags': u'أداة:استثناء:مجرور:مضاف',
    'vocalized': u'لِغَيْرَكَ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كَ'
}
STOPWORDS[u'لغيرك'] = {
    'word': u'لغيرك',
    'procletic': u'-لِ',
    'tags': u'أداة:استثناء:مجرور:مضاف',
    'vocalized': u'لِغَيْرَكِ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كِ'
}
STOPWORDS[u'لغيره'] = {
    'word': u'لغيره',
    'procletic': u'-لِ',
    'tags': u'أداة:استثناء:مجرور:مضاف',
    'vocalized': u'لِغَيْرَه',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'ه'
}
STOPWORDS[u'لغيركم'] = {
    'word': u'لغيركم',
    'procletic': u'-لِ',
    'tags': u'أداة:استثناء:مجرور:مضاف',
    'vocalized': u'لِغَيْرَكُمْ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'لغيركن'] = {
    'word': u'لغيركن',
    'procletic': u'-لِ',
    'tags': u'أداة:استثناء:مجرور:مضاف',
    'vocalized': u'لِغَيْرَكُنَّ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'لغيرها'] = {
    'word': u'لغيرها',
    'procletic': u'-لِ',
    'tags': u'أداة:استثناء:مجرور:مضاف',
    'vocalized': u'لِغَيْرَهَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هَا'
}
STOPWORDS[u'لغيرهم'] = {
    'word': u'لغيرهم',
    'procletic': u'-لِ',
    'tags': u'أداة:استثناء:مجرور:مضاف',
    'vocalized': u'لِغَيْرَهُمْ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'لغيرهن'] = {
    'word': u'لغيرهن',
    'procletic': u'-لِ',
    'tags': u'أداة:استثناء:مجرور:مضاف',
    'vocalized': u'لِغَيْرَهُنَّ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'لغيرنا'] = {
    'word': u'لغيرنا',
    'procletic': u'-لِ',
    'tags': u'أداة:استثناء:مجرور:مضاف',
    'vocalized': u'لِغَيْرَنَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'نَا'
}
STOPWORDS[u'لغيركما'] = {
    'word': u'لغيركما',
    'procletic': u'-لِ',
    'tags': u'أداة:استثناء:مجرور:مضاف',
    'vocalized': u'لِغَيْرَكُمَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'لغيرهما'] = {
    'word': u'لغيرهما',
    'procletic': u'-لِ',
    'tags': u'أداة:استثناء:مجرور:مضاف',
    'vocalized': u'لِغَيْرَهُمَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'لغيري'] = {
    'word': u'لغيري',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَغَيْرَِي',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'ِي'
}
STOPWORDS[u'لغيرك'] = {
    'word': u'لغيرك',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَغَيْرَكَ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كَ'
}
STOPWORDS[u'لغيرك'] = {
    'word': u'لغيرك',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَغَيْرَكِ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كِ'
}
STOPWORDS[u'لغيره'] = {
    'word': u'لغيره',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَغَيْرَه',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'ه'
}
STOPWORDS[u'لغيركم'] = {
    'word': u'لغيركم',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَغَيْرَكُمْ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'لغيركن'] = {
    'word': u'لغيركن',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَغَيْرَكُنَّ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'لغيرها'] = {
    'word': u'لغيرها',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَغَيْرَهَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هَا'
}
STOPWORDS[u'لغيرهم'] = {
    'word': u'لغيرهم',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَغَيْرَهُمْ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'لغيرهن'] = {
    'word': u'لغيرهن',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَغَيْرَهُنَّ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'لغيرنا'] = {
    'word': u'لغيرنا',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَغَيْرَنَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'نَا'
}
STOPWORDS[u'لغيركما'] = {
    'word': u'لغيركما',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَغَيْرَكُمَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'لغيرهما'] = {
    'word': u'لغيرهما',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَغَيْرَهُمَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'وغيري'] = {
    'word': u'وغيري',
    'procletic': u'-وَ',
    'tags': u'أداة:استثناء:معطوف:مضاف',
    'vocalized': u'وَغَيْرَِي',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'ِي'
}
STOPWORDS[u'وغيرك'] = {
    'word': u'وغيرك',
    'procletic': u'-وَ',
    'tags': u'أداة:استثناء:معطوف:مضاف',
    'vocalized': u'وَغَيْرَكَ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كَ'
}
STOPWORDS[u'وغيرك'] = {
    'word': u'وغيرك',
    'procletic': u'-وَ',
    'tags': u'أداة:استثناء:معطوف:مضاف',
    'vocalized': u'وَغَيْرَكِ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كِ'
}
STOPWORDS[u'وغيره'] = {
    'word': u'وغيره',
    'procletic': u'-وَ',
    'tags': u'أداة:استثناء:معطوف:مضاف',
    'vocalized': u'وَغَيْرَه',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'ه'
}
STOPWORDS[u'وغيركم'] = {
    'word': u'وغيركم',
    'procletic': u'-وَ',
    'tags': u'أداة:استثناء:معطوف:مضاف',
    'vocalized': u'وَغَيْرَكُمْ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'وغيركن'] = {
    'word': u'وغيركن',
    'procletic': u'-وَ',
    'tags': u'أداة:استثناء:معطوف:مضاف',
    'vocalized': u'وَغَيْرَكُنَّ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'وغيرها'] = {
    'word': u'وغيرها',
    'procletic': u'-وَ',
    'tags': u'أداة:استثناء:معطوف:مضاف',
    'vocalized': u'وَغَيْرَهَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هَا'
}
STOPWORDS[u'وغيرهم'] = {
    'word': u'وغيرهم',
    'procletic': u'-وَ',
    'tags': u'أداة:استثناء:معطوف:مضاف',
    'vocalized': u'وَغَيْرَهُمْ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'وغيرهن'] = {
    'word': u'وغيرهن',
    'procletic': u'-وَ',
    'tags': u'أداة:استثناء:معطوف:مضاف',
    'vocalized': u'وَغَيْرَهُنَّ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'وغيرنا'] = {
    'word': u'وغيرنا',
    'procletic': u'-وَ',
    'tags': u'أداة:استثناء:معطوف:مضاف',
    'vocalized': u'وَغَيْرَنَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'نَا'
}
STOPWORDS[u'وغيركما'] = {
    'word': u'وغيركما',
    'procletic': u'-وَ',
    'tags': u'أداة:استثناء:معطوف:مضاف',
    'vocalized': u'وَغَيْرَكُمَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'وغيرهما'] = {
    'word': u'وغيرهما',
    'procletic': u'-وَ',
    'tags': u'أداة:استثناء:معطوف:مضاف',
    'vocalized': u'وَغَيْرَهُمَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فغيري'] = {
    'word': u'فغيري',
    'procletic': u'-فَ',
    'tags': u'أداة:استثناء:معطوف:مضاف',
    'vocalized': u'فَغَيْرَِي',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'ِي'
}
STOPWORDS[u'فغيرك'] = {
    'word': u'فغيرك',
    'procletic': u'-فَ',
    'tags': u'أداة:استثناء:معطوف:مضاف',
    'vocalized': u'فَغَيْرَكَ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كَ'
}
STOPWORDS[u'فغيرك'] = {
    'word': u'فغيرك',
    'procletic': u'-فَ',
    'tags': u'أداة:استثناء:معطوف:مضاف',
    'vocalized': u'فَغَيْرَكِ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كِ'
}
STOPWORDS[u'فغيره'] = {
    'word': u'فغيره',
    'procletic': u'-فَ',
    'tags': u'أداة:استثناء:معطوف:مضاف',
    'vocalized': u'فَغَيْرَه',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'ه'
}
STOPWORDS[u'فغيركم'] = {
    'word': u'فغيركم',
    'procletic': u'-فَ',
    'tags': u'أداة:استثناء:معطوف:مضاف',
    'vocalized': u'فَغَيْرَكُمْ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فغيركن'] = {
    'word': u'فغيركن',
    'procletic': u'-فَ',
    'tags': u'أداة:استثناء:معطوف:مضاف',
    'vocalized': u'فَغَيْرَكُنَّ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فغيرها'] = {
    'word': u'فغيرها',
    'procletic': u'-فَ',
    'tags': u'أداة:استثناء:معطوف:مضاف',
    'vocalized': u'فَغَيْرَهَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هَا'
}
STOPWORDS[u'فغيرهم'] = {
    'word': u'فغيرهم',
    'procletic': u'-فَ',
    'tags': u'أداة:استثناء:معطوف:مضاف',
    'vocalized': u'فَغَيْرَهُمْ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فغيرهن'] = {
    'word': u'فغيرهن',
    'procletic': u'-فَ',
    'tags': u'أداة:استثناء:معطوف:مضاف',
    'vocalized': u'فَغَيْرَهُنَّ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فغيرنا'] = {
    'word': u'فغيرنا',
    'procletic': u'-فَ',
    'tags': u'أداة:استثناء:معطوف:مضاف',
    'vocalized': u'فَغَيْرَنَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'نَا'
}
STOPWORDS[u'فغيركما'] = {
    'word': u'فغيركما',
    'procletic': u'-فَ',
    'tags': u'أداة:استثناء:معطوف:مضاف',
    'vocalized': u'فَغَيْرَكُمَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فغيرهما'] = {
    'word': u'فغيرهما',
    'procletic': u'-فَ',
    'tags': u'أداة:استثناء:معطوف:مضاف',
    'vocalized': u'فَغَيْرَهُمَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'وبغيري'] = {
    'word': u'وبغيري',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِغَيْرَِي',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'ِي'
}
STOPWORDS[u'وبغيرك'] = {
    'word': u'وبغيرك',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِغَيْرَكَ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كَ'
}
STOPWORDS[u'وبغيرك'] = {
    'word': u'وبغيرك',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِغَيْرَكِ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كِ'
}
STOPWORDS[u'وبغيره'] = {
    'word': u'وبغيره',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِغَيْرَه',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'ه'
}
STOPWORDS[u'وبغيركم'] = {
    'word': u'وبغيركم',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِغَيْرَكُمْ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'وبغيركن'] = {
    'word': u'وبغيركن',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِغَيْرَكُنَّ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'وبغيرها'] = {
    'word': u'وبغيرها',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِغَيْرَهَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هَا'
}
STOPWORDS[u'وبغيرهم'] = {
    'word': u'وبغيرهم',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِغَيْرَهُمْ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'وبغيرهن'] = {
    'word': u'وبغيرهن',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِغَيْرَهُنَّ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'وبغيرنا'] = {
    'word': u'وبغيرنا',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِغَيْرَنَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'نَا'
}
STOPWORDS[u'وبغيركما'] = {
    'word': u'وبغيركما',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِغَيْرَكُمَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'وبغيرهما'] = {
    'word': u'وبغيرهما',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِغَيْرَهُمَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فبغيري'] = {
    'word': u'فبغيري',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِغَيْرَِي',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'ِي'
}
STOPWORDS[u'فبغيرك'] = {
    'word': u'فبغيرك',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِغَيْرَكَ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كَ'
}
STOPWORDS[u'فبغيرك'] = {
    'word': u'فبغيرك',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِغَيْرَكِ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كِ'
}
STOPWORDS[u'فبغيره'] = {
    'word': u'فبغيره',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِغَيْرَه',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'ه'
}
STOPWORDS[u'فبغيركم'] = {
    'word': u'فبغيركم',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِغَيْرَكُمْ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فبغيركن'] = {
    'word': u'فبغيركن',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِغَيْرَكُنَّ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فبغيرها'] = {
    'word': u'فبغيرها',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِغَيْرَهَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هَا'
}
STOPWORDS[u'فبغيرهم'] = {
    'word': u'فبغيرهم',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِغَيْرَهُمْ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فبغيرهن'] = {
    'word': u'فبغيرهن',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِغَيْرَهُنَّ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فبغيرنا'] = {
    'word': u'فبغيرنا',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِغَيْرَنَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'نَا'
}
STOPWORDS[u'فبغيركما'] = {
    'word': u'فبغيركما',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِغَيْرَكُمَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فبغيرهما'] = {
    'word': u'فبغيرهما',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِغَيْرَهُمَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'وكغيري'] = {
    'word': u'وكغيري',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَغَيْرَِي',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'ِي'
}
STOPWORDS[u'وكغيرك'] = {
    'word': u'وكغيرك',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَغَيْرَكَ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كَ'
}
STOPWORDS[u'وكغيرك'] = {
    'word': u'وكغيرك',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَغَيْرَكِ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كِ'
}
STOPWORDS[u'وكغيره'] = {
    'word': u'وكغيره',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَغَيْرَه',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'ه'
}
STOPWORDS[u'وكغيركم'] = {
    'word': u'وكغيركم',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَغَيْرَكُمْ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'وكغيركن'] = {
    'word': u'وكغيركن',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَغَيْرَكُنَّ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'وكغيرها'] = {
    'word': u'وكغيرها',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَغَيْرَهَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هَا'
}
STOPWORDS[u'وكغيرهم'] = {
    'word': u'وكغيرهم',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَغَيْرَهُمْ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'وكغيرهن'] = {
    'word': u'وكغيرهن',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَغَيْرَهُنَّ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'وكغيرنا'] = {
    'word': u'وكغيرنا',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَغَيْرَنَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'نَا'
}
STOPWORDS[u'وكغيركما'] = {
    'word': u'وكغيركما',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَغَيْرَكُمَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'وكغيرهما'] = {
    'word': u'وكغيرهما',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَغَيْرَهُمَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فكغيري'] = {
    'word': u'فكغيري',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَغَيْرَِي',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'ِي'
}
STOPWORDS[u'فكغيرك'] = {
    'word': u'فكغيرك',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَغَيْرَكَ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كَ'
}
STOPWORDS[u'فكغيرك'] = {
    'word': u'فكغيرك',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَغَيْرَكِ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كِ'
}
STOPWORDS[u'فكغيره'] = {
    'word': u'فكغيره',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَغَيْرَه',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'ه'
}
STOPWORDS[u'فكغيركم'] = {
    'word': u'فكغيركم',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَغَيْرَكُمْ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فكغيركن'] = {
    'word': u'فكغيركن',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَغَيْرَكُنَّ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فكغيرها'] = {
    'word': u'فكغيرها',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَغَيْرَهَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هَا'
}
STOPWORDS[u'فكغيرهم'] = {
    'word': u'فكغيرهم',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَغَيْرَهُمْ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فكغيرهن'] = {
    'word': u'فكغيرهن',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَغَيْرَهُنَّ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فكغيرنا'] = {
    'word': u'فكغيرنا',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَغَيْرَنَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'نَا'
}
STOPWORDS[u'فكغيركما'] = {
    'word': u'فكغيركما',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَغَيْرَكُمَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فكغيرهما'] = {
    'word': u'فكغيرهما',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَغَيْرَهُمَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ولغيري'] = {
    'word': u'ولغيري',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِغَيْرَِي',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'ِي'
}
STOPWORDS[u'ولغيرك'] = {
    'word': u'ولغيرك',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِغَيْرَكَ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كَ'
}
STOPWORDS[u'ولغيرك'] = {
    'word': u'ولغيرك',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِغَيْرَكِ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كِ'
}
STOPWORDS[u'ولغيره'] = {
    'word': u'ولغيره',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِغَيْرَه',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'ه'
}
STOPWORDS[u'ولغيركم'] = {
    'word': u'ولغيركم',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِغَيْرَكُمْ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ولغيركن'] = {
    'word': u'ولغيركن',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِغَيْرَكُنَّ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ولغيرها'] = {
    'word': u'ولغيرها',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِغَيْرَهَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هَا'
}
STOPWORDS[u'ولغيرهم'] = {
    'word': u'ولغيرهم',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِغَيْرَهُمْ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ولغيرهن'] = {
    'word': u'ولغيرهن',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِغَيْرَهُنَّ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ولغيرنا'] = {
    'word': u'ولغيرنا',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِغَيْرَنَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'نَا'
}
STOPWORDS[u'ولغيركما'] = {
    'word': u'ولغيركما',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِغَيْرَكُمَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ولغيرهما'] = {
    'word': u'ولغيرهما',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِغَيْرَهُمَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فلغيري'] = {
    'word': u'فلغيري',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِغَيْرَِي',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'ِي'
}
STOPWORDS[u'فلغيرك'] = {
    'word': u'فلغيرك',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِغَيْرَكَ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كَ'
}
STOPWORDS[u'فلغيرك'] = {
    'word': u'فلغيرك',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِغَيْرَكِ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كِ'
}
STOPWORDS[u'فلغيره'] = {
    'word': u'فلغيره',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِغَيْرَه',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'ه'
}
STOPWORDS[u'فلغيركم'] = {
    'word': u'فلغيركم',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِغَيْرَكُمْ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فلغيركن'] = {
    'word': u'فلغيركن',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِغَيْرَكُنَّ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فلغيرها'] = {
    'word': u'فلغيرها',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِغَيْرَهَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هَا'
}
STOPWORDS[u'فلغيرهم'] = {
    'word': u'فلغيرهم',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِغَيْرَهُمْ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فلغيرهن'] = {
    'word': u'فلغيرهن',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِغَيْرَهُنَّ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فلغيرنا'] = {
    'word': u'فلغيرنا',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِغَيْرَنَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'نَا'
}
STOPWORDS[u'فلغيركما'] = {
    'word': u'فلغيركما',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِغَيْرَكُمَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فلغيرهما'] = {
    'word': u'فلغيرهما',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِغَيْرَهُمَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ولغيري'] = {
    'word': u'ولغيري',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَغَيْرَِي',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'ِي'
}
STOPWORDS[u'ولغيرك'] = {
    'word': u'ولغيرك',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَغَيْرَكَ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كَ'
}
STOPWORDS[u'ولغيرك'] = {
    'word': u'ولغيرك',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَغَيْرَكِ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كِ'
}
STOPWORDS[u'ولغيره'] = {
    'word': u'ولغيره',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَغَيْرَه',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'ه'
}
STOPWORDS[u'ولغيركم'] = {
    'word': u'ولغيركم',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَغَيْرَكُمْ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ولغيركن'] = {
    'word': u'ولغيركن',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَغَيْرَكُنَّ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ولغيرها'] = {
    'word': u'ولغيرها',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَغَيْرَهَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هَا'
}
STOPWORDS[u'ولغيرهم'] = {
    'word': u'ولغيرهم',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَغَيْرَهُمْ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ولغيرهن'] = {
    'word': u'ولغيرهن',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَغَيْرَهُنَّ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ولغيرنا'] = {
    'word': u'ولغيرنا',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَغَيْرَنَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'نَا'
}
STOPWORDS[u'ولغيركما'] = {
    'word': u'ولغيركما',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَغَيْرَكُمَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ولغيرهما'] = {
    'word': u'ولغيرهما',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَغَيْرَهُمَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فلغيري'] = {
    'word': u'فلغيري',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَغَيْرَِي',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'ِي'
}
STOPWORDS[u'فلغيرك'] = {
    'word': u'فلغيرك',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَغَيْرَكَ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كَ'
}
STOPWORDS[u'فلغيرك'] = {
    'word': u'فلغيرك',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَغَيْرَكِ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كِ'
}
STOPWORDS[u'فلغيره'] = {
    'word': u'فلغيره',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَغَيْرَه',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'ه'
}
STOPWORDS[u'فلغيركم'] = {
    'word': u'فلغيركم',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَغَيْرَكُمْ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فلغيركن'] = {
    'word': u'فلغيركن',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَغَيْرَكُنَّ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فلغيرها'] = {
    'word': u'فلغيرها',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَغَيْرَهَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هَا'
}
STOPWORDS[u'فلغيرهم'] = {
    'word': u'فلغيرهم',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَغَيْرَهُمْ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فلغيرهن'] = {
    'word': u'فلغيرهن',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَغَيْرَهُنَّ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فلغيرنا'] = {
    'word': u'فلغيرنا',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَغَيْرَنَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'نَا'
}
STOPWORDS[u'فلغيركما'] = {
    'word': u'فلغيركما',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَغَيْرَكُمَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فلغيرهما'] = {
    'word': u'فلغيرهما',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَغَيْرَهُمَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أغيري'] = {
    'word': u'أغيري',
    'procletic': u'-أَ',
    'tags': u'أداة:استثناء:مجرور:مضاف',
    'vocalized': u'أَغَيْرَِي',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أغيرك'] = {
    'word': u'أغيرك',
    'procletic': u'-أَ',
    'tags': u'أداة:استثناء:مجرور:مضاف',
    'vocalized': u'أَغَيْرَكَ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أغيرك'] = {
    'word': u'أغيرك',
    'procletic': u'-أَ',
    'tags': u'أداة:استثناء:مجرور:مضاف',
    'vocalized': u'أَغَيْرَكِ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أغيره'] = {
    'word': u'أغيره',
    'procletic': u'-أَ',
    'tags': u'أداة:استثناء:مجرور:مضاف',
    'vocalized': u'أَغَيْرَه',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'ه'
}
STOPWORDS[u'أغيركم'] = {
    'word': u'أغيركم',
    'procletic': u'-أَ',
    'tags': u'أداة:استثناء:مجرور:مضاف',
    'vocalized': u'أَغَيْرَكُمْ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أغيركن'] = {
    'word': u'أغيركن',
    'procletic': u'-أَ',
    'tags': u'أداة:استثناء:مجرور:مضاف',
    'vocalized': u'أَغَيْرَكُنَّ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أغيرها'] = {
    'word': u'أغيرها',
    'procletic': u'-أَ',
    'tags': u'أداة:استثناء:مجرور:مضاف',
    'vocalized': u'أَغَيْرَهَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أغيرهم'] = {
    'word': u'أغيرهم',
    'procletic': u'-أَ',
    'tags': u'أداة:استثناء:مجرور:مضاف',
    'vocalized': u'أَغَيْرَهُمْ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أغيرهن'] = {
    'word': u'أغيرهن',
    'procletic': u'-أَ',
    'tags': u'أداة:استثناء:مجرور:مضاف',
    'vocalized': u'أَغَيْرَهُنَّ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أغيرنا'] = {
    'word': u'أغيرنا',
    'procletic': u'-أَ',
    'tags': u'أداة:استثناء:مجرور:مضاف',
    'vocalized': u'أَغَيْرَنَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أغيركما'] = {
    'word': u'أغيركما',
    'procletic': u'-أَ',
    'tags': u'أداة:استثناء:مجرور:مضاف',
    'vocalized': u'أَغَيْرَكُمَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أغيرهما'] = {
    'word': u'أغيرهما',
    'procletic': u'-أَ',
    'tags': u'أداة:استثناء:مجرور:مضاف',
    'vocalized': u'أَغَيْرَهُمَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أبغيري'] = {
    'word': u'أبغيري',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:استثناء:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِغَيْرَِي',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أبغيرك'] = {
    'word': u'أبغيرك',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:استثناء:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِغَيْرَكَ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أبغيرك'] = {
    'word': u'أبغيرك',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:استثناء:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِغَيْرَكِ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أبغيره'] = {
    'word': u'أبغيره',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:استثناء:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِغَيْرَه',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'ه'
}
STOPWORDS[u'أبغيركم'] = {
    'word': u'أبغيركم',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:استثناء:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِغَيْرَكُمْ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أبغيركن'] = {
    'word': u'أبغيركن',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:استثناء:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِغَيْرَكُنَّ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أبغيرها'] = {
    'word': u'أبغيرها',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:استثناء:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِغَيْرَهَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أبغيرهم'] = {
    'word': u'أبغيرهم',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:استثناء:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِغَيْرَهُمْ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أبغيرهن'] = {
    'word': u'أبغيرهن',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:استثناء:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِغَيْرَهُنَّ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أبغيرنا'] = {
    'word': u'أبغيرنا',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:استثناء:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِغَيْرَنَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أبغيركما'] = {
    'word': u'أبغيركما',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:استثناء:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِغَيْرَكُمَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أبغيرهما'] = {
    'word': u'أبغيرهما',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:استثناء:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِغَيْرَهُمَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أكغيري'] = {
    'word': u'أكغيري',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:استثناء:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَغَيْرَِي',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أكغيرك'] = {
    'word': u'أكغيرك',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:استثناء:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَغَيْرَكَ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أكغيرك'] = {
    'word': u'أكغيرك',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:استثناء:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَغَيْرَكِ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أكغيره'] = {
    'word': u'أكغيره',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:استثناء:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَغَيْرَه',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'ه'
}
STOPWORDS[u'أكغيركم'] = {
    'word': u'أكغيركم',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:استثناء:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَغَيْرَكُمْ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أكغيركن'] = {
    'word': u'أكغيركن',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:استثناء:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَغَيْرَكُنَّ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أكغيرها'] = {
    'word': u'أكغيرها',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:استثناء:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَغَيْرَهَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أكغيرهم'] = {
    'word': u'أكغيرهم',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:استثناء:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَغَيْرَهُمْ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أكغيرهن'] = {
    'word': u'أكغيرهن',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:استثناء:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَغَيْرَهُنَّ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أكغيرنا'] = {
    'word': u'أكغيرنا',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:استثناء:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَغَيْرَنَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أكغيركما'] = {
    'word': u'أكغيركما',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:استثناء:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَغَيْرَكُمَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أكغيرهما'] = {
    'word': u'أكغيرهما',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:استثناء:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَغَيْرَهُمَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ألغيري'] = {
    'word': u'ألغيري',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:استثناء:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِغَيْرَِي',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'ِي'
}
STOPWORDS[u'ألغيرك'] = {
    'word': u'ألغيرك',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:استثناء:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِغَيْرَكَ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كَ'
}
STOPWORDS[u'ألغيرك'] = {
    'word': u'ألغيرك',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:استثناء:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِغَيْرَكِ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كِ'
}
STOPWORDS[u'ألغيره'] = {
    'word': u'ألغيره',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:استثناء:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِغَيْرَه',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'ه'
}
STOPWORDS[u'ألغيركم'] = {
    'word': u'ألغيركم',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:استثناء:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِغَيْرَكُمْ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ألغيركن'] = {
    'word': u'ألغيركن',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:استثناء:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِغَيْرَكُنَّ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ألغيرها'] = {
    'word': u'ألغيرها',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:استثناء:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِغَيْرَهَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هَا'
}
STOPWORDS[u'ألغيرهم'] = {
    'word': u'ألغيرهم',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:استثناء:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِغَيْرَهُمْ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ألغيرهن'] = {
    'word': u'ألغيرهن',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:استثناء:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِغَيْرَهُنَّ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ألغيرنا'] = {
    'word': u'ألغيرنا',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:استثناء:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِغَيْرَنَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'نَا'
}
STOPWORDS[u'ألغيركما'] = {
    'word': u'ألغيركما',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:استثناء:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِغَيْرَكُمَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ألغيرهما'] = {
    'word': u'ألغيرهما',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:استثناء:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِغَيْرَهُمَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ألغيري'] = {
    'word': u'ألغيري',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَغَيْرَِي',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'ِي'
}
STOPWORDS[u'ألغيرك'] = {
    'word': u'ألغيرك',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَغَيْرَكَ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كَ'
}
STOPWORDS[u'ألغيرك'] = {
    'word': u'ألغيرك',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَغَيْرَكِ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كِ'
}
STOPWORDS[u'ألغيره'] = {
    'word': u'ألغيره',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَغَيْرَه',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'ه'
}
STOPWORDS[u'ألغيركم'] = {
    'word': u'ألغيركم',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَغَيْرَكُمْ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ألغيركن'] = {
    'word': u'ألغيركن',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَغَيْرَكُنَّ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ألغيرها'] = {
    'word': u'ألغيرها',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَغَيْرَهَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هَا'
}
STOPWORDS[u'ألغيرهم'] = {
    'word': u'ألغيرهم',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَغَيْرَهُمْ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ألغيرهن'] = {
    'word': u'ألغيرهن',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَغَيْرَهُنَّ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ألغيرنا'] = {
    'word': u'ألغيرنا',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَغَيْرَنَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'نَا'
}
STOPWORDS[u'ألغيركما'] = {
    'word': u'ألغيركما',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَغَيْرَكُمَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ألغيرهما'] = {
    'word': u'ألغيرهما',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَغَيْرَهُمَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أوغيري'] = {
    'word': u'أوغيري',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:استثناء:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَغَيْرَِي',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أوغيرك'] = {
    'word': u'أوغيرك',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:استثناء:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَغَيْرَكَ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أوغيرك'] = {
    'word': u'أوغيرك',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:استثناء:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَغَيْرَكِ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أوغيره'] = {
    'word': u'أوغيره',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:استثناء:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَغَيْرَه',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'ه'
}
STOPWORDS[u'أوغيركم'] = {
    'word': u'أوغيركم',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:استثناء:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَغَيْرَكُمْ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أوغيركن'] = {
    'word': u'أوغيركن',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:استثناء:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَغَيْرَكُنَّ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أوغيرها'] = {
    'word': u'أوغيرها',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:استثناء:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَغَيْرَهَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أوغيرهم'] = {
    'word': u'أوغيرهم',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:استثناء:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَغَيْرَهُمْ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أوغيرهن'] = {
    'word': u'أوغيرهن',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:استثناء:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَغَيْرَهُنَّ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أوغيرنا'] = {
    'word': u'أوغيرنا',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:استثناء:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَغَيْرَنَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أوغيركما'] = {
    'word': u'أوغيركما',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:استثناء:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَغَيْرَكُمَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أوغيرهما'] = {
    'word': u'أوغيرهما',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:استثناء:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَغَيْرَهُمَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أفغيري'] = {
    'word': u'أفغيري',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:استثناء:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَغَيْرَِي',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أفغيرك'] = {
    'word': u'أفغيرك',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:استثناء:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَغَيْرَكَ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أفغيرك'] = {
    'word': u'أفغيرك',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:استثناء:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَغَيْرَكِ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أفغيره'] = {
    'word': u'أفغيره',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:استثناء:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَغَيْرَه',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'ه'
}
STOPWORDS[u'أفغيركم'] = {
    'word': u'أفغيركم',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:استثناء:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَغَيْرَكُمْ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أفغيركن'] = {
    'word': u'أفغيركن',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:استثناء:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَغَيْرَكُنَّ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أفغيرها'] = {
    'word': u'أفغيرها',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:استثناء:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَغَيْرَهَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أفغيرهم'] = {
    'word': u'أفغيرهم',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:استثناء:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَغَيْرَهُمْ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أفغيرهن'] = {
    'word': u'أفغيرهن',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:استثناء:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَغَيْرَهُنَّ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أفغيرنا'] = {
    'word': u'أفغيرنا',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:استثناء:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَغَيْرَنَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أفغيركما'] = {
    'word': u'أفغيركما',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:استثناء:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَغَيْرَكُمَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أفغيرهما'] = {
    'word': u'أفغيرهما',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:استثناء:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَغَيْرَهُمَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أوبغيري'] = {
    'word': u'أوبغيري',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِغَيْرَِي',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أوبغيرك'] = {
    'word': u'أوبغيرك',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِغَيْرَكَ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أوبغيرك'] = {
    'word': u'أوبغيرك',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِغَيْرَكِ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أوبغيره'] = {
    'word': u'أوبغيره',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِغَيْرَه',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'ه'
}
STOPWORDS[u'أوبغيركم'] = {
    'word': u'أوبغيركم',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِغَيْرَكُمْ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أوبغيركن'] = {
    'word': u'أوبغيركن',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِغَيْرَكُنَّ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أوبغيرها'] = {
    'word': u'أوبغيرها',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِغَيْرَهَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أوبغيرهم'] = {
    'word': u'أوبغيرهم',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِغَيْرَهُمْ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أوبغيرهن'] = {
    'word': u'أوبغيرهن',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِغَيْرَهُنَّ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أوبغيرنا'] = {
    'word': u'أوبغيرنا',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِغَيْرَنَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أوبغيركما'] = {
    'word': u'أوبغيركما',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِغَيْرَكُمَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أوبغيرهما'] = {
    'word': u'أوبغيرهما',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِغَيْرَهُمَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أفبغيري'] = {
    'word': u'أفبغيري',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِغَيْرَِي',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أفبغيرك'] = {
    'word': u'أفبغيرك',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِغَيْرَكَ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أفبغيرك'] = {
    'word': u'أفبغيرك',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِغَيْرَكِ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أفبغيره'] = {
    'word': u'أفبغيره',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِغَيْرَه',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'ه'
}
STOPWORDS[u'أفبغيركم'] = {
    'word': u'أفبغيركم',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِغَيْرَكُمْ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أفبغيركن'] = {
    'word': u'أفبغيركن',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِغَيْرَكُنَّ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أفبغيرها'] = {
    'word': u'أفبغيرها',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِغَيْرَهَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أفبغيرهم'] = {
    'word': u'أفبغيرهم',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِغَيْرَهُمْ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أفبغيرهن'] = {
    'word': u'أفبغيرهن',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِغَيْرَهُنَّ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أفبغيرنا'] = {
    'word': u'أفبغيرنا',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِغَيْرَنَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أفبغيركما'] = {
    'word': u'أفبغيركما',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِغَيْرَكُمَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أفبغيرهما'] = {
    'word': u'أفبغيرهما',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِغَيْرَهُمَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أوكغيري'] = {
    'word': u'أوكغيري',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَغَيْرَِي',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أوكغيرك'] = {
    'word': u'أوكغيرك',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَغَيْرَكَ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أوكغيرك'] = {
    'word': u'أوكغيرك',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَغَيْرَكِ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أوكغيره'] = {
    'word': u'أوكغيره',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَغَيْرَه',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'ه'
}
STOPWORDS[u'أوكغيركم'] = {
    'word': u'أوكغيركم',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَغَيْرَكُمْ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أوكغيركن'] = {
    'word': u'أوكغيركن',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَغَيْرَكُنَّ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أوكغيرها'] = {
    'word': u'أوكغيرها',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَغَيْرَهَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أوكغيرهم'] = {
    'word': u'أوكغيرهم',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَغَيْرَهُمْ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أوكغيرهن'] = {
    'word': u'أوكغيرهن',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَغَيْرَهُنَّ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أوكغيرنا'] = {
    'word': u'أوكغيرنا',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَغَيْرَنَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أوكغيركما'] = {
    'word': u'أوكغيركما',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَغَيْرَكُمَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أوكغيرهما'] = {
    'word': u'أوكغيرهما',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَغَيْرَهُمَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أفكغيري'] = {
    'word': u'أفكغيري',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَغَيْرَِي',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أفكغيرك'] = {
    'word': u'أفكغيرك',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَغَيْرَكَ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أفكغيرك'] = {
    'word': u'أفكغيرك',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَغَيْرَكِ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أفكغيره'] = {
    'word': u'أفكغيره',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَغَيْرَه',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'ه'
}
STOPWORDS[u'أفكغيركم'] = {
    'word': u'أفكغيركم',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَغَيْرَكُمْ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أفكغيركن'] = {
    'word': u'أفكغيركن',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَغَيْرَكُنَّ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أفكغيرها'] = {
    'word': u'أفكغيرها',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَغَيْرَهَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أفكغيرهم'] = {
    'word': u'أفكغيرهم',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَغَيْرَهُمْ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أفكغيرهن'] = {
    'word': u'أفكغيرهن',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَغَيْرَهُنَّ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أفكغيرنا'] = {
    'word': u'أفكغيرنا',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَغَيْرَنَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أفكغيركما'] = {
    'word': u'أفكغيركما',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَغَيْرَكُمَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أفكغيرهما'] = {
    'word': u'أفكغيرهما',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَغَيْرَهُمَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أولغيري'] = {
    'word': u'أولغيري',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِغَيْرَِي',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أولغيرك'] = {
    'word': u'أولغيرك',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِغَيْرَكَ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أولغيرك'] = {
    'word': u'أولغيرك',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِغَيْرَكِ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أولغيره'] = {
    'word': u'أولغيره',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِغَيْرَه',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'ه'
}
STOPWORDS[u'أولغيركم'] = {
    'word': u'أولغيركم',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِغَيْرَكُمْ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أولغيركن'] = {
    'word': u'أولغيركن',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِغَيْرَكُنَّ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أولغيرها'] = {
    'word': u'أولغيرها',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِغَيْرَهَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أولغيرهم'] = {
    'word': u'أولغيرهم',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِغَيْرَهُمْ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أولغيرهن'] = {
    'word': u'أولغيرهن',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِغَيْرَهُنَّ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أولغيرنا'] = {
    'word': u'أولغيرنا',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِغَيْرَنَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أولغيركما'] = {
    'word': u'أولغيركما',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِغَيْرَكُمَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أولغيرهما'] = {
    'word': u'أولغيرهما',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِغَيْرَهُمَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أفلغيري'] = {
    'word': u'أفلغيري',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِغَيْرَِي',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أفلغيرك'] = {
    'word': u'أفلغيرك',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِغَيْرَكَ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أفلغيرك'] = {
    'word': u'أفلغيرك',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِغَيْرَكِ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أفلغيره'] = {
    'word': u'أفلغيره',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِغَيْرَه',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'ه'
}
STOPWORDS[u'أفلغيركم'] = {
    'word': u'أفلغيركم',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِغَيْرَكُمْ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أفلغيركن'] = {
    'word': u'أفلغيركن',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِغَيْرَكُنَّ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أفلغيرها'] = {
    'word': u'أفلغيرها',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِغَيْرَهَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أفلغيرهم'] = {
    'word': u'أفلغيرهم',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِغَيْرَهُمْ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أفلغيرهن'] = {
    'word': u'أفلغيرهن',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِغَيْرَهُنَّ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أفلغيرنا'] = {
    'word': u'أفلغيرنا',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِغَيْرَنَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أفلغيركما'] = {
    'word': u'أفلغيركما',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِغَيْرَكُمَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أفلغيرهما'] = {
    'word': u'أفلغيرهما',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:استثناء:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِغَيْرَهُمَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أولغيري'] = {
    'word': u'أولغيري',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَغَيْرَِي',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أولغيرك'] = {
    'word': u'أولغيرك',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَغَيْرَكَ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أولغيرك'] = {
    'word': u'أولغيرك',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَغَيْرَكِ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أولغيره'] = {
    'word': u'أولغيره',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَغَيْرَه',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'ه'
}
STOPWORDS[u'أولغيركم'] = {
    'word': u'أولغيركم',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَغَيْرَكُمْ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أولغيركن'] = {
    'word': u'أولغيركن',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَغَيْرَكُنَّ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أولغيرها'] = {
    'word': u'أولغيرها',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَغَيْرَهَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أولغيرهم'] = {
    'word': u'أولغيرهم',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَغَيْرَهُمْ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أولغيرهن'] = {
    'word': u'أولغيرهن',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَغَيْرَهُنَّ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أولغيرنا'] = {
    'word': u'أولغيرنا',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَغَيْرَنَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أولغيركما'] = {
    'word': u'أولغيركما',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَغَيْرَكُمَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أولغيرهما'] = {
    'word': u'أولغيرهما',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَغَيْرَهُمَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أفلغيري'] = {
    'word': u'أفلغيري',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَغَيْرَِي',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أفلغيرك'] = {
    'word': u'أفلغيرك',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَغَيْرَكَ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أفلغيرك'] = {
    'word': u'أفلغيرك',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَغَيْرَكِ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أفلغيره'] = {
    'word': u'أفلغيره',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَغَيْرَه',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'ه'
}
STOPWORDS[u'أفلغيركم'] = {
    'word': u'أفلغيركم',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَغَيْرَكُمْ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أفلغيركن'] = {
    'word': u'أفلغيركن',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَغَيْرَكُنَّ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أفلغيرها'] = {
    'word': u'أفلغيرها',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَغَيْرَهَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أفلغيرهم'] = {
    'word': u'أفلغيرهم',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَغَيْرَهُمْ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أفلغيرهن'] = {
    'word': u'أفلغيرهن',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَغَيْرَهُنَّ',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أفلغيرنا'] = {
    'word': u'أفلغيرنا',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَغَيْرَنَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أفلغيركما'] = {
    'word': u'أفلغيركما',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَغَيْرَكُمَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أفلغيرهما'] = {
    'word': u'أفلغيرهما',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَغَيْرَهُمَا',
    'stem': u'غَيْرَ',
    'type': u'STOPWORD',
    'original': u'غَيْرَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'لاسيما'] = {
    'word': u'لاسِيَّمَا',
    'procletic': u'',
    'tags': u'أداة:استثناء',
    'vocalized': u'لاسِيَّمَا',
    'stem': u'لاسِيَّمَا',
    'type': u'STOPWORD',
    'original': u'لاسِيَّمَا',
    'encletic': u''
}
STOPWORDS[u'ولاسيما'] = {
    'word': u'ولاسيما',
    'procletic': u'-وَ',
    'tags': u'أداة:استثناء:معطوف',
    'vocalized': u'وَلاسِيَّمَا',
    'stem': u'لاسِيَّمَا',
    'type': u'STOPWORD',
    'original': u'لاسِيَّمَا',
    'encletic': u''
}
STOPWORDS[u'فلاسيما'] = {
    'word': u'فلاسيما',
    'procletic': u'-فَ',
    'tags': u'أداة:استثناء:معطوف',
    'vocalized': u'فَلاسِيَّمَا',
    'stem': u'لاسِيَّمَا',
    'type': u'STOPWORD',
    'original': u'لاسِيَّمَا',
    'encletic': u''
}
STOPWORDS[u'متى'] = {
    'word': u'مَتَى',
    'procletic': u'',
    'tags': u'أداة:اسم استفهام/ظرف زمان',
    'vocalized': u'مَتَى',
    'stem': u'مَتَى',
    'type': u'STOPWORD',
    'original': u'مَتَى',
    'encletic': u''
}
STOPWORDS[u'ومتى'] = {
    'word': u'ومتى',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم استفهام/ظرف زمان:معطوف',
    'vocalized': u'وَمَتَى',
    'stem': u'مَتَى',
    'type': u'STOPWORD',
    'original': u'مَتَى',
    'encletic': u''
}
STOPWORDS[u'فمتى'] = {
    'word': u'فمتى',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم استفهام/ظرف زمان:معطوف',
    'vocalized': u'فَمَتَى',
    'stem': u'مَتَى',
    'type': u'STOPWORD',
    'original': u'مَتَى',
    'encletic': u''
}
STOPWORDS[u'أنى'] = {
    'word': u'أَنَّى',
    'procletic': u'',
    'tags': u'أداة:اسم الاستفهام',
    'vocalized': u'أَنَّى',
    'stem': u'أَنَّى',
    'type': u'STOPWORD',
    'original': u'أَنَّى',
    'encletic': u''
}
STOPWORDS[u'وأنى'] = {
    'word': u'وأنى',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم الاستفهام:معطوف',
    'vocalized': u'وَأَنَّى',
    'stem': u'أَنَّى',
    'type': u'STOPWORD',
    'original': u'أَنَّى',
    'encletic': u''
}
STOPWORDS[u'فأنى'] = {
    'word': u'فأنى',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم الاستفهام:معطوف',
    'vocalized': u'فَأَنَّى',
    'stem': u'أَنَّى',
    'type': u'STOPWORD',
    'original': u'أَنَّى',
    'encletic': u''
}
STOPWORDS[u'أي'] = {
    'word': u'أَيُّ',
    'procletic': u'',
    'tags': u'أداة:اسم الاستفهام',
    'vocalized': u'أَيُّ',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u''
}
STOPWORDS[u'وأي'] = {
    'word': u'وأي',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم الاستفهام:معطوف',
    'vocalized': u'وَأَيُّ',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u''
}
STOPWORDS[u'فأي'] = {
    'word': u'فأي',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم الاستفهام:معطوف',
    'vocalized': u'فَأَيُّ',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u''
}
STOPWORDS[u'أيان'] = {
    'word': u'ّأَيَّانَ',
    'procletic': u'',
    'tags': u'أداة:اسم الاستفهام',
    'vocalized': u'ّأَيَّانَ',
    'stem': u'ّأَيَّانَ',
    'type': u'STOPWORD',
    'original': u'ّأَيَّانَ',
    'encletic': u''
}
STOPWORDS[u'وأيان'] = {
    'word': u'وأيان',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم الاستفهام:معطوف',
    'vocalized': u'وَّأَيَّانَ',
    'stem': u'ّأَيَّانَ',
    'type': u'STOPWORD',
    'original': u'ّأَيَّانَ',
    'encletic': u''
}
STOPWORDS[u'فأيان'] = {
    'word': u'فأيان',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم الاستفهام:معطوف',
    'vocalized': u'فَّأَيَّانَ',
    'stem': u'ّأَيَّانَ',
    'type': u'STOPWORD',
    'original': u'ّأَيَّانَ',
    'encletic': u''
}
STOPWORDS[u'أين'] = {
    'word': u'أَيْنَ',
    'procletic': u'',
    'tags': u'أداة:اسم الاستفهام',
    'vocalized': u'أَيْنَ',
    'stem': u'أَيْنَ',
    'type': u'STOPWORD',
    'original': u'أَيْنَ',
    'encletic': u''
}
STOPWORDS[u'وأين'] = {
    'word': u'وأين',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم الاستفهام:معطوف',
    'vocalized': u'وَأَيْنَ',
    'stem': u'أَيْنَ',
    'type': u'STOPWORD',
    'original': u'أَيْنَ',
    'encletic': u''
}
STOPWORDS[u'فأين'] = {
    'word': u'فأين',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم الاستفهام:معطوف',
    'vocalized': u'فَأَيْنَ',
    'stem': u'أَيْنَ',
    'type': u'STOPWORD',
    'original': u'أَيْنَ',
    'encletic': u''
}
STOPWORDS[u'بكم'] = {
    'word': u'بِكَمْ',
    'procletic': u'',
    'tags': u'أداة:اسم الاستفهام',
    'vocalized': u'بِكَمْ',
    'stem': u'بِكَمْ',
    'type': u'STOPWORD',
    'original': u'بِكَمْ',
    'encletic': u''
}
STOPWORDS[u'وبكم'] = {
    'word': u'وبكم',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم الاستفهام:معطوف',
    'vocalized': u'وَبِكَمْ',
    'stem': u'بِكَمْ',
    'type': u'STOPWORD',
    'original': u'بِكَمْ',
    'encletic': u''
}
STOPWORDS[u'فبكم'] = {
    'word': u'فبكم',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم الاستفهام:معطوف',
    'vocalized': u'فَبِكَمْ',
    'stem': u'بِكَمْ',
    'type': u'STOPWORD',
    'original': u'بِكَمْ',
    'encletic': u''
}
STOPWORDS[u'بما'] = {
    'word': u'بِمَا',
    'procletic': u'',
    'tags': u'أداة:اسم الاستفهام',
    'vocalized': u'بِمَا',
    'stem': u'بِمَا',
    'type': u'STOPWORD',
    'original': u'بِمَا',
    'encletic': u''
}
STOPWORDS[u'وبما'] = {
    'word': u'وبما',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم الاستفهام:معطوف',
    'vocalized': u'وَبِمَا',
    'stem': u'بِمَا',
    'type': u'STOPWORD',
    'original': u'بِمَا',
    'encletic': u''
}
STOPWORDS[u'فبما'] = {
    'word': u'فبما',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم الاستفهام:معطوف',
    'vocalized': u'فَبِمَا',
    'stem': u'بِمَا',
    'type': u'STOPWORD',
    'original': u'بِمَا',
    'encletic': u''
}
STOPWORDS[u'أبما'] = {
    'word': u'أبما',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم الاستفهام:مجرور',
    'vocalized': u'أَبِمَا',
    'stem': u'بِمَا',
    'type': u'STOPWORD',
    'original': u'بِمَا',
    'encletic': u''
}
STOPWORDS[u'أوبما'] = {
    'word': u'أوبما',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم الاستفهام:معطوف:مجرور',
    'vocalized': u'أَوَبِمَا',
    'stem': u'بِمَا',
    'type': u'STOPWORD',
    'original': u'بِمَا',
    'encletic': u''
}
STOPWORDS[u'أفبما'] = {
    'word': u'أفبما',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم الاستفهام:معطوف:مجرور',
    'vocalized': u'أَفَبِمَا',
    'stem': u'بِمَا',
    'type': u'STOPWORD',
    'original': u'بِمَا',
    'encletic': u''
}
STOPWORDS[u'بماذا'] = {
    'word': u'بِمَاذَا',
    'procletic': u'',
    'tags': u'أداة:اسم الاستفهام',
    'vocalized': u'بِمَاذَا',
    'stem': u'بِمَاذَا',
    'type': u'STOPWORD',
    'original': u'بِمَاذَا',
    'encletic': u''
}
STOPWORDS[u'وبماذا'] = {
    'word': u'وبماذا',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم الاستفهام:معطوف',
    'vocalized': u'وَبِمَاذَا',
    'stem': u'بِمَاذَا',
    'type': u'STOPWORD',
    'original': u'بِمَاذَا',
    'encletic': u''
}
STOPWORDS[u'فبماذا'] = {
    'word': u'فبماذا',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم الاستفهام:معطوف',
    'vocalized': u'فَبِمَاذَا',
    'stem': u'بِمَاذَا',
    'type': u'STOPWORD',
    'original': u'بِمَاذَا',
    'encletic': u''
}
STOPWORDS[u'بمن'] = {
    'word': u'بِمَنْ',
    'procletic': u'',
    'tags': u'أداة:اسم الاستفهام',
    'vocalized': u'بِمَنْ',
    'stem': u'بِمَنْ',
    'type': u'STOPWORD',
    'original': u'بِمَنْ',
    'encletic': u''
}
STOPWORDS[u'وبمن'] = {
    'word': u'وبمن',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم الاستفهام:معطوف',
    'vocalized': u'وَبِمَنْ',
    'stem': u'بِمَنْ',
    'type': u'STOPWORD',
    'original': u'بِمَنْ',
    'encletic': u''
}
STOPWORDS[u'فبمن'] = {
    'word': u'فبمن',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم الاستفهام:معطوف',
    'vocalized': u'فَبِمَنْ',
    'stem': u'بِمَنْ',
    'type': u'STOPWORD',
    'original': u'بِمَنْ',
    'encletic': u''
}
STOPWORDS[u'كم'] = {
    'word': u'كَمْ',
    'procletic': u'',
    'tags': u'أداة:اسم الاستفهام',
    'vocalized': u'كَمْ',
    'stem': u'كَمْ',
    'type': u'STOPWORD',
    'original': u'كَمْ',
    'encletic': u''
}
STOPWORDS[u'وكم'] = {
    'word': u'وكم',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم الاستفهام:معطوف',
    'vocalized': u'وَكَمْ',
    'stem': u'كَمْ',
    'type': u'STOPWORD',
    'original': u'كَمْ',
    'encletic': u''
}
STOPWORDS[u'فكم'] = {
    'word': u'فكم',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم الاستفهام:معطوف',
    'vocalized': u'فَكَمْ',
    'stem': u'كَمْ',
    'type': u'STOPWORD',
    'original': u'كَمْ',
    'encletic': u''
}
STOPWORDS[u'كيف'] = {
    'word': u'كَيْفَ',
    'procletic': u'',
    'tags': u'أداة:اسم الاستفهام',
    'vocalized': u'كَيْفَ',
    'stem': u'كَيْفَ',
    'type': u'STOPWORD',
    'original': u'كَيْفَ',
    'encletic': u''
}
STOPWORDS[u'وكيف'] = {
    'word': u'وكيف',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم الاستفهام:معطوف',
    'vocalized': u'وَكَيْفَ',
    'stem': u'كَيْفَ',
    'type': u'STOPWORD',
    'original': u'كَيْفَ',
    'encletic': u''
}
STOPWORDS[u'فكيف'] = {
    'word': u'فكيف',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم الاستفهام:معطوف',
    'vocalized': u'فَكَيْفَ',
    'stem': u'كَيْفَ',
    'type': u'STOPWORD',
    'original': u'كَيْفَ',
    'encletic': u''
}
STOPWORDS[u'ما'] = {
    'word': u'مَا',
    'procletic': u'',
    'tags': u'أداة:اسم الاستفهام',
    'vocalized': u'مَا',
    'stem': u'مَا',
    'type': u'STOPWORD',
    'original': u'مَا',
    'encletic': u''
}
STOPWORDS[u'وما'] = {
    'word': u'وما',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم الاستفهام:معطوف',
    'vocalized': u'وَمَا',
    'stem': u'مَا',
    'type': u'STOPWORD',
    'original': u'مَا',
    'encletic': u''
}
STOPWORDS[u'فما'] = {
    'word': u'فما',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم الاستفهام:معطوف',
    'vocalized': u'فَمَا',
    'stem': u'مَا',
    'type': u'STOPWORD',
    'original': u'مَا',
    'encletic': u''
}
STOPWORDS[u'ماذا'] = {
    'word': u'مَاذَا',
    'procletic': u'',
    'tags': u'أداة:اسم الاستفهام',
    'vocalized': u'مَاذَا',
    'stem': u'مَاذَا',
    'type': u'STOPWORD',
    'original': u'مَاذَا',
    'encletic': u''
}
STOPWORDS[u'وماذا'] = {
    'word': u'وماذا',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم الاستفهام:معطوف',
    'vocalized': u'وَمَاذَا',
    'stem': u'مَاذَا',
    'type': u'STOPWORD',
    'original': u'مَاذَا',
    'encletic': u''
}
STOPWORDS[u'فماذا'] = {
    'word': u'فماذا',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم الاستفهام:معطوف',
    'vocalized': u'فَمَاذَا',
    'stem': u'مَاذَا',
    'type': u'STOPWORD',
    'original': u'مَاذَا',
    'encletic': u''
}
STOPWORDS[u'أماذا'] = {
    'word': u'أماذا',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم الاستفهام:مجرور',
    'vocalized': u'أَمَاذَا',
    'stem': u'مَاذَا',
    'type': u'STOPWORD',
    'original': u'مَاذَا',
    'encletic': u''
}
STOPWORDS[u'أوماذا'] = {
    'word': u'أوماذا',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم الاستفهام:معطوف:مجرور',
    'vocalized': u'أَوَمَاذَا',
    'stem': u'مَاذَا',
    'type': u'STOPWORD',
    'original': u'مَاذَا',
    'encletic': u''
}
STOPWORDS[u'أفماذا'] = {
    'word': u'أفماذا',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم الاستفهام:معطوف:مجرور',
    'vocalized': u'أَفَمَاذَا',
    'stem': u'مَاذَا',
    'type': u'STOPWORD',
    'original': u'مَاذَا',
    'encletic': u''
}
STOPWORDS[u'متى'] = {
    'word': u'مَتَى',
    'procletic': u'',
    'tags': u'أداة:اسم الاستفهام',
    'vocalized': u'مَتَى',
    'stem': u'مَتَى',
    'type': u'STOPWORD',
    'original': u'مَتَى',
    'encletic': u''
}
STOPWORDS[u'ومتى'] = {
    'word': u'ومتى',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم الاستفهام:معطوف',
    'vocalized': u'وَمَتَى',
    'stem': u'مَتَى',
    'type': u'STOPWORD',
    'original': u'مَتَى',
    'encletic': u''
}
STOPWORDS[u'فمتى'] = {
    'word': u'فمتى',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم الاستفهام:معطوف',
    'vocalized': u'فَمَتَى',
    'stem': u'مَتَى',
    'type': u'STOPWORD',
    'original': u'مَتَى',
    'encletic': u''
}
STOPWORDS[u'مما'] = {
    'word': u'مِمَّا',
    'procletic': u'',
    'tags': u'أداة:اسم الاستفهام',
    'vocalized': u'مِمَّا',
    'stem': u'مِمَّا',
    'type': u'STOPWORD',
    'original': u'مِمَّا',
    'encletic': u''
}
STOPWORDS[u'ومما'] = {
    'word': u'ومما',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم الاستفهام:معطوف',
    'vocalized': u'وَمِمَّا',
    'stem': u'مِمَّا',
    'type': u'STOPWORD',
    'original': u'مِمَّا',
    'encletic': u''
}
STOPWORDS[u'فمما'] = {
    'word': u'فمما',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم الاستفهام:معطوف',
    'vocalized': u'فَمِمَّا',
    'stem': u'مِمَّا',
    'type': u'STOPWORD',
    'original': u'مِمَّا',
    'encletic': u''
}
STOPWORDS[u'أمما'] = {
    'word': u'أمما',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم الاستفهام:مجرور',
    'vocalized': u'أَمِمَّا',
    'stem': u'مِمَّا',
    'type': u'STOPWORD',
    'original': u'مِمَّا',
    'encletic': u''
}
STOPWORDS[u'أومما'] = {
    'word': u'أومما',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم الاستفهام:معطوف:مجرور',
    'vocalized': u'أَوَمِمَّا',
    'stem': u'مِمَّا',
    'type': u'STOPWORD',
    'original': u'مِمَّا',
    'encletic': u''
}
STOPWORDS[u'أفمما'] = {
    'word': u'أفمما',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم الاستفهام:معطوف:مجرور',
    'vocalized': u'أَفَمِمَّا',
    'stem': u'مِمَّا',
    'type': u'STOPWORD',
    'original': u'مِمَّا',
    'encletic': u''
}
STOPWORDS[u'ممن'] = {
    'word': u'مِمَّنْ',
    'procletic': u'',
    'tags': u'أداة:اسم الاستفهام',
    'vocalized': u'مِمَّنْ',
    'stem': u'مِمَّنْ',
    'type': u'STOPWORD',
    'original': u'مِمَّنْ',
    'encletic': u''
}
STOPWORDS[u'وممن'] = {
    'word': u'وممن',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم الاستفهام:معطوف',
    'vocalized': u'وَمِمَّنْ',
    'stem': u'مِمَّنْ',
    'type': u'STOPWORD',
    'original': u'مِمَّنْ',
    'encletic': u''
}
STOPWORDS[u'فممن'] = {
    'word': u'فممن',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم الاستفهام:معطوف',
    'vocalized': u'فَمِمَّنْ',
    'stem': u'مِمَّنْ',
    'type': u'STOPWORD',
    'original': u'مِمَّنْ',
    'encletic': u''
}
STOPWORDS[u'من'] = {
    'word': u'مَنْ',
    'procletic': u'',
    'tags': u'أداة:اسم الاستفهام',
    'vocalized': u'مَنْ',
    'stem': u'مَنْ',
    'type': u'STOPWORD',
    'original': u'مَنْ',
    'encletic': u''
}
STOPWORDS[u'ومن'] = {
    'word': u'ومن',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم الاستفهام:معطوف',
    'vocalized': u'وَمَنْ',
    'stem': u'مَنْ',
    'type': u'STOPWORD',
    'original': u'مَنْ',
    'encletic': u''
}
STOPWORDS[u'فمن'] = {
    'word': u'فمن',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم الاستفهام:معطوف',
    'vocalized': u'فَمَنْ',
    'stem': u'مَنْ',
    'type': u'STOPWORD',
    'original': u'مَنْ',
    'encletic': u''
}
STOPWORDS[u'أنى'] = {
    'word': u'أَنَّى',
    'procletic': u'',
    'tags': u'أداة:اسم الشرط',
    'vocalized': u'أَنَّى',
    'stem': u'أَنَّى',
    'type': u'STOPWORD',
    'original': u'أَنَّى',
    'encletic': u''
}
STOPWORDS[u'وأنى'] = {
    'word': u'وأنى',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم الشرط:معطوف',
    'vocalized': u'وَأَنَّى',
    'stem': u'أَنَّى',
    'type': u'STOPWORD',
    'original': u'أَنَّى',
    'encletic': u''
}
STOPWORDS[u'فأنى'] = {
    'word': u'فأنى',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم الشرط:معطوف',
    'vocalized': u'فَأَنَّى',
    'stem': u'أَنَّى',
    'type': u'STOPWORD',
    'original': u'أَنَّى',
    'encletic': u''
}
STOPWORDS[u'أي'] = {
    'word': u'أَيُّ',
    'procletic': u'',
    'tags': u'أداة:اسم الشرط',
    'vocalized': u'أَيُّ',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u''
}
STOPWORDS[u'وأي'] = {
    'word': u'وأي',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم الشرط:معطوف',
    'vocalized': u'وَأَيُّ',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u''
}
STOPWORDS[u'فأي'] = {
    'word': u'فأي',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم الشرط:معطوف',
    'vocalized': u'فَأَيُّ',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u''
}
STOPWORDS[u'أيان'] = {
    'word': u'ّأَيَّانَ',
    'procletic': u'',
    'tags': u'أداة:اسم الشرط',
    'vocalized': u'ّأَيَّانَ',
    'stem': u'ّأَيَّانَ',
    'type': u'STOPWORD',
    'original': u'ّأَيَّانَ',
    'encletic': u''
}
STOPWORDS[u'وأيان'] = {
    'word': u'وأيان',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم الشرط:معطوف',
    'vocalized': u'وَّأَيَّانَ',
    'stem': u'ّأَيَّانَ',
    'type': u'STOPWORD',
    'original': u'ّأَيَّانَ',
    'encletic': u''
}
STOPWORDS[u'فأيان'] = {
    'word': u'فأيان',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم الشرط:معطوف',
    'vocalized': u'فَّأَيَّانَ',
    'stem': u'ّأَيَّانَ',
    'type': u'STOPWORD',
    'original': u'ّأَيَّانَ',
    'encletic': u''
}
STOPWORDS[u'أين'] = {
    'word': u'أَيْنَ',
    'procletic': u'',
    'tags': u'أداة:اسم الشرط',
    'vocalized': u'أَيْنَ',
    'stem': u'أَيْنَ',
    'type': u'STOPWORD',
    'original': u'أَيْنَ',
    'encletic': u''
}
STOPWORDS[u'وأين'] = {
    'word': u'وأين',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم الشرط:معطوف',
    'vocalized': u'وَأَيْنَ',
    'stem': u'أَيْنَ',
    'type': u'STOPWORD',
    'original': u'أَيْنَ',
    'encletic': u''
}
STOPWORDS[u'فأين'] = {
    'word': u'فأين',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم الشرط:معطوف',
    'vocalized': u'فَأَيْنَ',
    'stem': u'أَيْنَ',
    'type': u'STOPWORD',
    'original': u'أَيْنَ',
    'encletic': u''
}
STOPWORDS[u'أينما'] = {
    'word': u'أَيْنَمَا',
    'procletic': u'',
    'tags': u'أداة:اسم الشرط',
    'vocalized': u'أَيْنَمَا',
    'stem': u'أَيْنَمَا',
    'type': u'STOPWORD',
    'original': u'أَيْنَمَا',
    'encletic': u''
}
STOPWORDS[u'وأينما'] = {
    'word': u'وأينما',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم الشرط:معطوف',
    'vocalized': u'وَأَيْنَمَا',
    'stem': u'أَيْنَمَا',
    'type': u'STOPWORD',
    'original': u'أَيْنَمَا',
    'encletic': u''
}
STOPWORDS[u'فأينما'] = {
    'word': u'فأينما',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم الشرط:معطوف',
    'vocalized': u'فَأَيْنَمَا',
    'stem': u'أَيْنَمَا',
    'type': u'STOPWORD',
    'original': u'أَيْنَمَا',
    'encletic': u''
}
STOPWORDS[u'حيثما'] = {
    'word': u'حَيْثُمَا',
    'procletic': u'',
    'tags': u'أداة:اسم الشرط',
    'vocalized': u'حَيْثُمَا',
    'stem': u'حَيْثُمَا',
    'type': u'STOPWORD',
    'original': u'حَيْثُمَا',
    'encletic': u''
}
STOPWORDS[u'وحيثما'] = {
    'word': u'وحيثما',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم الشرط:معطوف',
    'vocalized': u'وَحَيْثُمَا',
    'stem': u'حَيْثُمَا',
    'type': u'STOPWORD',
    'original': u'حَيْثُمَا',
    'encletic': u''
}
STOPWORDS[u'فحيثما'] = {
    'word': u'فحيثما',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم الشرط:معطوف',
    'vocalized': u'فَحَيْثُمَا',
    'stem': u'حَيْثُمَا',
    'type': u'STOPWORD',
    'original': u'حَيْثُمَا',
    'encletic': u''
}
STOPWORDS[u'كيفما'] = {
    'word': u'كَيْفَمَا',
    'procletic': u'',
    'tags': u'أداة:اسم الشرط',
    'vocalized': u'كَيْفَمَا',
    'stem': u'كَيْفَمَا',
    'type': u'STOPWORD',
    'original': u'كَيْفَمَا',
    'encletic': u''
}
STOPWORDS[u'وكيفما'] = {
    'word': u'وكيفما',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم الشرط:معطوف',
    'vocalized': u'وَكَيْفَمَا',
    'stem': u'كَيْفَمَا',
    'type': u'STOPWORD',
    'original': u'كَيْفَمَا',
    'encletic': u''
}
STOPWORDS[u'فكيفما'] = {
    'word': u'فكيفما',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم الشرط:معطوف',
    'vocalized': u'فَكَيْفَمَا',
    'stem': u'كَيْفَمَا',
    'type': u'STOPWORD',
    'original': u'كَيْفَمَا',
    'encletic': u''
}
STOPWORDS[u'ما'] = {
    'word': u'مَا',
    'procletic': u'',
    'tags': u'أداة:اسم الشرط',
    'vocalized': u'مَا',
    'stem': u'مَا',
    'type': u'STOPWORD',
    'original': u'مَا',
    'encletic': u''
}
STOPWORDS[u'وما'] = {
    'word': u'وما',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم الشرط:معطوف',
    'vocalized': u'وَمَا',
    'stem': u'مَا',
    'type': u'STOPWORD',
    'original': u'مَا',
    'encletic': u''
}
STOPWORDS[u'فما'] = {
    'word': u'فما',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم الشرط:معطوف',
    'vocalized': u'فَمَا',
    'stem': u'مَا',
    'type': u'STOPWORD',
    'original': u'مَا',
    'encletic': u''
}
STOPWORDS[u'متى'] = {
    'word': u'مَتَى',
    'procletic': u'',
    'tags': u'أداة:اسم الشرط',
    'vocalized': u'مَتَى',
    'stem': u'مَتَى',
    'type': u'STOPWORD',
    'original': u'مَتَى',
    'encletic': u''
}
STOPWORDS[u'ومتى'] = {
    'word': u'ومتى',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم الشرط:معطوف',
    'vocalized': u'وَمَتَى',
    'stem': u'مَتَى',
    'type': u'STOPWORD',
    'original': u'مَتَى',
    'encletic': u''
}
STOPWORDS[u'فمتى'] = {
    'word': u'فمتى',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم الشرط:معطوف',
    'vocalized': u'فَمَتَى',
    'stem': u'مَتَى',
    'type': u'STOPWORD',
    'original': u'مَتَى',
    'encletic': u''
}
STOPWORDS[u'من'] = {
    'word': u'مَنْ',
    'procletic': u'',
    'tags': u'أداة:اسم الشرط',
    'vocalized': u'مَنْ',
    'stem': u'مَنْ',
    'type': u'STOPWORD',
    'original': u'مَنْ',
    'encletic': u''
}
STOPWORDS[u'ومن'] = {
    'word': u'ومن',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم الشرط:معطوف',
    'vocalized': u'وَمَنْ',
    'stem': u'مَنْ',
    'type': u'STOPWORD',
    'original': u'مَنْ',
    'encletic': u''
}
STOPWORDS[u'فمن'] = {
    'word': u'فمن',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم الشرط:معطوف',
    'vocalized': u'فَمَنْ',
    'stem': u'مَنْ',
    'type': u'STOPWORD',
    'original': u'مَنْ',
    'encletic': u''
}
STOPWORDS[u'مهما'] = {
    'word': u'مَهْمَا',
    'procletic': u'',
    'tags': u'أداة:اسم الشرط',
    'vocalized': u'مَهْمَا',
    'stem': u'مَهْمَا',
    'type': u'STOPWORD',
    'original': u'مَهْمَا',
    'encletic': u''
}
STOPWORDS[u'ومهما'] = {
    'word': u'ومهما',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم الشرط:معطوف',
    'vocalized': u'وَمَهْمَا',
    'stem': u'مَهْمَا',
    'type': u'STOPWORD',
    'original': u'مَهْمَا',
    'encletic': u''
}
STOPWORDS[u'فمهما'] = {
    'word': u'فمهما',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم الشرط:معطوف',
    'vocalized': u'فَمَهْمَا',
    'stem': u'مَهْمَا',
    'type': u'STOPWORD',
    'original': u'مَهْمَا',
    'encletic': u''
}
STOPWORDS[u'أمهما'] = {
    'word': u'أمهما',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم الشرط:مجرور',
    'vocalized': u'أَمَهْمَا',
    'stem': u'مَهْمَا',
    'type': u'STOPWORD',
    'original': u'مَهْمَا',
    'encletic': u''
}
STOPWORDS[u'أومهما'] = {
    'word': u'أومهما',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم الشرط:معطوف:مجرور',
    'vocalized': u'أَوَمَهْمَا',
    'stem': u'مَهْمَا',
    'type': u'STOPWORD',
    'original': u'مَهْمَا',
    'encletic': u''
}
STOPWORDS[u'أفمهما'] = {
    'word': u'أفمهما',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم الشرط:معطوف:مجرور',
    'vocalized': u'أَفَمَهْمَا',
    'stem': u'مَهْمَا',
    'type': u'STOPWORD',
    'original': u'مَهْمَا',
    'encletic': u''
}
STOPWORDS[u'أولئك'] = {
    'word': u'أُولَئِكَ',
    'procletic': u'',
    'tags': u'أداة:اسم إشارة',
    'vocalized': u'أُولَئِكَ',
    'stem': u'أُولَئِكَ',
    'type': u'STOPWORD',
    'original': u'أُولَئِكَ',
    'encletic': u''
}
STOPWORDS[u'بأولئك'] = {
    'word': u'بأولئك',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'بِأُولَئِكَ',
    'stem': u'أُولَئِكَ',
    'type': u'STOPWORD',
    'original': u'أُولَئِكَ',
    'encletic': u''
}
STOPWORDS[u'كأولئك'] = {
    'word': u'كأولئك',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'كَأُولَئِكَ',
    'stem': u'أُولَئِكَ',
    'type': u'STOPWORD',
    'original': u'أُولَئِكَ',
    'encletic': u''
}
STOPWORDS[u'لأولئك'] = {
    'word': u'لأولئك',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'لِأُولَئِكَ',
    'stem': u'أُولَئِكَ',
    'type': u'STOPWORD',
    'original': u'أُولَئِكَ',
    'encletic': u''
}
STOPWORDS[u'وأولئك'] = {
    'word': u'وأولئك',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إشارة:معطوف',
    'vocalized': u'وَأُولَئِكَ',
    'stem': u'أُولَئِكَ',
    'type': u'STOPWORD',
    'original': u'أُولَئِكَ',
    'encletic': u''
}
STOPWORDS[u'فأولئك'] = {
    'word': u'فأولئك',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إشارة:معطوف',
    'vocalized': u'فَأُولَئِكَ',
    'stem': u'أُولَئِكَ',
    'type': u'STOPWORD',
    'original': u'أُولَئِكَ',
    'encletic': u''
}
STOPWORDS[u'وبأولئك'] = {
    'word': u'وبأولئك',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'وَبِأُولَئِكَ',
    'stem': u'أُولَئِكَ',
    'type': u'STOPWORD',
    'original': u'أُولَئِكَ',
    'encletic': u''
}
STOPWORDS[u'فبأولئك'] = {
    'word': u'فبأولئك',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'فَبِأُولَئِكَ',
    'stem': u'أُولَئِكَ',
    'type': u'STOPWORD',
    'original': u'أُولَئِكَ',
    'encletic': u''
}
STOPWORDS[u'وكأولئك'] = {
    'word': u'وكأولئك',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'وَكَأُولَئِكَ',
    'stem': u'أُولَئِكَ',
    'type': u'STOPWORD',
    'original': u'أُولَئِكَ',
    'encletic': u''
}
STOPWORDS[u'فكأولئك'] = {
    'word': u'فكأولئك',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'فَكَأُولَئِكَ',
    'stem': u'أُولَئِكَ',
    'type': u'STOPWORD',
    'original': u'أُولَئِكَ',
    'encletic': u''
}
STOPWORDS[u'ولأولئك'] = {
    'word': u'ولأولئك',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'وَلِأُولَئِكَ',
    'stem': u'أُولَئِكَ',
    'type': u'STOPWORD',
    'original': u'أُولَئِكَ',
    'encletic': u''
}
STOPWORDS[u'فلأولئك'] = {
    'word': u'فلأولئك',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'فَلِأُولَئِكَ',
    'stem': u'أُولَئِكَ',
    'type': u'STOPWORD',
    'original': u'أُولَئِكَ',
    'encletic': u''
}
STOPWORDS[u'أأولئك'] = {
    'word': u'أأولئك',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'أَأُولَئِكَ',
    'stem': u'أُولَئِكَ',
    'type': u'STOPWORD',
    'original': u'أُولَئِكَ',
    'encletic': u''
}
STOPWORDS[u'أبأولئك'] = {
    'word': u'أبأولئك',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:مجرور',
    'vocalized': u'أَبِأُولَئِكَ',
    'stem': u'أُولَئِكَ',
    'type': u'STOPWORD',
    'original': u'أُولَئِكَ',
    'encletic': u''
}
STOPWORDS[u'أكأولئك'] = {
    'word': u'أكأولئك',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:مجرور',
    'vocalized': u'أَكَأُولَئِكَ',
    'stem': u'أُولَئِكَ',
    'type': u'STOPWORD',
    'original': u'أُولَئِكَ',
    'encletic': u''
}
STOPWORDS[u'ألأولئك'] = {
    'word': u'ألأولئك',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:مجرور',
    'vocalized': u'أَلِأُولَئِكَ',
    'stem': u'أُولَئِكَ',
    'type': u'STOPWORD',
    'original': u'أُولَئِكَ',
    'encletic': u''
}
STOPWORDS[u'أوأولئك'] = {
    'word': u'أوأولئك',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إشارة:معطوف:مجرور',
    'vocalized': u'أَوَأُولَئِكَ',
    'stem': u'أُولَئِكَ',
    'type': u'STOPWORD',
    'original': u'أُولَئِكَ',
    'encletic': u''
}
STOPWORDS[u'أفأولئك'] = {
    'word': u'أفأولئك',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إشارة:معطوف:مجرور',
    'vocalized': u'أَفَأُولَئِكَ',
    'stem': u'أُولَئِكَ',
    'type': u'STOPWORD',
    'original': u'أُولَئِكَ',
    'encletic': u''
}
STOPWORDS[u'أوبأولئك'] = {
    'word': u'أوبأولئك',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَبِأُولَئِكَ',
    'stem': u'أُولَئِكَ',
    'type': u'STOPWORD',
    'original': u'أُولَئِكَ',
    'encletic': u''
}
STOPWORDS[u'أفبأولئك'] = {
    'word': u'أفبأولئك',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَبِأُولَئِكَ',
    'stem': u'أُولَئِكَ',
    'type': u'STOPWORD',
    'original': u'أُولَئِكَ',
    'encletic': u''
}
STOPWORDS[u'أوكأولئك'] = {
    'word': u'أوكأولئك',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَكَأُولَئِكَ',
    'stem': u'أُولَئِكَ',
    'type': u'STOPWORD',
    'original': u'أُولَئِكَ',
    'encletic': u''
}
STOPWORDS[u'أفكأولئك'] = {
    'word': u'أفكأولئك',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَكَأُولَئِكَ',
    'stem': u'أُولَئِكَ',
    'type': u'STOPWORD',
    'original': u'أُولَئِكَ',
    'encletic': u''
}
STOPWORDS[u'أولأولئك'] = {
    'word': u'أولأولئك',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَلِأُولَئِكَ',
    'stem': u'أُولَئِكَ',
    'type': u'STOPWORD',
    'original': u'أُولَئِكَ',
    'encletic': u''
}
STOPWORDS[u'أفلأولئك'] = {
    'word': u'أفلأولئك',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَلِأُولَئِكَ',
    'stem': u'أُولَئِكَ',
    'type': u'STOPWORD',
    'original': u'أُولَئِكَ',
    'encletic': u''
}
STOPWORDS[u'أولئكم'] = {
    'word': u'أُولَئِكُمْ',
    'procletic': u'',
    'tags': u'أداة:اسم إشارة',
    'vocalized': u'أُولَئِكُمْ',
    'stem': u'أُولَئِكُمْ',
    'type': u'STOPWORD',
    'original': u'أُولَئِكُمْ',
    'encletic': u''
}
STOPWORDS[u'بأولئكم'] = {
    'word': u'بأولئكم',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'بِأُولَئِكُمْ',
    'stem': u'أُولَئِكُمْ',
    'type': u'STOPWORD',
    'original': u'أُولَئِكُمْ',
    'encletic': u''
}
STOPWORDS[u'كأولئكم'] = {
    'word': u'كأولئكم',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'كَأُولَئِكُمْ',
    'stem': u'أُولَئِكُمْ',
    'type': u'STOPWORD',
    'original': u'أُولَئِكُمْ',
    'encletic': u''
}
STOPWORDS[u'لأولئكم'] = {
    'word': u'لأولئكم',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'لِأُولَئِكُمْ',
    'stem': u'أُولَئِكُمْ',
    'type': u'STOPWORD',
    'original': u'أُولَئِكُمْ',
    'encletic': u''
}
STOPWORDS[u'وأولئكم'] = {
    'word': u'وأولئكم',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إشارة:معطوف',
    'vocalized': u'وَأُولَئِكُمْ',
    'stem': u'أُولَئِكُمْ',
    'type': u'STOPWORD',
    'original': u'أُولَئِكُمْ',
    'encletic': u''
}
STOPWORDS[u'فأولئكم'] = {
    'word': u'فأولئكم',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إشارة:معطوف',
    'vocalized': u'فَأُولَئِكُمْ',
    'stem': u'أُولَئِكُمْ',
    'type': u'STOPWORD',
    'original': u'أُولَئِكُمْ',
    'encletic': u''
}
STOPWORDS[u'وبأولئكم'] = {
    'word': u'وبأولئكم',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'وَبِأُولَئِكُمْ',
    'stem': u'أُولَئِكُمْ',
    'type': u'STOPWORD',
    'original': u'أُولَئِكُمْ',
    'encletic': u''
}
STOPWORDS[u'فبأولئكم'] = {
    'word': u'فبأولئكم',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'فَبِأُولَئِكُمْ',
    'stem': u'أُولَئِكُمْ',
    'type': u'STOPWORD',
    'original': u'أُولَئِكُمْ',
    'encletic': u''
}
STOPWORDS[u'وكأولئكم'] = {
    'word': u'وكأولئكم',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'وَكَأُولَئِكُمْ',
    'stem': u'أُولَئِكُمْ',
    'type': u'STOPWORD',
    'original': u'أُولَئِكُمْ',
    'encletic': u''
}
STOPWORDS[u'فكأولئكم'] = {
    'word': u'فكأولئكم',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'فَكَأُولَئِكُمْ',
    'stem': u'أُولَئِكُمْ',
    'type': u'STOPWORD',
    'original': u'أُولَئِكُمْ',
    'encletic': u''
}
STOPWORDS[u'ولأولئكم'] = {
    'word': u'ولأولئكم',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'وَلِأُولَئِكُمْ',
    'stem': u'أُولَئِكُمْ',
    'type': u'STOPWORD',
    'original': u'أُولَئِكُمْ',
    'encletic': u''
}
STOPWORDS[u'فلأولئكم'] = {
    'word': u'فلأولئكم',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'فَلِأُولَئِكُمْ',
    'stem': u'أُولَئِكُمْ',
    'type': u'STOPWORD',
    'original': u'أُولَئِكُمْ',
    'encletic': u''
}
STOPWORDS[u'أأولئكم'] = {
    'word': u'أأولئكم',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'أَأُولَئِكُمْ',
    'stem': u'أُولَئِكُمْ',
    'type': u'STOPWORD',
    'original': u'أُولَئِكُمْ',
    'encletic': u''
}
STOPWORDS[u'أبأولئكم'] = {
    'word': u'أبأولئكم',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:مجرور',
    'vocalized': u'أَبِأُولَئِكُمْ',
    'stem': u'أُولَئِكُمْ',
    'type': u'STOPWORD',
    'original': u'أُولَئِكُمْ',
    'encletic': u''
}
STOPWORDS[u'أكأولئكم'] = {
    'word': u'أكأولئكم',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:مجرور',
    'vocalized': u'أَكَأُولَئِكُمْ',
    'stem': u'أُولَئِكُمْ',
    'type': u'STOPWORD',
    'original': u'أُولَئِكُمْ',
    'encletic': u''
}
STOPWORDS[u'ألأولئكم'] = {
    'word': u'ألأولئكم',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:مجرور',
    'vocalized': u'أَلِأُولَئِكُمْ',
    'stem': u'أُولَئِكُمْ',
    'type': u'STOPWORD',
    'original': u'أُولَئِكُمْ',
    'encletic': u''
}
STOPWORDS[u'أوأولئكم'] = {
    'word': u'أوأولئكم',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إشارة:معطوف:مجرور',
    'vocalized': u'أَوَأُولَئِكُمْ',
    'stem': u'أُولَئِكُمْ',
    'type': u'STOPWORD',
    'original': u'أُولَئِكُمْ',
    'encletic': u''
}
STOPWORDS[u'أفأولئكم'] = {
    'word': u'أفأولئكم',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إشارة:معطوف:مجرور',
    'vocalized': u'أَفَأُولَئِكُمْ',
    'stem': u'أُولَئِكُمْ',
    'type': u'STOPWORD',
    'original': u'أُولَئِكُمْ',
    'encletic': u''
}
STOPWORDS[u'أوبأولئكم'] = {
    'word': u'أوبأولئكم',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَبِأُولَئِكُمْ',
    'stem': u'أُولَئِكُمْ',
    'type': u'STOPWORD',
    'original': u'أُولَئِكُمْ',
    'encletic': u''
}
STOPWORDS[u'أفبأولئكم'] = {
    'word': u'أفبأولئكم',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَبِأُولَئِكُمْ',
    'stem': u'أُولَئِكُمْ',
    'type': u'STOPWORD',
    'original': u'أُولَئِكُمْ',
    'encletic': u''
}
STOPWORDS[u'أوكأولئكم'] = {
    'word': u'أوكأولئكم',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَكَأُولَئِكُمْ',
    'stem': u'أُولَئِكُمْ',
    'type': u'STOPWORD',
    'original': u'أُولَئِكُمْ',
    'encletic': u''
}
STOPWORDS[u'أفكأولئكم'] = {
    'word': u'أفكأولئكم',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَكَأُولَئِكُمْ',
    'stem': u'أُولَئِكُمْ',
    'type': u'STOPWORD',
    'original': u'أُولَئِكُمْ',
    'encletic': u''
}
STOPWORDS[u'أولأولئكم'] = {
    'word': u'أولأولئكم',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَلِأُولَئِكُمْ',
    'stem': u'أُولَئِكُمْ',
    'type': u'STOPWORD',
    'original': u'أُولَئِكُمْ',
    'encletic': u''
}
STOPWORDS[u'أفلأولئكم'] = {
    'word': u'أفلأولئكم',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَلِأُولَئِكُمْ',
    'stem': u'أُولَئِكُمْ',
    'type': u'STOPWORD',
    'original': u'أُولَئِكُمْ',
    'encletic': u''
}
STOPWORDS[u'أولاء'] = {
    'word': u'أُولَاءِ',
    'procletic': u'',
    'tags': u'أداة:اسم إشارة',
    'vocalized': u'أُولَاءِ',
    'stem': u'أُولَاءِ',
    'type': u'STOPWORD',
    'original': u'أُولَاءِ',
    'encletic': u''
}
STOPWORDS[u'بأولاء'] = {
    'word': u'بأولاء',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'بِأُولَاءِ',
    'stem': u'أُولَاءِ',
    'type': u'STOPWORD',
    'original': u'أُولَاءِ',
    'encletic': u''
}
STOPWORDS[u'كأولاء'] = {
    'word': u'كأولاء',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'كَأُولَاءِ',
    'stem': u'أُولَاءِ',
    'type': u'STOPWORD',
    'original': u'أُولَاءِ',
    'encletic': u''
}
STOPWORDS[u'لأولاء'] = {
    'word': u'لأولاء',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'لِأُولَاءِ',
    'stem': u'أُولَاءِ',
    'type': u'STOPWORD',
    'original': u'أُولَاءِ',
    'encletic': u''
}
STOPWORDS[u'وأولاء'] = {
    'word': u'وأولاء',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إشارة:معطوف',
    'vocalized': u'وَأُولَاءِ',
    'stem': u'أُولَاءِ',
    'type': u'STOPWORD',
    'original': u'أُولَاءِ',
    'encletic': u''
}
STOPWORDS[u'فأولاء'] = {
    'word': u'فأولاء',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إشارة:معطوف',
    'vocalized': u'فَأُولَاءِ',
    'stem': u'أُولَاءِ',
    'type': u'STOPWORD',
    'original': u'أُولَاءِ',
    'encletic': u''
}
STOPWORDS[u'وبأولاء'] = {
    'word': u'وبأولاء',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'وَبِأُولَاءِ',
    'stem': u'أُولَاءِ',
    'type': u'STOPWORD',
    'original': u'أُولَاءِ',
    'encletic': u''
}
STOPWORDS[u'فبأولاء'] = {
    'word': u'فبأولاء',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'فَبِأُولَاءِ',
    'stem': u'أُولَاءِ',
    'type': u'STOPWORD',
    'original': u'أُولَاءِ',
    'encletic': u''
}
STOPWORDS[u'وكأولاء'] = {
    'word': u'وكأولاء',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'وَكَأُولَاءِ',
    'stem': u'أُولَاءِ',
    'type': u'STOPWORD',
    'original': u'أُولَاءِ',
    'encletic': u''
}
STOPWORDS[u'فكأولاء'] = {
    'word': u'فكأولاء',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'فَكَأُولَاءِ',
    'stem': u'أُولَاءِ',
    'type': u'STOPWORD',
    'original': u'أُولَاءِ',
    'encletic': u''
}
STOPWORDS[u'ولأولاء'] = {
    'word': u'ولأولاء',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'وَلِأُولَاءِ',
    'stem': u'أُولَاءِ',
    'type': u'STOPWORD',
    'original': u'أُولَاءِ',
    'encletic': u''
}
STOPWORDS[u'فلأولاء'] = {
    'word': u'فلأولاء',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'فَلِأُولَاءِ',
    'stem': u'أُولَاءِ',
    'type': u'STOPWORD',
    'original': u'أُولَاءِ',
    'encletic': u''
}
STOPWORDS[u'أأولاء'] = {
    'word': u'أأولاء',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'أَأُولَاءِ',
    'stem': u'أُولَاءِ',
    'type': u'STOPWORD',
    'original': u'أُولَاءِ',
    'encletic': u''
}
STOPWORDS[u'أبأولاء'] = {
    'word': u'أبأولاء',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:مجرور',
    'vocalized': u'أَبِأُولَاءِ',
    'stem': u'أُولَاءِ',
    'type': u'STOPWORD',
    'original': u'أُولَاءِ',
    'encletic': u''
}
STOPWORDS[u'أكأولاء'] = {
    'word': u'أكأولاء',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:مجرور',
    'vocalized': u'أَكَأُولَاءِ',
    'stem': u'أُولَاءِ',
    'type': u'STOPWORD',
    'original': u'أُولَاءِ',
    'encletic': u''
}
STOPWORDS[u'ألأولاء'] = {
    'word': u'ألأولاء',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:مجرور',
    'vocalized': u'أَلِأُولَاءِ',
    'stem': u'أُولَاءِ',
    'type': u'STOPWORD',
    'original': u'أُولَاءِ',
    'encletic': u''
}
STOPWORDS[u'أوأولاء'] = {
    'word': u'أوأولاء',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إشارة:معطوف:مجرور',
    'vocalized': u'أَوَأُولَاءِ',
    'stem': u'أُولَاءِ',
    'type': u'STOPWORD',
    'original': u'أُولَاءِ',
    'encletic': u''
}
STOPWORDS[u'أفأولاء'] = {
    'word': u'أفأولاء',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إشارة:معطوف:مجرور',
    'vocalized': u'أَفَأُولَاءِ',
    'stem': u'أُولَاءِ',
    'type': u'STOPWORD',
    'original': u'أُولَاءِ',
    'encletic': u''
}
STOPWORDS[u'أوبأولاء'] = {
    'word': u'أوبأولاء',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَبِأُولَاءِ',
    'stem': u'أُولَاءِ',
    'type': u'STOPWORD',
    'original': u'أُولَاءِ',
    'encletic': u''
}
STOPWORDS[u'أفبأولاء'] = {
    'word': u'أفبأولاء',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَبِأُولَاءِ',
    'stem': u'أُولَاءِ',
    'type': u'STOPWORD',
    'original': u'أُولَاءِ',
    'encletic': u''
}
STOPWORDS[u'أوكأولاء'] = {
    'word': u'أوكأولاء',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَكَأُولَاءِ',
    'stem': u'أُولَاءِ',
    'type': u'STOPWORD',
    'original': u'أُولَاءِ',
    'encletic': u''
}
STOPWORDS[u'أفكأولاء'] = {
    'word': u'أفكأولاء',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَكَأُولَاءِ',
    'stem': u'أُولَاءِ',
    'type': u'STOPWORD',
    'original': u'أُولَاءِ',
    'encletic': u''
}
STOPWORDS[u'أولأولاء'] = {
    'word': u'أولأولاء',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَلِأُولَاءِ',
    'stem': u'أُولَاءِ',
    'type': u'STOPWORD',
    'original': u'أُولَاءِ',
    'encletic': u''
}
STOPWORDS[u'أفلأولاء'] = {
    'word': u'أفلأولاء',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَلِأُولَاءِ',
    'stem': u'أُولَاءِ',
    'type': u'STOPWORD',
    'original': u'أُولَاءِ',
    'encletic': u''
}
STOPWORDS[u'أولالك'] = {
    'word': u'أُولَالِكَ',
    'procletic': u'',
    'tags': u'أداة:اسم إشارة',
    'vocalized': u'أُولَالِكَ',
    'stem': u'أُولَالِكَ',
    'type': u'STOPWORD',
    'original': u'أُولَالِكَ',
    'encletic': u''
}
STOPWORDS[u'بأولالك'] = {
    'word': u'بأولالك',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'بِأُولَالِكَ',
    'stem': u'أُولَالِكَ',
    'type': u'STOPWORD',
    'original': u'أُولَالِكَ',
    'encletic': u''
}
STOPWORDS[u'كأولالك'] = {
    'word': u'كأولالك',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'كَأُولَالِكَ',
    'stem': u'أُولَالِكَ',
    'type': u'STOPWORD',
    'original': u'أُولَالِكَ',
    'encletic': u''
}
STOPWORDS[u'لأولالك'] = {
    'word': u'لأولالك',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'لِأُولَالِكَ',
    'stem': u'أُولَالِكَ',
    'type': u'STOPWORD',
    'original': u'أُولَالِكَ',
    'encletic': u''
}
STOPWORDS[u'وأولالك'] = {
    'word': u'وأولالك',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إشارة:معطوف',
    'vocalized': u'وَأُولَالِكَ',
    'stem': u'أُولَالِكَ',
    'type': u'STOPWORD',
    'original': u'أُولَالِكَ',
    'encletic': u''
}
STOPWORDS[u'فأولالك'] = {
    'word': u'فأولالك',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إشارة:معطوف',
    'vocalized': u'فَأُولَالِكَ',
    'stem': u'أُولَالِكَ',
    'type': u'STOPWORD',
    'original': u'أُولَالِكَ',
    'encletic': u''
}
STOPWORDS[u'وبأولالك'] = {
    'word': u'وبأولالك',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'وَبِأُولَالِكَ',
    'stem': u'أُولَالِكَ',
    'type': u'STOPWORD',
    'original': u'أُولَالِكَ',
    'encletic': u''
}
STOPWORDS[u'فبأولالك'] = {
    'word': u'فبأولالك',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'فَبِأُولَالِكَ',
    'stem': u'أُولَالِكَ',
    'type': u'STOPWORD',
    'original': u'أُولَالِكَ',
    'encletic': u''
}
STOPWORDS[u'وكأولالك'] = {
    'word': u'وكأولالك',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'وَكَأُولَالِكَ',
    'stem': u'أُولَالِكَ',
    'type': u'STOPWORD',
    'original': u'أُولَالِكَ',
    'encletic': u''
}
STOPWORDS[u'فكأولالك'] = {
    'word': u'فكأولالك',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'فَكَأُولَالِكَ',
    'stem': u'أُولَالِكَ',
    'type': u'STOPWORD',
    'original': u'أُولَالِكَ',
    'encletic': u''
}
STOPWORDS[u'ولأولالك'] = {
    'word': u'ولأولالك',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'وَلِأُولَالِكَ',
    'stem': u'أُولَالِكَ',
    'type': u'STOPWORD',
    'original': u'أُولَالِكَ',
    'encletic': u''
}
STOPWORDS[u'فلأولالك'] = {
    'word': u'فلأولالك',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'فَلِأُولَالِكَ',
    'stem': u'أُولَالِكَ',
    'type': u'STOPWORD',
    'original': u'أُولَالِكَ',
    'encletic': u''
}
STOPWORDS[u'أأولالك'] = {
    'word': u'أأولالك',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'أَأُولَالِكَ',
    'stem': u'أُولَالِكَ',
    'type': u'STOPWORD',
    'original': u'أُولَالِكَ',
    'encletic': u''
}
STOPWORDS[u'أبأولالك'] = {
    'word': u'أبأولالك',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:مجرور',
    'vocalized': u'أَبِأُولَالِكَ',
    'stem': u'أُولَالِكَ',
    'type': u'STOPWORD',
    'original': u'أُولَالِكَ',
    'encletic': u''
}
STOPWORDS[u'أكأولالك'] = {
    'word': u'أكأولالك',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:مجرور',
    'vocalized': u'أَكَأُولَالِكَ',
    'stem': u'أُولَالِكَ',
    'type': u'STOPWORD',
    'original': u'أُولَالِكَ',
    'encletic': u''
}
STOPWORDS[u'ألأولالك'] = {
    'word': u'ألأولالك',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:مجرور',
    'vocalized': u'أَلِأُولَالِكَ',
    'stem': u'أُولَالِكَ',
    'type': u'STOPWORD',
    'original': u'أُولَالِكَ',
    'encletic': u''
}
STOPWORDS[u'أوأولالك'] = {
    'word': u'أوأولالك',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إشارة:معطوف:مجرور',
    'vocalized': u'أَوَأُولَالِكَ',
    'stem': u'أُولَالِكَ',
    'type': u'STOPWORD',
    'original': u'أُولَالِكَ',
    'encletic': u''
}
STOPWORDS[u'أفأولالك'] = {
    'word': u'أفأولالك',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إشارة:معطوف:مجرور',
    'vocalized': u'أَفَأُولَالِكَ',
    'stem': u'أُولَالِكَ',
    'type': u'STOPWORD',
    'original': u'أُولَالِكَ',
    'encletic': u''
}
STOPWORDS[u'أوبأولالك'] = {
    'word': u'أوبأولالك',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَبِأُولَالِكَ',
    'stem': u'أُولَالِكَ',
    'type': u'STOPWORD',
    'original': u'أُولَالِكَ',
    'encletic': u''
}
STOPWORDS[u'أفبأولالك'] = {
    'word': u'أفبأولالك',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَبِأُولَالِكَ',
    'stem': u'أُولَالِكَ',
    'type': u'STOPWORD',
    'original': u'أُولَالِكَ',
    'encletic': u''
}
STOPWORDS[u'أوكأولالك'] = {
    'word': u'أوكأولالك',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَكَأُولَالِكَ',
    'stem': u'أُولَالِكَ',
    'type': u'STOPWORD',
    'original': u'أُولَالِكَ',
    'encletic': u''
}
STOPWORDS[u'أفكأولالك'] = {
    'word': u'أفكأولالك',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَكَأُولَالِكَ',
    'stem': u'أُولَالِكَ',
    'type': u'STOPWORD',
    'original': u'أُولَالِكَ',
    'encletic': u''
}
STOPWORDS[u'أولأولالك'] = {
    'word': u'أولأولالك',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَلِأُولَالِكَ',
    'stem': u'أُولَالِكَ',
    'type': u'STOPWORD',
    'original': u'أُولَالِكَ',
    'encletic': u''
}
STOPWORDS[u'أفلأولالك'] = {
    'word': u'أفلأولالك',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَلِأُولَالِكَ',
    'stem': u'أُولَالِكَ',
    'type': u'STOPWORD',
    'original': u'أُولَالِكَ',
    'encletic': u''
}
STOPWORDS[u'تان'] = {
    'word': u'تَانِ',
    'procletic': u'',
    'tags': u'أداة:اسم إشارة',
    'vocalized': u'تَانِ',
    'stem': u'تَانِ',
    'type': u'STOPWORD',
    'original': u'تَانِ',
    'encletic': u''
}
STOPWORDS[u'وتان'] = {
    'word': u'وتان',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إشارة:معطوف',
    'vocalized': u'وَتَانِ',
    'stem': u'تَانِ',
    'type': u'STOPWORD',
    'original': u'تَانِ',
    'encletic': u''
}
STOPWORDS[u'فتان'] = {
    'word': u'فتان',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إشارة:معطوف',
    'vocalized': u'فَتَانِ',
    'stem': u'تَانِ',
    'type': u'STOPWORD',
    'original': u'تَانِ',
    'encletic': u''
}
STOPWORDS[u'تانك'] = {
    'word': u'تَانِكَ',
    'procletic': u'',
    'tags': u'أداة:اسم إشارة',
    'vocalized': u'تَانِكَ',
    'stem': u'تَانِكَ',
    'type': u'STOPWORD',
    'original': u'تَانِكَ',
    'encletic': u''
}
STOPWORDS[u'بتانك'] = {
    'word': u'بتانك',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'بِتَانِكَ',
    'stem': u'تَانِكَ',
    'type': u'STOPWORD',
    'original': u'تَانِكَ',
    'encletic': u''
}
STOPWORDS[u'كتانك'] = {
    'word': u'كتانك',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'كَتَانِكَ',
    'stem': u'تَانِكَ',
    'type': u'STOPWORD',
    'original': u'تَانِكَ',
    'encletic': u''
}
STOPWORDS[u'لتانك'] = {
    'word': u'لتانك',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'لِتَانِكَ',
    'stem': u'تَانِكَ',
    'type': u'STOPWORD',
    'original': u'تَانِكَ',
    'encletic': u''
}
STOPWORDS[u'وتانك'] = {
    'word': u'وتانك',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إشارة:معطوف',
    'vocalized': u'وَتَانِكَ',
    'stem': u'تَانِكَ',
    'type': u'STOPWORD',
    'original': u'تَانِكَ',
    'encletic': u''
}
STOPWORDS[u'فتانك'] = {
    'word': u'فتانك',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إشارة:معطوف',
    'vocalized': u'فَتَانِكَ',
    'stem': u'تَانِكَ',
    'type': u'STOPWORD',
    'original': u'تَانِكَ',
    'encletic': u''
}
STOPWORDS[u'وبتانك'] = {
    'word': u'وبتانك',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'وَبِتَانِكَ',
    'stem': u'تَانِكَ',
    'type': u'STOPWORD',
    'original': u'تَانِكَ',
    'encletic': u''
}
STOPWORDS[u'فبتانك'] = {
    'word': u'فبتانك',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'فَبِتَانِكَ',
    'stem': u'تَانِكَ',
    'type': u'STOPWORD',
    'original': u'تَانِكَ',
    'encletic': u''
}
STOPWORDS[u'وكتانك'] = {
    'word': u'وكتانك',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'وَكَتَانِكَ',
    'stem': u'تَانِكَ',
    'type': u'STOPWORD',
    'original': u'تَانِكَ',
    'encletic': u''
}
STOPWORDS[u'فكتانك'] = {
    'word': u'فكتانك',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'فَكَتَانِكَ',
    'stem': u'تَانِكَ',
    'type': u'STOPWORD',
    'original': u'تَانِكَ',
    'encletic': u''
}
STOPWORDS[u'ولتانك'] = {
    'word': u'ولتانك',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'وَلِتَانِكَ',
    'stem': u'تَانِكَ',
    'type': u'STOPWORD',
    'original': u'تَانِكَ',
    'encletic': u''
}
STOPWORDS[u'فلتانك'] = {
    'word': u'فلتانك',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'فَلِتَانِكَ',
    'stem': u'تَانِكَ',
    'type': u'STOPWORD',
    'original': u'تَانِكَ',
    'encletic': u''
}
STOPWORDS[u'تلك'] = {
    'word': u'تِلْكَ',
    'procletic': u'',
    'tags': u'أداة:اسم إشارة',
    'vocalized': u'تِلْكَ',
    'stem': u'تِلْكَ',
    'type': u'STOPWORD',
    'original': u'تِلْكَ',
    'encletic': u''
}
STOPWORDS[u'بتلك'] = {
    'word': u'بتلك',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'بِتِلْكَ',
    'stem': u'تِلْكَ',
    'type': u'STOPWORD',
    'original': u'تِلْكَ',
    'encletic': u''
}
STOPWORDS[u'كتلك'] = {
    'word': u'كتلك',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'كَتِلْكَ',
    'stem': u'تِلْكَ',
    'type': u'STOPWORD',
    'original': u'تِلْكَ',
    'encletic': u''
}
STOPWORDS[u'لتلك'] = {
    'word': u'لتلك',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'لِتِلْكَ',
    'stem': u'تِلْكَ',
    'type': u'STOPWORD',
    'original': u'تِلْكَ',
    'encletic': u''
}
STOPWORDS[u'وتلك'] = {
    'word': u'وتلك',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إشارة:معطوف',
    'vocalized': u'وَتِلْكَ',
    'stem': u'تِلْكَ',
    'type': u'STOPWORD',
    'original': u'تِلْكَ',
    'encletic': u''
}
STOPWORDS[u'فتلك'] = {
    'word': u'فتلك',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إشارة:معطوف',
    'vocalized': u'فَتِلْكَ',
    'stem': u'تِلْكَ',
    'type': u'STOPWORD',
    'original': u'تِلْكَ',
    'encletic': u''
}
STOPWORDS[u'وبتلك'] = {
    'word': u'وبتلك',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'وَبِتِلْكَ',
    'stem': u'تِلْكَ',
    'type': u'STOPWORD',
    'original': u'تِلْكَ',
    'encletic': u''
}
STOPWORDS[u'فبتلك'] = {
    'word': u'فبتلك',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'فَبِتِلْكَ',
    'stem': u'تِلْكَ',
    'type': u'STOPWORD',
    'original': u'تِلْكَ',
    'encletic': u''
}
STOPWORDS[u'وكتلك'] = {
    'word': u'وكتلك',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'وَكَتِلْكَ',
    'stem': u'تِلْكَ',
    'type': u'STOPWORD',
    'original': u'تِلْكَ',
    'encletic': u''
}
STOPWORDS[u'فكتلك'] = {
    'word': u'فكتلك',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'فَكَتِلْكَ',
    'stem': u'تِلْكَ',
    'type': u'STOPWORD',
    'original': u'تِلْكَ',
    'encletic': u''
}
STOPWORDS[u'ولتلك'] = {
    'word': u'ولتلك',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'وَلِتِلْكَ',
    'stem': u'تِلْكَ',
    'type': u'STOPWORD',
    'original': u'تِلْكَ',
    'encletic': u''
}
STOPWORDS[u'فلتلك'] = {
    'word': u'فلتلك',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'فَلِتِلْكَ',
    'stem': u'تِلْكَ',
    'type': u'STOPWORD',
    'original': u'تِلْكَ',
    'encletic': u''
}
STOPWORDS[u'أتلك'] = {
    'word': u'أتلك',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'أَتِلْكَ',
    'stem': u'تِلْكَ',
    'type': u'STOPWORD',
    'original': u'تِلْكَ',
    'encletic': u''
}
STOPWORDS[u'أبتلك'] = {
    'word': u'أبتلك',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:مجرور',
    'vocalized': u'أَبِتِلْكَ',
    'stem': u'تِلْكَ',
    'type': u'STOPWORD',
    'original': u'تِلْكَ',
    'encletic': u''
}
STOPWORDS[u'أكتلك'] = {
    'word': u'أكتلك',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:مجرور',
    'vocalized': u'أَكَتِلْكَ',
    'stem': u'تِلْكَ',
    'type': u'STOPWORD',
    'original': u'تِلْكَ',
    'encletic': u''
}
STOPWORDS[u'ألتلك'] = {
    'word': u'ألتلك',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:مجرور',
    'vocalized': u'أَلِتِلْكَ',
    'stem': u'تِلْكَ',
    'type': u'STOPWORD',
    'original': u'تِلْكَ',
    'encletic': u''
}
STOPWORDS[u'أوتلك'] = {
    'word': u'أوتلك',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إشارة:معطوف:مجرور',
    'vocalized': u'أَوَتِلْكَ',
    'stem': u'تِلْكَ',
    'type': u'STOPWORD',
    'original': u'تِلْكَ',
    'encletic': u''
}
STOPWORDS[u'أفتلك'] = {
    'word': u'أفتلك',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إشارة:معطوف:مجرور',
    'vocalized': u'أَفَتِلْكَ',
    'stem': u'تِلْكَ',
    'type': u'STOPWORD',
    'original': u'تِلْكَ',
    'encletic': u''
}
STOPWORDS[u'أوبتلك'] = {
    'word': u'أوبتلك',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَبِتِلْكَ',
    'stem': u'تِلْكَ',
    'type': u'STOPWORD',
    'original': u'تِلْكَ',
    'encletic': u''
}
STOPWORDS[u'أفبتلك'] = {
    'word': u'أفبتلك',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَبِتِلْكَ',
    'stem': u'تِلْكَ',
    'type': u'STOPWORD',
    'original': u'تِلْكَ',
    'encletic': u''
}
STOPWORDS[u'أوكتلك'] = {
    'word': u'أوكتلك',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَكَتِلْكَ',
    'stem': u'تِلْكَ',
    'type': u'STOPWORD',
    'original': u'تِلْكَ',
    'encletic': u''
}
STOPWORDS[u'أفكتلك'] = {
    'word': u'أفكتلك',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَكَتِلْكَ',
    'stem': u'تِلْكَ',
    'type': u'STOPWORD',
    'original': u'تِلْكَ',
    'encletic': u''
}
STOPWORDS[u'أولتلك'] = {
    'word': u'أولتلك',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَلِتِلْكَ',
    'stem': u'تِلْكَ',
    'type': u'STOPWORD',
    'original': u'تِلْكَ',
    'encletic': u''
}
STOPWORDS[u'أفلتلك'] = {
    'word': u'أفلتلك',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَلِتِلْكَ',
    'stem': u'تِلْكَ',
    'type': u'STOPWORD',
    'original': u'تِلْكَ',
    'encletic': u''
}
STOPWORDS[u'تلكم'] = {
    'word': u'تِلْكُمْ',
    'procletic': u'',
    'tags': u'أداة:اسم إشارة',
    'vocalized': u'تِلْكُمْ',
    'stem': u'تِلْكُمْ',
    'type': u'STOPWORD',
    'original': u'تِلْكُمْ',
    'encletic': u''
}
STOPWORDS[u'بتلكم'] = {
    'word': u'بتلكم',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'بِتِلْكُمْ',
    'stem': u'تِلْكُمْ',
    'type': u'STOPWORD',
    'original': u'تِلْكُمْ',
    'encletic': u''
}
STOPWORDS[u'كتلكم'] = {
    'word': u'كتلكم',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'كَتِلْكُمْ',
    'stem': u'تِلْكُمْ',
    'type': u'STOPWORD',
    'original': u'تِلْكُمْ',
    'encletic': u''
}
STOPWORDS[u'لتلكم'] = {
    'word': u'لتلكم',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'لِتِلْكُمْ',
    'stem': u'تِلْكُمْ',
    'type': u'STOPWORD',
    'original': u'تِلْكُمْ',
    'encletic': u''
}
STOPWORDS[u'وتلكم'] = {
    'word': u'وتلكم',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إشارة:معطوف',
    'vocalized': u'وَتِلْكُمْ',
    'stem': u'تِلْكُمْ',
    'type': u'STOPWORD',
    'original': u'تِلْكُمْ',
    'encletic': u''
}
STOPWORDS[u'فتلكم'] = {
    'word': u'فتلكم',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إشارة:معطوف',
    'vocalized': u'فَتِلْكُمْ',
    'stem': u'تِلْكُمْ',
    'type': u'STOPWORD',
    'original': u'تِلْكُمْ',
    'encletic': u''
}
STOPWORDS[u'وبتلكم'] = {
    'word': u'وبتلكم',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'وَبِتِلْكُمْ',
    'stem': u'تِلْكُمْ',
    'type': u'STOPWORD',
    'original': u'تِلْكُمْ',
    'encletic': u''
}
STOPWORDS[u'فبتلكم'] = {
    'word': u'فبتلكم',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'فَبِتِلْكُمْ',
    'stem': u'تِلْكُمْ',
    'type': u'STOPWORD',
    'original': u'تِلْكُمْ',
    'encletic': u''
}
STOPWORDS[u'وكتلكم'] = {
    'word': u'وكتلكم',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'وَكَتِلْكُمْ',
    'stem': u'تِلْكُمْ',
    'type': u'STOPWORD',
    'original': u'تِلْكُمْ',
    'encletic': u''
}
STOPWORDS[u'فكتلكم'] = {
    'word': u'فكتلكم',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'فَكَتِلْكُمْ',
    'stem': u'تِلْكُمْ',
    'type': u'STOPWORD',
    'original': u'تِلْكُمْ',
    'encletic': u''
}
STOPWORDS[u'ولتلكم'] = {
    'word': u'ولتلكم',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'وَلِتِلْكُمْ',
    'stem': u'تِلْكُمْ',
    'type': u'STOPWORD',
    'original': u'تِلْكُمْ',
    'encletic': u''
}
STOPWORDS[u'فلتلكم'] = {
    'word': u'فلتلكم',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'فَلِتِلْكُمْ',
    'stem': u'تِلْكُمْ',
    'type': u'STOPWORD',
    'original': u'تِلْكُمْ',
    'encletic': u''
}
STOPWORDS[u'أتلكم'] = {
    'word': u'أتلكم',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'أَتِلْكُمْ',
    'stem': u'تِلْكُمْ',
    'type': u'STOPWORD',
    'original': u'تِلْكُمْ',
    'encletic': u''
}
STOPWORDS[u'أبتلكم'] = {
    'word': u'أبتلكم',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:مجرور',
    'vocalized': u'أَبِتِلْكُمْ',
    'stem': u'تِلْكُمْ',
    'type': u'STOPWORD',
    'original': u'تِلْكُمْ',
    'encletic': u''
}
STOPWORDS[u'أكتلكم'] = {
    'word': u'أكتلكم',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:مجرور',
    'vocalized': u'أَكَتِلْكُمْ',
    'stem': u'تِلْكُمْ',
    'type': u'STOPWORD',
    'original': u'تِلْكُمْ',
    'encletic': u''
}
STOPWORDS[u'ألتلكم'] = {
    'word': u'ألتلكم',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:مجرور',
    'vocalized': u'أَلِتِلْكُمْ',
    'stem': u'تِلْكُمْ',
    'type': u'STOPWORD',
    'original': u'تِلْكُمْ',
    'encletic': u''
}
STOPWORDS[u'أوتلكم'] = {
    'word': u'أوتلكم',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إشارة:معطوف:مجرور',
    'vocalized': u'أَوَتِلْكُمْ',
    'stem': u'تِلْكُمْ',
    'type': u'STOPWORD',
    'original': u'تِلْكُمْ',
    'encletic': u''
}
STOPWORDS[u'أفتلكم'] = {
    'word': u'أفتلكم',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إشارة:معطوف:مجرور',
    'vocalized': u'أَفَتِلْكُمْ',
    'stem': u'تِلْكُمْ',
    'type': u'STOPWORD',
    'original': u'تِلْكُمْ',
    'encletic': u''
}
STOPWORDS[u'أوبتلكم'] = {
    'word': u'أوبتلكم',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَبِتِلْكُمْ',
    'stem': u'تِلْكُمْ',
    'type': u'STOPWORD',
    'original': u'تِلْكُمْ',
    'encletic': u''
}
STOPWORDS[u'أفبتلكم'] = {
    'word': u'أفبتلكم',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَبِتِلْكُمْ',
    'stem': u'تِلْكُمْ',
    'type': u'STOPWORD',
    'original': u'تِلْكُمْ',
    'encletic': u''
}
STOPWORDS[u'أوكتلكم'] = {
    'word': u'أوكتلكم',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَكَتِلْكُمْ',
    'stem': u'تِلْكُمْ',
    'type': u'STOPWORD',
    'original': u'تِلْكُمْ',
    'encletic': u''
}
STOPWORDS[u'أفكتلكم'] = {
    'word': u'أفكتلكم',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَكَتِلْكُمْ',
    'stem': u'تِلْكُمْ',
    'type': u'STOPWORD',
    'original': u'تِلْكُمْ',
    'encletic': u''
}
STOPWORDS[u'أولتلكم'] = {
    'word': u'أولتلكم',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَلِتِلْكُمْ',
    'stem': u'تِلْكُمْ',
    'type': u'STOPWORD',
    'original': u'تِلْكُمْ',
    'encletic': u''
}
STOPWORDS[u'أفلتلكم'] = {
    'word': u'أفلتلكم',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَلِتِلْكُمْ',
    'stem': u'تِلْكُمْ',
    'type': u'STOPWORD',
    'original': u'تِلْكُمْ',
    'encletic': u''
}
STOPWORDS[u'تلكما'] = {
    'word': u'تِلْكُمَا',
    'procletic': u'',
    'tags': u'أداة:اسم إشارة',
    'vocalized': u'تِلْكُمَا',
    'stem': u'تِلْكُمَا',
    'type': u'STOPWORD',
    'original': u'تِلْكُمَا',
    'encletic': u''
}
STOPWORDS[u'بتلكما'] = {
    'word': u'بتلكما',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'بِتِلْكُمَا',
    'stem': u'تِلْكُمَا',
    'type': u'STOPWORD',
    'original': u'تِلْكُمَا',
    'encletic': u''
}
STOPWORDS[u'كتلكما'] = {
    'word': u'كتلكما',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'كَتِلْكُمَا',
    'stem': u'تِلْكُمَا',
    'type': u'STOPWORD',
    'original': u'تِلْكُمَا',
    'encletic': u''
}
STOPWORDS[u'لتلكما'] = {
    'word': u'لتلكما',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'لِتِلْكُمَا',
    'stem': u'تِلْكُمَا',
    'type': u'STOPWORD',
    'original': u'تِلْكُمَا',
    'encletic': u''
}
STOPWORDS[u'وتلكما'] = {
    'word': u'وتلكما',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إشارة:معطوف',
    'vocalized': u'وَتِلْكُمَا',
    'stem': u'تِلْكُمَا',
    'type': u'STOPWORD',
    'original': u'تِلْكُمَا',
    'encletic': u''
}
STOPWORDS[u'فتلكما'] = {
    'word': u'فتلكما',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إشارة:معطوف',
    'vocalized': u'فَتِلْكُمَا',
    'stem': u'تِلْكُمَا',
    'type': u'STOPWORD',
    'original': u'تِلْكُمَا',
    'encletic': u''
}
STOPWORDS[u'وبتلكما'] = {
    'word': u'وبتلكما',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'وَبِتِلْكُمَا',
    'stem': u'تِلْكُمَا',
    'type': u'STOPWORD',
    'original': u'تِلْكُمَا',
    'encletic': u''
}
STOPWORDS[u'فبتلكما'] = {
    'word': u'فبتلكما',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'فَبِتِلْكُمَا',
    'stem': u'تِلْكُمَا',
    'type': u'STOPWORD',
    'original': u'تِلْكُمَا',
    'encletic': u''
}
STOPWORDS[u'وكتلكما'] = {
    'word': u'وكتلكما',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'وَكَتِلْكُمَا',
    'stem': u'تِلْكُمَا',
    'type': u'STOPWORD',
    'original': u'تِلْكُمَا',
    'encletic': u''
}
STOPWORDS[u'فكتلكما'] = {
    'word': u'فكتلكما',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'فَكَتِلْكُمَا',
    'stem': u'تِلْكُمَا',
    'type': u'STOPWORD',
    'original': u'تِلْكُمَا',
    'encletic': u''
}
STOPWORDS[u'ولتلكما'] = {
    'word': u'ولتلكما',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'وَلِتِلْكُمَا',
    'stem': u'تِلْكُمَا',
    'type': u'STOPWORD',
    'original': u'تِلْكُمَا',
    'encletic': u''
}
STOPWORDS[u'فلتلكما'] = {
    'word': u'فلتلكما',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'فَلِتِلْكُمَا',
    'stem': u'تِلْكُمَا',
    'type': u'STOPWORD',
    'original': u'تِلْكُمَا',
    'encletic': u''
}
STOPWORDS[u'أتلكما'] = {
    'word': u'أتلكما',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'أَتِلْكُمَا',
    'stem': u'تِلْكُمَا',
    'type': u'STOPWORD',
    'original': u'تِلْكُمَا',
    'encletic': u''
}
STOPWORDS[u'أبتلكما'] = {
    'word': u'أبتلكما',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:مجرور',
    'vocalized': u'أَبِتِلْكُمَا',
    'stem': u'تِلْكُمَا',
    'type': u'STOPWORD',
    'original': u'تِلْكُمَا',
    'encletic': u''
}
STOPWORDS[u'أكتلكما'] = {
    'word': u'أكتلكما',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:مجرور',
    'vocalized': u'أَكَتِلْكُمَا',
    'stem': u'تِلْكُمَا',
    'type': u'STOPWORD',
    'original': u'تِلْكُمَا',
    'encletic': u''
}
STOPWORDS[u'ألتلكما'] = {
    'word': u'ألتلكما',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:مجرور',
    'vocalized': u'أَلِتِلْكُمَا',
    'stem': u'تِلْكُمَا',
    'type': u'STOPWORD',
    'original': u'تِلْكُمَا',
    'encletic': u''
}
STOPWORDS[u'أوتلكما'] = {
    'word': u'أوتلكما',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إشارة:معطوف:مجرور',
    'vocalized': u'أَوَتِلْكُمَا',
    'stem': u'تِلْكُمَا',
    'type': u'STOPWORD',
    'original': u'تِلْكُمَا',
    'encletic': u''
}
STOPWORDS[u'أفتلكما'] = {
    'word': u'أفتلكما',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إشارة:معطوف:مجرور',
    'vocalized': u'أَفَتِلْكُمَا',
    'stem': u'تِلْكُمَا',
    'type': u'STOPWORD',
    'original': u'تِلْكُمَا',
    'encletic': u''
}
STOPWORDS[u'أوبتلكما'] = {
    'word': u'أوبتلكما',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَبِتِلْكُمَا',
    'stem': u'تِلْكُمَا',
    'type': u'STOPWORD',
    'original': u'تِلْكُمَا',
    'encletic': u''
}
STOPWORDS[u'أفبتلكما'] = {
    'word': u'أفبتلكما',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَبِتِلْكُمَا',
    'stem': u'تِلْكُمَا',
    'type': u'STOPWORD',
    'original': u'تِلْكُمَا',
    'encletic': u''
}
STOPWORDS[u'أوكتلكما'] = {
    'word': u'أوكتلكما',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَكَتِلْكُمَا',
    'stem': u'تِلْكُمَا',
    'type': u'STOPWORD',
    'original': u'تِلْكُمَا',
    'encletic': u''
}
STOPWORDS[u'أفكتلكما'] = {
    'word': u'أفكتلكما',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَكَتِلْكُمَا',
    'stem': u'تِلْكُمَا',
    'type': u'STOPWORD',
    'original': u'تِلْكُمَا',
    'encletic': u''
}
STOPWORDS[u'أولتلكما'] = {
    'word': u'أولتلكما',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَلِتِلْكُمَا',
    'stem': u'تِلْكُمَا',
    'type': u'STOPWORD',
    'original': u'تِلْكُمَا',
    'encletic': u''
}
STOPWORDS[u'أفلتلكما'] = {
    'word': u'أفلتلكما',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَلِتِلْكُمَا',
    'stem': u'تِلْكُمَا',
    'type': u'STOPWORD',
    'original': u'تِلْكُمَا',
    'encletic': u''
}
STOPWORDS[u'ته'] = {
    'word': u'تِهِ',
    'procletic': u'',
    'tags': u'أداة:اسم إشارة',
    'vocalized': u'تِهِ',
    'stem': u'تِهِ',
    'type': u'STOPWORD',
    'original': u'تِهِ',
    'encletic': u''
}
STOPWORDS[u'بته'] = {
    'word': u'بته',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'بِتِهِ',
    'stem': u'تِهِ',
    'type': u'STOPWORD',
    'original': u'تِهِ',
    'encletic': u''
}
STOPWORDS[u'كته'] = {
    'word': u'كته',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'كَتِهِ',
    'stem': u'تِهِ',
    'type': u'STOPWORD',
    'original': u'تِهِ',
    'encletic': u''
}
STOPWORDS[u'لته'] = {
    'word': u'لته',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'لِتِهِ',
    'stem': u'تِهِ',
    'type': u'STOPWORD',
    'original': u'تِهِ',
    'encletic': u''
}
STOPWORDS[u'وته'] = {
    'word': u'وته',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إشارة:معطوف',
    'vocalized': u'وَتِهِ',
    'stem': u'تِهِ',
    'type': u'STOPWORD',
    'original': u'تِهِ',
    'encletic': u''
}
STOPWORDS[u'فته'] = {
    'word': u'فته',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إشارة:معطوف',
    'vocalized': u'فَتِهِ',
    'stem': u'تِهِ',
    'type': u'STOPWORD',
    'original': u'تِهِ',
    'encletic': u''
}
STOPWORDS[u'وبته'] = {
    'word': u'وبته',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'وَبِتِهِ',
    'stem': u'تِهِ',
    'type': u'STOPWORD',
    'original': u'تِهِ',
    'encletic': u''
}
STOPWORDS[u'فبته'] = {
    'word': u'فبته',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'فَبِتِهِ',
    'stem': u'تِهِ',
    'type': u'STOPWORD',
    'original': u'تِهِ',
    'encletic': u''
}
STOPWORDS[u'وكته'] = {
    'word': u'وكته',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'وَكَتِهِ',
    'stem': u'تِهِ',
    'type': u'STOPWORD',
    'original': u'تِهِ',
    'encletic': u''
}
STOPWORDS[u'فكته'] = {
    'word': u'فكته',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'فَكَتِهِ',
    'stem': u'تِهِ',
    'type': u'STOPWORD',
    'original': u'تِهِ',
    'encletic': u''
}
STOPWORDS[u'ولته'] = {
    'word': u'ولته',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'وَلِتِهِ',
    'stem': u'تِهِ',
    'type': u'STOPWORD',
    'original': u'تِهِ',
    'encletic': u''
}
STOPWORDS[u'فلته'] = {
    'word': u'فلته',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'فَلِتِهِ',
    'stem': u'تِهِ',
    'type': u'STOPWORD',
    'original': u'تِهِ',
    'encletic': u''
}
STOPWORDS[u'تي'] = {
    'word': u'تِي',
    'procletic': u'',
    'tags': u'أداة:اسم إشارة',
    'vocalized': u'تِي',
    'stem': u'تِي',
    'type': u'STOPWORD',
    'original': u'تِي',
    'encletic': u''
}
STOPWORDS[u'بتي'] = {
    'word': u'بتي',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'بِتِي',
    'stem': u'تِي',
    'type': u'STOPWORD',
    'original': u'تِي',
    'encletic': u''
}
STOPWORDS[u'كتي'] = {
    'word': u'كتي',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'كَتِي',
    'stem': u'تِي',
    'type': u'STOPWORD',
    'original': u'تِي',
    'encletic': u''
}
STOPWORDS[u'لتي'] = {
    'word': u'لتي',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'لِتِي',
    'stem': u'تِي',
    'type': u'STOPWORD',
    'original': u'تِي',
    'encletic': u''
}
STOPWORDS[u'وتي'] = {
    'word': u'وتي',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إشارة:معطوف',
    'vocalized': u'وَتِي',
    'stem': u'تِي',
    'type': u'STOPWORD',
    'original': u'تِي',
    'encletic': u''
}
STOPWORDS[u'فتي'] = {
    'word': u'فتي',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إشارة:معطوف',
    'vocalized': u'فَتِي',
    'stem': u'تِي',
    'type': u'STOPWORD',
    'original': u'تِي',
    'encletic': u''
}
STOPWORDS[u'وبتي'] = {
    'word': u'وبتي',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'وَبِتِي',
    'stem': u'تِي',
    'type': u'STOPWORD',
    'original': u'تِي',
    'encletic': u''
}
STOPWORDS[u'فبتي'] = {
    'word': u'فبتي',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'فَبِتِي',
    'stem': u'تِي',
    'type': u'STOPWORD',
    'original': u'تِي',
    'encletic': u''
}
STOPWORDS[u'وكتي'] = {
    'word': u'وكتي',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'وَكَتِي',
    'stem': u'تِي',
    'type': u'STOPWORD',
    'original': u'تِي',
    'encletic': u''
}
STOPWORDS[u'فكتي'] = {
    'word': u'فكتي',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'فَكَتِي',
    'stem': u'تِي',
    'type': u'STOPWORD',
    'original': u'تِي',
    'encletic': u''
}
STOPWORDS[u'ولتي'] = {
    'word': u'ولتي',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'وَلِتِي',
    'stem': u'تِي',
    'type': u'STOPWORD',
    'original': u'تِي',
    'encletic': u''
}
STOPWORDS[u'فلتي'] = {
    'word': u'فلتي',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'فَلِتِي',
    'stem': u'تِي',
    'type': u'STOPWORD',
    'original': u'تِي',
    'encletic': u''
}
STOPWORDS[u'تين'] = {
    'word': u'تَيْنِ',
    'procletic': u'',
    'tags': u'أداة:اسم إشارة',
    'vocalized': u'تَيْنِ',
    'stem': u'تَيْنِ',
    'type': u'STOPWORD',
    'original': u'تَيْنِ',
    'encletic': u''
}
STOPWORDS[u'بتين'] = {
    'word': u'بتين',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'بِتَيْنِ',
    'stem': u'تَيْنِ',
    'type': u'STOPWORD',
    'original': u'تَيْنِ',
    'encletic': u''
}
STOPWORDS[u'كتين'] = {
    'word': u'كتين',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'كَتَيْنِ',
    'stem': u'تَيْنِ',
    'type': u'STOPWORD',
    'original': u'تَيْنِ',
    'encletic': u''
}
STOPWORDS[u'لتين'] = {
    'word': u'لتين',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'لِتَيْنِ',
    'stem': u'تَيْنِ',
    'type': u'STOPWORD',
    'original': u'تَيْنِ',
    'encletic': u''
}
STOPWORDS[u'وتين'] = {
    'word': u'وتين',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إشارة:معطوف',
    'vocalized': u'وَتَيْنِ',
    'stem': u'تَيْنِ',
    'type': u'STOPWORD',
    'original': u'تَيْنِ',
    'encletic': u''
}
STOPWORDS[u'فتين'] = {
    'word': u'فتين',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إشارة:معطوف',
    'vocalized': u'فَتَيْنِ',
    'stem': u'تَيْنِ',
    'type': u'STOPWORD',
    'original': u'تَيْنِ',
    'encletic': u''
}
STOPWORDS[u'وبتين'] = {
    'word': u'وبتين',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'وَبِتَيْنِ',
    'stem': u'تَيْنِ',
    'type': u'STOPWORD',
    'original': u'تَيْنِ',
    'encletic': u''
}
STOPWORDS[u'فبتين'] = {
    'word': u'فبتين',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'فَبِتَيْنِ',
    'stem': u'تَيْنِ',
    'type': u'STOPWORD',
    'original': u'تَيْنِ',
    'encletic': u''
}
STOPWORDS[u'وكتين'] = {
    'word': u'وكتين',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'وَكَتَيْنِ',
    'stem': u'تَيْنِ',
    'type': u'STOPWORD',
    'original': u'تَيْنِ',
    'encletic': u''
}
STOPWORDS[u'فكتين'] = {
    'word': u'فكتين',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'فَكَتَيْنِ',
    'stem': u'تَيْنِ',
    'type': u'STOPWORD',
    'original': u'تَيْنِ',
    'encletic': u''
}
STOPWORDS[u'ولتين'] = {
    'word': u'ولتين',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'وَلِتَيْنِ',
    'stem': u'تَيْنِ',
    'type': u'STOPWORD',
    'original': u'تَيْنِ',
    'encletic': u''
}
STOPWORDS[u'فلتين'] = {
    'word': u'فلتين',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'فَلِتَيْنِ',
    'stem': u'تَيْنِ',
    'type': u'STOPWORD',
    'original': u'تَيْنِ',
    'encletic': u''
}
STOPWORDS[u'تينك'] = {
    'word': u'تَيْنِكَ',
    'procletic': u'',
    'tags': u'أداة:اسم إشارة',
    'vocalized': u'تَيْنِكَ',
    'stem': u'تَيْنِكَ',
    'type': u'STOPWORD',
    'original': u'تَيْنِكَ',
    'encletic': u''
}
STOPWORDS[u'بتينك'] = {
    'word': u'بتينك',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'بِتَيْنِكَ',
    'stem': u'تَيْنِكَ',
    'type': u'STOPWORD',
    'original': u'تَيْنِكَ',
    'encletic': u''
}
STOPWORDS[u'كتينك'] = {
    'word': u'كتينك',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'كَتَيْنِكَ',
    'stem': u'تَيْنِكَ',
    'type': u'STOPWORD',
    'original': u'تَيْنِكَ',
    'encletic': u''
}
STOPWORDS[u'لتينك'] = {
    'word': u'لتينك',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'لِتَيْنِكَ',
    'stem': u'تَيْنِكَ',
    'type': u'STOPWORD',
    'original': u'تَيْنِكَ',
    'encletic': u''
}
STOPWORDS[u'وتينك'] = {
    'word': u'وتينك',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إشارة:معطوف',
    'vocalized': u'وَتَيْنِكَ',
    'stem': u'تَيْنِكَ',
    'type': u'STOPWORD',
    'original': u'تَيْنِكَ',
    'encletic': u''
}
STOPWORDS[u'فتينك'] = {
    'word': u'فتينك',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إشارة:معطوف',
    'vocalized': u'فَتَيْنِكَ',
    'stem': u'تَيْنِكَ',
    'type': u'STOPWORD',
    'original': u'تَيْنِكَ',
    'encletic': u''
}
STOPWORDS[u'وبتينك'] = {
    'word': u'وبتينك',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'وَبِتَيْنِكَ',
    'stem': u'تَيْنِكَ',
    'type': u'STOPWORD',
    'original': u'تَيْنِكَ',
    'encletic': u''
}
STOPWORDS[u'فبتينك'] = {
    'word': u'فبتينك',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'فَبِتَيْنِكَ',
    'stem': u'تَيْنِكَ',
    'type': u'STOPWORD',
    'original': u'تَيْنِكَ',
    'encletic': u''
}
STOPWORDS[u'وكتينك'] = {
    'word': u'وكتينك',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'وَكَتَيْنِكَ',
    'stem': u'تَيْنِكَ',
    'type': u'STOPWORD',
    'original': u'تَيْنِكَ',
    'encletic': u''
}
STOPWORDS[u'فكتينك'] = {
    'word': u'فكتينك',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'فَكَتَيْنِكَ',
    'stem': u'تَيْنِكَ',
    'type': u'STOPWORD',
    'original': u'تَيْنِكَ',
    'encletic': u''
}
STOPWORDS[u'ولتينك'] = {
    'word': u'ولتينك',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'وَلِتَيْنِكَ',
    'stem': u'تَيْنِكَ',
    'type': u'STOPWORD',
    'original': u'تَيْنِكَ',
    'encletic': u''
}
STOPWORDS[u'فلتينك'] = {
    'word': u'فلتينك',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'فَلِتَيْنِكَ',
    'stem': u'تَيْنِكَ',
    'type': u'STOPWORD',
    'original': u'تَيْنِكَ',
    'encletic': u''
}
STOPWORDS[u'أتينك'] = {
    'word': u'أتينك',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'أَتَيْنِكَ',
    'stem': u'تَيْنِكَ',
    'type': u'STOPWORD',
    'original': u'تَيْنِكَ',
    'encletic': u''
}
STOPWORDS[u'أبتينك'] = {
    'word': u'أبتينك',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:مجرور',
    'vocalized': u'أَبِتَيْنِكَ',
    'stem': u'تَيْنِكَ',
    'type': u'STOPWORD',
    'original': u'تَيْنِكَ',
    'encletic': u''
}
STOPWORDS[u'أكتينك'] = {
    'word': u'أكتينك',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:مجرور',
    'vocalized': u'أَكَتَيْنِكَ',
    'stem': u'تَيْنِكَ',
    'type': u'STOPWORD',
    'original': u'تَيْنِكَ',
    'encletic': u''
}
STOPWORDS[u'ألتينك'] = {
    'word': u'ألتينك',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:مجرور',
    'vocalized': u'أَلِتَيْنِكَ',
    'stem': u'تَيْنِكَ',
    'type': u'STOPWORD',
    'original': u'تَيْنِكَ',
    'encletic': u''
}
STOPWORDS[u'أوتينك'] = {
    'word': u'أوتينك',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إشارة:معطوف:مجرور',
    'vocalized': u'أَوَتَيْنِكَ',
    'stem': u'تَيْنِكَ',
    'type': u'STOPWORD',
    'original': u'تَيْنِكَ',
    'encletic': u''
}
STOPWORDS[u'أفتينك'] = {
    'word': u'أفتينك',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إشارة:معطوف:مجرور',
    'vocalized': u'أَفَتَيْنِكَ',
    'stem': u'تَيْنِكَ',
    'type': u'STOPWORD',
    'original': u'تَيْنِكَ',
    'encletic': u''
}
STOPWORDS[u'أوبتينك'] = {
    'word': u'أوبتينك',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَبِتَيْنِكَ',
    'stem': u'تَيْنِكَ',
    'type': u'STOPWORD',
    'original': u'تَيْنِكَ',
    'encletic': u''
}
STOPWORDS[u'أفبتينك'] = {
    'word': u'أفبتينك',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَبِتَيْنِكَ',
    'stem': u'تَيْنِكَ',
    'type': u'STOPWORD',
    'original': u'تَيْنِكَ',
    'encletic': u''
}
STOPWORDS[u'أوكتينك'] = {
    'word': u'أوكتينك',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَكَتَيْنِكَ',
    'stem': u'تَيْنِكَ',
    'type': u'STOPWORD',
    'original': u'تَيْنِكَ',
    'encletic': u''
}
STOPWORDS[u'أفكتينك'] = {
    'word': u'أفكتينك',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَكَتَيْنِكَ',
    'stem': u'تَيْنِكَ',
    'type': u'STOPWORD',
    'original': u'تَيْنِكَ',
    'encletic': u''
}
STOPWORDS[u'أولتينك'] = {
    'word': u'أولتينك',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَلِتَيْنِكَ',
    'stem': u'تَيْنِكَ',
    'type': u'STOPWORD',
    'original': u'تَيْنِكَ',
    'encletic': u''
}
STOPWORDS[u'أفلتينك'] = {
    'word': u'أفلتينك',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَلِتَيْنِكَ',
    'stem': u'تَيْنِكَ',
    'type': u'STOPWORD',
    'original': u'تَيْنِكَ',
    'encletic': u''
}
STOPWORDS[u'ثم'] = {
    'word': u'ثَمَّ',
    'procletic': u'',
    'tags': u'أداة:اسم إشارة',
    'vocalized': u'ثَمَّ',
    'stem': u'ثَمَّ',
    'type': u'STOPWORD',
    'original': u'ثَمَّ',
    'encletic': u''
}
STOPWORDS[u'وثم'] = {
    'word': u'وثم',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إشارة:معطوف',
    'vocalized': u'وَثَمَّ',
    'stem': u'ثَمَّ',
    'type': u'STOPWORD',
    'original': u'ثَمَّ',
    'encletic': u''
}
STOPWORDS[u'فثم'] = {
    'word': u'فثم',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إشارة:معطوف',
    'vocalized': u'فَثَمَّ',
    'stem': u'ثَمَّ',
    'type': u'STOPWORD',
    'original': u'ثَمَّ',
    'encletic': u''
}
STOPWORDS[u'أثم'] = {
    'word': u'أثم',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'أَثَمَّ',
    'stem': u'ثَمَّ',
    'type': u'STOPWORD',
    'original': u'ثَمَّ',
    'encletic': u''
}
STOPWORDS[u'أوثم'] = {
    'word': u'أوثم',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إشارة:معطوف:مجرور',
    'vocalized': u'أَوَثَمَّ',
    'stem': u'ثَمَّ',
    'type': u'STOPWORD',
    'original': u'ثَمَّ',
    'encletic': u''
}
STOPWORDS[u'أفثم'] = {
    'word': u'أفثم',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إشارة:معطوف:مجرور',
    'vocalized': u'أَفَثَمَّ',
    'stem': u'ثَمَّ',
    'type': u'STOPWORD',
    'original': u'ثَمَّ',
    'encletic': u''
}
STOPWORDS[u'ثمة'] = {
    'word': u'ثَمَّةَ',
    'procletic': u'',
    'tags': u'أداة:اسم إشارة',
    'vocalized': u'ثَمَّةَ',
    'stem': u'ثَمَّةَ',
    'type': u'STOPWORD',
    'original': u'ثَمَّةَ',
    'encletic': u''
}
STOPWORDS[u'وثمة'] = {
    'word': u'وثمة',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إشارة:معطوف',
    'vocalized': u'وَثَمَّةَ',
    'stem': u'ثَمَّةَ',
    'type': u'STOPWORD',
    'original': u'ثَمَّةَ',
    'encletic': u''
}
STOPWORDS[u'فثمة'] = {
    'word': u'فثمة',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إشارة:معطوف',
    'vocalized': u'فَثَمَّةَ',
    'stem': u'ثَمَّةَ',
    'type': u'STOPWORD',
    'original': u'ثَمَّةَ',
    'encletic': u''
}
STOPWORDS[u'أثمة'] = {
    'word': u'أثمة',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'أَثَمَّةَ',
    'stem': u'ثَمَّةَ',
    'type': u'STOPWORD',
    'original': u'ثَمَّةَ',
    'encletic': u''
}
STOPWORDS[u'أوثمة'] = {
    'word': u'أوثمة',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إشارة:معطوف:مجرور',
    'vocalized': u'أَوَثَمَّةَ',
    'stem': u'ثَمَّةَ',
    'type': u'STOPWORD',
    'original': u'ثَمَّةَ',
    'encletic': u''
}
STOPWORDS[u'أفثمة'] = {
    'word': u'أفثمة',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إشارة:معطوف:مجرور',
    'vocalized': u'أَفَثَمَّةَ',
    'stem': u'ثَمَّةَ',
    'type': u'STOPWORD',
    'original': u'ثَمَّةَ',
    'encletic': u''
}
STOPWORDS[u'ذا'] = {
    'word': u'ذَا',
    'procletic': u'',
    'tags': u'أداة:اسم إشارة',
    'vocalized': u'ذَا',
    'stem': u'ذَا',
    'type': u'STOPWORD',
    'original': u'ذَا',
    'encletic': u''
}
STOPWORDS[u'بذا'] = {
    'word': u'بذا',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'بِذَا',
    'stem': u'ذَا',
    'type': u'STOPWORD',
    'original': u'ذَا',
    'encletic': u''
}
STOPWORDS[u'كذا'] = {
    'word': u'كذا',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'كَذَا',
    'stem': u'ذَا',
    'type': u'STOPWORD',
    'original': u'ذَا',
    'encletic': u''
}
STOPWORDS[u'لذا'] = {
    'word': u'لذا',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'لِذَا',
    'stem': u'ذَا',
    'type': u'STOPWORD',
    'original': u'ذَا',
    'encletic': u''
}
STOPWORDS[u'وذا'] = {
    'word': u'وذا',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إشارة:معطوف',
    'vocalized': u'وَذَا',
    'stem': u'ذَا',
    'type': u'STOPWORD',
    'original': u'ذَا',
    'encletic': u''
}
STOPWORDS[u'فذا'] = {
    'word': u'فذا',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إشارة:معطوف',
    'vocalized': u'فَذَا',
    'stem': u'ذَا',
    'type': u'STOPWORD',
    'original': u'ذَا',
    'encletic': u''
}
STOPWORDS[u'وبذا'] = {
    'word': u'وبذا',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'وَبِذَا',
    'stem': u'ذَا',
    'type': u'STOPWORD',
    'original': u'ذَا',
    'encletic': u''
}
STOPWORDS[u'فبذا'] = {
    'word': u'فبذا',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'فَبِذَا',
    'stem': u'ذَا',
    'type': u'STOPWORD',
    'original': u'ذَا',
    'encletic': u''
}
STOPWORDS[u'وكذا'] = {
    'word': u'وكذا',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'وَكَذَا',
    'stem': u'ذَا',
    'type': u'STOPWORD',
    'original': u'ذَا',
    'encletic': u''
}
STOPWORDS[u'فكذا'] = {
    'word': u'فكذا',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'فَكَذَا',
    'stem': u'ذَا',
    'type': u'STOPWORD',
    'original': u'ذَا',
    'encletic': u''
}
STOPWORDS[u'ولذا'] = {
    'word': u'ولذا',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'وَلِذَا',
    'stem': u'ذَا',
    'type': u'STOPWORD',
    'original': u'ذَا',
    'encletic': u''
}
STOPWORDS[u'فلذا'] = {
    'word': u'فلذا',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'فَلِذَا',
    'stem': u'ذَا',
    'type': u'STOPWORD',
    'original': u'ذَا',
    'encletic': u''
}
STOPWORDS[u'ذاك'] = {
    'word': u'ذَاكَ',
    'procletic': u'',
    'tags': u'أداة:اسم إشارة',
    'vocalized': u'ذَاكَ',
    'stem': u'ذَاكَ',
    'type': u'STOPWORD',
    'original': u'ذَاكَ',
    'encletic': u''
}
STOPWORDS[u'بذاك'] = {
    'word': u'بذاك',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'بِذَاكَ',
    'stem': u'ذَاكَ',
    'type': u'STOPWORD',
    'original': u'ذَاكَ',
    'encletic': u''
}
STOPWORDS[u'كذاك'] = {
    'word': u'كذاك',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'كَذَاكَ',
    'stem': u'ذَاكَ',
    'type': u'STOPWORD',
    'original': u'ذَاكَ',
    'encletic': u''
}
STOPWORDS[u'لذاك'] = {
    'word': u'لذاك',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'لِذَاكَ',
    'stem': u'ذَاكَ',
    'type': u'STOPWORD',
    'original': u'ذَاكَ',
    'encletic': u''
}
STOPWORDS[u'وذاك'] = {
    'word': u'وذاك',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إشارة:معطوف',
    'vocalized': u'وَذَاكَ',
    'stem': u'ذَاكَ',
    'type': u'STOPWORD',
    'original': u'ذَاكَ',
    'encletic': u''
}
STOPWORDS[u'فذاك'] = {
    'word': u'فذاك',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إشارة:معطوف',
    'vocalized': u'فَذَاكَ',
    'stem': u'ذَاكَ',
    'type': u'STOPWORD',
    'original': u'ذَاكَ',
    'encletic': u''
}
STOPWORDS[u'وبذاك'] = {
    'word': u'وبذاك',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'وَبِذَاكَ',
    'stem': u'ذَاكَ',
    'type': u'STOPWORD',
    'original': u'ذَاكَ',
    'encletic': u''
}
STOPWORDS[u'فبذاك'] = {
    'word': u'فبذاك',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'فَبِذَاكَ',
    'stem': u'ذَاكَ',
    'type': u'STOPWORD',
    'original': u'ذَاكَ',
    'encletic': u''
}
STOPWORDS[u'وكذاك'] = {
    'word': u'وكذاك',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'وَكَذَاكَ',
    'stem': u'ذَاكَ',
    'type': u'STOPWORD',
    'original': u'ذَاكَ',
    'encletic': u''
}
STOPWORDS[u'فكذاك'] = {
    'word': u'فكذاك',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'فَكَذَاكَ',
    'stem': u'ذَاكَ',
    'type': u'STOPWORD',
    'original': u'ذَاكَ',
    'encletic': u''
}
STOPWORDS[u'ولذاك'] = {
    'word': u'ولذاك',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'وَلِذَاكَ',
    'stem': u'ذَاكَ',
    'type': u'STOPWORD',
    'original': u'ذَاكَ',
    'encletic': u''
}
STOPWORDS[u'فلذاك'] = {
    'word': u'فلذاك',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'فَلِذَاكَ',
    'stem': u'ذَاكَ',
    'type': u'STOPWORD',
    'original': u'ذَاكَ',
    'encletic': u''
}
STOPWORDS[u'أذاك'] = {
    'word': u'أذاك',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'أَذَاكَ',
    'stem': u'ذَاكَ',
    'type': u'STOPWORD',
    'original': u'ذَاكَ',
    'encletic': u''
}
STOPWORDS[u'أبذاك'] = {
    'word': u'أبذاك',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:مجرور',
    'vocalized': u'أَبِذَاكَ',
    'stem': u'ذَاكَ',
    'type': u'STOPWORD',
    'original': u'ذَاكَ',
    'encletic': u''
}
STOPWORDS[u'أكذاك'] = {
    'word': u'أكذاك',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:مجرور',
    'vocalized': u'أَكَذَاكَ',
    'stem': u'ذَاكَ',
    'type': u'STOPWORD',
    'original': u'ذَاكَ',
    'encletic': u''
}
STOPWORDS[u'ألذاك'] = {
    'word': u'ألذاك',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:مجرور',
    'vocalized': u'أَلِذَاكَ',
    'stem': u'ذَاكَ',
    'type': u'STOPWORD',
    'original': u'ذَاكَ',
    'encletic': u''
}
STOPWORDS[u'أوذاك'] = {
    'word': u'أوذاك',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إشارة:معطوف:مجرور',
    'vocalized': u'أَوَذَاكَ',
    'stem': u'ذَاكَ',
    'type': u'STOPWORD',
    'original': u'ذَاكَ',
    'encletic': u''
}
STOPWORDS[u'أفذاك'] = {
    'word': u'أفذاك',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إشارة:معطوف:مجرور',
    'vocalized': u'أَفَذَاكَ',
    'stem': u'ذَاكَ',
    'type': u'STOPWORD',
    'original': u'ذَاكَ',
    'encletic': u''
}
STOPWORDS[u'أوبذاك'] = {
    'word': u'أوبذاك',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَبِذَاكَ',
    'stem': u'ذَاكَ',
    'type': u'STOPWORD',
    'original': u'ذَاكَ',
    'encletic': u''
}
STOPWORDS[u'أفبذاك'] = {
    'word': u'أفبذاك',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَبِذَاكَ',
    'stem': u'ذَاكَ',
    'type': u'STOPWORD',
    'original': u'ذَاكَ',
    'encletic': u''
}
STOPWORDS[u'أوكذاك'] = {
    'word': u'أوكذاك',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَكَذَاكَ',
    'stem': u'ذَاكَ',
    'type': u'STOPWORD',
    'original': u'ذَاكَ',
    'encletic': u''
}
STOPWORDS[u'أفكذاك'] = {
    'word': u'أفكذاك',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَكَذَاكَ',
    'stem': u'ذَاكَ',
    'type': u'STOPWORD',
    'original': u'ذَاكَ',
    'encletic': u''
}
STOPWORDS[u'أولذاك'] = {
    'word': u'أولذاك',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَلِذَاكَ',
    'stem': u'ذَاكَ',
    'type': u'STOPWORD',
    'original': u'ذَاكَ',
    'encletic': u''
}
STOPWORDS[u'أفلذاك'] = {
    'word': u'أفلذاك',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَلِذَاكَ',
    'stem': u'ذَاكَ',
    'type': u'STOPWORD',
    'original': u'ذَاكَ',
    'encletic': u''
}
STOPWORDS[u'ذان'] = {
    'word': u'ذَانِ',
    'procletic': u'',
    'tags': u'أداة:اسم إشارة',
    'vocalized': u'ذَانِ',
    'stem': u'ذَانِ',
    'type': u'STOPWORD',
    'original': u'ذَانِ',
    'encletic': u''
}
STOPWORDS[u'وذان'] = {
    'word': u'وذان',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إشارة:معطوف',
    'vocalized': u'وَذَانِ',
    'stem': u'ذَانِ',
    'type': u'STOPWORD',
    'original': u'ذَانِ',
    'encletic': u''
}
STOPWORDS[u'فذان'] = {
    'word': u'فذان',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إشارة:معطوف',
    'vocalized': u'فَذَانِ',
    'stem': u'ذَانِ',
    'type': u'STOPWORD',
    'original': u'ذَانِ',
    'encletic': u''
}
STOPWORDS[u'ذانك'] = {
    'word': u'ذَانِكَ',
    'procletic': u'',
    'tags': u'أداة:اسم إشارة',
    'vocalized': u'ذَانِكَ',
    'stem': u'ذَانِكَ',
    'type': u'STOPWORD',
    'original': u'ذَانِكَ',
    'encletic': u''
}
STOPWORDS[u'بذانك'] = {
    'word': u'بذانك',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'بِذَانِكَ',
    'stem': u'ذَانِكَ',
    'type': u'STOPWORD',
    'original': u'ذَانِكَ',
    'encletic': u''
}
STOPWORDS[u'كذانك'] = {
    'word': u'كذانك',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'كَذَانِكَ',
    'stem': u'ذَانِكَ',
    'type': u'STOPWORD',
    'original': u'ذَانِكَ',
    'encletic': u''
}
STOPWORDS[u'لذانك'] = {
    'word': u'لذانك',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'لِذَانِكَ',
    'stem': u'ذَانِكَ',
    'type': u'STOPWORD',
    'original': u'ذَانِكَ',
    'encletic': u''
}
STOPWORDS[u'وذانك'] = {
    'word': u'وذانك',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إشارة:معطوف',
    'vocalized': u'وَذَانِكَ',
    'stem': u'ذَانِكَ',
    'type': u'STOPWORD',
    'original': u'ذَانِكَ',
    'encletic': u''
}
STOPWORDS[u'فذانك'] = {
    'word': u'فذانك',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إشارة:معطوف',
    'vocalized': u'فَذَانِكَ',
    'stem': u'ذَانِكَ',
    'type': u'STOPWORD',
    'original': u'ذَانِكَ',
    'encletic': u''
}
STOPWORDS[u'وبذانك'] = {
    'word': u'وبذانك',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'وَبِذَانِكَ',
    'stem': u'ذَانِكَ',
    'type': u'STOPWORD',
    'original': u'ذَانِكَ',
    'encletic': u''
}
STOPWORDS[u'فبذانك'] = {
    'word': u'فبذانك',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'فَبِذَانِكَ',
    'stem': u'ذَانِكَ',
    'type': u'STOPWORD',
    'original': u'ذَانِكَ',
    'encletic': u''
}
STOPWORDS[u'وكذانك'] = {
    'word': u'وكذانك',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'وَكَذَانِكَ',
    'stem': u'ذَانِكَ',
    'type': u'STOPWORD',
    'original': u'ذَانِكَ',
    'encletic': u''
}
STOPWORDS[u'فكذانك'] = {
    'word': u'فكذانك',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'فَكَذَانِكَ',
    'stem': u'ذَانِكَ',
    'type': u'STOPWORD',
    'original': u'ذَانِكَ',
    'encletic': u''
}
STOPWORDS[u'ولذانك'] = {
    'word': u'ولذانك',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'وَلِذَانِكَ',
    'stem': u'ذَانِكَ',
    'type': u'STOPWORD',
    'original': u'ذَانِكَ',
    'encletic': u''
}
STOPWORDS[u'فلذانك'] = {
    'word': u'فلذانك',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'فَلِذَانِكَ',
    'stem': u'ذَانِكَ',
    'type': u'STOPWORD',
    'original': u'ذَانِكَ',
    'encletic': u''
}
STOPWORDS[u'أذانك'] = {
    'word': u'أذانك',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'أَذَانِكَ',
    'stem': u'ذَانِكَ',
    'type': u'STOPWORD',
    'original': u'ذَانِكَ',
    'encletic': u''
}
STOPWORDS[u'أبذانك'] = {
    'word': u'أبذانك',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:مجرور',
    'vocalized': u'أَبِذَانِكَ',
    'stem': u'ذَانِكَ',
    'type': u'STOPWORD',
    'original': u'ذَانِكَ',
    'encletic': u''
}
STOPWORDS[u'أكذانك'] = {
    'word': u'أكذانك',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:مجرور',
    'vocalized': u'أَكَذَانِكَ',
    'stem': u'ذَانِكَ',
    'type': u'STOPWORD',
    'original': u'ذَانِكَ',
    'encletic': u''
}
STOPWORDS[u'ألذانك'] = {
    'word': u'ألذانك',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:مجرور',
    'vocalized': u'أَلِذَانِكَ',
    'stem': u'ذَانِكَ',
    'type': u'STOPWORD',
    'original': u'ذَانِكَ',
    'encletic': u''
}
STOPWORDS[u'أوذانك'] = {
    'word': u'أوذانك',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إشارة:معطوف:مجرور',
    'vocalized': u'أَوَذَانِكَ',
    'stem': u'ذَانِكَ',
    'type': u'STOPWORD',
    'original': u'ذَانِكَ',
    'encletic': u''
}
STOPWORDS[u'أفذانك'] = {
    'word': u'أفذانك',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إشارة:معطوف:مجرور',
    'vocalized': u'أَفَذَانِكَ',
    'stem': u'ذَانِكَ',
    'type': u'STOPWORD',
    'original': u'ذَانِكَ',
    'encletic': u''
}
STOPWORDS[u'أوبذانك'] = {
    'word': u'أوبذانك',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَبِذَانِكَ',
    'stem': u'ذَانِكَ',
    'type': u'STOPWORD',
    'original': u'ذَانِكَ',
    'encletic': u''
}
STOPWORDS[u'أفبذانك'] = {
    'word': u'أفبذانك',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَبِذَانِكَ',
    'stem': u'ذَانِكَ',
    'type': u'STOPWORD',
    'original': u'ذَانِكَ',
    'encletic': u''
}
STOPWORDS[u'أوكذانك'] = {
    'word': u'أوكذانك',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَكَذَانِكَ',
    'stem': u'ذَانِكَ',
    'type': u'STOPWORD',
    'original': u'ذَانِكَ',
    'encletic': u''
}
STOPWORDS[u'أفكذانك'] = {
    'word': u'أفكذانك',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَكَذَانِكَ',
    'stem': u'ذَانِكَ',
    'type': u'STOPWORD',
    'original': u'ذَانِكَ',
    'encletic': u''
}
STOPWORDS[u'أولذانك'] = {
    'word': u'أولذانك',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَلِذَانِكَ',
    'stem': u'ذَانِكَ',
    'type': u'STOPWORD',
    'original': u'ذَانِكَ',
    'encletic': u''
}
STOPWORDS[u'أفلذانك'] = {
    'word': u'أفلذانك',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَلِذَانِكَ',
    'stem': u'ذَانِكَ',
    'type': u'STOPWORD',
    'original': u'ذَانِكَ',
    'encletic': u''
}
STOPWORDS[u'ذلك'] = {
    'word': u'ذَلِكَ',
    'procletic': u'',
    'tags': u'أداة:اسم إشارة',
    'vocalized': u'ذَلِكَ',
    'stem': u'ذَلِكَ',
    'type': u'STOPWORD',
    'original': u'ذَلِكَ',
    'encletic': u''
}
STOPWORDS[u'بذلك'] = {
    'word': u'بذلك',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'بِذَلِكَ',
    'stem': u'ذَلِكَ',
    'type': u'STOPWORD',
    'original': u'ذَلِكَ',
    'encletic': u''
}
STOPWORDS[u'كذلك'] = {
    'word': u'كذلك',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'كَذَلِكَ',
    'stem': u'ذَلِكَ',
    'type': u'STOPWORD',
    'original': u'ذَلِكَ',
    'encletic': u''
}
STOPWORDS[u'لذلك'] = {
    'word': u'لذلك',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'لِذَلِكَ',
    'stem': u'ذَلِكَ',
    'type': u'STOPWORD',
    'original': u'ذَلِكَ',
    'encletic': u''
}
STOPWORDS[u'وذلك'] = {
    'word': u'وذلك',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إشارة:معطوف',
    'vocalized': u'وَذَلِكَ',
    'stem': u'ذَلِكَ',
    'type': u'STOPWORD',
    'original': u'ذَلِكَ',
    'encletic': u''
}
STOPWORDS[u'فذلك'] = {
    'word': u'فذلك',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إشارة:معطوف',
    'vocalized': u'فَذَلِكَ',
    'stem': u'ذَلِكَ',
    'type': u'STOPWORD',
    'original': u'ذَلِكَ',
    'encletic': u''
}
STOPWORDS[u'وبذلك'] = {
    'word': u'وبذلك',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'وَبِذَلِكَ',
    'stem': u'ذَلِكَ',
    'type': u'STOPWORD',
    'original': u'ذَلِكَ',
    'encletic': u''
}
STOPWORDS[u'فبذلك'] = {
    'word': u'فبذلك',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'فَبِذَلِكَ',
    'stem': u'ذَلِكَ',
    'type': u'STOPWORD',
    'original': u'ذَلِكَ',
    'encletic': u''
}
STOPWORDS[u'وكذلك'] = {
    'word': u'وكذلك',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'وَكَذَلِكَ',
    'stem': u'ذَلِكَ',
    'type': u'STOPWORD',
    'original': u'ذَلِكَ',
    'encletic': u''
}
STOPWORDS[u'فكذلك'] = {
    'word': u'فكذلك',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'فَكَذَلِكَ',
    'stem': u'ذَلِكَ',
    'type': u'STOPWORD',
    'original': u'ذَلِكَ',
    'encletic': u''
}
STOPWORDS[u'ولذلك'] = {
    'word': u'ولذلك',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'وَلِذَلِكَ',
    'stem': u'ذَلِكَ',
    'type': u'STOPWORD',
    'original': u'ذَلِكَ',
    'encletic': u''
}
STOPWORDS[u'فلذلك'] = {
    'word': u'فلذلك',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'فَلِذَلِكَ',
    'stem': u'ذَلِكَ',
    'type': u'STOPWORD',
    'original': u'ذَلِكَ',
    'encletic': u''
}
STOPWORDS[u'أذلك'] = {
    'word': u'أذلك',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'أَذَلِكَ',
    'stem': u'ذَلِكَ',
    'type': u'STOPWORD',
    'original': u'ذَلِكَ',
    'encletic': u''
}
STOPWORDS[u'أبذلك'] = {
    'word': u'أبذلك',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:مجرور',
    'vocalized': u'أَبِذَلِكَ',
    'stem': u'ذَلِكَ',
    'type': u'STOPWORD',
    'original': u'ذَلِكَ',
    'encletic': u''
}
STOPWORDS[u'أكذلك'] = {
    'word': u'أكذلك',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:مجرور',
    'vocalized': u'أَكَذَلِكَ',
    'stem': u'ذَلِكَ',
    'type': u'STOPWORD',
    'original': u'ذَلِكَ',
    'encletic': u''
}
STOPWORDS[u'ألذلك'] = {
    'word': u'ألذلك',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:مجرور',
    'vocalized': u'أَلِذَلِكَ',
    'stem': u'ذَلِكَ',
    'type': u'STOPWORD',
    'original': u'ذَلِكَ',
    'encletic': u''
}
STOPWORDS[u'أوذلك'] = {
    'word': u'أوذلك',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إشارة:معطوف:مجرور',
    'vocalized': u'أَوَذَلِكَ',
    'stem': u'ذَلِكَ',
    'type': u'STOPWORD',
    'original': u'ذَلِكَ',
    'encletic': u''
}
STOPWORDS[u'أفذلك'] = {
    'word': u'أفذلك',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إشارة:معطوف:مجرور',
    'vocalized': u'أَفَذَلِكَ',
    'stem': u'ذَلِكَ',
    'type': u'STOPWORD',
    'original': u'ذَلِكَ',
    'encletic': u''
}
STOPWORDS[u'أوبذلك'] = {
    'word': u'أوبذلك',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَبِذَلِكَ',
    'stem': u'ذَلِكَ',
    'type': u'STOPWORD',
    'original': u'ذَلِكَ',
    'encletic': u''
}
STOPWORDS[u'أفبذلك'] = {
    'word': u'أفبذلك',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَبِذَلِكَ',
    'stem': u'ذَلِكَ',
    'type': u'STOPWORD',
    'original': u'ذَلِكَ',
    'encletic': u''
}
STOPWORDS[u'أوكذلك'] = {
    'word': u'أوكذلك',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَكَذَلِكَ',
    'stem': u'ذَلِكَ',
    'type': u'STOPWORD',
    'original': u'ذَلِكَ',
    'encletic': u''
}
STOPWORDS[u'أفكذلك'] = {
    'word': u'أفكذلك',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَكَذَلِكَ',
    'stem': u'ذَلِكَ',
    'type': u'STOPWORD',
    'original': u'ذَلِكَ',
    'encletic': u''
}
STOPWORDS[u'أولذلك'] = {
    'word': u'أولذلك',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَلِذَلِكَ',
    'stem': u'ذَلِكَ',
    'type': u'STOPWORD',
    'original': u'ذَلِكَ',
    'encletic': u''
}
STOPWORDS[u'أفلذلك'] = {
    'word': u'أفلذلك',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَلِذَلِكَ',
    'stem': u'ذَلِكَ',
    'type': u'STOPWORD',
    'original': u'ذَلِكَ',
    'encletic': u''
}
STOPWORDS[u'ذلكم'] = {
    'word': u'ذَلِكُمْ',
    'procletic': u'',
    'tags': u'أداة:اسم إشارة',
    'vocalized': u'ذَلِكُمْ',
    'stem': u'ذَلِكُمْ',
    'type': u'STOPWORD',
    'original': u'ذَلِكُمْ',
    'encletic': u''
}
STOPWORDS[u'بذلكم'] = {
    'word': u'بذلكم',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'بِذَلِكُمْ',
    'stem': u'ذَلِكُمْ',
    'type': u'STOPWORD',
    'original': u'ذَلِكُمْ',
    'encletic': u''
}
STOPWORDS[u'كذلكم'] = {
    'word': u'كذلكم',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'كَذَلِكُمْ',
    'stem': u'ذَلِكُمْ',
    'type': u'STOPWORD',
    'original': u'ذَلِكُمْ',
    'encletic': u''
}
STOPWORDS[u'لذلكم'] = {
    'word': u'لذلكم',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'لِذَلِكُمْ',
    'stem': u'ذَلِكُمْ',
    'type': u'STOPWORD',
    'original': u'ذَلِكُمْ',
    'encletic': u''
}
STOPWORDS[u'وذلكم'] = {
    'word': u'وذلكم',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إشارة:معطوف',
    'vocalized': u'وَذَلِكُمْ',
    'stem': u'ذَلِكُمْ',
    'type': u'STOPWORD',
    'original': u'ذَلِكُمْ',
    'encletic': u''
}
STOPWORDS[u'فذلكم'] = {
    'word': u'فذلكم',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إشارة:معطوف',
    'vocalized': u'فَذَلِكُمْ',
    'stem': u'ذَلِكُمْ',
    'type': u'STOPWORD',
    'original': u'ذَلِكُمْ',
    'encletic': u''
}
STOPWORDS[u'وبذلكم'] = {
    'word': u'وبذلكم',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'وَبِذَلِكُمْ',
    'stem': u'ذَلِكُمْ',
    'type': u'STOPWORD',
    'original': u'ذَلِكُمْ',
    'encletic': u''
}
STOPWORDS[u'فبذلكم'] = {
    'word': u'فبذلكم',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'فَبِذَلِكُمْ',
    'stem': u'ذَلِكُمْ',
    'type': u'STOPWORD',
    'original': u'ذَلِكُمْ',
    'encletic': u''
}
STOPWORDS[u'وكذلكم'] = {
    'word': u'وكذلكم',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'وَكَذَلِكُمْ',
    'stem': u'ذَلِكُمْ',
    'type': u'STOPWORD',
    'original': u'ذَلِكُمْ',
    'encletic': u''
}
STOPWORDS[u'فكذلكم'] = {
    'word': u'فكذلكم',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'فَكَذَلِكُمْ',
    'stem': u'ذَلِكُمْ',
    'type': u'STOPWORD',
    'original': u'ذَلِكُمْ',
    'encletic': u''
}
STOPWORDS[u'ولذلكم'] = {
    'word': u'ولذلكم',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'وَلِذَلِكُمْ',
    'stem': u'ذَلِكُمْ',
    'type': u'STOPWORD',
    'original': u'ذَلِكُمْ',
    'encletic': u''
}
STOPWORDS[u'فلذلكم'] = {
    'word': u'فلذلكم',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'فَلِذَلِكُمْ',
    'stem': u'ذَلِكُمْ',
    'type': u'STOPWORD',
    'original': u'ذَلِكُمْ',
    'encletic': u''
}
STOPWORDS[u'أذلكم'] = {
    'word': u'أذلكم',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'أَذَلِكُمْ',
    'stem': u'ذَلِكُمْ',
    'type': u'STOPWORD',
    'original': u'ذَلِكُمْ',
    'encletic': u''
}
STOPWORDS[u'أبذلكم'] = {
    'word': u'أبذلكم',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:مجرور',
    'vocalized': u'أَبِذَلِكُمْ',
    'stem': u'ذَلِكُمْ',
    'type': u'STOPWORD',
    'original': u'ذَلِكُمْ',
    'encletic': u''
}
STOPWORDS[u'أكذلكم'] = {
    'word': u'أكذلكم',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:مجرور',
    'vocalized': u'أَكَذَلِكُمْ',
    'stem': u'ذَلِكُمْ',
    'type': u'STOPWORD',
    'original': u'ذَلِكُمْ',
    'encletic': u''
}
STOPWORDS[u'ألذلكم'] = {
    'word': u'ألذلكم',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:مجرور',
    'vocalized': u'أَلِذَلِكُمْ',
    'stem': u'ذَلِكُمْ',
    'type': u'STOPWORD',
    'original': u'ذَلِكُمْ',
    'encletic': u''
}
STOPWORDS[u'أوذلكم'] = {
    'word': u'أوذلكم',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إشارة:معطوف:مجرور',
    'vocalized': u'أَوَذَلِكُمْ',
    'stem': u'ذَلِكُمْ',
    'type': u'STOPWORD',
    'original': u'ذَلِكُمْ',
    'encletic': u''
}
STOPWORDS[u'أفذلكم'] = {
    'word': u'أفذلكم',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إشارة:معطوف:مجرور',
    'vocalized': u'أَفَذَلِكُمْ',
    'stem': u'ذَلِكُمْ',
    'type': u'STOPWORD',
    'original': u'ذَلِكُمْ',
    'encletic': u''
}
STOPWORDS[u'أوبذلكم'] = {
    'word': u'أوبذلكم',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَبِذَلِكُمْ',
    'stem': u'ذَلِكُمْ',
    'type': u'STOPWORD',
    'original': u'ذَلِكُمْ',
    'encletic': u''
}
STOPWORDS[u'أفبذلكم'] = {
    'word': u'أفبذلكم',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَبِذَلِكُمْ',
    'stem': u'ذَلِكُمْ',
    'type': u'STOPWORD',
    'original': u'ذَلِكُمْ',
    'encletic': u''
}
STOPWORDS[u'أوكذلكم'] = {
    'word': u'أوكذلكم',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَكَذَلِكُمْ',
    'stem': u'ذَلِكُمْ',
    'type': u'STOPWORD',
    'original': u'ذَلِكُمْ',
    'encletic': u''
}
STOPWORDS[u'أفكذلكم'] = {
    'word': u'أفكذلكم',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَكَذَلِكُمْ',
    'stem': u'ذَلِكُمْ',
    'type': u'STOPWORD',
    'original': u'ذَلِكُمْ',
    'encletic': u''
}
STOPWORDS[u'أولذلكم'] = {
    'word': u'أولذلكم',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَلِذَلِكُمْ',
    'stem': u'ذَلِكُمْ',
    'type': u'STOPWORD',
    'original': u'ذَلِكُمْ',
    'encletic': u''
}
STOPWORDS[u'أفلذلكم'] = {
    'word': u'أفلذلكم',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَلِذَلِكُمْ',
    'stem': u'ذَلِكُمْ',
    'type': u'STOPWORD',
    'original': u'ذَلِكُمْ',
    'encletic': u''
}
STOPWORDS[u'ذلكما'] = {
    'word': u'ذَلَكُمَا',
    'procletic': u'',
    'tags': u'أداة:اسم إشارة',
    'vocalized': u'ذَلَكُمَا',
    'stem': u'ذَلَكُمَا',
    'type': u'STOPWORD',
    'original': u'ذَلَكُمَا',
    'encletic': u''
}
STOPWORDS[u'بذلكما'] = {
    'word': u'بذلكما',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'بِذَلَكُمَا',
    'stem': u'ذَلَكُمَا',
    'type': u'STOPWORD',
    'original': u'ذَلَكُمَا',
    'encletic': u''
}
STOPWORDS[u'كذلكما'] = {
    'word': u'كذلكما',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'كَذَلَكُمَا',
    'stem': u'ذَلَكُمَا',
    'type': u'STOPWORD',
    'original': u'ذَلَكُمَا',
    'encletic': u''
}
STOPWORDS[u'لذلكما'] = {
    'word': u'لذلكما',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'لِذَلَكُمَا',
    'stem': u'ذَلَكُمَا',
    'type': u'STOPWORD',
    'original': u'ذَلَكُمَا',
    'encletic': u''
}
STOPWORDS[u'وذلكما'] = {
    'word': u'وذلكما',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إشارة:معطوف',
    'vocalized': u'وَذَلَكُمَا',
    'stem': u'ذَلَكُمَا',
    'type': u'STOPWORD',
    'original': u'ذَلَكُمَا',
    'encletic': u''
}
STOPWORDS[u'فذلكما'] = {
    'word': u'فذلكما',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إشارة:معطوف',
    'vocalized': u'فَذَلَكُمَا',
    'stem': u'ذَلَكُمَا',
    'type': u'STOPWORD',
    'original': u'ذَلَكُمَا',
    'encletic': u''
}
STOPWORDS[u'وبذلكما'] = {
    'word': u'وبذلكما',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'وَبِذَلَكُمَا',
    'stem': u'ذَلَكُمَا',
    'type': u'STOPWORD',
    'original': u'ذَلَكُمَا',
    'encletic': u''
}
STOPWORDS[u'فبذلكما'] = {
    'word': u'فبذلكما',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'فَبِذَلَكُمَا',
    'stem': u'ذَلَكُمَا',
    'type': u'STOPWORD',
    'original': u'ذَلَكُمَا',
    'encletic': u''
}
STOPWORDS[u'وكذلكما'] = {
    'word': u'وكذلكما',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'وَكَذَلَكُمَا',
    'stem': u'ذَلَكُمَا',
    'type': u'STOPWORD',
    'original': u'ذَلَكُمَا',
    'encletic': u''
}
STOPWORDS[u'فكذلكما'] = {
    'word': u'فكذلكما',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'فَكَذَلَكُمَا',
    'stem': u'ذَلَكُمَا',
    'type': u'STOPWORD',
    'original': u'ذَلَكُمَا',
    'encletic': u''
}
STOPWORDS[u'ولذلكما'] = {
    'word': u'ولذلكما',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'وَلِذَلَكُمَا',
    'stem': u'ذَلَكُمَا',
    'type': u'STOPWORD',
    'original': u'ذَلَكُمَا',
    'encletic': u''
}
STOPWORDS[u'فلذلكما'] = {
    'word': u'فلذلكما',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'فَلِذَلَكُمَا',
    'stem': u'ذَلَكُمَا',
    'type': u'STOPWORD',
    'original': u'ذَلَكُمَا',
    'encletic': u''
}
STOPWORDS[u'أذلكما'] = {
    'word': u'أذلكما',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'أَذَلَكُمَا',
    'stem': u'ذَلَكُمَا',
    'type': u'STOPWORD',
    'original': u'ذَلَكُمَا',
    'encletic': u''
}
STOPWORDS[u'أبذلكما'] = {
    'word': u'أبذلكما',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:مجرور',
    'vocalized': u'أَبِذَلَكُمَا',
    'stem': u'ذَلَكُمَا',
    'type': u'STOPWORD',
    'original': u'ذَلَكُمَا',
    'encletic': u''
}
STOPWORDS[u'أكذلكما'] = {
    'word': u'أكذلكما',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:مجرور',
    'vocalized': u'أَكَذَلَكُمَا',
    'stem': u'ذَلَكُمَا',
    'type': u'STOPWORD',
    'original': u'ذَلَكُمَا',
    'encletic': u''
}
STOPWORDS[u'ألذلكما'] = {
    'word': u'ألذلكما',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:مجرور',
    'vocalized': u'أَلِذَلَكُمَا',
    'stem': u'ذَلَكُمَا',
    'type': u'STOPWORD',
    'original': u'ذَلَكُمَا',
    'encletic': u''
}
STOPWORDS[u'أوذلكما'] = {
    'word': u'أوذلكما',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إشارة:معطوف:مجرور',
    'vocalized': u'أَوَذَلَكُمَا',
    'stem': u'ذَلَكُمَا',
    'type': u'STOPWORD',
    'original': u'ذَلَكُمَا',
    'encletic': u''
}
STOPWORDS[u'أفذلكما'] = {
    'word': u'أفذلكما',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إشارة:معطوف:مجرور',
    'vocalized': u'أَفَذَلَكُمَا',
    'stem': u'ذَلَكُمَا',
    'type': u'STOPWORD',
    'original': u'ذَلَكُمَا',
    'encletic': u''
}
STOPWORDS[u'أوبذلكما'] = {
    'word': u'أوبذلكما',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَبِذَلَكُمَا',
    'stem': u'ذَلَكُمَا',
    'type': u'STOPWORD',
    'original': u'ذَلَكُمَا',
    'encletic': u''
}
STOPWORDS[u'أفبذلكما'] = {
    'word': u'أفبذلكما',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَبِذَلَكُمَا',
    'stem': u'ذَلَكُمَا',
    'type': u'STOPWORD',
    'original': u'ذَلَكُمَا',
    'encletic': u''
}
STOPWORDS[u'أوكذلكما'] = {
    'word': u'أوكذلكما',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَكَذَلَكُمَا',
    'stem': u'ذَلَكُمَا',
    'type': u'STOPWORD',
    'original': u'ذَلَكُمَا',
    'encletic': u''
}
STOPWORDS[u'أفكذلكما'] = {
    'word': u'أفكذلكما',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَكَذَلَكُمَا',
    'stem': u'ذَلَكُمَا',
    'type': u'STOPWORD',
    'original': u'ذَلَكُمَا',
    'encletic': u''
}
STOPWORDS[u'أولذلكما'] = {
    'word': u'أولذلكما',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَلِذَلَكُمَا',
    'stem': u'ذَلَكُمَا',
    'type': u'STOPWORD',
    'original': u'ذَلَكُمَا',
    'encletic': u''
}
STOPWORDS[u'أفلذلكما'] = {
    'word': u'أفلذلكما',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَلِذَلَكُمَا',
    'stem': u'ذَلَكُمَا',
    'type': u'STOPWORD',
    'original': u'ذَلَكُمَا',
    'encletic': u''
}
STOPWORDS[u'ذلكن'] = {
    'word': u'ذَلِكُنَّ',
    'procletic': u'',
    'tags': u'أداة:اسم إشارة',
    'vocalized': u'ذَلِكُنَّ',
    'stem': u'ذَلِكُنَّ',
    'type': u'STOPWORD',
    'original': u'ذَلِكُنَّ',
    'encletic': u''
}
STOPWORDS[u'بذلكن'] = {
    'word': u'بذلكن',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'بِذَلِكُنَّ',
    'stem': u'ذَلِكُنَّ',
    'type': u'STOPWORD',
    'original': u'ذَلِكُنَّ',
    'encletic': u''
}
STOPWORDS[u'كذلكن'] = {
    'word': u'كذلكن',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'كَذَلِكُنَّ',
    'stem': u'ذَلِكُنَّ',
    'type': u'STOPWORD',
    'original': u'ذَلِكُنَّ',
    'encletic': u''
}
STOPWORDS[u'لذلكن'] = {
    'word': u'لذلكن',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'لِذَلِكُنَّ',
    'stem': u'ذَلِكُنَّ',
    'type': u'STOPWORD',
    'original': u'ذَلِكُنَّ',
    'encletic': u''
}
STOPWORDS[u'وذلكن'] = {
    'word': u'وذلكن',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إشارة:معطوف',
    'vocalized': u'وَذَلِكُنَّ',
    'stem': u'ذَلِكُنَّ',
    'type': u'STOPWORD',
    'original': u'ذَلِكُنَّ',
    'encletic': u''
}
STOPWORDS[u'فذلكن'] = {
    'word': u'فذلكن',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إشارة:معطوف',
    'vocalized': u'فَذَلِكُنَّ',
    'stem': u'ذَلِكُنَّ',
    'type': u'STOPWORD',
    'original': u'ذَلِكُنَّ',
    'encletic': u''
}
STOPWORDS[u'وبذلكن'] = {
    'word': u'وبذلكن',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'وَبِذَلِكُنَّ',
    'stem': u'ذَلِكُنَّ',
    'type': u'STOPWORD',
    'original': u'ذَلِكُنَّ',
    'encletic': u''
}
STOPWORDS[u'فبذلكن'] = {
    'word': u'فبذلكن',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'فَبِذَلِكُنَّ',
    'stem': u'ذَلِكُنَّ',
    'type': u'STOPWORD',
    'original': u'ذَلِكُنَّ',
    'encletic': u''
}
STOPWORDS[u'وكذلكن'] = {
    'word': u'وكذلكن',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'وَكَذَلِكُنَّ',
    'stem': u'ذَلِكُنَّ',
    'type': u'STOPWORD',
    'original': u'ذَلِكُنَّ',
    'encletic': u''
}
STOPWORDS[u'فكذلكن'] = {
    'word': u'فكذلكن',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'فَكَذَلِكُنَّ',
    'stem': u'ذَلِكُنَّ',
    'type': u'STOPWORD',
    'original': u'ذَلِكُنَّ',
    'encletic': u''
}
STOPWORDS[u'ولذلكن'] = {
    'word': u'ولذلكن',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'وَلِذَلِكُنَّ',
    'stem': u'ذَلِكُنَّ',
    'type': u'STOPWORD',
    'original': u'ذَلِكُنَّ',
    'encletic': u''
}
STOPWORDS[u'فلذلكن'] = {
    'word': u'فلذلكن',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'فَلِذَلِكُنَّ',
    'stem': u'ذَلِكُنَّ',
    'type': u'STOPWORD',
    'original': u'ذَلِكُنَّ',
    'encletic': u''
}
STOPWORDS[u'أذلكن'] = {
    'word': u'أذلكن',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'أَذَلِكُنَّ',
    'stem': u'ذَلِكُنَّ',
    'type': u'STOPWORD',
    'original': u'ذَلِكُنَّ',
    'encletic': u''
}
STOPWORDS[u'أبذلكن'] = {
    'word': u'أبذلكن',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:مجرور',
    'vocalized': u'أَبِذَلِكُنَّ',
    'stem': u'ذَلِكُنَّ',
    'type': u'STOPWORD',
    'original': u'ذَلِكُنَّ',
    'encletic': u''
}
STOPWORDS[u'أكذلكن'] = {
    'word': u'أكذلكن',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:مجرور',
    'vocalized': u'أَكَذَلِكُنَّ',
    'stem': u'ذَلِكُنَّ',
    'type': u'STOPWORD',
    'original': u'ذَلِكُنَّ',
    'encletic': u''
}
STOPWORDS[u'ألذلكن'] = {
    'word': u'ألذلكن',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:مجرور',
    'vocalized': u'أَلِذَلِكُنَّ',
    'stem': u'ذَلِكُنَّ',
    'type': u'STOPWORD',
    'original': u'ذَلِكُنَّ',
    'encletic': u''
}
STOPWORDS[u'أوذلكن'] = {
    'word': u'أوذلكن',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إشارة:معطوف:مجرور',
    'vocalized': u'أَوَذَلِكُنَّ',
    'stem': u'ذَلِكُنَّ',
    'type': u'STOPWORD',
    'original': u'ذَلِكُنَّ',
    'encletic': u''
}
STOPWORDS[u'أفذلكن'] = {
    'word': u'أفذلكن',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إشارة:معطوف:مجرور',
    'vocalized': u'أَفَذَلِكُنَّ',
    'stem': u'ذَلِكُنَّ',
    'type': u'STOPWORD',
    'original': u'ذَلِكُنَّ',
    'encletic': u''
}
STOPWORDS[u'أوبذلكن'] = {
    'word': u'أوبذلكن',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَبِذَلِكُنَّ',
    'stem': u'ذَلِكُنَّ',
    'type': u'STOPWORD',
    'original': u'ذَلِكُنَّ',
    'encletic': u''
}
STOPWORDS[u'أفبذلكن'] = {
    'word': u'أفبذلكن',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَبِذَلِكُنَّ',
    'stem': u'ذَلِكُنَّ',
    'type': u'STOPWORD',
    'original': u'ذَلِكُنَّ',
    'encletic': u''
}
STOPWORDS[u'أوكذلكن'] = {
    'word': u'أوكذلكن',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَكَذَلِكُنَّ',
    'stem': u'ذَلِكُنَّ',
    'type': u'STOPWORD',
    'original': u'ذَلِكُنَّ',
    'encletic': u''
}
STOPWORDS[u'أفكذلكن'] = {
    'word': u'أفكذلكن',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَكَذَلِكُنَّ',
    'stem': u'ذَلِكُنَّ',
    'type': u'STOPWORD',
    'original': u'ذَلِكُنَّ',
    'encletic': u''
}
STOPWORDS[u'أولذلكن'] = {
    'word': u'أولذلكن',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَلِذَلِكُنَّ',
    'stem': u'ذَلِكُنَّ',
    'type': u'STOPWORD',
    'original': u'ذَلِكُنَّ',
    'encletic': u''
}
STOPWORDS[u'أفلذلكن'] = {
    'word': u'أفلذلكن',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَلِذَلِكُنَّ',
    'stem': u'ذَلِكُنَّ',
    'type': u'STOPWORD',
    'original': u'ذَلِكُنَّ',
    'encletic': u''
}
STOPWORDS[u'ذه'] = {
    'word': u'ذِهِ',
    'procletic': u'',
    'tags': u'أداة:اسم إشارة',
    'vocalized': u'ذِهِ',
    'stem': u'ذِهِ',
    'type': u'STOPWORD',
    'original': u'ذِهِ',
    'encletic': u''
}
STOPWORDS[u'بذه'] = {
    'word': u'بذه',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'بِذِهِ',
    'stem': u'ذِهِ',
    'type': u'STOPWORD',
    'original': u'ذِهِ',
    'encletic': u''
}
STOPWORDS[u'كذه'] = {
    'word': u'كذه',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'كَذِهِ',
    'stem': u'ذِهِ',
    'type': u'STOPWORD',
    'original': u'ذِهِ',
    'encletic': u''
}
STOPWORDS[u'لذه'] = {
    'word': u'لذه',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'لِذِهِ',
    'stem': u'ذِهِ',
    'type': u'STOPWORD',
    'original': u'ذِهِ',
    'encletic': u''
}
STOPWORDS[u'وذه'] = {
    'word': u'وذه',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إشارة:معطوف',
    'vocalized': u'وَذِهِ',
    'stem': u'ذِهِ',
    'type': u'STOPWORD',
    'original': u'ذِهِ',
    'encletic': u''
}
STOPWORDS[u'فذه'] = {
    'word': u'فذه',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إشارة:معطوف',
    'vocalized': u'فَذِهِ',
    'stem': u'ذِهِ',
    'type': u'STOPWORD',
    'original': u'ذِهِ',
    'encletic': u''
}
STOPWORDS[u'وبذه'] = {
    'word': u'وبذه',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'وَبِذِهِ',
    'stem': u'ذِهِ',
    'type': u'STOPWORD',
    'original': u'ذِهِ',
    'encletic': u''
}
STOPWORDS[u'فبذه'] = {
    'word': u'فبذه',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'فَبِذِهِ',
    'stem': u'ذِهِ',
    'type': u'STOPWORD',
    'original': u'ذِهِ',
    'encletic': u''
}
STOPWORDS[u'وكذه'] = {
    'word': u'وكذه',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'وَكَذِهِ',
    'stem': u'ذِهِ',
    'type': u'STOPWORD',
    'original': u'ذِهِ',
    'encletic': u''
}
STOPWORDS[u'فكذه'] = {
    'word': u'فكذه',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'فَكَذِهِ',
    'stem': u'ذِهِ',
    'type': u'STOPWORD',
    'original': u'ذِهِ',
    'encletic': u''
}
STOPWORDS[u'ولذه'] = {
    'word': u'ولذه',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'وَلِذِهِ',
    'stem': u'ذِهِ',
    'type': u'STOPWORD',
    'original': u'ذِهِ',
    'encletic': u''
}
STOPWORDS[u'فلذه'] = {
    'word': u'فلذه',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'فَلِذِهِ',
    'stem': u'ذِهِ',
    'type': u'STOPWORD',
    'original': u'ذِهِ',
    'encletic': u''
}
STOPWORDS[u'ذوا'] = {
    'word': u'ذَوَا',
    'procletic': u'',
    'tags': u'أداة:اسم إشارة',
    'vocalized': u'ذَوَا',
    'stem': u'ذَوَا',
    'type': u'STOPWORD',
    'original': u'ذَوَا',
    'encletic': u''
}
STOPWORDS[u'وذوا'] = {
    'word': u'وذوا',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إشارة:معطوف',
    'vocalized': u'وَذَوَا',
    'stem': u'ذَوَا',
    'type': u'STOPWORD',
    'original': u'ذَوَا',
    'encletic': u''
}
STOPWORDS[u'فذوا'] = {
    'word': u'فذوا',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إشارة:معطوف',
    'vocalized': u'فَذَوَا',
    'stem': u'ذَوَا',
    'type': u'STOPWORD',
    'original': u'ذَوَا',
    'encletic': u''
}
STOPWORDS[u'ذواتا'] = {
    'word': u'ذَوَاتَا',
    'procletic': u'',
    'tags': u'أداة:اسم إشارة',
    'vocalized': u'ذَوَاتَا',
    'stem': u'ذَوَاتَا',
    'type': u'STOPWORD',
    'original': u'ذَوَاتَا',
    'encletic': u''
}
STOPWORDS[u'وذواتا'] = {
    'word': u'وذواتا',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إشارة:معطوف',
    'vocalized': u'وَذَوَاتَا',
    'stem': u'ذَوَاتَا',
    'type': u'STOPWORD',
    'original': u'ذَوَاتَا',
    'encletic': u''
}
STOPWORDS[u'فذواتا'] = {
    'word': u'فذواتا',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إشارة:معطوف',
    'vocalized': u'فَذَوَاتَا',
    'stem': u'ذَوَاتَا',
    'type': u'STOPWORD',
    'original': u'ذَوَاتَا',
    'encletic': u''
}
STOPWORDS[u'أذواتا'] = {
    'word': u'أذواتا',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'أَذَوَاتَا',
    'stem': u'ذَوَاتَا',
    'type': u'STOPWORD',
    'original': u'ذَوَاتَا',
    'encletic': u''
}
STOPWORDS[u'أوذواتا'] = {
    'word': u'أوذواتا',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إشارة:معطوف:مجرور',
    'vocalized': u'أَوَذَوَاتَا',
    'stem': u'ذَوَاتَا',
    'type': u'STOPWORD',
    'original': u'ذَوَاتَا',
    'encletic': u''
}
STOPWORDS[u'أفذواتا'] = {
    'word': u'أفذواتا',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إشارة:معطوف:مجرور',
    'vocalized': u'أَفَذَوَاتَا',
    'stem': u'ذَوَاتَا',
    'type': u'STOPWORD',
    'original': u'ذَوَاتَا',
    'encletic': u''
}
STOPWORDS[u'ذواتي'] = {
    'word': u'ذَوَاتَيْ',
    'procletic': u'',
    'tags': u'أداة:اسم إشارة',
    'vocalized': u'ذَوَاتَيْ',
    'stem': u'ذَوَاتَيْ',
    'type': u'STOPWORD',
    'original': u'ذَوَاتَيْ',
    'encletic': u''
}
STOPWORDS[u'بذواتي'] = {
    'word': u'بذواتي',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'بِذَوَاتَيْ',
    'stem': u'ذَوَاتَيْ',
    'type': u'STOPWORD',
    'original': u'ذَوَاتَيْ',
    'encletic': u''
}
STOPWORDS[u'كذواتي'] = {
    'word': u'كذواتي',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'كَذَوَاتَيْ',
    'stem': u'ذَوَاتَيْ',
    'type': u'STOPWORD',
    'original': u'ذَوَاتَيْ',
    'encletic': u''
}
STOPWORDS[u'لذواتي'] = {
    'word': u'لذواتي',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'لِذَوَاتَيْ',
    'stem': u'ذَوَاتَيْ',
    'type': u'STOPWORD',
    'original': u'ذَوَاتَيْ',
    'encletic': u''
}
STOPWORDS[u'وذواتي'] = {
    'word': u'وذواتي',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إشارة:معطوف',
    'vocalized': u'وَذَوَاتَيْ',
    'stem': u'ذَوَاتَيْ',
    'type': u'STOPWORD',
    'original': u'ذَوَاتَيْ',
    'encletic': u''
}
STOPWORDS[u'فذواتي'] = {
    'word': u'فذواتي',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إشارة:معطوف',
    'vocalized': u'فَذَوَاتَيْ',
    'stem': u'ذَوَاتَيْ',
    'type': u'STOPWORD',
    'original': u'ذَوَاتَيْ',
    'encletic': u''
}
STOPWORDS[u'وبذواتي'] = {
    'word': u'وبذواتي',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'وَبِذَوَاتَيْ',
    'stem': u'ذَوَاتَيْ',
    'type': u'STOPWORD',
    'original': u'ذَوَاتَيْ',
    'encletic': u''
}
STOPWORDS[u'فبذواتي'] = {
    'word': u'فبذواتي',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'فَبِذَوَاتَيْ',
    'stem': u'ذَوَاتَيْ',
    'type': u'STOPWORD',
    'original': u'ذَوَاتَيْ',
    'encletic': u''
}
STOPWORDS[u'وكذواتي'] = {
    'word': u'وكذواتي',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'وَكَذَوَاتَيْ',
    'stem': u'ذَوَاتَيْ',
    'type': u'STOPWORD',
    'original': u'ذَوَاتَيْ',
    'encletic': u''
}
STOPWORDS[u'فكذواتي'] = {
    'word': u'فكذواتي',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'فَكَذَوَاتَيْ',
    'stem': u'ذَوَاتَيْ',
    'type': u'STOPWORD',
    'original': u'ذَوَاتَيْ',
    'encletic': u''
}
STOPWORDS[u'ولذواتي'] = {
    'word': u'ولذواتي',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'وَلِذَوَاتَيْ',
    'stem': u'ذَوَاتَيْ',
    'type': u'STOPWORD',
    'original': u'ذَوَاتَيْ',
    'encletic': u''
}
STOPWORDS[u'فلذواتي'] = {
    'word': u'فلذواتي',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'فَلِذَوَاتَيْ',
    'stem': u'ذَوَاتَيْ',
    'type': u'STOPWORD',
    'original': u'ذَوَاتَيْ',
    'encletic': u''
}
STOPWORDS[u'أذواتي'] = {
    'word': u'أذواتي',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'أَذَوَاتَيْ',
    'stem': u'ذَوَاتَيْ',
    'type': u'STOPWORD',
    'original': u'ذَوَاتَيْ',
    'encletic': u''
}
STOPWORDS[u'أبذواتي'] = {
    'word': u'أبذواتي',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:مجرور',
    'vocalized': u'أَبِذَوَاتَيْ',
    'stem': u'ذَوَاتَيْ',
    'type': u'STOPWORD',
    'original': u'ذَوَاتَيْ',
    'encletic': u''
}
STOPWORDS[u'أكذواتي'] = {
    'word': u'أكذواتي',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:مجرور',
    'vocalized': u'أَكَذَوَاتَيْ',
    'stem': u'ذَوَاتَيْ',
    'type': u'STOPWORD',
    'original': u'ذَوَاتَيْ',
    'encletic': u''
}
STOPWORDS[u'ألذواتي'] = {
    'word': u'ألذواتي',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:مجرور',
    'vocalized': u'أَلِذَوَاتَيْ',
    'stem': u'ذَوَاتَيْ',
    'type': u'STOPWORD',
    'original': u'ذَوَاتَيْ',
    'encletic': u''
}
STOPWORDS[u'أوذواتي'] = {
    'word': u'أوذواتي',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إشارة:معطوف:مجرور',
    'vocalized': u'أَوَذَوَاتَيْ',
    'stem': u'ذَوَاتَيْ',
    'type': u'STOPWORD',
    'original': u'ذَوَاتَيْ',
    'encletic': u''
}
STOPWORDS[u'أفذواتي'] = {
    'word': u'أفذواتي',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إشارة:معطوف:مجرور',
    'vocalized': u'أَفَذَوَاتَيْ',
    'stem': u'ذَوَاتَيْ',
    'type': u'STOPWORD',
    'original': u'ذَوَاتَيْ',
    'encletic': u''
}
STOPWORDS[u'أوبذواتي'] = {
    'word': u'أوبذواتي',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَبِذَوَاتَيْ',
    'stem': u'ذَوَاتَيْ',
    'type': u'STOPWORD',
    'original': u'ذَوَاتَيْ',
    'encletic': u''
}
STOPWORDS[u'أفبذواتي'] = {
    'word': u'أفبذواتي',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَبِذَوَاتَيْ',
    'stem': u'ذَوَاتَيْ',
    'type': u'STOPWORD',
    'original': u'ذَوَاتَيْ',
    'encletic': u''
}
STOPWORDS[u'أوكذواتي'] = {
    'word': u'أوكذواتي',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَكَذَوَاتَيْ',
    'stem': u'ذَوَاتَيْ',
    'type': u'STOPWORD',
    'original': u'ذَوَاتَيْ',
    'encletic': u''
}
STOPWORDS[u'أفكذواتي'] = {
    'word': u'أفكذواتي',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَكَذَوَاتَيْ',
    'stem': u'ذَوَاتَيْ',
    'type': u'STOPWORD',
    'original': u'ذَوَاتَيْ',
    'encletic': u''
}
STOPWORDS[u'أولذواتي'] = {
    'word': u'أولذواتي',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَلِذَوَاتَيْ',
    'stem': u'ذَوَاتَيْ',
    'type': u'STOPWORD',
    'original': u'ذَوَاتَيْ',
    'encletic': u''
}
STOPWORDS[u'أفلذواتي'] = {
    'word': u'أفلذواتي',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَلِذَوَاتَيْ',
    'stem': u'ذَوَاتَيْ',
    'type': u'STOPWORD',
    'original': u'ذَوَاتَيْ',
    'encletic': u''
}
STOPWORDS[u'ذي'] = {
    'word': u'ذِي',
    'procletic': u'',
    'tags': u'أداة:اسم إشارة',
    'vocalized': u'ذِي',
    'stem': u'ذِي',
    'type': u'STOPWORD',
    'original': u'ذِي',
    'encletic': u''
}
STOPWORDS[u'بذي'] = {
    'word': u'بذي',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'بِذِي',
    'stem': u'ذِي',
    'type': u'STOPWORD',
    'original': u'ذِي',
    'encletic': u''
}
STOPWORDS[u'كذي'] = {
    'word': u'كذي',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'كَذِي',
    'stem': u'ذِي',
    'type': u'STOPWORD',
    'original': u'ذِي',
    'encletic': u''
}
STOPWORDS[u'لذي'] = {
    'word': u'لذي',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'لِذِي',
    'stem': u'ذِي',
    'type': u'STOPWORD',
    'original': u'ذِي',
    'encletic': u''
}
STOPWORDS[u'وذي'] = {
    'word': u'وذي',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إشارة:معطوف',
    'vocalized': u'وَذِي',
    'stem': u'ذِي',
    'type': u'STOPWORD',
    'original': u'ذِي',
    'encletic': u''
}
STOPWORDS[u'فذي'] = {
    'word': u'فذي',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إشارة:معطوف',
    'vocalized': u'فَذِي',
    'stem': u'ذِي',
    'type': u'STOPWORD',
    'original': u'ذِي',
    'encletic': u''
}
STOPWORDS[u'وبذي'] = {
    'word': u'وبذي',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'وَبِذِي',
    'stem': u'ذِي',
    'type': u'STOPWORD',
    'original': u'ذِي',
    'encletic': u''
}
STOPWORDS[u'فبذي'] = {
    'word': u'فبذي',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'فَبِذِي',
    'stem': u'ذِي',
    'type': u'STOPWORD',
    'original': u'ذِي',
    'encletic': u''
}
STOPWORDS[u'وكذي'] = {
    'word': u'وكذي',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'وَكَذِي',
    'stem': u'ذِي',
    'type': u'STOPWORD',
    'original': u'ذِي',
    'encletic': u''
}
STOPWORDS[u'فكذي'] = {
    'word': u'فكذي',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'فَكَذِي',
    'stem': u'ذِي',
    'type': u'STOPWORD',
    'original': u'ذِي',
    'encletic': u''
}
STOPWORDS[u'ولذي'] = {
    'word': u'ولذي',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'وَلِذِي',
    'stem': u'ذِي',
    'type': u'STOPWORD',
    'original': u'ذِي',
    'encletic': u''
}
STOPWORDS[u'فلذي'] = {
    'word': u'فلذي',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'فَلِذِي',
    'stem': u'ذِي',
    'type': u'STOPWORD',
    'original': u'ذِي',
    'encletic': u''
}
STOPWORDS[u'أذي'] = {
    'word': u'أذي',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'أَذِي',
    'stem': u'ذِي',
    'type': u'STOPWORD',
    'original': u'ذِي',
    'encletic': u''
}
STOPWORDS[u'أبذي'] = {
    'word': u'أبذي',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:مجرور',
    'vocalized': u'أَبِذِي',
    'stem': u'ذِي',
    'type': u'STOPWORD',
    'original': u'ذِي',
    'encletic': u''
}
STOPWORDS[u'أكذي'] = {
    'word': u'أكذي',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:مجرور',
    'vocalized': u'أَكَذِي',
    'stem': u'ذِي',
    'type': u'STOPWORD',
    'original': u'ذِي',
    'encletic': u''
}
STOPWORDS[u'ألذي'] = {
    'word': u'ألذي',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:مجرور',
    'vocalized': u'أَلِذِي',
    'stem': u'ذِي',
    'type': u'STOPWORD',
    'original': u'ذِي',
    'encletic': u''
}
STOPWORDS[u'أوذي'] = {
    'word': u'أوذي',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إشارة:معطوف:مجرور',
    'vocalized': u'أَوَذِي',
    'stem': u'ذِي',
    'type': u'STOPWORD',
    'original': u'ذِي',
    'encletic': u''
}
STOPWORDS[u'أفذي'] = {
    'word': u'أفذي',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إشارة:معطوف:مجرور',
    'vocalized': u'أَفَذِي',
    'stem': u'ذِي',
    'type': u'STOPWORD',
    'original': u'ذِي',
    'encletic': u''
}
STOPWORDS[u'أوبذي'] = {
    'word': u'أوبذي',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَبِذِي',
    'stem': u'ذِي',
    'type': u'STOPWORD',
    'original': u'ذِي',
    'encletic': u''
}
STOPWORDS[u'أفبذي'] = {
    'word': u'أفبذي',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَبِذِي',
    'stem': u'ذِي',
    'type': u'STOPWORD',
    'original': u'ذِي',
    'encletic': u''
}
STOPWORDS[u'أوكذي'] = {
    'word': u'أوكذي',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَكَذِي',
    'stem': u'ذِي',
    'type': u'STOPWORD',
    'original': u'ذِي',
    'encletic': u''
}
STOPWORDS[u'أفكذي'] = {
    'word': u'أفكذي',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَكَذِي',
    'stem': u'ذِي',
    'type': u'STOPWORD',
    'original': u'ذِي',
    'encletic': u''
}
STOPWORDS[u'أولذي'] = {
    'word': u'أولذي',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَلِذِي',
    'stem': u'ذِي',
    'type': u'STOPWORD',
    'original': u'ذِي',
    'encletic': u''
}
STOPWORDS[u'أفلذي'] = {
    'word': u'أفلذي',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَلِذِي',
    'stem': u'ذِي',
    'type': u'STOPWORD',
    'original': u'ذِي',
    'encletic': u''
}
STOPWORDS[u'ذين'] = {
    'word': u'ذَيْنِ',
    'procletic': u'',
    'tags': u'أداة:اسم إشارة',
    'vocalized': u'ذَيْنِ',
    'stem': u'ذَيْنِ',
    'type': u'STOPWORD',
    'original': u'ذَيْنِ',
    'encletic': u''
}
STOPWORDS[u'بذين'] = {
    'word': u'بذين',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'بِذَيْنِ',
    'stem': u'ذَيْنِ',
    'type': u'STOPWORD',
    'original': u'ذَيْنِ',
    'encletic': u''
}
STOPWORDS[u'كذين'] = {
    'word': u'كذين',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'كَذَيْنِ',
    'stem': u'ذَيْنِ',
    'type': u'STOPWORD',
    'original': u'ذَيْنِ',
    'encletic': u''
}
STOPWORDS[u'لذين'] = {
    'word': u'لذين',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'لِذَيْنِ',
    'stem': u'ذَيْنِ',
    'type': u'STOPWORD',
    'original': u'ذَيْنِ',
    'encletic': u''
}
STOPWORDS[u'وذين'] = {
    'word': u'وذين',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إشارة:معطوف',
    'vocalized': u'وَذَيْنِ',
    'stem': u'ذَيْنِ',
    'type': u'STOPWORD',
    'original': u'ذَيْنِ',
    'encletic': u''
}
STOPWORDS[u'فذين'] = {
    'word': u'فذين',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إشارة:معطوف',
    'vocalized': u'فَذَيْنِ',
    'stem': u'ذَيْنِ',
    'type': u'STOPWORD',
    'original': u'ذَيْنِ',
    'encletic': u''
}
STOPWORDS[u'وبذين'] = {
    'word': u'وبذين',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'وَبِذَيْنِ',
    'stem': u'ذَيْنِ',
    'type': u'STOPWORD',
    'original': u'ذَيْنِ',
    'encletic': u''
}
STOPWORDS[u'فبذين'] = {
    'word': u'فبذين',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'فَبِذَيْنِ',
    'stem': u'ذَيْنِ',
    'type': u'STOPWORD',
    'original': u'ذَيْنِ',
    'encletic': u''
}
STOPWORDS[u'وكذين'] = {
    'word': u'وكذين',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'وَكَذَيْنِ',
    'stem': u'ذَيْنِ',
    'type': u'STOPWORD',
    'original': u'ذَيْنِ',
    'encletic': u''
}
STOPWORDS[u'فكذين'] = {
    'word': u'فكذين',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'فَكَذَيْنِ',
    'stem': u'ذَيْنِ',
    'type': u'STOPWORD',
    'original': u'ذَيْنِ',
    'encletic': u''
}
STOPWORDS[u'ولذين'] = {
    'word': u'ولذين',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'وَلِذَيْنِ',
    'stem': u'ذَيْنِ',
    'type': u'STOPWORD',
    'original': u'ذَيْنِ',
    'encletic': u''
}
STOPWORDS[u'فلذين'] = {
    'word': u'فلذين',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'فَلِذَيْنِ',
    'stem': u'ذَيْنِ',
    'type': u'STOPWORD',
    'original': u'ذَيْنِ',
    'encletic': u''
}
STOPWORDS[u'أذين'] = {
    'word': u'أذين',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'أَذَيْنِ',
    'stem': u'ذَيْنِ',
    'type': u'STOPWORD',
    'original': u'ذَيْنِ',
    'encletic': u''
}
STOPWORDS[u'أبذين'] = {
    'word': u'أبذين',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:مجرور',
    'vocalized': u'أَبِذَيْنِ',
    'stem': u'ذَيْنِ',
    'type': u'STOPWORD',
    'original': u'ذَيْنِ',
    'encletic': u''
}
STOPWORDS[u'أكذين'] = {
    'word': u'أكذين',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:مجرور',
    'vocalized': u'أَكَذَيْنِ',
    'stem': u'ذَيْنِ',
    'type': u'STOPWORD',
    'original': u'ذَيْنِ',
    'encletic': u''
}
STOPWORDS[u'ألذين'] = {
    'word': u'ألذين',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:مجرور',
    'vocalized': u'أَلِذَيْنِ',
    'stem': u'ذَيْنِ',
    'type': u'STOPWORD',
    'original': u'ذَيْنِ',
    'encletic': u''
}
STOPWORDS[u'أوذين'] = {
    'word': u'أوذين',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إشارة:معطوف:مجرور',
    'vocalized': u'أَوَذَيْنِ',
    'stem': u'ذَيْنِ',
    'type': u'STOPWORD',
    'original': u'ذَيْنِ',
    'encletic': u''
}
STOPWORDS[u'أفذين'] = {
    'word': u'أفذين',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إشارة:معطوف:مجرور',
    'vocalized': u'أَفَذَيْنِ',
    'stem': u'ذَيْنِ',
    'type': u'STOPWORD',
    'original': u'ذَيْنِ',
    'encletic': u''
}
STOPWORDS[u'أوبذين'] = {
    'word': u'أوبذين',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَبِذَيْنِ',
    'stem': u'ذَيْنِ',
    'type': u'STOPWORD',
    'original': u'ذَيْنِ',
    'encletic': u''
}
STOPWORDS[u'أفبذين'] = {
    'word': u'أفبذين',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَبِذَيْنِ',
    'stem': u'ذَيْنِ',
    'type': u'STOPWORD',
    'original': u'ذَيْنِ',
    'encletic': u''
}
STOPWORDS[u'أوكذين'] = {
    'word': u'أوكذين',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَكَذَيْنِ',
    'stem': u'ذَيْنِ',
    'type': u'STOPWORD',
    'original': u'ذَيْنِ',
    'encletic': u''
}
STOPWORDS[u'أفكذين'] = {
    'word': u'أفكذين',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَكَذَيْنِ',
    'stem': u'ذَيْنِ',
    'type': u'STOPWORD',
    'original': u'ذَيْنِ',
    'encletic': u''
}
STOPWORDS[u'أولذين'] = {
    'word': u'أولذين',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَلِذَيْنِ',
    'stem': u'ذَيْنِ',
    'type': u'STOPWORD',
    'original': u'ذَيْنِ',
    'encletic': u''
}
STOPWORDS[u'أفلذين'] = {
    'word': u'أفلذين',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَلِذَيْنِ',
    'stem': u'ذَيْنِ',
    'type': u'STOPWORD',
    'original': u'ذَيْنِ',
    'encletic': u''
}
STOPWORDS[u'ذينك'] = {
    'word': u'ذَيْنِكَ',
    'procletic': u'',
    'tags': u'أداة:اسم إشارة',
    'vocalized': u'ذَيْنِكَ',
    'stem': u'ذَيْنِكَ',
    'type': u'STOPWORD',
    'original': u'ذَيْنِكَ',
    'encletic': u''
}
STOPWORDS[u'بذينك'] = {
    'word': u'بذينك',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'بِذَيْنِكَ',
    'stem': u'ذَيْنِكَ',
    'type': u'STOPWORD',
    'original': u'ذَيْنِكَ',
    'encletic': u''
}
STOPWORDS[u'كذينك'] = {
    'word': u'كذينك',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'كَذَيْنِكَ',
    'stem': u'ذَيْنِكَ',
    'type': u'STOPWORD',
    'original': u'ذَيْنِكَ',
    'encletic': u''
}
STOPWORDS[u'لذينك'] = {
    'word': u'لذينك',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'لِذَيْنِكَ',
    'stem': u'ذَيْنِكَ',
    'type': u'STOPWORD',
    'original': u'ذَيْنِكَ',
    'encletic': u''
}
STOPWORDS[u'وذينك'] = {
    'word': u'وذينك',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إشارة:معطوف',
    'vocalized': u'وَذَيْنِكَ',
    'stem': u'ذَيْنِكَ',
    'type': u'STOPWORD',
    'original': u'ذَيْنِكَ',
    'encletic': u''
}
STOPWORDS[u'فذينك'] = {
    'word': u'فذينك',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إشارة:معطوف',
    'vocalized': u'فَذَيْنِكَ',
    'stem': u'ذَيْنِكَ',
    'type': u'STOPWORD',
    'original': u'ذَيْنِكَ',
    'encletic': u''
}
STOPWORDS[u'وبذينك'] = {
    'word': u'وبذينك',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'وَبِذَيْنِكَ',
    'stem': u'ذَيْنِكَ',
    'type': u'STOPWORD',
    'original': u'ذَيْنِكَ',
    'encletic': u''
}
STOPWORDS[u'فبذينك'] = {
    'word': u'فبذينك',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'فَبِذَيْنِكَ',
    'stem': u'ذَيْنِكَ',
    'type': u'STOPWORD',
    'original': u'ذَيْنِكَ',
    'encletic': u''
}
STOPWORDS[u'وكذينك'] = {
    'word': u'وكذينك',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'وَكَذَيْنِكَ',
    'stem': u'ذَيْنِكَ',
    'type': u'STOPWORD',
    'original': u'ذَيْنِكَ',
    'encletic': u''
}
STOPWORDS[u'فكذينك'] = {
    'word': u'فكذينك',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'فَكَذَيْنِكَ',
    'stem': u'ذَيْنِكَ',
    'type': u'STOPWORD',
    'original': u'ذَيْنِكَ',
    'encletic': u''
}
STOPWORDS[u'ولذينك'] = {
    'word': u'ولذينك',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'وَلِذَيْنِكَ',
    'stem': u'ذَيْنِكَ',
    'type': u'STOPWORD',
    'original': u'ذَيْنِكَ',
    'encletic': u''
}
STOPWORDS[u'فلذينك'] = {
    'word': u'فلذينك',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'فَلِذَيْنِكَ',
    'stem': u'ذَيْنِكَ',
    'type': u'STOPWORD',
    'original': u'ذَيْنِكَ',
    'encletic': u''
}
STOPWORDS[u'أذينك'] = {
    'word': u'أذينك',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'أَذَيْنِكَ',
    'stem': u'ذَيْنِكَ',
    'type': u'STOPWORD',
    'original': u'ذَيْنِكَ',
    'encletic': u''
}
STOPWORDS[u'أبذينك'] = {
    'word': u'أبذينك',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:مجرور',
    'vocalized': u'أَبِذَيْنِكَ',
    'stem': u'ذَيْنِكَ',
    'type': u'STOPWORD',
    'original': u'ذَيْنِكَ',
    'encletic': u''
}
STOPWORDS[u'أكذينك'] = {
    'word': u'أكذينك',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:مجرور',
    'vocalized': u'أَكَذَيْنِكَ',
    'stem': u'ذَيْنِكَ',
    'type': u'STOPWORD',
    'original': u'ذَيْنِكَ',
    'encletic': u''
}
STOPWORDS[u'ألذينك'] = {
    'word': u'ألذينك',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:مجرور',
    'vocalized': u'أَلِذَيْنِكَ',
    'stem': u'ذَيْنِكَ',
    'type': u'STOPWORD',
    'original': u'ذَيْنِكَ',
    'encletic': u''
}
STOPWORDS[u'أوذينك'] = {
    'word': u'أوذينك',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إشارة:معطوف:مجرور',
    'vocalized': u'أَوَذَيْنِكَ',
    'stem': u'ذَيْنِكَ',
    'type': u'STOPWORD',
    'original': u'ذَيْنِكَ',
    'encletic': u''
}
STOPWORDS[u'أفذينك'] = {
    'word': u'أفذينك',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إشارة:معطوف:مجرور',
    'vocalized': u'أَفَذَيْنِكَ',
    'stem': u'ذَيْنِكَ',
    'type': u'STOPWORD',
    'original': u'ذَيْنِكَ',
    'encletic': u''
}
STOPWORDS[u'أوبذينك'] = {
    'word': u'أوبذينك',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَبِذَيْنِكَ',
    'stem': u'ذَيْنِكَ',
    'type': u'STOPWORD',
    'original': u'ذَيْنِكَ',
    'encletic': u''
}
STOPWORDS[u'أفبذينك'] = {
    'word': u'أفبذينك',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَبِذَيْنِكَ',
    'stem': u'ذَيْنِكَ',
    'type': u'STOPWORD',
    'original': u'ذَيْنِكَ',
    'encletic': u''
}
STOPWORDS[u'أوكذينك'] = {
    'word': u'أوكذينك',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَكَذَيْنِكَ',
    'stem': u'ذَيْنِكَ',
    'type': u'STOPWORD',
    'original': u'ذَيْنِكَ',
    'encletic': u''
}
STOPWORDS[u'أفكذينك'] = {
    'word': u'أفكذينك',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَكَذَيْنِكَ',
    'stem': u'ذَيْنِكَ',
    'type': u'STOPWORD',
    'original': u'ذَيْنِكَ',
    'encletic': u''
}
STOPWORDS[u'أولذينك'] = {
    'word': u'أولذينك',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَلِذَيْنِكَ',
    'stem': u'ذَيْنِكَ',
    'type': u'STOPWORD',
    'original': u'ذَيْنِكَ',
    'encletic': u''
}
STOPWORDS[u'أفلذينك'] = {
    'word': u'أفلذينك',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَلِذَيْنِكَ',
    'stem': u'ذَيْنِكَ',
    'type': u'STOPWORD',
    'original': u'ذَيْنِكَ',
    'encletic': u''
}
STOPWORDS[u'كذلك'] = {
    'word': u'كَذَلِكَ',
    'procletic': u'',
    'tags': u'أداة:اسم إشارة',
    'vocalized': u'كَذَلِكَ',
    'stem': u'كَذَلِكَ',
    'type': u'STOPWORD',
    'original': u'كَذَلِكَ',
    'encletic': u''
}
STOPWORDS[u'وكذلك'] = {
    'word': u'وكذلك',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إشارة:معطوف',
    'vocalized': u'وَكَذَلِكَ',
    'stem': u'كَذَلِكَ',
    'type': u'STOPWORD',
    'original': u'كَذَلِكَ',
    'encletic': u''
}
STOPWORDS[u'فكذلك'] = {
    'word': u'فكذلك',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إشارة:معطوف',
    'vocalized': u'فَكَذَلِكَ',
    'stem': u'كَذَلِكَ',
    'type': u'STOPWORD',
    'original': u'كَذَلِكَ',
    'encletic': u''
}
STOPWORDS[u'أكذلك'] = {
    'word': u'أكذلك',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'أَكَذَلِكَ',
    'stem': u'كَذَلِكَ',
    'type': u'STOPWORD',
    'original': u'كَذَلِكَ',
    'encletic': u''
}
STOPWORDS[u'أوكذلك'] = {
    'word': u'أوكذلك',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إشارة:معطوف:مجرور',
    'vocalized': u'أَوَكَذَلِكَ',
    'stem': u'كَذَلِكَ',
    'type': u'STOPWORD',
    'original': u'كَذَلِكَ',
    'encletic': u''
}
STOPWORDS[u'أفكذلك'] = {
    'word': u'أفكذلك',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إشارة:معطوف:مجرور',
    'vocalized': u'أَفَكَذَلِكَ',
    'stem': u'كَذَلِكَ',
    'type': u'STOPWORD',
    'original': u'كَذَلِكَ',
    'encletic': u''
}
STOPWORDS[u'هؤلاء'] = {
    'word': u'هَؤُلَاءِ',
    'procletic': u'',
    'tags': u'أداة:اسم إشارة',
    'vocalized': u'هَؤُلَاءِ',
    'stem': u'هَؤُلَاءِ',
    'type': u'STOPWORD',
    'original': u'هَؤُلَاءِ',
    'encletic': u''
}
STOPWORDS[u'بهؤلاء'] = {
    'word': u'بهؤلاء',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'بِهَؤُلَاءِ',
    'stem': u'هَؤُلَاءِ',
    'type': u'STOPWORD',
    'original': u'هَؤُلَاءِ',
    'encletic': u''
}
STOPWORDS[u'كهؤلاء'] = {
    'word': u'كهؤلاء',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'كَهَؤُلَاءِ',
    'stem': u'هَؤُلَاءِ',
    'type': u'STOPWORD',
    'original': u'هَؤُلَاءِ',
    'encletic': u''
}
STOPWORDS[u'لهؤلاء'] = {
    'word': u'لهؤلاء',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'لِهَؤُلَاءِ',
    'stem': u'هَؤُلَاءِ',
    'type': u'STOPWORD',
    'original': u'هَؤُلَاءِ',
    'encletic': u''
}
STOPWORDS[u'وهؤلاء'] = {
    'word': u'وهؤلاء',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إشارة:معطوف',
    'vocalized': u'وَهَؤُلَاءِ',
    'stem': u'هَؤُلَاءِ',
    'type': u'STOPWORD',
    'original': u'هَؤُلَاءِ',
    'encletic': u''
}
STOPWORDS[u'فهؤلاء'] = {
    'word': u'فهؤلاء',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إشارة:معطوف',
    'vocalized': u'فَهَؤُلَاءِ',
    'stem': u'هَؤُلَاءِ',
    'type': u'STOPWORD',
    'original': u'هَؤُلَاءِ',
    'encletic': u''
}
STOPWORDS[u'وبهؤلاء'] = {
    'word': u'وبهؤلاء',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'وَبِهَؤُلَاءِ',
    'stem': u'هَؤُلَاءِ',
    'type': u'STOPWORD',
    'original': u'هَؤُلَاءِ',
    'encletic': u''
}
STOPWORDS[u'فبهؤلاء'] = {
    'word': u'فبهؤلاء',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'فَبِهَؤُلَاءِ',
    'stem': u'هَؤُلَاءِ',
    'type': u'STOPWORD',
    'original': u'هَؤُلَاءِ',
    'encletic': u''
}
STOPWORDS[u'وكهؤلاء'] = {
    'word': u'وكهؤلاء',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'وَكَهَؤُلَاءِ',
    'stem': u'هَؤُلَاءِ',
    'type': u'STOPWORD',
    'original': u'هَؤُلَاءِ',
    'encletic': u''
}
STOPWORDS[u'فكهؤلاء'] = {
    'word': u'فكهؤلاء',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'فَكَهَؤُلَاءِ',
    'stem': u'هَؤُلَاءِ',
    'type': u'STOPWORD',
    'original': u'هَؤُلَاءِ',
    'encletic': u''
}
STOPWORDS[u'ولهؤلاء'] = {
    'word': u'ولهؤلاء',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'وَلِهَؤُلَاءِ',
    'stem': u'هَؤُلَاءِ',
    'type': u'STOPWORD',
    'original': u'هَؤُلَاءِ',
    'encletic': u''
}
STOPWORDS[u'فلهؤلاء'] = {
    'word': u'فلهؤلاء',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'فَلِهَؤُلَاءِ',
    'stem': u'هَؤُلَاءِ',
    'type': u'STOPWORD',
    'original': u'هَؤُلَاءِ',
    'encletic': u''
}
STOPWORDS[u'أهؤلاء'] = {
    'word': u'أهؤلاء',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'أَهَؤُلَاءِ',
    'stem': u'هَؤُلَاءِ',
    'type': u'STOPWORD',
    'original': u'هَؤُلَاءِ',
    'encletic': u''
}
STOPWORDS[u'أبهؤلاء'] = {
    'word': u'أبهؤلاء',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:مجرور',
    'vocalized': u'أَبِهَؤُلَاءِ',
    'stem': u'هَؤُلَاءِ',
    'type': u'STOPWORD',
    'original': u'هَؤُلَاءِ',
    'encletic': u''
}
STOPWORDS[u'أكهؤلاء'] = {
    'word': u'أكهؤلاء',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:مجرور',
    'vocalized': u'أَكَهَؤُلَاءِ',
    'stem': u'هَؤُلَاءِ',
    'type': u'STOPWORD',
    'original': u'هَؤُلَاءِ',
    'encletic': u''
}
STOPWORDS[u'ألهؤلاء'] = {
    'word': u'ألهؤلاء',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:مجرور',
    'vocalized': u'أَلِهَؤُلَاءِ',
    'stem': u'هَؤُلَاءِ',
    'type': u'STOPWORD',
    'original': u'هَؤُلَاءِ',
    'encletic': u''
}
STOPWORDS[u'أوهؤلاء'] = {
    'word': u'أوهؤلاء',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إشارة:معطوف:مجرور',
    'vocalized': u'أَوَهَؤُلَاءِ',
    'stem': u'هَؤُلَاءِ',
    'type': u'STOPWORD',
    'original': u'هَؤُلَاءِ',
    'encletic': u''
}
STOPWORDS[u'أفهؤلاء'] = {
    'word': u'أفهؤلاء',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إشارة:معطوف:مجرور',
    'vocalized': u'أَفَهَؤُلَاءِ',
    'stem': u'هَؤُلَاءِ',
    'type': u'STOPWORD',
    'original': u'هَؤُلَاءِ',
    'encletic': u''
}
STOPWORDS[u'أوبهؤلاء'] = {
    'word': u'أوبهؤلاء',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَبِهَؤُلَاءِ',
    'stem': u'هَؤُلَاءِ',
    'type': u'STOPWORD',
    'original': u'هَؤُلَاءِ',
    'encletic': u''
}
STOPWORDS[u'أفبهؤلاء'] = {
    'word': u'أفبهؤلاء',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَبِهَؤُلَاءِ',
    'stem': u'هَؤُلَاءِ',
    'type': u'STOPWORD',
    'original': u'هَؤُلَاءِ',
    'encletic': u''
}
STOPWORDS[u'أوكهؤلاء'] = {
    'word': u'أوكهؤلاء',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَكَهَؤُلَاءِ',
    'stem': u'هَؤُلَاءِ',
    'type': u'STOPWORD',
    'original': u'هَؤُلَاءِ',
    'encletic': u''
}
STOPWORDS[u'أفكهؤلاء'] = {
    'word': u'أفكهؤلاء',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَكَهَؤُلَاءِ',
    'stem': u'هَؤُلَاءِ',
    'type': u'STOPWORD',
    'original': u'هَؤُلَاءِ',
    'encletic': u''
}
STOPWORDS[u'أولهؤلاء'] = {
    'word': u'أولهؤلاء',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَلِهَؤُلَاءِ',
    'stem': u'هَؤُلَاءِ',
    'type': u'STOPWORD',
    'original': u'هَؤُلَاءِ',
    'encletic': u''
}
STOPWORDS[u'أفلهؤلاء'] = {
    'word': u'أفلهؤلاء',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَلِهَؤُلَاءِ',
    'stem': u'هَؤُلَاءِ',
    'type': u'STOPWORD',
    'original': u'هَؤُلَاءِ',
    'encletic': u''
}
STOPWORDS[u'هاتان'] = {
    'word': u'هَاتَانِ',
    'procletic': u'',
    'tags': u'أداة:اسم إشارة',
    'vocalized': u'هَاتَانِ',
    'stem': u'هَاتَانِ',
    'type': u'STOPWORD',
    'original': u'هَاتَانِ',
    'encletic': u''
}
STOPWORDS[u'وهاتان'] = {
    'word': u'وهاتان',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إشارة:معطوف',
    'vocalized': u'وَهَاتَانِ',
    'stem': u'هَاتَانِ',
    'type': u'STOPWORD',
    'original': u'هَاتَانِ',
    'encletic': u''
}
STOPWORDS[u'فهاتان'] = {
    'word': u'فهاتان',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إشارة:معطوف',
    'vocalized': u'فَهَاتَانِ',
    'stem': u'هَاتَانِ',
    'type': u'STOPWORD',
    'original': u'هَاتَانِ',
    'encletic': u''
}
STOPWORDS[u'أهاتان'] = {
    'word': u'أهاتان',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'أَهَاتَانِ',
    'stem': u'هَاتَانِ',
    'type': u'STOPWORD',
    'original': u'هَاتَانِ',
    'encletic': u''
}
STOPWORDS[u'أوهاتان'] = {
    'word': u'أوهاتان',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إشارة:معطوف:مجرور',
    'vocalized': u'أَوَهَاتَانِ',
    'stem': u'هَاتَانِ',
    'type': u'STOPWORD',
    'original': u'هَاتَانِ',
    'encletic': u''
}
STOPWORDS[u'أفهاتان'] = {
    'word': u'أفهاتان',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إشارة:معطوف:مجرور',
    'vocalized': u'أَفَهَاتَانِ',
    'stem': u'هَاتَانِ',
    'type': u'STOPWORD',
    'original': u'هَاتَانِ',
    'encletic': u''
}
STOPWORDS[u'هاته'] = {
    'word': u'هَاتِهِ',
    'procletic': u'',
    'tags': u'أداة:اسم إشارة',
    'vocalized': u'هَاتِهِ',
    'stem': u'هَاتِهِ',
    'type': u'STOPWORD',
    'original': u'هَاتِهِ',
    'encletic': u''
}
STOPWORDS[u'بهاته'] = {
    'word': u'بهاته',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'بِهَاتِهِ',
    'stem': u'هَاتِهِ',
    'type': u'STOPWORD',
    'original': u'هَاتِهِ',
    'encletic': u''
}
STOPWORDS[u'كهاته'] = {
    'word': u'كهاته',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'كَهَاتِهِ',
    'stem': u'هَاتِهِ',
    'type': u'STOPWORD',
    'original': u'هَاتِهِ',
    'encletic': u''
}
STOPWORDS[u'لهاته'] = {
    'word': u'لهاته',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'لِهَاتِهِ',
    'stem': u'هَاتِهِ',
    'type': u'STOPWORD',
    'original': u'هَاتِهِ',
    'encletic': u''
}
STOPWORDS[u'وهاته'] = {
    'word': u'وهاته',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إشارة:معطوف',
    'vocalized': u'وَهَاتِهِ',
    'stem': u'هَاتِهِ',
    'type': u'STOPWORD',
    'original': u'هَاتِهِ',
    'encletic': u''
}
STOPWORDS[u'فهاته'] = {
    'word': u'فهاته',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إشارة:معطوف',
    'vocalized': u'فَهَاتِهِ',
    'stem': u'هَاتِهِ',
    'type': u'STOPWORD',
    'original': u'هَاتِهِ',
    'encletic': u''
}
STOPWORDS[u'وبهاته'] = {
    'word': u'وبهاته',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'وَبِهَاتِهِ',
    'stem': u'هَاتِهِ',
    'type': u'STOPWORD',
    'original': u'هَاتِهِ',
    'encletic': u''
}
STOPWORDS[u'فبهاته'] = {
    'word': u'فبهاته',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'فَبِهَاتِهِ',
    'stem': u'هَاتِهِ',
    'type': u'STOPWORD',
    'original': u'هَاتِهِ',
    'encletic': u''
}
STOPWORDS[u'وكهاته'] = {
    'word': u'وكهاته',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'وَكَهَاتِهِ',
    'stem': u'هَاتِهِ',
    'type': u'STOPWORD',
    'original': u'هَاتِهِ',
    'encletic': u''
}
STOPWORDS[u'فكهاته'] = {
    'word': u'فكهاته',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'فَكَهَاتِهِ',
    'stem': u'هَاتِهِ',
    'type': u'STOPWORD',
    'original': u'هَاتِهِ',
    'encletic': u''
}
STOPWORDS[u'ولهاته'] = {
    'word': u'ولهاته',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'وَلِهَاتِهِ',
    'stem': u'هَاتِهِ',
    'type': u'STOPWORD',
    'original': u'هَاتِهِ',
    'encletic': u''
}
STOPWORDS[u'فلهاته'] = {
    'word': u'فلهاته',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'فَلِهَاتِهِ',
    'stem': u'هَاتِهِ',
    'type': u'STOPWORD',
    'original': u'هَاتِهِ',
    'encletic': u''
}
STOPWORDS[u'أهاته'] = {
    'word': u'أهاته',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'أَهَاتِهِ',
    'stem': u'هَاتِهِ',
    'type': u'STOPWORD',
    'original': u'هَاتِهِ',
    'encletic': u''
}
STOPWORDS[u'أبهاته'] = {
    'word': u'أبهاته',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:مجرور',
    'vocalized': u'أَبِهَاتِهِ',
    'stem': u'هَاتِهِ',
    'type': u'STOPWORD',
    'original': u'هَاتِهِ',
    'encletic': u''
}
STOPWORDS[u'أكهاته'] = {
    'word': u'أكهاته',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:مجرور',
    'vocalized': u'أَكَهَاتِهِ',
    'stem': u'هَاتِهِ',
    'type': u'STOPWORD',
    'original': u'هَاتِهِ',
    'encletic': u''
}
STOPWORDS[u'ألهاته'] = {
    'word': u'ألهاته',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:مجرور',
    'vocalized': u'أَلِهَاتِهِ',
    'stem': u'هَاتِهِ',
    'type': u'STOPWORD',
    'original': u'هَاتِهِ',
    'encletic': u''
}
STOPWORDS[u'أوهاته'] = {
    'word': u'أوهاته',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إشارة:معطوف:مجرور',
    'vocalized': u'أَوَهَاتِهِ',
    'stem': u'هَاتِهِ',
    'type': u'STOPWORD',
    'original': u'هَاتِهِ',
    'encletic': u''
}
STOPWORDS[u'أفهاته'] = {
    'word': u'أفهاته',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إشارة:معطوف:مجرور',
    'vocalized': u'أَفَهَاتِهِ',
    'stem': u'هَاتِهِ',
    'type': u'STOPWORD',
    'original': u'هَاتِهِ',
    'encletic': u''
}
STOPWORDS[u'أوبهاته'] = {
    'word': u'أوبهاته',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَبِهَاتِهِ',
    'stem': u'هَاتِهِ',
    'type': u'STOPWORD',
    'original': u'هَاتِهِ',
    'encletic': u''
}
STOPWORDS[u'أفبهاته'] = {
    'word': u'أفبهاته',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَبِهَاتِهِ',
    'stem': u'هَاتِهِ',
    'type': u'STOPWORD',
    'original': u'هَاتِهِ',
    'encletic': u''
}
STOPWORDS[u'أوكهاته'] = {
    'word': u'أوكهاته',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَكَهَاتِهِ',
    'stem': u'هَاتِهِ',
    'type': u'STOPWORD',
    'original': u'هَاتِهِ',
    'encletic': u''
}
STOPWORDS[u'أفكهاته'] = {
    'word': u'أفكهاته',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَكَهَاتِهِ',
    'stem': u'هَاتِهِ',
    'type': u'STOPWORD',
    'original': u'هَاتِهِ',
    'encletic': u''
}
STOPWORDS[u'أولهاته'] = {
    'word': u'أولهاته',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَلِهَاتِهِ',
    'stem': u'هَاتِهِ',
    'type': u'STOPWORD',
    'original': u'هَاتِهِ',
    'encletic': u''
}
STOPWORDS[u'أفلهاته'] = {
    'word': u'أفلهاته',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَلِهَاتِهِ',
    'stem': u'هَاتِهِ',
    'type': u'STOPWORD',
    'original': u'هَاتِهِ',
    'encletic': u''
}
STOPWORDS[u'هاتي'] = {
    'word': u'هَاتِي',
    'procletic': u'',
    'tags': u'أداة:اسم إشارة',
    'vocalized': u'هَاتِي',
    'stem': u'هَاتِي',
    'type': u'STOPWORD',
    'original': u'هَاتِي',
    'encletic': u''
}
STOPWORDS[u'بهاتي'] = {
    'word': u'بهاتي',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'بِهَاتِي',
    'stem': u'هَاتِي',
    'type': u'STOPWORD',
    'original': u'هَاتِي',
    'encletic': u''
}
STOPWORDS[u'كهاتي'] = {
    'word': u'كهاتي',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'كَهَاتِي',
    'stem': u'هَاتِي',
    'type': u'STOPWORD',
    'original': u'هَاتِي',
    'encletic': u''
}
STOPWORDS[u'لهاتي'] = {
    'word': u'لهاتي',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'لِهَاتِي',
    'stem': u'هَاتِي',
    'type': u'STOPWORD',
    'original': u'هَاتِي',
    'encletic': u''
}
STOPWORDS[u'وهاتي'] = {
    'word': u'وهاتي',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إشارة:معطوف',
    'vocalized': u'وَهَاتِي',
    'stem': u'هَاتِي',
    'type': u'STOPWORD',
    'original': u'هَاتِي',
    'encletic': u''
}
STOPWORDS[u'فهاتي'] = {
    'word': u'فهاتي',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إشارة:معطوف',
    'vocalized': u'فَهَاتِي',
    'stem': u'هَاتِي',
    'type': u'STOPWORD',
    'original': u'هَاتِي',
    'encletic': u''
}
STOPWORDS[u'وبهاتي'] = {
    'word': u'وبهاتي',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'وَبِهَاتِي',
    'stem': u'هَاتِي',
    'type': u'STOPWORD',
    'original': u'هَاتِي',
    'encletic': u''
}
STOPWORDS[u'فبهاتي'] = {
    'word': u'فبهاتي',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'فَبِهَاتِي',
    'stem': u'هَاتِي',
    'type': u'STOPWORD',
    'original': u'هَاتِي',
    'encletic': u''
}
STOPWORDS[u'وكهاتي'] = {
    'word': u'وكهاتي',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'وَكَهَاتِي',
    'stem': u'هَاتِي',
    'type': u'STOPWORD',
    'original': u'هَاتِي',
    'encletic': u''
}
STOPWORDS[u'فكهاتي'] = {
    'word': u'فكهاتي',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'فَكَهَاتِي',
    'stem': u'هَاتِي',
    'type': u'STOPWORD',
    'original': u'هَاتِي',
    'encletic': u''
}
STOPWORDS[u'ولهاتي'] = {
    'word': u'ولهاتي',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'وَلِهَاتِي',
    'stem': u'هَاتِي',
    'type': u'STOPWORD',
    'original': u'هَاتِي',
    'encletic': u''
}
STOPWORDS[u'فلهاتي'] = {
    'word': u'فلهاتي',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'فَلِهَاتِي',
    'stem': u'هَاتِي',
    'type': u'STOPWORD',
    'original': u'هَاتِي',
    'encletic': u''
}
STOPWORDS[u'أهاتي'] = {
    'word': u'أهاتي',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'أَهَاتِي',
    'stem': u'هَاتِي',
    'type': u'STOPWORD',
    'original': u'هَاتِي',
    'encletic': u''
}
STOPWORDS[u'أبهاتي'] = {
    'word': u'أبهاتي',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:مجرور',
    'vocalized': u'أَبِهَاتِي',
    'stem': u'هَاتِي',
    'type': u'STOPWORD',
    'original': u'هَاتِي',
    'encletic': u''
}
STOPWORDS[u'أكهاتي'] = {
    'word': u'أكهاتي',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:مجرور',
    'vocalized': u'أَكَهَاتِي',
    'stem': u'هَاتِي',
    'type': u'STOPWORD',
    'original': u'هَاتِي',
    'encletic': u''
}
STOPWORDS[u'ألهاتي'] = {
    'word': u'ألهاتي',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:مجرور',
    'vocalized': u'أَلِهَاتِي',
    'stem': u'هَاتِي',
    'type': u'STOPWORD',
    'original': u'هَاتِي',
    'encletic': u''
}
STOPWORDS[u'أوهاتي'] = {
    'word': u'أوهاتي',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إشارة:معطوف:مجرور',
    'vocalized': u'أَوَهَاتِي',
    'stem': u'هَاتِي',
    'type': u'STOPWORD',
    'original': u'هَاتِي',
    'encletic': u''
}
STOPWORDS[u'أفهاتي'] = {
    'word': u'أفهاتي',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إشارة:معطوف:مجرور',
    'vocalized': u'أَفَهَاتِي',
    'stem': u'هَاتِي',
    'type': u'STOPWORD',
    'original': u'هَاتِي',
    'encletic': u''
}
STOPWORDS[u'أوبهاتي'] = {
    'word': u'أوبهاتي',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَبِهَاتِي',
    'stem': u'هَاتِي',
    'type': u'STOPWORD',
    'original': u'هَاتِي',
    'encletic': u''
}
STOPWORDS[u'أفبهاتي'] = {
    'word': u'أفبهاتي',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَبِهَاتِي',
    'stem': u'هَاتِي',
    'type': u'STOPWORD',
    'original': u'هَاتِي',
    'encletic': u''
}
STOPWORDS[u'أوكهاتي'] = {
    'word': u'أوكهاتي',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَكَهَاتِي',
    'stem': u'هَاتِي',
    'type': u'STOPWORD',
    'original': u'هَاتِي',
    'encletic': u''
}
STOPWORDS[u'أفكهاتي'] = {
    'word': u'أفكهاتي',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَكَهَاتِي',
    'stem': u'هَاتِي',
    'type': u'STOPWORD',
    'original': u'هَاتِي',
    'encletic': u''
}
STOPWORDS[u'أولهاتي'] = {
    'word': u'أولهاتي',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَلِهَاتِي',
    'stem': u'هَاتِي',
    'type': u'STOPWORD',
    'original': u'هَاتِي',
    'encletic': u''
}
STOPWORDS[u'أفلهاتي'] = {
    'word': u'أفلهاتي',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَلِهَاتِي',
    'stem': u'هَاتِي',
    'type': u'STOPWORD',
    'original': u'هَاتِي',
    'encletic': u''
}
STOPWORDS[u'هاتين'] = {
    'word': u'هَاتَيْنِ',
    'procletic': u'',
    'tags': u'أداة:اسم إشارة',
    'vocalized': u'هَاتَيْنِ',
    'stem': u'هَاتَيْنِ',
    'type': u'STOPWORD',
    'original': u'هَاتَيْنِ',
    'encletic': u''
}
STOPWORDS[u'بهاتين'] = {
    'word': u'بهاتين',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'بِهَاتَيْنِ',
    'stem': u'هَاتَيْنِ',
    'type': u'STOPWORD',
    'original': u'هَاتَيْنِ',
    'encletic': u''
}
STOPWORDS[u'كهاتين'] = {
    'word': u'كهاتين',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'كَهَاتَيْنِ',
    'stem': u'هَاتَيْنِ',
    'type': u'STOPWORD',
    'original': u'هَاتَيْنِ',
    'encletic': u''
}
STOPWORDS[u'لهاتين'] = {
    'word': u'لهاتين',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'لِهَاتَيْنِ',
    'stem': u'هَاتَيْنِ',
    'type': u'STOPWORD',
    'original': u'هَاتَيْنِ',
    'encletic': u''
}
STOPWORDS[u'وهاتين'] = {
    'word': u'وهاتين',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إشارة:معطوف',
    'vocalized': u'وَهَاتَيْنِ',
    'stem': u'هَاتَيْنِ',
    'type': u'STOPWORD',
    'original': u'هَاتَيْنِ',
    'encletic': u''
}
STOPWORDS[u'فهاتين'] = {
    'word': u'فهاتين',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إشارة:معطوف',
    'vocalized': u'فَهَاتَيْنِ',
    'stem': u'هَاتَيْنِ',
    'type': u'STOPWORD',
    'original': u'هَاتَيْنِ',
    'encletic': u''
}
STOPWORDS[u'وبهاتين'] = {
    'word': u'وبهاتين',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'وَبِهَاتَيْنِ',
    'stem': u'هَاتَيْنِ',
    'type': u'STOPWORD',
    'original': u'هَاتَيْنِ',
    'encletic': u''
}
STOPWORDS[u'فبهاتين'] = {
    'word': u'فبهاتين',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'فَبِهَاتَيْنِ',
    'stem': u'هَاتَيْنِ',
    'type': u'STOPWORD',
    'original': u'هَاتَيْنِ',
    'encletic': u''
}
STOPWORDS[u'وكهاتين'] = {
    'word': u'وكهاتين',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'وَكَهَاتَيْنِ',
    'stem': u'هَاتَيْنِ',
    'type': u'STOPWORD',
    'original': u'هَاتَيْنِ',
    'encletic': u''
}
STOPWORDS[u'فكهاتين'] = {
    'word': u'فكهاتين',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'فَكَهَاتَيْنِ',
    'stem': u'هَاتَيْنِ',
    'type': u'STOPWORD',
    'original': u'هَاتَيْنِ',
    'encletic': u''
}
STOPWORDS[u'ولهاتين'] = {
    'word': u'ولهاتين',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'وَلِهَاتَيْنِ',
    'stem': u'هَاتَيْنِ',
    'type': u'STOPWORD',
    'original': u'هَاتَيْنِ',
    'encletic': u''
}
STOPWORDS[u'فلهاتين'] = {
    'word': u'فلهاتين',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'فَلِهَاتَيْنِ',
    'stem': u'هَاتَيْنِ',
    'type': u'STOPWORD',
    'original': u'هَاتَيْنِ',
    'encletic': u''
}
STOPWORDS[u'أهاتين'] = {
    'word': u'أهاتين',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'أَهَاتَيْنِ',
    'stem': u'هَاتَيْنِ',
    'type': u'STOPWORD',
    'original': u'هَاتَيْنِ',
    'encletic': u''
}
STOPWORDS[u'أبهاتين'] = {
    'word': u'أبهاتين',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:مجرور',
    'vocalized': u'أَبِهَاتَيْنِ',
    'stem': u'هَاتَيْنِ',
    'type': u'STOPWORD',
    'original': u'هَاتَيْنِ',
    'encletic': u''
}
STOPWORDS[u'أكهاتين'] = {
    'word': u'أكهاتين',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:مجرور',
    'vocalized': u'أَكَهَاتَيْنِ',
    'stem': u'هَاتَيْنِ',
    'type': u'STOPWORD',
    'original': u'هَاتَيْنِ',
    'encletic': u''
}
STOPWORDS[u'ألهاتين'] = {
    'word': u'ألهاتين',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:مجرور',
    'vocalized': u'أَلِهَاتَيْنِ',
    'stem': u'هَاتَيْنِ',
    'type': u'STOPWORD',
    'original': u'هَاتَيْنِ',
    'encletic': u''
}
STOPWORDS[u'أوهاتين'] = {
    'word': u'أوهاتين',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إشارة:معطوف:مجرور',
    'vocalized': u'أَوَهَاتَيْنِ',
    'stem': u'هَاتَيْنِ',
    'type': u'STOPWORD',
    'original': u'هَاتَيْنِ',
    'encletic': u''
}
STOPWORDS[u'أفهاتين'] = {
    'word': u'أفهاتين',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إشارة:معطوف:مجرور',
    'vocalized': u'أَفَهَاتَيْنِ',
    'stem': u'هَاتَيْنِ',
    'type': u'STOPWORD',
    'original': u'هَاتَيْنِ',
    'encletic': u''
}
STOPWORDS[u'أوبهاتين'] = {
    'word': u'أوبهاتين',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَبِهَاتَيْنِ',
    'stem': u'هَاتَيْنِ',
    'type': u'STOPWORD',
    'original': u'هَاتَيْنِ',
    'encletic': u''
}
STOPWORDS[u'أفبهاتين'] = {
    'word': u'أفبهاتين',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَبِهَاتَيْنِ',
    'stem': u'هَاتَيْنِ',
    'type': u'STOPWORD',
    'original': u'هَاتَيْنِ',
    'encletic': u''
}
STOPWORDS[u'أوكهاتين'] = {
    'word': u'أوكهاتين',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَكَهَاتَيْنِ',
    'stem': u'هَاتَيْنِ',
    'type': u'STOPWORD',
    'original': u'هَاتَيْنِ',
    'encletic': u''
}
STOPWORDS[u'أفكهاتين'] = {
    'word': u'أفكهاتين',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَكَهَاتَيْنِ',
    'stem': u'هَاتَيْنِ',
    'type': u'STOPWORD',
    'original': u'هَاتَيْنِ',
    'encletic': u''
}
STOPWORDS[u'أولهاتين'] = {
    'word': u'أولهاتين',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَلِهَاتَيْنِ',
    'stem': u'هَاتَيْنِ',
    'type': u'STOPWORD',
    'original': u'هَاتَيْنِ',
    'encletic': u''
}
STOPWORDS[u'أفلهاتين'] = {
    'word': u'أفلهاتين',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَلِهَاتَيْنِ',
    'stem': u'هَاتَيْنِ',
    'type': u'STOPWORD',
    'original': u'هَاتَيْنِ',
    'encletic': u''
}
STOPWORDS[u'هاهنا'] = {
    'word': u'هَاهُنَا',
    'procletic': u'',
    'tags': u'أداة:اسم إشارة',
    'vocalized': u'هَاهُنَا',
    'stem': u'هَاهُنَا',
    'type': u'STOPWORD',
    'original': u'هَاهُنَا',
    'encletic': u''
}
STOPWORDS[u'وهاهنا'] = {
    'word': u'وهاهنا',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إشارة:معطوف',
    'vocalized': u'وَهَاهُنَا',
    'stem': u'هَاهُنَا',
    'type': u'STOPWORD',
    'original': u'هَاهُنَا',
    'encletic': u''
}
STOPWORDS[u'فهاهنا'] = {
    'word': u'فهاهنا',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إشارة:معطوف',
    'vocalized': u'فَهَاهُنَا',
    'stem': u'هَاهُنَا',
    'type': u'STOPWORD',
    'original': u'هَاهُنَا',
    'encletic': u''
}
STOPWORDS[u'أهاهنا'] = {
    'word': u'أهاهنا',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'أَهَاهُنَا',
    'stem': u'هَاهُنَا',
    'type': u'STOPWORD',
    'original': u'هَاهُنَا',
    'encletic': u''
}
STOPWORDS[u'أوهاهنا'] = {
    'word': u'أوهاهنا',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إشارة:معطوف:مجرور',
    'vocalized': u'أَوَهَاهُنَا',
    'stem': u'هَاهُنَا',
    'type': u'STOPWORD',
    'original': u'هَاهُنَا',
    'encletic': u''
}
STOPWORDS[u'أفهاهنا'] = {
    'word': u'أفهاهنا',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إشارة:معطوف:مجرور',
    'vocalized': u'أَفَهَاهُنَا',
    'stem': u'هَاهُنَا',
    'type': u'STOPWORD',
    'original': u'هَاهُنَا',
    'encletic': u''
}
STOPWORDS[u'هذا'] = {
    'word': u'هَذَا',
    'procletic': u'',
    'tags': u'أداة:اسم إشارة',
    'vocalized': u'هَذَا',
    'stem': u'هَذَا',
    'type': u'STOPWORD',
    'original': u'هَذَا',
    'encletic': u''
}
STOPWORDS[u'بهذا'] = {
    'word': u'بهذا',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'بِهَذَا',
    'stem': u'هَذَا',
    'type': u'STOPWORD',
    'original': u'هَذَا',
    'encletic': u''
}
STOPWORDS[u'كهذا'] = {
    'word': u'كهذا',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'كَهَذَا',
    'stem': u'هَذَا',
    'type': u'STOPWORD',
    'original': u'هَذَا',
    'encletic': u''
}
STOPWORDS[u'لهذا'] = {
    'word': u'لهذا',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'لِهَذَا',
    'stem': u'هَذَا',
    'type': u'STOPWORD',
    'original': u'هَذَا',
    'encletic': u''
}
STOPWORDS[u'وهذا'] = {
    'word': u'وهذا',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إشارة:معطوف',
    'vocalized': u'وَهَذَا',
    'stem': u'هَذَا',
    'type': u'STOPWORD',
    'original': u'هَذَا',
    'encletic': u''
}
STOPWORDS[u'فهذا'] = {
    'word': u'فهذا',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إشارة:معطوف',
    'vocalized': u'فَهَذَا',
    'stem': u'هَذَا',
    'type': u'STOPWORD',
    'original': u'هَذَا',
    'encletic': u''
}
STOPWORDS[u'وبهذا'] = {
    'word': u'وبهذا',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'وَبِهَذَا',
    'stem': u'هَذَا',
    'type': u'STOPWORD',
    'original': u'هَذَا',
    'encletic': u''
}
STOPWORDS[u'فبهذا'] = {
    'word': u'فبهذا',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'فَبِهَذَا',
    'stem': u'هَذَا',
    'type': u'STOPWORD',
    'original': u'هَذَا',
    'encletic': u''
}
STOPWORDS[u'وكهذا'] = {
    'word': u'وكهذا',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'وَكَهَذَا',
    'stem': u'هَذَا',
    'type': u'STOPWORD',
    'original': u'هَذَا',
    'encletic': u''
}
STOPWORDS[u'فكهذا'] = {
    'word': u'فكهذا',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'فَكَهَذَا',
    'stem': u'هَذَا',
    'type': u'STOPWORD',
    'original': u'هَذَا',
    'encletic': u''
}
STOPWORDS[u'ولهذا'] = {
    'word': u'ولهذا',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'وَلِهَذَا',
    'stem': u'هَذَا',
    'type': u'STOPWORD',
    'original': u'هَذَا',
    'encletic': u''
}
STOPWORDS[u'فلهذا'] = {
    'word': u'فلهذا',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'فَلِهَذَا',
    'stem': u'هَذَا',
    'type': u'STOPWORD',
    'original': u'هَذَا',
    'encletic': u''
}
STOPWORDS[u'أهذا'] = {
    'word': u'أهذا',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'أَهَذَا',
    'stem': u'هَذَا',
    'type': u'STOPWORD',
    'original': u'هَذَا',
    'encletic': u''
}
STOPWORDS[u'أبهذا'] = {
    'word': u'أبهذا',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:مجرور',
    'vocalized': u'أَبِهَذَا',
    'stem': u'هَذَا',
    'type': u'STOPWORD',
    'original': u'هَذَا',
    'encletic': u''
}
STOPWORDS[u'أكهذا'] = {
    'word': u'أكهذا',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:مجرور',
    'vocalized': u'أَكَهَذَا',
    'stem': u'هَذَا',
    'type': u'STOPWORD',
    'original': u'هَذَا',
    'encletic': u''
}
STOPWORDS[u'ألهذا'] = {
    'word': u'ألهذا',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:مجرور',
    'vocalized': u'أَلِهَذَا',
    'stem': u'هَذَا',
    'type': u'STOPWORD',
    'original': u'هَذَا',
    'encletic': u''
}
STOPWORDS[u'أوهذا'] = {
    'word': u'أوهذا',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إشارة:معطوف:مجرور',
    'vocalized': u'أَوَهَذَا',
    'stem': u'هَذَا',
    'type': u'STOPWORD',
    'original': u'هَذَا',
    'encletic': u''
}
STOPWORDS[u'أفهذا'] = {
    'word': u'أفهذا',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إشارة:معطوف:مجرور',
    'vocalized': u'أَفَهَذَا',
    'stem': u'هَذَا',
    'type': u'STOPWORD',
    'original': u'هَذَا',
    'encletic': u''
}
STOPWORDS[u'أوبهذا'] = {
    'word': u'أوبهذا',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَبِهَذَا',
    'stem': u'هَذَا',
    'type': u'STOPWORD',
    'original': u'هَذَا',
    'encletic': u''
}
STOPWORDS[u'أفبهذا'] = {
    'word': u'أفبهذا',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَبِهَذَا',
    'stem': u'هَذَا',
    'type': u'STOPWORD',
    'original': u'هَذَا',
    'encletic': u''
}
STOPWORDS[u'أوكهذا'] = {
    'word': u'أوكهذا',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَكَهَذَا',
    'stem': u'هَذَا',
    'type': u'STOPWORD',
    'original': u'هَذَا',
    'encletic': u''
}
STOPWORDS[u'أفكهذا'] = {
    'word': u'أفكهذا',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَكَهَذَا',
    'stem': u'هَذَا',
    'type': u'STOPWORD',
    'original': u'هَذَا',
    'encletic': u''
}
STOPWORDS[u'أولهذا'] = {
    'word': u'أولهذا',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَلِهَذَا',
    'stem': u'هَذَا',
    'type': u'STOPWORD',
    'original': u'هَذَا',
    'encletic': u''
}
STOPWORDS[u'أفلهذا'] = {
    'word': u'أفلهذا',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَلِهَذَا',
    'stem': u'هَذَا',
    'type': u'STOPWORD',
    'original': u'هَذَا',
    'encletic': u''
}
STOPWORDS[u'هذان'] = {
    'word': u'هَذَانِ',
    'procletic': u'',
    'tags': u'أداة:اسم إشارة',
    'vocalized': u'هَذَانِ',
    'stem': u'هَذَانِ',
    'type': u'STOPWORD',
    'original': u'هَذَانِ',
    'encletic': u''
}
STOPWORDS[u'وهذان'] = {
    'word': u'وهذان',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إشارة:معطوف',
    'vocalized': u'وَهَذَانِ',
    'stem': u'هَذَانِ',
    'type': u'STOPWORD',
    'original': u'هَذَانِ',
    'encletic': u''
}
STOPWORDS[u'فهذان'] = {
    'word': u'فهذان',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إشارة:معطوف',
    'vocalized': u'فَهَذَانِ',
    'stem': u'هَذَانِ',
    'type': u'STOPWORD',
    'original': u'هَذَانِ',
    'encletic': u''
}
STOPWORDS[u'أهذان'] = {
    'word': u'أهذان',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'أَهَذَانِ',
    'stem': u'هَذَانِ',
    'type': u'STOPWORD',
    'original': u'هَذَانِ',
    'encletic': u''
}
STOPWORDS[u'أوهذان'] = {
    'word': u'أوهذان',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إشارة:معطوف:مجرور',
    'vocalized': u'أَوَهَذَانِ',
    'stem': u'هَذَانِ',
    'type': u'STOPWORD',
    'original': u'هَذَانِ',
    'encletic': u''
}
STOPWORDS[u'أفهذان'] = {
    'word': u'أفهذان',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إشارة:معطوف:مجرور',
    'vocalized': u'أَفَهَذَانِ',
    'stem': u'هَذَانِ',
    'type': u'STOPWORD',
    'original': u'هَذَانِ',
    'encletic': u''
}
STOPWORDS[u'هذه'] = {
    'word': u'هَذِهِ',
    'procletic': u'',
    'tags': u'أداة:اسم إشارة',
    'vocalized': u'هَذِهِ',
    'stem': u'هَذِهِ',
    'type': u'STOPWORD',
    'original': u'هَذِهِ',
    'encletic': u''
}
STOPWORDS[u'بهذه'] = {
    'word': u'بهذه',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'بِهَذِهِ',
    'stem': u'هَذِهِ',
    'type': u'STOPWORD',
    'original': u'هَذِهِ',
    'encletic': u''
}
STOPWORDS[u'كهذه'] = {
    'word': u'كهذه',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'كَهَذِهِ',
    'stem': u'هَذِهِ',
    'type': u'STOPWORD',
    'original': u'هَذِهِ',
    'encletic': u''
}
STOPWORDS[u'لهذه'] = {
    'word': u'لهذه',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'لِهَذِهِ',
    'stem': u'هَذِهِ',
    'type': u'STOPWORD',
    'original': u'هَذِهِ',
    'encletic': u''
}
STOPWORDS[u'وهذه'] = {
    'word': u'وهذه',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إشارة:معطوف',
    'vocalized': u'وَهَذِهِ',
    'stem': u'هَذِهِ',
    'type': u'STOPWORD',
    'original': u'هَذِهِ',
    'encletic': u''
}
STOPWORDS[u'فهذه'] = {
    'word': u'فهذه',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إشارة:معطوف',
    'vocalized': u'فَهَذِهِ',
    'stem': u'هَذِهِ',
    'type': u'STOPWORD',
    'original': u'هَذِهِ',
    'encletic': u''
}
STOPWORDS[u'وبهذه'] = {
    'word': u'وبهذه',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'وَبِهَذِهِ',
    'stem': u'هَذِهِ',
    'type': u'STOPWORD',
    'original': u'هَذِهِ',
    'encletic': u''
}
STOPWORDS[u'فبهذه'] = {
    'word': u'فبهذه',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'فَبِهَذِهِ',
    'stem': u'هَذِهِ',
    'type': u'STOPWORD',
    'original': u'هَذِهِ',
    'encletic': u''
}
STOPWORDS[u'وكهذه'] = {
    'word': u'وكهذه',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'وَكَهَذِهِ',
    'stem': u'هَذِهِ',
    'type': u'STOPWORD',
    'original': u'هَذِهِ',
    'encletic': u''
}
STOPWORDS[u'فكهذه'] = {
    'word': u'فكهذه',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'فَكَهَذِهِ',
    'stem': u'هَذِهِ',
    'type': u'STOPWORD',
    'original': u'هَذِهِ',
    'encletic': u''
}
STOPWORDS[u'ولهذه'] = {
    'word': u'ولهذه',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'وَلِهَذِهِ',
    'stem': u'هَذِهِ',
    'type': u'STOPWORD',
    'original': u'هَذِهِ',
    'encletic': u''
}
STOPWORDS[u'فلهذه'] = {
    'word': u'فلهذه',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'فَلِهَذِهِ',
    'stem': u'هَذِهِ',
    'type': u'STOPWORD',
    'original': u'هَذِهِ',
    'encletic': u''
}
STOPWORDS[u'أهذه'] = {
    'word': u'أهذه',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'أَهَذِهِ',
    'stem': u'هَذِهِ',
    'type': u'STOPWORD',
    'original': u'هَذِهِ',
    'encletic': u''
}
STOPWORDS[u'أبهذه'] = {
    'word': u'أبهذه',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:مجرور',
    'vocalized': u'أَبِهَذِهِ',
    'stem': u'هَذِهِ',
    'type': u'STOPWORD',
    'original': u'هَذِهِ',
    'encletic': u''
}
STOPWORDS[u'أكهذه'] = {
    'word': u'أكهذه',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:مجرور',
    'vocalized': u'أَكَهَذِهِ',
    'stem': u'هَذِهِ',
    'type': u'STOPWORD',
    'original': u'هَذِهِ',
    'encletic': u''
}
STOPWORDS[u'ألهذه'] = {
    'word': u'ألهذه',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:مجرور',
    'vocalized': u'أَلِهَذِهِ',
    'stem': u'هَذِهِ',
    'type': u'STOPWORD',
    'original': u'هَذِهِ',
    'encletic': u''
}
STOPWORDS[u'أوهذه'] = {
    'word': u'أوهذه',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إشارة:معطوف:مجرور',
    'vocalized': u'أَوَهَذِهِ',
    'stem': u'هَذِهِ',
    'type': u'STOPWORD',
    'original': u'هَذِهِ',
    'encletic': u''
}
STOPWORDS[u'أفهذه'] = {
    'word': u'أفهذه',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إشارة:معطوف:مجرور',
    'vocalized': u'أَفَهَذِهِ',
    'stem': u'هَذِهِ',
    'type': u'STOPWORD',
    'original': u'هَذِهِ',
    'encletic': u''
}
STOPWORDS[u'أوبهذه'] = {
    'word': u'أوبهذه',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَبِهَذِهِ',
    'stem': u'هَذِهِ',
    'type': u'STOPWORD',
    'original': u'هَذِهِ',
    'encletic': u''
}
STOPWORDS[u'أفبهذه'] = {
    'word': u'أفبهذه',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَبِهَذِهِ',
    'stem': u'هَذِهِ',
    'type': u'STOPWORD',
    'original': u'هَذِهِ',
    'encletic': u''
}
STOPWORDS[u'أوكهذه'] = {
    'word': u'أوكهذه',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَكَهَذِهِ',
    'stem': u'هَذِهِ',
    'type': u'STOPWORD',
    'original': u'هَذِهِ',
    'encletic': u''
}
STOPWORDS[u'أفكهذه'] = {
    'word': u'أفكهذه',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَكَهَذِهِ',
    'stem': u'هَذِهِ',
    'type': u'STOPWORD',
    'original': u'هَذِهِ',
    'encletic': u''
}
STOPWORDS[u'أولهذه'] = {
    'word': u'أولهذه',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَلِهَذِهِ',
    'stem': u'هَذِهِ',
    'type': u'STOPWORD',
    'original': u'هَذِهِ',
    'encletic': u''
}
STOPWORDS[u'أفلهذه'] = {
    'word': u'أفلهذه',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَلِهَذِهِ',
    'stem': u'هَذِهِ',
    'type': u'STOPWORD',
    'original': u'هَذِهِ',
    'encletic': u''
}
STOPWORDS[u'هذي'] = {
    'word': u'هَذِي',
    'procletic': u'',
    'tags': u'أداة:اسم إشارة',
    'vocalized': u'هَذِي',
    'stem': u'هَذِي',
    'type': u'STOPWORD',
    'original': u'هَذِي',
    'encletic': u''
}
STOPWORDS[u'بهذي'] = {
    'word': u'بهذي',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'بِهَذِي',
    'stem': u'هَذِي',
    'type': u'STOPWORD',
    'original': u'هَذِي',
    'encletic': u''
}
STOPWORDS[u'كهذي'] = {
    'word': u'كهذي',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'كَهَذِي',
    'stem': u'هَذِي',
    'type': u'STOPWORD',
    'original': u'هَذِي',
    'encletic': u''
}
STOPWORDS[u'لهذي'] = {
    'word': u'لهذي',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'لِهَذِي',
    'stem': u'هَذِي',
    'type': u'STOPWORD',
    'original': u'هَذِي',
    'encletic': u''
}
STOPWORDS[u'وهذي'] = {
    'word': u'وهذي',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إشارة:معطوف',
    'vocalized': u'وَهَذِي',
    'stem': u'هَذِي',
    'type': u'STOPWORD',
    'original': u'هَذِي',
    'encletic': u''
}
STOPWORDS[u'فهذي'] = {
    'word': u'فهذي',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إشارة:معطوف',
    'vocalized': u'فَهَذِي',
    'stem': u'هَذِي',
    'type': u'STOPWORD',
    'original': u'هَذِي',
    'encletic': u''
}
STOPWORDS[u'وبهذي'] = {
    'word': u'وبهذي',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'وَبِهَذِي',
    'stem': u'هَذِي',
    'type': u'STOPWORD',
    'original': u'هَذِي',
    'encletic': u''
}
STOPWORDS[u'فبهذي'] = {
    'word': u'فبهذي',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'فَبِهَذِي',
    'stem': u'هَذِي',
    'type': u'STOPWORD',
    'original': u'هَذِي',
    'encletic': u''
}
STOPWORDS[u'وكهذي'] = {
    'word': u'وكهذي',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'وَكَهَذِي',
    'stem': u'هَذِي',
    'type': u'STOPWORD',
    'original': u'هَذِي',
    'encletic': u''
}
STOPWORDS[u'فكهذي'] = {
    'word': u'فكهذي',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'فَكَهَذِي',
    'stem': u'هَذِي',
    'type': u'STOPWORD',
    'original': u'هَذِي',
    'encletic': u''
}
STOPWORDS[u'ولهذي'] = {
    'word': u'ولهذي',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'وَلِهَذِي',
    'stem': u'هَذِي',
    'type': u'STOPWORD',
    'original': u'هَذِي',
    'encletic': u''
}
STOPWORDS[u'فلهذي'] = {
    'word': u'فلهذي',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'فَلِهَذِي',
    'stem': u'هَذِي',
    'type': u'STOPWORD',
    'original': u'هَذِي',
    'encletic': u''
}
STOPWORDS[u'هذين'] = {
    'word': u'هَذَيْنِ',
    'procletic': u'',
    'tags': u'أداة:اسم إشارة',
    'vocalized': u'هَذَيْنِ',
    'stem': u'هَذَيْنِ',
    'type': u'STOPWORD',
    'original': u'هَذَيْنِ',
    'encletic': u''
}
STOPWORDS[u'بهذين'] = {
    'word': u'بهذين',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'بِهَذَيْنِ',
    'stem': u'هَذَيْنِ',
    'type': u'STOPWORD',
    'original': u'هَذَيْنِ',
    'encletic': u''
}
STOPWORDS[u'كهذين'] = {
    'word': u'كهذين',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'كَهَذَيْنِ',
    'stem': u'هَذَيْنِ',
    'type': u'STOPWORD',
    'original': u'هَذَيْنِ',
    'encletic': u''
}
STOPWORDS[u'لهذين'] = {
    'word': u'لهذين',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'لِهَذَيْنِ',
    'stem': u'هَذَيْنِ',
    'type': u'STOPWORD',
    'original': u'هَذَيْنِ',
    'encletic': u''
}
STOPWORDS[u'وهذين'] = {
    'word': u'وهذين',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إشارة:معطوف',
    'vocalized': u'وَهَذَيْنِ',
    'stem': u'هَذَيْنِ',
    'type': u'STOPWORD',
    'original': u'هَذَيْنِ',
    'encletic': u''
}
STOPWORDS[u'فهذين'] = {
    'word': u'فهذين',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إشارة:معطوف',
    'vocalized': u'فَهَذَيْنِ',
    'stem': u'هَذَيْنِ',
    'type': u'STOPWORD',
    'original': u'هَذَيْنِ',
    'encletic': u''
}
STOPWORDS[u'وبهذين'] = {
    'word': u'وبهذين',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'وَبِهَذَيْنِ',
    'stem': u'هَذَيْنِ',
    'type': u'STOPWORD',
    'original': u'هَذَيْنِ',
    'encletic': u''
}
STOPWORDS[u'فبهذين'] = {
    'word': u'فبهذين',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'فَبِهَذَيْنِ',
    'stem': u'هَذَيْنِ',
    'type': u'STOPWORD',
    'original': u'هَذَيْنِ',
    'encletic': u''
}
STOPWORDS[u'وكهذين'] = {
    'word': u'وكهذين',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'وَكَهَذَيْنِ',
    'stem': u'هَذَيْنِ',
    'type': u'STOPWORD',
    'original': u'هَذَيْنِ',
    'encletic': u''
}
STOPWORDS[u'فكهذين'] = {
    'word': u'فكهذين',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'فَكَهَذَيْنِ',
    'stem': u'هَذَيْنِ',
    'type': u'STOPWORD',
    'original': u'هَذَيْنِ',
    'encletic': u''
}
STOPWORDS[u'ولهذين'] = {
    'word': u'ولهذين',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'وَلِهَذَيْنِ',
    'stem': u'هَذَيْنِ',
    'type': u'STOPWORD',
    'original': u'هَذَيْنِ',
    'encletic': u''
}
STOPWORDS[u'فلهذين'] = {
    'word': u'فلهذين',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'فَلِهَذَيْنِ',
    'stem': u'هَذَيْنِ',
    'type': u'STOPWORD',
    'original': u'هَذَيْنِ',
    'encletic': u''
}
STOPWORDS[u'هكذا'] = {
    'word': u'هَكَذَا',
    'procletic': u'',
    'tags': u'أداة:اسم إشارة',
    'vocalized': u'هَكَذَا',
    'stem': u'هَكَذَا',
    'type': u'STOPWORD',
    'original': u'هَكَذَا',
    'encletic': u''
}
STOPWORDS[u'وهكذا'] = {
    'word': u'وهكذا',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إشارة:معطوف',
    'vocalized': u'وَهَكَذَا',
    'stem': u'هَكَذَا',
    'type': u'STOPWORD',
    'original': u'هَكَذَا',
    'encletic': u''
}
STOPWORDS[u'فهكذا'] = {
    'word': u'فهكذا',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إشارة:معطوف',
    'vocalized': u'فَهَكَذَا',
    'stem': u'هَكَذَا',
    'type': u'STOPWORD',
    'original': u'هَكَذَا',
    'encletic': u''
}
STOPWORDS[u'أهكذا'] = {
    'word': u'أهكذا',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'أَهَكَذَا',
    'stem': u'هَكَذَا',
    'type': u'STOPWORD',
    'original': u'هَكَذَا',
    'encletic': u''
}
STOPWORDS[u'أوهكذا'] = {
    'word': u'أوهكذا',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إشارة:معطوف:مجرور',
    'vocalized': u'أَوَهَكَذَا',
    'stem': u'هَكَذَا',
    'type': u'STOPWORD',
    'original': u'هَكَذَا',
    'encletic': u''
}
STOPWORDS[u'أفهكذا'] = {
    'word': u'أفهكذا',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إشارة:معطوف:مجرور',
    'vocalized': u'أَفَهَكَذَا',
    'stem': u'هَكَذَا',
    'type': u'STOPWORD',
    'original': u'هَكَذَا',
    'encletic': u''
}
STOPWORDS[u'هنا'] = {
    'word': u'هُنَا',
    'procletic': u'',
    'tags': u'أداة:اسم إشارة',
    'vocalized': u'هُنَا',
    'stem': u'هُنَا',
    'type': u'STOPWORD',
    'original': u'هُنَا',
    'encletic': u''
}
STOPWORDS[u'وهنا'] = {
    'word': u'وهنا',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إشارة:معطوف',
    'vocalized': u'وَهُنَا',
    'stem': u'هُنَا',
    'type': u'STOPWORD',
    'original': u'هُنَا',
    'encletic': u''
}
STOPWORDS[u'فهنا'] = {
    'word': u'فهنا',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إشارة:معطوف',
    'vocalized': u'فَهُنَا',
    'stem': u'هُنَا',
    'type': u'STOPWORD',
    'original': u'هُنَا',
    'encletic': u''
}
STOPWORDS[u'أهنا'] = {
    'word': u'أهنا',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'أَهُنَا',
    'stem': u'هُنَا',
    'type': u'STOPWORD',
    'original': u'هُنَا',
    'encletic': u''
}
STOPWORDS[u'أوهنا'] = {
    'word': u'أوهنا',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إشارة:معطوف:مجرور',
    'vocalized': u'أَوَهُنَا',
    'stem': u'هُنَا',
    'type': u'STOPWORD',
    'original': u'هُنَا',
    'encletic': u''
}
STOPWORDS[u'أفهنا'] = {
    'word': u'أفهنا',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إشارة:معطوف:مجرور',
    'vocalized': u'أَفَهُنَا',
    'stem': u'هُنَا',
    'type': u'STOPWORD',
    'original': u'هُنَا',
    'encletic': u''
}
STOPWORDS[u'هناك'] = {
    'word': u'هُنَاكَ',
    'procletic': u'',
    'tags': u'أداة:اسم إشارة',
    'vocalized': u'هُنَاكَ',
    'stem': u'هُنَاكَ',
    'type': u'STOPWORD',
    'original': u'هُنَاكَ',
    'encletic': u''
}
STOPWORDS[u'بهناك'] = {
    'word': u'بهناك',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'بِهُنَاكَ',
    'stem': u'هُنَاكَ',
    'type': u'STOPWORD',
    'original': u'هُنَاكَ',
    'encletic': u''
}
STOPWORDS[u'كهناك'] = {
    'word': u'كهناك',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'كَهُنَاكَ',
    'stem': u'هُنَاكَ',
    'type': u'STOPWORD',
    'original': u'هُنَاكَ',
    'encletic': u''
}
STOPWORDS[u'لهناك'] = {
    'word': u'لهناك',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'لِهُنَاكَ',
    'stem': u'هُنَاكَ',
    'type': u'STOPWORD',
    'original': u'هُنَاكَ',
    'encletic': u''
}
STOPWORDS[u'وهناك'] = {
    'word': u'وهناك',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إشارة:معطوف',
    'vocalized': u'وَهُنَاكَ',
    'stem': u'هُنَاكَ',
    'type': u'STOPWORD',
    'original': u'هُنَاكَ',
    'encletic': u''
}
STOPWORDS[u'فهناك'] = {
    'word': u'فهناك',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إشارة:معطوف',
    'vocalized': u'فَهُنَاكَ',
    'stem': u'هُنَاكَ',
    'type': u'STOPWORD',
    'original': u'هُنَاكَ',
    'encletic': u''
}
STOPWORDS[u'وبهناك'] = {
    'word': u'وبهناك',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'وَبِهُنَاكَ',
    'stem': u'هُنَاكَ',
    'type': u'STOPWORD',
    'original': u'هُنَاكَ',
    'encletic': u''
}
STOPWORDS[u'فبهناك'] = {
    'word': u'فبهناك',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'فَبِهُنَاكَ',
    'stem': u'هُنَاكَ',
    'type': u'STOPWORD',
    'original': u'هُنَاكَ',
    'encletic': u''
}
STOPWORDS[u'وكهناك'] = {
    'word': u'وكهناك',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'وَكَهُنَاكَ',
    'stem': u'هُنَاكَ',
    'type': u'STOPWORD',
    'original': u'هُنَاكَ',
    'encletic': u''
}
STOPWORDS[u'فكهناك'] = {
    'word': u'فكهناك',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'فَكَهُنَاكَ',
    'stem': u'هُنَاكَ',
    'type': u'STOPWORD',
    'original': u'هُنَاكَ',
    'encletic': u''
}
STOPWORDS[u'ولهناك'] = {
    'word': u'ولهناك',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'وَلِهُنَاكَ',
    'stem': u'هُنَاكَ',
    'type': u'STOPWORD',
    'original': u'هُنَاكَ',
    'encletic': u''
}
STOPWORDS[u'فلهناك'] = {
    'word': u'فلهناك',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'فَلِهُنَاكَ',
    'stem': u'هُنَاكَ',
    'type': u'STOPWORD',
    'original': u'هُنَاكَ',
    'encletic': u''
}
STOPWORDS[u'أهناك'] = {
    'word': u'أهناك',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'أَهُنَاكَ',
    'stem': u'هُنَاكَ',
    'type': u'STOPWORD',
    'original': u'هُنَاكَ',
    'encletic': u''
}
STOPWORDS[u'أبهناك'] = {
    'word': u'أبهناك',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:مجرور',
    'vocalized': u'أَبِهُنَاكَ',
    'stem': u'هُنَاكَ',
    'type': u'STOPWORD',
    'original': u'هُنَاكَ',
    'encletic': u''
}
STOPWORDS[u'أكهناك'] = {
    'word': u'أكهناك',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:مجرور',
    'vocalized': u'أَكَهُنَاكَ',
    'stem': u'هُنَاكَ',
    'type': u'STOPWORD',
    'original': u'هُنَاكَ',
    'encletic': u''
}
STOPWORDS[u'ألهناك'] = {
    'word': u'ألهناك',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:مجرور',
    'vocalized': u'أَلِهُنَاكَ',
    'stem': u'هُنَاكَ',
    'type': u'STOPWORD',
    'original': u'هُنَاكَ',
    'encletic': u''
}
STOPWORDS[u'أوهناك'] = {
    'word': u'أوهناك',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إشارة:معطوف:مجرور',
    'vocalized': u'أَوَهُنَاكَ',
    'stem': u'هُنَاكَ',
    'type': u'STOPWORD',
    'original': u'هُنَاكَ',
    'encletic': u''
}
STOPWORDS[u'أفهناك'] = {
    'word': u'أفهناك',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إشارة:معطوف:مجرور',
    'vocalized': u'أَفَهُنَاكَ',
    'stem': u'هُنَاكَ',
    'type': u'STOPWORD',
    'original': u'هُنَاكَ',
    'encletic': u''
}
STOPWORDS[u'أوبهناك'] = {
    'word': u'أوبهناك',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَبِهُنَاكَ',
    'stem': u'هُنَاكَ',
    'type': u'STOPWORD',
    'original': u'هُنَاكَ',
    'encletic': u''
}
STOPWORDS[u'أفبهناك'] = {
    'word': u'أفبهناك',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَبِهُنَاكَ',
    'stem': u'هُنَاكَ',
    'type': u'STOPWORD',
    'original': u'هُنَاكَ',
    'encletic': u''
}
STOPWORDS[u'أوكهناك'] = {
    'word': u'أوكهناك',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَكَهُنَاكَ',
    'stem': u'هُنَاكَ',
    'type': u'STOPWORD',
    'original': u'هُنَاكَ',
    'encletic': u''
}
STOPWORDS[u'أفكهناك'] = {
    'word': u'أفكهناك',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَكَهُنَاكَ',
    'stem': u'هُنَاكَ',
    'type': u'STOPWORD',
    'original': u'هُنَاكَ',
    'encletic': u''
}
STOPWORDS[u'أولهناك'] = {
    'word': u'أولهناك',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَلِهُنَاكَ',
    'stem': u'هُنَاكَ',
    'type': u'STOPWORD',
    'original': u'هُنَاكَ',
    'encletic': u''
}
STOPWORDS[u'أفلهناك'] = {
    'word': u'أفلهناك',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَلِهُنَاكَ',
    'stem': u'هُنَاكَ',
    'type': u'STOPWORD',
    'original': u'هُنَاكَ',
    'encletic': u''
}
STOPWORDS[u'هنالك'] = {
    'word': u'هُنَالِكَ',
    'procletic': u'',
    'tags': u'أداة:اسم إشارة',
    'vocalized': u'هُنَالِكَ',
    'stem': u'هُنَالِكَ',
    'type': u'STOPWORD',
    'original': u'هُنَالِكَ',
    'encletic': u''
}
STOPWORDS[u'بهنالك'] = {
    'word': u'بهنالك',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'بِهُنَالِكَ',
    'stem': u'هُنَالِكَ',
    'type': u'STOPWORD',
    'original': u'هُنَالِكَ',
    'encletic': u''
}
STOPWORDS[u'كهنالك'] = {
    'word': u'كهنالك',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'كَهُنَالِكَ',
    'stem': u'هُنَالِكَ',
    'type': u'STOPWORD',
    'original': u'هُنَالِكَ',
    'encletic': u''
}
STOPWORDS[u'لهنالك'] = {
    'word': u'لهنالك',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'لِهُنَالِكَ',
    'stem': u'هُنَالِكَ',
    'type': u'STOPWORD',
    'original': u'هُنَالِكَ',
    'encletic': u''
}
STOPWORDS[u'وهنالك'] = {
    'word': u'وهنالك',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إشارة:معطوف',
    'vocalized': u'وَهُنَالِكَ',
    'stem': u'هُنَالِكَ',
    'type': u'STOPWORD',
    'original': u'هُنَالِكَ',
    'encletic': u''
}
STOPWORDS[u'فهنالك'] = {
    'word': u'فهنالك',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إشارة:معطوف',
    'vocalized': u'فَهُنَالِكَ',
    'stem': u'هُنَالِكَ',
    'type': u'STOPWORD',
    'original': u'هُنَالِكَ',
    'encletic': u''
}
STOPWORDS[u'وبهنالك'] = {
    'word': u'وبهنالك',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'وَبِهُنَالِكَ',
    'stem': u'هُنَالِكَ',
    'type': u'STOPWORD',
    'original': u'هُنَالِكَ',
    'encletic': u''
}
STOPWORDS[u'فبهنالك'] = {
    'word': u'فبهنالك',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'فَبِهُنَالِكَ',
    'stem': u'هُنَالِكَ',
    'type': u'STOPWORD',
    'original': u'هُنَالِكَ',
    'encletic': u''
}
STOPWORDS[u'وكهنالك'] = {
    'word': u'وكهنالك',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'وَكَهُنَالِكَ',
    'stem': u'هُنَالِكَ',
    'type': u'STOPWORD',
    'original': u'هُنَالِكَ',
    'encletic': u''
}
STOPWORDS[u'فكهنالك'] = {
    'word': u'فكهنالك',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'فَكَهُنَالِكَ',
    'stem': u'هُنَالِكَ',
    'type': u'STOPWORD',
    'original': u'هُنَالِكَ',
    'encletic': u''
}
STOPWORDS[u'ولهنالك'] = {
    'word': u'ولهنالك',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'وَلِهُنَالِكَ',
    'stem': u'هُنَالِكَ',
    'type': u'STOPWORD',
    'original': u'هُنَالِكَ',
    'encletic': u''
}
STOPWORDS[u'فلهنالك'] = {
    'word': u'فلهنالك',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف',
    'vocalized': u'فَلِهُنَالِكَ',
    'stem': u'هُنَالِكَ',
    'type': u'STOPWORD',
    'original': u'هُنَالِكَ',
    'encletic': u''
}
STOPWORDS[u'أهنالك'] = {
    'word': u'أهنالك',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إشارة:مجرور',
    'vocalized': u'أَهُنَالِكَ',
    'stem': u'هُنَالِكَ',
    'type': u'STOPWORD',
    'original': u'هُنَالِكَ',
    'encletic': u''
}
STOPWORDS[u'أبهنالك'] = {
    'word': u'أبهنالك',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:مجرور',
    'vocalized': u'أَبِهُنَالِكَ',
    'stem': u'هُنَالِكَ',
    'type': u'STOPWORD',
    'original': u'هُنَالِكَ',
    'encletic': u''
}
STOPWORDS[u'أكهنالك'] = {
    'word': u'أكهنالك',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:مجرور',
    'vocalized': u'أَكَهُنَالِكَ',
    'stem': u'هُنَالِكَ',
    'type': u'STOPWORD',
    'original': u'هُنَالِكَ',
    'encletic': u''
}
STOPWORDS[u'ألهنالك'] = {
    'word': u'ألهنالك',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:مجرور',
    'vocalized': u'أَلِهُنَالِكَ',
    'stem': u'هُنَالِكَ',
    'type': u'STOPWORD',
    'original': u'هُنَالِكَ',
    'encletic': u''
}
STOPWORDS[u'أوهنالك'] = {
    'word': u'أوهنالك',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إشارة:معطوف:مجرور',
    'vocalized': u'أَوَهُنَالِكَ',
    'stem': u'هُنَالِكَ',
    'type': u'STOPWORD',
    'original': u'هُنَالِكَ',
    'encletic': u''
}
STOPWORDS[u'أفهنالك'] = {
    'word': u'أفهنالك',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إشارة:معطوف:مجرور',
    'vocalized': u'أَفَهُنَالِكَ',
    'stem': u'هُنَالِكَ',
    'type': u'STOPWORD',
    'original': u'هُنَالِكَ',
    'encletic': u''
}
STOPWORDS[u'أوبهنالك'] = {
    'word': u'أوبهنالك',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَبِهُنَالِكَ',
    'stem': u'هُنَالِكَ',
    'type': u'STOPWORD',
    'original': u'هُنَالِكَ',
    'encletic': u''
}
STOPWORDS[u'أفبهنالك'] = {
    'word': u'أفبهنالك',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَبِهُنَالِكَ',
    'stem': u'هُنَالِكَ',
    'type': u'STOPWORD',
    'original': u'هُنَالِكَ',
    'encletic': u''
}
STOPWORDS[u'أوكهنالك'] = {
    'word': u'أوكهنالك',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَكَهُنَالِكَ',
    'stem': u'هُنَالِكَ',
    'type': u'STOPWORD',
    'original': u'هُنَالِكَ',
    'encletic': u''
}
STOPWORDS[u'أفكهنالك'] = {
    'word': u'أفكهنالك',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَكَهُنَالِكَ',
    'stem': u'هُنَالِكَ',
    'type': u'STOPWORD',
    'original': u'هُنَالِكَ',
    'encletic': u''
}
STOPWORDS[u'أولهنالك'] = {
    'word': u'أولهنالك',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَلِهُنَالِكَ',
    'stem': u'هُنَالِكَ',
    'type': u'STOPWORD',
    'original': u'هُنَالِكَ',
    'encletic': u''
}
STOPWORDS[u'أفلهنالك'] = {
    'word': u'أفلهنالك',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إشارة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَلِهُنَالِكَ',
    'stem': u'هُنَالِكَ',
    'type': u'STOPWORD',
    'original': u'هُنَالِكَ',
    'encletic': u''
}
STOPWORDS[u'أي'] = {
    'word': u'أَيُّ',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة',
    'vocalized': u'أَيُّ',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u''
}
STOPWORDS[u'بأي'] = {
    'word': u'بأي',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور',
    'vocalized': u'بِأَيُّ',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u''
}
STOPWORDS[u'كأي'] = {
    'word': u'كأي',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور',
    'vocalized': u'كَأَيُّ',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u''
}
STOPWORDS[u'لأي'] = {
    'word': u'لأي',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور',
    'vocalized': u'لِأَيُّ',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u''
}
STOPWORDS[u'وأي'] = {
    'word': u'وأي',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف',
    'vocalized': u'وَأَيُّ',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u''
}
STOPWORDS[u'فأي'] = {
    'word': u'فأي',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف',
    'vocalized': u'فَأَيُّ',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u''
}
STOPWORDS[u'وبأي'] = {
    'word': u'وبأي',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف',
    'vocalized': u'وَبِأَيُّ',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u''
}
STOPWORDS[u'فبأي'] = {
    'word': u'فبأي',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف',
    'vocalized': u'فَبِأَيُّ',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u''
}
STOPWORDS[u'وكأي'] = {
    'word': u'وكأي',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف',
    'vocalized': u'وَكَأَيُّ',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u''
}
STOPWORDS[u'فكأي'] = {
    'word': u'فكأي',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف',
    'vocalized': u'فَكَأَيُّ',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u''
}
STOPWORDS[u'ولأي'] = {
    'word': u'ولأي',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف',
    'vocalized': u'وَلِأَيُّ',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u''
}
STOPWORDS[u'فلأي'] = {
    'word': u'فلأي',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف',
    'vocalized': u'فَلِأَيُّ',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u''
}
STOPWORDS[u'أيي'] = {
    'word': u'أيي',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'أَيُِّي',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'ِي'
}
STOPWORDS[u'أيك'] = {
    'word': u'أيك',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'أَيُّكَ',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'كَ'
}
STOPWORDS[u'أيك'] = {
    'word': u'أيك',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'أَيُّكِ',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'كِ'
}
STOPWORDS[u'أيه'] = {
    'word': u'أيه',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'أَيُّه',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'ه'
}
STOPWORDS[u'أيكم'] = {
    'word': u'أيكم',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'أَيُّكُمْ',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أيكن'] = {
    'word': u'أيكن',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'أَيُّكُنَّ',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أيها'] = {
    'word': u'أيها',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'أَيُّهَا',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'هَا'
}
STOPWORDS[u'أيهم'] = {
    'word': u'أيهم',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'أَيُّهُمْ',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أيهن'] = {
    'word': u'أيهن',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'أَيُّهُنَّ',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أينا'] = {
    'word': u'أينا',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'أَيُّنَا',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'نَا'
}
STOPWORDS[u'أيكما'] = {
    'word': u'أيكما',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'أَيُّكُمَا',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أيهما'] = {
    'word': u'أيهما',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'أَيُّهُمَا',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'بأيي'] = {
    'word': u'بأيي',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِأَيُِّي',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'ِي'
}
STOPWORDS[u'بأيك'] = {
    'word': u'بأيك',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِأَيُّكَ',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'كَ'
}
STOPWORDS[u'بأيك'] = {
    'word': u'بأيك',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِأَيُّكِ',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'كِ'
}
STOPWORDS[u'بأيه'] = {
    'word': u'بأيه',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِأَيُّه',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'ه'
}
STOPWORDS[u'بأيكم'] = {
    'word': u'بأيكم',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِأَيُّكُمْ',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'بأيكن'] = {
    'word': u'بأيكن',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِأَيُّكُنَّ',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'بأيها'] = {
    'word': u'بأيها',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِأَيُّهَا',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'هَا'
}
STOPWORDS[u'بأيهم'] = {
    'word': u'بأيهم',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِأَيُّهُمْ',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'بأيهن'] = {
    'word': u'بأيهن',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِأَيُّهُنَّ',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'بأينا'] = {
    'word': u'بأينا',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِأَيُّنَا',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'نَا'
}
STOPWORDS[u'بأيكما'] = {
    'word': u'بأيكما',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِأَيُّكُمَا',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'بأيهما'] = {
    'word': u'بأيهما',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِأَيُّهُمَا',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'كأيي'] = {
    'word': u'كأيي',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَأَيُِّي',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'ِي'
}
STOPWORDS[u'كأيك'] = {
    'word': u'كأيك',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَأَيُّكَ',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'كَ'
}
STOPWORDS[u'كأيك'] = {
    'word': u'كأيك',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَأَيُّكِ',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'كِ'
}
STOPWORDS[u'كأيه'] = {
    'word': u'كأيه',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَأَيُّه',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'ه'
}
STOPWORDS[u'كأيكم'] = {
    'word': u'كأيكم',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَأَيُّكُمْ',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'كأيكن'] = {
    'word': u'كأيكن',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَأَيُّكُنَّ',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'كأيها'] = {
    'word': u'كأيها',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَأَيُّهَا',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'هَا'
}
STOPWORDS[u'كأيهم'] = {
    'word': u'كأيهم',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَأَيُّهُمْ',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'كأيهن'] = {
    'word': u'كأيهن',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَأَيُّهُنَّ',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'كأينا'] = {
    'word': u'كأينا',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَأَيُّنَا',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'نَا'
}
STOPWORDS[u'كأيكما'] = {
    'word': u'كأيكما',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَأَيُّكُمَا',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'كأيهما'] = {
    'word': u'كأيهما',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَأَيُّهُمَا',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'لأيي'] = {
    'word': u'لأيي',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِأَيُِّي',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'ِي'
}
STOPWORDS[u'لأيك'] = {
    'word': u'لأيك',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِأَيُّكَ',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'كَ'
}
STOPWORDS[u'لأيك'] = {
    'word': u'لأيك',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِأَيُّكِ',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'كِ'
}
STOPWORDS[u'لأيه'] = {
    'word': u'لأيه',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِأَيُّه',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'ه'
}
STOPWORDS[u'لأيكم'] = {
    'word': u'لأيكم',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِأَيُّكُمْ',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'لأيكن'] = {
    'word': u'لأيكن',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِأَيُّكُنَّ',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'لأيها'] = {
    'word': u'لأيها',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِأَيُّهَا',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'هَا'
}
STOPWORDS[u'لأيهم'] = {
    'word': u'لأيهم',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِأَيُّهُمْ',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'لأيهن'] = {
    'word': u'لأيهن',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِأَيُّهُنَّ',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'لأينا'] = {
    'word': u'لأينا',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِأَيُّنَا',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'نَا'
}
STOPWORDS[u'لأيكما'] = {
    'word': u'لأيكما',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِأَيُّكُمَا',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'لأيهما'] = {
    'word': u'لأيهما',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِأَيُّهُمَا',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'وأيي'] = {
    'word': u'وأيي',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَأَيُِّي',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'ِي'
}
STOPWORDS[u'وأيك'] = {
    'word': u'وأيك',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَأَيُّكَ',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'كَ'
}
STOPWORDS[u'وأيك'] = {
    'word': u'وأيك',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَأَيُّكِ',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'كِ'
}
STOPWORDS[u'وأيه'] = {
    'word': u'وأيه',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَأَيُّه',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'ه'
}
STOPWORDS[u'وأيكم'] = {
    'word': u'وأيكم',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَأَيُّكُمْ',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'وأيكن'] = {
    'word': u'وأيكن',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَأَيُّكُنَّ',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'وأيها'] = {
    'word': u'وأيها',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَأَيُّهَا',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'هَا'
}
STOPWORDS[u'وأيهم'] = {
    'word': u'وأيهم',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَأَيُّهُمْ',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'وأيهن'] = {
    'word': u'وأيهن',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَأَيُّهُنَّ',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'وأينا'] = {
    'word': u'وأينا',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَأَيُّنَا',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'نَا'
}
STOPWORDS[u'وأيكما'] = {
    'word': u'وأيكما',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَأَيُّكُمَا',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'وأيهما'] = {
    'word': u'وأيهما',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَأَيُّهُمَا',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فأيي'] = {
    'word': u'فأيي',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَأَيُِّي',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'ِي'
}
STOPWORDS[u'فأيك'] = {
    'word': u'فأيك',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَأَيُّكَ',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'كَ'
}
STOPWORDS[u'فأيك'] = {
    'word': u'فأيك',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَأَيُّكِ',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'كِ'
}
STOPWORDS[u'فأيه'] = {
    'word': u'فأيه',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَأَيُّه',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'ه'
}
STOPWORDS[u'فأيكم'] = {
    'word': u'فأيكم',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَأَيُّكُمْ',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فأيكن'] = {
    'word': u'فأيكن',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَأَيُّكُنَّ',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فأيها'] = {
    'word': u'فأيها',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَأَيُّهَا',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'هَا'
}
STOPWORDS[u'فأيهم'] = {
    'word': u'فأيهم',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَأَيُّهُمْ',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فأيهن'] = {
    'word': u'فأيهن',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَأَيُّهُنَّ',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فأينا'] = {
    'word': u'فأينا',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَأَيُّنَا',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'نَا'
}
STOPWORDS[u'فأيكما'] = {
    'word': u'فأيكما',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَأَيُّكُمَا',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فأيهما'] = {
    'word': u'فأيهما',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَأَيُّهُمَا',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'وبأيي'] = {
    'word': u'وبأيي',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِأَيُِّي',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'ِي'
}
STOPWORDS[u'وبأيك'] = {
    'word': u'وبأيك',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِأَيُّكَ',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'كَ'
}
STOPWORDS[u'وبأيك'] = {
    'word': u'وبأيك',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِأَيُّكِ',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'كِ'
}
STOPWORDS[u'وبأيه'] = {
    'word': u'وبأيه',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِأَيُّه',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'ه'
}
STOPWORDS[u'وبأيكم'] = {
    'word': u'وبأيكم',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِأَيُّكُمْ',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'وبأيكن'] = {
    'word': u'وبأيكن',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِأَيُّكُنَّ',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'وبأيها'] = {
    'word': u'وبأيها',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِأَيُّهَا',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'هَا'
}
STOPWORDS[u'وبأيهم'] = {
    'word': u'وبأيهم',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِأَيُّهُمْ',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'وبأيهن'] = {
    'word': u'وبأيهن',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِأَيُّهُنَّ',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'وبأينا'] = {
    'word': u'وبأينا',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِأَيُّنَا',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'نَا'
}
STOPWORDS[u'وبأيكما'] = {
    'word': u'وبأيكما',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِأَيُّكُمَا',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'وبأيهما'] = {
    'word': u'وبأيهما',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِأَيُّهُمَا',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فبأيي'] = {
    'word': u'فبأيي',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِأَيُِّي',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'ِي'
}
STOPWORDS[u'فبأيك'] = {
    'word': u'فبأيك',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِأَيُّكَ',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'كَ'
}
STOPWORDS[u'فبأيك'] = {
    'word': u'فبأيك',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِأَيُّكِ',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'كِ'
}
STOPWORDS[u'فبأيه'] = {
    'word': u'فبأيه',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِأَيُّه',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'ه'
}
STOPWORDS[u'فبأيكم'] = {
    'word': u'فبأيكم',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِأَيُّكُمْ',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فبأيكن'] = {
    'word': u'فبأيكن',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِأَيُّكُنَّ',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فبأيها'] = {
    'word': u'فبأيها',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِأَيُّهَا',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'هَا'
}
STOPWORDS[u'فبأيهم'] = {
    'word': u'فبأيهم',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِأَيُّهُمْ',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فبأيهن'] = {
    'word': u'فبأيهن',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِأَيُّهُنَّ',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فبأينا'] = {
    'word': u'فبأينا',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِأَيُّنَا',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'نَا'
}
STOPWORDS[u'فبأيكما'] = {
    'word': u'فبأيكما',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِأَيُّكُمَا',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فبأيهما'] = {
    'word': u'فبأيهما',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِأَيُّهُمَا',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'وكأيي'] = {
    'word': u'وكأيي',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَأَيُِّي',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'ِي'
}
STOPWORDS[u'وكأيك'] = {
    'word': u'وكأيك',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَأَيُّكَ',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'كَ'
}
STOPWORDS[u'وكأيك'] = {
    'word': u'وكأيك',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَأَيُّكِ',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'كِ'
}
STOPWORDS[u'وكأيه'] = {
    'word': u'وكأيه',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَأَيُّه',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'ه'
}
STOPWORDS[u'وكأيكم'] = {
    'word': u'وكأيكم',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَأَيُّكُمْ',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'وكأيكن'] = {
    'word': u'وكأيكن',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَأَيُّكُنَّ',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'وكأيها'] = {
    'word': u'وكأيها',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَأَيُّهَا',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'هَا'
}
STOPWORDS[u'وكأيهم'] = {
    'word': u'وكأيهم',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَأَيُّهُمْ',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'وكأيهن'] = {
    'word': u'وكأيهن',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَأَيُّهُنَّ',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'وكأينا'] = {
    'word': u'وكأينا',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَأَيُّنَا',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'نَا'
}
STOPWORDS[u'وكأيكما'] = {
    'word': u'وكأيكما',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَأَيُّكُمَا',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'وكأيهما'] = {
    'word': u'وكأيهما',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَأَيُّهُمَا',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فكأيي'] = {
    'word': u'فكأيي',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَأَيُِّي',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'ِي'
}
STOPWORDS[u'فكأيك'] = {
    'word': u'فكأيك',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَأَيُّكَ',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'كَ'
}
STOPWORDS[u'فكأيك'] = {
    'word': u'فكأيك',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَأَيُّكِ',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'كِ'
}
STOPWORDS[u'فكأيه'] = {
    'word': u'فكأيه',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَأَيُّه',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'ه'
}
STOPWORDS[u'فكأيكم'] = {
    'word': u'فكأيكم',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَأَيُّكُمْ',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فكأيكن'] = {
    'word': u'فكأيكن',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَأَيُّكُنَّ',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فكأيها'] = {
    'word': u'فكأيها',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَأَيُّهَا',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'هَا'
}
STOPWORDS[u'فكأيهم'] = {
    'word': u'فكأيهم',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَأَيُّهُمْ',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فكأيهن'] = {
    'word': u'فكأيهن',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَأَيُّهُنَّ',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فكأينا'] = {
    'word': u'فكأينا',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَأَيُّنَا',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'نَا'
}
STOPWORDS[u'فكأيكما'] = {
    'word': u'فكأيكما',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَأَيُّكُمَا',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فكأيهما'] = {
    'word': u'فكأيهما',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَأَيُّهُمَا',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ولأيي'] = {
    'word': u'ولأيي',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِأَيُِّي',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'ِي'
}
STOPWORDS[u'ولأيك'] = {
    'word': u'ولأيك',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِأَيُّكَ',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'كَ'
}
STOPWORDS[u'ولأيك'] = {
    'word': u'ولأيك',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِأَيُّكِ',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'كِ'
}
STOPWORDS[u'ولأيه'] = {
    'word': u'ولأيه',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِأَيُّه',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'ه'
}
STOPWORDS[u'ولأيكم'] = {
    'word': u'ولأيكم',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِأَيُّكُمْ',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ولأيكن'] = {
    'word': u'ولأيكن',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِأَيُّكُنَّ',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ولأيها'] = {
    'word': u'ولأيها',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِأَيُّهَا',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'هَا'
}
STOPWORDS[u'ولأيهم'] = {
    'word': u'ولأيهم',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِأَيُّهُمْ',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ولأيهن'] = {
    'word': u'ولأيهن',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِأَيُّهُنَّ',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ولأينا'] = {
    'word': u'ولأينا',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِأَيُّنَا',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'نَا'
}
STOPWORDS[u'ولأيكما'] = {
    'word': u'ولأيكما',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِأَيُّكُمَا',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ولأيهما'] = {
    'word': u'ولأيهما',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِأَيُّهُمَا',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فلأيي'] = {
    'word': u'فلأيي',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِأَيُِّي',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'ِي'
}
STOPWORDS[u'فلأيك'] = {
    'word': u'فلأيك',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِأَيُّكَ',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'كَ'
}
STOPWORDS[u'فلأيك'] = {
    'word': u'فلأيك',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِأَيُّكِ',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'كِ'
}
STOPWORDS[u'فلأيه'] = {
    'word': u'فلأيه',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِأَيُّه',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'ه'
}
STOPWORDS[u'فلأيكم'] = {
    'word': u'فلأيكم',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِأَيُّكُمْ',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فلأيكن'] = {
    'word': u'فلأيكن',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِأَيُّكُنَّ',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فلأيها'] = {
    'word': u'فلأيها',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِأَيُّهَا',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'هَا'
}
STOPWORDS[u'فلأيهم'] = {
    'word': u'فلأيهم',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِأَيُّهُمْ',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فلأيهن'] = {
    'word': u'فلأيهن',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِأَيُّهُنَّ',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فلأينا'] = {
    'word': u'فلأينا',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِأَيُّنَا',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'نَا'
}
STOPWORDS[u'فلأيكما'] = {
    'word': u'فلأيكما',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِأَيُّكُمَا',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فلأيهما'] = {
    'word': u'فلأيهما',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِأَيُّهُمَا',
    'stem': u'أَيُّ',
    'type': u'STOPWORD',
    'original': u'أَيُّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'إذ'] = {
    'word': u'إِذْ',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة',
    'vocalized': u'إِذْ',
    'stem': u'إِذْ',
    'type': u'STOPWORD',
    'original': u'إِذْ',
    'encletic': u''
}
STOPWORDS[u'وإذ'] = {
    'word': u'وإذ',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف',
    'vocalized': u'وَإِذْ',
    'stem': u'إِذْ',
    'type': u'STOPWORD',
    'original': u'إِذْ',
    'encletic': u''
}
STOPWORDS[u'فإذ'] = {
    'word': u'فإذ',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف',
    'vocalized': u'فَإِذْ',
    'stem': u'إِذْ',
    'type': u'STOPWORD',
    'original': u'إِذْ',
    'encletic': u''
}
STOPWORDS[u'إذا'] = {
    'word': u'إِذَا',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة',
    'vocalized': u'إِذَا',
    'stem': u'إِذَا',
    'type': u'STOPWORD',
    'original': u'إِذَا',
    'encletic': u''
}
STOPWORDS[u'وإذا'] = {
    'word': u'وإذا',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف',
    'vocalized': u'وَإِذَا',
    'stem': u'إِذَا',
    'type': u'STOPWORD',
    'original': u'إِذَا',
    'encletic': u''
}
STOPWORDS[u'فإذا'] = {
    'word': u'فإذا',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف',
    'vocalized': u'فَإِذَا',
    'stem': u'إِذَا',
    'type': u'STOPWORD',
    'original': u'إِذَا',
    'encletic': u''
}
STOPWORDS[u'بعض'] = {
    'word': u'بَعْضُ',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة',
    'vocalized': u'بَعْضُ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u''
}
STOPWORDS[u'ببعض'] = {
    'word': u'ببعض',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور',
    'vocalized': u'بِبَعْضُ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u''
}
STOPWORDS[u'كبعض'] = {
    'word': u'كبعض',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور',
    'vocalized': u'كَبَعْضُ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u''
}
STOPWORDS[u'لبعض'] = {
    'word': u'لبعض',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور',
    'vocalized': u'لِبَعْضُ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u''
}
STOPWORDS[u'لبعض'] = {
    'word': u'لبعض',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور',
    'vocalized': u'لَبَعْضُ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u''
}
STOPWORDS[u'وبعض'] = {
    'word': u'وبعض',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف',
    'vocalized': u'وَبَعْضُ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u''
}
STOPWORDS[u'فبعض'] = {
    'word': u'فبعض',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف',
    'vocalized': u'فَبَعْضُ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u''
}
STOPWORDS[u'وببعض'] = {
    'word': u'وببعض',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف',
    'vocalized': u'وَبِبَعْضُ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u''
}
STOPWORDS[u'فببعض'] = {
    'word': u'فببعض',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف',
    'vocalized': u'فَبِبَعْضُ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u''
}
STOPWORDS[u'وكبعض'] = {
    'word': u'وكبعض',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف',
    'vocalized': u'وَكَبَعْضُ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u''
}
STOPWORDS[u'فكبعض'] = {
    'word': u'فكبعض',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف',
    'vocalized': u'فَكَبَعْضُ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u''
}
STOPWORDS[u'ولبعض'] = {
    'word': u'ولبعض',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف',
    'vocalized': u'وَلِبَعْضُ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u''
}
STOPWORDS[u'فلبعض'] = {
    'word': u'فلبعض',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف',
    'vocalized': u'فَلِبَعْضُ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u''
}
STOPWORDS[u'ولبعض'] = {
    'word': u'ولبعض',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'وَلَبَعْضُ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u''
}
STOPWORDS[u'فلبعض'] = {
    'word': u'فلبعض',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'فَلَبَعْضُ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u''
}
STOPWORDS[u'أبعض'] = {
    'word': u'أبعض',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور',
    'vocalized': u'أَبَعْضُ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u''
}
STOPWORDS[u'أببعض'] = {
    'word': u'أببعض',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور',
    'vocalized': u'أَبِبَعْضُ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u''
}
STOPWORDS[u'أكبعض'] = {
    'word': u'أكبعض',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور',
    'vocalized': u'أَكَبَعْضُ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u''
}
STOPWORDS[u'ألبعض'] = {
    'word': u'ألبعض',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور',
    'vocalized': u'أَلِبَعْضُ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u''
}
STOPWORDS[u'ألبعض'] = {
    'word': u'ألبعض',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور',
    'vocalized': u'أَلَبَعْضُ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u''
}
STOPWORDS[u'أوبعض'] = {
    'word': u'أوبعض',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور',
    'vocalized': u'أَوَبَعْضُ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u''
}
STOPWORDS[u'أفبعض'] = {
    'word': u'أفبعض',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور',
    'vocalized': u'أَفَبَعْضُ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u''
}
STOPWORDS[u'أوببعض'] = {
    'word': u'أوببعض',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَبِبَعْضُ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u''
}
STOPWORDS[u'أفببعض'] = {
    'word': u'أفببعض',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَبِبَعْضُ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u''
}
STOPWORDS[u'أوكبعض'] = {
    'word': u'أوكبعض',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَكَبَعْضُ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u''
}
STOPWORDS[u'أفكبعض'] = {
    'word': u'أفكبعض',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَكَبَعْضُ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u''
}
STOPWORDS[u'أولبعض'] = {
    'word': u'أولبعض',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَلِبَعْضُ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u''
}
STOPWORDS[u'أفلبعض'] = {
    'word': u'أفلبعض',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَلِبَعْضُ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u''
}
STOPWORDS[u'أولبعض'] = {
    'word': u'أولبعض',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَلَبَعْضُ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u''
}
STOPWORDS[u'أفلبعض'] = {
    'word': u'أفلبعض',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَلَبَعْضُ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u''
}
STOPWORDS[u'بعضي'] = {
    'word': u'بعضي',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'بَعْضُِي',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'ِي'
}
STOPWORDS[u'بعضك'] = {
    'word': u'بعضك',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'بَعْضُكَ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كَ'
}
STOPWORDS[u'بعضك'] = {
    'word': u'بعضك',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'بَعْضُكِ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كِ'
}
STOPWORDS[u'بعضه'] = {
    'word': u'بعضه',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'بَعْضُه',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'ه'
}
STOPWORDS[u'بعضكم'] = {
    'word': u'بعضكم',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'بَعْضُكُمْ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'بعضكن'] = {
    'word': u'بعضكن',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'بَعْضُكُنَّ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'بعضها'] = {
    'word': u'بعضها',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'بَعْضُهَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هَا'
}
STOPWORDS[u'بعضهم'] = {
    'word': u'بعضهم',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'بَعْضُهُمْ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'بعضهن'] = {
    'word': u'بعضهن',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'بَعْضُهُنَّ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'بعضنا'] = {
    'word': u'بعضنا',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'بَعْضُنَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'نَا'
}
STOPWORDS[u'بعضكما'] = {
    'word': u'بعضكما',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'بَعْضُكُمَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'بعضهما'] = {
    'word': u'بعضهما',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'بَعْضُهُمَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ببعضي'] = {
    'word': u'ببعضي',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِبَعْضُِي',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'ِي'
}
STOPWORDS[u'ببعضك'] = {
    'word': u'ببعضك',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِبَعْضُكَ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كَ'
}
STOPWORDS[u'ببعضك'] = {
    'word': u'ببعضك',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِبَعْضُكِ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كِ'
}
STOPWORDS[u'ببعضه'] = {
    'word': u'ببعضه',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِبَعْضُه',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'ه'
}
STOPWORDS[u'ببعضكم'] = {
    'word': u'ببعضكم',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِبَعْضُكُمْ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ببعضكن'] = {
    'word': u'ببعضكن',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِبَعْضُكُنَّ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ببعضها'] = {
    'word': u'ببعضها',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِبَعْضُهَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هَا'
}
STOPWORDS[u'ببعضهم'] = {
    'word': u'ببعضهم',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِبَعْضُهُمْ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ببعضهن'] = {
    'word': u'ببعضهن',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِبَعْضُهُنَّ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ببعضنا'] = {
    'word': u'ببعضنا',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِبَعْضُنَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'نَا'
}
STOPWORDS[u'ببعضكما'] = {
    'word': u'ببعضكما',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِبَعْضُكُمَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ببعضهما'] = {
    'word': u'ببعضهما',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِبَعْضُهُمَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'كبعضي'] = {
    'word': u'كبعضي',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَبَعْضُِي',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'ِي'
}
STOPWORDS[u'كبعضك'] = {
    'word': u'كبعضك',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَبَعْضُكَ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كَ'
}
STOPWORDS[u'كبعضك'] = {
    'word': u'كبعضك',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَبَعْضُكِ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كِ'
}
STOPWORDS[u'كبعضه'] = {
    'word': u'كبعضه',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَبَعْضُه',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'ه'
}
STOPWORDS[u'كبعضكم'] = {
    'word': u'كبعضكم',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَبَعْضُكُمْ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'كبعضكن'] = {
    'word': u'كبعضكن',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَبَعْضُكُنَّ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'كبعضها'] = {
    'word': u'كبعضها',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَبَعْضُهَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هَا'
}
STOPWORDS[u'كبعضهم'] = {
    'word': u'كبعضهم',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَبَعْضُهُمْ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'كبعضهن'] = {
    'word': u'كبعضهن',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَبَعْضُهُنَّ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'كبعضنا'] = {
    'word': u'كبعضنا',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَبَعْضُنَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'نَا'
}
STOPWORDS[u'كبعضكما'] = {
    'word': u'كبعضكما',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَبَعْضُكُمَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'كبعضهما'] = {
    'word': u'كبعضهما',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَبَعْضُهُمَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'لبعضي'] = {
    'word': u'لبعضي',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِبَعْضُِي',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'ِي'
}
STOPWORDS[u'لبعضك'] = {
    'word': u'لبعضك',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِبَعْضُكَ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كَ'
}
STOPWORDS[u'لبعضك'] = {
    'word': u'لبعضك',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِبَعْضُكِ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كِ'
}
STOPWORDS[u'لبعضه'] = {
    'word': u'لبعضه',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِبَعْضُه',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'ه'
}
STOPWORDS[u'لبعضكم'] = {
    'word': u'لبعضكم',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِبَعْضُكُمْ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'لبعضكن'] = {
    'word': u'لبعضكن',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِبَعْضُكُنَّ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'لبعضها'] = {
    'word': u'لبعضها',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِبَعْضُهَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هَا'
}
STOPWORDS[u'لبعضهم'] = {
    'word': u'لبعضهم',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِبَعْضُهُمْ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'لبعضهن'] = {
    'word': u'لبعضهن',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِبَعْضُهُنَّ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'لبعضنا'] = {
    'word': u'لبعضنا',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِبَعْضُنَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'نَا'
}
STOPWORDS[u'لبعضكما'] = {
    'word': u'لبعضكما',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِبَعْضُكُمَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'لبعضهما'] = {
    'word': u'لبعضهما',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِبَعْضُهُمَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'لبعضي'] = {
    'word': u'لبعضي',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَبَعْضُِي',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'ِي'
}
STOPWORDS[u'لبعضك'] = {
    'word': u'لبعضك',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَبَعْضُكَ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كَ'
}
STOPWORDS[u'لبعضك'] = {
    'word': u'لبعضك',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَبَعْضُكِ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كِ'
}
STOPWORDS[u'لبعضه'] = {
    'word': u'لبعضه',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَبَعْضُه',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'ه'
}
STOPWORDS[u'لبعضكم'] = {
    'word': u'لبعضكم',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَبَعْضُكُمْ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'لبعضكن'] = {
    'word': u'لبعضكن',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَبَعْضُكُنَّ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'لبعضها'] = {
    'word': u'لبعضها',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَبَعْضُهَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هَا'
}
STOPWORDS[u'لبعضهم'] = {
    'word': u'لبعضهم',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَبَعْضُهُمْ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'لبعضهن'] = {
    'word': u'لبعضهن',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَبَعْضُهُنَّ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'لبعضنا'] = {
    'word': u'لبعضنا',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَبَعْضُنَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'نَا'
}
STOPWORDS[u'لبعضكما'] = {
    'word': u'لبعضكما',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَبَعْضُكُمَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'لبعضهما'] = {
    'word': u'لبعضهما',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَبَعْضُهُمَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'وبعضي'] = {
    'word': u'وبعضي',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَبَعْضُِي',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'ِي'
}
STOPWORDS[u'وبعضك'] = {
    'word': u'وبعضك',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَبَعْضُكَ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كَ'
}
STOPWORDS[u'وبعضك'] = {
    'word': u'وبعضك',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَبَعْضُكِ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كِ'
}
STOPWORDS[u'وبعضه'] = {
    'word': u'وبعضه',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَبَعْضُه',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'ه'
}
STOPWORDS[u'وبعضكم'] = {
    'word': u'وبعضكم',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَبَعْضُكُمْ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'وبعضكن'] = {
    'word': u'وبعضكن',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَبَعْضُكُنَّ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'وبعضها'] = {
    'word': u'وبعضها',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَبَعْضُهَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هَا'
}
STOPWORDS[u'وبعضهم'] = {
    'word': u'وبعضهم',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَبَعْضُهُمْ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'وبعضهن'] = {
    'word': u'وبعضهن',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَبَعْضُهُنَّ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'وبعضنا'] = {
    'word': u'وبعضنا',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَبَعْضُنَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'نَا'
}
STOPWORDS[u'وبعضكما'] = {
    'word': u'وبعضكما',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَبَعْضُكُمَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'وبعضهما'] = {
    'word': u'وبعضهما',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَبَعْضُهُمَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فبعضي'] = {
    'word': u'فبعضي',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَبَعْضُِي',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'ِي'
}
STOPWORDS[u'فبعضك'] = {
    'word': u'فبعضك',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَبَعْضُكَ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كَ'
}
STOPWORDS[u'فبعضك'] = {
    'word': u'فبعضك',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَبَعْضُكِ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كِ'
}
STOPWORDS[u'فبعضه'] = {
    'word': u'فبعضه',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَبَعْضُه',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'ه'
}
STOPWORDS[u'فبعضكم'] = {
    'word': u'فبعضكم',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَبَعْضُكُمْ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فبعضكن'] = {
    'word': u'فبعضكن',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَبَعْضُكُنَّ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فبعضها'] = {
    'word': u'فبعضها',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَبَعْضُهَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هَا'
}
STOPWORDS[u'فبعضهم'] = {
    'word': u'فبعضهم',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَبَعْضُهُمْ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فبعضهن'] = {
    'word': u'فبعضهن',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَبَعْضُهُنَّ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فبعضنا'] = {
    'word': u'فبعضنا',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَبَعْضُنَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'نَا'
}
STOPWORDS[u'فبعضكما'] = {
    'word': u'فبعضكما',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَبَعْضُكُمَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فبعضهما'] = {
    'word': u'فبعضهما',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَبَعْضُهُمَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'وببعضي'] = {
    'word': u'وببعضي',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِبَعْضُِي',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'ِي'
}
STOPWORDS[u'وببعضك'] = {
    'word': u'وببعضك',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِبَعْضُكَ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كَ'
}
STOPWORDS[u'وببعضك'] = {
    'word': u'وببعضك',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِبَعْضُكِ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كِ'
}
STOPWORDS[u'وببعضه'] = {
    'word': u'وببعضه',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِبَعْضُه',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'ه'
}
STOPWORDS[u'وببعضكم'] = {
    'word': u'وببعضكم',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِبَعْضُكُمْ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'وببعضكن'] = {
    'word': u'وببعضكن',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِبَعْضُكُنَّ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'وببعضها'] = {
    'word': u'وببعضها',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِبَعْضُهَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هَا'
}
STOPWORDS[u'وببعضهم'] = {
    'word': u'وببعضهم',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِبَعْضُهُمْ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'وببعضهن'] = {
    'word': u'وببعضهن',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِبَعْضُهُنَّ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'وببعضنا'] = {
    'word': u'وببعضنا',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِبَعْضُنَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'نَا'
}
STOPWORDS[u'وببعضكما'] = {
    'word': u'وببعضكما',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِبَعْضُكُمَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'وببعضهما'] = {
    'word': u'وببعضهما',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِبَعْضُهُمَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فببعضي'] = {
    'word': u'فببعضي',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِبَعْضُِي',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'ِي'
}
STOPWORDS[u'فببعضك'] = {
    'word': u'فببعضك',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِبَعْضُكَ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كَ'
}
STOPWORDS[u'فببعضك'] = {
    'word': u'فببعضك',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِبَعْضُكِ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كِ'
}
STOPWORDS[u'فببعضه'] = {
    'word': u'فببعضه',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِبَعْضُه',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'ه'
}
STOPWORDS[u'فببعضكم'] = {
    'word': u'فببعضكم',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِبَعْضُكُمْ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فببعضكن'] = {
    'word': u'فببعضكن',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِبَعْضُكُنَّ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فببعضها'] = {
    'word': u'فببعضها',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِبَعْضُهَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هَا'
}
STOPWORDS[u'فببعضهم'] = {
    'word': u'فببعضهم',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِبَعْضُهُمْ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فببعضهن'] = {
    'word': u'فببعضهن',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِبَعْضُهُنَّ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فببعضنا'] = {
    'word': u'فببعضنا',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِبَعْضُنَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'نَا'
}
STOPWORDS[u'فببعضكما'] = {
    'word': u'فببعضكما',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِبَعْضُكُمَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فببعضهما'] = {
    'word': u'فببعضهما',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِبَعْضُهُمَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'وكبعضي'] = {
    'word': u'وكبعضي',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَبَعْضُِي',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'ِي'
}
STOPWORDS[u'وكبعضك'] = {
    'word': u'وكبعضك',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَبَعْضُكَ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كَ'
}
STOPWORDS[u'وكبعضك'] = {
    'word': u'وكبعضك',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَبَعْضُكِ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كِ'
}
STOPWORDS[u'وكبعضه'] = {
    'word': u'وكبعضه',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَبَعْضُه',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'ه'
}
STOPWORDS[u'وكبعضكم'] = {
    'word': u'وكبعضكم',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَبَعْضُكُمْ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'وكبعضكن'] = {
    'word': u'وكبعضكن',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَبَعْضُكُنَّ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'وكبعضها'] = {
    'word': u'وكبعضها',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَبَعْضُهَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هَا'
}
STOPWORDS[u'وكبعضهم'] = {
    'word': u'وكبعضهم',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَبَعْضُهُمْ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'وكبعضهن'] = {
    'word': u'وكبعضهن',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَبَعْضُهُنَّ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'وكبعضنا'] = {
    'word': u'وكبعضنا',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَبَعْضُنَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'نَا'
}
STOPWORDS[u'وكبعضكما'] = {
    'word': u'وكبعضكما',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَبَعْضُكُمَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'وكبعضهما'] = {
    'word': u'وكبعضهما',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَبَعْضُهُمَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فكبعضي'] = {
    'word': u'فكبعضي',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَبَعْضُِي',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'ِي'
}
STOPWORDS[u'فكبعضك'] = {
    'word': u'فكبعضك',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَبَعْضُكَ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كَ'
}
STOPWORDS[u'فكبعضك'] = {
    'word': u'فكبعضك',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَبَعْضُكِ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كِ'
}
STOPWORDS[u'فكبعضه'] = {
    'word': u'فكبعضه',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَبَعْضُه',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'ه'
}
STOPWORDS[u'فكبعضكم'] = {
    'word': u'فكبعضكم',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَبَعْضُكُمْ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فكبعضكن'] = {
    'word': u'فكبعضكن',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَبَعْضُكُنَّ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فكبعضها'] = {
    'word': u'فكبعضها',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَبَعْضُهَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هَا'
}
STOPWORDS[u'فكبعضهم'] = {
    'word': u'فكبعضهم',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَبَعْضُهُمْ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فكبعضهن'] = {
    'word': u'فكبعضهن',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَبَعْضُهُنَّ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فكبعضنا'] = {
    'word': u'فكبعضنا',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَبَعْضُنَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'نَا'
}
STOPWORDS[u'فكبعضكما'] = {
    'word': u'فكبعضكما',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَبَعْضُكُمَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فكبعضهما'] = {
    'word': u'فكبعضهما',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَبَعْضُهُمَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ولبعضي'] = {
    'word': u'ولبعضي',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِبَعْضُِي',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'ِي'
}
STOPWORDS[u'ولبعضك'] = {
    'word': u'ولبعضك',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِبَعْضُكَ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كَ'
}
STOPWORDS[u'ولبعضك'] = {
    'word': u'ولبعضك',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِبَعْضُكِ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كِ'
}
STOPWORDS[u'ولبعضه'] = {
    'word': u'ولبعضه',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِبَعْضُه',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'ه'
}
STOPWORDS[u'ولبعضكم'] = {
    'word': u'ولبعضكم',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِبَعْضُكُمْ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ولبعضكن'] = {
    'word': u'ولبعضكن',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِبَعْضُكُنَّ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ولبعضها'] = {
    'word': u'ولبعضها',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِبَعْضُهَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هَا'
}
STOPWORDS[u'ولبعضهم'] = {
    'word': u'ولبعضهم',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِبَعْضُهُمْ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ولبعضهن'] = {
    'word': u'ولبعضهن',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِبَعْضُهُنَّ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ولبعضنا'] = {
    'word': u'ولبعضنا',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِبَعْضُنَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'نَا'
}
STOPWORDS[u'ولبعضكما'] = {
    'word': u'ولبعضكما',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِبَعْضُكُمَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ولبعضهما'] = {
    'word': u'ولبعضهما',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِبَعْضُهُمَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فلبعضي'] = {
    'word': u'فلبعضي',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِبَعْضُِي',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'ِي'
}
STOPWORDS[u'فلبعضك'] = {
    'word': u'فلبعضك',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِبَعْضُكَ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كَ'
}
STOPWORDS[u'فلبعضك'] = {
    'word': u'فلبعضك',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِبَعْضُكِ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كِ'
}
STOPWORDS[u'فلبعضه'] = {
    'word': u'فلبعضه',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِبَعْضُه',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'ه'
}
STOPWORDS[u'فلبعضكم'] = {
    'word': u'فلبعضكم',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِبَعْضُكُمْ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فلبعضكن'] = {
    'word': u'فلبعضكن',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِبَعْضُكُنَّ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فلبعضها'] = {
    'word': u'فلبعضها',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِبَعْضُهَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هَا'
}
STOPWORDS[u'فلبعضهم'] = {
    'word': u'فلبعضهم',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِبَعْضُهُمْ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فلبعضهن'] = {
    'word': u'فلبعضهن',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِبَعْضُهُنَّ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فلبعضنا'] = {
    'word': u'فلبعضنا',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِبَعْضُنَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'نَا'
}
STOPWORDS[u'فلبعضكما'] = {
    'word': u'فلبعضكما',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِبَعْضُكُمَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فلبعضهما'] = {
    'word': u'فلبعضهما',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِبَعْضُهُمَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ولبعضي'] = {
    'word': u'ولبعضي',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَبَعْضُِي',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'ِي'
}
STOPWORDS[u'ولبعضك'] = {
    'word': u'ولبعضك',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَبَعْضُكَ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كَ'
}
STOPWORDS[u'ولبعضك'] = {
    'word': u'ولبعضك',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَبَعْضُكِ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كِ'
}
STOPWORDS[u'ولبعضه'] = {
    'word': u'ولبعضه',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَبَعْضُه',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'ه'
}
STOPWORDS[u'ولبعضكم'] = {
    'word': u'ولبعضكم',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَبَعْضُكُمْ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ولبعضكن'] = {
    'word': u'ولبعضكن',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَبَعْضُكُنَّ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ولبعضها'] = {
    'word': u'ولبعضها',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَبَعْضُهَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هَا'
}
STOPWORDS[u'ولبعضهم'] = {
    'word': u'ولبعضهم',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَبَعْضُهُمْ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ولبعضهن'] = {
    'word': u'ولبعضهن',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَبَعْضُهُنَّ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ولبعضنا'] = {
    'word': u'ولبعضنا',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَبَعْضُنَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'نَا'
}
STOPWORDS[u'ولبعضكما'] = {
    'word': u'ولبعضكما',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَبَعْضُكُمَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ولبعضهما'] = {
    'word': u'ولبعضهما',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَبَعْضُهُمَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فلبعضي'] = {
    'word': u'فلبعضي',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَبَعْضُِي',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'ِي'
}
STOPWORDS[u'فلبعضك'] = {
    'word': u'فلبعضك',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَبَعْضُكَ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كَ'
}
STOPWORDS[u'فلبعضك'] = {
    'word': u'فلبعضك',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَبَعْضُكِ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كِ'
}
STOPWORDS[u'فلبعضه'] = {
    'word': u'فلبعضه',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَبَعْضُه',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'ه'
}
STOPWORDS[u'فلبعضكم'] = {
    'word': u'فلبعضكم',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَبَعْضُكُمْ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فلبعضكن'] = {
    'word': u'فلبعضكن',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَبَعْضُكُنَّ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فلبعضها'] = {
    'word': u'فلبعضها',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَبَعْضُهَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هَا'
}
STOPWORDS[u'فلبعضهم'] = {
    'word': u'فلبعضهم',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَبَعْضُهُمْ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فلبعضهن'] = {
    'word': u'فلبعضهن',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَبَعْضُهُنَّ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فلبعضنا'] = {
    'word': u'فلبعضنا',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَبَعْضُنَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'نَا'
}
STOPWORDS[u'فلبعضكما'] = {
    'word': u'فلبعضكما',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَبَعْضُكُمَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فلبعضهما'] = {
    'word': u'فلبعضهما',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَبَعْضُهُمَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أبعضي'] = {
    'word': u'أبعضي',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'أَبَعْضُِي',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'ِي'
}
STOPWORDS[u'أبعضك'] = {
    'word': u'أبعضك',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'أَبَعْضُكَ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كَ'
}
STOPWORDS[u'أبعضك'] = {
    'word': u'أبعضك',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'أَبَعْضُكِ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كِ'
}
STOPWORDS[u'أبعضه'] = {
    'word': u'أبعضه',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'أَبَعْضُه',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'ه'
}
STOPWORDS[u'أبعضكم'] = {
    'word': u'أبعضكم',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'أَبَعْضُكُمْ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أبعضكن'] = {
    'word': u'أبعضكن',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'أَبَعْضُكُنَّ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أبعضها'] = {
    'word': u'أبعضها',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'أَبَعْضُهَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هَا'
}
STOPWORDS[u'أبعضهم'] = {
    'word': u'أبعضهم',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'أَبَعْضُهُمْ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أبعضهن'] = {
    'word': u'أبعضهن',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'أَبَعْضُهُنَّ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أبعضنا'] = {
    'word': u'أبعضنا',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'أَبَعْضُنَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'نَا'
}
STOPWORDS[u'أبعضكما'] = {
    'word': u'أبعضكما',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'أَبَعْضُكُمَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أبعضهما'] = {
    'word': u'أبعضهما',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'أَبَعْضُهُمَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أببعضي'] = {
    'word': u'أببعضي',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِبَعْضُِي',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'ِي'
}
STOPWORDS[u'أببعضك'] = {
    'word': u'أببعضك',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِبَعْضُكَ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كَ'
}
STOPWORDS[u'أببعضك'] = {
    'word': u'أببعضك',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِبَعْضُكِ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كِ'
}
STOPWORDS[u'أببعضه'] = {
    'word': u'أببعضه',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِبَعْضُه',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'ه'
}
STOPWORDS[u'أببعضكم'] = {
    'word': u'أببعضكم',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِبَعْضُكُمْ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أببعضكن'] = {
    'word': u'أببعضكن',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِبَعْضُكُنَّ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أببعضها'] = {
    'word': u'أببعضها',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِبَعْضُهَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هَا'
}
STOPWORDS[u'أببعضهم'] = {
    'word': u'أببعضهم',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِبَعْضُهُمْ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أببعضهن'] = {
    'word': u'أببعضهن',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِبَعْضُهُنَّ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أببعضنا'] = {
    'word': u'أببعضنا',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِبَعْضُنَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'نَا'
}
STOPWORDS[u'أببعضكما'] = {
    'word': u'أببعضكما',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِبَعْضُكُمَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أببعضهما'] = {
    'word': u'أببعضهما',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِبَعْضُهُمَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أكبعضي'] = {
    'word': u'أكبعضي',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَبَعْضُِي',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'ِي'
}
STOPWORDS[u'أكبعضك'] = {
    'word': u'أكبعضك',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَبَعْضُكَ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كَ'
}
STOPWORDS[u'أكبعضك'] = {
    'word': u'أكبعضك',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَبَعْضُكِ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كِ'
}
STOPWORDS[u'أكبعضه'] = {
    'word': u'أكبعضه',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَبَعْضُه',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'ه'
}
STOPWORDS[u'أكبعضكم'] = {
    'word': u'أكبعضكم',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَبَعْضُكُمْ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أكبعضكن'] = {
    'word': u'أكبعضكن',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَبَعْضُكُنَّ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أكبعضها'] = {
    'word': u'أكبعضها',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَبَعْضُهَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هَا'
}
STOPWORDS[u'أكبعضهم'] = {
    'word': u'أكبعضهم',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَبَعْضُهُمْ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أكبعضهن'] = {
    'word': u'أكبعضهن',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَبَعْضُهُنَّ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أكبعضنا'] = {
    'word': u'أكبعضنا',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَبَعْضُنَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'نَا'
}
STOPWORDS[u'أكبعضكما'] = {
    'word': u'أكبعضكما',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَبَعْضُكُمَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أكبعضهما'] = {
    'word': u'أكبعضهما',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَبَعْضُهُمَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ألبعضي'] = {
    'word': u'ألبعضي',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِبَعْضُِي',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'ِي'
}
STOPWORDS[u'ألبعضك'] = {
    'word': u'ألبعضك',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِبَعْضُكَ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كَ'
}
STOPWORDS[u'ألبعضك'] = {
    'word': u'ألبعضك',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِبَعْضُكِ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كِ'
}
STOPWORDS[u'ألبعضه'] = {
    'word': u'ألبعضه',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِبَعْضُه',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'ه'
}
STOPWORDS[u'ألبعضكم'] = {
    'word': u'ألبعضكم',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِبَعْضُكُمْ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ألبعضكن'] = {
    'word': u'ألبعضكن',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِبَعْضُكُنَّ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ألبعضها'] = {
    'word': u'ألبعضها',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِبَعْضُهَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هَا'
}
STOPWORDS[u'ألبعضهم'] = {
    'word': u'ألبعضهم',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِبَعْضُهُمْ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ألبعضهن'] = {
    'word': u'ألبعضهن',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِبَعْضُهُنَّ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ألبعضنا'] = {
    'word': u'ألبعضنا',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِبَعْضُنَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'نَا'
}
STOPWORDS[u'ألبعضكما'] = {
    'word': u'ألبعضكما',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِبَعْضُكُمَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ألبعضهما'] = {
    'word': u'ألبعضهما',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِبَعْضُهُمَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ألبعضي'] = {
    'word': u'ألبعضي',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَبَعْضُِي',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'ِي'
}
STOPWORDS[u'ألبعضك'] = {
    'word': u'ألبعضك',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَبَعْضُكَ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كَ'
}
STOPWORDS[u'ألبعضك'] = {
    'word': u'ألبعضك',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَبَعْضُكِ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كِ'
}
STOPWORDS[u'ألبعضه'] = {
    'word': u'ألبعضه',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَبَعْضُه',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'ه'
}
STOPWORDS[u'ألبعضكم'] = {
    'word': u'ألبعضكم',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَبَعْضُكُمْ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ألبعضكن'] = {
    'word': u'ألبعضكن',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَبَعْضُكُنَّ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ألبعضها'] = {
    'word': u'ألبعضها',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَبَعْضُهَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هَا'
}
STOPWORDS[u'ألبعضهم'] = {
    'word': u'ألبعضهم',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَبَعْضُهُمْ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ألبعضهن'] = {
    'word': u'ألبعضهن',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَبَعْضُهُنَّ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ألبعضنا'] = {
    'word': u'ألبعضنا',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَبَعْضُنَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'نَا'
}
STOPWORDS[u'ألبعضكما'] = {
    'word': u'ألبعضكما',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَبَعْضُكُمَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ألبعضهما'] = {
    'word': u'ألبعضهما',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَبَعْضُهُمَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أوبعضي'] = {
    'word': u'أوبعضي',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبَعْضُِي',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'ِي'
}
STOPWORDS[u'أوبعضك'] = {
    'word': u'أوبعضك',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبَعْضُكَ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كَ'
}
STOPWORDS[u'أوبعضك'] = {
    'word': u'أوبعضك',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبَعْضُكِ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كِ'
}
STOPWORDS[u'أوبعضه'] = {
    'word': u'أوبعضه',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبَعْضُه',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'ه'
}
STOPWORDS[u'أوبعضكم'] = {
    'word': u'أوبعضكم',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبَعْضُكُمْ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أوبعضكن'] = {
    'word': u'أوبعضكن',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبَعْضُكُنَّ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أوبعضها'] = {
    'word': u'أوبعضها',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبَعْضُهَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هَا'
}
STOPWORDS[u'أوبعضهم'] = {
    'word': u'أوبعضهم',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبَعْضُهُمْ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أوبعضهن'] = {
    'word': u'أوبعضهن',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبَعْضُهُنَّ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أوبعضنا'] = {
    'word': u'أوبعضنا',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبَعْضُنَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'نَا'
}
STOPWORDS[u'أوبعضكما'] = {
    'word': u'أوبعضكما',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبَعْضُكُمَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أوبعضهما'] = {
    'word': u'أوبعضهما',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبَعْضُهُمَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أفبعضي'] = {
    'word': u'أفبعضي',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبَعْضُِي',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'ِي'
}
STOPWORDS[u'أفبعضك'] = {
    'word': u'أفبعضك',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبَعْضُكَ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كَ'
}
STOPWORDS[u'أفبعضك'] = {
    'word': u'أفبعضك',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبَعْضُكِ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كِ'
}
STOPWORDS[u'أفبعضه'] = {
    'word': u'أفبعضه',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبَعْضُه',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'ه'
}
STOPWORDS[u'أفبعضكم'] = {
    'word': u'أفبعضكم',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبَعْضُكُمْ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أفبعضكن'] = {
    'word': u'أفبعضكن',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبَعْضُكُنَّ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أفبعضها'] = {
    'word': u'أفبعضها',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبَعْضُهَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هَا'
}
STOPWORDS[u'أفبعضهم'] = {
    'word': u'أفبعضهم',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبَعْضُهُمْ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أفبعضهن'] = {
    'word': u'أفبعضهن',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبَعْضُهُنَّ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أفبعضنا'] = {
    'word': u'أفبعضنا',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبَعْضُنَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'نَا'
}
STOPWORDS[u'أفبعضكما'] = {
    'word': u'أفبعضكما',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبَعْضُكُمَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أفبعضهما'] = {
    'word': u'أفبعضهما',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبَعْضُهُمَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أوببعضي'] = {
    'word': u'أوببعضي',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِبَعْضُِي',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'ِي'
}
STOPWORDS[u'أوببعضك'] = {
    'word': u'أوببعضك',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِبَعْضُكَ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كَ'
}
STOPWORDS[u'أوببعضك'] = {
    'word': u'أوببعضك',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِبَعْضُكِ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كِ'
}
STOPWORDS[u'أوببعضه'] = {
    'word': u'أوببعضه',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِبَعْضُه',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'ه'
}
STOPWORDS[u'أوببعضكم'] = {
    'word': u'أوببعضكم',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِبَعْضُكُمْ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أوببعضكن'] = {
    'word': u'أوببعضكن',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِبَعْضُكُنَّ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أوببعضها'] = {
    'word': u'أوببعضها',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِبَعْضُهَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هَا'
}
STOPWORDS[u'أوببعضهم'] = {
    'word': u'أوببعضهم',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِبَعْضُهُمْ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أوببعضهن'] = {
    'word': u'أوببعضهن',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِبَعْضُهُنَّ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أوببعضنا'] = {
    'word': u'أوببعضنا',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِبَعْضُنَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'نَا'
}
STOPWORDS[u'أوببعضكما'] = {
    'word': u'أوببعضكما',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِبَعْضُكُمَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أوببعضهما'] = {
    'word': u'أوببعضهما',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِبَعْضُهُمَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أفببعضي'] = {
    'word': u'أفببعضي',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِبَعْضُِي',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'ِي'
}
STOPWORDS[u'أفببعضك'] = {
    'word': u'أفببعضك',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِبَعْضُكَ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كَ'
}
STOPWORDS[u'أفببعضك'] = {
    'word': u'أفببعضك',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِبَعْضُكِ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كِ'
}
STOPWORDS[u'أفببعضه'] = {
    'word': u'أفببعضه',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِبَعْضُه',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'ه'
}
STOPWORDS[u'أفببعضكم'] = {
    'word': u'أفببعضكم',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِبَعْضُكُمْ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أفببعضكن'] = {
    'word': u'أفببعضكن',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِبَعْضُكُنَّ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أفببعضها'] = {
    'word': u'أفببعضها',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِبَعْضُهَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هَا'
}
STOPWORDS[u'أفببعضهم'] = {
    'word': u'أفببعضهم',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِبَعْضُهُمْ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أفببعضهن'] = {
    'word': u'أفببعضهن',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِبَعْضُهُنَّ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أفببعضنا'] = {
    'word': u'أفببعضنا',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِبَعْضُنَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'نَا'
}
STOPWORDS[u'أفببعضكما'] = {
    'word': u'أفببعضكما',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِبَعْضُكُمَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أفببعضهما'] = {
    'word': u'أفببعضهما',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِبَعْضُهُمَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أوكبعضي'] = {
    'word': u'أوكبعضي',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَبَعْضُِي',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'ِي'
}
STOPWORDS[u'أوكبعضك'] = {
    'word': u'أوكبعضك',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَبَعْضُكَ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كَ'
}
STOPWORDS[u'أوكبعضك'] = {
    'word': u'أوكبعضك',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَبَعْضُكِ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كِ'
}
STOPWORDS[u'أوكبعضه'] = {
    'word': u'أوكبعضه',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَبَعْضُه',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'ه'
}
STOPWORDS[u'أوكبعضكم'] = {
    'word': u'أوكبعضكم',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَبَعْضُكُمْ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أوكبعضكن'] = {
    'word': u'أوكبعضكن',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَبَعْضُكُنَّ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أوكبعضها'] = {
    'word': u'أوكبعضها',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَبَعْضُهَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هَا'
}
STOPWORDS[u'أوكبعضهم'] = {
    'word': u'أوكبعضهم',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَبَعْضُهُمْ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أوكبعضهن'] = {
    'word': u'أوكبعضهن',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَبَعْضُهُنَّ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أوكبعضنا'] = {
    'word': u'أوكبعضنا',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَبَعْضُنَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'نَا'
}
STOPWORDS[u'أوكبعضكما'] = {
    'word': u'أوكبعضكما',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَبَعْضُكُمَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أوكبعضهما'] = {
    'word': u'أوكبعضهما',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَبَعْضُهُمَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أفكبعضي'] = {
    'word': u'أفكبعضي',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَبَعْضُِي',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'ِي'
}
STOPWORDS[u'أفكبعضك'] = {
    'word': u'أفكبعضك',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَبَعْضُكَ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كَ'
}
STOPWORDS[u'أفكبعضك'] = {
    'word': u'أفكبعضك',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَبَعْضُكِ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كِ'
}
STOPWORDS[u'أفكبعضه'] = {
    'word': u'أفكبعضه',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَبَعْضُه',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'ه'
}
STOPWORDS[u'أفكبعضكم'] = {
    'word': u'أفكبعضكم',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَبَعْضُكُمْ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أفكبعضكن'] = {
    'word': u'أفكبعضكن',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَبَعْضُكُنَّ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أفكبعضها'] = {
    'word': u'أفكبعضها',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَبَعْضُهَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هَا'
}
STOPWORDS[u'أفكبعضهم'] = {
    'word': u'أفكبعضهم',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَبَعْضُهُمْ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أفكبعضهن'] = {
    'word': u'أفكبعضهن',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَبَعْضُهُنَّ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أفكبعضنا'] = {
    'word': u'أفكبعضنا',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَبَعْضُنَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'نَا'
}
STOPWORDS[u'أفكبعضكما'] = {
    'word': u'أفكبعضكما',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَبَعْضُكُمَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أفكبعضهما'] = {
    'word': u'أفكبعضهما',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَبَعْضُهُمَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أولبعضي'] = {
    'word': u'أولبعضي',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِبَعْضُِي',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'ِي'
}
STOPWORDS[u'أولبعضك'] = {
    'word': u'أولبعضك',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِبَعْضُكَ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كَ'
}
STOPWORDS[u'أولبعضك'] = {
    'word': u'أولبعضك',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِبَعْضُكِ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كِ'
}
STOPWORDS[u'أولبعضه'] = {
    'word': u'أولبعضه',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِبَعْضُه',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'ه'
}
STOPWORDS[u'أولبعضكم'] = {
    'word': u'أولبعضكم',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِبَعْضُكُمْ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أولبعضكن'] = {
    'word': u'أولبعضكن',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِبَعْضُكُنَّ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أولبعضها'] = {
    'word': u'أولبعضها',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِبَعْضُهَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هَا'
}
STOPWORDS[u'أولبعضهم'] = {
    'word': u'أولبعضهم',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِبَعْضُهُمْ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أولبعضهن'] = {
    'word': u'أولبعضهن',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِبَعْضُهُنَّ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أولبعضنا'] = {
    'word': u'أولبعضنا',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِبَعْضُنَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'نَا'
}
STOPWORDS[u'أولبعضكما'] = {
    'word': u'أولبعضكما',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِبَعْضُكُمَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أولبعضهما'] = {
    'word': u'أولبعضهما',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِبَعْضُهُمَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أفلبعضي'] = {
    'word': u'أفلبعضي',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِبَعْضُِي',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'ِي'
}
STOPWORDS[u'أفلبعضك'] = {
    'word': u'أفلبعضك',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِبَعْضُكَ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كَ'
}
STOPWORDS[u'أفلبعضك'] = {
    'word': u'أفلبعضك',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِبَعْضُكِ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كِ'
}
STOPWORDS[u'أفلبعضه'] = {
    'word': u'أفلبعضه',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِبَعْضُه',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'ه'
}
STOPWORDS[u'أفلبعضكم'] = {
    'word': u'أفلبعضكم',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِبَعْضُكُمْ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أفلبعضكن'] = {
    'word': u'أفلبعضكن',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِبَعْضُكُنَّ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أفلبعضها'] = {
    'word': u'أفلبعضها',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِبَعْضُهَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هَا'
}
STOPWORDS[u'أفلبعضهم'] = {
    'word': u'أفلبعضهم',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِبَعْضُهُمْ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أفلبعضهن'] = {
    'word': u'أفلبعضهن',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِبَعْضُهُنَّ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أفلبعضنا'] = {
    'word': u'أفلبعضنا',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِبَعْضُنَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'نَا'
}
STOPWORDS[u'أفلبعضكما'] = {
    'word': u'أفلبعضكما',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِبَعْضُكُمَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أفلبعضهما'] = {
    'word': u'أفلبعضهما',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِبَعْضُهُمَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أولبعضي'] = {
    'word': u'أولبعضي',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَبَعْضُِي',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'ِي'
}
STOPWORDS[u'أولبعضك'] = {
    'word': u'أولبعضك',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَبَعْضُكَ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كَ'
}
STOPWORDS[u'أولبعضك'] = {
    'word': u'أولبعضك',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَبَعْضُكِ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كِ'
}
STOPWORDS[u'أولبعضه'] = {
    'word': u'أولبعضه',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَبَعْضُه',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'ه'
}
STOPWORDS[u'أولبعضكم'] = {
    'word': u'أولبعضكم',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَبَعْضُكُمْ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أولبعضكن'] = {
    'word': u'أولبعضكن',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَبَعْضُكُنَّ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أولبعضها'] = {
    'word': u'أولبعضها',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَبَعْضُهَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هَا'
}
STOPWORDS[u'أولبعضهم'] = {
    'word': u'أولبعضهم',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَبَعْضُهُمْ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أولبعضهن'] = {
    'word': u'أولبعضهن',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَبَعْضُهُنَّ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أولبعضنا'] = {
    'word': u'أولبعضنا',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَبَعْضُنَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'نَا'
}
STOPWORDS[u'أولبعضكما'] = {
    'word': u'أولبعضكما',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَبَعْضُكُمَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أولبعضهما'] = {
    'word': u'أولبعضهما',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَبَعْضُهُمَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أفلبعضي'] = {
    'word': u'أفلبعضي',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَبَعْضُِي',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'ِي'
}
STOPWORDS[u'أفلبعضك'] = {
    'word': u'أفلبعضك',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَبَعْضُكَ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كَ'
}
STOPWORDS[u'أفلبعضك'] = {
    'word': u'أفلبعضك',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَبَعْضُكِ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كِ'
}
STOPWORDS[u'أفلبعضه'] = {
    'word': u'أفلبعضه',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَبَعْضُه',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'ه'
}
STOPWORDS[u'أفلبعضكم'] = {
    'word': u'أفلبعضكم',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَبَعْضُكُمْ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أفلبعضكن'] = {
    'word': u'أفلبعضكن',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَبَعْضُكُنَّ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أفلبعضها'] = {
    'word': u'أفلبعضها',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَبَعْضُهَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هَا'
}
STOPWORDS[u'أفلبعضهم'] = {
    'word': u'أفلبعضهم',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَبَعْضُهُمْ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أفلبعضهن'] = {
    'word': u'أفلبعضهن',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَبَعْضُهُنَّ',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أفلبعضنا'] = {
    'word': u'أفلبعضنا',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَبَعْضُنَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'نَا'
}
STOPWORDS[u'أفلبعضكما'] = {
    'word': u'أفلبعضكما',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَبَعْضُكُمَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أفلبعضهما'] = {
    'word': u'أفلبعضهما',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَبَعْضُهُمَا',
    'stem': u'بَعْضُ',
    'type': u'STOPWORD',
    'original': u'بَعْضُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'تجاه'] = {
    'word': u'تُجَاهَ',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة',
    'vocalized': u'تُجَاهَ',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u''
}
STOPWORDS[u'بتجاه'] = {
    'word': u'بتجاه',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور',
    'vocalized': u'بِتُجَاهَ',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u''
}
STOPWORDS[u'كتجاه'] = {
    'word': u'كتجاه',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور',
    'vocalized': u'كَتُجَاهَ',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u''
}
STOPWORDS[u'لتجاه'] = {
    'word': u'لتجاه',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور',
    'vocalized': u'لِتُجَاهَ',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u''
}
STOPWORDS[u'وتجاه'] = {
    'word': u'وتجاه',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف',
    'vocalized': u'وَتُجَاهَ',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u''
}
STOPWORDS[u'فتجاه'] = {
    'word': u'فتجاه',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف',
    'vocalized': u'فَتُجَاهَ',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u''
}
STOPWORDS[u'وبتجاه'] = {
    'word': u'وبتجاه',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف',
    'vocalized': u'وَبِتُجَاهَ',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u''
}
STOPWORDS[u'فبتجاه'] = {
    'word': u'فبتجاه',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف',
    'vocalized': u'فَبِتُجَاهَ',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u''
}
STOPWORDS[u'وكتجاه'] = {
    'word': u'وكتجاه',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف',
    'vocalized': u'وَكَتُجَاهَ',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u''
}
STOPWORDS[u'فكتجاه'] = {
    'word': u'فكتجاه',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف',
    'vocalized': u'فَكَتُجَاهَ',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u''
}
STOPWORDS[u'ولتجاه'] = {
    'word': u'ولتجاه',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف',
    'vocalized': u'وَلِتُجَاهَ',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u''
}
STOPWORDS[u'فلتجاه'] = {
    'word': u'فلتجاه',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف',
    'vocalized': u'فَلِتُجَاهَ',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u''
}
STOPWORDS[u'تجاهي'] = {
    'word': u'تجاهي',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'تُجَاهَِي',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'ِي'
}
STOPWORDS[u'تجاهك'] = {
    'word': u'تجاهك',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'تُجَاهَكَ',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'كَ'
}
STOPWORDS[u'تجاهك'] = {
    'word': u'تجاهك',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'تُجَاهَكِ',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'كِ'
}
STOPWORDS[u'تجاهه'] = {
    'word': u'تجاهه',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'تُجَاهَه',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'ه'
}
STOPWORDS[u'تجاهكم'] = {
    'word': u'تجاهكم',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'تُجَاهَكُمْ',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'تجاهكن'] = {
    'word': u'تجاهكن',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'تُجَاهَكُنَّ',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'تجاهها'] = {
    'word': u'تجاهها',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'تُجَاهَهَا',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'هَا'
}
STOPWORDS[u'تجاههم'] = {
    'word': u'تجاههم',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'تُجَاهَهُمْ',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'تجاههن'] = {
    'word': u'تجاههن',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'تُجَاهَهُنَّ',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'تجاهنا'] = {
    'word': u'تجاهنا',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'تُجَاهَنَا',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'نَا'
}
STOPWORDS[u'تجاهكما'] = {
    'word': u'تجاهكما',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'تُجَاهَكُمَا',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'تجاههما'] = {
    'word': u'تجاههما',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'تُجَاهَهُمَا',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'بتجاهي'] = {
    'word': u'بتجاهي',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِتُجَاهَِي',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'ِي'
}
STOPWORDS[u'بتجاهك'] = {
    'word': u'بتجاهك',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِتُجَاهَكَ',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'كَ'
}
STOPWORDS[u'بتجاهك'] = {
    'word': u'بتجاهك',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِتُجَاهَكِ',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'كِ'
}
STOPWORDS[u'بتجاهه'] = {
    'word': u'بتجاهه',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِتُجَاهَه',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'ه'
}
STOPWORDS[u'بتجاهكم'] = {
    'word': u'بتجاهكم',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِتُجَاهَكُمْ',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'بتجاهكن'] = {
    'word': u'بتجاهكن',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِتُجَاهَكُنَّ',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'بتجاهها'] = {
    'word': u'بتجاهها',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِتُجَاهَهَا',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'هَا'
}
STOPWORDS[u'بتجاههم'] = {
    'word': u'بتجاههم',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِتُجَاهَهُمْ',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'بتجاههن'] = {
    'word': u'بتجاههن',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِتُجَاهَهُنَّ',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'بتجاهنا'] = {
    'word': u'بتجاهنا',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِتُجَاهَنَا',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'نَا'
}
STOPWORDS[u'بتجاهكما'] = {
    'word': u'بتجاهكما',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِتُجَاهَكُمَا',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'بتجاههما'] = {
    'word': u'بتجاههما',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِتُجَاهَهُمَا',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'كتجاهي'] = {
    'word': u'كتجاهي',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَتُجَاهَِي',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'ِي'
}
STOPWORDS[u'كتجاهك'] = {
    'word': u'كتجاهك',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَتُجَاهَكَ',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'كَ'
}
STOPWORDS[u'كتجاهك'] = {
    'word': u'كتجاهك',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَتُجَاهَكِ',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'كِ'
}
STOPWORDS[u'كتجاهه'] = {
    'word': u'كتجاهه',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَتُجَاهَه',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'ه'
}
STOPWORDS[u'كتجاهكم'] = {
    'word': u'كتجاهكم',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَتُجَاهَكُمْ',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'كتجاهكن'] = {
    'word': u'كتجاهكن',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَتُجَاهَكُنَّ',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'كتجاهها'] = {
    'word': u'كتجاهها',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَتُجَاهَهَا',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'هَا'
}
STOPWORDS[u'كتجاههم'] = {
    'word': u'كتجاههم',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَتُجَاهَهُمْ',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'كتجاههن'] = {
    'word': u'كتجاههن',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَتُجَاهَهُنَّ',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'كتجاهنا'] = {
    'word': u'كتجاهنا',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَتُجَاهَنَا',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'نَا'
}
STOPWORDS[u'كتجاهكما'] = {
    'word': u'كتجاهكما',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَتُجَاهَكُمَا',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'كتجاههما'] = {
    'word': u'كتجاههما',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَتُجَاهَهُمَا',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'لتجاهي'] = {
    'word': u'لتجاهي',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِتُجَاهَِي',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'ِي'
}
STOPWORDS[u'لتجاهك'] = {
    'word': u'لتجاهك',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِتُجَاهَكَ',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'كَ'
}
STOPWORDS[u'لتجاهك'] = {
    'word': u'لتجاهك',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِتُجَاهَكِ',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'كِ'
}
STOPWORDS[u'لتجاهه'] = {
    'word': u'لتجاهه',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِتُجَاهَه',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'ه'
}
STOPWORDS[u'لتجاهكم'] = {
    'word': u'لتجاهكم',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِتُجَاهَكُمْ',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'لتجاهكن'] = {
    'word': u'لتجاهكن',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِتُجَاهَكُنَّ',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'لتجاهها'] = {
    'word': u'لتجاهها',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِتُجَاهَهَا',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'هَا'
}
STOPWORDS[u'لتجاههم'] = {
    'word': u'لتجاههم',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِتُجَاهَهُمْ',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'لتجاههن'] = {
    'word': u'لتجاههن',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِتُجَاهَهُنَّ',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'لتجاهنا'] = {
    'word': u'لتجاهنا',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِتُجَاهَنَا',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'نَا'
}
STOPWORDS[u'لتجاهكما'] = {
    'word': u'لتجاهكما',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِتُجَاهَكُمَا',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'لتجاههما'] = {
    'word': u'لتجاههما',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِتُجَاهَهُمَا',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'وتجاهي'] = {
    'word': u'وتجاهي',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَتُجَاهَِي',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'ِي'
}
STOPWORDS[u'وتجاهك'] = {
    'word': u'وتجاهك',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَتُجَاهَكَ',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'كَ'
}
STOPWORDS[u'وتجاهك'] = {
    'word': u'وتجاهك',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَتُجَاهَكِ',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'كِ'
}
STOPWORDS[u'وتجاهه'] = {
    'word': u'وتجاهه',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَتُجَاهَه',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'ه'
}
STOPWORDS[u'وتجاهكم'] = {
    'word': u'وتجاهكم',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَتُجَاهَكُمْ',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'وتجاهكن'] = {
    'word': u'وتجاهكن',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَتُجَاهَكُنَّ',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'وتجاهها'] = {
    'word': u'وتجاهها',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَتُجَاهَهَا',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'هَا'
}
STOPWORDS[u'وتجاههم'] = {
    'word': u'وتجاههم',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَتُجَاهَهُمْ',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'وتجاههن'] = {
    'word': u'وتجاههن',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَتُجَاهَهُنَّ',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'وتجاهنا'] = {
    'word': u'وتجاهنا',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَتُجَاهَنَا',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'نَا'
}
STOPWORDS[u'وتجاهكما'] = {
    'word': u'وتجاهكما',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَتُجَاهَكُمَا',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'وتجاههما'] = {
    'word': u'وتجاههما',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَتُجَاهَهُمَا',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فتجاهي'] = {
    'word': u'فتجاهي',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَتُجَاهَِي',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'ِي'
}
STOPWORDS[u'فتجاهك'] = {
    'word': u'فتجاهك',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَتُجَاهَكَ',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'كَ'
}
STOPWORDS[u'فتجاهك'] = {
    'word': u'فتجاهك',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَتُجَاهَكِ',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'كِ'
}
STOPWORDS[u'فتجاهه'] = {
    'word': u'فتجاهه',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَتُجَاهَه',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'ه'
}
STOPWORDS[u'فتجاهكم'] = {
    'word': u'فتجاهكم',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَتُجَاهَكُمْ',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فتجاهكن'] = {
    'word': u'فتجاهكن',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَتُجَاهَكُنَّ',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فتجاهها'] = {
    'word': u'فتجاهها',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَتُجَاهَهَا',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'هَا'
}
STOPWORDS[u'فتجاههم'] = {
    'word': u'فتجاههم',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَتُجَاهَهُمْ',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فتجاههن'] = {
    'word': u'فتجاههن',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَتُجَاهَهُنَّ',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فتجاهنا'] = {
    'word': u'فتجاهنا',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَتُجَاهَنَا',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'نَا'
}
STOPWORDS[u'فتجاهكما'] = {
    'word': u'فتجاهكما',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَتُجَاهَكُمَا',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فتجاههما'] = {
    'word': u'فتجاههما',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَتُجَاهَهُمَا',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'وبتجاهي'] = {
    'word': u'وبتجاهي',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِتُجَاهَِي',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'ِي'
}
STOPWORDS[u'وبتجاهك'] = {
    'word': u'وبتجاهك',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِتُجَاهَكَ',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'كَ'
}
STOPWORDS[u'وبتجاهك'] = {
    'word': u'وبتجاهك',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِتُجَاهَكِ',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'كِ'
}
STOPWORDS[u'وبتجاهه'] = {
    'word': u'وبتجاهه',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِتُجَاهَه',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'ه'
}
STOPWORDS[u'وبتجاهكم'] = {
    'word': u'وبتجاهكم',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِتُجَاهَكُمْ',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'وبتجاهكن'] = {
    'word': u'وبتجاهكن',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِتُجَاهَكُنَّ',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'وبتجاهها'] = {
    'word': u'وبتجاهها',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِتُجَاهَهَا',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'هَا'
}
STOPWORDS[u'وبتجاههم'] = {
    'word': u'وبتجاههم',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِتُجَاهَهُمْ',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'وبتجاههن'] = {
    'word': u'وبتجاههن',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِتُجَاهَهُنَّ',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'وبتجاهنا'] = {
    'word': u'وبتجاهنا',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِتُجَاهَنَا',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'نَا'
}
STOPWORDS[u'وبتجاهكما'] = {
    'word': u'وبتجاهكما',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِتُجَاهَكُمَا',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'وبتجاههما'] = {
    'word': u'وبتجاههما',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِتُجَاهَهُمَا',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فبتجاهي'] = {
    'word': u'فبتجاهي',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِتُجَاهَِي',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'ِي'
}
STOPWORDS[u'فبتجاهك'] = {
    'word': u'فبتجاهك',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِتُجَاهَكَ',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'كَ'
}
STOPWORDS[u'فبتجاهك'] = {
    'word': u'فبتجاهك',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِتُجَاهَكِ',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'كِ'
}
STOPWORDS[u'فبتجاهه'] = {
    'word': u'فبتجاهه',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِتُجَاهَه',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'ه'
}
STOPWORDS[u'فبتجاهكم'] = {
    'word': u'فبتجاهكم',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِتُجَاهَكُمْ',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فبتجاهكن'] = {
    'word': u'فبتجاهكن',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِتُجَاهَكُنَّ',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فبتجاهها'] = {
    'word': u'فبتجاهها',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِتُجَاهَهَا',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'هَا'
}
STOPWORDS[u'فبتجاههم'] = {
    'word': u'فبتجاههم',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِتُجَاهَهُمْ',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فبتجاههن'] = {
    'word': u'فبتجاههن',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِتُجَاهَهُنَّ',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فبتجاهنا'] = {
    'word': u'فبتجاهنا',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِتُجَاهَنَا',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'نَا'
}
STOPWORDS[u'فبتجاهكما'] = {
    'word': u'فبتجاهكما',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِتُجَاهَكُمَا',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فبتجاههما'] = {
    'word': u'فبتجاههما',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِتُجَاهَهُمَا',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'وكتجاهي'] = {
    'word': u'وكتجاهي',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَتُجَاهَِي',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'ِي'
}
STOPWORDS[u'وكتجاهك'] = {
    'word': u'وكتجاهك',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَتُجَاهَكَ',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'كَ'
}
STOPWORDS[u'وكتجاهك'] = {
    'word': u'وكتجاهك',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَتُجَاهَكِ',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'كِ'
}
STOPWORDS[u'وكتجاهه'] = {
    'word': u'وكتجاهه',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَتُجَاهَه',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'ه'
}
STOPWORDS[u'وكتجاهكم'] = {
    'word': u'وكتجاهكم',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَتُجَاهَكُمْ',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'وكتجاهكن'] = {
    'word': u'وكتجاهكن',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَتُجَاهَكُنَّ',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'وكتجاهها'] = {
    'word': u'وكتجاهها',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَتُجَاهَهَا',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'هَا'
}
STOPWORDS[u'وكتجاههم'] = {
    'word': u'وكتجاههم',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَتُجَاهَهُمْ',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'وكتجاههن'] = {
    'word': u'وكتجاههن',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَتُجَاهَهُنَّ',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'وكتجاهنا'] = {
    'word': u'وكتجاهنا',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَتُجَاهَنَا',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'نَا'
}
STOPWORDS[u'وكتجاهكما'] = {
    'word': u'وكتجاهكما',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَتُجَاهَكُمَا',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'وكتجاههما'] = {
    'word': u'وكتجاههما',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَتُجَاهَهُمَا',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فكتجاهي'] = {
    'word': u'فكتجاهي',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَتُجَاهَِي',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'ِي'
}
STOPWORDS[u'فكتجاهك'] = {
    'word': u'فكتجاهك',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَتُجَاهَكَ',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'كَ'
}
STOPWORDS[u'فكتجاهك'] = {
    'word': u'فكتجاهك',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَتُجَاهَكِ',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'كِ'
}
STOPWORDS[u'فكتجاهه'] = {
    'word': u'فكتجاهه',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَتُجَاهَه',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'ه'
}
STOPWORDS[u'فكتجاهكم'] = {
    'word': u'فكتجاهكم',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَتُجَاهَكُمْ',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فكتجاهكن'] = {
    'word': u'فكتجاهكن',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَتُجَاهَكُنَّ',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فكتجاهها'] = {
    'word': u'فكتجاهها',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَتُجَاهَهَا',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'هَا'
}
STOPWORDS[u'فكتجاههم'] = {
    'word': u'فكتجاههم',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَتُجَاهَهُمْ',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فكتجاههن'] = {
    'word': u'فكتجاههن',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَتُجَاهَهُنَّ',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فكتجاهنا'] = {
    'word': u'فكتجاهنا',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَتُجَاهَنَا',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'نَا'
}
STOPWORDS[u'فكتجاهكما'] = {
    'word': u'فكتجاهكما',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَتُجَاهَكُمَا',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فكتجاههما'] = {
    'word': u'فكتجاههما',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَتُجَاهَهُمَا',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ولتجاهي'] = {
    'word': u'ولتجاهي',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِتُجَاهَِي',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'ِي'
}
STOPWORDS[u'ولتجاهك'] = {
    'word': u'ولتجاهك',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِتُجَاهَكَ',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'كَ'
}
STOPWORDS[u'ولتجاهك'] = {
    'word': u'ولتجاهك',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِتُجَاهَكِ',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'كِ'
}
STOPWORDS[u'ولتجاهه'] = {
    'word': u'ولتجاهه',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِتُجَاهَه',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'ه'
}
STOPWORDS[u'ولتجاهكم'] = {
    'word': u'ولتجاهكم',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِتُجَاهَكُمْ',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ولتجاهكن'] = {
    'word': u'ولتجاهكن',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِتُجَاهَكُنَّ',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ولتجاهها'] = {
    'word': u'ولتجاهها',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِتُجَاهَهَا',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'هَا'
}
STOPWORDS[u'ولتجاههم'] = {
    'word': u'ولتجاههم',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِتُجَاهَهُمْ',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ولتجاههن'] = {
    'word': u'ولتجاههن',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِتُجَاهَهُنَّ',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ولتجاهنا'] = {
    'word': u'ولتجاهنا',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِتُجَاهَنَا',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'نَا'
}
STOPWORDS[u'ولتجاهكما'] = {
    'word': u'ولتجاهكما',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِتُجَاهَكُمَا',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ولتجاههما'] = {
    'word': u'ولتجاههما',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِتُجَاهَهُمَا',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فلتجاهي'] = {
    'word': u'فلتجاهي',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِتُجَاهَِي',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'ِي'
}
STOPWORDS[u'فلتجاهك'] = {
    'word': u'فلتجاهك',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِتُجَاهَكَ',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'كَ'
}
STOPWORDS[u'فلتجاهك'] = {
    'word': u'فلتجاهك',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِتُجَاهَكِ',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'كِ'
}
STOPWORDS[u'فلتجاهه'] = {
    'word': u'فلتجاهه',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِتُجَاهَه',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'ه'
}
STOPWORDS[u'فلتجاهكم'] = {
    'word': u'فلتجاهكم',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِتُجَاهَكُمْ',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فلتجاهكن'] = {
    'word': u'فلتجاهكن',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِتُجَاهَكُنَّ',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فلتجاهها'] = {
    'word': u'فلتجاهها',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِتُجَاهَهَا',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'هَا'
}
STOPWORDS[u'فلتجاههم'] = {
    'word': u'فلتجاههم',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِتُجَاهَهُمْ',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فلتجاههن'] = {
    'word': u'فلتجاههن',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِتُجَاهَهُنَّ',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فلتجاهنا'] = {
    'word': u'فلتجاهنا',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِتُجَاهَنَا',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'نَا'
}
STOPWORDS[u'فلتجاهكما'] = {
    'word': u'فلتجاهكما',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِتُجَاهَكُمَا',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فلتجاههما'] = {
    'word': u'فلتجاههما',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِتُجَاهَهُمَا',
    'stem': u'تُجَاهَ',
    'type': u'STOPWORD',
    'original': u'تُجَاهَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'تلقاء'] = {
    'word': u'تِلْقَاءَ',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة',
    'vocalized': u'تِلْقَاءَ',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u''
}
STOPWORDS[u'بتلقاء'] = {
    'word': u'بتلقاء',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور',
    'vocalized': u'بِتِلْقَاءَ',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u''
}
STOPWORDS[u'كتلقاء'] = {
    'word': u'كتلقاء',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور',
    'vocalized': u'كَتِلْقَاءَ',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u''
}
STOPWORDS[u'لتلقاء'] = {
    'word': u'لتلقاء',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور',
    'vocalized': u'لِتِلْقَاءَ',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u''
}
STOPWORDS[u'وتلقاء'] = {
    'word': u'وتلقاء',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف',
    'vocalized': u'وَتِلْقَاءَ',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u''
}
STOPWORDS[u'فتلقاء'] = {
    'word': u'فتلقاء',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف',
    'vocalized': u'فَتِلْقَاءَ',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u''
}
STOPWORDS[u'وبتلقاء'] = {
    'word': u'وبتلقاء',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف',
    'vocalized': u'وَبِتِلْقَاءَ',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u''
}
STOPWORDS[u'فبتلقاء'] = {
    'word': u'فبتلقاء',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف',
    'vocalized': u'فَبِتِلْقَاءَ',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u''
}
STOPWORDS[u'وكتلقاء'] = {
    'word': u'وكتلقاء',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف',
    'vocalized': u'وَكَتِلْقَاءَ',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u''
}
STOPWORDS[u'فكتلقاء'] = {
    'word': u'فكتلقاء',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف',
    'vocalized': u'فَكَتِلْقَاءَ',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u''
}
STOPWORDS[u'ولتلقاء'] = {
    'word': u'ولتلقاء',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف',
    'vocalized': u'وَلِتِلْقَاءَ',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u''
}
STOPWORDS[u'فلتلقاء'] = {
    'word': u'فلتلقاء',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف',
    'vocalized': u'فَلِتِلْقَاءَ',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u''
}
STOPWORDS[u'تلقاءي'] = {
    'word': u'تلقاءي',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'تِلْقَاءَِي',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'ِي'
}
STOPWORDS[u'تلقاءك'] = {
    'word': u'تلقاءك',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'تِلْقَاءَكَ',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'كَ'
}
STOPWORDS[u'تلقاءك'] = {
    'word': u'تلقاءك',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'تِلْقَاءَكِ',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'كِ'
}
STOPWORDS[u'تلقاءه'] = {
    'word': u'تلقاءه',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'تِلْقَاءَه',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'ه'
}
STOPWORDS[u'تلقاءكم'] = {
    'word': u'تلقاءكم',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'تِلْقَاءَكُمْ',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'تلقاءكن'] = {
    'word': u'تلقاءكن',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'تِلْقَاءَكُنَّ',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'تلقاءها'] = {
    'word': u'تلقاءها',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'تِلْقَاءَهَا',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'هَا'
}
STOPWORDS[u'تلقاءهم'] = {
    'word': u'تلقاءهم',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'تِلْقَاءَهُمْ',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'تلقاءهن'] = {
    'word': u'تلقاءهن',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'تِلْقَاءَهُنَّ',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'تلقاءنا'] = {
    'word': u'تلقاءنا',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'تِلْقَاءَنَا',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'نَا'
}
STOPWORDS[u'تلقاءكما'] = {
    'word': u'تلقاءكما',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'تِلْقَاءَكُمَا',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'تلقاءهما'] = {
    'word': u'تلقاءهما',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'تِلْقَاءَهُمَا',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'بتلقاءي'] = {
    'word': u'بتلقاءي',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِتِلْقَاءَِي',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'ِي'
}
STOPWORDS[u'بتلقاءك'] = {
    'word': u'بتلقاءك',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِتِلْقَاءَكَ',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'كَ'
}
STOPWORDS[u'بتلقاءك'] = {
    'word': u'بتلقاءك',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِتِلْقَاءَكِ',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'كِ'
}
STOPWORDS[u'بتلقاءه'] = {
    'word': u'بتلقاءه',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِتِلْقَاءَه',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'ه'
}
STOPWORDS[u'بتلقاءكم'] = {
    'word': u'بتلقاءكم',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِتِلْقَاءَكُمْ',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'بتلقاءكن'] = {
    'word': u'بتلقاءكن',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِتِلْقَاءَكُنَّ',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'بتلقاءها'] = {
    'word': u'بتلقاءها',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِتِلْقَاءَهَا',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'هَا'
}
STOPWORDS[u'بتلقاءهم'] = {
    'word': u'بتلقاءهم',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِتِلْقَاءَهُمْ',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'بتلقاءهن'] = {
    'word': u'بتلقاءهن',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِتِلْقَاءَهُنَّ',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'بتلقاءنا'] = {
    'word': u'بتلقاءنا',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِتِلْقَاءَنَا',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'نَا'
}
STOPWORDS[u'بتلقاءكما'] = {
    'word': u'بتلقاءكما',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِتِلْقَاءَكُمَا',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'بتلقاءهما'] = {
    'word': u'بتلقاءهما',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِتِلْقَاءَهُمَا',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'كتلقاءي'] = {
    'word': u'كتلقاءي',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَتِلْقَاءَِي',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'ِي'
}
STOPWORDS[u'كتلقاءك'] = {
    'word': u'كتلقاءك',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَتِلْقَاءَكَ',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'كَ'
}
STOPWORDS[u'كتلقاءك'] = {
    'word': u'كتلقاءك',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَتِلْقَاءَكِ',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'كِ'
}
STOPWORDS[u'كتلقاءه'] = {
    'word': u'كتلقاءه',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَتِلْقَاءَه',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'ه'
}
STOPWORDS[u'كتلقاءكم'] = {
    'word': u'كتلقاءكم',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَتِلْقَاءَكُمْ',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'كتلقاءكن'] = {
    'word': u'كتلقاءكن',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَتِلْقَاءَكُنَّ',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'كتلقاءها'] = {
    'word': u'كتلقاءها',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَتِلْقَاءَهَا',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'هَا'
}
STOPWORDS[u'كتلقاءهم'] = {
    'word': u'كتلقاءهم',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَتِلْقَاءَهُمْ',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'كتلقاءهن'] = {
    'word': u'كتلقاءهن',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَتِلْقَاءَهُنَّ',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'كتلقاءنا'] = {
    'word': u'كتلقاءنا',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَتِلْقَاءَنَا',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'نَا'
}
STOPWORDS[u'كتلقاءكما'] = {
    'word': u'كتلقاءكما',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَتِلْقَاءَكُمَا',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'كتلقاءهما'] = {
    'word': u'كتلقاءهما',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَتِلْقَاءَهُمَا',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'لتلقاءي'] = {
    'word': u'لتلقاءي',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِتِلْقَاءَِي',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'ِي'
}
STOPWORDS[u'لتلقاءك'] = {
    'word': u'لتلقاءك',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِتِلْقَاءَكَ',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'كَ'
}
STOPWORDS[u'لتلقاءك'] = {
    'word': u'لتلقاءك',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِتِلْقَاءَكِ',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'كِ'
}
STOPWORDS[u'لتلقاءه'] = {
    'word': u'لتلقاءه',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِتِلْقَاءَه',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'ه'
}
STOPWORDS[u'لتلقاءكم'] = {
    'word': u'لتلقاءكم',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِتِلْقَاءَكُمْ',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'لتلقاءكن'] = {
    'word': u'لتلقاءكن',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِتِلْقَاءَكُنَّ',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'لتلقاءها'] = {
    'word': u'لتلقاءها',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِتِلْقَاءَهَا',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'هَا'
}
STOPWORDS[u'لتلقاءهم'] = {
    'word': u'لتلقاءهم',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِتِلْقَاءَهُمْ',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'لتلقاءهن'] = {
    'word': u'لتلقاءهن',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِتِلْقَاءَهُنَّ',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'لتلقاءنا'] = {
    'word': u'لتلقاءنا',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِتِلْقَاءَنَا',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'نَا'
}
STOPWORDS[u'لتلقاءكما'] = {
    'word': u'لتلقاءكما',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِتِلْقَاءَكُمَا',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'لتلقاءهما'] = {
    'word': u'لتلقاءهما',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِتِلْقَاءَهُمَا',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'وتلقاءي'] = {
    'word': u'وتلقاءي',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَتِلْقَاءَِي',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'ِي'
}
STOPWORDS[u'وتلقاءك'] = {
    'word': u'وتلقاءك',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَتِلْقَاءَكَ',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'كَ'
}
STOPWORDS[u'وتلقاءك'] = {
    'word': u'وتلقاءك',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَتِلْقَاءَكِ',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'كِ'
}
STOPWORDS[u'وتلقاءه'] = {
    'word': u'وتلقاءه',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَتِلْقَاءَه',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'ه'
}
STOPWORDS[u'وتلقاءكم'] = {
    'word': u'وتلقاءكم',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَتِلْقَاءَكُمْ',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'وتلقاءكن'] = {
    'word': u'وتلقاءكن',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَتِلْقَاءَكُنَّ',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'وتلقاءها'] = {
    'word': u'وتلقاءها',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَتِلْقَاءَهَا',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'هَا'
}
STOPWORDS[u'وتلقاءهم'] = {
    'word': u'وتلقاءهم',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَتِلْقَاءَهُمْ',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'وتلقاءهن'] = {
    'word': u'وتلقاءهن',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَتِلْقَاءَهُنَّ',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'وتلقاءنا'] = {
    'word': u'وتلقاءنا',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَتِلْقَاءَنَا',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'نَا'
}
STOPWORDS[u'وتلقاءكما'] = {
    'word': u'وتلقاءكما',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَتِلْقَاءَكُمَا',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'وتلقاءهما'] = {
    'word': u'وتلقاءهما',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَتِلْقَاءَهُمَا',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فتلقاءي'] = {
    'word': u'فتلقاءي',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَتِلْقَاءَِي',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'ِي'
}
STOPWORDS[u'فتلقاءك'] = {
    'word': u'فتلقاءك',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَتِلْقَاءَكَ',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'كَ'
}
STOPWORDS[u'فتلقاءك'] = {
    'word': u'فتلقاءك',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَتِلْقَاءَكِ',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'كِ'
}
STOPWORDS[u'فتلقاءه'] = {
    'word': u'فتلقاءه',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَتِلْقَاءَه',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'ه'
}
STOPWORDS[u'فتلقاءكم'] = {
    'word': u'فتلقاءكم',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَتِلْقَاءَكُمْ',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فتلقاءكن'] = {
    'word': u'فتلقاءكن',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَتِلْقَاءَكُنَّ',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فتلقاءها'] = {
    'word': u'فتلقاءها',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَتِلْقَاءَهَا',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'هَا'
}
STOPWORDS[u'فتلقاءهم'] = {
    'word': u'فتلقاءهم',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَتِلْقَاءَهُمْ',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فتلقاءهن'] = {
    'word': u'فتلقاءهن',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَتِلْقَاءَهُنَّ',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فتلقاءنا'] = {
    'word': u'فتلقاءنا',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَتِلْقَاءَنَا',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'نَا'
}
STOPWORDS[u'فتلقاءكما'] = {
    'word': u'فتلقاءكما',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَتِلْقَاءَكُمَا',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فتلقاءهما'] = {
    'word': u'فتلقاءهما',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَتِلْقَاءَهُمَا',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'وبتلقاءي'] = {
    'word': u'وبتلقاءي',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِتِلْقَاءَِي',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'ِي'
}
STOPWORDS[u'وبتلقاءك'] = {
    'word': u'وبتلقاءك',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِتِلْقَاءَكَ',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'كَ'
}
STOPWORDS[u'وبتلقاءك'] = {
    'word': u'وبتلقاءك',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِتِلْقَاءَكِ',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'كِ'
}
STOPWORDS[u'وبتلقاءه'] = {
    'word': u'وبتلقاءه',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِتِلْقَاءَه',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'ه'
}
STOPWORDS[u'وبتلقاءكم'] = {
    'word': u'وبتلقاءكم',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِتِلْقَاءَكُمْ',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'وبتلقاءكن'] = {
    'word': u'وبتلقاءكن',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِتِلْقَاءَكُنَّ',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'وبتلقاءها'] = {
    'word': u'وبتلقاءها',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِتِلْقَاءَهَا',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'هَا'
}
STOPWORDS[u'وبتلقاءهم'] = {
    'word': u'وبتلقاءهم',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِتِلْقَاءَهُمْ',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'وبتلقاءهن'] = {
    'word': u'وبتلقاءهن',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِتِلْقَاءَهُنَّ',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'وبتلقاءنا'] = {
    'word': u'وبتلقاءنا',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِتِلْقَاءَنَا',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'نَا'
}
STOPWORDS[u'وبتلقاءكما'] = {
    'word': u'وبتلقاءكما',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِتِلْقَاءَكُمَا',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'وبتلقاءهما'] = {
    'word': u'وبتلقاءهما',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِتِلْقَاءَهُمَا',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فبتلقاءي'] = {
    'word': u'فبتلقاءي',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِتِلْقَاءَِي',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'ِي'
}
STOPWORDS[u'فبتلقاءك'] = {
    'word': u'فبتلقاءك',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِتِلْقَاءَكَ',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'كَ'
}
STOPWORDS[u'فبتلقاءك'] = {
    'word': u'فبتلقاءك',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِتِلْقَاءَكِ',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'كِ'
}
STOPWORDS[u'فبتلقاءه'] = {
    'word': u'فبتلقاءه',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِتِلْقَاءَه',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'ه'
}
STOPWORDS[u'فبتلقاءكم'] = {
    'word': u'فبتلقاءكم',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِتِلْقَاءَكُمْ',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فبتلقاءكن'] = {
    'word': u'فبتلقاءكن',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِتِلْقَاءَكُنَّ',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فبتلقاءها'] = {
    'word': u'فبتلقاءها',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِتِلْقَاءَهَا',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'هَا'
}
STOPWORDS[u'فبتلقاءهم'] = {
    'word': u'فبتلقاءهم',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِتِلْقَاءَهُمْ',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فبتلقاءهن'] = {
    'word': u'فبتلقاءهن',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِتِلْقَاءَهُنَّ',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فبتلقاءنا'] = {
    'word': u'فبتلقاءنا',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِتِلْقَاءَنَا',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'نَا'
}
STOPWORDS[u'فبتلقاءكما'] = {
    'word': u'فبتلقاءكما',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِتِلْقَاءَكُمَا',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فبتلقاءهما'] = {
    'word': u'فبتلقاءهما',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِتِلْقَاءَهُمَا',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'وكتلقاءي'] = {
    'word': u'وكتلقاءي',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَتِلْقَاءَِي',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'ِي'
}
STOPWORDS[u'وكتلقاءك'] = {
    'word': u'وكتلقاءك',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَتِلْقَاءَكَ',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'كَ'
}
STOPWORDS[u'وكتلقاءك'] = {
    'word': u'وكتلقاءك',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَتِلْقَاءَكِ',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'كِ'
}
STOPWORDS[u'وكتلقاءه'] = {
    'word': u'وكتلقاءه',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَتِلْقَاءَه',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'ه'
}
STOPWORDS[u'وكتلقاءكم'] = {
    'word': u'وكتلقاءكم',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَتِلْقَاءَكُمْ',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'وكتلقاءكن'] = {
    'word': u'وكتلقاءكن',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَتِلْقَاءَكُنَّ',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'وكتلقاءها'] = {
    'word': u'وكتلقاءها',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَتِلْقَاءَهَا',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'هَا'
}
STOPWORDS[u'وكتلقاءهم'] = {
    'word': u'وكتلقاءهم',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَتِلْقَاءَهُمْ',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'وكتلقاءهن'] = {
    'word': u'وكتلقاءهن',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَتِلْقَاءَهُنَّ',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'وكتلقاءنا'] = {
    'word': u'وكتلقاءنا',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَتِلْقَاءَنَا',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'نَا'
}
STOPWORDS[u'وكتلقاءكما'] = {
    'word': u'وكتلقاءكما',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَتِلْقَاءَكُمَا',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'وكتلقاءهما'] = {
    'word': u'وكتلقاءهما',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَتِلْقَاءَهُمَا',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فكتلقاءي'] = {
    'word': u'فكتلقاءي',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَتِلْقَاءَِي',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'ِي'
}
STOPWORDS[u'فكتلقاءك'] = {
    'word': u'فكتلقاءك',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَتِلْقَاءَكَ',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'كَ'
}
STOPWORDS[u'فكتلقاءك'] = {
    'word': u'فكتلقاءك',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَتِلْقَاءَكِ',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'كِ'
}
STOPWORDS[u'فكتلقاءه'] = {
    'word': u'فكتلقاءه',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَتِلْقَاءَه',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'ه'
}
STOPWORDS[u'فكتلقاءكم'] = {
    'word': u'فكتلقاءكم',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَتِلْقَاءَكُمْ',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فكتلقاءكن'] = {
    'word': u'فكتلقاءكن',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَتِلْقَاءَكُنَّ',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فكتلقاءها'] = {
    'word': u'فكتلقاءها',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَتِلْقَاءَهَا',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'هَا'
}
STOPWORDS[u'فكتلقاءهم'] = {
    'word': u'فكتلقاءهم',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَتِلْقَاءَهُمْ',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فكتلقاءهن'] = {
    'word': u'فكتلقاءهن',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَتِلْقَاءَهُنَّ',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فكتلقاءنا'] = {
    'word': u'فكتلقاءنا',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَتِلْقَاءَنَا',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'نَا'
}
STOPWORDS[u'فكتلقاءكما'] = {
    'word': u'فكتلقاءكما',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَتِلْقَاءَكُمَا',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فكتلقاءهما'] = {
    'word': u'فكتلقاءهما',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَتِلْقَاءَهُمَا',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ولتلقاءي'] = {
    'word': u'ولتلقاءي',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِتِلْقَاءَِي',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'ِي'
}
STOPWORDS[u'ولتلقاءك'] = {
    'word': u'ولتلقاءك',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِتِلْقَاءَكَ',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'كَ'
}
STOPWORDS[u'ولتلقاءك'] = {
    'word': u'ولتلقاءك',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِتِلْقَاءَكِ',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'كِ'
}
STOPWORDS[u'ولتلقاءه'] = {
    'word': u'ولتلقاءه',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِتِلْقَاءَه',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'ه'
}
STOPWORDS[u'ولتلقاءكم'] = {
    'word': u'ولتلقاءكم',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِتِلْقَاءَكُمْ',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ولتلقاءكن'] = {
    'word': u'ولتلقاءكن',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِتِلْقَاءَكُنَّ',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ولتلقاءها'] = {
    'word': u'ولتلقاءها',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِتِلْقَاءَهَا',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'هَا'
}
STOPWORDS[u'ولتلقاءهم'] = {
    'word': u'ولتلقاءهم',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِتِلْقَاءَهُمْ',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ولتلقاءهن'] = {
    'word': u'ولتلقاءهن',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِتِلْقَاءَهُنَّ',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ولتلقاءنا'] = {
    'word': u'ولتلقاءنا',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِتِلْقَاءَنَا',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'نَا'
}
STOPWORDS[u'ولتلقاءكما'] = {
    'word': u'ولتلقاءكما',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِتِلْقَاءَكُمَا',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ولتلقاءهما'] = {
    'word': u'ولتلقاءهما',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِتِلْقَاءَهُمَا',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فلتلقاءي'] = {
    'word': u'فلتلقاءي',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِتِلْقَاءَِي',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'ِي'
}
STOPWORDS[u'فلتلقاءك'] = {
    'word': u'فلتلقاءك',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِتِلْقَاءَكَ',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'كَ'
}
STOPWORDS[u'فلتلقاءك'] = {
    'word': u'فلتلقاءك',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِتِلْقَاءَكِ',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'كِ'
}
STOPWORDS[u'فلتلقاءه'] = {
    'word': u'فلتلقاءه',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِتِلْقَاءَه',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'ه'
}
STOPWORDS[u'فلتلقاءكم'] = {
    'word': u'فلتلقاءكم',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِتِلْقَاءَكُمْ',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فلتلقاءكن'] = {
    'word': u'فلتلقاءكن',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِتِلْقَاءَكُنَّ',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فلتلقاءها'] = {
    'word': u'فلتلقاءها',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِتِلْقَاءَهَا',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'هَا'
}
STOPWORDS[u'فلتلقاءهم'] = {
    'word': u'فلتلقاءهم',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِتِلْقَاءَهُمْ',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فلتلقاءهن'] = {
    'word': u'فلتلقاءهن',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِتِلْقَاءَهُنَّ',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فلتلقاءنا'] = {
    'word': u'فلتلقاءنا',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِتِلْقَاءَنَا',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'نَا'
}
STOPWORDS[u'فلتلقاءكما'] = {
    'word': u'فلتلقاءكما',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِتِلْقَاءَكُمَا',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فلتلقاءهما'] = {
    'word': u'فلتلقاءهما',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِتِلْقَاءَهُمَا',
    'stem': u'تِلْقَاءَ',
    'type': u'STOPWORD',
    'original': u'تِلْقَاءَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'جميع'] = {
    'word': u'جَمِيعَ',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة',
    'vocalized': u'جَمِيعَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u''
}
STOPWORDS[u'بجميع'] = {
    'word': u'بجميع',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور',
    'vocalized': u'بِجَمِيعَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u''
}
STOPWORDS[u'كجميع'] = {
    'word': u'كجميع',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور',
    'vocalized': u'كَجَمِيعَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u''
}
STOPWORDS[u'لجميع'] = {
    'word': u'لجميع',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور',
    'vocalized': u'لِجَمِيعَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u''
}
STOPWORDS[u'لجميع'] = {
    'word': u'لجميع',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور',
    'vocalized': u'لَجَمِيعَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u''
}
STOPWORDS[u'وجميع'] = {
    'word': u'وجميع',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف',
    'vocalized': u'وَجَمِيعَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u''
}
STOPWORDS[u'فجميع'] = {
    'word': u'فجميع',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف',
    'vocalized': u'فَجَمِيعَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u''
}
STOPWORDS[u'وبجميع'] = {
    'word': u'وبجميع',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف',
    'vocalized': u'وَبِجَمِيعَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u''
}
STOPWORDS[u'فبجميع'] = {
    'word': u'فبجميع',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف',
    'vocalized': u'فَبِجَمِيعَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u''
}
STOPWORDS[u'وكجميع'] = {
    'word': u'وكجميع',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف',
    'vocalized': u'وَكَجَمِيعَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u''
}
STOPWORDS[u'فكجميع'] = {
    'word': u'فكجميع',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف',
    'vocalized': u'فَكَجَمِيعَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u''
}
STOPWORDS[u'ولجميع'] = {
    'word': u'ولجميع',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف',
    'vocalized': u'وَلِجَمِيعَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u''
}
STOPWORDS[u'فلجميع'] = {
    'word': u'فلجميع',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف',
    'vocalized': u'فَلِجَمِيعَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u''
}
STOPWORDS[u'ولجميع'] = {
    'word': u'ولجميع',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'وَلَجَمِيعَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u''
}
STOPWORDS[u'فلجميع'] = {
    'word': u'فلجميع',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'فَلَجَمِيعَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u''
}
STOPWORDS[u'أجميع'] = {
    'word': u'أجميع',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور',
    'vocalized': u'أَجَمِيعَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u''
}
STOPWORDS[u'أبجميع'] = {
    'word': u'أبجميع',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور',
    'vocalized': u'أَبِجَمِيعَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u''
}
STOPWORDS[u'أكجميع'] = {
    'word': u'أكجميع',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور',
    'vocalized': u'أَكَجَمِيعَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u''
}
STOPWORDS[u'ألجميع'] = {
    'word': u'ألجميع',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور',
    'vocalized': u'أَلِجَمِيعَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u''
}
STOPWORDS[u'ألجميع'] = {
    'word': u'ألجميع',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور',
    'vocalized': u'أَلَجَمِيعَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u''
}
STOPWORDS[u'أوجميع'] = {
    'word': u'أوجميع',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور',
    'vocalized': u'أَوَجَمِيعَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u''
}
STOPWORDS[u'أفجميع'] = {
    'word': u'أفجميع',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور',
    'vocalized': u'أَفَجَمِيعَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u''
}
STOPWORDS[u'أوبجميع'] = {
    'word': u'أوبجميع',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَبِجَمِيعَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u''
}
STOPWORDS[u'أفبجميع'] = {
    'word': u'أفبجميع',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَبِجَمِيعَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u''
}
STOPWORDS[u'أوكجميع'] = {
    'word': u'أوكجميع',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَكَجَمِيعَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u''
}
STOPWORDS[u'أفكجميع'] = {
    'word': u'أفكجميع',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَكَجَمِيعَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u''
}
STOPWORDS[u'أولجميع'] = {
    'word': u'أولجميع',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَلِجَمِيعَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u''
}
STOPWORDS[u'أفلجميع'] = {
    'word': u'أفلجميع',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَلِجَمِيعَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u''
}
STOPWORDS[u'أولجميع'] = {
    'word': u'أولجميع',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَلَجَمِيعَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u''
}
STOPWORDS[u'أفلجميع'] = {
    'word': u'أفلجميع',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَلَجَمِيعَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u''
}
STOPWORDS[u'جميعي'] = {
    'word': u'جميعي',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'جَمِيعَِي',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ِي'
}
STOPWORDS[u'جميعك'] = {
    'word': u'جميعك',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'جَمِيعَكَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كَ'
}
STOPWORDS[u'جميعك'] = {
    'word': u'جميعك',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'جَمِيعَكِ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كِ'
}
STOPWORDS[u'جميعه'] = {
    'word': u'جميعه',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'جَمِيعَه',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ه'
}
STOPWORDS[u'جميعكم'] = {
    'word': u'جميعكم',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'جَمِيعَكُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'جميعكن'] = {
    'word': u'جميعكن',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'جَمِيعَكُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'جميعها'] = {
    'word': u'جميعها',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'جَمِيعَهَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هَا'
}
STOPWORDS[u'جميعهم'] = {
    'word': u'جميعهم',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'جَمِيعَهُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'جميعهن'] = {
    'word': u'جميعهن',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'جَمِيعَهُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'جميعنا'] = {
    'word': u'جميعنا',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'جَمِيعَنَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'نَا'
}
STOPWORDS[u'جميعكما'] = {
    'word': u'جميعكما',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'جَمِيعَكُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'جميعهما'] = {
    'word': u'جميعهما',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'جَمِيعَهُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'بجميعي'] = {
    'word': u'بجميعي',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِجَمِيعَِي',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ِي'
}
STOPWORDS[u'بجميعك'] = {
    'word': u'بجميعك',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِجَمِيعَكَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كَ'
}
STOPWORDS[u'بجميعك'] = {
    'word': u'بجميعك',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِجَمِيعَكِ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كِ'
}
STOPWORDS[u'بجميعه'] = {
    'word': u'بجميعه',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِجَمِيعَه',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ه'
}
STOPWORDS[u'بجميعكم'] = {
    'word': u'بجميعكم',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِجَمِيعَكُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'بجميعكن'] = {
    'word': u'بجميعكن',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِجَمِيعَكُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'بجميعها'] = {
    'word': u'بجميعها',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِجَمِيعَهَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هَا'
}
STOPWORDS[u'بجميعهم'] = {
    'word': u'بجميعهم',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِجَمِيعَهُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'بجميعهن'] = {
    'word': u'بجميعهن',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِجَمِيعَهُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'بجميعنا'] = {
    'word': u'بجميعنا',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِجَمِيعَنَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'نَا'
}
STOPWORDS[u'بجميعكما'] = {
    'word': u'بجميعكما',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِجَمِيعَكُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'بجميعهما'] = {
    'word': u'بجميعهما',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِجَمِيعَهُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'كجميعي'] = {
    'word': u'كجميعي',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَجَمِيعَِي',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ِي'
}
STOPWORDS[u'كجميعك'] = {
    'word': u'كجميعك',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَجَمِيعَكَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كَ'
}
STOPWORDS[u'كجميعك'] = {
    'word': u'كجميعك',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَجَمِيعَكِ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كِ'
}
STOPWORDS[u'كجميعه'] = {
    'word': u'كجميعه',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَجَمِيعَه',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ه'
}
STOPWORDS[u'كجميعكم'] = {
    'word': u'كجميعكم',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَجَمِيعَكُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'كجميعكن'] = {
    'word': u'كجميعكن',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَجَمِيعَكُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'كجميعها'] = {
    'word': u'كجميعها',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَجَمِيعَهَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هَا'
}
STOPWORDS[u'كجميعهم'] = {
    'word': u'كجميعهم',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَجَمِيعَهُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'كجميعهن'] = {
    'word': u'كجميعهن',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَجَمِيعَهُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'كجميعنا'] = {
    'word': u'كجميعنا',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَجَمِيعَنَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'نَا'
}
STOPWORDS[u'كجميعكما'] = {
    'word': u'كجميعكما',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَجَمِيعَكُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'كجميعهما'] = {
    'word': u'كجميعهما',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَجَمِيعَهُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'لجميعي'] = {
    'word': u'لجميعي',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِجَمِيعَِي',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ِي'
}
STOPWORDS[u'لجميعك'] = {
    'word': u'لجميعك',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِجَمِيعَكَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كَ'
}
STOPWORDS[u'لجميعك'] = {
    'word': u'لجميعك',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِجَمِيعَكِ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كِ'
}
STOPWORDS[u'لجميعه'] = {
    'word': u'لجميعه',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِجَمِيعَه',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ه'
}
STOPWORDS[u'لجميعكم'] = {
    'word': u'لجميعكم',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِجَمِيعَكُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'لجميعكن'] = {
    'word': u'لجميعكن',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِجَمِيعَكُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'لجميعها'] = {
    'word': u'لجميعها',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِجَمِيعَهَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هَا'
}
STOPWORDS[u'لجميعهم'] = {
    'word': u'لجميعهم',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِجَمِيعَهُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'لجميعهن'] = {
    'word': u'لجميعهن',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِجَمِيعَهُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'لجميعنا'] = {
    'word': u'لجميعنا',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِجَمِيعَنَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'نَا'
}
STOPWORDS[u'لجميعكما'] = {
    'word': u'لجميعكما',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِجَمِيعَكُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'لجميعهما'] = {
    'word': u'لجميعهما',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِجَمِيعَهُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'لجميعي'] = {
    'word': u'لجميعي',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَجَمِيعَِي',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ِي'
}
STOPWORDS[u'لجميعك'] = {
    'word': u'لجميعك',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَجَمِيعَكَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كَ'
}
STOPWORDS[u'لجميعك'] = {
    'word': u'لجميعك',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَجَمِيعَكِ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كِ'
}
STOPWORDS[u'لجميعه'] = {
    'word': u'لجميعه',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَجَمِيعَه',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ه'
}
STOPWORDS[u'لجميعكم'] = {
    'word': u'لجميعكم',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَجَمِيعَكُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'لجميعكن'] = {
    'word': u'لجميعكن',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَجَمِيعَكُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'لجميعها'] = {
    'word': u'لجميعها',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَجَمِيعَهَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هَا'
}
STOPWORDS[u'لجميعهم'] = {
    'word': u'لجميعهم',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَجَمِيعَهُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'لجميعهن'] = {
    'word': u'لجميعهن',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَجَمِيعَهُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'لجميعنا'] = {
    'word': u'لجميعنا',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَجَمِيعَنَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'نَا'
}
STOPWORDS[u'لجميعكما'] = {
    'word': u'لجميعكما',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَجَمِيعَكُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'لجميعهما'] = {
    'word': u'لجميعهما',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَجَمِيعَهُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'وجميعي'] = {
    'word': u'وجميعي',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَجَمِيعَِي',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ِي'
}
STOPWORDS[u'وجميعك'] = {
    'word': u'وجميعك',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَجَمِيعَكَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كَ'
}
STOPWORDS[u'وجميعك'] = {
    'word': u'وجميعك',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَجَمِيعَكِ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كِ'
}
STOPWORDS[u'وجميعه'] = {
    'word': u'وجميعه',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَجَمِيعَه',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ه'
}
STOPWORDS[u'وجميعكم'] = {
    'word': u'وجميعكم',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَجَمِيعَكُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'وجميعكن'] = {
    'word': u'وجميعكن',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَجَمِيعَكُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'وجميعها'] = {
    'word': u'وجميعها',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَجَمِيعَهَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هَا'
}
STOPWORDS[u'وجميعهم'] = {
    'word': u'وجميعهم',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَجَمِيعَهُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'وجميعهن'] = {
    'word': u'وجميعهن',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَجَمِيعَهُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'وجميعنا'] = {
    'word': u'وجميعنا',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَجَمِيعَنَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'نَا'
}
STOPWORDS[u'وجميعكما'] = {
    'word': u'وجميعكما',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَجَمِيعَكُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'وجميعهما'] = {
    'word': u'وجميعهما',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَجَمِيعَهُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فجميعي'] = {
    'word': u'فجميعي',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَجَمِيعَِي',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ِي'
}
STOPWORDS[u'فجميعك'] = {
    'word': u'فجميعك',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَجَمِيعَكَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كَ'
}
STOPWORDS[u'فجميعك'] = {
    'word': u'فجميعك',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَجَمِيعَكِ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كِ'
}
STOPWORDS[u'فجميعه'] = {
    'word': u'فجميعه',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَجَمِيعَه',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ه'
}
STOPWORDS[u'فجميعكم'] = {
    'word': u'فجميعكم',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَجَمِيعَكُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فجميعكن'] = {
    'word': u'فجميعكن',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَجَمِيعَكُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فجميعها'] = {
    'word': u'فجميعها',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَجَمِيعَهَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هَا'
}
STOPWORDS[u'فجميعهم'] = {
    'word': u'فجميعهم',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَجَمِيعَهُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فجميعهن'] = {
    'word': u'فجميعهن',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَجَمِيعَهُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فجميعنا'] = {
    'word': u'فجميعنا',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَجَمِيعَنَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'نَا'
}
STOPWORDS[u'فجميعكما'] = {
    'word': u'فجميعكما',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَجَمِيعَكُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فجميعهما'] = {
    'word': u'فجميعهما',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَجَمِيعَهُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'وبجميعي'] = {
    'word': u'وبجميعي',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِجَمِيعَِي',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ِي'
}
STOPWORDS[u'وبجميعك'] = {
    'word': u'وبجميعك',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِجَمِيعَكَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كَ'
}
STOPWORDS[u'وبجميعك'] = {
    'word': u'وبجميعك',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِجَمِيعَكِ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كِ'
}
STOPWORDS[u'وبجميعه'] = {
    'word': u'وبجميعه',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِجَمِيعَه',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ه'
}
STOPWORDS[u'وبجميعكم'] = {
    'word': u'وبجميعكم',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِجَمِيعَكُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'وبجميعكن'] = {
    'word': u'وبجميعكن',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِجَمِيعَكُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'وبجميعها'] = {
    'word': u'وبجميعها',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِجَمِيعَهَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هَا'
}
STOPWORDS[u'وبجميعهم'] = {
    'word': u'وبجميعهم',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِجَمِيعَهُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'وبجميعهن'] = {
    'word': u'وبجميعهن',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِجَمِيعَهُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'وبجميعنا'] = {
    'word': u'وبجميعنا',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِجَمِيعَنَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'نَا'
}
STOPWORDS[u'وبجميعكما'] = {
    'word': u'وبجميعكما',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِجَمِيعَكُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'وبجميعهما'] = {
    'word': u'وبجميعهما',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِجَمِيعَهُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فبجميعي'] = {
    'word': u'فبجميعي',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِجَمِيعَِي',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ِي'
}
STOPWORDS[u'فبجميعك'] = {
    'word': u'فبجميعك',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِجَمِيعَكَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كَ'
}
STOPWORDS[u'فبجميعك'] = {
    'word': u'فبجميعك',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِجَمِيعَكِ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كِ'
}
STOPWORDS[u'فبجميعه'] = {
    'word': u'فبجميعه',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِجَمِيعَه',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ه'
}
STOPWORDS[u'فبجميعكم'] = {
    'word': u'فبجميعكم',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِجَمِيعَكُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فبجميعكن'] = {
    'word': u'فبجميعكن',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِجَمِيعَكُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فبجميعها'] = {
    'word': u'فبجميعها',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِجَمِيعَهَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هَا'
}
STOPWORDS[u'فبجميعهم'] = {
    'word': u'فبجميعهم',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِجَمِيعَهُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فبجميعهن'] = {
    'word': u'فبجميعهن',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِجَمِيعَهُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فبجميعنا'] = {
    'word': u'فبجميعنا',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِجَمِيعَنَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'نَا'
}
STOPWORDS[u'فبجميعكما'] = {
    'word': u'فبجميعكما',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِجَمِيعَكُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فبجميعهما'] = {
    'word': u'فبجميعهما',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِجَمِيعَهُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'وكجميعي'] = {
    'word': u'وكجميعي',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَجَمِيعَِي',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ِي'
}
STOPWORDS[u'وكجميعك'] = {
    'word': u'وكجميعك',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَجَمِيعَكَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كَ'
}
STOPWORDS[u'وكجميعك'] = {
    'word': u'وكجميعك',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَجَمِيعَكِ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كِ'
}
STOPWORDS[u'وكجميعه'] = {
    'word': u'وكجميعه',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَجَمِيعَه',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ه'
}
STOPWORDS[u'وكجميعكم'] = {
    'word': u'وكجميعكم',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَجَمِيعَكُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'وكجميعكن'] = {
    'word': u'وكجميعكن',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَجَمِيعَكُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'وكجميعها'] = {
    'word': u'وكجميعها',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَجَمِيعَهَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هَا'
}
STOPWORDS[u'وكجميعهم'] = {
    'word': u'وكجميعهم',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَجَمِيعَهُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'وكجميعهن'] = {
    'word': u'وكجميعهن',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَجَمِيعَهُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'وكجميعنا'] = {
    'word': u'وكجميعنا',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَجَمِيعَنَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'نَا'
}
STOPWORDS[u'وكجميعكما'] = {
    'word': u'وكجميعكما',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَجَمِيعَكُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'وكجميعهما'] = {
    'word': u'وكجميعهما',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَجَمِيعَهُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فكجميعي'] = {
    'word': u'فكجميعي',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَجَمِيعَِي',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ِي'
}
STOPWORDS[u'فكجميعك'] = {
    'word': u'فكجميعك',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَجَمِيعَكَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كَ'
}
STOPWORDS[u'فكجميعك'] = {
    'word': u'فكجميعك',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَجَمِيعَكِ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كِ'
}
STOPWORDS[u'فكجميعه'] = {
    'word': u'فكجميعه',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَجَمِيعَه',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ه'
}
STOPWORDS[u'فكجميعكم'] = {
    'word': u'فكجميعكم',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَجَمِيعَكُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فكجميعكن'] = {
    'word': u'فكجميعكن',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَجَمِيعَكُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فكجميعها'] = {
    'word': u'فكجميعها',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَجَمِيعَهَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هَا'
}
STOPWORDS[u'فكجميعهم'] = {
    'word': u'فكجميعهم',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَجَمِيعَهُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فكجميعهن'] = {
    'word': u'فكجميعهن',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَجَمِيعَهُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فكجميعنا'] = {
    'word': u'فكجميعنا',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَجَمِيعَنَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'نَا'
}
STOPWORDS[u'فكجميعكما'] = {
    'word': u'فكجميعكما',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَجَمِيعَكُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فكجميعهما'] = {
    'word': u'فكجميعهما',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَجَمِيعَهُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ولجميعي'] = {
    'word': u'ولجميعي',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِجَمِيعَِي',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ِي'
}
STOPWORDS[u'ولجميعك'] = {
    'word': u'ولجميعك',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِجَمِيعَكَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كَ'
}
STOPWORDS[u'ولجميعك'] = {
    'word': u'ولجميعك',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِجَمِيعَكِ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كِ'
}
STOPWORDS[u'ولجميعه'] = {
    'word': u'ولجميعه',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِجَمِيعَه',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ه'
}
STOPWORDS[u'ولجميعكم'] = {
    'word': u'ولجميعكم',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِجَمِيعَكُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ولجميعكن'] = {
    'word': u'ولجميعكن',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِجَمِيعَكُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ولجميعها'] = {
    'word': u'ولجميعها',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِجَمِيعَهَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هَا'
}
STOPWORDS[u'ولجميعهم'] = {
    'word': u'ولجميعهم',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِجَمِيعَهُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ولجميعهن'] = {
    'word': u'ولجميعهن',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِجَمِيعَهُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ولجميعنا'] = {
    'word': u'ولجميعنا',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِجَمِيعَنَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'نَا'
}
STOPWORDS[u'ولجميعكما'] = {
    'word': u'ولجميعكما',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِجَمِيعَكُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ولجميعهما'] = {
    'word': u'ولجميعهما',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِجَمِيعَهُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فلجميعي'] = {
    'word': u'فلجميعي',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِجَمِيعَِي',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ِي'
}
STOPWORDS[u'فلجميعك'] = {
    'word': u'فلجميعك',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِجَمِيعَكَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كَ'
}
STOPWORDS[u'فلجميعك'] = {
    'word': u'فلجميعك',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِجَمِيعَكِ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كِ'
}
STOPWORDS[u'فلجميعه'] = {
    'word': u'فلجميعه',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِجَمِيعَه',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ه'
}
STOPWORDS[u'فلجميعكم'] = {
    'word': u'فلجميعكم',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِجَمِيعَكُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فلجميعكن'] = {
    'word': u'فلجميعكن',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِجَمِيعَكُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فلجميعها'] = {
    'word': u'فلجميعها',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِجَمِيعَهَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هَا'
}
STOPWORDS[u'فلجميعهم'] = {
    'word': u'فلجميعهم',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِجَمِيعَهُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فلجميعهن'] = {
    'word': u'فلجميعهن',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِجَمِيعَهُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فلجميعنا'] = {
    'word': u'فلجميعنا',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِجَمِيعَنَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'نَا'
}
STOPWORDS[u'فلجميعكما'] = {
    'word': u'فلجميعكما',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِجَمِيعَكُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فلجميعهما'] = {
    'word': u'فلجميعهما',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِجَمِيعَهُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ولجميعي'] = {
    'word': u'ولجميعي',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَجَمِيعَِي',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ِي'
}
STOPWORDS[u'ولجميعك'] = {
    'word': u'ولجميعك',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَجَمِيعَكَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كَ'
}
STOPWORDS[u'ولجميعك'] = {
    'word': u'ولجميعك',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَجَمِيعَكِ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كِ'
}
STOPWORDS[u'ولجميعه'] = {
    'word': u'ولجميعه',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَجَمِيعَه',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ه'
}
STOPWORDS[u'ولجميعكم'] = {
    'word': u'ولجميعكم',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَجَمِيعَكُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ولجميعكن'] = {
    'word': u'ولجميعكن',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَجَمِيعَكُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ولجميعها'] = {
    'word': u'ولجميعها',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَجَمِيعَهَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هَا'
}
STOPWORDS[u'ولجميعهم'] = {
    'word': u'ولجميعهم',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَجَمِيعَهُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ولجميعهن'] = {
    'word': u'ولجميعهن',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَجَمِيعَهُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ولجميعنا'] = {
    'word': u'ولجميعنا',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَجَمِيعَنَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'نَا'
}
STOPWORDS[u'ولجميعكما'] = {
    'word': u'ولجميعكما',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَجَمِيعَكُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ولجميعهما'] = {
    'word': u'ولجميعهما',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَجَمِيعَهُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فلجميعي'] = {
    'word': u'فلجميعي',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَجَمِيعَِي',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ِي'
}
STOPWORDS[u'فلجميعك'] = {
    'word': u'فلجميعك',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَجَمِيعَكَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كَ'
}
STOPWORDS[u'فلجميعك'] = {
    'word': u'فلجميعك',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَجَمِيعَكِ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كِ'
}
STOPWORDS[u'فلجميعه'] = {
    'word': u'فلجميعه',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَجَمِيعَه',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ه'
}
STOPWORDS[u'فلجميعكم'] = {
    'word': u'فلجميعكم',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَجَمِيعَكُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فلجميعكن'] = {
    'word': u'فلجميعكن',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَجَمِيعَكُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فلجميعها'] = {
    'word': u'فلجميعها',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَجَمِيعَهَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هَا'
}
STOPWORDS[u'فلجميعهم'] = {
    'word': u'فلجميعهم',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَجَمِيعَهُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فلجميعهن'] = {
    'word': u'فلجميعهن',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَجَمِيعَهُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فلجميعنا'] = {
    'word': u'فلجميعنا',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَجَمِيعَنَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'نَا'
}
STOPWORDS[u'فلجميعكما'] = {
    'word': u'فلجميعكما',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَجَمِيعَكُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فلجميعهما'] = {
    'word': u'فلجميعهما',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَجَمِيعَهُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أجميعي'] = {
    'word': u'أجميعي',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'أَجَمِيعَِي',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أجميعك'] = {
    'word': u'أجميعك',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'أَجَمِيعَكَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أجميعك'] = {
    'word': u'أجميعك',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'أَجَمِيعَكِ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أجميعه'] = {
    'word': u'أجميعه',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'أَجَمِيعَه',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ه'
}
STOPWORDS[u'أجميعكم'] = {
    'word': u'أجميعكم',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'أَجَمِيعَكُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أجميعكن'] = {
    'word': u'أجميعكن',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'أَجَمِيعَكُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أجميعها'] = {
    'word': u'أجميعها',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'أَجَمِيعَهَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أجميعهم'] = {
    'word': u'أجميعهم',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'أَجَمِيعَهُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أجميعهن'] = {
    'word': u'أجميعهن',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'أَجَمِيعَهُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أجميعنا'] = {
    'word': u'أجميعنا',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'أَجَمِيعَنَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أجميعكما'] = {
    'word': u'أجميعكما',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'أَجَمِيعَكُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أجميعهما'] = {
    'word': u'أجميعهما',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'أَجَمِيعَهُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أبجميعي'] = {
    'word': u'أبجميعي',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِجَمِيعَِي',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أبجميعك'] = {
    'word': u'أبجميعك',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِجَمِيعَكَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أبجميعك'] = {
    'word': u'أبجميعك',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِجَمِيعَكِ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أبجميعه'] = {
    'word': u'أبجميعه',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِجَمِيعَه',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ه'
}
STOPWORDS[u'أبجميعكم'] = {
    'word': u'أبجميعكم',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِجَمِيعَكُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أبجميعكن'] = {
    'word': u'أبجميعكن',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِجَمِيعَكُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أبجميعها'] = {
    'word': u'أبجميعها',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِجَمِيعَهَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أبجميعهم'] = {
    'word': u'أبجميعهم',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِجَمِيعَهُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أبجميعهن'] = {
    'word': u'أبجميعهن',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِجَمِيعَهُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أبجميعنا'] = {
    'word': u'أبجميعنا',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِجَمِيعَنَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أبجميعكما'] = {
    'word': u'أبجميعكما',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِجَمِيعَكُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أبجميعهما'] = {
    'word': u'أبجميعهما',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِجَمِيعَهُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أكجميعي'] = {
    'word': u'أكجميعي',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَجَمِيعَِي',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أكجميعك'] = {
    'word': u'أكجميعك',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَجَمِيعَكَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أكجميعك'] = {
    'word': u'أكجميعك',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَجَمِيعَكِ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أكجميعه'] = {
    'word': u'أكجميعه',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَجَمِيعَه',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ه'
}
STOPWORDS[u'أكجميعكم'] = {
    'word': u'أكجميعكم',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَجَمِيعَكُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أكجميعكن'] = {
    'word': u'أكجميعكن',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَجَمِيعَكُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أكجميعها'] = {
    'word': u'أكجميعها',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَجَمِيعَهَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أكجميعهم'] = {
    'word': u'أكجميعهم',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَجَمِيعَهُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أكجميعهن'] = {
    'word': u'أكجميعهن',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَجَمِيعَهُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أكجميعنا'] = {
    'word': u'أكجميعنا',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَجَمِيعَنَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أكجميعكما'] = {
    'word': u'أكجميعكما',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَجَمِيعَكُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أكجميعهما'] = {
    'word': u'أكجميعهما',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَجَمِيعَهُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ألجميعي'] = {
    'word': u'ألجميعي',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِجَمِيعَِي',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ِي'
}
STOPWORDS[u'ألجميعك'] = {
    'word': u'ألجميعك',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِجَمِيعَكَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كَ'
}
STOPWORDS[u'ألجميعك'] = {
    'word': u'ألجميعك',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِجَمِيعَكِ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كِ'
}
STOPWORDS[u'ألجميعه'] = {
    'word': u'ألجميعه',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِجَمِيعَه',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ه'
}
STOPWORDS[u'ألجميعكم'] = {
    'word': u'ألجميعكم',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِجَمِيعَكُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ألجميعكن'] = {
    'word': u'ألجميعكن',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِجَمِيعَكُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ألجميعها'] = {
    'word': u'ألجميعها',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِجَمِيعَهَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هَا'
}
STOPWORDS[u'ألجميعهم'] = {
    'word': u'ألجميعهم',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِجَمِيعَهُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ألجميعهن'] = {
    'word': u'ألجميعهن',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِجَمِيعَهُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ألجميعنا'] = {
    'word': u'ألجميعنا',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِجَمِيعَنَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'نَا'
}
STOPWORDS[u'ألجميعكما'] = {
    'word': u'ألجميعكما',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِجَمِيعَكُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ألجميعهما'] = {
    'word': u'ألجميعهما',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِجَمِيعَهُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ألجميعي'] = {
    'word': u'ألجميعي',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَجَمِيعَِي',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ِي'
}
STOPWORDS[u'ألجميعك'] = {
    'word': u'ألجميعك',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَجَمِيعَكَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كَ'
}
STOPWORDS[u'ألجميعك'] = {
    'word': u'ألجميعك',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَجَمِيعَكِ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كِ'
}
STOPWORDS[u'ألجميعه'] = {
    'word': u'ألجميعه',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَجَمِيعَه',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ه'
}
STOPWORDS[u'ألجميعكم'] = {
    'word': u'ألجميعكم',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَجَمِيعَكُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ألجميعكن'] = {
    'word': u'ألجميعكن',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَجَمِيعَكُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ألجميعها'] = {
    'word': u'ألجميعها',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَجَمِيعَهَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هَا'
}
STOPWORDS[u'ألجميعهم'] = {
    'word': u'ألجميعهم',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَجَمِيعَهُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ألجميعهن'] = {
    'word': u'ألجميعهن',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَجَمِيعَهُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ألجميعنا'] = {
    'word': u'ألجميعنا',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَجَمِيعَنَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'نَا'
}
STOPWORDS[u'ألجميعكما'] = {
    'word': u'ألجميعكما',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَجَمِيعَكُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ألجميعهما'] = {
    'word': u'ألجميعهما',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَجَمِيعَهُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أوجميعي'] = {
    'word': u'أوجميعي',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَجَمِيعَِي',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أوجميعك'] = {
    'word': u'أوجميعك',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَجَمِيعَكَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أوجميعك'] = {
    'word': u'أوجميعك',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَجَمِيعَكِ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أوجميعه'] = {
    'word': u'أوجميعه',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَجَمِيعَه',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ه'
}
STOPWORDS[u'أوجميعكم'] = {
    'word': u'أوجميعكم',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَجَمِيعَكُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أوجميعكن'] = {
    'word': u'أوجميعكن',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَجَمِيعَكُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أوجميعها'] = {
    'word': u'أوجميعها',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَجَمِيعَهَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أوجميعهم'] = {
    'word': u'أوجميعهم',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَجَمِيعَهُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أوجميعهن'] = {
    'word': u'أوجميعهن',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَجَمِيعَهُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أوجميعنا'] = {
    'word': u'أوجميعنا',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَجَمِيعَنَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أوجميعكما'] = {
    'word': u'أوجميعكما',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَجَمِيعَكُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أوجميعهما'] = {
    'word': u'أوجميعهما',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَجَمِيعَهُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أفجميعي'] = {
    'word': u'أفجميعي',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَجَمِيعَِي',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أفجميعك'] = {
    'word': u'أفجميعك',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَجَمِيعَكَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أفجميعك'] = {
    'word': u'أفجميعك',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَجَمِيعَكِ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أفجميعه'] = {
    'word': u'أفجميعه',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَجَمِيعَه',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ه'
}
STOPWORDS[u'أفجميعكم'] = {
    'word': u'أفجميعكم',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَجَمِيعَكُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أفجميعكن'] = {
    'word': u'أفجميعكن',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَجَمِيعَكُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أفجميعها'] = {
    'word': u'أفجميعها',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَجَمِيعَهَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أفجميعهم'] = {
    'word': u'أفجميعهم',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَجَمِيعَهُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أفجميعهن'] = {
    'word': u'أفجميعهن',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَجَمِيعَهُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أفجميعنا'] = {
    'word': u'أفجميعنا',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَجَمِيعَنَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أفجميعكما'] = {
    'word': u'أفجميعكما',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَجَمِيعَكُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أفجميعهما'] = {
    'word': u'أفجميعهما',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَجَمِيعَهُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أوبجميعي'] = {
    'word': u'أوبجميعي',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِجَمِيعَِي',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أوبجميعك'] = {
    'word': u'أوبجميعك',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِجَمِيعَكَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أوبجميعك'] = {
    'word': u'أوبجميعك',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِجَمِيعَكِ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أوبجميعه'] = {
    'word': u'أوبجميعه',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِجَمِيعَه',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ه'
}
STOPWORDS[u'أوبجميعكم'] = {
    'word': u'أوبجميعكم',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِجَمِيعَكُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أوبجميعكن'] = {
    'word': u'أوبجميعكن',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِجَمِيعَكُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أوبجميعها'] = {
    'word': u'أوبجميعها',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِجَمِيعَهَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أوبجميعهم'] = {
    'word': u'أوبجميعهم',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِجَمِيعَهُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أوبجميعهن'] = {
    'word': u'أوبجميعهن',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِجَمِيعَهُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أوبجميعنا'] = {
    'word': u'أوبجميعنا',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِجَمِيعَنَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أوبجميعكما'] = {
    'word': u'أوبجميعكما',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِجَمِيعَكُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أوبجميعهما'] = {
    'word': u'أوبجميعهما',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِجَمِيعَهُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أفبجميعي'] = {
    'word': u'أفبجميعي',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِجَمِيعَِي',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أفبجميعك'] = {
    'word': u'أفبجميعك',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِجَمِيعَكَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أفبجميعك'] = {
    'word': u'أفبجميعك',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِجَمِيعَكِ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أفبجميعه'] = {
    'word': u'أفبجميعه',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِجَمِيعَه',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ه'
}
STOPWORDS[u'أفبجميعكم'] = {
    'word': u'أفبجميعكم',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِجَمِيعَكُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أفبجميعكن'] = {
    'word': u'أفبجميعكن',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِجَمِيعَكُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أفبجميعها'] = {
    'word': u'أفبجميعها',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِجَمِيعَهَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أفبجميعهم'] = {
    'word': u'أفبجميعهم',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِجَمِيعَهُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أفبجميعهن'] = {
    'word': u'أفبجميعهن',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِجَمِيعَهُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أفبجميعنا'] = {
    'word': u'أفبجميعنا',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِجَمِيعَنَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أفبجميعكما'] = {
    'word': u'أفبجميعكما',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِجَمِيعَكُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أفبجميعهما'] = {
    'word': u'أفبجميعهما',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِجَمِيعَهُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أوكجميعي'] = {
    'word': u'أوكجميعي',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَجَمِيعَِي',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أوكجميعك'] = {
    'word': u'أوكجميعك',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَجَمِيعَكَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أوكجميعك'] = {
    'word': u'أوكجميعك',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَجَمِيعَكِ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أوكجميعه'] = {
    'word': u'أوكجميعه',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَجَمِيعَه',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ه'
}
STOPWORDS[u'أوكجميعكم'] = {
    'word': u'أوكجميعكم',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَجَمِيعَكُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أوكجميعكن'] = {
    'word': u'أوكجميعكن',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَجَمِيعَكُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أوكجميعها'] = {
    'word': u'أوكجميعها',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَجَمِيعَهَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أوكجميعهم'] = {
    'word': u'أوكجميعهم',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَجَمِيعَهُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أوكجميعهن'] = {
    'word': u'أوكجميعهن',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَجَمِيعَهُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أوكجميعنا'] = {
    'word': u'أوكجميعنا',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَجَمِيعَنَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أوكجميعكما'] = {
    'word': u'أوكجميعكما',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَجَمِيعَكُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أوكجميعهما'] = {
    'word': u'أوكجميعهما',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَجَمِيعَهُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أفكجميعي'] = {
    'word': u'أفكجميعي',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَجَمِيعَِي',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أفكجميعك'] = {
    'word': u'أفكجميعك',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَجَمِيعَكَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أفكجميعك'] = {
    'word': u'أفكجميعك',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَجَمِيعَكِ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أفكجميعه'] = {
    'word': u'أفكجميعه',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَجَمِيعَه',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ه'
}
STOPWORDS[u'أفكجميعكم'] = {
    'word': u'أفكجميعكم',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَجَمِيعَكُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أفكجميعكن'] = {
    'word': u'أفكجميعكن',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَجَمِيعَكُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أفكجميعها'] = {
    'word': u'أفكجميعها',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَجَمِيعَهَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أفكجميعهم'] = {
    'word': u'أفكجميعهم',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَجَمِيعَهُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أفكجميعهن'] = {
    'word': u'أفكجميعهن',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَجَمِيعَهُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أفكجميعنا'] = {
    'word': u'أفكجميعنا',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَجَمِيعَنَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أفكجميعكما'] = {
    'word': u'أفكجميعكما',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَجَمِيعَكُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أفكجميعهما'] = {
    'word': u'أفكجميعهما',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَجَمِيعَهُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أولجميعي'] = {
    'word': u'أولجميعي',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِجَمِيعَِي',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أولجميعك'] = {
    'word': u'أولجميعك',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِجَمِيعَكَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أولجميعك'] = {
    'word': u'أولجميعك',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِجَمِيعَكِ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أولجميعه'] = {
    'word': u'أولجميعه',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِجَمِيعَه',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ه'
}
STOPWORDS[u'أولجميعكم'] = {
    'word': u'أولجميعكم',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِجَمِيعَكُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أولجميعكن'] = {
    'word': u'أولجميعكن',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِجَمِيعَكُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أولجميعها'] = {
    'word': u'أولجميعها',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِجَمِيعَهَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أولجميعهم'] = {
    'word': u'أولجميعهم',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِجَمِيعَهُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أولجميعهن'] = {
    'word': u'أولجميعهن',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِجَمِيعَهُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أولجميعنا'] = {
    'word': u'أولجميعنا',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِجَمِيعَنَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أولجميعكما'] = {
    'word': u'أولجميعكما',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِجَمِيعَكُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أولجميعهما'] = {
    'word': u'أولجميعهما',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِجَمِيعَهُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أفلجميعي'] = {
    'word': u'أفلجميعي',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِجَمِيعَِي',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أفلجميعك'] = {
    'word': u'أفلجميعك',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِجَمِيعَكَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أفلجميعك'] = {
    'word': u'أفلجميعك',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِجَمِيعَكِ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أفلجميعه'] = {
    'word': u'أفلجميعه',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِجَمِيعَه',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ه'
}
STOPWORDS[u'أفلجميعكم'] = {
    'word': u'أفلجميعكم',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِجَمِيعَكُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أفلجميعكن'] = {
    'word': u'أفلجميعكن',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِجَمِيعَكُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أفلجميعها'] = {
    'word': u'أفلجميعها',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِجَمِيعَهَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أفلجميعهم'] = {
    'word': u'أفلجميعهم',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِجَمِيعَهُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أفلجميعهن'] = {
    'word': u'أفلجميعهن',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِجَمِيعَهُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أفلجميعنا'] = {
    'word': u'أفلجميعنا',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِجَمِيعَنَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أفلجميعكما'] = {
    'word': u'أفلجميعكما',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِجَمِيعَكُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أفلجميعهما'] = {
    'word': u'أفلجميعهما',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِجَمِيعَهُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أولجميعي'] = {
    'word': u'أولجميعي',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَجَمِيعَِي',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أولجميعك'] = {
    'word': u'أولجميعك',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَجَمِيعَكَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أولجميعك'] = {
    'word': u'أولجميعك',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَجَمِيعَكِ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أولجميعه'] = {
    'word': u'أولجميعه',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَجَمِيعَه',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ه'
}
STOPWORDS[u'أولجميعكم'] = {
    'word': u'أولجميعكم',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَجَمِيعَكُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أولجميعكن'] = {
    'word': u'أولجميعكن',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَجَمِيعَكُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أولجميعها'] = {
    'word': u'أولجميعها',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَجَمِيعَهَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أولجميعهم'] = {
    'word': u'أولجميعهم',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَجَمِيعَهُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أولجميعهن'] = {
    'word': u'أولجميعهن',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَجَمِيعَهُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أولجميعنا'] = {
    'word': u'أولجميعنا',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَجَمِيعَنَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أولجميعكما'] = {
    'word': u'أولجميعكما',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَجَمِيعَكُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أولجميعهما'] = {
    'word': u'أولجميعهما',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَجَمِيعَهُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أفلجميعي'] = {
    'word': u'أفلجميعي',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَجَمِيعَِي',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أفلجميعك'] = {
    'word': u'أفلجميعك',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَجَمِيعَكَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أفلجميعك'] = {
    'word': u'أفلجميعك',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَجَمِيعَكِ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أفلجميعه'] = {
    'word': u'أفلجميعه',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَجَمِيعَه',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ه'
}
STOPWORDS[u'أفلجميعكم'] = {
    'word': u'أفلجميعكم',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَجَمِيعَكُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أفلجميعكن'] = {
    'word': u'أفلجميعكن',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَجَمِيعَكُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أفلجميعها'] = {
    'word': u'أفلجميعها',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَجَمِيعَهَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أفلجميعهم'] = {
    'word': u'أفلجميعهم',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَجَمِيعَهُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أفلجميعهن'] = {
    'word': u'أفلجميعهن',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَجَمِيعَهُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أفلجميعنا'] = {
    'word': u'أفلجميعنا',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَجَمِيعَنَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أفلجميعكما'] = {
    'word': u'أفلجميعكما',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَجَمِيعَكُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أفلجميعهما'] = {
    'word': u'أفلجميعهما',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَجَمِيعَهُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'حسب'] = {
    'word': u'حَسْبَ',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة',
    'vocalized': u'حَسْبَ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u''
}
STOPWORDS[u'بحسب'] = {
    'word': u'بحسب',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور',
    'vocalized': u'بِحَسْبَ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u''
}
STOPWORDS[u'كحسب'] = {
    'word': u'كحسب',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور',
    'vocalized': u'كَحَسْبَ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u''
}
STOPWORDS[u'لحسب'] = {
    'word': u'لحسب',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور',
    'vocalized': u'لِحَسْبَ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u''
}
STOPWORDS[u'وحسب'] = {
    'word': u'وحسب',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف',
    'vocalized': u'وَحَسْبَ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u''
}
STOPWORDS[u'فحسب'] = {
    'word': u'فحسب',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف',
    'vocalized': u'فَحَسْبَ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u''
}
STOPWORDS[u'وبحسب'] = {
    'word': u'وبحسب',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف',
    'vocalized': u'وَبِحَسْبَ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u''
}
STOPWORDS[u'فبحسب'] = {
    'word': u'فبحسب',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف',
    'vocalized': u'فَبِحَسْبَ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u''
}
STOPWORDS[u'وكحسب'] = {
    'word': u'وكحسب',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف',
    'vocalized': u'وَكَحَسْبَ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u''
}
STOPWORDS[u'فكحسب'] = {
    'word': u'فكحسب',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف',
    'vocalized': u'فَكَحَسْبَ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u''
}
STOPWORDS[u'ولحسب'] = {
    'word': u'ولحسب',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف',
    'vocalized': u'وَلِحَسْبَ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u''
}
STOPWORDS[u'فلحسب'] = {
    'word': u'فلحسب',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف',
    'vocalized': u'فَلِحَسْبَ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u''
}
STOPWORDS[u'أحسب'] = {
    'word': u'أحسب',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور',
    'vocalized': u'أَحَسْبَ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u''
}
STOPWORDS[u'أبحسب'] = {
    'word': u'أبحسب',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور',
    'vocalized': u'أَبِحَسْبَ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u''
}
STOPWORDS[u'أكحسب'] = {
    'word': u'أكحسب',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور',
    'vocalized': u'أَكَحَسْبَ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u''
}
STOPWORDS[u'ألحسب'] = {
    'word': u'ألحسب',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور',
    'vocalized': u'أَلِحَسْبَ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u''
}
STOPWORDS[u'أوحسب'] = {
    'word': u'أوحسب',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور',
    'vocalized': u'أَوَحَسْبَ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u''
}
STOPWORDS[u'أفحسب'] = {
    'word': u'أفحسب',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور',
    'vocalized': u'أَفَحَسْبَ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u''
}
STOPWORDS[u'أوبحسب'] = {
    'word': u'أوبحسب',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَبِحَسْبَ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u''
}
STOPWORDS[u'أفبحسب'] = {
    'word': u'أفبحسب',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَبِحَسْبَ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u''
}
STOPWORDS[u'أوكحسب'] = {
    'word': u'أوكحسب',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَكَحَسْبَ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u''
}
STOPWORDS[u'أفكحسب'] = {
    'word': u'أفكحسب',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَكَحَسْبَ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u''
}
STOPWORDS[u'أولحسب'] = {
    'word': u'أولحسب',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَلِحَسْبَ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u''
}
STOPWORDS[u'أفلحسب'] = {
    'word': u'أفلحسب',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَلِحَسْبَ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u''
}
STOPWORDS[u'حسبي'] = {
    'word': u'حسبي',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'حَسْبَِي',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'ِي'
}
STOPWORDS[u'حسبك'] = {
    'word': u'حسبك',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'حَسْبَكَ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كَ'
}
STOPWORDS[u'حسبك'] = {
    'word': u'حسبك',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'حَسْبَكِ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كِ'
}
STOPWORDS[u'حسبه'] = {
    'word': u'حسبه',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'حَسْبَه',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'ه'
}
STOPWORDS[u'حسبكم'] = {
    'word': u'حسبكم',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'حَسْبَكُمْ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'حسبكن'] = {
    'word': u'حسبكن',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'حَسْبَكُنَّ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'حسبها'] = {
    'word': u'حسبها',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'حَسْبَهَا',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'هَا'
}
STOPWORDS[u'حسبهم'] = {
    'word': u'حسبهم',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'حَسْبَهُمْ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'حسبهن'] = {
    'word': u'حسبهن',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'حَسْبَهُنَّ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'حسبنا'] = {
    'word': u'حسبنا',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'حَسْبَنَا',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'نَا'
}
STOPWORDS[u'حسبكما'] = {
    'word': u'حسبكما',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'حَسْبَكُمَا',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'حسبهما'] = {
    'word': u'حسبهما',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'حَسْبَهُمَا',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'بحسبي'] = {
    'word': u'بحسبي',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِحَسْبَِي',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'ِي'
}
STOPWORDS[u'بحسبك'] = {
    'word': u'بحسبك',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِحَسْبَكَ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كَ'
}
STOPWORDS[u'بحسبك'] = {
    'word': u'بحسبك',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِحَسْبَكِ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كِ'
}
STOPWORDS[u'بحسبه'] = {
    'word': u'بحسبه',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِحَسْبَه',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'ه'
}
STOPWORDS[u'بحسبكم'] = {
    'word': u'بحسبكم',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِحَسْبَكُمْ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'بحسبكن'] = {
    'word': u'بحسبكن',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِحَسْبَكُنَّ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'بحسبها'] = {
    'word': u'بحسبها',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِحَسْبَهَا',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'هَا'
}
STOPWORDS[u'بحسبهم'] = {
    'word': u'بحسبهم',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِحَسْبَهُمْ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'بحسبهن'] = {
    'word': u'بحسبهن',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِحَسْبَهُنَّ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'بحسبنا'] = {
    'word': u'بحسبنا',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِحَسْبَنَا',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'نَا'
}
STOPWORDS[u'بحسبكما'] = {
    'word': u'بحسبكما',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِحَسْبَكُمَا',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'بحسبهما'] = {
    'word': u'بحسبهما',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِحَسْبَهُمَا',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'كحسبي'] = {
    'word': u'كحسبي',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَحَسْبَِي',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'ِي'
}
STOPWORDS[u'كحسبك'] = {
    'word': u'كحسبك',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَحَسْبَكَ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كَ'
}
STOPWORDS[u'كحسبك'] = {
    'word': u'كحسبك',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَحَسْبَكِ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كِ'
}
STOPWORDS[u'كحسبه'] = {
    'word': u'كحسبه',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَحَسْبَه',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'ه'
}
STOPWORDS[u'كحسبكم'] = {
    'word': u'كحسبكم',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَحَسْبَكُمْ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'كحسبكن'] = {
    'word': u'كحسبكن',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَحَسْبَكُنَّ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'كحسبها'] = {
    'word': u'كحسبها',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَحَسْبَهَا',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'هَا'
}
STOPWORDS[u'كحسبهم'] = {
    'word': u'كحسبهم',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَحَسْبَهُمْ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'كحسبهن'] = {
    'word': u'كحسبهن',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَحَسْبَهُنَّ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'كحسبنا'] = {
    'word': u'كحسبنا',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَحَسْبَنَا',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'نَا'
}
STOPWORDS[u'كحسبكما'] = {
    'word': u'كحسبكما',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَحَسْبَكُمَا',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'كحسبهما'] = {
    'word': u'كحسبهما',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَحَسْبَهُمَا',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'لحسبي'] = {
    'word': u'لحسبي',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِحَسْبَِي',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'ِي'
}
STOPWORDS[u'لحسبك'] = {
    'word': u'لحسبك',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِحَسْبَكَ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كَ'
}
STOPWORDS[u'لحسبك'] = {
    'word': u'لحسبك',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِحَسْبَكِ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كِ'
}
STOPWORDS[u'لحسبه'] = {
    'word': u'لحسبه',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِحَسْبَه',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'ه'
}
STOPWORDS[u'لحسبكم'] = {
    'word': u'لحسبكم',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِحَسْبَكُمْ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'لحسبكن'] = {
    'word': u'لحسبكن',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِحَسْبَكُنَّ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'لحسبها'] = {
    'word': u'لحسبها',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِحَسْبَهَا',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'هَا'
}
STOPWORDS[u'لحسبهم'] = {
    'word': u'لحسبهم',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِحَسْبَهُمْ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'لحسبهن'] = {
    'word': u'لحسبهن',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِحَسْبَهُنَّ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'لحسبنا'] = {
    'word': u'لحسبنا',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِحَسْبَنَا',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'نَا'
}
STOPWORDS[u'لحسبكما'] = {
    'word': u'لحسبكما',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِحَسْبَكُمَا',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'لحسبهما'] = {
    'word': u'لحسبهما',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِحَسْبَهُمَا',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'وحسبي'] = {
    'word': u'وحسبي',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَحَسْبَِي',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'ِي'
}
STOPWORDS[u'وحسبك'] = {
    'word': u'وحسبك',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَحَسْبَكَ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كَ'
}
STOPWORDS[u'وحسبك'] = {
    'word': u'وحسبك',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَحَسْبَكِ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كِ'
}
STOPWORDS[u'وحسبه'] = {
    'word': u'وحسبه',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَحَسْبَه',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'ه'
}
STOPWORDS[u'وحسبكم'] = {
    'word': u'وحسبكم',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَحَسْبَكُمْ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'وحسبكن'] = {
    'word': u'وحسبكن',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَحَسْبَكُنَّ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'وحسبها'] = {
    'word': u'وحسبها',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَحَسْبَهَا',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'هَا'
}
STOPWORDS[u'وحسبهم'] = {
    'word': u'وحسبهم',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَحَسْبَهُمْ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'وحسبهن'] = {
    'word': u'وحسبهن',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَحَسْبَهُنَّ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'وحسبنا'] = {
    'word': u'وحسبنا',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَحَسْبَنَا',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'نَا'
}
STOPWORDS[u'وحسبكما'] = {
    'word': u'وحسبكما',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَحَسْبَكُمَا',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'وحسبهما'] = {
    'word': u'وحسبهما',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَحَسْبَهُمَا',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فحسبي'] = {
    'word': u'فحسبي',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَحَسْبَِي',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'ِي'
}
STOPWORDS[u'فحسبك'] = {
    'word': u'فحسبك',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَحَسْبَكَ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كَ'
}
STOPWORDS[u'فحسبك'] = {
    'word': u'فحسبك',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَحَسْبَكِ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كِ'
}
STOPWORDS[u'فحسبه'] = {
    'word': u'فحسبه',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَحَسْبَه',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'ه'
}
STOPWORDS[u'فحسبكم'] = {
    'word': u'فحسبكم',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَحَسْبَكُمْ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فحسبكن'] = {
    'word': u'فحسبكن',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَحَسْبَكُنَّ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فحسبها'] = {
    'word': u'فحسبها',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَحَسْبَهَا',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'هَا'
}
STOPWORDS[u'فحسبهم'] = {
    'word': u'فحسبهم',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَحَسْبَهُمْ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فحسبهن'] = {
    'word': u'فحسبهن',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَحَسْبَهُنَّ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فحسبنا'] = {
    'word': u'فحسبنا',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَحَسْبَنَا',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'نَا'
}
STOPWORDS[u'فحسبكما'] = {
    'word': u'فحسبكما',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَحَسْبَكُمَا',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فحسبهما'] = {
    'word': u'فحسبهما',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَحَسْبَهُمَا',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'وبحسبي'] = {
    'word': u'وبحسبي',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِحَسْبَِي',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'ِي'
}
STOPWORDS[u'وبحسبك'] = {
    'word': u'وبحسبك',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِحَسْبَكَ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كَ'
}
STOPWORDS[u'وبحسبك'] = {
    'word': u'وبحسبك',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِحَسْبَكِ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كِ'
}
STOPWORDS[u'وبحسبه'] = {
    'word': u'وبحسبه',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِحَسْبَه',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'ه'
}
STOPWORDS[u'وبحسبكم'] = {
    'word': u'وبحسبكم',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِحَسْبَكُمْ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'وبحسبكن'] = {
    'word': u'وبحسبكن',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِحَسْبَكُنَّ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'وبحسبها'] = {
    'word': u'وبحسبها',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِحَسْبَهَا',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'هَا'
}
STOPWORDS[u'وبحسبهم'] = {
    'word': u'وبحسبهم',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِحَسْبَهُمْ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'وبحسبهن'] = {
    'word': u'وبحسبهن',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِحَسْبَهُنَّ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'وبحسبنا'] = {
    'word': u'وبحسبنا',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِحَسْبَنَا',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'نَا'
}
STOPWORDS[u'وبحسبكما'] = {
    'word': u'وبحسبكما',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِحَسْبَكُمَا',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'وبحسبهما'] = {
    'word': u'وبحسبهما',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِحَسْبَهُمَا',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فبحسبي'] = {
    'word': u'فبحسبي',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِحَسْبَِي',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'ِي'
}
STOPWORDS[u'فبحسبك'] = {
    'word': u'فبحسبك',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِحَسْبَكَ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كَ'
}
STOPWORDS[u'فبحسبك'] = {
    'word': u'فبحسبك',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِحَسْبَكِ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كِ'
}
STOPWORDS[u'فبحسبه'] = {
    'word': u'فبحسبه',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِحَسْبَه',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'ه'
}
STOPWORDS[u'فبحسبكم'] = {
    'word': u'فبحسبكم',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِحَسْبَكُمْ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فبحسبكن'] = {
    'word': u'فبحسبكن',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِحَسْبَكُنَّ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فبحسبها'] = {
    'word': u'فبحسبها',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِحَسْبَهَا',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'هَا'
}
STOPWORDS[u'فبحسبهم'] = {
    'word': u'فبحسبهم',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِحَسْبَهُمْ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فبحسبهن'] = {
    'word': u'فبحسبهن',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِحَسْبَهُنَّ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فبحسبنا'] = {
    'word': u'فبحسبنا',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِحَسْبَنَا',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'نَا'
}
STOPWORDS[u'فبحسبكما'] = {
    'word': u'فبحسبكما',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِحَسْبَكُمَا',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فبحسبهما'] = {
    'word': u'فبحسبهما',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِحَسْبَهُمَا',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'وكحسبي'] = {
    'word': u'وكحسبي',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَحَسْبَِي',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'ِي'
}
STOPWORDS[u'وكحسبك'] = {
    'word': u'وكحسبك',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَحَسْبَكَ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كَ'
}
STOPWORDS[u'وكحسبك'] = {
    'word': u'وكحسبك',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَحَسْبَكِ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كِ'
}
STOPWORDS[u'وكحسبه'] = {
    'word': u'وكحسبه',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَحَسْبَه',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'ه'
}
STOPWORDS[u'وكحسبكم'] = {
    'word': u'وكحسبكم',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَحَسْبَكُمْ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'وكحسبكن'] = {
    'word': u'وكحسبكن',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَحَسْبَكُنَّ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'وكحسبها'] = {
    'word': u'وكحسبها',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَحَسْبَهَا',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'هَا'
}
STOPWORDS[u'وكحسبهم'] = {
    'word': u'وكحسبهم',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَحَسْبَهُمْ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'وكحسبهن'] = {
    'word': u'وكحسبهن',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَحَسْبَهُنَّ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'وكحسبنا'] = {
    'word': u'وكحسبنا',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَحَسْبَنَا',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'نَا'
}
STOPWORDS[u'وكحسبكما'] = {
    'word': u'وكحسبكما',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَحَسْبَكُمَا',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'وكحسبهما'] = {
    'word': u'وكحسبهما',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَحَسْبَهُمَا',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فكحسبي'] = {
    'word': u'فكحسبي',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَحَسْبَِي',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'ِي'
}
STOPWORDS[u'فكحسبك'] = {
    'word': u'فكحسبك',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَحَسْبَكَ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كَ'
}
STOPWORDS[u'فكحسبك'] = {
    'word': u'فكحسبك',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَحَسْبَكِ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كِ'
}
STOPWORDS[u'فكحسبه'] = {
    'word': u'فكحسبه',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَحَسْبَه',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'ه'
}
STOPWORDS[u'فكحسبكم'] = {
    'word': u'فكحسبكم',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَحَسْبَكُمْ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فكحسبكن'] = {
    'word': u'فكحسبكن',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَحَسْبَكُنَّ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فكحسبها'] = {
    'word': u'فكحسبها',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَحَسْبَهَا',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'هَا'
}
STOPWORDS[u'فكحسبهم'] = {
    'word': u'فكحسبهم',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَحَسْبَهُمْ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فكحسبهن'] = {
    'word': u'فكحسبهن',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَحَسْبَهُنَّ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فكحسبنا'] = {
    'word': u'فكحسبنا',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَحَسْبَنَا',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'نَا'
}
STOPWORDS[u'فكحسبكما'] = {
    'word': u'فكحسبكما',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَحَسْبَكُمَا',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فكحسبهما'] = {
    'word': u'فكحسبهما',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَحَسْبَهُمَا',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ولحسبي'] = {
    'word': u'ولحسبي',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِحَسْبَِي',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'ِي'
}
STOPWORDS[u'ولحسبك'] = {
    'word': u'ولحسبك',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِحَسْبَكَ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كَ'
}
STOPWORDS[u'ولحسبك'] = {
    'word': u'ولحسبك',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِحَسْبَكِ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كِ'
}
STOPWORDS[u'ولحسبه'] = {
    'word': u'ولحسبه',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِحَسْبَه',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'ه'
}
STOPWORDS[u'ولحسبكم'] = {
    'word': u'ولحسبكم',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِحَسْبَكُمْ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ولحسبكن'] = {
    'word': u'ولحسبكن',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِحَسْبَكُنَّ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ولحسبها'] = {
    'word': u'ولحسبها',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِحَسْبَهَا',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'هَا'
}
STOPWORDS[u'ولحسبهم'] = {
    'word': u'ولحسبهم',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِحَسْبَهُمْ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ولحسبهن'] = {
    'word': u'ولحسبهن',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِحَسْبَهُنَّ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ولحسبنا'] = {
    'word': u'ولحسبنا',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِحَسْبَنَا',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'نَا'
}
STOPWORDS[u'ولحسبكما'] = {
    'word': u'ولحسبكما',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِحَسْبَكُمَا',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ولحسبهما'] = {
    'word': u'ولحسبهما',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِحَسْبَهُمَا',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فلحسبي'] = {
    'word': u'فلحسبي',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِحَسْبَِي',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'ِي'
}
STOPWORDS[u'فلحسبك'] = {
    'word': u'فلحسبك',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِحَسْبَكَ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كَ'
}
STOPWORDS[u'فلحسبك'] = {
    'word': u'فلحسبك',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِحَسْبَكِ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كِ'
}
STOPWORDS[u'فلحسبه'] = {
    'word': u'فلحسبه',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِحَسْبَه',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'ه'
}
STOPWORDS[u'فلحسبكم'] = {
    'word': u'فلحسبكم',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِحَسْبَكُمْ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فلحسبكن'] = {
    'word': u'فلحسبكن',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِحَسْبَكُنَّ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فلحسبها'] = {
    'word': u'فلحسبها',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِحَسْبَهَا',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'هَا'
}
STOPWORDS[u'فلحسبهم'] = {
    'word': u'فلحسبهم',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِحَسْبَهُمْ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فلحسبهن'] = {
    'word': u'فلحسبهن',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِحَسْبَهُنَّ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فلحسبنا'] = {
    'word': u'فلحسبنا',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِحَسْبَنَا',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'نَا'
}
STOPWORDS[u'فلحسبكما'] = {
    'word': u'فلحسبكما',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِحَسْبَكُمَا',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فلحسبهما'] = {
    'word': u'فلحسبهما',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِحَسْبَهُمَا',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أحسبي'] = {
    'word': u'أحسبي',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'أَحَسْبَِي',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أحسبك'] = {
    'word': u'أحسبك',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'أَحَسْبَكَ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أحسبك'] = {
    'word': u'أحسبك',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'أَحَسْبَكِ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أحسبه'] = {
    'word': u'أحسبه',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'أَحَسْبَه',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'ه'
}
STOPWORDS[u'أحسبكم'] = {
    'word': u'أحسبكم',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'أَحَسْبَكُمْ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أحسبكن'] = {
    'word': u'أحسبكن',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'أَحَسْبَكُنَّ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أحسبها'] = {
    'word': u'أحسبها',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'أَحَسْبَهَا',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أحسبهم'] = {
    'word': u'أحسبهم',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'أَحَسْبَهُمْ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أحسبهن'] = {
    'word': u'أحسبهن',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'أَحَسْبَهُنَّ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أحسبنا'] = {
    'word': u'أحسبنا',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'أَحَسْبَنَا',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أحسبكما'] = {
    'word': u'أحسبكما',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'أَحَسْبَكُمَا',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أحسبهما'] = {
    'word': u'أحسبهما',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'أَحَسْبَهُمَا',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أبحسبي'] = {
    'word': u'أبحسبي',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِحَسْبَِي',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أبحسبك'] = {
    'word': u'أبحسبك',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِحَسْبَكَ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أبحسبك'] = {
    'word': u'أبحسبك',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِحَسْبَكِ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أبحسبه'] = {
    'word': u'أبحسبه',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِحَسْبَه',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'ه'
}
STOPWORDS[u'أبحسبكم'] = {
    'word': u'أبحسبكم',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِحَسْبَكُمْ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أبحسبكن'] = {
    'word': u'أبحسبكن',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِحَسْبَكُنَّ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أبحسبها'] = {
    'word': u'أبحسبها',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِحَسْبَهَا',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أبحسبهم'] = {
    'word': u'أبحسبهم',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِحَسْبَهُمْ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أبحسبهن'] = {
    'word': u'أبحسبهن',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِحَسْبَهُنَّ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أبحسبنا'] = {
    'word': u'أبحسبنا',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِحَسْبَنَا',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أبحسبكما'] = {
    'word': u'أبحسبكما',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِحَسْبَكُمَا',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أبحسبهما'] = {
    'word': u'أبحسبهما',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِحَسْبَهُمَا',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أكحسبي'] = {
    'word': u'أكحسبي',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَحَسْبَِي',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أكحسبك'] = {
    'word': u'أكحسبك',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَحَسْبَكَ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أكحسبك'] = {
    'word': u'أكحسبك',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَحَسْبَكِ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أكحسبه'] = {
    'word': u'أكحسبه',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَحَسْبَه',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'ه'
}
STOPWORDS[u'أكحسبكم'] = {
    'word': u'أكحسبكم',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَحَسْبَكُمْ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أكحسبكن'] = {
    'word': u'أكحسبكن',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَحَسْبَكُنَّ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أكحسبها'] = {
    'word': u'أكحسبها',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَحَسْبَهَا',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أكحسبهم'] = {
    'word': u'أكحسبهم',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَحَسْبَهُمْ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أكحسبهن'] = {
    'word': u'أكحسبهن',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَحَسْبَهُنَّ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أكحسبنا'] = {
    'word': u'أكحسبنا',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَحَسْبَنَا',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أكحسبكما'] = {
    'word': u'أكحسبكما',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَحَسْبَكُمَا',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أكحسبهما'] = {
    'word': u'أكحسبهما',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَحَسْبَهُمَا',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ألحسبي'] = {
    'word': u'ألحسبي',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِحَسْبَِي',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'ِي'
}
STOPWORDS[u'ألحسبك'] = {
    'word': u'ألحسبك',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِحَسْبَكَ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كَ'
}
STOPWORDS[u'ألحسبك'] = {
    'word': u'ألحسبك',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِحَسْبَكِ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كِ'
}
STOPWORDS[u'ألحسبه'] = {
    'word': u'ألحسبه',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِحَسْبَه',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'ه'
}
STOPWORDS[u'ألحسبكم'] = {
    'word': u'ألحسبكم',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِحَسْبَكُمْ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ألحسبكن'] = {
    'word': u'ألحسبكن',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِحَسْبَكُنَّ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ألحسبها'] = {
    'word': u'ألحسبها',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِحَسْبَهَا',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'هَا'
}
STOPWORDS[u'ألحسبهم'] = {
    'word': u'ألحسبهم',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِحَسْبَهُمْ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ألحسبهن'] = {
    'word': u'ألحسبهن',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِحَسْبَهُنَّ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ألحسبنا'] = {
    'word': u'ألحسبنا',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِحَسْبَنَا',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'نَا'
}
STOPWORDS[u'ألحسبكما'] = {
    'word': u'ألحسبكما',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِحَسْبَكُمَا',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ألحسبهما'] = {
    'word': u'ألحسبهما',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِحَسْبَهُمَا',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أوحسبي'] = {
    'word': u'أوحسبي',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَحَسْبَِي',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أوحسبك'] = {
    'word': u'أوحسبك',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَحَسْبَكَ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أوحسبك'] = {
    'word': u'أوحسبك',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَحَسْبَكِ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أوحسبه'] = {
    'word': u'أوحسبه',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَحَسْبَه',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'ه'
}
STOPWORDS[u'أوحسبكم'] = {
    'word': u'أوحسبكم',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَحَسْبَكُمْ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أوحسبكن'] = {
    'word': u'أوحسبكن',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَحَسْبَكُنَّ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أوحسبها'] = {
    'word': u'أوحسبها',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَحَسْبَهَا',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أوحسبهم'] = {
    'word': u'أوحسبهم',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَحَسْبَهُمْ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أوحسبهن'] = {
    'word': u'أوحسبهن',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَحَسْبَهُنَّ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أوحسبنا'] = {
    'word': u'أوحسبنا',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَحَسْبَنَا',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أوحسبكما'] = {
    'word': u'أوحسبكما',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَحَسْبَكُمَا',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أوحسبهما'] = {
    'word': u'أوحسبهما',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَحَسْبَهُمَا',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أفحسبي'] = {
    'word': u'أفحسبي',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَحَسْبَِي',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أفحسبك'] = {
    'word': u'أفحسبك',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَحَسْبَكَ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أفحسبك'] = {
    'word': u'أفحسبك',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَحَسْبَكِ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أفحسبه'] = {
    'word': u'أفحسبه',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَحَسْبَه',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'ه'
}
STOPWORDS[u'أفحسبكم'] = {
    'word': u'أفحسبكم',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَحَسْبَكُمْ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أفحسبكن'] = {
    'word': u'أفحسبكن',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَحَسْبَكُنَّ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أفحسبها'] = {
    'word': u'أفحسبها',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَحَسْبَهَا',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أفحسبهم'] = {
    'word': u'أفحسبهم',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَحَسْبَهُمْ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أفحسبهن'] = {
    'word': u'أفحسبهن',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَحَسْبَهُنَّ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أفحسبنا'] = {
    'word': u'أفحسبنا',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَحَسْبَنَا',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أفحسبكما'] = {
    'word': u'أفحسبكما',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَحَسْبَكُمَا',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أفحسبهما'] = {
    'word': u'أفحسبهما',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَحَسْبَهُمَا',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أوبحسبي'] = {
    'word': u'أوبحسبي',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِحَسْبَِي',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أوبحسبك'] = {
    'word': u'أوبحسبك',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِحَسْبَكَ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أوبحسبك'] = {
    'word': u'أوبحسبك',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِحَسْبَكِ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أوبحسبه'] = {
    'word': u'أوبحسبه',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِحَسْبَه',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'ه'
}
STOPWORDS[u'أوبحسبكم'] = {
    'word': u'أوبحسبكم',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِحَسْبَكُمْ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أوبحسبكن'] = {
    'word': u'أوبحسبكن',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِحَسْبَكُنَّ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أوبحسبها'] = {
    'word': u'أوبحسبها',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِحَسْبَهَا',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أوبحسبهم'] = {
    'word': u'أوبحسبهم',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِحَسْبَهُمْ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أوبحسبهن'] = {
    'word': u'أوبحسبهن',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِحَسْبَهُنَّ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أوبحسبنا'] = {
    'word': u'أوبحسبنا',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِحَسْبَنَا',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أوبحسبكما'] = {
    'word': u'أوبحسبكما',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِحَسْبَكُمَا',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أوبحسبهما'] = {
    'word': u'أوبحسبهما',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِحَسْبَهُمَا',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أفبحسبي'] = {
    'word': u'أفبحسبي',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِحَسْبَِي',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أفبحسبك'] = {
    'word': u'أفبحسبك',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِحَسْبَكَ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أفبحسبك'] = {
    'word': u'أفبحسبك',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِحَسْبَكِ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أفبحسبه'] = {
    'word': u'أفبحسبه',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِحَسْبَه',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'ه'
}
STOPWORDS[u'أفبحسبكم'] = {
    'word': u'أفبحسبكم',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِحَسْبَكُمْ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أفبحسبكن'] = {
    'word': u'أفبحسبكن',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِحَسْبَكُنَّ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أفبحسبها'] = {
    'word': u'أفبحسبها',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِحَسْبَهَا',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أفبحسبهم'] = {
    'word': u'أفبحسبهم',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِحَسْبَهُمْ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أفبحسبهن'] = {
    'word': u'أفبحسبهن',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِحَسْبَهُنَّ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أفبحسبنا'] = {
    'word': u'أفبحسبنا',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِحَسْبَنَا',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أفبحسبكما'] = {
    'word': u'أفبحسبكما',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِحَسْبَكُمَا',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أفبحسبهما'] = {
    'word': u'أفبحسبهما',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِحَسْبَهُمَا',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أوكحسبي'] = {
    'word': u'أوكحسبي',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَحَسْبَِي',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أوكحسبك'] = {
    'word': u'أوكحسبك',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَحَسْبَكَ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أوكحسبك'] = {
    'word': u'أوكحسبك',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَحَسْبَكِ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أوكحسبه'] = {
    'word': u'أوكحسبه',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَحَسْبَه',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'ه'
}
STOPWORDS[u'أوكحسبكم'] = {
    'word': u'أوكحسبكم',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَحَسْبَكُمْ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أوكحسبكن'] = {
    'word': u'أوكحسبكن',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَحَسْبَكُنَّ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أوكحسبها'] = {
    'word': u'أوكحسبها',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَحَسْبَهَا',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أوكحسبهم'] = {
    'word': u'أوكحسبهم',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَحَسْبَهُمْ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أوكحسبهن'] = {
    'word': u'أوكحسبهن',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَحَسْبَهُنَّ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أوكحسبنا'] = {
    'word': u'أوكحسبنا',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَحَسْبَنَا',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أوكحسبكما'] = {
    'word': u'أوكحسبكما',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَحَسْبَكُمَا',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أوكحسبهما'] = {
    'word': u'أوكحسبهما',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَحَسْبَهُمَا',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أفكحسبي'] = {
    'word': u'أفكحسبي',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَحَسْبَِي',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أفكحسبك'] = {
    'word': u'أفكحسبك',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَحَسْبَكَ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أفكحسبك'] = {
    'word': u'أفكحسبك',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَحَسْبَكِ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أفكحسبه'] = {
    'word': u'أفكحسبه',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَحَسْبَه',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'ه'
}
STOPWORDS[u'أفكحسبكم'] = {
    'word': u'أفكحسبكم',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَحَسْبَكُمْ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أفكحسبكن'] = {
    'word': u'أفكحسبكن',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَحَسْبَكُنَّ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أفكحسبها'] = {
    'word': u'أفكحسبها',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَحَسْبَهَا',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أفكحسبهم'] = {
    'word': u'أفكحسبهم',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَحَسْبَهُمْ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أفكحسبهن'] = {
    'word': u'أفكحسبهن',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَحَسْبَهُنَّ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أفكحسبنا'] = {
    'word': u'أفكحسبنا',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَحَسْبَنَا',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أفكحسبكما'] = {
    'word': u'أفكحسبكما',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَحَسْبَكُمَا',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أفكحسبهما'] = {
    'word': u'أفكحسبهما',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَحَسْبَهُمَا',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أولحسبي'] = {
    'word': u'أولحسبي',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِحَسْبَِي',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أولحسبك'] = {
    'word': u'أولحسبك',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِحَسْبَكَ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أولحسبك'] = {
    'word': u'أولحسبك',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِحَسْبَكِ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أولحسبه'] = {
    'word': u'أولحسبه',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِحَسْبَه',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'ه'
}
STOPWORDS[u'أولحسبكم'] = {
    'word': u'أولحسبكم',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِحَسْبَكُمْ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أولحسبكن'] = {
    'word': u'أولحسبكن',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِحَسْبَكُنَّ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أولحسبها'] = {
    'word': u'أولحسبها',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِحَسْبَهَا',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أولحسبهم'] = {
    'word': u'أولحسبهم',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِحَسْبَهُمْ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أولحسبهن'] = {
    'word': u'أولحسبهن',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِحَسْبَهُنَّ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أولحسبنا'] = {
    'word': u'أولحسبنا',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِحَسْبَنَا',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أولحسبكما'] = {
    'word': u'أولحسبكما',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِحَسْبَكُمَا',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أولحسبهما'] = {
    'word': u'أولحسبهما',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِحَسْبَهُمَا',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أفلحسبي'] = {
    'word': u'أفلحسبي',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِحَسْبَِي',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أفلحسبك'] = {
    'word': u'أفلحسبك',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِحَسْبَكَ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أفلحسبك'] = {
    'word': u'أفلحسبك',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِحَسْبَكِ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أفلحسبه'] = {
    'word': u'أفلحسبه',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِحَسْبَه',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'ه'
}
STOPWORDS[u'أفلحسبكم'] = {
    'word': u'أفلحسبكم',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِحَسْبَكُمْ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أفلحسبكن'] = {
    'word': u'أفلحسبكن',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِحَسْبَكُنَّ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أفلحسبها'] = {
    'word': u'أفلحسبها',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِحَسْبَهَا',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أفلحسبهم'] = {
    'word': u'أفلحسبهم',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِحَسْبَهُمْ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أفلحسبهن'] = {
    'word': u'أفلحسبهن',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِحَسْبَهُنَّ',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أفلحسبنا'] = {
    'word': u'أفلحسبنا',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِحَسْبَنَا',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أفلحسبكما'] = {
    'word': u'أفلحسبكما',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِحَسْبَكُمَا',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أفلحسبهما'] = {
    'word': u'أفلحسبهما',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِحَسْبَهُمَا',
    'stem': u'حَسْبَ',
    'type': u'STOPWORD',
    'original': u'حَسْبَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'حيث'] = {
    'word': u'حَيْثُ',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة',
    'vocalized': u'حَيْثُ',
    'stem': u'حَيْثُ',
    'type': u'STOPWORD',
    'original': u'حَيْثُ',
    'encletic': u''
}
STOPWORDS[u'بحيث'] = {
    'word': u'بحيث',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور',
    'vocalized': u'بِحَيْثُ',
    'stem': u'حَيْثُ',
    'type': u'STOPWORD',
    'original': u'حَيْثُ',
    'encletic': u''
}
STOPWORDS[u'كحيث'] = {
    'word': u'كحيث',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور',
    'vocalized': u'كَحَيْثُ',
    'stem': u'حَيْثُ',
    'type': u'STOPWORD',
    'original': u'حَيْثُ',
    'encletic': u''
}
STOPWORDS[u'لحيث'] = {
    'word': u'لحيث',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور',
    'vocalized': u'لِحَيْثُ',
    'stem': u'حَيْثُ',
    'type': u'STOPWORD',
    'original': u'حَيْثُ',
    'encletic': u''
}
STOPWORDS[u'وحيث'] = {
    'word': u'وحيث',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف',
    'vocalized': u'وَحَيْثُ',
    'stem': u'حَيْثُ',
    'type': u'STOPWORD',
    'original': u'حَيْثُ',
    'encletic': u''
}
STOPWORDS[u'فحيث'] = {
    'word': u'فحيث',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف',
    'vocalized': u'فَحَيْثُ',
    'stem': u'حَيْثُ',
    'type': u'STOPWORD',
    'original': u'حَيْثُ',
    'encletic': u''
}
STOPWORDS[u'وبحيث'] = {
    'word': u'وبحيث',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف',
    'vocalized': u'وَبِحَيْثُ',
    'stem': u'حَيْثُ',
    'type': u'STOPWORD',
    'original': u'حَيْثُ',
    'encletic': u''
}
STOPWORDS[u'فبحيث'] = {
    'word': u'فبحيث',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف',
    'vocalized': u'فَبِحَيْثُ',
    'stem': u'حَيْثُ',
    'type': u'STOPWORD',
    'original': u'حَيْثُ',
    'encletic': u''
}
STOPWORDS[u'وكحيث'] = {
    'word': u'وكحيث',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف',
    'vocalized': u'وَكَحَيْثُ',
    'stem': u'حَيْثُ',
    'type': u'STOPWORD',
    'original': u'حَيْثُ',
    'encletic': u''
}
STOPWORDS[u'فكحيث'] = {
    'word': u'فكحيث',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف',
    'vocalized': u'فَكَحَيْثُ',
    'stem': u'حَيْثُ',
    'type': u'STOPWORD',
    'original': u'حَيْثُ',
    'encletic': u''
}
STOPWORDS[u'ولحيث'] = {
    'word': u'ولحيث',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف',
    'vocalized': u'وَلِحَيْثُ',
    'stem': u'حَيْثُ',
    'type': u'STOPWORD',
    'original': u'حَيْثُ',
    'encletic': u''
}
STOPWORDS[u'فلحيث'] = {
    'word': u'فلحيث',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف',
    'vocalized': u'فَلِحَيْثُ',
    'stem': u'حَيْثُ',
    'type': u'STOPWORD',
    'original': u'حَيْثُ',
    'encletic': u''
}
STOPWORDS[u'سبحان'] = {
    'word': u'سُبْحَانَ',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة',
    'vocalized': u'سُبْحَانَ',
    'stem': u'سُبْحَانَ',
    'type': u'STOPWORD',
    'original': u'سُبْحَانَ',
    'encletic': u''
}
STOPWORDS[u'وسبحان'] = {
    'word': u'وسبحان',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف',
    'vocalized': u'وَسُبْحَانَ',
    'stem': u'سُبْحَانَ',
    'type': u'STOPWORD',
    'original': u'سُبْحَانَ',
    'encletic': u''
}
STOPWORDS[u'فسبحان'] = {
    'word': u'فسبحان',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف',
    'vocalized': u'فَسُبْحَانَ',
    'stem': u'سُبْحَانَ',
    'type': u'STOPWORD',
    'original': u'سُبْحَانَ',
    'encletic': u''
}
STOPWORDS[u'سبحاني'] = {
    'word': u'سبحاني',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'سُبْحَانَِي',
    'stem': u'سُبْحَانَ',
    'type': u'STOPWORD',
    'original': u'سُبْحَانَ',
    'encletic': u'ِي'
}
STOPWORDS[u'سبحانك'] = {
    'word': u'سبحانك',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'سُبْحَانَكَ',
    'stem': u'سُبْحَانَ',
    'type': u'STOPWORD',
    'original': u'سُبْحَانَ',
    'encletic': u'كَ'
}
STOPWORDS[u'سبحانك'] = {
    'word': u'سبحانك',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'سُبْحَانَكِ',
    'stem': u'سُبْحَانَ',
    'type': u'STOPWORD',
    'original': u'سُبْحَانَ',
    'encletic': u'كِ'
}
STOPWORDS[u'سبحانه'] = {
    'word': u'سبحانه',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'سُبْحَانَه',
    'stem': u'سُبْحَانَ',
    'type': u'STOPWORD',
    'original': u'سُبْحَانَ',
    'encletic': u'ه'
}
STOPWORDS[u'سبحانكم'] = {
    'word': u'سبحانكم',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'سُبْحَانَكُمْ',
    'stem': u'سُبْحَانَ',
    'type': u'STOPWORD',
    'original': u'سُبْحَانَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'سبحانكن'] = {
    'word': u'سبحانكن',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'سُبْحَانَكُنَّ',
    'stem': u'سُبْحَانَ',
    'type': u'STOPWORD',
    'original': u'سُبْحَانَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'سبحانها'] = {
    'word': u'سبحانها',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'سُبْحَانَهَا',
    'stem': u'سُبْحَانَ',
    'type': u'STOPWORD',
    'original': u'سُبْحَانَ',
    'encletic': u'هَا'
}
STOPWORDS[u'سبحانهم'] = {
    'word': u'سبحانهم',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'سُبْحَانَهُمْ',
    'stem': u'سُبْحَانَ',
    'type': u'STOPWORD',
    'original': u'سُبْحَانَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'سبحانهن'] = {
    'word': u'سبحانهن',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'سُبْحَانَهُنَّ',
    'stem': u'سُبْحَانَ',
    'type': u'STOPWORD',
    'original': u'سُبْحَانَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'سبحاننا'] = {
    'word': u'سبحاننا',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'سُبْحَانَنَا',
    'stem': u'سُبْحَانَ',
    'type': u'STOPWORD',
    'original': u'سُبْحَانَ',
    'encletic': u'نَا'
}
STOPWORDS[u'سبحانكما'] = {
    'word': u'سبحانكما',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'سُبْحَانَكُمَا',
    'stem': u'سُبْحَانَ',
    'type': u'STOPWORD',
    'original': u'سُبْحَانَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'سبحانهما'] = {
    'word': u'سبحانهما',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'سُبْحَانَهُمَا',
    'stem': u'سُبْحَانَ',
    'type': u'STOPWORD',
    'original': u'سُبْحَانَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'وسبحاني'] = {
    'word': u'وسبحاني',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَسُبْحَانَِي',
    'stem': u'سُبْحَانَ',
    'type': u'STOPWORD',
    'original': u'سُبْحَانَ',
    'encletic': u'ِي'
}
STOPWORDS[u'وسبحانك'] = {
    'word': u'وسبحانك',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَسُبْحَانَكَ',
    'stem': u'سُبْحَانَ',
    'type': u'STOPWORD',
    'original': u'سُبْحَانَ',
    'encletic': u'كَ'
}
STOPWORDS[u'وسبحانك'] = {
    'word': u'وسبحانك',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَسُبْحَانَكِ',
    'stem': u'سُبْحَانَ',
    'type': u'STOPWORD',
    'original': u'سُبْحَانَ',
    'encletic': u'كِ'
}
STOPWORDS[u'وسبحانه'] = {
    'word': u'وسبحانه',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَسُبْحَانَه',
    'stem': u'سُبْحَانَ',
    'type': u'STOPWORD',
    'original': u'سُبْحَانَ',
    'encletic': u'ه'
}
STOPWORDS[u'وسبحانكم'] = {
    'word': u'وسبحانكم',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَسُبْحَانَكُمْ',
    'stem': u'سُبْحَانَ',
    'type': u'STOPWORD',
    'original': u'سُبْحَانَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'وسبحانكن'] = {
    'word': u'وسبحانكن',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَسُبْحَانَكُنَّ',
    'stem': u'سُبْحَانَ',
    'type': u'STOPWORD',
    'original': u'سُبْحَانَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'وسبحانها'] = {
    'word': u'وسبحانها',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَسُبْحَانَهَا',
    'stem': u'سُبْحَانَ',
    'type': u'STOPWORD',
    'original': u'سُبْحَانَ',
    'encletic': u'هَا'
}
STOPWORDS[u'وسبحانهم'] = {
    'word': u'وسبحانهم',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَسُبْحَانَهُمْ',
    'stem': u'سُبْحَانَ',
    'type': u'STOPWORD',
    'original': u'سُبْحَانَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'وسبحانهن'] = {
    'word': u'وسبحانهن',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَسُبْحَانَهُنَّ',
    'stem': u'سُبْحَانَ',
    'type': u'STOPWORD',
    'original': u'سُبْحَانَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'وسبحاننا'] = {
    'word': u'وسبحاننا',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَسُبْحَانَنَا',
    'stem': u'سُبْحَانَ',
    'type': u'STOPWORD',
    'original': u'سُبْحَانَ',
    'encletic': u'نَا'
}
STOPWORDS[u'وسبحانكما'] = {
    'word': u'وسبحانكما',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَسُبْحَانَكُمَا',
    'stem': u'سُبْحَانَ',
    'type': u'STOPWORD',
    'original': u'سُبْحَانَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'وسبحانهما'] = {
    'word': u'وسبحانهما',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَسُبْحَانَهُمَا',
    'stem': u'سُبْحَانَ',
    'type': u'STOPWORD',
    'original': u'سُبْحَانَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فسبحاني'] = {
    'word': u'فسبحاني',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَسُبْحَانَِي',
    'stem': u'سُبْحَانَ',
    'type': u'STOPWORD',
    'original': u'سُبْحَانَ',
    'encletic': u'ِي'
}
STOPWORDS[u'فسبحانك'] = {
    'word': u'فسبحانك',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَسُبْحَانَكَ',
    'stem': u'سُبْحَانَ',
    'type': u'STOPWORD',
    'original': u'سُبْحَانَ',
    'encletic': u'كَ'
}
STOPWORDS[u'فسبحانك'] = {
    'word': u'فسبحانك',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَسُبْحَانَكِ',
    'stem': u'سُبْحَانَ',
    'type': u'STOPWORD',
    'original': u'سُبْحَانَ',
    'encletic': u'كِ'
}
STOPWORDS[u'فسبحانه'] = {
    'word': u'فسبحانه',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَسُبْحَانَه',
    'stem': u'سُبْحَانَ',
    'type': u'STOPWORD',
    'original': u'سُبْحَانَ',
    'encletic': u'ه'
}
STOPWORDS[u'فسبحانكم'] = {
    'word': u'فسبحانكم',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَسُبْحَانَكُمْ',
    'stem': u'سُبْحَانَ',
    'type': u'STOPWORD',
    'original': u'سُبْحَانَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فسبحانكن'] = {
    'word': u'فسبحانكن',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَسُبْحَانَكُنَّ',
    'stem': u'سُبْحَانَ',
    'type': u'STOPWORD',
    'original': u'سُبْحَانَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فسبحانها'] = {
    'word': u'فسبحانها',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَسُبْحَانَهَا',
    'stem': u'سُبْحَانَ',
    'type': u'STOPWORD',
    'original': u'سُبْحَانَ',
    'encletic': u'هَا'
}
STOPWORDS[u'فسبحانهم'] = {
    'word': u'فسبحانهم',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَسُبْحَانَهُمْ',
    'stem': u'سُبْحَانَ',
    'type': u'STOPWORD',
    'original': u'سُبْحَانَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فسبحانهن'] = {
    'word': u'فسبحانهن',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَسُبْحَانَهُنَّ',
    'stem': u'سُبْحَانَ',
    'type': u'STOPWORD',
    'original': u'سُبْحَانَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فسبحاننا'] = {
    'word': u'فسبحاننا',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَسُبْحَانَنَا',
    'stem': u'سُبْحَانَ',
    'type': u'STOPWORD',
    'original': u'سُبْحَانَ',
    'encletic': u'نَا'
}
STOPWORDS[u'فسبحانكما'] = {
    'word': u'فسبحانكما',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَسُبْحَانَكُمَا',
    'stem': u'سُبْحَانَ',
    'type': u'STOPWORD',
    'original': u'سُبْحَانَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فسبحانهما'] = {
    'word': u'فسبحانهما',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَسُبْحَانَهُمَا',
    'stem': u'سُبْحَانَ',
    'type': u'STOPWORD',
    'original': u'سُبْحَانَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'سوى'] = {
    'word': u'سِوَى',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة',
    'vocalized': u'سِوَى',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u''
}
STOPWORDS[u'بسوى'] = {
    'word': u'بسوى',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور',
    'vocalized': u'بِسِوَى',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u''
}
STOPWORDS[u'كسوى'] = {
    'word': u'كسوى',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور',
    'vocalized': u'كَسِوَى',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u''
}
STOPWORDS[u'لسوى'] = {
    'word': u'لسوى',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور',
    'vocalized': u'لِسِوَى',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u''
}
STOPWORDS[u'وسوى'] = {
    'word': u'وسوى',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف',
    'vocalized': u'وَسِوَى',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u''
}
STOPWORDS[u'فسوى'] = {
    'word': u'فسوى',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف',
    'vocalized': u'فَسِوَى',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u''
}
STOPWORDS[u'وبسوى'] = {
    'word': u'وبسوى',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف',
    'vocalized': u'وَبِسِوَى',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u''
}
STOPWORDS[u'فبسوى'] = {
    'word': u'فبسوى',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف',
    'vocalized': u'فَبِسِوَى',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u''
}
STOPWORDS[u'وكسوى'] = {
    'word': u'وكسوى',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف',
    'vocalized': u'وَكَسِوَى',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u''
}
STOPWORDS[u'فكسوى'] = {
    'word': u'فكسوى',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف',
    'vocalized': u'فَكَسِوَى',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u''
}
STOPWORDS[u'ولسوى'] = {
    'word': u'ولسوى',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف',
    'vocalized': u'وَلِسِوَى',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u''
}
STOPWORDS[u'فلسوى'] = {
    'word': u'فلسوى',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف',
    'vocalized': u'فَلِسِوَى',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u''
}
STOPWORDS[u'أسوى'] = {
    'word': u'أسوى',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور',
    'vocalized': u'أَسِوَى',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u''
}
STOPWORDS[u'أبسوى'] = {
    'word': u'أبسوى',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور',
    'vocalized': u'أَبِسِوَى',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u''
}
STOPWORDS[u'أكسوى'] = {
    'word': u'أكسوى',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور',
    'vocalized': u'أَكَسِوَى',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u''
}
STOPWORDS[u'ألسوى'] = {
    'word': u'ألسوى',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور',
    'vocalized': u'أَلِسِوَى',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u''
}
STOPWORDS[u'أوسوى'] = {
    'word': u'أوسوى',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور',
    'vocalized': u'أَوَسِوَى',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u''
}
STOPWORDS[u'أفسوى'] = {
    'word': u'أفسوى',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور',
    'vocalized': u'أَفَسِوَى',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u''
}
STOPWORDS[u'أوبسوى'] = {
    'word': u'أوبسوى',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَبِسِوَى',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u''
}
STOPWORDS[u'أفبسوى'] = {
    'word': u'أفبسوى',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَبِسِوَى',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u''
}
STOPWORDS[u'أوكسوى'] = {
    'word': u'أوكسوى',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَكَسِوَى',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u''
}
STOPWORDS[u'أفكسوى'] = {
    'word': u'أفكسوى',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَكَسِوَى',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u''
}
STOPWORDS[u'أولسوى'] = {
    'word': u'أولسوى',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَلِسِوَى',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u''
}
STOPWORDS[u'أفلسوى'] = {
    'word': u'أفلسوى',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَلِسِوَى',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u''
}
STOPWORDS[u'سوي'] = {
    'word': u'سوي',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'سِوَيِي',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'ِي'
}
STOPWORDS[u'سويك'] = {
    'word': u'سويك',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'سِوَيكَ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كَ'
}
STOPWORDS[u'سويك'] = {
    'word': u'سويك',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'سِوَيكِ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كِ'
}
STOPWORDS[u'سويه'] = {
    'word': u'سويه',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'سِوَيه',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'ه'
}
STOPWORDS[u'سويكم'] = {
    'word': u'سويكم',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'سِوَيكُمْ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كُمْ'
}
STOPWORDS[u'سويكن'] = {
    'word': u'سويكن',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'سِوَيكُنَّ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'سويها'] = {
    'word': u'سويها',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'سِوَيهَا',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'هَا'
}
STOPWORDS[u'سويهم'] = {
    'word': u'سويهم',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'سِوَيهُمْ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'هُمْ'
}
STOPWORDS[u'سويهن'] = {
    'word': u'سويهن',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'سِوَيهُنَّ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'سوينا'] = {
    'word': u'سوينا',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'سِوَينَا',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'نَا'
}
STOPWORDS[u'سويكما'] = {
    'word': u'سويكما',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'سِوَيكُمَا',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كُمَا'
}
STOPWORDS[u'سويهما'] = {
    'word': u'سويهما',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'سِوَيهُمَا',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'هُمَا'
}
STOPWORDS[u'بسوي'] = {
    'word': u'بسوي',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِسِوَيِي',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'ِي'
}
STOPWORDS[u'بسويك'] = {
    'word': u'بسويك',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِسِوَيكَ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كَ'
}
STOPWORDS[u'بسويك'] = {
    'word': u'بسويك',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِسِوَيكِ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كِ'
}
STOPWORDS[u'بسويه'] = {
    'word': u'بسويه',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِسِوَيه',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'ه'
}
STOPWORDS[u'بسويكم'] = {
    'word': u'بسويكم',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِسِوَيكُمْ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كُمْ'
}
STOPWORDS[u'بسويكن'] = {
    'word': u'بسويكن',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِسِوَيكُنَّ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'بسويها'] = {
    'word': u'بسويها',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِسِوَيهَا',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'هَا'
}
STOPWORDS[u'بسويهم'] = {
    'word': u'بسويهم',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِسِوَيهُمْ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'هُمْ'
}
STOPWORDS[u'بسويهن'] = {
    'word': u'بسويهن',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِسِوَيهُنَّ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'بسوينا'] = {
    'word': u'بسوينا',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِسِوَينَا',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'نَا'
}
STOPWORDS[u'بسويكما'] = {
    'word': u'بسويكما',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِسِوَيكُمَا',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كُمَا'
}
STOPWORDS[u'بسويهما'] = {
    'word': u'بسويهما',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِسِوَيهُمَا',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'هُمَا'
}
STOPWORDS[u'كسوي'] = {
    'word': u'كسوي',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَسِوَيِي',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'ِي'
}
STOPWORDS[u'كسويك'] = {
    'word': u'كسويك',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَسِوَيكَ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كَ'
}
STOPWORDS[u'كسويك'] = {
    'word': u'كسويك',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَسِوَيكِ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كِ'
}
STOPWORDS[u'كسويه'] = {
    'word': u'كسويه',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَسِوَيه',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'ه'
}
STOPWORDS[u'كسويكم'] = {
    'word': u'كسويكم',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَسِوَيكُمْ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كُمْ'
}
STOPWORDS[u'كسويكن'] = {
    'word': u'كسويكن',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَسِوَيكُنَّ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'كسويها'] = {
    'word': u'كسويها',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَسِوَيهَا',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'هَا'
}
STOPWORDS[u'كسويهم'] = {
    'word': u'كسويهم',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَسِوَيهُمْ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'هُمْ'
}
STOPWORDS[u'كسويهن'] = {
    'word': u'كسويهن',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَسِوَيهُنَّ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'كسوينا'] = {
    'word': u'كسوينا',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَسِوَينَا',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'نَا'
}
STOPWORDS[u'كسويكما'] = {
    'word': u'كسويكما',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَسِوَيكُمَا',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كُمَا'
}
STOPWORDS[u'كسويهما'] = {
    'word': u'كسويهما',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَسِوَيهُمَا',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'هُمَا'
}
STOPWORDS[u'لسوي'] = {
    'word': u'لسوي',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِسِوَيِي',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'ِي'
}
STOPWORDS[u'لسويك'] = {
    'word': u'لسويك',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِسِوَيكَ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كَ'
}
STOPWORDS[u'لسويك'] = {
    'word': u'لسويك',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِسِوَيكِ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كِ'
}
STOPWORDS[u'لسويه'] = {
    'word': u'لسويه',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِسِوَيه',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'ه'
}
STOPWORDS[u'لسويكم'] = {
    'word': u'لسويكم',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِسِوَيكُمْ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كُمْ'
}
STOPWORDS[u'لسويكن'] = {
    'word': u'لسويكن',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِسِوَيكُنَّ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'لسويها'] = {
    'word': u'لسويها',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِسِوَيهَا',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'هَا'
}
STOPWORDS[u'لسويهم'] = {
    'word': u'لسويهم',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِسِوَيهُمْ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'هُمْ'
}
STOPWORDS[u'لسويهن'] = {
    'word': u'لسويهن',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِسِوَيهُنَّ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'لسوينا'] = {
    'word': u'لسوينا',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِسِوَينَا',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'نَا'
}
STOPWORDS[u'لسويكما'] = {
    'word': u'لسويكما',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِسِوَيكُمَا',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كُمَا'
}
STOPWORDS[u'لسويهما'] = {
    'word': u'لسويهما',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِسِوَيهُمَا',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'هُمَا'
}
STOPWORDS[u'وسوي'] = {
    'word': u'وسوي',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَسِوَيِي',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'ِي'
}
STOPWORDS[u'وسويك'] = {
    'word': u'وسويك',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَسِوَيكَ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كَ'
}
STOPWORDS[u'وسويك'] = {
    'word': u'وسويك',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَسِوَيكِ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كِ'
}
STOPWORDS[u'وسويه'] = {
    'word': u'وسويه',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَسِوَيه',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'ه'
}
STOPWORDS[u'وسويكم'] = {
    'word': u'وسويكم',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَسِوَيكُمْ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كُمْ'
}
STOPWORDS[u'وسويكن'] = {
    'word': u'وسويكن',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَسِوَيكُنَّ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'وسويها'] = {
    'word': u'وسويها',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَسِوَيهَا',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'هَا'
}
STOPWORDS[u'وسويهم'] = {
    'word': u'وسويهم',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَسِوَيهُمْ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'هُمْ'
}
STOPWORDS[u'وسويهن'] = {
    'word': u'وسويهن',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَسِوَيهُنَّ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'وسوينا'] = {
    'word': u'وسوينا',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَسِوَينَا',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'نَا'
}
STOPWORDS[u'وسويكما'] = {
    'word': u'وسويكما',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَسِوَيكُمَا',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كُمَا'
}
STOPWORDS[u'وسويهما'] = {
    'word': u'وسويهما',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَسِوَيهُمَا',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فسوي'] = {
    'word': u'فسوي',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَسِوَيِي',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'ِي'
}
STOPWORDS[u'فسويك'] = {
    'word': u'فسويك',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَسِوَيكَ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كَ'
}
STOPWORDS[u'فسويك'] = {
    'word': u'فسويك',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَسِوَيكِ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كِ'
}
STOPWORDS[u'فسويه'] = {
    'word': u'فسويه',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَسِوَيه',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'ه'
}
STOPWORDS[u'فسويكم'] = {
    'word': u'فسويكم',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَسِوَيكُمْ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فسويكن'] = {
    'word': u'فسويكن',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَسِوَيكُنَّ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فسويها'] = {
    'word': u'فسويها',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَسِوَيهَا',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'هَا'
}
STOPWORDS[u'فسويهم'] = {
    'word': u'فسويهم',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَسِوَيهُمْ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فسويهن'] = {
    'word': u'فسويهن',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَسِوَيهُنَّ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فسوينا'] = {
    'word': u'فسوينا',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَسِوَينَا',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'نَا'
}
STOPWORDS[u'فسويكما'] = {
    'word': u'فسويكما',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَسِوَيكُمَا',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فسويهما'] = {
    'word': u'فسويهما',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَسِوَيهُمَا',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'هُمَا'
}
STOPWORDS[u'وبسوي'] = {
    'word': u'وبسوي',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِسِوَيِي',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'ِي'
}
STOPWORDS[u'وبسويك'] = {
    'word': u'وبسويك',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِسِوَيكَ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كَ'
}
STOPWORDS[u'وبسويك'] = {
    'word': u'وبسويك',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِسِوَيكِ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كِ'
}
STOPWORDS[u'وبسويه'] = {
    'word': u'وبسويه',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِسِوَيه',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'ه'
}
STOPWORDS[u'وبسويكم'] = {
    'word': u'وبسويكم',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِسِوَيكُمْ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كُمْ'
}
STOPWORDS[u'وبسويكن'] = {
    'word': u'وبسويكن',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِسِوَيكُنَّ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'وبسويها'] = {
    'word': u'وبسويها',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِسِوَيهَا',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'هَا'
}
STOPWORDS[u'وبسويهم'] = {
    'word': u'وبسويهم',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِسِوَيهُمْ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'هُمْ'
}
STOPWORDS[u'وبسويهن'] = {
    'word': u'وبسويهن',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِسِوَيهُنَّ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'وبسوينا'] = {
    'word': u'وبسوينا',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِسِوَينَا',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'نَا'
}
STOPWORDS[u'وبسويكما'] = {
    'word': u'وبسويكما',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِسِوَيكُمَا',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كُمَا'
}
STOPWORDS[u'وبسويهما'] = {
    'word': u'وبسويهما',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِسِوَيهُمَا',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فبسوي'] = {
    'word': u'فبسوي',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِسِوَيِي',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'ِي'
}
STOPWORDS[u'فبسويك'] = {
    'word': u'فبسويك',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِسِوَيكَ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كَ'
}
STOPWORDS[u'فبسويك'] = {
    'word': u'فبسويك',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِسِوَيكِ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كِ'
}
STOPWORDS[u'فبسويه'] = {
    'word': u'فبسويه',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِسِوَيه',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'ه'
}
STOPWORDS[u'فبسويكم'] = {
    'word': u'فبسويكم',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِسِوَيكُمْ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فبسويكن'] = {
    'word': u'فبسويكن',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِسِوَيكُنَّ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فبسويها'] = {
    'word': u'فبسويها',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِسِوَيهَا',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'هَا'
}
STOPWORDS[u'فبسويهم'] = {
    'word': u'فبسويهم',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِسِوَيهُمْ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فبسويهن'] = {
    'word': u'فبسويهن',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِسِوَيهُنَّ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فبسوينا'] = {
    'word': u'فبسوينا',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِسِوَينَا',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'نَا'
}
STOPWORDS[u'فبسويكما'] = {
    'word': u'فبسويكما',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِسِوَيكُمَا',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فبسويهما'] = {
    'word': u'فبسويهما',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِسِوَيهُمَا',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'هُمَا'
}
STOPWORDS[u'وكسوي'] = {
    'word': u'وكسوي',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَسِوَيِي',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'ِي'
}
STOPWORDS[u'وكسويك'] = {
    'word': u'وكسويك',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَسِوَيكَ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كَ'
}
STOPWORDS[u'وكسويك'] = {
    'word': u'وكسويك',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَسِوَيكِ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كِ'
}
STOPWORDS[u'وكسويه'] = {
    'word': u'وكسويه',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَسِوَيه',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'ه'
}
STOPWORDS[u'وكسويكم'] = {
    'word': u'وكسويكم',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَسِوَيكُمْ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كُمْ'
}
STOPWORDS[u'وكسويكن'] = {
    'word': u'وكسويكن',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَسِوَيكُنَّ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'وكسويها'] = {
    'word': u'وكسويها',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَسِوَيهَا',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'هَا'
}
STOPWORDS[u'وكسويهم'] = {
    'word': u'وكسويهم',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَسِوَيهُمْ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'هُمْ'
}
STOPWORDS[u'وكسويهن'] = {
    'word': u'وكسويهن',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَسِوَيهُنَّ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'وكسوينا'] = {
    'word': u'وكسوينا',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَسِوَينَا',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'نَا'
}
STOPWORDS[u'وكسويكما'] = {
    'word': u'وكسويكما',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَسِوَيكُمَا',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كُمَا'
}
STOPWORDS[u'وكسويهما'] = {
    'word': u'وكسويهما',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَسِوَيهُمَا',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فكسوي'] = {
    'word': u'فكسوي',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَسِوَيِي',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'ِي'
}
STOPWORDS[u'فكسويك'] = {
    'word': u'فكسويك',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَسِوَيكَ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كَ'
}
STOPWORDS[u'فكسويك'] = {
    'word': u'فكسويك',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَسِوَيكِ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كِ'
}
STOPWORDS[u'فكسويه'] = {
    'word': u'فكسويه',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَسِوَيه',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'ه'
}
STOPWORDS[u'فكسويكم'] = {
    'word': u'فكسويكم',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَسِوَيكُمْ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فكسويكن'] = {
    'word': u'فكسويكن',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَسِوَيكُنَّ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فكسويها'] = {
    'word': u'فكسويها',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَسِوَيهَا',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'هَا'
}
STOPWORDS[u'فكسويهم'] = {
    'word': u'فكسويهم',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَسِوَيهُمْ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فكسويهن'] = {
    'word': u'فكسويهن',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَسِوَيهُنَّ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فكسوينا'] = {
    'word': u'فكسوينا',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَسِوَينَا',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'نَا'
}
STOPWORDS[u'فكسويكما'] = {
    'word': u'فكسويكما',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَسِوَيكُمَا',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فكسويهما'] = {
    'word': u'فكسويهما',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَسِوَيهُمَا',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ولسوي'] = {
    'word': u'ولسوي',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِسِوَيِي',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'ِي'
}
STOPWORDS[u'ولسويك'] = {
    'word': u'ولسويك',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِسِوَيكَ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كَ'
}
STOPWORDS[u'ولسويك'] = {
    'word': u'ولسويك',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِسِوَيكِ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كِ'
}
STOPWORDS[u'ولسويه'] = {
    'word': u'ولسويه',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِسِوَيه',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'ه'
}
STOPWORDS[u'ولسويكم'] = {
    'word': u'ولسويكم',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِسِوَيكُمْ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ولسويكن'] = {
    'word': u'ولسويكن',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِسِوَيكُنَّ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ولسويها'] = {
    'word': u'ولسويها',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِسِوَيهَا',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'هَا'
}
STOPWORDS[u'ولسويهم'] = {
    'word': u'ولسويهم',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِسِوَيهُمْ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ولسويهن'] = {
    'word': u'ولسويهن',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِسِوَيهُنَّ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ولسوينا'] = {
    'word': u'ولسوينا',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِسِوَينَا',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'نَا'
}
STOPWORDS[u'ولسويكما'] = {
    'word': u'ولسويكما',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِسِوَيكُمَا',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ولسويهما'] = {
    'word': u'ولسويهما',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِسِوَيهُمَا',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فلسوي'] = {
    'word': u'فلسوي',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِسِوَيِي',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'ِي'
}
STOPWORDS[u'فلسويك'] = {
    'word': u'فلسويك',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِسِوَيكَ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كَ'
}
STOPWORDS[u'فلسويك'] = {
    'word': u'فلسويك',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِسِوَيكِ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كِ'
}
STOPWORDS[u'فلسويه'] = {
    'word': u'فلسويه',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِسِوَيه',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'ه'
}
STOPWORDS[u'فلسويكم'] = {
    'word': u'فلسويكم',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِسِوَيكُمْ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فلسويكن'] = {
    'word': u'فلسويكن',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِسِوَيكُنَّ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فلسويها'] = {
    'word': u'فلسويها',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِسِوَيهَا',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'هَا'
}
STOPWORDS[u'فلسويهم'] = {
    'word': u'فلسويهم',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِسِوَيهُمْ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فلسويهن'] = {
    'word': u'فلسويهن',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِسِوَيهُنَّ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فلسوينا'] = {
    'word': u'فلسوينا',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِسِوَينَا',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'نَا'
}
STOPWORDS[u'فلسويكما'] = {
    'word': u'فلسويكما',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِسِوَيكُمَا',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فلسويهما'] = {
    'word': u'فلسويهما',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِسِوَيهُمَا',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أسوي'] = {
    'word': u'أسوي',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'أَسِوَيِي',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'ِي'
}
STOPWORDS[u'أسويك'] = {
    'word': u'أسويك',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'أَسِوَيكَ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كَ'
}
STOPWORDS[u'أسويك'] = {
    'word': u'أسويك',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'أَسِوَيكِ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كِ'
}
STOPWORDS[u'أسويه'] = {
    'word': u'أسويه',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'أَسِوَيه',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'ه'
}
STOPWORDS[u'أسويكم'] = {
    'word': u'أسويكم',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'أَسِوَيكُمْ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أسويكن'] = {
    'word': u'أسويكن',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'أَسِوَيكُنَّ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أسويها'] = {
    'word': u'أسويها',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'أَسِوَيهَا',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'هَا'
}
STOPWORDS[u'أسويهم'] = {
    'word': u'أسويهم',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'أَسِوَيهُمْ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أسويهن'] = {
    'word': u'أسويهن',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'أَسِوَيهُنَّ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أسوينا'] = {
    'word': u'أسوينا',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'أَسِوَينَا',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'نَا'
}
STOPWORDS[u'أسويكما'] = {
    'word': u'أسويكما',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'أَسِوَيكُمَا',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أسويهما'] = {
    'word': u'أسويهما',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'أَسِوَيهُمَا',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أبسوي'] = {
    'word': u'أبسوي',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِسِوَيِي',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'ِي'
}
STOPWORDS[u'أبسويك'] = {
    'word': u'أبسويك',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِسِوَيكَ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كَ'
}
STOPWORDS[u'أبسويك'] = {
    'word': u'أبسويك',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِسِوَيكِ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كِ'
}
STOPWORDS[u'أبسويه'] = {
    'word': u'أبسويه',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِسِوَيه',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'ه'
}
STOPWORDS[u'أبسويكم'] = {
    'word': u'أبسويكم',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِسِوَيكُمْ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أبسويكن'] = {
    'word': u'أبسويكن',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِسِوَيكُنَّ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أبسويها'] = {
    'word': u'أبسويها',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِسِوَيهَا',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'هَا'
}
STOPWORDS[u'أبسويهم'] = {
    'word': u'أبسويهم',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِسِوَيهُمْ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أبسويهن'] = {
    'word': u'أبسويهن',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِسِوَيهُنَّ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أبسوينا'] = {
    'word': u'أبسوينا',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِسِوَينَا',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'نَا'
}
STOPWORDS[u'أبسويكما'] = {
    'word': u'أبسويكما',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِسِوَيكُمَا',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أبسويهما'] = {
    'word': u'أبسويهما',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِسِوَيهُمَا',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أكسوي'] = {
    'word': u'أكسوي',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَسِوَيِي',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'ِي'
}
STOPWORDS[u'أكسويك'] = {
    'word': u'أكسويك',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَسِوَيكَ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كَ'
}
STOPWORDS[u'أكسويك'] = {
    'word': u'أكسويك',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَسِوَيكِ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كِ'
}
STOPWORDS[u'أكسويه'] = {
    'word': u'أكسويه',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَسِوَيه',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'ه'
}
STOPWORDS[u'أكسويكم'] = {
    'word': u'أكسويكم',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَسِوَيكُمْ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أكسويكن'] = {
    'word': u'أكسويكن',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَسِوَيكُنَّ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أكسويها'] = {
    'word': u'أكسويها',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَسِوَيهَا',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'هَا'
}
STOPWORDS[u'أكسويهم'] = {
    'word': u'أكسويهم',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَسِوَيهُمْ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أكسويهن'] = {
    'word': u'أكسويهن',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَسِوَيهُنَّ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أكسوينا'] = {
    'word': u'أكسوينا',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَسِوَينَا',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'نَا'
}
STOPWORDS[u'أكسويكما'] = {
    'word': u'أكسويكما',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَسِوَيكُمَا',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أكسويهما'] = {
    'word': u'أكسويهما',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَسِوَيهُمَا',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ألسوي'] = {
    'word': u'ألسوي',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِسِوَيِي',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'ِي'
}
STOPWORDS[u'ألسويك'] = {
    'word': u'ألسويك',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِسِوَيكَ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كَ'
}
STOPWORDS[u'ألسويك'] = {
    'word': u'ألسويك',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِسِوَيكِ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كِ'
}
STOPWORDS[u'ألسويه'] = {
    'word': u'ألسويه',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِسِوَيه',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'ه'
}
STOPWORDS[u'ألسويكم'] = {
    'word': u'ألسويكم',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِسِوَيكُمْ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ألسويكن'] = {
    'word': u'ألسويكن',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِسِوَيكُنَّ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ألسويها'] = {
    'word': u'ألسويها',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِسِوَيهَا',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'هَا'
}
STOPWORDS[u'ألسويهم'] = {
    'word': u'ألسويهم',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِسِوَيهُمْ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ألسويهن'] = {
    'word': u'ألسويهن',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِسِوَيهُنَّ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ألسوينا'] = {
    'word': u'ألسوينا',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِسِوَينَا',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'نَا'
}
STOPWORDS[u'ألسويكما'] = {
    'word': u'ألسويكما',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِسِوَيكُمَا',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ألسويهما'] = {
    'word': u'ألسويهما',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِسِوَيهُمَا',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أوسوي'] = {
    'word': u'أوسوي',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَسِوَيِي',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'ِي'
}
STOPWORDS[u'أوسويك'] = {
    'word': u'أوسويك',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَسِوَيكَ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كَ'
}
STOPWORDS[u'أوسويك'] = {
    'word': u'أوسويك',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَسِوَيكِ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كِ'
}
STOPWORDS[u'أوسويه'] = {
    'word': u'أوسويه',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَسِوَيه',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'ه'
}
STOPWORDS[u'أوسويكم'] = {
    'word': u'أوسويكم',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَسِوَيكُمْ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أوسويكن'] = {
    'word': u'أوسويكن',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَسِوَيكُنَّ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أوسويها'] = {
    'word': u'أوسويها',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَسِوَيهَا',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'هَا'
}
STOPWORDS[u'أوسويهم'] = {
    'word': u'أوسويهم',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَسِوَيهُمْ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أوسويهن'] = {
    'word': u'أوسويهن',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَسِوَيهُنَّ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أوسوينا'] = {
    'word': u'أوسوينا',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَسِوَينَا',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'نَا'
}
STOPWORDS[u'أوسويكما'] = {
    'word': u'أوسويكما',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَسِوَيكُمَا',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أوسويهما'] = {
    'word': u'أوسويهما',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَسِوَيهُمَا',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أفسوي'] = {
    'word': u'أفسوي',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَسِوَيِي',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'ِي'
}
STOPWORDS[u'أفسويك'] = {
    'word': u'أفسويك',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَسِوَيكَ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كَ'
}
STOPWORDS[u'أفسويك'] = {
    'word': u'أفسويك',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَسِوَيكِ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كِ'
}
STOPWORDS[u'أفسويه'] = {
    'word': u'أفسويه',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَسِوَيه',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'ه'
}
STOPWORDS[u'أفسويكم'] = {
    'word': u'أفسويكم',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَسِوَيكُمْ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أفسويكن'] = {
    'word': u'أفسويكن',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَسِوَيكُنَّ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أفسويها'] = {
    'word': u'أفسويها',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَسِوَيهَا',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'هَا'
}
STOPWORDS[u'أفسويهم'] = {
    'word': u'أفسويهم',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَسِوَيهُمْ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أفسويهن'] = {
    'word': u'أفسويهن',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَسِوَيهُنَّ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أفسوينا'] = {
    'word': u'أفسوينا',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَسِوَينَا',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'نَا'
}
STOPWORDS[u'أفسويكما'] = {
    'word': u'أفسويكما',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَسِوَيكُمَا',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أفسويهما'] = {
    'word': u'أفسويهما',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَسِوَيهُمَا',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أوبسوي'] = {
    'word': u'أوبسوي',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِسِوَيِي',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'ِي'
}
STOPWORDS[u'أوبسويك'] = {
    'word': u'أوبسويك',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِسِوَيكَ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كَ'
}
STOPWORDS[u'أوبسويك'] = {
    'word': u'أوبسويك',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِسِوَيكِ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كِ'
}
STOPWORDS[u'أوبسويه'] = {
    'word': u'أوبسويه',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِسِوَيه',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'ه'
}
STOPWORDS[u'أوبسويكم'] = {
    'word': u'أوبسويكم',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِسِوَيكُمْ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أوبسويكن'] = {
    'word': u'أوبسويكن',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِسِوَيكُنَّ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أوبسويها'] = {
    'word': u'أوبسويها',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِسِوَيهَا',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'هَا'
}
STOPWORDS[u'أوبسويهم'] = {
    'word': u'أوبسويهم',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِسِوَيهُمْ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أوبسويهن'] = {
    'word': u'أوبسويهن',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِسِوَيهُنَّ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أوبسوينا'] = {
    'word': u'أوبسوينا',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِسِوَينَا',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'نَا'
}
STOPWORDS[u'أوبسويكما'] = {
    'word': u'أوبسويكما',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِسِوَيكُمَا',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أوبسويهما'] = {
    'word': u'أوبسويهما',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِسِوَيهُمَا',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أفبسوي'] = {
    'word': u'أفبسوي',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِسِوَيِي',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'ِي'
}
STOPWORDS[u'أفبسويك'] = {
    'word': u'أفبسويك',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِسِوَيكَ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كَ'
}
STOPWORDS[u'أفبسويك'] = {
    'word': u'أفبسويك',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِسِوَيكِ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كِ'
}
STOPWORDS[u'أفبسويه'] = {
    'word': u'أفبسويه',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِسِوَيه',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'ه'
}
STOPWORDS[u'أفبسويكم'] = {
    'word': u'أفبسويكم',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِسِوَيكُمْ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أفبسويكن'] = {
    'word': u'أفبسويكن',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِسِوَيكُنَّ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أفبسويها'] = {
    'word': u'أفبسويها',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِسِوَيهَا',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'هَا'
}
STOPWORDS[u'أفبسويهم'] = {
    'word': u'أفبسويهم',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِسِوَيهُمْ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أفبسويهن'] = {
    'word': u'أفبسويهن',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِسِوَيهُنَّ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أفبسوينا'] = {
    'word': u'أفبسوينا',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِسِوَينَا',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'نَا'
}
STOPWORDS[u'أفبسويكما'] = {
    'word': u'أفبسويكما',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِسِوَيكُمَا',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أفبسويهما'] = {
    'word': u'أفبسويهما',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِسِوَيهُمَا',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أوكسوي'] = {
    'word': u'أوكسوي',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَسِوَيِي',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'ِي'
}
STOPWORDS[u'أوكسويك'] = {
    'word': u'أوكسويك',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَسِوَيكَ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كَ'
}
STOPWORDS[u'أوكسويك'] = {
    'word': u'أوكسويك',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَسِوَيكِ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كِ'
}
STOPWORDS[u'أوكسويه'] = {
    'word': u'أوكسويه',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَسِوَيه',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'ه'
}
STOPWORDS[u'أوكسويكم'] = {
    'word': u'أوكسويكم',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَسِوَيكُمْ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أوكسويكن'] = {
    'word': u'أوكسويكن',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَسِوَيكُنَّ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أوكسويها'] = {
    'word': u'أوكسويها',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَسِوَيهَا',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'هَا'
}
STOPWORDS[u'أوكسويهم'] = {
    'word': u'أوكسويهم',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَسِوَيهُمْ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أوكسويهن'] = {
    'word': u'أوكسويهن',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَسِوَيهُنَّ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أوكسوينا'] = {
    'word': u'أوكسوينا',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَسِوَينَا',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'نَا'
}
STOPWORDS[u'أوكسويكما'] = {
    'word': u'أوكسويكما',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَسِوَيكُمَا',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أوكسويهما'] = {
    'word': u'أوكسويهما',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَسِوَيهُمَا',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أفكسوي'] = {
    'word': u'أفكسوي',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَسِوَيِي',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'ِي'
}
STOPWORDS[u'أفكسويك'] = {
    'word': u'أفكسويك',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَسِوَيكَ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كَ'
}
STOPWORDS[u'أفكسويك'] = {
    'word': u'أفكسويك',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَسِوَيكِ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كِ'
}
STOPWORDS[u'أفكسويه'] = {
    'word': u'أفكسويه',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَسِوَيه',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'ه'
}
STOPWORDS[u'أفكسويكم'] = {
    'word': u'أفكسويكم',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَسِوَيكُمْ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أفكسويكن'] = {
    'word': u'أفكسويكن',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَسِوَيكُنَّ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أفكسويها'] = {
    'word': u'أفكسويها',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَسِوَيهَا',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'هَا'
}
STOPWORDS[u'أفكسويهم'] = {
    'word': u'أفكسويهم',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَسِوَيهُمْ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أفكسويهن'] = {
    'word': u'أفكسويهن',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَسِوَيهُنَّ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أفكسوينا'] = {
    'word': u'أفكسوينا',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَسِوَينَا',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'نَا'
}
STOPWORDS[u'أفكسويكما'] = {
    'word': u'أفكسويكما',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَسِوَيكُمَا',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أفكسويهما'] = {
    'word': u'أفكسويهما',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَسِوَيهُمَا',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أولسوي'] = {
    'word': u'أولسوي',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِسِوَيِي',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'ِي'
}
STOPWORDS[u'أولسويك'] = {
    'word': u'أولسويك',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِسِوَيكَ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كَ'
}
STOPWORDS[u'أولسويك'] = {
    'word': u'أولسويك',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِسِوَيكِ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كِ'
}
STOPWORDS[u'أولسويه'] = {
    'word': u'أولسويه',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِسِوَيه',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'ه'
}
STOPWORDS[u'أولسويكم'] = {
    'word': u'أولسويكم',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِسِوَيكُمْ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أولسويكن'] = {
    'word': u'أولسويكن',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِسِوَيكُنَّ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أولسويها'] = {
    'word': u'أولسويها',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِسِوَيهَا',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'هَا'
}
STOPWORDS[u'أولسويهم'] = {
    'word': u'أولسويهم',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِسِوَيهُمْ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أولسويهن'] = {
    'word': u'أولسويهن',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِسِوَيهُنَّ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أولسوينا'] = {
    'word': u'أولسوينا',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِسِوَينَا',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'نَا'
}
STOPWORDS[u'أولسويكما'] = {
    'word': u'أولسويكما',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِسِوَيكُمَا',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أولسويهما'] = {
    'word': u'أولسويهما',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِسِوَيهُمَا',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أفلسوي'] = {
    'word': u'أفلسوي',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِسِوَيِي',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'ِي'
}
STOPWORDS[u'أفلسويك'] = {
    'word': u'أفلسويك',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِسِوَيكَ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كَ'
}
STOPWORDS[u'أفلسويك'] = {
    'word': u'أفلسويك',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِسِوَيكِ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كِ'
}
STOPWORDS[u'أفلسويه'] = {
    'word': u'أفلسويه',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِسِوَيه',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'ه'
}
STOPWORDS[u'أفلسويكم'] = {
    'word': u'أفلسويكم',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِسِوَيكُمْ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أفلسويكن'] = {
    'word': u'أفلسويكن',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِسِوَيكُنَّ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أفلسويها'] = {
    'word': u'أفلسويها',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِسِوَيهَا',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'هَا'
}
STOPWORDS[u'أفلسويهم'] = {
    'word': u'أفلسويهم',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِسِوَيهُمْ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أفلسويهن'] = {
    'word': u'أفلسويهن',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِسِوَيهُنَّ',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أفلسوينا'] = {
    'word': u'أفلسوينا',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِسِوَينَا',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'نَا'
}
STOPWORDS[u'أفلسويكما'] = {
    'word': u'أفلسويكما',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِسِوَيكُمَا',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أفلسويهما'] = {
    'word': u'أفلسويهما',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِسِوَيهُمَا',
    'stem': u'سِوَى',
    'type': u'STOPWORD',
    'original': u'سِوَى',
    'encletic': u'هُمَا'
}
STOPWORDS[u'شبه'] = {
    'word': u'شِبْهُ',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة',
    'vocalized': u'شِبْهُ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u''
}
STOPWORDS[u'بشبه'] = {
    'word': u'بشبه',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور',
    'vocalized': u'بِشِبْهُ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u''
}
STOPWORDS[u'كشبه'] = {
    'word': u'كشبه',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور',
    'vocalized': u'كَشِبْهُ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u''
}
STOPWORDS[u'لشبه'] = {
    'word': u'لشبه',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور',
    'vocalized': u'لِشِبْهُ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u''
}
STOPWORDS[u'لشبه'] = {
    'word': u'لشبه',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور',
    'vocalized': u'لَشِبْهُ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u''
}
STOPWORDS[u'وشبه'] = {
    'word': u'وشبه',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف',
    'vocalized': u'وَشِبْهُ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u''
}
STOPWORDS[u'فشبه'] = {
    'word': u'فشبه',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف',
    'vocalized': u'فَشِبْهُ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u''
}
STOPWORDS[u'وبشبه'] = {
    'word': u'وبشبه',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف',
    'vocalized': u'وَبِشِبْهُ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u''
}
STOPWORDS[u'فبشبه'] = {
    'word': u'فبشبه',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف',
    'vocalized': u'فَبِشِبْهُ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u''
}
STOPWORDS[u'وكشبه'] = {
    'word': u'وكشبه',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف',
    'vocalized': u'وَكَشِبْهُ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u''
}
STOPWORDS[u'فكشبه'] = {
    'word': u'فكشبه',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف',
    'vocalized': u'فَكَشِبْهُ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u''
}
STOPWORDS[u'ولشبه'] = {
    'word': u'ولشبه',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف',
    'vocalized': u'وَلِشِبْهُ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u''
}
STOPWORDS[u'فلشبه'] = {
    'word': u'فلشبه',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف',
    'vocalized': u'فَلِشِبْهُ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u''
}
STOPWORDS[u'ولشبه'] = {
    'word': u'ولشبه',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'وَلَشِبْهُ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u''
}
STOPWORDS[u'فلشبه'] = {
    'word': u'فلشبه',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'فَلَشِبْهُ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u''
}
STOPWORDS[u'أشبه'] = {
    'word': u'أشبه',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور',
    'vocalized': u'أَشِبْهُ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u''
}
STOPWORDS[u'أبشبه'] = {
    'word': u'أبشبه',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور',
    'vocalized': u'أَبِشِبْهُ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u''
}
STOPWORDS[u'أكشبه'] = {
    'word': u'أكشبه',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور',
    'vocalized': u'أَكَشِبْهُ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u''
}
STOPWORDS[u'ألشبه'] = {
    'word': u'ألشبه',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور',
    'vocalized': u'أَلِشِبْهُ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u''
}
STOPWORDS[u'ألشبه'] = {
    'word': u'ألشبه',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور',
    'vocalized': u'أَلَشِبْهُ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u''
}
STOPWORDS[u'أوشبه'] = {
    'word': u'أوشبه',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور',
    'vocalized': u'أَوَشِبْهُ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u''
}
STOPWORDS[u'أفشبه'] = {
    'word': u'أفشبه',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور',
    'vocalized': u'أَفَشِبْهُ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u''
}
STOPWORDS[u'أوبشبه'] = {
    'word': u'أوبشبه',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَبِشِبْهُ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u''
}
STOPWORDS[u'أفبشبه'] = {
    'word': u'أفبشبه',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَبِشِبْهُ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u''
}
STOPWORDS[u'أوكشبه'] = {
    'word': u'أوكشبه',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَكَشِبْهُ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u''
}
STOPWORDS[u'أفكشبه'] = {
    'word': u'أفكشبه',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَكَشِبْهُ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u''
}
STOPWORDS[u'أولشبه'] = {
    'word': u'أولشبه',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَلِشِبْهُ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u''
}
STOPWORDS[u'أفلشبه'] = {
    'word': u'أفلشبه',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَلِشِبْهُ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u''
}
STOPWORDS[u'أولشبه'] = {
    'word': u'أولشبه',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَلَشِبْهُ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u''
}
STOPWORDS[u'أفلشبه'] = {
    'word': u'أفلشبه',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَلَشِبْهُ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u''
}
STOPWORDS[u'شبهي'] = {
    'word': u'شبهي',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'شِبْهُِي',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'ِي'
}
STOPWORDS[u'شبهك'] = {
    'word': u'شبهك',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'شِبْهُكَ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كَ'
}
STOPWORDS[u'شبهك'] = {
    'word': u'شبهك',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'شِبْهُكِ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كِ'
}
STOPWORDS[u'شبهه'] = {
    'word': u'شبهه',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'شِبْهُه',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'ه'
}
STOPWORDS[u'شبهكم'] = {
    'word': u'شبهكم',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'شِبْهُكُمْ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'شبهكن'] = {
    'word': u'شبهكن',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'شِبْهُكُنَّ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'شبهها'] = {
    'word': u'شبهها',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'شِبْهُهَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هَا'
}
STOPWORDS[u'شبههم'] = {
    'word': u'شبههم',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'شِبْهُهُمْ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'شبههن'] = {
    'word': u'شبههن',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'شِبْهُهُنَّ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'شبهنا'] = {
    'word': u'شبهنا',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'شِبْهُنَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'نَا'
}
STOPWORDS[u'شبهكما'] = {
    'word': u'شبهكما',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'شِبْهُكُمَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'شبههما'] = {
    'word': u'شبههما',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'شِبْهُهُمَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'بشبهي'] = {
    'word': u'بشبهي',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِشِبْهُِي',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'ِي'
}
STOPWORDS[u'بشبهك'] = {
    'word': u'بشبهك',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِشِبْهُكَ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كَ'
}
STOPWORDS[u'بشبهك'] = {
    'word': u'بشبهك',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِشِبْهُكِ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كِ'
}
STOPWORDS[u'بشبهه'] = {
    'word': u'بشبهه',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِشِبْهُه',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'ه'
}
STOPWORDS[u'بشبهكم'] = {
    'word': u'بشبهكم',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِشِبْهُكُمْ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'بشبهكن'] = {
    'word': u'بشبهكن',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِشِبْهُكُنَّ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'بشبهها'] = {
    'word': u'بشبهها',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِشِبْهُهَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هَا'
}
STOPWORDS[u'بشبههم'] = {
    'word': u'بشبههم',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِشِبْهُهُمْ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'بشبههن'] = {
    'word': u'بشبههن',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِشِبْهُهُنَّ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'بشبهنا'] = {
    'word': u'بشبهنا',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِشِبْهُنَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'نَا'
}
STOPWORDS[u'بشبهكما'] = {
    'word': u'بشبهكما',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِشِبْهُكُمَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'بشبههما'] = {
    'word': u'بشبههما',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِشِبْهُهُمَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'كشبهي'] = {
    'word': u'كشبهي',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَشِبْهُِي',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'ِي'
}
STOPWORDS[u'كشبهك'] = {
    'word': u'كشبهك',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَشِبْهُكَ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كَ'
}
STOPWORDS[u'كشبهك'] = {
    'word': u'كشبهك',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَشِبْهُكِ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كِ'
}
STOPWORDS[u'كشبهه'] = {
    'word': u'كشبهه',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَشِبْهُه',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'ه'
}
STOPWORDS[u'كشبهكم'] = {
    'word': u'كشبهكم',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَشِبْهُكُمْ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'كشبهكن'] = {
    'word': u'كشبهكن',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَشِبْهُكُنَّ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'كشبهها'] = {
    'word': u'كشبهها',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَشِبْهُهَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هَا'
}
STOPWORDS[u'كشبههم'] = {
    'word': u'كشبههم',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَشِبْهُهُمْ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'كشبههن'] = {
    'word': u'كشبههن',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَشِبْهُهُنَّ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'كشبهنا'] = {
    'word': u'كشبهنا',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَشِبْهُنَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'نَا'
}
STOPWORDS[u'كشبهكما'] = {
    'word': u'كشبهكما',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَشِبْهُكُمَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'كشبههما'] = {
    'word': u'كشبههما',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَشِبْهُهُمَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'لشبهي'] = {
    'word': u'لشبهي',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِشِبْهُِي',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'ِي'
}
STOPWORDS[u'لشبهك'] = {
    'word': u'لشبهك',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِشِبْهُكَ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كَ'
}
STOPWORDS[u'لشبهك'] = {
    'word': u'لشبهك',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِشِبْهُكِ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كِ'
}
STOPWORDS[u'لشبهه'] = {
    'word': u'لشبهه',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِشِبْهُه',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'ه'
}
STOPWORDS[u'لشبهكم'] = {
    'word': u'لشبهكم',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِشِبْهُكُمْ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'لشبهكن'] = {
    'word': u'لشبهكن',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِشِبْهُكُنَّ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'لشبهها'] = {
    'word': u'لشبهها',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِشِبْهُهَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هَا'
}
STOPWORDS[u'لشبههم'] = {
    'word': u'لشبههم',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِشِبْهُهُمْ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'لشبههن'] = {
    'word': u'لشبههن',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِشِبْهُهُنَّ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'لشبهنا'] = {
    'word': u'لشبهنا',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِشِبْهُنَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'نَا'
}
STOPWORDS[u'لشبهكما'] = {
    'word': u'لشبهكما',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِشِبْهُكُمَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'لشبههما'] = {
    'word': u'لشبههما',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِشِبْهُهُمَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'لشبهي'] = {
    'word': u'لشبهي',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَشِبْهُِي',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'ِي'
}
STOPWORDS[u'لشبهك'] = {
    'word': u'لشبهك',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَشِبْهُكَ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كَ'
}
STOPWORDS[u'لشبهك'] = {
    'word': u'لشبهك',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَشِبْهُكِ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كِ'
}
STOPWORDS[u'لشبهه'] = {
    'word': u'لشبهه',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَشِبْهُه',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'ه'
}
STOPWORDS[u'لشبهكم'] = {
    'word': u'لشبهكم',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَشِبْهُكُمْ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'لشبهكن'] = {
    'word': u'لشبهكن',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَشِبْهُكُنَّ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'لشبهها'] = {
    'word': u'لشبهها',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَشِبْهُهَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هَا'
}
STOPWORDS[u'لشبههم'] = {
    'word': u'لشبههم',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَشِبْهُهُمْ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'لشبههن'] = {
    'word': u'لشبههن',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَشِبْهُهُنَّ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'لشبهنا'] = {
    'word': u'لشبهنا',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَشِبْهُنَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'نَا'
}
STOPWORDS[u'لشبهكما'] = {
    'word': u'لشبهكما',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَشِبْهُكُمَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'لشبههما'] = {
    'word': u'لشبههما',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَشِبْهُهُمَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'وشبهي'] = {
    'word': u'وشبهي',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَشِبْهُِي',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'ِي'
}
STOPWORDS[u'وشبهك'] = {
    'word': u'وشبهك',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَشِبْهُكَ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كَ'
}
STOPWORDS[u'وشبهك'] = {
    'word': u'وشبهك',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَشِبْهُكِ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كِ'
}
STOPWORDS[u'وشبهه'] = {
    'word': u'وشبهه',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَشِبْهُه',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'ه'
}
STOPWORDS[u'وشبهكم'] = {
    'word': u'وشبهكم',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَشِبْهُكُمْ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'وشبهكن'] = {
    'word': u'وشبهكن',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَشِبْهُكُنَّ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'وشبهها'] = {
    'word': u'وشبهها',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَشِبْهُهَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هَا'
}
STOPWORDS[u'وشبههم'] = {
    'word': u'وشبههم',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَشِبْهُهُمْ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'وشبههن'] = {
    'word': u'وشبههن',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَشِبْهُهُنَّ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'وشبهنا'] = {
    'word': u'وشبهنا',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَشِبْهُنَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'نَا'
}
STOPWORDS[u'وشبهكما'] = {
    'word': u'وشبهكما',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَشِبْهُكُمَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'وشبههما'] = {
    'word': u'وشبههما',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَشِبْهُهُمَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فشبهي'] = {
    'word': u'فشبهي',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَشِبْهُِي',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'ِي'
}
STOPWORDS[u'فشبهك'] = {
    'word': u'فشبهك',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَشِبْهُكَ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كَ'
}
STOPWORDS[u'فشبهك'] = {
    'word': u'فشبهك',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَشِبْهُكِ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كِ'
}
STOPWORDS[u'فشبهه'] = {
    'word': u'فشبهه',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَشِبْهُه',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'ه'
}
STOPWORDS[u'فشبهكم'] = {
    'word': u'فشبهكم',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَشِبْهُكُمْ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فشبهكن'] = {
    'word': u'فشبهكن',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَشِبْهُكُنَّ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فشبهها'] = {
    'word': u'فشبهها',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَشِبْهُهَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هَا'
}
STOPWORDS[u'فشبههم'] = {
    'word': u'فشبههم',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَشِبْهُهُمْ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فشبههن'] = {
    'word': u'فشبههن',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَشِبْهُهُنَّ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فشبهنا'] = {
    'word': u'فشبهنا',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَشِبْهُنَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'نَا'
}
STOPWORDS[u'فشبهكما'] = {
    'word': u'فشبهكما',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَشِبْهُكُمَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فشبههما'] = {
    'word': u'فشبههما',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَشِبْهُهُمَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'وبشبهي'] = {
    'word': u'وبشبهي',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِشِبْهُِي',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'ِي'
}
STOPWORDS[u'وبشبهك'] = {
    'word': u'وبشبهك',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِشِبْهُكَ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كَ'
}
STOPWORDS[u'وبشبهك'] = {
    'word': u'وبشبهك',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِشِبْهُكِ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كِ'
}
STOPWORDS[u'وبشبهه'] = {
    'word': u'وبشبهه',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِشِبْهُه',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'ه'
}
STOPWORDS[u'وبشبهكم'] = {
    'word': u'وبشبهكم',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِشِبْهُكُمْ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'وبشبهكن'] = {
    'word': u'وبشبهكن',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِشِبْهُكُنَّ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'وبشبهها'] = {
    'word': u'وبشبهها',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِشِبْهُهَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هَا'
}
STOPWORDS[u'وبشبههم'] = {
    'word': u'وبشبههم',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِشِبْهُهُمْ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'وبشبههن'] = {
    'word': u'وبشبههن',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِشِبْهُهُنَّ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'وبشبهنا'] = {
    'word': u'وبشبهنا',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِشِبْهُنَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'نَا'
}
STOPWORDS[u'وبشبهكما'] = {
    'word': u'وبشبهكما',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِشِبْهُكُمَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'وبشبههما'] = {
    'word': u'وبشبههما',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِشِبْهُهُمَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فبشبهي'] = {
    'word': u'فبشبهي',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِشِبْهُِي',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'ِي'
}
STOPWORDS[u'فبشبهك'] = {
    'word': u'فبشبهك',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِشِبْهُكَ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كَ'
}
STOPWORDS[u'فبشبهك'] = {
    'word': u'فبشبهك',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِشِبْهُكِ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كِ'
}
STOPWORDS[u'فبشبهه'] = {
    'word': u'فبشبهه',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِشِبْهُه',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'ه'
}
STOPWORDS[u'فبشبهكم'] = {
    'word': u'فبشبهكم',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِشِبْهُكُمْ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فبشبهكن'] = {
    'word': u'فبشبهكن',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِشِبْهُكُنَّ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فبشبهها'] = {
    'word': u'فبشبهها',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِشِبْهُهَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هَا'
}
STOPWORDS[u'فبشبههم'] = {
    'word': u'فبشبههم',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِشِبْهُهُمْ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فبشبههن'] = {
    'word': u'فبشبههن',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِشِبْهُهُنَّ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فبشبهنا'] = {
    'word': u'فبشبهنا',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِشِبْهُنَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'نَا'
}
STOPWORDS[u'فبشبهكما'] = {
    'word': u'فبشبهكما',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِشِبْهُكُمَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فبشبههما'] = {
    'word': u'فبشبههما',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِشِبْهُهُمَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'وكشبهي'] = {
    'word': u'وكشبهي',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَشِبْهُِي',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'ِي'
}
STOPWORDS[u'وكشبهك'] = {
    'word': u'وكشبهك',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَشِبْهُكَ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كَ'
}
STOPWORDS[u'وكشبهك'] = {
    'word': u'وكشبهك',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَشِبْهُكِ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كِ'
}
STOPWORDS[u'وكشبهه'] = {
    'word': u'وكشبهه',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَشِبْهُه',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'ه'
}
STOPWORDS[u'وكشبهكم'] = {
    'word': u'وكشبهكم',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَشِبْهُكُمْ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'وكشبهكن'] = {
    'word': u'وكشبهكن',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَشِبْهُكُنَّ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'وكشبهها'] = {
    'word': u'وكشبهها',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَشِبْهُهَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هَا'
}
STOPWORDS[u'وكشبههم'] = {
    'word': u'وكشبههم',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَشِبْهُهُمْ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'وكشبههن'] = {
    'word': u'وكشبههن',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَشِبْهُهُنَّ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'وكشبهنا'] = {
    'word': u'وكشبهنا',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَشِبْهُنَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'نَا'
}
STOPWORDS[u'وكشبهكما'] = {
    'word': u'وكشبهكما',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَشِبْهُكُمَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'وكشبههما'] = {
    'word': u'وكشبههما',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَشِبْهُهُمَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فكشبهي'] = {
    'word': u'فكشبهي',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَشِبْهُِي',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'ِي'
}
STOPWORDS[u'فكشبهك'] = {
    'word': u'فكشبهك',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَشِبْهُكَ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كَ'
}
STOPWORDS[u'فكشبهك'] = {
    'word': u'فكشبهك',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَشِبْهُكِ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كِ'
}
STOPWORDS[u'فكشبهه'] = {
    'word': u'فكشبهه',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَشِبْهُه',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'ه'
}
STOPWORDS[u'فكشبهكم'] = {
    'word': u'فكشبهكم',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَشِبْهُكُمْ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فكشبهكن'] = {
    'word': u'فكشبهكن',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَشِبْهُكُنَّ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فكشبهها'] = {
    'word': u'فكشبهها',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَشِبْهُهَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هَا'
}
STOPWORDS[u'فكشبههم'] = {
    'word': u'فكشبههم',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَشِبْهُهُمْ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فكشبههن'] = {
    'word': u'فكشبههن',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَشِبْهُهُنَّ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فكشبهنا'] = {
    'word': u'فكشبهنا',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَشِبْهُنَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'نَا'
}
STOPWORDS[u'فكشبهكما'] = {
    'word': u'فكشبهكما',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَشِبْهُكُمَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فكشبههما'] = {
    'word': u'فكشبههما',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَشِبْهُهُمَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ولشبهي'] = {
    'word': u'ولشبهي',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِشِبْهُِي',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'ِي'
}
STOPWORDS[u'ولشبهك'] = {
    'word': u'ولشبهك',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِشِبْهُكَ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كَ'
}
STOPWORDS[u'ولشبهك'] = {
    'word': u'ولشبهك',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِشِبْهُكِ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كِ'
}
STOPWORDS[u'ولشبهه'] = {
    'word': u'ولشبهه',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِشِبْهُه',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'ه'
}
STOPWORDS[u'ولشبهكم'] = {
    'word': u'ولشبهكم',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِشِبْهُكُمْ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ولشبهكن'] = {
    'word': u'ولشبهكن',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِشِبْهُكُنَّ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ولشبهها'] = {
    'word': u'ولشبهها',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِشِبْهُهَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هَا'
}
STOPWORDS[u'ولشبههم'] = {
    'word': u'ولشبههم',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِشِبْهُهُمْ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ولشبههن'] = {
    'word': u'ولشبههن',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِشِبْهُهُنَّ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ولشبهنا'] = {
    'word': u'ولشبهنا',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِشِبْهُنَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'نَا'
}
STOPWORDS[u'ولشبهكما'] = {
    'word': u'ولشبهكما',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِشِبْهُكُمَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ولشبههما'] = {
    'word': u'ولشبههما',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِشِبْهُهُمَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فلشبهي'] = {
    'word': u'فلشبهي',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِشِبْهُِي',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'ِي'
}
STOPWORDS[u'فلشبهك'] = {
    'word': u'فلشبهك',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِشِبْهُكَ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كَ'
}
STOPWORDS[u'فلشبهك'] = {
    'word': u'فلشبهك',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِشِبْهُكِ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كِ'
}
STOPWORDS[u'فلشبهه'] = {
    'word': u'فلشبهه',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِشِبْهُه',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'ه'
}
STOPWORDS[u'فلشبهكم'] = {
    'word': u'فلشبهكم',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِشِبْهُكُمْ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فلشبهكن'] = {
    'word': u'فلشبهكن',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِشِبْهُكُنَّ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فلشبهها'] = {
    'word': u'فلشبهها',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِشِبْهُهَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هَا'
}
STOPWORDS[u'فلشبههم'] = {
    'word': u'فلشبههم',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِشِبْهُهُمْ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فلشبههن'] = {
    'word': u'فلشبههن',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِشِبْهُهُنَّ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فلشبهنا'] = {
    'word': u'فلشبهنا',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِشِبْهُنَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'نَا'
}
STOPWORDS[u'فلشبهكما'] = {
    'word': u'فلشبهكما',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِشِبْهُكُمَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فلشبههما'] = {
    'word': u'فلشبههما',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِشِبْهُهُمَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ولشبهي'] = {
    'word': u'ولشبهي',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَشِبْهُِي',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'ِي'
}
STOPWORDS[u'ولشبهك'] = {
    'word': u'ولشبهك',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَشِبْهُكَ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كَ'
}
STOPWORDS[u'ولشبهك'] = {
    'word': u'ولشبهك',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَشِبْهُكِ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كِ'
}
STOPWORDS[u'ولشبهه'] = {
    'word': u'ولشبهه',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَشِبْهُه',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'ه'
}
STOPWORDS[u'ولشبهكم'] = {
    'word': u'ولشبهكم',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَشِبْهُكُمْ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ولشبهكن'] = {
    'word': u'ولشبهكن',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَشِبْهُكُنَّ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ولشبهها'] = {
    'word': u'ولشبهها',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَشِبْهُهَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هَا'
}
STOPWORDS[u'ولشبههم'] = {
    'word': u'ولشبههم',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَشِبْهُهُمْ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ولشبههن'] = {
    'word': u'ولشبههن',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَشِبْهُهُنَّ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ولشبهنا'] = {
    'word': u'ولشبهنا',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَشِبْهُنَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'نَا'
}
STOPWORDS[u'ولشبهكما'] = {
    'word': u'ولشبهكما',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَشِبْهُكُمَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ولشبههما'] = {
    'word': u'ولشبههما',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَشِبْهُهُمَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فلشبهي'] = {
    'word': u'فلشبهي',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَشِبْهُِي',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'ِي'
}
STOPWORDS[u'فلشبهك'] = {
    'word': u'فلشبهك',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَشِبْهُكَ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كَ'
}
STOPWORDS[u'فلشبهك'] = {
    'word': u'فلشبهك',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَشِبْهُكِ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كِ'
}
STOPWORDS[u'فلشبهه'] = {
    'word': u'فلشبهه',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَشِبْهُه',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'ه'
}
STOPWORDS[u'فلشبهكم'] = {
    'word': u'فلشبهكم',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَشِبْهُكُمْ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فلشبهكن'] = {
    'word': u'فلشبهكن',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَشِبْهُكُنَّ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فلشبهها'] = {
    'word': u'فلشبهها',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَشِبْهُهَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هَا'
}
STOPWORDS[u'فلشبههم'] = {
    'word': u'فلشبههم',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَشِبْهُهُمْ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فلشبههن'] = {
    'word': u'فلشبههن',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَشِبْهُهُنَّ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فلشبهنا'] = {
    'word': u'فلشبهنا',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَشِبْهُنَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'نَا'
}
STOPWORDS[u'فلشبهكما'] = {
    'word': u'فلشبهكما',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَشِبْهُكُمَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فلشبههما'] = {
    'word': u'فلشبههما',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَشِبْهُهُمَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أشبهي'] = {
    'word': u'أشبهي',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'أَشِبْهُِي',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'ِي'
}
STOPWORDS[u'أشبهك'] = {
    'word': u'أشبهك',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'أَشِبْهُكَ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كَ'
}
STOPWORDS[u'أشبهك'] = {
    'word': u'أشبهك',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'أَشِبْهُكِ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كِ'
}
STOPWORDS[u'أشبهه'] = {
    'word': u'أشبهه',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'أَشِبْهُه',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'ه'
}
STOPWORDS[u'أشبهكم'] = {
    'word': u'أشبهكم',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'أَشِبْهُكُمْ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أشبهكن'] = {
    'word': u'أشبهكن',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'أَشِبْهُكُنَّ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أشبهها'] = {
    'word': u'أشبهها',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'أَشِبْهُهَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هَا'
}
STOPWORDS[u'أشبههم'] = {
    'word': u'أشبههم',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'أَشِبْهُهُمْ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أشبههن'] = {
    'word': u'أشبههن',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'أَشِبْهُهُنَّ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أشبهنا'] = {
    'word': u'أشبهنا',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'أَشِبْهُنَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'نَا'
}
STOPWORDS[u'أشبهكما'] = {
    'word': u'أشبهكما',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'أَشِبْهُكُمَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أشبههما'] = {
    'word': u'أشبههما',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'أَشِبْهُهُمَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أبشبهي'] = {
    'word': u'أبشبهي',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِشِبْهُِي',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'ِي'
}
STOPWORDS[u'أبشبهك'] = {
    'word': u'أبشبهك',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِشِبْهُكَ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كَ'
}
STOPWORDS[u'أبشبهك'] = {
    'word': u'أبشبهك',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِشِبْهُكِ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كِ'
}
STOPWORDS[u'أبشبهه'] = {
    'word': u'أبشبهه',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِشِبْهُه',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'ه'
}
STOPWORDS[u'أبشبهكم'] = {
    'word': u'أبشبهكم',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِشِبْهُكُمْ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أبشبهكن'] = {
    'word': u'أبشبهكن',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِشِبْهُكُنَّ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أبشبهها'] = {
    'word': u'أبشبهها',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِشِبْهُهَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هَا'
}
STOPWORDS[u'أبشبههم'] = {
    'word': u'أبشبههم',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِشِبْهُهُمْ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أبشبههن'] = {
    'word': u'أبشبههن',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِشِبْهُهُنَّ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أبشبهنا'] = {
    'word': u'أبشبهنا',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِشِبْهُنَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'نَا'
}
STOPWORDS[u'أبشبهكما'] = {
    'word': u'أبشبهكما',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِشِبْهُكُمَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أبشبههما'] = {
    'word': u'أبشبههما',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِشِبْهُهُمَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أكشبهي'] = {
    'word': u'أكشبهي',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَشِبْهُِي',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'ِي'
}
STOPWORDS[u'أكشبهك'] = {
    'word': u'أكشبهك',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَشِبْهُكَ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كَ'
}
STOPWORDS[u'أكشبهك'] = {
    'word': u'أكشبهك',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَشِبْهُكِ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كِ'
}
STOPWORDS[u'أكشبهه'] = {
    'word': u'أكشبهه',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَشِبْهُه',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'ه'
}
STOPWORDS[u'أكشبهكم'] = {
    'word': u'أكشبهكم',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَشِبْهُكُمْ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أكشبهكن'] = {
    'word': u'أكشبهكن',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَشِبْهُكُنَّ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أكشبهها'] = {
    'word': u'أكشبهها',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَشِبْهُهَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هَا'
}
STOPWORDS[u'أكشبههم'] = {
    'word': u'أكشبههم',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَشِبْهُهُمْ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أكشبههن'] = {
    'word': u'أكشبههن',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَشِبْهُهُنَّ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أكشبهنا'] = {
    'word': u'أكشبهنا',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَشِبْهُنَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'نَا'
}
STOPWORDS[u'أكشبهكما'] = {
    'word': u'أكشبهكما',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَشِبْهُكُمَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أكشبههما'] = {
    'word': u'أكشبههما',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَشِبْهُهُمَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ألشبهي'] = {
    'word': u'ألشبهي',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِشِبْهُِي',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'ِي'
}
STOPWORDS[u'ألشبهك'] = {
    'word': u'ألشبهك',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِشِبْهُكَ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كَ'
}
STOPWORDS[u'ألشبهك'] = {
    'word': u'ألشبهك',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِشِبْهُكِ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كِ'
}
STOPWORDS[u'ألشبهه'] = {
    'word': u'ألشبهه',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِشِبْهُه',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'ه'
}
STOPWORDS[u'ألشبهكم'] = {
    'word': u'ألشبهكم',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِشِبْهُكُمْ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ألشبهكن'] = {
    'word': u'ألشبهكن',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِشِبْهُكُنَّ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ألشبهها'] = {
    'word': u'ألشبهها',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِشِبْهُهَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هَا'
}
STOPWORDS[u'ألشبههم'] = {
    'word': u'ألشبههم',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِشِبْهُهُمْ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ألشبههن'] = {
    'word': u'ألشبههن',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِشِبْهُهُنَّ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ألشبهنا'] = {
    'word': u'ألشبهنا',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِشِبْهُنَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'نَا'
}
STOPWORDS[u'ألشبهكما'] = {
    'word': u'ألشبهكما',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِشِبْهُكُمَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ألشبههما'] = {
    'word': u'ألشبههما',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِشِبْهُهُمَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ألشبهي'] = {
    'word': u'ألشبهي',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَشِبْهُِي',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'ِي'
}
STOPWORDS[u'ألشبهك'] = {
    'word': u'ألشبهك',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَشِبْهُكَ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كَ'
}
STOPWORDS[u'ألشبهك'] = {
    'word': u'ألشبهك',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَشِبْهُكِ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كِ'
}
STOPWORDS[u'ألشبهه'] = {
    'word': u'ألشبهه',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَشِبْهُه',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'ه'
}
STOPWORDS[u'ألشبهكم'] = {
    'word': u'ألشبهكم',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَشِبْهُكُمْ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ألشبهكن'] = {
    'word': u'ألشبهكن',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَشِبْهُكُنَّ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ألشبهها'] = {
    'word': u'ألشبهها',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَشِبْهُهَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هَا'
}
STOPWORDS[u'ألشبههم'] = {
    'word': u'ألشبههم',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَشِبْهُهُمْ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ألشبههن'] = {
    'word': u'ألشبههن',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَشِبْهُهُنَّ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ألشبهنا'] = {
    'word': u'ألشبهنا',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَشِبْهُنَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'نَا'
}
STOPWORDS[u'ألشبهكما'] = {
    'word': u'ألشبهكما',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَشِبْهُكُمَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ألشبههما'] = {
    'word': u'ألشبههما',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَشِبْهُهُمَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أوشبهي'] = {
    'word': u'أوشبهي',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَشِبْهُِي',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'ِي'
}
STOPWORDS[u'أوشبهك'] = {
    'word': u'أوشبهك',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَشِبْهُكَ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كَ'
}
STOPWORDS[u'أوشبهك'] = {
    'word': u'أوشبهك',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَشِبْهُكِ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كِ'
}
STOPWORDS[u'أوشبهه'] = {
    'word': u'أوشبهه',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَشِبْهُه',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'ه'
}
STOPWORDS[u'أوشبهكم'] = {
    'word': u'أوشبهكم',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَشِبْهُكُمْ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أوشبهكن'] = {
    'word': u'أوشبهكن',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَشِبْهُكُنَّ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أوشبهها'] = {
    'word': u'أوشبهها',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَشِبْهُهَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هَا'
}
STOPWORDS[u'أوشبههم'] = {
    'word': u'أوشبههم',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَشِبْهُهُمْ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أوشبههن'] = {
    'word': u'أوشبههن',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَشِبْهُهُنَّ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أوشبهنا'] = {
    'word': u'أوشبهنا',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَشِبْهُنَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'نَا'
}
STOPWORDS[u'أوشبهكما'] = {
    'word': u'أوشبهكما',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَشِبْهُكُمَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أوشبههما'] = {
    'word': u'أوشبههما',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَشِبْهُهُمَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أفشبهي'] = {
    'word': u'أفشبهي',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَشِبْهُِي',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'ِي'
}
STOPWORDS[u'أفشبهك'] = {
    'word': u'أفشبهك',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَشِبْهُكَ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كَ'
}
STOPWORDS[u'أفشبهك'] = {
    'word': u'أفشبهك',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَشِبْهُكِ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كِ'
}
STOPWORDS[u'أفشبهه'] = {
    'word': u'أفشبهه',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَشِبْهُه',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'ه'
}
STOPWORDS[u'أفشبهكم'] = {
    'word': u'أفشبهكم',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَشِبْهُكُمْ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أفشبهكن'] = {
    'word': u'أفشبهكن',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَشِبْهُكُنَّ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أفشبهها'] = {
    'word': u'أفشبهها',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَشِبْهُهَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هَا'
}
STOPWORDS[u'أفشبههم'] = {
    'word': u'أفشبههم',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَشِبْهُهُمْ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أفشبههن'] = {
    'word': u'أفشبههن',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَشِبْهُهُنَّ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أفشبهنا'] = {
    'word': u'أفشبهنا',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَشِبْهُنَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'نَا'
}
STOPWORDS[u'أفشبهكما'] = {
    'word': u'أفشبهكما',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَشِبْهُكُمَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أفشبههما'] = {
    'word': u'أفشبههما',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَشِبْهُهُمَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أوبشبهي'] = {
    'word': u'أوبشبهي',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِشِبْهُِي',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'ِي'
}
STOPWORDS[u'أوبشبهك'] = {
    'word': u'أوبشبهك',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِشِبْهُكَ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كَ'
}
STOPWORDS[u'أوبشبهك'] = {
    'word': u'أوبشبهك',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِشِبْهُكِ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كِ'
}
STOPWORDS[u'أوبشبهه'] = {
    'word': u'أوبشبهه',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِشِبْهُه',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'ه'
}
STOPWORDS[u'أوبشبهكم'] = {
    'word': u'أوبشبهكم',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِشِبْهُكُمْ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أوبشبهكن'] = {
    'word': u'أوبشبهكن',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِشِبْهُكُنَّ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أوبشبهها'] = {
    'word': u'أوبشبهها',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِشِبْهُهَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هَا'
}
STOPWORDS[u'أوبشبههم'] = {
    'word': u'أوبشبههم',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِشِبْهُهُمْ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أوبشبههن'] = {
    'word': u'أوبشبههن',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِشِبْهُهُنَّ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أوبشبهنا'] = {
    'word': u'أوبشبهنا',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِشِبْهُنَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'نَا'
}
STOPWORDS[u'أوبشبهكما'] = {
    'word': u'أوبشبهكما',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِشِبْهُكُمَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أوبشبههما'] = {
    'word': u'أوبشبههما',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِشِبْهُهُمَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أفبشبهي'] = {
    'word': u'أفبشبهي',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِشِبْهُِي',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'ِي'
}
STOPWORDS[u'أفبشبهك'] = {
    'word': u'أفبشبهك',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِشِبْهُكَ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كَ'
}
STOPWORDS[u'أفبشبهك'] = {
    'word': u'أفبشبهك',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِشِبْهُكِ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كِ'
}
STOPWORDS[u'أفبشبهه'] = {
    'word': u'أفبشبهه',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِشِبْهُه',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'ه'
}
STOPWORDS[u'أفبشبهكم'] = {
    'word': u'أفبشبهكم',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِشِبْهُكُمْ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أفبشبهكن'] = {
    'word': u'أفبشبهكن',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِشِبْهُكُنَّ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أفبشبهها'] = {
    'word': u'أفبشبهها',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِشِبْهُهَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هَا'
}
STOPWORDS[u'أفبشبههم'] = {
    'word': u'أفبشبههم',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِشِبْهُهُمْ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أفبشبههن'] = {
    'word': u'أفبشبههن',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِشِبْهُهُنَّ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أفبشبهنا'] = {
    'word': u'أفبشبهنا',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِشِبْهُنَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'نَا'
}
STOPWORDS[u'أفبشبهكما'] = {
    'word': u'أفبشبهكما',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِشِبْهُكُمَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أفبشبههما'] = {
    'word': u'أفبشبههما',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِشِبْهُهُمَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أوكشبهي'] = {
    'word': u'أوكشبهي',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَشِبْهُِي',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'ِي'
}
STOPWORDS[u'أوكشبهك'] = {
    'word': u'أوكشبهك',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَشِبْهُكَ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كَ'
}
STOPWORDS[u'أوكشبهك'] = {
    'word': u'أوكشبهك',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَشِبْهُكِ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كِ'
}
STOPWORDS[u'أوكشبهه'] = {
    'word': u'أوكشبهه',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَشِبْهُه',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'ه'
}
STOPWORDS[u'أوكشبهكم'] = {
    'word': u'أوكشبهكم',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَشِبْهُكُمْ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أوكشبهكن'] = {
    'word': u'أوكشبهكن',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَشِبْهُكُنَّ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أوكشبهها'] = {
    'word': u'أوكشبهها',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَشِبْهُهَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هَا'
}
STOPWORDS[u'أوكشبههم'] = {
    'word': u'أوكشبههم',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَشِبْهُهُمْ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أوكشبههن'] = {
    'word': u'أوكشبههن',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَشِبْهُهُنَّ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أوكشبهنا'] = {
    'word': u'أوكشبهنا',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَشِبْهُنَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'نَا'
}
STOPWORDS[u'أوكشبهكما'] = {
    'word': u'أوكشبهكما',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَشِبْهُكُمَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أوكشبههما'] = {
    'word': u'أوكشبههما',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَشِبْهُهُمَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أفكشبهي'] = {
    'word': u'أفكشبهي',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَشِبْهُِي',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'ِي'
}
STOPWORDS[u'أفكشبهك'] = {
    'word': u'أفكشبهك',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَشِبْهُكَ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كَ'
}
STOPWORDS[u'أفكشبهك'] = {
    'word': u'أفكشبهك',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَشِبْهُكِ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كِ'
}
STOPWORDS[u'أفكشبهه'] = {
    'word': u'أفكشبهه',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَشِبْهُه',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'ه'
}
STOPWORDS[u'أفكشبهكم'] = {
    'word': u'أفكشبهكم',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَشِبْهُكُمْ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أفكشبهكن'] = {
    'word': u'أفكشبهكن',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَشِبْهُكُنَّ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أفكشبهها'] = {
    'word': u'أفكشبهها',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَشِبْهُهَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هَا'
}
STOPWORDS[u'أفكشبههم'] = {
    'word': u'أفكشبههم',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَشِبْهُهُمْ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أفكشبههن'] = {
    'word': u'أفكشبههن',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَشِبْهُهُنَّ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أفكشبهنا'] = {
    'word': u'أفكشبهنا',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَشِبْهُنَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'نَا'
}
STOPWORDS[u'أفكشبهكما'] = {
    'word': u'أفكشبهكما',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَشِبْهُكُمَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أفكشبههما'] = {
    'word': u'أفكشبههما',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَشِبْهُهُمَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أولشبهي'] = {
    'word': u'أولشبهي',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِشِبْهُِي',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'ِي'
}
STOPWORDS[u'أولشبهك'] = {
    'word': u'أولشبهك',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِشِبْهُكَ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كَ'
}
STOPWORDS[u'أولشبهك'] = {
    'word': u'أولشبهك',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِشِبْهُكِ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كِ'
}
STOPWORDS[u'أولشبهه'] = {
    'word': u'أولشبهه',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِشِبْهُه',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'ه'
}
STOPWORDS[u'أولشبهكم'] = {
    'word': u'أولشبهكم',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِشِبْهُكُمْ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أولشبهكن'] = {
    'word': u'أولشبهكن',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِشِبْهُكُنَّ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أولشبهها'] = {
    'word': u'أولشبهها',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِشِبْهُهَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هَا'
}
STOPWORDS[u'أولشبههم'] = {
    'word': u'أولشبههم',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِشِبْهُهُمْ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أولشبههن'] = {
    'word': u'أولشبههن',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِشِبْهُهُنَّ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أولشبهنا'] = {
    'word': u'أولشبهنا',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِشِبْهُنَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'نَا'
}
STOPWORDS[u'أولشبهكما'] = {
    'word': u'أولشبهكما',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِشِبْهُكُمَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أولشبههما'] = {
    'word': u'أولشبههما',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِشِبْهُهُمَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أفلشبهي'] = {
    'word': u'أفلشبهي',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِشِبْهُِي',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'ِي'
}
STOPWORDS[u'أفلشبهك'] = {
    'word': u'أفلشبهك',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِشِبْهُكَ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كَ'
}
STOPWORDS[u'أفلشبهك'] = {
    'word': u'أفلشبهك',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِشِبْهُكِ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كِ'
}
STOPWORDS[u'أفلشبهه'] = {
    'word': u'أفلشبهه',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِشِبْهُه',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'ه'
}
STOPWORDS[u'أفلشبهكم'] = {
    'word': u'أفلشبهكم',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِشِبْهُكُمْ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أفلشبهكن'] = {
    'word': u'أفلشبهكن',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِشِبْهُكُنَّ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أفلشبهها'] = {
    'word': u'أفلشبهها',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِشِبْهُهَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هَا'
}
STOPWORDS[u'أفلشبههم'] = {
    'word': u'أفلشبههم',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِشِبْهُهُمْ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أفلشبههن'] = {
    'word': u'أفلشبههن',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِشِبْهُهُنَّ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أفلشبهنا'] = {
    'word': u'أفلشبهنا',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِشِبْهُنَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'نَا'
}
STOPWORDS[u'أفلشبهكما'] = {
    'word': u'أفلشبهكما',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِشِبْهُكُمَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أفلشبههما'] = {
    'word': u'أفلشبههما',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِشِبْهُهُمَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أولشبهي'] = {
    'word': u'أولشبهي',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَشِبْهُِي',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'ِي'
}
STOPWORDS[u'أولشبهك'] = {
    'word': u'أولشبهك',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَشِبْهُكَ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كَ'
}
STOPWORDS[u'أولشبهك'] = {
    'word': u'أولشبهك',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَشِبْهُكِ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كِ'
}
STOPWORDS[u'أولشبهه'] = {
    'word': u'أولشبهه',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَشِبْهُه',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'ه'
}
STOPWORDS[u'أولشبهكم'] = {
    'word': u'أولشبهكم',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَشِبْهُكُمْ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أولشبهكن'] = {
    'word': u'أولشبهكن',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَشِبْهُكُنَّ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أولشبهها'] = {
    'word': u'أولشبهها',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَشِبْهُهَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هَا'
}
STOPWORDS[u'أولشبههم'] = {
    'word': u'أولشبههم',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَشِبْهُهُمْ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أولشبههن'] = {
    'word': u'أولشبههن',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَشِبْهُهُنَّ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أولشبهنا'] = {
    'word': u'أولشبهنا',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَشِبْهُنَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'نَا'
}
STOPWORDS[u'أولشبهكما'] = {
    'word': u'أولشبهكما',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَشِبْهُكُمَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أولشبههما'] = {
    'word': u'أولشبههما',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَشِبْهُهُمَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أفلشبهي'] = {
    'word': u'أفلشبهي',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَشِبْهُِي',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'ِي'
}
STOPWORDS[u'أفلشبهك'] = {
    'word': u'أفلشبهك',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَشِبْهُكَ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كَ'
}
STOPWORDS[u'أفلشبهك'] = {
    'word': u'أفلشبهك',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَشِبْهُكِ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كِ'
}
STOPWORDS[u'أفلشبهه'] = {
    'word': u'أفلشبهه',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَشِبْهُه',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'ه'
}
STOPWORDS[u'أفلشبهكم'] = {
    'word': u'أفلشبهكم',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَشِبْهُكُمْ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أفلشبهكن'] = {
    'word': u'أفلشبهكن',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَشِبْهُكُنَّ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أفلشبهها'] = {
    'word': u'أفلشبهها',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَشِبْهُهَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هَا'
}
STOPWORDS[u'أفلشبههم'] = {
    'word': u'أفلشبههم',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَشِبْهُهُمْ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أفلشبههن'] = {
    'word': u'أفلشبههن',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَشِبْهُهُنَّ',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أفلشبهنا'] = {
    'word': u'أفلشبهنا',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَشِبْهُنَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'نَا'
}
STOPWORDS[u'أفلشبهكما'] = {
    'word': u'أفلشبهكما',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَشِبْهُكُمَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أفلشبههما'] = {
    'word': u'أفلشبههما',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَشِبْهُهُمَا',
    'stem': u'شِبْهُ',
    'type': u'STOPWORD',
    'original': u'شِبْهُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'كل'] = {
    'word': u'كُلُّ',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة',
    'vocalized': u'كُلُّ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u''
}
STOPWORDS[u'بكل'] = {
    'word': u'بكل',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور',
    'vocalized': u'بِكُلُّ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u''
}
STOPWORDS[u'ككل'] = {
    'word': u'ككل',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور',
    'vocalized': u'كَكُلُّ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u''
}
STOPWORDS[u'لكل'] = {
    'word': u'لكل',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور',
    'vocalized': u'لِكُلُّ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u''
}
STOPWORDS[u'لكل'] = {
    'word': u'لكل',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور',
    'vocalized': u'لَكُلُّ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u''
}
STOPWORDS[u'وكل'] = {
    'word': u'وكل',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف',
    'vocalized': u'وَكُلُّ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u''
}
STOPWORDS[u'فكل'] = {
    'word': u'فكل',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف',
    'vocalized': u'فَكُلُّ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u''
}
STOPWORDS[u'وبكل'] = {
    'word': u'وبكل',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف',
    'vocalized': u'وَبِكُلُّ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u''
}
STOPWORDS[u'فبكل'] = {
    'word': u'فبكل',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف',
    'vocalized': u'فَبِكُلُّ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u''
}
STOPWORDS[u'وككل'] = {
    'word': u'وككل',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف',
    'vocalized': u'وَكَكُلُّ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u''
}
STOPWORDS[u'فككل'] = {
    'word': u'فككل',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف',
    'vocalized': u'فَكَكُلُّ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u''
}
STOPWORDS[u'ولكل'] = {
    'word': u'ولكل',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف',
    'vocalized': u'وَلِكُلُّ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u''
}
STOPWORDS[u'فلكل'] = {
    'word': u'فلكل',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف',
    'vocalized': u'فَلِكُلُّ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u''
}
STOPWORDS[u'ولكل'] = {
    'word': u'ولكل',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'وَلَكُلُّ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u''
}
STOPWORDS[u'فلكل'] = {
    'word': u'فلكل',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'فَلَكُلُّ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u''
}
STOPWORDS[u'أكل'] = {
    'word': u'أكل',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور',
    'vocalized': u'أَكُلُّ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u''
}
STOPWORDS[u'أبكل'] = {
    'word': u'أبكل',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور',
    'vocalized': u'أَبِكُلُّ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u''
}
STOPWORDS[u'أككل'] = {
    'word': u'أككل',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور',
    'vocalized': u'أَكَكُلُّ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u''
}
STOPWORDS[u'ألكل'] = {
    'word': u'ألكل',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور',
    'vocalized': u'أَلِكُلُّ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u''
}
STOPWORDS[u'ألكل'] = {
    'word': u'ألكل',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور',
    'vocalized': u'أَلَكُلُّ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u''
}
STOPWORDS[u'أوكل'] = {
    'word': u'أوكل',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور',
    'vocalized': u'أَوَكُلُّ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u''
}
STOPWORDS[u'أفكل'] = {
    'word': u'أفكل',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور',
    'vocalized': u'أَفَكُلُّ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u''
}
STOPWORDS[u'أوبكل'] = {
    'word': u'أوبكل',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَبِكُلُّ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u''
}
STOPWORDS[u'أفبكل'] = {
    'word': u'أفبكل',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَبِكُلُّ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u''
}
STOPWORDS[u'أوككل'] = {
    'word': u'أوككل',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَكَكُلُّ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u''
}
STOPWORDS[u'أفككل'] = {
    'word': u'أفككل',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَكَكُلُّ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u''
}
STOPWORDS[u'أولكل'] = {
    'word': u'أولكل',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَلِكُلُّ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u''
}
STOPWORDS[u'أفلكل'] = {
    'word': u'أفلكل',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَلِكُلُّ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u''
}
STOPWORDS[u'أولكل'] = {
    'word': u'أولكل',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَلَكُلُّ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u''
}
STOPWORDS[u'أفلكل'] = {
    'word': u'أفلكل',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَلَكُلُّ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u''
}
STOPWORDS[u'كلي'] = {
    'word': u'كلي',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'كُلُِّي',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'ِي'
}
STOPWORDS[u'كلك'] = {
    'word': u'كلك',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'كُلُّكَ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كَ'
}
STOPWORDS[u'كلك'] = {
    'word': u'كلك',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'كُلُّكِ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كِ'
}
STOPWORDS[u'كله'] = {
    'word': u'كله',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'كُلُّه',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'ه'
}
STOPWORDS[u'كلكم'] = {
    'word': u'كلكم',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'كُلُّكُمْ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'كلكن'] = {
    'word': u'كلكن',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'كُلُّكُنَّ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'كلها'] = {
    'word': u'كلها',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'كُلُّهَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هَا'
}
STOPWORDS[u'كلهم'] = {
    'word': u'كلهم',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'كُلُّهُمْ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'كلهن'] = {
    'word': u'كلهن',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'كُلُّهُنَّ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'كلنا'] = {
    'word': u'كلنا',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'كُلُّنَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'نَا'
}
STOPWORDS[u'كلكما'] = {
    'word': u'كلكما',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'كُلُّكُمَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'كلهما'] = {
    'word': u'كلهما',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'كُلُّهُمَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'بكلي'] = {
    'word': u'بكلي',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِكُلُِّي',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'ِي'
}
STOPWORDS[u'بكلك'] = {
    'word': u'بكلك',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِكُلُّكَ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كَ'
}
STOPWORDS[u'بكلك'] = {
    'word': u'بكلك',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِكُلُّكِ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كِ'
}
STOPWORDS[u'بكله'] = {
    'word': u'بكله',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِكُلُّه',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'ه'
}
STOPWORDS[u'بكلكم'] = {
    'word': u'بكلكم',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِكُلُّكُمْ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'بكلكن'] = {
    'word': u'بكلكن',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِكُلُّكُنَّ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'بكلها'] = {
    'word': u'بكلها',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِكُلُّهَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هَا'
}
STOPWORDS[u'بكلهم'] = {
    'word': u'بكلهم',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِكُلُّهُمْ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'بكلهن'] = {
    'word': u'بكلهن',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِكُلُّهُنَّ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'بكلنا'] = {
    'word': u'بكلنا',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِكُلُّنَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'نَا'
}
STOPWORDS[u'بكلكما'] = {
    'word': u'بكلكما',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِكُلُّكُمَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'بكلهما'] = {
    'word': u'بكلهما',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِكُلُّهُمَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ككلي'] = {
    'word': u'ككلي',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَكُلُِّي',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'ِي'
}
STOPWORDS[u'ككلك'] = {
    'word': u'ككلك',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَكُلُّكَ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كَ'
}
STOPWORDS[u'ككلك'] = {
    'word': u'ككلك',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَكُلُّكِ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كِ'
}
STOPWORDS[u'ككله'] = {
    'word': u'ككله',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَكُلُّه',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'ه'
}
STOPWORDS[u'ككلكم'] = {
    'word': u'ككلكم',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَكُلُّكُمْ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ككلكن'] = {
    'word': u'ككلكن',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَكُلُّكُنَّ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ككلها'] = {
    'word': u'ككلها',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَكُلُّهَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هَا'
}
STOPWORDS[u'ككلهم'] = {
    'word': u'ككلهم',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَكُلُّهُمْ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ككلهن'] = {
    'word': u'ككلهن',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَكُلُّهُنَّ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ككلنا'] = {
    'word': u'ككلنا',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَكُلُّنَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'نَا'
}
STOPWORDS[u'ككلكما'] = {
    'word': u'ككلكما',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَكُلُّكُمَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ككلهما'] = {
    'word': u'ككلهما',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَكُلُّهُمَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'لكلي'] = {
    'word': u'لكلي',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِكُلُِّي',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'ِي'
}
STOPWORDS[u'لكلك'] = {
    'word': u'لكلك',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِكُلُّكَ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كَ'
}
STOPWORDS[u'لكلك'] = {
    'word': u'لكلك',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِكُلُّكِ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كِ'
}
STOPWORDS[u'لكله'] = {
    'word': u'لكله',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِكُلُّه',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'ه'
}
STOPWORDS[u'لكلكم'] = {
    'word': u'لكلكم',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِكُلُّكُمْ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'لكلكن'] = {
    'word': u'لكلكن',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِكُلُّكُنَّ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'لكلها'] = {
    'word': u'لكلها',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِكُلُّهَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هَا'
}
STOPWORDS[u'لكلهم'] = {
    'word': u'لكلهم',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِكُلُّهُمْ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'لكلهن'] = {
    'word': u'لكلهن',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِكُلُّهُنَّ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'لكلنا'] = {
    'word': u'لكلنا',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِكُلُّنَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'نَا'
}
STOPWORDS[u'لكلكما'] = {
    'word': u'لكلكما',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِكُلُّكُمَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'لكلهما'] = {
    'word': u'لكلهما',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِكُلُّهُمَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'لكلي'] = {
    'word': u'لكلي',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَكُلُِّي',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'ِي'
}
STOPWORDS[u'لكلك'] = {
    'word': u'لكلك',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَكُلُّكَ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كَ'
}
STOPWORDS[u'لكلك'] = {
    'word': u'لكلك',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَكُلُّكِ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كِ'
}
STOPWORDS[u'لكله'] = {
    'word': u'لكله',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَكُلُّه',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'ه'
}
STOPWORDS[u'لكلكم'] = {
    'word': u'لكلكم',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَكُلُّكُمْ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'لكلكن'] = {
    'word': u'لكلكن',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَكُلُّكُنَّ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'لكلها'] = {
    'word': u'لكلها',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَكُلُّهَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هَا'
}
STOPWORDS[u'لكلهم'] = {
    'word': u'لكلهم',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَكُلُّهُمْ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'لكلهن'] = {
    'word': u'لكلهن',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَكُلُّهُنَّ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'لكلنا'] = {
    'word': u'لكلنا',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَكُلُّنَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'نَا'
}
STOPWORDS[u'لكلكما'] = {
    'word': u'لكلكما',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَكُلُّكُمَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'لكلهما'] = {
    'word': u'لكلهما',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَكُلُّهُمَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'وكلي'] = {
    'word': u'وكلي',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَكُلُِّي',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'ِي'
}
STOPWORDS[u'وكلك'] = {
    'word': u'وكلك',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَكُلُّكَ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كَ'
}
STOPWORDS[u'وكلك'] = {
    'word': u'وكلك',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَكُلُّكِ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كِ'
}
STOPWORDS[u'وكله'] = {
    'word': u'وكله',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَكُلُّه',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'ه'
}
STOPWORDS[u'وكلكم'] = {
    'word': u'وكلكم',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَكُلُّكُمْ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'وكلكن'] = {
    'word': u'وكلكن',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَكُلُّكُنَّ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'وكلها'] = {
    'word': u'وكلها',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَكُلُّهَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هَا'
}
STOPWORDS[u'وكلهم'] = {
    'word': u'وكلهم',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَكُلُّهُمْ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'وكلهن'] = {
    'word': u'وكلهن',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَكُلُّهُنَّ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'وكلنا'] = {
    'word': u'وكلنا',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَكُلُّنَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'نَا'
}
STOPWORDS[u'وكلكما'] = {
    'word': u'وكلكما',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَكُلُّكُمَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'وكلهما'] = {
    'word': u'وكلهما',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَكُلُّهُمَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فكلي'] = {
    'word': u'فكلي',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَكُلُِّي',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'ِي'
}
STOPWORDS[u'فكلك'] = {
    'word': u'فكلك',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَكُلُّكَ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كَ'
}
STOPWORDS[u'فكلك'] = {
    'word': u'فكلك',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَكُلُّكِ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كِ'
}
STOPWORDS[u'فكله'] = {
    'word': u'فكله',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَكُلُّه',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'ه'
}
STOPWORDS[u'فكلكم'] = {
    'word': u'فكلكم',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَكُلُّكُمْ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فكلكن'] = {
    'word': u'فكلكن',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَكُلُّكُنَّ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فكلها'] = {
    'word': u'فكلها',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَكُلُّهَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هَا'
}
STOPWORDS[u'فكلهم'] = {
    'word': u'فكلهم',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَكُلُّهُمْ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فكلهن'] = {
    'word': u'فكلهن',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَكُلُّهُنَّ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فكلنا'] = {
    'word': u'فكلنا',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَكُلُّنَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'نَا'
}
STOPWORDS[u'فكلكما'] = {
    'word': u'فكلكما',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَكُلُّكُمَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فكلهما'] = {
    'word': u'فكلهما',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَكُلُّهُمَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'وبكلي'] = {
    'word': u'وبكلي',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِكُلُِّي',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'ِي'
}
STOPWORDS[u'وبكلك'] = {
    'word': u'وبكلك',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِكُلُّكَ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كَ'
}
STOPWORDS[u'وبكلك'] = {
    'word': u'وبكلك',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِكُلُّكِ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كِ'
}
STOPWORDS[u'وبكله'] = {
    'word': u'وبكله',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِكُلُّه',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'ه'
}
STOPWORDS[u'وبكلكم'] = {
    'word': u'وبكلكم',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِكُلُّكُمْ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'وبكلكن'] = {
    'word': u'وبكلكن',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِكُلُّكُنَّ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'وبكلها'] = {
    'word': u'وبكلها',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِكُلُّهَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هَا'
}
STOPWORDS[u'وبكلهم'] = {
    'word': u'وبكلهم',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِكُلُّهُمْ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'وبكلهن'] = {
    'word': u'وبكلهن',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِكُلُّهُنَّ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'وبكلنا'] = {
    'word': u'وبكلنا',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِكُلُّنَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'نَا'
}
STOPWORDS[u'وبكلكما'] = {
    'word': u'وبكلكما',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِكُلُّكُمَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'وبكلهما'] = {
    'word': u'وبكلهما',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِكُلُّهُمَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فبكلي'] = {
    'word': u'فبكلي',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِكُلُِّي',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'ِي'
}
STOPWORDS[u'فبكلك'] = {
    'word': u'فبكلك',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِكُلُّكَ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كَ'
}
STOPWORDS[u'فبكلك'] = {
    'word': u'فبكلك',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِكُلُّكِ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كِ'
}
STOPWORDS[u'فبكله'] = {
    'word': u'فبكله',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِكُلُّه',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'ه'
}
STOPWORDS[u'فبكلكم'] = {
    'word': u'فبكلكم',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِكُلُّكُمْ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فبكلكن'] = {
    'word': u'فبكلكن',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِكُلُّكُنَّ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فبكلها'] = {
    'word': u'فبكلها',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِكُلُّهَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هَا'
}
STOPWORDS[u'فبكلهم'] = {
    'word': u'فبكلهم',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِكُلُّهُمْ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فبكلهن'] = {
    'word': u'فبكلهن',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِكُلُّهُنَّ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فبكلنا'] = {
    'word': u'فبكلنا',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِكُلُّنَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'نَا'
}
STOPWORDS[u'فبكلكما'] = {
    'word': u'فبكلكما',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِكُلُّكُمَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فبكلهما'] = {
    'word': u'فبكلهما',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِكُلُّهُمَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'وككلي'] = {
    'word': u'وككلي',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَكُلُِّي',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'ِي'
}
STOPWORDS[u'وككلك'] = {
    'word': u'وككلك',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَكُلُّكَ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كَ'
}
STOPWORDS[u'وككلك'] = {
    'word': u'وككلك',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَكُلُّكِ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كِ'
}
STOPWORDS[u'وككله'] = {
    'word': u'وككله',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَكُلُّه',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'ه'
}
STOPWORDS[u'وككلكم'] = {
    'word': u'وككلكم',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَكُلُّكُمْ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'وككلكن'] = {
    'word': u'وككلكن',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَكُلُّكُنَّ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'وككلها'] = {
    'word': u'وككلها',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَكُلُّهَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هَا'
}
STOPWORDS[u'وككلهم'] = {
    'word': u'وككلهم',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَكُلُّهُمْ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'وككلهن'] = {
    'word': u'وككلهن',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَكُلُّهُنَّ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'وككلنا'] = {
    'word': u'وككلنا',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَكُلُّنَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'نَا'
}
STOPWORDS[u'وككلكما'] = {
    'word': u'وككلكما',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَكُلُّكُمَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'وككلهما'] = {
    'word': u'وككلهما',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَكُلُّهُمَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فككلي'] = {
    'word': u'فككلي',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَكُلُِّي',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'ِي'
}
STOPWORDS[u'فككلك'] = {
    'word': u'فككلك',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَكُلُّكَ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كَ'
}
STOPWORDS[u'فككلك'] = {
    'word': u'فككلك',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَكُلُّكِ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كِ'
}
STOPWORDS[u'فككله'] = {
    'word': u'فككله',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَكُلُّه',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'ه'
}
STOPWORDS[u'فككلكم'] = {
    'word': u'فككلكم',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَكُلُّكُمْ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فككلكن'] = {
    'word': u'فككلكن',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَكُلُّكُنَّ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فككلها'] = {
    'word': u'فككلها',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَكُلُّهَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هَا'
}
STOPWORDS[u'فككلهم'] = {
    'word': u'فككلهم',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَكُلُّهُمْ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فككلهن'] = {
    'word': u'فككلهن',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَكُلُّهُنَّ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فككلنا'] = {
    'word': u'فككلنا',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَكُلُّنَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'نَا'
}
STOPWORDS[u'فككلكما'] = {
    'word': u'فككلكما',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَكُلُّكُمَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فككلهما'] = {
    'word': u'فككلهما',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَكُلُّهُمَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ولكلي'] = {
    'word': u'ولكلي',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِكُلُِّي',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'ِي'
}
STOPWORDS[u'ولكلك'] = {
    'word': u'ولكلك',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِكُلُّكَ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كَ'
}
STOPWORDS[u'ولكلك'] = {
    'word': u'ولكلك',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِكُلُّكِ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كِ'
}
STOPWORDS[u'ولكله'] = {
    'word': u'ولكله',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِكُلُّه',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'ه'
}
STOPWORDS[u'ولكلكم'] = {
    'word': u'ولكلكم',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِكُلُّكُمْ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ولكلكن'] = {
    'word': u'ولكلكن',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِكُلُّكُنَّ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ولكلها'] = {
    'word': u'ولكلها',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِكُلُّهَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هَا'
}
STOPWORDS[u'ولكلهم'] = {
    'word': u'ولكلهم',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِكُلُّهُمْ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ولكلهن'] = {
    'word': u'ولكلهن',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِكُلُّهُنَّ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ولكلنا'] = {
    'word': u'ولكلنا',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِكُلُّنَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'نَا'
}
STOPWORDS[u'ولكلكما'] = {
    'word': u'ولكلكما',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِكُلُّكُمَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ولكلهما'] = {
    'word': u'ولكلهما',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِكُلُّهُمَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فلكلي'] = {
    'word': u'فلكلي',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِكُلُِّي',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'ِي'
}
STOPWORDS[u'فلكلك'] = {
    'word': u'فلكلك',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِكُلُّكَ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كَ'
}
STOPWORDS[u'فلكلك'] = {
    'word': u'فلكلك',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِكُلُّكِ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كِ'
}
STOPWORDS[u'فلكله'] = {
    'word': u'فلكله',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِكُلُّه',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'ه'
}
STOPWORDS[u'فلكلكم'] = {
    'word': u'فلكلكم',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِكُلُّكُمْ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فلكلكن'] = {
    'word': u'فلكلكن',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِكُلُّكُنَّ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فلكلها'] = {
    'word': u'فلكلها',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِكُلُّهَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هَا'
}
STOPWORDS[u'فلكلهم'] = {
    'word': u'فلكلهم',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِكُلُّهُمْ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فلكلهن'] = {
    'word': u'فلكلهن',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِكُلُّهُنَّ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فلكلنا'] = {
    'word': u'فلكلنا',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِكُلُّنَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'نَا'
}
STOPWORDS[u'فلكلكما'] = {
    'word': u'فلكلكما',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِكُلُّكُمَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فلكلهما'] = {
    'word': u'فلكلهما',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِكُلُّهُمَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ولكلي'] = {
    'word': u'ولكلي',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَكُلُِّي',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'ِي'
}
STOPWORDS[u'ولكلك'] = {
    'word': u'ولكلك',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَكُلُّكَ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كَ'
}
STOPWORDS[u'ولكلك'] = {
    'word': u'ولكلك',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَكُلُّكِ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كِ'
}
STOPWORDS[u'ولكله'] = {
    'word': u'ولكله',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَكُلُّه',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'ه'
}
STOPWORDS[u'ولكلكم'] = {
    'word': u'ولكلكم',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَكُلُّكُمْ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ولكلكن'] = {
    'word': u'ولكلكن',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَكُلُّكُنَّ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ولكلها'] = {
    'word': u'ولكلها',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَكُلُّهَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هَا'
}
STOPWORDS[u'ولكلهم'] = {
    'word': u'ولكلهم',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَكُلُّهُمْ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ولكلهن'] = {
    'word': u'ولكلهن',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَكُلُّهُنَّ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ولكلنا'] = {
    'word': u'ولكلنا',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَكُلُّنَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'نَا'
}
STOPWORDS[u'ولكلكما'] = {
    'word': u'ولكلكما',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَكُلُّكُمَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ولكلهما'] = {
    'word': u'ولكلهما',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَكُلُّهُمَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فلكلي'] = {
    'word': u'فلكلي',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَكُلُِّي',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'ِي'
}
STOPWORDS[u'فلكلك'] = {
    'word': u'فلكلك',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَكُلُّكَ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كَ'
}
STOPWORDS[u'فلكلك'] = {
    'word': u'فلكلك',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَكُلُّكِ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كِ'
}
STOPWORDS[u'فلكله'] = {
    'word': u'فلكله',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَكُلُّه',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'ه'
}
STOPWORDS[u'فلكلكم'] = {
    'word': u'فلكلكم',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَكُلُّكُمْ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فلكلكن'] = {
    'word': u'فلكلكن',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَكُلُّكُنَّ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فلكلها'] = {
    'word': u'فلكلها',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَكُلُّهَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هَا'
}
STOPWORDS[u'فلكلهم'] = {
    'word': u'فلكلهم',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَكُلُّهُمْ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فلكلهن'] = {
    'word': u'فلكلهن',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَكُلُّهُنَّ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فلكلنا'] = {
    'word': u'فلكلنا',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَكُلُّنَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'نَا'
}
STOPWORDS[u'فلكلكما'] = {
    'word': u'فلكلكما',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَكُلُّكُمَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فلكلهما'] = {
    'word': u'فلكلهما',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَكُلُّهُمَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أكلي'] = {
    'word': u'أكلي',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'أَكُلُِّي',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'ِي'
}
STOPWORDS[u'أكلك'] = {
    'word': u'أكلك',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'أَكُلُّكَ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كَ'
}
STOPWORDS[u'أكلك'] = {
    'word': u'أكلك',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'أَكُلُّكِ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كِ'
}
STOPWORDS[u'أكله'] = {
    'word': u'أكله',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'أَكُلُّه',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'ه'
}
STOPWORDS[u'أكلكم'] = {
    'word': u'أكلكم',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'أَكُلُّكُمْ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أكلكن'] = {
    'word': u'أكلكن',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'أَكُلُّكُنَّ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أكلها'] = {
    'word': u'أكلها',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'أَكُلُّهَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هَا'
}
STOPWORDS[u'أكلهم'] = {
    'word': u'أكلهم',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'أَكُلُّهُمْ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أكلهن'] = {
    'word': u'أكلهن',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'أَكُلُّهُنَّ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أكلنا'] = {
    'word': u'أكلنا',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'أَكُلُّنَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'نَا'
}
STOPWORDS[u'أكلكما'] = {
    'word': u'أكلكما',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'أَكُلُّكُمَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أكلهما'] = {
    'word': u'أكلهما',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'أَكُلُّهُمَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أبكلي'] = {
    'word': u'أبكلي',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِكُلُِّي',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'ِي'
}
STOPWORDS[u'أبكلك'] = {
    'word': u'أبكلك',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِكُلُّكَ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كَ'
}
STOPWORDS[u'أبكلك'] = {
    'word': u'أبكلك',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِكُلُّكِ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كِ'
}
STOPWORDS[u'أبكله'] = {
    'word': u'أبكله',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِكُلُّه',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'ه'
}
STOPWORDS[u'أبكلكم'] = {
    'word': u'أبكلكم',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِكُلُّكُمْ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أبكلكن'] = {
    'word': u'أبكلكن',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِكُلُّكُنَّ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أبكلها'] = {
    'word': u'أبكلها',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِكُلُّهَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هَا'
}
STOPWORDS[u'أبكلهم'] = {
    'word': u'أبكلهم',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِكُلُّهُمْ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أبكلهن'] = {
    'word': u'أبكلهن',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِكُلُّهُنَّ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أبكلنا'] = {
    'word': u'أبكلنا',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِكُلُّنَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'نَا'
}
STOPWORDS[u'أبكلكما'] = {
    'word': u'أبكلكما',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِكُلُّكُمَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أبكلهما'] = {
    'word': u'أبكلهما',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِكُلُّهُمَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أككلي'] = {
    'word': u'أككلي',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَكُلُِّي',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'ِي'
}
STOPWORDS[u'أككلك'] = {
    'word': u'أككلك',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَكُلُّكَ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كَ'
}
STOPWORDS[u'أككلك'] = {
    'word': u'أككلك',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَكُلُّكِ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كِ'
}
STOPWORDS[u'أككله'] = {
    'word': u'أككله',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَكُلُّه',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'ه'
}
STOPWORDS[u'أككلكم'] = {
    'word': u'أككلكم',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَكُلُّكُمْ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أككلكن'] = {
    'word': u'أككلكن',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَكُلُّكُنَّ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أككلها'] = {
    'word': u'أككلها',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَكُلُّهَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هَا'
}
STOPWORDS[u'أككلهم'] = {
    'word': u'أككلهم',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَكُلُّهُمْ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أككلهن'] = {
    'word': u'أككلهن',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَكُلُّهُنَّ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أككلنا'] = {
    'word': u'أككلنا',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَكُلُّنَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'نَا'
}
STOPWORDS[u'أككلكما'] = {
    'word': u'أككلكما',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَكُلُّكُمَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أككلهما'] = {
    'word': u'أككلهما',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَكُلُّهُمَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ألكلي'] = {
    'word': u'ألكلي',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِكُلُِّي',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'ِي'
}
STOPWORDS[u'ألكلك'] = {
    'word': u'ألكلك',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِكُلُّكَ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كَ'
}
STOPWORDS[u'ألكلك'] = {
    'word': u'ألكلك',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِكُلُّكِ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كِ'
}
STOPWORDS[u'ألكله'] = {
    'word': u'ألكله',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِكُلُّه',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'ه'
}
STOPWORDS[u'ألكلكم'] = {
    'word': u'ألكلكم',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِكُلُّكُمْ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ألكلكن'] = {
    'word': u'ألكلكن',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِكُلُّكُنَّ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ألكلها'] = {
    'word': u'ألكلها',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِكُلُّهَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هَا'
}
STOPWORDS[u'ألكلهم'] = {
    'word': u'ألكلهم',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِكُلُّهُمْ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ألكلهن'] = {
    'word': u'ألكلهن',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِكُلُّهُنَّ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ألكلنا'] = {
    'word': u'ألكلنا',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِكُلُّنَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'نَا'
}
STOPWORDS[u'ألكلكما'] = {
    'word': u'ألكلكما',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِكُلُّكُمَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ألكلهما'] = {
    'word': u'ألكلهما',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِكُلُّهُمَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ألكلي'] = {
    'word': u'ألكلي',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَكُلُِّي',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'ِي'
}
STOPWORDS[u'ألكلك'] = {
    'word': u'ألكلك',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَكُلُّكَ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كَ'
}
STOPWORDS[u'ألكلك'] = {
    'word': u'ألكلك',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَكُلُّكِ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كِ'
}
STOPWORDS[u'ألكله'] = {
    'word': u'ألكله',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَكُلُّه',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'ه'
}
STOPWORDS[u'ألكلكم'] = {
    'word': u'ألكلكم',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَكُلُّكُمْ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ألكلكن'] = {
    'word': u'ألكلكن',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَكُلُّكُنَّ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ألكلها'] = {
    'word': u'ألكلها',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَكُلُّهَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هَا'
}
STOPWORDS[u'ألكلهم'] = {
    'word': u'ألكلهم',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَكُلُّهُمْ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ألكلهن'] = {
    'word': u'ألكلهن',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَكُلُّهُنَّ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ألكلنا'] = {
    'word': u'ألكلنا',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَكُلُّنَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'نَا'
}
STOPWORDS[u'ألكلكما'] = {
    'word': u'ألكلكما',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَكُلُّكُمَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ألكلهما'] = {
    'word': u'ألكلهما',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَكُلُّهُمَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أوكلي'] = {
    'word': u'أوكلي',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكُلُِّي',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'ِي'
}
STOPWORDS[u'أوكلك'] = {
    'word': u'أوكلك',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكُلُّكَ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كَ'
}
STOPWORDS[u'أوكلك'] = {
    'word': u'أوكلك',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكُلُّكِ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كِ'
}
STOPWORDS[u'أوكله'] = {
    'word': u'أوكله',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكُلُّه',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'ه'
}
STOPWORDS[u'أوكلكم'] = {
    'word': u'أوكلكم',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكُلُّكُمْ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أوكلكن'] = {
    'word': u'أوكلكن',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكُلُّكُنَّ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أوكلها'] = {
    'word': u'أوكلها',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكُلُّهَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هَا'
}
STOPWORDS[u'أوكلهم'] = {
    'word': u'أوكلهم',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكُلُّهُمْ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أوكلهن'] = {
    'word': u'أوكلهن',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكُلُّهُنَّ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أوكلنا'] = {
    'word': u'أوكلنا',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكُلُّنَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'نَا'
}
STOPWORDS[u'أوكلكما'] = {
    'word': u'أوكلكما',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكُلُّكُمَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أوكلهما'] = {
    'word': u'أوكلهما',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكُلُّهُمَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أفكلي'] = {
    'word': u'أفكلي',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكُلُِّي',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'ِي'
}
STOPWORDS[u'أفكلك'] = {
    'word': u'أفكلك',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكُلُّكَ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كَ'
}
STOPWORDS[u'أفكلك'] = {
    'word': u'أفكلك',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكُلُّكِ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كِ'
}
STOPWORDS[u'أفكله'] = {
    'word': u'أفكله',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكُلُّه',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'ه'
}
STOPWORDS[u'أفكلكم'] = {
    'word': u'أفكلكم',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكُلُّكُمْ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أفكلكن'] = {
    'word': u'أفكلكن',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكُلُّكُنَّ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أفكلها'] = {
    'word': u'أفكلها',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكُلُّهَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هَا'
}
STOPWORDS[u'أفكلهم'] = {
    'word': u'أفكلهم',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكُلُّهُمْ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أفكلهن'] = {
    'word': u'أفكلهن',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكُلُّهُنَّ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أفكلنا'] = {
    'word': u'أفكلنا',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكُلُّنَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'نَا'
}
STOPWORDS[u'أفكلكما'] = {
    'word': u'أفكلكما',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكُلُّكُمَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أفكلهما'] = {
    'word': u'أفكلهما',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكُلُّهُمَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أوبكلي'] = {
    'word': u'أوبكلي',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِكُلُِّي',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'ِي'
}
STOPWORDS[u'أوبكلك'] = {
    'word': u'أوبكلك',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِكُلُّكَ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كَ'
}
STOPWORDS[u'أوبكلك'] = {
    'word': u'أوبكلك',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِكُلُّكِ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كِ'
}
STOPWORDS[u'أوبكله'] = {
    'word': u'أوبكله',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِكُلُّه',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'ه'
}
STOPWORDS[u'أوبكلكم'] = {
    'word': u'أوبكلكم',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِكُلُّكُمْ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أوبكلكن'] = {
    'word': u'أوبكلكن',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِكُلُّكُنَّ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أوبكلها'] = {
    'word': u'أوبكلها',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِكُلُّهَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هَا'
}
STOPWORDS[u'أوبكلهم'] = {
    'word': u'أوبكلهم',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِكُلُّهُمْ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أوبكلهن'] = {
    'word': u'أوبكلهن',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِكُلُّهُنَّ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أوبكلنا'] = {
    'word': u'أوبكلنا',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِكُلُّنَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'نَا'
}
STOPWORDS[u'أوبكلكما'] = {
    'word': u'أوبكلكما',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِكُلُّكُمَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أوبكلهما'] = {
    'word': u'أوبكلهما',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِكُلُّهُمَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أفبكلي'] = {
    'word': u'أفبكلي',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِكُلُِّي',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'ِي'
}
STOPWORDS[u'أفبكلك'] = {
    'word': u'أفبكلك',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِكُلُّكَ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كَ'
}
STOPWORDS[u'أفبكلك'] = {
    'word': u'أفبكلك',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِكُلُّكِ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كِ'
}
STOPWORDS[u'أفبكله'] = {
    'word': u'أفبكله',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِكُلُّه',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'ه'
}
STOPWORDS[u'أفبكلكم'] = {
    'word': u'أفبكلكم',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِكُلُّكُمْ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أفبكلكن'] = {
    'word': u'أفبكلكن',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِكُلُّكُنَّ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أفبكلها'] = {
    'word': u'أفبكلها',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِكُلُّهَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هَا'
}
STOPWORDS[u'أفبكلهم'] = {
    'word': u'أفبكلهم',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِكُلُّهُمْ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أفبكلهن'] = {
    'word': u'أفبكلهن',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِكُلُّهُنَّ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أفبكلنا'] = {
    'word': u'أفبكلنا',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِكُلُّنَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'نَا'
}
STOPWORDS[u'أفبكلكما'] = {
    'word': u'أفبكلكما',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِكُلُّكُمَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أفبكلهما'] = {
    'word': u'أفبكلهما',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِكُلُّهُمَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أوككلي'] = {
    'word': u'أوككلي',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَكُلُِّي',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'ِي'
}
STOPWORDS[u'أوككلك'] = {
    'word': u'أوككلك',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَكُلُّكَ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كَ'
}
STOPWORDS[u'أوككلك'] = {
    'word': u'أوككلك',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَكُلُّكِ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كِ'
}
STOPWORDS[u'أوككله'] = {
    'word': u'أوككله',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَكُلُّه',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'ه'
}
STOPWORDS[u'أوككلكم'] = {
    'word': u'أوككلكم',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَكُلُّكُمْ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أوككلكن'] = {
    'word': u'أوككلكن',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَكُلُّكُنَّ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أوككلها'] = {
    'word': u'أوككلها',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَكُلُّهَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هَا'
}
STOPWORDS[u'أوككلهم'] = {
    'word': u'أوككلهم',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَكُلُّهُمْ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أوككلهن'] = {
    'word': u'أوككلهن',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَكُلُّهُنَّ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أوككلنا'] = {
    'word': u'أوككلنا',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَكُلُّنَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'نَا'
}
STOPWORDS[u'أوككلكما'] = {
    'word': u'أوككلكما',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَكُلُّكُمَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أوككلهما'] = {
    'word': u'أوككلهما',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَكُلُّهُمَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أفككلي'] = {
    'word': u'أفككلي',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَكُلُِّي',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'ِي'
}
STOPWORDS[u'أفككلك'] = {
    'word': u'أفككلك',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَكُلُّكَ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كَ'
}
STOPWORDS[u'أفككلك'] = {
    'word': u'أفككلك',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَكُلُّكِ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كِ'
}
STOPWORDS[u'أفككله'] = {
    'word': u'أفككله',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَكُلُّه',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'ه'
}
STOPWORDS[u'أفككلكم'] = {
    'word': u'أفككلكم',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَكُلُّكُمْ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أفككلكن'] = {
    'word': u'أفككلكن',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَكُلُّكُنَّ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أفككلها'] = {
    'word': u'أفككلها',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَكُلُّهَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هَا'
}
STOPWORDS[u'أفككلهم'] = {
    'word': u'أفككلهم',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَكُلُّهُمْ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أفككلهن'] = {
    'word': u'أفككلهن',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَكُلُّهُنَّ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أفككلنا'] = {
    'word': u'أفككلنا',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَكُلُّنَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'نَا'
}
STOPWORDS[u'أفككلكما'] = {
    'word': u'أفككلكما',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَكُلُّكُمَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أفككلهما'] = {
    'word': u'أفككلهما',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَكُلُّهُمَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أولكلي'] = {
    'word': u'أولكلي',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِكُلُِّي',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'ِي'
}
STOPWORDS[u'أولكلك'] = {
    'word': u'أولكلك',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِكُلُّكَ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كَ'
}
STOPWORDS[u'أولكلك'] = {
    'word': u'أولكلك',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِكُلُّكِ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كِ'
}
STOPWORDS[u'أولكله'] = {
    'word': u'أولكله',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِكُلُّه',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'ه'
}
STOPWORDS[u'أولكلكم'] = {
    'word': u'أولكلكم',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِكُلُّكُمْ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أولكلكن'] = {
    'word': u'أولكلكن',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِكُلُّكُنَّ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أولكلها'] = {
    'word': u'أولكلها',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِكُلُّهَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هَا'
}
STOPWORDS[u'أولكلهم'] = {
    'word': u'أولكلهم',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِكُلُّهُمْ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أولكلهن'] = {
    'word': u'أولكلهن',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِكُلُّهُنَّ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أولكلنا'] = {
    'word': u'أولكلنا',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِكُلُّنَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'نَا'
}
STOPWORDS[u'أولكلكما'] = {
    'word': u'أولكلكما',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِكُلُّكُمَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أولكلهما'] = {
    'word': u'أولكلهما',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِكُلُّهُمَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أفلكلي'] = {
    'word': u'أفلكلي',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِكُلُِّي',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'ِي'
}
STOPWORDS[u'أفلكلك'] = {
    'word': u'أفلكلك',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِكُلُّكَ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كَ'
}
STOPWORDS[u'أفلكلك'] = {
    'word': u'أفلكلك',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِكُلُّكِ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كِ'
}
STOPWORDS[u'أفلكله'] = {
    'word': u'أفلكله',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِكُلُّه',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'ه'
}
STOPWORDS[u'أفلكلكم'] = {
    'word': u'أفلكلكم',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِكُلُّكُمْ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أفلكلكن'] = {
    'word': u'أفلكلكن',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِكُلُّكُنَّ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أفلكلها'] = {
    'word': u'أفلكلها',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِكُلُّهَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هَا'
}
STOPWORDS[u'أفلكلهم'] = {
    'word': u'أفلكلهم',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِكُلُّهُمْ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أفلكلهن'] = {
    'word': u'أفلكلهن',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِكُلُّهُنَّ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أفلكلنا'] = {
    'word': u'أفلكلنا',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِكُلُّنَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'نَا'
}
STOPWORDS[u'أفلكلكما'] = {
    'word': u'أفلكلكما',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِكُلُّكُمَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أفلكلهما'] = {
    'word': u'أفلكلهما',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِكُلُّهُمَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أولكلي'] = {
    'word': u'أولكلي',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَكُلُِّي',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'ِي'
}
STOPWORDS[u'أولكلك'] = {
    'word': u'أولكلك',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَكُلُّكَ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كَ'
}
STOPWORDS[u'أولكلك'] = {
    'word': u'أولكلك',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَكُلُّكِ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كِ'
}
STOPWORDS[u'أولكله'] = {
    'word': u'أولكله',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَكُلُّه',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'ه'
}
STOPWORDS[u'أولكلكم'] = {
    'word': u'أولكلكم',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَكُلُّكُمْ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أولكلكن'] = {
    'word': u'أولكلكن',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَكُلُّكُنَّ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أولكلها'] = {
    'word': u'أولكلها',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَكُلُّهَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هَا'
}
STOPWORDS[u'أولكلهم'] = {
    'word': u'أولكلهم',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَكُلُّهُمْ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أولكلهن'] = {
    'word': u'أولكلهن',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَكُلُّهُنَّ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أولكلنا'] = {
    'word': u'أولكلنا',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَكُلُّنَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'نَا'
}
STOPWORDS[u'أولكلكما'] = {
    'word': u'أولكلكما',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَكُلُّكُمَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أولكلهما'] = {
    'word': u'أولكلهما',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَكُلُّهُمَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أفلكلي'] = {
    'word': u'أفلكلي',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَكُلُِّي',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'ِي'
}
STOPWORDS[u'أفلكلك'] = {
    'word': u'أفلكلك',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَكُلُّكَ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كَ'
}
STOPWORDS[u'أفلكلك'] = {
    'word': u'أفلكلك',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَكُلُّكِ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كِ'
}
STOPWORDS[u'أفلكله'] = {
    'word': u'أفلكله',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَكُلُّه',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'ه'
}
STOPWORDS[u'أفلكلكم'] = {
    'word': u'أفلكلكم',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَكُلُّكُمْ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أفلكلكن'] = {
    'word': u'أفلكلكن',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَكُلُّكُنَّ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أفلكلها'] = {
    'word': u'أفلكلها',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَكُلُّهَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هَا'
}
STOPWORDS[u'أفلكلهم'] = {
    'word': u'أفلكلهم',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَكُلُّهُمْ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أفلكلهن'] = {
    'word': u'أفلكلهن',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَكُلُّهُنَّ',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أفلكلنا'] = {
    'word': u'أفلكلنا',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَكُلُّنَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'نَا'
}
STOPWORDS[u'أفلكلكما'] = {
    'word': u'أفلكلكما',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَكُلُّكُمَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أفلكلهما'] = {
    'word': u'أفلكلهما',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَكُلُّهُمَا',
    'stem': u'كُلُّ',
    'type': u'STOPWORD',
    'original': u'كُلُّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'لعمر'] = {
    'word': u'لَعَمْرُ',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة',
    'vocalized': u'لَعَمْرُ',
    'stem': u'لَعَمْرُ',
    'type': u'STOPWORD',
    'original': u'لَعَمْرُ',
    'encletic': u''
}
STOPWORDS[u'ولعمر'] = {
    'word': u'ولعمر',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف',
    'vocalized': u'وَلَعَمْرُ',
    'stem': u'لَعَمْرُ',
    'type': u'STOPWORD',
    'original': u'لَعَمْرُ',
    'encletic': u''
}
STOPWORDS[u'فلعمر'] = {
    'word': u'فلعمر',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف',
    'vocalized': u'فَلَعَمْرُ',
    'stem': u'لَعَمْرُ',
    'type': u'STOPWORD',
    'original': u'لَعَمْرُ',
    'encletic': u''
}
STOPWORDS[u'لعمري'] = {
    'word': u'لعمري',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'لَعَمْرُِي',
    'stem': u'لَعَمْرُ',
    'type': u'STOPWORD',
    'original': u'لَعَمْرُ',
    'encletic': u'ِي'
}
STOPWORDS[u'لعمرك'] = {
    'word': u'لعمرك',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'لَعَمْرُكَ',
    'stem': u'لَعَمْرُ',
    'type': u'STOPWORD',
    'original': u'لَعَمْرُ',
    'encletic': u'كَ'
}
STOPWORDS[u'لعمرك'] = {
    'word': u'لعمرك',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'لَعَمْرُكِ',
    'stem': u'لَعَمْرُ',
    'type': u'STOPWORD',
    'original': u'لَعَمْرُ',
    'encletic': u'كِ'
}
STOPWORDS[u'لعمره'] = {
    'word': u'لعمره',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'لَعَمْرُه',
    'stem': u'لَعَمْرُ',
    'type': u'STOPWORD',
    'original': u'لَعَمْرُ',
    'encletic': u'ه'
}
STOPWORDS[u'لعمركم'] = {
    'word': u'لعمركم',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'لَعَمْرُكُمْ',
    'stem': u'لَعَمْرُ',
    'type': u'STOPWORD',
    'original': u'لَعَمْرُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'لعمركن'] = {
    'word': u'لعمركن',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'لَعَمْرُكُنَّ',
    'stem': u'لَعَمْرُ',
    'type': u'STOPWORD',
    'original': u'لَعَمْرُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'لعمرها'] = {
    'word': u'لعمرها',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'لَعَمْرُهَا',
    'stem': u'لَعَمْرُ',
    'type': u'STOPWORD',
    'original': u'لَعَمْرُ',
    'encletic': u'هَا'
}
STOPWORDS[u'لعمرهم'] = {
    'word': u'لعمرهم',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'لَعَمْرُهُمْ',
    'stem': u'لَعَمْرُ',
    'type': u'STOPWORD',
    'original': u'لَعَمْرُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'لعمرهن'] = {
    'word': u'لعمرهن',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'لَعَمْرُهُنَّ',
    'stem': u'لَعَمْرُ',
    'type': u'STOPWORD',
    'original': u'لَعَمْرُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'لعمرنا'] = {
    'word': u'لعمرنا',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'لَعَمْرُنَا',
    'stem': u'لَعَمْرُ',
    'type': u'STOPWORD',
    'original': u'لَعَمْرُ',
    'encletic': u'نَا'
}
STOPWORDS[u'لعمركما'] = {
    'word': u'لعمركما',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'لَعَمْرُكُمَا',
    'stem': u'لَعَمْرُ',
    'type': u'STOPWORD',
    'original': u'لَعَمْرُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'لعمرهما'] = {
    'word': u'لعمرهما',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'لَعَمْرُهُمَا',
    'stem': u'لَعَمْرُ',
    'type': u'STOPWORD',
    'original': u'لَعَمْرُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ولعمري'] = {
    'word': u'ولعمري',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَلَعَمْرُِي',
    'stem': u'لَعَمْرُ',
    'type': u'STOPWORD',
    'original': u'لَعَمْرُ',
    'encletic': u'ِي'
}
STOPWORDS[u'ولعمرك'] = {
    'word': u'ولعمرك',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَلَعَمْرُكَ',
    'stem': u'لَعَمْرُ',
    'type': u'STOPWORD',
    'original': u'لَعَمْرُ',
    'encletic': u'كَ'
}
STOPWORDS[u'ولعمرك'] = {
    'word': u'ولعمرك',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَلَعَمْرُكِ',
    'stem': u'لَعَمْرُ',
    'type': u'STOPWORD',
    'original': u'لَعَمْرُ',
    'encletic': u'كِ'
}
STOPWORDS[u'ولعمره'] = {
    'word': u'ولعمره',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَلَعَمْرُه',
    'stem': u'لَعَمْرُ',
    'type': u'STOPWORD',
    'original': u'لَعَمْرُ',
    'encletic': u'ه'
}
STOPWORDS[u'ولعمركم'] = {
    'word': u'ولعمركم',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَلَعَمْرُكُمْ',
    'stem': u'لَعَمْرُ',
    'type': u'STOPWORD',
    'original': u'لَعَمْرُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ولعمركن'] = {
    'word': u'ولعمركن',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَلَعَمْرُكُنَّ',
    'stem': u'لَعَمْرُ',
    'type': u'STOPWORD',
    'original': u'لَعَمْرُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ولعمرها'] = {
    'word': u'ولعمرها',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَلَعَمْرُهَا',
    'stem': u'لَعَمْرُ',
    'type': u'STOPWORD',
    'original': u'لَعَمْرُ',
    'encletic': u'هَا'
}
STOPWORDS[u'ولعمرهم'] = {
    'word': u'ولعمرهم',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَلَعَمْرُهُمْ',
    'stem': u'لَعَمْرُ',
    'type': u'STOPWORD',
    'original': u'لَعَمْرُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ولعمرهن'] = {
    'word': u'ولعمرهن',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَلَعَمْرُهُنَّ',
    'stem': u'لَعَمْرُ',
    'type': u'STOPWORD',
    'original': u'لَعَمْرُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ولعمرنا'] = {
    'word': u'ولعمرنا',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَلَعَمْرُنَا',
    'stem': u'لَعَمْرُ',
    'type': u'STOPWORD',
    'original': u'لَعَمْرُ',
    'encletic': u'نَا'
}
STOPWORDS[u'ولعمركما'] = {
    'word': u'ولعمركما',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَلَعَمْرُكُمَا',
    'stem': u'لَعَمْرُ',
    'type': u'STOPWORD',
    'original': u'لَعَمْرُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ولعمرهما'] = {
    'word': u'ولعمرهما',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَلَعَمْرُهُمَا',
    'stem': u'لَعَمْرُ',
    'type': u'STOPWORD',
    'original': u'لَعَمْرُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فلعمري'] = {
    'word': u'فلعمري',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَلَعَمْرُِي',
    'stem': u'لَعَمْرُ',
    'type': u'STOPWORD',
    'original': u'لَعَمْرُ',
    'encletic': u'ِي'
}
STOPWORDS[u'فلعمرك'] = {
    'word': u'فلعمرك',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَلَعَمْرُكَ',
    'stem': u'لَعَمْرُ',
    'type': u'STOPWORD',
    'original': u'لَعَمْرُ',
    'encletic': u'كَ'
}
STOPWORDS[u'فلعمرك'] = {
    'word': u'فلعمرك',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَلَعَمْرُكِ',
    'stem': u'لَعَمْرُ',
    'type': u'STOPWORD',
    'original': u'لَعَمْرُ',
    'encletic': u'كِ'
}
STOPWORDS[u'فلعمره'] = {
    'word': u'فلعمره',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَلَعَمْرُه',
    'stem': u'لَعَمْرُ',
    'type': u'STOPWORD',
    'original': u'لَعَمْرُ',
    'encletic': u'ه'
}
STOPWORDS[u'فلعمركم'] = {
    'word': u'فلعمركم',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَلَعَمْرُكُمْ',
    'stem': u'لَعَمْرُ',
    'type': u'STOPWORD',
    'original': u'لَعَمْرُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فلعمركن'] = {
    'word': u'فلعمركن',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَلَعَمْرُكُنَّ',
    'stem': u'لَعَمْرُ',
    'type': u'STOPWORD',
    'original': u'لَعَمْرُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فلعمرها'] = {
    'word': u'فلعمرها',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَلَعَمْرُهَا',
    'stem': u'لَعَمْرُ',
    'type': u'STOPWORD',
    'original': u'لَعَمْرُ',
    'encletic': u'هَا'
}
STOPWORDS[u'فلعمرهم'] = {
    'word': u'فلعمرهم',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَلَعَمْرُهُمْ',
    'stem': u'لَعَمْرُ',
    'type': u'STOPWORD',
    'original': u'لَعَمْرُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فلعمرهن'] = {
    'word': u'فلعمرهن',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَلَعَمْرُهُنَّ',
    'stem': u'لَعَمْرُ',
    'type': u'STOPWORD',
    'original': u'لَعَمْرُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فلعمرنا'] = {
    'word': u'فلعمرنا',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَلَعَمْرُنَا',
    'stem': u'لَعَمْرُ',
    'type': u'STOPWORD',
    'original': u'لَعَمْرُ',
    'encletic': u'نَا'
}
STOPWORDS[u'فلعمركما'] = {
    'word': u'فلعمركما',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَلَعَمْرُكُمَا',
    'stem': u'لَعَمْرُ',
    'type': u'STOPWORD',
    'original': u'لَعَمْرُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فلعمرهما'] = {
    'word': u'فلعمرهما',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَلَعَمْرُهُمَا',
    'stem': u'لَعَمْرُ',
    'type': u'STOPWORD',
    'original': u'لَعَمْرُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'لما'] = {
    'word': u'لَمَّا',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة',
    'vocalized': u'لَمَّا',
    'stem': u'لَمَّا',
    'type': u'STOPWORD',
    'original': u'لَمَّا',
    'encletic': u''
}
STOPWORDS[u'ولما'] = {
    'word': u'ولما',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف',
    'vocalized': u'وَلَمَّا',
    'stem': u'لَمَّا',
    'type': u'STOPWORD',
    'original': u'لَمَّا',
    'encletic': u''
}
STOPWORDS[u'فلما'] = {
    'word': u'فلما',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف',
    'vocalized': u'فَلَمَّا',
    'stem': u'لَمَّا',
    'type': u'STOPWORD',
    'original': u'لَمَّا',
    'encletic': u''
}
STOPWORDS[u'ألما'] = {
    'word': u'ألما',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور',
    'vocalized': u'أَلَمَّا',
    'stem': u'لَمَّا',
    'type': u'STOPWORD',
    'original': u'لَمَّا',
    'encletic': u''
}
STOPWORDS[u'أولما'] = {
    'word': u'أولما',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور',
    'vocalized': u'أَوَلَمَّا',
    'stem': u'لَمَّا',
    'type': u'STOPWORD',
    'original': u'لَمَّا',
    'encletic': u''
}
STOPWORDS[u'أفلما'] = {
    'word': u'أفلما',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور',
    'vocalized': u'أَفَلَمَّا',
    'stem': u'لَمَّا',
    'type': u'STOPWORD',
    'original': u'لَمَّا',
    'encletic': u''
}
STOPWORDS[u'مثل'] = {
    'word': u'مِثْلُ',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة',
    'vocalized': u'مِثْلُ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u''
}
STOPWORDS[u'بمثل'] = {
    'word': u'بمثل',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور',
    'vocalized': u'بِمِثْلُ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u''
}
STOPWORDS[u'كمثل'] = {
    'word': u'كمثل',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور',
    'vocalized': u'كَمِثْلُ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u''
}
STOPWORDS[u'لمثل'] = {
    'word': u'لمثل',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور',
    'vocalized': u'لِمِثْلُ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u''
}
STOPWORDS[u'لمثل'] = {
    'word': u'لمثل',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور',
    'vocalized': u'لَمِثْلُ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u''
}
STOPWORDS[u'ومثل'] = {
    'word': u'ومثل',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف',
    'vocalized': u'وَمِثْلُ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u''
}
STOPWORDS[u'فمثل'] = {
    'word': u'فمثل',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف',
    'vocalized': u'فَمِثْلُ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u''
}
STOPWORDS[u'وبمثل'] = {
    'word': u'وبمثل',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف',
    'vocalized': u'وَبِمِثْلُ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u''
}
STOPWORDS[u'فبمثل'] = {
    'word': u'فبمثل',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف',
    'vocalized': u'فَبِمِثْلُ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u''
}
STOPWORDS[u'وكمثل'] = {
    'word': u'وكمثل',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف',
    'vocalized': u'وَكَمِثْلُ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u''
}
STOPWORDS[u'فكمثل'] = {
    'word': u'فكمثل',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف',
    'vocalized': u'فَكَمِثْلُ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u''
}
STOPWORDS[u'ولمثل'] = {
    'word': u'ولمثل',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف',
    'vocalized': u'وَلِمِثْلُ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u''
}
STOPWORDS[u'فلمثل'] = {
    'word': u'فلمثل',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف',
    'vocalized': u'فَلِمِثْلُ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u''
}
STOPWORDS[u'ولمثل'] = {
    'word': u'ولمثل',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'وَلَمِثْلُ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u''
}
STOPWORDS[u'فلمثل'] = {
    'word': u'فلمثل',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'فَلَمِثْلُ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u''
}
STOPWORDS[u'أمثل'] = {
    'word': u'أمثل',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور',
    'vocalized': u'أَمِثْلُ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u''
}
STOPWORDS[u'أبمثل'] = {
    'word': u'أبمثل',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور',
    'vocalized': u'أَبِمِثْلُ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u''
}
STOPWORDS[u'أكمثل'] = {
    'word': u'أكمثل',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور',
    'vocalized': u'أَكَمِثْلُ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u''
}
STOPWORDS[u'ألمثل'] = {
    'word': u'ألمثل',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور',
    'vocalized': u'أَلِمِثْلُ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u''
}
STOPWORDS[u'ألمثل'] = {
    'word': u'ألمثل',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور',
    'vocalized': u'أَلَمِثْلُ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u''
}
STOPWORDS[u'أومثل'] = {
    'word': u'أومثل',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور',
    'vocalized': u'أَوَمِثْلُ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u''
}
STOPWORDS[u'أفمثل'] = {
    'word': u'أفمثل',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور',
    'vocalized': u'أَفَمِثْلُ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u''
}
STOPWORDS[u'أوبمثل'] = {
    'word': u'أوبمثل',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَبِمِثْلُ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u''
}
STOPWORDS[u'أفبمثل'] = {
    'word': u'أفبمثل',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَبِمِثْلُ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u''
}
STOPWORDS[u'أوكمثل'] = {
    'word': u'أوكمثل',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَكَمِثْلُ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u''
}
STOPWORDS[u'أفكمثل'] = {
    'word': u'أفكمثل',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَكَمِثْلُ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u''
}
STOPWORDS[u'أولمثل'] = {
    'word': u'أولمثل',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَلِمِثْلُ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u''
}
STOPWORDS[u'أفلمثل'] = {
    'word': u'أفلمثل',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَلِمِثْلُ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u''
}
STOPWORDS[u'أولمثل'] = {
    'word': u'أولمثل',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَلَمِثْلُ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u''
}
STOPWORDS[u'أفلمثل'] = {
    'word': u'أفلمثل',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَلَمِثْلُ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u''
}
STOPWORDS[u'مثلي'] = {
    'word': u'مثلي',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'مِثْلُِي',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'ِي'
}
STOPWORDS[u'مثلك'] = {
    'word': u'مثلك',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'مِثْلُكَ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كَ'
}
STOPWORDS[u'مثلك'] = {
    'word': u'مثلك',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'مِثْلُكِ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كِ'
}
STOPWORDS[u'مثله'] = {
    'word': u'مثله',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'مِثْلُه',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'ه'
}
STOPWORDS[u'مثلكم'] = {
    'word': u'مثلكم',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'مِثْلُكُمْ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'مثلكن'] = {
    'word': u'مثلكن',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'مِثْلُكُنَّ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'مثلها'] = {
    'word': u'مثلها',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'مِثْلُهَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هَا'
}
STOPWORDS[u'مثلهم'] = {
    'word': u'مثلهم',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'مِثْلُهُمْ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'مثلهن'] = {
    'word': u'مثلهن',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'مِثْلُهُنَّ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'مثلنا'] = {
    'word': u'مثلنا',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'مِثْلُنَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'نَا'
}
STOPWORDS[u'مثلكما'] = {
    'word': u'مثلكما',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'مِثْلُكُمَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'مثلهما'] = {
    'word': u'مثلهما',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'مِثْلُهُمَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'بمثلي'] = {
    'word': u'بمثلي',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِمِثْلُِي',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'ِي'
}
STOPWORDS[u'بمثلك'] = {
    'word': u'بمثلك',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِمِثْلُكَ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كَ'
}
STOPWORDS[u'بمثلك'] = {
    'word': u'بمثلك',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِمِثْلُكِ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كِ'
}
STOPWORDS[u'بمثله'] = {
    'word': u'بمثله',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِمِثْلُه',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'ه'
}
STOPWORDS[u'بمثلكم'] = {
    'word': u'بمثلكم',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِمِثْلُكُمْ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'بمثلكن'] = {
    'word': u'بمثلكن',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِمِثْلُكُنَّ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'بمثلها'] = {
    'word': u'بمثلها',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِمِثْلُهَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هَا'
}
STOPWORDS[u'بمثلهم'] = {
    'word': u'بمثلهم',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِمِثْلُهُمْ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'بمثلهن'] = {
    'word': u'بمثلهن',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِمِثْلُهُنَّ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'بمثلنا'] = {
    'word': u'بمثلنا',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِمِثْلُنَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'نَا'
}
STOPWORDS[u'بمثلكما'] = {
    'word': u'بمثلكما',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِمِثْلُكُمَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'بمثلهما'] = {
    'word': u'بمثلهما',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِمِثْلُهُمَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'كمثلي'] = {
    'word': u'كمثلي',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَمِثْلُِي',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'ِي'
}
STOPWORDS[u'كمثلك'] = {
    'word': u'كمثلك',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَمِثْلُكَ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كَ'
}
STOPWORDS[u'كمثلك'] = {
    'word': u'كمثلك',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَمِثْلُكِ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كِ'
}
STOPWORDS[u'كمثله'] = {
    'word': u'كمثله',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَمِثْلُه',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'ه'
}
STOPWORDS[u'كمثلكم'] = {
    'word': u'كمثلكم',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَمِثْلُكُمْ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'كمثلكن'] = {
    'word': u'كمثلكن',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَمِثْلُكُنَّ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'كمثلها'] = {
    'word': u'كمثلها',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَمِثْلُهَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هَا'
}
STOPWORDS[u'كمثلهم'] = {
    'word': u'كمثلهم',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَمِثْلُهُمْ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'كمثلهن'] = {
    'word': u'كمثلهن',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَمِثْلُهُنَّ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'كمثلنا'] = {
    'word': u'كمثلنا',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَمِثْلُنَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'نَا'
}
STOPWORDS[u'كمثلكما'] = {
    'word': u'كمثلكما',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَمِثْلُكُمَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'كمثلهما'] = {
    'word': u'كمثلهما',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَمِثْلُهُمَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'لمثلي'] = {
    'word': u'لمثلي',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِمِثْلُِي',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'ِي'
}
STOPWORDS[u'لمثلك'] = {
    'word': u'لمثلك',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِمِثْلُكَ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كَ'
}
STOPWORDS[u'لمثلك'] = {
    'word': u'لمثلك',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِمِثْلُكِ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كِ'
}
STOPWORDS[u'لمثله'] = {
    'word': u'لمثله',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِمِثْلُه',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'ه'
}
STOPWORDS[u'لمثلكم'] = {
    'word': u'لمثلكم',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِمِثْلُكُمْ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'لمثلكن'] = {
    'word': u'لمثلكن',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِمِثْلُكُنَّ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'لمثلها'] = {
    'word': u'لمثلها',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِمِثْلُهَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هَا'
}
STOPWORDS[u'لمثلهم'] = {
    'word': u'لمثلهم',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِمِثْلُهُمْ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'لمثلهن'] = {
    'word': u'لمثلهن',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِمِثْلُهُنَّ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'لمثلنا'] = {
    'word': u'لمثلنا',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِمِثْلُنَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'نَا'
}
STOPWORDS[u'لمثلكما'] = {
    'word': u'لمثلكما',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِمِثْلُكُمَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'لمثلهما'] = {
    'word': u'لمثلهما',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِمِثْلُهُمَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'لمثلي'] = {
    'word': u'لمثلي',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَمِثْلُِي',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'ِي'
}
STOPWORDS[u'لمثلك'] = {
    'word': u'لمثلك',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَمِثْلُكَ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كَ'
}
STOPWORDS[u'لمثلك'] = {
    'word': u'لمثلك',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَمِثْلُكِ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كِ'
}
STOPWORDS[u'لمثله'] = {
    'word': u'لمثله',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَمِثْلُه',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'ه'
}
STOPWORDS[u'لمثلكم'] = {
    'word': u'لمثلكم',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَمِثْلُكُمْ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'لمثلكن'] = {
    'word': u'لمثلكن',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَمِثْلُكُنَّ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'لمثلها'] = {
    'word': u'لمثلها',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَمِثْلُهَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هَا'
}
STOPWORDS[u'لمثلهم'] = {
    'word': u'لمثلهم',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَمِثْلُهُمْ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'لمثلهن'] = {
    'word': u'لمثلهن',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَمِثْلُهُنَّ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'لمثلنا'] = {
    'word': u'لمثلنا',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَمِثْلُنَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'نَا'
}
STOPWORDS[u'لمثلكما'] = {
    'word': u'لمثلكما',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَمِثْلُكُمَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'لمثلهما'] = {
    'word': u'لمثلهما',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَمِثْلُهُمَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ومثلي'] = {
    'word': u'ومثلي',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَمِثْلُِي',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'ِي'
}
STOPWORDS[u'ومثلك'] = {
    'word': u'ومثلك',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَمِثْلُكَ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كَ'
}
STOPWORDS[u'ومثلك'] = {
    'word': u'ومثلك',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَمِثْلُكِ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كِ'
}
STOPWORDS[u'ومثله'] = {
    'word': u'ومثله',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَمِثْلُه',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'ه'
}
STOPWORDS[u'ومثلكم'] = {
    'word': u'ومثلكم',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَمِثْلُكُمْ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ومثلكن'] = {
    'word': u'ومثلكن',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَمِثْلُكُنَّ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ومثلها'] = {
    'word': u'ومثلها',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَمِثْلُهَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هَا'
}
STOPWORDS[u'ومثلهم'] = {
    'word': u'ومثلهم',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَمِثْلُهُمْ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ومثلهن'] = {
    'word': u'ومثلهن',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَمِثْلُهُنَّ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ومثلنا'] = {
    'word': u'ومثلنا',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَمِثْلُنَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'نَا'
}
STOPWORDS[u'ومثلكما'] = {
    'word': u'ومثلكما',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَمِثْلُكُمَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ومثلهما'] = {
    'word': u'ومثلهما',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَمِثْلُهُمَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فمثلي'] = {
    'word': u'فمثلي',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَمِثْلُِي',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'ِي'
}
STOPWORDS[u'فمثلك'] = {
    'word': u'فمثلك',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَمِثْلُكَ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كَ'
}
STOPWORDS[u'فمثلك'] = {
    'word': u'فمثلك',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَمِثْلُكِ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كِ'
}
STOPWORDS[u'فمثله'] = {
    'word': u'فمثله',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَمِثْلُه',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'ه'
}
STOPWORDS[u'فمثلكم'] = {
    'word': u'فمثلكم',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَمِثْلُكُمْ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فمثلكن'] = {
    'word': u'فمثلكن',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَمِثْلُكُنَّ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فمثلها'] = {
    'word': u'فمثلها',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَمِثْلُهَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هَا'
}
STOPWORDS[u'فمثلهم'] = {
    'word': u'فمثلهم',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَمِثْلُهُمْ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فمثلهن'] = {
    'word': u'فمثلهن',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَمِثْلُهُنَّ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فمثلنا'] = {
    'word': u'فمثلنا',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَمِثْلُنَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'نَا'
}
STOPWORDS[u'فمثلكما'] = {
    'word': u'فمثلكما',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَمِثْلُكُمَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فمثلهما'] = {
    'word': u'فمثلهما',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَمِثْلُهُمَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'وبمثلي'] = {
    'word': u'وبمثلي',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِمِثْلُِي',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'ِي'
}
STOPWORDS[u'وبمثلك'] = {
    'word': u'وبمثلك',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِمِثْلُكَ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كَ'
}
STOPWORDS[u'وبمثلك'] = {
    'word': u'وبمثلك',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِمِثْلُكِ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كِ'
}
STOPWORDS[u'وبمثله'] = {
    'word': u'وبمثله',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِمِثْلُه',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'ه'
}
STOPWORDS[u'وبمثلكم'] = {
    'word': u'وبمثلكم',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِمِثْلُكُمْ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'وبمثلكن'] = {
    'word': u'وبمثلكن',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِمِثْلُكُنَّ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'وبمثلها'] = {
    'word': u'وبمثلها',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِمِثْلُهَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هَا'
}
STOPWORDS[u'وبمثلهم'] = {
    'word': u'وبمثلهم',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِمِثْلُهُمْ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'وبمثلهن'] = {
    'word': u'وبمثلهن',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِمِثْلُهُنَّ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'وبمثلنا'] = {
    'word': u'وبمثلنا',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِمِثْلُنَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'نَا'
}
STOPWORDS[u'وبمثلكما'] = {
    'word': u'وبمثلكما',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِمِثْلُكُمَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'وبمثلهما'] = {
    'word': u'وبمثلهما',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِمِثْلُهُمَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فبمثلي'] = {
    'word': u'فبمثلي',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِمِثْلُِي',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'ِي'
}
STOPWORDS[u'فبمثلك'] = {
    'word': u'فبمثلك',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِمِثْلُكَ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كَ'
}
STOPWORDS[u'فبمثلك'] = {
    'word': u'فبمثلك',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِمِثْلُكِ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كِ'
}
STOPWORDS[u'فبمثله'] = {
    'word': u'فبمثله',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِمِثْلُه',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'ه'
}
STOPWORDS[u'فبمثلكم'] = {
    'word': u'فبمثلكم',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِمِثْلُكُمْ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فبمثلكن'] = {
    'word': u'فبمثلكن',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِمِثْلُكُنَّ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فبمثلها'] = {
    'word': u'فبمثلها',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِمِثْلُهَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هَا'
}
STOPWORDS[u'فبمثلهم'] = {
    'word': u'فبمثلهم',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِمِثْلُهُمْ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فبمثلهن'] = {
    'word': u'فبمثلهن',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِمِثْلُهُنَّ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فبمثلنا'] = {
    'word': u'فبمثلنا',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِمِثْلُنَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'نَا'
}
STOPWORDS[u'فبمثلكما'] = {
    'word': u'فبمثلكما',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِمِثْلُكُمَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فبمثلهما'] = {
    'word': u'فبمثلهما',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِمِثْلُهُمَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'وكمثلي'] = {
    'word': u'وكمثلي',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَمِثْلُِي',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'ِي'
}
STOPWORDS[u'وكمثلك'] = {
    'word': u'وكمثلك',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَمِثْلُكَ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كَ'
}
STOPWORDS[u'وكمثلك'] = {
    'word': u'وكمثلك',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَمِثْلُكِ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كِ'
}
STOPWORDS[u'وكمثله'] = {
    'word': u'وكمثله',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَمِثْلُه',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'ه'
}
STOPWORDS[u'وكمثلكم'] = {
    'word': u'وكمثلكم',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَمِثْلُكُمْ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'وكمثلكن'] = {
    'word': u'وكمثلكن',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَمِثْلُكُنَّ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'وكمثلها'] = {
    'word': u'وكمثلها',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَمِثْلُهَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هَا'
}
STOPWORDS[u'وكمثلهم'] = {
    'word': u'وكمثلهم',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَمِثْلُهُمْ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'وكمثلهن'] = {
    'word': u'وكمثلهن',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَمِثْلُهُنَّ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'وكمثلنا'] = {
    'word': u'وكمثلنا',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَمِثْلُنَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'نَا'
}
STOPWORDS[u'وكمثلكما'] = {
    'word': u'وكمثلكما',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَمِثْلُكُمَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'وكمثلهما'] = {
    'word': u'وكمثلهما',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَمِثْلُهُمَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فكمثلي'] = {
    'word': u'فكمثلي',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَمِثْلُِي',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'ِي'
}
STOPWORDS[u'فكمثلك'] = {
    'word': u'فكمثلك',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَمِثْلُكَ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كَ'
}
STOPWORDS[u'فكمثلك'] = {
    'word': u'فكمثلك',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَمِثْلُكِ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كِ'
}
STOPWORDS[u'فكمثله'] = {
    'word': u'فكمثله',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَمِثْلُه',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'ه'
}
STOPWORDS[u'فكمثلكم'] = {
    'word': u'فكمثلكم',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَمِثْلُكُمْ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فكمثلكن'] = {
    'word': u'فكمثلكن',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَمِثْلُكُنَّ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فكمثلها'] = {
    'word': u'فكمثلها',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَمِثْلُهَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هَا'
}
STOPWORDS[u'فكمثلهم'] = {
    'word': u'فكمثلهم',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَمِثْلُهُمْ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فكمثلهن'] = {
    'word': u'فكمثلهن',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَمِثْلُهُنَّ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فكمثلنا'] = {
    'word': u'فكمثلنا',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَمِثْلُنَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'نَا'
}
STOPWORDS[u'فكمثلكما'] = {
    'word': u'فكمثلكما',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَمِثْلُكُمَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فكمثلهما'] = {
    'word': u'فكمثلهما',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَمِثْلُهُمَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ولمثلي'] = {
    'word': u'ولمثلي',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِمِثْلُِي',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'ِي'
}
STOPWORDS[u'ولمثلك'] = {
    'word': u'ولمثلك',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِمِثْلُكَ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كَ'
}
STOPWORDS[u'ولمثلك'] = {
    'word': u'ولمثلك',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِمِثْلُكِ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كِ'
}
STOPWORDS[u'ولمثله'] = {
    'word': u'ولمثله',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِمِثْلُه',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'ه'
}
STOPWORDS[u'ولمثلكم'] = {
    'word': u'ولمثلكم',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِمِثْلُكُمْ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ولمثلكن'] = {
    'word': u'ولمثلكن',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِمِثْلُكُنَّ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ولمثلها'] = {
    'word': u'ولمثلها',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِمِثْلُهَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هَا'
}
STOPWORDS[u'ولمثلهم'] = {
    'word': u'ولمثلهم',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِمِثْلُهُمْ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ولمثلهن'] = {
    'word': u'ولمثلهن',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِمِثْلُهُنَّ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ولمثلنا'] = {
    'word': u'ولمثلنا',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِمِثْلُنَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'نَا'
}
STOPWORDS[u'ولمثلكما'] = {
    'word': u'ولمثلكما',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِمِثْلُكُمَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ولمثلهما'] = {
    'word': u'ولمثلهما',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِمِثْلُهُمَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فلمثلي'] = {
    'word': u'فلمثلي',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِمِثْلُِي',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'ِي'
}
STOPWORDS[u'فلمثلك'] = {
    'word': u'فلمثلك',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِمِثْلُكَ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كَ'
}
STOPWORDS[u'فلمثلك'] = {
    'word': u'فلمثلك',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِمِثْلُكِ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كِ'
}
STOPWORDS[u'فلمثله'] = {
    'word': u'فلمثله',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِمِثْلُه',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'ه'
}
STOPWORDS[u'فلمثلكم'] = {
    'word': u'فلمثلكم',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِمِثْلُكُمْ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فلمثلكن'] = {
    'word': u'فلمثلكن',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِمِثْلُكُنَّ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فلمثلها'] = {
    'word': u'فلمثلها',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِمِثْلُهَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هَا'
}
STOPWORDS[u'فلمثلهم'] = {
    'word': u'فلمثلهم',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِمِثْلُهُمْ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فلمثلهن'] = {
    'word': u'فلمثلهن',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِمِثْلُهُنَّ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فلمثلنا'] = {
    'word': u'فلمثلنا',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِمِثْلُنَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'نَا'
}
STOPWORDS[u'فلمثلكما'] = {
    'word': u'فلمثلكما',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِمِثْلُكُمَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فلمثلهما'] = {
    'word': u'فلمثلهما',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِمِثْلُهُمَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ولمثلي'] = {
    'word': u'ولمثلي',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَمِثْلُِي',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'ِي'
}
STOPWORDS[u'ولمثلك'] = {
    'word': u'ولمثلك',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَمِثْلُكَ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كَ'
}
STOPWORDS[u'ولمثلك'] = {
    'word': u'ولمثلك',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَمِثْلُكِ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كِ'
}
STOPWORDS[u'ولمثله'] = {
    'word': u'ولمثله',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَمِثْلُه',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'ه'
}
STOPWORDS[u'ولمثلكم'] = {
    'word': u'ولمثلكم',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَمِثْلُكُمْ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ولمثلكن'] = {
    'word': u'ولمثلكن',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَمِثْلُكُنَّ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ولمثلها'] = {
    'word': u'ولمثلها',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَمِثْلُهَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هَا'
}
STOPWORDS[u'ولمثلهم'] = {
    'word': u'ولمثلهم',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَمِثْلُهُمْ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ولمثلهن'] = {
    'word': u'ولمثلهن',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَمِثْلُهُنَّ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ولمثلنا'] = {
    'word': u'ولمثلنا',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَمِثْلُنَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'نَا'
}
STOPWORDS[u'ولمثلكما'] = {
    'word': u'ولمثلكما',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَمِثْلُكُمَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ولمثلهما'] = {
    'word': u'ولمثلهما',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَمِثْلُهُمَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فلمثلي'] = {
    'word': u'فلمثلي',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَمِثْلُِي',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'ِي'
}
STOPWORDS[u'فلمثلك'] = {
    'word': u'فلمثلك',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَمِثْلُكَ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كَ'
}
STOPWORDS[u'فلمثلك'] = {
    'word': u'فلمثلك',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَمِثْلُكِ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كِ'
}
STOPWORDS[u'فلمثله'] = {
    'word': u'فلمثله',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَمِثْلُه',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'ه'
}
STOPWORDS[u'فلمثلكم'] = {
    'word': u'فلمثلكم',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَمِثْلُكُمْ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فلمثلكن'] = {
    'word': u'فلمثلكن',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَمِثْلُكُنَّ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فلمثلها'] = {
    'word': u'فلمثلها',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَمِثْلُهَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هَا'
}
STOPWORDS[u'فلمثلهم'] = {
    'word': u'فلمثلهم',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَمِثْلُهُمْ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فلمثلهن'] = {
    'word': u'فلمثلهن',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَمِثْلُهُنَّ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فلمثلنا'] = {
    'word': u'فلمثلنا',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَمِثْلُنَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'نَا'
}
STOPWORDS[u'فلمثلكما'] = {
    'word': u'فلمثلكما',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَمِثْلُكُمَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فلمثلهما'] = {
    'word': u'فلمثلهما',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَمِثْلُهُمَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أمثلي'] = {
    'word': u'أمثلي',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'أَمِثْلُِي',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'ِي'
}
STOPWORDS[u'أمثلك'] = {
    'word': u'أمثلك',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'أَمِثْلُكَ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كَ'
}
STOPWORDS[u'أمثلك'] = {
    'word': u'أمثلك',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'أَمِثْلُكِ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كِ'
}
STOPWORDS[u'أمثله'] = {
    'word': u'أمثله',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'أَمِثْلُه',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'ه'
}
STOPWORDS[u'أمثلكم'] = {
    'word': u'أمثلكم',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'أَمِثْلُكُمْ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أمثلكن'] = {
    'word': u'أمثلكن',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'أَمِثْلُكُنَّ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أمثلها'] = {
    'word': u'أمثلها',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'أَمِثْلُهَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هَا'
}
STOPWORDS[u'أمثلهم'] = {
    'word': u'أمثلهم',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'أَمِثْلُهُمْ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أمثلهن'] = {
    'word': u'أمثلهن',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'أَمِثْلُهُنَّ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أمثلنا'] = {
    'word': u'أمثلنا',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'أَمِثْلُنَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'نَا'
}
STOPWORDS[u'أمثلكما'] = {
    'word': u'أمثلكما',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'أَمِثْلُكُمَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أمثلهما'] = {
    'word': u'أمثلهما',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'أَمِثْلُهُمَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أبمثلي'] = {
    'word': u'أبمثلي',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِمِثْلُِي',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'ِي'
}
STOPWORDS[u'أبمثلك'] = {
    'word': u'أبمثلك',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِمِثْلُكَ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كَ'
}
STOPWORDS[u'أبمثلك'] = {
    'word': u'أبمثلك',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِمِثْلُكِ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كِ'
}
STOPWORDS[u'أبمثله'] = {
    'word': u'أبمثله',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِمِثْلُه',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'ه'
}
STOPWORDS[u'أبمثلكم'] = {
    'word': u'أبمثلكم',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِمِثْلُكُمْ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أبمثلكن'] = {
    'word': u'أبمثلكن',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِمِثْلُكُنَّ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أبمثلها'] = {
    'word': u'أبمثلها',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِمِثْلُهَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هَا'
}
STOPWORDS[u'أبمثلهم'] = {
    'word': u'أبمثلهم',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِمِثْلُهُمْ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أبمثلهن'] = {
    'word': u'أبمثلهن',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِمِثْلُهُنَّ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أبمثلنا'] = {
    'word': u'أبمثلنا',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِمِثْلُنَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'نَا'
}
STOPWORDS[u'أبمثلكما'] = {
    'word': u'أبمثلكما',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِمِثْلُكُمَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أبمثلهما'] = {
    'word': u'أبمثلهما',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِمِثْلُهُمَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أكمثلي'] = {
    'word': u'أكمثلي',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَمِثْلُِي',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'ِي'
}
STOPWORDS[u'أكمثلك'] = {
    'word': u'أكمثلك',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَمِثْلُكَ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كَ'
}
STOPWORDS[u'أكمثلك'] = {
    'word': u'أكمثلك',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَمِثْلُكِ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كِ'
}
STOPWORDS[u'أكمثله'] = {
    'word': u'أكمثله',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَمِثْلُه',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'ه'
}
STOPWORDS[u'أكمثلكم'] = {
    'word': u'أكمثلكم',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَمِثْلُكُمْ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أكمثلكن'] = {
    'word': u'أكمثلكن',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَمِثْلُكُنَّ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أكمثلها'] = {
    'word': u'أكمثلها',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَمِثْلُهَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هَا'
}
STOPWORDS[u'أكمثلهم'] = {
    'word': u'أكمثلهم',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَمِثْلُهُمْ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أكمثلهن'] = {
    'word': u'أكمثلهن',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَمِثْلُهُنَّ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أكمثلنا'] = {
    'word': u'أكمثلنا',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَمِثْلُنَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'نَا'
}
STOPWORDS[u'أكمثلكما'] = {
    'word': u'أكمثلكما',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَمِثْلُكُمَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أكمثلهما'] = {
    'word': u'أكمثلهما',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَمِثْلُهُمَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ألمثلي'] = {
    'word': u'ألمثلي',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِمِثْلُِي',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'ِي'
}
STOPWORDS[u'ألمثلك'] = {
    'word': u'ألمثلك',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِمِثْلُكَ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كَ'
}
STOPWORDS[u'ألمثلك'] = {
    'word': u'ألمثلك',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِمِثْلُكِ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كِ'
}
STOPWORDS[u'ألمثله'] = {
    'word': u'ألمثله',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِمِثْلُه',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'ه'
}
STOPWORDS[u'ألمثلكم'] = {
    'word': u'ألمثلكم',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِمِثْلُكُمْ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ألمثلكن'] = {
    'word': u'ألمثلكن',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِمِثْلُكُنَّ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ألمثلها'] = {
    'word': u'ألمثلها',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِمِثْلُهَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هَا'
}
STOPWORDS[u'ألمثلهم'] = {
    'word': u'ألمثلهم',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِمِثْلُهُمْ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ألمثلهن'] = {
    'word': u'ألمثلهن',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِمِثْلُهُنَّ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ألمثلنا'] = {
    'word': u'ألمثلنا',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِمِثْلُنَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'نَا'
}
STOPWORDS[u'ألمثلكما'] = {
    'word': u'ألمثلكما',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِمِثْلُكُمَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ألمثلهما'] = {
    'word': u'ألمثلهما',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِمِثْلُهُمَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ألمثلي'] = {
    'word': u'ألمثلي',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَمِثْلُِي',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'ِي'
}
STOPWORDS[u'ألمثلك'] = {
    'word': u'ألمثلك',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَمِثْلُكَ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كَ'
}
STOPWORDS[u'ألمثلك'] = {
    'word': u'ألمثلك',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَمِثْلُكِ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كِ'
}
STOPWORDS[u'ألمثله'] = {
    'word': u'ألمثله',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَمِثْلُه',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'ه'
}
STOPWORDS[u'ألمثلكم'] = {
    'word': u'ألمثلكم',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَمِثْلُكُمْ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ألمثلكن'] = {
    'word': u'ألمثلكن',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَمِثْلُكُنَّ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ألمثلها'] = {
    'word': u'ألمثلها',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَمِثْلُهَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هَا'
}
STOPWORDS[u'ألمثلهم'] = {
    'word': u'ألمثلهم',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَمِثْلُهُمْ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ألمثلهن'] = {
    'word': u'ألمثلهن',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَمِثْلُهُنَّ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ألمثلنا'] = {
    'word': u'ألمثلنا',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَمِثْلُنَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'نَا'
}
STOPWORDS[u'ألمثلكما'] = {
    'word': u'ألمثلكما',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَمِثْلُكُمَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ألمثلهما'] = {
    'word': u'ألمثلهما',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَمِثْلُهُمَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أومثلي'] = {
    'word': u'أومثلي',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَمِثْلُِي',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'ِي'
}
STOPWORDS[u'أومثلك'] = {
    'word': u'أومثلك',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَمِثْلُكَ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كَ'
}
STOPWORDS[u'أومثلك'] = {
    'word': u'أومثلك',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَمِثْلُكِ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كِ'
}
STOPWORDS[u'أومثله'] = {
    'word': u'أومثله',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَمِثْلُه',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'ه'
}
STOPWORDS[u'أومثلكم'] = {
    'word': u'أومثلكم',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَمِثْلُكُمْ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أومثلكن'] = {
    'word': u'أومثلكن',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَمِثْلُكُنَّ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أومثلها'] = {
    'word': u'أومثلها',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَمِثْلُهَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هَا'
}
STOPWORDS[u'أومثلهم'] = {
    'word': u'أومثلهم',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَمِثْلُهُمْ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أومثلهن'] = {
    'word': u'أومثلهن',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَمِثْلُهُنَّ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أومثلنا'] = {
    'word': u'أومثلنا',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَمِثْلُنَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'نَا'
}
STOPWORDS[u'أومثلكما'] = {
    'word': u'أومثلكما',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَمِثْلُكُمَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أومثلهما'] = {
    'word': u'أومثلهما',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَمِثْلُهُمَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أفمثلي'] = {
    'word': u'أفمثلي',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَمِثْلُِي',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'ِي'
}
STOPWORDS[u'أفمثلك'] = {
    'word': u'أفمثلك',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَمِثْلُكَ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كَ'
}
STOPWORDS[u'أفمثلك'] = {
    'word': u'أفمثلك',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَمِثْلُكِ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كِ'
}
STOPWORDS[u'أفمثله'] = {
    'word': u'أفمثله',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَمِثْلُه',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'ه'
}
STOPWORDS[u'أفمثلكم'] = {
    'word': u'أفمثلكم',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَمِثْلُكُمْ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أفمثلكن'] = {
    'word': u'أفمثلكن',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَمِثْلُكُنَّ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أفمثلها'] = {
    'word': u'أفمثلها',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَمِثْلُهَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هَا'
}
STOPWORDS[u'أفمثلهم'] = {
    'word': u'أفمثلهم',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَمِثْلُهُمْ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أفمثلهن'] = {
    'word': u'أفمثلهن',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَمِثْلُهُنَّ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أفمثلنا'] = {
    'word': u'أفمثلنا',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَمِثْلُنَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'نَا'
}
STOPWORDS[u'أفمثلكما'] = {
    'word': u'أفمثلكما',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَمِثْلُكُمَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أفمثلهما'] = {
    'word': u'أفمثلهما',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَمِثْلُهُمَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أوبمثلي'] = {
    'word': u'أوبمثلي',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِمِثْلُِي',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'ِي'
}
STOPWORDS[u'أوبمثلك'] = {
    'word': u'أوبمثلك',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِمِثْلُكَ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كَ'
}
STOPWORDS[u'أوبمثلك'] = {
    'word': u'أوبمثلك',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِمِثْلُكِ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كِ'
}
STOPWORDS[u'أوبمثله'] = {
    'word': u'أوبمثله',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِمِثْلُه',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'ه'
}
STOPWORDS[u'أوبمثلكم'] = {
    'word': u'أوبمثلكم',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِمِثْلُكُمْ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أوبمثلكن'] = {
    'word': u'أوبمثلكن',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِمِثْلُكُنَّ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أوبمثلها'] = {
    'word': u'أوبمثلها',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِمِثْلُهَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هَا'
}
STOPWORDS[u'أوبمثلهم'] = {
    'word': u'أوبمثلهم',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِمِثْلُهُمْ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أوبمثلهن'] = {
    'word': u'أوبمثلهن',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِمِثْلُهُنَّ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أوبمثلنا'] = {
    'word': u'أوبمثلنا',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِمِثْلُنَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'نَا'
}
STOPWORDS[u'أوبمثلكما'] = {
    'word': u'أوبمثلكما',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِمِثْلُكُمَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أوبمثلهما'] = {
    'word': u'أوبمثلهما',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِمِثْلُهُمَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أفبمثلي'] = {
    'word': u'أفبمثلي',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِمِثْلُِي',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'ِي'
}
STOPWORDS[u'أفبمثلك'] = {
    'word': u'أفبمثلك',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِمِثْلُكَ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كَ'
}
STOPWORDS[u'أفبمثلك'] = {
    'word': u'أفبمثلك',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِمِثْلُكِ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كِ'
}
STOPWORDS[u'أفبمثله'] = {
    'word': u'أفبمثله',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِمِثْلُه',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'ه'
}
STOPWORDS[u'أفبمثلكم'] = {
    'word': u'أفبمثلكم',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِمِثْلُكُمْ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أفبمثلكن'] = {
    'word': u'أفبمثلكن',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِمِثْلُكُنَّ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أفبمثلها'] = {
    'word': u'أفبمثلها',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِمِثْلُهَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هَا'
}
STOPWORDS[u'أفبمثلهم'] = {
    'word': u'أفبمثلهم',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِمِثْلُهُمْ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أفبمثلهن'] = {
    'word': u'أفبمثلهن',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِمِثْلُهُنَّ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أفبمثلنا'] = {
    'word': u'أفبمثلنا',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِمِثْلُنَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'نَا'
}
STOPWORDS[u'أفبمثلكما'] = {
    'word': u'أفبمثلكما',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِمِثْلُكُمَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أفبمثلهما'] = {
    'word': u'أفبمثلهما',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِمِثْلُهُمَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أوكمثلي'] = {
    'word': u'أوكمثلي',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَمِثْلُِي',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'ِي'
}
STOPWORDS[u'أوكمثلك'] = {
    'word': u'أوكمثلك',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَمِثْلُكَ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كَ'
}
STOPWORDS[u'أوكمثلك'] = {
    'word': u'أوكمثلك',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَمِثْلُكِ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كِ'
}
STOPWORDS[u'أوكمثله'] = {
    'word': u'أوكمثله',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَمِثْلُه',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'ه'
}
STOPWORDS[u'أوكمثلكم'] = {
    'word': u'أوكمثلكم',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَمِثْلُكُمْ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أوكمثلكن'] = {
    'word': u'أوكمثلكن',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَمِثْلُكُنَّ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أوكمثلها'] = {
    'word': u'أوكمثلها',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَمِثْلُهَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هَا'
}
STOPWORDS[u'أوكمثلهم'] = {
    'word': u'أوكمثلهم',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَمِثْلُهُمْ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أوكمثلهن'] = {
    'word': u'أوكمثلهن',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَمِثْلُهُنَّ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أوكمثلنا'] = {
    'word': u'أوكمثلنا',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَمِثْلُنَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'نَا'
}
STOPWORDS[u'أوكمثلكما'] = {
    'word': u'أوكمثلكما',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَمِثْلُكُمَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أوكمثلهما'] = {
    'word': u'أوكمثلهما',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَمِثْلُهُمَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أفكمثلي'] = {
    'word': u'أفكمثلي',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَمِثْلُِي',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'ِي'
}
STOPWORDS[u'أفكمثلك'] = {
    'word': u'أفكمثلك',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَمِثْلُكَ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كَ'
}
STOPWORDS[u'أفكمثلك'] = {
    'word': u'أفكمثلك',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَمِثْلُكِ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كِ'
}
STOPWORDS[u'أفكمثله'] = {
    'word': u'أفكمثله',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَمِثْلُه',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'ه'
}
STOPWORDS[u'أفكمثلكم'] = {
    'word': u'أفكمثلكم',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَمِثْلُكُمْ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أفكمثلكن'] = {
    'word': u'أفكمثلكن',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَمِثْلُكُنَّ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أفكمثلها'] = {
    'word': u'أفكمثلها',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَمِثْلُهَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هَا'
}
STOPWORDS[u'أفكمثلهم'] = {
    'word': u'أفكمثلهم',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَمِثْلُهُمْ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أفكمثلهن'] = {
    'word': u'أفكمثلهن',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَمِثْلُهُنَّ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أفكمثلنا'] = {
    'word': u'أفكمثلنا',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَمِثْلُنَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'نَا'
}
STOPWORDS[u'أفكمثلكما'] = {
    'word': u'أفكمثلكما',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَمِثْلُكُمَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أفكمثلهما'] = {
    'word': u'أفكمثلهما',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَمِثْلُهُمَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أولمثلي'] = {
    'word': u'أولمثلي',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِمِثْلُِي',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'ِي'
}
STOPWORDS[u'أولمثلك'] = {
    'word': u'أولمثلك',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِمِثْلُكَ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كَ'
}
STOPWORDS[u'أولمثلك'] = {
    'word': u'أولمثلك',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِمِثْلُكِ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كِ'
}
STOPWORDS[u'أولمثله'] = {
    'word': u'أولمثله',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِمِثْلُه',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'ه'
}
STOPWORDS[u'أولمثلكم'] = {
    'word': u'أولمثلكم',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِمِثْلُكُمْ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أولمثلكن'] = {
    'word': u'أولمثلكن',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِمِثْلُكُنَّ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أولمثلها'] = {
    'word': u'أولمثلها',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِمِثْلُهَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هَا'
}
STOPWORDS[u'أولمثلهم'] = {
    'word': u'أولمثلهم',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِمِثْلُهُمْ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أولمثلهن'] = {
    'word': u'أولمثلهن',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِمِثْلُهُنَّ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أولمثلنا'] = {
    'word': u'أولمثلنا',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِمِثْلُنَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'نَا'
}
STOPWORDS[u'أولمثلكما'] = {
    'word': u'أولمثلكما',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِمِثْلُكُمَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أولمثلهما'] = {
    'word': u'أولمثلهما',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِمِثْلُهُمَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أفلمثلي'] = {
    'word': u'أفلمثلي',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِمِثْلُِي',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'ِي'
}
STOPWORDS[u'أفلمثلك'] = {
    'word': u'أفلمثلك',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِمِثْلُكَ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كَ'
}
STOPWORDS[u'أفلمثلك'] = {
    'word': u'أفلمثلك',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِمِثْلُكِ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كِ'
}
STOPWORDS[u'أفلمثله'] = {
    'word': u'أفلمثله',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِمِثْلُه',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'ه'
}
STOPWORDS[u'أفلمثلكم'] = {
    'word': u'أفلمثلكم',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِمِثْلُكُمْ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أفلمثلكن'] = {
    'word': u'أفلمثلكن',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِمِثْلُكُنَّ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أفلمثلها'] = {
    'word': u'أفلمثلها',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِمِثْلُهَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هَا'
}
STOPWORDS[u'أفلمثلهم'] = {
    'word': u'أفلمثلهم',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِمِثْلُهُمْ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أفلمثلهن'] = {
    'word': u'أفلمثلهن',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِمِثْلُهُنَّ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أفلمثلنا'] = {
    'word': u'أفلمثلنا',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِمِثْلُنَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'نَا'
}
STOPWORDS[u'أفلمثلكما'] = {
    'word': u'أفلمثلكما',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِمِثْلُكُمَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أفلمثلهما'] = {
    'word': u'أفلمثلهما',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِمِثْلُهُمَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أولمثلي'] = {
    'word': u'أولمثلي',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَمِثْلُِي',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'ِي'
}
STOPWORDS[u'أولمثلك'] = {
    'word': u'أولمثلك',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَمِثْلُكَ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كَ'
}
STOPWORDS[u'أولمثلك'] = {
    'word': u'أولمثلك',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَمِثْلُكِ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كِ'
}
STOPWORDS[u'أولمثله'] = {
    'word': u'أولمثله',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَمِثْلُه',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'ه'
}
STOPWORDS[u'أولمثلكم'] = {
    'word': u'أولمثلكم',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَمِثْلُكُمْ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أولمثلكن'] = {
    'word': u'أولمثلكن',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَمِثْلُكُنَّ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أولمثلها'] = {
    'word': u'أولمثلها',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَمِثْلُهَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هَا'
}
STOPWORDS[u'أولمثلهم'] = {
    'word': u'أولمثلهم',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَمِثْلُهُمْ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أولمثلهن'] = {
    'word': u'أولمثلهن',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَمِثْلُهُنَّ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أولمثلنا'] = {
    'word': u'أولمثلنا',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَمِثْلُنَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'نَا'
}
STOPWORDS[u'أولمثلكما'] = {
    'word': u'أولمثلكما',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَمِثْلُكُمَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أولمثلهما'] = {
    'word': u'أولمثلهما',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَمِثْلُهُمَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أفلمثلي'] = {
    'word': u'أفلمثلي',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَمِثْلُِي',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'ِي'
}
STOPWORDS[u'أفلمثلك'] = {
    'word': u'أفلمثلك',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَمِثْلُكَ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كَ'
}
STOPWORDS[u'أفلمثلك'] = {
    'word': u'أفلمثلك',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَمِثْلُكِ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كِ'
}
STOPWORDS[u'أفلمثله'] = {
    'word': u'أفلمثله',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَمِثْلُه',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'ه'
}
STOPWORDS[u'أفلمثلكم'] = {
    'word': u'أفلمثلكم',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَمِثْلُكُمْ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أفلمثلكن'] = {
    'word': u'أفلمثلكن',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَمِثْلُكُنَّ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أفلمثلها'] = {
    'word': u'أفلمثلها',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَمِثْلُهَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هَا'
}
STOPWORDS[u'أفلمثلهم'] = {
    'word': u'أفلمثلهم',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَمِثْلُهُمْ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أفلمثلهن'] = {
    'word': u'أفلمثلهن',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَمِثْلُهُنَّ',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أفلمثلنا'] = {
    'word': u'أفلمثلنا',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَمِثْلُنَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'نَا'
}
STOPWORDS[u'أفلمثلكما'] = {
    'word': u'أفلمثلكما',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَمِثْلُكُمَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أفلمثلهما'] = {
    'word': u'أفلمثلهما',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَمِثْلُهُمَا',
    'stem': u'مِثْلُ',
    'type': u'STOPWORD',
    'original': u'مِثْلُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'مع'] = {
    'word': u'مَعَ',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة',
    'vocalized': u'مَعَ',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u''
}
STOPWORDS[u'لمع'] = {
    'word': u'لمع',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور',
    'vocalized': u'لَمَعَ',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u''
}
STOPWORDS[u'ومع'] = {
    'word': u'ومع',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف',
    'vocalized': u'وَمَعَ',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u''
}
STOPWORDS[u'فمع'] = {
    'word': u'فمع',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف',
    'vocalized': u'فَمَعَ',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u''
}
STOPWORDS[u'ولمع'] = {
    'word': u'ولمع',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'وَلَمَعَ',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u''
}
STOPWORDS[u'فلمع'] = {
    'word': u'فلمع',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'فَلَمَعَ',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u''
}
STOPWORDS[u'أمع'] = {
    'word': u'أمع',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور',
    'vocalized': u'أَمَعَ',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u''
}
STOPWORDS[u'ألمع'] = {
    'word': u'ألمع',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور',
    'vocalized': u'أَلَمَعَ',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u''
}
STOPWORDS[u'أومع'] = {
    'word': u'أومع',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور',
    'vocalized': u'أَوَمَعَ',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u''
}
STOPWORDS[u'أفمع'] = {
    'word': u'أفمع',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور',
    'vocalized': u'أَفَمَعَ',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u''
}
STOPWORDS[u'أولمع'] = {
    'word': u'أولمع',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَلَمَعَ',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u''
}
STOPWORDS[u'أفلمع'] = {
    'word': u'أفلمع',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَلَمَعَ',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u''
}
STOPWORDS[u'معي'] = {
    'word': u'معي',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'مَعَِي',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'ِي'
}
STOPWORDS[u'معك'] = {
    'word': u'معك',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'مَعَكَ',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'كَ'
}
STOPWORDS[u'معك'] = {
    'word': u'معك',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'مَعَكِ',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'كِ'
}
STOPWORDS[u'معه'] = {
    'word': u'معه',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'مَعَه',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'ه'
}
STOPWORDS[u'معكم'] = {
    'word': u'معكم',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'مَعَكُمْ',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'معكن'] = {
    'word': u'معكن',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'مَعَكُنَّ',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'معها'] = {
    'word': u'معها',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'مَعَهَا',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'هَا'
}
STOPWORDS[u'معهم'] = {
    'word': u'معهم',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'مَعَهُمْ',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'معهن'] = {
    'word': u'معهن',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'مَعَهُنَّ',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'معنا'] = {
    'word': u'معنا',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'مَعَنَا',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'نَا'
}
STOPWORDS[u'معكما'] = {
    'word': u'معكما',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'مَعَكُمَا',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'معهما'] = {
    'word': u'معهما',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'مَعَهُمَا',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'لمعي'] = {
    'word': u'لمعي',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَمَعَِي',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'ِي'
}
STOPWORDS[u'لمعك'] = {
    'word': u'لمعك',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَمَعَكَ',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'كَ'
}
STOPWORDS[u'لمعك'] = {
    'word': u'لمعك',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَمَعَكِ',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'كِ'
}
STOPWORDS[u'لمعه'] = {
    'word': u'لمعه',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَمَعَه',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'ه'
}
STOPWORDS[u'لمعكم'] = {
    'word': u'لمعكم',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَمَعَكُمْ',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'لمعكن'] = {
    'word': u'لمعكن',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَمَعَكُنَّ',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'لمعها'] = {
    'word': u'لمعها',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَمَعَهَا',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'هَا'
}
STOPWORDS[u'لمعهم'] = {
    'word': u'لمعهم',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَمَعَهُمْ',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'لمعهن'] = {
    'word': u'لمعهن',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَمَعَهُنَّ',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'لمعنا'] = {
    'word': u'لمعنا',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَمَعَنَا',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'نَا'
}
STOPWORDS[u'لمعكما'] = {
    'word': u'لمعكما',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَمَعَكُمَا',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'لمعهما'] = {
    'word': u'لمعهما',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَمَعَهُمَا',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ومعي'] = {
    'word': u'ومعي',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَمَعَِي',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'ِي'
}
STOPWORDS[u'ومعك'] = {
    'word': u'ومعك',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَمَعَكَ',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'كَ'
}
STOPWORDS[u'ومعك'] = {
    'word': u'ومعك',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَمَعَكِ',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'كِ'
}
STOPWORDS[u'ومعه'] = {
    'word': u'ومعه',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَمَعَه',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'ه'
}
STOPWORDS[u'ومعكم'] = {
    'word': u'ومعكم',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَمَعَكُمْ',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ومعكن'] = {
    'word': u'ومعكن',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَمَعَكُنَّ',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ومعها'] = {
    'word': u'ومعها',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَمَعَهَا',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'هَا'
}
STOPWORDS[u'ومعهم'] = {
    'word': u'ومعهم',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَمَعَهُمْ',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ومعهن'] = {
    'word': u'ومعهن',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَمَعَهُنَّ',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ومعنا'] = {
    'word': u'ومعنا',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَمَعَنَا',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'نَا'
}
STOPWORDS[u'ومعكما'] = {
    'word': u'ومعكما',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَمَعَكُمَا',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ومعهما'] = {
    'word': u'ومعهما',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَمَعَهُمَا',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فمعي'] = {
    'word': u'فمعي',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَمَعَِي',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'ِي'
}
STOPWORDS[u'فمعك'] = {
    'word': u'فمعك',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَمَعَكَ',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'كَ'
}
STOPWORDS[u'فمعك'] = {
    'word': u'فمعك',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَمَعَكِ',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'كِ'
}
STOPWORDS[u'فمعه'] = {
    'word': u'فمعه',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَمَعَه',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'ه'
}
STOPWORDS[u'فمعكم'] = {
    'word': u'فمعكم',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَمَعَكُمْ',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فمعكن'] = {
    'word': u'فمعكن',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَمَعَكُنَّ',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فمعها'] = {
    'word': u'فمعها',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَمَعَهَا',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'هَا'
}
STOPWORDS[u'فمعهم'] = {
    'word': u'فمعهم',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَمَعَهُمْ',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فمعهن'] = {
    'word': u'فمعهن',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَمَعَهُنَّ',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فمعنا'] = {
    'word': u'فمعنا',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَمَعَنَا',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'نَا'
}
STOPWORDS[u'فمعكما'] = {
    'word': u'فمعكما',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَمَعَكُمَا',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فمعهما'] = {
    'word': u'فمعهما',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَمَعَهُمَا',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ولمعي'] = {
    'word': u'ولمعي',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَمَعَِي',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'ِي'
}
STOPWORDS[u'ولمعك'] = {
    'word': u'ولمعك',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَمَعَكَ',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'كَ'
}
STOPWORDS[u'ولمعك'] = {
    'word': u'ولمعك',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَمَعَكِ',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'كِ'
}
STOPWORDS[u'ولمعه'] = {
    'word': u'ولمعه',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَمَعَه',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'ه'
}
STOPWORDS[u'ولمعكم'] = {
    'word': u'ولمعكم',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَمَعَكُمْ',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ولمعكن'] = {
    'word': u'ولمعكن',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَمَعَكُنَّ',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ولمعها'] = {
    'word': u'ولمعها',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَمَعَهَا',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'هَا'
}
STOPWORDS[u'ولمعهم'] = {
    'word': u'ولمعهم',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَمَعَهُمْ',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ولمعهن'] = {
    'word': u'ولمعهن',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَمَعَهُنَّ',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ولمعنا'] = {
    'word': u'ولمعنا',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَمَعَنَا',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'نَا'
}
STOPWORDS[u'ولمعكما'] = {
    'word': u'ولمعكما',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَمَعَكُمَا',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ولمعهما'] = {
    'word': u'ولمعهما',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَمَعَهُمَا',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فلمعي'] = {
    'word': u'فلمعي',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَمَعَِي',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'ِي'
}
STOPWORDS[u'فلمعك'] = {
    'word': u'فلمعك',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَمَعَكَ',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'كَ'
}
STOPWORDS[u'فلمعك'] = {
    'word': u'فلمعك',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَمَعَكِ',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'كِ'
}
STOPWORDS[u'فلمعه'] = {
    'word': u'فلمعه',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَمَعَه',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'ه'
}
STOPWORDS[u'فلمعكم'] = {
    'word': u'فلمعكم',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَمَعَكُمْ',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فلمعكن'] = {
    'word': u'فلمعكن',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَمَعَكُنَّ',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فلمعها'] = {
    'word': u'فلمعها',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَمَعَهَا',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'هَا'
}
STOPWORDS[u'فلمعهم'] = {
    'word': u'فلمعهم',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَمَعَهُمْ',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فلمعهن'] = {
    'word': u'فلمعهن',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَمَعَهُنَّ',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فلمعنا'] = {
    'word': u'فلمعنا',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَمَعَنَا',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'نَا'
}
STOPWORDS[u'فلمعكما'] = {
    'word': u'فلمعكما',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَمَعَكُمَا',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فلمعهما'] = {
    'word': u'فلمعهما',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَمَعَهُمَا',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أمعي'] = {
    'word': u'أمعي',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'أَمَعَِي',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أمعك'] = {
    'word': u'أمعك',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'أَمَعَكَ',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أمعك'] = {
    'word': u'أمعك',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'أَمَعَكِ',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أمعه'] = {
    'word': u'أمعه',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'أَمَعَه',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'ه'
}
STOPWORDS[u'أمعكم'] = {
    'word': u'أمعكم',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'أَمَعَكُمْ',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أمعكن'] = {
    'word': u'أمعكن',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'أَمَعَكُنَّ',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أمعها'] = {
    'word': u'أمعها',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'أَمَعَهَا',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أمعهم'] = {
    'word': u'أمعهم',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'أَمَعَهُمْ',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أمعهن'] = {
    'word': u'أمعهن',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'أَمَعَهُنَّ',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أمعنا'] = {
    'word': u'أمعنا',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'أَمَعَنَا',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أمعكما'] = {
    'word': u'أمعكما',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'أَمَعَكُمَا',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أمعهما'] = {
    'word': u'أمعهما',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'أَمَعَهُمَا',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ألمعي'] = {
    'word': u'ألمعي',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَمَعَِي',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'ِي'
}
STOPWORDS[u'ألمعك'] = {
    'word': u'ألمعك',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَمَعَكَ',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'كَ'
}
STOPWORDS[u'ألمعك'] = {
    'word': u'ألمعك',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَمَعَكِ',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'كِ'
}
STOPWORDS[u'ألمعه'] = {
    'word': u'ألمعه',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَمَعَه',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'ه'
}
STOPWORDS[u'ألمعكم'] = {
    'word': u'ألمعكم',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَمَعَكُمْ',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ألمعكن'] = {
    'word': u'ألمعكن',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَمَعَكُنَّ',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ألمعها'] = {
    'word': u'ألمعها',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَمَعَهَا',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'هَا'
}
STOPWORDS[u'ألمعهم'] = {
    'word': u'ألمعهم',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَمَعَهُمْ',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ألمعهن'] = {
    'word': u'ألمعهن',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَمَعَهُنَّ',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ألمعنا'] = {
    'word': u'ألمعنا',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَمَعَنَا',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'نَا'
}
STOPWORDS[u'ألمعكما'] = {
    'word': u'ألمعكما',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَمَعَكُمَا',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ألمعهما'] = {
    'word': u'ألمعهما',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَمَعَهُمَا',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أومعي'] = {
    'word': u'أومعي',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَمَعَِي',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أومعك'] = {
    'word': u'أومعك',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَمَعَكَ',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أومعك'] = {
    'word': u'أومعك',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَمَعَكِ',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أومعه'] = {
    'word': u'أومعه',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَمَعَه',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'ه'
}
STOPWORDS[u'أومعكم'] = {
    'word': u'أومعكم',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَمَعَكُمْ',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أومعكن'] = {
    'word': u'أومعكن',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَمَعَكُنَّ',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أومعها'] = {
    'word': u'أومعها',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَمَعَهَا',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أومعهم'] = {
    'word': u'أومعهم',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَمَعَهُمْ',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أومعهن'] = {
    'word': u'أومعهن',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَمَعَهُنَّ',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أومعنا'] = {
    'word': u'أومعنا',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَمَعَنَا',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أومعكما'] = {
    'word': u'أومعكما',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَمَعَكُمَا',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أومعهما'] = {
    'word': u'أومعهما',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَمَعَهُمَا',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أفمعي'] = {
    'word': u'أفمعي',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَمَعَِي',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أفمعك'] = {
    'word': u'أفمعك',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَمَعَكَ',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أفمعك'] = {
    'word': u'أفمعك',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَمَعَكِ',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أفمعه'] = {
    'word': u'أفمعه',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَمَعَه',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'ه'
}
STOPWORDS[u'أفمعكم'] = {
    'word': u'أفمعكم',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَمَعَكُمْ',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أفمعكن'] = {
    'word': u'أفمعكن',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَمَعَكُنَّ',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أفمعها'] = {
    'word': u'أفمعها',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَمَعَهَا',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أفمعهم'] = {
    'word': u'أفمعهم',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَمَعَهُمْ',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أفمعهن'] = {
    'word': u'أفمعهن',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَمَعَهُنَّ',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أفمعنا'] = {
    'word': u'أفمعنا',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَمَعَنَا',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أفمعكما'] = {
    'word': u'أفمعكما',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَمَعَكُمَا',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أفمعهما'] = {
    'word': u'أفمعهما',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَمَعَهُمَا',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أولمعي'] = {
    'word': u'أولمعي',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَمَعَِي',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أولمعك'] = {
    'word': u'أولمعك',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَمَعَكَ',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أولمعك'] = {
    'word': u'أولمعك',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَمَعَكِ',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أولمعه'] = {
    'word': u'أولمعه',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَمَعَه',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'ه'
}
STOPWORDS[u'أولمعكم'] = {
    'word': u'أولمعكم',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَمَعَكُمْ',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أولمعكن'] = {
    'word': u'أولمعكن',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَمَعَكُنَّ',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أولمعها'] = {
    'word': u'أولمعها',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَمَعَهَا',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أولمعهم'] = {
    'word': u'أولمعهم',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَمَعَهُمْ',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أولمعهن'] = {
    'word': u'أولمعهن',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَمَعَهُنَّ',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أولمعنا'] = {
    'word': u'أولمعنا',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَمَعَنَا',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أولمعكما'] = {
    'word': u'أولمعكما',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَمَعَكُمَا',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أولمعهما'] = {
    'word': u'أولمعهما',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَمَعَهُمَا',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أفلمعي'] = {
    'word': u'أفلمعي',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَمَعَِي',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أفلمعك'] = {
    'word': u'أفلمعك',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَمَعَكَ',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أفلمعك'] = {
    'word': u'أفلمعك',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَمَعَكِ',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أفلمعه'] = {
    'word': u'أفلمعه',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَمَعَه',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'ه'
}
STOPWORDS[u'أفلمعكم'] = {
    'word': u'أفلمعكم',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَمَعَكُمْ',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أفلمعكن'] = {
    'word': u'أفلمعكن',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَمَعَكُنَّ',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أفلمعها'] = {
    'word': u'أفلمعها',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَمَعَهَا',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أفلمعهم'] = {
    'word': u'أفلمعهم',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَمَعَهُمْ',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أفلمعهن'] = {
    'word': u'أفلمعهن',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَمَعَهُنَّ',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أفلمعنا'] = {
    'word': u'أفلمعنا',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَمَعَنَا',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أفلمعكما'] = {
    'word': u'أفلمعكما',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَمَعَكُمَا',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أفلمعهما'] = {
    'word': u'أفلمعهما',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَمَعَهُمَا',
    'stem': u'مَعَ',
    'type': u'STOPWORD',
    'original': u'مَعَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'معاذ'] = {
    'word': u'مَعَاذَ',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة',
    'vocalized': u'مَعَاذَ',
    'stem': u'مَعَاذَ',
    'type': u'STOPWORD',
    'original': u'مَعَاذَ',
    'encletic': u''
}
STOPWORDS[u'ومعاذ'] = {
    'word': u'ومعاذ',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف',
    'vocalized': u'وَمَعَاذَ',
    'stem': u'مَعَاذَ',
    'type': u'STOPWORD',
    'original': u'مَعَاذَ',
    'encletic': u''
}
STOPWORDS[u'فمعاذ'] = {
    'word': u'فمعاذ',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف',
    'vocalized': u'فَمَعَاذَ',
    'stem': u'مَعَاذَ',
    'type': u'STOPWORD',
    'original': u'مَعَاذَ',
    'encletic': u''
}
STOPWORDS[u'معاذي'] = {
    'word': u'معاذي',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'مَعَاذَِي',
    'stem': u'مَعَاذَ',
    'type': u'STOPWORD',
    'original': u'مَعَاذَ',
    'encletic': u'ِي'
}
STOPWORDS[u'معاذك'] = {
    'word': u'معاذك',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'مَعَاذَكَ',
    'stem': u'مَعَاذَ',
    'type': u'STOPWORD',
    'original': u'مَعَاذَ',
    'encletic': u'كَ'
}
STOPWORDS[u'معاذك'] = {
    'word': u'معاذك',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'مَعَاذَكِ',
    'stem': u'مَعَاذَ',
    'type': u'STOPWORD',
    'original': u'مَعَاذَ',
    'encletic': u'كِ'
}
STOPWORDS[u'معاذه'] = {
    'word': u'معاذه',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'مَعَاذَه',
    'stem': u'مَعَاذَ',
    'type': u'STOPWORD',
    'original': u'مَعَاذَ',
    'encletic': u'ه'
}
STOPWORDS[u'معاذكم'] = {
    'word': u'معاذكم',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'مَعَاذَكُمْ',
    'stem': u'مَعَاذَ',
    'type': u'STOPWORD',
    'original': u'مَعَاذَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'معاذكن'] = {
    'word': u'معاذكن',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'مَعَاذَكُنَّ',
    'stem': u'مَعَاذَ',
    'type': u'STOPWORD',
    'original': u'مَعَاذَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'معاذها'] = {
    'word': u'معاذها',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'مَعَاذَهَا',
    'stem': u'مَعَاذَ',
    'type': u'STOPWORD',
    'original': u'مَعَاذَ',
    'encletic': u'هَا'
}
STOPWORDS[u'معاذهم'] = {
    'word': u'معاذهم',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'مَعَاذَهُمْ',
    'stem': u'مَعَاذَ',
    'type': u'STOPWORD',
    'original': u'مَعَاذَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'معاذهن'] = {
    'word': u'معاذهن',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'مَعَاذَهُنَّ',
    'stem': u'مَعَاذَ',
    'type': u'STOPWORD',
    'original': u'مَعَاذَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'معاذنا'] = {
    'word': u'معاذنا',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'مَعَاذَنَا',
    'stem': u'مَعَاذَ',
    'type': u'STOPWORD',
    'original': u'مَعَاذَ',
    'encletic': u'نَا'
}
STOPWORDS[u'معاذكما'] = {
    'word': u'معاذكما',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'مَعَاذَكُمَا',
    'stem': u'مَعَاذَ',
    'type': u'STOPWORD',
    'original': u'مَعَاذَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'معاذهما'] = {
    'word': u'معاذهما',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'مَعَاذَهُمَا',
    'stem': u'مَعَاذَ',
    'type': u'STOPWORD',
    'original': u'مَعَاذَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ومعاذي'] = {
    'word': u'ومعاذي',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَمَعَاذَِي',
    'stem': u'مَعَاذَ',
    'type': u'STOPWORD',
    'original': u'مَعَاذَ',
    'encletic': u'ِي'
}
STOPWORDS[u'ومعاذك'] = {
    'word': u'ومعاذك',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَمَعَاذَكَ',
    'stem': u'مَعَاذَ',
    'type': u'STOPWORD',
    'original': u'مَعَاذَ',
    'encletic': u'كَ'
}
STOPWORDS[u'ومعاذك'] = {
    'word': u'ومعاذك',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَمَعَاذَكِ',
    'stem': u'مَعَاذَ',
    'type': u'STOPWORD',
    'original': u'مَعَاذَ',
    'encletic': u'كِ'
}
STOPWORDS[u'ومعاذه'] = {
    'word': u'ومعاذه',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَمَعَاذَه',
    'stem': u'مَعَاذَ',
    'type': u'STOPWORD',
    'original': u'مَعَاذَ',
    'encletic': u'ه'
}
STOPWORDS[u'ومعاذكم'] = {
    'word': u'ومعاذكم',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَمَعَاذَكُمْ',
    'stem': u'مَعَاذَ',
    'type': u'STOPWORD',
    'original': u'مَعَاذَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ومعاذكن'] = {
    'word': u'ومعاذكن',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَمَعَاذَكُنَّ',
    'stem': u'مَعَاذَ',
    'type': u'STOPWORD',
    'original': u'مَعَاذَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ومعاذها'] = {
    'word': u'ومعاذها',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَمَعَاذَهَا',
    'stem': u'مَعَاذَ',
    'type': u'STOPWORD',
    'original': u'مَعَاذَ',
    'encletic': u'هَا'
}
STOPWORDS[u'ومعاذهم'] = {
    'word': u'ومعاذهم',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَمَعَاذَهُمْ',
    'stem': u'مَعَاذَ',
    'type': u'STOPWORD',
    'original': u'مَعَاذَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ومعاذهن'] = {
    'word': u'ومعاذهن',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَمَعَاذَهُنَّ',
    'stem': u'مَعَاذَ',
    'type': u'STOPWORD',
    'original': u'مَعَاذَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ومعاذنا'] = {
    'word': u'ومعاذنا',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَمَعَاذَنَا',
    'stem': u'مَعَاذَ',
    'type': u'STOPWORD',
    'original': u'مَعَاذَ',
    'encletic': u'نَا'
}
STOPWORDS[u'ومعاذكما'] = {
    'word': u'ومعاذكما',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَمَعَاذَكُمَا',
    'stem': u'مَعَاذَ',
    'type': u'STOPWORD',
    'original': u'مَعَاذَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ومعاذهما'] = {
    'word': u'ومعاذهما',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَمَعَاذَهُمَا',
    'stem': u'مَعَاذَ',
    'type': u'STOPWORD',
    'original': u'مَعَاذَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فمعاذي'] = {
    'word': u'فمعاذي',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَمَعَاذَِي',
    'stem': u'مَعَاذَ',
    'type': u'STOPWORD',
    'original': u'مَعَاذَ',
    'encletic': u'ِي'
}
STOPWORDS[u'فمعاذك'] = {
    'word': u'فمعاذك',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَمَعَاذَكَ',
    'stem': u'مَعَاذَ',
    'type': u'STOPWORD',
    'original': u'مَعَاذَ',
    'encletic': u'كَ'
}
STOPWORDS[u'فمعاذك'] = {
    'word': u'فمعاذك',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَمَعَاذَكِ',
    'stem': u'مَعَاذَ',
    'type': u'STOPWORD',
    'original': u'مَعَاذَ',
    'encletic': u'كِ'
}
STOPWORDS[u'فمعاذه'] = {
    'word': u'فمعاذه',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَمَعَاذَه',
    'stem': u'مَعَاذَ',
    'type': u'STOPWORD',
    'original': u'مَعَاذَ',
    'encletic': u'ه'
}
STOPWORDS[u'فمعاذكم'] = {
    'word': u'فمعاذكم',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَمَعَاذَكُمْ',
    'stem': u'مَعَاذَ',
    'type': u'STOPWORD',
    'original': u'مَعَاذَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فمعاذكن'] = {
    'word': u'فمعاذكن',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَمَعَاذَكُنَّ',
    'stem': u'مَعَاذَ',
    'type': u'STOPWORD',
    'original': u'مَعَاذَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فمعاذها'] = {
    'word': u'فمعاذها',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَمَعَاذَهَا',
    'stem': u'مَعَاذَ',
    'type': u'STOPWORD',
    'original': u'مَعَاذَ',
    'encletic': u'هَا'
}
STOPWORDS[u'فمعاذهم'] = {
    'word': u'فمعاذهم',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَمَعَاذَهُمْ',
    'stem': u'مَعَاذَ',
    'type': u'STOPWORD',
    'original': u'مَعَاذَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فمعاذهن'] = {
    'word': u'فمعاذهن',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَمَعَاذَهُنَّ',
    'stem': u'مَعَاذَ',
    'type': u'STOPWORD',
    'original': u'مَعَاذَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فمعاذنا'] = {
    'word': u'فمعاذنا',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَمَعَاذَنَا',
    'stem': u'مَعَاذَ',
    'type': u'STOPWORD',
    'original': u'مَعَاذَ',
    'encletic': u'نَا'
}
STOPWORDS[u'فمعاذكما'] = {
    'word': u'فمعاذكما',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَمَعَاذَكُمَا',
    'stem': u'مَعَاذَ',
    'type': u'STOPWORD',
    'original': u'مَعَاذَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فمعاذهما'] = {
    'word': u'فمعاذهما',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَمَعَاذَهُمَا',
    'stem': u'مَعَاذَ',
    'type': u'STOPWORD',
    'original': u'مَعَاذَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'نحو'] = {
    'word': u'نَحْوَ',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة',
    'vocalized': u'نَحْوَ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u''
}
STOPWORDS[u'بنحو'] = {
    'word': u'بنحو',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور',
    'vocalized': u'بِنَحْوَ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u''
}
STOPWORDS[u'كنحو'] = {
    'word': u'كنحو',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور',
    'vocalized': u'كَنَحْوَ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u''
}
STOPWORDS[u'لنحو'] = {
    'word': u'لنحو',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور',
    'vocalized': u'لِنَحْوَ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u''
}
STOPWORDS[u'ونحو'] = {
    'word': u'ونحو',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف',
    'vocalized': u'وَنَحْوَ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u''
}
STOPWORDS[u'فنحو'] = {
    'word': u'فنحو',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف',
    'vocalized': u'فَنَحْوَ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u''
}
STOPWORDS[u'وبنحو'] = {
    'word': u'وبنحو',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف',
    'vocalized': u'وَبِنَحْوَ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u''
}
STOPWORDS[u'فبنحو'] = {
    'word': u'فبنحو',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف',
    'vocalized': u'فَبِنَحْوَ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u''
}
STOPWORDS[u'وكنحو'] = {
    'word': u'وكنحو',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف',
    'vocalized': u'وَكَنَحْوَ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u''
}
STOPWORDS[u'فكنحو'] = {
    'word': u'فكنحو',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف',
    'vocalized': u'فَكَنَحْوَ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u''
}
STOPWORDS[u'ولنحو'] = {
    'word': u'ولنحو',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف',
    'vocalized': u'وَلِنَحْوَ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u''
}
STOPWORDS[u'فلنحو'] = {
    'word': u'فلنحو',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف',
    'vocalized': u'فَلِنَحْوَ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u''
}
STOPWORDS[u'أنحو'] = {
    'word': u'أنحو',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور',
    'vocalized': u'أَنَحْوَ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u''
}
STOPWORDS[u'أبنحو'] = {
    'word': u'أبنحو',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور',
    'vocalized': u'أَبِنَحْوَ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u''
}
STOPWORDS[u'أكنحو'] = {
    'word': u'أكنحو',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور',
    'vocalized': u'أَكَنَحْوَ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u''
}
STOPWORDS[u'ألنحو'] = {
    'word': u'ألنحو',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور',
    'vocalized': u'أَلِنَحْوَ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u''
}
STOPWORDS[u'أونحو'] = {
    'word': u'أونحو',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور',
    'vocalized': u'أَوَنَحْوَ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u''
}
STOPWORDS[u'أفنحو'] = {
    'word': u'أفنحو',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور',
    'vocalized': u'أَفَنَحْوَ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u''
}
STOPWORDS[u'أوبنحو'] = {
    'word': u'أوبنحو',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَبِنَحْوَ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u''
}
STOPWORDS[u'أفبنحو'] = {
    'word': u'أفبنحو',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَبِنَحْوَ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u''
}
STOPWORDS[u'أوكنحو'] = {
    'word': u'أوكنحو',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَكَنَحْوَ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u''
}
STOPWORDS[u'أفكنحو'] = {
    'word': u'أفكنحو',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَكَنَحْوَ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u''
}
STOPWORDS[u'أولنحو'] = {
    'word': u'أولنحو',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَلِنَحْوَ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u''
}
STOPWORDS[u'أفلنحو'] = {
    'word': u'أفلنحو',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَلِنَحْوَ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u''
}
STOPWORDS[u'نحوي'] = {
    'word': u'نحوي',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'نَحْوَِي',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'ِي'
}
STOPWORDS[u'نحوك'] = {
    'word': u'نحوك',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'نَحْوَكَ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كَ'
}
STOPWORDS[u'نحوك'] = {
    'word': u'نحوك',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'نَحْوَكِ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كِ'
}
STOPWORDS[u'نحوه'] = {
    'word': u'نحوه',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'نَحْوَه',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'ه'
}
STOPWORDS[u'نحوكم'] = {
    'word': u'نحوكم',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'نَحْوَكُمْ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'نحوكن'] = {
    'word': u'نحوكن',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'نَحْوَكُنَّ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'نحوها'] = {
    'word': u'نحوها',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'نَحْوَهَا',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'هَا'
}
STOPWORDS[u'نحوهم'] = {
    'word': u'نحوهم',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'نَحْوَهُمْ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'نحوهن'] = {
    'word': u'نحوهن',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'نَحْوَهُنَّ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'نحونا'] = {
    'word': u'نحونا',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'نَحْوَنَا',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'نَا'
}
STOPWORDS[u'نحوكما'] = {
    'word': u'نحوكما',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'نَحْوَكُمَا',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'نحوهما'] = {
    'word': u'نحوهما',
    'procletic': u'',
    'tags': u'أداة:اسم إضافة:مضاف',
    'vocalized': u'نَحْوَهُمَا',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'بنحوي'] = {
    'word': u'بنحوي',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِنَحْوَِي',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'ِي'
}
STOPWORDS[u'بنحوك'] = {
    'word': u'بنحوك',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِنَحْوَكَ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كَ'
}
STOPWORDS[u'بنحوك'] = {
    'word': u'بنحوك',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِنَحْوَكِ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كِ'
}
STOPWORDS[u'بنحوه'] = {
    'word': u'بنحوه',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِنَحْوَه',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'ه'
}
STOPWORDS[u'بنحوكم'] = {
    'word': u'بنحوكم',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِنَحْوَكُمْ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'بنحوكن'] = {
    'word': u'بنحوكن',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِنَحْوَكُنَّ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'بنحوها'] = {
    'word': u'بنحوها',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِنَحْوَهَا',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'هَا'
}
STOPWORDS[u'بنحوهم'] = {
    'word': u'بنحوهم',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِنَحْوَهُمْ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'بنحوهن'] = {
    'word': u'بنحوهن',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِنَحْوَهُنَّ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'بنحونا'] = {
    'word': u'بنحونا',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِنَحْوَنَا',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'نَا'
}
STOPWORDS[u'بنحوكما'] = {
    'word': u'بنحوكما',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِنَحْوَكُمَا',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'بنحوهما'] = {
    'word': u'بنحوهما',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'بِنَحْوَهُمَا',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'كنحوي'] = {
    'word': u'كنحوي',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَنَحْوَِي',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'ِي'
}
STOPWORDS[u'كنحوك'] = {
    'word': u'كنحوك',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَنَحْوَكَ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كَ'
}
STOPWORDS[u'كنحوك'] = {
    'word': u'كنحوك',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَنَحْوَكِ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كِ'
}
STOPWORDS[u'كنحوه'] = {
    'word': u'كنحوه',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَنَحْوَه',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'ه'
}
STOPWORDS[u'كنحوكم'] = {
    'word': u'كنحوكم',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَنَحْوَكُمْ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'كنحوكن'] = {
    'word': u'كنحوكن',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَنَحْوَكُنَّ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'كنحوها'] = {
    'word': u'كنحوها',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَنَحْوَهَا',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'هَا'
}
STOPWORDS[u'كنحوهم'] = {
    'word': u'كنحوهم',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَنَحْوَهُمْ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'كنحوهن'] = {
    'word': u'كنحوهن',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَنَحْوَهُنَّ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'كنحونا'] = {
    'word': u'كنحونا',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَنَحْوَنَا',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'نَا'
}
STOPWORDS[u'كنحوكما'] = {
    'word': u'كنحوكما',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَنَحْوَكُمَا',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'كنحوهما'] = {
    'word': u'كنحوهما',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'كَنَحْوَهُمَا',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'لنحوي'] = {
    'word': u'لنحوي',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِنَحْوَِي',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'ِي'
}
STOPWORDS[u'لنحوك'] = {
    'word': u'لنحوك',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِنَحْوَكَ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كَ'
}
STOPWORDS[u'لنحوك'] = {
    'word': u'لنحوك',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِنَحْوَكِ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كِ'
}
STOPWORDS[u'لنحوه'] = {
    'word': u'لنحوه',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِنَحْوَه',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'ه'
}
STOPWORDS[u'لنحوكم'] = {
    'word': u'لنحوكم',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِنَحْوَكُمْ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'لنحوكن'] = {
    'word': u'لنحوكن',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِنَحْوَكُنَّ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'لنحوها'] = {
    'word': u'لنحوها',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِنَحْوَهَا',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'هَا'
}
STOPWORDS[u'لنحوهم'] = {
    'word': u'لنحوهم',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِنَحْوَهُمْ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'لنحوهن'] = {
    'word': u'لنحوهن',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِنَحْوَهُنَّ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'لنحونا'] = {
    'word': u'لنحونا',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِنَحْوَنَا',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'نَا'
}
STOPWORDS[u'لنحوكما'] = {
    'word': u'لنحوكما',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِنَحْوَكُمَا',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'لنحوهما'] = {
    'word': u'لنحوهما',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'لِنَحْوَهُمَا',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ونحوي'] = {
    'word': u'ونحوي',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَنَحْوَِي',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'ِي'
}
STOPWORDS[u'ونحوك'] = {
    'word': u'ونحوك',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَنَحْوَكَ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كَ'
}
STOPWORDS[u'ونحوك'] = {
    'word': u'ونحوك',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَنَحْوَكِ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كِ'
}
STOPWORDS[u'ونحوه'] = {
    'word': u'ونحوه',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَنَحْوَه',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'ه'
}
STOPWORDS[u'ونحوكم'] = {
    'word': u'ونحوكم',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَنَحْوَكُمْ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ونحوكن'] = {
    'word': u'ونحوكن',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَنَحْوَكُنَّ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ونحوها'] = {
    'word': u'ونحوها',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَنَحْوَهَا',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'هَا'
}
STOPWORDS[u'ونحوهم'] = {
    'word': u'ونحوهم',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَنَحْوَهُمْ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ونحوهن'] = {
    'word': u'ونحوهن',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَنَحْوَهُنَّ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ونحونا'] = {
    'word': u'ونحونا',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَنَحْوَنَا',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'نَا'
}
STOPWORDS[u'ونحوكما'] = {
    'word': u'ونحوكما',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَنَحْوَكُمَا',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ونحوهما'] = {
    'word': u'ونحوهما',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'وَنَحْوَهُمَا',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فنحوي'] = {
    'word': u'فنحوي',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَنَحْوَِي',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'ِي'
}
STOPWORDS[u'فنحوك'] = {
    'word': u'فنحوك',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَنَحْوَكَ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كَ'
}
STOPWORDS[u'فنحوك'] = {
    'word': u'فنحوك',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَنَحْوَكِ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كِ'
}
STOPWORDS[u'فنحوه'] = {
    'word': u'فنحوه',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَنَحْوَه',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'ه'
}
STOPWORDS[u'فنحوكم'] = {
    'word': u'فنحوكم',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَنَحْوَكُمْ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فنحوكن'] = {
    'word': u'فنحوكن',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَنَحْوَكُنَّ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فنحوها'] = {
    'word': u'فنحوها',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَنَحْوَهَا',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'هَا'
}
STOPWORDS[u'فنحوهم'] = {
    'word': u'فنحوهم',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَنَحْوَهُمْ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فنحوهن'] = {
    'word': u'فنحوهن',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَنَحْوَهُنَّ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فنحونا'] = {
    'word': u'فنحونا',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَنَحْوَنَا',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'نَا'
}
STOPWORDS[u'فنحوكما'] = {
    'word': u'فنحوكما',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَنَحْوَكُمَا',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فنحوهما'] = {
    'word': u'فنحوهما',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم إضافة:معطوف:مضاف',
    'vocalized': u'فَنَحْوَهُمَا',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'وبنحوي'] = {
    'word': u'وبنحوي',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِنَحْوَِي',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'ِي'
}
STOPWORDS[u'وبنحوك'] = {
    'word': u'وبنحوك',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِنَحْوَكَ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كَ'
}
STOPWORDS[u'وبنحوك'] = {
    'word': u'وبنحوك',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِنَحْوَكِ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كِ'
}
STOPWORDS[u'وبنحوه'] = {
    'word': u'وبنحوه',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِنَحْوَه',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'ه'
}
STOPWORDS[u'وبنحوكم'] = {
    'word': u'وبنحوكم',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِنَحْوَكُمْ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'وبنحوكن'] = {
    'word': u'وبنحوكن',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِنَحْوَكُنَّ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'وبنحوها'] = {
    'word': u'وبنحوها',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِنَحْوَهَا',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'هَا'
}
STOPWORDS[u'وبنحوهم'] = {
    'word': u'وبنحوهم',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِنَحْوَهُمْ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'وبنحوهن'] = {
    'word': u'وبنحوهن',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِنَحْوَهُنَّ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'وبنحونا'] = {
    'word': u'وبنحونا',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِنَحْوَنَا',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'نَا'
}
STOPWORDS[u'وبنحوكما'] = {
    'word': u'وبنحوكما',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِنَحْوَكُمَا',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'وبنحوهما'] = {
    'word': u'وبنحوهما',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِنَحْوَهُمَا',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فبنحوي'] = {
    'word': u'فبنحوي',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِنَحْوَِي',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'ِي'
}
STOPWORDS[u'فبنحوك'] = {
    'word': u'فبنحوك',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِنَحْوَكَ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كَ'
}
STOPWORDS[u'فبنحوك'] = {
    'word': u'فبنحوك',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِنَحْوَكِ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كِ'
}
STOPWORDS[u'فبنحوه'] = {
    'word': u'فبنحوه',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِنَحْوَه',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'ه'
}
STOPWORDS[u'فبنحوكم'] = {
    'word': u'فبنحوكم',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِنَحْوَكُمْ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فبنحوكن'] = {
    'word': u'فبنحوكن',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِنَحْوَكُنَّ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فبنحوها'] = {
    'word': u'فبنحوها',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِنَحْوَهَا',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'هَا'
}
STOPWORDS[u'فبنحوهم'] = {
    'word': u'فبنحوهم',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِنَحْوَهُمْ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فبنحوهن'] = {
    'word': u'فبنحوهن',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِنَحْوَهُنَّ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فبنحونا'] = {
    'word': u'فبنحونا',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِنَحْوَنَا',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'نَا'
}
STOPWORDS[u'فبنحوكما'] = {
    'word': u'فبنحوكما',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِنَحْوَكُمَا',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فبنحوهما'] = {
    'word': u'فبنحوهما',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِنَحْوَهُمَا',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'وكنحوي'] = {
    'word': u'وكنحوي',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَنَحْوَِي',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'ِي'
}
STOPWORDS[u'وكنحوك'] = {
    'word': u'وكنحوك',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَنَحْوَكَ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كَ'
}
STOPWORDS[u'وكنحوك'] = {
    'word': u'وكنحوك',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَنَحْوَكِ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كِ'
}
STOPWORDS[u'وكنحوه'] = {
    'word': u'وكنحوه',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَنَحْوَه',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'ه'
}
STOPWORDS[u'وكنحوكم'] = {
    'word': u'وكنحوكم',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَنَحْوَكُمْ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'وكنحوكن'] = {
    'word': u'وكنحوكن',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَنَحْوَكُنَّ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'وكنحوها'] = {
    'word': u'وكنحوها',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَنَحْوَهَا',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'هَا'
}
STOPWORDS[u'وكنحوهم'] = {
    'word': u'وكنحوهم',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَنَحْوَهُمْ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'وكنحوهن'] = {
    'word': u'وكنحوهن',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَنَحْوَهُنَّ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'وكنحونا'] = {
    'word': u'وكنحونا',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَنَحْوَنَا',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'نَا'
}
STOPWORDS[u'وكنحوكما'] = {
    'word': u'وكنحوكما',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَنَحْوَكُمَا',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'وكنحوهما'] = {
    'word': u'وكنحوهما',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَنَحْوَهُمَا',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فكنحوي'] = {
    'word': u'فكنحوي',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَنَحْوَِي',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'ِي'
}
STOPWORDS[u'فكنحوك'] = {
    'word': u'فكنحوك',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَنَحْوَكَ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كَ'
}
STOPWORDS[u'فكنحوك'] = {
    'word': u'فكنحوك',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَنَحْوَكِ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كِ'
}
STOPWORDS[u'فكنحوه'] = {
    'word': u'فكنحوه',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَنَحْوَه',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'ه'
}
STOPWORDS[u'فكنحوكم'] = {
    'word': u'فكنحوكم',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَنَحْوَكُمْ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فكنحوكن'] = {
    'word': u'فكنحوكن',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَنَحْوَكُنَّ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فكنحوها'] = {
    'word': u'فكنحوها',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَنَحْوَهَا',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'هَا'
}
STOPWORDS[u'فكنحوهم'] = {
    'word': u'فكنحوهم',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَنَحْوَهُمْ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فكنحوهن'] = {
    'word': u'فكنحوهن',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَنَحْوَهُنَّ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فكنحونا'] = {
    'word': u'فكنحونا',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَنَحْوَنَا',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'نَا'
}
STOPWORDS[u'فكنحوكما'] = {
    'word': u'فكنحوكما',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَنَحْوَكُمَا',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فكنحوهما'] = {
    'word': u'فكنحوهما',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَنَحْوَهُمَا',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ولنحوي'] = {
    'word': u'ولنحوي',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِنَحْوَِي',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'ِي'
}
STOPWORDS[u'ولنحوك'] = {
    'word': u'ولنحوك',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِنَحْوَكَ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كَ'
}
STOPWORDS[u'ولنحوك'] = {
    'word': u'ولنحوك',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِنَحْوَكِ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كِ'
}
STOPWORDS[u'ولنحوه'] = {
    'word': u'ولنحوه',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِنَحْوَه',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'ه'
}
STOPWORDS[u'ولنحوكم'] = {
    'word': u'ولنحوكم',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِنَحْوَكُمْ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ولنحوكن'] = {
    'word': u'ولنحوكن',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِنَحْوَكُنَّ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ولنحوها'] = {
    'word': u'ولنحوها',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِنَحْوَهَا',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'هَا'
}
STOPWORDS[u'ولنحوهم'] = {
    'word': u'ولنحوهم',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِنَحْوَهُمْ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ولنحوهن'] = {
    'word': u'ولنحوهن',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِنَحْوَهُنَّ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ولنحونا'] = {
    'word': u'ولنحونا',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِنَحْوَنَا',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'نَا'
}
STOPWORDS[u'ولنحوكما'] = {
    'word': u'ولنحوكما',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِنَحْوَكُمَا',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ولنحوهما'] = {
    'word': u'ولنحوهما',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِنَحْوَهُمَا',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فلنحوي'] = {
    'word': u'فلنحوي',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِنَحْوَِي',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'ِي'
}
STOPWORDS[u'فلنحوك'] = {
    'word': u'فلنحوك',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِنَحْوَكَ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كَ'
}
STOPWORDS[u'فلنحوك'] = {
    'word': u'فلنحوك',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِنَحْوَكِ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كِ'
}
STOPWORDS[u'فلنحوه'] = {
    'word': u'فلنحوه',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِنَحْوَه',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'ه'
}
STOPWORDS[u'فلنحوكم'] = {
    'word': u'فلنحوكم',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِنَحْوَكُمْ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فلنحوكن'] = {
    'word': u'فلنحوكن',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِنَحْوَكُنَّ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فلنحوها'] = {
    'word': u'فلنحوها',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِنَحْوَهَا',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'هَا'
}
STOPWORDS[u'فلنحوهم'] = {
    'word': u'فلنحوهم',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِنَحْوَهُمْ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فلنحوهن'] = {
    'word': u'فلنحوهن',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِنَحْوَهُنَّ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فلنحونا'] = {
    'word': u'فلنحونا',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِنَحْوَنَا',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'نَا'
}
STOPWORDS[u'فلنحوكما'] = {
    'word': u'فلنحوكما',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِنَحْوَكُمَا',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فلنحوهما'] = {
    'word': u'فلنحوهما',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِنَحْوَهُمَا',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أنحوي'] = {
    'word': u'أنحوي',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'أَنَحْوَِي',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أنحوك'] = {
    'word': u'أنحوك',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'أَنَحْوَكَ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أنحوك'] = {
    'word': u'أنحوك',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'أَنَحْوَكِ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أنحوه'] = {
    'word': u'أنحوه',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'أَنَحْوَه',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'ه'
}
STOPWORDS[u'أنحوكم'] = {
    'word': u'أنحوكم',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'أَنَحْوَكُمْ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أنحوكن'] = {
    'word': u'أنحوكن',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'أَنَحْوَكُنَّ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أنحوها'] = {
    'word': u'أنحوها',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'أَنَحْوَهَا',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أنحوهم'] = {
    'word': u'أنحوهم',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'أَنَحْوَهُمْ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أنحوهن'] = {
    'word': u'أنحوهن',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'أَنَحْوَهُنَّ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أنحونا'] = {
    'word': u'أنحونا',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'أَنَحْوَنَا',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أنحوكما'] = {
    'word': u'أنحوكما',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'أَنَحْوَكُمَا',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أنحوهما'] = {
    'word': u'أنحوهما',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مضاف',
    'vocalized': u'أَنَحْوَهُمَا',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أبنحوي'] = {
    'word': u'أبنحوي',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِنَحْوَِي',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أبنحوك'] = {
    'word': u'أبنحوك',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِنَحْوَكَ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أبنحوك'] = {
    'word': u'أبنحوك',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِنَحْوَكِ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أبنحوه'] = {
    'word': u'أبنحوه',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِنَحْوَه',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'ه'
}
STOPWORDS[u'أبنحوكم'] = {
    'word': u'أبنحوكم',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِنَحْوَكُمْ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أبنحوكن'] = {
    'word': u'أبنحوكن',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِنَحْوَكُنَّ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أبنحوها'] = {
    'word': u'أبنحوها',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِنَحْوَهَا',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أبنحوهم'] = {
    'word': u'أبنحوهم',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِنَحْوَهُمْ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أبنحوهن'] = {
    'word': u'أبنحوهن',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِنَحْوَهُنَّ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أبنحونا'] = {
    'word': u'أبنحونا',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِنَحْوَنَا',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أبنحوكما'] = {
    'word': u'أبنحوكما',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِنَحْوَكُمَا',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أبنحوهما'] = {
    'word': u'أبنحوهما',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِنَحْوَهُمَا',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أكنحوي'] = {
    'word': u'أكنحوي',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَنَحْوَِي',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أكنحوك'] = {
    'word': u'أكنحوك',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَنَحْوَكَ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أكنحوك'] = {
    'word': u'أكنحوك',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَنَحْوَكِ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أكنحوه'] = {
    'word': u'أكنحوه',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَنَحْوَه',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'ه'
}
STOPWORDS[u'أكنحوكم'] = {
    'word': u'أكنحوكم',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَنَحْوَكُمْ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أكنحوكن'] = {
    'word': u'أكنحوكن',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَنَحْوَكُنَّ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أكنحوها'] = {
    'word': u'أكنحوها',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَنَحْوَهَا',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أكنحوهم'] = {
    'word': u'أكنحوهم',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَنَحْوَهُمْ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أكنحوهن'] = {
    'word': u'أكنحوهن',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَنَحْوَهُنَّ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أكنحونا'] = {
    'word': u'أكنحونا',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَنَحْوَنَا',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أكنحوكما'] = {
    'word': u'أكنحوكما',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَنَحْوَكُمَا',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أكنحوهما'] = {
    'word': u'أكنحوهما',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَنَحْوَهُمَا',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ألنحوي'] = {
    'word': u'ألنحوي',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِنَحْوَِي',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'ِي'
}
STOPWORDS[u'ألنحوك'] = {
    'word': u'ألنحوك',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِنَحْوَكَ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كَ'
}
STOPWORDS[u'ألنحوك'] = {
    'word': u'ألنحوك',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِنَحْوَكِ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كِ'
}
STOPWORDS[u'ألنحوه'] = {
    'word': u'ألنحوه',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِنَحْوَه',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'ه'
}
STOPWORDS[u'ألنحوكم'] = {
    'word': u'ألنحوكم',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِنَحْوَكُمْ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ألنحوكن'] = {
    'word': u'ألنحوكن',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِنَحْوَكُنَّ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ألنحوها'] = {
    'word': u'ألنحوها',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِنَحْوَهَا',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'هَا'
}
STOPWORDS[u'ألنحوهم'] = {
    'word': u'ألنحوهم',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِنَحْوَهُمْ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ألنحوهن'] = {
    'word': u'ألنحوهن',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِنَحْوَهُنَّ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ألنحونا'] = {
    'word': u'ألنحونا',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِنَحْوَنَا',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'نَا'
}
STOPWORDS[u'ألنحوكما'] = {
    'word': u'ألنحوكما',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِنَحْوَكُمَا',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ألنحوهما'] = {
    'word': u'ألنحوهما',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِنَحْوَهُمَا',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أونحوي'] = {
    'word': u'أونحوي',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَنَحْوَِي',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أونحوك'] = {
    'word': u'أونحوك',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَنَحْوَكَ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أونحوك'] = {
    'word': u'أونحوك',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَنَحْوَكِ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أونحوه'] = {
    'word': u'أونحوه',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَنَحْوَه',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'ه'
}
STOPWORDS[u'أونحوكم'] = {
    'word': u'أونحوكم',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَنَحْوَكُمْ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أونحوكن'] = {
    'word': u'أونحوكن',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَنَحْوَكُنَّ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أونحوها'] = {
    'word': u'أونحوها',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَنَحْوَهَا',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أونحوهم'] = {
    'word': u'أونحوهم',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَنَحْوَهُمْ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أونحوهن'] = {
    'word': u'أونحوهن',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَنَحْوَهُنَّ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أونحونا'] = {
    'word': u'أونحونا',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَنَحْوَنَا',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أونحوكما'] = {
    'word': u'أونحوكما',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَنَحْوَكُمَا',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أونحوهما'] = {
    'word': u'أونحوهما',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَنَحْوَهُمَا',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أفنحوي'] = {
    'word': u'أفنحوي',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَنَحْوَِي',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أفنحوك'] = {
    'word': u'أفنحوك',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَنَحْوَكَ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أفنحوك'] = {
    'word': u'أفنحوك',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَنَحْوَكِ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أفنحوه'] = {
    'word': u'أفنحوه',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَنَحْوَه',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'ه'
}
STOPWORDS[u'أفنحوكم'] = {
    'word': u'أفنحوكم',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَنَحْوَكُمْ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أفنحوكن'] = {
    'word': u'أفنحوكن',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَنَحْوَكُنَّ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أفنحوها'] = {
    'word': u'أفنحوها',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَنَحْوَهَا',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أفنحوهم'] = {
    'word': u'أفنحوهم',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَنَحْوَهُمْ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أفنحوهن'] = {
    'word': u'أفنحوهن',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَنَحْوَهُنَّ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أفنحونا'] = {
    'word': u'أفنحونا',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَنَحْوَنَا',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أفنحوكما'] = {
    'word': u'أفنحوكما',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَنَحْوَكُمَا',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أفنحوهما'] = {
    'word': u'أفنحوهما',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم إضافة:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَنَحْوَهُمَا',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أوبنحوي'] = {
    'word': u'أوبنحوي',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِنَحْوَِي',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أوبنحوك'] = {
    'word': u'أوبنحوك',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِنَحْوَكَ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أوبنحوك'] = {
    'word': u'أوبنحوك',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِنَحْوَكِ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أوبنحوه'] = {
    'word': u'أوبنحوه',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِنَحْوَه',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'ه'
}
STOPWORDS[u'أوبنحوكم'] = {
    'word': u'أوبنحوكم',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِنَحْوَكُمْ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أوبنحوكن'] = {
    'word': u'أوبنحوكن',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِنَحْوَكُنَّ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أوبنحوها'] = {
    'word': u'أوبنحوها',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِنَحْوَهَا',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أوبنحوهم'] = {
    'word': u'أوبنحوهم',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِنَحْوَهُمْ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أوبنحوهن'] = {
    'word': u'أوبنحوهن',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِنَحْوَهُنَّ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أوبنحونا'] = {
    'word': u'أوبنحونا',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِنَحْوَنَا',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أوبنحوكما'] = {
    'word': u'أوبنحوكما',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِنَحْوَكُمَا',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أوبنحوهما'] = {
    'word': u'أوبنحوهما',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِنَحْوَهُمَا',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أفبنحوي'] = {
    'word': u'أفبنحوي',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِنَحْوَِي',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أفبنحوك'] = {
    'word': u'أفبنحوك',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِنَحْوَكَ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أفبنحوك'] = {
    'word': u'أفبنحوك',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِنَحْوَكِ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أفبنحوه'] = {
    'word': u'أفبنحوه',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِنَحْوَه',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'ه'
}
STOPWORDS[u'أفبنحوكم'] = {
    'word': u'أفبنحوكم',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِنَحْوَكُمْ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أفبنحوكن'] = {
    'word': u'أفبنحوكن',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِنَحْوَكُنَّ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أفبنحوها'] = {
    'word': u'أفبنحوها',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِنَحْوَهَا',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أفبنحوهم'] = {
    'word': u'أفبنحوهم',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِنَحْوَهُمْ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أفبنحوهن'] = {
    'word': u'أفبنحوهن',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِنَحْوَهُنَّ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أفبنحونا'] = {
    'word': u'أفبنحونا',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِنَحْوَنَا',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أفبنحوكما'] = {
    'word': u'أفبنحوكما',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِنَحْوَكُمَا',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أفبنحوهما'] = {
    'word': u'أفبنحوهما',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِنَحْوَهُمَا',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أوكنحوي'] = {
    'word': u'أوكنحوي',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَنَحْوَِي',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أوكنحوك'] = {
    'word': u'أوكنحوك',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَنَحْوَكَ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أوكنحوك'] = {
    'word': u'أوكنحوك',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَنَحْوَكِ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أوكنحوه'] = {
    'word': u'أوكنحوه',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَنَحْوَه',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'ه'
}
STOPWORDS[u'أوكنحوكم'] = {
    'word': u'أوكنحوكم',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَنَحْوَكُمْ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أوكنحوكن'] = {
    'word': u'أوكنحوكن',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَنَحْوَكُنَّ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أوكنحوها'] = {
    'word': u'أوكنحوها',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَنَحْوَهَا',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أوكنحوهم'] = {
    'word': u'أوكنحوهم',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَنَحْوَهُمْ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أوكنحوهن'] = {
    'word': u'أوكنحوهن',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَنَحْوَهُنَّ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أوكنحونا'] = {
    'word': u'أوكنحونا',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَنَحْوَنَا',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أوكنحوكما'] = {
    'word': u'أوكنحوكما',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَنَحْوَكُمَا',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أوكنحوهما'] = {
    'word': u'أوكنحوهما',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَنَحْوَهُمَا',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أفكنحوي'] = {
    'word': u'أفكنحوي',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَنَحْوَِي',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أفكنحوك'] = {
    'word': u'أفكنحوك',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَنَحْوَكَ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أفكنحوك'] = {
    'word': u'أفكنحوك',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَنَحْوَكِ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أفكنحوه'] = {
    'word': u'أفكنحوه',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَنَحْوَه',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'ه'
}
STOPWORDS[u'أفكنحوكم'] = {
    'word': u'أفكنحوكم',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَنَحْوَكُمْ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أفكنحوكن'] = {
    'word': u'أفكنحوكن',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَنَحْوَكُنَّ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أفكنحوها'] = {
    'word': u'أفكنحوها',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَنَحْوَهَا',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أفكنحوهم'] = {
    'word': u'أفكنحوهم',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَنَحْوَهُمْ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أفكنحوهن'] = {
    'word': u'أفكنحوهن',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَنَحْوَهُنَّ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أفكنحونا'] = {
    'word': u'أفكنحونا',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَنَحْوَنَا',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أفكنحوكما'] = {
    'word': u'أفكنحوكما',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَنَحْوَكُمَا',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أفكنحوهما'] = {
    'word': u'أفكنحوهما',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَنَحْوَهُمَا',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أولنحوي'] = {
    'word': u'أولنحوي',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِنَحْوَِي',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أولنحوك'] = {
    'word': u'أولنحوك',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِنَحْوَكَ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أولنحوك'] = {
    'word': u'أولنحوك',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِنَحْوَكِ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أولنحوه'] = {
    'word': u'أولنحوه',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِنَحْوَه',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'ه'
}
STOPWORDS[u'أولنحوكم'] = {
    'word': u'أولنحوكم',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِنَحْوَكُمْ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أولنحوكن'] = {
    'word': u'أولنحوكن',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِنَحْوَكُنَّ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أولنحوها'] = {
    'word': u'أولنحوها',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِنَحْوَهَا',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أولنحوهم'] = {
    'word': u'أولنحوهم',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِنَحْوَهُمْ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أولنحوهن'] = {
    'word': u'أولنحوهن',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِنَحْوَهُنَّ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أولنحونا'] = {
    'word': u'أولنحونا',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِنَحْوَنَا',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أولنحوكما'] = {
    'word': u'أولنحوكما',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِنَحْوَكُمَا',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أولنحوهما'] = {
    'word': u'أولنحوهما',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِنَحْوَهُمَا',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أفلنحوي'] = {
    'word': u'أفلنحوي',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِنَحْوَِي',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أفلنحوك'] = {
    'word': u'أفلنحوك',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِنَحْوَكَ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أفلنحوك'] = {
    'word': u'أفلنحوك',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِنَحْوَكِ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أفلنحوه'] = {
    'word': u'أفلنحوه',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِنَحْوَه',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'ه'
}
STOPWORDS[u'أفلنحوكم'] = {
    'word': u'أفلنحوكم',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِنَحْوَكُمْ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أفلنحوكن'] = {
    'word': u'أفلنحوكن',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِنَحْوَكُنَّ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أفلنحوها'] = {
    'word': u'أفلنحوها',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِنَحْوَهَا',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أفلنحوهم'] = {
    'word': u'أفلنحوهم',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِنَحْوَهُمْ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أفلنحوهن'] = {
    'word': u'أفلنحوهن',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِنَحْوَهُنَّ',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أفلنحونا'] = {
    'word': u'أفلنحونا',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِنَحْوَنَا',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أفلنحوكما'] = {
    'word': u'أفلنحوكما',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِنَحْوَكُمَا',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أفلنحوهما'] = {
    'word': u'أفلنحوهما',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم إضافة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِنَحْوَهُمَا',
    'stem': u'نَحْوَ',
    'type': u'STOPWORD',
    'original': u'نَحْوَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أقل'] = {
    'word': u'أَقَلُّ',
    'procletic': u'',
    'tags': u'أداة:اسم تفضيل',
    'vocalized': u'أَقَلُّ',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u''
}
STOPWORDS[u'بأقل'] = {
    'word': u'بأقل',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم تفضيل:مجرور',
    'vocalized': u'بِأَقَلُّ',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u''
}
STOPWORDS[u'كأقل'] = {
    'word': u'كأقل',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم تفضيل:مجرور',
    'vocalized': u'كَأَقَلُّ',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u''
}
STOPWORDS[u'لأقل'] = {
    'word': u'لأقل',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم تفضيل:مجرور',
    'vocalized': u'لِأَقَلُّ',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u''
}
STOPWORDS[u'وأقل'] = {
    'word': u'وأقل',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم تفضيل:معطوف',
    'vocalized': u'وَأَقَلُّ',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u''
}
STOPWORDS[u'فأقل'] = {
    'word': u'فأقل',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم تفضيل:معطوف',
    'vocalized': u'فَأَقَلُّ',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u''
}
STOPWORDS[u'وبأقل'] = {
    'word': u'وبأقل',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف',
    'vocalized': u'وَبِأَقَلُّ',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u''
}
STOPWORDS[u'فبأقل'] = {
    'word': u'فبأقل',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف',
    'vocalized': u'فَبِأَقَلُّ',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u''
}
STOPWORDS[u'وكأقل'] = {
    'word': u'وكأقل',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف',
    'vocalized': u'وَكَأَقَلُّ',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u''
}
STOPWORDS[u'فكأقل'] = {
    'word': u'فكأقل',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف',
    'vocalized': u'فَكَأَقَلُّ',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u''
}
STOPWORDS[u'ولأقل'] = {
    'word': u'ولأقل',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف',
    'vocalized': u'وَلِأَقَلُّ',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u''
}
STOPWORDS[u'فلأقل'] = {
    'word': u'فلأقل',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف',
    'vocalized': u'فَلِأَقَلُّ',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u''
}
STOPWORDS[u'أقلي'] = {
    'word': u'أقلي',
    'procletic': u'',
    'tags': u'أداة:اسم تفضيل:مضاف',
    'vocalized': u'أَقَلُِّي',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'ِي'
}
STOPWORDS[u'أقلك'] = {
    'word': u'أقلك',
    'procletic': u'',
    'tags': u'أداة:اسم تفضيل:مضاف',
    'vocalized': u'أَقَلُّكَ',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'كَ'
}
STOPWORDS[u'أقلك'] = {
    'word': u'أقلك',
    'procletic': u'',
    'tags': u'أداة:اسم تفضيل:مضاف',
    'vocalized': u'أَقَلُّكِ',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'كِ'
}
STOPWORDS[u'أقله'] = {
    'word': u'أقله',
    'procletic': u'',
    'tags': u'أداة:اسم تفضيل:مضاف',
    'vocalized': u'أَقَلُّه',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'ه'
}
STOPWORDS[u'أقلكم'] = {
    'word': u'أقلكم',
    'procletic': u'',
    'tags': u'أداة:اسم تفضيل:مضاف',
    'vocalized': u'أَقَلُّكُمْ',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أقلكن'] = {
    'word': u'أقلكن',
    'procletic': u'',
    'tags': u'أداة:اسم تفضيل:مضاف',
    'vocalized': u'أَقَلُّكُنَّ',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أقلها'] = {
    'word': u'أقلها',
    'procletic': u'',
    'tags': u'أداة:اسم تفضيل:مضاف',
    'vocalized': u'أَقَلُّهَا',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'هَا'
}
STOPWORDS[u'أقلهم'] = {
    'word': u'أقلهم',
    'procletic': u'',
    'tags': u'أداة:اسم تفضيل:مضاف',
    'vocalized': u'أَقَلُّهُمْ',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أقلهن'] = {
    'word': u'أقلهن',
    'procletic': u'',
    'tags': u'أداة:اسم تفضيل:مضاف',
    'vocalized': u'أَقَلُّهُنَّ',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أقلنا'] = {
    'word': u'أقلنا',
    'procletic': u'',
    'tags': u'أداة:اسم تفضيل:مضاف',
    'vocalized': u'أَقَلُّنَا',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'نَا'
}
STOPWORDS[u'أقلكما'] = {
    'word': u'أقلكما',
    'procletic': u'',
    'tags': u'أداة:اسم تفضيل:مضاف',
    'vocalized': u'أَقَلُّكُمَا',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أقلهما'] = {
    'word': u'أقلهما',
    'procletic': u'',
    'tags': u'أداة:اسم تفضيل:مضاف',
    'vocalized': u'أَقَلُّهُمَا',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'بأقلي'] = {
    'word': u'بأقلي',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم تفضيل:مجرور:مضاف',
    'vocalized': u'بِأَقَلُِّي',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'ِي'
}
STOPWORDS[u'بأقلك'] = {
    'word': u'بأقلك',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم تفضيل:مجرور:مضاف',
    'vocalized': u'بِأَقَلُّكَ',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'كَ'
}
STOPWORDS[u'بأقلك'] = {
    'word': u'بأقلك',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم تفضيل:مجرور:مضاف',
    'vocalized': u'بِأَقَلُّكِ',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'كِ'
}
STOPWORDS[u'بأقله'] = {
    'word': u'بأقله',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم تفضيل:مجرور:مضاف',
    'vocalized': u'بِأَقَلُّه',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'ه'
}
STOPWORDS[u'بأقلكم'] = {
    'word': u'بأقلكم',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم تفضيل:مجرور:مضاف',
    'vocalized': u'بِأَقَلُّكُمْ',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'بأقلكن'] = {
    'word': u'بأقلكن',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم تفضيل:مجرور:مضاف',
    'vocalized': u'بِأَقَلُّكُنَّ',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'بأقلها'] = {
    'word': u'بأقلها',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم تفضيل:مجرور:مضاف',
    'vocalized': u'بِأَقَلُّهَا',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'هَا'
}
STOPWORDS[u'بأقلهم'] = {
    'word': u'بأقلهم',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم تفضيل:مجرور:مضاف',
    'vocalized': u'بِأَقَلُّهُمْ',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'بأقلهن'] = {
    'word': u'بأقلهن',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم تفضيل:مجرور:مضاف',
    'vocalized': u'بِأَقَلُّهُنَّ',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'بأقلنا'] = {
    'word': u'بأقلنا',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم تفضيل:مجرور:مضاف',
    'vocalized': u'بِأَقَلُّنَا',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'نَا'
}
STOPWORDS[u'بأقلكما'] = {
    'word': u'بأقلكما',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم تفضيل:مجرور:مضاف',
    'vocalized': u'بِأَقَلُّكُمَا',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'بأقلهما'] = {
    'word': u'بأقلهما',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم تفضيل:مجرور:مضاف',
    'vocalized': u'بِأَقَلُّهُمَا',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'كأقلي'] = {
    'word': u'كأقلي',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم تفضيل:مجرور:مضاف',
    'vocalized': u'كَأَقَلُِّي',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'ِي'
}
STOPWORDS[u'كأقلك'] = {
    'word': u'كأقلك',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم تفضيل:مجرور:مضاف',
    'vocalized': u'كَأَقَلُّكَ',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'كَ'
}
STOPWORDS[u'كأقلك'] = {
    'word': u'كأقلك',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم تفضيل:مجرور:مضاف',
    'vocalized': u'كَأَقَلُّكِ',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'كِ'
}
STOPWORDS[u'كأقله'] = {
    'word': u'كأقله',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم تفضيل:مجرور:مضاف',
    'vocalized': u'كَأَقَلُّه',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'ه'
}
STOPWORDS[u'كأقلكم'] = {
    'word': u'كأقلكم',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم تفضيل:مجرور:مضاف',
    'vocalized': u'كَأَقَلُّكُمْ',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'كأقلكن'] = {
    'word': u'كأقلكن',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم تفضيل:مجرور:مضاف',
    'vocalized': u'كَأَقَلُّكُنَّ',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'كأقلها'] = {
    'word': u'كأقلها',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم تفضيل:مجرور:مضاف',
    'vocalized': u'كَأَقَلُّهَا',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'هَا'
}
STOPWORDS[u'كأقلهم'] = {
    'word': u'كأقلهم',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم تفضيل:مجرور:مضاف',
    'vocalized': u'كَأَقَلُّهُمْ',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'كأقلهن'] = {
    'word': u'كأقلهن',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم تفضيل:مجرور:مضاف',
    'vocalized': u'كَأَقَلُّهُنَّ',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'كأقلنا'] = {
    'word': u'كأقلنا',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم تفضيل:مجرور:مضاف',
    'vocalized': u'كَأَقَلُّنَا',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'نَا'
}
STOPWORDS[u'كأقلكما'] = {
    'word': u'كأقلكما',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم تفضيل:مجرور:مضاف',
    'vocalized': u'كَأَقَلُّكُمَا',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'كأقلهما'] = {
    'word': u'كأقلهما',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم تفضيل:مجرور:مضاف',
    'vocalized': u'كَأَقَلُّهُمَا',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'لأقلي'] = {
    'word': u'لأقلي',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم تفضيل:مجرور:مضاف',
    'vocalized': u'لِأَقَلُِّي',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'ِي'
}
STOPWORDS[u'لأقلك'] = {
    'word': u'لأقلك',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم تفضيل:مجرور:مضاف',
    'vocalized': u'لِأَقَلُّكَ',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'كَ'
}
STOPWORDS[u'لأقلك'] = {
    'word': u'لأقلك',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم تفضيل:مجرور:مضاف',
    'vocalized': u'لِأَقَلُّكِ',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'كِ'
}
STOPWORDS[u'لأقله'] = {
    'word': u'لأقله',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم تفضيل:مجرور:مضاف',
    'vocalized': u'لِأَقَلُّه',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'ه'
}
STOPWORDS[u'لأقلكم'] = {
    'word': u'لأقلكم',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم تفضيل:مجرور:مضاف',
    'vocalized': u'لِأَقَلُّكُمْ',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'لأقلكن'] = {
    'word': u'لأقلكن',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم تفضيل:مجرور:مضاف',
    'vocalized': u'لِأَقَلُّكُنَّ',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'لأقلها'] = {
    'word': u'لأقلها',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم تفضيل:مجرور:مضاف',
    'vocalized': u'لِأَقَلُّهَا',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'هَا'
}
STOPWORDS[u'لأقلهم'] = {
    'word': u'لأقلهم',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم تفضيل:مجرور:مضاف',
    'vocalized': u'لِأَقَلُّهُمْ',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'لأقلهن'] = {
    'word': u'لأقلهن',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم تفضيل:مجرور:مضاف',
    'vocalized': u'لِأَقَلُّهُنَّ',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'لأقلنا'] = {
    'word': u'لأقلنا',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم تفضيل:مجرور:مضاف',
    'vocalized': u'لِأَقَلُّنَا',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'نَا'
}
STOPWORDS[u'لأقلكما'] = {
    'word': u'لأقلكما',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم تفضيل:مجرور:مضاف',
    'vocalized': u'لِأَقَلُّكُمَا',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'لأقلهما'] = {
    'word': u'لأقلهما',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم تفضيل:مجرور:مضاف',
    'vocalized': u'لِأَقَلُّهُمَا',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'وأقلي'] = {
    'word': u'وأقلي',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم تفضيل:معطوف:مضاف',
    'vocalized': u'وَأَقَلُِّي',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'ِي'
}
STOPWORDS[u'وأقلك'] = {
    'word': u'وأقلك',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم تفضيل:معطوف:مضاف',
    'vocalized': u'وَأَقَلُّكَ',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'كَ'
}
STOPWORDS[u'وأقلك'] = {
    'word': u'وأقلك',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم تفضيل:معطوف:مضاف',
    'vocalized': u'وَأَقَلُّكِ',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'كِ'
}
STOPWORDS[u'وأقله'] = {
    'word': u'وأقله',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم تفضيل:معطوف:مضاف',
    'vocalized': u'وَأَقَلُّه',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'ه'
}
STOPWORDS[u'وأقلكم'] = {
    'word': u'وأقلكم',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم تفضيل:معطوف:مضاف',
    'vocalized': u'وَأَقَلُّكُمْ',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'وأقلكن'] = {
    'word': u'وأقلكن',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم تفضيل:معطوف:مضاف',
    'vocalized': u'وَأَقَلُّكُنَّ',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'وأقلها'] = {
    'word': u'وأقلها',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم تفضيل:معطوف:مضاف',
    'vocalized': u'وَأَقَلُّهَا',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'هَا'
}
STOPWORDS[u'وأقلهم'] = {
    'word': u'وأقلهم',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم تفضيل:معطوف:مضاف',
    'vocalized': u'وَأَقَلُّهُمْ',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'وأقلهن'] = {
    'word': u'وأقلهن',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم تفضيل:معطوف:مضاف',
    'vocalized': u'وَأَقَلُّهُنَّ',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'وأقلنا'] = {
    'word': u'وأقلنا',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم تفضيل:معطوف:مضاف',
    'vocalized': u'وَأَقَلُّنَا',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'نَا'
}
STOPWORDS[u'وأقلكما'] = {
    'word': u'وأقلكما',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم تفضيل:معطوف:مضاف',
    'vocalized': u'وَأَقَلُّكُمَا',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'وأقلهما'] = {
    'word': u'وأقلهما',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم تفضيل:معطوف:مضاف',
    'vocalized': u'وَأَقَلُّهُمَا',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فأقلي'] = {
    'word': u'فأقلي',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم تفضيل:معطوف:مضاف',
    'vocalized': u'فَأَقَلُِّي',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'ِي'
}
STOPWORDS[u'فأقلك'] = {
    'word': u'فأقلك',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم تفضيل:معطوف:مضاف',
    'vocalized': u'فَأَقَلُّكَ',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'كَ'
}
STOPWORDS[u'فأقلك'] = {
    'word': u'فأقلك',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم تفضيل:معطوف:مضاف',
    'vocalized': u'فَأَقَلُّكِ',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'كِ'
}
STOPWORDS[u'فأقله'] = {
    'word': u'فأقله',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم تفضيل:معطوف:مضاف',
    'vocalized': u'فَأَقَلُّه',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'ه'
}
STOPWORDS[u'فأقلكم'] = {
    'word': u'فأقلكم',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم تفضيل:معطوف:مضاف',
    'vocalized': u'فَأَقَلُّكُمْ',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فأقلكن'] = {
    'word': u'فأقلكن',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم تفضيل:معطوف:مضاف',
    'vocalized': u'فَأَقَلُّكُنَّ',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فأقلها'] = {
    'word': u'فأقلها',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم تفضيل:معطوف:مضاف',
    'vocalized': u'فَأَقَلُّهَا',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'هَا'
}
STOPWORDS[u'فأقلهم'] = {
    'word': u'فأقلهم',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم تفضيل:معطوف:مضاف',
    'vocalized': u'فَأَقَلُّهُمْ',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فأقلهن'] = {
    'word': u'فأقلهن',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم تفضيل:معطوف:مضاف',
    'vocalized': u'فَأَقَلُّهُنَّ',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فأقلنا'] = {
    'word': u'فأقلنا',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم تفضيل:معطوف:مضاف',
    'vocalized': u'فَأَقَلُّنَا',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'نَا'
}
STOPWORDS[u'فأقلكما'] = {
    'word': u'فأقلكما',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم تفضيل:معطوف:مضاف',
    'vocalized': u'فَأَقَلُّكُمَا',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فأقلهما'] = {
    'word': u'فأقلهما',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم تفضيل:معطوف:مضاف',
    'vocalized': u'فَأَقَلُّهُمَا',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'وبأقلي'] = {
    'word': u'وبأقلي',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِأَقَلُِّي',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'ِي'
}
STOPWORDS[u'وبأقلك'] = {
    'word': u'وبأقلك',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِأَقَلُّكَ',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'كَ'
}
STOPWORDS[u'وبأقلك'] = {
    'word': u'وبأقلك',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِأَقَلُّكِ',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'كِ'
}
STOPWORDS[u'وبأقله'] = {
    'word': u'وبأقله',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِأَقَلُّه',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'ه'
}
STOPWORDS[u'وبأقلكم'] = {
    'word': u'وبأقلكم',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِأَقَلُّكُمْ',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'وبأقلكن'] = {
    'word': u'وبأقلكن',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِأَقَلُّكُنَّ',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'وبأقلها'] = {
    'word': u'وبأقلها',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِأَقَلُّهَا',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'هَا'
}
STOPWORDS[u'وبأقلهم'] = {
    'word': u'وبأقلهم',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِأَقَلُّهُمْ',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'وبأقلهن'] = {
    'word': u'وبأقلهن',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِأَقَلُّهُنَّ',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'وبأقلنا'] = {
    'word': u'وبأقلنا',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِأَقَلُّنَا',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'نَا'
}
STOPWORDS[u'وبأقلكما'] = {
    'word': u'وبأقلكما',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِأَقَلُّكُمَا',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'وبأقلهما'] = {
    'word': u'وبأقلهما',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِأَقَلُّهُمَا',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فبأقلي'] = {
    'word': u'فبأقلي',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِأَقَلُِّي',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'ِي'
}
STOPWORDS[u'فبأقلك'] = {
    'word': u'فبأقلك',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِأَقَلُّكَ',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'كَ'
}
STOPWORDS[u'فبأقلك'] = {
    'word': u'فبأقلك',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِأَقَلُّكِ',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'كِ'
}
STOPWORDS[u'فبأقله'] = {
    'word': u'فبأقله',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِأَقَلُّه',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'ه'
}
STOPWORDS[u'فبأقلكم'] = {
    'word': u'فبأقلكم',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِأَقَلُّكُمْ',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فبأقلكن'] = {
    'word': u'فبأقلكن',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِأَقَلُّكُنَّ',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فبأقلها'] = {
    'word': u'فبأقلها',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِأَقَلُّهَا',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'هَا'
}
STOPWORDS[u'فبأقلهم'] = {
    'word': u'فبأقلهم',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِأَقَلُّهُمْ',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فبأقلهن'] = {
    'word': u'فبأقلهن',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِأَقَلُّهُنَّ',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فبأقلنا'] = {
    'word': u'فبأقلنا',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِأَقَلُّنَا',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'نَا'
}
STOPWORDS[u'فبأقلكما'] = {
    'word': u'فبأقلكما',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِأَقَلُّكُمَا',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فبأقلهما'] = {
    'word': u'فبأقلهما',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِأَقَلُّهُمَا',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'وكأقلي'] = {
    'word': u'وكأقلي',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَأَقَلُِّي',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'ِي'
}
STOPWORDS[u'وكأقلك'] = {
    'word': u'وكأقلك',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَأَقَلُّكَ',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'كَ'
}
STOPWORDS[u'وكأقلك'] = {
    'word': u'وكأقلك',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَأَقَلُّكِ',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'كِ'
}
STOPWORDS[u'وكأقله'] = {
    'word': u'وكأقله',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَأَقَلُّه',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'ه'
}
STOPWORDS[u'وكأقلكم'] = {
    'word': u'وكأقلكم',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَأَقَلُّكُمْ',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'وكأقلكن'] = {
    'word': u'وكأقلكن',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَأَقَلُّكُنَّ',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'وكأقلها'] = {
    'word': u'وكأقلها',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَأَقَلُّهَا',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'هَا'
}
STOPWORDS[u'وكأقلهم'] = {
    'word': u'وكأقلهم',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَأَقَلُّهُمْ',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'وكأقلهن'] = {
    'word': u'وكأقلهن',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَأَقَلُّهُنَّ',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'وكأقلنا'] = {
    'word': u'وكأقلنا',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَأَقَلُّنَا',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'نَا'
}
STOPWORDS[u'وكأقلكما'] = {
    'word': u'وكأقلكما',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَأَقَلُّكُمَا',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'وكأقلهما'] = {
    'word': u'وكأقلهما',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَأَقَلُّهُمَا',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فكأقلي'] = {
    'word': u'فكأقلي',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَأَقَلُِّي',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'ِي'
}
STOPWORDS[u'فكأقلك'] = {
    'word': u'فكأقلك',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَأَقَلُّكَ',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'كَ'
}
STOPWORDS[u'فكأقلك'] = {
    'word': u'فكأقلك',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَأَقَلُّكِ',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'كِ'
}
STOPWORDS[u'فكأقله'] = {
    'word': u'فكأقله',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَأَقَلُّه',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'ه'
}
STOPWORDS[u'فكأقلكم'] = {
    'word': u'فكأقلكم',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَأَقَلُّكُمْ',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فكأقلكن'] = {
    'word': u'فكأقلكن',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَأَقَلُّكُنَّ',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فكأقلها'] = {
    'word': u'فكأقلها',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَأَقَلُّهَا',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'هَا'
}
STOPWORDS[u'فكأقلهم'] = {
    'word': u'فكأقلهم',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَأَقَلُّهُمْ',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فكأقلهن'] = {
    'word': u'فكأقلهن',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَأَقَلُّهُنَّ',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فكأقلنا'] = {
    'word': u'فكأقلنا',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَأَقَلُّنَا',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'نَا'
}
STOPWORDS[u'فكأقلكما'] = {
    'word': u'فكأقلكما',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَأَقَلُّكُمَا',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فكأقلهما'] = {
    'word': u'فكأقلهما',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَأَقَلُّهُمَا',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ولأقلي'] = {
    'word': u'ولأقلي',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِأَقَلُِّي',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'ِي'
}
STOPWORDS[u'ولأقلك'] = {
    'word': u'ولأقلك',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِأَقَلُّكَ',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'كَ'
}
STOPWORDS[u'ولأقلك'] = {
    'word': u'ولأقلك',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِأَقَلُّكِ',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'كِ'
}
STOPWORDS[u'ولأقله'] = {
    'word': u'ولأقله',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِأَقَلُّه',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'ه'
}
STOPWORDS[u'ولأقلكم'] = {
    'word': u'ولأقلكم',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِأَقَلُّكُمْ',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ولأقلكن'] = {
    'word': u'ولأقلكن',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِأَقَلُّكُنَّ',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ولأقلها'] = {
    'word': u'ولأقلها',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِأَقَلُّهَا',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'هَا'
}
STOPWORDS[u'ولأقلهم'] = {
    'word': u'ولأقلهم',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِأَقَلُّهُمْ',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ولأقلهن'] = {
    'word': u'ولأقلهن',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِأَقَلُّهُنَّ',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ولأقلنا'] = {
    'word': u'ولأقلنا',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِأَقَلُّنَا',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'نَا'
}
STOPWORDS[u'ولأقلكما'] = {
    'word': u'ولأقلكما',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِأَقَلُّكُمَا',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ولأقلهما'] = {
    'word': u'ولأقلهما',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِأَقَلُّهُمَا',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فلأقلي'] = {
    'word': u'فلأقلي',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِأَقَلُِّي',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'ِي'
}
STOPWORDS[u'فلأقلك'] = {
    'word': u'فلأقلك',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِأَقَلُّكَ',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'كَ'
}
STOPWORDS[u'فلأقلك'] = {
    'word': u'فلأقلك',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِأَقَلُّكِ',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'كِ'
}
STOPWORDS[u'فلأقله'] = {
    'word': u'فلأقله',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِأَقَلُّه',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'ه'
}
STOPWORDS[u'فلأقلكم'] = {
    'word': u'فلأقلكم',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِأَقَلُّكُمْ',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فلأقلكن'] = {
    'word': u'فلأقلكن',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِأَقَلُّكُنَّ',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فلأقلها'] = {
    'word': u'فلأقلها',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِأَقَلُّهَا',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'هَا'
}
STOPWORDS[u'فلأقلهم'] = {
    'word': u'فلأقلهم',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِأَقَلُّهُمْ',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فلأقلهن'] = {
    'word': u'فلأقلهن',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِأَقَلُّهُنَّ',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فلأقلنا'] = {
    'word': u'فلأقلنا',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِأَقَلُّنَا',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'نَا'
}
STOPWORDS[u'فلأقلكما'] = {
    'word': u'فلأقلكما',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِأَقَلُّكُمَا',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فلأقلهما'] = {
    'word': u'فلأقلهما',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِأَقَلُّهُمَا',
    'stem': u'أَقَلُّ',
    'type': u'STOPWORD',
    'original': u'أَقَلُّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أكثر'] = {
    'word': u'أَكْثَرُ',
    'procletic': u'',
    'tags': u'أداة:اسم تفضيل',
    'vocalized': u'أَكْثَرُ',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u''
}
STOPWORDS[u'بأكثر'] = {
    'word': u'بأكثر',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم تفضيل:مجرور',
    'vocalized': u'بِأَكْثَرُ',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u''
}
STOPWORDS[u'كأكثر'] = {
    'word': u'كأكثر',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم تفضيل:مجرور',
    'vocalized': u'كَأَكْثَرُ',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u''
}
STOPWORDS[u'لأكثر'] = {
    'word': u'لأكثر',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم تفضيل:مجرور',
    'vocalized': u'لِأَكْثَرُ',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u''
}
STOPWORDS[u'وأكثر'] = {
    'word': u'وأكثر',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم تفضيل:معطوف',
    'vocalized': u'وَأَكْثَرُ',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u''
}
STOPWORDS[u'فأكثر'] = {
    'word': u'فأكثر',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم تفضيل:معطوف',
    'vocalized': u'فَأَكْثَرُ',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u''
}
STOPWORDS[u'وبأكثر'] = {
    'word': u'وبأكثر',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف',
    'vocalized': u'وَبِأَكْثَرُ',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u''
}
STOPWORDS[u'فبأكثر'] = {
    'word': u'فبأكثر',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف',
    'vocalized': u'فَبِأَكْثَرُ',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u''
}
STOPWORDS[u'وكأكثر'] = {
    'word': u'وكأكثر',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف',
    'vocalized': u'وَكَأَكْثَرُ',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u''
}
STOPWORDS[u'فكأكثر'] = {
    'word': u'فكأكثر',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف',
    'vocalized': u'فَكَأَكْثَرُ',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u''
}
STOPWORDS[u'ولأكثر'] = {
    'word': u'ولأكثر',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف',
    'vocalized': u'وَلِأَكْثَرُ',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u''
}
STOPWORDS[u'فلأكثر'] = {
    'word': u'فلأكثر',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف',
    'vocalized': u'فَلِأَكْثَرُ',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u''
}
STOPWORDS[u'أكثري'] = {
    'word': u'أكثري',
    'procletic': u'',
    'tags': u'أداة:اسم تفضيل:مضاف',
    'vocalized': u'أَكْثَرُِي',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'ِي'
}
STOPWORDS[u'أكثرك'] = {
    'word': u'أكثرك',
    'procletic': u'',
    'tags': u'أداة:اسم تفضيل:مضاف',
    'vocalized': u'أَكْثَرُكَ',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'كَ'
}
STOPWORDS[u'أكثرك'] = {
    'word': u'أكثرك',
    'procletic': u'',
    'tags': u'أداة:اسم تفضيل:مضاف',
    'vocalized': u'أَكْثَرُكِ',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'كِ'
}
STOPWORDS[u'أكثره'] = {
    'word': u'أكثره',
    'procletic': u'',
    'tags': u'أداة:اسم تفضيل:مضاف',
    'vocalized': u'أَكْثَرُه',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'ه'
}
STOPWORDS[u'أكثركم'] = {
    'word': u'أكثركم',
    'procletic': u'',
    'tags': u'أداة:اسم تفضيل:مضاف',
    'vocalized': u'أَكْثَرُكُمْ',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أكثركن'] = {
    'word': u'أكثركن',
    'procletic': u'',
    'tags': u'أداة:اسم تفضيل:مضاف',
    'vocalized': u'أَكْثَرُكُنَّ',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أكثرها'] = {
    'word': u'أكثرها',
    'procletic': u'',
    'tags': u'أداة:اسم تفضيل:مضاف',
    'vocalized': u'أَكْثَرُهَا',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'هَا'
}
STOPWORDS[u'أكثرهم'] = {
    'word': u'أكثرهم',
    'procletic': u'',
    'tags': u'أداة:اسم تفضيل:مضاف',
    'vocalized': u'أَكْثَرُهُمْ',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أكثرهن'] = {
    'word': u'أكثرهن',
    'procletic': u'',
    'tags': u'أداة:اسم تفضيل:مضاف',
    'vocalized': u'أَكْثَرُهُنَّ',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أكثرنا'] = {
    'word': u'أكثرنا',
    'procletic': u'',
    'tags': u'أداة:اسم تفضيل:مضاف',
    'vocalized': u'أَكْثَرُنَا',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'نَا'
}
STOPWORDS[u'أكثركما'] = {
    'word': u'أكثركما',
    'procletic': u'',
    'tags': u'أداة:اسم تفضيل:مضاف',
    'vocalized': u'أَكْثَرُكُمَا',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أكثرهما'] = {
    'word': u'أكثرهما',
    'procletic': u'',
    'tags': u'أداة:اسم تفضيل:مضاف',
    'vocalized': u'أَكْثَرُهُمَا',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'بأكثري'] = {
    'word': u'بأكثري',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم تفضيل:مجرور:مضاف',
    'vocalized': u'بِأَكْثَرُِي',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'ِي'
}
STOPWORDS[u'بأكثرك'] = {
    'word': u'بأكثرك',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم تفضيل:مجرور:مضاف',
    'vocalized': u'بِأَكْثَرُكَ',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'كَ'
}
STOPWORDS[u'بأكثرك'] = {
    'word': u'بأكثرك',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم تفضيل:مجرور:مضاف',
    'vocalized': u'بِأَكْثَرُكِ',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'كِ'
}
STOPWORDS[u'بأكثره'] = {
    'word': u'بأكثره',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم تفضيل:مجرور:مضاف',
    'vocalized': u'بِأَكْثَرُه',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'ه'
}
STOPWORDS[u'بأكثركم'] = {
    'word': u'بأكثركم',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم تفضيل:مجرور:مضاف',
    'vocalized': u'بِأَكْثَرُكُمْ',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'بأكثركن'] = {
    'word': u'بأكثركن',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم تفضيل:مجرور:مضاف',
    'vocalized': u'بِأَكْثَرُكُنَّ',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'بأكثرها'] = {
    'word': u'بأكثرها',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم تفضيل:مجرور:مضاف',
    'vocalized': u'بِأَكْثَرُهَا',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'هَا'
}
STOPWORDS[u'بأكثرهم'] = {
    'word': u'بأكثرهم',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم تفضيل:مجرور:مضاف',
    'vocalized': u'بِأَكْثَرُهُمْ',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'بأكثرهن'] = {
    'word': u'بأكثرهن',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم تفضيل:مجرور:مضاف',
    'vocalized': u'بِأَكْثَرُهُنَّ',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'بأكثرنا'] = {
    'word': u'بأكثرنا',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم تفضيل:مجرور:مضاف',
    'vocalized': u'بِأَكْثَرُنَا',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'نَا'
}
STOPWORDS[u'بأكثركما'] = {
    'word': u'بأكثركما',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم تفضيل:مجرور:مضاف',
    'vocalized': u'بِأَكْثَرُكُمَا',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'بأكثرهما'] = {
    'word': u'بأكثرهما',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم تفضيل:مجرور:مضاف',
    'vocalized': u'بِأَكْثَرُهُمَا',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'كأكثري'] = {
    'word': u'كأكثري',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم تفضيل:مجرور:مضاف',
    'vocalized': u'كَأَكْثَرُِي',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'ِي'
}
STOPWORDS[u'كأكثرك'] = {
    'word': u'كأكثرك',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم تفضيل:مجرور:مضاف',
    'vocalized': u'كَأَكْثَرُكَ',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'كَ'
}
STOPWORDS[u'كأكثرك'] = {
    'word': u'كأكثرك',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم تفضيل:مجرور:مضاف',
    'vocalized': u'كَأَكْثَرُكِ',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'كِ'
}
STOPWORDS[u'كأكثره'] = {
    'word': u'كأكثره',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم تفضيل:مجرور:مضاف',
    'vocalized': u'كَأَكْثَرُه',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'ه'
}
STOPWORDS[u'كأكثركم'] = {
    'word': u'كأكثركم',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم تفضيل:مجرور:مضاف',
    'vocalized': u'كَأَكْثَرُكُمْ',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'كأكثركن'] = {
    'word': u'كأكثركن',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم تفضيل:مجرور:مضاف',
    'vocalized': u'كَأَكْثَرُكُنَّ',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'كأكثرها'] = {
    'word': u'كأكثرها',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم تفضيل:مجرور:مضاف',
    'vocalized': u'كَأَكْثَرُهَا',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'هَا'
}
STOPWORDS[u'كأكثرهم'] = {
    'word': u'كأكثرهم',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم تفضيل:مجرور:مضاف',
    'vocalized': u'كَأَكْثَرُهُمْ',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'كأكثرهن'] = {
    'word': u'كأكثرهن',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم تفضيل:مجرور:مضاف',
    'vocalized': u'كَأَكْثَرُهُنَّ',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'كأكثرنا'] = {
    'word': u'كأكثرنا',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم تفضيل:مجرور:مضاف',
    'vocalized': u'كَأَكْثَرُنَا',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'نَا'
}
STOPWORDS[u'كأكثركما'] = {
    'word': u'كأكثركما',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم تفضيل:مجرور:مضاف',
    'vocalized': u'كَأَكْثَرُكُمَا',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'كأكثرهما'] = {
    'word': u'كأكثرهما',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم تفضيل:مجرور:مضاف',
    'vocalized': u'كَأَكْثَرُهُمَا',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'لأكثري'] = {
    'word': u'لأكثري',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم تفضيل:مجرور:مضاف',
    'vocalized': u'لِأَكْثَرُِي',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'ِي'
}
STOPWORDS[u'لأكثرك'] = {
    'word': u'لأكثرك',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم تفضيل:مجرور:مضاف',
    'vocalized': u'لِأَكْثَرُكَ',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'كَ'
}
STOPWORDS[u'لأكثرك'] = {
    'word': u'لأكثرك',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم تفضيل:مجرور:مضاف',
    'vocalized': u'لِأَكْثَرُكِ',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'كِ'
}
STOPWORDS[u'لأكثره'] = {
    'word': u'لأكثره',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم تفضيل:مجرور:مضاف',
    'vocalized': u'لِأَكْثَرُه',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'ه'
}
STOPWORDS[u'لأكثركم'] = {
    'word': u'لأكثركم',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم تفضيل:مجرور:مضاف',
    'vocalized': u'لِأَكْثَرُكُمْ',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'لأكثركن'] = {
    'word': u'لأكثركن',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم تفضيل:مجرور:مضاف',
    'vocalized': u'لِأَكْثَرُكُنَّ',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'لأكثرها'] = {
    'word': u'لأكثرها',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم تفضيل:مجرور:مضاف',
    'vocalized': u'لِأَكْثَرُهَا',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'هَا'
}
STOPWORDS[u'لأكثرهم'] = {
    'word': u'لأكثرهم',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم تفضيل:مجرور:مضاف',
    'vocalized': u'لِأَكْثَرُهُمْ',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'لأكثرهن'] = {
    'word': u'لأكثرهن',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم تفضيل:مجرور:مضاف',
    'vocalized': u'لِأَكْثَرُهُنَّ',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'لأكثرنا'] = {
    'word': u'لأكثرنا',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم تفضيل:مجرور:مضاف',
    'vocalized': u'لِأَكْثَرُنَا',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'نَا'
}
STOPWORDS[u'لأكثركما'] = {
    'word': u'لأكثركما',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم تفضيل:مجرور:مضاف',
    'vocalized': u'لِأَكْثَرُكُمَا',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'لأكثرهما'] = {
    'word': u'لأكثرهما',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم تفضيل:مجرور:مضاف',
    'vocalized': u'لِأَكْثَرُهُمَا',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'وأكثري'] = {
    'word': u'وأكثري',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم تفضيل:معطوف:مضاف',
    'vocalized': u'وَأَكْثَرُِي',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'ِي'
}
STOPWORDS[u'وأكثرك'] = {
    'word': u'وأكثرك',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم تفضيل:معطوف:مضاف',
    'vocalized': u'وَأَكْثَرُكَ',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'كَ'
}
STOPWORDS[u'وأكثرك'] = {
    'word': u'وأكثرك',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم تفضيل:معطوف:مضاف',
    'vocalized': u'وَأَكْثَرُكِ',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'كِ'
}
STOPWORDS[u'وأكثره'] = {
    'word': u'وأكثره',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم تفضيل:معطوف:مضاف',
    'vocalized': u'وَأَكْثَرُه',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'ه'
}
STOPWORDS[u'وأكثركم'] = {
    'word': u'وأكثركم',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم تفضيل:معطوف:مضاف',
    'vocalized': u'وَأَكْثَرُكُمْ',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'وأكثركن'] = {
    'word': u'وأكثركن',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم تفضيل:معطوف:مضاف',
    'vocalized': u'وَأَكْثَرُكُنَّ',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'وأكثرها'] = {
    'word': u'وأكثرها',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم تفضيل:معطوف:مضاف',
    'vocalized': u'وَأَكْثَرُهَا',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'هَا'
}
STOPWORDS[u'وأكثرهم'] = {
    'word': u'وأكثرهم',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم تفضيل:معطوف:مضاف',
    'vocalized': u'وَأَكْثَرُهُمْ',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'وأكثرهن'] = {
    'word': u'وأكثرهن',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم تفضيل:معطوف:مضاف',
    'vocalized': u'وَأَكْثَرُهُنَّ',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'وأكثرنا'] = {
    'word': u'وأكثرنا',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم تفضيل:معطوف:مضاف',
    'vocalized': u'وَأَكْثَرُنَا',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'نَا'
}
STOPWORDS[u'وأكثركما'] = {
    'word': u'وأكثركما',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم تفضيل:معطوف:مضاف',
    'vocalized': u'وَأَكْثَرُكُمَا',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'وأكثرهما'] = {
    'word': u'وأكثرهما',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم تفضيل:معطوف:مضاف',
    'vocalized': u'وَأَكْثَرُهُمَا',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فأكثري'] = {
    'word': u'فأكثري',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم تفضيل:معطوف:مضاف',
    'vocalized': u'فَأَكْثَرُِي',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'ِي'
}
STOPWORDS[u'فأكثرك'] = {
    'word': u'فأكثرك',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم تفضيل:معطوف:مضاف',
    'vocalized': u'فَأَكْثَرُكَ',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'كَ'
}
STOPWORDS[u'فأكثرك'] = {
    'word': u'فأكثرك',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم تفضيل:معطوف:مضاف',
    'vocalized': u'فَأَكْثَرُكِ',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'كِ'
}
STOPWORDS[u'فأكثره'] = {
    'word': u'فأكثره',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم تفضيل:معطوف:مضاف',
    'vocalized': u'فَأَكْثَرُه',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'ه'
}
STOPWORDS[u'فأكثركم'] = {
    'word': u'فأكثركم',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم تفضيل:معطوف:مضاف',
    'vocalized': u'فَأَكْثَرُكُمْ',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فأكثركن'] = {
    'word': u'فأكثركن',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم تفضيل:معطوف:مضاف',
    'vocalized': u'فَأَكْثَرُكُنَّ',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فأكثرها'] = {
    'word': u'فأكثرها',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم تفضيل:معطوف:مضاف',
    'vocalized': u'فَأَكْثَرُهَا',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'هَا'
}
STOPWORDS[u'فأكثرهم'] = {
    'word': u'فأكثرهم',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم تفضيل:معطوف:مضاف',
    'vocalized': u'فَأَكْثَرُهُمْ',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فأكثرهن'] = {
    'word': u'فأكثرهن',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم تفضيل:معطوف:مضاف',
    'vocalized': u'فَأَكْثَرُهُنَّ',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فأكثرنا'] = {
    'word': u'فأكثرنا',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم تفضيل:معطوف:مضاف',
    'vocalized': u'فَأَكْثَرُنَا',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'نَا'
}
STOPWORDS[u'فأكثركما'] = {
    'word': u'فأكثركما',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم تفضيل:معطوف:مضاف',
    'vocalized': u'فَأَكْثَرُكُمَا',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فأكثرهما'] = {
    'word': u'فأكثرهما',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم تفضيل:معطوف:مضاف',
    'vocalized': u'فَأَكْثَرُهُمَا',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'وبأكثري'] = {
    'word': u'وبأكثري',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِأَكْثَرُِي',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'ِي'
}
STOPWORDS[u'وبأكثرك'] = {
    'word': u'وبأكثرك',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِأَكْثَرُكَ',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'كَ'
}
STOPWORDS[u'وبأكثرك'] = {
    'word': u'وبأكثرك',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِأَكْثَرُكِ',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'كِ'
}
STOPWORDS[u'وبأكثره'] = {
    'word': u'وبأكثره',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِأَكْثَرُه',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'ه'
}
STOPWORDS[u'وبأكثركم'] = {
    'word': u'وبأكثركم',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِأَكْثَرُكُمْ',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'وبأكثركن'] = {
    'word': u'وبأكثركن',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِأَكْثَرُكُنَّ',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'وبأكثرها'] = {
    'word': u'وبأكثرها',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِأَكْثَرُهَا',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'هَا'
}
STOPWORDS[u'وبأكثرهم'] = {
    'word': u'وبأكثرهم',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِأَكْثَرُهُمْ',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'وبأكثرهن'] = {
    'word': u'وبأكثرهن',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِأَكْثَرُهُنَّ',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'وبأكثرنا'] = {
    'word': u'وبأكثرنا',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِأَكْثَرُنَا',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'نَا'
}
STOPWORDS[u'وبأكثركما'] = {
    'word': u'وبأكثركما',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِأَكْثَرُكُمَا',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'وبأكثرهما'] = {
    'word': u'وبأكثرهما',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِأَكْثَرُهُمَا',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فبأكثري'] = {
    'word': u'فبأكثري',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِأَكْثَرُِي',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'ِي'
}
STOPWORDS[u'فبأكثرك'] = {
    'word': u'فبأكثرك',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِأَكْثَرُكَ',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'كَ'
}
STOPWORDS[u'فبأكثرك'] = {
    'word': u'فبأكثرك',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِأَكْثَرُكِ',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'كِ'
}
STOPWORDS[u'فبأكثره'] = {
    'word': u'فبأكثره',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِأَكْثَرُه',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'ه'
}
STOPWORDS[u'فبأكثركم'] = {
    'word': u'فبأكثركم',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِأَكْثَرُكُمْ',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فبأكثركن'] = {
    'word': u'فبأكثركن',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِأَكْثَرُكُنَّ',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فبأكثرها'] = {
    'word': u'فبأكثرها',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِأَكْثَرُهَا',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'هَا'
}
STOPWORDS[u'فبأكثرهم'] = {
    'word': u'فبأكثرهم',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِأَكْثَرُهُمْ',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فبأكثرهن'] = {
    'word': u'فبأكثرهن',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِأَكْثَرُهُنَّ',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فبأكثرنا'] = {
    'word': u'فبأكثرنا',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِأَكْثَرُنَا',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'نَا'
}
STOPWORDS[u'فبأكثركما'] = {
    'word': u'فبأكثركما',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِأَكْثَرُكُمَا',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فبأكثرهما'] = {
    'word': u'فبأكثرهما',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِأَكْثَرُهُمَا',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'وكأكثري'] = {
    'word': u'وكأكثري',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَأَكْثَرُِي',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'ِي'
}
STOPWORDS[u'وكأكثرك'] = {
    'word': u'وكأكثرك',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَأَكْثَرُكَ',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'كَ'
}
STOPWORDS[u'وكأكثرك'] = {
    'word': u'وكأكثرك',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَأَكْثَرُكِ',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'كِ'
}
STOPWORDS[u'وكأكثره'] = {
    'word': u'وكأكثره',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَأَكْثَرُه',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'ه'
}
STOPWORDS[u'وكأكثركم'] = {
    'word': u'وكأكثركم',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَأَكْثَرُكُمْ',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'وكأكثركن'] = {
    'word': u'وكأكثركن',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَأَكْثَرُكُنَّ',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'وكأكثرها'] = {
    'word': u'وكأكثرها',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَأَكْثَرُهَا',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'هَا'
}
STOPWORDS[u'وكأكثرهم'] = {
    'word': u'وكأكثرهم',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَأَكْثَرُهُمْ',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'وكأكثرهن'] = {
    'word': u'وكأكثرهن',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَأَكْثَرُهُنَّ',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'وكأكثرنا'] = {
    'word': u'وكأكثرنا',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَأَكْثَرُنَا',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'نَا'
}
STOPWORDS[u'وكأكثركما'] = {
    'word': u'وكأكثركما',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَأَكْثَرُكُمَا',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'وكأكثرهما'] = {
    'word': u'وكأكثرهما',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَأَكْثَرُهُمَا',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فكأكثري'] = {
    'word': u'فكأكثري',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَأَكْثَرُِي',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'ِي'
}
STOPWORDS[u'فكأكثرك'] = {
    'word': u'فكأكثرك',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَأَكْثَرُكَ',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'كَ'
}
STOPWORDS[u'فكأكثرك'] = {
    'word': u'فكأكثرك',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَأَكْثَرُكِ',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'كِ'
}
STOPWORDS[u'فكأكثره'] = {
    'word': u'فكأكثره',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَأَكْثَرُه',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'ه'
}
STOPWORDS[u'فكأكثركم'] = {
    'word': u'فكأكثركم',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَأَكْثَرُكُمْ',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فكأكثركن'] = {
    'word': u'فكأكثركن',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَأَكْثَرُكُنَّ',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فكأكثرها'] = {
    'word': u'فكأكثرها',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَأَكْثَرُهَا',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'هَا'
}
STOPWORDS[u'فكأكثرهم'] = {
    'word': u'فكأكثرهم',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَأَكْثَرُهُمْ',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فكأكثرهن'] = {
    'word': u'فكأكثرهن',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَأَكْثَرُهُنَّ',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فكأكثرنا'] = {
    'word': u'فكأكثرنا',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَأَكْثَرُنَا',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'نَا'
}
STOPWORDS[u'فكأكثركما'] = {
    'word': u'فكأكثركما',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَأَكْثَرُكُمَا',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فكأكثرهما'] = {
    'word': u'فكأكثرهما',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَأَكْثَرُهُمَا',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ولأكثري'] = {
    'word': u'ولأكثري',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِأَكْثَرُِي',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'ِي'
}
STOPWORDS[u'ولأكثرك'] = {
    'word': u'ولأكثرك',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِأَكْثَرُكَ',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'كَ'
}
STOPWORDS[u'ولأكثرك'] = {
    'word': u'ولأكثرك',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِأَكْثَرُكِ',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'كِ'
}
STOPWORDS[u'ولأكثره'] = {
    'word': u'ولأكثره',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِأَكْثَرُه',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'ه'
}
STOPWORDS[u'ولأكثركم'] = {
    'word': u'ولأكثركم',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِأَكْثَرُكُمْ',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ولأكثركن'] = {
    'word': u'ولأكثركن',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِأَكْثَرُكُنَّ',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ولأكثرها'] = {
    'word': u'ولأكثرها',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِأَكْثَرُهَا',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'هَا'
}
STOPWORDS[u'ولأكثرهم'] = {
    'word': u'ولأكثرهم',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِأَكْثَرُهُمْ',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ولأكثرهن'] = {
    'word': u'ولأكثرهن',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِأَكْثَرُهُنَّ',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ولأكثرنا'] = {
    'word': u'ولأكثرنا',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِأَكْثَرُنَا',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'نَا'
}
STOPWORDS[u'ولأكثركما'] = {
    'word': u'ولأكثركما',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِأَكْثَرُكُمَا',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ولأكثرهما'] = {
    'word': u'ولأكثرهما',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِأَكْثَرُهُمَا',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فلأكثري'] = {
    'word': u'فلأكثري',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِأَكْثَرُِي',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'ِي'
}
STOPWORDS[u'فلأكثرك'] = {
    'word': u'فلأكثرك',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِأَكْثَرُكَ',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'كَ'
}
STOPWORDS[u'فلأكثرك'] = {
    'word': u'فلأكثرك',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِأَكْثَرُكِ',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'كِ'
}
STOPWORDS[u'فلأكثره'] = {
    'word': u'فلأكثره',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِأَكْثَرُه',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'ه'
}
STOPWORDS[u'فلأكثركم'] = {
    'word': u'فلأكثركم',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِأَكْثَرُكُمْ',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فلأكثركن'] = {
    'word': u'فلأكثركن',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِأَكْثَرُكُنَّ',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فلأكثرها'] = {
    'word': u'فلأكثرها',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِأَكْثَرُهَا',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'هَا'
}
STOPWORDS[u'فلأكثرهم'] = {
    'word': u'فلأكثرهم',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِأَكْثَرُهُمْ',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فلأكثرهن'] = {
    'word': u'فلأكثرهن',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِأَكْثَرُهُنَّ',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فلأكثرنا'] = {
    'word': u'فلأكثرنا',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِأَكْثَرُنَا',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'نَا'
}
STOPWORDS[u'فلأكثركما'] = {
    'word': u'فلأكثركما',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِأَكْثَرُكُمَا',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فلأكثرهما'] = {
    'word': u'فلأكثرهما',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم تفضيل:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِأَكْثَرُهُمَا',
    'stem': u'أَكْثَرُ',
    'type': u'STOPWORD',
    'original': u'أَكْثَرُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'إذا'] = {
    'word': u'إذاً',
    'procletic': u'',
    'tags': u'أداة:اسم شرط',
    'vocalized': u'إذاً',
    'stem': u'إذاً',
    'type': u'STOPWORD',
    'original': u'إذاً',
    'encletic': u''
}
STOPWORDS[u'وإذا'] = {
    'word': u'وإذا',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم شرط:معطوف',
    'vocalized': u'وَإذاً',
    'stem': u'إذاً',
    'type': u'STOPWORD',
    'original': u'إذاً',
    'encletic': u''
}
STOPWORDS[u'فإذا'] = {
    'word': u'فإذا',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم شرط:معطوف',
    'vocalized': u'فَإذاً',
    'stem': u'إذاً',
    'type': u'STOPWORD',
    'original': u'إذاً',
    'encletic': u''
}
STOPWORDS[u'آها'] = {
    'word': u'آهاً',
    'procletic': u'',
    'tags': u'أداة:اسم صوت',
    'vocalized': u'آهاً',
    'stem': u'آهاً',
    'type': u'STOPWORD',
    'original': u'آهاً',
    'encletic': u''
}
STOPWORDS[u'بس'] = {
    'word': u'بَسّْ',
    'procletic': u'',
    'tags': u'أداة:اسم صوت',
    'vocalized': u'بَسّْ',
    'stem': u'بَسّْ',
    'type': u'STOPWORD',
    'original': u'بَسّْ',
    'encletic': u''
}
STOPWORDS[u'حاي'] = {
    'word': u'حَايْ',
    'procletic': u'',
    'tags': u'أداة:اسم صوت',
    'vocalized': u'حَايْ',
    'stem': u'حَايْ',
    'type': u'STOPWORD',
    'original': u'حَايْ',
    'encletic': u''
}
STOPWORDS[u'صه'] = {
    'word': u'صَهْ',
    'procletic': u'',
    'tags': u'أداة:اسم صوت',
    'vocalized': u'صَهْ',
    'stem': u'صَهْ',
    'type': u'STOPWORD',
    'original': u'صَهْ',
    'encletic': u''
}
STOPWORDS[u'صه'] = {
    'word': u'صَهٍ',
    'procletic': u'',
    'tags': u'أداة:اسم صوت',
    'vocalized': u'صَهٍ',
    'stem': u'صَهٍ',
    'type': u'STOPWORD',
    'original': u'صَهٍ',
    'encletic': u''
}
STOPWORDS[u'طاق'] = {
    'word': u'طَاقْ',
    'procletic': u'',
    'tags': u'أداة:اسم صوت',
    'vocalized': u'طَاقْ',
    'stem': u'طَاقْ',
    'type': u'STOPWORD',
    'original': u'طَاقْ',
    'encletic': u''
}
STOPWORDS[u'طق'] = {
    'word': u'طَقْ',
    'procletic': u'',
    'tags': u'أداة:اسم صوت',
    'vocalized': u'طَقْ',
    'stem': u'طَقْ',
    'type': u'STOPWORD',
    'original': u'طَقْ',
    'encletic': u''
}
STOPWORDS[u'عدس'] = {
    'word': u'عَدَسْ',
    'procletic': u'',
    'tags': u'أداة:اسم صوت',
    'vocalized': u'عَدَسْ',
    'stem': u'عَدَسْ',
    'type': u'STOPWORD',
    'original': u'عَدَسْ',
    'encletic': u''
}
STOPWORDS[u'كخ'] = {
    'word': u'كِخْ',
    'procletic': u'',
    'tags': u'أداة:اسم صوت',
    'vocalized': u'كِخْ',
    'stem': u'كِخْ',
    'type': u'STOPWORD',
    'original': u'كِخْ',
    'encletic': u''
}
STOPWORDS[u'نخ'] = {
    'word': u'نَخْ',
    'procletic': u'',
    'tags': u'أداة:اسم صوت',
    'vocalized': u'نَخْ',
    'stem': u'نَخْ',
    'type': u'STOPWORD',
    'original': u'نَخْ',
    'encletic': u''
}
STOPWORDS[u'هج'] = {
    'word': u'هَجْ',
    'procletic': u'',
    'tags': u'أداة:اسم صوت',
    'vocalized': u'هَجْ',
    'stem': u'هَجْ',
    'type': u'STOPWORD',
    'original': u'هَجْ',
    'encletic': u''
}
STOPWORDS[u'وا'] = {
    'word': u'وَا',
    'procletic': u'',
    'tags': u'أداة:اسم صوت',
    'vocalized': u'وَا',
    'stem': u'وَا',
    'type': u'STOPWORD',
    'original': u'وَا',
    'encletic': u''
}
STOPWORDS[u'واها'] = {
    'word': u'وَاهاً',
    'procletic': u'',
    'tags': u'أداة:اسم صوت',
    'vocalized': u'وَاهاً',
    'stem': u'وَاهاً',
    'type': u'STOPWORD',
    'original': u'وَاهاً',
    'encletic': u''
}
STOPWORDS[u'وي'] = {
    'word': u'وَيْ',
    'procletic': u'',
    'tags': u'أداة:اسم صوت',
    'vocalized': u'وَيْ',
    'stem': u'وَيْ',
    'type': u'STOPWORD',
    'original': u'وَيْ',
    'encletic': u''
}
STOPWORDS[u'آمين'] = {
    'word': u'آمِينَ',
    'procletic': u'',
    'tags': u'أداة:اسم فعل',
    'vocalized': u'آمِينَ',
    'stem': u'آمِينَ',
    'type': u'STOPWORD',
    'original': u'آمِينَ',
    'encletic': u''
}
STOPWORDS[u'وآمين'] = {
    'word': u'وآمين',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم فعل:معطوف',
    'vocalized': u'وَآمِينَ',
    'stem': u'آمِينَ',
    'type': u'STOPWORD',
    'original': u'آمِينَ',
    'encletic': u''
}
STOPWORDS[u'فآمين'] = {
    'word': u'فآمين',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم فعل:معطوف',
    'vocalized': u'فَآمِينَ',
    'stem': u'آمِينَ',
    'type': u'STOPWORD',
    'original': u'آمِينَ',
    'encletic': u''
}
STOPWORDS[u'آه'] = {
    'word': u'آهٍ',
    'procletic': u'',
    'tags': u'أداة:اسم فعل',
    'vocalized': u'آهٍ',
    'stem': u'آهٍ',
    'type': u'STOPWORD',
    'original': u'آهٍ',
    'encletic': u''
}
STOPWORDS[u'وآه'] = {
    'word': u'وآه',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم فعل:معطوف',
    'vocalized': u'وَآهٍ',
    'stem': u'آهٍ',
    'type': u'STOPWORD',
    'original': u'آهٍ',
    'encletic': u''
}
STOPWORDS[u'فآه'] = {
    'word': u'فآه',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم فعل:معطوف',
    'vocalized': u'فَآهٍ',
    'stem': u'آهٍ',
    'type': u'STOPWORD',
    'original': u'آهٍ',
    'encletic': u''
}
STOPWORDS[u'أف'] = {
    'word': u'أُفٍّ',
    'procletic': u'',
    'tags': u'أداة:اسم فعل',
    'vocalized': u'أُفٍّ',
    'stem': u'أُفٍّ',
    'type': u'STOPWORD',
    'original': u'أُفٍّ',
    'encletic': u''
}
STOPWORDS[u'وأف'] = {
    'word': u'وأف',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم فعل:معطوف',
    'vocalized': u'وَأُفٍّ',
    'stem': u'أُفٍّ',
    'type': u'STOPWORD',
    'original': u'أُفٍّ',
    'encletic': u''
}
STOPWORDS[u'فأف'] = {
    'word': u'فأف',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم فعل:معطوف',
    'vocalized': u'فَأُفٍّ',
    'stem': u'أُفٍّ',
    'type': u'STOPWORD',
    'original': u'أُفٍّ',
    'encletic': u''
}
STOPWORDS[u'أف'] = {
    'word': u'أُفٍّ',
    'procletic': u'',
    'tags': u'أداة:اسم فعل',
    'vocalized': u'أُفٍّ',
    'stem': u'أُفٍّ',
    'type': u'STOPWORD',
    'original': u'أُفٍّ',
    'encletic': u''
}
STOPWORDS[u'وأف'] = {
    'word': u'وأف',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم فعل:معطوف',
    'vocalized': u'وَأُفٍّ',
    'stem': u'أُفٍّ',
    'type': u'STOPWORD',
    'original': u'أُفٍّ',
    'encletic': u''
}
STOPWORDS[u'فأف'] = {
    'word': u'فأف',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم فعل:معطوف',
    'vocalized': u'فَأُفٍّ',
    'stem': u'أُفٍّ',
    'type': u'STOPWORD',
    'original': u'أُفٍّ',
    'encletic': u''
}
STOPWORDS[u'أمامك'] = {
    'word': u'أَمَامَكَ',
    'procletic': u'',
    'tags': u'أداة:اسم فعل',
    'vocalized': u'أَمَامَكَ',
    'stem': u'أَمَامَكَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَكَ',
    'encletic': u''
}
STOPWORDS[u'وأمامك'] = {
    'word': u'وأمامك',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم فعل:معطوف',
    'vocalized': u'وَأَمَامَكَ',
    'stem': u'أَمَامَكَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَكَ',
    'encletic': u''
}
STOPWORDS[u'فأمامك'] = {
    'word': u'فأمامك',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم فعل:معطوف',
    'vocalized': u'فَأَمَامَكَ',
    'stem': u'أَمَامَكَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَكَ',
    'encletic': u''
}
STOPWORDS[u'أوه'] = {
    'word': u'أَوَّهْ',
    'procletic': u'',
    'tags': u'أداة:اسم فعل',
    'vocalized': u'أَوَّهْ',
    'stem': u'أَوَّهْ',
    'type': u'STOPWORD',
    'original': u'أَوَّهْ',
    'encletic': u''
}
STOPWORDS[u'وأوه'] = {
    'word': u'وأوه',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم فعل:معطوف',
    'vocalized': u'وَأَوَّهْ',
    'stem': u'أَوَّهْ',
    'type': u'STOPWORD',
    'original': u'أَوَّهْ',
    'encletic': u''
}
STOPWORDS[u'فأوه'] = {
    'word': u'فأوه',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم فعل:معطوف',
    'vocalized': u'فَأَوَّهْ',
    'stem': u'أَوَّهْ',
    'type': u'STOPWORD',
    'original': u'أَوَّهْ',
    'encletic': u''
}
STOPWORDS[u'إليك'] = {
    'word': u'إِلَيْكَ',
    'procletic': u'',
    'tags': u'أداة:اسم فعل',
    'vocalized': u'إِلَيْكَ',
    'stem': u'إِلَيْكَ',
    'type': u'STOPWORD',
    'original': u'إِلَيْكَ',
    'encletic': u''
}
STOPWORDS[u'وإليك'] = {
    'word': u'وإليك',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم فعل:معطوف',
    'vocalized': u'وَإِلَيْكَ',
    'stem': u'إِلَيْكَ',
    'type': u'STOPWORD',
    'original': u'إِلَيْكَ',
    'encletic': u''
}
STOPWORDS[u'فإليك'] = {
    'word': u'فإليك',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم فعل:معطوف',
    'vocalized': u'فَإِلَيْكَ',
    'stem': u'إِلَيْكَ',
    'type': u'STOPWORD',
    'original': u'إِلَيْكَ',
    'encletic': u''
}
STOPWORDS[u'إليكم'] = {
    'word': u'إِلَيْكُمْ',
    'procletic': u'',
    'tags': u'أداة:اسم فعل',
    'vocalized': u'إِلَيْكُمْ',
    'stem': u'إِلَيْكُمْ',
    'type': u'STOPWORD',
    'original': u'إِلَيْكُمْ',
    'encletic': u''
}
STOPWORDS[u'وإليكم'] = {
    'word': u'وإليكم',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم فعل:معطوف',
    'vocalized': u'وَإِلَيْكُمْ',
    'stem': u'إِلَيْكُمْ',
    'type': u'STOPWORD',
    'original': u'إِلَيْكُمْ',
    'encletic': u''
}
STOPWORDS[u'فإليكم'] = {
    'word': u'فإليكم',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم فعل:معطوف',
    'vocalized': u'فَإِلَيْكُمْ',
    'stem': u'إِلَيْكُمْ',
    'type': u'STOPWORD',
    'original': u'إِلَيْكُمْ',
    'encletic': u''
}
STOPWORDS[u'إليكما'] = {
    'word': u'إِلَيْكُمَا',
    'procletic': u'',
    'tags': u'أداة:اسم فعل',
    'vocalized': u'إِلَيْكُمَا',
    'stem': u'إِلَيْكُمَا',
    'type': u'STOPWORD',
    'original': u'إِلَيْكُمَا',
    'encletic': u''
}
STOPWORDS[u'وإليكما'] = {
    'word': u'وإليكما',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم فعل:معطوف',
    'vocalized': u'وَإِلَيْكُمَا',
    'stem': u'إِلَيْكُمَا',
    'type': u'STOPWORD',
    'original': u'إِلَيْكُمَا',
    'encletic': u''
}
STOPWORDS[u'فإليكما'] = {
    'word': u'فإليكما',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم فعل:معطوف',
    'vocalized': u'فَإِلَيْكُمَا',
    'stem': u'إِلَيْكُمَا',
    'type': u'STOPWORD',
    'original': u'إِلَيْكُمَا',
    'encletic': u''
}
STOPWORDS[u'إليكن'] = {
    'word': u'إِلَيْكُنَّ',
    'procletic': u'',
    'tags': u'أداة:اسم فعل',
    'vocalized': u'إِلَيْكُنَّ',
    'stem': u'إِلَيْكُنَّ',
    'type': u'STOPWORD',
    'original': u'إِلَيْكُنَّ',
    'encletic': u''
}
STOPWORDS[u'وإليكن'] = {
    'word': u'وإليكن',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم فعل:معطوف',
    'vocalized': u'وَإِلَيْكُنَّ',
    'stem': u'إِلَيْكُنَّ',
    'type': u'STOPWORD',
    'original': u'إِلَيْكُنَّ',
    'encletic': u''
}
STOPWORDS[u'فإليكن'] = {
    'word': u'فإليكن',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم فعل:معطوف',
    'vocalized': u'فَإِلَيْكُنَّ',
    'stem': u'إِلَيْكُنَّ',
    'type': u'STOPWORD',
    'original': u'إِلَيْكُنَّ',
    'encletic': u''
}
STOPWORDS[u'إيه'] = {
    'word': u'إيهِ',
    'procletic': u'',
    'tags': u'أداة:اسم فعل',
    'vocalized': u'إيهِ',
    'stem': u'إيهِ',
    'type': u'STOPWORD',
    'original': u'إيهِ',
    'encletic': u''
}
STOPWORDS[u'وإيه'] = {
    'word': u'وإيه',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم فعل:معطوف',
    'vocalized': u'وَإيهِ',
    'stem': u'إيهِ',
    'type': u'STOPWORD',
    'original': u'إيهِ',
    'encletic': u''
}
STOPWORDS[u'فإيه'] = {
    'word': u'فإيه',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم فعل:معطوف',
    'vocalized': u'فَإيهِ',
    'stem': u'إيهِ',
    'type': u'STOPWORD',
    'original': u'إيهِ',
    'encletic': u''
}
STOPWORDS[u'بخ'] = {
    'word': u'بخٍ',
    'procletic': u'',
    'tags': u'أداة:اسم فعل',
    'vocalized': u'بخٍ',
    'stem': u'بخٍ',
    'type': u'STOPWORD',
    'original': u'بخٍ',
    'encletic': u''
}
STOPWORDS[u'وبخ'] = {
    'word': u'وبخ',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم فعل:معطوف',
    'vocalized': u'وَبخٍ',
    'stem': u'بخٍ',
    'type': u'STOPWORD',
    'original': u'بخٍ',
    'encletic': u''
}
STOPWORDS[u'فبخ'] = {
    'word': u'فبخ',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم فعل:معطوف',
    'vocalized': u'فَبخٍ',
    'stem': u'بخٍ',
    'type': u'STOPWORD',
    'original': u'بخٍ',
    'encletic': u''
}
STOPWORDS[u'بس'] = {
    'word': u'بَسْ',
    'procletic': u'',
    'tags': u'أداة:اسم فعل',
    'vocalized': u'بَسْ',
    'stem': u'بَسْ',
    'type': u'STOPWORD',
    'original': u'بَسْ',
    'encletic': u''
}
STOPWORDS[u'وبس'] = {
    'word': u'وبس',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم فعل:معطوف',
    'vocalized': u'وَبَسْ',
    'stem': u'بَسْ',
    'type': u'STOPWORD',
    'original': u'بَسْ',
    'encletic': u''
}
STOPWORDS[u'فبس'] = {
    'word': u'فبس',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم فعل:معطوف',
    'vocalized': u'فَبَسْ',
    'stem': u'بَسْ',
    'type': u'STOPWORD',
    'original': u'بَسْ',
    'encletic': u''
}
STOPWORDS[u'بطآن'] = {
    'word': u'بُطْآنَ',
    'procletic': u'',
    'tags': u'أداة:اسم فعل',
    'vocalized': u'بُطْآنَ',
    'stem': u'بُطْآنَ',
    'type': u'STOPWORD',
    'original': u'بُطْآنَ',
    'encletic': u''
}
STOPWORDS[u'وبطآن'] = {
    'word': u'وبطآن',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم فعل:معطوف',
    'vocalized': u'وَبُطْآنَ',
    'stem': u'بُطْآنَ',
    'type': u'STOPWORD',
    'original': u'بُطْآنَ',
    'encletic': u''
}
STOPWORDS[u'فبطآن'] = {
    'word': u'فبطآن',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم فعل:معطوف',
    'vocalized': u'فَبُطْآنَ',
    'stem': u'بُطْآنَ',
    'type': u'STOPWORD',
    'original': u'بُطْآنَ',
    'encletic': u''
}
STOPWORDS[u'بله'] = {
    'word': u'بَلْهَ',
    'procletic': u'',
    'tags': u'أداة:اسم فعل',
    'vocalized': u'بَلْهَ',
    'stem': u'بَلْهَ',
    'type': u'STOPWORD',
    'original': u'بَلْهَ',
    'encletic': u''
}
STOPWORDS[u'وبله'] = {
    'word': u'وبله',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم فعل:معطوف',
    'vocalized': u'وَبَلْهَ',
    'stem': u'بَلْهَ',
    'type': u'STOPWORD',
    'original': u'بَلْهَ',
    'encletic': u''
}
STOPWORDS[u'فبله'] = {
    'word': u'فبله',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم فعل:معطوف',
    'vocalized': u'فَبَلْهَ',
    'stem': u'بَلْهَ',
    'type': u'STOPWORD',
    'original': u'بَلْهَ',
    'encletic': u''
}
STOPWORDS[u'حذار'] = {
    'word': u'حَذَارِ',
    'procletic': u'',
    'tags': u'أداة:اسم فعل',
    'vocalized': u'حَذَارِ',
    'stem': u'حَذَارِ',
    'type': u'STOPWORD',
    'original': u'حَذَارِ',
    'encletic': u''
}
STOPWORDS[u'وحذار'] = {
    'word': u'وحذار',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم فعل:معطوف',
    'vocalized': u'وَحَذَارِ',
    'stem': u'حَذَارِ',
    'type': u'STOPWORD',
    'original': u'حَذَارِ',
    'encletic': u''
}
STOPWORDS[u'فحذار'] = {
    'word': u'فحذار',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم فعل:معطوف',
    'vocalized': u'فَحَذَارِ',
    'stem': u'حَذَارِ',
    'type': u'STOPWORD',
    'original': u'حَذَارِ',
    'encletic': u''
}
STOPWORDS[u'حي'] = {
    'word': u'حَيَّ',
    'procletic': u'',
    'tags': u'أداة:اسم فعل',
    'vocalized': u'حَيَّ',
    'stem': u'حَيَّ',
    'type': u'STOPWORD',
    'original': u'حَيَّ',
    'encletic': u''
}
STOPWORDS[u'وحي'] = {
    'word': u'وحي',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم فعل:معطوف',
    'vocalized': u'وَحَيَّ',
    'stem': u'حَيَّ',
    'type': u'STOPWORD',
    'original': u'حَيَّ',
    'encletic': u''
}
STOPWORDS[u'فحي'] = {
    'word': u'فحي',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم فعل:معطوف',
    'vocalized': u'فَحَيَّ',
    'stem': u'حَيَّ',
    'type': u'STOPWORD',
    'original': u'حَيَّ',
    'encletic': u''
}
STOPWORDS[u'دونك'] = {
    'word': u'دُونَكَ',
    'procletic': u'',
    'tags': u'أداة:اسم فعل',
    'vocalized': u'دُونَكَ',
    'stem': u'دُونَكَ',
    'type': u'STOPWORD',
    'original': u'دُونَكَ',
    'encletic': u''
}
STOPWORDS[u'ودونك'] = {
    'word': u'ودونك',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم فعل:معطوف',
    'vocalized': u'وَدُونَكَ',
    'stem': u'دُونَكَ',
    'type': u'STOPWORD',
    'original': u'دُونَكَ',
    'encletic': u''
}
STOPWORDS[u'فدونك'] = {
    'word': u'فدونك',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم فعل:معطوف',
    'vocalized': u'فَدُونَكَ',
    'stem': u'دُونَكَ',
    'type': u'STOPWORD',
    'original': u'دُونَكَ',
    'encletic': u''
}
STOPWORDS[u'رويدك'] = {
    'word': u'رُوَيْدَكَ',
    'procletic': u'',
    'tags': u'أداة:اسم فعل',
    'vocalized': u'رُوَيْدَكَ',
    'stem': u'رُوَيْدَكَ',
    'type': u'STOPWORD',
    'original': u'رُوَيْدَكَ',
    'encletic': u''
}
STOPWORDS[u'ورويدك'] = {
    'word': u'ورويدك',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم فعل:معطوف',
    'vocalized': u'وَرُوَيْدَكَ',
    'stem': u'رُوَيْدَكَ',
    'type': u'STOPWORD',
    'original': u'رُوَيْدَكَ',
    'encletic': u''
}
STOPWORDS[u'فرويدك'] = {
    'word': u'فرويدك',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم فعل:معطوف',
    'vocalized': u'فَرُوَيْدَكَ',
    'stem': u'رُوَيْدَكَ',
    'type': u'STOPWORD',
    'original': u'رُوَيْدَكَ',
    'encletic': u''
}
STOPWORDS[u'سرعان'] = {
    'word': u'سُرْعَانَ',
    'procletic': u'',
    'tags': u'أداة:اسم فعل',
    'vocalized': u'سُرْعَانَ',
    'stem': u'سُرْعَانَ',
    'type': u'STOPWORD',
    'original': u'سُرْعَانَ',
    'encletic': u''
}
STOPWORDS[u'وسرعان'] = {
    'word': u'وسرعان',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم فعل:معطوف',
    'vocalized': u'وَسُرْعَانَ',
    'stem': u'سُرْعَانَ',
    'type': u'STOPWORD',
    'original': u'سُرْعَانَ',
    'encletic': u''
}
STOPWORDS[u'فسرعان'] = {
    'word': u'فسرعان',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم فعل:معطوف',
    'vocalized': u'فَسُرْعَانَ',
    'stem': u'سُرْعَانَ',
    'type': u'STOPWORD',
    'original': u'سُرْعَانَ',
    'encletic': u''
}
STOPWORDS[u'شتان'] = {
    'word': u'شَتَّانَ',
    'procletic': u'',
    'tags': u'أداة:اسم فعل',
    'vocalized': u'شَتَّانَ',
    'stem': u'شَتَّانَ',
    'type': u'STOPWORD',
    'original': u'شَتَّانَ',
    'encletic': u''
}
STOPWORDS[u'وشتان'] = {
    'word': u'وشتان',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم فعل:معطوف',
    'vocalized': u'وَشَتَّانَ',
    'stem': u'شَتَّانَ',
    'type': u'STOPWORD',
    'original': u'شَتَّانَ',
    'encletic': u''
}
STOPWORDS[u'فشتان'] = {
    'word': u'فشتان',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم فعل:معطوف',
    'vocalized': u'فَشَتَّانَ',
    'stem': u'شَتَّانَ',
    'type': u'STOPWORD',
    'original': u'شَتَّانَ',
    'encletic': u''
}
STOPWORDS[u'عليك'] = {
    'word': u'عَلَيْكَ',
    'procletic': u'',
    'tags': u'أداة:اسم فعل',
    'vocalized': u'عَلَيْكَ',
    'stem': u'عَلَيْكَ',
    'type': u'STOPWORD',
    'original': u'عَلَيْكَ',
    'encletic': u''
}
STOPWORDS[u'وعليك'] = {
    'word': u'وعليك',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم فعل:معطوف',
    'vocalized': u'وَعَلَيْكَ',
    'stem': u'عَلَيْكَ',
    'type': u'STOPWORD',
    'original': u'عَلَيْكَ',
    'encletic': u''
}
STOPWORDS[u'فعليك'] = {
    'word': u'فعليك',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم فعل:معطوف',
    'vocalized': u'فَعَلَيْكَ',
    'stem': u'عَلَيْكَ',
    'type': u'STOPWORD',
    'original': u'عَلَيْكَ',
    'encletic': u''
}
STOPWORDS[u'مكانك'] = {
    'word': u'مَكَانَكَ',
    'procletic': u'',
    'tags': u'أداة:اسم فعل',
    'vocalized': u'مَكَانَكَ',
    'stem': u'مَكَانَكَ',
    'type': u'STOPWORD',
    'original': u'مَكَانَكَ',
    'encletic': u''
}
STOPWORDS[u'ومكانك'] = {
    'word': u'ومكانك',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم فعل:معطوف',
    'vocalized': u'وَمَكَانَكَ',
    'stem': u'مَكَانَكَ',
    'type': u'STOPWORD',
    'original': u'مَكَانَكَ',
    'encletic': u''
}
STOPWORDS[u'فمكانك'] = {
    'word': u'فمكانك',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم فعل:معطوف',
    'vocalized': u'فَمَكَانَكَ',
    'stem': u'مَكَانَكَ',
    'type': u'STOPWORD',
    'original': u'مَكَانَكَ',
    'encletic': u''
}
STOPWORDS[u'مكانك'] = {
    'word': u'مَكَانَكِ',
    'procletic': u'',
    'tags': u'أداة:اسم فعل',
    'vocalized': u'مَكَانَكِ',
    'stem': u'مَكَانَكِ',
    'type': u'STOPWORD',
    'original': u'مَكَانَكِ',
    'encletic': u''
}
STOPWORDS[u'ومكانك'] = {
    'word': u'ومكانك',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم فعل:معطوف',
    'vocalized': u'وَمَكَانَكِ',
    'stem': u'مَكَانَكِ',
    'type': u'STOPWORD',
    'original': u'مَكَانَكِ',
    'encletic': u''
}
STOPWORDS[u'فمكانك'] = {
    'word': u'فمكانك',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم فعل:معطوف',
    'vocalized': u'فَمَكَانَكِ',
    'stem': u'مَكَانَكِ',
    'type': u'STOPWORD',
    'original': u'مَكَانَكِ',
    'encletic': u''
}
STOPWORDS[u'مكانكم'] = {
    'word': u'مَكَانَكُمْ',
    'procletic': u'',
    'tags': u'أداة:اسم فعل',
    'vocalized': u'مَكَانَكُمْ',
    'stem': u'مَكَانَكُمْ',
    'type': u'STOPWORD',
    'original': u'مَكَانَكُمْ',
    'encletic': u''
}
STOPWORDS[u'ومكانكم'] = {
    'word': u'ومكانكم',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم فعل:معطوف',
    'vocalized': u'وَمَكَانَكُمْ',
    'stem': u'مَكَانَكُمْ',
    'type': u'STOPWORD',
    'original': u'مَكَانَكُمْ',
    'encletic': u''
}
STOPWORDS[u'فمكانكم'] = {
    'word': u'فمكانكم',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم فعل:معطوف',
    'vocalized': u'فَمَكَانَكُمْ',
    'stem': u'مَكَانَكُمْ',
    'type': u'STOPWORD',
    'original': u'مَكَانَكُمْ',
    'encletic': u''
}
STOPWORDS[u'مكانكما'] = {
    'word': u'مَكَانَكُمَا',
    'procletic': u'',
    'tags': u'أداة:اسم فعل',
    'vocalized': u'مَكَانَكُمَا',
    'stem': u'مَكَانَكُمَا',
    'type': u'STOPWORD',
    'original': u'مَكَانَكُمَا',
    'encletic': u''
}
STOPWORDS[u'ومكانكما'] = {
    'word': u'ومكانكما',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم فعل:معطوف',
    'vocalized': u'وَمَكَانَكُمَا',
    'stem': u'مَكَانَكُمَا',
    'type': u'STOPWORD',
    'original': u'مَكَانَكُمَا',
    'encletic': u''
}
STOPWORDS[u'فمكانكما'] = {
    'word': u'فمكانكما',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم فعل:معطوف',
    'vocalized': u'فَمَكَانَكُمَا',
    'stem': u'مَكَانَكُمَا',
    'type': u'STOPWORD',
    'original': u'مَكَانَكُمَا',
    'encletic': u''
}
STOPWORDS[u'مكانكن'] = {
    'word': u'مَكَانَكُنَّ',
    'procletic': u'',
    'tags': u'أداة:اسم فعل',
    'vocalized': u'مَكَانَكُنَّ',
    'stem': u'مَكَانَكُنَّ',
    'type': u'STOPWORD',
    'original': u'مَكَانَكُنَّ',
    'encletic': u''
}
STOPWORDS[u'ومكانكن'] = {
    'word': u'ومكانكن',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم فعل:معطوف',
    'vocalized': u'وَمَكَانَكُنَّ',
    'stem': u'مَكَانَكُنَّ',
    'type': u'STOPWORD',
    'original': u'مَكَانَكُنَّ',
    'encletic': u''
}
STOPWORDS[u'فمكانكن'] = {
    'word': u'فمكانكن',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم فعل:معطوف',
    'vocalized': u'فَمَكَانَكُنَّ',
    'stem': u'مَكَانَكُنَّ',
    'type': u'STOPWORD',
    'original': u'مَكَانَكُنَّ',
    'encletic': u''
}
STOPWORDS[u'مه'] = {
    'word': u'مَهْ',
    'procletic': u'',
    'tags': u'أداة:اسم فعل',
    'vocalized': u'مَهْ',
    'stem': u'مَهْ',
    'type': u'STOPWORD',
    'original': u'مَهْ',
    'encletic': u''
}
STOPWORDS[u'ها'] = {
    'word': u'هَا',
    'procletic': u'',
    'tags': u'أداة:اسم فعل',
    'vocalized': u'هَا',
    'stem': u'هَا',
    'type': u'STOPWORD',
    'original': u'هَا',
    'encletic': u''
}
STOPWORDS[u'هاؤم'] = {
    'word': u'هَاؤُمُ',
    'procletic': u'',
    'tags': u'أداة:اسم فعل',
    'vocalized': u'هَاؤُمُ',
    'stem': u'هَاؤُمُ',
    'type': u'STOPWORD',
    'original': u'هَاؤُمُ',
    'encletic': u''
}
STOPWORDS[u'وهاؤم'] = {
    'word': u'وهاؤم',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم فعل:معطوف',
    'vocalized': u'وَهَاؤُمُ',
    'stem': u'هَاؤُمُ',
    'type': u'STOPWORD',
    'original': u'هَاؤُمُ',
    'encletic': u''
}
STOPWORDS[u'فهاؤم'] = {
    'word': u'فهاؤم',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم فعل:معطوف',
    'vocalized': u'فَهَاؤُمُ',
    'stem': u'هَاؤُمُ',
    'type': u'STOPWORD',
    'original': u'هَاؤُمُ',
    'encletic': u''
}
STOPWORDS[u'هاك'] = {
    'word': u'هَاكَ',
    'procletic': u'',
    'tags': u'أداة:اسم فعل',
    'vocalized': u'هَاكَ',
    'stem': u'هَاكَ',
    'type': u'STOPWORD',
    'original': u'هَاكَ',
    'encletic': u''
}
STOPWORDS[u'هلم'] = {
    'word': u'هَلُمَّ',
    'procletic': u'',
    'tags': u'أداة:اسم فعل',
    'vocalized': u'هَلُمَّ',
    'stem': u'هَلُمَّ',
    'type': u'STOPWORD',
    'original': u'هَلُمَّ',
    'encletic': u''
}
STOPWORDS[u'هيا'] = {
    'word': u'هَيَّا',
    'procletic': u'',
    'tags': u'أداة:اسم فعل',
    'vocalized': u'هَيَّا',
    'stem': u'هَيَّا',
    'type': u'STOPWORD',
    'original': u'هَيَّا',
    'encletic': u''
}
STOPWORDS[u'هيت'] = {
    'word': u'هِيتَ',
    'procletic': u'',
    'tags': u'أداة:اسم فعل',
    'vocalized': u'هِيتَ',
    'stem': u'هِيتَ',
    'type': u'STOPWORD',
    'original': u'هِيتَ',
    'encletic': u''
}
STOPWORDS[u'وهيت'] = {
    'word': u'وهيت',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم فعل:معطوف',
    'vocalized': u'وَهِيتَ',
    'stem': u'هِيتَ',
    'type': u'STOPWORD',
    'original': u'هِيتَ',
    'encletic': u''
}
STOPWORDS[u'فهيت'] = {
    'word': u'فهيت',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم فعل:معطوف',
    'vocalized': u'فَهِيتَ',
    'stem': u'هِيتَ',
    'type': u'STOPWORD',
    'original': u'هِيتَ',
    'encletic': u''
}
STOPWORDS[u'هيهات'] = {
    'word': u'هَيْهَاتَ',
    'procletic': u'',
    'tags': u'أداة:اسم فعل',
    'vocalized': u'هَيْهَاتَ',
    'stem': u'هَيْهَاتَ',
    'type': u'STOPWORD',
    'original': u'هَيْهَاتَ',
    'encletic': u''
}
STOPWORDS[u'وا'] = {
    'word': u'وَا',
    'procletic': u'',
    'tags': u'أداة:اسم فعل',
    'vocalized': u'وَا',
    'stem': u'وَا',
    'type': u'STOPWORD',
    'original': u'وَا',
    'encletic': u''
}
STOPWORDS[u'وراءك'] = {
    'word': u'وَرَاءَكَ',
    'procletic': u'',
    'tags': u'أداة:اسم فعل',
    'vocalized': u'وَرَاءَكَ',
    'stem': u'وَرَاءَكَ',
    'type': u'STOPWORD',
    'original': u'وَرَاءَكَ',
    'encletic': u''
}
STOPWORDS[u'وشكان'] = {
    'word': u'وُشْكَانََ',
    'procletic': u'',
    'tags': u'أداة:اسم فعل',
    'vocalized': u'وُشْكَانََ',
    'stem': u'وُشْكَانََ',
    'type': u'STOPWORD',
    'original': u'وُشْكَانََ',
    'encletic': u''
}
STOPWORDS[u'ويكأن'] = {
    'word': u'وَيْكَأَنَّ',
    'procletic': u'',
    'tags': u'أداة:اسم فعل',
    'vocalized': u'وَيْكَأَنَّ',
    'stem': u'وَيْكَأَنَّ',
    'type': u'STOPWORD',
    'original': u'وَيْكَأَنَّ',
    'encletic': u''
}
STOPWORDS[u'ويكأني'] = {
    'word': u'ويكأني',
    'procletic': u'',
    'tags': u'أداة:اسم فعل:مضاف',
    'vocalized': u'وَيْكَأَنَِّي',
    'stem': u'وَيْكَأَنَّ',
    'type': u'STOPWORD',
    'original': u'وَيْكَأَنَّ',
    'encletic': u'ِي'
}
STOPWORDS[u'ويكأنك'] = {
    'word': u'ويكأنك',
    'procletic': u'',
    'tags': u'أداة:اسم فعل:مضاف',
    'vocalized': u'وَيْكَأَنَّكَ',
    'stem': u'وَيْكَأَنَّ',
    'type': u'STOPWORD',
    'original': u'وَيْكَأَنَّ',
    'encletic': u'كَ'
}
STOPWORDS[u'ويكأنك'] = {
    'word': u'ويكأنك',
    'procletic': u'',
    'tags': u'أداة:اسم فعل:مضاف',
    'vocalized': u'وَيْكَأَنَّكِ',
    'stem': u'وَيْكَأَنَّ',
    'type': u'STOPWORD',
    'original': u'وَيْكَأَنَّ',
    'encletic': u'كِ'
}
STOPWORDS[u'ويكأنه'] = {
    'word': u'ويكأنه',
    'procletic': u'',
    'tags': u'أداة:اسم فعل:مضاف',
    'vocalized': u'وَيْكَأَنَّه',
    'stem': u'وَيْكَأَنَّ',
    'type': u'STOPWORD',
    'original': u'وَيْكَأَنَّ',
    'encletic': u'ه'
}
STOPWORDS[u'ويكأنكم'] = {
    'word': u'ويكأنكم',
    'procletic': u'',
    'tags': u'أداة:اسم فعل:مضاف',
    'vocalized': u'وَيْكَأَنَّكُمْ',
    'stem': u'وَيْكَأَنَّ',
    'type': u'STOPWORD',
    'original': u'وَيْكَأَنَّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ويكأنكن'] = {
    'word': u'ويكأنكن',
    'procletic': u'',
    'tags': u'أداة:اسم فعل:مضاف',
    'vocalized': u'وَيْكَأَنَّكُنَّ',
    'stem': u'وَيْكَأَنَّ',
    'type': u'STOPWORD',
    'original': u'وَيْكَأَنَّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ويكأنها'] = {
    'word': u'ويكأنها',
    'procletic': u'',
    'tags': u'أداة:اسم فعل:مضاف',
    'vocalized': u'وَيْكَأَنَّهَا',
    'stem': u'وَيْكَأَنَّ',
    'type': u'STOPWORD',
    'original': u'وَيْكَأَنَّ',
    'encletic': u'هَا'
}
STOPWORDS[u'ويكأنهم'] = {
    'word': u'ويكأنهم',
    'procletic': u'',
    'tags': u'أداة:اسم فعل:مضاف',
    'vocalized': u'وَيْكَأَنَّهُمْ',
    'stem': u'وَيْكَأَنَّ',
    'type': u'STOPWORD',
    'original': u'وَيْكَأَنَّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ويكأنهن'] = {
    'word': u'ويكأنهن',
    'procletic': u'',
    'tags': u'أداة:اسم فعل:مضاف',
    'vocalized': u'وَيْكَأَنَّهُنَّ',
    'stem': u'وَيْكَأَنَّ',
    'type': u'STOPWORD',
    'original': u'وَيْكَأَنَّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ويكأننا'] = {
    'word': u'ويكأننا',
    'procletic': u'',
    'tags': u'أداة:اسم فعل:مضاف',
    'vocalized': u'وَيْكَأَنَّنَا',
    'stem': u'وَيْكَأَنَّ',
    'type': u'STOPWORD',
    'original': u'وَيْكَأَنَّ',
    'encletic': u'نَا'
}
STOPWORDS[u'ويكأنكما'] = {
    'word': u'ويكأنكما',
    'procletic': u'',
    'tags': u'أداة:اسم فعل:مضاف',
    'vocalized': u'وَيْكَأَنَّكُمَا',
    'stem': u'وَيْكَأَنَّ',
    'type': u'STOPWORD',
    'original': u'وَيْكَأَنَّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ويكأنهما'] = {
    'word': u'ويكأنهما',
    'procletic': u'',
    'tags': u'أداة:اسم فعل:مضاف',
    'vocalized': u'وَيْكَأَنَّهُمَا',
    'stem': u'وَيْكَأَنَّ',
    'type': u'STOPWORD',
    'original': u'وَيْكَأَنَّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'الألاء'] = {
    'word': u'الْأَلَاءُ',
    'procletic': u'',
    'tags': u'أداة:اسم موصول',
    'vocalized': u'الْأَلَاءُ',
    'stem': u'الْأَلَاءُ',
    'type': u'STOPWORD',
    'original': u'الْأَلَاءُ',
    'encletic': u''
}
STOPWORDS[u'بالألاء'] = {
    'word': u'بالألاء',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم موصول:مجرور',
    'vocalized': u'بِالْأَلَاءُ',
    'stem': u'الْأَلَاءُ',
    'type': u'STOPWORD',
    'original': u'الْأَلَاءُ',
    'encletic': u''
}
STOPWORDS[u'كالألاء'] = {
    'word': u'كالألاء',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم موصول:مجرور',
    'vocalized': u'كَالْأَلَاءُ',
    'stem': u'الْأَلَاءُ',
    'type': u'STOPWORD',
    'original': u'الْأَلَاءُ',
    'encletic': u''
}
STOPWORDS[u'للألاء'] = {
    'word': u'للألاء',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم موصول:مجرور',
    'vocalized': u'لِالْأَلَاءُ',
    'stem': u'الْأَلَاءُ',
    'type': u'STOPWORD',
    'original': u'الْأَلَاءُ',
    'encletic': u''
}
STOPWORDS[u'والألاء'] = {
    'word': u'والألاء',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم موصول:معطوف',
    'vocalized': u'وَالْأَلَاءُ',
    'stem': u'الْأَلَاءُ',
    'type': u'STOPWORD',
    'original': u'الْأَلَاءُ',
    'encletic': u''
}
STOPWORDS[u'فالألاء'] = {
    'word': u'فالألاء',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم موصول:معطوف',
    'vocalized': u'فَالْأَلَاءُ',
    'stem': u'الْأَلَاءُ',
    'type': u'STOPWORD',
    'original': u'الْأَلَاءُ',
    'encletic': u''
}
STOPWORDS[u'وبالألاء'] = {
    'word': u'وبالألاء',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم موصول:مجرور:معطوف',
    'vocalized': u'وَبِالْأَلَاءُ',
    'stem': u'الْأَلَاءُ',
    'type': u'STOPWORD',
    'original': u'الْأَلَاءُ',
    'encletic': u''
}
STOPWORDS[u'فبالألاء'] = {
    'word': u'فبالألاء',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم موصول:مجرور:معطوف',
    'vocalized': u'فَبِالْأَلَاءُ',
    'stem': u'الْأَلَاءُ',
    'type': u'STOPWORD',
    'original': u'الْأَلَاءُ',
    'encletic': u''
}
STOPWORDS[u'وكالألاء'] = {
    'word': u'وكالألاء',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم موصول:مجرور:معطوف',
    'vocalized': u'وَكَالْأَلَاءُ',
    'stem': u'الْأَلَاءُ',
    'type': u'STOPWORD',
    'original': u'الْأَلَاءُ',
    'encletic': u''
}
STOPWORDS[u'فكالألاء'] = {
    'word': u'فكالألاء',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم موصول:مجرور:معطوف',
    'vocalized': u'فَكَالْأَلَاءُ',
    'stem': u'الْأَلَاءُ',
    'type': u'STOPWORD',
    'original': u'الْأَلَاءُ',
    'encletic': u''
}
STOPWORDS[u'وللألاء'] = {
    'word': u'وللألاء',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم موصول:مجرور:معطوف',
    'vocalized': u'وَلِالْأَلَاءُ',
    'stem': u'الْأَلَاءُ',
    'type': u'STOPWORD',
    'original': u'الْأَلَاءُ',
    'encletic': u''
}
STOPWORDS[u'فللألاء'] = {
    'word': u'فللألاء',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم موصول:مجرور:معطوف',
    'vocalized': u'فَلِالْأَلَاءُ',
    'stem': u'الْأَلَاءُ',
    'type': u'STOPWORD',
    'original': u'الْأَلَاءُ',
    'encletic': u''
}
STOPWORDS[u'الألى'] = {
    'word': u'الْأُلَى',
    'procletic': u'',
    'tags': u'أداة:اسم موصول',
    'vocalized': u'الْأُلَى',
    'stem': u'الْأُلَى',
    'type': u'STOPWORD',
    'original': u'الْأُلَى',
    'encletic': u''
}
STOPWORDS[u'بالألى'] = {
    'word': u'بالألى',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم موصول:مجرور',
    'vocalized': u'بِالْأُلَى',
    'stem': u'الْأُلَى',
    'type': u'STOPWORD',
    'original': u'الْأُلَى',
    'encletic': u''
}
STOPWORDS[u'كالألى'] = {
    'word': u'كالألى',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم موصول:مجرور',
    'vocalized': u'كَالْأُلَى',
    'stem': u'الْأُلَى',
    'type': u'STOPWORD',
    'original': u'الْأُلَى',
    'encletic': u''
}
STOPWORDS[u'للألى'] = {
    'word': u'للألى',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم موصول:مجرور',
    'vocalized': u'لِالْأُلَى',
    'stem': u'الْأُلَى',
    'type': u'STOPWORD',
    'original': u'الْأُلَى',
    'encletic': u''
}
STOPWORDS[u'والألى'] = {
    'word': u'والألى',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم موصول:معطوف',
    'vocalized': u'وَالْأُلَى',
    'stem': u'الْأُلَى',
    'type': u'STOPWORD',
    'original': u'الْأُلَى',
    'encletic': u''
}
STOPWORDS[u'فالألى'] = {
    'word': u'فالألى',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم موصول:معطوف',
    'vocalized': u'فَالْأُلَى',
    'stem': u'الْأُلَى',
    'type': u'STOPWORD',
    'original': u'الْأُلَى',
    'encletic': u''
}
STOPWORDS[u'وبالألى'] = {
    'word': u'وبالألى',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم موصول:مجرور:معطوف',
    'vocalized': u'وَبِالْأُلَى',
    'stem': u'الْأُلَى',
    'type': u'STOPWORD',
    'original': u'الْأُلَى',
    'encletic': u''
}
STOPWORDS[u'فبالألى'] = {
    'word': u'فبالألى',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم موصول:مجرور:معطوف',
    'vocalized': u'فَبِالْأُلَى',
    'stem': u'الْأُلَى',
    'type': u'STOPWORD',
    'original': u'الْأُلَى',
    'encletic': u''
}
STOPWORDS[u'وكالألى'] = {
    'word': u'وكالألى',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم موصول:مجرور:معطوف',
    'vocalized': u'وَكَالْأُلَى',
    'stem': u'الْأُلَى',
    'type': u'STOPWORD',
    'original': u'الْأُلَى',
    'encletic': u''
}
STOPWORDS[u'فكالألى'] = {
    'word': u'فكالألى',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم موصول:مجرور:معطوف',
    'vocalized': u'فَكَالْأُلَى',
    'stem': u'الْأُلَى',
    'type': u'STOPWORD',
    'original': u'الْأُلَى',
    'encletic': u''
}
STOPWORDS[u'وللألى'] = {
    'word': u'وللألى',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم موصول:مجرور:معطوف',
    'vocalized': u'وَلِالْأُلَى',
    'stem': u'الْأُلَى',
    'type': u'STOPWORD',
    'original': u'الْأُلَى',
    'encletic': u''
}
STOPWORDS[u'فللألى'] = {
    'word': u'فللألى',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم موصول:مجرور:معطوف',
    'vocalized': u'فَلِالْأُلَى',
    'stem': u'الْأُلَى',
    'type': u'STOPWORD',
    'original': u'الْأُلَى',
    'encletic': u''
}
STOPWORDS[u'التي'] = {
    'word': u'الَّتِي',
    'procletic': u'',
    'tags': u'أداة:اسم موصول',
    'vocalized': u'الَّتِي',
    'stem': u'الَّتِي',
    'type': u'STOPWORD',
    'original': u'الَّتِي',
    'encletic': u''
}
STOPWORDS[u'بالتي'] = {
    'word': u'بالتي',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم موصول:مجرور',
    'vocalized': u'بِالَّتِي',
    'stem': u'الَّتِي',
    'type': u'STOPWORD',
    'original': u'الَّتِي',
    'encletic': u''
}
STOPWORDS[u'كالتي'] = {
    'word': u'كالتي',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم موصول:مجرور',
    'vocalized': u'كَالَّتِي',
    'stem': u'الَّتِي',
    'type': u'STOPWORD',
    'original': u'الَّتِي',
    'encletic': u''
}
STOPWORDS[u'للتي'] = {
    'word': u'للتي',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم موصول:مجرور',
    'vocalized': u'لِالَّتِي',
    'stem': u'الَّتِي',
    'type': u'STOPWORD',
    'original': u'الَّتِي',
    'encletic': u''
}
STOPWORDS[u'والتي'] = {
    'word': u'والتي',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم موصول:معطوف',
    'vocalized': u'وَالَّتِي',
    'stem': u'الَّتِي',
    'type': u'STOPWORD',
    'original': u'الَّتِي',
    'encletic': u''
}
STOPWORDS[u'فالتي'] = {
    'word': u'فالتي',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم موصول:معطوف',
    'vocalized': u'فَالَّتِي',
    'stem': u'الَّتِي',
    'type': u'STOPWORD',
    'original': u'الَّتِي',
    'encletic': u''
}
STOPWORDS[u'وبالتي'] = {
    'word': u'وبالتي',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم موصول:مجرور:معطوف',
    'vocalized': u'وَبِالَّتِي',
    'stem': u'الَّتِي',
    'type': u'STOPWORD',
    'original': u'الَّتِي',
    'encletic': u''
}
STOPWORDS[u'فبالتي'] = {
    'word': u'فبالتي',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم موصول:مجرور:معطوف',
    'vocalized': u'فَبِالَّتِي',
    'stem': u'الَّتِي',
    'type': u'STOPWORD',
    'original': u'الَّتِي',
    'encletic': u''
}
STOPWORDS[u'وكالتي'] = {
    'word': u'وكالتي',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم موصول:مجرور:معطوف',
    'vocalized': u'وَكَالَّتِي',
    'stem': u'الَّتِي',
    'type': u'STOPWORD',
    'original': u'الَّتِي',
    'encletic': u''
}
STOPWORDS[u'فكالتي'] = {
    'word': u'فكالتي',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم موصول:مجرور:معطوف',
    'vocalized': u'فَكَالَّتِي',
    'stem': u'الَّتِي',
    'type': u'STOPWORD',
    'original': u'الَّتِي',
    'encletic': u''
}
STOPWORDS[u'وللتي'] = {
    'word': u'وللتي',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم موصول:مجرور:معطوف',
    'vocalized': u'وَلِالَّتِي',
    'stem': u'الَّتِي',
    'type': u'STOPWORD',
    'original': u'الَّتِي',
    'encletic': u''
}
STOPWORDS[u'فللتي'] = {
    'word': u'فللتي',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم موصول:مجرور:معطوف',
    'vocalized': u'فَلِالَّتِي',
    'stem': u'الَّتِي',
    'type': u'STOPWORD',
    'original': u'الَّتِي',
    'encletic': u''
}
STOPWORDS[u'الذي'] = {
    'word': u'الَّذِي',
    'procletic': u'',
    'tags': u'أداة:اسم موصول',
    'vocalized': u'الَّذِي',
    'stem': u'الَّذِي',
    'type': u'STOPWORD',
    'original': u'الَّذِي',
    'encletic': u''
}
STOPWORDS[u'بالذي'] = {
    'word': u'بالذي',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم موصول:مجرور',
    'vocalized': u'بِالَّذِي',
    'stem': u'الَّذِي',
    'type': u'STOPWORD',
    'original': u'الَّذِي',
    'encletic': u''
}
STOPWORDS[u'كالذي'] = {
    'word': u'كالذي',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم موصول:مجرور',
    'vocalized': u'كَالَّذِي',
    'stem': u'الَّذِي',
    'type': u'STOPWORD',
    'original': u'الَّذِي',
    'encletic': u''
}
STOPWORDS[u'للذي'] = {
    'word': u'للذي',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم موصول:مجرور',
    'vocalized': u'لِالَّذِي',
    'stem': u'الَّذِي',
    'type': u'STOPWORD',
    'original': u'الَّذِي',
    'encletic': u''
}
STOPWORDS[u'والذي'] = {
    'word': u'والذي',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم موصول:معطوف',
    'vocalized': u'وَالَّذِي',
    'stem': u'الَّذِي',
    'type': u'STOPWORD',
    'original': u'الَّذِي',
    'encletic': u''
}
STOPWORDS[u'فالذي'] = {
    'word': u'فالذي',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم موصول:معطوف',
    'vocalized': u'فَالَّذِي',
    'stem': u'الَّذِي',
    'type': u'STOPWORD',
    'original': u'الَّذِي',
    'encletic': u''
}
STOPWORDS[u'وبالذي'] = {
    'word': u'وبالذي',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم موصول:مجرور:معطوف',
    'vocalized': u'وَبِالَّذِي',
    'stem': u'الَّذِي',
    'type': u'STOPWORD',
    'original': u'الَّذِي',
    'encletic': u''
}
STOPWORDS[u'فبالذي'] = {
    'word': u'فبالذي',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم موصول:مجرور:معطوف',
    'vocalized': u'فَبِالَّذِي',
    'stem': u'الَّذِي',
    'type': u'STOPWORD',
    'original': u'الَّذِي',
    'encletic': u''
}
STOPWORDS[u'وكالذي'] = {
    'word': u'وكالذي',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم موصول:مجرور:معطوف',
    'vocalized': u'وَكَالَّذِي',
    'stem': u'الَّذِي',
    'type': u'STOPWORD',
    'original': u'الَّذِي',
    'encletic': u''
}
STOPWORDS[u'فكالذي'] = {
    'word': u'فكالذي',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم موصول:مجرور:معطوف',
    'vocalized': u'فَكَالَّذِي',
    'stem': u'الَّذِي',
    'type': u'STOPWORD',
    'original': u'الَّذِي',
    'encletic': u''
}
STOPWORDS[u'وللذي'] = {
    'word': u'وللذي',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم موصول:مجرور:معطوف',
    'vocalized': u'وَلِالَّذِي',
    'stem': u'الَّذِي',
    'type': u'STOPWORD',
    'original': u'الَّذِي',
    'encletic': u''
}
STOPWORDS[u'فللذي'] = {
    'word': u'فللذي',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم موصول:مجرور:معطوف',
    'vocalized': u'فَلِالَّذِي',
    'stem': u'الَّذِي',
    'type': u'STOPWORD',
    'original': u'الَّذِي',
    'encletic': u''
}
STOPWORDS[u'الذين'] = {
    'word': u'الَّذِينَ',
    'procletic': u'',
    'tags': u'أداة:اسم موصول',
    'vocalized': u'الَّذِينَ',
    'stem': u'الَّذِينَ',
    'type': u'STOPWORD',
    'original': u'الَّذِينَ',
    'encletic': u''
}
STOPWORDS[u'بالذين'] = {
    'word': u'بالذين',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم موصول:مجرور',
    'vocalized': u'بِالَّذِينَ',
    'stem': u'الَّذِينَ',
    'type': u'STOPWORD',
    'original': u'الَّذِينَ',
    'encletic': u''
}
STOPWORDS[u'كالذين'] = {
    'word': u'كالذين',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم موصول:مجرور',
    'vocalized': u'كَالَّذِينَ',
    'stem': u'الَّذِينَ',
    'type': u'STOPWORD',
    'original': u'الَّذِينَ',
    'encletic': u''
}
STOPWORDS[u'للذين'] = {
    'word': u'للذين',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم موصول:مجرور',
    'vocalized': u'لِالَّذِينَ',
    'stem': u'الَّذِينَ',
    'type': u'STOPWORD',
    'original': u'الَّذِينَ',
    'encletic': u''
}
STOPWORDS[u'والذين'] = {
    'word': u'والذين',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم موصول:معطوف',
    'vocalized': u'وَالَّذِينَ',
    'stem': u'الَّذِينَ',
    'type': u'STOPWORD',
    'original': u'الَّذِينَ',
    'encletic': u''
}
STOPWORDS[u'فالذين'] = {
    'word': u'فالذين',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم موصول:معطوف',
    'vocalized': u'فَالَّذِينَ',
    'stem': u'الَّذِينَ',
    'type': u'STOPWORD',
    'original': u'الَّذِينَ',
    'encletic': u''
}
STOPWORDS[u'وبالذين'] = {
    'word': u'وبالذين',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم موصول:مجرور:معطوف',
    'vocalized': u'وَبِالَّذِينَ',
    'stem': u'الَّذِينَ',
    'type': u'STOPWORD',
    'original': u'الَّذِينَ',
    'encletic': u''
}
STOPWORDS[u'فبالذين'] = {
    'word': u'فبالذين',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم موصول:مجرور:معطوف',
    'vocalized': u'فَبِالَّذِينَ',
    'stem': u'الَّذِينَ',
    'type': u'STOPWORD',
    'original': u'الَّذِينَ',
    'encletic': u''
}
STOPWORDS[u'وكالذين'] = {
    'word': u'وكالذين',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم موصول:مجرور:معطوف',
    'vocalized': u'وَكَالَّذِينَ',
    'stem': u'الَّذِينَ',
    'type': u'STOPWORD',
    'original': u'الَّذِينَ',
    'encletic': u''
}
STOPWORDS[u'فكالذين'] = {
    'word': u'فكالذين',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم موصول:مجرور:معطوف',
    'vocalized': u'فَكَالَّذِينَ',
    'stem': u'الَّذِينَ',
    'type': u'STOPWORD',
    'original': u'الَّذِينَ',
    'encletic': u''
}
STOPWORDS[u'وللذين'] = {
    'word': u'وللذين',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم موصول:مجرور:معطوف',
    'vocalized': u'وَلِالَّذِينَ',
    'stem': u'الَّذِينَ',
    'type': u'STOPWORD',
    'original': u'الَّذِينَ',
    'encletic': u''
}
STOPWORDS[u'فللذين'] = {
    'word': u'فللذين',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم موصول:مجرور:معطوف',
    'vocalized': u'فَلِالَّذِينَ',
    'stem': u'الَّذِينَ',
    'type': u'STOPWORD',
    'original': u'الَّذِينَ',
    'encletic': u''
}
STOPWORDS[u'اللائي'] = {
    'word': u'الْلَائِي',
    'procletic': u'',
    'tags': u'أداة:اسم موصول',
    'vocalized': u'الْلَائِي',
    'stem': u'الْلَائِي',
    'type': u'STOPWORD',
    'original': u'الْلَائِي',
    'encletic': u''
}
STOPWORDS[u'باللائي'] = {
    'word': u'باللائي',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم موصول:مجرور',
    'vocalized': u'بِالْلَائِي',
    'stem': u'الْلَائِي',
    'type': u'STOPWORD',
    'original': u'الْلَائِي',
    'encletic': u''
}
STOPWORDS[u'كاللائي'] = {
    'word': u'كاللائي',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم موصول:مجرور',
    'vocalized': u'كَالْلَائِي',
    'stem': u'الْلَائِي',
    'type': u'STOPWORD',
    'original': u'الْلَائِي',
    'encletic': u''
}
STOPWORDS[u'للائي'] = {
    'word': u'للائي',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم موصول:مجرور',
    'vocalized': u'لِالْلَائِي',
    'stem': u'الْلَائِي',
    'type': u'STOPWORD',
    'original': u'الْلَائِي',
    'encletic': u''
}
STOPWORDS[u'واللائي'] = {
    'word': u'واللائي',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم موصول:معطوف',
    'vocalized': u'وَالْلَائِي',
    'stem': u'الْلَائِي',
    'type': u'STOPWORD',
    'original': u'الْلَائِي',
    'encletic': u''
}
STOPWORDS[u'فاللائي'] = {
    'word': u'فاللائي',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم موصول:معطوف',
    'vocalized': u'فَالْلَائِي',
    'stem': u'الْلَائِي',
    'type': u'STOPWORD',
    'original': u'الْلَائِي',
    'encletic': u''
}
STOPWORDS[u'وباللائي'] = {
    'word': u'وباللائي',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم موصول:مجرور:معطوف',
    'vocalized': u'وَبِالْلَائِي',
    'stem': u'الْلَائِي',
    'type': u'STOPWORD',
    'original': u'الْلَائِي',
    'encletic': u''
}
STOPWORDS[u'فباللائي'] = {
    'word': u'فباللائي',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم موصول:مجرور:معطوف',
    'vocalized': u'فَبِالْلَائِي',
    'stem': u'الْلَائِي',
    'type': u'STOPWORD',
    'original': u'الْلَائِي',
    'encletic': u''
}
STOPWORDS[u'وكاللائي'] = {
    'word': u'وكاللائي',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم موصول:مجرور:معطوف',
    'vocalized': u'وَكَالْلَائِي',
    'stem': u'الْلَائِي',
    'type': u'STOPWORD',
    'original': u'الْلَائِي',
    'encletic': u''
}
STOPWORDS[u'فكاللائي'] = {
    'word': u'فكاللائي',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم موصول:مجرور:معطوف',
    'vocalized': u'فَكَالْلَائِي',
    'stem': u'الْلَائِي',
    'type': u'STOPWORD',
    'original': u'الْلَائِي',
    'encletic': u''
}
STOPWORDS[u'وللائي'] = {
    'word': u'وللائي',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم موصول:مجرور:معطوف',
    'vocalized': u'وَلِالْلَائِي',
    'stem': u'الْلَائِي',
    'type': u'STOPWORD',
    'original': u'الْلَائِي',
    'encletic': u''
}
STOPWORDS[u'فللائي'] = {
    'word': u'فللائي',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم موصول:مجرور:معطوف',
    'vocalized': u'فَلِالْلَائِي',
    'stem': u'الْلَائِي',
    'type': u'STOPWORD',
    'original': u'الْلَائِي',
    'encletic': u''
}
STOPWORDS[u'اللاتي'] = {
    'word': u'الْلَاتِي',
    'procletic': u'',
    'tags': u'أداة:اسم موصول',
    'vocalized': u'الْلَاتِي',
    'stem': u'الْلَاتِي',
    'type': u'STOPWORD',
    'original': u'الْلَاتِي',
    'encletic': u''
}
STOPWORDS[u'باللاتي'] = {
    'word': u'باللاتي',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم موصول:مجرور',
    'vocalized': u'بِالْلَاتِي',
    'stem': u'الْلَاتِي',
    'type': u'STOPWORD',
    'original': u'الْلَاتِي',
    'encletic': u''
}
STOPWORDS[u'كاللاتي'] = {
    'word': u'كاللاتي',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم موصول:مجرور',
    'vocalized': u'كَالْلَاتِي',
    'stem': u'الْلَاتِي',
    'type': u'STOPWORD',
    'original': u'الْلَاتِي',
    'encletic': u''
}
STOPWORDS[u'للاتي'] = {
    'word': u'للاتي',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم موصول:مجرور',
    'vocalized': u'لِالْلَاتِي',
    'stem': u'الْلَاتِي',
    'type': u'STOPWORD',
    'original': u'الْلَاتِي',
    'encletic': u''
}
STOPWORDS[u'واللاتي'] = {
    'word': u'واللاتي',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم موصول:معطوف',
    'vocalized': u'وَالْلَاتِي',
    'stem': u'الْلَاتِي',
    'type': u'STOPWORD',
    'original': u'الْلَاتِي',
    'encletic': u''
}
STOPWORDS[u'فاللاتي'] = {
    'word': u'فاللاتي',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم موصول:معطوف',
    'vocalized': u'فَالْلَاتِي',
    'stem': u'الْلَاتِي',
    'type': u'STOPWORD',
    'original': u'الْلَاتِي',
    'encletic': u''
}
STOPWORDS[u'وباللاتي'] = {
    'word': u'وباللاتي',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم موصول:مجرور:معطوف',
    'vocalized': u'وَبِالْلَاتِي',
    'stem': u'الْلَاتِي',
    'type': u'STOPWORD',
    'original': u'الْلَاتِي',
    'encletic': u''
}
STOPWORDS[u'فباللاتي'] = {
    'word': u'فباللاتي',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم موصول:مجرور:معطوف',
    'vocalized': u'فَبِالْلَاتِي',
    'stem': u'الْلَاتِي',
    'type': u'STOPWORD',
    'original': u'الْلَاتِي',
    'encletic': u''
}
STOPWORDS[u'وكاللاتي'] = {
    'word': u'وكاللاتي',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم موصول:مجرور:معطوف',
    'vocalized': u'وَكَالْلَاتِي',
    'stem': u'الْلَاتِي',
    'type': u'STOPWORD',
    'original': u'الْلَاتِي',
    'encletic': u''
}
STOPWORDS[u'فكاللاتي'] = {
    'word': u'فكاللاتي',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم موصول:مجرور:معطوف',
    'vocalized': u'فَكَالْلَاتِي',
    'stem': u'الْلَاتِي',
    'type': u'STOPWORD',
    'original': u'الْلَاتِي',
    'encletic': u''
}
STOPWORDS[u'وللاتي'] = {
    'word': u'وللاتي',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم موصول:مجرور:معطوف',
    'vocalized': u'وَلِالْلَاتِي',
    'stem': u'الْلَاتِي',
    'type': u'STOPWORD',
    'original': u'الْلَاتِي',
    'encletic': u''
}
STOPWORDS[u'فللاتي'] = {
    'word': u'فللاتي',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم موصول:مجرور:معطوف',
    'vocalized': u'فَلِالْلَاتِي',
    'stem': u'الْلَاتِي',
    'type': u'STOPWORD',
    'original': u'الْلَاتِي',
    'encletic': u''
}
STOPWORDS[u'اللتان'] = {
    'word': u'الْلَتَانِ',
    'procletic': u'',
    'tags': u'أداة:اسم موصول',
    'vocalized': u'الْلَتَانِ',
    'stem': u'الْلَتَانِ',
    'type': u'STOPWORD',
    'original': u'الْلَتَانِ',
    'encletic': u''
}
STOPWORDS[u'واللتان'] = {
    'word': u'واللتان',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم موصول:معطوف',
    'vocalized': u'وَالْلَتَانِ',
    'stem': u'الْلَتَانِ',
    'type': u'STOPWORD',
    'original': u'الْلَتَانِ',
    'encletic': u''
}
STOPWORDS[u'فاللتان'] = {
    'word': u'فاللتان',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم موصول:معطوف',
    'vocalized': u'فَالْلَتَانِ',
    'stem': u'الْلَتَانِ',
    'type': u'STOPWORD',
    'original': u'الْلَتَانِ',
    'encletic': u''
}
STOPWORDS[u'اللتيا'] = {
    'word': u'الْلَتَيَّا',
    'procletic': u'',
    'tags': u'أداة:اسم موصول',
    'vocalized': u'الْلَتَيَّا',
    'stem': u'الْلَتَيَّا',
    'type': u'STOPWORD',
    'original': u'الْلَتَيَّا',
    'encletic': u''
}
STOPWORDS[u'باللتيا'] = {
    'word': u'باللتيا',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم موصول:مجرور',
    'vocalized': u'بِالْلَتَيَّا',
    'stem': u'الْلَتَيَّا',
    'type': u'STOPWORD',
    'original': u'الْلَتَيَّا',
    'encletic': u''
}
STOPWORDS[u'كاللتيا'] = {
    'word': u'كاللتيا',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم موصول:مجرور',
    'vocalized': u'كَالْلَتَيَّا',
    'stem': u'الْلَتَيَّا',
    'type': u'STOPWORD',
    'original': u'الْلَتَيَّا',
    'encletic': u''
}
STOPWORDS[u'للتيا'] = {
    'word': u'للتيا',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم موصول:مجرور',
    'vocalized': u'لِالْلَتَيَّا',
    'stem': u'الْلَتَيَّا',
    'type': u'STOPWORD',
    'original': u'الْلَتَيَّا',
    'encletic': u''
}
STOPWORDS[u'واللتيا'] = {
    'word': u'واللتيا',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم موصول:معطوف',
    'vocalized': u'وَالْلَتَيَّا',
    'stem': u'الْلَتَيَّا',
    'type': u'STOPWORD',
    'original': u'الْلَتَيَّا',
    'encletic': u''
}
STOPWORDS[u'فاللتيا'] = {
    'word': u'فاللتيا',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم موصول:معطوف',
    'vocalized': u'فَالْلَتَيَّا',
    'stem': u'الْلَتَيَّا',
    'type': u'STOPWORD',
    'original': u'الْلَتَيَّا',
    'encletic': u''
}
STOPWORDS[u'وباللتيا'] = {
    'word': u'وباللتيا',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم موصول:مجرور:معطوف',
    'vocalized': u'وَبِالْلَتَيَّا',
    'stem': u'الْلَتَيَّا',
    'type': u'STOPWORD',
    'original': u'الْلَتَيَّا',
    'encletic': u''
}
STOPWORDS[u'فباللتيا'] = {
    'word': u'فباللتيا',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم موصول:مجرور:معطوف',
    'vocalized': u'فَبِالْلَتَيَّا',
    'stem': u'الْلَتَيَّا',
    'type': u'STOPWORD',
    'original': u'الْلَتَيَّا',
    'encletic': u''
}
STOPWORDS[u'وكاللتيا'] = {
    'word': u'وكاللتيا',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم موصول:مجرور:معطوف',
    'vocalized': u'وَكَالْلَتَيَّا',
    'stem': u'الْلَتَيَّا',
    'type': u'STOPWORD',
    'original': u'الْلَتَيَّا',
    'encletic': u''
}
STOPWORDS[u'فكاللتيا'] = {
    'word': u'فكاللتيا',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم موصول:مجرور:معطوف',
    'vocalized': u'فَكَالْلَتَيَّا',
    'stem': u'الْلَتَيَّا',
    'type': u'STOPWORD',
    'original': u'الْلَتَيَّا',
    'encletic': u''
}
STOPWORDS[u'وللتيا'] = {
    'word': u'وللتيا',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم موصول:مجرور:معطوف',
    'vocalized': u'وَلِالْلَتَيَّا',
    'stem': u'الْلَتَيَّا',
    'type': u'STOPWORD',
    'original': u'الْلَتَيَّا',
    'encletic': u''
}
STOPWORDS[u'فللتيا'] = {
    'word': u'فللتيا',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم موصول:مجرور:معطوف',
    'vocalized': u'فَلِالْلَتَيَّا',
    'stem': u'الْلَتَيَّا',
    'type': u'STOPWORD',
    'original': u'الْلَتَيَّا',
    'encletic': u''
}
STOPWORDS[u'اللتين'] = {
    'word': u'الْلَتَيْنِ',
    'procletic': u'',
    'tags': u'أداة:اسم موصول',
    'vocalized': u'الْلَتَيْنِ',
    'stem': u'الْلَتَيْنِ',
    'type': u'STOPWORD',
    'original': u'الْلَتَيْنِ',
    'encletic': u''
}
STOPWORDS[u'باللتين'] = {
    'word': u'باللتين',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم موصول:مجرور',
    'vocalized': u'بِالْلَتَيْنِ',
    'stem': u'الْلَتَيْنِ',
    'type': u'STOPWORD',
    'original': u'الْلَتَيْنِ',
    'encletic': u''
}
STOPWORDS[u'كاللتين'] = {
    'word': u'كاللتين',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم موصول:مجرور',
    'vocalized': u'كَالْلَتَيْنِ',
    'stem': u'الْلَتَيْنِ',
    'type': u'STOPWORD',
    'original': u'الْلَتَيْنِ',
    'encletic': u''
}
STOPWORDS[u'للتين'] = {
    'word': u'للتين',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم موصول:مجرور',
    'vocalized': u'لِالْلَتَيْنِ',
    'stem': u'الْلَتَيْنِ',
    'type': u'STOPWORD',
    'original': u'الْلَتَيْنِ',
    'encletic': u''
}
STOPWORDS[u'واللتين'] = {
    'word': u'واللتين',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم موصول:معطوف',
    'vocalized': u'وَالْلَتَيْنِ',
    'stem': u'الْلَتَيْنِ',
    'type': u'STOPWORD',
    'original': u'الْلَتَيْنِ',
    'encletic': u''
}
STOPWORDS[u'فاللتين'] = {
    'word': u'فاللتين',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم موصول:معطوف',
    'vocalized': u'فَالْلَتَيْنِ',
    'stem': u'الْلَتَيْنِ',
    'type': u'STOPWORD',
    'original': u'الْلَتَيْنِ',
    'encletic': u''
}
STOPWORDS[u'وباللتين'] = {
    'word': u'وباللتين',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم موصول:مجرور:معطوف',
    'vocalized': u'وَبِالْلَتَيْنِ',
    'stem': u'الْلَتَيْنِ',
    'type': u'STOPWORD',
    'original': u'الْلَتَيْنِ',
    'encletic': u''
}
STOPWORDS[u'فباللتين'] = {
    'word': u'فباللتين',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم موصول:مجرور:معطوف',
    'vocalized': u'فَبِالْلَتَيْنِ',
    'stem': u'الْلَتَيْنِ',
    'type': u'STOPWORD',
    'original': u'الْلَتَيْنِ',
    'encletic': u''
}
STOPWORDS[u'وكاللتين'] = {
    'word': u'وكاللتين',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم موصول:مجرور:معطوف',
    'vocalized': u'وَكَالْلَتَيْنِ',
    'stem': u'الْلَتَيْنِ',
    'type': u'STOPWORD',
    'original': u'الْلَتَيْنِ',
    'encletic': u''
}
STOPWORDS[u'فكاللتين'] = {
    'word': u'فكاللتين',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم موصول:مجرور:معطوف',
    'vocalized': u'فَكَالْلَتَيْنِ',
    'stem': u'الْلَتَيْنِ',
    'type': u'STOPWORD',
    'original': u'الْلَتَيْنِ',
    'encletic': u''
}
STOPWORDS[u'وللتين'] = {
    'word': u'وللتين',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم موصول:مجرور:معطوف',
    'vocalized': u'وَلِالْلَتَيْنِ',
    'stem': u'الْلَتَيْنِ',
    'type': u'STOPWORD',
    'original': u'الْلَتَيْنِ',
    'encletic': u''
}
STOPWORDS[u'فللتين'] = {
    'word': u'فللتين',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم موصول:مجرور:معطوف',
    'vocalized': u'فَلِالْلَتَيْنِ',
    'stem': u'الْلَتَيْنِ',
    'type': u'STOPWORD',
    'original': u'الْلَتَيْنِ',
    'encletic': u''
}
STOPWORDS[u'اللذان'] = {
    'word': u'الْلَذَانِ',
    'procletic': u'',
    'tags': u'أداة:اسم موصول',
    'vocalized': u'الْلَذَانِ',
    'stem': u'الْلَذَانِ',
    'type': u'STOPWORD',
    'original': u'الْلَذَانِ',
    'encletic': u''
}
STOPWORDS[u'واللذان'] = {
    'word': u'واللذان',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم موصول:معطوف',
    'vocalized': u'وَالْلَذَانِ',
    'stem': u'الْلَذَانِ',
    'type': u'STOPWORD',
    'original': u'الْلَذَانِ',
    'encletic': u''
}
STOPWORDS[u'فاللذان'] = {
    'word': u'فاللذان',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم موصول:معطوف',
    'vocalized': u'فَالْلَذَانِ',
    'stem': u'الْلَذَانِ',
    'type': u'STOPWORD',
    'original': u'الْلَذَانِ',
    'encletic': u''
}
STOPWORDS[u'اللذين'] = {
    'word': u'الْلَذَيْنِ',
    'procletic': u'',
    'tags': u'أداة:اسم موصول',
    'vocalized': u'الْلَذَيْنِ',
    'stem': u'الْلَذَيْنِ',
    'type': u'STOPWORD',
    'original': u'الْلَذَيْنِ',
    'encletic': u''
}
STOPWORDS[u'باللذين'] = {
    'word': u'باللذين',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم موصول:مجرور',
    'vocalized': u'بِالْلَذَيْنِ',
    'stem': u'الْلَذَيْنِ',
    'type': u'STOPWORD',
    'original': u'الْلَذَيْنِ',
    'encletic': u''
}
STOPWORDS[u'كاللذين'] = {
    'word': u'كاللذين',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم موصول:مجرور',
    'vocalized': u'كَالْلَذَيْنِ',
    'stem': u'الْلَذَيْنِ',
    'type': u'STOPWORD',
    'original': u'الْلَذَيْنِ',
    'encletic': u''
}
STOPWORDS[u'للذين'] = {
    'word': u'للذين',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم موصول:مجرور',
    'vocalized': u'لِالْلَذَيْنِ',
    'stem': u'الْلَذَيْنِ',
    'type': u'STOPWORD',
    'original': u'الْلَذَيْنِ',
    'encletic': u''
}
STOPWORDS[u'واللذين'] = {
    'word': u'واللذين',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم موصول:معطوف',
    'vocalized': u'وَالْلَذَيْنِ',
    'stem': u'الْلَذَيْنِ',
    'type': u'STOPWORD',
    'original': u'الْلَذَيْنِ',
    'encletic': u''
}
STOPWORDS[u'فاللذين'] = {
    'word': u'فاللذين',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم موصول:معطوف',
    'vocalized': u'فَالْلَذَيْنِ',
    'stem': u'الْلَذَيْنِ',
    'type': u'STOPWORD',
    'original': u'الْلَذَيْنِ',
    'encletic': u''
}
STOPWORDS[u'وباللذين'] = {
    'word': u'وباللذين',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم موصول:مجرور:معطوف',
    'vocalized': u'وَبِالْلَذَيْنِ',
    'stem': u'الْلَذَيْنِ',
    'type': u'STOPWORD',
    'original': u'الْلَذَيْنِ',
    'encletic': u''
}
STOPWORDS[u'فباللذين'] = {
    'word': u'فباللذين',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم موصول:مجرور:معطوف',
    'vocalized': u'فَبِالْلَذَيْنِ',
    'stem': u'الْلَذَيْنِ',
    'type': u'STOPWORD',
    'original': u'الْلَذَيْنِ',
    'encletic': u''
}
STOPWORDS[u'وكاللذين'] = {
    'word': u'وكاللذين',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم موصول:مجرور:معطوف',
    'vocalized': u'وَكَالْلَذَيْنِ',
    'stem': u'الْلَذَيْنِ',
    'type': u'STOPWORD',
    'original': u'الْلَذَيْنِ',
    'encletic': u''
}
STOPWORDS[u'فكاللذين'] = {
    'word': u'فكاللذين',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم موصول:مجرور:معطوف',
    'vocalized': u'فَكَالْلَذَيْنِ',
    'stem': u'الْلَذَيْنِ',
    'type': u'STOPWORD',
    'original': u'الْلَذَيْنِ',
    'encletic': u''
}
STOPWORDS[u'وللذين'] = {
    'word': u'وللذين',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم موصول:مجرور:معطوف',
    'vocalized': u'وَلِالْلَذَيْنِ',
    'stem': u'الْلَذَيْنِ',
    'type': u'STOPWORD',
    'original': u'الْلَذَيْنِ',
    'encletic': u''
}
STOPWORDS[u'فللذين'] = {
    'word': u'فللذين',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم موصول:مجرور:معطوف',
    'vocalized': u'فَلِالْلَذَيْنِ',
    'stem': u'الْلَذَيْنِ',
    'type': u'STOPWORD',
    'original': u'الْلَذَيْنِ',
    'encletic': u''
}
STOPWORDS[u'اللواتي'] = {
    'word': u'الْلَوَاتِي',
    'procletic': u'',
    'tags': u'أداة:اسم موصول',
    'vocalized': u'الْلَوَاتِي',
    'stem': u'الْلَوَاتِي',
    'type': u'STOPWORD',
    'original': u'الْلَوَاتِي',
    'encletic': u''
}
STOPWORDS[u'باللواتي'] = {
    'word': u'باللواتي',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم موصول:مجرور',
    'vocalized': u'بِالْلَوَاتِي',
    'stem': u'الْلَوَاتِي',
    'type': u'STOPWORD',
    'original': u'الْلَوَاتِي',
    'encletic': u''
}
STOPWORDS[u'كاللواتي'] = {
    'word': u'كاللواتي',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم موصول:مجرور',
    'vocalized': u'كَالْلَوَاتِي',
    'stem': u'الْلَوَاتِي',
    'type': u'STOPWORD',
    'original': u'الْلَوَاتِي',
    'encletic': u''
}
STOPWORDS[u'للواتي'] = {
    'word': u'للواتي',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم موصول:مجرور',
    'vocalized': u'لِالْلَوَاتِي',
    'stem': u'الْلَوَاتِي',
    'type': u'STOPWORD',
    'original': u'الْلَوَاتِي',
    'encletic': u''
}
STOPWORDS[u'واللواتي'] = {
    'word': u'واللواتي',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم موصول:معطوف',
    'vocalized': u'وَالْلَوَاتِي',
    'stem': u'الْلَوَاتِي',
    'type': u'STOPWORD',
    'original': u'الْلَوَاتِي',
    'encletic': u''
}
STOPWORDS[u'فاللواتي'] = {
    'word': u'فاللواتي',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم موصول:معطوف',
    'vocalized': u'فَالْلَوَاتِي',
    'stem': u'الْلَوَاتِي',
    'type': u'STOPWORD',
    'original': u'الْلَوَاتِي',
    'encletic': u''
}
STOPWORDS[u'وباللواتي'] = {
    'word': u'وباللواتي',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم موصول:مجرور:معطوف',
    'vocalized': u'وَبِالْلَوَاتِي',
    'stem': u'الْلَوَاتِي',
    'type': u'STOPWORD',
    'original': u'الْلَوَاتِي',
    'encletic': u''
}
STOPWORDS[u'فباللواتي'] = {
    'word': u'فباللواتي',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم موصول:مجرور:معطوف',
    'vocalized': u'فَبِالْلَوَاتِي',
    'stem': u'الْلَوَاتِي',
    'type': u'STOPWORD',
    'original': u'الْلَوَاتِي',
    'encletic': u''
}
STOPWORDS[u'وكاللواتي'] = {
    'word': u'وكاللواتي',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم موصول:مجرور:معطوف',
    'vocalized': u'وَكَالْلَوَاتِي',
    'stem': u'الْلَوَاتِي',
    'type': u'STOPWORD',
    'original': u'الْلَوَاتِي',
    'encletic': u''
}
STOPWORDS[u'فكاللواتي'] = {
    'word': u'فكاللواتي',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم موصول:مجرور:معطوف',
    'vocalized': u'فَكَالْلَوَاتِي',
    'stem': u'الْلَوَاتِي',
    'type': u'STOPWORD',
    'original': u'الْلَوَاتِي',
    'encletic': u''
}
STOPWORDS[u'وللواتي'] = {
    'word': u'وللواتي',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم موصول:مجرور:معطوف',
    'vocalized': u'وَلِالْلَوَاتِي',
    'stem': u'الْلَوَاتِي',
    'type': u'STOPWORD',
    'original': u'الْلَوَاتِي',
    'encletic': u''
}
STOPWORDS[u'فللواتي'] = {
    'word': u'فللواتي',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم موصول:مجرور:معطوف',
    'vocalized': u'فَلِالْلَوَاتِي',
    'stem': u'الْلَوَاتِي',
    'type': u'STOPWORD',
    'original': u'الْلَوَاتِي',
    'encletic': u''
}
STOPWORDS[u'أي'] = {
    'word': u'أََيُّ',
    'procletic': u'',
    'tags': u'أداة:اسم موصول',
    'vocalized': u'أََيُّ',
    'stem': u'أََيُّ',
    'type': u'STOPWORD',
    'original': u'أََيُّ',
    'encletic': u''
}
STOPWORDS[u'بأي'] = {
    'word': u'بأي',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم موصول:مجرور',
    'vocalized': u'بِأََيُّ',
    'stem': u'أََيُّ',
    'type': u'STOPWORD',
    'original': u'أََيُّ',
    'encletic': u''
}
STOPWORDS[u'كأي'] = {
    'word': u'كأي',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم موصول:مجرور',
    'vocalized': u'كَأََيُّ',
    'stem': u'أََيُّ',
    'type': u'STOPWORD',
    'original': u'أََيُّ',
    'encletic': u''
}
STOPWORDS[u'لأي'] = {
    'word': u'لأي',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم موصول:مجرور',
    'vocalized': u'لِأََيُّ',
    'stem': u'أََيُّ',
    'type': u'STOPWORD',
    'original': u'أََيُّ',
    'encletic': u''
}
STOPWORDS[u'وأي'] = {
    'word': u'وأي',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم موصول:معطوف',
    'vocalized': u'وَأََيُّ',
    'stem': u'أََيُّ',
    'type': u'STOPWORD',
    'original': u'أََيُّ',
    'encletic': u''
}
STOPWORDS[u'فأي'] = {
    'word': u'فأي',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم موصول:معطوف',
    'vocalized': u'فَأََيُّ',
    'stem': u'أََيُّ',
    'type': u'STOPWORD',
    'original': u'أََيُّ',
    'encletic': u''
}
STOPWORDS[u'وبأي'] = {
    'word': u'وبأي',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم موصول:مجرور:معطوف',
    'vocalized': u'وَبِأََيُّ',
    'stem': u'أََيُّ',
    'type': u'STOPWORD',
    'original': u'أََيُّ',
    'encletic': u''
}
STOPWORDS[u'فبأي'] = {
    'word': u'فبأي',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم موصول:مجرور:معطوف',
    'vocalized': u'فَبِأََيُّ',
    'stem': u'أََيُّ',
    'type': u'STOPWORD',
    'original': u'أََيُّ',
    'encletic': u''
}
STOPWORDS[u'وكأي'] = {
    'word': u'وكأي',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم موصول:مجرور:معطوف',
    'vocalized': u'وَكَأََيُّ',
    'stem': u'أََيُّ',
    'type': u'STOPWORD',
    'original': u'أََيُّ',
    'encletic': u''
}
STOPWORDS[u'فكأي'] = {
    'word': u'فكأي',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم موصول:مجرور:معطوف',
    'vocalized': u'فَكَأََيُّ',
    'stem': u'أََيُّ',
    'type': u'STOPWORD',
    'original': u'أََيُّ',
    'encletic': u''
}
STOPWORDS[u'ولأي'] = {
    'word': u'ولأي',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم موصول:مجرور:معطوف',
    'vocalized': u'وَلِأََيُّ',
    'stem': u'أََيُّ',
    'type': u'STOPWORD',
    'original': u'أََيُّ',
    'encletic': u''
}
STOPWORDS[u'فلأي'] = {
    'word': u'فلأي',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم موصول:مجرور:معطوف',
    'vocalized': u'فَلِأََيُّ',
    'stem': u'أََيُّ',
    'type': u'STOPWORD',
    'original': u'أََيُّ',
    'encletic': u''
}
STOPWORDS[u'ذا'] = {
    'word': u'ذَا',
    'procletic': u'',
    'tags': u'أداة:اسم موصول',
    'vocalized': u'ذَا',
    'stem': u'ذَا',
    'type': u'STOPWORD',
    'original': u'ذَا',
    'encletic': u''
}
STOPWORDS[u'بذا'] = {
    'word': u'بذا',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم موصول:مجرور',
    'vocalized': u'بِذَا',
    'stem': u'ذَا',
    'type': u'STOPWORD',
    'original': u'ذَا',
    'encletic': u''
}
STOPWORDS[u'كذا'] = {
    'word': u'كذا',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم موصول:مجرور',
    'vocalized': u'كَذَا',
    'stem': u'ذَا',
    'type': u'STOPWORD',
    'original': u'ذَا',
    'encletic': u''
}
STOPWORDS[u'لذا'] = {
    'word': u'لذا',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم موصول:مجرور',
    'vocalized': u'لِذَا',
    'stem': u'ذَا',
    'type': u'STOPWORD',
    'original': u'ذَا',
    'encletic': u''
}
STOPWORDS[u'وذا'] = {
    'word': u'وذا',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم موصول:معطوف',
    'vocalized': u'وَذَا',
    'stem': u'ذَا',
    'type': u'STOPWORD',
    'original': u'ذَا',
    'encletic': u''
}
STOPWORDS[u'فذا'] = {
    'word': u'فذا',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم موصول:معطوف',
    'vocalized': u'فَذَا',
    'stem': u'ذَا',
    'type': u'STOPWORD',
    'original': u'ذَا',
    'encletic': u''
}
STOPWORDS[u'وبذا'] = {
    'word': u'وبذا',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم موصول:مجرور:معطوف',
    'vocalized': u'وَبِذَا',
    'stem': u'ذَا',
    'type': u'STOPWORD',
    'original': u'ذَا',
    'encletic': u''
}
STOPWORDS[u'فبذا'] = {
    'word': u'فبذا',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم موصول:مجرور:معطوف',
    'vocalized': u'فَبِذَا',
    'stem': u'ذَا',
    'type': u'STOPWORD',
    'original': u'ذَا',
    'encletic': u''
}
STOPWORDS[u'وكذا'] = {
    'word': u'وكذا',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم موصول:مجرور:معطوف',
    'vocalized': u'وَكَذَا',
    'stem': u'ذَا',
    'type': u'STOPWORD',
    'original': u'ذَا',
    'encletic': u''
}
STOPWORDS[u'فكذا'] = {
    'word': u'فكذا',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم موصول:مجرور:معطوف',
    'vocalized': u'فَكَذَا',
    'stem': u'ذَا',
    'type': u'STOPWORD',
    'original': u'ذَا',
    'encletic': u''
}
STOPWORDS[u'ولذا'] = {
    'word': u'ولذا',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم موصول:مجرور:معطوف',
    'vocalized': u'وَلِذَا',
    'stem': u'ذَا',
    'type': u'STOPWORD',
    'original': u'ذَا',
    'encletic': u''
}
STOPWORDS[u'فلذا'] = {
    'word': u'فلذا',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم موصول:مجرور:معطوف',
    'vocalized': u'فَلِذَا',
    'stem': u'ذَا',
    'type': u'STOPWORD',
    'original': u'ذَا',
    'encletic': u''
}
STOPWORDS[u'ذات'] = {
    'word': u'ذَاتُ',
    'procletic': u'',
    'tags': u'أداة:اسم موصول',
    'vocalized': u'ذَاتُ',
    'stem': u'ذَاتُ',
    'type': u'STOPWORD',
    'original': u'ذَاتُ',
    'encletic': u''
}
STOPWORDS[u'بذات'] = {
    'word': u'بذات',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم موصول:مجرور',
    'vocalized': u'بِذَاتُ',
    'stem': u'ذَاتُ',
    'type': u'STOPWORD',
    'original': u'ذَاتُ',
    'encletic': u''
}
STOPWORDS[u'كذات'] = {
    'word': u'كذات',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم موصول:مجرور',
    'vocalized': u'كَذَاتُ',
    'stem': u'ذَاتُ',
    'type': u'STOPWORD',
    'original': u'ذَاتُ',
    'encletic': u''
}
STOPWORDS[u'لذات'] = {
    'word': u'لذات',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم موصول:مجرور',
    'vocalized': u'لِذَاتُ',
    'stem': u'ذَاتُ',
    'type': u'STOPWORD',
    'original': u'ذَاتُ',
    'encletic': u''
}
STOPWORDS[u'وذات'] = {
    'word': u'وذات',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم موصول:معطوف',
    'vocalized': u'وَذَاتُ',
    'stem': u'ذَاتُ',
    'type': u'STOPWORD',
    'original': u'ذَاتُ',
    'encletic': u''
}
STOPWORDS[u'فذات'] = {
    'word': u'فذات',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم موصول:معطوف',
    'vocalized': u'فَذَاتُ',
    'stem': u'ذَاتُ',
    'type': u'STOPWORD',
    'original': u'ذَاتُ',
    'encletic': u''
}
STOPWORDS[u'وبذات'] = {
    'word': u'وبذات',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم موصول:مجرور:معطوف',
    'vocalized': u'وَبِذَاتُ',
    'stem': u'ذَاتُ',
    'type': u'STOPWORD',
    'original': u'ذَاتُ',
    'encletic': u''
}
STOPWORDS[u'فبذات'] = {
    'word': u'فبذات',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم موصول:مجرور:معطوف',
    'vocalized': u'فَبِذَاتُ',
    'stem': u'ذَاتُ',
    'type': u'STOPWORD',
    'original': u'ذَاتُ',
    'encletic': u''
}
STOPWORDS[u'وكذات'] = {
    'word': u'وكذات',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم موصول:مجرور:معطوف',
    'vocalized': u'وَكَذَاتُ',
    'stem': u'ذَاتُ',
    'type': u'STOPWORD',
    'original': u'ذَاتُ',
    'encletic': u''
}
STOPWORDS[u'فكذات'] = {
    'word': u'فكذات',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم موصول:مجرور:معطوف',
    'vocalized': u'فَكَذَاتُ',
    'stem': u'ذَاتُ',
    'type': u'STOPWORD',
    'original': u'ذَاتُ',
    'encletic': u''
}
STOPWORDS[u'ولذات'] = {
    'word': u'ولذات',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم موصول:مجرور:معطوف',
    'vocalized': u'وَلِذَاتُ',
    'stem': u'ذَاتُ',
    'type': u'STOPWORD',
    'original': u'ذَاتُ',
    'encletic': u''
}
STOPWORDS[u'فلذات'] = {
    'word': u'فلذات',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم موصول:مجرور:معطوف',
    'vocalized': u'فَلِذَاتُ',
    'stem': u'ذَاتُ',
    'type': u'STOPWORD',
    'original': u'ذَاتُ',
    'encletic': u''
}
STOPWORDS[u'ما'] = {
    'word': u'مَا',
    'procletic': u'',
    'tags': u'أداة:اسم موصول',
    'vocalized': u'مَا',
    'stem': u'مَا',
    'type': u'STOPWORD',
    'original': u'مَا',
    'encletic': u''
}
STOPWORDS[u'وما'] = {
    'word': u'وما',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم موصول:معطوف',
    'vocalized': u'وَمَا',
    'stem': u'مَا',
    'type': u'STOPWORD',
    'original': u'مَا',
    'encletic': u''
}
STOPWORDS[u'فما'] = {
    'word': u'فما',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم موصول:معطوف',
    'vocalized': u'فَمَا',
    'stem': u'مَا',
    'type': u'STOPWORD',
    'original': u'مَا',
    'encletic': u''
}
STOPWORDS[u'من'] = {
    'word': u'مَنْ',
    'procletic': u'',
    'tags': u'أداة:اسم موصول',
    'vocalized': u'مَنْ',
    'stem': u'مَنْ',
    'type': u'STOPWORD',
    'original': u'مَنْ',
    'encletic': u''
}
STOPWORDS[u'بمن'] = {
    'word': u'بمن',
    'procletic': u'-بِ',
    'tags': u'أداة:اسم موصول:مجرور',
    'vocalized': u'بِمَنْ',
    'stem': u'مَنْ',
    'type': u'STOPWORD',
    'original': u'مَنْ',
    'encletic': u''
}
STOPWORDS[u'كمن'] = {
    'word': u'كمن',
    'procletic': u'-كَ',
    'tags': u'أداة:اسم موصول:مجرور',
    'vocalized': u'كَمَنْ',
    'stem': u'مَنْ',
    'type': u'STOPWORD',
    'original': u'مَنْ',
    'encletic': u''
}
STOPWORDS[u'لمن'] = {
    'word': u'لمن',
    'procletic': u'-لِ',
    'tags': u'أداة:اسم موصول:مجرور',
    'vocalized': u'لِمَنْ',
    'stem': u'مَنْ',
    'type': u'STOPWORD',
    'original': u'مَنْ',
    'encletic': u''
}
STOPWORDS[u'ومن'] = {
    'word': u'ومن',
    'procletic': u'-وَ',
    'tags': u'أداة:اسم موصول:معطوف',
    'vocalized': u'وَمَنْ',
    'stem': u'مَنْ',
    'type': u'STOPWORD',
    'original': u'مَنْ',
    'encletic': u''
}
STOPWORDS[u'فمن'] = {
    'word': u'فمن',
    'procletic': u'-فَ',
    'tags': u'أداة:اسم موصول:معطوف',
    'vocalized': u'فَمَنْ',
    'stem': u'مَنْ',
    'type': u'STOPWORD',
    'original': u'مَنْ',
    'encletic': u''
}
STOPWORDS[u'وبمن'] = {
    'word': u'وبمن',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:اسم موصول:مجرور:معطوف',
    'vocalized': u'وَبِمَنْ',
    'stem': u'مَنْ',
    'type': u'STOPWORD',
    'original': u'مَنْ',
    'encletic': u''
}
STOPWORDS[u'فبمن'] = {
    'word': u'فبمن',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:اسم موصول:مجرور:معطوف',
    'vocalized': u'فَبِمَنْ',
    'stem': u'مَنْ',
    'type': u'STOPWORD',
    'original': u'مَنْ',
    'encletic': u''
}
STOPWORDS[u'وكمن'] = {
    'word': u'وكمن',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:اسم موصول:مجرور:معطوف',
    'vocalized': u'وَكَمَنْ',
    'stem': u'مَنْ',
    'type': u'STOPWORD',
    'original': u'مَنْ',
    'encletic': u''
}
STOPWORDS[u'فكمن'] = {
    'word': u'فكمن',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:اسم موصول:مجرور:معطوف',
    'vocalized': u'فَكَمَنْ',
    'stem': u'مَنْ',
    'type': u'STOPWORD',
    'original': u'مَنْ',
    'encletic': u''
}
STOPWORDS[u'ولمن'] = {
    'word': u'ولمن',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:اسم موصول:مجرور:معطوف',
    'vocalized': u'وَلِمَنْ',
    'stem': u'مَنْ',
    'type': u'STOPWORD',
    'original': u'مَنْ',
    'encletic': u''
}
STOPWORDS[u'فلمن'] = {
    'word': u'فلمن',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:اسم موصول:مجرور:معطوف',
    'vocalized': u'فَلِمَنْ',
    'stem': u'مَنْ',
    'type': u'STOPWORD',
    'original': u'مَنْ',
    'encletic': u''
}
STOPWORDS[u'أمن'] = {
    'word': u'أمن',
    'procletic': u'-أَ',
    'tags': u'أداة:اسم موصول:مجرور',
    'vocalized': u'أَمَنْ',
    'stem': u'مَنْ',
    'type': u'STOPWORD',
    'original': u'مَنْ',
    'encletic': u''
}
STOPWORDS[u'أبمن'] = {
    'word': u'أبمن',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:اسم موصول:مجرور:مجرور',
    'vocalized': u'أَبِمَنْ',
    'stem': u'مَنْ',
    'type': u'STOPWORD',
    'original': u'مَنْ',
    'encletic': u''
}
STOPWORDS[u'أكمن'] = {
    'word': u'أكمن',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:اسم موصول:مجرور:مجرور',
    'vocalized': u'أَكَمَنْ',
    'stem': u'مَنْ',
    'type': u'STOPWORD',
    'original': u'مَنْ',
    'encletic': u''
}
STOPWORDS[u'ألمن'] = {
    'word': u'ألمن',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:اسم موصول:مجرور:مجرور',
    'vocalized': u'أَلِمَنْ',
    'stem': u'مَنْ',
    'type': u'STOPWORD',
    'original': u'مَنْ',
    'encletic': u''
}
STOPWORDS[u'أومن'] = {
    'word': u'أومن',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:اسم موصول:معطوف:مجرور',
    'vocalized': u'أَوَمَنْ',
    'stem': u'مَنْ',
    'type': u'STOPWORD',
    'original': u'مَنْ',
    'encletic': u''
}
STOPWORDS[u'أفمن'] = {
    'word': u'أفمن',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:اسم موصول:معطوف:مجرور',
    'vocalized': u'أَفَمَنْ',
    'stem': u'مَنْ',
    'type': u'STOPWORD',
    'original': u'مَنْ',
    'encletic': u''
}
STOPWORDS[u'أوبمن'] = {
    'word': u'أوبمن',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:اسم موصول:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَبِمَنْ',
    'stem': u'مَنْ',
    'type': u'STOPWORD',
    'original': u'مَنْ',
    'encletic': u''
}
STOPWORDS[u'أفبمن'] = {
    'word': u'أفبمن',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:اسم موصول:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَبِمَنْ',
    'stem': u'مَنْ',
    'type': u'STOPWORD',
    'original': u'مَنْ',
    'encletic': u''
}
STOPWORDS[u'أوكمن'] = {
    'word': u'أوكمن',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:اسم موصول:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَكَمَنْ',
    'stem': u'مَنْ',
    'type': u'STOPWORD',
    'original': u'مَنْ',
    'encletic': u''
}
STOPWORDS[u'أفكمن'] = {
    'word': u'أفكمن',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:اسم موصول:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَكَمَنْ',
    'stem': u'مَنْ',
    'type': u'STOPWORD',
    'original': u'مَنْ',
    'encletic': u''
}
STOPWORDS[u'أولمن'] = {
    'word': u'أولمن',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:اسم موصول:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَلِمَنْ',
    'stem': u'مَنْ',
    'type': u'STOPWORD',
    'original': u'مَنْ',
    'encletic': u''
}
STOPWORDS[u'أفلمن'] = {
    'word': u'أفلمن',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:اسم موصول:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَلِمَنْ',
    'stem': u'مَنْ',
    'type': u'STOPWORD',
    'original': u'مَنْ',
    'encletic': u''
}
STOPWORDS[u'أب'] = {
    'word': u'أَبُ',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة',
    'vocalized': u'أَبُ',
    'stem': u'أَبُ',
    'type': u'STOPWORD',
    'original': u'أَبُ',
    'encletic': u''
}
STOPWORDS[u'وأب'] = {
    'word': u'وأب',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف',
    'vocalized': u'وَأَبُ',
    'stem': u'أَبُ',
    'type': u'STOPWORD',
    'original': u'أَبُ',
    'encletic': u''
}
STOPWORDS[u'فأب'] = {
    'word': u'فأب',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف',
    'vocalized': u'فَأَبُ',
    'stem': u'أَبُ',
    'type': u'STOPWORD',
    'original': u'أَبُ',
    'encletic': u''
}
STOPWORDS[u'أخ'] = {
    'word': u'أَخُ',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة',
    'vocalized': u'أَخُ',
    'stem': u'أَخُ',
    'type': u'STOPWORD',
    'original': u'أَخُ',
    'encletic': u''
}
STOPWORDS[u'وأخ'] = {
    'word': u'وأخ',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف',
    'vocalized': u'وَأَخُ',
    'stem': u'أَخُ',
    'type': u'STOPWORD',
    'original': u'أَخُ',
    'encletic': u''
}
STOPWORDS[u'فأخ'] = {
    'word': u'فأخ',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف',
    'vocalized': u'فَأَخُ',
    'stem': u'أَخُ',
    'type': u'STOPWORD',
    'original': u'أَخُ',
    'encletic': u''
}
STOPWORDS[u'حم'] = {
    'word': u'حَمُ',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة',
    'vocalized': u'حَمُ',
    'stem': u'حَمُ',
    'type': u'STOPWORD',
    'original': u'حَمُ',
    'encletic': u''
}
STOPWORDS[u'وحم'] = {
    'word': u'وحم',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف',
    'vocalized': u'وَحَمُ',
    'stem': u'حَمُ',
    'type': u'STOPWORD',
    'original': u'حَمُ',
    'encletic': u''
}
STOPWORDS[u'فحم'] = {
    'word': u'فحم',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف',
    'vocalized': u'فَحَمُ',
    'stem': u'حَمُ',
    'type': u'STOPWORD',
    'original': u'حَمُ',
    'encletic': u''
}
STOPWORDS[u'ذو'] = {
    'word': u'ذُو',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة',
    'vocalized': u'ذُو',
    'stem': u'ذُو',
    'type': u'STOPWORD',
    'original': u'ذُو',
    'encletic': u''
}
STOPWORDS[u'وذو'] = {
    'word': u'وذو',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف',
    'vocalized': u'وَذُو',
    'stem': u'ذُو',
    'type': u'STOPWORD',
    'original': u'ذُو',
    'encletic': u''
}
STOPWORDS[u'فذو'] = {
    'word': u'فذو',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف',
    'vocalized': u'فَذُو',
    'stem': u'ذُو',
    'type': u'STOPWORD',
    'original': u'ذُو',
    'encletic': u''
}
STOPWORDS[u'فو'] = {
    'word': u'فُو',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة',
    'vocalized': u'فُو',
    'stem': u'فُو',
    'type': u'STOPWORD',
    'original': u'فُو',
    'encletic': u''
}
STOPWORDS[u'وفو'] = {
    'word': u'وفو',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف',
    'vocalized': u'وَفُو',
    'stem': u'فُو',
    'type': u'STOPWORD',
    'original': u'فُو',
    'encletic': u''
}
STOPWORDS[u'ففو'] = {
    'word': u'ففو',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف',
    'vocalized': u'فَفُو',
    'stem': u'فُو',
    'type': u'STOPWORD',
    'original': u'فُو',
    'encletic': u''
}
STOPWORDS[u'أبو'] = {
    'word': u'أَبُو',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة',
    'vocalized': u'أَبُو',
    'stem': u'أَبُو',
    'type': u'STOPWORD',
    'original': u'أَبُو',
    'encletic': u''
}
STOPWORDS[u'وأبو'] = {
    'word': u'وأبو',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف',
    'vocalized': u'وَأَبُو',
    'stem': u'أَبُو',
    'type': u'STOPWORD',
    'original': u'أَبُو',
    'encletic': u''
}
STOPWORDS[u'فأبو'] = {
    'word': u'فأبو',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف',
    'vocalized': u'فَأَبُو',
    'stem': u'أَبُو',
    'type': u'STOPWORD',
    'original': u'أَبُو',
    'encletic': u''
}
STOPWORDS[u'أبوي'] = {
    'word': u'أبوي',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'أَبُوِي',
    'stem': u'أَبُو',
    'type': u'STOPWORD',
    'original': u'أَبُو',
    'encletic': u'ِي'
}
STOPWORDS[u'أبوك'] = {
    'word': u'أبوك',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'أَبُوكَ',
    'stem': u'أَبُو',
    'type': u'STOPWORD',
    'original': u'أَبُو',
    'encletic': u'كَ'
}
STOPWORDS[u'أبوك'] = {
    'word': u'أبوك',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'أَبُوكِ',
    'stem': u'أَبُو',
    'type': u'STOPWORD',
    'original': u'أَبُو',
    'encletic': u'كِ'
}
STOPWORDS[u'أبوه'] = {
    'word': u'أبوه',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'أَبُوه',
    'stem': u'أَبُو',
    'type': u'STOPWORD',
    'original': u'أَبُو',
    'encletic': u'ه'
}
STOPWORDS[u'أبوكم'] = {
    'word': u'أبوكم',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'أَبُوكُمْ',
    'stem': u'أَبُو',
    'type': u'STOPWORD',
    'original': u'أَبُو',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أبوكن'] = {
    'word': u'أبوكن',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'أَبُوكُنَّ',
    'stem': u'أَبُو',
    'type': u'STOPWORD',
    'original': u'أَبُو',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أبوها'] = {
    'word': u'أبوها',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'أَبُوهَا',
    'stem': u'أَبُو',
    'type': u'STOPWORD',
    'original': u'أَبُو',
    'encletic': u'هَا'
}
STOPWORDS[u'أبوهم'] = {
    'word': u'أبوهم',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'أَبُوهُمْ',
    'stem': u'أَبُو',
    'type': u'STOPWORD',
    'original': u'أَبُو',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أبوهن'] = {
    'word': u'أبوهن',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'أَبُوهُنَّ',
    'stem': u'أَبُو',
    'type': u'STOPWORD',
    'original': u'أَبُو',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أبونا'] = {
    'word': u'أبونا',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'أَبُونَا',
    'stem': u'أَبُو',
    'type': u'STOPWORD',
    'original': u'أَبُو',
    'encletic': u'نَا'
}
STOPWORDS[u'أبوكما'] = {
    'word': u'أبوكما',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'أَبُوكُمَا',
    'stem': u'أَبُو',
    'type': u'STOPWORD',
    'original': u'أَبُو',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أبوهما'] = {
    'word': u'أبوهما',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'أَبُوهُمَا',
    'stem': u'أَبُو',
    'type': u'STOPWORD',
    'original': u'أَبُو',
    'encletic': u'هُمَا'
}
STOPWORDS[u'وأبوي'] = {
    'word': u'وأبوي',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَأَبُوِي',
    'stem': u'أَبُو',
    'type': u'STOPWORD',
    'original': u'أَبُو',
    'encletic': u'ِي'
}
STOPWORDS[u'وأبوك'] = {
    'word': u'وأبوك',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَأَبُوكَ',
    'stem': u'أَبُو',
    'type': u'STOPWORD',
    'original': u'أَبُو',
    'encletic': u'كَ'
}
STOPWORDS[u'وأبوك'] = {
    'word': u'وأبوك',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَأَبُوكِ',
    'stem': u'أَبُو',
    'type': u'STOPWORD',
    'original': u'أَبُو',
    'encletic': u'كِ'
}
STOPWORDS[u'وأبوه'] = {
    'word': u'وأبوه',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَأَبُوه',
    'stem': u'أَبُو',
    'type': u'STOPWORD',
    'original': u'أَبُو',
    'encletic': u'ه'
}
STOPWORDS[u'وأبوكم'] = {
    'word': u'وأبوكم',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَأَبُوكُمْ',
    'stem': u'أَبُو',
    'type': u'STOPWORD',
    'original': u'أَبُو',
    'encletic': u'كُمْ'
}
STOPWORDS[u'وأبوكن'] = {
    'word': u'وأبوكن',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَأَبُوكُنَّ',
    'stem': u'أَبُو',
    'type': u'STOPWORD',
    'original': u'أَبُو',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'وأبوها'] = {
    'word': u'وأبوها',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَأَبُوهَا',
    'stem': u'أَبُو',
    'type': u'STOPWORD',
    'original': u'أَبُو',
    'encletic': u'هَا'
}
STOPWORDS[u'وأبوهم'] = {
    'word': u'وأبوهم',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَأَبُوهُمْ',
    'stem': u'أَبُو',
    'type': u'STOPWORD',
    'original': u'أَبُو',
    'encletic': u'هُمْ'
}
STOPWORDS[u'وأبوهن'] = {
    'word': u'وأبوهن',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَأَبُوهُنَّ',
    'stem': u'أَبُو',
    'type': u'STOPWORD',
    'original': u'أَبُو',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'وأبونا'] = {
    'word': u'وأبونا',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَأَبُونَا',
    'stem': u'أَبُو',
    'type': u'STOPWORD',
    'original': u'أَبُو',
    'encletic': u'نَا'
}
STOPWORDS[u'وأبوكما'] = {
    'word': u'وأبوكما',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَأَبُوكُمَا',
    'stem': u'أَبُو',
    'type': u'STOPWORD',
    'original': u'أَبُو',
    'encletic': u'كُمَا'
}
STOPWORDS[u'وأبوهما'] = {
    'word': u'وأبوهما',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَأَبُوهُمَا',
    'stem': u'أَبُو',
    'type': u'STOPWORD',
    'original': u'أَبُو',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فأبوي'] = {
    'word': u'فأبوي',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَأَبُوِي',
    'stem': u'أَبُو',
    'type': u'STOPWORD',
    'original': u'أَبُو',
    'encletic': u'ِي'
}
STOPWORDS[u'فأبوك'] = {
    'word': u'فأبوك',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَأَبُوكَ',
    'stem': u'أَبُو',
    'type': u'STOPWORD',
    'original': u'أَبُو',
    'encletic': u'كَ'
}
STOPWORDS[u'فأبوك'] = {
    'word': u'فأبوك',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَأَبُوكِ',
    'stem': u'أَبُو',
    'type': u'STOPWORD',
    'original': u'أَبُو',
    'encletic': u'كِ'
}
STOPWORDS[u'فأبوه'] = {
    'word': u'فأبوه',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَأَبُوه',
    'stem': u'أَبُو',
    'type': u'STOPWORD',
    'original': u'أَبُو',
    'encletic': u'ه'
}
STOPWORDS[u'فأبوكم'] = {
    'word': u'فأبوكم',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَأَبُوكُمْ',
    'stem': u'أَبُو',
    'type': u'STOPWORD',
    'original': u'أَبُو',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فأبوكن'] = {
    'word': u'فأبوكن',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَأَبُوكُنَّ',
    'stem': u'أَبُو',
    'type': u'STOPWORD',
    'original': u'أَبُو',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فأبوها'] = {
    'word': u'فأبوها',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَأَبُوهَا',
    'stem': u'أَبُو',
    'type': u'STOPWORD',
    'original': u'أَبُو',
    'encletic': u'هَا'
}
STOPWORDS[u'فأبوهم'] = {
    'word': u'فأبوهم',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَأَبُوهُمْ',
    'stem': u'أَبُو',
    'type': u'STOPWORD',
    'original': u'أَبُو',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فأبوهن'] = {
    'word': u'فأبوهن',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَأَبُوهُنَّ',
    'stem': u'أَبُو',
    'type': u'STOPWORD',
    'original': u'أَبُو',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فأبونا'] = {
    'word': u'فأبونا',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَأَبُونَا',
    'stem': u'أَبُو',
    'type': u'STOPWORD',
    'original': u'أَبُو',
    'encletic': u'نَا'
}
STOPWORDS[u'فأبوكما'] = {
    'word': u'فأبوكما',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَأَبُوكُمَا',
    'stem': u'أَبُو',
    'type': u'STOPWORD',
    'original': u'أَبُو',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فأبوهما'] = {
    'word': u'فأبوهما',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَأَبُوهُمَا',
    'stem': u'أَبُو',
    'type': u'STOPWORD',
    'original': u'أَبُو',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أخو'] = {
    'word': u'أَخُو',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة',
    'vocalized': u'أَخُو',
    'stem': u'أَخُو',
    'type': u'STOPWORD',
    'original': u'أَخُو',
    'encletic': u''
}
STOPWORDS[u'وأخو'] = {
    'word': u'وأخو',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف',
    'vocalized': u'وَأَخُو',
    'stem': u'أَخُو',
    'type': u'STOPWORD',
    'original': u'أَخُو',
    'encletic': u''
}
STOPWORDS[u'فأخو'] = {
    'word': u'فأخو',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف',
    'vocalized': u'فَأَخُو',
    'stem': u'أَخُو',
    'type': u'STOPWORD',
    'original': u'أَخُو',
    'encletic': u''
}
STOPWORDS[u'أخوي'] = {
    'word': u'أخوي',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'أَخُوِي',
    'stem': u'أَخُو',
    'type': u'STOPWORD',
    'original': u'أَخُو',
    'encletic': u'ِي'
}
STOPWORDS[u'أخوك'] = {
    'word': u'أخوك',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'أَخُوكَ',
    'stem': u'أَخُو',
    'type': u'STOPWORD',
    'original': u'أَخُو',
    'encletic': u'كَ'
}
STOPWORDS[u'أخوك'] = {
    'word': u'أخوك',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'أَخُوكِ',
    'stem': u'أَخُو',
    'type': u'STOPWORD',
    'original': u'أَخُو',
    'encletic': u'كِ'
}
STOPWORDS[u'أخوه'] = {
    'word': u'أخوه',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'أَخُوه',
    'stem': u'أَخُو',
    'type': u'STOPWORD',
    'original': u'أَخُو',
    'encletic': u'ه'
}
STOPWORDS[u'أخوكم'] = {
    'word': u'أخوكم',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'أَخُوكُمْ',
    'stem': u'أَخُو',
    'type': u'STOPWORD',
    'original': u'أَخُو',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أخوكن'] = {
    'word': u'أخوكن',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'أَخُوكُنَّ',
    'stem': u'أَخُو',
    'type': u'STOPWORD',
    'original': u'أَخُو',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أخوها'] = {
    'word': u'أخوها',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'أَخُوهَا',
    'stem': u'أَخُو',
    'type': u'STOPWORD',
    'original': u'أَخُو',
    'encletic': u'هَا'
}
STOPWORDS[u'أخوهم'] = {
    'word': u'أخوهم',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'أَخُوهُمْ',
    'stem': u'أَخُو',
    'type': u'STOPWORD',
    'original': u'أَخُو',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أخوهن'] = {
    'word': u'أخوهن',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'أَخُوهُنَّ',
    'stem': u'أَخُو',
    'type': u'STOPWORD',
    'original': u'أَخُو',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أخونا'] = {
    'word': u'أخونا',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'أَخُونَا',
    'stem': u'أَخُو',
    'type': u'STOPWORD',
    'original': u'أَخُو',
    'encletic': u'نَا'
}
STOPWORDS[u'أخوكما'] = {
    'word': u'أخوكما',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'أَخُوكُمَا',
    'stem': u'أَخُو',
    'type': u'STOPWORD',
    'original': u'أَخُو',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أخوهما'] = {
    'word': u'أخوهما',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'أَخُوهُمَا',
    'stem': u'أَخُو',
    'type': u'STOPWORD',
    'original': u'أَخُو',
    'encletic': u'هُمَا'
}
STOPWORDS[u'وأخوي'] = {
    'word': u'وأخوي',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَأَخُوِي',
    'stem': u'أَخُو',
    'type': u'STOPWORD',
    'original': u'أَخُو',
    'encletic': u'ِي'
}
STOPWORDS[u'وأخوك'] = {
    'word': u'وأخوك',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَأَخُوكَ',
    'stem': u'أَخُو',
    'type': u'STOPWORD',
    'original': u'أَخُو',
    'encletic': u'كَ'
}
STOPWORDS[u'وأخوك'] = {
    'word': u'وأخوك',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَأَخُوكِ',
    'stem': u'أَخُو',
    'type': u'STOPWORD',
    'original': u'أَخُو',
    'encletic': u'كِ'
}
STOPWORDS[u'وأخوه'] = {
    'word': u'وأخوه',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَأَخُوه',
    'stem': u'أَخُو',
    'type': u'STOPWORD',
    'original': u'أَخُو',
    'encletic': u'ه'
}
STOPWORDS[u'وأخوكم'] = {
    'word': u'وأخوكم',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَأَخُوكُمْ',
    'stem': u'أَخُو',
    'type': u'STOPWORD',
    'original': u'أَخُو',
    'encletic': u'كُمْ'
}
STOPWORDS[u'وأخوكن'] = {
    'word': u'وأخوكن',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَأَخُوكُنَّ',
    'stem': u'أَخُو',
    'type': u'STOPWORD',
    'original': u'أَخُو',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'وأخوها'] = {
    'word': u'وأخوها',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَأَخُوهَا',
    'stem': u'أَخُو',
    'type': u'STOPWORD',
    'original': u'أَخُو',
    'encletic': u'هَا'
}
STOPWORDS[u'وأخوهم'] = {
    'word': u'وأخوهم',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَأَخُوهُمْ',
    'stem': u'أَخُو',
    'type': u'STOPWORD',
    'original': u'أَخُو',
    'encletic': u'هُمْ'
}
STOPWORDS[u'وأخوهن'] = {
    'word': u'وأخوهن',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَأَخُوهُنَّ',
    'stem': u'أَخُو',
    'type': u'STOPWORD',
    'original': u'أَخُو',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'وأخونا'] = {
    'word': u'وأخونا',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَأَخُونَا',
    'stem': u'أَخُو',
    'type': u'STOPWORD',
    'original': u'أَخُو',
    'encletic': u'نَا'
}
STOPWORDS[u'وأخوكما'] = {
    'word': u'وأخوكما',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَأَخُوكُمَا',
    'stem': u'أَخُو',
    'type': u'STOPWORD',
    'original': u'أَخُو',
    'encletic': u'كُمَا'
}
STOPWORDS[u'وأخوهما'] = {
    'word': u'وأخوهما',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَأَخُوهُمَا',
    'stem': u'أَخُو',
    'type': u'STOPWORD',
    'original': u'أَخُو',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فأخوي'] = {
    'word': u'فأخوي',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَأَخُوِي',
    'stem': u'أَخُو',
    'type': u'STOPWORD',
    'original': u'أَخُو',
    'encletic': u'ِي'
}
STOPWORDS[u'فأخوك'] = {
    'word': u'فأخوك',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَأَخُوكَ',
    'stem': u'أَخُو',
    'type': u'STOPWORD',
    'original': u'أَخُو',
    'encletic': u'كَ'
}
STOPWORDS[u'فأخوك'] = {
    'word': u'فأخوك',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَأَخُوكِ',
    'stem': u'أَخُو',
    'type': u'STOPWORD',
    'original': u'أَخُو',
    'encletic': u'كِ'
}
STOPWORDS[u'فأخوه'] = {
    'word': u'فأخوه',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَأَخُوه',
    'stem': u'أَخُو',
    'type': u'STOPWORD',
    'original': u'أَخُو',
    'encletic': u'ه'
}
STOPWORDS[u'فأخوكم'] = {
    'word': u'فأخوكم',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَأَخُوكُمْ',
    'stem': u'أَخُو',
    'type': u'STOPWORD',
    'original': u'أَخُو',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فأخوكن'] = {
    'word': u'فأخوكن',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَأَخُوكُنَّ',
    'stem': u'أَخُو',
    'type': u'STOPWORD',
    'original': u'أَخُو',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فأخوها'] = {
    'word': u'فأخوها',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَأَخُوهَا',
    'stem': u'أَخُو',
    'type': u'STOPWORD',
    'original': u'أَخُو',
    'encletic': u'هَا'
}
STOPWORDS[u'فأخوهم'] = {
    'word': u'فأخوهم',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَأَخُوهُمْ',
    'stem': u'أَخُو',
    'type': u'STOPWORD',
    'original': u'أَخُو',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فأخوهن'] = {
    'word': u'فأخوهن',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَأَخُوهُنَّ',
    'stem': u'أَخُو',
    'type': u'STOPWORD',
    'original': u'أَخُو',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فأخونا'] = {
    'word': u'فأخونا',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَأَخُونَا',
    'stem': u'أَخُو',
    'type': u'STOPWORD',
    'original': u'أَخُو',
    'encletic': u'نَا'
}
STOPWORDS[u'فأخوكما'] = {
    'word': u'فأخوكما',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَأَخُوكُمَا',
    'stem': u'أَخُو',
    'type': u'STOPWORD',
    'original': u'أَخُو',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فأخوهما'] = {
    'word': u'فأخوهما',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَأَخُوهُمَا',
    'stem': u'أَخُو',
    'type': u'STOPWORD',
    'original': u'أَخُو',
    'encletic': u'هُمَا'
}
STOPWORDS[u'حمو'] = {
    'word': u'حَمُو',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة',
    'vocalized': u'حَمُو',
    'stem': u'حَمُو',
    'type': u'STOPWORD',
    'original': u'حَمُو',
    'encletic': u''
}
STOPWORDS[u'وحمو'] = {
    'word': u'وحمو',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف',
    'vocalized': u'وَحَمُو',
    'stem': u'حَمُو',
    'type': u'STOPWORD',
    'original': u'حَمُو',
    'encletic': u''
}
STOPWORDS[u'فحمو'] = {
    'word': u'فحمو',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف',
    'vocalized': u'فَحَمُو',
    'stem': u'حَمُو',
    'type': u'STOPWORD',
    'original': u'حَمُو',
    'encletic': u''
}
STOPWORDS[u'حموي'] = {
    'word': u'حموي',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'حَمُوِي',
    'stem': u'حَمُو',
    'type': u'STOPWORD',
    'original': u'حَمُو',
    'encletic': u'ِي'
}
STOPWORDS[u'حموك'] = {
    'word': u'حموك',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'حَمُوكَ',
    'stem': u'حَمُو',
    'type': u'STOPWORD',
    'original': u'حَمُو',
    'encletic': u'كَ'
}
STOPWORDS[u'حموك'] = {
    'word': u'حموك',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'حَمُوكِ',
    'stem': u'حَمُو',
    'type': u'STOPWORD',
    'original': u'حَمُو',
    'encletic': u'كِ'
}
STOPWORDS[u'حموه'] = {
    'word': u'حموه',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'حَمُوه',
    'stem': u'حَمُو',
    'type': u'STOPWORD',
    'original': u'حَمُو',
    'encletic': u'ه'
}
STOPWORDS[u'حموكم'] = {
    'word': u'حموكم',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'حَمُوكُمْ',
    'stem': u'حَمُو',
    'type': u'STOPWORD',
    'original': u'حَمُو',
    'encletic': u'كُمْ'
}
STOPWORDS[u'حموكن'] = {
    'word': u'حموكن',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'حَمُوكُنَّ',
    'stem': u'حَمُو',
    'type': u'STOPWORD',
    'original': u'حَمُو',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'حموها'] = {
    'word': u'حموها',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'حَمُوهَا',
    'stem': u'حَمُو',
    'type': u'STOPWORD',
    'original': u'حَمُو',
    'encletic': u'هَا'
}
STOPWORDS[u'حموهم'] = {
    'word': u'حموهم',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'حَمُوهُمْ',
    'stem': u'حَمُو',
    'type': u'STOPWORD',
    'original': u'حَمُو',
    'encletic': u'هُمْ'
}
STOPWORDS[u'حموهن'] = {
    'word': u'حموهن',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'حَمُوهُنَّ',
    'stem': u'حَمُو',
    'type': u'STOPWORD',
    'original': u'حَمُو',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'حمونا'] = {
    'word': u'حمونا',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'حَمُونَا',
    'stem': u'حَمُو',
    'type': u'STOPWORD',
    'original': u'حَمُو',
    'encletic': u'نَا'
}
STOPWORDS[u'حموكما'] = {
    'word': u'حموكما',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'حَمُوكُمَا',
    'stem': u'حَمُو',
    'type': u'STOPWORD',
    'original': u'حَمُو',
    'encletic': u'كُمَا'
}
STOPWORDS[u'حموهما'] = {
    'word': u'حموهما',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'حَمُوهُمَا',
    'stem': u'حَمُو',
    'type': u'STOPWORD',
    'original': u'حَمُو',
    'encletic': u'هُمَا'
}
STOPWORDS[u'وحموي'] = {
    'word': u'وحموي',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَحَمُوِي',
    'stem': u'حَمُو',
    'type': u'STOPWORD',
    'original': u'حَمُو',
    'encletic': u'ِي'
}
STOPWORDS[u'وحموك'] = {
    'word': u'وحموك',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَحَمُوكَ',
    'stem': u'حَمُو',
    'type': u'STOPWORD',
    'original': u'حَمُو',
    'encletic': u'كَ'
}
STOPWORDS[u'وحموك'] = {
    'word': u'وحموك',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَحَمُوكِ',
    'stem': u'حَمُو',
    'type': u'STOPWORD',
    'original': u'حَمُو',
    'encletic': u'كِ'
}
STOPWORDS[u'وحموه'] = {
    'word': u'وحموه',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَحَمُوه',
    'stem': u'حَمُو',
    'type': u'STOPWORD',
    'original': u'حَمُو',
    'encletic': u'ه'
}
STOPWORDS[u'وحموكم'] = {
    'word': u'وحموكم',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَحَمُوكُمْ',
    'stem': u'حَمُو',
    'type': u'STOPWORD',
    'original': u'حَمُو',
    'encletic': u'كُمْ'
}
STOPWORDS[u'وحموكن'] = {
    'word': u'وحموكن',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَحَمُوكُنَّ',
    'stem': u'حَمُو',
    'type': u'STOPWORD',
    'original': u'حَمُو',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'وحموها'] = {
    'word': u'وحموها',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَحَمُوهَا',
    'stem': u'حَمُو',
    'type': u'STOPWORD',
    'original': u'حَمُو',
    'encletic': u'هَا'
}
STOPWORDS[u'وحموهم'] = {
    'word': u'وحموهم',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَحَمُوهُمْ',
    'stem': u'حَمُو',
    'type': u'STOPWORD',
    'original': u'حَمُو',
    'encletic': u'هُمْ'
}
STOPWORDS[u'وحموهن'] = {
    'word': u'وحموهن',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَحَمُوهُنَّ',
    'stem': u'حَمُو',
    'type': u'STOPWORD',
    'original': u'حَمُو',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'وحمونا'] = {
    'word': u'وحمونا',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَحَمُونَا',
    'stem': u'حَمُو',
    'type': u'STOPWORD',
    'original': u'حَمُو',
    'encletic': u'نَا'
}
STOPWORDS[u'وحموكما'] = {
    'word': u'وحموكما',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَحَمُوكُمَا',
    'stem': u'حَمُو',
    'type': u'STOPWORD',
    'original': u'حَمُو',
    'encletic': u'كُمَا'
}
STOPWORDS[u'وحموهما'] = {
    'word': u'وحموهما',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَحَمُوهُمَا',
    'stem': u'حَمُو',
    'type': u'STOPWORD',
    'original': u'حَمُو',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فحموي'] = {
    'word': u'فحموي',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَحَمُوِي',
    'stem': u'حَمُو',
    'type': u'STOPWORD',
    'original': u'حَمُو',
    'encletic': u'ِي'
}
STOPWORDS[u'فحموك'] = {
    'word': u'فحموك',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَحَمُوكَ',
    'stem': u'حَمُو',
    'type': u'STOPWORD',
    'original': u'حَمُو',
    'encletic': u'كَ'
}
STOPWORDS[u'فحموك'] = {
    'word': u'فحموك',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَحَمُوكِ',
    'stem': u'حَمُو',
    'type': u'STOPWORD',
    'original': u'حَمُو',
    'encletic': u'كِ'
}
STOPWORDS[u'فحموه'] = {
    'word': u'فحموه',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَحَمُوه',
    'stem': u'حَمُو',
    'type': u'STOPWORD',
    'original': u'حَمُو',
    'encletic': u'ه'
}
STOPWORDS[u'فحموكم'] = {
    'word': u'فحموكم',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَحَمُوكُمْ',
    'stem': u'حَمُو',
    'type': u'STOPWORD',
    'original': u'حَمُو',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فحموكن'] = {
    'word': u'فحموكن',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَحَمُوكُنَّ',
    'stem': u'حَمُو',
    'type': u'STOPWORD',
    'original': u'حَمُو',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فحموها'] = {
    'word': u'فحموها',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَحَمُوهَا',
    'stem': u'حَمُو',
    'type': u'STOPWORD',
    'original': u'حَمُو',
    'encletic': u'هَا'
}
STOPWORDS[u'فحموهم'] = {
    'word': u'فحموهم',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَحَمُوهُمْ',
    'stem': u'حَمُو',
    'type': u'STOPWORD',
    'original': u'حَمُو',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فحموهن'] = {
    'word': u'فحموهن',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَحَمُوهُنَّ',
    'stem': u'حَمُو',
    'type': u'STOPWORD',
    'original': u'حَمُو',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فحمونا'] = {
    'word': u'فحمونا',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَحَمُونَا',
    'stem': u'حَمُو',
    'type': u'STOPWORD',
    'original': u'حَمُو',
    'encletic': u'نَا'
}
STOPWORDS[u'فحموكما'] = {
    'word': u'فحموكما',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَحَمُوكُمَا',
    'stem': u'حَمُو',
    'type': u'STOPWORD',
    'original': u'حَمُو',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فحموهما'] = {
    'word': u'فحموهما',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَحَمُوهُمَا',
    'stem': u'حَمُو',
    'type': u'STOPWORD',
    'original': u'حَمُو',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ذو'] = {
    'word': u'ذُو',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة',
    'vocalized': u'ذُو',
    'stem': u'ذُو',
    'type': u'STOPWORD',
    'original': u'ذُو',
    'encletic': u''
}
STOPWORDS[u'وذو'] = {
    'word': u'وذو',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف',
    'vocalized': u'وَذُو',
    'stem': u'ذُو',
    'type': u'STOPWORD',
    'original': u'ذُو',
    'encletic': u''
}
STOPWORDS[u'فذو'] = {
    'word': u'فذو',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف',
    'vocalized': u'فَذُو',
    'stem': u'ذُو',
    'type': u'STOPWORD',
    'original': u'ذُو',
    'encletic': u''
}
STOPWORDS[u'فو'] = {
    'word': u'فُو',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة',
    'vocalized': u'فُو',
    'stem': u'فُو',
    'type': u'STOPWORD',
    'original': u'فُو',
    'encletic': u''
}
STOPWORDS[u'وفو'] = {
    'word': u'وفو',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف',
    'vocalized': u'وَفُو',
    'stem': u'فُو',
    'type': u'STOPWORD',
    'original': u'فُو',
    'encletic': u''
}
STOPWORDS[u'ففو'] = {
    'word': u'ففو',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف',
    'vocalized': u'فَفُو',
    'stem': u'فُو',
    'type': u'STOPWORD',
    'original': u'فُو',
    'encletic': u''
}
STOPWORDS[u'فوي'] = {
    'word': u'فوي',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'فُوِي',
    'stem': u'فُو',
    'type': u'STOPWORD',
    'original': u'فُو',
    'encletic': u'ِي'
}
STOPWORDS[u'فوك'] = {
    'word': u'فوك',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'فُوكَ',
    'stem': u'فُو',
    'type': u'STOPWORD',
    'original': u'فُو',
    'encletic': u'كَ'
}
STOPWORDS[u'فوك'] = {
    'word': u'فوك',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'فُوكِ',
    'stem': u'فُو',
    'type': u'STOPWORD',
    'original': u'فُو',
    'encletic': u'كِ'
}
STOPWORDS[u'فوه'] = {
    'word': u'فوه',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'فُوه',
    'stem': u'فُو',
    'type': u'STOPWORD',
    'original': u'فُو',
    'encletic': u'ه'
}
STOPWORDS[u'فوكم'] = {
    'word': u'فوكم',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'فُوكُمْ',
    'stem': u'فُو',
    'type': u'STOPWORD',
    'original': u'فُو',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فوكن'] = {
    'word': u'فوكن',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'فُوكُنَّ',
    'stem': u'فُو',
    'type': u'STOPWORD',
    'original': u'فُو',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فوها'] = {
    'word': u'فوها',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'فُوهَا',
    'stem': u'فُو',
    'type': u'STOPWORD',
    'original': u'فُو',
    'encletic': u'هَا'
}
STOPWORDS[u'فوهم'] = {
    'word': u'فوهم',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'فُوهُمْ',
    'stem': u'فُو',
    'type': u'STOPWORD',
    'original': u'فُو',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فوهن'] = {
    'word': u'فوهن',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'فُوهُنَّ',
    'stem': u'فُو',
    'type': u'STOPWORD',
    'original': u'فُو',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فونا'] = {
    'word': u'فونا',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'فُونَا',
    'stem': u'فُو',
    'type': u'STOPWORD',
    'original': u'فُو',
    'encletic': u'نَا'
}
STOPWORDS[u'فوكما'] = {
    'word': u'فوكما',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'فُوكُمَا',
    'stem': u'فُو',
    'type': u'STOPWORD',
    'original': u'فُو',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فوهما'] = {
    'word': u'فوهما',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'فُوهُمَا',
    'stem': u'فُو',
    'type': u'STOPWORD',
    'original': u'فُو',
    'encletic': u'هُمَا'
}
STOPWORDS[u'وفوي'] = {
    'word': u'وفوي',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَفُوِي',
    'stem': u'فُو',
    'type': u'STOPWORD',
    'original': u'فُو',
    'encletic': u'ِي'
}
STOPWORDS[u'وفوك'] = {
    'word': u'وفوك',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَفُوكَ',
    'stem': u'فُو',
    'type': u'STOPWORD',
    'original': u'فُو',
    'encletic': u'كَ'
}
STOPWORDS[u'وفوك'] = {
    'word': u'وفوك',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَفُوكِ',
    'stem': u'فُو',
    'type': u'STOPWORD',
    'original': u'فُو',
    'encletic': u'كِ'
}
STOPWORDS[u'وفوه'] = {
    'word': u'وفوه',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَفُوه',
    'stem': u'فُو',
    'type': u'STOPWORD',
    'original': u'فُو',
    'encletic': u'ه'
}
STOPWORDS[u'وفوكم'] = {
    'word': u'وفوكم',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَفُوكُمْ',
    'stem': u'فُو',
    'type': u'STOPWORD',
    'original': u'فُو',
    'encletic': u'كُمْ'
}
STOPWORDS[u'وفوكن'] = {
    'word': u'وفوكن',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَفُوكُنَّ',
    'stem': u'فُو',
    'type': u'STOPWORD',
    'original': u'فُو',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'وفوها'] = {
    'word': u'وفوها',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَفُوهَا',
    'stem': u'فُو',
    'type': u'STOPWORD',
    'original': u'فُو',
    'encletic': u'هَا'
}
STOPWORDS[u'وفوهم'] = {
    'word': u'وفوهم',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَفُوهُمْ',
    'stem': u'فُو',
    'type': u'STOPWORD',
    'original': u'فُو',
    'encletic': u'هُمْ'
}
STOPWORDS[u'وفوهن'] = {
    'word': u'وفوهن',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَفُوهُنَّ',
    'stem': u'فُو',
    'type': u'STOPWORD',
    'original': u'فُو',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'وفونا'] = {
    'word': u'وفونا',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَفُونَا',
    'stem': u'فُو',
    'type': u'STOPWORD',
    'original': u'فُو',
    'encletic': u'نَا'
}
STOPWORDS[u'وفوكما'] = {
    'word': u'وفوكما',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَفُوكُمَا',
    'stem': u'فُو',
    'type': u'STOPWORD',
    'original': u'فُو',
    'encletic': u'كُمَا'
}
STOPWORDS[u'وفوهما'] = {
    'word': u'وفوهما',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَفُوهُمَا',
    'stem': u'فُو',
    'type': u'STOPWORD',
    'original': u'فُو',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ففوي'] = {
    'word': u'ففوي',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَفُوِي',
    'stem': u'فُو',
    'type': u'STOPWORD',
    'original': u'فُو',
    'encletic': u'ِي'
}
STOPWORDS[u'ففوك'] = {
    'word': u'ففوك',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَفُوكَ',
    'stem': u'فُو',
    'type': u'STOPWORD',
    'original': u'فُو',
    'encletic': u'كَ'
}
STOPWORDS[u'ففوك'] = {
    'word': u'ففوك',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَفُوكِ',
    'stem': u'فُو',
    'type': u'STOPWORD',
    'original': u'فُو',
    'encletic': u'كِ'
}
STOPWORDS[u'ففوه'] = {
    'word': u'ففوه',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَفُوه',
    'stem': u'فُو',
    'type': u'STOPWORD',
    'original': u'فُو',
    'encletic': u'ه'
}
STOPWORDS[u'ففوكم'] = {
    'word': u'ففوكم',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَفُوكُمْ',
    'stem': u'فُو',
    'type': u'STOPWORD',
    'original': u'فُو',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ففوكن'] = {
    'word': u'ففوكن',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَفُوكُنَّ',
    'stem': u'فُو',
    'type': u'STOPWORD',
    'original': u'فُو',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ففوها'] = {
    'word': u'ففوها',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَفُوهَا',
    'stem': u'فُو',
    'type': u'STOPWORD',
    'original': u'فُو',
    'encletic': u'هَا'
}
STOPWORDS[u'ففوهم'] = {
    'word': u'ففوهم',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَفُوهُمْ',
    'stem': u'فُو',
    'type': u'STOPWORD',
    'original': u'فُو',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ففوهن'] = {
    'word': u'ففوهن',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَفُوهُنَّ',
    'stem': u'فُو',
    'type': u'STOPWORD',
    'original': u'فُو',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ففونا'] = {
    'word': u'ففونا',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَفُونَا',
    'stem': u'فُو',
    'type': u'STOPWORD',
    'original': u'فُو',
    'encletic': u'نَا'
}
STOPWORDS[u'ففوكما'] = {
    'word': u'ففوكما',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَفُوكُمَا',
    'stem': u'فُو',
    'type': u'STOPWORD',
    'original': u'فُو',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ففوهما'] = {
    'word': u'ففوهما',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَفُوهُمَا',
    'stem': u'فُو',
    'type': u'STOPWORD',
    'original': u'فُو',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أبا'] = {
    'word': u'أَبَا',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة',
    'vocalized': u'أَبَا',
    'stem': u'أَبَا',
    'type': u'STOPWORD',
    'original': u'أَبَا',
    'encletic': u''
}
STOPWORDS[u'وأبا'] = {
    'word': u'وأبا',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف',
    'vocalized': u'وَأَبَا',
    'stem': u'أَبَا',
    'type': u'STOPWORD',
    'original': u'أَبَا',
    'encletic': u''
}
STOPWORDS[u'فأبا'] = {
    'word': u'فأبا',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف',
    'vocalized': u'فَأَبَا',
    'stem': u'أَبَا',
    'type': u'STOPWORD',
    'original': u'أَبَا',
    'encletic': u''
}
STOPWORDS[u'أباي'] = {
    'word': u'أباي',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'أَبَاِي',
    'stem': u'أَبَا',
    'type': u'STOPWORD',
    'original': u'أَبَا',
    'encletic': u'ِي'
}
STOPWORDS[u'أباك'] = {
    'word': u'أباك',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'أَبَاكَ',
    'stem': u'أَبَا',
    'type': u'STOPWORD',
    'original': u'أَبَا',
    'encletic': u'كَ'
}
STOPWORDS[u'أباك'] = {
    'word': u'أباك',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'أَبَاكِ',
    'stem': u'أَبَا',
    'type': u'STOPWORD',
    'original': u'أَبَا',
    'encletic': u'كِ'
}
STOPWORDS[u'أباه'] = {
    'word': u'أباه',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'أَبَاه',
    'stem': u'أَبَا',
    'type': u'STOPWORD',
    'original': u'أَبَا',
    'encletic': u'ه'
}
STOPWORDS[u'أباكم'] = {
    'word': u'أباكم',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'أَبَاكُمْ',
    'stem': u'أَبَا',
    'type': u'STOPWORD',
    'original': u'أَبَا',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أباكن'] = {
    'word': u'أباكن',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'أَبَاكُنَّ',
    'stem': u'أَبَا',
    'type': u'STOPWORD',
    'original': u'أَبَا',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أباها'] = {
    'word': u'أباها',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'أَبَاهَا',
    'stem': u'أَبَا',
    'type': u'STOPWORD',
    'original': u'أَبَا',
    'encletic': u'هَا'
}
STOPWORDS[u'أباهم'] = {
    'word': u'أباهم',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'أَبَاهُمْ',
    'stem': u'أَبَا',
    'type': u'STOPWORD',
    'original': u'أَبَا',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أباهن'] = {
    'word': u'أباهن',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'أَبَاهُنَّ',
    'stem': u'أَبَا',
    'type': u'STOPWORD',
    'original': u'أَبَا',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أبانا'] = {
    'word': u'أبانا',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'أَبَانَا',
    'stem': u'أَبَا',
    'type': u'STOPWORD',
    'original': u'أَبَا',
    'encletic': u'نَا'
}
STOPWORDS[u'أباكما'] = {
    'word': u'أباكما',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'أَبَاكُمَا',
    'stem': u'أَبَا',
    'type': u'STOPWORD',
    'original': u'أَبَا',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أباهما'] = {
    'word': u'أباهما',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'أَبَاهُمَا',
    'stem': u'أَبَا',
    'type': u'STOPWORD',
    'original': u'أَبَا',
    'encletic': u'هُمَا'
}
STOPWORDS[u'وأباي'] = {
    'word': u'وأباي',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَأَبَاِي',
    'stem': u'أَبَا',
    'type': u'STOPWORD',
    'original': u'أَبَا',
    'encletic': u'ِي'
}
STOPWORDS[u'وأباك'] = {
    'word': u'وأباك',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَأَبَاكَ',
    'stem': u'أَبَا',
    'type': u'STOPWORD',
    'original': u'أَبَا',
    'encletic': u'كَ'
}
STOPWORDS[u'وأباك'] = {
    'word': u'وأباك',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَأَبَاكِ',
    'stem': u'أَبَا',
    'type': u'STOPWORD',
    'original': u'أَبَا',
    'encletic': u'كِ'
}
STOPWORDS[u'وأباه'] = {
    'word': u'وأباه',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَأَبَاه',
    'stem': u'أَبَا',
    'type': u'STOPWORD',
    'original': u'أَبَا',
    'encletic': u'ه'
}
STOPWORDS[u'وأباكم'] = {
    'word': u'وأباكم',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَأَبَاكُمْ',
    'stem': u'أَبَا',
    'type': u'STOPWORD',
    'original': u'أَبَا',
    'encletic': u'كُمْ'
}
STOPWORDS[u'وأباكن'] = {
    'word': u'وأباكن',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَأَبَاكُنَّ',
    'stem': u'أَبَا',
    'type': u'STOPWORD',
    'original': u'أَبَا',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'وأباها'] = {
    'word': u'وأباها',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَأَبَاهَا',
    'stem': u'أَبَا',
    'type': u'STOPWORD',
    'original': u'أَبَا',
    'encletic': u'هَا'
}
STOPWORDS[u'وأباهم'] = {
    'word': u'وأباهم',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَأَبَاهُمْ',
    'stem': u'أَبَا',
    'type': u'STOPWORD',
    'original': u'أَبَا',
    'encletic': u'هُمْ'
}
STOPWORDS[u'وأباهن'] = {
    'word': u'وأباهن',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَأَبَاهُنَّ',
    'stem': u'أَبَا',
    'type': u'STOPWORD',
    'original': u'أَبَا',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'وأبانا'] = {
    'word': u'وأبانا',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَأَبَانَا',
    'stem': u'أَبَا',
    'type': u'STOPWORD',
    'original': u'أَبَا',
    'encletic': u'نَا'
}
STOPWORDS[u'وأباكما'] = {
    'word': u'وأباكما',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَأَبَاكُمَا',
    'stem': u'أَبَا',
    'type': u'STOPWORD',
    'original': u'أَبَا',
    'encletic': u'كُمَا'
}
STOPWORDS[u'وأباهما'] = {
    'word': u'وأباهما',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَأَبَاهُمَا',
    'stem': u'أَبَا',
    'type': u'STOPWORD',
    'original': u'أَبَا',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فأباي'] = {
    'word': u'فأباي',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَأَبَاِي',
    'stem': u'أَبَا',
    'type': u'STOPWORD',
    'original': u'أَبَا',
    'encletic': u'ِي'
}
STOPWORDS[u'فأباك'] = {
    'word': u'فأباك',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَأَبَاكَ',
    'stem': u'أَبَا',
    'type': u'STOPWORD',
    'original': u'أَبَا',
    'encletic': u'كَ'
}
STOPWORDS[u'فأباك'] = {
    'word': u'فأباك',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَأَبَاكِ',
    'stem': u'أَبَا',
    'type': u'STOPWORD',
    'original': u'أَبَا',
    'encletic': u'كِ'
}
STOPWORDS[u'فأباه'] = {
    'word': u'فأباه',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَأَبَاه',
    'stem': u'أَبَا',
    'type': u'STOPWORD',
    'original': u'أَبَا',
    'encletic': u'ه'
}
STOPWORDS[u'فأباكم'] = {
    'word': u'فأباكم',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَأَبَاكُمْ',
    'stem': u'أَبَا',
    'type': u'STOPWORD',
    'original': u'أَبَا',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فأباكن'] = {
    'word': u'فأباكن',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَأَبَاكُنَّ',
    'stem': u'أَبَا',
    'type': u'STOPWORD',
    'original': u'أَبَا',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فأباها'] = {
    'word': u'فأباها',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَأَبَاهَا',
    'stem': u'أَبَا',
    'type': u'STOPWORD',
    'original': u'أَبَا',
    'encletic': u'هَا'
}
STOPWORDS[u'فأباهم'] = {
    'word': u'فأباهم',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَأَبَاهُمْ',
    'stem': u'أَبَا',
    'type': u'STOPWORD',
    'original': u'أَبَا',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فأباهن'] = {
    'word': u'فأباهن',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَأَبَاهُنَّ',
    'stem': u'أَبَا',
    'type': u'STOPWORD',
    'original': u'أَبَا',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فأبانا'] = {
    'word': u'فأبانا',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَأَبَانَا',
    'stem': u'أَبَا',
    'type': u'STOPWORD',
    'original': u'أَبَا',
    'encletic': u'نَا'
}
STOPWORDS[u'فأباكما'] = {
    'word': u'فأباكما',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَأَبَاكُمَا',
    'stem': u'أَبَا',
    'type': u'STOPWORD',
    'original': u'أَبَا',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فأباهما'] = {
    'word': u'فأباهما',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَأَبَاهُمَا',
    'stem': u'أَبَا',
    'type': u'STOPWORD',
    'original': u'أَبَا',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أخا'] = {
    'word': u'أَخَا',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة',
    'vocalized': u'أَخَا',
    'stem': u'أَخَا',
    'type': u'STOPWORD',
    'original': u'أَخَا',
    'encletic': u''
}
STOPWORDS[u'وأخا'] = {
    'word': u'وأخا',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف',
    'vocalized': u'وَأَخَا',
    'stem': u'أَخَا',
    'type': u'STOPWORD',
    'original': u'أَخَا',
    'encletic': u''
}
STOPWORDS[u'فأخا'] = {
    'word': u'فأخا',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف',
    'vocalized': u'فَأَخَا',
    'stem': u'أَخَا',
    'type': u'STOPWORD',
    'original': u'أَخَا',
    'encletic': u''
}
STOPWORDS[u'أخاي'] = {
    'word': u'أخاي',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'أَخَاِي',
    'stem': u'أَخَا',
    'type': u'STOPWORD',
    'original': u'أَخَا',
    'encletic': u'ِي'
}
STOPWORDS[u'أخاك'] = {
    'word': u'أخاك',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'أَخَاكَ',
    'stem': u'أَخَا',
    'type': u'STOPWORD',
    'original': u'أَخَا',
    'encletic': u'كَ'
}
STOPWORDS[u'أخاك'] = {
    'word': u'أخاك',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'أَخَاكِ',
    'stem': u'أَخَا',
    'type': u'STOPWORD',
    'original': u'أَخَا',
    'encletic': u'كِ'
}
STOPWORDS[u'أخاه'] = {
    'word': u'أخاه',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'أَخَاه',
    'stem': u'أَخَا',
    'type': u'STOPWORD',
    'original': u'أَخَا',
    'encletic': u'ه'
}
STOPWORDS[u'أخاكم'] = {
    'word': u'أخاكم',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'أَخَاكُمْ',
    'stem': u'أَخَا',
    'type': u'STOPWORD',
    'original': u'أَخَا',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أخاكن'] = {
    'word': u'أخاكن',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'أَخَاكُنَّ',
    'stem': u'أَخَا',
    'type': u'STOPWORD',
    'original': u'أَخَا',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أخاها'] = {
    'word': u'أخاها',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'أَخَاهَا',
    'stem': u'أَخَا',
    'type': u'STOPWORD',
    'original': u'أَخَا',
    'encletic': u'هَا'
}
STOPWORDS[u'أخاهم'] = {
    'word': u'أخاهم',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'أَخَاهُمْ',
    'stem': u'أَخَا',
    'type': u'STOPWORD',
    'original': u'أَخَا',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أخاهن'] = {
    'word': u'أخاهن',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'أَخَاهُنَّ',
    'stem': u'أَخَا',
    'type': u'STOPWORD',
    'original': u'أَخَا',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أخانا'] = {
    'word': u'أخانا',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'أَخَانَا',
    'stem': u'أَخَا',
    'type': u'STOPWORD',
    'original': u'أَخَا',
    'encletic': u'نَا'
}
STOPWORDS[u'أخاكما'] = {
    'word': u'أخاكما',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'أَخَاكُمَا',
    'stem': u'أَخَا',
    'type': u'STOPWORD',
    'original': u'أَخَا',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أخاهما'] = {
    'word': u'أخاهما',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'أَخَاهُمَا',
    'stem': u'أَخَا',
    'type': u'STOPWORD',
    'original': u'أَخَا',
    'encletic': u'هُمَا'
}
STOPWORDS[u'وأخاي'] = {
    'word': u'وأخاي',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَأَخَاِي',
    'stem': u'أَخَا',
    'type': u'STOPWORD',
    'original': u'أَخَا',
    'encletic': u'ِي'
}
STOPWORDS[u'وأخاك'] = {
    'word': u'وأخاك',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَأَخَاكَ',
    'stem': u'أَخَا',
    'type': u'STOPWORD',
    'original': u'أَخَا',
    'encletic': u'كَ'
}
STOPWORDS[u'وأخاك'] = {
    'word': u'وأخاك',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَأَخَاكِ',
    'stem': u'أَخَا',
    'type': u'STOPWORD',
    'original': u'أَخَا',
    'encletic': u'كِ'
}
STOPWORDS[u'وأخاه'] = {
    'word': u'وأخاه',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَأَخَاه',
    'stem': u'أَخَا',
    'type': u'STOPWORD',
    'original': u'أَخَا',
    'encletic': u'ه'
}
STOPWORDS[u'وأخاكم'] = {
    'word': u'وأخاكم',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَأَخَاكُمْ',
    'stem': u'أَخَا',
    'type': u'STOPWORD',
    'original': u'أَخَا',
    'encletic': u'كُمْ'
}
STOPWORDS[u'وأخاكن'] = {
    'word': u'وأخاكن',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَأَخَاكُنَّ',
    'stem': u'أَخَا',
    'type': u'STOPWORD',
    'original': u'أَخَا',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'وأخاها'] = {
    'word': u'وأخاها',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَأَخَاهَا',
    'stem': u'أَخَا',
    'type': u'STOPWORD',
    'original': u'أَخَا',
    'encletic': u'هَا'
}
STOPWORDS[u'وأخاهم'] = {
    'word': u'وأخاهم',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَأَخَاهُمْ',
    'stem': u'أَخَا',
    'type': u'STOPWORD',
    'original': u'أَخَا',
    'encletic': u'هُمْ'
}
STOPWORDS[u'وأخاهن'] = {
    'word': u'وأخاهن',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَأَخَاهُنَّ',
    'stem': u'أَخَا',
    'type': u'STOPWORD',
    'original': u'أَخَا',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'وأخانا'] = {
    'word': u'وأخانا',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَأَخَانَا',
    'stem': u'أَخَا',
    'type': u'STOPWORD',
    'original': u'أَخَا',
    'encletic': u'نَا'
}
STOPWORDS[u'وأخاكما'] = {
    'word': u'وأخاكما',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَأَخَاكُمَا',
    'stem': u'أَخَا',
    'type': u'STOPWORD',
    'original': u'أَخَا',
    'encletic': u'كُمَا'
}
STOPWORDS[u'وأخاهما'] = {
    'word': u'وأخاهما',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَأَخَاهُمَا',
    'stem': u'أَخَا',
    'type': u'STOPWORD',
    'original': u'أَخَا',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فأخاي'] = {
    'word': u'فأخاي',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَأَخَاِي',
    'stem': u'أَخَا',
    'type': u'STOPWORD',
    'original': u'أَخَا',
    'encletic': u'ِي'
}
STOPWORDS[u'فأخاك'] = {
    'word': u'فأخاك',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَأَخَاكَ',
    'stem': u'أَخَا',
    'type': u'STOPWORD',
    'original': u'أَخَا',
    'encletic': u'كَ'
}
STOPWORDS[u'فأخاك'] = {
    'word': u'فأخاك',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَأَخَاكِ',
    'stem': u'أَخَا',
    'type': u'STOPWORD',
    'original': u'أَخَا',
    'encletic': u'كِ'
}
STOPWORDS[u'فأخاه'] = {
    'word': u'فأخاه',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَأَخَاه',
    'stem': u'أَخَا',
    'type': u'STOPWORD',
    'original': u'أَخَا',
    'encletic': u'ه'
}
STOPWORDS[u'فأخاكم'] = {
    'word': u'فأخاكم',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَأَخَاكُمْ',
    'stem': u'أَخَا',
    'type': u'STOPWORD',
    'original': u'أَخَا',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فأخاكن'] = {
    'word': u'فأخاكن',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَأَخَاكُنَّ',
    'stem': u'أَخَا',
    'type': u'STOPWORD',
    'original': u'أَخَا',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فأخاها'] = {
    'word': u'فأخاها',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَأَخَاهَا',
    'stem': u'أَخَا',
    'type': u'STOPWORD',
    'original': u'أَخَا',
    'encletic': u'هَا'
}
STOPWORDS[u'فأخاهم'] = {
    'word': u'فأخاهم',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَأَخَاهُمْ',
    'stem': u'أَخَا',
    'type': u'STOPWORD',
    'original': u'أَخَا',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فأخاهن'] = {
    'word': u'فأخاهن',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَأَخَاهُنَّ',
    'stem': u'أَخَا',
    'type': u'STOPWORD',
    'original': u'أَخَا',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فأخانا'] = {
    'word': u'فأخانا',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَأَخَانَا',
    'stem': u'أَخَا',
    'type': u'STOPWORD',
    'original': u'أَخَا',
    'encletic': u'نَا'
}
STOPWORDS[u'فأخاكما'] = {
    'word': u'فأخاكما',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَأَخَاكُمَا',
    'stem': u'أَخَا',
    'type': u'STOPWORD',
    'original': u'أَخَا',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فأخاهما'] = {
    'word': u'فأخاهما',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَأَخَاهُمَا',
    'stem': u'أَخَا',
    'type': u'STOPWORD',
    'original': u'أَخَا',
    'encletic': u'هُمَا'
}
STOPWORDS[u'حما'] = {
    'word': u'حَمَا',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة',
    'vocalized': u'حَمَا',
    'stem': u'حَمَا',
    'type': u'STOPWORD',
    'original': u'حَمَا',
    'encletic': u''
}
STOPWORDS[u'وحما'] = {
    'word': u'وحما',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف',
    'vocalized': u'وَحَمَا',
    'stem': u'حَمَا',
    'type': u'STOPWORD',
    'original': u'حَمَا',
    'encletic': u''
}
STOPWORDS[u'فحما'] = {
    'word': u'فحما',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف',
    'vocalized': u'فَحَمَا',
    'stem': u'حَمَا',
    'type': u'STOPWORD',
    'original': u'حَمَا',
    'encletic': u''
}
STOPWORDS[u'حماي'] = {
    'word': u'حماي',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'حَمَاِي',
    'stem': u'حَمَا',
    'type': u'STOPWORD',
    'original': u'حَمَا',
    'encletic': u'ِي'
}
STOPWORDS[u'حماك'] = {
    'word': u'حماك',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'حَمَاكَ',
    'stem': u'حَمَا',
    'type': u'STOPWORD',
    'original': u'حَمَا',
    'encletic': u'كَ'
}
STOPWORDS[u'حماك'] = {
    'word': u'حماك',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'حَمَاكِ',
    'stem': u'حَمَا',
    'type': u'STOPWORD',
    'original': u'حَمَا',
    'encletic': u'كِ'
}
STOPWORDS[u'حماه'] = {
    'word': u'حماه',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'حَمَاه',
    'stem': u'حَمَا',
    'type': u'STOPWORD',
    'original': u'حَمَا',
    'encletic': u'ه'
}
STOPWORDS[u'حماكم'] = {
    'word': u'حماكم',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'حَمَاكُمْ',
    'stem': u'حَمَا',
    'type': u'STOPWORD',
    'original': u'حَمَا',
    'encletic': u'كُمْ'
}
STOPWORDS[u'حماكن'] = {
    'word': u'حماكن',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'حَمَاكُنَّ',
    'stem': u'حَمَا',
    'type': u'STOPWORD',
    'original': u'حَمَا',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'حماها'] = {
    'word': u'حماها',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'حَمَاهَا',
    'stem': u'حَمَا',
    'type': u'STOPWORD',
    'original': u'حَمَا',
    'encletic': u'هَا'
}
STOPWORDS[u'حماهم'] = {
    'word': u'حماهم',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'حَمَاهُمْ',
    'stem': u'حَمَا',
    'type': u'STOPWORD',
    'original': u'حَمَا',
    'encletic': u'هُمْ'
}
STOPWORDS[u'حماهن'] = {
    'word': u'حماهن',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'حَمَاهُنَّ',
    'stem': u'حَمَا',
    'type': u'STOPWORD',
    'original': u'حَمَا',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'حمانا'] = {
    'word': u'حمانا',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'حَمَانَا',
    'stem': u'حَمَا',
    'type': u'STOPWORD',
    'original': u'حَمَا',
    'encletic': u'نَا'
}
STOPWORDS[u'حماكما'] = {
    'word': u'حماكما',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'حَمَاكُمَا',
    'stem': u'حَمَا',
    'type': u'STOPWORD',
    'original': u'حَمَا',
    'encletic': u'كُمَا'
}
STOPWORDS[u'حماهما'] = {
    'word': u'حماهما',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'حَمَاهُمَا',
    'stem': u'حَمَا',
    'type': u'STOPWORD',
    'original': u'حَمَا',
    'encletic': u'هُمَا'
}
STOPWORDS[u'وحماي'] = {
    'word': u'وحماي',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَحَمَاِي',
    'stem': u'حَمَا',
    'type': u'STOPWORD',
    'original': u'حَمَا',
    'encletic': u'ِي'
}
STOPWORDS[u'وحماك'] = {
    'word': u'وحماك',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَحَمَاكَ',
    'stem': u'حَمَا',
    'type': u'STOPWORD',
    'original': u'حَمَا',
    'encletic': u'كَ'
}
STOPWORDS[u'وحماك'] = {
    'word': u'وحماك',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَحَمَاكِ',
    'stem': u'حَمَا',
    'type': u'STOPWORD',
    'original': u'حَمَا',
    'encletic': u'كِ'
}
STOPWORDS[u'وحماه'] = {
    'word': u'وحماه',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَحَمَاه',
    'stem': u'حَمَا',
    'type': u'STOPWORD',
    'original': u'حَمَا',
    'encletic': u'ه'
}
STOPWORDS[u'وحماكم'] = {
    'word': u'وحماكم',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَحَمَاكُمْ',
    'stem': u'حَمَا',
    'type': u'STOPWORD',
    'original': u'حَمَا',
    'encletic': u'كُمْ'
}
STOPWORDS[u'وحماكن'] = {
    'word': u'وحماكن',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَحَمَاكُنَّ',
    'stem': u'حَمَا',
    'type': u'STOPWORD',
    'original': u'حَمَا',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'وحماها'] = {
    'word': u'وحماها',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَحَمَاهَا',
    'stem': u'حَمَا',
    'type': u'STOPWORD',
    'original': u'حَمَا',
    'encletic': u'هَا'
}
STOPWORDS[u'وحماهم'] = {
    'word': u'وحماهم',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَحَمَاهُمْ',
    'stem': u'حَمَا',
    'type': u'STOPWORD',
    'original': u'حَمَا',
    'encletic': u'هُمْ'
}
STOPWORDS[u'وحماهن'] = {
    'word': u'وحماهن',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَحَمَاهُنَّ',
    'stem': u'حَمَا',
    'type': u'STOPWORD',
    'original': u'حَمَا',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'وحمانا'] = {
    'word': u'وحمانا',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَحَمَانَا',
    'stem': u'حَمَا',
    'type': u'STOPWORD',
    'original': u'حَمَا',
    'encletic': u'نَا'
}
STOPWORDS[u'وحماكما'] = {
    'word': u'وحماكما',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَحَمَاكُمَا',
    'stem': u'حَمَا',
    'type': u'STOPWORD',
    'original': u'حَمَا',
    'encletic': u'كُمَا'
}
STOPWORDS[u'وحماهما'] = {
    'word': u'وحماهما',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَحَمَاهُمَا',
    'stem': u'حَمَا',
    'type': u'STOPWORD',
    'original': u'حَمَا',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فحماي'] = {
    'word': u'فحماي',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَحَمَاِي',
    'stem': u'حَمَا',
    'type': u'STOPWORD',
    'original': u'حَمَا',
    'encletic': u'ِي'
}
STOPWORDS[u'فحماك'] = {
    'word': u'فحماك',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَحَمَاكَ',
    'stem': u'حَمَا',
    'type': u'STOPWORD',
    'original': u'حَمَا',
    'encletic': u'كَ'
}
STOPWORDS[u'فحماك'] = {
    'word': u'فحماك',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَحَمَاكِ',
    'stem': u'حَمَا',
    'type': u'STOPWORD',
    'original': u'حَمَا',
    'encletic': u'كِ'
}
STOPWORDS[u'فحماه'] = {
    'word': u'فحماه',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَحَمَاه',
    'stem': u'حَمَا',
    'type': u'STOPWORD',
    'original': u'حَمَا',
    'encletic': u'ه'
}
STOPWORDS[u'فحماكم'] = {
    'word': u'فحماكم',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَحَمَاكُمْ',
    'stem': u'حَمَا',
    'type': u'STOPWORD',
    'original': u'حَمَا',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فحماكن'] = {
    'word': u'فحماكن',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَحَمَاكُنَّ',
    'stem': u'حَمَا',
    'type': u'STOPWORD',
    'original': u'حَمَا',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فحماها'] = {
    'word': u'فحماها',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَحَمَاهَا',
    'stem': u'حَمَا',
    'type': u'STOPWORD',
    'original': u'حَمَا',
    'encletic': u'هَا'
}
STOPWORDS[u'فحماهم'] = {
    'word': u'فحماهم',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَحَمَاهُمْ',
    'stem': u'حَمَا',
    'type': u'STOPWORD',
    'original': u'حَمَا',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فحماهن'] = {
    'word': u'فحماهن',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَحَمَاهُنَّ',
    'stem': u'حَمَا',
    'type': u'STOPWORD',
    'original': u'حَمَا',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فحمانا'] = {
    'word': u'فحمانا',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَحَمَانَا',
    'stem': u'حَمَا',
    'type': u'STOPWORD',
    'original': u'حَمَا',
    'encletic': u'نَا'
}
STOPWORDS[u'فحماكما'] = {
    'word': u'فحماكما',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَحَمَاكُمَا',
    'stem': u'حَمَا',
    'type': u'STOPWORD',
    'original': u'حَمَا',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فحماهما'] = {
    'word': u'فحماهما',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَحَمَاهُمَا',
    'stem': u'حَمَا',
    'type': u'STOPWORD',
    'original': u'حَمَا',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ذا'] = {
    'word': u'ذََا',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة',
    'vocalized': u'ذََا',
    'stem': u'ذََا',
    'type': u'STOPWORD',
    'original': u'ذََا',
    'encletic': u''
}
STOPWORDS[u'وذا'] = {
    'word': u'وذا',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف',
    'vocalized': u'وَذََا',
    'stem': u'ذََا',
    'type': u'STOPWORD',
    'original': u'ذََا',
    'encletic': u''
}
STOPWORDS[u'فذا'] = {
    'word': u'فذا',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف',
    'vocalized': u'فَذََا',
    'stem': u'ذََا',
    'type': u'STOPWORD',
    'original': u'ذََا',
    'encletic': u''
}
STOPWORDS[u'فا'] = {
    'word': u'فَا',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة',
    'vocalized': u'فَا',
    'stem': u'فَا',
    'type': u'STOPWORD',
    'original': u'فَا',
    'encletic': u''
}
STOPWORDS[u'وفا'] = {
    'word': u'وفا',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف',
    'vocalized': u'وَفَا',
    'stem': u'فَا',
    'type': u'STOPWORD',
    'original': u'فَا',
    'encletic': u''
}
STOPWORDS[u'ففا'] = {
    'word': u'ففا',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف',
    'vocalized': u'فَفَا',
    'stem': u'فَا',
    'type': u'STOPWORD',
    'original': u'فَا',
    'encletic': u''
}
STOPWORDS[u'فاي'] = {
    'word': u'فاي',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'فَاِي',
    'stem': u'فَا',
    'type': u'STOPWORD',
    'original': u'فَا',
    'encletic': u'ِي'
}
STOPWORDS[u'فاك'] = {
    'word': u'فاك',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'فَاكَ',
    'stem': u'فَا',
    'type': u'STOPWORD',
    'original': u'فَا',
    'encletic': u'كَ'
}
STOPWORDS[u'فاك'] = {
    'word': u'فاك',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'فَاكِ',
    'stem': u'فَا',
    'type': u'STOPWORD',
    'original': u'فَا',
    'encletic': u'كِ'
}
STOPWORDS[u'فاه'] = {
    'word': u'فاه',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'فَاه',
    'stem': u'فَا',
    'type': u'STOPWORD',
    'original': u'فَا',
    'encletic': u'ه'
}
STOPWORDS[u'فاكم'] = {
    'word': u'فاكم',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'فَاكُمْ',
    'stem': u'فَا',
    'type': u'STOPWORD',
    'original': u'فَا',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فاكن'] = {
    'word': u'فاكن',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'فَاكُنَّ',
    'stem': u'فَا',
    'type': u'STOPWORD',
    'original': u'فَا',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فاها'] = {
    'word': u'فاها',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'فَاهَا',
    'stem': u'فَا',
    'type': u'STOPWORD',
    'original': u'فَا',
    'encletic': u'هَا'
}
STOPWORDS[u'فاهم'] = {
    'word': u'فاهم',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'فَاهُمْ',
    'stem': u'فَا',
    'type': u'STOPWORD',
    'original': u'فَا',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فاهن'] = {
    'word': u'فاهن',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'فَاهُنَّ',
    'stem': u'فَا',
    'type': u'STOPWORD',
    'original': u'فَا',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فانا'] = {
    'word': u'فانا',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'فَانَا',
    'stem': u'فَا',
    'type': u'STOPWORD',
    'original': u'فَا',
    'encletic': u'نَا'
}
STOPWORDS[u'فاكما'] = {
    'word': u'فاكما',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'فَاكُمَا',
    'stem': u'فَا',
    'type': u'STOPWORD',
    'original': u'فَا',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فاهما'] = {
    'word': u'فاهما',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'فَاهُمَا',
    'stem': u'فَا',
    'type': u'STOPWORD',
    'original': u'فَا',
    'encletic': u'هُمَا'
}
STOPWORDS[u'وفاي'] = {
    'word': u'وفاي',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَفَاِي',
    'stem': u'فَا',
    'type': u'STOPWORD',
    'original': u'فَا',
    'encletic': u'ِي'
}
STOPWORDS[u'وفاك'] = {
    'word': u'وفاك',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَفَاكَ',
    'stem': u'فَا',
    'type': u'STOPWORD',
    'original': u'فَا',
    'encletic': u'كَ'
}
STOPWORDS[u'وفاك'] = {
    'word': u'وفاك',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَفَاكِ',
    'stem': u'فَا',
    'type': u'STOPWORD',
    'original': u'فَا',
    'encletic': u'كِ'
}
STOPWORDS[u'وفاه'] = {
    'word': u'وفاه',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَفَاه',
    'stem': u'فَا',
    'type': u'STOPWORD',
    'original': u'فَا',
    'encletic': u'ه'
}
STOPWORDS[u'وفاكم'] = {
    'word': u'وفاكم',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَفَاكُمْ',
    'stem': u'فَا',
    'type': u'STOPWORD',
    'original': u'فَا',
    'encletic': u'كُمْ'
}
STOPWORDS[u'وفاكن'] = {
    'word': u'وفاكن',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَفَاكُنَّ',
    'stem': u'فَا',
    'type': u'STOPWORD',
    'original': u'فَا',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'وفاها'] = {
    'word': u'وفاها',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَفَاهَا',
    'stem': u'فَا',
    'type': u'STOPWORD',
    'original': u'فَا',
    'encletic': u'هَا'
}
STOPWORDS[u'وفاهم'] = {
    'word': u'وفاهم',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَفَاهُمْ',
    'stem': u'فَا',
    'type': u'STOPWORD',
    'original': u'فَا',
    'encletic': u'هُمْ'
}
STOPWORDS[u'وفاهن'] = {
    'word': u'وفاهن',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَفَاهُنَّ',
    'stem': u'فَا',
    'type': u'STOPWORD',
    'original': u'فَا',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'وفانا'] = {
    'word': u'وفانا',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَفَانَا',
    'stem': u'فَا',
    'type': u'STOPWORD',
    'original': u'فَا',
    'encletic': u'نَا'
}
STOPWORDS[u'وفاكما'] = {
    'word': u'وفاكما',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَفَاكُمَا',
    'stem': u'فَا',
    'type': u'STOPWORD',
    'original': u'فَا',
    'encletic': u'كُمَا'
}
STOPWORDS[u'وفاهما'] = {
    'word': u'وفاهما',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَفَاهُمَا',
    'stem': u'فَا',
    'type': u'STOPWORD',
    'original': u'فَا',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ففاي'] = {
    'word': u'ففاي',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَفَاِي',
    'stem': u'فَا',
    'type': u'STOPWORD',
    'original': u'فَا',
    'encletic': u'ِي'
}
STOPWORDS[u'ففاك'] = {
    'word': u'ففاك',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَفَاكَ',
    'stem': u'فَا',
    'type': u'STOPWORD',
    'original': u'فَا',
    'encletic': u'كَ'
}
STOPWORDS[u'ففاك'] = {
    'word': u'ففاك',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَفَاكِ',
    'stem': u'فَا',
    'type': u'STOPWORD',
    'original': u'فَا',
    'encletic': u'كِ'
}
STOPWORDS[u'ففاه'] = {
    'word': u'ففاه',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَفَاه',
    'stem': u'فَا',
    'type': u'STOPWORD',
    'original': u'فَا',
    'encletic': u'ه'
}
STOPWORDS[u'ففاكم'] = {
    'word': u'ففاكم',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَفَاكُمْ',
    'stem': u'فَا',
    'type': u'STOPWORD',
    'original': u'فَا',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ففاكن'] = {
    'word': u'ففاكن',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَفَاكُنَّ',
    'stem': u'فَا',
    'type': u'STOPWORD',
    'original': u'فَا',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ففاها'] = {
    'word': u'ففاها',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَفَاهَا',
    'stem': u'فَا',
    'type': u'STOPWORD',
    'original': u'فَا',
    'encletic': u'هَا'
}
STOPWORDS[u'ففاهم'] = {
    'word': u'ففاهم',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَفَاهُمْ',
    'stem': u'فَا',
    'type': u'STOPWORD',
    'original': u'فَا',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ففاهن'] = {
    'word': u'ففاهن',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَفَاهُنَّ',
    'stem': u'فَا',
    'type': u'STOPWORD',
    'original': u'فَا',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ففانا'] = {
    'word': u'ففانا',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَفَانَا',
    'stem': u'فَا',
    'type': u'STOPWORD',
    'original': u'فَا',
    'encletic': u'نَا'
}
STOPWORDS[u'ففاكما'] = {
    'word': u'ففاكما',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَفَاكُمَا',
    'stem': u'فَا',
    'type': u'STOPWORD',
    'original': u'فَا',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ففاهما'] = {
    'word': u'ففاهما',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَفَاهُمَا',
    'stem': u'فَا',
    'type': u'STOPWORD',
    'original': u'فَا',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أبي'] = {
    'word': u'أَبِي',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة',
    'vocalized': u'أَبِي',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u''
}
STOPWORDS[u'بأبي'] = {
    'word': u'بأبي',
    'procletic': u'-بِ',
    'tags': u'أداة:الأسماء الخمسة:مجرور',
    'vocalized': u'بِأَبِي',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u''
}
STOPWORDS[u'كأبي'] = {
    'word': u'كأبي',
    'procletic': u'-كَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور',
    'vocalized': u'كَأَبِي',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u''
}
STOPWORDS[u'لأبي'] = {
    'word': u'لأبي',
    'procletic': u'-لِ',
    'tags': u'أداة:الأسماء الخمسة:مجرور',
    'vocalized': u'لِأَبِي',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u''
}
STOPWORDS[u'وأبي'] = {
    'word': u'وأبي',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف',
    'vocalized': u'وَأَبِي',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u''
}
STOPWORDS[u'فأبي'] = {
    'word': u'فأبي',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف',
    'vocalized': u'فَأَبِي',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u''
}
STOPWORDS[u'وبأبي'] = {
    'word': u'وبأبي',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف',
    'vocalized': u'وَبِأَبِي',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u''
}
STOPWORDS[u'فبأبي'] = {
    'word': u'فبأبي',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف',
    'vocalized': u'فَبِأَبِي',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u''
}
STOPWORDS[u'وكأبي'] = {
    'word': u'وكأبي',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف',
    'vocalized': u'وَكَأَبِي',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u''
}
STOPWORDS[u'فكأبي'] = {
    'word': u'فكأبي',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف',
    'vocalized': u'فَكَأَبِي',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u''
}
STOPWORDS[u'ولأبي'] = {
    'word': u'ولأبي',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف',
    'vocalized': u'وَلِأَبِي',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u''
}
STOPWORDS[u'فلأبي'] = {
    'word': u'فلأبي',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف',
    'vocalized': u'فَلِأَبِي',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u''
}
STOPWORDS[u'أبيي'] = {
    'word': u'أبيي',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'أَبِيِي',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'ِي'
}
STOPWORDS[u'أبيك'] = {
    'word': u'أبيك',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'أَبِيكَ',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'كَ'
}
STOPWORDS[u'أبيك'] = {
    'word': u'أبيك',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'أَبِيكِ',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'كِ'
}
STOPWORDS[u'أبيه'] = {
    'word': u'أبيه',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'أَبِيه',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'ه'
}
STOPWORDS[u'أبيكم'] = {
    'word': u'أبيكم',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'أَبِيكُمْ',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أبيكن'] = {
    'word': u'أبيكن',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'أَبِيكُنَّ',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أبيها'] = {
    'word': u'أبيها',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'أَبِيهَا',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'هَا'
}
STOPWORDS[u'أبيهم'] = {
    'word': u'أبيهم',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'أَبِيهُمْ',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أبيهن'] = {
    'word': u'أبيهن',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'أَبِيهُنَّ',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أبينا'] = {
    'word': u'أبينا',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'أَبِينَا',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'نَا'
}
STOPWORDS[u'أبيكما'] = {
    'word': u'أبيكما',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'أَبِيكُمَا',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أبيهما'] = {
    'word': u'أبيهما',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'أَبِيهُمَا',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'هُمَا'
}
STOPWORDS[u'بأبيي'] = {
    'word': u'بأبيي',
    'procletic': u'-بِ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'بِأَبِيِي',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'ِي'
}
STOPWORDS[u'بأبيك'] = {
    'word': u'بأبيك',
    'procletic': u'-بِ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'بِأَبِيكَ',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'كَ'
}
STOPWORDS[u'بأبيك'] = {
    'word': u'بأبيك',
    'procletic': u'-بِ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'بِأَبِيكِ',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'كِ'
}
STOPWORDS[u'بأبيه'] = {
    'word': u'بأبيه',
    'procletic': u'-بِ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'بِأَبِيه',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'ه'
}
STOPWORDS[u'بأبيكم'] = {
    'word': u'بأبيكم',
    'procletic': u'-بِ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'بِأَبِيكُمْ',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'كُمْ'
}
STOPWORDS[u'بأبيكن'] = {
    'word': u'بأبيكن',
    'procletic': u'-بِ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'بِأَبِيكُنَّ',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'بأبيها'] = {
    'word': u'بأبيها',
    'procletic': u'-بِ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'بِأَبِيهَا',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'هَا'
}
STOPWORDS[u'بأبيهم'] = {
    'word': u'بأبيهم',
    'procletic': u'-بِ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'بِأَبِيهُمْ',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'هُمْ'
}
STOPWORDS[u'بأبيهن'] = {
    'word': u'بأبيهن',
    'procletic': u'-بِ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'بِأَبِيهُنَّ',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'بأبينا'] = {
    'word': u'بأبينا',
    'procletic': u'-بِ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'بِأَبِينَا',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'نَا'
}
STOPWORDS[u'بأبيكما'] = {
    'word': u'بأبيكما',
    'procletic': u'-بِ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'بِأَبِيكُمَا',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'كُمَا'
}
STOPWORDS[u'بأبيهما'] = {
    'word': u'بأبيهما',
    'procletic': u'-بِ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'بِأَبِيهُمَا',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'هُمَا'
}
STOPWORDS[u'كأبيي'] = {
    'word': u'كأبيي',
    'procletic': u'-كَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'كَأَبِيِي',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'ِي'
}
STOPWORDS[u'كأبيك'] = {
    'word': u'كأبيك',
    'procletic': u'-كَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'كَأَبِيكَ',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'كَ'
}
STOPWORDS[u'كأبيك'] = {
    'word': u'كأبيك',
    'procletic': u'-كَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'كَأَبِيكِ',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'كِ'
}
STOPWORDS[u'كأبيه'] = {
    'word': u'كأبيه',
    'procletic': u'-كَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'كَأَبِيه',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'ه'
}
STOPWORDS[u'كأبيكم'] = {
    'word': u'كأبيكم',
    'procletic': u'-كَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'كَأَبِيكُمْ',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'كُمْ'
}
STOPWORDS[u'كأبيكن'] = {
    'word': u'كأبيكن',
    'procletic': u'-كَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'كَأَبِيكُنَّ',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'كأبيها'] = {
    'word': u'كأبيها',
    'procletic': u'-كَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'كَأَبِيهَا',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'هَا'
}
STOPWORDS[u'كأبيهم'] = {
    'word': u'كأبيهم',
    'procletic': u'-كَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'كَأَبِيهُمْ',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'هُمْ'
}
STOPWORDS[u'كأبيهن'] = {
    'word': u'كأبيهن',
    'procletic': u'-كَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'كَأَبِيهُنَّ',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'كأبينا'] = {
    'word': u'كأبينا',
    'procletic': u'-كَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'كَأَبِينَا',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'نَا'
}
STOPWORDS[u'كأبيكما'] = {
    'word': u'كأبيكما',
    'procletic': u'-كَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'كَأَبِيكُمَا',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'كُمَا'
}
STOPWORDS[u'كأبيهما'] = {
    'word': u'كأبيهما',
    'procletic': u'-كَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'كَأَبِيهُمَا',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'هُمَا'
}
STOPWORDS[u'لأبيي'] = {
    'word': u'لأبيي',
    'procletic': u'-لِ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'لِأَبِيِي',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'ِي'
}
STOPWORDS[u'لأبيك'] = {
    'word': u'لأبيك',
    'procletic': u'-لِ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'لِأَبِيكَ',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'كَ'
}
STOPWORDS[u'لأبيك'] = {
    'word': u'لأبيك',
    'procletic': u'-لِ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'لِأَبِيكِ',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'كِ'
}
STOPWORDS[u'لأبيه'] = {
    'word': u'لأبيه',
    'procletic': u'-لِ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'لِأَبِيه',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'ه'
}
STOPWORDS[u'لأبيكم'] = {
    'word': u'لأبيكم',
    'procletic': u'-لِ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'لِأَبِيكُمْ',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'كُمْ'
}
STOPWORDS[u'لأبيكن'] = {
    'word': u'لأبيكن',
    'procletic': u'-لِ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'لِأَبِيكُنَّ',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'لأبيها'] = {
    'word': u'لأبيها',
    'procletic': u'-لِ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'لِأَبِيهَا',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'هَا'
}
STOPWORDS[u'لأبيهم'] = {
    'word': u'لأبيهم',
    'procletic': u'-لِ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'لِأَبِيهُمْ',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'هُمْ'
}
STOPWORDS[u'لأبيهن'] = {
    'word': u'لأبيهن',
    'procletic': u'-لِ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'لِأَبِيهُنَّ',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'لأبينا'] = {
    'word': u'لأبينا',
    'procletic': u'-لِ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'لِأَبِينَا',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'نَا'
}
STOPWORDS[u'لأبيكما'] = {
    'word': u'لأبيكما',
    'procletic': u'-لِ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'لِأَبِيكُمَا',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'كُمَا'
}
STOPWORDS[u'لأبيهما'] = {
    'word': u'لأبيهما',
    'procletic': u'-لِ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'لِأَبِيهُمَا',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'هُمَا'
}
STOPWORDS[u'وأبيي'] = {
    'word': u'وأبيي',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَأَبِيِي',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'ِي'
}
STOPWORDS[u'وأبيك'] = {
    'word': u'وأبيك',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَأَبِيكَ',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'كَ'
}
STOPWORDS[u'وأبيك'] = {
    'word': u'وأبيك',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَأَبِيكِ',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'كِ'
}
STOPWORDS[u'وأبيه'] = {
    'word': u'وأبيه',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَأَبِيه',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'ه'
}
STOPWORDS[u'وأبيكم'] = {
    'word': u'وأبيكم',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَأَبِيكُمْ',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'كُمْ'
}
STOPWORDS[u'وأبيكن'] = {
    'word': u'وأبيكن',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَأَبِيكُنَّ',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'وأبيها'] = {
    'word': u'وأبيها',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَأَبِيهَا',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'هَا'
}
STOPWORDS[u'وأبيهم'] = {
    'word': u'وأبيهم',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَأَبِيهُمْ',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'هُمْ'
}
STOPWORDS[u'وأبيهن'] = {
    'word': u'وأبيهن',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَأَبِيهُنَّ',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'وأبينا'] = {
    'word': u'وأبينا',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَأَبِينَا',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'نَا'
}
STOPWORDS[u'وأبيكما'] = {
    'word': u'وأبيكما',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَأَبِيكُمَا',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'كُمَا'
}
STOPWORDS[u'وأبيهما'] = {
    'word': u'وأبيهما',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَأَبِيهُمَا',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فأبيي'] = {
    'word': u'فأبيي',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَأَبِيِي',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'ِي'
}
STOPWORDS[u'فأبيك'] = {
    'word': u'فأبيك',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَأَبِيكَ',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'كَ'
}
STOPWORDS[u'فأبيك'] = {
    'word': u'فأبيك',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَأَبِيكِ',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'كِ'
}
STOPWORDS[u'فأبيه'] = {
    'word': u'فأبيه',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَأَبِيه',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'ه'
}
STOPWORDS[u'فأبيكم'] = {
    'word': u'فأبيكم',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَأَبِيكُمْ',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فأبيكن'] = {
    'word': u'فأبيكن',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَأَبِيكُنَّ',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فأبيها'] = {
    'word': u'فأبيها',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَأَبِيهَا',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'هَا'
}
STOPWORDS[u'فأبيهم'] = {
    'word': u'فأبيهم',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَأَبِيهُمْ',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فأبيهن'] = {
    'word': u'فأبيهن',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَأَبِيهُنَّ',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فأبينا'] = {
    'word': u'فأبينا',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَأَبِينَا',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'نَا'
}
STOPWORDS[u'فأبيكما'] = {
    'word': u'فأبيكما',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَأَبِيكُمَا',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فأبيهما'] = {
    'word': u'فأبيهما',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَأَبِيهُمَا',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'هُمَا'
}
STOPWORDS[u'وبأبيي'] = {
    'word': u'وبأبيي',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِأَبِيِي',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'ِي'
}
STOPWORDS[u'وبأبيك'] = {
    'word': u'وبأبيك',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِأَبِيكَ',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'كَ'
}
STOPWORDS[u'وبأبيك'] = {
    'word': u'وبأبيك',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِأَبِيكِ',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'كِ'
}
STOPWORDS[u'وبأبيه'] = {
    'word': u'وبأبيه',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِأَبِيه',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'ه'
}
STOPWORDS[u'وبأبيكم'] = {
    'word': u'وبأبيكم',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِأَبِيكُمْ',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'كُمْ'
}
STOPWORDS[u'وبأبيكن'] = {
    'word': u'وبأبيكن',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِأَبِيكُنَّ',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'وبأبيها'] = {
    'word': u'وبأبيها',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِأَبِيهَا',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'هَا'
}
STOPWORDS[u'وبأبيهم'] = {
    'word': u'وبأبيهم',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِأَبِيهُمْ',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'هُمْ'
}
STOPWORDS[u'وبأبيهن'] = {
    'word': u'وبأبيهن',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِأَبِيهُنَّ',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'وبأبينا'] = {
    'word': u'وبأبينا',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِأَبِينَا',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'نَا'
}
STOPWORDS[u'وبأبيكما'] = {
    'word': u'وبأبيكما',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِأَبِيكُمَا',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'كُمَا'
}
STOPWORDS[u'وبأبيهما'] = {
    'word': u'وبأبيهما',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِأَبِيهُمَا',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فبأبيي'] = {
    'word': u'فبأبيي',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِأَبِيِي',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'ِي'
}
STOPWORDS[u'فبأبيك'] = {
    'word': u'فبأبيك',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِأَبِيكَ',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'كَ'
}
STOPWORDS[u'فبأبيك'] = {
    'word': u'فبأبيك',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِأَبِيكِ',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'كِ'
}
STOPWORDS[u'فبأبيه'] = {
    'word': u'فبأبيه',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِأَبِيه',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'ه'
}
STOPWORDS[u'فبأبيكم'] = {
    'word': u'فبأبيكم',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِأَبِيكُمْ',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فبأبيكن'] = {
    'word': u'فبأبيكن',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِأَبِيكُنَّ',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فبأبيها'] = {
    'word': u'فبأبيها',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِأَبِيهَا',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'هَا'
}
STOPWORDS[u'فبأبيهم'] = {
    'word': u'فبأبيهم',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِأَبِيهُمْ',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فبأبيهن'] = {
    'word': u'فبأبيهن',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِأَبِيهُنَّ',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فبأبينا'] = {
    'word': u'فبأبينا',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِأَبِينَا',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'نَا'
}
STOPWORDS[u'فبأبيكما'] = {
    'word': u'فبأبيكما',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِأَبِيكُمَا',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فبأبيهما'] = {
    'word': u'فبأبيهما',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِأَبِيهُمَا',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'هُمَا'
}
STOPWORDS[u'وكأبيي'] = {
    'word': u'وكأبيي',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَأَبِيِي',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'ِي'
}
STOPWORDS[u'وكأبيك'] = {
    'word': u'وكأبيك',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَأَبِيكَ',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'كَ'
}
STOPWORDS[u'وكأبيك'] = {
    'word': u'وكأبيك',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَأَبِيكِ',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'كِ'
}
STOPWORDS[u'وكأبيه'] = {
    'word': u'وكأبيه',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَأَبِيه',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'ه'
}
STOPWORDS[u'وكأبيكم'] = {
    'word': u'وكأبيكم',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَأَبِيكُمْ',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'كُمْ'
}
STOPWORDS[u'وكأبيكن'] = {
    'word': u'وكأبيكن',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَأَبِيكُنَّ',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'وكأبيها'] = {
    'word': u'وكأبيها',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَأَبِيهَا',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'هَا'
}
STOPWORDS[u'وكأبيهم'] = {
    'word': u'وكأبيهم',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَأَبِيهُمْ',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'هُمْ'
}
STOPWORDS[u'وكأبيهن'] = {
    'word': u'وكأبيهن',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَأَبِيهُنَّ',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'وكأبينا'] = {
    'word': u'وكأبينا',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَأَبِينَا',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'نَا'
}
STOPWORDS[u'وكأبيكما'] = {
    'word': u'وكأبيكما',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَأَبِيكُمَا',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'كُمَا'
}
STOPWORDS[u'وكأبيهما'] = {
    'word': u'وكأبيهما',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَأَبِيهُمَا',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فكأبيي'] = {
    'word': u'فكأبيي',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَأَبِيِي',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'ِي'
}
STOPWORDS[u'فكأبيك'] = {
    'word': u'فكأبيك',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَأَبِيكَ',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'كَ'
}
STOPWORDS[u'فكأبيك'] = {
    'word': u'فكأبيك',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَأَبِيكِ',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'كِ'
}
STOPWORDS[u'فكأبيه'] = {
    'word': u'فكأبيه',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَأَبِيه',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'ه'
}
STOPWORDS[u'فكأبيكم'] = {
    'word': u'فكأبيكم',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَأَبِيكُمْ',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فكأبيكن'] = {
    'word': u'فكأبيكن',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَأَبِيكُنَّ',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فكأبيها'] = {
    'word': u'فكأبيها',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَأَبِيهَا',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'هَا'
}
STOPWORDS[u'فكأبيهم'] = {
    'word': u'فكأبيهم',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَأَبِيهُمْ',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فكأبيهن'] = {
    'word': u'فكأبيهن',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَأَبِيهُنَّ',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فكأبينا'] = {
    'word': u'فكأبينا',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَأَبِينَا',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'نَا'
}
STOPWORDS[u'فكأبيكما'] = {
    'word': u'فكأبيكما',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَأَبِيكُمَا',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فكأبيهما'] = {
    'word': u'فكأبيهما',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَأَبِيهُمَا',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ولأبيي'] = {
    'word': u'ولأبيي',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِأَبِيِي',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'ِي'
}
STOPWORDS[u'ولأبيك'] = {
    'word': u'ولأبيك',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِأَبِيكَ',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'كَ'
}
STOPWORDS[u'ولأبيك'] = {
    'word': u'ولأبيك',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِأَبِيكِ',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'كِ'
}
STOPWORDS[u'ولأبيه'] = {
    'word': u'ولأبيه',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِأَبِيه',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'ه'
}
STOPWORDS[u'ولأبيكم'] = {
    'word': u'ولأبيكم',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِأَبِيكُمْ',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ولأبيكن'] = {
    'word': u'ولأبيكن',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِأَبِيكُنَّ',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ولأبيها'] = {
    'word': u'ولأبيها',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِأَبِيهَا',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'هَا'
}
STOPWORDS[u'ولأبيهم'] = {
    'word': u'ولأبيهم',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِأَبِيهُمْ',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ولأبيهن'] = {
    'word': u'ولأبيهن',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِأَبِيهُنَّ',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ولأبينا'] = {
    'word': u'ولأبينا',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِأَبِينَا',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'نَا'
}
STOPWORDS[u'ولأبيكما'] = {
    'word': u'ولأبيكما',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِأَبِيكُمَا',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ولأبيهما'] = {
    'word': u'ولأبيهما',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِأَبِيهُمَا',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فلأبيي'] = {
    'word': u'فلأبيي',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِأَبِيِي',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'ِي'
}
STOPWORDS[u'فلأبيك'] = {
    'word': u'فلأبيك',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِأَبِيكَ',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'كَ'
}
STOPWORDS[u'فلأبيك'] = {
    'word': u'فلأبيك',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِأَبِيكِ',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'كِ'
}
STOPWORDS[u'فلأبيه'] = {
    'word': u'فلأبيه',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِأَبِيه',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'ه'
}
STOPWORDS[u'فلأبيكم'] = {
    'word': u'فلأبيكم',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِأَبِيكُمْ',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فلأبيكن'] = {
    'word': u'فلأبيكن',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِأَبِيكُنَّ',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فلأبيها'] = {
    'word': u'فلأبيها',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِأَبِيهَا',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'هَا'
}
STOPWORDS[u'فلأبيهم'] = {
    'word': u'فلأبيهم',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِأَبِيهُمْ',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فلأبيهن'] = {
    'word': u'فلأبيهن',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِأَبِيهُنَّ',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فلأبينا'] = {
    'word': u'فلأبينا',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِأَبِينَا',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'نَا'
}
STOPWORDS[u'فلأبيكما'] = {
    'word': u'فلأبيكما',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِأَبِيكُمَا',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فلأبيهما'] = {
    'word': u'فلأبيهما',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِأَبِيهُمَا',
    'stem': u'أَبِي',
    'type': u'STOPWORD',
    'original': u'أَبِي',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أخي'] = {
    'word': u'أَخِي',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة',
    'vocalized': u'أَخِي',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u''
}
STOPWORDS[u'بأخي'] = {
    'word': u'بأخي',
    'procletic': u'-بِ',
    'tags': u'أداة:الأسماء الخمسة:مجرور',
    'vocalized': u'بِأَخِي',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u''
}
STOPWORDS[u'كأخي'] = {
    'word': u'كأخي',
    'procletic': u'-كَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور',
    'vocalized': u'كَأَخِي',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u''
}
STOPWORDS[u'لأخي'] = {
    'word': u'لأخي',
    'procletic': u'-لِ',
    'tags': u'أداة:الأسماء الخمسة:مجرور',
    'vocalized': u'لِأَخِي',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u''
}
STOPWORDS[u'وأخي'] = {
    'word': u'وأخي',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف',
    'vocalized': u'وَأَخِي',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u''
}
STOPWORDS[u'فأخي'] = {
    'word': u'فأخي',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف',
    'vocalized': u'فَأَخِي',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u''
}
STOPWORDS[u'وبأخي'] = {
    'word': u'وبأخي',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف',
    'vocalized': u'وَبِأَخِي',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u''
}
STOPWORDS[u'فبأخي'] = {
    'word': u'فبأخي',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف',
    'vocalized': u'فَبِأَخِي',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u''
}
STOPWORDS[u'وكأخي'] = {
    'word': u'وكأخي',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف',
    'vocalized': u'وَكَأَخِي',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u''
}
STOPWORDS[u'فكأخي'] = {
    'word': u'فكأخي',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف',
    'vocalized': u'فَكَأَخِي',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u''
}
STOPWORDS[u'ولأخي'] = {
    'word': u'ولأخي',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف',
    'vocalized': u'وَلِأَخِي',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u''
}
STOPWORDS[u'فلأخي'] = {
    'word': u'فلأخي',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف',
    'vocalized': u'فَلِأَخِي',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u''
}
STOPWORDS[u'أخيي'] = {
    'word': u'أخيي',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'أَخِيِي',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'ِي'
}
STOPWORDS[u'أخيك'] = {
    'word': u'أخيك',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'أَخِيكَ',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'كَ'
}
STOPWORDS[u'أخيك'] = {
    'word': u'أخيك',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'أَخِيكِ',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'كِ'
}
STOPWORDS[u'أخيه'] = {
    'word': u'أخيه',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'أَخِيه',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'ه'
}
STOPWORDS[u'أخيكم'] = {
    'word': u'أخيكم',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'أَخِيكُمْ',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أخيكن'] = {
    'word': u'أخيكن',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'أَخِيكُنَّ',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أخيها'] = {
    'word': u'أخيها',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'أَخِيهَا',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'هَا'
}
STOPWORDS[u'أخيهم'] = {
    'word': u'أخيهم',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'أَخِيهُمْ',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أخيهن'] = {
    'word': u'أخيهن',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'أَخِيهُنَّ',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أخينا'] = {
    'word': u'أخينا',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'أَخِينَا',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'نَا'
}
STOPWORDS[u'أخيكما'] = {
    'word': u'أخيكما',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'أَخِيكُمَا',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أخيهما'] = {
    'word': u'أخيهما',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'أَخِيهُمَا',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'هُمَا'
}
STOPWORDS[u'بأخيي'] = {
    'word': u'بأخيي',
    'procletic': u'-بِ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'بِأَخِيِي',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'ِي'
}
STOPWORDS[u'بأخيك'] = {
    'word': u'بأخيك',
    'procletic': u'-بِ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'بِأَخِيكَ',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'كَ'
}
STOPWORDS[u'بأخيك'] = {
    'word': u'بأخيك',
    'procletic': u'-بِ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'بِأَخِيكِ',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'كِ'
}
STOPWORDS[u'بأخيه'] = {
    'word': u'بأخيه',
    'procletic': u'-بِ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'بِأَخِيه',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'ه'
}
STOPWORDS[u'بأخيكم'] = {
    'word': u'بأخيكم',
    'procletic': u'-بِ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'بِأَخِيكُمْ',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'كُمْ'
}
STOPWORDS[u'بأخيكن'] = {
    'word': u'بأخيكن',
    'procletic': u'-بِ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'بِأَخِيكُنَّ',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'بأخيها'] = {
    'word': u'بأخيها',
    'procletic': u'-بِ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'بِأَخِيهَا',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'هَا'
}
STOPWORDS[u'بأخيهم'] = {
    'word': u'بأخيهم',
    'procletic': u'-بِ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'بِأَخِيهُمْ',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'هُمْ'
}
STOPWORDS[u'بأخيهن'] = {
    'word': u'بأخيهن',
    'procletic': u'-بِ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'بِأَخِيهُنَّ',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'بأخينا'] = {
    'word': u'بأخينا',
    'procletic': u'-بِ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'بِأَخِينَا',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'نَا'
}
STOPWORDS[u'بأخيكما'] = {
    'word': u'بأخيكما',
    'procletic': u'-بِ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'بِأَخِيكُمَا',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'كُمَا'
}
STOPWORDS[u'بأخيهما'] = {
    'word': u'بأخيهما',
    'procletic': u'-بِ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'بِأَخِيهُمَا',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'هُمَا'
}
STOPWORDS[u'كأخيي'] = {
    'word': u'كأخيي',
    'procletic': u'-كَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'كَأَخِيِي',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'ِي'
}
STOPWORDS[u'كأخيك'] = {
    'word': u'كأخيك',
    'procletic': u'-كَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'كَأَخِيكَ',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'كَ'
}
STOPWORDS[u'كأخيك'] = {
    'word': u'كأخيك',
    'procletic': u'-كَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'كَأَخِيكِ',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'كِ'
}
STOPWORDS[u'كأخيه'] = {
    'word': u'كأخيه',
    'procletic': u'-كَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'كَأَخِيه',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'ه'
}
STOPWORDS[u'كأخيكم'] = {
    'word': u'كأخيكم',
    'procletic': u'-كَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'كَأَخِيكُمْ',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'كُمْ'
}
STOPWORDS[u'كأخيكن'] = {
    'word': u'كأخيكن',
    'procletic': u'-كَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'كَأَخِيكُنَّ',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'كأخيها'] = {
    'word': u'كأخيها',
    'procletic': u'-كَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'كَأَخِيهَا',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'هَا'
}
STOPWORDS[u'كأخيهم'] = {
    'word': u'كأخيهم',
    'procletic': u'-كَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'كَأَخِيهُمْ',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'هُمْ'
}
STOPWORDS[u'كأخيهن'] = {
    'word': u'كأخيهن',
    'procletic': u'-كَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'كَأَخِيهُنَّ',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'كأخينا'] = {
    'word': u'كأخينا',
    'procletic': u'-كَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'كَأَخِينَا',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'نَا'
}
STOPWORDS[u'كأخيكما'] = {
    'word': u'كأخيكما',
    'procletic': u'-كَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'كَأَخِيكُمَا',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'كُمَا'
}
STOPWORDS[u'كأخيهما'] = {
    'word': u'كأخيهما',
    'procletic': u'-كَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'كَأَخِيهُمَا',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'هُمَا'
}
STOPWORDS[u'لأخيي'] = {
    'word': u'لأخيي',
    'procletic': u'-لِ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'لِأَخِيِي',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'ِي'
}
STOPWORDS[u'لأخيك'] = {
    'word': u'لأخيك',
    'procletic': u'-لِ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'لِأَخِيكَ',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'كَ'
}
STOPWORDS[u'لأخيك'] = {
    'word': u'لأخيك',
    'procletic': u'-لِ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'لِأَخِيكِ',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'كِ'
}
STOPWORDS[u'لأخيه'] = {
    'word': u'لأخيه',
    'procletic': u'-لِ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'لِأَخِيه',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'ه'
}
STOPWORDS[u'لأخيكم'] = {
    'word': u'لأخيكم',
    'procletic': u'-لِ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'لِأَخِيكُمْ',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'كُمْ'
}
STOPWORDS[u'لأخيكن'] = {
    'word': u'لأخيكن',
    'procletic': u'-لِ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'لِأَخِيكُنَّ',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'لأخيها'] = {
    'word': u'لأخيها',
    'procletic': u'-لِ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'لِأَخِيهَا',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'هَا'
}
STOPWORDS[u'لأخيهم'] = {
    'word': u'لأخيهم',
    'procletic': u'-لِ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'لِأَخِيهُمْ',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'هُمْ'
}
STOPWORDS[u'لأخيهن'] = {
    'word': u'لأخيهن',
    'procletic': u'-لِ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'لِأَخِيهُنَّ',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'لأخينا'] = {
    'word': u'لأخينا',
    'procletic': u'-لِ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'لِأَخِينَا',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'نَا'
}
STOPWORDS[u'لأخيكما'] = {
    'word': u'لأخيكما',
    'procletic': u'-لِ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'لِأَخِيكُمَا',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'كُمَا'
}
STOPWORDS[u'لأخيهما'] = {
    'word': u'لأخيهما',
    'procletic': u'-لِ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'لِأَخِيهُمَا',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'هُمَا'
}
STOPWORDS[u'وأخيي'] = {
    'word': u'وأخيي',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَأَخِيِي',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'ِي'
}
STOPWORDS[u'وأخيك'] = {
    'word': u'وأخيك',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَأَخِيكَ',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'كَ'
}
STOPWORDS[u'وأخيك'] = {
    'word': u'وأخيك',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَأَخِيكِ',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'كِ'
}
STOPWORDS[u'وأخيه'] = {
    'word': u'وأخيه',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَأَخِيه',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'ه'
}
STOPWORDS[u'وأخيكم'] = {
    'word': u'وأخيكم',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَأَخِيكُمْ',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'كُمْ'
}
STOPWORDS[u'وأخيكن'] = {
    'word': u'وأخيكن',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَأَخِيكُنَّ',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'وأخيها'] = {
    'word': u'وأخيها',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَأَخِيهَا',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'هَا'
}
STOPWORDS[u'وأخيهم'] = {
    'word': u'وأخيهم',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَأَخِيهُمْ',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'هُمْ'
}
STOPWORDS[u'وأخيهن'] = {
    'word': u'وأخيهن',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَأَخِيهُنَّ',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'وأخينا'] = {
    'word': u'وأخينا',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَأَخِينَا',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'نَا'
}
STOPWORDS[u'وأخيكما'] = {
    'word': u'وأخيكما',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَأَخِيكُمَا',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'كُمَا'
}
STOPWORDS[u'وأخيهما'] = {
    'word': u'وأخيهما',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَأَخِيهُمَا',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فأخيي'] = {
    'word': u'فأخيي',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَأَخِيِي',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'ِي'
}
STOPWORDS[u'فأخيك'] = {
    'word': u'فأخيك',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَأَخِيكَ',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'كَ'
}
STOPWORDS[u'فأخيك'] = {
    'word': u'فأخيك',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَأَخِيكِ',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'كِ'
}
STOPWORDS[u'فأخيه'] = {
    'word': u'فأخيه',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَأَخِيه',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'ه'
}
STOPWORDS[u'فأخيكم'] = {
    'word': u'فأخيكم',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَأَخِيكُمْ',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فأخيكن'] = {
    'word': u'فأخيكن',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَأَخِيكُنَّ',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فأخيها'] = {
    'word': u'فأخيها',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَأَخِيهَا',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'هَا'
}
STOPWORDS[u'فأخيهم'] = {
    'word': u'فأخيهم',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَأَخِيهُمْ',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فأخيهن'] = {
    'word': u'فأخيهن',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَأَخِيهُنَّ',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فأخينا'] = {
    'word': u'فأخينا',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَأَخِينَا',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'نَا'
}
STOPWORDS[u'فأخيكما'] = {
    'word': u'فأخيكما',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَأَخِيكُمَا',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فأخيهما'] = {
    'word': u'فأخيهما',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَأَخِيهُمَا',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'هُمَا'
}
STOPWORDS[u'وبأخيي'] = {
    'word': u'وبأخيي',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِأَخِيِي',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'ِي'
}
STOPWORDS[u'وبأخيك'] = {
    'word': u'وبأخيك',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِأَخِيكَ',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'كَ'
}
STOPWORDS[u'وبأخيك'] = {
    'word': u'وبأخيك',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِأَخِيكِ',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'كِ'
}
STOPWORDS[u'وبأخيه'] = {
    'word': u'وبأخيه',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِأَخِيه',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'ه'
}
STOPWORDS[u'وبأخيكم'] = {
    'word': u'وبأخيكم',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِأَخِيكُمْ',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'كُمْ'
}
STOPWORDS[u'وبأخيكن'] = {
    'word': u'وبأخيكن',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِأَخِيكُنَّ',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'وبأخيها'] = {
    'word': u'وبأخيها',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِأَخِيهَا',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'هَا'
}
STOPWORDS[u'وبأخيهم'] = {
    'word': u'وبأخيهم',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِأَخِيهُمْ',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'هُمْ'
}
STOPWORDS[u'وبأخيهن'] = {
    'word': u'وبأخيهن',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِأَخِيهُنَّ',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'وبأخينا'] = {
    'word': u'وبأخينا',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِأَخِينَا',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'نَا'
}
STOPWORDS[u'وبأخيكما'] = {
    'word': u'وبأخيكما',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِأَخِيكُمَا',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'كُمَا'
}
STOPWORDS[u'وبأخيهما'] = {
    'word': u'وبأخيهما',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِأَخِيهُمَا',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فبأخيي'] = {
    'word': u'فبأخيي',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِأَخِيِي',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'ِي'
}
STOPWORDS[u'فبأخيك'] = {
    'word': u'فبأخيك',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِأَخِيكَ',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'كَ'
}
STOPWORDS[u'فبأخيك'] = {
    'word': u'فبأخيك',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِأَخِيكِ',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'كِ'
}
STOPWORDS[u'فبأخيه'] = {
    'word': u'فبأخيه',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِأَخِيه',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'ه'
}
STOPWORDS[u'فبأخيكم'] = {
    'word': u'فبأخيكم',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِأَخِيكُمْ',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فبأخيكن'] = {
    'word': u'فبأخيكن',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِأَخِيكُنَّ',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فبأخيها'] = {
    'word': u'فبأخيها',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِأَخِيهَا',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'هَا'
}
STOPWORDS[u'فبأخيهم'] = {
    'word': u'فبأخيهم',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِأَخِيهُمْ',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فبأخيهن'] = {
    'word': u'فبأخيهن',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِأَخِيهُنَّ',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فبأخينا'] = {
    'word': u'فبأخينا',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِأَخِينَا',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'نَا'
}
STOPWORDS[u'فبأخيكما'] = {
    'word': u'فبأخيكما',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِأَخِيكُمَا',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فبأخيهما'] = {
    'word': u'فبأخيهما',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِأَخِيهُمَا',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'هُمَا'
}
STOPWORDS[u'وكأخيي'] = {
    'word': u'وكأخيي',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَأَخِيِي',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'ِي'
}
STOPWORDS[u'وكأخيك'] = {
    'word': u'وكأخيك',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَأَخِيكَ',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'كَ'
}
STOPWORDS[u'وكأخيك'] = {
    'word': u'وكأخيك',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَأَخِيكِ',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'كِ'
}
STOPWORDS[u'وكأخيه'] = {
    'word': u'وكأخيه',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَأَخِيه',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'ه'
}
STOPWORDS[u'وكأخيكم'] = {
    'word': u'وكأخيكم',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَأَخِيكُمْ',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'كُمْ'
}
STOPWORDS[u'وكأخيكن'] = {
    'word': u'وكأخيكن',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَأَخِيكُنَّ',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'وكأخيها'] = {
    'word': u'وكأخيها',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَأَخِيهَا',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'هَا'
}
STOPWORDS[u'وكأخيهم'] = {
    'word': u'وكأخيهم',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَأَخِيهُمْ',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'هُمْ'
}
STOPWORDS[u'وكأخيهن'] = {
    'word': u'وكأخيهن',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَأَخِيهُنَّ',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'وكأخينا'] = {
    'word': u'وكأخينا',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَأَخِينَا',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'نَا'
}
STOPWORDS[u'وكأخيكما'] = {
    'word': u'وكأخيكما',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَأَخِيكُمَا',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'كُمَا'
}
STOPWORDS[u'وكأخيهما'] = {
    'word': u'وكأخيهما',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَأَخِيهُمَا',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فكأخيي'] = {
    'word': u'فكأخيي',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَأَخِيِي',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'ِي'
}
STOPWORDS[u'فكأخيك'] = {
    'word': u'فكأخيك',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَأَخِيكَ',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'كَ'
}
STOPWORDS[u'فكأخيك'] = {
    'word': u'فكأخيك',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَأَخِيكِ',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'كِ'
}
STOPWORDS[u'فكأخيه'] = {
    'word': u'فكأخيه',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَأَخِيه',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'ه'
}
STOPWORDS[u'فكأخيكم'] = {
    'word': u'فكأخيكم',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَأَخِيكُمْ',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فكأخيكن'] = {
    'word': u'فكأخيكن',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَأَخِيكُنَّ',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فكأخيها'] = {
    'word': u'فكأخيها',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَأَخِيهَا',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'هَا'
}
STOPWORDS[u'فكأخيهم'] = {
    'word': u'فكأخيهم',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَأَخِيهُمْ',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فكأخيهن'] = {
    'word': u'فكأخيهن',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَأَخِيهُنَّ',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فكأخينا'] = {
    'word': u'فكأخينا',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَأَخِينَا',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'نَا'
}
STOPWORDS[u'فكأخيكما'] = {
    'word': u'فكأخيكما',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَأَخِيكُمَا',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فكأخيهما'] = {
    'word': u'فكأخيهما',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَأَخِيهُمَا',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ولأخيي'] = {
    'word': u'ولأخيي',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِأَخِيِي',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'ِي'
}
STOPWORDS[u'ولأخيك'] = {
    'word': u'ولأخيك',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِأَخِيكَ',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'كَ'
}
STOPWORDS[u'ولأخيك'] = {
    'word': u'ولأخيك',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِأَخِيكِ',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'كِ'
}
STOPWORDS[u'ولأخيه'] = {
    'word': u'ولأخيه',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِأَخِيه',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'ه'
}
STOPWORDS[u'ولأخيكم'] = {
    'word': u'ولأخيكم',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِأَخِيكُمْ',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ولأخيكن'] = {
    'word': u'ولأخيكن',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِأَخِيكُنَّ',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ولأخيها'] = {
    'word': u'ولأخيها',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِأَخِيهَا',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'هَا'
}
STOPWORDS[u'ولأخيهم'] = {
    'word': u'ولأخيهم',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِأَخِيهُمْ',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ولأخيهن'] = {
    'word': u'ولأخيهن',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِأَخِيهُنَّ',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ولأخينا'] = {
    'word': u'ولأخينا',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِأَخِينَا',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'نَا'
}
STOPWORDS[u'ولأخيكما'] = {
    'word': u'ولأخيكما',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِأَخِيكُمَا',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ولأخيهما'] = {
    'word': u'ولأخيهما',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِأَخِيهُمَا',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فلأخيي'] = {
    'word': u'فلأخيي',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِأَخِيِي',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'ِي'
}
STOPWORDS[u'فلأخيك'] = {
    'word': u'فلأخيك',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِأَخِيكَ',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'كَ'
}
STOPWORDS[u'فلأخيك'] = {
    'word': u'فلأخيك',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِأَخِيكِ',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'كِ'
}
STOPWORDS[u'فلأخيه'] = {
    'word': u'فلأخيه',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِأَخِيه',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'ه'
}
STOPWORDS[u'فلأخيكم'] = {
    'word': u'فلأخيكم',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِأَخِيكُمْ',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فلأخيكن'] = {
    'word': u'فلأخيكن',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِأَخِيكُنَّ',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فلأخيها'] = {
    'word': u'فلأخيها',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِأَخِيهَا',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'هَا'
}
STOPWORDS[u'فلأخيهم'] = {
    'word': u'فلأخيهم',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِأَخِيهُمْ',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فلأخيهن'] = {
    'word': u'فلأخيهن',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِأَخِيهُنَّ',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فلأخينا'] = {
    'word': u'فلأخينا',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِأَخِينَا',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'نَا'
}
STOPWORDS[u'فلأخيكما'] = {
    'word': u'فلأخيكما',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِأَخِيكُمَا',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فلأخيهما'] = {
    'word': u'فلأخيهما',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِأَخِيهُمَا',
    'stem': u'أَخِي',
    'type': u'STOPWORD',
    'original': u'أَخِي',
    'encletic': u'هُمَا'
}
STOPWORDS[u'حمي'] = {
    'word': u'حَمِي',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة',
    'vocalized': u'حَمِي',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u''
}
STOPWORDS[u'بحمي'] = {
    'word': u'بحمي',
    'procletic': u'-بِ',
    'tags': u'أداة:الأسماء الخمسة:مجرور',
    'vocalized': u'بِحَمِي',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u''
}
STOPWORDS[u'كحمي'] = {
    'word': u'كحمي',
    'procletic': u'-كَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور',
    'vocalized': u'كَحَمِي',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u''
}
STOPWORDS[u'لحمي'] = {
    'word': u'لحمي',
    'procletic': u'-لِ',
    'tags': u'أداة:الأسماء الخمسة:مجرور',
    'vocalized': u'لِحَمِي',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u''
}
STOPWORDS[u'وحمي'] = {
    'word': u'وحمي',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف',
    'vocalized': u'وَحَمِي',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u''
}
STOPWORDS[u'فحمي'] = {
    'word': u'فحمي',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف',
    'vocalized': u'فَحَمِي',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u''
}
STOPWORDS[u'وبحمي'] = {
    'word': u'وبحمي',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف',
    'vocalized': u'وَبِحَمِي',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u''
}
STOPWORDS[u'فبحمي'] = {
    'word': u'فبحمي',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف',
    'vocalized': u'فَبِحَمِي',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u''
}
STOPWORDS[u'وكحمي'] = {
    'word': u'وكحمي',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف',
    'vocalized': u'وَكَحَمِي',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u''
}
STOPWORDS[u'فكحمي'] = {
    'word': u'فكحمي',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف',
    'vocalized': u'فَكَحَمِي',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u''
}
STOPWORDS[u'ولحمي'] = {
    'word': u'ولحمي',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف',
    'vocalized': u'وَلِحَمِي',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u''
}
STOPWORDS[u'فلحمي'] = {
    'word': u'فلحمي',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف',
    'vocalized': u'فَلِحَمِي',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u''
}
STOPWORDS[u'حميي'] = {
    'word': u'حميي',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'حَمِيِي',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'ِي'
}
STOPWORDS[u'حميك'] = {
    'word': u'حميك',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'حَمِيكَ',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'كَ'
}
STOPWORDS[u'حميك'] = {
    'word': u'حميك',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'حَمِيكِ',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'كِ'
}
STOPWORDS[u'حميه'] = {
    'word': u'حميه',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'حَمِيه',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'ه'
}
STOPWORDS[u'حميكم'] = {
    'word': u'حميكم',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'حَمِيكُمْ',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'كُمْ'
}
STOPWORDS[u'حميكن'] = {
    'word': u'حميكن',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'حَمِيكُنَّ',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'حميها'] = {
    'word': u'حميها',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'حَمِيهَا',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'هَا'
}
STOPWORDS[u'حميهم'] = {
    'word': u'حميهم',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'حَمِيهُمْ',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'هُمْ'
}
STOPWORDS[u'حميهن'] = {
    'word': u'حميهن',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'حَمِيهُنَّ',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'حمينا'] = {
    'word': u'حمينا',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'حَمِينَا',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'نَا'
}
STOPWORDS[u'حميكما'] = {
    'word': u'حميكما',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'حَمِيكُمَا',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'كُمَا'
}
STOPWORDS[u'حميهما'] = {
    'word': u'حميهما',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'حَمِيهُمَا',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'هُمَا'
}
STOPWORDS[u'بحميي'] = {
    'word': u'بحميي',
    'procletic': u'-بِ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'بِحَمِيِي',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'ِي'
}
STOPWORDS[u'بحميك'] = {
    'word': u'بحميك',
    'procletic': u'-بِ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'بِحَمِيكَ',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'كَ'
}
STOPWORDS[u'بحميك'] = {
    'word': u'بحميك',
    'procletic': u'-بِ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'بِحَمِيكِ',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'كِ'
}
STOPWORDS[u'بحميه'] = {
    'word': u'بحميه',
    'procletic': u'-بِ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'بِحَمِيه',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'ه'
}
STOPWORDS[u'بحميكم'] = {
    'word': u'بحميكم',
    'procletic': u'-بِ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'بِحَمِيكُمْ',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'كُمْ'
}
STOPWORDS[u'بحميكن'] = {
    'word': u'بحميكن',
    'procletic': u'-بِ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'بِحَمِيكُنَّ',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'بحميها'] = {
    'word': u'بحميها',
    'procletic': u'-بِ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'بِحَمِيهَا',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'هَا'
}
STOPWORDS[u'بحميهم'] = {
    'word': u'بحميهم',
    'procletic': u'-بِ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'بِحَمِيهُمْ',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'هُمْ'
}
STOPWORDS[u'بحميهن'] = {
    'word': u'بحميهن',
    'procletic': u'-بِ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'بِحَمِيهُنَّ',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'بحمينا'] = {
    'word': u'بحمينا',
    'procletic': u'-بِ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'بِحَمِينَا',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'نَا'
}
STOPWORDS[u'بحميكما'] = {
    'word': u'بحميكما',
    'procletic': u'-بِ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'بِحَمِيكُمَا',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'كُمَا'
}
STOPWORDS[u'بحميهما'] = {
    'word': u'بحميهما',
    'procletic': u'-بِ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'بِحَمِيهُمَا',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'هُمَا'
}
STOPWORDS[u'كحميي'] = {
    'word': u'كحميي',
    'procletic': u'-كَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'كَحَمِيِي',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'ِي'
}
STOPWORDS[u'كحميك'] = {
    'word': u'كحميك',
    'procletic': u'-كَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'كَحَمِيكَ',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'كَ'
}
STOPWORDS[u'كحميك'] = {
    'word': u'كحميك',
    'procletic': u'-كَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'كَحَمِيكِ',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'كِ'
}
STOPWORDS[u'كحميه'] = {
    'word': u'كحميه',
    'procletic': u'-كَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'كَحَمِيه',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'ه'
}
STOPWORDS[u'كحميكم'] = {
    'word': u'كحميكم',
    'procletic': u'-كَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'كَحَمِيكُمْ',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'كُمْ'
}
STOPWORDS[u'كحميكن'] = {
    'word': u'كحميكن',
    'procletic': u'-كَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'كَحَمِيكُنَّ',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'كحميها'] = {
    'word': u'كحميها',
    'procletic': u'-كَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'كَحَمِيهَا',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'هَا'
}
STOPWORDS[u'كحميهم'] = {
    'word': u'كحميهم',
    'procletic': u'-كَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'كَحَمِيهُمْ',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'هُمْ'
}
STOPWORDS[u'كحميهن'] = {
    'word': u'كحميهن',
    'procletic': u'-كَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'كَحَمِيهُنَّ',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'كحمينا'] = {
    'word': u'كحمينا',
    'procletic': u'-كَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'كَحَمِينَا',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'نَا'
}
STOPWORDS[u'كحميكما'] = {
    'word': u'كحميكما',
    'procletic': u'-كَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'كَحَمِيكُمَا',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'كُمَا'
}
STOPWORDS[u'كحميهما'] = {
    'word': u'كحميهما',
    'procletic': u'-كَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'كَحَمِيهُمَا',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'هُمَا'
}
STOPWORDS[u'لحميي'] = {
    'word': u'لحميي',
    'procletic': u'-لِ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'لِحَمِيِي',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'ِي'
}
STOPWORDS[u'لحميك'] = {
    'word': u'لحميك',
    'procletic': u'-لِ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'لِحَمِيكَ',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'كَ'
}
STOPWORDS[u'لحميك'] = {
    'word': u'لحميك',
    'procletic': u'-لِ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'لِحَمِيكِ',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'كِ'
}
STOPWORDS[u'لحميه'] = {
    'word': u'لحميه',
    'procletic': u'-لِ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'لِحَمِيه',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'ه'
}
STOPWORDS[u'لحميكم'] = {
    'word': u'لحميكم',
    'procletic': u'-لِ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'لِحَمِيكُمْ',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'كُمْ'
}
STOPWORDS[u'لحميكن'] = {
    'word': u'لحميكن',
    'procletic': u'-لِ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'لِحَمِيكُنَّ',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'لحميها'] = {
    'word': u'لحميها',
    'procletic': u'-لِ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'لِحَمِيهَا',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'هَا'
}
STOPWORDS[u'لحميهم'] = {
    'word': u'لحميهم',
    'procletic': u'-لِ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'لِحَمِيهُمْ',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'هُمْ'
}
STOPWORDS[u'لحميهن'] = {
    'word': u'لحميهن',
    'procletic': u'-لِ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'لِحَمِيهُنَّ',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'لحمينا'] = {
    'word': u'لحمينا',
    'procletic': u'-لِ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'لِحَمِينَا',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'نَا'
}
STOPWORDS[u'لحميكما'] = {
    'word': u'لحميكما',
    'procletic': u'-لِ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'لِحَمِيكُمَا',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'كُمَا'
}
STOPWORDS[u'لحميهما'] = {
    'word': u'لحميهما',
    'procletic': u'-لِ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'لِحَمِيهُمَا',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'هُمَا'
}
STOPWORDS[u'وحميي'] = {
    'word': u'وحميي',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَحَمِيِي',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'ِي'
}
STOPWORDS[u'وحميك'] = {
    'word': u'وحميك',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَحَمِيكَ',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'كَ'
}
STOPWORDS[u'وحميك'] = {
    'word': u'وحميك',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَحَمِيكِ',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'كِ'
}
STOPWORDS[u'وحميه'] = {
    'word': u'وحميه',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَحَمِيه',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'ه'
}
STOPWORDS[u'وحميكم'] = {
    'word': u'وحميكم',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَحَمِيكُمْ',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'كُمْ'
}
STOPWORDS[u'وحميكن'] = {
    'word': u'وحميكن',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَحَمِيكُنَّ',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'وحميها'] = {
    'word': u'وحميها',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَحَمِيهَا',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'هَا'
}
STOPWORDS[u'وحميهم'] = {
    'word': u'وحميهم',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَحَمِيهُمْ',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'هُمْ'
}
STOPWORDS[u'وحميهن'] = {
    'word': u'وحميهن',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَحَمِيهُنَّ',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'وحمينا'] = {
    'word': u'وحمينا',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَحَمِينَا',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'نَا'
}
STOPWORDS[u'وحميكما'] = {
    'word': u'وحميكما',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَحَمِيكُمَا',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'كُمَا'
}
STOPWORDS[u'وحميهما'] = {
    'word': u'وحميهما',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَحَمِيهُمَا',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فحميي'] = {
    'word': u'فحميي',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَحَمِيِي',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'ِي'
}
STOPWORDS[u'فحميك'] = {
    'word': u'فحميك',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَحَمِيكَ',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'كَ'
}
STOPWORDS[u'فحميك'] = {
    'word': u'فحميك',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَحَمِيكِ',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'كِ'
}
STOPWORDS[u'فحميه'] = {
    'word': u'فحميه',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَحَمِيه',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'ه'
}
STOPWORDS[u'فحميكم'] = {
    'word': u'فحميكم',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَحَمِيكُمْ',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فحميكن'] = {
    'word': u'فحميكن',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَحَمِيكُنَّ',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فحميها'] = {
    'word': u'فحميها',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَحَمِيهَا',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'هَا'
}
STOPWORDS[u'فحميهم'] = {
    'word': u'فحميهم',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَحَمِيهُمْ',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فحميهن'] = {
    'word': u'فحميهن',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَحَمِيهُنَّ',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فحمينا'] = {
    'word': u'فحمينا',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَحَمِينَا',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'نَا'
}
STOPWORDS[u'فحميكما'] = {
    'word': u'فحميكما',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَحَمِيكُمَا',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فحميهما'] = {
    'word': u'فحميهما',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَحَمِيهُمَا',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'هُمَا'
}
STOPWORDS[u'وبحميي'] = {
    'word': u'وبحميي',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِحَمِيِي',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'ِي'
}
STOPWORDS[u'وبحميك'] = {
    'word': u'وبحميك',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِحَمِيكَ',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'كَ'
}
STOPWORDS[u'وبحميك'] = {
    'word': u'وبحميك',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِحَمِيكِ',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'كِ'
}
STOPWORDS[u'وبحميه'] = {
    'word': u'وبحميه',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِحَمِيه',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'ه'
}
STOPWORDS[u'وبحميكم'] = {
    'word': u'وبحميكم',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِحَمِيكُمْ',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'كُمْ'
}
STOPWORDS[u'وبحميكن'] = {
    'word': u'وبحميكن',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِحَمِيكُنَّ',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'وبحميها'] = {
    'word': u'وبحميها',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِحَمِيهَا',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'هَا'
}
STOPWORDS[u'وبحميهم'] = {
    'word': u'وبحميهم',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِحَمِيهُمْ',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'هُمْ'
}
STOPWORDS[u'وبحميهن'] = {
    'word': u'وبحميهن',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِحَمِيهُنَّ',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'وبحمينا'] = {
    'word': u'وبحمينا',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِحَمِينَا',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'نَا'
}
STOPWORDS[u'وبحميكما'] = {
    'word': u'وبحميكما',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِحَمِيكُمَا',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'كُمَا'
}
STOPWORDS[u'وبحميهما'] = {
    'word': u'وبحميهما',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِحَمِيهُمَا',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فبحميي'] = {
    'word': u'فبحميي',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِحَمِيِي',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'ِي'
}
STOPWORDS[u'فبحميك'] = {
    'word': u'فبحميك',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِحَمِيكَ',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'كَ'
}
STOPWORDS[u'فبحميك'] = {
    'word': u'فبحميك',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِحَمِيكِ',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'كِ'
}
STOPWORDS[u'فبحميه'] = {
    'word': u'فبحميه',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِحَمِيه',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'ه'
}
STOPWORDS[u'فبحميكم'] = {
    'word': u'فبحميكم',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِحَمِيكُمْ',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فبحميكن'] = {
    'word': u'فبحميكن',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِحَمِيكُنَّ',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فبحميها'] = {
    'word': u'فبحميها',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِحَمِيهَا',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'هَا'
}
STOPWORDS[u'فبحميهم'] = {
    'word': u'فبحميهم',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِحَمِيهُمْ',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فبحميهن'] = {
    'word': u'فبحميهن',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِحَمِيهُنَّ',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فبحمينا'] = {
    'word': u'فبحمينا',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِحَمِينَا',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'نَا'
}
STOPWORDS[u'فبحميكما'] = {
    'word': u'فبحميكما',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِحَمِيكُمَا',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فبحميهما'] = {
    'word': u'فبحميهما',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِحَمِيهُمَا',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'هُمَا'
}
STOPWORDS[u'وكحميي'] = {
    'word': u'وكحميي',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَحَمِيِي',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'ِي'
}
STOPWORDS[u'وكحميك'] = {
    'word': u'وكحميك',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَحَمِيكَ',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'كَ'
}
STOPWORDS[u'وكحميك'] = {
    'word': u'وكحميك',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَحَمِيكِ',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'كِ'
}
STOPWORDS[u'وكحميه'] = {
    'word': u'وكحميه',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَحَمِيه',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'ه'
}
STOPWORDS[u'وكحميكم'] = {
    'word': u'وكحميكم',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَحَمِيكُمْ',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'كُمْ'
}
STOPWORDS[u'وكحميكن'] = {
    'word': u'وكحميكن',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَحَمِيكُنَّ',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'وكحميها'] = {
    'word': u'وكحميها',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَحَمِيهَا',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'هَا'
}
STOPWORDS[u'وكحميهم'] = {
    'word': u'وكحميهم',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَحَمِيهُمْ',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'هُمْ'
}
STOPWORDS[u'وكحميهن'] = {
    'word': u'وكحميهن',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَحَمِيهُنَّ',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'وكحمينا'] = {
    'word': u'وكحمينا',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَحَمِينَا',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'نَا'
}
STOPWORDS[u'وكحميكما'] = {
    'word': u'وكحميكما',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَحَمِيكُمَا',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'كُمَا'
}
STOPWORDS[u'وكحميهما'] = {
    'word': u'وكحميهما',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَحَمِيهُمَا',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فكحميي'] = {
    'word': u'فكحميي',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَحَمِيِي',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'ِي'
}
STOPWORDS[u'فكحميك'] = {
    'word': u'فكحميك',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَحَمِيكَ',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'كَ'
}
STOPWORDS[u'فكحميك'] = {
    'word': u'فكحميك',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَحَمِيكِ',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'كِ'
}
STOPWORDS[u'فكحميه'] = {
    'word': u'فكحميه',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَحَمِيه',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'ه'
}
STOPWORDS[u'فكحميكم'] = {
    'word': u'فكحميكم',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَحَمِيكُمْ',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فكحميكن'] = {
    'word': u'فكحميكن',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَحَمِيكُنَّ',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فكحميها'] = {
    'word': u'فكحميها',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَحَمِيهَا',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'هَا'
}
STOPWORDS[u'فكحميهم'] = {
    'word': u'فكحميهم',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَحَمِيهُمْ',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فكحميهن'] = {
    'word': u'فكحميهن',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَحَمِيهُنَّ',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فكحمينا'] = {
    'word': u'فكحمينا',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَحَمِينَا',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'نَا'
}
STOPWORDS[u'فكحميكما'] = {
    'word': u'فكحميكما',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَحَمِيكُمَا',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فكحميهما'] = {
    'word': u'فكحميهما',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَحَمِيهُمَا',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ولحميي'] = {
    'word': u'ولحميي',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِحَمِيِي',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'ِي'
}
STOPWORDS[u'ولحميك'] = {
    'word': u'ولحميك',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِحَمِيكَ',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'كَ'
}
STOPWORDS[u'ولحميك'] = {
    'word': u'ولحميك',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِحَمِيكِ',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'كِ'
}
STOPWORDS[u'ولحميه'] = {
    'word': u'ولحميه',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِحَمِيه',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'ه'
}
STOPWORDS[u'ولحميكم'] = {
    'word': u'ولحميكم',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِحَمِيكُمْ',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ولحميكن'] = {
    'word': u'ولحميكن',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِحَمِيكُنَّ',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ولحميها'] = {
    'word': u'ولحميها',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِحَمِيهَا',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'هَا'
}
STOPWORDS[u'ولحميهم'] = {
    'word': u'ولحميهم',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِحَمِيهُمْ',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ولحميهن'] = {
    'word': u'ولحميهن',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِحَمِيهُنَّ',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ولحمينا'] = {
    'word': u'ولحمينا',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِحَمِينَا',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'نَا'
}
STOPWORDS[u'ولحميكما'] = {
    'word': u'ولحميكما',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِحَمِيكُمَا',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ولحميهما'] = {
    'word': u'ولحميهما',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِحَمِيهُمَا',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فلحميي'] = {
    'word': u'فلحميي',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِحَمِيِي',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'ِي'
}
STOPWORDS[u'فلحميك'] = {
    'word': u'فلحميك',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِحَمِيكَ',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'كَ'
}
STOPWORDS[u'فلحميك'] = {
    'word': u'فلحميك',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِحَمِيكِ',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'كِ'
}
STOPWORDS[u'فلحميه'] = {
    'word': u'فلحميه',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِحَمِيه',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'ه'
}
STOPWORDS[u'فلحميكم'] = {
    'word': u'فلحميكم',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِحَمِيكُمْ',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فلحميكن'] = {
    'word': u'فلحميكن',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِحَمِيكُنَّ',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فلحميها'] = {
    'word': u'فلحميها',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِحَمِيهَا',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'هَا'
}
STOPWORDS[u'فلحميهم'] = {
    'word': u'فلحميهم',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِحَمِيهُمْ',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فلحميهن'] = {
    'word': u'فلحميهن',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِحَمِيهُنَّ',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فلحمينا'] = {
    'word': u'فلحمينا',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِحَمِينَا',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'نَا'
}
STOPWORDS[u'فلحميكما'] = {
    'word': u'فلحميكما',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِحَمِيكُمَا',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فلحميهما'] = {
    'word': u'فلحميهما',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِحَمِيهُمَا',
    'stem': u'حَمِي',
    'type': u'STOPWORD',
    'original': u'حَمِي',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ذي'] = {
    'word': u'ذِي',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة',
    'vocalized': u'ذِي',
    'stem': u'ذِي',
    'type': u'STOPWORD',
    'original': u'ذِي',
    'encletic': u''
}
STOPWORDS[u'وذي'] = {
    'word': u'وذي',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف',
    'vocalized': u'وَذِي',
    'stem': u'ذِي',
    'type': u'STOPWORD',
    'original': u'ذِي',
    'encletic': u''
}
STOPWORDS[u'فذي'] = {
    'word': u'فذي',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف',
    'vocalized': u'فَذِي',
    'stem': u'ذِي',
    'type': u'STOPWORD',
    'original': u'ذِي',
    'encletic': u''
}
STOPWORDS[u'في'] = {
    'word': u'فِي',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة',
    'vocalized': u'فِي',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u''
}
STOPWORDS[u'بفي'] = {
    'word': u'بفي',
    'procletic': u'-بِ',
    'tags': u'أداة:الأسماء الخمسة:مجرور',
    'vocalized': u'بِفِي',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u''
}
STOPWORDS[u'كفي'] = {
    'word': u'كفي',
    'procletic': u'-كَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور',
    'vocalized': u'كَفِي',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u''
}
STOPWORDS[u'لفي'] = {
    'word': u'لفي',
    'procletic': u'-لِ',
    'tags': u'أداة:الأسماء الخمسة:مجرور',
    'vocalized': u'لِفِي',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u''
}
STOPWORDS[u'وفي'] = {
    'word': u'وفي',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف',
    'vocalized': u'وَفِي',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u''
}
STOPWORDS[u'ففي'] = {
    'word': u'ففي',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف',
    'vocalized': u'فَفِي',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u''
}
STOPWORDS[u'وبفي'] = {
    'word': u'وبفي',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف',
    'vocalized': u'وَبِفِي',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u''
}
STOPWORDS[u'فبفي'] = {
    'word': u'فبفي',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف',
    'vocalized': u'فَبِفِي',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u''
}
STOPWORDS[u'وكفي'] = {
    'word': u'وكفي',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف',
    'vocalized': u'وَكَفِي',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u''
}
STOPWORDS[u'فكفي'] = {
    'word': u'فكفي',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف',
    'vocalized': u'فَكَفِي',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u''
}
STOPWORDS[u'ولفي'] = {
    'word': u'ولفي',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف',
    'vocalized': u'وَلِفِي',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u''
}
STOPWORDS[u'فلفي'] = {
    'word': u'فلفي',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف',
    'vocalized': u'فَلِفِي',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u''
}
STOPWORDS[u'فيي'] = {
    'word': u'فيي',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'فِيِي',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'ِي'
}
STOPWORDS[u'فيك'] = {
    'word': u'فيك',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'فِيكَ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كَ'
}
STOPWORDS[u'فيك'] = {
    'word': u'فيك',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'فِيكِ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كِ'
}
STOPWORDS[u'فيه'] = {
    'word': u'فيه',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'فِيه',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'ه'
}
STOPWORDS[u'فيكم'] = {
    'word': u'فيكم',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'فِيكُمْ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فيكن'] = {
    'word': u'فيكن',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'فِيكُنَّ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فيها'] = {
    'word': u'فيها',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'فِيهَا',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'هَا'
}
STOPWORDS[u'فيهم'] = {
    'word': u'فيهم',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'فِيهُمْ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فيهن'] = {
    'word': u'فيهن',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'فِيهُنَّ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فينا'] = {
    'word': u'فينا',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'فِينَا',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'نَا'
}
STOPWORDS[u'فيكما'] = {
    'word': u'فيكما',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'فِيكُمَا',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فيهما'] = {
    'word': u'فيهما',
    'procletic': u'',
    'tags': u'أداة:الأسماء الخمسة:مضاف',
    'vocalized': u'فِيهُمَا',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'هُمَا'
}
STOPWORDS[u'بفيي'] = {
    'word': u'بفيي',
    'procletic': u'-بِ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'بِفِيِي',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'ِي'
}
STOPWORDS[u'بفيك'] = {
    'word': u'بفيك',
    'procletic': u'-بِ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'بِفِيكَ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كَ'
}
STOPWORDS[u'بفيك'] = {
    'word': u'بفيك',
    'procletic': u'-بِ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'بِفِيكِ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كِ'
}
STOPWORDS[u'بفيه'] = {
    'word': u'بفيه',
    'procletic': u'-بِ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'بِفِيه',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'ه'
}
STOPWORDS[u'بفيكم'] = {
    'word': u'بفيكم',
    'procletic': u'-بِ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'بِفِيكُمْ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كُمْ'
}
STOPWORDS[u'بفيكن'] = {
    'word': u'بفيكن',
    'procletic': u'-بِ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'بِفِيكُنَّ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'بفيها'] = {
    'word': u'بفيها',
    'procletic': u'-بِ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'بِفِيهَا',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'هَا'
}
STOPWORDS[u'بفيهم'] = {
    'word': u'بفيهم',
    'procletic': u'-بِ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'بِفِيهُمْ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'هُمْ'
}
STOPWORDS[u'بفيهن'] = {
    'word': u'بفيهن',
    'procletic': u'-بِ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'بِفِيهُنَّ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'بفينا'] = {
    'word': u'بفينا',
    'procletic': u'-بِ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'بِفِينَا',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'نَا'
}
STOPWORDS[u'بفيكما'] = {
    'word': u'بفيكما',
    'procletic': u'-بِ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'بِفِيكُمَا',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كُمَا'
}
STOPWORDS[u'بفيهما'] = {
    'word': u'بفيهما',
    'procletic': u'-بِ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'بِفِيهُمَا',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'هُمَا'
}
STOPWORDS[u'كفيي'] = {
    'word': u'كفيي',
    'procletic': u'-كَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'كَفِيِي',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'ِي'
}
STOPWORDS[u'كفيك'] = {
    'word': u'كفيك',
    'procletic': u'-كَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'كَفِيكَ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كَ'
}
STOPWORDS[u'كفيك'] = {
    'word': u'كفيك',
    'procletic': u'-كَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'كَفِيكِ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كِ'
}
STOPWORDS[u'كفيه'] = {
    'word': u'كفيه',
    'procletic': u'-كَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'كَفِيه',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'ه'
}
STOPWORDS[u'كفيكم'] = {
    'word': u'كفيكم',
    'procletic': u'-كَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'كَفِيكُمْ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كُمْ'
}
STOPWORDS[u'كفيكن'] = {
    'word': u'كفيكن',
    'procletic': u'-كَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'كَفِيكُنَّ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'كفيها'] = {
    'word': u'كفيها',
    'procletic': u'-كَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'كَفِيهَا',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'هَا'
}
STOPWORDS[u'كفيهم'] = {
    'word': u'كفيهم',
    'procletic': u'-كَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'كَفِيهُمْ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'هُمْ'
}
STOPWORDS[u'كفيهن'] = {
    'word': u'كفيهن',
    'procletic': u'-كَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'كَفِيهُنَّ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'كفينا'] = {
    'word': u'كفينا',
    'procletic': u'-كَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'كَفِينَا',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'نَا'
}
STOPWORDS[u'كفيكما'] = {
    'word': u'كفيكما',
    'procletic': u'-كَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'كَفِيكُمَا',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كُمَا'
}
STOPWORDS[u'كفيهما'] = {
    'word': u'كفيهما',
    'procletic': u'-كَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'كَفِيهُمَا',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'هُمَا'
}
STOPWORDS[u'لفيي'] = {
    'word': u'لفيي',
    'procletic': u'-لِ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'لِفِيِي',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'ِي'
}
STOPWORDS[u'لفيك'] = {
    'word': u'لفيك',
    'procletic': u'-لِ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'لِفِيكَ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كَ'
}
STOPWORDS[u'لفيك'] = {
    'word': u'لفيك',
    'procletic': u'-لِ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'لِفِيكِ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كِ'
}
STOPWORDS[u'لفيه'] = {
    'word': u'لفيه',
    'procletic': u'-لِ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'لِفِيه',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'ه'
}
STOPWORDS[u'لفيكم'] = {
    'word': u'لفيكم',
    'procletic': u'-لِ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'لِفِيكُمْ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كُمْ'
}
STOPWORDS[u'لفيكن'] = {
    'word': u'لفيكن',
    'procletic': u'-لِ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'لِفِيكُنَّ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'لفيها'] = {
    'word': u'لفيها',
    'procletic': u'-لِ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'لِفِيهَا',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'هَا'
}
STOPWORDS[u'لفيهم'] = {
    'word': u'لفيهم',
    'procletic': u'-لِ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'لِفِيهُمْ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'هُمْ'
}
STOPWORDS[u'لفيهن'] = {
    'word': u'لفيهن',
    'procletic': u'-لِ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'لِفِيهُنَّ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'لفينا'] = {
    'word': u'لفينا',
    'procletic': u'-لِ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'لِفِينَا',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'نَا'
}
STOPWORDS[u'لفيكما'] = {
    'word': u'لفيكما',
    'procletic': u'-لِ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'لِفِيكُمَا',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كُمَا'
}
STOPWORDS[u'لفيهما'] = {
    'word': u'لفيهما',
    'procletic': u'-لِ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:مضاف',
    'vocalized': u'لِفِيهُمَا',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'هُمَا'
}
STOPWORDS[u'وفيي'] = {
    'word': u'وفيي',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَفِيِي',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'ِي'
}
STOPWORDS[u'وفيك'] = {
    'word': u'وفيك',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَفِيكَ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كَ'
}
STOPWORDS[u'وفيك'] = {
    'word': u'وفيك',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَفِيكِ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كِ'
}
STOPWORDS[u'وفيه'] = {
    'word': u'وفيه',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَفِيه',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'ه'
}
STOPWORDS[u'وفيكم'] = {
    'word': u'وفيكم',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَفِيكُمْ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كُمْ'
}
STOPWORDS[u'وفيكن'] = {
    'word': u'وفيكن',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَفِيكُنَّ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'وفيها'] = {
    'word': u'وفيها',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَفِيهَا',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'هَا'
}
STOPWORDS[u'وفيهم'] = {
    'word': u'وفيهم',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَفِيهُمْ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'هُمْ'
}
STOPWORDS[u'وفيهن'] = {
    'word': u'وفيهن',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَفِيهُنَّ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'وفينا'] = {
    'word': u'وفينا',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَفِينَا',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'نَا'
}
STOPWORDS[u'وفيكما'] = {
    'word': u'وفيكما',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَفِيكُمَا',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كُمَا'
}
STOPWORDS[u'وفيهما'] = {
    'word': u'وفيهما',
    'procletic': u'-وَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'وَفِيهُمَا',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ففيي'] = {
    'word': u'ففيي',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَفِيِي',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'ِي'
}
STOPWORDS[u'ففيك'] = {
    'word': u'ففيك',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَفِيكَ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كَ'
}
STOPWORDS[u'ففيك'] = {
    'word': u'ففيك',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَفِيكِ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كِ'
}
STOPWORDS[u'ففيه'] = {
    'word': u'ففيه',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَفِيه',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'ه'
}
STOPWORDS[u'ففيكم'] = {
    'word': u'ففيكم',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَفِيكُمْ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ففيكن'] = {
    'word': u'ففيكن',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَفِيكُنَّ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ففيها'] = {
    'word': u'ففيها',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَفِيهَا',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'هَا'
}
STOPWORDS[u'ففيهم'] = {
    'word': u'ففيهم',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَفِيهُمْ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ففيهن'] = {
    'word': u'ففيهن',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَفِيهُنَّ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ففينا'] = {
    'word': u'ففينا',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَفِينَا',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'نَا'
}
STOPWORDS[u'ففيكما'] = {
    'word': u'ففيكما',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَفِيكُمَا',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ففيهما'] = {
    'word': u'ففيهما',
    'procletic': u'-فَ',
    'tags': u'أداة:الأسماء الخمسة:معطوف:مضاف',
    'vocalized': u'فَفِيهُمَا',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'هُمَا'
}
STOPWORDS[u'وبفيي'] = {
    'word': u'وبفيي',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِفِيِي',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'ِي'
}
STOPWORDS[u'وبفيك'] = {
    'word': u'وبفيك',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِفِيكَ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كَ'
}
STOPWORDS[u'وبفيك'] = {
    'word': u'وبفيك',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِفِيكِ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كِ'
}
STOPWORDS[u'وبفيه'] = {
    'word': u'وبفيه',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِفِيه',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'ه'
}
STOPWORDS[u'وبفيكم'] = {
    'word': u'وبفيكم',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِفِيكُمْ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كُمْ'
}
STOPWORDS[u'وبفيكن'] = {
    'word': u'وبفيكن',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِفِيكُنَّ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'وبفيها'] = {
    'word': u'وبفيها',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِفِيهَا',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'هَا'
}
STOPWORDS[u'وبفيهم'] = {
    'word': u'وبفيهم',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِفِيهُمْ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'هُمْ'
}
STOPWORDS[u'وبفيهن'] = {
    'word': u'وبفيهن',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِفِيهُنَّ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'وبفينا'] = {
    'word': u'وبفينا',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِفِينَا',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'نَا'
}
STOPWORDS[u'وبفيكما'] = {
    'word': u'وبفيكما',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِفِيكُمَا',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كُمَا'
}
STOPWORDS[u'وبفيهما'] = {
    'word': u'وبفيهما',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِفِيهُمَا',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فبفيي'] = {
    'word': u'فبفيي',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِفِيِي',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'ِي'
}
STOPWORDS[u'فبفيك'] = {
    'word': u'فبفيك',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِفِيكَ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كَ'
}
STOPWORDS[u'فبفيك'] = {
    'word': u'فبفيك',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِفِيكِ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كِ'
}
STOPWORDS[u'فبفيه'] = {
    'word': u'فبفيه',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِفِيه',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'ه'
}
STOPWORDS[u'فبفيكم'] = {
    'word': u'فبفيكم',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِفِيكُمْ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فبفيكن'] = {
    'word': u'فبفيكن',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِفِيكُنَّ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فبفيها'] = {
    'word': u'فبفيها',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِفِيهَا',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'هَا'
}
STOPWORDS[u'فبفيهم'] = {
    'word': u'فبفيهم',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِفِيهُمْ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فبفيهن'] = {
    'word': u'فبفيهن',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِفِيهُنَّ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فبفينا'] = {
    'word': u'فبفينا',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِفِينَا',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'نَا'
}
STOPWORDS[u'فبفيكما'] = {
    'word': u'فبفيكما',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِفِيكُمَا',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فبفيهما'] = {
    'word': u'فبفيهما',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِفِيهُمَا',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'هُمَا'
}
STOPWORDS[u'وكفيي'] = {
    'word': u'وكفيي',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَفِيِي',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'ِي'
}
STOPWORDS[u'وكفيك'] = {
    'word': u'وكفيك',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَفِيكَ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كَ'
}
STOPWORDS[u'وكفيك'] = {
    'word': u'وكفيك',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَفِيكِ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كِ'
}
STOPWORDS[u'وكفيه'] = {
    'word': u'وكفيه',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَفِيه',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'ه'
}
STOPWORDS[u'وكفيكم'] = {
    'word': u'وكفيكم',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَفِيكُمْ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كُمْ'
}
STOPWORDS[u'وكفيكن'] = {
    'word': u'وكفيكن',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَفِيكُنَّ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'وكفيها'] = {
    'word': u'وكفيها',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَفِيهَا',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'هَا'
}
STOPWORDS[u'وكفيهم'] = {
    'word': u'وكفيهم',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَفِيهُمْ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'هُمْ'
}
STOPWORDS[u'وكفيهن'] = {
    'word': u'وكفيهن',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَفِيهُنَّ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'وكفينا'] = {
    'word': u'وكفينا',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَفِينَا',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'نَا'
}
STOPWORDS[u'وكفيكما'] = {
    'word': u'وكفيكما',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَفِيكُمَا',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كُمَا'
}
STOPWORDS[u'وكفيهما'] = {
    'word': u'وكفيهما',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَفِيهُمَا',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فكفيي'] = {
    'word': u'فكفيي',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَفِيِي',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'ِي'
}
STOPWORDS[u'فكفيك'] = {
    'word': u'فكفيك',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَفِيكَ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كَ'
}
STOPWORDS[u'فكفيك'] = {
    'word': u'فكفيك',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَفِيكِ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كِ'
}
STOPWORDS[u'فكفيه'] = {
    'word': u'فكفيه',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَفِيه',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'ه'
}
STOPWORDS[u'فكفيكم'] = {
    'word': u'فكفيكم',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَفِيكُمْ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فكفيكن'] = {
    'word': u'فكفيكن',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَفِيكُنَّ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فكفيها'] = {
    'word': u'فكفيها',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَفِيهَا',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'هَا'
}
STOPWORDS[u'فكفيهم'] = {
    'word': u'فكفيهم',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَفِيهُمْ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فكفيهن'] = {
    'word': u'فكفيهن',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَفِيهُنَّ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فكفينا'] = {
    'word': u'فكفينا',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَفِينَا',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'نَا'
}
STOPWORDS[u'فكفيكما'] = {
    'word': u'فكفيكما',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَفِيكُمَا',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فكفيهما'] = {
    'word': u'فكفيهما',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَفِيهُمَا',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ولفيي'] = {
    'word': u'ولفيي',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِفِيِي',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'ِي'
}
STOPWORDS[u'ولفيك'] = {
    'word': u'ولفيك',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِفِيكَ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كَ'
}
STOPWORDS[u'ولفيك'] = {
    'word': u'ولفيك',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِفِيكِ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كِ'
}
STOPWORDS[u'ولفيه'] = {
    'word': u'ولفيه',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِفِيه',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'ه'
}
STOPWORDS[u'ولفيكم'] = {
    'word': u'ولفيكم',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِفِيكُمْ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ولفيكن'] = {
    'word': u'ولفيكن',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِفِيكُنَّ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ولفيها'] = {
    'word': u'ولفيها',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِفِيهَا',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'هَا'
}
STOPWORDS[u'ولفيهم'] = {
    'word': u'ولفيهم',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِفِيهُمْ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ولفيهن'] = {
    'word': u'ولفيهن',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِفِيهُنَّ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ولفينا'] = {
    'word': u'ولفينا',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِفِينَا',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'نَا'
}
STOPWORDS[u'ولفيكما'] = {
    'word': u'ولفيكما',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِفِيكُمَا',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ولفيهما'] = {
    'word': u'ولفيهما',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِفِيهُمَا',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فلفيي'] = {
    'word': u'فلفيي',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِفِيِي',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'ِي'
}
STOPWORDS[u'فلفيك'] = {
    'word': u'فلفيك',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِفِيكَ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كَ'
}
STOPWORDS[u'فلفيك'] = {
    'word': u'فلفيك',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِفِيكِ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كِ'
}
STOPWORDS[u'فلفيه'] = {
    'word': u'فلفيه',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِفِيه',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'ه'
}
STOPWORDS[u'فلفيكم'] = {
    'word': u'فلفيكم',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِفِيكُمْ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فلفيكن'] = {
    'word': u'فلفيكن',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِفِيكُنَّ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فلفيها'] = {
    'word': u'فلفيها',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِفِيهَا',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'هَا'
}
STOPWORDS[u'فلفيهم'] = {
    'word': u'فلفيهم',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِفِيهُمْ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فلفيهن'] = {
    'word': u'فلفيهن',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِفِيهُنَّ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فلفينا'] = {
    'word': u'فلفينا',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِفِينَا',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'نَا'
}
STOPWORDS[u'فلفيكما'] = {
    'word': u'فلفيكما',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِفِيكُمَا',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فلفيهما'] = {
    'word': u'فلفيهما',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:الأسماء الخمسة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِفِيهُمَا',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'هُمَا'
}
STOPWORDS[u'لن'] = {
    'word': u'لَنْ',
    'procletic': u'',
    'tags': u'أداة:الحروف(حروف)',
    'vocalized': u'لَنْ',
    'stem': u'لَنْ',
    'type': u'STOPWORD',
    'original': u'لَنْ',
    'encletic': u''
}
STOPWORDS[u'ولن'] = {
    'word': u'ولن',
    'procletic': u'-وَ',
    'tags': u'أداة:الحروف(حروف):معطوف',
    'vocalized': u'وَلَنْ',
    'stem': u'لَنْ',
    'type': u'STOPWORD',
    'original': u'لَنْ',
    'encletic': u''
}
STOPWORDS[u'فلن'] = {
    'word': u'فلن',
    'procletic': u'-فَ',
    'tags': u'أداة:الحروف(حروف):معطوف',
    'vocalized': u'فَلَنْ',
    'stem': u'لَنْ',
    'type': u'STOPWORD',
    'original': u'لَنْ',
    'encletic': u''
}
STOPWORDS[u'ألن'] = {
    'word': u'ألن',
    'procletic': u'-أَ',
    'tags': u'أداة:الحروف(حروف):مجرور',
    'vocalized': u'أَلَنْ',
    'stem': u'لَنْ',
    'type': u'STOPWORD',
    'original': u'لَنْ',
    'encletic': u''
}
STOPWORDS[u'أولن'] = {
    'word': u'أولن',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:الحروف(حروف):معطوف:مجرور',
    'vocalized': u'أَوَلَنْ',
    'stem': u'لَنْ',
    'type': u'STOPWORD',
    'original': u'لَنْ',
    'encletic': u''
}
STOPWORDS[u'أفلن'] = {
    'word': u'أفلن',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:الحروف(حروف):معطوف:مجرور',
    'vocalized': u'أَفَلَنْ',
    'stem': u'لَنْ',
    'type': u'STOPWORD',
    'original': u'لَنْ',
    'encletic': u''
}
STOPWORDS[u'لو'] = {
    'word': u'لَوْ',
    'procletic': u'',
    'tags': u'أداة:الحروف(حروف)',
    'vocalized': u'لَوْ',
    'stem': u'لَوْ',
    'type': u'STOPWORD',
    'original': u'لَوْ',
    'encletic': u''
}
STOPWORDS[u'ولو'] = {
    'word': u'ولو',
    'procletic': u'-وَ',
    'tags': u'أداة:الحروف(حروف):معطوف',
    'vocalized': u'وَلَوْ',
    'stem': u'لَوْ',
    'type': u'STOPWORD',
    'original': u'لَوْ',
    'encletic': u''
}
STOPWORDS[u'فلو'] = {
    'word': u'فلو',
    'procletic': u'-فَ',
    'tags': u'أداة:الحروف(حروف):معطوف',
    'vocalized': u'فَلَوْ',
    'stem': u'لَوْ',
    'type': u'STOPWORD',
    'original': u'لَوْ',
    'encletic': u''
}
STOPWORDS[u'ألو'] = {
    'word': u'ألو',
    'procletic': u'-أَ',
    'tags': u'أداة:الحروف(حروف):مجرور',
    'vocalized': u'أَلَوْ',
    'stem': u'لَوْ',
    'type': u'STOPWORD',
    'original': u'لَوْ',
    'encletic': u''
}
STOPWORDS[u'أولو'] = {
    'word': u'أولو',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:الحروف(حروف):معطوف:مجرور',
    'vocalized': u'أَوَلَوْ',
    'stem': u'لَوْ',
    'type': u'STOPWORD',
    'original': u'لَوْ',
    'encletic': u''
}
STOPWORDS[u'أفلو'] = {
    'word': u'أفلو',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:الحروف(حروف):معطوف:مجرور',
    'vocalized': u'أَفَلَوْ',
    'stem': u'لَوْ',
    'type': u'STOPWORD',
    'original': u'لَوْ',
    'encletic': u''
}
STOPWORDS[u'لولا'] = {
    'word': u'لَوْلَا',
    'procletic': u'',
    'tags': u'أداة:الحروف(حروف)',
    'vocalized': u'لَوْلَا',
    'stem': u'لَوْلَا',
    'type': u'STOPWORD',
    'original': u'لَوْلَا',
    'encletic': u''
}
STOPWORDS[u'ولولا'] = {
    'word': u'ولولا',
    'procletic': u'-وَ',
    'tags': u'أداة:الحروف(حروف):معطوف',
    'vocalized': u'وَلَوْلَا',
    'stem': u'لَوْلَا',
    'type': u'STOPWORD',
    'original': u'لَوْلَا',
    'encletic': u''
}
STOPWORDS[u'فلولا'] = {
    'word': u'فلولا',
    'procletic': u'-فَ',
    'tags': u'أداة:الحروف(حروف):معطوف',
    'vocalized': u'فَلَوْلَا',
    'stem': u'لَوْلَا',
    'type': u'STOPWORD',
    'original': u'لَوْلَا',
    'encletic': u''
}
STOPWORDS[u'لوما'] = {
    'word': u'لَوْمَا',
    'procletic': u'',
    'tags': u'أداة:الحروف(حروف)',
    'vocalized': u'لَوْمَا',
    'stem': u'لَوْمَا',
    'type': u'STOPWORD',
    'original': u'لَوْمَا',
    'encletic': u''
}
STOPWORDS[u'ولوما'] = {
    'word': u'ولوما',
    'procletic': u'-وَ',
    'tags': u'أداة:الحروف(حروف):معطوف',
    'vocalized': u'وَلَوْمَا',
    'stem': u'لَوْمَا',
    'type': u'STOPWORD',
    'original': u'لَوْمَا',
    'encletic': u''
}
STOPWORDS[u'فلوما'] = {
    'word': u'فلوما',
    'procletic': u'-فَ',
    'tags': u'أداة:الحروف(حروف):معطوف',
    'vocalized': u'فَلَوْمَا',
    'stem': u'لَوْمَا',
    'type': u'STOPWORD',
    'original': u'لَوْمَا',
    'encletic': u''
}
STOPWORDS[u'نعم'] = {
    'word': u'نَعَمْ',
    'procletic': u'',
    'tags': u'أداة:الحروف(حروف)',
    'vocalized': u'نَعَمْ',
    'stem': u'نَعَمْ',
    'type': u'STOPWORD',
    'original': u'نَعَمْ',
    'encletic': u''
}
STOPWORDS[u'ونعم'] = {
    'word': u'ونعم',
    'procletic': u'-وَ',
    'tags': u'أداة:الحروف(حروف):معطوف',
    'vocalized': u'وَنَعَمْ',
    'stem': u'نَعَمْ',
    'type': u'STOPWORD',
    'original': u'نَعَمْ',
    'encletic': u''
}
STOPWORDS[u'فنعم'] = {
    'word': u'فنعم',
    'procletic': u'-فَ',
    'tags': u'أداة:الحروف(حروف):معطوف',
    'vocalized': u'فَنَعَمْ',
    'stem': u'نَعَمْ',
    'type': u'STOPWORD',
    'original': u'نَعَمْ',
    'encletic': u''
}
STOPWORDS[u'بئس'] = {
    'word': u'بِئْسَ',
    'procletic': u'',
    'tags': u'أداة:المدح والذم',
    'vocalized': u'بِئْسَ',
    'stem': u'بِئْسَ',
    'type': u'STOPWORD',
    'original': u'بِئْسَ',
    'encletic': u''
}
STOPWORDS[u'وبئس'] = {
    'word': u'وبئس',
    'procletic': u'-وَ',
    'tags': u'أداة:المدح والذم:معطوف',
    'vocalized': u'وَبِئْسَ',
    'stem': u'بِئْسَ',
    'type': u'STOPWORD',
    'original': u'بِئْسَ',
    'encletic': u''
}
STOPWORDS[u'فبئس'] = {
    'word': u'فبئس',
    'procletic': u'-فَ',
    'tags': u'أداة:المدح والذم:معطوف',
    'vocalized': u'فَبِئْسَ',
    'stem': u'بِئْسَ',
    'type': u'STOPWORD',
    'original': u'بِئْسَ',
    'encletic': u''
}
STOPWORDS[u'حبذا'] = {
    'word': u'حَبَّذَا',
    'procletic': u'',
    'tags': u'أداة:المدح والذم',
    'vocalized': u'حَبَّذَا',
    'stem': u'حَبَّذَا',
    'type': u'STOPWORD',
    'original': u'حَبَّذَا',
    'encletic': u''
}
STOPWORDS[u'وحبذا'] = {
    'word': u'وحبذا',
    'procletic': u'-وَ',
    'tags': u'أداة:المدح والذم:معطوف',
    'vocalized': u'وَحَبَّذَا',
    'stem': u'حَبَّذَا',
    'type': u'STOPWORD',
    'original': u'حَبَّذَا',
    'encletic': u''
}
STOPWORDS[u'فحبذا'] = {
    'word': u'فحبذا',
    'procletic': u'-فَ',
    'tags': u'أداة:المدح والذم:معطوف',
    'vocalized': u'فَحَبَّذَا',
    'stem': u'حَبَّذَا',
    'type': u'STOPWORD',
    'original': u'حَبَّذَا',
    'encletic': u''
}
STOPWORDS[u'ساء'] = {
    'word': u'سَاءَ',
    'procletic': u'',
    'tags': u'أداة:المدح والذم',
    'vocalized': u'سَاءَ',
    'stem': u'سَاءَ',
    'type': u'STOPWORD',
    'original': u'سَاءَ',
    'encletic': u''
}
STOPWORDS[u'وساء'] = {
    'word': u'وساء',
    'procletic': u'-وَ',
    'tags': u'أداة:المدح والذم:معطوف',
    'vocalized': u'وَسَاءَ',
    'stem': u'سَاءَ',
    'type': u'STOPWORD',
    'original': u'سَاءَ',
    'encletic': u''
}
STOPWORDS[u'فساء'] = {
    'word': u'فساء',
    'procletic': u'-فَ',
    'tags': u'أداة:المدح والذم:معطوف',
    'vocalized': u'فَسَاءَ',
    'stem': u'سَاءَ',
    'type': u'STOPWORD',
    'original': u'سَاءَ',
    'encletic': u''
}
STOPWORDS[u'ساءما'] = {
    'word': u'سَاءَمَا',
    'procletic': u'',
    'tags': u'أداة:المدح والذم',
    'vocalized': u'سَاءَمَا',
    'stem': u'سَاءَمَا',
    'type': u'STOPWORD',
    'original': u'سَاءَمَا',
    'encletic': u''
}
STOPWORDS[u'وساءما'] = {
    'word': u'وساءما',
    'procletic': u'-وَ',
    'tags': u'أداة:المدح والذم:معطوف',
    'vocalized': u'وَسَاءَمَا',
    'stem': u'سَاءَمَا',
    'type': u'STOPWORD',
    'original': u'سَاءَمَا',
    'encletic': u''
}
STOPWORDS[u'فساءما'] = {
    'word': u'فساءما',
    'procletic': u'-فَ',
    'tags': u'أداة:المدح والذم:معطوف',
    'vocalized': u'فَسَاءَمَا',
    'stem': u'سَاءَمَا',
    'type': u'STOPWORD',
    'original': u'سَاءَمَا',
    'encletic': u''
}
STOPWORDS[u'نعم'] = {
    'word': u'نِعْمَ',
    'procletic': u'',
    'tags': u'أداة:المدح والذم',
    'vocalized': u'نِعْمَ',
    'stem': u'نِعْمَ',
    'type': u'STOPWORD',
    'original': u'نِعْمَ',
    'encletic': u''
}
STOPWORDS[u'ونعم'] = {
    'word': u'ونعم',
    'procletic': u'-وَ',
    'tags': u'أداة:المدح والذم:معطوف',
    'vocalized': u'وَنِعْمَ',
    'stem': u'نِعْمَ',
    'type': u'STOPWORD',
    'original': u'نِعْمَ',
    'encletic': u''
}
STOPWORDS[u'فنعم'] = {
    'word': u'فنعم',
    'procletic': u'-فَ',
    'tags': u'أداة:المدح والذم:معطوف',
    'vocalized': u'فَنِعْمَ',
    'stem': u'نِعْمَ',
    'type': u'STOPWORD',
    'original': u'نِعْمَ',
    'encletic': u''
}
STOPWORDS[u'نعما'] = {
    'word': u'نِعِمّا',
    'procletic': u'',
    'tags': u'أداة:المدح والذم',
    'vocalized': u'نِعِمّا',
    'stem': u'نِعِمّا',
    'type': u'STOPWORD',
    'original': u'نِعِمّا',
    'encletic': u''
}
STOPWORDS[u'ونعما'] = {
    'word': u'ونعما',
    'procletic': u'-وَ',
    'tags': u'أداة:المدح والذم:معطوف',
    'vocalized': u'وَنِعِمّا',
    'stem': u'نِعِمّا',
    'type': u'STOPWORD',
    'original': u'نِعِمّا',
    'encletic': u''
}
STOPWORDS[u'فنعما'] = {
    'word': u'فنعما',
    'procletic': u'-فَ',
    'tags': u'أداة:المدح والذم:معطوف',
    'vocalized': u'فَنِعِمّا',
    'stem': u'نِعِمّا',
    'type': u'STOPWORD',
    'original': u'نِعِمّا',
    'encletic': u''
}
STOPWORDS[u'إن'] = {
    'word': u'إَنَّ',
    'procletic': u'',
    'tags': u'أداة:المشبهة بليس',
    'vocalized': u'إَنَّ',
    'stem': u'إَنَّ',
    'type': u'STOPWORD',
    'original': u'إَنَّ',
    'encletic': u''
}
STOPWORDS[u'بإن'] = {
    'word': u'بإن',
    'procletic': u'-بِ',
    'tags': u'أداة:المشبهة بليس:مجرور',
    'vocalized': u'بِإَنَّ',
    'stem': u'إَنَّ',
    'type': u'STOPWORD',
    'original': u'إَنَّ',
    'encletic': u''
}
STOPWORDS[u'كإن'] = {
    'word': u'كإن',
    'procletic': u'-كَ',
    'tags': u'أداة:المشبهة بليس:مجرور',
    'vocalized': u'كَإَنَّ',
    'stem': u'إَنَّ',
    'type': u'STOPWORD',
    'original': u'إَنَّ',
    'encletic': u''
}
STOPWORDS[u'لإن'] = {
    'word': u'لإن',
    'procletic': u'-لِ',
    'tags': u'أداة:المشبهة بليس:مجرور',
    'vocalized': u'لِإَنَّ',
    'stem': u'إَنَّ',
    'type': u'STOPWORD',
    'original': u'إَنَّ',
    'encletic': u''
}
STOPWORDS[u'وإن'] = {
    'word': u'وإن',
    'procletic': u'-وَ',
    'tags': u'أداة:المشبهة بليس:معطوف',
    'vocalized': u'وَإَنَّ',
    'stem': u'إَنَّ',
    'type': u'STOPWORD',
    'original': u'إَنَّ',
    'encletic': u''
}
STOPWORDS[u'فإن'] = {
    'word': u'فإن',
    'procletic': u'-فَ',
    'tags': u'أداة:المشبهة بليس:معطوف',
    'vocalized': u'فَإَنَّ',
    'stem': u'إَنَّ',
    'type': u'STOPWORD',
    'original': u'إَنَّ',
    'encletic': u''
}
STOPWORDS[u'وبإن'] = {
    'word': u'وبإن',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:المشبهة بليس:مجرور:معطوف',
    'vocalized': u'وَبِإَنَّ',
    'stem': u'إَنَّ',
    'type': u'STOPWORD',
    'original': u'إَنَّ',
    'encletic': u''
}
STOPWORDS[u'فبإن'] = {
    'word': u'فبإن',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:المشبهة بليس:مجرور:معطوف',
    'vocalized': u'فَبِإَنَّ',
    'stem': u'إَنَّ',
    'type': u'STOPWORD',
    'original': u'إَنَّ',
    'encletic': u''
}
STOPWORDS[u'وكإن'] = {
    'word': u'وكإن',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:المشبهة بليس:مجرور:معطوف',
    'vocalized': u'وَكَإَنَّ',
    'stem': u'إَنَّ',
    'type': u'STOPWORD',
    'original': u'إَنَّ',
    'encletic': u''
}
STOPWORDS[u'فكإن'] = {
    'word': u'فكإن',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:المشبهة بليس:مجرور:معطوف',
    'vocalized': u'فَكَإَنَّ',
    'stem': u'إَنَّ',
    'type': u'STOPWORD',
    'original': u'إَنَّ',
    'encletic': u''
}
STOPWORDS[u'ولإن'] = {
    'word': u'ولإن',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:المشبهة بليس:مجرور:معطوف',
    'vocalized': u'وَلِإَنَّ',
    'stem': u'إَنَّ',
    'type': u'STOPWORD',
    'original': u'إَنَّ',
    'encletic': u''
}
STOPWORDS[u'فلإن'] = {
    'word': u'فلإن',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:المشبهة بليس:مجرور:معطوف',
    'vocalized': u'فَلِإَنَّ',
    'stem': u'إَنَّ',
    'type': u'STOPWORD',
    'original': u'إَنَّ',
    'encletic': u''
}
STOPWORDS[u'لات'] = {
    'word': u'لَاتَ',
    'procletic': u'',
    'tags': u'أداة:المشبهة بليس',
    'vocalized': u'لَاتَ',
    'stem': u'لَاتَ',
    'type': u'STOPWORD',
    'original': u'لَاتَ',
    'encletic': u''
}
STOPWORDS[u'ولات'] = {
    'word': u'ولات',
    'procletic': u'-وَ',
    'tags': u'أداة:المشبهة بليس:معطوف',
    'vocalized': u'وَلَاتَ',
    'stem': u'لَاتَ',
    'type': u'STOPWORD',
    'original': u'لَاتَ',
    'encletic': u''
}
STOPWORDS[u'فلات'] = {
    'word': u'فلات',
    'procletic': u'-فَ',
    'tags': u'أداة:المشبهة بليس:معطوف',
    'vocalized': u'فَلَاتَ',
    'stem': u'لَاتَ',
    'type': u'STOPWORD',
    'original': u'لَاتَ',
    'encletic': u''
}
STOPWORDS[u'ما'] = {
    'word': u'مَا',
    'procletic': u'',
    'tags': u'أداة:المشبهة بليس',
    'vocalized': u'مَا',
    'stem': u'مَا',
    'type': u'STOPWORD',
    'original': u'مَا',
    'encletic': u''
}
STOPWORDS[u'وما'] = {
    'word': u'وما',
    'procletic': u'-وَ',
    'tags': u'أداة:المشبهة بليس:معطوف',
    'vocalized': u'وَمَا',
    'stem': u'مَا',
    'type': u'STOPWORD',
    'original': u'مَا',
    'encletic': u''
}
STOPWORDS[u'فما'] = {
    'word': u'فما',
    'procletic': u'-فَ',
    'tags': u'أداة:المشبهة بليس:معطوف',
    'vocalized': u'فَمَا',
    'stem': u'مَا',
    'type': u'STOPWORD',
    'original': u'مَا',
    'encletic': u''
}
STOPWORDS[u'لا'] = {
    'word': u'لَا',
    'procletic': u'',
    'tags': u'أداة:المشبهة بليس/نافية للجنس/ناهية',
    'vocalized': u'لَا',
    'stem': u'لَا',
    'type': u'STOPWORD',
    'original': u'لَا',
    'encletic': u''
}
STOPWORDS[u'ولا'] = {
    'word': u'ولا',
    'procletic': u'-وَ',
    'tags': u'أداة:المشبهة بليس/نافية للجنس/ناهية:معطوف',
    'vocalized': u'وَلَا',
    'stem': u'لَا',
    'type': u'STOPWORD',
    'original': u'لَا',
    'encletic': u''
}
STOPWORDS[u'فلا'] = {
    'word': u'فلا',
    'procletic': u'-فَ',
    'tags': u'أداة:المشبهة بليس/نافية للجنس/ناهية:معطوف',
    'vocalized': u'فَلَا',
    'stem': u'لَا',
    'type': u'STOPWORD',
    'original': u'لَا',
    'encletic': u''
}
STOPWORDS[u'ألا'] = {
    'word': u'ألا',
    'procletic': u'-أَ',
    'tags': u'أداة:المشبهة بليس/نافية للجنس/ناهية:مجرور',
    'vocalized': u'أَلَا',
    'stem': u'لَا',
    'type': u'STOPWORD',
    'original': u'لَا',
    'encletic': u''
}
STOPWORDS[u'أولا'] = {
    'word': u'أولا',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:المشبهة بليس/نافية للجنس/ناهية:معطوف:مجرور',
    'vocalized': u'أَوَلَا',
    'stem': u'لَا',
    'type': u'STOPWORD',
    'original': u'لَا',
    'encletic': u''
}
STOPWORDS[u'أفلا'] = {
    'word': u'أفلا',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:المشبهة بليس/نافية للجنس/ناهية:معطوف:مجرور',
    'vocalized': u'أَفَلَا',
    'stem': u'لَا',
    'type': u'STOPWORD',
    'original': u'لَا',
    'encletic': u''
}
STOPWORDS[u'أن'] = {
    'word': u'أَنَّ',
    'procletic': u'',
    'tags': u'أداة:إن و أخواتها',
    'vocalized': u'أَنَّ',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u''
}
STOPWORDS[u'بأن'] = {
    'word': u'بأن',
    'procletic': u'-بِ',
    'tags': u'أداة:إن و أخواتها:مجرور',
    'vocalized': u'بِأَنَّ',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u''
}
STOPWORDS[u'كأن'] = {
    'word': u'كأن',
    'procletic': u'-كَ',
    'tags': u'أداة:إن و أخواتها:مجرور',
    'vocalized': u'كَأَنَّ',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u''
}
STOPWORDS[u'لأن'] = {
    'word': u'لأن',
    'procletic': u'-لِ',
    'tags': u'أداة:إن و أخواتها:مجرور',
    'vocalized': u'لِأَنَّ',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u''
}
STOPWORDS[u'وأن'] = {
    'word': u'وأن',
    'procletic': u'-وَ',
    'tags': u'أداة:إن و أخواتها:معطوف',
    'vocalized': u'وَأَنَّ',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u''
}
STOPWORDS[u'فأن'] = {
    'word': u'فأن',
    'procletic': u'-فَ',
    'tags': u'أداة:إن و أخواتها:معطوف',
    'vocalized': u'فَأَنَّ',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u''
}
STOPWORDS[u'وبأن'] = {
    'word': u'وبأن',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:إن و أخواتها:مجرور:معطوف',
    'vocalized': u'وَبِأَنَّ',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u''
}
STOPWORDS[u'فبأن'] = {
    'word': u'فبأن',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:إن و أخواتها:مجرور:معطوف',
    'vocalized': u'فَبِأَنَّ',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u''
}
STOPWORDS[u'وكأن'] = {
    'word': u'وكأن',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:إن و أخواتها:مجرور:معطوف',
    'vocalized': u'وَكَأَنَّ',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u''
}
STOPWORDS[u'فكأن'] = {
    'word': u'فكأن',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:إن و أخواتها:مجرور:معطوف',
    'vocalized': u'فَكَأَنَّ',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u''
}
STOPWORDS[u'ولأن'] = {
    'word': u'ولأن',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:إن و أخواتها:مجرور:معطوف',
    'vocalized': u'وَلِأَنَّ',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u''
}
STOPWORDS[u'فلأن'] = {
    'word': u'فلأن',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:إن و أخواتها:مجرور:معطوف',
    'vocalized': u'فَلِأَنَّ',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u''
}
STOPWORDS[u'أني'] = {
    'word': u'أني',
    'procletic': u'',
    'tags': u'أداة:إن و أخواتها:مضاف',
    'vocalized': u'أَنَِّي',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'ِي'
}
STOPWORDS[u'أنك'] = {
    'word': u'أنك',
    'procletic': u'',
    'tags': u'أداة:إن و أخواتها:مضاف',
    'vocalized': u'أَنَّكَ',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'كَ'
}
STOPWORDS[u'أنك'] = {
    'word': u'أنك',
    'procletic': u'',
    'tags': u'أداة:إن و أخواتها:مضاف',
    'vocalized': u'أَنَّكِ',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'كِ'
}
STOPWORDS[u'أنه'] = {
    'word': u'أنه',
    'procletic': u'',
    'tags': u'أداة:إن و أخواتها:مضاف',
    'vocalized': u'أَنَّه',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'ه'
}
STOPWORDS[u'أنكم'] = {
    'word': u'أنكم',
    'procletic': u'',
    'tags': u'أداة:إن و أخواتها:مضاف',
    'vocalized': u'أَنَّكُمْ',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أنكن'] = {
    'word': u'أنكن',
    'procletic': u'',
    'tags': u'أداة:إن و أخواتها:مضاف',
    'vocalized': u'أَنَّكُنَّ',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أنها'] = {
    'word': u'أنها',
    'procletic': u'',
    'tags': u'أداة:إن و أخواتها:مضاف',
    'vocalized': u'أَنَّهَا',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'هَا'
}
STOPWORDS[u'أنهم'] = {
    'word': u'أنهم',
    'procletic': u'',
    'tags': u'أداة:إن و أخواتها:مضاف',
    'vocalized': u'أَنَّهُمْ',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أنهن'] = {
    'word': u'أنهن',
    'procletic': u'',
    'tags': u'أداة:إن و أخواتها:مضاف',
    'vocalized': u'أَنَّهُنَّ',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أننا'] = {
    'word': u'أننا',
    'procletic': u'',
    'tags': u'أداة:إن و أخواتها:مضاف',
    'vocalized': u'أَنَّنَا',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'نَا'
}
STOPWORDS[u'أنكما'] = {
    'word': u'أنكما',
    'procletic': u'',
    'tags': u'أداة:إن و أخواتها:مضاف',
    'vocalized': u'أَنَّكُمَا',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أنهما'] = {
    'word': u'أنهما',
    'procletic': u'',
    'tags': u'أداة:إن و أخواتها:مضاف',
    'vocalized': u'أَنَّهُمَا',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'بأني'] = {
    'word': u'بأني',
    'procletic': u'-بِ',
    'tags': u'أداة:إن و أخواتها:مجرور:مضاف',
    'vocalized': u'بِأَنَِّي',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'ِي'
}
STOPWORDS[u'بأنك'] = {
    'word': u'بأنك',
    'procletic': u'-بِ',
    'tags': u'أداة:إن و أخواتها:مجرور:مضاف',
    'vocalized': u'بِأَنَّكَ',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'كَ'
}
STOPWORDS[u'بأنك'] = {
    'word': u'بأنك',
    'procletic': u'-بِ',
    'tags': u'أداة:إن و أخواتها:مجرور:مضاف',
    'vocalized': u'بِأَنَّكِ',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'كِ'
}
STOPWORDS[u'بأنه'] = {
    'word': u'بأنه',
    'procletic': u'-بِ',
    'tags': u'أداة:إن و أخواتها:مجرور:مضاف',
    'vocalized': u'بِأَنَّه',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'ه'
}
STOPWORDS[u'بأنكم'] = {
    'word': u'بأنكم',
    'procletic': u'-بِ',
    'tags': u'أداة:إن و أخواتها:مجرور:مضاف',
    'vocalized': u'بِأَنَّكُمْ',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'بأنكن'] = {
    'word': u'بأنكن',
    'procletic': u'-بِ',
    'tags': u'أداة:إن و أخواتها:مجرور:مضاف',
    'vocalized': u'بِأَنَّكُنَّ',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'بأنها'] = {
    'word': u'بأنها',
    'procletic': u'-بِ',
    'tags': u'أداة:إن و أخواتها:مجرور:مضاف',
    'vocalized': u'بِأَنَّهَا',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'هَا'
}
STOPWORDS[u'بأنهم'] = {
    'word': u'بأنهم',
    'procletic': u'-بِ',
    'tags': u'أداة:إن و أخواتها:مجرور:مضاف',
    'vocalized': u'بِأَنَّهُمْ',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'بأنهن'] = {
    'word': u'بأنهن',
    'procletic': u'-بِ',
    'tags': u'أداة:إن و أخواتها:مجرور:مضاف',
    'vocalized': u'بِأَنَّهُنَّ',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'بأننا'] = {
    'word': u'بأننا',
    'procletic': u'-بِ',
    'tags': u'أداة:إن و أخواتها:مجرور:مضاف',
    'vocalized': u'بِأَنَّنَا',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'نَا'
}
STOPWORDS[u'بأنكما'] = {
    'word': u'بأنكما',
    'procletic': u'-بِ',
    'tags': u'أداة:إن و أخواتها:مجرور:مضاف',
    'vocalized': u'بِأَنَّكُمَا',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'بأنهما'] = {
    'word': u'بأنهما',
    'procletic': u'-بِ',
    'tags': u'أداة:إن و أخواتها:مجرور:مضاف',
    'vocalized': u'بِأَنَّهُمَا',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'كأني'] = {
    'word': u'كأني',
    'procletic': u'-كَ',
    'tags': u'أداة:إن و أخواتها:مجرور:مضاف',
    'vocalized': u'كَأَنَِّي',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'ِي'
}
STOPWORDS[u'كأنك'] = {
    'word': u'كأنك',
    'procletic': u'-كَ',
    'tags': u'أداة:إن و أخواتها:مجرور:مضاف',
    'vocalized': u'كَأَنَّكَ',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'كَ'
}
STOPWORDS[u'كأنك'] = {
    'word': u'كأنك',
    'procletic': u'-كَ',
    'tags': u'أداة:إن و أخواتها:مجرور:مضاف',
    'vocalized': u'كَأَنَّكِ',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'كِ'
}
STOPWORDS[u'كأنه'] = {
    'word': u'كأنه',
    'procletic': u'-كَ',
    'tags': u'أداة:إن و أخواتها:مجرور:مضاف',
    'vocalized': u'كَأَنَّه',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'ه'
}
STOPWORDS[u'كأنكم'] = {
    'word': u'كأنكم',
    'procletic': u'-كَ',
    'tags': u'أداة:إن و أخواتها:مجرور:مضاف',
    'vocalized': u'كَأَنَّكُمْ',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'كأنكن'] = {
    'word': u'كأنكن',
    'procletic': u'-كَ',
    'tags': u'أداة:إن و أخواتها:مجرور:مضاف',
    'vocalized': u'كَأَنَّكُنَّ',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'كأنها'] = {
    'word': u'كأنها',
    'procletic': u'-كَ',
    'tags': u'أداة:إن و أخواتها:مجرور:مضاف',
    'vocalized': u'كَأَنَّهَا',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'هَا'
}
STOPWORDS[u'كأنهم'] = {
    'word': u'كأنهم',
    'procletic': u'-كَ',
    'tags': u'أداة:إن و أخواتها:مجرور:مضاف',
    'vocalized': u'كَأَنَّهُمْ',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'كأنهن'] = {
    'word': u'كأنهن',
    'procletic': u'-كَ',
    'tags': u'أداة:إن و أخواتها:مجرور:مضاف',
    'vocalized': u'كَأَنَّهُنَّ',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'كأننا'] = {
    'word': u'كأننا',
    'procletic': u'-كَ',
    'tags': u'أداة:إن و أخواتها:مجرور:مضاف',
    'vocalized': u'كَأَنَّنَا',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'نَا'
}
STOPWORDS[u'كأنكما'] = {
    'word': u'كأنكما',
    'procletic': u'-كَ',
    'tags': u'أداة:إن و أخواتها:مجرور:مضاف',
    'vocalized': u'كَأَنَّكُمَا',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'كأنهما'] = {
    'word': u'كأنهما',
    'procletic': u'-كَ',
    'tags': u'أداة:إن و أخواتها:مجرور:مضاف',
    'vocalized': u'كَأَنَّهُمَا',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'لأني'] = {
    'word': u'لأني',
    'procletic': u'-لِ',
    'tags': u'أداة:إن و أخواتها:مجرور:مضاف',
    'vocalized': u'لِأَنَِّي',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'ِي'
}
STOPWORDS[u'لأنك'] = {
    'word': u'لأنك',
    'procletic': u'-لِ',
    'tags': u'أداة:إن و أخواتها:مجرور:مضاف',
    'vocalized': u'لِأَنَّكَ',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'كَ'
}
STOPWORDS[u'لأنك'] = {
    'word': u'لأنك',
    'procletic': u'-لِ',
    'tags': u'أداة:إن و أخواتها:مجرور:مضاف',
    'vocalized': u'لِأَنَّكِ',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'كِ'
}
STOPWORDS[u'لأنه'] = {
    'word': u'لأنه',
    'procletic': u'-لِ',
    'tags': u'أداة:إن و أخواتها:مجرور:مضاف',
    'vocalized': u'لِأَنَّه',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'ه'
}
STOPWORDS[u'لأنكم'] = {
    'word': u'لأنكم',
    'procletic': u'-لِ',
    'tags': u'أداة:إن و أخواتها:مجرور:مضاف',
    'vocalized': u'لِأَنَّكُمْ',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'لأنكن'] = {
    'word': u'لأنكن',
    'procletic': u'-لِ',
    'tags': u'أداة:إن و أخواتها:مجرور:مضاف',
    'vocalized': u'لِأَنَّكُنَّ',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'لأنها'] = {
    'word': u'لأنها',
    'procletic': u'-لِ',
    'tags': u'أداة:إن و أخواتها:مجرور:مضاف',
    'vocalized': u'لِأَنَّهَا',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'هَا'
}
STOPWORDS[u'لأنهم'] = {
    'word': u'لأنهم',
    'procletic': u'-لِ',
    'tags': u'أداة:إن و أخواتها:مجرور:مضاف',
    'vocalized': u'لِأَنَّهُمْ',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'لأنهن'] = {
    'word': u'لأنهن',
    'procletic': u'-لِ',
    'tags': u'أداة:إن و أخواتها:مجرور:مضاف',
    'vocalized': u'لِأَنَّهُنَّ',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'لأننا'] = {
    'word': u'لأننا',
    'procletic': u'-لِ',
    'tags': u'أداة:إن و أخواتها:مجرور:مضاف',
    'vocalized': u'لِأَنَّنَا',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'نَا'
}
STOPWORDS[u'لأنكما'] = {
    'word': u'لأنكما',
    'procletic': u'-لِ',
    'tags': u'أداة:إن و أخواتها:مجرور:مضاف',
    'vocalized': u'لِأَنَّكُمَا',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'لأنهما'] = {
    'word': u'لأنهما',
    'procletic': u'-لِ',
    'tags': u'أداة:إن و أخواتها:مجرور:مضاف',
    'vocalized': u'لِأَنَّهُمَا',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'وأني'] = {
    'word': u'وأني',
    'procletic': u'-وَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'وَأَنَِّي',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'ِي'
}
STOPWORDS[u'وأنك'] = {
    'word': u'وأنك',
    'procletic': u'-وَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'وَأَنَّكَ',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'كَ'
}
STOPWORDS[u'وأنك'] = {
    'word': u'وأنك',
    'procletic': u'-وَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'وَأَنَّكِ',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'كِ'
}
STOPWORDS[u'وأنه'] = {
    'word': u'وأنه',
    'procletic': u'-وَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'وَأَنَّه',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'ه'
}
STOPWORDS[u'وأنكم'] = {
    'word': u'وأنكم',
    'procletic': u'-وَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'وَأَنَّكُمْ',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'وأنكن'] = {
    'word': u'وأنكن',
    'procletic': u'-وَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'وَأَنَّكُنَّ',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'وأنها'] = {
    'word': u'وأنها',
    'procletic': u'-وَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'وَأَنَّهَا',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'هَا'
}
STOPWORDS[u'وأنهم'] = {
    'word': u'وأنهم',
    'procletic': u'-وَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'وَأَنَّهُمْ',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'وأنهن'] = {
    'word': u'وأنهن',
    'procletic': u'-وَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'وَأَنَّهُنَّ',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'وأننا'] = {
    'word': u'وأننا',
    'procletic': u'-وَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'وَأَنَّنَا',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'نَا'
}
STOPWORDS[u'وأنكما'] = {
    'word': u'وأنكما',
    'procletic': u'-وَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'وَأَنَّكُمَا',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'وأنهما'] = {
    'word': u'وأنهما',
    'procletic': u'-وَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'وَأَنَّهُمَا',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فأني'] = {
    'word': u'فأني',
    'procletic': u'-فَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'فَأَنَِّي',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'ِي'
}
STOPWORDS[u'فأنك'] = {
    'word': u'فأنك',
    'procletic': u'-فَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'فَأَنَّكَ',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'كَ'
}
STOPWORDS[u'فأنك'] = {
    'word': u'فأنك',
    'procletic': u'-فَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'فَأَنَّكِ',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'كِ'
}
STOPWORDS[u'فأنه'] = {
    'word': u'فأنه',
    'procletic': u'-فَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'فَأَنَّه',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'ه'
}
STOPWORDS[u'فأنكم'] = {
    'word': u'فأنكم',
    'procletic': u'-فَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'فَأَنَّكُمْ',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فأنكن'] = {
    'word': u'فأنكن',
    'procletic': u'-فَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'فَأَنَّكُنَّ',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فأنها'] = {
    'word': u'فأنها',
    'procletic': u'-فَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'فَأَنَّهَا',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'هَا'
}
STOPWORDS[u'فأنهم'] = {
    'word': u'فأنهم',
    'procletic': u'-فَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'فَأَنَّهُمْ',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فأنهن'] = {
    'word': u'فأنهن',
    'procletic': u'-فَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'فَأَنَّهُنَّ',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فأننا'] = {
    'word': u'فأننا',
    'procletic': u'-فَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'فَأَنَّنَا',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'نَا'
}
STOPWORDS[u'فأنكما'] = {
    'word': u'فأنكما',
    'procletic': u'-فَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'فَأَنَّكُمَا',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فأنهما'] = {
    'word': u'فأنهما',
    'procletic': u'-فَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'فَأَنَّهُمَا',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'وبأني'] = {
    'word': u'وبأني',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:إن و أخواتها:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِأَنَِّي',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'ِي'
}
STOPWORDS[u'وبأنك'] = {
    'word': u'وبأنك',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:إن و أخواتها:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِأَنَّكَ',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'كَ'
}
STOPWORDS[u'وبأنك'] = {
    'word': u'وبأنك',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:إن و أخواتها:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِأَنَّكِ',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'كِ'
}
STOPWORDS[u'وبأنه'] = {
    'word': u'وبأنه',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:إن و أخواتها:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِأَنَّه',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'ه'
}
STOPWORDS[u'وبأنكم'] = {
    'word': u'وبأنكم',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:إن و أخواتها:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِأَنَّكُمْ',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'وبأنكن'] = {
    'word': u'وبأنكن',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:إن و أخواتها:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِأَنَّكُنَّ',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'وبأنها'] = {
    'word': u'وبأنها',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:إن و أخواتها:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِأَنَّهَا',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'هَا'
}
STOPWORDS[u'وبأنهم'] = {
    'word': u'وبأنهم',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:إن و أخواتها:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِأَنَّهُمْ',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'وبأنهن'] = {
    'word': u'وبأنهن',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:إن و أخواتها:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِأَنَّهُنَّ',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'وبأننا'] = {
    'word': u'وبأننا',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:إن و أخواتها:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِأَنَّنَا',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'نَا'
}
STOPWORDS[u'وبأنكما'] = {
    'word': u'وبأنكما',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:إن و أخواتها:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِأَنَّكُمَا',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'وبأنهما'] = {
    'word': u'وبأنهما',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:إن و أخواتها:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِأَنَّهُمَا',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فبأني'] = {
    'word': u'فبأني',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:إن و أخواتها:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِأَنَِّي',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'ِي'
}
STOPWORDS[u'فبأنك'] = {
    'word': u'فبأنك',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:إن و أخواتها:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِأَنَّكَ',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'كَ'
}
STOPWORDS[u'فبأنك'] = {
    'word': u'فبأنك',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:إن و أخواتها:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِأَنَّكِ',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'كِ'
}
STOPWORDS[u'فبأنه'] = {
    'word': u'فبأنه',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:إن و أخواتها:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِأَنَّه',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'ه'
}
STOPWORDS[u'فبأنكم'] = {
    'word': u'فبأنكم',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:إن و أخواتها:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِأَنَّكُمْ',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فبأنكن'] = {
    'word': u'فبأنكن',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:إن و أخواتها:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِأَنَّكُنَّ',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فبأنها'] = {
    'word': u'فبأنها',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:إن و أخواتها:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِأَنَّهَا',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'هَا'
}
STOPWORDS[u'فبأنهم'] = {
    'word': u'فبأنهم',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:إن و أخواتها:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِأَنَّهُمْ',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فبأنهن'] = {
    'word': u'فبأنهن',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:إن و أخواتها:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِأَنَّهُنَّ',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فبأننا'] = {
    'word': u'فبأننا',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:إن و أخواتها:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِأَنَّنَا',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'نَا'
}
STOPWORDS[u'فبأنكما'] = {
    'word': u'فبأنكما',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:إن و أخواتها:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِأَنَّكُمَا',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فبأنهما'] = {
    'word': u'فبأنهما',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:إن و أخواتها:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِأَنَّهُمَا',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'وكأني'] = {
    'word': u'وكأني',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:إن و أخواتها:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَأَنَِّي',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'ِي'
}
STOPWORDS[u'وكأنك'] = {
    'word': u'وكأنك',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:إن و أخواتها:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَأَنَّكَ',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'كَ'
}
STOPWORDS[u'وكأنك'] = {
    'word': u'وكأنك',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:إن و أخواتها:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَأَنَّكِ',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'كِ'
}
STOPWORDS[u'وكأنه'] = {
    'word': u'وكأنه',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:إن و أخواتها:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَأَنَّه',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'ه'
}
STOPWORDS[u'وكأنكم'] = {
    'word': u'وكأنكم',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:إن و أخواتها:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَأَنَّكُمْ',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'وكأنكن'] = {
    'word': u'وكأنكن',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:إن و أخواتها:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَأَنَّكُنَّ',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'وكأنها'] = {
    'word': u'وكأنها',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:إن و أخواتها:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَأَنَّهَا',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'هَا'
}
STOPWORDS[u'وكأنهم'] = {
    'word': u'وكأنهم',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:إن و أخواتها:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَأَنَّهُمْ',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'وكأنهن'] = {
    'word': u'وكأنهن',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:إن و أخواتها:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَأَنَّهُنَّ',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'وكأننا'] = {
    'word': u'وكأننا',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:إن و أخواتها:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَأَنَّنَا',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'نَا'
}
STOPWORDS[u'وكأنكما'] = {
    'word': u'وكأنكما',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:إن و أخواتها:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَأَنَّكُمَا',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'وكأنهما'] = {
    'word': u'وكأنهما',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:إن و أخواتها:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَأَنَّهُمَا',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فكأني'] = {
    'word': u'فكأني',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:إن و أخواتها:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَأَنَِّي',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'ِي'
}
STOPWORDS[u'فكأنك'] = {
    'word': u'فكأنك',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:إن و أخواتها:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَأَنَّكَ',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'كَ'
}
STOPWORDS[u'فكأنك'] = {
    'word': u'فكأنك',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:إن و أخواتها:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَأَنَّكِ',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'كِ'
}
STOPWORDS[u'فكأنه'] = {
    'word': u'فكأنه',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:إن و أخواتها:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَأَنَّه',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'ه'
}
STOPWORDS[u'فكأنكم'] = {
    'word': u'فكأنكم',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:إن و أخواتها:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَأَنَّكُمْ',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فكأنكن'] = {
    'word': u'فكأنكن',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:إن و أخواتها:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَأَنَّكُنَّ',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فكأنها'] = {
    'word': u'فكأنها',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:إن و أخواتها:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَأَنَّهَا',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'هَا'
}
STOPWORDS[u'فكأنهم'] = {
    'word': u'فكأنهم',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:إن و أخواتها:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَأَنَّهُمْ',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فكأنهن'] = {
    'word': u'فكأنهن',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:إن و أخواتها:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَأَنَّهُنَّ',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فكأننا'] = {
    'word': u'فكأننا',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:إن و أخواتها:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَأَنَّنَا',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'نَا'
}
STOPWORDS[u'فكأنكما'] = {
    'word': u'فكأنكما',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:إن و أخواتها:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَأَنَّكُمَا',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فكأنهما'] = {
    'word': u'فكأنهما',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:إن و أخواتها:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَأَنَّهُمَا',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ولأني'] = {
    'word': u'ولأني',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:إن و أخواتها:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِأَنَِّي',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'ِي'
}
STOPWORDS[u'ولأنك'] = {
    'word': u'ولأنك',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:إن و أخواتها:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِأَنَّكَ',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'كَ'
}
STOPWORDS[u'ولأنك'] = {
    'word': u'ولأنك',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:إن و أخواتها:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِأَنَّكِ',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'كِ'
}
STOPWORDS[u'ولأنه'] = {
    'word': u'ولأنه',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:إن و أخواتها:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِأَنَّه',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'ه'
}
STOPWORDS[u'ولأنكم'] = {
    'word': u'ولأنكم',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:إن و أخواتها:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِأَنَّكُمْ',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ولأنكن'] = {
    'word': u'ولأنكن',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:إن و أخواتها:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِأَنَّكُنَّ',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ولأنها'] = {
    'word': u'ولأنها',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:إن و أخواتها:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِأَنَّهَا',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'هَا'
}
STOPWORDS[u'ولأنهم'] = {
    'word': u'ولأنهم',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:إن و أخواتها:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِأَنَّهُمْ',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ولأنهن'] = {
    'word': u'ولأنهن',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:إن و أخواتها:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِأَنَّهُنَّ',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ولأننا'] = {
    'word': u'ولأننا',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:إن و أخواتها:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِأَنَّنَا',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'نَا'
}
STOPWORDS[u'ولأنكما'] = {
    'word': u'ولأنكما',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:إن و أخواتها:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِأَنَّكُمَا',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ولأنهما'] = {
    'word': u'ولأنهما',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:إن و أخواتها:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِأَنَّهُمَا',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فلأني'] = {
    'word': u'فلأني',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:إن و أخواتها:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِأَنَِّي',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'ِي'
}
STOPWORDS[u'فلأنك'] = {
    'word': u'فلأنك',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:إن و أخواتها:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِأَنَّكَ',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'كَ'
}
STOPWORDS[u'فلأنك'] = {
    'word': u'فلأنك',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:إن و أخواتها:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِأَنَّكِ',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'كِ'
}
STOPWORDS[u'فلأنه'] = {
    'word': u'فلأنه',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:إن و أخواتها:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِأَنَّه',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'ه'
}
STOPWORDS[u'فلأنكم'] = {
    'word': u'فلأنكم',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:إن و أخواتها:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِأَنَّكُمْ',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فلأنكن'] = {
    'word': u'فلأنكن',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:إن و أخواتها:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِأَنَّكُنَّ',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فلأنها'] = {
    'word': u'فلأنها',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:إن و أخواتها:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِأَنَّهَا',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'هَا'
}
STOPWORDS[u'فلأنهم'] = {
    'word': u'فلأنهم',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:إن و أخواتها:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِأَنَّهُمْ',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فلأنهن'] = {
    'word': u'فلأنهن',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:إن و أخواتها:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِأَنَّهُنَّ',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فلأننا'] = {
    'word': u'فلأننا',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:إن و أخواتها:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِأَنَّنَا',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'نَا'
}
STOPWORDS[u'فلأنكما'] = {
    'word': u'فلأنكما',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:إن و أخواتها:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِأَنَّكُمَا',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فلأنهما'] = {
    'word': u'فلأنهما',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:إن و أخواتها:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِأَنَّهُمَا',
    'stem': u'أَنَّ',
    'type': u'STOPWORD',
    'original': u'أَنَّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'إن'] = {
    'word': u'إِنَّ',
    'procletic': u'',
    'tags': u'أداة:إن و أخواتها',
    'vocalized': u'إِنَّ',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u''
}
STOPWORDS[u'لإن'] = {
    'word': u'لإن',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور',
    'vocalized': u'لَإِنَّ',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u''
}
STOPWORDS[u'وإن'] = {
    'word': u'وإن',
    'procletic': u'-وَ',
    'tags': u'أداة:إن و أخواتها:معطوف',
    'vocalized': u'وَإِنَّ',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u''
}
STOPWORDS[u'فإن'] = {
    'word': u'فإن',
    'procletic': u'-فَ',
    'tags': u'أداة:إن و أخواتها:معطوف',
    'vocalized': u'فَإِنَّ',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u''
}
STOPWORDS[u'ولإن'] = {
    'word': u'ولإن',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'وَلَإِنَّ',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u''
}
STOPWORDS[u'فلإن'] = {
    'word': u'فلإن',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'فَلَإِنَّ',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u''
}
STOPWORDS[u'أإن'] = {
    'word': u'أإن',
    'procletic': u'-أَ',
    'tags': u'أداة:إن و أخواتها:مجرور',
    'vocalized': u'أَإِنَّ',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u''
}
STOPWORDS[u'ألإن'] = {
    'word': u'ألإن',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور',
    'vocalized': u'أَلَإِنَّ',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u''
}
STOPWORDS[u'أوإن'] = {
    'word': u'أوإن',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مجرور',
    'vocalized': u'أَوَإِنَّ',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u''
}
STOPWORDS[u'أفإن'] = {
    'word': u'أفإن',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مجرور',
    'vocalized': u'أَفَإِنَّ',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u''
}
STOPWORDS[u'أولإن'] = {
    'word': u'أولإن',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَلَإِنَّ',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u''
}
STOPWORDS[u'أفلإن'] = {
    'word': u'أفلإن',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَلَإِنَّ',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u''
}
STOPWORDS[u'إني'] = {
    'word': u'إني',
    'procletic': u'',
    'tags': u'أداة:إن و أخواتها:مضاف',
    'vocalized': u'إِنَِّي',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'ِي'
}
STOPWORDS[u'إنك'] = {
    'word': u'إنك',
    'procletic': u'',
    'tags': u'أداة:إن و أخواتها:مضاف',
    'vocalized': u'إِنَّكَ',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'كَ'
}
STOPWORDS[u'إنك'] = {
    'word': u'إنك',
    'procletic': u'',
    'tags': u'أداة:إن و أخواتها:مضاف',
    'vocalized': u'إِنَّكِ',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'كِ'
}
STOPWORDS[u'إنه'] = {
    'word': u'إنه',
    'procletic': u'',
    'tags': u'أداة:إن و أخواتها:مضاف',
    'vocalized': u'إِنَّه',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'ه'
}
STOPWORDS[u'إنكم'] = {
    'word': u'إنكم',
    'procletic': u'',
    'tags': u'أداة:إن و أخواتها:مضاف',
    'vocalized': u'إِنَّكُمْ',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'إنكن'] = {
    'word': u'إنكن',
    'procletic': u'',
    'tags': u'أداة:إن و أخواتها:مضاف',
    'vocalized': u'إِنَّكُنَّ',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'إنها'] = {
    'word': u'إنها',
    'procletic': u'',
    'tags': u'أداة:إن و أخواتها:مضاف',
    'vocalized': u'إِنَّهَا',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'هَا'
}
STOPWORDS[u'إنهم'] = {
    'word': u'إنهم',
    'procletic': u'',
    'tags': u'أداة:إن و أخواتها:مضاف',
    'vocalized': u'إِنَّهُمْ',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'إنهن'] = {
    'word': u'إنهن',
    'procletic': u'',
    'tags': u'أداة:إن و أخواتها:مضاف',
    'vocalized': u'إِنَّهُنَّ',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'إننا'] = {
    'word': u'إننا',
    'procletic': u'',
    'tags': u'أداة:إن و أخواتها:مضاف',
    'vocalized': u'إِنَّنَا',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'نَا'
}
STOPWORDS[u'إنكما'] = {
    'word': u'إنكما',
    'procletic': u'',
    'tags': u'أداة:إن و أخواتها:مضاف',
    'vocalized': u'إِنَّكُمَا',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'إنهما'] = {
    'word': u'إنهما',
    'procletic': u'',
    'tags': u'أداة:إن و أخواتها:مضاف',
    'vocalized': u'إِنَّهُمَا',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'لإني'] = {
    'word': u'لإني',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَإِنَِّي',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'ِي'
}
STOPWORDS[u'لإنك'] = {
    'word': u'لإنك',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَإِنَّكَ',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'كَ'
}
STOPWORDS[u'لإنك'] = {
    'word': u'لإنك',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَإِنَّكِ',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'كِ'
}
STOPWORDS[u'لإنه'] = {
    'word': u'لإنه',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَإِنَّه',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'ه'
}
STOPWORDS[u'لإنكم'] = {
    'word': u'لإنكم',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَإِنَّكُمْ',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'لإنكن'] = {
    'word': u'لإنكن',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَإِنَّكُنَّ',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'لإنها'] = {
    'word': u'لإنها',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَإِنَّهَا',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'هَا'
}
STOPWORDS[u'لإنهم'] = {
    'word': u'لإنهم',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَإِنَّهُمْ',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'لإنهن'] = {
    'word': u'لإنهن',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَإِنَّهُنَّ',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'لإننا'] = {
    'word': u'لإننا',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَإِنَّنَا',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'نَا'
}
STOPWORDS[u'لإنكما'] = {
    'word': u'لإنكما',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَإِنَّكُمَا',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'لإنهما'] = {
    'word': u'لإنهما',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَإِنَّهُمَا',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'وإني'] = {
    'word': u'وإني',
    'procletic': u'-وَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'وَإِنَِّي',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'ِي'
}
STOPWORDS[u'وإنك'] = {
    'word': u'وإنك',
    'procletic': u'-وَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'وَإِنَّكَ',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'كَ'
}
STOPWORDS[u'وإنك'] = {
    'word': u'وإنك',
    'procletic': u'-وَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'وَإِنَّكِ',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'كِ'
}
STOPWORDS[u'وإنه'] = {
    'word': u'وإنه',
    'procletic': u'-وَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'وَإِنَّه',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'ه'
}
STOPWORDS[u'وإنكم'] = {
    'word': u'وإنكم',
    'procletic': u'-وَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'وَإِنَّكُمْ',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'وإنكن'] = {
    'word': u'وإنكن',
    'procletic': u'-وَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'وَإِنَّكُنَّ',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'وإنها'] = {
    'word': u'وإنها',
    'procletic': u'-وَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'وَإِنَّهَا',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'هَا'
}
STOPWORDS[u'وإنهم'] = {
    'word': u'وإنهم',
    'procletic': u'-وَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'وَإِنَّهُمْ',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'وإنهن'] = {
    'word': u'وإنهن',
    'procletic': u'-وَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'وَإِنَّهُنَّ',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'وإننا'] = {
    'word': u'وإننا',
    'procletic': u'-وَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'وَإِنَّنَا',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'نَا'
}
STOPWORDS[u'وإنكما'] = {
    'word': u'وإنكما',
    'procletic': u'-وَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'وَإِنَّكُمَا',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'وإنهما'] = {
    'word': u'وإنهما',
    'procletic': u'-وَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'وَإِنَّهُمَا',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فإني'] = {
    'word': u'فإني',
    'procletic': u'-فَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'فَإِنَِّي',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'ِي'
}
STOPWORDS[u'فإنك'] = {
    'word': u'فإنك',
    'procletic': u'-فَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'فَإِنَّكَ',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'كَ'
}
STOPWORDS[u'فإنك'] = {
    'word': u'فإنك',
    'procletic': u'-فَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'فَإِنَّكِ',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'كِ'
}
STOPWORDS[u'فإنه'] = {
    'word': u'فإنه',
    'procletic': u'-فَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'فَإِنَّه',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'ه'
}
STOPWORDS[u'فإنكم'] = {
    'word': u'فإنكم',
    'procletic': u'-فَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'فَإِنَّكُمْ',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فإنكن'] = {
    'word': u'فإنكن',
    'procletic': u'-فَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'فَإِنَّكُنَّ',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فإنها'] = {
    'word': u'فإنها',
    'procletic': u'-فَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'فَإِنَّهَا',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'هَا'
}
STOPWORDS[u'فإنهم'] = {
    'word': u'فإنهم',
    'procletic': u'-فَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'فَإِنَّهُمْ',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فإنهن'] = {
    'word': u'فإنهن',
    'procletic': u'-فَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'فَإِنَّهُنَّ',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فإننا'] = {
    'word': u'فإننا',
    'procletic': u'-فَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'فَإِنَّنَا',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'نَا'
}
STOPWORDS[u'فإنكما'] = {
    'word': u'فإنكما',
    'procletic': u'-فَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'فَإِنَّكُمَا',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فإنهما'] = {
    'word': u'فإنهما',
    'procletic': u'-فَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'فَإِنَّهُمَا',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ولإني'] = {
    'word': u'ولإني',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَإِنَِّي',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'ِي'
}
STOPWORDS[u'ولإنك'] = {
    'word': u'ولإنك',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَإِنَّكَ',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'كَ'
}
STOPWORDS[u'ولإنك'] = {
    'word': u'ولإنك',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَإِنَّكِ',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'كِ'
}
STOPWORDS[u'ولإنه'] = {
    'word': u'ولإنه',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَإِنَّه',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'ه'
}
STOPWORDS[u'ولإنكم'] = {
    'word': u'ولإنكم',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَإِنَّكُمْ',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ولإنكن'] = {
    'word': u'ولإنكن',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَإِنَّكُنَّ',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ولإنها'] = {
    'word': u'ولإنها',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَإِنَّهَا',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'هَا'
}
STOPWORDS[u'ولإنهم'] = {
    'word': u'ولإنهم',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَإِنَّهُمْ',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ولإنهن'] = {
    'word': u'ولإنهن',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَإِنَّهُنَّ',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ولإننا'] = {
    'word': u'ولإننا',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَإِنَّنَا',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'نَا'
}
STOPWORDS[u'ولإنكما'] = {
    'word': u'ولإنكما',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَإِنَّكُمَا',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ولإنهما'] = {
    'word': u'ولإنهما',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَإِنَّهُمَا',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فلإني'] = {
    'word': u'فلإني',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَإِنَِّي',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'ِي'
}
STOPWORDS[u'فلإنك'] = {
    'word': u'فلإنك',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَإِنَّكَ',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'كَ'
}
STOPWORDS[u'فلإنك'] = {
    'word': u'فلإنك',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَإِنَّكِ',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'كِ'
}
STOPWORDS[u'فلإنه'] = {
    'word': u'فلإنه',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَإِنَّه',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'ه'
}
STOPWORDS[u'فلإنكم'] = {
    'word': u'فلإنكم',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَإِنَّكُمْ',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فلإنكن'] = {
    'word': u'فلإنكن',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَإِنَّكُنَّ',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فلإنها'] = {
    'word': u'فلإنها',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَإِنَّهَا',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'هَا'
}
STOPWORDS[u'فلإنهم'] = {
    'word': u'فلإنهم',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَإِنَّهُمْ',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فلإنهن'] = {
    'word': u'فلإنهن',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَإِنَّهُنَّ',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فلإننا'] = {
    'word': u'فلإننا',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَإِنَّنَا',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'نَا'
}
STOPWORDS[u'فلإنكما'] = {
    'word': u'فلإنكما',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَإِنَّكُمَا',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فلإنهما'] = {
    'word': u'فلإنهما',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَإِنَّهُمَا',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أإني'] = {
    'word': u'أإني',
    'procletic': u'-أَ',
    'tags': u'أداة:إن و أخواتها:مجرور:مضاف',
    'vocalized': u'أَإِنَِّي',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'ِي'
}
STOPWORDS[u'أإنك'] = {
    'word': u'أإنك',
    'procletic': u'-أَ',
    'tags': u'أداة:إن و أخواتها:مجرور:مضاف',
    'vocalized': u'أَإِنَّكَ',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'كَ'
}
STOPWORDS[u'أإنك'] = {
    'word': u'أإنك',
    'procletic': u'-أَ',
    'tags': u'أداة:إن و أخواتها:مجرور:مضاف',
    'vocalized': u'أَإِنَّكِ',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'كِ'
}
STOPWORDS[u'أإنه'] = {
    'word': u'أإنه',
    'procletic': u'-أَ',
    'tags': u'أداة:إن و أخواتها:مجرور:مضاف',
    'vocalized': u'أَإِنَّه',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'ه'
}
STOPWORDS[u'أإنكم'] = {
    'word': u'أإنكم',
    'procletic': u'-أَ',
    'tags': u'أداة:إن و أخواتها:مجرور:مضاف',
    'vocalized': u'أَإِنَّكُمْ',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أإنكن'] = {
    'word': u'أإنكن',
    'procletic': u'-أَ',
    'tags': u'أداة:إن و أخواتها:مجرور:مضاف',
    'vocalized': u'أَإِنَّكُنَّ',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أإنها'] = {
    'word': u'أإنها',
    'procletic': u'-أَ',
    'tags': u'أداة:إن و أخواتها:مجرور:مضاف',
    'vocalized': u'أَإِنَّهَا',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'هَا'
}
STOPWORDS[u'أإنهم'] = {
    'word': u'أإنهم',
    'procletic': u'-أَ',
    'tags': u'أداة:إن و أخواتها:مجرور:مضاف',
    'vocalized': u'أَإِنَّهُمْ',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أإنهن'] = {
    'word': u'أإنهن',
    'procletic': u'-أَ',
    'tags': u'أداة:إن و أخواتها:مجرور:مضاف',
    'vocalized': u'أَإِنَّهُنَّ',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أإننا'] = {
    'word': u'أإننا',
    'procletic': u'-أَ',
    'tags': u'أداة:إن و أخواتها:مجرور:مضاف',
    'vocalized': u'أَإِنَّنَا',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'نَا'
}
STOPWORDS[u'أإنكما'] = {
    'word': u'أإنكما',
    'procletic': u'-أَ',
    'tags': u'أداة:إن و أخواتها:مجرور:مضاف',
    'vocalized': u'أَإِنَّكُمَا',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أإنهما'] = {
    'word': u'أإنهما',
    'procletic': u'-أَ',
    'tags': u'أداة:إن و أخواتها:مجرور:مضاف',
    'vocalized': u'أَإِنَّهُمَا',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ألإني'] = {
    'word': u'ألإني',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَإِنَِّي',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'ِي'
}
STOPWORDS[u'ألإنك'] = {
    'word': u'ألإنك',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَإِنَّكَ',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'كَ'
}
STOPWORDS[u'ألإنك'] = {
    'word': u'ألإنك',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَإِنَّكِ',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'كِ'
}
STOPWORDS[u'ألإنه'] = {
    'word': u'ألإنه',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَإِنَّه',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'ه'
}
STOPWORDS[u'ألإنكم'] = {
    'word': u'ألإنكم',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَإِنَّكُمْ',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ألإنكن'] = {
    'word': u'ألإنكن',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَإِنَّكُنَّ',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ألإنها'] = {
    'word': u'ألإنها',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَإِنَّهَا',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'هَا'
}
STOPWORDS[u'ألإنهم'] = {
    'word': u'ألإنهم',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَإِنَّهُمْ',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ألإنهن'] = {
    'word': u'ألإنهن',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَإِنَّهُنَّ',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ألإننا'] = {
    'word': u'ألإننا',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَإِنَّنَا',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'نَا'
}
STOPWORDS[u'ألإنكما'] = {
    'word': u'ألإنكما',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَإِنَّكُمَا',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ألإنهما'] = {
    'word': u'ألإنهما',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَإِنَّهُمَا',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أوإني'] = {
    'word': u'أوإني',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَإِنَِّي',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'ِي'
}
STOPWORDS[u'أوإنك'] = {
    'word': u'أوإنك',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَإِنَّكَ',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'كَ'
}
STOPWORDS[u'أوإنك'] = {
    'word': u'أوإنك',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَإِنَّكِ',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'كِ'
}
STOPWORDS[u'أوإنه'] = {
    'word': u'أوإنه',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَإِنَّه',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'ه'
}
STOPWORDS[u'أوإنكم'] = {
    'word': u'أوإنكم',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَإِنَّكُمْ',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أوإنكن'] = {
    'word': u'أوإنكن',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَإِنَّكُنَّ',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أوإنها'] = {
    'word': u'أوإنها',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَإِنَّهَا',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'هَا'
}
STOPWORDS[u'أوإنهم'] = {
    'word': u'أوإنهم',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَإِنَّهُمْ',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أوإنهن'] = {
    'word': u'أوإنهن',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَإِنَّهُنَّ',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أوإننا'] = {
    'word': u'أوإننا',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَإِنَّنَا',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'نَا'
}
STOPWORDS[u'أوإنكما'] = {
    'word': u'أوإنكما',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَإِنَّكُمَا',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أوإنهما'] = {
    'word': u'أوإنهما',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَإِنَّهُمَا',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أفإني'] = {
    'word': u'أفإني',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَإِنَِّي',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'ِي'
}
STOPWORDS[u'أفإنك'] = {
    'word': u'أفإنك',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَإِنَّكَ',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'كَ'
}
STOPWORDS[u'أفإنك'] = {
    'word': u'أفإنك',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَإِنَّكِ',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'كِ'
}
STOPWORDS[u'أفإنه'] = {
    'word': u'أفإنه',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَإِنَّه',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'ه'
}
STOPWORDS[u'أفإنكم'] = {
    'word': u'أفإنكم',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَإِنَّكُمْ',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أفإنكن'] = {
    'word': u'أفإنكن',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَإِنَّكُنَّ',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أفإنها'] = {
    'word': u'أفإنها',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَإِنَّهَا',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'هَا'
}
STOPWORDS[u'أفإنهم'] = {
    'word': u'أفإنهم',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَإِنَّهُمْ',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أفإنهن'] = {
    'word': u'أفإنهن',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَإِنَّهُنَّ',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أفإننا'] = {
    'word': u'أفإننا',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَإِنَّنَا',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'نَا'
}
STOPWORDS[u'أفإنكما'] = {
    'word': u'أفإنكما',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَإِنَّكُمَا',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أفإنهما'] = {
    'word': u'أفإنهما',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَإِنَّهُمَا',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أولإني'] = {
    'word': u'أولإني',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَإِنَِّي',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'ِي'
}
STOPWORDS[u'أولإنك'] = {
    'word': u'أولإنك',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَإِنَّكَ',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'كَ'
}
STOPWORDS[u'أولإنك'] = {
    'word': u'أولإنك',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَإِنَّكِ',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'كِ'
}
STOPWORDS[u'أولإنه'] = {
    'word': u'أولإنه',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَإِنَّه',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'ه'
}
STOPWORDS[u'أولإنكم'] = {
    'word': u'أولإنكم',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَإِنَّكُمْ',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أولإنكن'] = {
    'word': u'أولإنكن',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَإِنَّكُنَّ',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أولإنها'] = {
    'word': u'أولإنها',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَإِنَّهَا',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'هَا'
}
STOPWORDS[u'أولإنهم'] = {
    'word': u'أولإنهم',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَإِنَّهُمْ',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أولإنهن'] = {
    'word': u'أولإنهن',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَإِنَّهُنَّ',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أولإننا'] = {
    'word': u'أولإننا',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَإِنَّنَا',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'نَا'
}
STOPWORDS[u'أولإنكما'] = {
    'word': u'أولإنكما',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَإِنَّكُمَا',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أولإنهما'] = {
    'word': u'أولإنهما',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَإِنَّهُمَا',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أفلإني'] = {
    'word': u'أفلإني',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَإِنَِّي',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'ِي'
}
STOPWORDS[u'أفلإنك'] = {
    'word': u'أفلإنك',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَإِنَّكَ',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'كَ'
}
STOPWORDS[u'أفلإنك'] = {
    'word': u'أفلإنك',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَإِنَّكِ',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'كِ'
}
STOPWORDS[u'أفلإنه'] = {
    'word': u'أفلإنه',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَإِنَّه',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'ه'
}
STOPWORDS[u'أفلإنكم'] = {
    'word': u'أفلإنكم',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَإِنَّكُمْ',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أفلإنكن'] = {
    'word': u'أفلإنكن',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَإِنَّكُنَّ',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أفلإنها'] = {
    'word': u'أفلإنها',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَإِنَّهَا',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'هَا'
}
STOPWORDS[u'أفلإنهم'] = {
    'word': u'أفلإنهم',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَإِنَّهُمْ',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أفلإنهن'] = {
    'word': u'أفلإنهن',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَإِنَّهُنَّ',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أفلإننا'] = {
    'word': u'أفلإننا',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَإِنَّنَا',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'نَا'
}
STOPWORDS[u'أفلإنكما'] = {
    'word': u'أفلإنكما',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَإِنَّكُمَا',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أفلإنهما'] = {
    'word': u'أفلإنهما',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَإِنَّهُمَا',
    'stem': u'إِنَّ',
    'type': u'STOPWORD',
    'original': u'إِنَّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'عل'] = {
    'word': u'عَلًّ',
    'procletic': u'',
    'tags': u'أداة:إن و أخواتها',
    'vocalized': u'عَلًّ',
    'stem': u'عَلًّ',
    'type': u'STOPWORD',
    'original': u'عَلًّ',
    'encletic': u''
}
STOPWORDS[u'وعل'] = {
    'word': u'وعل',
    'procletic': u'-وَ',
    'tags': u'أداة:إن و أخواتها:معطوف',
    'vocalized': u'وَعَلًّ',
    'stem': u'عَلًّ',
    'type': u'STOPWORD',
    'original': u'عَلًّ',
    'encletic': u''
}
STOPWORDS[u'فعل'] = {
    'word': u'فعل',
    'procletic': u'-فَ',
    'tags': u'أداة:إن و أخواتها:معطوف',
    'vocalized': u'فَعَلًّ',
    'stem': u'عَلًّ',
    'type': u'STOPWORD',
    'original': u'عَلًّ',
    'encletic': u''
}
STOPWORDS[u'كأن'] = {
    'word': u'كَأَنَّ',
    'procletic': u'',
    'tags': u'أداة:إن و أخواتها',
    'vocalized': u'كَأَنَّ',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u''
}
STOPWORDS[u'لكأن'] = {
    'word': u'لكأن',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور',
    'vocalized': u'لَكَأَنَّ',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u''
}
STOPWORDS[u'وكأن'] = {
    'word': u'وكأن',
    'procletic': u'-وَ',
    'tags': u'أداة:إن و أخواتها:معطوف',
    'vocalized': u'وَكَأَنَّ',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u''
}
STOPWORDS[u'فكأن'] = {
    'word': u'فكأن',
    'procletic': u'-فَ',
    'tags': u'أداة:إن و أخواتها:معطوف',
    'vocalized': u'فَكَأَنَّ',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u''
}
STOPWORDS[u'ولكأن'] = {
    'word': u'ولكأن',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'وَلَكَأَنَّ',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u''
}
STOPWORDS[u'فلكأن'] = {
    'word': u'فلكأن',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'فَلَكَأَنَّ',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u''
}
STOPWORDS[u'أكأن'] = {
    'word': u'أكأن',
    'procletic': u'-أَ',
    'tags': u'أداة:إن و أخواتها:مجرور',
    'vocalized': u'أَكَأَنَّ',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u''
}
STOPWORDS[u'ألكأن'] = {
    'word': u'ألكأن',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور',
    'vocalized': u'أَلَكَأَنَّ',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u''
}
STOPWORDS[u'أوكأن'] = {
    'word': u'أوكأن',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مجرور',
    'vocalized': u'أَوَكَأَنَّ',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u''
}
STOPWORDS[u'أفكأن'] = {
    'word': u'أفكأن',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مجرور',
    'vocalized': u'أَفَكَأَنَّ',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u''
}
STOPWORDS[u'أولكأن'] = {
    'word': u'أولكأن',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَلَكَأَنَّ',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u''
}
STOPWORDS[u'أفلكأن'] = {
    'word': u'أفلكأن',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَلَكَأَنَّ',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u''
}
STOPWORDS[u'كأني'] = {
    'word': u'كأني',
    'procletic': u'',
    'tags': u'أداة:إن و أخواتها:مضاف',
    'vocalized': u'كَأَنَِّي',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'ِي'
}
STOPWORDS[u'كأنك'] = {
    'word': u'كأنك',
    'procletic': u'',
    'tags': u'أداة:إن و أخواتها:مضاف',
    'vocalized': u'كَأَنَّكَ',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'كَ'
}
STOPWORDS[u'كأنك'] = {
    'word': u'كأنك',
    'procletic': u'',
    'tags': u'أداة:إن و أخواتها:مضاف',
    'vocalized': u'كَأَنَّكِ',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'كِ'
}
STOPWORDS[u'كأنه'] = {
    'word': u'كأنه',
    'procletic': u'',
    'tags': u'أداة:إن و أخواتها:مضاف',
    'vocalized': u'كَأَنَّه',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'ه'
}
STOPWORDS[u'كأنكم'] = {
    'word': u'كأنكم',
    'procletic': u'',
    'tags': u'أداة:إن و أخواتها:مضاف',
    'vocalized': u'كَأَنَّكُمْ',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'كأنكن'] = {
    'word': u'كأنكن',
    'procletic': u'',
    'tags': u'أداة:إن و أخواتها:مضاف',
    'vocalized': u'كَأَنَّكُنَّ',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'كأنها'] = {
    'word': u'كأنها',
    'procletic': u'',
    'tags': u'أداة:إن و أخواتها:مضاف',
    'vocalized': u'كَأَنَّهَا',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'هَا'
}
STOPWORDS[u'كأنهم'] = {
    'word': u'كأنهم',
    'procletic': u'',
    'tags': u'أداة:إن و أخواتها:مضاف',
    'vocalized': u'كَأَنَّهُمْ',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'كأنهن'] = {
    'word': u'كأنهن',
    'procletic': u'',
    'tags': u'أداة:إن و أخواتها:مضاف',
    'vocalized': u'كَأَنَّهُنَّ',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'كأننا'] = {
    'word': u'كأننا',
    'procletic': u'',
    'tags': u'أداة:إن و أخواتها:مضاف',
    'vocalized': u'كَأَنَّنَا',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'نَا'
}
STOPWORDS[u'كأنكما'] = {
    'word': u'كأنكما',
    'procletic': u'',
    'tags': u'أداة:إن و أخواتها:مضاف',
    'vocalized': u'كَأَنَّكُمَا',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'كأنهما'] = {
    'word': u'كأنهما',
    'procletic': u'',
    'tags': u'أداة:إن و أخواتها:مضاف',
    'vocalized': u'كَأَنَّهُمَا',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'لكأني'] = {
    'word': u'لكأني',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَكَأَنَِّي',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'ِي'
}
STOPWORDS[u'لكأنك'] = {
    'word': u'لكأنك',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَكَأَنَّكَ',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'كَ'
}
STOPWORDS[u'لكأنك'] = {
    'word': u'لكأنك',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَكَأَنَّكِ',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'كِ'
}
STOPWORDS[u'لكأنه'] = {
    'word': u'لكأنه',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَكَأَنَّه',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'ه'
}
STOPWORDS[u'لكأنكم'] = {
    'word': u'لكأنكم',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَكَأَنَّكُمْ',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'لكأنكن'] = {
    'word': u'لكأنكن',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَكَأَنَّكُنَّ',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'لكأنها'] = {
    'word': u'لكأنها',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَكَأَنَّهَا',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'هَا'
}
STOPWORDS[u'لكأنهم'] = {
    'word': u'لكأنهم',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَكَأَنَّهُمْ',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'لكأنهن'] = {
    'word': u'لكأنهن',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَكَأَنَّهُنَّ',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'لكأننا'] = {
    'word': u'لكأننا',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَكَأَنَّنَا',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'نَا'
}
STOPWORDS[u'لكأنكما'] = {
    'word': u'لكأنكما',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَكَأَنَّكُمَا',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'لكأنهما'] = {
    'word': u'لكأنهما',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَكَأَنَّهُمَا',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'وكأني'] = {
    'word': u'وكأني',
    'procletic': u'-وَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'وَكَأَنَِّي',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'ِي'
}
STOPWORDS[u'وكأنك'] = {
    'word': u'وكأنك',
    'procletic': u'-وَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'وَكَأَنَّكَ',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'كَ'
}
STOPWORDS[u'وكأنك'] = {
    'word': u'وكأنك',
    'procletic': u'-وَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'وَكَأَنَّكِ',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'كِ'
}
STOPWORDS[u'وكأنه'] = {
    'word': u'وكأنه',
    'procletic': u'-وَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'وَكَأَنَّه',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'ه'
}
STOPWORDS[u'وكأنكم'] = {
    'word': u'وكأنكم',
    'procletic': u'-وَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'وَكَأَنَّكُمْ',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'وكأنكن'] = {
    'word': u'وكأنكن',
    'procletic': u'-وَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'وَكَأَنَّكُنَّ',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'وكأنها'] = {
    'word': u'وكأنها',
    'procletic': u'-وَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'وَكَأَنَّهَا',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'هَا'
}
STOPWORDS[u'وكأنهم'] = {
    'word': u'وكأنهم',
    'procletic': u'-وَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'وَكَأَنَّهُمْ',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'وكأنهن'] = {
    'word': u'وكأنهن',
    'procletic': u'-وَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'وَكَأَنَّهُنَّ',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'وكأننا'] = {
    'word': u'وكأننا',
    'procletic': u'-وَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'وَكَأَنَّنَا',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'نَا'
}
STOPWORDS[u'وكأنكما'] = {
    'word': u'وكأنكما',
    'procletic': u'-وَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'وَكَأَنَّكُمَا',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'وكأنهما'] = {
    'word': u'وكأنهما',
    'procletic': u'-وَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'وَكَأَنَّهُمَا',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فكأني'] = {
    'word': u'فكأني',
    'procletic': u'-فَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'فَكَأَنَِّي',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'ِي'
}
STOPWORDS[u'فكأنك'] = {
    'word': u'فكأنك',
    'procletic': u'-فَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'فَكَأَنَّكَ',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'كَ'
}
STOPWORDS[u'فكأنك'] = {
    'word': u'فكأنك',
    'procletic': u'-فَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'فَكَأَنَّكِ',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'كِ'
}
STOPWORDS[u'فكأنه'] = {
    'word': u'فكأنه',
    'procletic': u'-فَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'فَكَأَنَّه',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'ه'
}
STOPWORDS[u'فكأنكم'] = {
    'word': u'فكأنكم',
    'procletic': u'-فَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'فَكَأَنَّكُمْ',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فكأنكن'] = {
    'word': u'فكأنكن',
    'procletic': u'-فَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'فَكَأَنَّكُنَّ',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فكأنها'] = {
    'word': u'فكأنها',
    'procletic': u'-فَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'فَكَأَنَّهَا',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'هَا'
}
STOPWORDS[u'فكأنهم'] = {
    'word': u'فكأنهم',
    'procletic': u'-فَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'فَكَأَنَّهُمْ',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فكأنهن'] = {
    'word': u'فكأنهن',
    'procletic': u'-فَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'فَكَأَنَّهُنَّ',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فكأننا'] = {
    'word': u'فكأننا',
    'procletic': u'-فَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'فَكَأَنَّنَا',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'نَا'
}
STOPWORDS[u'فكأنكما'] = {
    'word': u'فكأنكما',
    'procletic': u'-فَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'فَكَأَنَّكُمَا',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فكأنهما'] = {
    'word': u'فكأنهما',
    'procletic': u'-فَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'فَكَأَنَّهُمَا',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ولكأني'] = {
    'word': u'ولكأني',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَكَأَنَِّي',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'ِي'
}
STOPWORDS[u'ولكأنك'] = {
    'word': u'ولكأنك',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَكَأَنَّكَ',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'كَ'
}
STOPWORDS[u'ولكأنك'] = {
    'word': u'ولكأنك',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَكَأَنَّكِ',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'كِ'
}
STOPWORDS[u'ولكأنه'] = {
    'word': u'ولكأنه',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَكَأَنَّه',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'ه'
}
STOPWORDS[u'ولكأنكم'] = {
    'word': u'ولكأنكم',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَكَأَنَّكُمْ',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ولكأنكن'] = {
    'word': u'ولكأنكن',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَكَأَنَّكُنَّ',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ولكأنها'] = {
    'word': u'ولكأنها',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَكَأَنَّهَا',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'هَا'
}
STOPWORDS[u'ولكأنهم'] = {
    'word': u'ولكأنهم',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَكَأَنَّهُمْ',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ولكأنهن'] = {
    'word': u'ولكأنهن',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَكَأَنَّهُنَّ',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ولكأننا'] = {
    'word': u'ولكأننا',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَكَأَنَّنَا',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'نَا'
}
STOPWORDS[u'ولكأنكما'] = {
    'word': u'ولكأنكما',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَكَأَنَّكُمَا',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ولكأنهما'] = {
    'word': u'ولكأنهما',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَكَأَنَّهُمَا',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فلكأني'] = {
    'word': u'فلكأني',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَكَأَنَِّي',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'ِي'
}
STOPWORDS[u'فلكأنك'] = {
    'word': u'فلكأنك',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَكَأَنَّكَ',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'كَ'
}
STOPWORDS[u'فلكأنك'] = {
    'word': u'فلكأنك',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَكَأَنَّكِ',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'كِ'
}
STOPWORDS[u'فلكأنه'] = {
    'word': u'فلكأنه',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَكَأَنَّه',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'ه'
}
STOPWORDS[u'فلكأنكم'] = {
    'word': u'فلكأنكم',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَكَأَنَّكُمْ',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فلكأنكن'] = {
    'word': u'فلكأنكن',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَكَأَنَّكُنَّ',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فلكأنها'] = {
    'word': u'فلكأنها',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَكَأَنَّهَا',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'هَا'
}
STOPWORDS[u'فلكأنهم'] = {
    'word': u'فلكأنهم',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَكَأَنَّهُمْ',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فلكأنهن'] = {
    'word': u'فلكأنهن',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَكَأَنَّهُنَّ',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فلكأننا'] = {
    'word': u'فلكأننا',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَكَأَنَّنَا',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'نَا'
}
STOPWORDS[u'فلكأنكما'] = {
    'word': u'فلكأنكما',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَكَأَنَّكُمَا',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فلكأنهما'] = {
    'word': u'فلكأنهما',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَكَأَنَّهُمَا',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أكأني'] = {
    'word': u'أكأني',
    'procletic': u'-أَ',
    'tags': u'أداة:إن و أخواتها:مجرور:مضاف',
    'vocalized': u'أَكَأَنَِّي',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'ِي'
}
STOPWORDS[u'أكأنك'] = {
    'word': u'أكأنك',
    'procletic': u'-أَ',
    'tags': u'أداة:إن و أخواتها:مجرور:مضاف',
    'vocalized': u'أَكَأَنَّكَ',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'كَ'
}
STOPWORDS[u'أكأنك'] = {
    'word': u'أكأنك',
    'procletic': u'-أَ',
    'tags': u'أداة:إن و أخواتها:مجرور:مضاف',
    'vocalized': u'أَكَأَنَّكِ',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'كِ'
}
STOPWORDS[u'أكأنه'] = {
    'word': u'أكأنه',
    'procletic': u'-أَ',
    'tags': u'أداة:إن و أخواتها:مجرور:مضاف',
    'vocalized': u'أَكَأَنَّه',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'ه'
}
STOPWORDS[u'أكأنكم'] = {
    'word': u'أكأنكم',
    'procletic': u'-أَ',
    'tags': u'أداة:إن و أخواتها:مجرور:مضاف',
    'vocalized': u'أَكَأَنَّكُمْ',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أكأنكن'] = {
    'word': u'أكأنكن',
    'procletic': u'-أَ',
    'tags': u'أداة:إن و أخواتها:مجرور:مضاف',
    'vocalized': u'أَكَأَنَّكُنَّ',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أكأنها'] = {
    'word': u'أكأنها',
    'procletic': u'-أَ',
    'tags': u'أداة:إن و أخواتها:مجرور:مضاف',
    'vocalized': u'أَكَأَنَّهَا',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'هَا'
}
STOPWORDS[u'أكأنهم'] = {
    'word': u'أكأنهم',
    'procletic': u'-أَ',
    'tags': u'أداة:إن و أخواتها:مجرور:مضاف',
    'vocalized': u'أَكَأَنَّهُمْ',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أكأنهن'] = {
    'word': u'أكأنهن',
    'procletic': u'-أَ',
    'tags': u'أداة:إن و أخواتها:مجرور:مضاف',
    'vocalized': u'أَكَأَنَّهُنَّ',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أكأننا'] = {
    'word': u'أكأننا',
    'procletic': u'-أَ',
    'tags': u'أداة:إن و أخواتها:مجرور:مضاف',
    'vocalized': u'أَكَأَنَّنَا',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'نَا'
}
STOPWORDS[u'أكأنكما'] = {
    'word': u'أكأنكما',
    'procletic': u'-أَ',
    'tags': u'أداة:إن و أخواتها:مجرور:مضاف',
    'vocalized': u'أَكَأَنَّكُمَا',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أكأنهما'] = {
    'word': u'أكأنهما',
    'procletic': u'-أَ',
    'tags': u'أداة:إن و أخواتها:مجرور:مضاف',
    'vocalized': u'أَكَأَنَّهُمَا',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ألكأني'] = {
    'word': u'ألكأني',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَكَأَنَِّي',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'ِي'
}
STOPWORDS[u'ألكأنك'] = {
    'word': u'ألكأنك',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَكَأَنَّكَ',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'كَ'
}
STOPWORDS[u'ألكأنك'] = {
    'word': u'ألكأنك',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَكَأَنَّكِ',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'كِ'
}
STOPWORDS[u'ألكأنه'] = {
    'word': u'ألكأنه',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَكَأَنَّه',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'ه'
}
STOPWORDS[u'ألكأنكم'] = {
    'word': u'ألكأنكم',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَكَأَنَّكُمْ',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ألكأنكن'] = {
    'word': u'ألكأنكن',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَكَأَنَّكُنَّ',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ألكأنها'] = {
    'word': u'ألكأنها',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَكَأَنَّهَا',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'هَا'
}
STOPWORDS[u'ألكأنهم'] = {
    'word': u'ألكأنهم',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَكَأَنَّهُمْ',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ألكأنهن'] = {
    'word': u'ألكأنهن',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَكَأَنَّهُنَّ',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ألكأننا'] = {
    'word': u'ألكأننا',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَكَأَنَّنَا',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'نَا'
}
STOPWORDS[u'ألكأنكما'] = {
    'word': u'ألكأنكما',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَكَأَنَّكُمَا',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ألكأنهما'] = {
    'word': u'ألكأنهما',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَكَأَنَّهُمَا',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أوكأني'] = {
    'word': u'أوكأني',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَأَنَِّي',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'ِي'
}
STOPWORDS[u'أوكأنك'] = {
    'word': u'أوكأنك',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَأَنَّكَ',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'كَ'
}
STOPWORDS[u'أوكأنك'] = {
    'word': u'أوكأنك',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَأَنَّكِ',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'كِ'
}
STOPWORDS[u'أوكأنه'] = {
    'word': u'أوكأنه',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَأَنَّه',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'ه'
}
STOPWORDS[u'أوكأنكم'] = {
    'word': u'أوكأنكم',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَأَنَّكُمْ',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أوكأنكن'] = {
    'word': u'أوكأنكن',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَأَنَّكُنَّ',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أوكأنها'] = {
    'word': u'أوكأنها',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَأَنَّهَا',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'هَا'
}
STOPWORDS[u'أوكأنهم'] = {
    'word': u'أوكأنهم',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَأَنَّهُمْ',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أوكأنهن'] = {
    'word': u'أوكأنهن',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَأَنَّهُنَّ',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أوكأننا'] = {
    'word': u'أوكأننا',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَأَنَّنَا',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'نَا'
}
STOPWORDS[u'أوكأنكما'] = {
    'word': u'أوكأنكما',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَأَنَّكُمَا',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أوكأنهما'] = {
    'word': u'أوكأنهما',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَأَنَّهُمَا',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أفكأني'] = {
    'word': u'أفكأني',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَأَنَِّي',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'ِي'
}
STOPWORDS[u'أفكأنك'] = {
    'word': u'أفكأنك',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَأَنَّكَ',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'كَ'
}
STOPWORDS[u'أفكأنك'] = {
    'word': u'أفكأنك',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَأَنَّكِ',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'كِ'
}
STOPWORDS[u'أفكأنه'] = {
    'word': u'أفكأنه',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَأَنَّه',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'ه'
}
STOPWORDS[u'أفكأنكم'] = {
    'word': u'أفكأنكم',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَأَنَّكُمْ',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أفكأنكن'] = {
    'word': u'أفكأنكن',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَأَنَّكُنَّ',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أفكأنها'] = {
    'word': u'أفكأنها',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَأَنَّهَا',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'هَا'
}
STOPWORDS[u'أفكأنهم'] = {
    'word': u'أفكأنهم',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَأَنَّهُمْ',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أفكأنهن'] = {
    'word': u'أفكأنهن',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَأَنَّهُنَّ',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أفكأننا'] = {
    'word': u'أفكأننا',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَأَنَّنَا',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'نَا'
}
STOPWORDS[u'أفكأنكما'] = {
    'word': u'أفكأنكما',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَأَنَّكُمَا',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أفكأنهما'] = {
    'word': u'أفكأنهما',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَأَنَّهُمَا',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أولكأني'] = {
    'word': u'أولكأني',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَكَأَنَِّي',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'ِي'
}
STOPWORDS[u'أولكأنك'] = {
    'word': u'أولكأنك',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَكَأَنَّكَ',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'كَ'
}
STOPWORDS[u'أولكأنك'] = {
    'word': u'أولكأنك',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَكَأَنَّكِ',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'كِ'
}
STOPWORDS[u'أولكأنه'] = {
    'word': u'أولكأنه',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَكَأَنَّه',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'ه'
}
STOPWORDS[u'أولكأنكم'] = {
    'word': u'أولكأنكم',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَكَأَنَّكُمْ',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أولكأنكن'] = {
    'word': u'أولكأنكن',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَكَأَنَّكُنَّ',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أولكأنها'] = {
    'word': u'أولكأنها',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَكَأَنَّهَا',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'هَا'
}
STOPWORDS[u'أولكأنهم'] = {
    'word': u'أولكأنهم',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَكَأَنَّهُمْ',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أولكأنهن'] = {
    'word': u'أولكأنهن',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَكَأَنَّهُنَّ',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أولكأننا'] = {
    'word': u'أولكأننا',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَكَأَنَّنَا',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'نَا'
}
STOPWORDS[u'أولكأنكما'] = {
    'word': u'أولكأنكما',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَكَأَنَّكُمَا',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أولكأنهما'] = {
    'word': u'أولكأنهما',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَكَأَنَّهُمَا',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أفلكأني'] = {
    'word': u'أفلكأني',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَكَأَنَِّي',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'ِي'
}
STOPWORDS[u'أفلكأنك'] = {
    'word': u'أفلكأنك',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَكَأَنَّكَ',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'كَ'
}
STOPWORDS[u'أفلكأنك'] = {
    'word': u'أفلكأنك',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَكَأَنَّكِ',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'كِ'
}
STOPWORDS[u'أفلكأنه'] = {
    'word': u'أفلكأنه',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَكَأَنَّه',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'ه'
}
STOPWORDS[u'أفلكأنكم'] = {
    'word': u'أفلكأنكم',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَكَأَنَّكُمْ',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أفلكأنكن'] = {
    'word': u'أفلكأنكن',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَكَأَنَّكُنَّ',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أفلكأنها'] = {
    'word': u'أفلكأنها',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَكَأَنَّهَا',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'هَا'
}
STOPWORDS[u'أفلكأنهم'] = {
    'word': u'أفلكأنهم',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَكَأَنَّهُمْ',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أفلكأنهن'] = {
    'word': u'أفلكأنهن',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَكَأَنَّهُنَّ',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أفلكأننا'] = {
    'word': u'أفلكأننا',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَكَأَنَّنَا',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'نَا'
}
STOPWORDS[u'أفلكأنكما'] = {
    'word': u'أفلكأنكما',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَكَأَنَّكُمَا',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أفلكأنهما'] = {
    'word': u'أفلكأنهما',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَكَأَنَّهُمَا',
    'stem': u'كَأَنَّ',
    'type': u'STOPWORD',
    'original': u'كَأَنَّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'لعل'] = {
    'word': u'لَعَلَّ',
    'procletic': u'',
    'tags': u'أداة:إن و أخواتها',
    'vocalized': u'لَعَلَّ',
    'stem': u'لَعَلَّ',
    'type': u'STOPWORD',
    'original': u'لَعَلَّ',
    'encletic': u''
}
STOPWORDS[u'ولعل'] = {
    'word': u'ولعل',
    'procletic': u'-وَ',
    'tags': u'أداة:إن و أخواتها:معطوف',
    'vocalized': u'وَلَعَلَّ',
    'stem': u'لَعَلَّ',
    'type': u'STOPWORD',
    'original': u'لَعَلَّ',
    'encletic': u''
}
STOPWORDS[u'فلعل'] = {
    'word': u'فلعل',
    'procletic': u'-فَ',
    'tags': u'أداة:إن و أخواتها:معطوف',
    'vocalized': u'فَلَعَلَّ',
    'stem': u'لَعَلَّ',
    'type': u'STOPWORD',
    'original': u'لَعَلَّ',
    'encletic': u''
}
STOPWORDS[u'ألعل'] = {
    'word': u'ألعل',
    'procletic': u'-أَ',
    'tags': u'أداة:إن و أخواتها:مجرور',
    'vocalized': u'أَلَعَلَّ',
    'stem': u'لَعَلَّ',
    'type': u'STOPWORD',
    'original': u'لَعَلَّ',
    'encletic': u''
}
STOPWORDS[u'أولعل'] = {
    'word': u'أولعل',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مجرور',
    'vocalized': u'أَوَلَعَلَّ',
    'stem': u'لَعَلَّ',
    'type': u'STOPWORD',
    'original': u'لَعَلَّ',
    'encletic': u''
}
STOPWORDS[u'أفلعل'] = {
    'word': u'أفلعل',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مجرور',
    'vocalized': u'أَفَلَعَلَّ',
    'stem': u'لَعَلَّ',
    'type': u'STOPWORD',
    'original': u'لَعَلَّ',
    'encletic': u''
}
STOPWORDS[u'لعلي'] = {
    'word': u'لعلي',
    'procletic': u'',
    'tags': u'أداة:إن و أخواتها:مضاف',
    'vocalized': u'لَعَلَِّي',
    'stem': u'لَعَلَّ',
    'type': u'STOPWORD',
    'original': u'لَعَلَّ',
    'encletic': u'ِي'
}
STOPWORDS[u'لعلك'] = {
    'word': u'لعلك',
    'procletic': u'',
    'tags': u'أداة:إن و أخواتها:مضاف',
    'vocalized': u'لَعَلَّكَ',
    'stem': u'لَعَلَّ',
    'type': u'STOPWORD',
    'original': u'لَعَلَّ',
    'encletic': u'كَ'
}
STOPWORDS[u'لعلك'] = {
    'word': u'لعلك',
    'procletic': u'',
    'tags': u'أداة:إن و أخواتها:مضاف',
    'vocalized': u'لَعَلَّكِ',
    'stem': u'لَعَلَّ',
    'type': u'STOPWORD',
    'original': u'لَعَلَّ',
    'encletic': u'كِ'
}
STOPWORDS[u'لعله'] = {
    'word': u'لعله',
    'procletic': u'',
    'tags': u'أداة:إن و أخواتها:مضاف',
    'vocalized': u'لَعَلَّه',
    'stem': u'لَعَلَّ',
    'type': u'STOPWORD',
    'original': u'لَعَلَّ',
    'encletic': u'ه'
}
STOPWORDS[u'لعلكم'] = {
    'word': u'لعلكم',
    'procletic': u'',
    'tags': u'أداة:إن و أخواتها:مضاف',
    'vocalized': u'لَعَلَّكُمْ',
    'stem': u'لَعَلَّ',
    'type': u'STOPWORD',
    'original': u'لَعَلَّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'لعلكن'] = {
    'word': u'لعلكن',
    'procletic': u'',
    'tags': u'أداة:إن و أخواتها:مضاف',
    'vocalized': u'لَعَلَّكُنَّ',
    'stem': u'لَعَلَّ',
    'type': u'STOPWORD',
    'original': u'لَعَلَّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'لعلها'] = {
    'word': u'لعلها',
    'procletic': u'',
    'tags': u'أداة:إن و أخواتها:مضاف',
    'vocalized': u'لَعَلَّهَا',
    'stem': u'لَعَلَّ',
    'type': u'STOPWORD',
    'original': u'لَعَلَّ',
    'encletic': u'هَا'
}
STOPWORDS[u'لعلهم'] = {
    'word': u'لعلهم',
    'procletic': u'',
    'tags': u'أداة:إن و أخواتها:مضاف',
    'vocalized': u'لَعَلَّهُمْ',
    'stem': u'لَعَلَّ',
    'type': u'STOPWORD',
    'original': u'لَعَلَّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'لعلهن'] = {
    'word': u'لعلهن',
    'procletic': u'',
    'tags': u'أداة:إن و أخواتها:مضاف',
    'vocalized': u'لَعَلَّهُنَّ',
    'stem': u'لَعَلَّ',
    'type': u'STOPWORD',
    'original': u'لَعَلَّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'لعلنا'] = {
    'word': u'لعلنا',
    'procletic': u'',
    'tags': u'أداة:إن و أخواتها:مضاف',
    'vocalized': u'لَعَلَّنَا',
    'stem': u'لَعَلَّ',
    'type': u'STOPWORD',
    'original': u'لَعَلَّ',
    'encletic': u'نَا'
}
STOPWORDS[u'لعلكما'] = {
    'word': u'لعلكما',
    'procletic': u'',
    'tags': u'أداة:إن و أخواتها:مضاف',
    'vocalized': u'لَعَلَّكُمَا',
    'stem': u'لَعَلَّ',
    'type': u'STOPWORD',
    'original': u'لَعَلَّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'لعلهما'] = {
    'word': u'لعلهما',
    'procletic': u'',
    'tags': u'أداة:إن و أخواتها:مضاف',
    'vocalized': u'لَعَلَّهُمَا',
    'stem': u'لَعَلَّ',
    'type': u'STOPWORD',
    'original': u'لَعَلَّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ولعلي'] = {
    'word': u'ولعلي',
    'procletic': u'-وَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'وَلَعَلَِّي',
    'stem': u'لَعَلَّ',
    'type': u'STOPWORD',
    'original': u'لَعَلَّ',
    'encletic': u'ِي'
}
STOPWORDS[u'ولعلك'] = {
    'word': u'ولعلك',
    'procletic': u'-وَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'وَلَعَلَّكَ',
    'stem': u'لَعَلَّ',
    'type': u'STOPWORD',
    'original': u'لَعَلَّ',
    'encletic': u'كَ'
}
STOPWORDS[u'ولعلك'] = {
    'word': u'ولعلك',
    'procletic': u'-وَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'وَلَعَلَّكِ',
    'stem': u'لَعَلَّ',
    'type': u'STOPWORD',
    'original': u'لَعَلَّ',
    'encletic': u'كِ'
}
STOPWORDS[u'ولعله'] = {
    'word': u'ولعله',
    'procletic': u'-وَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'وَلَعَلَّه',
    'stem': u'لَعَلَّ',
    'type': u'STOPWORD',
    'original': u'لَعَلَّ',
    'encletic': u'ه'
}
STOPWORDS[u'ولعلكم'] = {
    'word': u'ولعلكم',
    'procletic': u'-وَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'وَلَعَلَّكُمْ',
    'stem': u'لَعَلَّ',
    'type': u'STOPWORD',
    'original': u'لَعَلَّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ولعلكن'] = {
    'word': u'ولعلكن',
    'procletic': u'-وَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'وَلَعَلَّكُنَّ',
    'stem': u'لَعَلَّ',
    'type': u'STOPWORD',
    'original': u'لَعَلَّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ولعلها'] = {
    'word': u'ولعلها',
    'procletic': u'-وَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'وَلَعَلَّهَا',
    'stem': u'لَعَلَّ',
    'type': u'STOPWORD',
    'original': u'لَعَلَّ',
    'encletic': u'هَا'
}
STOPWORDS[u'ولعلهم'] = {
    'word': u'ولعلهم',
    'procletic': u'-وَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'وَلَعَلَّهُمْ',
    'stem': u'لَعَلَّ',
    'type': u'STOPWORD',
    'original': u'لَعَلَّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ولعلهن'] = {
    'word': u'ولعلهن',
    'procletic': u'-وَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'وَلَعَلَّهُنَّ',
    'stem': u'لَعَلَّ',
    'type': u'STOPWORD',
    'original': u'لَعَلَّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ولعلنا'] = {
    'word': u'ولعلنا',
    'procletic': u'-وَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'وَلَعَلَّنَا',
    'stem': u'لَعَلَّ',
    'type': u'STOPWORD',
    'original': u'لَعَلَّ',
    'encletic': u'نَا'
}
STOPWORDS[u'ولعلكما'] = {
    'word': u'ولعلكما',
    'procletic': u'-وَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'وَلَعَلَّكُمَا',
    'stem': u'لَعَلَّ',
    'type': u'STOPWORD',
    'original': u'لَعَلَّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ولعلهما'] = {
    'word': u'ولعلهما',
    'procletic': u'-وَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'وَلَعَلَّهُمَا',
    'stem': u'لَعَلَّ',
    'type': u'STOPWORD',
    'original': u'لَعَلَّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فلعلي'] = {
    'word': u'فلعلي',
    'procletic': u'-فَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'فَلَعَلَِّي',
    'stem': u'لَعَلَّ',
    'type': u'STOPWORD',
    'original': u'لَعَلَّ',
    'encletic': u'ِي'
}
STOPWORDS[u'فلعلك'] = {
    'word': u'فلعلك',
    'procletic': u'-فَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'فَلَعَلَّكَ',
    'stem': u'لَعَلَّ',
    'type': u'STOPWORD',
    'original': u'لَعَلَّ',
    'encletic': u'كَ'
}
STOPWORDS[u'فلعلك'] = {
    'word': u'فلعلك',
    'procletic': u'-فَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'فَلَعَلَّكِ',
    'stem': u'لَعَلَّ',
    'type': u'STOPWORD',
    'original': u'لَعَلَّ',
    'encletic': u'كِ'
}
STOPWORDS[u'فلعله'] = {
    'word': u'فلعله',
    'procletic': u'-فَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'فَلَعَلَّه',
    'stem': u'لَعَلَّ',
    'type': u'STOPWORD',
    'original': u'لَعَلَّ',
    'encletic': u'ه'
}
STOPWORDS[u'فلعلكم'] = {
    'word': u'فلعلكم',
    'procletic': u'-فَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'فَلَعَلَّكُمْ',
    'stem': u'لَعَلَّ',
    'type': u'STOPWORD',
    'original': u'لَعَلَّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فلعلكن'] = {
    'word': u'فلعلكن',
    'procletic': u'-فَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'فَلَعَلَّكُنَّ',
    'stem': u'لَعَلَّ',
    'type': u'STOPWORD',
    'original': u'لَعَلَّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فلعلها'] = {
    'word': u'فلعلها',
    'procletic': u'-فَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'فَلَعَلَّهَا',
    'stem': u'لَعَلَّ',
    'type': u'STOPWORD',
    'original': u'لَعَلَّ',
    'encletic': u'هَا'
}
STOPWORDS[u'فلعلهم'] = {
    'word': u'فلعلهم',
    'procletic': u'-فَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'فَلَعَلَّهُمْ',
    'stem': u'لَعَلَّ',
    'type': u'STOPWORD',
    'original': u'لَعَلَّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فلعلهن'] = {
    'word': u'فلعلهن',
    'procletic': u'-فَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'فَلَعَلَّهُنَّ',
    'stem': u'لَعَلَّ',
    'type': u'STOPWORD',
    'original': u'لَعَلَّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فلعلنا'] = {
    'word': u'فلعلنا',
    'procletic': u'-فَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'فَلَعَلَّنَا',
    'stem': u'لَعَلَّ',
    'type': u'STOPWORD',
    'original': u'لَعَلَّ',
    'encletic': u'نَا'
}
STOPWORDS[u'فلعلكما'] = {
    'word': u'فلعلكما',
    'procletic': u'-فَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'فَلَعَلَّكُمَا',
    'stem': u'لَعَلَّ',
    'type': u'STOPWORD',
    'original': u'لَعَلَّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فلعلهما'] = {
    'word': u'فلعلهما',
    'procletic': u'-فَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'فَلَعَلَّهُمَا',
    'stem': u'لَعَلَّ',
    'type': u'STOPWORD',
    'original': u'لَعَلَّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ألعلي'] = {
    'word': u'ألعلي',
    'procletic': u'-أَ',
    'tags': u'أداة:إن و أخواتها:مجرور:مضاف',
    'vocalized': u'أَلَعَلَِّي',
    'stem': u'لَعَلَّ',
    'type': u'STOPWORD',
    'original': u'لَعَلَّ',
    'encletic': u'ِي'
}
STOPWORDS[u'ألعلك'] = {
    'word': u'ألعلك',
    'procletic': u'-أَ',
    'tags': u'أداة:إن و أخواتها:مجرور:مضاف',
    'vocalized': u'أَلَعَلَّكَ',
    'stem': u'لَعَلَّ',
    'type': u'STOPWORD',
    'original': u'لَعَلَّ',
    'encletic': u'كَ'
}
STOPWORDS[u'ألعلك'] = {
    'word': u'ألعلك',
    'procletic': u'-أَ',
    'tags': u'أداة:إن و أخواتها:مجرور:مضاف',
    'vocalized': u'أَلَعَلَّكِ',
    'stem': u'لَعَلَّ',
    'type': u'STOPWORD',
    'original': u'لَعَلَّ',
    'encletic': u'كِ'
}
STOPWORDS[u'ألعله'] = {
    'word': u'ألعله',
    'procletic': u'-أَ',
    'tags': u'أداة:إن و أخواتها:مجرور:مضاف',
    'vocalized': u'أَلَعَلَّه',
    'stem': u'لَعَلَّ',
    'type': u'STOPWORD',
    'original': u'لَعَلَّ',
    'encletic': u'ه'
}
STOPWORDS[u'ألعلكم'] = {
    'word': u'ألعلكم',
    'procletic': u'-أَ',
    'tags': u'أداة:إن و أخواتها:مجرور:مضاف',
    'vocalized': u'أَلَعَلَّكُمْ',
    'stem': u'لَعَلَّ',
    'type': u'STOPWORD',
    'original': u'لَعَلَّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ألعلكن'] = {
    'word': u'ألعلكن',
    'procletic': u'-أَ',
    'tags': u'أداة:إن و أخواتها:مجرور:مضاف',
    'vocalized': u'أَلَعَلَّكُنَّ',
    'stem': u'لَعَلَّ',
    'type': u'STOPWORD',
    'original': u'لَعَلَّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ألعلها'] = {
    'word': u'ألعلها',
    'procletic': u'-أَ',
    'tags': u'أداة:إن و أخواتها:مجرور:مضاف',
    'vocalized': u'أَلَعَلَّهَا',
    'stem': u'لَعَلَّ',
    'type': u'STOPWORD',
    'original': u'لَعَلَّ',
    'encletic': u'هَا'
}
STOPWORDS[u'ألعلهم'] = {
    'word': u'ألعلهم',
    'procletic': u'-أَ',
    'tags': u'أداة:إن و أخواتها:مجرور:مضاف',
    'vocalized': u'أَلَعَلَّهُمْ',
    'stem': u'لَعَلَّ',
    'type': u'STOPWORD',
    'original': u'لَعَلَّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ألعلهن'] = {
    'word': u'ألعلهن',
    'procletic': u'-أَ',
    'tags': u'أداة:إن و أخواتها:مجرور:مضاف',
    'vocalized': u'أَلَعَلَّهُنَّ',
    'stem': u'لَعَلَّ',
    'type': u'STOPWORD',
    'original': u'لَعَلَّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ألعلنا'] = {
    'word': u'ألعلنا',
    'procletic': u'-أَ',
    'tags': u'أداة:إن و أخواتها:مجرور:مضاف',
    'vocalized': u'أَلَعَلَّنَا',
    'stem': u'لَعَلَّ',
    'type': u'STOPWORD',
    'original': u'لَعَلَّ',
    'encletic': u'نَا'
}
STOPWORDS[u'ألعلكما'] = {
    'word': u'ألعلكما',
    'procletic': u'-أَ',
    'tags': u'أداة:إن و أخواتها:مجرور:مضاف',
    'vocalized': u'أَلَعَلَّكُمَا',
    'stem': u'لَعَلَّ',
    'type': u'STOPWORD',
    'original': u'لَعَلَّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ألعلهما'] = {
    'word': u'ألعلهما',
    'procletic': u'-أَ',
    'tags': u'أداة:إن و أخواتها:مجرور:مضاف',
    'vocalized': u'أَلَعَلَّهُمَا',
    'stem': u'لَعَلَّ',
    'type': u'STOPWORD',
    'original': u'لَعَلَّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أولعلي'] = {
    'word': u'أولعلي',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَعَلَِّي',
    'stem': u'لَعَلَّ',
    'type': u'STOPWORD',
    'original': u'لَعَلَّ',
    'encletic': u'ِي'
}
STOPWORDS[u'أولعلك'] = {
    'word': u'أولعلك',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَعَلَّكَ',
    'stem': u'لَعَلَّ',
    'type': u'STOPWORD',
    'original': u'لَعَلَّ',
    'encletic': u'كَ'
}
STOPWORDS[u'أولعلك'] = {
    'word': u'أولعلك',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَعَلَّكِ',
    'stem': u'لَعَلَّ',
    'type': u'STOPWORD',
    'original': u'لَعَلَّ',
    'encletic': u'كِ'
}
STOPWORDS[u'أولعله'] = {
    'word': u'أولعله',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَعَلَّه',
    'stem': u'لَعَلَّ',
    'type': u'STOPWORD',
    'original': u'لَعَلَّ',
    'encletic': u'ه'
}
STOPWORDS[u'أولعلكم'] = {
    'word': u'أولعلكم',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَعَلَّكُمْ',
    'stem': u'لَعَلَّ',
    'type': u'STOPWORD',
    'original': u'لَعَلَّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أولعلكن'] = {
    'word': u'أولعلكن',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَعَلَّكُنَّ',
    'stem': u'لَعَلَّ',
    'type': u'STOPWORD',
    'original': u'لَعَلَّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أولعلها'] = {
    'word': u'أولعلها',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَعَلَّهَا',
    'stem': u'لَعَلَّ',
    'type': u'STOPWORD',
    'original': u'لَعَلَّ',
    'encletic': u'هَا'
}
STOPWORDS[u'أولعلهم'] = {
    'word': u'أولعلهم',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَعَلَّهُمْ',
    'stem': u'لَعَلَّ',
    'type': u'STOPWORD',
    'original': u'لَعَلَّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أولعلهن'] = {
    'word': u'أولعلهن',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَعَلَّهُنَّ',
    'stem': u'لَعَلَّ',
    'type': u'STOPWORD',
    'original': u'لَعَلَّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أولعلنا'] = {
    'word': u'أولعلنا',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَعَلَّنَا',
    'stem': u'لَعَلَّ',
    'type': u'STOPWORD',
    'original': u'لَعَلَّ',
    'encletic': u'نَا'
}
STOPWORDS[u'أولعلكما'] = {
    'word': u'أولعلكما',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَعَلَّكُمَا',
    'stem': u'لَعَلَّ',
    'type': u'STOPWORD',
    'original': u'لَعَلَّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أولعلهما'] = {
    'word': u'أولعلهما',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَعَلَّهُمَا',
    'stem': u'لَعَلَّ',
    'type': u'STOPWORD',
    'original': u'لَعَلَّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أفلعلي'] = {
    'word': u'أفلعلي',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَعَلَِّي',
    'stem': u'لَعَلَّ',
    'type': u'STOPWORD',
    'original': u'لَعَلَّ',
    'encletic': u'ِي'
}
STOPWORDS[u'أفلعلك'] = {
    'word': u'أفلعلك',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَعَلَّكَ',
    'stem': u'لَعَلَّ',
    'type': u'STOPWORD',
    'original': u'لَعَلَّ',
    'encletic': u'كَ'
}
STOPWORDS[u'أفلعلك'] = {
    'word': u'أفلعلك',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَعَلَّكِ',
    'stem': u'لَعَلَّ',
    'type': u'STOPWORD',
    'original': u'لَعَلَّ',
    'encletic': u'كِ'
}
STOPWORDS[u'أفلعله'] = {
    'word': u'أفلعله',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَعَلَّه',
    'stem': u'لَعَلَّ',
    'type': u'STOPWORD',
    'original': u'لَعَلَّ',
    'encletic': u'ه'
}
STOPWORDS[u'أفلعلكم'] = {
    'word': u'أفلعلكم',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَعَلَّكُمْ',
    'stem': u'لَعَلَّ',
    'type': u'STOPWORD',
    'original': u'لَعَلَّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أفلعلكن'] = {
    'word': u'أفلعلكن',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَعَلَّكُنَّ',
    'stem': u'لَعَلَّ',
    'type': u'STOPWORD',
    'original': u'لَعَلَّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أفلعلها'] = {
    'word': u'أفلعلها',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَعَلَّهَا',
    'stem': u'لَعَلَّ',
    'type': u'STOPWORD',
    'original': u'لَعَلَّ',
    'encletic': u'هَا'
}
STOPWORDS[u'أفلعلهم'] = {
    'word': u'أفلعلهم',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَعَلَّهُمْ',
    'stem': u'لَعَلَّ',
    'type': u'STOPWORD',
    'original': u'لَعَلَّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أفلعلهن'] = {
    'word': u'أفلعلهن',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَعَلَّهُنَّ',
    'stem': u'لَعَلَّ',
    'type': u'STOPWORD',
    'original': u'لَعَلَّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أفلعلنا'] = {
    'word': u'أفلعلنا',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَعَلَّنَا',
    'stem': u'لَعَلَّ',
    'type': u'STOPWORD',
    'original': u'لَعَلَّ',
    'encletic': u'نَا'
}
STOPWORDS[u'أفلعلكما'] = {
    'word': u'أفلعلكما',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَعَلَّكُمَا',
    'stem': u'لَعَلَّ',
    'type': u'STOPWORD',
    'original': u'لَعَلَّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أفلعلهما'] = {
    'word': u'أفلعلهما',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَعَلَّهُمَا',
    'stem': u'لَعَلَّ',
    'type': u'STOPWORD',
    'original': u'لَعَلَّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'لكن'] = {
    'word': u'لَكِنَّ',
    'procletic': u'',
    'tags': u'أداة:إن و أخواتها',
    'vocalized': u'لَكِنَّ',
    'stem': u'لَكِنَّ',
    'type': u'STOPWORD',
    'original': u'لَكِنَّ',
    'encletic': u''
}
STOPWORDS[u'ولكن'] = {
    'word': u'ولكن',
    'procletic': u'-وَ',
    'tags': u'أداة:إن و أخواتها:معطوف',
    'vocalized': u'وَلَكِنَّ',
    'stem': u'لَكِنَّ',
    'type': u'STOPWORD',
    'original': u'لَكِنَّ',
    'encletic': u''
}
STOPWORDS[u'فلكن'] = {
    'word': u'فلكن',
    'procletic': u'-فَ',
    'tags': u'أداة:إن و أخواتها:معطوف',
    'vocalized': u'فَلَكِنَّ',
    'stem': u'لَكِنَّ',
    'type': u'STOPWORD',
    'original': u'لَكِنَّ',
    'encletic': u''
}
STOPWORDS[u'لكني'] = {
    'word': u'لكني',
    'procletic': u'',
    'tags': u'أداة:إن و أخواتها:مضاف',
    'vocalized': u'لَكِنَِّي',
    'stem': u'لَكِنَّ',
    'type': u'STOPWORD',
    'original': u'لَكِنَّ',
    'encletic': u'ِي'
}
STOPWORDS[u'لكنك'] = {
    'word': u'لكنك',
    'procletic': u'',
    'tags': u'أداة:إن و أخواتها:مضاف',
    'vocalized': u'لَكِنَّكَ',
    'stem': u'لَكِنَّ',
    'type': u'STOPWORD',
    'original': u'لَكِنَّ',
    'encletic': u'كَ'
}
STOPWORDS[u'لكنك'] = {
    'word': u'لكنك',
    'procletic': u'',
    'tags': u'أداة:إن و أخواتها:مضاف',
    'vocalized': u'لَكِنَّكِ',
    'stem': u'لَكِنَّ',
    'type': u'STOPWORD',
    'original': u'لَكِنَّ',
    'encletic': u'كِ'
}
STOPWORDS[u'لكنه'] = {
    'word': u'لكنه',
    'procletic': u'',
    'tags': u'أداة:إن و أخواتها:مضاف',
    'vocalized': u'لَكِنَّه',
    'stem': u'لَكِنَّ',
    'type': u'STOPWORD',
    'original': u'لَكِنَّ',
    'encletic': u'ه'
}
STOPWORDS[u'لكنكم'] = {
    'word': u'لكنكم',
    'procletic': u'',
    'tags': u'أداة:إن و أخواتها:مضاف',
    'vocalized': u'لَكِنَّكُمْ',
    'stem': u'لَكِنَّ',
    'type': u'STOPWORD',
    'original': u'لَكِنَّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'لكنكن'] = {
    'word': u'لكنكن',
    'procletic': u'',
    'tags': u'أداة:إن و أخواتها:مضاف',
    'vocalized': u'لَكِنَّكُنَّ',
    'stem': u'لَكِنَّ',
    'type': u'STOPWORD',
    'original': u'لَكِنَّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'لكنها'] = {
    'word': u'لكنها',
    'procletic': u'',
    'tags': u'أداة:إن و أخواتها:مضاف',
    'vocalized': u'لَكِنَّهَا',
    'stem': u'لَكِنَّ',
    'type': u'STOPWORD',
    'original': u'لَكِنَّ',
    'encletic': u'هَا'
}
STOPWORDS[u'لكنهم'] = {
    'word': u'لكنهم',
    'procletic': u'',
    'tags': u'أداة:إن و أخواتها:مضاف',
    'vocalized': u'لَكِنَّهُمْ',
    'stem': u'لَكِنَّ',
    'type': u'STOPWORD',
    'original': u'لَكِنَّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'لكنهن'] = {
    'word': u'لكنهن',
    'procletic': u'',
    'tags': u'أداة:إن و أخواتها:مضاف',
    'vocalized': u'لَكِنَّهُنَّ',
    'stem': u'لَكِنَّ',
    'type': u'STOPWORD',
    'original': u'لَكِنَّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'لكننا'] = {
    'word': u'لكننا',
    'procletic': u'',
    'tags': u'أداة:إن و أخواتها:مضاف',
    'vocalized': u'لَكِنَّنَا',
    'stem': u'لَكِنَّ',
    'type': u'STOPWORD',
    'original': u'لَكِنَّ',
    'encletic': u'نَا'
}
STOPWORDS[u'لكنكما'] = {
    'word': u'لكنكما',
    'procletic': u'',
    'tags': u'أداة:إن و أخواتها:مضاف',
    'vocalized': u'لَكِنَّكُمَا',
    'stem': u'لَكِنَّ',
    'type': u'STOPWORD',
    'original': u'لَكِنَّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'لكنهما'] = {
    'word': u'لكنهما',
    'procletic': u'',
    'tags': u'أداة:إن و أخواتها:مضاف',
    'vocalized': u'لَكِنَّهُمَا',
    'stem': u'لَكِنَّ',
    'type': u'STOPWORD',
    'original': u'لَكِنَّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ولكني'] = {
    'word': u'ولكني',
    'procletic': u'-وَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'وَلَكِنَِّي',
    'stem': u'لَكِنَّ',
    'type': u'STOPWORD',
    'original': u'لَكِنَّ',
    'encletic': u'ِي'
}
STOPWORDS[u'ولكنك'] = {
    'word': u'ولكنك',
    'procletic': u'-وَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'وَلَكِنَّكَ',
    'stem': u'لَكِنَّ',
    'type': u'STOPWORD',
    'original': u'لَكِنَّ',
    'encletic': u'كَ'
}
STOPWORDS[u'ولكنك'] = {
    'word': u'ولكنك',
    'procletic': u'-وَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'وَلَكِنَّكِ',
    'stem': u'لَكِنَّ',
    'type': u'STOPWORD',
    'original': u'لَكِنَّ',
    'encletic': u'كِ'
}
STOPWORDS[u'ولكنه'] = {
    'word': u'ولكنه',
    'procletic': u'-وَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'وَلَكِنَّه',
    'stem': u'لَكِنَّ',
    'type': u'STOPWORD',
    'original': u'لَكِنَّ',
    'encletic': u'ه'
}
STOPWORDS[u'ولكنكم'] = {
    'word': u'ولكنكم',
    'procletic': u'-وَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'وَلَكِنَّكُمْ',
    'stem': u'لَكِنَّ',
    'type': u'STOPWORD',
    'original': u'لَكِنَّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ولكنكن'] = {
    'word': u'ولكنكن',
    'procletic': u'-وَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'وَلَكِنَّكُنَّ',
    'stem': u'لَكِنَّ',
    'type': u'STOPWORD',
    'original': u'لَكِنَّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ولكنها'] = {
    'word': u'ولكنها',
    'procletic': u'-وَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'وَلَكِنَّهَا',
    'stem': u'لَكِنَّ',
    'type': u'STOPWORD',
    'original': u'لَكِنَّ',
    'encletic': u'هَا'
}
STOPWORDS[u'ولكنهم'] = {
    'word': u'ولكنهم',
    'procletic': u'-وَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'وَلَكِنَّهُمْ',
    'stem': u'لَكِنَّ',
    'type': u'STOPWORD',
    'original': u'لَكِنَّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ولكنهن'] = {
    'word': u'ولكنهن',
    'procletic': u'-وَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'وَلَكِنَّهُنَّ',
    'stem': u'لَكِنَّ',
    'type': u'STOPWORD',
    'original': u'لَكِنَّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ولكننا'] = {
    'word': u'ولكننا',
    'procletic': u'-وَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'وَلَكِنَّنَا',
    'stem': u'لَكِنَّ',
    'type': u'STOPWORD',
    'original': u'لَكِنَّ',
    'encletic': u'نَا'
}
STOPWORDS[u'ولكنكما'] = {
    'word': u'ولكنكما',
    'procletic': u'-وَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'وَلَكِنَّكُمَا',
    'stem': u'لَكِنَّ',
    'type': u'STOPWORD',
    'original': u'لَكِنَّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ولكنهما'] = {
    'word': u'ولكنهما',
    'procletic': u'-وَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'وَلَكِنَّهُمَا',
    'stem': u'لَكِنَّ',
    'type': u'STOPWORD',
    'original': u'لَكِنَّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فلكني'] = {
    'word': u'فلكني',
    'procletic': u'-فَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'فَلَكِنَِّي',
    'stem': u'لَكِنَّ',
    'type': u'STOPWORD',
    'original': u'لَكِنَّ',
    'encletic': u'ِي'
}
STOPWORDS[u'فلكنك'] = {
    'word': u'فلكنك',
    'procletic': u'-فَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'فَلَكِنَّكَ',
    'stem': u'لَكِنَّ',
    'type': u'STOPWORD',
    'original': u'لَكِنَّ',
    'encletic': u'كَ'
}
STOPWORDS[u'فلكنك'] = {
    'word': u'فلكنك',
    'procletic': u'-فَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'فَلَكِنَّكِ',
    'stem': u'لَكِنَّ',
    'type': u'STOPWORD',
    'original': u'لَكِنَّ',
    'encletic': u'كِ'
}
STOPWORDS[u'فلكنه'] = {
    'word': u'فلكنه',
    'procletic': u'-فَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'فَلَكِنَّه',
    'stem': u'لَكِنَّ',
    'type': u'STOPWORD',
    'original': u'لَكِنَّ',
    'encletic': u'ه'
}
STOPWORDS[u'فلكنكم'] = {
    'word': u'فلكنكم',
    'procletic': u'-فَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'فَلَكِنَّكُمْ',
    'stem': u'لَكِنَّ',
    'type': u'STOPWORD',
    'original': u'لَكِنَّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فلكنكن'] = {
    'word': u'فلكنكن',
    'procletic': u'-فَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'فَلَكِنَّكُنَّ',
    'stem': u'لَكِنَّ',
    'type': u'STOPWORD',
    'original': u'لَكِنَّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فلكنها'] = {
    'word': u'فلكنها',
    'procletic': u'-فَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'فَلَكِنَّهَا',
    'stem': u'لَكِنَّ',
    'type': u'STOPWORD',
    'original': u'لَكِنَّ',
    'encletic': u'هَا'
}
STOPWORDS[u'فلكنهم'] = {
    'word': u'فلكنهم',
    'procletic': u'-فَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'فَلَكِنَّهُمْ',
    'stem': u'لَكِنَّ',
    'type': u'STOPWORD',
    'original': u'لَكِنَّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فلكنهن'] = {
    'word': u'فلكنهن',
    'procletic': u'-فَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'فَلَكِنَّهُنَّ',
    'stem': u'لَكِنَّ',
    'type': u'STOPWORD',
    'original': u'لَكِنَّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فلكننا'] = {
    'word': u'فلكننا',
    'procletic': u'-فَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'فَلَكِنَّنَا',
    'stem': u'لَكِنَّ',
    'type': u'STOPWORD',
    'original': u'لَكِنَّ',
    'encletic': u'نَا'
}
STOPWORDS[u'فلكنكما'] = {
    'word': u'فلكنكما',
    'procletic': u'-فَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'فَلَكِنَّكُمَا',
    'stem': u'لَكِنَّ',
    'type': u'STOPWORD',
    'original': u'لَكِنَّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فلكنهما'] = {
    'word': u'فلكنهما',
    'procletic': u'-فَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'فَلَكِنَّهُمَا',
    'stem': u'لَكِنَّ',
    'type': u'STOPWORD',
    'original': u'لَكِنَّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ليت'] = {
    'word': u'لَيْتَ',
    'procletic': u'',
    'tags': u'أداة:إن و أخواتها',
    'vocalized': u'لَيْتَ',
    'stem': u'لَيْتَ',
    'type': u'STOPWORD',
    'original': u'لَيْتَ',
    'encletic': u''
}
STOPWORDS[u'وليت'] = {
    'word': u'وليت',
    'procletic': u'-وَ',
    'tags': u'أداة:إن و أخواتها:معطوف',
    'vocalized': u'وَلَيْتَ',
    'stem': u'لَيْتَ',
    'type': u'STOPWORD',
    'original': u'لَيْتَ',
    'encletic': u''
}
STOPWORDS[u'فليت'] = {
    'word': u'فليت',
    'procletic': u'-فَ',
    'tags': u'أداة:إن و أخواتها:معطوف',
    'vocalized': u'فَلَيْتَ',
    'stem': u'لَيْتَ',
    'type': u'STOPWORD',
    'original': u'لَيْتَ',
    'encletic': u''
}
STOPWORDS[u'ليتي'] = {
    'word': u'ليتي',
    'procletic': u'',
    'tags': u'أداة:إن و أخواتها:مضاف',
    'vocalized': u'لَيْتَِي',
    'stem': u'لَيْتَ',
    'type': u'STOPWORD',
    'original': u'لَيْتَ',
    'encletic': u'ِي'
}
STOPWORDS[u'ليتك'] = {
    'word': u'ليتك',
    'procletic': u'',
    'tags': u'أداة:إن و أخواتها:مضاف',
    'vocalized': u'لَيْتَكَ',
    'stem': u'لَيْتَ',
    'type': u'STOPWORD',
    'original': u'لَيْتَ',
    'encletic': u'كَ'
}
STOPWORDS[u'ليتك'] = {
    'word': u'ليتك',
    'procletic': u'',
    'tags': u'أداة:إن و أخواتها:مضاف',
    'vocalized': u'لَيْتَكِ',
    'stem': u'لَيْتَ',
    'type': u'STOPWORD',
    'original': u'لَيْتَ',
    'encletic': u'كِ'
}
STOPWORDS[u'ليته'] = {
    'word': u'ليته',
    'procletic': u'',
    'tags': u'أداة:إن و أخواتها:مضاف',
    'vocalized': u'لَيْتَه',
    'stem': u'لَيْتَ',
    'type': u'STOPWORD',
    'original': u'لَيْتَ',
    'encletic': u'ه'
}
STOPWORDS[u'ليتكم'] = {
    'word': u'ليتكم',
    'procletic': u'',
    'tags': u'أداة:إن و أخواتها:مضاف',
    'vocalized': u'لَيْتَكُمْ',
    'stem': u'لَيْتَ',
    'type': u'STOPWORD',
    'original': u'لَيْتَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ليتكن'] = {
    'word': u'ليتكن',
    'procletic': u'',
    'tags': u'أداة:إن و أخواتها:مضاف',
    'vocalized': u'لَيْتَكُنَّ',
    'stem': u'لَيْتَ',
    'type': u'STOPWORD',
    'original': u'لَيْتَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ليتها'] = {
    'word': u'ليتها',
    'procletic': u'',
    'tags': u'أداة:إن و أخواتها:مضاف',
    'vocalized': u'لَيْتَهَا',
    'stem': u'لَيْتَ',
    'type': u'STOPWORD',
    'original': u'لَيْتَ',
    'encletic': u'هَا'
}
STOPWORDS[u'ليتهم'] = {
    'word': u'ليتهم',
    'procletic': u'',
    'tags': u'أداة:إن و أخواتها:مضاف',
    'vocalized': u'لَيْتَهُمْ',
    'stem': u'لَيْتَ',
    'type': u'STOPWORD',
    'original': u'لَيْتَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ليتهن'] = {
    'word': u'ليتهن',
    'procletic': u'',
    'tags': u'أداة:إن و أخواتها:مضاف',
    'vocalized': u'لَيْتَهُنَّ',
    'stem': u'لَيْتَ',
    'type': u'STOPWORD',
    'original': u'لَيْتَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ليتنا'] = {
    'word': u'ليتنا',
    'procletic': u'',
    'tags': u'أداة:إن و أخواتها:مضاف',
    'vocalized': u'لَيْتَنَا',
    'stem': u'لَيْتَ',
    'type': u'STOPWORD',
    'original': u'لَيْتَ',
    'encletic': u'نَا'
}
STOPWORDS[u'ليتكما'] = {
    'word': u'ليتكما',
    'procletic': u'',
    'tags': u'أداة:إن و أخواتها:مضاف',
    'vocalized': u'لَيْتَكُمَا',
    'stem': u'لَيْتَ',
    'type': u'STOPWORD',
    'original': u'لَيْتَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ليتهما'] = {
    'word': u'ليتهما',
    'procletic': u'',
    'tags': u'أداة:إن و أخواتها:مضاف',
    'vocalized': u'لَيْتَهُمَا',
    'stem': u'لَيْتَ',
    'type': u'STOPWORD',
    'original': u'لَيْتَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'وليتي'] = {
    'word': u'وليتي',
    'procletic': u'-وَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'وَلَيْتَِي',
    'stem': u'لَيْتَ',
    'type': u'STOPWORD',
    'original': u'لَيْتَ',
    'encletic': u'ِي'
}
STOPWORDS[u'وليتك'] = {
    'word': u'وليتك',
    'procletic': u'-وَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'وَلَيْتَكَ',
    'stem': u'لَيْتَ',
    'type': u'STOPWORD',
    'original': u'لَيْتَ',
    'encletic': u'كَ'
}
STOPWORDS[u'وليتك'] = {
    'word': u'وليتك',
    'procletic': u'-وَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'وَلَيْتَكِ',
    'stem': u'لَيْتَ',
    'type': u'STOPWORD',
    'original': u'لَيْتَ',
    'encletic': u'كِ'
}
STOPWORDS[u'وليته'] = {
    'word': u'وليته',
    'procletic': u'-وَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'وَلَيْتَه',
    'stem': u'لَيْتَ',
    'type': u'STOPWORD',
    'original': u'لَيْتَ',
    'encletic': u'ه'
}
STOPWORDS[u'وليتكم'] = {
    'word': u'وليتكم',
    'procletic': u'-وَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'وَلَيْتَكُمْ',
    'stem': u'لَيْتَ',
    'type': u'STOPWORD',
    'original': u'لَيْتَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'وليتكن'] = {
    'word': u'وليتكن',
    'procletic': u'-وَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'وَلَيْتَكُنَّ',
    'stem': u'لَيْتَ',
    'type': u'STOPWORD',
    'original': u'لَيْتَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'وليتها'] = {
    'word': u'وليتها',
    'procletic': u'-وَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'وَلَيْتَهَا',
    'stem': u'لَيْتَ',
    'type': u'STOPWORD',
    'original': u'لَيْتَ',
    'encletic': u'هَا'
}
STOPWORDS[u'وليتهم'] = {
    'word': u'وليتهم',
    'procletic': u'-وَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'وَلَيْتَهُمْ',
    'stem': u'لَيْتَ',
    'type': u'STOPWORD',
    'original': u'لَيْتَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'وليتهن'] = {
    'word': u'وليتهن',
    'procletic': u'-وَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'وَلَيْتَهُنَّ',
    'stem': u'لَيْتَ',
    'type': u'STOPWORD',
    'original': u'لَيْتَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'وليتنا'] = {
    'word': u'وليتنا',
    'procletic': u'-وَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'وَلَيْتَنَا',
    'stem': u'لَيْتَ',
    'type': u'STOPWORD',
    'original': u'لَيْتَ',
    'encletic': u'نَا'
}
STOPWORDS[u'وليتكما'] = {
    'word': u'وليتكما',
    'procletic': u'-وَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'وَلَيْتَكُمَا',
    'stem': u'لَيْتَ',
    'type': u'STOPWORD',
    'original': u'لَيْتَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'وليتهما'] = {
    'word': u'وليتهما',
    'procletic': u'-وَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'وَلَيْتَهُمَا',
    'stem': u'لَيْتَ',
    'type': u'STOPWORD',
    'original': u'لَيْتَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فليتي'] = {
    'word': u'فليتي',
    'procletic': u'-فَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'فَلَيْتَِي',
    'stem': u'لَيْتَ',
    'type': u'STOPWORD',
    'original': u'لَيْتَ',
    'encletic': u'ِي'
}
STOPWORDS[u'فليتك'] = {
    'word': u'فليتك',
    'procletic': u'-فَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'فَلَيْتَكَ',
    'stem': u'لَيْتَ',
    'type': u'STOPWORD',
    'original': u'لَيْتَ',
    'encletic': u'كَ'
}
STOPWORDS[u'فليتك'] = {
    'word': u'فليتك',
    'procletic': u'-فَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'فَلَيْتَكِ',
    'stem': u'لَيْتَ',
    'type': u'STOPWORD',
    'original': u'لَيْتَ',
    'encletic': u'كِ'
}
STOPWORDS[u'فليته'] = {
    'word': u'فليته',
    'procletic': u'-فَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'فَلَيْتَه',
    'stem': u'لَيْتَ',
    'type': u'STOPWORD',
    'original': u'لَيْتَ',
    'encletic': u'ه'
}
STOPWORDS[u'فليتكم'] = {
    'word': u'فليتكم',
    'procletic': u'-فَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'فَلَيْتَكُمْ',
    'stem': u'لَيْتَ',
    'type': u'STOPWORD',
    'original': u'لَيْتَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فليتكن'] = {
    'word': u'فليتكن',
    'procletic': u'-فَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'فَلَيْتَكُنَّ',
    'stem': u'لَيْتَ',
    'type': u'STOPWORD',
    'original': u'لَيْتَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فليتها'] = {
    'word': u'فليتها',
    'procletic': u'-فَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'فَلَيْتَهَا',
    'stem': u'لَيْتَ',
    'type': u'STOPWORD',
    'original': u'لَيْتَ',
    'encletic': u'هَا'
}
STOPWORDS[u'فليتهم'] = {
    'word': u'فليتهم',
    'procletic': u'-فَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'فَلَيْتَهُمْ',
    'stem': u'لَيْتَ',
    'type': u'STOPWORD',
    'original': u'لَيْتَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فليتهن'] = {
    'word': u'فليتهن',
    'procletic': u'-فَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'فَلَيْتَهُنَّ',
    'stem': u'لَيْتَ',
    'type': u'STOPWORD',
    'original': u'لَيْتَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فليتنا'] = {
    'word': u'فليتنا',
    'procletic': u'-فَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'فَلَيْتَنَا',
    'stem': u'لَيْتَ',
    'type': u'STOPWORD',
    'original': u'لَيْتَ',
    'encletic': u'نَا'
}
STOPWORDS[u'فليتكما'] = {
    'word': u'فليتكما',
    'procletic': u'-فَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'فَلَيْتَكُمَا',
    'stem': u'لَيْتَ',
    'type': u'STOPWORD',
    'original': u'لَيْتَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فليتهما'] = {
    'word': u'فليتهما',
    'procletic': u'-فَ',
    'tags': u'أداة:إن و أخواتها:معطوف:مضاف',
    'vocalized': u'فَلَيْتَهُمَا',
    'stem': u'لَيْتَ',
    'type': u'STOPWORD',
    'original': u'لَيْتَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'آي'] = {
    'word': u'آي',
    'procletic': u'',
    'tags': u'أداة:تعليل',
    'vocalized': u'آي',
    'stem': u'آي',
    'type': u'STOPWORD',
    'original': u'آي',
    'encletic': u''
}
STOPWORDS[u'كي'] = {
    'word': u'كَيْ',
    'procletic': u'',
    'tags': u'أداة:تعليل',
    'vocalized': u'كَيْ',
    'stem': u'كَيْ',
    'type': u'STOPWORD',
    'original': u'كَيْ',
    'encletic': u''
}
STOPWORDS[u'وكي'] = {
    'word': u'وكي',
    'procletic': u'-وَ',
    'tags': u'أداة:تعليل:معطوف',
    'vocalized': u'وَكَيْ',
    'stem': u'كَيْ',
    'type': u'STOPWORD',
    'original': u'كَيْ',
    'encletic': u''
}
STOPWORDS[u'فكي'] = {
    'word': u'فكي',
    'procletic': u'-فَ',
    'tags': u'أداة:تعليل:معطوف',
    'vocalized': u'فَكَيْ',
    'stem': u'كَيْ',
    'type': u'STOPWORD',
    'original': u'كَيْ',
    'encletic': u''
}
STOPWORDS[u'أجمع'] = {
    'word': u'أَجْمَعَ',
    'procletic': u'',
    'tags': u'أداة:توكيد',
    'vocalized': u'أَجْمَعَ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u''
}
STOPWORDS[u'بأجمع'] = {
    'word': u'بأجمع',
    'procletic': u'-بِ',
    'tags': u'أداة:توكيد:مجرور',
    'vocalized': u'بِأَجْمَعَ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u''
}
STOPWORDS[u'كأجمع'] = {
    'word': u'كأجمع',
    'procletic': u'-كَ',
    'tags': u'أداة:توكيد:مجرور',
    'vocalized': u'كَأَجْمَعَ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u''
}
STOPWORDS[u'لأجمع'] = {
    'word': u'لأجمع',
    'procletic': u'-لِ',
    'tags': u'أداة:توكيد:مجرور',
    'vocalized': u'لِأَجْمَعَ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u''
}
STOPWORDS[u'لأجمع'] = {
    'word': u'لأجمع',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور',
    'vocalized': u'لَأَجْمَعَ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u''
}
STOPWORDS[u'وأجمع'] = {
    'word': u'وأجمع',
    'procletic': u'-وَ',
    'tags': u'أداة:توكيد:معطوف',
    'vocalized': u'وَأَجْمَعَ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u''
}
STOPWORDS[u'فأجمع'] = {
    'word': u'فأجمع',
    'procletic': u'-فَ',
    'tags': u'أداة:توكيد:معطوف',
    'vocalized': u'فَأَجْمَعَ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u''
}
STOPWORDS[u'وبأجمع'] = {
    'word': u'وبأجمع',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف',
    'vocalized': u'وَبِأَجْمَعَ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u''
}
STOPWORDS[u'فبأجمع'] = {
    'word': u'فبأجمع',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف',
    'vocalized': u'فَبِأَجْمَعَ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u''
}
STOPWORDS[u'وكأجمع'] = {
    'word': u'وكأجمع',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف',
    'vocalized': u'وَكَأَجْمَعَ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u''
}
STOPWORDS[u'فكأجمع'] = {
    'word': u'فكأجمع',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف',
    'vocalized': u'فَكَأَجْمَعَ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u''
}
STOPWORDS[u'ولأجمع'] = {
    'word': u'ولأجمع',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف',
    'vocalized': u'وَلِأَجْمَعَ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u''
}
STOPWORDS[u'فلأجمع'] = {
    'word': u'فلأجمع',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف',
    'vocalized': u'فَلِأَجْمَعَ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u''
}
STOPWORDS[u'ولأجمع'] = {
    'word': u'ولأجمع',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'وَلَأَجْمَعَ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u''
}
STOPWORDS[u'فلأجمع'] = {
    'word': u'فلأجمع',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'فَلَأَجْمَعَ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u''
}
STOPWORDS[u'أأجمع'] = {
    'word': u'أأجمع',
    'procletic': u'-أَ',
    'tags': u'أداة:توكيد:مجرور',
    'vocalized': u'أَأَجْمَعَ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u''
}
STOPWORDS[u'أبأجمع'] = {
    'word': u'أبأجمع',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور',
    'vocalized': u'أَبِأَجْمَعَ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u''
}
STOPWORDS[u'أكأجمع'] = {
    'word': u'أكأجمع',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور',
    'vocalized': u'أَكَأَجْمَعَ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u''
}
STOPWORDS[u'ألأجمع'] = {
    'word': u'ألأجمع',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور',
    'vocalized': u'أَلِأَجْمَعَ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u''
}
STOPWORDS[u'ألأجمع'] = {
    'word': u'ألأجمع',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور',
    'vocalized': u'أَلَأَجْمَعَ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u''
}
STOPWORDS[u'أوأجمع'] = {
    'word': u'أوأجمع',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور',
    'vocalized': u'أَوَأَجْمَعَ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u''
}
STOPWORDS[u'أفأجمع'] = {
    'word': u'أفأجمع',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور',
    'vocalized': u'أَفَأَجْمَعَ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u''
}
STOPWORDS[u'أوبأجمع'] = {
    'word': u'أوبأجمع',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَبِأَجْمَعَ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u''
}
STOPWORDS[u'أفبأجمع'] = {
    'word': u'أفبأجمع',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَبِأَجْمَعَ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u''
}
STOPWORDS[u'أوكأجمع'] = {
    'word': u'أوكأجمع',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَكَأَجْمَعَ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u''
}
STOPWORDS[u'أفكأجمع'] = {
    'word': u'أفكأجمع',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَكَأَجْمَعَ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u''
}
STOPWORDS[u'أولأجمع'] = {
    'word': u'أولأجمع',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَلِأَجْمَعَ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u''
}
STOPWORDS[u'أفلأجمع'] = {
    'word': u'أفلأجمع',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَلِأَجْمَعَ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u''
}
STOPWORDS[u'أولأجمع'] = {
    'word': u'أولأجمع',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَلَأَجْمَعَ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u''
}
STOPWORDS[u'أفلأجمع'] = {
    'word': u'أفلأجمع',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَلَأَجْمَعَ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u''
}
STOPWORDS[u'أجمعي'] = {
    'word': u'أجمعي',
    'procletic': u'',
    'tags': u'أداة:توكيد:مضاف',
    'vocalized': u'أَجْمَعَِي',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أجمعك'] = {
    'word': u'أجمعك',
    'procletic': u'',
    'tags': u'أداة:توكيد:مضاف',
    'vocalized': u'أَجْمَعَكَ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أجمعك'] = {
    'word': u'أجمعك',
    'procletic': u'',
    'tags': u'أداة:توكيد:مضاف',
    'vocalized': u'أَجْمَعَكِ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أجمعه'] = {
    'word': u'أجمعه',
    'procletic': u'',
    'tags': u'أداة:توكيد:مضاف',
    'vocalized': u'أَجْمَعَه',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'ه'
}
STOPWORDS[u'أجمعكم'] = {
    'word': u'أجمعكم',
    'procletic': u'',
    'tags': u'أداة:توكيد:مضاف',
    'vocalized': u'أَجْمَعَكُمْ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أجمعكن'] = {
    'word': u'أجمعكن',
    'procletic': u'',
    'tags': u'أداة:توكيد:مضاف',
    'vocalized': u'أَجْمَعَكُنَّ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أجمعها'] = {
    'word': u'أجمعها',
    'procletic': u'',
    'tags': u'أداة:توكيد:مضاف',
    'vocalized': u'أَجْمَعَهَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أجمعهم'] = {
    'word': u'أجمعهم',
    'procletic': u'',
    'tags': u'أداة:توكيد:مضاف',
    'vocalized': u'أَجْمَعَهُمْ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أجمعهن'] = {
    'word': u'أجمعهن',
    'procletic': u'',
    'tags': u'أداة:توكيد:مضاف',
    'vocalized': u'أَجْمَعَهُنَّ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أجمعنا'] = {
    'word': u'أجمعنا',
    'procletic': u'',
    'tags': u'أداة:توكيد:مضاف',
    'vocalized': u'أَجْمَعَنَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أجمعكما'] = {
    'word': u'أجمعكما',
    'procletic': u'',
    'tags': u'أداة:توكيد:مضاف',
    'vocalized': u'أَجْمَعَكُمَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أجمعهما'] = {
    'word': u'أجمعهما',
    'procletic': u'',
    'tags': u'أداة:توكيد:مضاف',
    'vocalized': u'أَجْمَعَهُمَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'بأجمعي'] = {
    'word': u'بأجمعي',
    'procletic': u'-بِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'بِأَجْمَعَِي',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'ِي'
}
STOPWORDS[u'بأجمعك'] = {
    'word': u'بأجمعك',
    'procletic': u'-بِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'بِأَجْمَعَكَ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كَ'
}
STOPWORDS[u'بأجمعك'] = {
    'word': u'بأجمعك',
    'procletic': u'-بِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'بِأَجْمَعَكِ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كِ'
}
STOPWORDS[u'بأجمعه'] = {
    'word': u'بأجمعه',
    'procletic': u'-بِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'بِأَجْمَعَه',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'ه'
}
STOPWORDS[u'بأجمعكم'] = {
    'word': u'بأجمعكم',
    'procletic': u'-بِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'بِأَجْمَعَكُمْ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'بأجمعكن'] = {
    'word': u'بأجمعكن',
    'procletic': u'-بِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'بِأَجْمَعَكُنَّ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'بأجمعها'] = {
    'word': u'بأجمعها',
    'procletic': u'-بِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'بِأَجْمَعَهَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هَا'
}
STOPWORDS[u'بأجمعهم'] = {
    'word': u'بأجمعهم',
    'procletic': u'-بِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'بِأَجْمَعَهُمْ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'بأجمعهن'] = {
    'word': u'بأجمعهن',
    'procletic': u'-بِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'بِأَجْمَعَهُنَّ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'بأجمعنا'] = {
    'word': u'بأجمعنا',
    'procletic': u'-بِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'بِأَجْمَعَنَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'نَا'
}
STOPWORDS[u'بأجمعكما'] = {
    'word': u'بأجمعكما',
    'procletic': u'-بِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'بِأَجْمَعَكُمَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'بأجمعهما'] = {
    'word': u'بأجمعهما',
    'procletic': u'-بِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'بِأَجْمَعَهُمَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'كأجمعي'] = {
    'word': u'كأجمعي',
    'procletic': u'-كَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'كَأَجْمَعَِي',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'ِي'
}
STOPWORDS[u'كأجمعك'] = {
    'word': u'كأجمعك',
    'procletic': u'-كَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'كَأَجْمَعَكَ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كَ'
}
STOPWORDS[u'كأجمعك'] = {
    'word': u'كأجمعك',
    'procletic': u'-كَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'كَأَجْمَعَكِ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كِ'
}
STOPWORDS[u'كأجمعه'] = {
    'word': u'كأجمعه',
    'procletic': u'-كَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'كَأَجْمَعَه',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'ه'
}
STOPWORDS[u'كأجمعكم'] = {
    'word': u'كأجمعكم',
    'procletic': u'-كَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'كَأَجْمَعَكُمْ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'كأجمعكن'] = {
    'word': u'كأجمعكن',
    'procletic': u'-كَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'كَأَجْمَعَكُنَّ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'كأجمعها'] = {
    'word': u'كأجمعها',
    'procletic': u'-كَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'كَأَجْمَعَهَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هَا'
}
STOPWORDS[u'كأجمعهم'] = {
    'word': u'كأجمعهم',
    'procletic': u'-كَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'كَأَجْمَعَهُمْ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'كأجمعهن'] = {
    'word': u'كأجمعهن',
    'procletic': u'-كَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'كَأَجْمَعَهُنَّ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'كأجمعنا'] = {
    'word': u'كأجمعنا',
    'procletic': u'-كَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'كَأَجْمَعَنَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'نَا'
}
STOPWORDS[u'كأجمعكما'] = {
    'word': u'كأجمعكما',
    'procletic': u'-كَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'كَأَجْمَعَكُمَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'كأجمعهما'] = {
    'word': u'كأجمعهما',
    'procletic': u'-كَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'كَأَجْمَعَهُمَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'لأجمعي'] = {
    'word': u'لأجمعي',
    'procletic': u'-لِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'لِأَجْمَعَِي',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'ِي'
}
STOPWORDS[u'لأجمعك'] = {
    'word': u'لأجمعك',
    'procletic': u'-لِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'لِأَجْمَعَكَ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كَ'
}
STOPWORDS[u'لأجمعك'] = {
    'word': u'لأجمعك',
    'procletic': u'-لِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'لِأَجْمَعَكِ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كِ'
}
STOPWORDS[u'لأجمعه'] = {
    'word': u'لأجمعه',
    'procletic': u'-لِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'لِأَجْمَعَه',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'ه'
}
STOPWORDS[u'لأجمعكم'] = {
    'word': u'لأجمعكم',
    'procletic': u'-لِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'لِأَجْمَعَكُمْ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'لأجمعكن'] = {
    'word': u'لأجمعكن',
    'procletic': u'-لِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'لِأَجْمَعَكُنَّ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'لأجمعها'] = {
    'word': u'لأجمعها',
    'procletic': u'-لِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'لِأَجْمَعَهَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هَا'
}
STOPWORDS[u'لأجمعهم'] = {
    'word': u'لأجمعهم',
    'procletic': u'-لِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'لِأَجْمَعَهُمْ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'لأجمعهن'] = {
    'word': u'لأجمعهن',
    'procletic': u'-لِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'لِأَجْمَعَهُنَّ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'لأجمعنا'] = {
    'word': u'لأجمعنا',
    'procletic': u'-لِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'لِأَجْمَعَنَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'نَا'
}
STOPWORDS[u'لأجمعكما'] = {
    'word': u'لأجمعكما',
    'procletic': u'-لِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'لِأَجْمَعَكُمَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'لأجمعهما'] = {
    'word': u'لأجمعهما',
    'procletic': u'-لِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'لِأَجْمَعَهُمَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'لأجمعي'] = {
    'word': u'لأجمعي',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَأَجْمَعَِي',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'ِي'
}
STOPWORDS[u'لأجمعك'] = {
    'word': u'لأجمعك',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَأَجْمَعَكَ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كَ'
}
STOPWORDS[u'لأجمعك'] = {
    'word': u'لأجمعك',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَأَجْمَعَكِ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كِ'
}
STOPWORDS[u'لأجمعه'] = {
    'word': u'لأجمعه',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَأَجْمَعَه',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'ه'
}
STOPWORDS[u'لأجمعكم'] = {
    'word': u'لأجمعكم',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَأَجْمَعَكُمْ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'لأجمعكن'] = {
    'word': u'لأجمعكن',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَأَجْمَعَكُنَّ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'لأجمعها'] = {
    'word': u'لأجمعها',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَأَجْمَعَهَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هَا'
}
STOPWORDS[u'لأجمعهم'] = {
    'word': u'لأجمعهم',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَأَجْمَعَهُمْ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'لأجمعهن'] = {
    'word': u'لأجمعهن',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَأَجْمَعَهُنَّ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'لأجمعنا'] = {
    'word': u'لأجمعنا',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَأَجْمَعَنَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'نَا'
}
STOPWORDS[u'لأجمعكما'] = {
    'word': u'لأجمعكما',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَأَجْمَعَكُمَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'لأجمعهما'] = {
    'word': u'لأجمعهما',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَأَجْمَعَهُمَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'وأجمعي'] = {
    'word': u'وأجمعي',
    'procletic': u'-وَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'وَأَجْمَعَِي',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'ِي'
}
STOPWORDS[u'وأجمعك'] = {
    'word': u'وأجمعك',
    'procletic': u'-وَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'وَأَجْمَعَكَ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كَ'
}
STOPWORDS[u'وأجمعك'] = {
    'word': u'وأجمعك',
    'procletic': u'-وَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'وَأَجْمَعَكِ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كِ'
}
STOPWORDS[u'وأجمعه'] = {
    'word': u'وأجمعه',
    'procletic': u'-وَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'وَأَجْمَعَه',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'ه'
}
STOPWORDS[u'وأجمعكم'] = {
    'word': u'وأجمعكم',
    'procletic': u'-وَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'وَأَجْمَعَكُمْ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'وأجمعكن'] = {
    'word': u'وأجمعكن',
    'procletic': u'-وَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'وَأَجْمَعَكُنَّ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'وأجمعها'] = {
    'word': u'وأجمعها',
    'procletic': u'-وَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'وَأَجْمَعَهَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هَا'
}
STOPWORDS[u'وأجمعهم'] = {
    'word': u'وأجمعهم',
    'procletic': u'-وَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'وَأَجْمَعَهُمْ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'وأجمعهن'] = {
    'word': u'وأجمعهن',
    'procletic': u'-وَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'وَأَجْمَعَهُنَّ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'وأجمعنا'] = {
    'word': u'وأجمعنا',
    'procletic': u'-وَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'وَأَجْمَعَنَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'نَا'
}
STOPWORDS[u'وأجمعكما'] = {
    'word': u'وأجمعكما',
    'procletic': u'-وَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'وَأَجْمَعَكُمَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'وأجمعهما'] = {
    'word': u'وأجمعهما',
    'procletic': u'-وَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'وَأَجْمَعَهُمَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فأجمعي'] = {
    'word': u'فأجمعي',
    'procletic': u'-فَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'فَأَجْمَعَِي',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'ِي'
}
STOPWORDS[u'فأجمعك'] = {
    'word': u'فأجمعك',
    'procletic': u'-فَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'فَأَجْمَعَكَ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كَ'
}
STOPWORDS[u'فأجمعك'] = {
    'word': u'فأجمعك',
    'procletic': u'-فَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'فَأَجْمَعَكِ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كِ'
}
STOPWORDS[u'فأجمعه'] = {
    'word': u'فأجمعه',
    'procletic': u'-فَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'فَأَجْمَعَه',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'ه'
}
STOPWORDS[u'فأجمعكم'] = {
    'word': u'فأجمعكم',
    'procletic': u'-فَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'فَأَجْمَعَكُمْ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فأجمعكن'] = {
    'word': u'فأجمعكن',
    'procletic': u'-فَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'فَأَجْمَعَكُنَّ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فأجمعها'] = {
    'word': u'فأجمعها',
    'procletic': u'-فَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'فَأَجْمَعَهَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هَا'
}
STOPWORDS[u'فأجمعهم'] = {
    'word': u'فأجمعهم',
    'procletic': u'-فَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'فَأَجْمَعَهُمْ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فأجمعهن'] = {
    'word': u'فأجمعهن',
    'procletic': u'-فَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'فَأَجْمَعَهُنَّ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فأجمعنا'] = {
    'word': u'فأجمعنا',
    'procletic': u'-فَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'فَأَجْمَعَنَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'نَا'
}
STOPWORDS[u'فأجمعكما'] = {
    'word': u'فأجمعكما',
    'procletic': u'-فَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'فَأَجْمَعَكُمَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فأجمعهما'] = {
    'word': u'فأجمعهما',
    'procletic': u'-فَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'فَأَجْمَعَهُمَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'وبأجمعي'] = {
    'word': u'وبأجمعي',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِأَجْمَعَِي',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'ِي'
}
STOPWORDS[u'وبأجمعك'] = {
    'word': u'وبأجمعك',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِأَجْمَعَكَ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كَ'
}
STOPWORDS[u'وبأجمعك'] = {
    'word': u'وبأجمعك',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِأَجْمَعَكِ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كِ'
}
STOPWORDS[u'وبأجمعه'] = {
    'word': u'وبأجمعه',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِأَجْمَعَه',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'ه'
}
STOPWORDS[u'وبأجمعكم'] = {
    'word': u'وبأجمعكم',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِأَجْمَعَكُمْ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'وبأجمعكن'] = {
    'word': u'وبأجمعكن',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِأَجْمَعَكُنَّ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'وبأجمعها'] = {
    'word': u'وبأجمعها',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِأَجْمَعَهَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هَا'
}
STOPWORDS[u'وبأجمعهم'] = {
    'word': u'وبأجمعهم',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِأَجْمَعَهُمْ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'وبأجمعهن'] = {
    'word': u'وبأجمعهن',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِأَجْمَعَهُنَّ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'وبأجمعنا'] = {
    'word': u'وبأجمعنا',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِأَجْمَعَنَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'نَا'
}
STOPWORDS[u'وبأجمعكما'] = {
    'word': u'وبأجمعكما',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِأَجْمَعَكُمَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'وبأجمعهما'] = {
    'word': u'وبأجمعهما',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِأَجْمَعَهُمَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فبأجمعي'] = {
    'word': u'فبأجمعي',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِأَجْمَعَِي',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'ِي'
}
STOPWORDS[u'فبأجمعك'] = {
    'word': u'فبأجمعك',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِأَجْمَعَكَ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كَ'
}
STOPWORDS[u'فبأجمعك'] = {
    'word': u'فبأجمعك',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِأَجْمَعَكِ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كِ'
}
STOPWORDS[u'فبأجمعه'] = {
    'word': u'فبأجمعه',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِأَجْمَعَه',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'ه'
}
STOPWORDS[u'فبأجمعكم'] = {
    'word': u'فبأجمعكم',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِأَجْمَعَكُمْ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فبأجمعكن'] = {
    'word': u'فبأجمعكن',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِأَجْمَعَكُنَّ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فبأجمعها'] = {
    'word': u'فبأجمعها',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِأَجْمَعَهَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هَا'
}
STOPWORDS[u'فبأجمعهم'] = {
    'word': u'فبأجمعهم',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِأَجْمَعَهُمْ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فبأجمعهن'] = {
    'word': u'فبأجمعهن',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِأَجْمَعَهُنَّ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فبأجمعنا'] = {
    'word': u'فبأجمعنا',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِأَجْمَعَنَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'نَا'
}
STOPWORDS[u'فبأجمعكما'] = {
    'word': u'فبأجمعكما',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِأَجْمَعَكُمَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فبأجمعهما'] = {
    'word': u'فبأجمعهما',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِأَجْمَعَهُمَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'وكأجمعي'] = {
    'word': u'وكأجمعي',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَأَجْمَعَِي',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'ِي'
}
STOPWORDS[u'وكأجمعك'] = {
    'word': u'وكأجمعك',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَأَجْمَعَكَ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كَ'
}
STOPWORDS[u'وكأجمعك'] = {
    'word': u'وكأجمعك',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَأَجْمَعَكِ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كِ'
}
STOPWORDS[u'وكأجمعه'] = {
    'word': u'وكأجمعه',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَأَجْمَعَه',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'ه'
}
STOPWORDS[u'وكأجمعكم'] = {
    'word': u'وكأجمعكم',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَأَجْمَعَكُمْ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'وكأجمعكن'] = {
    'word': u'وكأجمعكن',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَأَجْمَعَكُنَّ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'وكأجمعها'] = {
    'word': u'وكأجمعها',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَأَجْمَعَهَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هَا'
}
STOPWORDS[u'وكأجمعهم'] = {
    'word': u'وكأجمعهم',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَأَجْمَعَهُمْ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'وكأجمعهن'] = {
    'word': u'وكأجمعهن',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَأَجْمَعَهُنَّ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'وكأجمعنا'] = {
    'word': u'وكأجمعنا',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَأَجْمَعَنَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'نَا'
}
STOPWORDS[u'وكأجمعكما'] = {
    'word': u'وكأجمعكما',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَأَجْمَعَكُمَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'وكأجمعهما'] = {
    'word': u'وكأجمعهما',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَأَجْمَعَهُمَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فكأجمعي'] = {
    'word': u'فكأجمعي',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَأَجْمَعَِي',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'ِي'
}
STOPWORDS[u'فكأجمعك'] = {
    'word': u'فكأجمعك',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَأَجْمَعَكَ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كَ'
}
STOPWORDS[u'فكأجمعك'] = {
    'word': u'فكأجمعك',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَأَجْمَعَكِ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كِ'
}
STOPWORDS[u'فكأجمعه'] = {
    'word': u'فكأجمعه',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَأَجْمَعَه',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'ه'
}
STOPWORDS[u'فكأجمعكم'] = {
    'word': u'فكأجمعكم',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَأَجْمَعَكُمْ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فكأجمعكن'] = {
    'word': u'فكأجمعكن',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَأَجْمَعَكُنَّ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فكأجمعها'] = {
    'word': u'فكأجمعها',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَأَجْمَعَهَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هَا'
}
STOPWORDS[u'فكأجمعهم'] = {
    'word': u'فكأجمعهم',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَأَجْمَعَهُمْ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فكأجمعهن'] = {
    'word': u'فكأجمعهن',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَأَجْمَعَهُنَّ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فكأجمعنا'] = {
    'word': u'فكأجمعنا',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَأَجْمَعَنَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'نَا'
}
STOPWORDS[u'فكأجمعكما'] = {
    'word': u'فكأجمعكما',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَأَجْمَعَكُمَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فكأجمعهما'] = {
    'word': u'فكأجمعهما',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَأَجْمَعَهُمَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ولأجمعي'] = {
    'word': u'ولأجمعي',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِأَجْمَعَِي',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'ِي'
}
STOPWORDS[u'ولأجمعك'] = {
    'word': u'ولأجمعك',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِأَجْمَعَكَ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كَ'
}
STOPWORDS[u'ولأجمعك'] = {
    'word': u'ولأجمعك',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِأَجْمَعَكِ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كِ'
}
STOPWORDS[u'ولأجمعه'] = {
    'word': u'ولأجمعه',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِأَجْمَعَه',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'ه'
}
STOPWORDS[u'ولأجمعكم'] = {
    'word': u'ولأجمعكم',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِأَجْمَعَكُمْ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ولأجمعكن'] = {
    'word': u'ولأجمعكن',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِأَجْمَعَكُنَّ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ولأجمعها'] = {
    'word': u'ولأجمعها',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِأَجْمَعَهَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هَا'
}
STOPWORDS[u'ولأجمعهم'] = {
    'word': u'ولأجمعهم',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِأَجْمَعَهُمْ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ولأجمعهن'] = {
    'word': u'ولأجمعهن',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِأَجْمَعَهُنَّ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ولأجمعنا'] = {
    'word': u'ولأجمعنا',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِأَجْمَعَنَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'نَا'
}
STOPWORDS[u'ولأجمعكما'] = {
    'word': u'ولأجمعكما',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِأَجْمَعَكُمَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ولأجمعهما'] = {
    'word': u'ولأجمعهما',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِأَجْمَعَهُمَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فلأجمعي'] = {
    'word': u'فلأجمعي',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِأَجْمَعَِي',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'ِي'
}
STOPWORDS[u'فلأجمعك'] = {
    'word': u'فلأجمعك',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِأَجْمَعَكَ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كَ'
}
STOPWORDS[u'فلأجمعك'] = {
    'word': u'فلأجمعك',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِأَجْمَعَكِ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كِ'
}
STOPWORDS[u'فلأجمعه'] = {
    'word': u'فلأجمعه',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِأَجْمَعَه',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'ه'
}
STOPWORDS[u'فلأجمعكم'] = {
    'word': u'فلأجمعكم',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِأَجْمَعَكُمْ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فلأجمعكن'] = {
    'word': u'فلأجمعكن',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِأَجْمَعَكُنَّ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فلأجمعها'] = {
    'word': u'فلأجمعها',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِأَجْمَعَهَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هَا'
}
STOPWORDS[u'فلأجمعهم'] = {
    'word': u'فلأجمعهم',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِأَجْمَعَهُمْ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فلأجمعهن'] = {
    'word': u'فلأجمعهن',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِأَجْمَعَهُنَّ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فلأجمعنا'] = {
    'word': u'فلأجمعنا',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِأَجْمَعَنَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'نَا'
}
STOPWORDS[u'فلأجمعكما'] = {
    'word': u'فلأجمعكما',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِأَجْمَعَكُمَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فلأجمعهما'] = {
    'word': u'فلأجمعهما',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِأَجْمَعَهُمَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ولأجمعي'] = {
    'word': u'ولأجمعي',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَأَجْمَعَِي',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'ِي'
}
STOPWORDS[u'ولأجمعك'] = {
    'word': u'ولأجمعك',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَأَجْمَعَكَ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كَ'
}
STOPWORDS[u'ولأجمعك'] = {
    'word': u'ولأجمعك',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَأَجْمَعَكِ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كِ'
}
STOPWORDS[u'ولأجمعه'] = {
    'word': u'ولأجمعه',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَأَجْمَعَه',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'ه'
}
STOPWORDS[u'ولأجمعكم'] = {
    'word': u'ولأجمعكم',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَأَجْمَعَكُمْ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ولأجمعكن'] = {
    'word': u'ولأجمعكن',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَأَجْمَعَكُنَّ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ولأجمعها'] = {
    'word': u'ولأجمعها',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَأَجْمَعَهَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هَا'
}
STOPWORDS[u'ولأجمعهم'] = {
    'word': u'ولأجمعهم',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَأَجْمَعَهُمْ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ولأجمعهن'] = {
    'word': u'ولأجمعهن',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَأَجْمَعَهُنَّ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ولأجمعنا'] = {
    'word': u'ولأجمعنا',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَأَجْمَعَنَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'نَا'
}
STOPWORDS[u'ولأجمعكما'] = {
    'word': u'ولأجمعكما',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَأَجْمَعَكُمَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ولأجمعهما'] = {
    'word': u'ولأجمعهما',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَأَجْمَعَهُمَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فلأجمعي'] = {
    'word': u'فلأجمعي',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَأَجْمَعَِي',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'ِي'
}
STOPWORDS[u'فلأجمعك'] = {
    'word': u'فلأجمعك',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَأَجْمَعَكَ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كَ'
}
STOPWORDS[u'فلأجمعك'] = {
    'word': u'فلأجمعك',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَأَجْمَعَكِ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كِ'
}
STOPWORDS[u'فلأجمعه'] = {
    'word': u'فلأجمعه',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَأَجْمَعَه',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'ه'
}
STOPWORDS[u'فلأجمعكم'] = {
    'word': u'فلأجمعكم',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَأَجْمَعَكُمْ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فلأجمعكن'] = {
    'word': u'فلأجمعكن',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَأَجْمَعَكُنَّ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فلأجمعها'] = {
    'word': u'فلأجمعها',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَأَجْمَعَهَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هَا'
}
STOPWORDS[u'فلأجمعهم'] = {
    'word': u'فلأجمعهم',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَأَجْمَعَهُمْ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فلأجمعهن'] = {
    'word': u'فلأجمعهن',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَأَجْمَعَهُنَّ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فلأجمعنا'] = {
    'word': u'فلأجمعنا',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَأَجْمَعَنَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'نَا'
}
STOPWORDS[u'فلأجمعكما'] = {
    'word': u'فلأجمعكما',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَأَجْمَعَكُمَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فلأجمعهما'] = {
    'word': u'فلأجمعهما',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَأَجْمَعَهُمَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أأجمعي'] = {
    'word': u'أأجمعي',
    'procletic': u'-أَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'أَأَجْمَعَِي',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أأجمعك'] = {
    'word': u'أأجمعك',
    'procletic': u'-أَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'أَأَجْمَعَكَ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أأجمعك'] = {
    'word': u'أأجمعك',
    'procletic': u'-أَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'أَأَجْمَعَكِ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أأجمعه'] = {
    'word': u'أأجمعه',
    'procletic': u'-أَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'أَأَجْمَعَه',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'ه'
}
STOPWORDS[u'أأجمعكم'] = {
    'word': u'أأجمعكم',
    'procletic': u'-أَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'أَأَجْمَعَكُمْ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أأجمعكن'] = {
    'word': u'أأجمعكن',
    'procletic': u'-أَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'أَأَجْمَعَكُنَّ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أأجمعها'] = {
    'word': u'أأجمعها',
    'procletic': u'-أَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'أَأَجْمَعَهَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أأجمعهم'] = {
    'word': u'أأجمعهم',
    'procletic': u'-أَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'أَأَجْمَعَهُمْ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أأجمعهن'] = {
    'word': u'أأجمعهن',
    'procletic': u'-أَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'أَأَجْمَعَهُنَّ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أأجمعنا'] = {
    'word': u'أأجمعنا',
    'procletic': u'-أَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'أَأَجْمَعَنَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أأجمعكما'] = {
    'word': u'أأجمعكما',
    'procletic': u'-أَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'أَأَجْمَعَكُمَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أأجمعهما'] = {
    'word': u'أأجمعهما',
    'procletic': u'-أَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'أَأَجْمَعَهُمَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أبأجمعي'] = {
    'word': u'أبأجمعي',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِأَجْمَعَِي',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أبأجمعك'] = {
    'word': u'أبأجمعك',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِأَجْمَعَكَ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أبأجمعك'] = {
    'word': u'أبأجمعك',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِأَجْمَعَكِ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أبأجمعه'] = {
    'word': u'أبأجمعه',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِأَجْمَعَه',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'ه'
}
STOPWORDS[u'أبأجمعكم'] = {
    'word': u'أبأجمعكم',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِأَجْمَعَكُمْ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أبأجمعكن'] = {
    'word': u'أبأجمعكن',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِأَجْمَعَكُنَّ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أبأجمعها'] = {
    'word': u'أبأجمعها',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِأَجْمَعَهَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أبأجمعهم'] = {
    'word': u'أبأجمعهم',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِأَجْمَعَهُمْ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أبأجمعهن'] = {
    'word': u'أبأجمعهن',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِأَجْمَعَهُنَّ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أبأجمعنا'] = {
    'word': u'أبأجمعنا',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِأَجْمَعَنَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أبأجمعكما'] = {
    'word': u'أبأجمعكما',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِأَجْمَعَكُمَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أبأجمعهما'] = {
    'word': u'أبأجمعهما',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِأَجْمَعَهُمَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أكأجمعي'] = {
    'word': u'أكأجمعي',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَأَجْمَعَِي',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أكأجمعك'] = {
    'word': u'أكأجمعك',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَأَجْمَعَكَ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أكأجمعك'] = {
    'word': u'أكأجمعك',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَأَجْمَعَكِ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أكأجمعه'] = {
    'word': u'أكأجمعه',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَأَجْمَعَه',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'ه'
}
STOPWORDS[u'أكأجمعكم'] = {
    'word': u'أكأجمعكم',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَأَجْمَعَكُمْ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أكأجمعكن'] = {
    'word': u'أكأجمعكن',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَأَجْمَعَكُنَّ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أكأجمعها'] = {
    'word': u'أكأجمعها',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَأَجْمَعَهَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أكأجمعهم'] = {
    'word': u'أكأجمعهم',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَأَجْمَعَهُمْ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أكأجمعهن'] = {
    'word': u'أكأجمعهن',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَأَجْمَعَهُنَّ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أكأجمعنا'] = {
    'word': u'أكأجمعنا',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَأَجْمَعَنَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أكأجمعكما'] = {
    'word': u'أكأجمعكما',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَأَجْمَعَكُمَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أكأجمعهما'] = {
    'word': u'أكأجمعهما',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَأَجْمَعَهُمَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ألأجمعي'] = {
    'word': u'ألأجمعي',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِأَجْمَعَِي',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'ِي'
}
STOPWORDS[u'ألأجمعك'] = {
    'word': u'ألأجمعك',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِأَجْمَعَكَ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كَ'
}
STOPWORDS[u'ألأجمعك'] = {
    'word': u'ألأجمعك',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِأَجْمَعَكِ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كِ'
}
STOPWORDS[u'ألأجمعه'] = {
    'word': u'ألأجمعه',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِأَجْمَعَه',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'ه'
}
STOPWORDS[u'ألأجمعكم'] = {
    'word': u'ألأجمعكم',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِأَجْمَعَكُمْ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ألأجمعكن'] = {
    'word': u'ألأجمعكن',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِأَجْمَعَكُنَّ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ألأجمعها'] = {
    'word': u'ألأجمعها',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِأَجْمَعَهَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هَا'
}
STOPWORDS[u'ألأجمعهم'] = {
    'word': u'ألأجمعهم',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِأَجْمَعَهُمْ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ألأجمعهن'] = {
    'word': u'ألأجمعهن',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِأَجْمَعَهُنَّ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ألأجمعنا'] = {
    'word': u'ألأجمعنا',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِأَجْمَعَنَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'نَا'
}
STOPWORDS[u'ألأجمعكما'] = {
    'word': u'ألأجمعكما',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِأَجْمَعَكُمَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ألأجمعهما'] = {
    'word': u'ألأجمعهما',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِأَجْمَعَهُمَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ألأجمعي'] = {
    'word': u'ألأجمعي',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَأَجْمَعَِي',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'ِي'
}
STOPWORDS[u'ألأجمعك'] = {
    'word': u'ألأجمعك',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَأَجْمَعَكَ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كَ'
}
STOPWORDS[u'ألأجمعك'] = {
    'word': u'ألأجمعك',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَأَجْمَعَكِ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كِ'
}
STOPWORDS[u'ألأجمعه'] = {
    'word': u'ألأجمعه',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَأَجْمَعَه',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'ه'
}
STOPWORDS[u'ألأجمعكم'] = {
    'word': u'ألأجمعكم',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَأَجْمَعَكُمْ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ألأجمعكن'] = {
    'word': u'ألأجمعكن',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَأَجْمَعَكُنَّ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ألأجمعها'] = {
    'word': u'ألأجمعها',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَأَجْمَعَهَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هَا'
}
STOPWORDS[u'ألأجمعهم'] = {
    'word': u'ألأجمعهم',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَأَجْمَعَهُمْ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ألأجمعهن'] = {
    'word': u'ألأجمعهن',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَأَجْمَعَهُنَّ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ألأجمعنا'] = {
    'word': u'ألأجمعنا',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَأَجْمَعَنَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'نَا'
}
STOPWORDS[u'ألأجمعكما'] = {
    'word': u'ألأجمعكما',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَأَجْمَعَكُمَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ألأجمعهما'] = {
    'word': u'ألأجمعهما',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَأَجْمَعَهُمَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أوأجمعي'] = {
    'word': u'أوأجمعي',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَأَجْمَعَِي',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أوأجمعك'] = {
    'word': u'أوأجمعك',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَأَجْمَعَكَ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أوأجمعك'] = {
    'word': u'أوأجمعك',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَأَجْمَعَكِ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أوأجمعه'] = {
    'word': u'أوأجمعه',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَأَجْمَعَه',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'ه'
}
STOPWORDS[u'أوأجمعكم'] = {
    'word': u'أوأجمعكم',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَأَجْمَعَكُمْ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أوأجمعكن'] = {
    'word': u'أوأجمعكن',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَأَجْمَعَكُنَّ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أوأجمعها'] = {
    'word': u'أوأجمعها',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَأَجْمَعَهَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أوأجمعهم'] = {
    'word': u'أوأجمعهم',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَأَجْمَعَهُمْ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أوأجمعهن'] = {
    'word': u'أوأجمعهن',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَأَجْمَعَهُنَّ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أوأجمعنا'] = {
    'word': u'أوأجمعنا',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَأَجْمَعَنَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أوأجمعكما'] = {
    'word': u'أوأجمعكما',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَأَجْمَعَكُمَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أوأجمعهما'] = {
    'word': u'أوأجمعهما',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَأَجْمَعَهُمَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أفأجمعي'] = {
    'word': u'أفأجمعي',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَأَجْمَعَِي',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أفأجمعك'] = {
    'word': u'أفأجمعك',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَأَجْمَعَكَ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أفأجمعك'] = {
    'word': u'أفأجمعك',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَأَجْمَعَكِ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أفأجمعه'] = {
    'word': u'أفأجمعه',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَأَجْمَعَه',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'ه'
}
STOPWORDS[u'أفأجمعكم'] = {
    'word': u'أفأجمعكم',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَأَجْمَعَكُمْ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أفأجمعكن'] = {
    'word': u'أفأجمعكن',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَأَجْمَعَكُنَّ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أفأجمعها'] = {
    'word': u'أفأجمعها',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَأَجْمَعَهَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أفأجمعهم'] = {
    'word': u'أفأجمعهم',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَأَجْمَعَهُمْ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أفأجمعهن'] = {
    'word': u'أفأجمعهن',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَأَجْمَعَهُنَّ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أفأجمعنا'] = {
    'word': u'أفأجمعنا',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَأَجْمَعَنَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أفأجمعكما'] = {
    'word': u'أفأجمعكما',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَأَجْمَعَكُمَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أفأجمعهما'] = {
    'word': u'أفأجمعهما',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَأَجْمَعَهُمَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أوبأجمعي'] = {
    'word': u'أوبأجمعي',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِأَجْمَعَِي',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أوبأجمعك'] = {
    'word': u'أوبأجمعك',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِأَجْمَعَكَ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أوبأجمعك'] = {
    'word': u'أوبأجمعك',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِأَجْمَعَكِ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أوبأجمعه'] = {
    'word': u'أوبأجمعه',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِأَجْمَعَه',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'ه'
}
STOPWORDS[u'أوبأجمعكم'] = {
    'word': u'أوبأجمعكم',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِأَجْمَعَكُمْ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أوبأجمعكن'] = {
    'word': u'أوبأجمعكن',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِأَجْمَعَكُنَّ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أوبأجمعها'] = {
    'word': u'أوبأجمعها',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِأَجْمَعَهَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أوبأجمعهم'] = {
    'word': u'أوبأجمعهم',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِأَجْمَعَهُمْ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أوبأجمعهن'] = {
    'word': u'أوبأجمعهن',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِأَجْمَعَهُنَّ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أوبأجمعنا'] = {
    'word': u'أوبأجمعنا',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِأَجْمَعَنَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أوبأجمعكما'] = {
    'word': u'أوبأجمعكما',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِأَجْمَعَكُمَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أوبأجمعهما'] = {
    'word': u'أوبأجمعهما',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِأَجْمَعَهُمَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أفبأجمعي'] = {
    'word': u'أفبأجمعي',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِأَجْمَعَِي',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أفبأجمعك'] = {
    'word': u'أفبأجمعك',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِأَجْمَعَكَ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أفبأجمعك'] = {
    'word': u'أفبأجمعك',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِأَجْمَعَكِ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أفبأجمعه'] = {
    'word': u'أفبأجمعه',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِأَجْمَعَه',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'ه'
}
STOPWORDS[u'أفبأجمعكم'] = {
    'word': u'أفبأجمعكم',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِأَجْمَعَكُمْ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أفبأجمعكن'] = {
    'word': u'أفبأجمعكن',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِأَجْمَعَكُنَّ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أفبأجمعها'] = {
    'word': u'أفبأجمعها',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِأَجْمَعَهَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أفبأجمعهم'] = {
    'word': u'أفبأجمعهم',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِأَجْمَعَهُمْ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أفبأجمعهن'] = {
    'word': u'أفبأجمعهن',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِأَجْمَعَهُنَّ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أفبأجمعنا'] = {
    'word': u'أفبأجمعنا',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِأَجْمَعَنَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أفبأجمعكما'] = {
    'word': u'أفبأجمعكما',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِأَجْمَعَكُمَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أفبأجمعهما'] = {
    'word': u'أفبأجمعهما',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِأَجْمَعَهُمَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أوكأجمعي'] = {
    'word': u'أوكأجمعي',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَأَجْمَعَِي',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أوكأجمعك'] = {
    'word': u'أوكأجمعك',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَأَجْمَعَكَ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أوكأجمعك'] = {
    'word': u'أوكأجمعك',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَأَجْمَعَكِ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أوكأجمعه'] = {
    'word': u'أوكأجمعه',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَأَجْمَعَه',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'ه'
}
STOPWORDS[u'أوكأجمعكم'] = {
    'word': u'أوكأجمعكم',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَأَجْمَعَكُمْ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أوكأجمعكن'] = {
    'word': u'أوكأجمعكن',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَأَجْمَعَكُنَّ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أوكأجمعها'] = {
    'word': u'أوكأجمعها',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَأَجْمَعَهَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أوكأجمعهم'] = {
    'word': u'أوكأجمعهم',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَأَجْمَعَهُمْ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أوكأجمعهن'] = {
    'word': u'أوكأجمعهن',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَأَجْمَعَهُنَّ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أوكأجمعنا'] = {
    'word': u'أوكأجمعنا',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَأَجْمَعَنَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أوكأجمعكما'] = {
    'word': u'أوكأجمعكما',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَأَجْمَعَكُمَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أوكأجمعهما'] = {
    'word': u'أوكأجمعهما',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَأَجْمَعَهُمَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أفكأجمعي'] = {
    'word': u'أفكأجمعي',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَأَجْمَعَِي',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أفكأجمعك'] = {
    'word': u'أفكأجمعك',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَأَجْمَعَكَ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أفكأجمعك'] = {
    'word': u'أفكأجمعك',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَأَجْمَعَكِ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أفكأجمعه'] = {
    'word': u'أفكأجمعه',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَأَجْمَعَه',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'ه'
}
STOPWORDS[u'أفكأجمعكم'] = {
    'word': u'أفكأجمعكم',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَأَجْمَعَكُمْ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أفكأجمعكن'] = {
    'word': u'أفكأجمعكن',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَأَجْمَعَكُنَّ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أفكأجمعها'] = {
    'word': u'أفكأجمعها',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَأَجْمَعَهَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أفكأجمعهم'] = {
    'word': u'أفكأجمعهم',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَأَجْمَعَهُمْ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أفكأجمعهن'] = {
    'word': u'أفكأجمعهن',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَأَجْمَعَهُنَّ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أفكأجمعنا'] = {
    'word': u'أفكأجمعنا',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَأَجْمَعَنَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أفكأجمعكما'] = {
    'word': u'أفكأجمعكما',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَأَجْمَعَكُمَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أفكأجمعهما'] = {
    'word': u'أفكأجمعهما',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَأَجْمَعَهُمَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أولأجمعي'] = {
    'word': u'أولأجمعي',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِأَجْمَعَِي',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أولأجمعك'] = {
    'word': u'أولأجمعك',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِأَجْمَعَكَ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أولأجمعك'] = {
    'word': u'أولأجمعك',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِأَجْمَعَكِ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أولأجمعه'] = {
    'word': u'أولأجمعه',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِأَجْمَعَه',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'ه'
}
STOPWORDS[u'أولأجمعكم'] = {
    'word': u'أولأجمعكم',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِأَجْمَعَكُمْ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أولأجمعكن'] = {
    'word': u'أولأجمعكن',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِأَجْمَعَكُنَّ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أولأجمعها'] = {
    'word': u'أولأجمعها',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِأَجْمَعَهَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أولأجمعهم'] = {
    'word': u'أولأجمعهم',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِأَجْمَعَهُمْ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أولأجمعهن'] = {
    'word': u'أولأجمعهن',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِأَجْمَعَهُنَّ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أولأجمعنا'] = {
    'word': u'أولأجمعنا',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِأَجْمَعَنَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أولأجمعكما'] = {
    'word': u'أولأجمعكما',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِأَجْمَعَكُمَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أولأجمعهما'] = {
    'word': u'أولأجمعهما',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِأَجْمَعَهُمَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أفلأجمعي'] = {
    'word': u'أفلأجمعي',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِأَجْمَعَِي',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أفلأجمعك'] = {
    'word': u'أفلأجمعك',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِأَجْمَعَكَ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أفلأجمعك'] = {
    'word': u'أفلأجمعك',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِأَجْمَعَكِ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أفلأجمعه'] = {
    'word': u'أفلأجمعه',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِأَجْمَعَه',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'ه'
}
STOPWORDS[u'أفلأجمعكم'] = {
    'word': u'أفلأجمعكم',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِأَجْمَعَكُمْ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أفلأجمعكن'] = {
    'word': u'أفلأجمعكن',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِأَجْمَعَكُنَّ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أفلأجمعها'] = {
    'word': u'أفلأجمعها',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِأَجْمَعَهَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أفلأجمعهم'] = {
    'word': u'أفلأجمعهم',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِأَجْمَعَهُمْ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أفلأجمعهن'] = {
    'word': u'أفلأجمعهن',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِأَجْمَعَهُنَّ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أفلأجمعنا'] = {
    'word': u'أفلأجمعنا',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِأَجْمَعَنَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أفلأجمعكما'] = {
    'word': u'أفلأجمعكما',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِأَجْمَعَكُمَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أفلأجمعهما'] = {
    'word': u'أفلأجمعهما',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِأَجْمَعَهُمَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أولأجمعي'] = {
    'word': u'أولأجمعي',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَأَجْمَعَِي',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أولأجمعك'] = {
    'word': u'أولأجمعك',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَأَجْمَعَكَ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أولأجمعك'] = {
    'word': u'أولأجمعك',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَأَجْمَعَكِ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أولأجمعه'] = {
    'word': u'أولأجمعه',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَأَجْمَعَه',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'ه'
}
STOPWORDS[u'أولأجمعكم'] = {
    'word': u'أولأجمعكم',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَأَجْمَعَكُمْ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أولأجمعكن'] = {
    'word': u'أولأجمعكن',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَأَجْمَعَكُنَّ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أولأجمعها'] = {
    'word': u'أولأجمعها',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَأَجْمَعَهَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أولأجمعهم'] = {
    'word': u'أولأجمعهم',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَأَجْمَعَهُمْ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أولأجمعهن'] = {
    'word': u'أولأجمعهن',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَأَجْمَعَهُنَّ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أولأجمعنا'] = {
    'word': u'أولأجمعنا',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَأَجْمَعَنَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أولأجمعكما'] = {
    'word': u'أولأجمعكما',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَأَجْمَعَكُمَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أولأجمعهما'] = {
    'word': u'أولأجمعهما',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَأَجْمَعَهُمَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أفلأجمعي'] = {
    'word': u'أفلأجمعي',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَأَجْمَعَِي',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أفلأجمعك'] = {
    'word': u'أفلأجمعك',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَأَجْمَعَكَ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أفلأجمعك'] = {
    'word': u'أفلأجمعك',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَأَجْمَعَكِ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أفلأجمعه'] = {
    'word': u'أفلأجمعه',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَأَجْمَعَه',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'ه'
}
STOPWORDS[u'أفلأجمعكم'] = {
    'word': u'أفلأجمعكم',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَأَجْمَعَكُمْ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أفلأجمعكن'] = {
    'word': u'أفلأجمعكن',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَأَجْمَعَكُنَّ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أفلأجمعها'] = {
    'word': u'أفلأجمعها',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَأَجْمَعَهَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أفلأجمعهم'] = {
    'word': u'أفلأجمعهم',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَأَجْمَعَهُمْ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أفلأجمعهن'] = {
    'word': u'أفلأجمعهن',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَأَجْمَعَهُنَّ',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أفلأجمعنا'] = {
    'word': u'أفلأجمعنا',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَأَجْمَعَنَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أفلأجمعكما'] = {
    'word': u'أفلأجمعكما',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَأَجْمَعَكُمَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أفلأجمعهما'] = {
    'word': u'أفلأجمعهما',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَأَجْمَعَهُمَا',
    'stem': u'أَجْمَعَ',
    'type': u'STOPWORD',
    'original': u'أَجْمَعَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'جميع'] = {
    'word': u'جَمِيعَ',
    'procletic': u'',
    'tags': u'أداة:توكيد',
    'vocalized': u'جَمِيعَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u''
}
STOPWORDS[u'بجميع'] = {
    'word': u'بجميع',
    'procletic': u'-بِ',
    'tags': u'أداة:توكيد:مجرور',
    'vocalized': u'بِجَمِيعَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u''
}
STOPWORDS[u'كجميع'] = {
    'word': u'كجميع',
    'procletic': u'-كَ',
    'tags': u'أداة:توكيد:مجرور',
    'vocalized': u'كَجَمِيعَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u''
}
STOPWORDS[u'لجميع'] = {
    'word': u'لجميع',
    'procletic': u'-لِ',
    'tags': u'أداة:توكيد:مجرور',
    'vocalized': u'لِجَمِيعَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u''
}
STOPWORDS[u'لجميع'] = {
    'word': u'لجميع',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور',
    'vocalized': u'لَجَمِيعَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u''
}
STOPWORDS[u'وجميع'] = {
    'word': u'وجميع',
    'procletic': u'-وَ',
    'tags': u'أداة:توكيد:معطوف',
    'vocalized': u'وَجَمِيعَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u''
}
STOPWORDS[u'فجميع'] = {
    'word': u'فجميع',
    'procletic': u'-فَ',
    'tags': u'أداة:توكيد:معطوف',
    'vocalized': u'فَجَمِيعَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u''
}
STOPWORDS[u'وبجميع'] = {
    'word': u'وبجميع',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف',
    'vocalized': u'وَبِجَمِيعَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u''
}
STOPWORDS[u'فبجميع'] = {
    'word': u'فبجميع',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف',
    'vocalized': u'فَبِجَمِيعَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u''
}
STOPWORDS[u'وكجميع'] = {
    'word': u'وكجميع',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف',
    'vocalized': u'وَكَجَمِيعَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u''
}
STOPWORDS[u'فكجميع'] = {
    'word': u'فكجميع',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف',
    'vocalized': u'فَكَجَمِيعَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u''
}
STOPWORDS[u'ولجميع'] = {
    'word': u'ولجميع',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف',
    'vocalized': u'وَلِجَمِيعَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u''
}
STOPWORDS[u'فلجميع'] = {
    'word': u'فلجميع',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف',
    'vocalized': u'فَلِجَمِيعَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u''
}
STOPWORDS[u'ولجميع'] = {
    'word': u'ولجميع',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'وَلَجَمِيعَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u''
}
STOPWORDS[u'فلجميع'] = {
    'word': u'فلجميع',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'فَلَجَمِيعَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u''
}
STOPWORDS[u'أجميع'] = {
    'word': u'أجميع',
    'procletic': u'-أَ',
    'tags': u'أداة:توكيد:مجرور',
    'vocalized': u'أَجَمِيعَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u''
}
STOPWORDS[u'أبجميع'] = {
    'word': u'أبجميع',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور',
    'vocalized': u'أَبِجَمِيعَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u''
}
STOPWORDS[u'أكجميع'] = {
    'word': u'أكجميع',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور',
    'vocalized': u'أَكَجَمِيعَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u''
}
STOPWORDS[u'ألجميع'] = {
    'word': u'ألجميع',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور',
    'vocalized': u'أَلِجَمِيعَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u''
}
STOPWORDS[u'ألجميع'] = {
    'word': u'ألجميع',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور',
    'vocalized': u'أَلَجَمِيعَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u''
}
STOPWORDS[u'أوجميع'] = {
    'word': u'أوجميع',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور',
    'vocalized': u'أَوَجَمِيعَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u''
}
STOPWORDS[u'أفجميع'] = {
    'word': u'أفجميع',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور',
    'vocalized': u'أَفَجَمِيعَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u''
}
STOPWORDS[u'أوبجميع'] = {
    'word': u'أوبجميع',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَبِجَمِيعَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u''
}
STOPWORDS[u'أفبجميع'] = {
    'word': u'أفبجميع',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَبِجَمِيعَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u''
}
STOPWORDS[u'أوكجميع'] = {
    'word': u'أوكجميع',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَكَجَمِيعَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u''
}
STOPWORDS[u'أفكجميع'] = {
    'word': u'أفكجميع',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَكَجَمِيعَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u''
}
STOPWORDS[u'أولجميع'] = {
    'word': u'أولجميع',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَلِجَمِيعَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u''
}
STOPWORDS[u'أفلجميع'] = {
    'word': u'أفلجميع',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَلِجَمِيعَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u''
}
STOPWORDS[u'أولجميع'] = {
    'word': u'أولجميع',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَلَجَمِيعَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u''
}
STOPWORDS[u'أفلجميع'] = {
    'word': u'أفلجميع',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَلَجَمِيعَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u''
}
STOPWORDS[u'جميعي'] = {
    'word': u'جميعي',
    'procletic': u'',
    'tags': u'أداة:توكيد:مضاف',
    'vocalized': u'جَمِيعَِي',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ِي'
}
STOPWORDS[u'جميعك'] = {
    'word': u'جميعك',
    'procletic': u'',
    'tags': u'أداة:توكيد:مضاف',
    'vocalized': u'جَمِيعَكَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كَ'
}
STOPWORDS[u'جميعك'] = {
    'word': u'جميعك',
    'procletic': u'',
    'tags': u'أداة:توكيد:مضاف',
    'vocalized': u'جَمِيعَكِ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كِ'
}
STOPWORDS[u'جميعه'] = {
    'word': u'جميعه',
    'procletic': u'',
    'tags': u'أداة:توكيد:مضاف',
    'vocalized': u'جَمِيعَه',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ه'
}
STOPWORDS[u'جميعكم'] = {
    'word': u'جميعكم',
    'procletic': u'',
    'tags': u'أداة:توكيد:مضاف',
    'vocalized': u'جَمِيعَكُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'جميعكن'] = {
    'word': u'جميعكن',
    'procletic': u'',
    'tags': u'أداة:توكيد:مضاف',
    'vocalized': u'جَمِيعَكُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'جميعها'] = {
    'word': u'جميعها',
    'procletic': u'',
    'tags': u'أداة:توكيد:مضاف',
    'vocalized': u'جَمِيعَهَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هَا'
}
STOPWORDS[u'جميعهم'] = {
    'word': u'جميعهم',
    'procletic': u'',
    'tags': u'أداة:توكيد:مضاف',
    'vocalized': u'جَمِيعَهُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'جميعهن'] = {
    'word': u'جميعهن',
    'procletic': u'',
    'tags': u'أداة:توكيد:مضاف',
    'vocalized': u'جَمِيعَهُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'جميعنا'] = {
    'word': u'جميعنا',
    'procletic': u'',
    'tags': u'أداة:توكيد:مضاف',
    'vocalized': u'جَمِيعَنَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'نَا'
}
STOPWORDS[u'جميعكما'] = {
    'word': u'جميعكما',
    'procletic': u'',
    'tags': u'أداة:توكيد:مضاف',
    'vocalized': u'جَمِيعَكُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'جميعهما'] = {
    'word': u'جميعهما',
    'procletic': u'',
    'tags': u'أداة:توكيد:مضاف',
    'vocalized': u'جَمِيعَهُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'بجميعي'] = {
    'word': u'بجميعي',
    'procletic': u'-بِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'بِجَمِيعَِي',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ِي'
}
STOPWORDS[u'بجميعك'] = {
    'word': u'بجميعك',
    'procletic': u'-بِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'بِجَمِيعَكَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كَ'
}
STOPWORDS[u'بجميعك'] = {
    'word': u'بجميعك',
    'procletic': u'-بِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'بِجَمِيعَكِ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كِ'
}
STOPWORDS[u'بجميعه'] = {
    'word': u'بجميعه',
    'procletic': u'-بِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'بِجَمِيعَه',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ه'
}
STOPWORDS[u'بجميعكم'] = {
    'word': u'بجميعكم',
    'procletic': u'-بِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'بِجَمِيعَكُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'بجميعكن'] = {
    'word': u'بجميعكن',
    'procletic': u'-بِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'بِجَمِيعَكُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'بجميعها'] = {
    'word': u'بجميعها',
    'procletic': u'-بِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'بِجَمِيعَهَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هَا'
}
STOPWORDS[u'بجميعهم'] = {
    'word': u'بجميعهم',
    'procletic': u'-بِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'بِجَمِيعَهُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'بجميعهن'] = {
    'word': u'بجميعهن',
    'procletic': u'-بِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'بِجَمِيعَهُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'بجميعنا'] = {
    'word': u'بجميعنا',
    'procletic': u'-بِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'بِجَمِيعَنَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'نَا'
}
STOPWORDS[u'بجميعكما'] = {
    'word': u'بجميعكما',
    'procletic': u'-بِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'بِجَمِيعَكُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'بجميعهما'] = {
    'word': u'بجميعهما',
    'procletic': u'-بِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'بِجَمِيعَهُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'كجميعي'] = {
    'word': u'كجميعي',
    'procletic': u'-كَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'كَجَمِيعَِي',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ِي'
}
STOPWORDS[u'كجميعك'] = {
    'word': u'كجميعك',
    'procletic': u'-كَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'كَجَمِيعَكَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كَ'
}
STOPWORDS[u'كجميعك'] = {
    'word': u'كجميعك',
    'procletic': u'-كَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'كَجَمِيعَكِ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كِ'
}
STOPWORDS[u'كجميعه'] = {
    'word': u'كجميعه',
    'procletic': u'-كَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'كَجَمِيعَه',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ه'
}
STOPWORDS[u'كجميعكم'] = {
    'word': u'كجميعكم',
    'procletic': u'-كَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'كَجَمِيعَكُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'كجميعكن'] = {
    'word': u'كجميعكن',
    'procletic': u'-كَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'كَجَمِيعَكُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'كجميعها'] = {
    'word': u'كجميعها',
    'procletic': u'-كَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'كَجَمِيعَهَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هَا'
}
STOPWORDS[u'كجميعهم'] = {
    'word': u'كجميعهم',
    'procletic': u'-كَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'كَجَمِيعَهُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'كجميعهن'] = {
    'word': u'كجميعهن',
    'procletic': u'-كَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'كَجَمِيعَهُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'كجميعنا'] = {
    'word': u'كجميعنا',
    'procletic': u'-كَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'كَجَمِيعَنَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'نَا'
}
STOPWORDS[u'كجميعكما'] = {
    'word': u'كجميعكما',
    'procletic': u'-كَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'كَجَمِيعَكُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'كجميعهما'] = {
    'word': u'كجميعهما',
    'procletic': u'-كَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'كَجَمِيعَهُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'لجميعي'] = {
    'word': u'لجميعي',
    'procletic': u'-لِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'لِجَمِيعَِي',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ِي'
}
STOPWORDS[u'لجميعك'] = {
    'word': u'لجميعك',
    'procletic': u'-لِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'لِجَمِيعَكَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كَ'
}
STOPWORDS[u'لجميعك'] = {
    'word': u'لجميعك',
    'procletic': u'-لِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'لِجَمِيعَكِ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كِ'
}
STOPWORDS[u'لجميعه'] = {
    'word': u'لجميعه',
    'procletic': u'-لِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'لِجَمِيعَه',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ه'
}
STOPWORDS[u'لجميعكم'] = {
    'word': u'لجميعكم',
    'procletic': u'-لِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'لِجَمِيعَكُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'لجميعكن'] = {
    'word': u'لجميعكن',
    'procletic': u'-لِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'لِجَمِيعَكُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'لجميعها'] = {
    'word': u'لجميعها',
    'procletic': u'-لِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'لِجَمِيعَهَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هَا'
}
STOPWORDS[u'لجميعهم'] = {
    'word': u'لجميعهم',
    'procletic': u'-لِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'لِجَمِيعَهُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'لجميعهن'] = {
    'word': u'لجميعهن',
    'procletic': u'-لِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'لِجَمِيعَهُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'لجميعنا'] = {
    'word': u'لجميعنا',
    'procletic': u'-لِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'لِجَمِيعَنَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'نَا'
}
STOPWORDS[u'لجميعكما'] = {
    'word': u'لجميعكما',
    'procletic': u'-لِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'لِجَمِيعَكُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'لجميعهما'] = {
    'word': u'لجميعهما',
    'procletic': u'-لِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'لِجَمِيعَهُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'لجميعي'] = {
    'word': u'لجميعي',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَجَمِيعَِي',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ِي'
}
STOPWORDS[u'لجميعك'] = {
    'word': u'لجميعك',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَجَمِيعَكَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كَ'
}
STOPWORDS[u'لجميعك'] = {
    'word': u'لجميعك',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَجَمِيعَكِ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كِ'
}
STOPWORDS[u'لجميعه'] = {
    'word': u'لجميعه',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَجَمِيعَه',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ه'
}
STOPWORDS[u'لجميعكم'] = {
    'word': u'لجميعكم',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَجَمِيعَكُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'لجميعكن'] = {
    'word': u'لجميعكن',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَجَمِيعَكُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'لجميعها'] = {
    'word': u'لجميعها',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَجَمِيعَهَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هَا'
}
STOPWORDS[u'لجميعهم'] = {
    'word': u'لجميعهم',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَجَمِيعَهُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'لجميعهن'] = {
    'word': u'لجميعهن',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَجَمِيعَهُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'لجميعنا'] = {
    'word': u'لجميعنا',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَجَمِيعَنَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'نَا'
}
STOPWORDS[u'لجميعكما'] = {
    'word': u'لجميعكما',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَجَمِيعَكُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'لجميعهما'] = {
    'word': u'لجميعهما',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَجَمِيعَهُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'وجميعي'] = {
    'word': u'وجميعي',
    'procletic': u'-وَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'وَجَمِيعَِي',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ِي'
}
STOPWORDS[u'وجميعك'] = {
    'word': u'وجميعك',
    'procletic': u'-وَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'وَجَمِيعَكَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كَ'
}
STOPWORDS[u'وجميعك'] = {
    'word': u'وجميعك',
    'procletic': u'-وَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'وَجَمِيعَكِ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كِ'
}
STOPWORDS[u'وجميعه'] = {
    'word': u'وجميعه',
    'procletic': u'-وَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'وَجَمِيعَه',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ه'
}
STOPWORDS[u'وجميعكم'] = {
    'word': u'وجميعكم',
    'procletic': u'-وَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'وَجَمِيعَكُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'وجميعكن'] = {
    'word': u'وجميعكن',
    'procletic': u'-وَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'وَجَمِيعَكُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'وجميعها'] = {
    'word': u'وجميعها',
    'procletic': u'-وَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'وَجَمِيعَهَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هَا'
}
STOPWORDS[u'وجميعهم'] = {
    'word': u'وجميعهم',
    'procletic': u'-وَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'وَجَمِيعَهُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'وجميعهن'] = {
    'word': u'وجميعهن',
    'procletic': u'-وَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'وَجَمِيعَهُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'وجميعنا'] = {
    'word': u'وجميعنا',
    'procletic': u'-وَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'وَجَمِيعَنَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'نَا'
}
STOPWORDS[u'وجميعكما'] = {
    'word': u'وجميعكما',
    'procletic': u'-وَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'وَجَمِيعَكُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'وجميعهما'] = {
    'word': u'وجميعهما',
    'procletic': u'-وَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'وَجَمِيعَهُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فجميعي'] = {
    'word': u'فجميعي',
    'procletic': u'-فَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'فَجَمِيعَِي',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ِي'
}
STOPWORDS[u'فجميعك'] = {
    'word': u'فجميعك',
    'procletic': u'-فَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'فَجَمِيعَكَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كَ'
}
STOPWORDS[u'فجميعك'] = {
    'word': u'فجميعك',
    'procletic': u'-فَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'فَجَمِيعَكِ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كِ'
}
STOPWORDS[u'فجميعه'] = {
    'word': u'فجميعه',
    'procletic': u'-فَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'فَجَمِيعَه',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ه'
}
STOPWORDS[u'فجميعكم'] = {
    'word': u'فجميعكم',
    'procletic': u'-فَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'فَجَمِيعَكُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فجميعكن'] = {
    'word': u'فجميعكن',
    'procletic': u'-فَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'فَجَمِيعَكُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فجميعها'] = {
    'word': u'فجميعها',
    'procletic': u'-فَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'فَجَمِيعَهَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هَا'
}
STOPWORDS[u'فجميعهم'] = {
    'word': u'فجميعهم',
    'procletic': u'-فَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'فَجَمِيعَهُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فجميعهن'] = {
    'word': u'فجميعهن',
    'procletic': u'-فَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'فَجَمِيعَهُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فجميعنا'] = {
    'word': u'فجميعنا',
    'procletic': u'-فَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'فَجَمِيعَنَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'نَا'
}
STOPWORDS[u'فجميعكما'] = {
    'word': u'فجميعكما',
    'procletic': u'-فَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'فَجَمِيعَكُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فجميعهما'] = {
    'word': u'فجميعهما',
    'procletic': u'-فَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'فَجَمِيعَهُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'وبجميعي'] = {
    'word': u'وبجميعي',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِجَمِيعَِي',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ِي'
}
STOPWORDS[u'وبجميعك'] = {
    'word': u'وبجميعك',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِجَمِيعَكَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كَ'
}
STOPWORDS[u'وبجميعك'] = {
    'word': u'وبجميعك',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِجَمِيعَكِ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كِ'
}
STOPWORDS[u'وبجميعه'] = {
    'word': u'وبجميعه',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِجَمِيعَه',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ه'
}
STOPWORDS[u'وبجميعكم'] = {
    'word': u'وبجميعكم',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِجَمِيعَكُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'وبجميعكن'] = {
    'word': u'وبجميعكن',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِجَمِيعَكُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'وبجميعها'] = {
    'word': u'وبجميعها',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِجَمِيعَهَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هَا'
}
STOPWORDS[u'وبجميعهم'] = {
    'word': u'وبجميعهم',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِجَمِيعَهُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'وبجميعهن'] = {
    'word': u'وبجميعهن',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِجَمِيعَهُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'وبجميعنا'] = {
    'word': u'وبجميعنا',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِجَمِيعَنَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'نَا'
}
STOPWORDS[u'وبجميعكما'] = {
    'word': u'وبجميعكما',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِجَمِيعَكُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'وبجميعهما'] = {
    'word': u'وبجميعهما',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِجَمِيعَهُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فبجميعي'] = {
    'word': u'فبجميعي',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِجَمِيعَِي',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ِي'
}
STOPWORDS[u'فبجميعك'] = {
    'word': u'فبجميعك',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِجَمِيعَكَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كَ'
}
STOPWORDS[u'فبجميعك'] = {
    'word': u'فبجميعك',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِجَمِيعَكِ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كِ'
}
STOPWORDS[u'فبجميعه'] = {
    'word': u'فبجميعه',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِجَمِيعَه',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ه'
}
STOPWORDS[u'فبجميعكم'] = {
    'word': u'فبجميعكم',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِجَمِيعَكُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فبجميعكن'] = {
    'word': u'فبجميعكن',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِجَمِيعَكُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فبجميعها'] = {
    'word': u'فبجميعها',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِجَمِيعَهَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هَا'
}
STOPWORDS[u'فبجميعهم'] = {
    'word': u'فبجميعهم',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِجَمِيعَهُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فبجميعهن'] = {
    'word': u'فبجميعهن',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِجَمِيعَهُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فبجميعنا'] = {
    'word': u'فبجميعنا',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِجَمِيعَنَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'نَا'
}
STOPWORDS[u'فبجميعكما'] = {
    'word': u'فبجميعكما',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِجَمِيعَكُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فبجميعهما'] = {
    'word': u'فبجميعهما',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِجَمِيعَهُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'وكجميعي'] = {
    'word': u'وكجميعي',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَجَمِيعَِي',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ِي'
}
STOPWORDS[u'وكجميعك'] = {
    'word': u'وكجميعك',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَجَمِيعَكَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كَ'
}
STOPWORDS[u'وكجميعك'] = {
    'word': u'وكجميعك',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَجَمِيعَكِ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كِ'
}
STOPWORDS[u'وكجميعه'] = {
    'word': u'وكجميعه',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَجَمِيعَه',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ه'
}
STOPWORDS[u'وكجميعكم'] = {
    'word': u'وكجميعكم',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَجَمِيعَكُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'وكجميعكن'] = {
    'word': u'وكجميعكن',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَجَمِيعَكُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'وكجميعها'] = {
    'word': u'وكجميعها',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَجَمِيعَهَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هَا'
}
STOPWORDS[u'وكجميعهم'] = {
    'word': u'وكجميعهم',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَجَمِيعَهُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'وكجميعهن'] = {
    'word': u'وكجميعهن',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَجَمِيعَهُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'وكجميعنا'] = {
    'word': u'وكجميعنا',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَجَمِيعَنَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'نَا'
}
STOPWORDS[u'وكجميعكما'] = {
    'word': u'وكجميعكما',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَجَمِيعَكُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'وكجميعهما'] = {
    'word': u'وكجميعهما',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَجَمِيعَهُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فكجميعي'] = {
    'word': u'فكجميعي',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَجَمِيعَِي',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ِي'
}
STOPWORDS[u'فكجميعك'] = {
    'word': u'فكجميعك',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَجَمِيعَكَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كَ'
}
STOPWORDS[u'فكجميعك'] = {
    'word': u'فكجميعك',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَجَمِيعَكِ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كِ'
}
STOPWORDS[u'فكجميعه'] = {
    'word': u'فكجميعه',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَجَمِيعَه',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ه'
}
STOPWORDS[u'فكجميعكم'] = {
    'word': u'فكجميعكم',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَجَمِيعَكُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فكجميعكن'] = {
    'word': u'فكجميعكن',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَجَمِيعَكُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فكجميعها'] = {
    'word': u'فكجميعها',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَجَمِيعَهَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هَا'
}
STOPWORDS[u'فكجميعهم'] = {
    'word': u'فكجميعهم',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَجَمِيعَهُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فكجميعهن'] = {
    'word': u'فكجميعهن',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَجَمِيعَهُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فكجميعنا'] = {
    'word': u'فكجميعنا',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَجَمِيعَنَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'نَا'
}
STOPWORDS[u'فكجميعكما'] = {
    'word': u'فكجميعكما',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَجَمِيعَكُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فكجميعهما'] = {
    'word': u'فكجميعهما',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَجَمِيعَهُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ولجميعي'] = {
    'word': u'ولجميعي',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِجَمِيعَِي',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ِي'
}
STOPWORDS[u'ولجميعك'] = {
    'word': u'ولجميعك',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِجَمِيعَكَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كَ'
}
STOPWORDS[u'ولجميعك'] = {
    'word': u'ولجميعك',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِجَمِيعَكِ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كِ'
}
STOPWORDS[u'ولجميعه'] = {
    'word': u'ولجميعه',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِجَمِيعَه',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ه'
}
STOPWORDS[u'ولجميعكم'] = {
    'word': u'ولجميعكم',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِجَمِيعَكُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ولجميعكن'] = {
    'word': u'ولجميعكن',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِجَمِيعَكُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ولجميعها'] = {
    'word': u'ولجميعها',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِجَمِيعَهَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هَا'
}
STOPWORDS[u'ولجميعهم'] = {
    'word': u'ولجميعهم',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِجَمِيعَهُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ولجميعهن'] = {
    'word': u'ولجميعهن',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِجَمِيعَهُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ولجميعنا'] = {
    'word': u'ولجميعنا',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِجَمِيعَنَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'نَا'
}
STOPWORDS[u'ولجميعكما'] = {
    'word': u'ولجميعكما',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِجَمِيعَكُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ولجميعهما'] = {
    'word': u'ولجميعهما',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِجَمِيعَهُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فلجميعي'] = {
    'word': u'فلجميعي',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِجَمِيعَِي',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ِي'
}
STOPWORDS[u'فلجميعك'] = {
    'word': u'فلجميعك',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِجَمِيعَكَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كَ'
}
STOPWORDS[u'فلجميعك'] = {
    'word': u'فلجميعك',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِجَمِيعَكِ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كِ'
}
STOPWORDS[u'فلجميعه'] = {
    'word': u'فلجميعه',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِجَمِيعَه',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ه'
}
STOPWORDS[u'فلجميعكم'] = {
    'word': u'فلجميعكم',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِجَمِيعَكُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فلجميعكن'] = {
    'word': u'فلجميعكن',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِجَمِيعَكُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فلجميعها'] = {
    'word': u'فلجميعها',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِجَمِيعَهَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هَا'
}
STOPWORDS[u'فلجميعهم'] = {
    'word': u'فلجميعهم',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِجَمِيعَهُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فلجميعهن'] = {
    'word': u'فلجميعهن',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِجَمِيعَهُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فلجميعنا'] = {
    'word': u'فلجميعنا',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِجَمِيعَنَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'نَا'
}
STOPWORDS[u'فلجميعكما'] = {
    'word': u'فلجميعكما',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِجَمِيعَكُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فلجميعهما'] = {
    'word': u'فلجميعهما',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِجَمِيعَهُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ولجميعي'] = {
    'word': u'ولجميعي',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَجَمِيعَِي',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ِي'
}
STOPWORDS[u'ولجميعك'] = {
    'word': u'ولجميعك',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَجَمِيعَكَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كَ'
}
STOPWORDS[u'ولجميعك'] = {
    'word': u'ولجميعك',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَجَمِيعَكِ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كِ'
}
STOPWORDS[u'ولجميعه'] = {
    'word': u'ولجميعه',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَجَمِيعَه',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ه'
}
STOPWORDS[u'ولجميعكم'] = {
    'word': u'ولجميعكم',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَجَمِيعَكُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ولجميعكن'] = {
    'word': u'ولجميعكن',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَجَمِيعَكُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ولجميعها'] = {
    'word': u'ولجميعها',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَجَمِيعَهَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هَا'
}
STOPWORDS[u'ولجميعهم'] = {
    'word': u'ولجميعهم',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَجَمِيعَهُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ولجميعهن'] = {
    'word': u'ولجميعهن',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَجَمِيعَهُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ولجميعنا'] = {
    'word': u'ولجميعنا',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَجَمِيعَنَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'نَا'
}
STOPWORDS[u'ولجميعكما'] = {
    'word': u'ولجميعكما',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَجَمِيعَكُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ولجميعهما'] = {
    'word': u'ولجميعهما',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَجَمِيعَهُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فلجميعي'] = {
    'word': u'فلجميعي',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَجَمِيعَِي',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ِي'
}
STOPWORDS[u'فلجميعك'] = {
    'word': u'فلجميعك',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَجَمِيعَكَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كَ'
}
STOPWORDS[u'فلجميعك'] = {
    'word': u'فلجميعك',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَجَمِيعَكِ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كِ'
}
STOPWORDS[u'فلجميعه'] = {
    'word': u'فلجميعه',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَجَمِيعَه',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ه'
}
STOPWORDS[u'فلجميعكم'] = {
    'word': u'فلجميعكم',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَجَمِيعَكُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فلجميعكن'] = {
    'word': u'فلجميعكن',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَجَمِيعَكُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فلجميعها'] = {
    'word': u'فلجميعها',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَجَمِيعَهَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هَا'
}
STOPWORDS[u'فلجميعهم'] = {
    'word': u'فلجميعهم',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَجَمِيعَهُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فلجميعهن'] = {
    'word': u'فلجميعهن',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَجَمِيعَهُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فلجميعنا'] = {
    'word': u'فلجميعنا',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَجَمِيعَنَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'نَا'
}
STOPWORDS[u'فلجميعكما'] = {
    'word': u'فلجميعكما',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَجَمِيعَكُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فلجميعهما'] = {
    'word': u'فلجميعهما',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَجَمِيعَهُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أجميعي'] = {
    'word': u'أجميعي',
    'procletic': u'-أَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'أَجَمِيعَِي',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أجميعك'] = {
    'word': u'أجميعك',
    'procletic': u'-أَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'أَجَمِيعَكَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أجميعك'] = {
    'word': u'أجميعك',
    'procletic': u'-أَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'أَجَمِيعَكِ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أجميعه'] = {
    'word': u'أجميعه',
    'procletic': u'-أَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'أَجَمِيعَه',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ه'
}
STOPWORDS[u'أجميعكم'] = {
    'word': u'أجميعكم',
    'procletic': u'-أَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'أَجَمِيعَكُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أجميعكن'] = {
    'word': u'أجميعكن',
    'procletic': u'-أَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'أَجَمِيعَكُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أجميعها'] = {
    'word': u'أجميعها',
    'procletic': u'-أَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'أَجَمِيعَهَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أجميعهم'] = {
    'word': u'أجميعهم',
    'procletic': u'-أَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'أَجَمِيعَهُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أجميعهن'] = {
    'word': u'أجميعهن',
    'procletic': u'-أَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'أَجَمِيعَهُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أجميعنا'] = {
    'word': u'أجميعنا',
    'procletic': u'-أَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'أَجَمِيعَنَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أجميعكما'] = {
    'word': u'أجميعكما',
    'procletic': u'-أَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'أَجَمِيعَكُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أجميعهما'] = {
    'word': u'أجميعهما',
    'procletic': u'-أَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'أَجَمِيعَهُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أبجميعي'] = {
    'word': u'أبجميعي',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِجَمِيعَِي',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أبجميعك'] = {
    'word': u'أبجميعك',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِجَمِيعَكَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أبجميعك'] = {
    'word': u'أبجميعك',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِجَمِيعَكِ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أبجميعه'] = {
    'word': u'أبجميعه',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِجَمِيعَه',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ه'
}
STOPWORDS[u'أبجميعكم'] = {
    'word': u'أبجميعكم',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِجَمِيعَكُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أبجميعكن'] = {
    'word': u'أبجميعكن',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِجَمِيعَكُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أبجميعها'] = {
    'word': u'أبجميعها',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِجَمِيعَهَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أبجميعهم'] = {
    'word': u'أبجميعهم',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِجَمِيعَهُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أبجميعهن'] = {
    'word': u'أبجميعهن',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِجَمِيعَهُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أبجميعنا'] = {
    'word': u'أبجميعنا',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِجَمِيعَنَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أبجميعكما'] = {
    'word': u'أبجميعكما',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِجَمِيعَكُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أبجميعهما'] = {
    'word': u'أبجميعهما',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِجَمِيعَهُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أكجميعي'] = {
    'word': u'أكجميعي',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَجَمِيعَِي',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أكجميعك'] = {
    'word': u'أكجميعك',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَجَمِيعَكَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أكجميعك'] = {
    'word': u'أكجميعك',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَجَمِيعَكِ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أكجميعه'] = {
    'word': u'أكجميعه',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَجَمِيعَه',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ه'
}
STOPWORDS[u'أكجميعكم'] = {
    'word': u'أكجميعكم',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَجَمِيعَكُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أكجميعكن'] = {
    'word': u'أكجميعكن',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَجَمِيعَكُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أكجميعها'] = {
    'word': u'أكجميعها',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَجَمِيعَهَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أكجميعهم'] = {
    'word': u'أكجميعهم',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَجَمِيعَهُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أكجميعهن'] = {
    'word': u'أكجميعهن',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَجَمِيعَهُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أكجميعنا'] = {
    'word': u'أكجميعنا',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَجَمِيعَنَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أكجميعكما'] = {
    'word': u'أكجميعكما',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَجَمِيعَكُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أكجميعهما'] = {
    'word': u'أكجميعهما',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَجَمِيعَهُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ألجميعي'] = {
    'word': u'ألجميعي',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِجَمِيعَِي',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ِي'
}
STOPWORDS[u'ألجميعك'] = {
    'word': u'ألجميعك',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِجَمِيعَكَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كَ'
}
STOPWORDS[u'ألجميعك'] = {
    'word': u'ألجميعك',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِجَمِيعَكِ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كِ'
}
STOPWORDS[u'ألجميعه'] = {
    'word': u'ألجميعه',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِجَمِيعَه',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ه'
}
STOPWORDS[u'ألجميعكم'] = {
    'word': u'ألجميعكم',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِجَمِيعَكُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ألجميعكن'] = {
    'word': u'ألجميعكن',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِجَمِيعَكُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ألجميعها'] = {
    'word': u'ألجميعها',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِجَمِيعَهَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هَا'
}
STOPWORDS[u'ألجميعهم'] = {
    'word': u'ألجميعهم',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِجَمِيعَهُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ألجميعهن'] = {
    'word': u'ألجميعهن',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِجَمِيعَهُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ألجميعنا'] = {
    'word': u'ألجميعنا',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِجَمِيعَنَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'نَا'
}
STOPWORDS[u'ألجميعكما'] = {
    'word': u'ألجميعكما',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِجَمِيعَكُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ألجميعهما'] = {
    'word': u'ألجميعهما',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِجَمِيعَهُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ألجميعي'] = {
    'word': u'ألجميعي',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَجَمِيعَِي',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ِي'
}
STOPWORDS[u'ألجميعك'] = {
    'word': u'ألجميعك',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَجَمِيعَكَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كَ'
}
STOPWORDS[u'ألجميعك'] = {
    'word': u'ألجميعك',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَجَمِيعَكِ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كِ'
}
STOPWORDS[u'ألجميعه'] = {
    'word': u'ألجميعه',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَجَمِيعَه',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ه'
}
STOPWORDS[u'ألجميعكم'] = {
    'word': u'ألجميعكم',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَجَمِيعَكُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ألجميعكن'] = {
    'word': u'ألجميعكن',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَجَمِيعَكُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ألجميعها'] = {
    'word': u'ألجميعها',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَجَمِيعَهَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هَا'
}
STOPWORDS[u'ألجميعهم'] = {
    'word': u'ألجميعهم',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَجَمِيعَهُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ألجميعهن'] = {
    'word': u'ألجميعهن',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَجَمِيعَهُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ألجميعنا'] = {
    'word': u'ألجميعنا',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَجَمِيعَنَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'نَا'
}
STOPWORDS[u'ألجميعكما'] = {
    'word': u'ألجميعكما',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَجَمِيعَكُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ألجميعهما'] = {
    'word': u'ألجميعهما',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَجَمِيعَهُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أوجميعي'] = {
    'word': u'أوجميعي',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَجَمِيعَِي',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أوجميعك'] = {
    'word': u'أوجميعك',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَجَمِيعَكَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أوجميعك'] = {
    'word': u'أوجميعك',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَجَمِيعَكِ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أوجميعه'] = {
    'word': u'أوجميعه',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَجَمِيعَه',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ه'
}
STOPWORDS[u'أوجميعكم'] = {
    'word': u'أوجميعكم',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَجَمِيعَكُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أوجميعكن'] = {
    'word': u'أوجميعكن',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَجَمِيعَكُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أوجميعها'] = {
    'word': u'أوجميعها',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَجَمِيعَهَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أوجميعهم'] = {
    'word': u'أوجميعهم',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَجَمِيعَهُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أوجميعهن'] = {
    'word': u'أوجميعهن',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَجَمِيعَهُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أوجميعنا'] = {
    'word': u'أوجميعنا',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَجَمِيعَنَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أوجميعكما'] = {
    'word': u'أوجميعكما',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَجَمِيعَكُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أوجميعهما'] = {
    'word': u'أوجميعهما',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَجَمِيعَهُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أفجميعي'] = {
    'word': u'أفجميعي',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَجَمِيعَِي',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أفجميعك'] = {
    'word': u'أفجميعك',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَجَمِيعَكَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أفجميعك'] = {
    'word': u'أفجميعك',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَجَمِيعَكِ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أفجميعه'] = {
    'word': u'أفجميعه',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَجَمِيعَه',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ه'
}
STOPWORDS[u'أفجميعكم'] = {
    'word': u'أفجميعكم',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَجَمِيعَكُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أفجميعكن'] = {
    'word': u'أفجميعكن',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَجَمِيعَكُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أفجميعها'] = {
    'word': u'أفجميعها',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَجَمِيعَهَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أفجميعهم'] = {
    'word': u'أفجميعهم',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَجَمِيعَهُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أفجميعهن'] = {
    'word': u'أفجميعهن',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَجَمِيعَهُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أفجميعنا'] = {
    'word': u'أفجميعنا',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَجَمِيعَنَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أفجميعكما'] = {
    'word': u'أفجميعكما',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَجَمِيعَكُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أفجميعهما'] = {
    'word': u'أفجميعهما',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَجَمِيعَهُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أوبجميعي'] = {
    'word': u'أوبجميعي',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِجَمِيعَِي',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أوبجميعك'] = {
    'word': u'أوبجميعك',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِجَمِيعَكَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أوبجميعك'] = {
    'word': u'أوبجميعك',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِجَمِيعَكِ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أوبجميعه'] = {
    'word': u'أوبجميعه',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِجَمِيعَه',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ه'
}
STOPWORDS[u'أوبجميعكم'] = {
    'word': u'أوبجميعكم',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِجَمِيعَكُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أوبجميعكن'] = {
    'word': u'أوبجميعكن',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِجَمِيعَكُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أوبجميعها'] = {
    'word': u'أوبجميعها',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِجَمِيعَهَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أوبجميعهم'] = {
    'word': u'أوبجميعهم',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِجَمِيعَهُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أوبجميعهن'] = {
    'word': u'أوبجميعهن',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِجَمِيعَهُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أوبجميعنا'] = {
    'word': u'أوبجميعنا',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِجَمِيعَنَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أوبجميعكما'] = {
    'word': u'أوبجميعكما',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِجَمِيعَكُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أوبجميعهما'] = {
    'word': u'أوبجميعهما',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِجَمِيعَهُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أفبجميعي'] = {
    'word': u'أفبجميعي',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِجَمِيعَِي',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أفبجميعك'] = {
    'word': u'أفبجميعك',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِجَمِيعَكَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أفبجميعك'] = {
    'word': u'أفبجميعك',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِجَمِيعَكِ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أفبجميعه'] = {
    'word': u'أفبجميعه',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِجَمِيعَه',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ه'
}
STOPWORDS[u'أفبجميعكم'] = {
    'word': u'أفبجميعكم',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِجَمِيعَكُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أفبجميعكن'] = {
    'word': u'أفبجميعكن',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِجَمِيعَكُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أفبجميعها'] = {
    'word': u'أفبجميعها',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِجَمِيعَهَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أفبجميعهم'] = {
    'word': u'أفبجميعهم',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِجَمِيعَهُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أفبجميعهن'] = {
    'word': u'أفبجميعهن',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِجَمِيعَهُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أفبجميعنا'] = {
    'word': u'أفبجميعنا',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِجَمِيعَنَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أفبجميعكما'] = {
    'word': u'أفبجميعكما',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِجَمِيعَكُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أفبجميعهما'] = {
    'word': u'أفبجميعهما',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِجَمِيعَهُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أوكجميعي'] = {
    'word': u'أوكجميعي',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَجَمِيعَِي',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أوكجميعك'] = {
    'word': u'أوكجميعك',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَجَمِيعَكَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أوكجميعك'] = {
    'word': u'أوكجميعك',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَجَمِيعَكِ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أوكجميعه'] = {
    'word': u'أوكجميعه',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَجَمِيعَه',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ه'
}
STOPWORDS[u'أوكجميعكم'] = {
    'word': u'أوكجميعكم',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَجَمِيعَكُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أوكجميعكن'] = {
    'word': u'أوكجميعكن',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَجَمِيعَكُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أوكجميعها'] = {
    'word': u'أوكجميعها',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَجَمِيعَهَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أوكجميعهم'] = {
    'word': u'أوكجميعهم',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَجَمِيعَهُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أوكجميعهن'] = {
    'word': u'أوكجميعهن',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَجَمِيعَهُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أوكجميعنا'] = {
    'word': u'أوكجميعنا',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَجَمِيعَنَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أوكجميعكما'] = {
    'word': u'أوكجميعكما',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَجَمِيعَكُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أوكجميعهما'] = {
    'word': u'أوكجميعهما',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَجَمِيعَهُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أفكجميعي'] = {
    'word': u'أفكجميعي',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَجَمِيعَِي',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أفكجميعك'] = {
    'word': u'أفكجميعك',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَجَمِيعَكَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أفكجميعك'] = {
    'word': u'أفكجميعك',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَجَمِيعَكِ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أفكجميعه'] = {
    'word': u'أفكجميعه',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَجَمِيعَه',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ه'
}
STOPWORDS[u'أفكجميعكم'] = {
    'word': u'أفكجميعكم',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَجَمِيعَكُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أفكجميعكن'] = {
    'word': u'أفكجميعكن',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَجَمِيعَكُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أفكجميعها'] = {
    'word': u'أفكجميعها',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَجَمِيعَهَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أفكجميعهم'] = {
    'word': u'أفكجميعهم',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَجَمِيعَهُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أفكجميعهن'] = {
    'word': u'أفكجميعهن',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَجَمِيعَهُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أفكجميعنا'] = {
    'word': u'أفكجميعنا',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَجَمِيعَنَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أفكجميعكما'] = {
    'word': u'أفكجميعكما',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَجَمِيعَكُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أفكجميعهما'] = {
    'word': u'أفكجميعهما',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَجَمِيعَهُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أولجميعي'] = {
    'word': u'أولجميعي',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِجَمِيعَِي',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أولجميعك'] = {
    'word': u'أولجميعك',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِجَمِيعَكَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أولجميعك'] = {
    'word': u'أولجميعك',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِجَمِيعَكِ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أولجميعه'] = {
    'word': u'أولجميعه',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِجَمِيعَه',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ه'
}
STOPWORDS[u'أولجميعكم'] = {
    'word': u'أولجميعكم',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِجَمِيعَكُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أولجميعكن'] = {
    'word': u'أولجميعكن',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِجَمِيعَكُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أولجميعها'] = {
    'word': u'أولجميعها',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِجَمِيعَهَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أولجميعهم'] = {
    'word': u'أولجميعهم',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِجَمِيعَهُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أولجميعهن'] = {
    'word': u'أولجميعهن',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِجَمِيعَهُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أولجميعنا'] = {
    'word': u'أولجميعنا',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِجَمِيعَنَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أولجميعكما'] = {
    'word': u'أولجميعكما',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِجَمِيعَكُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أولجميعهما'] = {
    'word': u'أولجميعهما',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِجَمِيعَهُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أفلجميعي'] = {
    'word': u'أفلجميعي',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِجَمِيعَِي',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أفلجميعك'] = {
    'word': u'أفلجميعك',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِجَمِيعَكَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أفلجميعك'] = {
    'word': u'أفلجميعك',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِجَمِيعَكِ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أفلجميعه'] = {
    'word': u'أفلجميعه',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِجَمِيعَه',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ه'
}
STOPWORDS[u'أفلجميعكم'] = {
    'word': u'أفلجميعكم',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِجَمِيعَكُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أفلجميعكن'] = {
    'word': u'أفلجميعكن',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِجَمِيعَكُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أفلجميعها'] = {
    'word': u'أفلجميعها',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِجَمِيعَهَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أفلجميعهم'] = {
    'word': u'أفلجميعهم',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِجَمِيعَهُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أفلجميعهن'] = {
    'word': u'أفلجميعهن',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِجَمِيعَهُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أفلجميعنا'] = {
    'word': u'أفلجميعنا',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِجَمِيعَنَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أفلجميعكما'] = {
    'word': u'أفلجميعكما',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِجَمِيعَكُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أفلجميعهما'] = {
    'word': u'أفلجميعهما',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِجَمِيعَهُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أولجميعي'] = {
    'word': u'أولجميعي',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَجَمِيعَِي',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أولجميعك'] = {
    'word': u'أولجميعك',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَجَمِيعَكَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أولجميعك'] = {
    'word': u'أولجميعك',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَجَمِيعَكِ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أولجميعه'] = {
    'word': u'أولجميعه',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَجَمِيعَه',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ه'
}
STOPWORDS[u'أولجميعكم'] = {
    'word': u'أولجميعكم',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَجَمِيعَكُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أولجميعكن'] = {
    'word': u'أولجميعكن',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَجَمِيعَكُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أولجميعها'] = {
    'word': u'أولجميعها',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَجَمِيعَهَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أولجميعهم'] = {
    'word': u'أولجميعهم',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَجَمِيعَهُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أولجميعهن'] = {
    'word': u'أولجميعهن',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَجَمِيعَهُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أولجميعنا'] = {
    'word': u'أولجميعنا',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَجَمِيعَنَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أولجميعكما'] = {
    'word': u'أولجميعكما',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَجَمِيعَكُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أولجميعهما'] = {
    'word': u'أولجميعهما',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَجَمِيعَهُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أفلجميعي'] = {
    'word': u'أفلجميعي',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَجَمِيعَِي',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أفلجميعك'] = {
    'word': u'أفلجميعك',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَجَمِيعَكَ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أفلجميعك'] = {
    'word': u'أفلجميعك',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَجَمِيعَكِ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أفلجميعه'] = {
    'word': u'أفلجميعه',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَجَمِيعَه',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'ه'
}
STOPWORDS[u'أفلجميعكم'] = {
    'word': u'أفلجميعكم',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَجَمِيعَكُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أفلجميعكن'] = {
    'word': u'أفلجميعكن',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَجَمِيعَكُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أفلجميعها'] = {
    'word': u'أفلجميعها',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَجَمِيعَهَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أفلجميعهم'] = {
    'word': u'أفلجميعهم',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَجَمِيعَهُمْ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أفلجميعهن'] = {
    'word': u'أفلجميعهن',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَجَمِيعَهُنَّ',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أفلجميعنا'] = {
    'word': u'أفلجميعنا',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَجَمِيعَنَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أفلجميعكما'] = {
    'word': u'أفلجميعكما',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَجَمِيعَكُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أفلجميعهما'] = {
    'word': u'أفلجميعهما',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَجَمِيعَهُمَا',
    'stem': u'جَمِيعَ',
    'type': u'STOPWORD',
    'original': u'جَمِيعَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'عامة'] = {
    'word': u'عَامَّةَ',
    'procletic': u'',
    'tags': u'أداة:توكيد',
    'vocalized': u'عَامَّةَ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u''
}
STOPWORDS[u'بعامة'] = {
    'word': u'بعامة',
    'procletic': u'-بِ',
    'tags': u'أداة:توكيد:مجرور',
    'vocalized': u'بِعَامَّةَ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u''
}
STOPWORDS[u'كعامة'] = {
    'word': u'كعامة',
    'procletic': u'-كَ',
    'tags': u'أداة:توكيد:مجرور',
    'vocalized': u'كَعَامَّةَ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u''
}
STOPWORDS[u'لعامة'] = {
    'word': u'لعامة',
    'procletic': u'-لِ',
    'tags': u'أداة:توكيد:مجرور',
    'vocalized': u'لِعَامَّةَ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u''
}
STOPWORDS[u'لعامة'] = {
    'word': u'لعامة',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور',
    'vocalized': u'لَعَامَّةَ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u''
}
STOPWORDS[u'وعامة'] = {
    'word': u'وعامة',
    'procletic': u'-وَ',
    'tags': u'أداة:توكيد:معطوف',
    'vocalized': u'وَعَامَّةَ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u''
}
STOPWORDS[u'فعامة'] = {
    'word': u'فعامة',
    'procletic': u'-فَ',
    'tags': u'أداة:توكيد:معطوف',
    'vocalized': u'فَعَامَّةَ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u''
}
STOPWORDS[u'وبعامة'] = {
    'word': u'وبعامة',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف',
    'vocalized': u'وَبِعَامَّةَ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u''
}
STOPWORDS[u'فبعامة'] = {
    'word': u'فبعامة',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف',
    'vocalized': u'فَبِعَامَّةَ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u''
}
STOPWORDS[u'وكعامة'] = {
    'word': u'وكعامة',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف',
    'vocalized': u'وَكَعَامَّةَ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u''
}
STOPWORDS[u'فكعامة'] = {
    'word': u'فكعامة',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف',
    'vocalized': u'فَكَعَامَّةَ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u''
}
STOPWORDS[u'ولعامة'] = {
    'word': u'ولعامة',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف',
    'vocalized': u'وَلِعَامَّةَ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u''
}
STOPWORDS[u'فلعامة'] = {
    'word': u'فلعامة',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف',
    'vocalized': u'فَلِعَامَّةَ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u''
}
STOPWORDS[u'ولعامة'] = {
    'word': u'ولعامة',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'وَلَعَامَّةَ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u''
}
STOPWORDS[u'فلعامة'] = {
    'word': u'فلعامة',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'فَلَعَامَّةَ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u''
}
STOPWORDS[u'أعامة'] = {
    'word': u'أعامة',
    'procletic': u'-أَ',
    'tags': u'أداة:توكيد:مجرور',
    'vocalized': u'أَعَامَّةَ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u''
}
STOPWORDS[u'أبعامة'] = {
    'word': u'أبعامة',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور',
    'vocalized': u'أَبِعَامَّةَ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u''
}
STOPWORDS[u'أكعامة'] = {
    'word': u'أكعامة',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور',
    'vocalized': u'أَكَعَامَّةَ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u''
}
STOPWORDS[u'ألعامة'] = {
    'word': u'ألعامة',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور',
    'vocalized': u'أَلِعَامَّةَ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u''
}
STOPWORDS[u'ألعامة'] = {
    'word': u'ألعامة',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور',
    'vocalized': u'أَلَعَامَّةَ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u''
}
STOPWORDS[u'أوعامة'] = {
    'word': u'أوعامة',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور',
    'vocalized': u'أَوَعَامَّةَ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u''
}
STOPWORDS[u'أفعامة'] = {
    'word': u'أفعامة',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور',
    'vocalized': u'أَفَعَامَّةَ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u''
}
STOPWORDS[u'أوبعامة'] = {
    'word': u'أوبعامة',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَبِعَامَّةَ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u''
}
STOPWORDS[u'أفبعامة'] = {
    'word': u'أفبعامة',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَبِعَامَّةَ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u''
}
STOPWORDS[u'أوكعامة'] = {
    'word': u'أوكعامة',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَكَعَامَّةَ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u''
}
STOPWORDS[u'أفكعامة'] = {
    'word': u'أفكعامة',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَكَعَامَّةَ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u''
}
STOPWORDS[u'أولعامة'] = {
    'word': u'أولعامة',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَلِعَامَّةَ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u''
}
STOPWORDS[u'أفلعامة'] = {
    'word': u'أفلعامة',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَلِعَامَّةَ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u''
}
STOPWORDS[u'أولعامة'] = {
    'word': u'أولعامة',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَلَعَامَّةَ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u''
}
STOPWORDS[u'أفلعامة'] = {
    'word': u'أفلعامة',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَلَعَامَّةَ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u''
}
STOPWORDS[u'عامتي'] = {
    'word': u'عامتي',
    'procletic': u'',
    'tags': u'أداة:توكيد:مضاف',
    'vocalized': u'عَامَّةَِي',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'ِي'
}
STOPWORDS[u'عامتك'] = {
    'word': u'عامتك',
    'procletic': u'',
    'tags': u'أداة:توكيد:مضاف',
    'vocalized': u'عَامَّةَكَ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كَ'
}
STOPWORDS[u'عامتك'] = {
    'word': u'عامتك',
    'procletic': u'',
    'tags': u'أداة:توكيد:مضاف',
    'vocalized': u'عَامَّةَكِ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كِ'
}
STOPWORDS[u'عامته'] = {
    'word': u'عامته',
    'procletic': u'',
    'tags': u'أداة:توكيد:مضاف',
    'vocalized': u'عَامَّةَه',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'ه'
}
STOPWORDS[u'عامتكم'] = {
    'word': u'عامتكم',
    'procletic': u'',
    'tags': u'أداة:توكيد:مضاف',
    'vocalized': u'عَامَّةَكُمْ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'عامتكن'] = {
    'word': u'عامتكن',
    'procletic': u'',
    'tags': u'أداة:توكيد:مضاف',
    'vocalized': u'عَامَّةَكُنَّ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'عامتها'] = {
    'word': u'عامتها',
    'procletic': u'',
    'tags': u'أداة:توكيد:مضاف',
    'vocalized': u'عَامَّةَهَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هَا'
}
STOPWORDS[u'عامتهم'] = {
    'word': u'عامتهم',
    'procletic': u'',
    'tags': u'أداة:توكيد:مضاف',
    'vocalized': u'عَامَّةَهُمْ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'عامتهن'] = {
    'word': u'عامتهن',
    'procletic': u'',
    'tags': u'أداة:توكيد:مضاف',
    'vocalized': u'عَامَّةَهُنَّ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'عامتنا'] = {
    'word': u'عامتنا',
    'procletic': u'',
    'tags': u'أداة:توكيد:مضاف',
    'vocalized': u'عَامَّةَنَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'نَا'
}
STOPWORDS[u'عامتكما'] = {
    'word': u'عامتكما',
    'procletic': u'',
    'tags': u'أداة:توكيد:مضاف',
    'vocalized': u'عَامَّةَكُمَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'عامتهما'] = {
    'word': u'عامتهما',
    'procletic': u'',
    'tags': u'أداة:توكيد:مضاف',
    'vocalized': u'عَامَّةَهُمَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'بعامتي'] = {
    'word': u'بعامتي',
    'procletic': u'-بِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'بِعَامَّةَِي',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'ِي'
}
STOPWORDS[u'بعامتك'] = {
    'word': u'بعامتك',
    'procletic': u'-بِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'بِعَامَّةَكَ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كَ'
}
STOPWORDS[u'بعامتك'] = {
    'word': u'بعامتك',
    'procletic': u'-بِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'بِعَامَّةَكِ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كِ'
}
STOPWORDS[u'بعامته'] = {
    'word': u'بعامته',
    'procletic': u'-بِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'بِعَامَّةَه',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'ه'
}
STOPWORDS[u'بعامتكم'] = {
    'word': u'بعامتكم',
    'procletic': u'-بِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'بِعَامَّةَكُمْ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'بعامتكن'] = {
    'word': u'بعامتكن',
    'procletic': u'-بِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'بِعَامَّةَكُنَّ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'بعامتها'] = {
    'word': u'بعامتها',
    'procletic': u'-بِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'بِعَامَّةَهَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هَا'
}
STOPWORDS[u'بعامتهم'] = {
    'word': u'بعامتهم',
    'procletic': u'-بِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'بِعَامَّةَهُمْ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'بعامتهن'] = {
    'word': u'بعامتهن',
    'procletic': u'-بِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'بِعَامَّةَهُنَّ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'بعامتنا'] = {
    'word': u'بعامتنا',
    'procletic': u'-بِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'بِعَامَّةَنَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'نَا'
}
STOPWORDS[u'بعامتكما'] = {
    'word': u'بعامتكما',
    'procletic': u'-بِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'بِعَامَّةَكُمَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'بعامتهما'] = {
    'word': u'بعامتهما',
    'procletic': u'-بِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'بِعَامَّةَهُمَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'كعامتي'] = {
    'word': u'كعامتي',
    'procletic': u'-كَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'كَعَامَّةَِي',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'ِي'
}
STOPWORDS[u'كعامتك'] = {
    'word': u'كعامتك',
    'procletic': u'-كَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'كَعَامَّةَكَ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كَ'
}
STOPWORDS[u'كعامتك'] = {
    'word': u'كعامتك',
    'procletic': u'-كَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'كَعَامَّةَكِ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كِ'
}
STOPWORDS[u'كعامته'] = {
    'word': u'كعامته',
    'procletic': u'-كَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'كَعَامَّةَه',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'ه'
}
STOPWORDS[u'كعامتكم'] = {
    'word': u'كعامتكم',
    'procletic': u'-كَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'كَعَامَّةَكُمْ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'كعامتكن'] = {
    'word': u'كعامتكن',
    'procletic': u'-كَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'كَعَامَّةَكُنَّ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'كعامتها'] = {
    'word': u'كعامتها',
    'procletic': u'-كَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'كَعَامَّةَهَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هَا'
}
STOPWORDS[u'كعامتهم'] = {
    'word': u'كعامتهم',
    'procletic': u'-كَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'كَعَامَّةَهُمْ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'كعامتهن'] = {
    'word': u'كعامتهن',
    'procletic': u'-كَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'كَعَامَّةَهُنَّ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'كعامتنا'] = {
    'word': u'كعامتنا',
    'procletic': u'-كَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'كَعَامَّةَنَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'نَا'
}
STOPWORDS[u'كعامتكما'] = {
    'word': u'كعامتكما',
    'procletic': u'-كَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'كَعَامَّةَكُمَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'كعامتهما'] = {
    'word': u'كعامتهما',
    'procletic': u'-كَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'كَعَامَّةَهُمَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'لعامتي'] = {
    'word': u'لعامتي',
    'procletic': u'-لِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'لِعَامَّةَِي',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'ِي'
}
STOPWORDS[u'لعامتك'] = {
    'word': u'لعامتك',
    'procletic': u'-لِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'لِعَامَّةَكَ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كَ'
}
STOPWORDS[u'لعامتك'] = {
    'word': u'لعامتك',
    'procletic': u'-لِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'لِعَامَّةَكِ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كِ'
}
STOPWORDS[u'لعامته'] = {
    'word': u'لعامته',
    'procletic': u'-لِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'لِعَامَّةَه',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'ه'
}
STOPWORDS[u'لعامتكم'] = {
    'word': u'لعامتكم',
    'procletic': u'-لِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'لِعَامَّةَكُمْ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'لعامتكن'] = {
    'word': u'لعامتكن',
    'procletic': u'-لِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'لِعَامَّةَكُنَّ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'لعامتها'] = {
    'word': u'لعامتها',
    'procletic': u'-لِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'لِعَامَّةَهَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هَا'
}
STOPWORDS[u'لعامتهم'] = {
    'word': u'لعامتهم',
    'procletic': u'-لِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'لِعَامَّةَهُمْ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'لعامتهن'] = {
    'word': u'لعامتهن',
    'procletic': u'-لِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'لِعَامَّةَهُنَّ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'لعامتنا'] = {
    'word': u'لعامتنا',
    'procletic': u'-لِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'لِعَامَّةَنَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'نَا'
}
STOPWORDS[u'لعامتكما'] = {
    'word': u'لعامتكما',
    'procletic': u'-لِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'لِعَامَّةَكُمَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'لعامتهما'] = {
    'word': u'لعامتهما',
    'procletic': u'-لِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'لِعَامَّةَهُمَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'لعامتي'] = {
    'word': u'لعامتي',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَعَامَّةَِي',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'ِي'
}
STOPWORDS[u'لعامتك'] = {
    'word': u'لعامتك',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَعَامَّةَكَ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كَ'
}
STOPWORDS[u'لعامتك'] = {
    'word': u'لعامتك',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَعَامَّةَكِ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كِ'
}
STOPWORDS[u'لعامته'] = {
    'word': u'لعامته',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَعَامَّةَه',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'ه'
}
STOPWORDS[u'لعامتكم'] = {
    'word': u'لعامتكم',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَعَامَّةَكُمْ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'لعامتكن'] = {
    'word': u'لعامتكن',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَعَامَّةَكُنَّ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'لعامتها'] = {
    'word': u'لعامتها',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَعَامَّةَهَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هَا'
}
STOPWORDS[u'لعامتهم'] = {
    'word': u'لعامتهم',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَعَامَّةَهُمْ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'لعامتهن'] = {
    'word': u'لعامتهن',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَعَامَّةَهُنَّ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'لعامتنا'] = {
    'word': u'لعامتنا',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَعَامَّةَنَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'نَا'
}
STOPWORDS[u'لعامتكما'] = {
    'word': u'لعامتكما',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَعَامَّةَكُمَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'لعامتهما'] = {
    'word': u'لعامتهما',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَعَامَّةَهُمَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'وعامتي'] = {
    'word': u'وعامتي',
    'procletic': u'-وَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'وَعَامَّةَِي',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'ِي'
}
STOPWORDS[u'وعامتك'] = {
    'word': u'وعامتك',
    'procletic': u'-وَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'وَعَامَّةَكَ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كَ'
}
STOPWORDS[u'وعامتك'] = {
    'word': u'وعامتك',
    'procletic': u'-وَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'وَعَامَّةَكِ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كِ'
}
STOPWORDS[u'وعامته'] = {
    'word': u'وعامته',
    'procletic': u'-وَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'وَعَامَّةَه',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'ه'
}
STOPWORDS[u'وعامتكم'] = {
    'word': u'وعامتكم',
    'procletic': u'-وَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'وَعَامَّةَكُمْ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'وعامتكن'] = {
    'word': u'وعامتكن',
    'procletic': u'-وَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'وَعَامَّةَكُنَّ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'وعامتها'] = {
    'word': u'وعامتها',
    'procletic': u'-وَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'وَعَامَّةَهَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هَا'
}
STOPWORDS[u'وعامتهم'] = {
    'word': u'وعامتهم',
    'procletic': u'-وَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'وَعَامَّةَهُمْ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'وعامتهن'] = {
    'word': u'وعامتهن',
    'procletic': u'-وَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'وَعَامَّةَهُنَّ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'وعامتنا'] = {
    'word': u'وعامتنا',
    'procletic': u'-وَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'وَعَامَّةَنَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'نَا'
}
STOPWORDS[u'وعامتكما'] = {
    'word': u'وعامتكما',
    'procletic': u'-وَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'وَعَامَّةَكُمَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'وعامتهما'] = {
    'word': u'وعامتهما',
    'procletic': u'-وَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'وَعَامَّةَهُمَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فعامتي'] = {
    'word': u'فعامتي',
    'procletic': u'-فَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'فَعَامَّةَِي',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'ِي'
}
STOPWORDS[u'فعامتك'] = {
    'word': u'فعامتك',
    'procletic': u'-فَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'فَعَامَّةَكَ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كَ'
}
STOPWORDS[u'فعامتك'] = {
    'word': u'فعامتك',
    'procletic': u'-فَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'فَعَامَّةَكِ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كِ'
}
STOPWORDS[u'فعامته'] = {
    'word': u'فعامته',
    'procletic': u'-فَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'فَعَامَّةَه',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'ه'
}
STOPWORDS[u'فعامتكم'] = {
    'word': u'فعامتكم',
    'procletic': u'-فَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'فَعَامَّةَكُمْ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فعامتكن'] = {
    'word': u'فعامتكن',
    'procletic': u'-فَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'فَعَامَّةَكُنَّ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فعامتها'] = {
    'word': u'فعامتها',
    'procletic': u'-فَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'فَعَامَّةَهَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هَا'
}
STOPWORDS[u'فعامتهم'] = {
    'word': u'فعامتهم',
    'procletic': u'-فَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'فَعَامَّةَهُمْ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فعامتهن'] = {
    'word': u'فعامتهن',
    'procletic': u'-فَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'فَعَامَّةَهُنَّ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فعامتنا'] = {
    'word': u'فعامتنا',
    'procletic': u'-فَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'فَعَامَّةَنَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'نَا'
}
STOPWORDS[u'فعامتكما'] = {
    'word': u'فعامتكما',
    'procletic': u'-فَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'فَعَامَّةَكُمَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فعامتهما'] = {
    'word': u'فعامتهما',
    'procletic': u'-فَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'فَعَامَّةَهُمَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'وبعامتي'] = {
    'word': u'وبعامتي',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِعَامَّةَِي',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'ِي'
}
STOPWORDS[u'وبعامتك'] = {
    'word': u'وبعامتك',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِعَامَّةَكَ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كَ'
}
STOPWORDS[u'وبعامتك'] = {
    'word': u'وبعامتك',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِعَامَّةَكِ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كِ'
}
STOPWORDS[u'وبعامته'] = {
    'word': u'وبعامته',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِعَامَّةَه',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'ه'
}
STOPWORDS[u'وبعامتكم'] = {
    'word': u'وبعامتكم',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِعَامَّةَكُمْ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'وبعامتكن'] = {
    'word': u'وبعامتكن',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِعَامَّةَكُنَّ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'وبعامتها'] = {
    'word': u'وبعامتها',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِعَامَّةَهَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هَا'
}
STOPWORDS[u'وبعامتهم'] = {
    'word': u'وبعامتهم',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِعَامَّةَهُمْ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'وبعامتهن'] = {
    'word': u'وبعامتهن',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِعَامَّةَهُنَّ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'وبعامتنا'] = {
    'word': u'وبعامتنا',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِعَامَّةَنَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'نَا'
}
STOPWORDS[u'وبعامتكما'] = {
    'word': u'وبعامتكما',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِعَامَّةَكُمَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'وبعامتهما'] = {
    'word': u'وبعامتهما',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِعَامَّةَهُمَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فبعامتي'] = {
    'word': u'فبعامتي',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِعَامَّةَِي',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'ِي'
}
STOPWORDS[u'فبعامتك'] = {
    'word': u'فبعامتك',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِعَامَّةَكَ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كَ'
}
STOPWORDS[u'فبعامتك'] = {
    'word': u'فبعامتك',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِعَامَّةَكِ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كِ'
}
STOPWORDS[u'فبعامته'] = {
    'word': u'فبعامته',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِعَامَّةَه',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'ه'
}
STOPWORDS[u'فبعامتكم'] = {
    'word': u'فبعامتكم',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِعَامَّةَكُمْ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فبعامتكن'] = {
    'word': u'فبعامتكن',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِعَامَّةَكُنَّ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فبعامتها'] = {
    'word': u'فبعامتها',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِعَامَّةَهَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هَا'
}
STOPWORDS[u'فبعامتهم'] = {
    'word': u'فبعامتهم',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِعَامَّةَهُمْ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فبعامتهن'] = {
    'word': u'فبعامتهن',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِعَامَّةَهُنَّ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فبعامتنا'] = {
    'word': u'فبعامتنا',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِعَامَّةَنَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'نَا'
}
STOPWORDS[u'فبعامتكما'] = {
    'word': u'فبعامتكما',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِعَامَّةَكُمَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فبعامتهما'] = {
    'word': u'فبعامتهما',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِعَامَّةَهُمَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'وكعامتي'] = {
    'word': u'وكعامتي',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَعَامَّةَِي',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'ِي'
}
STOPWORDS[u'وكعامتك'] = {
    'word': u'وكعامتك',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَعَامَّةَكَ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كَ'
}
STOPWORDS[u'وكعامتك'] = {
    'word': u'وكعامتك',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَعَامَّةَكِ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كِ'
}
STOPWORDS[u'وكعامته'] = {
    'word': u'وكعامته',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَعَامَّةَه',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'ه'
}
STOPWORDS[u'وكعامتكم'] = {
    'word': u'وكعامتكم',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَعَامَّةَكُمْ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'وكعامتكن'] = {
    'word': u'وكعامتكن',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَعَامَّةَكُنَّ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'وكعامتها'] = {
    'word': u'وكعامتها',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَعَامَّةَهَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هَا'
}
STOPWORDS[u'وكعامتهم'] = {
    'word': u'وكعامتهم',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَعَامَّةَهُمْ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'وكعامتهن'] = {
    'word': u'وكعامتهن',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَعَامَّةَهُنَّ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'وكعامتنا'] = {
    'word': u'وكعامتنا',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَعَامَّةَنَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'نَا'
}
STOPWORDS[u'وكعامتكما'] = {
    'word': u'وكعامتكما',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَعَامَّةَكُمَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'وكعامتهما'] = {
    'word': u'وكعامتهما',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَعَامَّةَهُمَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فكعامتي'] = {
    'word': u'فكعامتي',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَعَامَّةَِي',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'ِي'
}
STOPWORDS[u'فكعامتك'] = {
    'word': u'فكعامتك',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَعَامَّةَكَ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كَ'
}
STOPWORDS[u'فكعامتك'] = {
    'word': u'فكعامتك',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَعَامَّةَكِ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كِ'
}
STOPWORDS[u'فكعامته'] = {
    'word': u'فكعامته',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَعَامَّةَه',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'ه'
}
STOPWORDS[u'فكعامتكم'] = {
    'word': u'فكعامتكم',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَعَامَّةَكُمْ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فكعامتكن'] = {
    'word': u'فكعامتكن',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَعَامَّةَكُنَّ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فكعامتها'] = {
    'word': u'فكعامتها',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَعَامَّةَهَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هَا'
}
STOPWORDS[u'فكعامتهم'] = {
    'word': u'فكعامتهم',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَعَامَّةَهُمْ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فكعامتهن'] = {
    'word': u'فكعامتهن',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَعَامَّةَهُنَّ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فكعامتنا'] = {
    'word': u'فكعامتنا',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَعَامَّةَنَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'نَا'
}
STOPWORDS[u'فكعامتكما'] = {
    'word': u'فكعامتكما',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَعَامَّةَكُمَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فكعامتهما'] = {
    'word': u'فكعامتهما',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَعَامَّةَهُمَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ولعامتي'] = {
    'word': u'ولعامتي',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِعَامَّةَِي',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'ِي'
}
STOPWORDS[u'ولعامتك'] = {
    'word': u'ولعامتك',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِعَامَّةَكَ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كَ'
}
STOPWORDS[u'ولعامتك'] = {
    'word': u'ولعامتك',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِعَامَّةَكِ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كِ'
}
STOPWORDS[u'ولعامته'] = {
    'word': u'ولعامته',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِعَامَّةَه',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'ه'
}
STOPWORDS[u'ولعامتكم'] = {
    'word': u'ولعامتكم',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِعَامَّةَكُمْ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ولعامتكن'] = {
    'word': u'ولعامتكن',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِعَامَّةَكُنَّ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ولعامتها'] = {
    'word': u'ولعامتها',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِعَامَّةَهَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هَا'
}
STOPWORDS[u'ولعامتهم'] = {
    'word': u'ولعامتهم',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِعَامَّةَهُمْ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ولعامتهن'] = {
    'word': u'ولعامتهن',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِعَامَّةَهُنَّ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ولعامتنا'] = {
    'word': u'ولعامتنا',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِعَامَّةَنَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'نَا'
}
STOPWORDS[u'ولعامتكما'] = {
    'word': u'ولعامتكما',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِعَامَّةَكُمَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ولعامتهما'] = {
    'word': u'ولعامتهما',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِعَامَّةَهُمَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فلعامتي'] = {
    'word': u'فلعامتي',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِعَامَّةَِي',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'ِي'
}
STOPWORDS[u'فلعامتك'] = {
    'word': u'فلعامتك',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِعَامَّةَكَ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كَ'
}
STOPWORDS[u'فلعامتك'] = {
    'word': u'فلعامتك',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِعَامَّةَكِ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كِ'
}
STOPWORDS[u'فلعامته'] = {
    'word': u'فلعامته',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِعَامَّةَه',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'ه'
}
STOPWORDS[u'فلعامتكم'] = {
    'word': u'فلعامتكم',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِعَامَّةَكُمْ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فلعامتكن'] = {
    'word': u'فلعامتكن',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِعَامَّةَكُنَّ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فلعامتها'] = {
    'word': u'فلعامتها',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِعَامَّةَهَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هَا'
}
STOPWORDS[u'فلعامتهم'] = {
    'word': u'فلعامتهم',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِعَامَّةَهُمْ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فلعامتهن'] = {
    'word': u'فلعامتهن',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِعَامَّةَهُنَّ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فلعامتنا'] = {
    'word': u'فلعامتنا',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِعَامَّةَنَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'نَا'
}
STOPWORDS[u'فلعامتكما'] = {
    'word': u'فلعامتكما',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِعَامَّةَكُمَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فلعامتهما'] = {
    'word': u'فلعامتهما',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِعَامَّةَهُمَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ولعامتي'] = {
    'word': u'ولعامتي',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَعَامَّةَِي',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'ِي'
}
STOPWORDS[u'ولعامتك'] = {
    'word': u'ولعامتك',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَعَامَّةَكَ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كَ'
}
STOPWORDS[u'ولعامتك'] = {
    'word': u'ولعامتك',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَعَامَّةَكِ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كِ'
}
STOPWORDS[u'ولعامته'] = {
    'word': u'ولعامته',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَعَامَّةَه',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'ه'
}
STOPWORDS[u'ولعامتكم'] = {
    'word': u'ولعامتكم',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَعَامَّةَكُمْ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ولعامتكن'] = {
    'word': u'ولعامتكن',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَعَامَّةَكُنَّ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ولعامتها'] = {
    'word': u'ولعامتها',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَعَامَّةَهَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هَا'
}
STOPWORDS[u'ولعامتهم'] = {
    'word': u'ولعامتهم',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَعَامَّةَهُمْ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ولعامتهن'] = {
    'word': u'ولعامتهن',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَعَامَّةَهُنَّ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ولعامتنا'] = {
    'word': u'ولعامتنا',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَعَامَّةَنَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'نَا'
}
STOPWORDS[u'ولعامتكما'] = {
    'word': u'ولعامتكما',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَعَامَّةَكُمَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ولعامتهما'] = {
    'word': u'ولعامتهما',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَعَامَّةَهُمَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فلعامتي'] = {
    'word': u'فلعامتي',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَعَامَّةَِي',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'ِي'
}
STOPWORDS[u'فلعامتك'] = {
    'word': u'فلعامتك',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَعَامَّةَكَ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كَ'
}
STOPWORDS[u'فلعامتك'] = {
    'word': u'فلعامتك',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَعَامَّةَكِ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كِ'
}
STOPWORDS[u'فلعامته'] = {
    'word': u'فلعامته',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَعَامَّةَه',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'ه'
}
STOPWORDS[u'فلعامتكم'] = {
    'word': u'فلعامتكم',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَعَامَّةَكُمْ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فلعامتكن'] = {
    'word': u'فلعامتكن',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَعَامَّةَكُنَّ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فلعامتها'] = {
    'word': u'فلعامتها',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَعَامَّةَهَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هَا'
}
STOPWORDS[u'فلعامتهم'] = {
    'word': u'فلعامتهم',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَعَامَّةَهُمْ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فلعامتهن'] = {
    'word': u'فلعامتهن',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَعَامَّةَهُنَّ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فلعامتنا'] = {
    'word': u'فلعامتنا',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَعَامَّةَنَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'نَا'
}
STOPWORDS[u'فلعامتكما'] = {
    'word': u'فلعامتكما',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَعَامَّةَكُمَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فلعامتهما'] = {
    'word': u'فلعامتهما',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَعَامَّةَهُمَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أعامتي'] = {
    'word': u'أعامتي',
    'procletic': u'-أَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'أَعَامَّةَِي',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أعامتك'] = {
    'word': u'أعامتك',
    'procletic': u'-أَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'أَعَامَّةَكَ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أعامتك'] = {
    'word': u'أعامتك',
    'procletic': u'-أَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'أَعَامَّةَكِ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أعامته'] = {
    'word': u'أعامته',
    'procletic': u'-أَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'أَعَامَّةَه',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'ه'
}
STOPWORDS[u'أعامتكم'] = {
    'word': u'أعامتكم',
    'procletic': u'-أَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'أَعَامَّةَكُمْ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أعامتكن'] = {
    'word': u'أعامتكن',
    'procletic': u'-أَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'أَعَامَّةَكُنَّ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أعامتها'] = {
    'word': u'أعامتها',
    'procletic': u'-أَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'أَعَامَّةَهَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أعامتهم'] = {
    'word': u'أعامتهم',
    'procletic': u'-أَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'أَعَامَّةَهُمْ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أعامتهن'] = {
    'word': u'أعامتهن',
    'procletic': u'-أَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'أَعَامَّةَهُنَّ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أعامتنا'] = {
    'word': u'أعامتنا',
    'procletic': u'-أَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'أَعَامَّةَنَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أعامتكما'] = {
    'word': u'أعامتكما',
    'procletic': u'-أَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'أَعَامَّةَكُمَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أعامتهما'] = {
    'word': u'أعامتهما',
    'procletic': u'-أَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'أَعَامَّةَهُمَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أبعامتي'] = {
    'word': u'أبعامتي',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِعَامَّةَِي',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أبعامتك'] = {
    'word': u'أبعامتك',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِعَامَّةَكَ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أبعامتك'] = {
    'word': u'أبعامتك',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِعَامَّةَكِ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أبعامته'] = {
    'word': u'أبعامته',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِعَامَّةَه',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'ه'
}
STOPWORDS[u'أبعامتكم'] = {
    'word': u'أبعامتكم',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِعَامَّةَكُمْ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أبعامتكن'] = {
    'word': u'أبعامتكن',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِعَامَّةَكُنَّ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أبعامتها'] = {
    'word': u'أبعامتها',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِعَامَّةَهَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أبعامتهم'] = {
    'word': u'أبعامتهم',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِعَامَّةَهُمْ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أبعامتهن'] = {
    'word': u'أبعامتهن',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِعَامَّةَهُنَّ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أبعامتنا'] = {
    'word': u'أبعامتنا',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِعَامَّةَنَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أبعامتكما'] = {
    'word': u'أبعامتكما',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِعَامَّةَكُمَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أبعامتهما'] = {
    'word': u'أبعامتهما',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِعَامَّةَهُمَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أكعامتي'] = {
    'word': u'أكعامتي',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَعَامَّةَِي',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أكعامتك'] = {
    'word': u'أكعامتك',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَعَامَّةَكَ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أكعامتك'] = {
    'word': u'أكعامتك',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَعَامَّةَكِ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أكعامته'] = {
    'word': u'أكعامته',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَعَامَّةَه',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'ه'
}
STOPWORDS[u'أكعامتكم'] = {
    'word': u'أكعامتكم',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَعَامَّةَكُمْ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أكعامتكن'] = {
    'word': u'أكعامتكن',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَعَامَّةَكُنَّ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أكعامتها'] = {
    'word': u'أكعامتها',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَعَامَّةَهَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أكعامتهم'] = {
    'word': u'أكعامتهم',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَعَامَّةَهُمْ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أكعامتهن'] = {
    'word': u'أكعامتهن',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَعَامَّةَهُنَّ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أكعامتنا'] = {
    'word': u'أكعامتنا',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَعَامَّةَنَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أكعامتكما'] = {
    'word': u'أكعامتكما',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَعَامَّةَكُمَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أكعامتهما'] = {
    'word': u'أكعامتهما',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَعَامَّةَهُمَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ألعامتي'] = {
    'word': u'ألعامتي',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِعَامَّةَِي',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'ِي'
}
STOPWORDS[u'ألعامتك'] = {
    'word': u'ألعامتك',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِعَامَّةَكَ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كَ'
}
STOPWORDS[u'ألعامتك'] = {
    'word': u'ألعامتك',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِعَامَّةَكِ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كِ'
}
STOPWORDS[u'ألعامته'] = {
    'word': u'ألعامته',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِعَامَّةَه',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'ه'
}
STOPWORDS[u'ألعامتكم'] = {
    'word': u'ألعامتكم',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِعَامَّةَكُمْ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ألعامتكن'] = {
    'word': u'ألعامتكن',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِعَامَّةَكُنَّ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ألعامتها'] = {
    'word': u'ألعامتها',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِعَامَّةَهَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هَا'
}
STOPWORDS[u'ألعامتهم'] = {
    'word': u'ألعامتهم',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِعَامَّةَهُمْ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ألعامتهن'] = {
    'word': u'ألعامتهن',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِعَامَّةَهُنَّ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ألعامتنا'] = {
    'word': u'ألعامتنا',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِعَامَّةَنَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'نَا'
}
STOPWORDS[u'ألعامتكما'] = {
    'word': u'ألعامتكما',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِعَامَّةَكُمَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ألعامتهما'] = {
    'word': u'ألعامتهما',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِعَامَّةَهُمَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ألعامتي'] = {
    'word': u'ألعامتي',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَعَامَّةَِي',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'ِي'
}
STOPWORDS[u'ألعامتك'] = {
    'word': u'ألعامتك',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَعَامَّةَكَ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كَ'
}
STOPWORDS[u'ألعامتك'] = {
    'word': u'ألعامتك',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَعَامَّةَكِ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كِ'
}
STOPWORDS[u'ألعامته'] = {
    'word': u'ألعامته',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَعَامَّةَه',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'ه'
}
STOPWORDS[u'ألعامتكم'] = {
    'word': u'ألعامتكم',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَعَامَّةَكُمْ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ألعامتكن'] = {
    'word': u'ألعامتكن',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَعَامَّةَكُنَّ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ألعامتها'] = {
    'word': u'ألعامتها',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَعَامَّةَهَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هَا'
}
STOPWORDS[u'ألعامتهم'] = {
    'word': u'ألعامتهم',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَعَامَّةَهُمْ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ألعامتهن'] = {
    'word': u'ألعامتهن',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَعَامَّةَهُنَّ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ألعامتنا'] = {
    'word': u'ألعامتنا',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَعَامَّةَنَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'نَا'
}
STOPWORDS[u'ألعامتكما'] = {
    'word': u'ألعامتكما',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَعَامَّةَكُمَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ألعامتهما'] = {
    'word': u'ألعامتهما',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَعَامَّةَهُمَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أوعامتي'] = {
    'word': u'أوعامتي',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَعَامَّةَِي',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أوعامتك'] = {
    'word': u'أوعامتك',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَعَامَّةَكَ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أوعامتك'] = {
    'word': u'أوعامتك',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَعَامَّةَكِ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أوعامته'] = {
    'word': u'أوعامته',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَعَامَّةَه',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'ه'
}
STOPWORDS[u'أوعامتكم'] = {
    'word': u'أوعامتكم',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَعَامَّةَكُمْ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أوعامتكن'] = {
    'word': u'أوعامتكن',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَعَامَّةَكُنَّ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أوعامتها'] = {
    'word': u'أوعامتها',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَعَامَّةَهَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أوعامتهم'] = {
    'word': u'أوعامتهم',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَعَامَّةَهُمْ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أوعامتهن'] = {
    'word': u'أوعامتهن',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَعَامَّةَهُنَّ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أوعامتنا'] = {
    'word': u'أوعامتنا',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَعَامَّةَنَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أوعامتكما'] = {
    'word': u'أوعامتكما',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَعَامَّةَكُمَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أوعامتهما'] = {
    'word': u'أوعامتهما',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَعَامَّةَهُمَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أفعامتي'] = {
    'word': u'أفعامتي',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَعَامَّةَِي',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أفعامتك'] = {
    'word': u'أفعامتك',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَعَامَّةَكَ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أفعامتك'] = {
    'word': u'أفعامتك',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَعَامَّةَكِ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أفعامته'] = {
    'word': u'أفعامته',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَعَامَّةَه',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'ه'
}
STOPWORDS[u'أفعامتكم'] = {
    'word': u'أفعامتكم',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَعَامَّةَكُمْ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أفعامتكن'] = {
    'word': u'أفعامتكن',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَعَامَّةَكُنَّ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أفعامتها'] = {
    'word': u'أفعامتها',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَعَامَّةَهَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أفعامتهم'] = {
    'word': u'أفعامتهم',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَعَامَّةَهُمْ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أفعامتهن'] = {
    'word': u'أفعامتهن',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَعَامَّةَهُنَّ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أفعامتنا'] = {
    'word': u'أفعامتنا',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَعَامَّةَنَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أفعامتكما'] = {
    'word': u'أفعامتكما',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَعَامَّةَكُمَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أفعامتهما'] = {
    'word': u'أفعامتهما',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَعَامَّةَهُمَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أوبعامتي'] = {
    'word': u'أوبعامتي',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِعَامَّةَِي',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أوبعامتك'] = {
    'word': u'أوبعامتك',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِعَامَّةَكَ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أوبعامتك'] = {
    'word': u'أوبعامتك',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِعَامَّةَكِ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أوبعامته'] = {
    'word': u'أوبعامته',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِعَامَّةَه',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'ه'
}
STOPWORDS[u'أوبعامتكم'] = {
    'word': u'أوبعامتكم',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِعَامَّةَكُمْ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أوبعامتكن'] = {
    'word': u'أوبعامتكن',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِعَامَّةَكُنَّ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أوبعامتها'] = {
    'word': u'أوبعامتها',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِعَامَّةَهَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أوبعامتهم'] = {
    'word': u'أوبعامتهم',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِعَامَّةَهُمْ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أوبعامتهن'] = {
    'word': u'أوبعامتهن',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِعَامَّةَهُنَّ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أوبعامتنا'] = {
    'word': u'أوبعامتنا',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِعَامَّةَنَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أوبعامتكما'] = {
    'word': u'أوبعامتكما',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِعَامَّةَكُمَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أوبعامتهما'] = {
    'word': u'أوبعامتهما',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِعَامَّةَهُمَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أفبعامتي'] = {
    'word': u'أفبعامتي',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِعَامَّةَِي',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أفبعامتك'] = {
    'word': u'أفبعامتك',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِعَامَّةَكَ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أفبعامتك'] = {
    'word': u'أفبعامتك',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِعَامَّةَكِ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أفبعامته'] = {
    'word': u'أفبعامته',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِعَامَّةَه',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'ه'
}
STOPWORDS[u'أفبعامتكم'] = {
    'word': u'أفبعامتكم',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِعَامَّةَكُمْ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أفبعامتكن'] = {
    'word': u'أفبعامتكن',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِعَامَّةَكُنَّ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أفبعامتها'] = {
    'word': u'أفبعامتها',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِعَامَّةَهَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أفبعامتهم'] = {
    'word': u'أفبعامتهم',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِعَامَّةَهُمْ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أفبعامتهن'] = {
    'word': u'أفبعامتهن',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِعَامَّةَهُنَّ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أفبعامتنا'] = {
    'word': u'أفبعامتنا',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِعَامَّةَنَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أفبعامتكما'] = {
    'word': u'أفبعامتكما',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِعَامَّةَكُمَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أفبعامتهما'] = {
    'word': u'أفبعامتهما',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِعَامَّةَهُمَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أوكعامتي'] = {
    'word': u'أوكعامتي',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَعَامَّةَِي',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أوكعامتك'] = {
    'word': u'أوكعامتك',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَعَامَّةَكَ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أوكعامتك'] = {
    'word': u'أوكعامتك',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَعَامَّةَكِ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أوكعامته'] = {
    'word': u'أوكعامته',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَعَامَّةَه',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'ه'
}
STOPWORDS[u'أوكعامتكم'] = {
    'word': u'أوكعامتكم',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَعَامَّةَكُمْ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أوكعامتكن'] = {
    'word': u'أوكعامتكن',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَعَامَّةَكُنَّ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أوكعامتها'] = {
    'word': u'أوكعامتها',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَعَامَّةَهَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أوكعامتهم'] = {
    'word': u'أوكعامتهم',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَعَامَّةَهُمْ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أوكعامتهن'] = {
    'word': u'أوكعامتهن',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَعَامَّةَهُنَّ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أوكعامتنا'] = {
    'word': u'أوكعامتنا',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَعَامَّةَنَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أوكعامتكما'] = {
    'word': u'أوكعامتكما',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَعَامَّةَكُمَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أوكعامتهما'] = {
    'word': u'أوكعامتهما',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَعَامَّةَهُمَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أفكعامتي'] = {
    'word': u'أفكعامتي',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَعَامَّةَِي',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أفكعامتك'] = {
    'word': u'أفكعامتك',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَعَامَّةَكَ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أفكعامتك'] = {
    'word': u'أفكعامتك',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَعَامَّةَكِ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أفكعامته'] = {
    'word': u'أفكعامته',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَعَامَّةَه',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'ه'
}
STOPWORDS[u'أفكعامتكم'] = {
    'word': u'أفكعامتكم',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَعَامَّةَكُمْ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أفكعامتكن'] = {
    'word': u'أفكعامتكن',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَعَامَّةَكُنَّ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أفكعامتها'] = {
    'word': u'أفكعامتها',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَعَامَّةَهَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أفكعامتهم'] = {
    'word': u'أفكعامتهم',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَعَامَّةَهُمْ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أفكعامتهن'] = {
    'word': u'أفكعامتهن',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَعَامَّةَهُنَّ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أفكعامتنا'] = {
    'word': u'أفكعامتنا',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَعَامَّةَنَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أفكعامتكما'] = {
    'word': u'أفكعامتكما',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَعَامَّةَكُمَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أفكعامتهما'] = {
    'word': u'أفكعامتهما',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَعَامَّةَهُمَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أولعامتي'] = {
    'word': u'أولعامتي',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِعَامَّةَِي',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أولعامتك'] = {
    'word': u'أولعامتك',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِعَامَّةَكَ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أولعامتك'] = {
    'word': u'أولعامتك',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِعَامَّةَكِ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أولعامته'] = {
    'word': u'أولعامته',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِعَامَّةَه',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'ه'
}
STOPWORDS[u'أولعامتكم'] = {
    'word': u'أولعامتكم',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِعَامَّةَكُمْ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أولعامتكن'] = {
    'word': u'أولعامتكن',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِعَامَّةَكُنَّ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أولعامتها'] = {
    'word': u'أولعامتها',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِعَامَّةَهَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أولعامتهم'] = {
    'word': u'أولعامتهم',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِعَامَّةَهُمْ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أولعامتهن'] = {
    'word': u'أولعامتهن',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِعَامَّةَهُنَّ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أولعامتنا'] = {
    'word': u'أولعامتنا',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِعَامَّةَنَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أولعامتكما'] = {
    'word': u'أولعامتكما',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِعَامَّةَكُمَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أولعامتهما'] = {
    'word': u'أولعامتهما',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِعَامَّةَهُمَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أفلعامتي'] = {
    'word': u'أفلعامتي',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِعَامَّةَِي',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أفلعامتك'] = {
    'word': u'أفلعامتك',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِعَامَّةَكَ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أفلعامتك'] = {
    'word': u'أفلعامتك',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِعَامَّةَكِ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أفلعامته'] = {
    'word': u'أفلعامته',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِعَامَّةَه',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'ه'
}
STOPWORDS[u'أفلعامتكم'] = {
    'word': u'أفلعامتكم',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِعَامَّةَكُمْ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أفلعامتكن'] = {
    'word': u'أفلعامتكن',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِعَامَّةَكُنَّ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أفلعامتها'] = {
    'word': u'أفلعامتها',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِعَامَّةَهَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أفلعامتهم'] = {
    'word': u'أفلعامتهم',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِعَامَّةَهُمْ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أفلعامتهن'] = {
    'word': u'أفلعامتهن',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِعَامَّةَهُنَّ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أفلعامتنا'] = {
    'word': u'أفلعامتنا',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِعَامَّةَنَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أفلعامتكما'] = {
    'word': u'أفلعامتكما',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِعَامَّةَكُمَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أفلعامتهما'] = {
    'word': u'أفلعامتهما',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِعَامَّةَهُمَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أولعامتي'] = {
    'word': u'أولعامتي',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَعَامَّةَِي',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أولعامتك'] = {
    'word': u'أولعامتك',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَعَامَّةَكَ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أولعامتك'] = {
    'word': u'أولعامتك',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَعَامَّةَكِ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أولعامته'] = {
    'word': u'أولعامته',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَعَامَّةَه',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'ه'
}
STOPWORDS[u'أولعامتكم'] = {
    'word': u'أولعامتكم',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَعَامَّةَكُمْ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أولعامتكن'] = {
    'word': u'أولعامتكن',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَعَامَّةَكُنَّ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أولعامتها'] = {
    'word': u'أولعامتها',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَعَامَّةَهَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أولعامتهم'] = {
    'word': u'أولعامتهم',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَعَامَّةَهُمْ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أولعامتهن'] = {
    'word': u'أولعامتهن',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَعَامَّةَهُنَّ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أولعامتنا'] = {
    'word': u'أولعامتنا',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَعَامَّةَنَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أولعامتكما'] = {
    'word': u'أولعامتكما',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَعَامَّةَكُمَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أولعامتهما'] = {
    'word': u'أولعامتهما',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَعَامَّةَهُمَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أفلعامتي'] = {
    'word': u'أفلعامتي',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَعَامَّةَِي',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أفلعامتك'] = {
    'word': u'أفلعامتك',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَعَامَّةَكَ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أفلعامتك'] = {
    'word': u'أفلعامتك',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَعَامَّةَكِ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أفلعامته'] = {
    'word': u'أفلعامته',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَعَامَّةَه',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'ه'
}
STOPWORDS[u'أفلعامتكم'] = {
    'word': u'أفلعامتكم',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَعَامَّةَكُمْ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أفلعامتكن'] = {
    'word': u'أفلعامتكن',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَعَامَّةَكُنَّ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أفلعامتها'] = {
    'word': u'أفلعامتها',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَعَامَّةَهَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أفلعامتهم'] = {
    'word': u'أفلعامتهم',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَعَامَّةَهُمْ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أفلعامتهن'] = {
    'word': u'أفلعامتهن',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَعَامَّةَهُنَّ',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أفلعامتنا'] = {
    'word': u'أفلعامتنا',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَعَامَّةَنَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أفلعامتكما'] = {
    'word': u'أفلعامتكما',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَعَامَّةَكُمَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أفلعامتهما'] = {
    'word': u'أفلعامتهما',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَعَامَّةَهُمَا',
    'stem': u'عَامَّةَ',
    'type': u'STOPWORD',
    'original': u'عَامَّةَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'عين'] = {
    'word': u'عَيْنَ',
    'procletic': u'',
    'tags': u'أداة:توكيد',
    'vocalized': u'عَيْنَ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u''
}
STOPWORDS[u'بعين'] = {
    'word': u'بعين',
    'procletic': u'-بِ',
    'tags': u'أداة:توكيد:مجرور',
    'vocalized': u'بِعَيْنَ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u''
}
STOPWORDS[u'كعين'] = {
    'word': u'كعين',
    'procletic': u'-كَ',
    'tags': u'أداة:توكيد:مجرور',
    'vocalized': u'كَعَيْنَ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u''
}
STOPWORDS[u'لعين'] = {
    'word': u'لعين',
    'procletic': u'-لِ',
    'tags': u'أداة:توكيد:مجرور',
    'vocalized': u'لِعَيْنَ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u''
}
STOPWORDS[u'لعين'] = {
    'word': u'لعين',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور',
    'vocalized': u'لَعَيْنَ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u''
}
STOPWORDS[u'وعين'] = {
    'word': u'وعين',
    'procletic': u'-وَ',
    'tags': u'أداة:توكيد:معطوف',
    'vocalized': u'وَعَيْنَ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u''
}
STOPWORDS[u'فعين'] = {
    'word': u'فعين',
    'procletic': u'-فَ',
    'tags': u'أداة:توكيد:معطوف',
    'vocalized': u'فَعَيْنَ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u''
}
STOPWORDS[u'وبعين'] = {
    'word': u'وبعين',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف',
    'vocalized': u'وَبِعَيْنَ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u''
}
STOPWORDS[u'فبعين'] = {
    'word': u'فبعين',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف',
    'vocalized': u'فَبِعَيْنَ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u''
}
STOPWORDS[u'وكعين'] = {
    'word': u'وكعين',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف',
    'vocalized': u'وَكَعَيْنَ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u''
}
STOPWORDS[u'فكعين'] = {
    'word': u'فكعين',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف',
    'vocalized': u'فَكَعَيْنَ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u''
}
STOPWORDS[u'ولعين'] = {
    'word': u'ولعين',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف',
    'vocalized': u'وَلِعَيْنَ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u''
}
STOPWORDS[u'فلعين'] = {
    'word': u'فلعين',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف',
    'vocalized': u'فَلِعَيْنَ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u''
}
STOPWORDS[u'ولعين'] = {
    'word': u'ولعين',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'وَلَعَيْنَ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u''
}
STOPWORDS[u'فلعين'] = {
    'word': u'فلعين',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'فَلَعَيْنَ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u''
}
STOPWORDS[u'أعين'] = {
    'word': u'أعين',
    'procletic': u'-أَ',
    'tags': u'أداة:توكيد:مجرور',
    'vocalized': u'أَعَيْنَ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u''
}
STOPWORDS[u'أبعين'] = {
    'word': u'أبعين',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور',
    'vocalized': u'أَبِعَيْنَ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u''
}
STOPWORDS[u'أكعين'] = {
    'word': u'أكعين',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور',
    'vocalized': u'أَكَعَيْنَ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u''
}
STOPWORDS[u'ألعين'] = {
    'word': u'ألعين',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور',
    'vocalized': u'أَلِعَيْنَ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u''
}
STOPWORDS[u'ألعين'] = {
    'word': u'ألعين',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور',
    'vocalized': u'أَلَعَيْنَ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u''
}
STOPWORDS[u'أوعين'] = {
    'word': u'أوعين',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور',
    'vocalized': u'أَوَعَيْنَ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u''
}
STOPWORDS[u'أفعين'] = {
    'word': u'أفعين',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور',
    'vocalized': u'أَفَعَيْنَ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u''
}
STOPWORDS[u'أوبعين'] = {
    'word': u'أوبعين',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَبِعَيْنَ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u''
}
STOPWORDS[u'أفبعين'] = {
    'word': u'أفبعين',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَبِعَيْنَ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u''
}
STOPWORDS[u'أوكعين'] = {
    'word': u'أوكعين',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَكَعَيْنَ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u''
}
STOPWORDS[u'أفكعين'] = {
    'word': u'أفكعين',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَكَعَيْنَ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u''
}
STOPWORDS[u'أولعين'] = {
    'word': u'أولعين',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَلِعَيْنَ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u''
}
STOPWORDS[u'أفلعين'] = {
    'word': u'أفلعين',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَلِعَيْنَ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u''
}
STOPWORDS[u'أولعين'] = {
    'word': u'أولعين',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَلَعَيْنَ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u''
}
STOPWORDS[u'أفلعين'] = {
    'word': u'أفلعين',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَلَعَيْنَ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u''
}
STOPWORDS[u'عيني'] = {
    'word': u'عيني',
    'procletic': u'',
    'tags': u'أداة:توكيد:مضاف',
    'vocalized': u'عَيْنَِي',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'ِي'
}
STOPWORDS[u'عينك'] = {
    'word': u'عينك',
    'procletic': u'',
    'tags': u'أداة:توكيد:مضاف',
    'vocalized': u'عَيْنَكَ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كَ'
}
STOPWORDS[u'عينك'] = {
    'word': u'عينك',
    'procletic': u'',
    'tags': u'أداة:توكيد:مضاف',
    'vocalized': u'عَيْنَكِ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كِ'
}
STOPWORDS[u'عينه'] = {
    'word': u'عينه',
    'procletic': u'',
    'tags': u'أداة:توكيد:مضاف',
    'vocalized': u'عَيْنَه',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'ه'
}
STOPWORDS[u'عينكم'] = {
    'word': u'عينكم',
    'procletic': u'',
    'tags': u'أداة:توكيد:مضاف',
    'vocalized': u'عَيْنَكُمْ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'عينكن'] = {
    'word': u'عينكن',
    'procletic': u'',
    'tags': u'أداة:توكيد:مضاف',
    'vocalized': u'عَيْنَكُنَّ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'عينها'] = {
    'word': u'عينها',
    'procletic': u'',
    'tags': u'أداة:توكيد:مضاف',
    'vocalized': u'عَيْنَهَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هَا'
}
STOPWORDS[u'عينهم'] = {
    'word': u'عينهم',
    'procletic': u'',
    'tags': u'أداة:توكيد:مضاف',
    'vocalized': u'عَيْنَهُمْ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'عينهن'] = {
    'word': u'عينهن',
    'procletic': u'',
    'tags': u'أداة:توكيد:مضاف',
    'vocalized': u'عَيْنَهُنَّ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'عيننا'] = {
    'word': u'عيننا',
    'procletic': u'',
    'tags': u'أداة:توكيد:مضاف',
    'vocalized': u'عَيْنَنَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'نَا'
}
STOPWORDS[u'عينكما'] = {
    'word': u'عينكما',
    'procletic': u'',
    'tags': u'أداة:توكيد:مضاف',
    'vocalized': u'عَيْنَكُمَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'عينهما'] = {
    'word': u'عينهما',
    'procletic': u'',
    'tags': u'أداة:توكيد:مضاف',
    'vocalized': u'عَيْنَهُمَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'بعيني'] = {
    'word': u'بعيني',
    'procletic': u'-بِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'بِعَيْنَِي',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'ِي'
}
STOPWORDS[u'بعينك'] = {
    'word': u'بعينك',
    'procletic': u'-بِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'بِعَيْنَكَ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كَ'
}
STOPWORDS[u'بعينك'] = {
    'word': u'بعينك',
    'procletic': u'-بِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'بِعَيْنَكِ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كِ'
}
STOPWORDS[u'بعينه'] = {
    'word': u'بعينه',
    'procletic': u'-بِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'بِعَيْنَه',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'ه'
}
STOPWORDS[u'بعينكم'] = {
    'word': u'بعينكم',
    'procletic': u'-بِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'بِعَيْنَكُمْ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'بعينكن'] = {
    'word': u'بعينكن',
    'procletic': u'-بِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'بِعَيْنَكُنَّ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'بعينها'] = {
    'word': u'بعينها',
    'procletic': u'-بِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'بِعَيْنَهَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هَا'
}
STOPWORDS[u'بعينهم'] = {
    'word': u'بعينهم',
    'procletic': u'-بِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'بِعَيْنَهُمْ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'بعينهن'] = {
    'word': u'بعينهن',
    'procletic': u'-بِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'بِعَيْنَهُنَّ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'بعيننا'] = {
    'word': u'بعيننا',
    'procletic': u'-بِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'بِعَيْنَنَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'نَا'
}
STOPWORDS[u'بعينكما'] = {
    'word': u'بعينكما',
    'procletic': u'-بِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'بِعَيْنَكُمَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'بعينهما'] = {
    'word': u'بعينهما',
    'procletic': u'-بِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'بِعَيْنَهُمَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'كعيني'] = {
    'word': u'كعيني',
    'procletic': u'-كَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'كَعَيْنَِي',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'ِي'
}
STOPWORDS[u'كعينك'] = {
    'word': u'كعينك',
    'procletic': u'-كَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'كَعَيْنَكَ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كَ'
}
STOPWORDS[u'كعينك'] = {
    'word': u'كعينك',
    'procletic': u'-كَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'كَعَيْنَكِ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كِ'
}
STOPWORDS[u'كعينه'] = {
    'word': u'كعينه',
    'procletic': u'-كَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'كَعَيْنَه',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'ه'
}
STOPWORDS[u'كعينكم'] = {
    'word': u'كعينكم',
    'procletic': u'-كَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'كَعَيْنَكُمْ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'كعينكن'] = {
    'word': u'كعينكن',
    'procletic': u'-كَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'كَعَيْنَكُنَّ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'كعينها'] = {
    'word': u'كعينها',
    'procletic': u'-كَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'كَعَيْنَهَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هَا'
}
STOPWORDS[u'كعينهم'] = {
    'word': u'كعينهم',
    'procletic': u'-كَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'كَعَيْنَهُمْ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'كعينهن'] = {
    'word': u'كعينهن',
    'procletic': u'-كَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'كَعَيْنَهُنَّ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'كعيننا'] = {
    'word': u'كعيننا',
    'procletic': u'-كَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'كَعَيْنَنَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'نَا'
}
STOPWORDS[u'كعينكما'] = {
    'word': u'كعينكما',
    'procletic': u'-كَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'كَعَيْنَكُمَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'كعينهما'] = {
    'word': u'كعينهما',
    'procletic': u'-كَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'كَعَيْنَهُمَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'لعيني'] = {
    'word': u'لعيني',
    'procletic': u'-لِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'لِعَيْنَِي',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'ِي'
}
STOPWORDS[u'لعينك'] = {
    'word': u'لعينك',
    'procletic': u'-لِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'لِعَيْنَكَ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كَ'
}
STOPWORDS[u'لعينك'] = {
    'word': u'لعينك',
    'procletic': u'-لِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'لِعَيْنَكِ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كِ'
}
STOPWORDS[u'لعينه'] = {
    'word': u'لعينه',
    'procletic': u'-لِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'لِعَيْنَه',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'ه'
}
STOPWORDS[u'لعينكم'] = {
    'word': u'لعينكم',
    'procletic': u'-لِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'لِعَيْنَكُمْ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'لعينكن'] = {
    'word': u'لعينكن',
    'procletic': u'-لِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'لِعَيْنَكُنَّ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'لعينها'] = {
    'word': u'لعينها',
    'procletic': u'-لِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'لِعَيْنَهَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هَا'
}
STOPWORDS[u'لعينهم'] = {
    'word': u'لعينهم',
    'procletic': u'-لِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'لِعَيْنَهُمْ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'لعينهن'] = {
    'word': u'لعينهن',
    'procletic': u'-لِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'لِعَيْنَهُنَّ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'لعيننا'] = {
    'word': u'لعيننا',
    'procletic': u'-لِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'لِعَيْنَنَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'نَا'
}
STOPWORDS[u'لعينكما'] = {
    'word': u'لعينكما',
    'procletic': u'-لِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'لِعَيْنَكُمَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'لعينهما'] = {
    'word': u'لعينهما',
    'procletic': u'-لِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'لِعَيْنَهُمَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'لعيني'] = {
    'word': u'لعيني',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَعَيْنَِي',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'ِي'
}
STOPWORDS[u'لعينك'] = {
    'word': u'لعينك',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَعَيْنَكَ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كَ'
}
STOPWORDS[u'لعينك'] = {
    'word': u'لعينك',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَعَيْنَكِ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كِ'
}
STOPWORDS[u'لعينه'] = {
    'word': u'لعينه',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَعَيْنَه',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'ه'
}
STOPWORDS[u'لعينكم'] = {
    'word': u'لعينكم',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَعَيْنَكُمْ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'لعينكن'] = {
    'word': u'لعينكن',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَعَيْنَكُنَّ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'لعينها'] = {
    'word': u'لعينها',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَعَيْنَهَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هَا'
}
STOPWORDS[u'لعينهم'] = {
    'word': u'لعينهم',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَعَيْنَهُمْ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'لعينهن'] = {
    'word': u'لعينهن',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَعَيْنَهُنَّ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'لعيننا'] = {
    'word': u'لعيننا',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَعَيْنَنَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'نَا'
}
STOPWORDS[u'لعينكما'] = {
    'word': u'لعينكما',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَعَيْنَكُمَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'لعينهما'] = {
    'word': u'لعينهما',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَعَيْنَهُمَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'وعيني'] = {
    'word': u'وعيني',
    'procletic': u'-وَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'وَعَيْنَِي',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'ِي'
}
STOPWORDS[u'وعينك'] = {
    'word': u'وعينك',
    'procletic': u'-وَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'وَعَيْنَكَ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كَ'
}
STOPWORDS[u'وعينك'] = {
    'word': u'وعينك',
    'procletic': u'-وَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'وَعَيْنَكِ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كِ'
}
STOPWORDS[u'وعينه'] = {
    'word': u'وعينه',
    'procletic': u'-وَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'وَعَيْنَه',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'ه'
}
STOPWORDS[u'وعينكم'] = {
    'word': u'وعينكم',
    'procletic': u'-وَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'وَعَيْنَكُمْ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'وعينكن'] = {
    'word': u'وعينكن',
    'procletic': u'-وَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'وَعَيْنَكُنَّ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'وعينها'] = {
    'word': u'وعينها',
    'procletic': u'-وَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'وَعَيْنَهَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هَا'
}
STOPWORDS[u'وعينهم'] = {
    'word': u'وعينهم',
    'procletic': u'-وَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'وَعَيْنَهُمْ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'وعينهن'] = {
    'word': u'وعينهن',
    'procletic': u'-وَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'وَعَيْنَهُنَّ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'وعيننا'] = {
    'word': u'وعيننا',
    'procletic': u'-وَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'وَعَيْنَنَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'نَا'
}
STOPWORDS[u'وعينكما'] = {
    'word': u'وعينكما',
    'procletic': u'-وَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'وَعَيْنَكُمَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'وعينهما'] = {
    'word': u'وعينهما',
    'procletic': u'-وَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'وَعَيْنَهُمَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فعيني'] = {
    'word': u'فعيني',
    'procletic': u'-فَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'فَعَيْنَِي',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'ِي'
}
STOPWORDS[u'فعينك'] = {
    'word': u'فعينك',
    'procletic': u'-فَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'فَعَيْنَكَ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كَ'
}
STOPWORDS[u'فعينك'] = {
    'word': u'فعينك',
    'procletic': u'-فَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'فَعَيْنَكِ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كِ'
}
STOPWORDS[u'فعينه'] = {
    'word': u'فعينه',
    'procletic': u'-فَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'فَعَيْنَه',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'ه'
}
STOPWORDS[u'فعينكم'] = {
    'word': u'فعينكم',
    'procletic': u'-فَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'فَعَيْنَكُمْ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فعينكن'] = {
    'word': u'فعينكن',
    'procletic': u'-فَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'فَعَيْنَكُنَّ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فعينها'] = {
    'word': u'فعينها',
    'procletic': u'-فَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'فَعَيْنَهَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هَا'
}
STOPWORDS[u'فعينهم'] = {
    'word': u'فعينهم',
    'procletic': u'-فَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'فَعَيْنَهُمْ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فعينهن'] = {
    'word': u'فعينهن',
    'procletic': u'-فَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'فَعَيْنَهُنَّ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فعيننا'] = {
    'word': u'فعيننا',
    'procletic': u'-فَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'فَعَيْنَنَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'نَا'
}
STOPWORDS[u'فعينكما'] = {
    'word': u'فعينكما',
    'procletic': u'-فَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'فَعَيْنَكُمَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فعينهما'] = {
    'word': u'فعينهما',
    'procletic': u'-فَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'فَعَيْنَهُمَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'وبعيني'] = {
    'word': u'وبعيني',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِعَيْنَِي',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'ِي'
}
STOPWORDS[u'وبعينك'] = {
    'word': u'وبعينك',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِعَيْنَكَ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كَ'
}
STOPWORDS[u'وبعينك'] = {
    'word': u'وبعينك',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِعَيْنَكِ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كِ'
}
STOPWORDS[u'وبعينه'] = {
    'word': u'وبعينه',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِعَيْنَه',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'ه'
}
STOPWORDS[u'وبعينكم'] = {
    'word': u'وبعينكم',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِعَيْنَكُمْ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'وبعينكن'] = {
    'word': u'وبعينكن',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِعَيْنَكُنَّ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'وبعينها'] = {
    'word': u'وبعينها',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِعَيْنَهَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هَا'
}
STOPWORDS[u'وبعينهم'] = {
    'word': u'وبعينهم',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِعَيْنَهُمْ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'وبعينهن'] = {
    'word': u'وبعينهن',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِعَيْنَهُنَّ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'وبعيننا'] = {
    'word': u'وبعيننا',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِعَيْنَنَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'نَا'
}
STOPWORDS[u'وبعينكما'] = {
    'word': u'وبعينكما',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِعَيْنَكُمَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'وبعينهما'] = {
    'word': u'وبعينهما',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِعَيْنَهُمَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فبعيني'] = {
    'word': u'فبعيني',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِعَيْنَِي',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'ِي'
}
STOPWORDS[u'فبعينك'] = {
    'word': u'فبعينك',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِعَيْنَكَ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كَ'
}
STOPWORDS[u'فبعينك'] = {
    'word': u'فبعينك',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِعَيْنَكِ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كِ'
}
STOPWORDS[u'فبعينه'] = {
    'word': u'فبعينه',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِعَيْنَه',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'ه'
}
STOPWORDS[u'فبعينكم'] = {
    'word': u'فبعينكم',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِعَيْنَكُمْ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فبعينكن'] = {
    'word': u'فبعينكن',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِعَيْنَكُنَّ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فبعينها'] = {
    'word': u'فبعينها',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِعَيْنَهَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هَا'
}
STOPWORDS[u'فبعينهم'] = {
    'word': u'فبعينهم',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِعَيْنَهُمْ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فبعينهن'] = {
    'word': u'فبعينهن',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِعَيْنَهُنَّ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فبعيننا'] = {
    'word': u'فبعيننا',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِعَيْنَنَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'نَا'
}
STOPWORDS[u'فبعينكما'] = {
    'word': u'فبعينكما',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِعَيْنَكُمَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فبعينهما'] = {
    'word': u'فبعينهما',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِعَيْنَهُمَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'وكعيني'] = {
    'word': u'وكعيني',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَعَيْنَِي',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'ِي'
}
STOPWORDS[u'وكعينك'] = {
    'word': u'وكعينك',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَعَيْنَكَ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كَ'
}
STOPWORDS[u'وكعينك'] = {
    'word': u'وكعينك',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَعَيْنَكِ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كِ'
}
STOPWORDS[u'وكعينه'] = {
    'word': u'وكعينه',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَعَيْنَه',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'ه'
}
STOPWORDS[u'وكعينكم'] = {
    'word': u'وكعينكم',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَعَيْنَكُمْ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'وكعينكن'] = {
    'word': u'وكعينكن',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَعَيْنَكُنَّ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'وكعينها'] = {
    'word': u'وكعينها',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَعَيْنَهَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هَا'
}
STOPWORDS[u'وكعينهم'] = {
    'word': u'وكعينهم',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَعَيْنَهُمْ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'وكعينهن'] = {
    'word': u'وكعينهن',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَعَيْنَهُنَّ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'وكعيننا'] = {
    'word': u'وكعيننا',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَعَيْنَنَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'نَا'
}
STOPWORDS[u'وكعينكما'] = {
    'word': u'وكعينكما',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَعَيْنَكُمَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'وكعينهما'] = {
    'word': u'وكعينهما',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَعَيْنَهُمَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فكعيني'] = {
    'word': u'فكعيني',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَعَيْنَِي',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'ِي'
}
STOPWORDS[u'فكعينك'] = {
    'word': u'فكعينك',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَعَيْنَكَ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كَ'
}
STOPWORDS[u'فكعينك'] = {
    'word': u'فكعينك',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَعَيْنَكِ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كِ'
}
STOPWORDS[u'فكعينه'] = {
    'word': u'فكعينه',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَعَيْنَه',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'ه'
}
STOPWORDS[u'فكعينكم'] = {
    'word': u'فكعينكم',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَعَيْنَكُمْ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فكعينكن'] = {
    'word': u'فكعينكن',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَعَيْنَكُنَّ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فكعينها'] = {
    'word': u'فكعينها',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَعَيْنَهَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هَا'
}
STOPWORDS[u'فكعينهم'] = {
    'word': u'فكعينهم',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَعَيْنَهُمْ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فكعينهن'] = {
    'word': u'فكعينهن',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَعَيْنَهُنَّ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فكعيننا'] = {
    'word': u'فكعيننا',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَعَيْنَنَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'نَا'
}
STOPWORDS[u'فكعينكما'] = {
    'word': u'فكعينكما',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَعَيْنَكُمَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فكعينهما'] = {
    'word': u'فكعينهما',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَعَيْنَهُمَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ولعيني'] = {
    'word': u'ولعيني',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِعَيْنَِي',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'ِي'
}
STOPWORDS[u'ولعينك'] = {
    'word': u'ولعينك',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِعَيْنَكَ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كَ'
}
STOPWORDS[u'ولعينك'] = {
    'word': u'ولعينك',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِعَيْنَكِ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كِ'
}
STOPWORDS[u'ولعينه'] = {
    'word': u'ولعينه',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِعَيْنَه',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'ه'
}
STOPWORDS[u'ولعينكم'] = {
    'word': u'ولعينكم',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِعَيْنَكُمْ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ولعينكن'] = {
    'word': u'ولعينكن',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِعَيْنَكُنَّ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ولعينها'] = {
    'word': u'ولعينها',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِعَيْنَهَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هَا'
}
STOPWORDS[u'ولعينهم'] = {
    'word': u'ولعينهم',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِعَيْنَهُمْ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ولعينهن'] = {
    'word': u'ولعينهن',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِعَيْنَهُنَّ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ولعيننا'] = {
    'word': u'ولعيننا',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِعَيْنَنَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'نَا'
}
STOPWORDS[u'ولعينكما'] = {
    'word': u'ولعينكما',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِعَيْنَكُمَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ولعينهما'] = {
    'word': u'ولعينهما',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِعَيْنَهُمَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فلعيني'] = {
    'word': u'فلعيني',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِعَيْنَِي',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'ِي'
}
STOPWORDS[u'فلعينك'] = {
    'word': u'فلعينك',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِعَيْنَكَ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كَ'
}
STOPWORDS[u'فلعينك'] = {
    'word': u'فلعينك',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِعَيْنَكِ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كِ'
}
STOPWORDS[u'فلعينه'] = {
    'word': u'فلعينه',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِعَيْنَه',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'ه'
}
STOPWORDS[u'فلعينكم'] = {
    'word': u'فلعينكم',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِعَيْنَكُمْ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فلعينكن'] = {
    'word': u'فلعينكن',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِعَيْنَكُنَّ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فلعينها'] = {
    'word': u'فلعينها',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِعَيْنَهَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هَا'
}
STOPWORDS[u'فلعينهم'] = {
    'word': u'فلعينهم',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِعَيْنَهُمْ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فلعينهن'] = {
    'word': u'فلعينهن',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِعَيْنَهُنَّ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فلعيننا'] = {
    'word': u'فلعيننا',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِعَيْنَنَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'نَا'
}
STOPWORDS[u'فلعينكما'] = {
    'word': u'فلعينكما',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِعَيْنَكُمَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فلعينهما'] = {
    'word': u'فلعينهما',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِعَيْنَهُمَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ولعيني'] = {
    'word': u'ولعيني',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَعَيْنَِي',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'ِي'
}
STOPWORDS[u'ولعينك'] = {
    'word': u'ولعينك',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَعَيْنَكَ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كَ'
}
STOPWORDS[u'ولعينك'] = {
    'word': u'ولعينك',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَعَيْنَكِ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كِ'
}
STOPWORDS[u'ولعينه'] = {
    'word': u'ولعينه',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَعَيْنَه',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'ه'
}
STOPWORDS[u'ولعينكم'] = {
    'word': u'ولعينكم',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَعَيْنَكُمْ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ولعينكن'] = {
    'word': u'ولعينكن',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَعَيْنَكُنَّ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ولعينها'] = {
    'word': u'ولعينها',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَعَيْنَهَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هَا'
}
STOPWORDS[u'ولعينهم'] = {
    'word': u'ولعينهم',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَعَيْنَهُمْ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ولعينهن'] = {
    'word': u'ولعينهن',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَعَيْنَهُنَّ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ولعيننا'] = {
    'word': u'ولعيننا',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَعَيْنَنَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'نَا'
}
STOPWORDS[u'ولعينكما'] = {
    'word': u'ولعينكما',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَعَيْنَكُمَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ولعينهما'] = {
    'word': u'ولعينهما',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَعَيْنَهُمَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فلعيني'] = {
    'word': u'فلعيني',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَعَيْنَِي',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'ِي'
}
STOPWORDS[u'فلعينك'] = {
    'word': u'فلعينك',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَعَيْنَكَ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كَ'
}
STOPWORDS[u'فلعينك'] = {
    'word': u'فلعينك',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَعَيْنَكِ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كِ'
}
STOPWORDS[u'فلعينه'] = {
    'word': u'فلعينه',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَعَيْنَه',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'ه'
}
STOPWORDS[u'فلعينكم'] = {
    'word': u'فلعينكم',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَعَيْنَكُمْ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فلعينكن'] = {
    'word': u'فلعينكن',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَعَيْنَكُنَّ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فلعينها'] = {
    'word': u'فلعينها',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَعَيْنَهَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هَا'
}
STOPWORDS[u'فلعينهم'] = {
    'word': u'فلعينهم',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَعَيْنَهُمْ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فلعينهن'] = {
    'word': u'فلعينهن',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَعَيْنَهُنَّ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فلعيننا'] = {
    'word': u'فلعيننا',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَعَيْنَنَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'نَا'
}
STOPWORDS[u'فلعينكما'] = {
    'word': u'فلعينكما',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَعَيْنَكُمَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فلعينهما'] = {
    'word': u'فلعينهما',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَعَيْنَهُمَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أعيني'] = {
    'word': u'أعيني',
    'procletic': u'-أَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'أَعَيْنَِي',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أعينك'] = {
    'word': u'أعينك',
    'procletic': u'-أَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'أَعَيْنَكَ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أعينك'] = {
    'word': u'أعينك',
    'procletic': u'-أَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'أَعَيْنَكِ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أعينه'] = {
    'word': u'أعينه',
    'procletic': u'-أَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'أَعَيْنَه',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'ه'
}
STOPWORDS[u'أعينكم'] = {
    'word': u'أعينكم',
    'procletic': u'-أَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'أَعَيْنَكُمْ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أعينكن'] = {
    'word': u'أعينكن',
    'procletic': u'-أَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'أَعَيْنَكُنَّ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أعينها'] = {
    'word': u'أعينها',
    'procletic': u'-أَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'أَعَيْنَهَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أعينهم'] = {
    'word': u'أعينهم',
    'procletic': u'-أَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'أَعَيْنَهُمْ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أعينهن'] = {
    'word': u'أعينهن',
    'procletic': u'-أَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'أَعَيْنَهُنَّ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أعيننا'] = {
    'word': u'أعيننا',
    'procletic': u'-أَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'أَعَيْنَنَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أعينكما'] = {
    'word': u'أعينكما',
    'procletic': u'-أَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'أَعَيْنَكُمَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أعينهما'] = {
    'word': u'أعينهما',
    'procletic': u'-أَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'أَعَيْنَهُمَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أبعيني'] = {
    'word': u'أبعيني',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِعَيْنَِي',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أبعينك'] = {
    'word': u'أبعينك',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِعَيْنَكَ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أبعينك'] = {
    'word': u'أبعينك',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِعَيْنَكِ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أبعينه'] = {
    'word': u'أبعينه',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِعَيْنَه',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'ه'
}
STOPWORDS[u'أبعينكم'] = {
    'word': u'أبعينكم',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِعَيْنَكُمْ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أبعينكن'] = {
    'word': u'أبعينكن',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِعَيْنَكُنَّ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أبعينها'] = {
    'word': u'أبعينها',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِعَيْنَهَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أبعينهم'] = {
    'word': u'أبعينهم',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِعَيْنَهُمْ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أبعينهن'] = {
    'word': u'أبعينهن',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِعَيْنَهُنَّ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أبعيننا'] = {
    'word': u'أبعيننا',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِعَيْنَنَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أبعينكما'] = {
    'word': u'أبعينكما',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِعَيْنَكُمَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أبعينهما'] = {
    'word': u'أبعينهما',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِعَيْنَهُمَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أكعيني'] = {
    'word': u'أكعيني',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَعَيْنَِي',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أكعينك'] = {
    'word': u'أكعينك',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَعَيْنَكَ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أكعينك'] = {
    'word': u'أكعينك',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَعَيْنَكِ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أكعينه'] = {
    'word': u'أكعينه',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَعَيْنَه',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'ه'
}
STOPWORDS[u'أكعينكم'] = {
    'word': u'أكعينكم',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَعَيْنَكُمْ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أكعينكن'] = {
    'word': u'أكعينكن',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَعَيْنَكُنَّ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أكعينها'] = {
    'word': u'أكعينها',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَعَيْنَهَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أكعينهم'] = {
    'word': u'أكعينهم',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَعَيْنَهُمْ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أكعينهن'] = {
    'word': u'أكعينهن',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَعَيْنَهُنَّ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أكعيننا'] = {
    'word': u'أكعيننا',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَعَيْنَنَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أكعينكما'] = {
    'word': u'أكعينكما',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَعَيْنَكُمَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أكعينهما'] = {
    'word': u'أكعينهما',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَعَيْنَهُمَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ألعيني'] = {
    'word': u'ألعيني',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِعَيْنَِي',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'ِي'
}
STOPWORDS[u'ألعينك'] = {
    'word': u'ألعينك',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِعَيْنَكَ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كَ'
}
STOPWORDS[u'ألعينك'] = {
    'word': u'ألعينك',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِعَيْنَكِ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كِ'
}
STOPWORDS[u'ألعينه'] = {
    'word': u'ألعينه',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِعَيْنَه',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'ه'
}
STOPWORDS[u'ألعينكم'] = {
    'word': u'ألعينكم',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِعَيْنَكُمْ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ألعينكن'] = {
    'word': u'ألعينكن',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِعَيْنَكُنَّ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ألعينها'] = {
    'word': u'ألعينها',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِعَيْنَهَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هَا'
}
STOPWORDS[u'ألعينهم'] = {
    'word': u'ألعينهم',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِعَيْنَهُمْ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ألعينهن'] = {
    'word': u'ألعينهن',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِعَيْنَهُنَّ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ألعيننا'] = {
    'word': u'ألعيننا',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِعَيْنَنَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'نَا'
}
STOPWORDS[u'ألعينكما'] = {
    'word': u'ألعينكما',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِعَيْنَكُمَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ألعينهما'] = {
    'word': u'ألعينهما',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِعَيْنَهُمَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ألعيني'] = {
    'word': u'ألعيني',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَعَيْنَِي',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'ِي'
}
STOPWORDS[u'ألعينك'] = {
    'word': u'ألعينك',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَعَيْنَكَ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كَ'
}
STOPWORDS[u'ألعينك'] = {
    'word': u'ألعينك',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَعَيْنَكِ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كِ'
}
STOPWORDS[u'ألعينه'] = {
    'word': u'ألعينه',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَعَيْنَه',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'ه'
}
STOPWORDS[u'ألعينكم'] = {
    'word': u'ألعينكم',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَعَيْنَكُمْ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ألعينكن'] = {
    'word': u'ألعينكن',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَعَيْنَكُنَّ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ألعينها'] = {
    'word': u'ألعينها',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَعَيْنَهَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هَا'
}
STOPWORDS[u'ألعينهم'] = {
    'word': u'ألعينهم',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَعَيْنَهُمْ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ألعينهن'] = {
    'word': u'ألعينهن',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَعَيْنَهُنَّ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ألعيننا'] = {
    'word': u'ألعيننا',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَعَيْنَنَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'نَا'
}
STOPWORDS[u'ألعينكما'] = {
    'word': u'ألعينكما',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَعَيْنَكُمَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ألعينهما'] = {
    'word': u'ألعينهما',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَعَيْنَهُمَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أوعيني'] = {
    'word': u'أوعيني',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَعَيْنَِي',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أوعينك'] = {
    'word': u'أوعينك',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَعَيْنَكَ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أوعينك'] = {
    'word': u'أوعينك',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَعَيْنَكِ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أوعينه'] = {
    'word': u'أوعينه',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَعَيْنَه',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'ه'
}
STOPWORDS[u'أوعينكم'] = {
    'word': u'أوعينكم',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَعَيْنَكُمْ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أوعينكن'] = {
    'word': u'أوعينكن',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَعَيْنَكُنَّ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أوعينها'] = {
    'word': u'أوعينها',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَعَيْنَهَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أوعينهم'] = {
    'word': u'أوعينهم',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَعَيْنَهُمْ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أوعينهن'] = {
    'word': u'أوعينهن',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَعَيْنَهُنَّ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أوعيننا'] = {
    'word': u'أوعيننا',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَعَيْنَنَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أوعينكما'] = {
    'word': u'أوعينكما',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَعَيْنَكُمَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أوعينهما'] = {
    'word': u'أوعينهما',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَعَيْنَهُمَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أفعيني'] = {
    'word': u'أفعيني',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَعَيْنَِي',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أفعينك'] = {
    'word': u'أفعينك',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَعَيْنَكَ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أفعينك'] = {
    'word': u'أفعينك',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَعَيْنَكِ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أفعينه'] = {
    'word': u'أفعينه',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَعَيْنَه',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'ه'
}
STOPWORDS[u'أفعينكم'] = {
    'word': u'أفعينكم',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَعَيْنَكُمْ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أفعينكن'] = {
    'word': u'أفعينكن',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَعَيْنَكُنَّ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أفعينها'] = {
    'word': u'أفعينها',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَعَيْنَهَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أفعينهم'] = {
    'word': u'أفعينهم',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَعَيْنَهُمْ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أفعينهن'] = {
    'word': u'أفعينهن',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَعَيْنَهُنَّ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أفعيننا'] = {
    'word': u'أفعيننا',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَعَيْنَنَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أفعينكما'] = {
    'word': u'أفعينكما',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَعَيْنَكُمَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أفعينهما'] = {
    'word': u'أفعينهما',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَعَيْنَهُمَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أوبعيني'] = {
    'word': u'أوبعيني',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِعَيْنَِي',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أوبعينك'] = {
    'word': u'أوبعينك',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِعَيْنَكَ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أوبعينك'] = {
    'word': u'أوبعينك',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِعَيْنَكِ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أوبعينه'] = {
    'word': u'أوبعينه',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِعَيْنَه',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'ه'
}
STOPWORDS[u'أوبعينكم'] = {
    'word': u'أوبعينكم',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِعَيْنَكُمْ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أوبعينكن'] = {
    'word': u'أوبعينكن',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِعَيْنَكُنَّ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أوبعينها'] = {
    'word': u'أوبعينها',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِعَيْنَهَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أوبعينهم'] = {
    'word': u'أوبعينهم',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِعَيْنَهُمْ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أوبعينهن'] = {
    'word': u'أوبعينهن',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِعَيْنَهُنَّ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أوبعيننا'] = {
    'word': u'أوبعيننا',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِعَيْنَنَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أوبعينكما'] = {
    'word': u'أوبعينكما',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِعَيْنَكُمَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أوبعينهما'] = {
    'word': u'أوبعينهما',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِعَيْنَهُمَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أفبعيني'] = {
    'word': u'أفبعيني',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِعَيْنَِي',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أفبعينك'] = {
    'word': u'أفبعينك',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِعَيْنَكَ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أفبعينك'] = {
    'word': u'أفبعينك',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِعَيْنَكِ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أفبعينه'] = {
    'word': u'أفبعينه',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِعَيْنَه',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'ه'
}
STOPWORDS[u'أفبعينكم'] = {
    'word': u'أفبعينكم',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِعَيْنَكُمْ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أفبعينكن'] = {
    'word': u'أفبعينكن',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِعَيْنَكُنَّ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أفبعينها'] = {
    'word': u'أفبعينها',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِعَيْنَهَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أفبعينهم'] = {
    'word': u'أفبعينهم',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِعَيْنَهُمْ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أفبعينهن'] = {
    'word': u'أفبعينهن',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِعَيْنَهُنَّ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أفبعيننا'] = {
    'word': u'أفبعيننا',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِعَيْنَنَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أفبعينكما'] = {
    'word': u'أفبعينكما',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِعَيْنَكُمَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أفبعينهما'] = {
    'word': u'أفبعينهما',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِعَيْنَهُمَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أوكعيني'] = {
    'word': u'أوكعيني',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَعَيْنَِي',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أوكعينك'] = {
    'word': u'أوكعينك',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَعَيْنَكَ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أوكعينك'] = {
    'word': u'أوكعينك',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَعَيْنَكِ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أوكعينه'] = {
    'word': u'أوكعينه',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَعَيْنَه',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'ه'
}
STOPWORDS[u'أوكعينكم'] = {
    'word': u'أوكعينكم',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَعَيْنَكُمْ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أوكعينكن'] = {
    'word': u'أوكعينكن',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَعَيْنَكُنَّ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أوكعينها'] = {
    'word': u'أوكعينها',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَعَيْنَهَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أوكعينهم'] = {
    'word': u'أوكعينهم',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَعَيْنَهُمْ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أوكعينهن'] = {
    'word': u'أوكعينهن',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَعَيْنَهُنَّ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أوكعيننا'] = {
    'word': u'أوكعيننا',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَعَيْنَنَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أوكعينكما'] = {
    'word': u'أوكعينكما',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَعَيْنَكُمَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أوكعينهما'] = {
    'word': u'أوكعينهما',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَعَيْنَهُمَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أفكعيني'] = {
    'word': u'أفكعيني',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَعَيْنَِي',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أفكعينك'] = {
    'word': u'أفكعينك',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَعَيْنَكَ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أفكعينك'] = {
    'word': u'أفكعينك',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَعَيْنَكِ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أفكعينه'] = {
    'word': u'أفكعينه',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَعَيْنَه',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'ه'
}
STOPWORDS[u'أفكعينكم'] = {
    'word': u'أفكعينكم',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَعَيْنَكُمْ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أفكعينكن'] = {
    'word': u'أفكعينكن',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَعَيْنَكُنَّ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أفكعينها'] = {
    'word': u'أفكعينها',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَعَيْنَهَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أفكعينهم'] = {
    'word': u'أفكعينهم',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَعَيْنَهُمْ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أفكعينهن'] = {
    'word': u'أفكعينهن',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَعَيْنَهُنَّ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أفكعيننا'] = {
    'word': u'أفكعيننا',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَعَيْنَنَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أفكعينكما'] = {
    'word': u'أفكعينكما',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَعَيْنَكُمَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أفكعينهما'] = {
    'word': u'أفكعينهما',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَعَيْنَهُمَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أولعيني'] = {
    'word': u'أولعيني',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِعَيْنَِي',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أولعينك'] = {
    'word': u'أولعينك',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِعَيْنَكَ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أولعينك'] = {
    'word': u'أولعينك',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِعَيْنَكِ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أولعينه'] = {
    'word': u'أولعينه',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِعَيْنَه',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'ه'
}
STOPWORDS[u'أولعينكم'] = {
    'word': u'أولعينكم',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِعَيْنَكُمْ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أولعينكن'] = {
    'word': u'أولعينكن',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِعَيْنَكُنَّ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أولعينها'] = {
    'word': u'أولعينها',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِعَيْنَهَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أولعينهم'] = {
    'word': u'أولعينهم',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِعَيْنَهُمْ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أولعينهن'] = {
    'word': u'أولعينهن',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِعَيْنَهُنَّ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أولعيننا'] = {
    'word': u'أولعيننا',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِعَيْنَنَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أولعينكما'] = {
    'word': u'أولعينكما',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِعَيْنَكُمَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أولعينهما'] = {
    'word': u'أولعينهما',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِعَيْنَهُمَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أفلعيني'] = {
    'word': u'أفلعيني',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِعَيْنَِي',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أفلعينك'] = {
    'word': u'أفلعينك',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِعَيْنَكَ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أفلعينك'] = {
    'word': u'أفلعينك',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِعَيْنَكِ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أفلعينه'] = {
    'word': u'أفلعينه',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِعَيْنَه',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'ه'
}
STOPWORDS[u'أفلعينكم'] = {
    'word': u'أفلعينكم',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِعَيْنَكُمْ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أفلعينكن'] = {
    'word': u'أفلعينكن',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِعَيْنَكُنَّ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أفلعينها'] = {
    'word': u'أفلعينها',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِعَيْنَهَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أفلعينهم'] = {
    'word': u'أفلعينهم',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِعَيْنَهُمْ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أفلعينهن'] = {
    'word': u'أفلعينهن',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِعَيْنَهُنَّ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أفلعيننا'] = {
    'word': u'أفلعيننا',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِعَيْنَنَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أفلعينكما'] = {
    'word': u'أفلعينكما',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِعَيْنَكُمَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أفلعينهما'] = {
    'word': u'أفلعينهما',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِعَيْنَهُمَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أولعيني'] = {
    'word': u'أولعيني',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَعَيْنَِي',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أولعينك'] = {
    'word': u'أولعينك',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَعَيْنَكَ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أولعينك'] = {
    'word': u'أولعينك',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَعَيْنَكِ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أولعينه'] = {
    'word': u'أولعينه',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَعَيْنَه',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'ه'
}
STOPWORDS[u'أولعينكم'] = {
    'word': u'أولعينكم',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَعَيْنَكُمْ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أولعينكن'] = {
    'word': u'أولعينكن',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَعَيْنَكُنَّ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أولعينها'] = {
    'word': u'أولعينها',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَعَيْنَهَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أولعينهم'] = {
    'word': u'أولعينهم',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَعَيْنَهُمْ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أولعينهن'] = {
    'word': u'أولعينهن',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَعَيْنَهُنَّ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أولعيننا'] = {
    'word': u'أولعيننا',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَعَيْنَنَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أولعينكما'] = {
    'word': u'أولعينكما',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَعَيْنَكُمَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أولعينهما'] = {
    'word': u'أولعينهما',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَعَيْنَهُمَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أفلعيني'] = {
    'word': u'أفلعيني',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَعَيْنَِي',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أفلعينك'] = {
    'word': u'أفلعينك',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَعَيْنَكَ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أفلعينك'] = {
    'word': u'أفلعينك',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَعَيْنَكِ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أفلعينه'] = {
    'word': u'أفلعينه',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَعَيْنَه',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'ه'
}
STOPWORDS[u'أفلعينكم'] = {
    'word': u'أفلعينكم',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَعَيْنَكُمْ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أفلعينكن'] = {
    'word': u'أفلعينكن',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَعَيْنَكُنَّ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أفلعينها'] = {
    'word': u'أفلعينها',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَعَيْنَهَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أفلعينهم'] = {
    'word': u'أفلعينهم',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَعَيْنَهُمْ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أفلعينهن'] = {
    'word': u'أفلعينهن',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَعَيْنَهُنَّ',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أفلعيننا'] = {
    'word': u'أفلعيننا',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَعَيْنَنَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أفلعينكما'] = {
    'word': u'أفلعينكما',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَعَيْنَكُمَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أفلعينهما'] = {
    'word': u'أفلعينهما',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَعَيْنَهُمَا',
    'stem': u'عَيْنَ',
    'type': u'STOPWORD',
    'original': u'عَيْنَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'كل'] = {
    'word': u'كُلَّ',
    'procletic': u'',
    'tags': u'أداة:توكيد',
    'vocalized': u'كُلَّ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u''
}
STOPWORDS[u'بكل'] = {
    'word': u'بكل',
    'procletic': u'-بِ',
    'tags': u'أداة:توكيد:مجرور',
    'vocalized': u'بِكُلَّ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u''
}
STOPWORDS[u'ككل'] = {
    'word': u'ككل',
    'procletic': u'-كَ',
    'tags': u'أداة:توكيد:مجرور',
    'vocalized': u'كَكُلَّ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u''
}
STOPWORDS[u'لكل'] = {
    'word': u'لكل',
    'procletic': u'-لِ',
    'tags': u'أداة:توكيد:مجرور',
    'vocalized': u'لِكُلَّ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u''
}
STOPWORDS[u'لكل'] = {
    'word': u'لكل',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور',
    'vocalized': u'لَكُلَّ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u''
}
STOPWORDS[u'وكل'] = {
    'word': u'وكل',
    'procletic': u'-وَ',
    'tags': u'أداة:توكيد:معطوف',
    'vocalized': u'وَكُلَّ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u''
}
STOPWORDS[u'فكل'] = {
    'word': u'فكل',
    'procletic': u'-فَ',
    'tags': u'أداة:توكيد:معطوف',
    'vocalized': u'فَكُلَّ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u''
}
STOPWORDS[u'وبكل'] = {
    'word': u'وبكل',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف',
    'vocalized': u'وَبِكُلَّ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u''
}
STOPWORDS[u'فبكل'] = {
    'word': u'فبكل',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف',
    'vocalized': u'فَبِكُلَّ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u''
}
STOPWORDS[u'وككل'] = {
    'word': u'وككل',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف',
    'vocalized': u'وَكَكُلَّ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u''
}
STOPWORDS[u'فككل'] = {
    'word': u'فككل',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف',
    'vocalized': u'فَكَكُلَّ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u''
}
STOPWORDS[u'ولكل'] = {
    'word': u'ولكل',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف',
    'vocalized': u'وَلِكُلَّ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u''
}
STOPWORDS[u'فلكل'] = {
    'word': u'فلكل',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف',
    'vocalized': u'فَلِكُلَّ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u''
}
STOPWORDS[u'ولكل'] = {
    'word': u'ولكل',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'وَلَكُلَّ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u''
}
STOPWORDS[u'فلكل'] = {
    'word': u'فلكل',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'فَلَكُلَّ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u''
}
STOPWORDS[u'أكل'] = {
    'word': u'أكل',
    'procletic': u'-أَ',
    'tags': u'أداة:توكيد:مجرور',
    'vocalized': u'أَكُلَّ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u''
}
STOPWORDS[u'أبكل'] = {
    'word': u'أبكل',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور',
    'vocalized': u'أَبِكُلَّ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u''
}
STOPWORDS[u'أككل'] = {
    'word': u'أككل',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور',
    'vocalized': u'أَكَكُلَّ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u''
}
STOPWORDS[u'ألكل'] = {
    'word': u'ألكل',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور',
    'vocalized': u'أَلِكُلَّ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u''
}
STOPWORDS[u'ألكل'] = {
    'word': u'ألكل',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور',
    'vocalized': u'أَلَكُلَّ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u''
}
STOPWORDS[u'أوكل'] = {
    'word': u'أوكل',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور',
    'vocalized': u'أَوَكُلَّ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u''
}
STOPWORDS[u'أفكل'] = {
    'word': u'أفكل',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور',
    'vocalized': u'أَفَكُلَّ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u''
}
STOPWORDS[u'أوبكل'] = {
    'word': u'أوبكل',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَبِكُلَّ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u''
}
STOPWORDS[u'أفبكل'] = {
    'word': u'أفبكل',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَبِكُلَّ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u''
}
STOPWORDS[u'أوككل'] = {
    'word': u'أوككل',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَكَكُلَّ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u''
}
STOPWORDS[u'أفككل'] = {
    'word': u'أفككل',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَكَكُلَّ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u''
}
STOPWORDS[u'أولكل'] = {
    'word': u'أولكل',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَلِكُلَّ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u''
}
STOPWORDS[u'أفلكل'] = {
    'word': u'أفلكل',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَلِكُلَّ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u''
}
STOPWORDS[u'أولكل'] = {
    'word': u'أولكل',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَلَكُلَّ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u''
}
STOPWORDS[u'أفلكل'] = {
    'word': u'أفلكل',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَلَكُلَّ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u''
}
STOPWORDS[u'كلي'] = {
    'word': u'كلي',
    'procletic': u'',
    'tags': u'أداة:توكيد:مضاف',
    'vocalized': u'كُلَِّي',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'ِي'
}
STOPWORDS[u'كلك'] = {
    'word': u'كلك',
    'procletic': u'',
    'tags': u'أداة:توكيد:مضاف',
    'vocalized': u'كُلَّكَ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كَ'
}
STOPWORDS[u'كلك'] = {
    'word': u'كلك',
    'procletic': u'',
    'tags': u'أداة:توكيد:مضاف',
    'vocalized': u'كُلَّكِ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كِ'
}
STOPWORDS[u'كله'] = {
    'word': u'كله',
    'procletic': u'',
    'tags': u'أداة:توكيد:مضاف',
    'vocalized': u'كُلَّه',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'ه'
}
STOPWORDS[u'كلكم'] = {
    'word': u'كلكم',
    'procletic': u'',
    'tags': u'أداة:توكيد:مضاف',
    'vocalized': u'كُلَّكُمْ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'كلكن'] = {
    'word': u'كلكن',
    'procletic': u'',
    'tags': u'أداة:توكيد:مضاف',
    'vocalized': u'كُلَّكُنَّ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'كلها'] = {
    'word': u'كلها',
    'procletic': u'',
    'tags': u'أداة:توكيد:مضاف',
    'vocalized': u'كُلَّهَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هَا'
}
STOPWORDS[u'كلهم'] = {
    'word': u'كلهم',
    'procletic': u'',
    'tags': u'أداة:توكيد:مضاف',
    'vocalized': u'كُلَّهُمْ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'كلهن'] = {
    'word': u'كلهن',
    'procletic': u'',
    'tags': u'أداة:توكيد:مضاف',
    'vocalized': u'كُلَّهُنَّ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'كلنا'] = {
    'word': u'كلنا',
    'procletic': u'',
    'tags': u'أداة:توكيد:مضاف',
    'vocalized': u'كُلَّنَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'نَا'
}
STOPWORDS[u'كلكما'] = {
    'word': u'كلكما',
    'procletic': u'',
    'tags': u'أداة:توكيد:مضاف',
    'vocalized': u'كُلَّكُمَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'كلهما'] = {
    'word': u'كلهما',
    'procletic': u'',
    'tags': u'أداة:توكيد:مضاف',
    'vocalized': u'كُلَّهُمَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'بكلي'] = {
    'word': u'بكلي',
    'procletic': u'-بِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'بِكُلَِّي',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'ِي'
}
STOPWORDS[u'بكلك'] = {
    'word': u'بكلك',
    'procletic': u'-بِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'بِكُلَّكَ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كَ'
}
STOPWORDS[u'بكلك'] = {
    'word': u'بكلك',
    'procletic': u'-بِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'بِكُلَّكِ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كِ'
}
STOPWORDS[u'بكله'] = {
    'word': u'بكله',
    'procletic': u'-بِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'بِكُلَّه',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'ه'
}
STOPWORDS[u'بكلكم'] = {
    'word': u'بكلكم',
    'procletic': u'-بِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'بِكُلَّكُمْ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'بكلكن'] = {
    'word': u'بكلكن',
    'procletic': u'-بِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'بِكُلَّكُنَّ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'بكلها'] = {
    'word': u'بكلها',
    'procletic': u'-بِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'بِكُلَّهَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هَا'
}
STOPWORDS[u'بكلهم'] = {
    'word': u'بكلهم',
    'procletic': u'-بِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'بِكُلَّهُمْ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'بكلهن'] = {
    'word': u'بكلهن',
    'procletic': u'-بِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'بِكُلَّهُنَّ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'بكلنا'] = {
    'word': u'بكلنا',
    'procletic': u'-بِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'بِكُلَّنَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'نَا'
}
STOPWORDS[u'بكلكما'] = {
    'word': u'بكلكما',
    'procletic': u'-بِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'بِكُلَّكُمَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'بكلهما'] = {
    'word': u'بكلهما',
    'procletic': u'-بِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'بِكُلَّهُمَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ككلي'] = {
    'word': u'ككلي',
    'procletic': u'-كَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'كَكُلَِّي',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'ِي'
}
STOPWORDS[u'ككلك'] = {
    'word': u'ككلك',
    'procletic': u'-كَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'كَكُلَّكَ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كَ'
}
STOPWORDS[u'ككلك'] = {
    'word': u'ككلك',
    'procletic': u'-كَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'كَكُلَّكِ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كِ'
}
STOPWORDS[u'ككله'] = {
    'word': u'ككله',
    'procletic': u'-كَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'كَكُلَّه',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'ه'
}
STOPWORDS[u'ككلكم'] = {
    'word': u'ككلكم',
    'procletic': u'-كَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'كَكُلَّكُمْ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ككلكن'] = {
    'word': u'ككلكن',
    'procletic': u'-كَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'كَكُلَّكُنَّ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ككلها'] = {
    'word': u'ككلها',
    'procletic': u'-كَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'كَكُلَّهَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هَا'
}
STOPWORDS[u'ككلهم'] = {
    'word': u'ككلهم',
    'procletic': u'-كَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'كَكُلَّهُمْ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ككلهن'] = {
    'word': u'ككلهن',
    'procletic': u'-كَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'كَكُلَّهُنَّ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ككلنا'] = {
    'word': u'ككلنا',
    'procletic': u'-كَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'كَكُلَّنَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'نَا'
}
STOPWORDS[u'ككلكما'] = {
    'word': u'ككلكما',
    'procletic': u'-كَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'كَكُلَّكُمَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ككلهما'] = {
    'word': u'ككلهما',
    'procletic': u'-كَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'كَكُلَّهُمَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'لكلي'] = {
    'word': u'لكلي',
    'procletic': u'-لِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'لِكُلَِّي',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'ِي'
}
STOPWORDS[u'لكلك'] = {
    'word': u'لكلك',
    'procletic': u'-لِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'لِكُلَّكَ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كَ'
}
STOPWORDS[u'لكلك'] = {
    'word': u'لكلك',
    'procletic': u'-لِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'لِكُلَّكِ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كِ'
}
STOPWORDS[u'لكله'] = {
    'word': u'لكله',
    'procletic': u'-لِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'لِكُلَّه',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'ه'
}
STOPWORDS[u'لكلكم'] = {
    'word': u'لكلكم',
    'procletic': u'-لِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'لِكُلَّكُمْ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'لكلكن'] = {
    'word': u'لكلكن',
    'procletic': u'-لِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'لِكُلَّكُنَّ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'لكلها'] = {
    'word': u'لكلها',
    'procletic': u'-لِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'لِكُلَّهَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هَا'
}
STOPWORDS[u'لكلهم'] = {
    'word': u'لكلهم',
    'procletic': u'-لِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'لِكُلَّهُمْ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'لكلهن'] = {
    'word': u'لكلهن',
    'procletic': u'-لِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'لِكُلَّهُنَّ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'لكلنا'] = {
    'word': u'لكلنا',
    'procletic': u'-لِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'لِكُلَّنَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'نَا'
}
STOPWORDS[u'لكلكما'] = {
    'word': u'لكلكما',
    'procletic': u'-لِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'لِكُلَّكُمَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'لكلهما'] = {
    'word': u'لكلهما',
    'procletic': u'-لِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'لِكُلَّهُمَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'لكلي'] = {
    'word': u'لكلي',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَكُلَِّي',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'ِي'
}
STOPWORDS[u'لكلك'] = {
    'word': u'لكلك',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَكُلَّكَ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كَ'
}
STOPWORDS[u'لكلك'] = {
    'word': u'لكلك',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَكُلَّكِ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كِ'
}
STOPWORDS[u'لكله'] = {
    'word': u'لكله',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَكُلَّه',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'ه'
}
STOPWORDS[u'لكلكم'] = {
    'word': u'لكلكم',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَكُلَّكُمْ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'لكلكن'] = {
    'word': u'لكلكن',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَكُلَّكُنَّ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'لكلها'] = {
    'word': u'لكلها',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَكُلَّهَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هَا'
}
STOPWORDS[u'لكلهم'] = {
    'word': u'لكلهم',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَكُلَّهُمْ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'لكلهن'] = {
    'word': u'لكلهن',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَكُلَّهُنَّ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'لكلنا'] = {
    'word': u'لكلنا',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَكُلَّنَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'نَا'
}
STOPWORDS[u'لكلكما'] = {
    'word': u'لكلكما',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَكُلَّكُمَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'لكلهما'] = {
    'word': u'لكلهما',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَكُلَّهُمَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'وكلي'] = {
    'word': u'وكلي',
    'procletic': u'-وَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'وَكُلَِّي',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'ِي'
}
STOPWORDS[u'وكلك'] = {
    'word': u'وكلك',
    'procletic': u'-وَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'وَكُلَّكَ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كَ'
}
STOPWORDS[u'وكلك'] = {
    'word': u'وكلك',
    'procletic': u'-وَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'وَكُلَّكِ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كِ'
}
STOPWORDS[u'وكله'] = {
    'word': u'وكله',
    'procletic': u'-وَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'وَكُلَّه',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'ه'
}
STOPWORDS[u'وكلكم'] = {
    'word': u'وكلكم',
    'procletic': u'-وَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'وَكُلَّكُمْ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'وكلكن'] = {
    'word': u'وكلكن',
    'procletic': u'-وَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'وَكُلَّكُنَّ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'وكلها'] = {
    'word': u'وكلها',
    'procletic': u'-وَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'وَكُلَّهَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هَا'
}
STOPWORDS[u'وكلهم'] = {
    'word': u'وكلهم',
    'procletic': u'-وَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'وَكُلَّهُمْ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'وكلهن'] = {
    'word': u'وكلهن',
    'procletic': u'-وَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'وَكُلَّهُنَّ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'وكلنا'] = {
    'word': u'وكلنا',
    'procletic': u'-وَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'وَكُلَّنَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'نَا'
}
STOPWORDS[u'وكلكما'] = {
    'word': u'وكلكما',
    'procletic': u'-وَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'وَكُلَّكُمَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'وكلهما'] = {
    'word': u'وكلهما',
    'procletic': u'-وَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'وَكُلَّهُمَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فكلي'] = {
    'word': u'فكلي',
    'procletic': u'-فَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'فَكُلَِّي',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'ِي'
}
STOPWORDS[u'فكلك'] = {
    'word': u'فكلك',
    'procletic': u'-فَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'فَكُلَّكَ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كَ'
}
STOPWORDS[u'فكلك'] = {
    'word': u'فكلك',
    'procletic': u'-فَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'فَكُلَّكِ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كِ'
}
STOPWORDS[u'فكله'] = {
    'word': u'فكله',
    'procletic': u'-فَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'فَكُلَّه',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'ه'
}
STOPWORDS[u'فكلكم'] = {
    'word': u'فكلكم',
    'procletic': u'-فَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'فَكُلَّكُمْ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فكلكن'] = {
    'word': u'فكلكن',
    'procletic': u'-فَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'فَكُلَّكُنَّ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فكلها'] = {
    'word': u'فكلها',
    'procletic': u'-فَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'فَكُلَّهَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هَا'
}
STOPWORDS[u'فكلهم'] = {
    'word': u'فكلهم',
    'procletic': u'-فَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'فَكُلَّهُمْ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فكلهن'] = {
    'word': u'فكلهن',
    'procletic': u'-فَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'فَكُلَّهُنَّ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فكلنا'] = {
    'word': u'فكلنا',
    'procletic': u'-فَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'فَكُلَّنَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'نَا'
}
STOPWORDS[u'فكلكما'] = {
    'word': u'فكلكما',
    'procletic': u'-فَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'فَكُلَّكُمَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فكلهما'] = {
    'word': u'فكلهما',
    'procletic': u'-فَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'فَكُلَّهُمَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'وبكلي'] = {
    'word': u'وبكلي',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِكُلَِّي',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'ِي'
}
STOPWORDS[u'وبكلك'] = {
    'word': u'وبكلك',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِكُلَّكَ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كَ'
}
STOPWORDS[u'وبكلك'] = {
    'word': u'وبكلك',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِكُلَّكِ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كِ'
}
STOPWORDS[u'وبكله'] = {
    'word': u'وبكله',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِكُلَّه',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'ه'
}
STOPWORDS[u'وبكلكم'] = {
    'word': u'وبكلكم',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِكُلَّكُمْ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'وبكلكن'] = {
    'word': u'وبكلكن',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِكُلَّكُنَّ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'وبكلها'] = {
    'word': u'وبكلها',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِكُلَّهَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هَا'
}
STOPWORDS[u'وبكلهم'] = {
    'word': u'وبكلهم',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِكُلَّهُمْ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'وبكلهن'] = {
    'word': u'وبكلهن',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِكُلَّهُنَّ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'وبكلنا'] = {
    'word': u'وبكلنا',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِكُلَّنَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'نَا'
}
STOPWORDS[u'وبكلكما'] = {
    'word': u'وبكلكما',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِكُلَّكُمَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'وبكلهما'] = {
    'word': u'وبكلهما',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِكُلَّهُمَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فبكلي'] = {
    'word': u'فبكلي',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِكُلَِّي',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'ِي'
}
STOPWORDS[u'فبكلك'] = {
    'word': u'فبكلك',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِكُلَّكَ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كَ'
}
STOPWORDS[u'فبكلك'] = {
    'word': u'فبكلك',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِكُلَّكِ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كِ'
}
STOPWORDS[u'فبكله'] = {
    'word': u'فبكله',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِكُلَّه',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'ه'
}
STOPWORDS[u'فبكلكم'] = {
    'word': u'فبكلكم',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِكُلَّكُمْ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فبكلكن'] = {
    'word': u'فبكلكن',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِكُلَّكُنَّ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فبكلها'] = {
    'word': u'فبكلها',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِكُلَّهَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هَا'
}
STOPWORDS[u'فبكلهم'] = {
    'word': u'فبكلهم',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِكُلَّهُمْ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فبكلهن'] = {
    'word': u'فبكلهن',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِكُلَّهُنَّ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فبكلنا'] = {
    'word': u'فبكلنا',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِكُلَّنَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'نَا'
}
STOPWORDS[u'فبكلكما'] = {
    'word': u'فبكلكما',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِكُلَّكُمَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فبكلهما'] = {
    'word': u'فبكلهما',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِكُلَّهُمَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'وككلي'] = {
    'word': u'وككلي',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَكُلَِّي',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'ِي'
}
STOPWORDS[u'وككلك'] = {
    'word': u'وككلك',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَكُلَّكَ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كَ'
}
STOPWORDS[u'وككلك'] = {
    'word': u'وككلك',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَكُلَّكِ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كِ'
}
STOPWORDS[u'وككله'] = {
    'word': u'وككله',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَكُلَّه',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'ه'
}
STOPWORDS[u'وككلكم'] = {
    'word': u'وككلكم',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَكُلَّكُمْ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'وككلكن'] = {
    'word': u'وككلكن',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَكُلَّكُنَّ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'وككلها'] = {
    'word': u'وككلها',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَكُلَّهَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هَا'
}
STOPWORDS[u'وككلهم'] = {
    'word': u'وككلهم',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَكُلَّهُمْ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'وككلهن'] = {
    'word': u'وككلهن',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَكُلَّهُنَّ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'وككلنا'] = {
    'word': u'وككلنا',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَكُلَّنَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'نَا'
}
STOPWORDS[u'وككلكما'] = {
    'word': u'وككلكما',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَكُلَّكُمَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'وككلهما'] = {
    'word': u'وككلهما',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَكُلَّهُمَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فككلي'] = {
    'word': u'فككلي',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَكُلَِّي',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'ِي'
}
STOPWORDS[u'فككلك'] = {
    'word': u'فككلك',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَكُلَّكَ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كَ'
}
STOPWORDS[u'فككلك'] = {
    'word': u'فككلك',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَكُلَّكِ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كِ'
}
STOPWORDS[u'فككله'] = {
    'word': u'فككله',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَكُلَّه',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'ه'
}
STOPWORDS[u'فككلكم'] = {
    'word': u'فككلكم',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَكُلَّكُمْ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فككلكن'] = {
    'word': u'فككلكن',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَكُلَّكُنَّ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فككلها'] = {
    'word': u'فككلها',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَكُلَّهَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هَا'
}
STOPWORDS[u'فككلهم'] = {
    'word': u'فككلهم',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَكُلَّهُمْ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فككلهن'] = {
    'word': u'فككلهن',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَكُلَّهُنَّ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فككلنا'] = {
    'word': u'فككلنا',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَكُلَّنَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'نَا'
}
STOPWORDS[u'فككلكما'] = {
    'word': u'فككلكما',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَكُلَّكُمَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فككلهما'] = {
    'word': u'فككلهما',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَكُلَّهُمَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ولكلي'] = {
    'word': u'ولكلي',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِكُلَِّي',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'ِي'
}
STOPWORDS[u'ولكلك'] = {
    'word': u'ولكلك',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِكُلَّكَ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كَ'
}
STOPWORDS[u'ولكلك'] = {
    'word': u'ولكلك',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِكُلَّكِ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كِ'
}
STOPWORDS[u'ولكله'] = {
    'word': u'ولكله',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِكُلَّه',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'ه'
}
STOPWORDS[u'ولكلكم'] = {
    'word': u'ولكلكم',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِكُلَّكُمْ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ولكلكن'] = {
    'word': u'ولكلكن',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِكُلَّكُنَّ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ولكلها'] = {
    'word': u'ولكلها',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِكُلَّهَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هَا'
}
STOPWORDS[u'ولكلهم'] = {
    'word': u'ولكلهم',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِكُلَّهُمْ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ولكلهن'] = {
    'word': u'ولكلهن',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِكُلَّهُنَّ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ولكلنا'] = {
    'word': u'ولكلنا',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِكُلَّنَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'نَا'
}
STOPWORDS[u'ولكلكما'] = {
    'word': u'ولكلكما',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِكُلَّكُمَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ولكلهما'] = {
    'word': u'ولكلهما',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِكُلَّهُمَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فلكلي'] = {
    'word': u'فلكلي',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِكُلَِّي',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'ِي'
}
STOPWORDS[u'فلكلك'] = {
    'word': u'فلكلك',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِكُلَّكَ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كَ'
}
STOPWORDS[u'فلكلك'] = {
    'word': u'فلكلك',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِكُلَّكِ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كِ'
}
STOPWORDS[u'فلكله'] = {
    'word': u'فلكله',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِكُلَّه',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'ه'
}
STOPWORDS[u'فلكلكم'] = {
    'word': u'فلكلكم',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِكُلَّكُمْ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فلكلكن'] = {
    'word': u'فلكلكن',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِكُلَّكُنَّ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فلكلها'] = {
    'word': u'فلكلها',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِكُلَّهَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هَا'
}
STOPWORDS[u'فلكلهم'] = {
    'word': u'فلكلهم',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِكُلَّهُمْ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فلكلهن'] = {
    'word': u'فلكلهن',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِكُلَّهُنَّ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فلكلنا'] = {
    'word': u'فلكلنا',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِكُلَّنَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'نَا'
}
STOPWORDS[u'فلكلكما'] = {
    'word': u'فلكلكما',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِكُلَّكُمَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فلكلهما'] = {
    'word': u'فلكلهما',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِكُلَّهُمَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ولكلي'] = {
    'word': u'ولكلي',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَكُلَِّي',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'ِي'
}
STOPWORDS[u'ولكلك'] = {
    'word': u'ولكلك',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَكُلَّكَ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كَ'
}
STOPWORDS[u'ولكلك'] = {
    'word': u'ولكلك',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَكُلَّكِ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كِ'
}
STOPWORDS[u'ولكله'] = {
    'word': u'ولكله',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَكُلَّه',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'ه'
}
STOPWORDS[u'ولكلكم'] = {
    'word': u'ولكلكم',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَكُلَّكُمْ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ولكلكن'] = {
    'word': u'ولكلكن',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَكُلَّكُنَّ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ولكلها'] = {
    'word': u'ولكلها',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَكُلَّهَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هَا'
}
STOPWORDS[u'ولكلهم'] = {
    'word': u'ولكلهم',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَكُلَّهُمْ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ولكلهن'] = {
    'word': u'ولكلهن',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَكُلَّهُنَّ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ولكلنا'] = {
    'word': u'ولكلنا',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَكُلَّنَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'نَا'
}
STOPWORDS[u'ولكلكما'] = {
    'word': u'ولكلكما',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَكُلَّكُمَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ولكلهما'] = {
    'word': u'ولكلهما',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَكُلَّهُمَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فلكلي'] = {
    'word': u'فلكلي',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَكُلَِّي',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'ِي'
}
STOPWORDS[u'فلكلك'] = {
    'word': u'فلكلك',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَكُلَّكَ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كَ'
}
STOPWORDS[u'فلكلك'] = {
    'word': u'فلكلك',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَكُلَّكِ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كِ'
}
STOPWORDS[u'فلكله'] = {
    'word': u'فلكله',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَكُلَّه',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'ه'
}
STOPWORDS[u'فلكلكم'] = {
    'word': u'فلكلكم',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَكُلَّكُمْ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فلكلكن'] = {
    'word': u'فلكلكن',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَكُلَّكُنَّ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فلكلها'] = {
    'word': u'فلكلها',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَكُلَّهَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هَا'
}
STOPWORDS[u'فلكلهم'] = {
    'word': u'فلكلهم',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَكُلَّهُمْ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فلكلهن'] = {
    'word': u'فلكلهن',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَكُلَّهُنَّ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فلكلنا'] = {
    'word': u'فلكلنا',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَكُلَّنَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'نَا'
}
STOPWORDS[u'فلكلكما'] = {
    'word': u'فلكلكما',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَكُلَّكُمَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فلكلهما'] = {
    'word': u'فلكلهما',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَكُلَّهُمَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أكلي'] = {
    'word': u'أكلي',
    'procletic': u'-أَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'أَكُلَِّي',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'ِي'
}
STOPWORDS[u'أكلك'] = {
    'word': u'أكلك',
    'procletic': u'-أَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'أَكُلَّكَ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كَ'
}
STOPWORDS[u'أكلك'] = {
    'word': u'أكلك',
    'procletic': u'-أَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'أَكُلَّكِ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كِ'
}
STOPWORDS[u'أكله'] = {
    'word': u'أكله',
    'procletic': u'-أَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'أَكُلَّه',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'ه'
}
STOPWORDS[u'أكلكم'] = {
    'word': u'أكلكم',
    'procletic': u'-أَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'أَكُلَّكُمْ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أكلكن'] = {
    'word': u'أكلكن',
    'procletic': u'-أَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'أَكُلَّكُنَّ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أكلها'] = {
    'word': u'أكلها',
    'procletic': u'-أَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'أَكُلَّهَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هَا'
}
STOPWORDS[u'أكلهم'] = {
    'word': u'أكلهم',
    'procletic': u'-أَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'أَكُلَّهُمْ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أكلهن'] = {
    'word': u'أكلهن',
    'procletic': u'-أَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'أَكُلَّهُنَّ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أكلنا'] = {
    'word': u'أكلنا',
    'procletic': u'-أَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'أَكُلَّنَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'نَا'
}
STOPWORDS[u'أكلكما'] = {
    'word': u'أكلكما',
    'procletic': u'-أَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'أَكُلَّكُمَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أكلهما'] = {
    'word': u'أكلهما',
    'procletic': u'-أَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'أَكُلَّهُمَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أبكلي'] = {
    'word': u'أبكلي',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِكُلَِّي',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'ِي'
}
STOPWORDS[u'أبكلك'] = {
    'word': u'أبكلك',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِكُلَّكَ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كَ'
}
STOPWORDS[u'أبكلك'] = {
    'word': u'أبكلك',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِكُلَّكِ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كِ'
}
STOPWORDS[u'أبكله'] = {
    'word': u'أبكله',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِكُلَّه',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'ه'
}
STOPWORDS[u'أبكلكم'] = {
    'word': u'أبكلكم',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِكُلَّكُمْ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أبكلكن'] = {
    'word': u'أبكلكن',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِكُلَّكُنَّ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أبكلها'] = {
    'word': u'أبكلها',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِكُلَّهَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هَا'
}
STOPWORDS[u'أبكلهم'] = {
    'word': u'أبكلهم',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِكُلَّهُمْ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أبكلهن'] = {
    'word': u'أبكلهن',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِكُلَّهُنَّ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أبكلنا'] = {
    'word': u'أبكلنا',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِكُلَّنَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'نَا'
}
STOPWORDS[u'أبكلكما'] = {
    'word': u'أبكلكما',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِكُلَّكُمَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أبكلهما'] = {
    'word': u'أبكلهما',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِكُلَّهُمَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أككلي'] = {
    'word': u'أككلي',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَكُلَِّي',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'ِي'
}
STOPWORDS[u'أككلك'] = {
    'word': u'أككلك',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَكُلَّكَ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كَ'
}
STOPWORDS[u'أككلك'] = {
    'word': u'أككلك',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَكُلَّكِ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كِ'
}
STOPWORDS[u'أككله'] = {
    'word': u'أككله',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَكُلَّه',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'ه'
}
STOPWORDS[u'أككلكم'] = {
    'word': u'أككلكم',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَكُلَّكُمْ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أككلكن'] = {
    'word': u'أككلكن',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَكُلَّكُنَّ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أككلها'] = {
    'word': u'أككلها',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَكُلَّهَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هَا'
}
STOPWORDS[u'أككلهم'] = {
    'word': u'أككلهم',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَكُلَّهُمْ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أككلهن'] = {
    'word': u'أككلهن',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَكُلَّهُنَّ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أككلنا'] = {
    'word': u'أككلنا',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَكُلَّنَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'نَا'
}
STOPWORDS[u'أككلكما'] = {
    'word': u'أككلكما',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَكُلَّكُمَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أككلهما'] = {
    'word': u'أككلهما',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَكُلَّهُمَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ألكلي'] = {
    'word': u'ألكلي',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِكُلَِّي',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'ِي'
}
STOPWORDS[u'ألكلك'] = {
    'word': u'ألكلك',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِكُلَّكَ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كَ'
}
STOPWORDS[u'ألكلك'] = {
    'word': u'ألكلك',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِكُلَّكِ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كِ'
}
STOPWORDS[u'ألكله'] = {
    'word': u'ألكله',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِكُلَّه',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'ه'
}
STOPWORDS[u'ألكلكم'] = {
    'word': u'ألكلكم',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِكُلَّكُمْ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ألكلكن'] = {
    'word': u'ألكلكن',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِكُلَّكُنَّ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ألكلها'] = {
    'word': u'ألكلها',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِكُلَّهَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هَا'
}
STOPWORDS[u'ألكلهم'] = {
    'word': u'ألكلهم',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِكُلَّهُمْ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ألكلهن'] = {
    'word': u'ألكلهن',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِكُلَّهُنَّ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ألكلنا'] = {
    'word': u'ألكلنا',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِكُلَّنَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'نَا'
}
STOPWORDS[u'ألكلكما'] = {
    'word': u'ألكلكما',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِكُلَّكُمَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ألكلهما'] = {
    'word': u'ألكلهما',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِكُلَّهُمَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ألكلي'] = {
    'word': u'ألكلي',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَكُلَِّي',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'ِي'
}
STOPWORDS[u'ألكلك'] = {
    'word': u'ألكلك',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَكُلَّكَ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كَ'
}
STOPWORDS[u'ألكلك'] = {
    'word': u'ألكلك',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَكُلَّكِ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كِ'
}
STOPWORDS[u'ألكله'] = {
    'word': u'ألكله',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَكُلَّه',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'ه'
}
STOPWORDS[u'ألكلكم'] = {
    'word': u'ألكلكم',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَكُلَّكُمْ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ألكلكن'] = {
    'word': u'ألكلكن',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَكُلَّكُنَّ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ألكلها'] = {
    'word': u'ألكلها',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَكُلَّهَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هَا'
}
STOPWORDS[u'ألكلهم'] = {
    'word': u'ألكلهم',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَكُلَّهُمْ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ألكلهن'] = {
    'word': u'ألكلهن',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَكُلَّهُنَّ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ألكلنا'] = {
    'word': u'ألكلنا',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَكُلَّنَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'نَا'
}
STOPWORDS[u'ألكلكما'] = {
    'word': u'ألكلكما',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَكُلَّكُمَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ألكلهما'] = {
    'word': u'ألكلهما',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَكُلَّهُمَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أوكلي'] = {
    'word': u'أوكلي',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكُلَِّي',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'ِي'
}
STOPWORDS[u'أوكلك'] = {
    'word': u'أوكلك',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكُلَّكَ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كَ'
}
STOPWORDS[u'أوكلك'] = {
    'word': u'أوكلك',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكُلَّكِ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كِ'
}
STOPWORDS[u'أوكله'] = {
    'word': u'أوكله',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكُلَّه',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'ه'
}
STOPWORDS[u'أوكلكم'] = {
    'word': u'أوكلكم',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكُلَّكُمْ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أوكلكن'] = {
    'word': u'أوكلكن',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكُلَّكُنَّ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أوكلها'] = {
    'word': u'أوكلها',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكُلَّهَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هَا'
}
STOPWORDS[u'أوكلهم'] = {
    'word': u'أوكلهم',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكُلَّهُمْ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أوكلهن'] = {
    'word': u'أوكلهن',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكُلَّهُنَّ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أوكلنا'] = {
    'word': u'أوكلنا',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكُلَّنَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'نَا'
}
STOPWORDS[u'أوكلكما'] = {
    'word': u'أوكلكما',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكُلَّكُمَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أوكلهما'] = {
    'word': u'أوكلهما',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكُلَّهُمَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أفكلي'] = {
    'word': u'أفكلي',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكُلَِّي',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'ِي'
}
STOPWORDS[u'أفكلك'] = {
    'word': u'أفكلك',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكُلَّكَ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كَ'
}
STOPWORDS[u'أفكلك'] = {
    'word': u'أفكلك',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكُلَّكِ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كِ'
}
STOPWORDS[u'أفكله'] = {
    'word': u'أفكله',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكُلَّه',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'ه'
}
STOPWORDS[u'أفكلكم'] = {
    'word': u'أفكلكم',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكُلَّكُمْ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أفكلكن'] = {
    'word': u'أفكلكن',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكُلَّكُنَّ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أفكلها'] = {
    'word': u'أفكلها',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكُلَّهَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هَا'
}
STOPWORDS[u'أفكلهم'] = {
    'word': u'أفكلهم',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكُلَّهُمْ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أفكلهن'] = {
    'word': u'أفكلهن',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكُلَّهُنَّ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أفكلنا'] = {
    'word': u'أفكلنا',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكُلَّنَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'نَا'
}
STOPWORDS[u'أفكلكما'] = {
    'word': u'أفكلكما',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكُلَّكُمَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أفكلهما'] = {
    'word': u'أفكلهما',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكُلَّهُمَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أوبكلي'] = {
    'word': u'أوبكلي',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِكُلَِّي',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'ِي'
}
STOPWORDS[u'أوبكلك'] = {
    'word': u'أوبكلك',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِكُلَّكَ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كَ'
}
STOPWORDS[u'أوبكلك'] = {
    'word': u'أوبكلك',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِكُلَّكِ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كِ'
}
STOPWORDS[u'أوبكله'] = {
    'word': u'أوبكله',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِكُلَّه',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'ه'
}
STOPWORDS[u'أوبكلكم'] = {
    'word': u'أوبكلكم',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِكُلَّكُمْ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أوبكلكن'] = {
    'word': u'أوبكلكن',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِكُلَّكُنَّ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أوبكلها'] = {
    'word': u'أوبكلها',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِكُلَّهَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هَا'
}
STOPWORDS[u'أوبكلهم'] = {
    'word': u'أوبكلهم',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِكُلَّهُمْ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أوبكلهن'] = {
    'word': u'أوبكلهن',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِكُلَّهُنَّ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أوبكلنا'] = {
    'word': u'أوبكلنا',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِكُلَّنَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'نَا'
}
STOPWORDS[u'أوبكلكما'] = {
    'word': u'أوبكلكما',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِكُلَّكُمَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أوبكلهما'] = {
    'word': u'أوبكلهما',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِكُلَّهُمَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أفبكلي'] = {
    'word': u'أفبكلي',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِكُلَِّي',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'ِي'
}
STOPWORDS[u'أفبكلك'] = {
    'word': u'أفبكلك',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِكُلَّكَ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كَ'
}
STOPWORDS[u'أفبكلك'] = {
    'word': u'أفبكلك',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِكُلَّكِ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كِ'
}
STOPWORDS[u'أفبكله'] = {
    'word': u'أفبكله',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِكُلَّه',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'ه'
}
STOPWORDS[u'أفبكلكم'] = {
    'word': u'أفبكلكم',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِكُلَّكُمْ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أفبكلكن'] = {
    'word': u'أفبكلكن',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِكُلَّكُنَّ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أفبكلها'] = {
    'word': u'أفبكلها',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِكُلَّهَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هَا'
}
STOPWORDS[u'أفبكلهم'] = {
    'word': u'أفبكلهم',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِكُلَّهُمْ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أفبكلهن'] = {
    'word': u'أفبكلهن',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِكُلَّهُنَّ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أفبكلنا'] = {
    'word': u'أفبكلنا',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِكُلَّنَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'نَا'
}
STOPWORDS[u'أفبكلكما'] = {
    'word': u'أفبكلكما',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِكُلَّكُمَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أفبكلهما'] = {
    'word': u'أفبكلهما',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِكُلَّهُمَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أوككلي'] = {
    'word': u'أوككلي',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَكُلَِّي',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'ِي'
}
STOPWORDS[u'أوككلك'] = {
    'word': u'أوككلك',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَكُلَّكَ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كَ'
}
STOPWORDS[u'أوككلك'] = {
    'word': u'أوككلك',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَكُلَّكِ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كِ'
}
STOPWORDS[u'أوككله'] = {
    'word': u'أوككله',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَكُلَّه',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'ه'
}
STOPWORDS[u'أوككلكم'] = {
    'word': u'أوككلكم',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَكُلَّكُمْ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أوككلكن'] = {
    'word': u'أوككلكن',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَكُلَّكُنَّ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أوككلها'] = {
    'word': u'أوككلها',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَكُلَّهَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هَا'
}
STOPWORDS[u'أوككلهم'] = {
    'word': u'أوككلهم',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَكُلَّهُمْ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أوككلهن'] = {
    'word': u'أوككلهن',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَكُلَّهُنَّ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أوككلنا'] = {
    'word': u'أوككلنا',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَكُلَّنَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'نَا'
}
STOPWORDS[u'أوككلكما'] = {
    'word': u'أوككلكما',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَكُلَّكُمَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أوككلهما'] = {
    'word': u'أوككلهما',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَكُلَّهُمَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أفككلي'] = {
    'word': u'أفككلي',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَكُلَِّي',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'ِي'
}
STOPWORDS[u'أفككلك'] = {
    'word': u'أفككلك',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَكُلَّكَ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كَ'
}
STOPWORDS[u'أفككلك'] = {
    'word': u'أفككلك',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَكُلَّكِ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كِ'
}
STOPWORDS[u'أفككله'] = {
    'word': u'أفككله',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَكُلَّه',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'ه'
}
STOPWORDS[u'أفككلكم'] = {
    'word': u'أفككلكم',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَكُلَّكُمْ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أفككلكن'] = {
    'word': u'أفككلكن',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَكُلَّكُنَّ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أفككلها'] = {
    'word': u'أفككلها',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَكُلَّهَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هَا'
}
STOPWORDS[u'أفككلهم'] = {
    'word': u'أفككلهم',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَكُلَّهُمْ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أفككلهن'] = {
    'word': u'أفككلهن',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَكُلَّهُنَّ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أفككلنا'] = {
    'word': u'أفككلنا',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَكُلَّنَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'نَا'
}
STOPWORDS[u'أفككلكما'] = {
    'word': u'أفككلكما',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَكُلَّكُمَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أفككلهما'] = {
    'word': u'أفككلهما',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَكُلَّهُمَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أولكلي'] = {
    'word': u'أولكلي',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِكُلَِّي',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'ِي'
}
STOPWORDS[u'أولكلك'] = {
    'word': u'أولكلك',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِكُلَّكَ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كَ'
}
STOPWORDS[u'أولكلك'] = {
    'word': u'أولكلك',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِكُلَّكِ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كِ'
}
STOPWORDS[u'أولكله'] = {
    'word': u'أولكله',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِكُلَّه',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'ه'
}
STOPWORDS[u'أولكلكم'] = {
    'word': u'أولكلكم',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِكُلَّكُمْ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أولكلكن'] = {
    'word': u'أولكلكن',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِكُلَّكُنَّ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أولكلها'] = {
    'word': u'أولكلها',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِكُلَّهَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هَا'
}
STOPWORDS[u'أولكلهم'] = {
    'word': u'أولكلهم',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِكُلَّهُمْ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أولكلهن'] = {
    'word': u'أولكلهن',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِكُلَّهُنَّ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أولكلنا'] = {
    'word': u'أولكلنا',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِكُلَّنَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'نَا'
}
STOPWORDS[u'أولكلكما'] = {
    'word': u'أولكلكما',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِكُلَّكُمَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أولكلهما'] = {
    'word': u'أولكلهما',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِكُلَّهُمَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أفلكلي'] = {
    'word': u'أفلكلي',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِكُلَِّي',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'ِي'
}
STOPWORDS[u'أفلكلك'] = {
    'word': u'أفلكلك',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِكُلَّكَ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كَ'
}
STOPWORDS[u'أفلكلك'] = {
    'word': u'أفلكلك',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِكُلَّكِ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كِ'
}
STOPWORDS[u'أفلكله'] = {
    'word': u'أفلكله',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِكُلَّه',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'ه'
}
STOPWORDS[u'أفلكلكم'] = {
    'word': u'أفلكلكم',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِكُلَّكُمْ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أفلكلكن'] = {
    'word': u'أفلكلكن',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِكُلَّكُنَّ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أفلكلها'] = {
    'word': u'أفلكلها',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِكُلَّهَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هَا'
}
STOPWORDS[u'أفلكلهم'] = {
    'word': u'أفلكلهم',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِكُلَّهُمْ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أفلكلهن'] = {
    'word': u'أفلكلهن',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِكُلَّهُنَّ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أفلكلنا'] = {
    'word': u'أفلكلنا',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِكُلَّنَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'نَا'
}
STOPWORDS[u'أفلكلكما'] = {
    'word': u'أفلكلكما',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِكُلَّكُمَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أفلكلهما'] = {
    'word': u'أفلكلهما',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِكُلَّهُمَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أولكلي'] = {
    'word': u'أولكلي',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَكُلَِّي',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'ِي'
}
STOPWORDS[u'أولكلك'] = {
    'word': u'أولكلك',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَكُلَّكَ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كَ'
}
STOPWORDS[u'أولكلك'] = {
    'word': u'أولكلك',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَكُلَّكِ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كِ'
}
STOPWORDS[u'أولكله'] = {
    'word': u'أولكله',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَكُلَّه',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'ه'
}
STOPWORDS[u'أولكلكم'] = {
    'word': u'أولكلكم',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَكُلَّكُمْ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أولكلكن'] = {
    'word': u'أولكلكن',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَكُلَّكُنَّ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أولكلها'] = {
    'word': u'أولكلها',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَكُلَّهَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هَا'
}
STOPWORDS[u'أولكلهم'] = {
    'word': u'أولكلهم',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَكُلَّهُمْ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أولكلهن'] = {
    'word': u'أولكلهن',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَكُلَّهُنَّ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أولكلنا'] = {
    'word': u'أولكلنا',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَكُلَّنَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'نَا'
}
STOPWORDS[u'أولكلكما'] = {
    'word': u'أولكلكما',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَكُلَّكُمَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أولكلهما'] = {
    'word': u'أولكلهما',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَكُلَّهُمَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أفلكلي'] = {
    'word': u'أفلكلي',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَكُلَِّي',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'ِي'
}
STOPWORDS[u'أفلكلك'] = {
    'word': u'أفلكلك',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَكُلَّكَ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كَ'
}
STOPWORDS[u'أفلكلك'] = {
    'word': u'أفلكلك',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَكُلَّكِ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كِ'
}
STOPWORDS[u'أفلكله'] = {
    'word': u'أفلكله',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَكُلَّه',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'ه'
}
STOPWORDS[u'أفلكلكم'] = {
    'word': u'أفلكلكم',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَكُلَّكُمْ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أفلكلكن'] = {
    'word': u'أفلكلكن',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَكُلَّكُنَّ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أفلكلها'] = {
    'word': u'أفلكلها',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَكُلَّهَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هَا'
}
STOPWORDS[u'أفلكلهم'] = {
    'word': u'أفلكلهم',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَكُلَّهُمْ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أفلكلهن'] = {
    'word': u'أفلكلهن',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَكُلَّهُنَّ',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أفلكلنا'] = {
    'word': u'أفلكلنا',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَكُلَّنَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'نَا'
}
STOPWORDS[u'أفلكلكما'] = {
    'word': u'أفلكلكما',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَكُلَّكُمَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أفلكلهما'] = {
    'word': u'أفلكلهما',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَكُلَّهُمَا',
    'stem': u'كُلَّ',
    'type': u'STOPWORD',
    'original': u'كُلَّ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'كلا'] = {
    'word': u'كِلَا',
    'procletic': u'',
    'tags': u'أداة:توكيد',
    'vocalized': u'كِلَا',
    'stem': u'كِلَا',
    'type': u'STOPWORD',
    'original': u'كِلَا',
    'encletic': u''
}
STOPWORDS[u'وكلا'] = {
    'word': u'وكلا',
    'procletic': u'-وَ',
    'tags': u'أداة:توكيد:معطوف',
    'vocalized': u'وَكِلَا',
    'stem': u'كِلَا',
    'type': u'STOPWORD',
    'original': u'كِلَا',
    'encletic': u''
}
STOPWORDS[u'فكلا'] = {
    'word': u'فكلا',
    'procletic': u'-فَ',
    'tags': u'أداة:توكيد:معطوف',
    'vocalized': u'فَكِلَا',
    'stem': u'كِلَا',
    'type': u'STOPWORD',
    'original': u'كِلَا',
    'encletic': u''
}
STOPWORDS[u'أكلا'] = {
    'word': u'أكلا',
    'procletic': u'-أَ',
    'tags': u'أداة:توكيد:مجرور',
    'vocalized': u'أَكِلَا',
    'stem': u'كِلَا',
    'type': u'STOPWORD',
    'original': u'كِلَا',
    'encletic': u''
}
STOPWORDS[u'أوكلا'] = {
    'word': u'أوكلا',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور',
    'vocalized': u'أَوَكِلَا',
    'stem': u'كِلَا',
    'type': u'STOPWORD',
    'original': u'كِلَا',
    'encletic': u''
}
STOPWORDS[u'أفكلا'] = {
    'word': u'أفكلا',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور',
    'vocalized': u'أَفَكِلَا',
    'stem': u'كِلَا',
    'type': u'STOPWORD',
    'original': u'كِلَا',
    'encletic': u''
}
STOPWORDS[u'كلاهما'] = {
    'word': u'كِلَاهُمَا',
    'procletic': u'',
    'tags': u'أداة:توكيد',
    'vocalized': u'كِلَاهُمَا',
    'stem': u'كِلَاهُمَا',
    'type': u'STOPWORD',
    'original': u'كِلَاهُمَا',
    'encletic': u''
}
STOPWORDS[u'وكلاهما'] = {
    'word': u'وكلاهما',
    'procletic': u'-وَ',
    'tags': u'أداة:توكيد:معطوف',
    'vocalized': u'وَكِلَاهُمَا',
    'stem': u'كِلَاهُمَا',
    'type': u'STOPWORD',
    'original': u'كِلَاهُمَا',
    'encletic': u''
}
STOPWORDS[u'فكلاهما'] = {
    'word': u'فكلاهما',
    'procletic': u'-فَ',
    'tags': u'أداة:توكيد:معطوف',
    'vocalized': u'فَكِلَاهُمَا',
    'stem': u'كِلَاهُمَا',
    'type': u'STOPWORD',
    'original': u'كِلَاهُمَا',
    'encletic': u''
}
STOPWORDS[u'أكلاهما'] = {
    'word': u'أكلاهما',
    'procletic': u'-أَ',
    'tags': u'أداة:توكيد:مجرور',
    'vocalized': u'أَكِلَاهُمَا',
    'stem': u'كِلَاهُمَا',
    'type': u'STOPWORD',
    'original': u'كِلَاهُمَا',
    'encletic': u''
}
STOPWORDS[u'أوكلاهما'] = {
    'word': u'أوكلاهما',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور',
    'vocalized': u'أَوَكِلَاهُمَا',
    'stem': u'كِلَاهُمَا',
    'type': u'STOPWORD',
    'original': u'كِلَاهُمَا',
    'encletic': u''
}
STOPWORDS[u'أفكلاهما'] = {
    'word': u'أفكلاهما',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور',
    'vocalized': u'أَفَكِلَاهُمَا',
    'stem': u'كِلَاهُمَا',
    'type': u'STOPWORD',
    'original': u'كِلَاهُمَا',
    'encletic': u''
}
STOPWORDS[u'كلتا'] = {
    'word': u'كِلْتَا',
    'procletic': u'',
    'tags': u'أداة:توكيد',
    'vocalized': u'كِلْتَا',
    'stem': u'كِلْتَا',
    'type': u'STOPWORD',
    'original': u'كِلْتَا',
    'encletic': u''
}
STOPWORDS[u'بكلتا'] = {
    'word': u'بكلتا',
    'procletic': u'-بِ',
    'tags': u'أداة:توكيد:مجرور',
    'vocalized': u'بِكِلْتَا',
    'stem': u'كِلْتَا',
    'type': u'STOPWORD',
    'original': u'كِلْتَا',
    'encletic': u''
}
STOPWORDS[u'ككلتا'] = {
    'word': u'ككلتا',
    'procletic': u'-كَ',
    'tags': u'أداة:توكيد:مجرور',
    'vocalized': u'كَكِلْتَا',
    'stem': u'كِلْتَا',
    'type': u'STOPWORD',
    'original': u'كِلْتَا',
    'encletic': u''
}
STOPWORDS[u'لكلتا'] = {
    'word': u'لكلتا',
    'procletic': u'-لِ',
    'tags': u'أداة:توكيد:مجرور',
    'vocalized': u'لِكِلْتَا',
    'stem': u'كِلْتَا',
    'type': u'STOPWORD',
    'original': u'كِلْتَا',
    'encletic': u''
}
STOPWORDS[u'لكلتا'] = {
    'word': u'لكلتا',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور',
    'vocalized': u'لَكِلْتَا',
    'stem': u'كِلْتَا',
    'type': u'STOPWORD',
    'original': u'كِلْتَا',
    'encletic': u''
}
STOPWORDS[u'وكلتا'] = {
    'word': u'وكلتا',
    'procletic': u'-وَ',
    'tags': u'أداة:توكيد:معطوف',
    'vocalized': u'وَكِلْتَا',
    'stem': u'كِلْتَا',
    'type': u'STOPWORD',
    'original': u'كِلْتَا',
    'encletic': u''
}
STOPWORDS[u'فكلتا'] = {
    'word': u'فكلتا',
    'procletic': u'-فَ',
    'tags': u'أداة:توكيد:معطوف',
    'vocalized': u'فَكِلْتَا',
    'stem': u'كِلْتَا',
    'type': u'STOPWORD',
    'original': u'كِلْتَا',
    'encletic': u''
}
STOPWORDS[u'وبكلتا'] = {
    'word': u'وبكلتا',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف',
    'vocalized': u'وَبِكِلْتَا',
    'stem': u'كِلْتَا',
    'type': u'STOPWORD',
    'original': u'كِلْتَا',
    'encletic': u''
}
STOPWORDS[u'فبكلتا'] = {
    'word': u'فبكلتا',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف',
    'vocalized': u'فَبِكِلْتَا',
    'stem': u'كِلْتَا',
    'type': u'STOPWORD',
    'original': u'كِلْتَا',
    'encletic': u''
}
STOPWORDS[u'وككلتا'] = {
    'word': u'وككلتا',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف',
    'vocalized': u'وَكَكِلْتَا',
    'stem': u'كِلْتَا',
    'type': u'STOPWORD',
    'original': u'كِلْتَا',
    'encletic': u''
}
STOPWORDS[u'فككلتا'] = {
    'word': u'فككلتا',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف',
    'vocalized': u'فَكَكِلْتَا',
    'stem': u'كِلْتَا',
    'type': u'STOPWORD',
    'original': u'كِلْتَا',
    'encletic': u''
}
STOPWORDS[u'ولكلتا'] = {
    'word': u'ولكلتا',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف',
    'vocalized': u'وَلِكِلْتَا',
    'stem': u'كِلْتَا',
    'type': u'STOPWORD',
    'original': u'كِلْتَا',
    'encletic': u''
}
STOPWORDS[u'فلكلتا'] = {
    'word': u'فلكلتا',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف',
    'vocalized': u'فَلِكِلْتَا',
    'stem': u'كِلْتَا',
    'type': u'STOPWORD',
    'original': u'كِلْتَا',
    'encletic': u''
}
STOPWORDS[u'ولكلتا'] = {
    'word': u'ولكلتا',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'وَلَكِلْتَا',
    'stem': u'كِلْتَا',
    'type': u'STOPWORD',
    'original': u'كِلْتَا',
    'encletic': u''
}
STOPWORDS[u'فلكلتا'] = {
    'word': u'فلكلتا',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'فَلَكِلْتَا',
    'stem': u'كِلْتَا',
    'type': u'STOPWORD',
    'original': u'كِلْتَا',
    'encletic': u''
}
STOPWORDS[u'أكلتا'] = {
    'word': u'أكلتا',
    'procletic': u'-أَ',
    'tags': u'أداة:توكيد:مجرور',
    'vocalized': u'أَكِلْتَا',
    'stem': u'كِلْتَا',
    'type': u'STOPWORD',
    'original': u'كِلْتَا',
    'encletic': u''
}
STOPWORDS[u'أبكلتا'] = {
    'word': u'أبكلتا',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور',
    'vocalized': u'أَبِكِلْتَا',
    'stem': u'كِلْتَا',
    'type': u'STOPWORD',
    'original': u'كِلْتَا',
    'encletic': u''
}
STOPWORDS[u'أككلتا'] = {
    'word': u'أككلتا',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور',
    'vocalized': u'أَكَكِلْتَا',
    'stem': u'كِلْتَا',
    'type': u'STOPWORD',
    'original': u'كِلْتَا',
    'encletic': u''
}
STOPWORDS[u'ألكلتا'] = {
    'word': u'ألكلتا',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور',
    'vocalized': u'أَلِكِلْتَا',
    'stem': u'كِلْتَا',
    'type': u'STOPWORD',
    'original': u'كِلْتَا',
    'encletic': u''
}
STOPWORDS[u'ألكلتا'] = {
    'word': u'ألكلتا',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور',
    'vocalized': u'أَلَكِلْتَا',
    'stem': u'كِلْتَا',
    'type': u'STOPWORD',
    'original': u'كِلْتَا',
    'encletic': u''
}
STOPWORDS[u'أوكلتا'] = {
    'word': u'أوكلتا',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور',
    'vocalized': u'أَوَكِلْتَا',
    'stem': u'كِلْتَا',
    'type': u'STOPWORD',
    'original': u'كِلْتَا',
    'encletic': u''
}
STOPWORDS[u'أفكلتا'] = {
    'word': u'أفكلتا',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور',
    'vocalized': u'أَفَكِلْتَا',
    'stem': u'كِلْتَا',
    'type': u'STOPWORD',
    'original': u'كِلْتَا',
    'encletic': u''
}
STOPWORDS[u'أوبكلتا'] = {
    'word': u'أوبكلتا',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَبِكِلْتَا',
    'stem': u'كِلْتَا',
    'type': u'STOPWORD',
    'original': u'كِلْتَا',
    'encletic': u''
}
STOPWORDS[u'أفبكلتا'] = {
    'word': u'أفبكلتا',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَبِكِلْتَا',
    'stem': u'كِلْتَا',
    'type': u'STOPWORD',
    'original': u'كِلْتَا',
    'encletic': u''
}
STOPWORDS[u'أوككلتا'] = {
    'word': u'أوككلتا',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَكَكِلْتَا',
    'stem': u'كِلْتَا',
    'type': u'STOPWORD',
    'original': u'كِلْتَا',
    'encletic': u''
}
STOPWORDS[u'أفككلتا'] = {
    'word': u'أفككلتا',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَكَكِلْتَا',
    'stem': u'كِلْتَا',
    'type': u'STOPWORD',
    'original': u'كِلْتَا',
    'encletic': u''
}
STOPWORDS[u'أولكلتا'] = {
    'word': u'أولكلتا',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَلِكِلْتَا',
    'stem': u'كِلْتَا',
    'type': u'STOPWORD',
    'original': u'كِلْتَا',
    'encletic': u''
}
STOPWORDS[u'أفلكلتا'] = {
    'word': u'أفلكلتا',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَلِكِلْتَا',
    'stem': u'كِلْتَا',
    'type': u'STOPWORD',
    'original': u'كِلْتَا',
    'encletic': u''
}
STOPWORDS[u'أولكلتا'] = {
    'word': u'أولكلتا',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَلَكِلْتَا',
    'stem': u'كِلْتَا',
    'type': u'STOPWORD',
    'original': u'كِلْتَا',
    'encletic': u''
}
STOPWORDS[u'أفلكلتا'] = {
    'word': u'أفلكلتا',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَلَكِلْتَا',
    'stem': u'كِلْتَا',
    'type': u'STOPWORD',
    'original': u'كِلْتَا',
    'encletic': u''
}
STOPWORDS[u'كليكما'] = {
    'word': u'كِلَيْكُمَا',
    'procletic': u'',
    'tags': u'أداة:توكيد',
    'vocalized': u'كِلَيْكُمَا',
    'stem': u'كِلَيْكُمَا',
    'type': u'STOPWORD',
    'original': u'كِلَيْكُمَا',
    'encletic': u''
}
STOPWORDS[u'بكليكما'] = {
    'word': u'بكليكما',
    'procletic': u'-بِ',
    'tags': u'أداة:توكيد:مجرور',
    'vocalized': u'بِكِلَيْكُمَا',
    'stem': u'كِلَيْكُمَا',
    'type': u'STOPWORD',
    'original': u'كِلَيْكُمَا',
    'encletic': u''
}
STOPWORDS[u'ككليكما'] = {
    'word': u'ككليكما',
    'procletic': u'-كَ',
    'tags': u'أداة:توكيد:مجرور',
    'vocalized': u'كَكِلَيْكُمَا',
    'stem': u'كِلَيْكُمَا',
    'type': u'STOPWORD',
    'original': u'كِلَيْكُمَا',
    'encletic': u''
}
STOPWORDS[u'لكليكما'] = {
    'word': u'لكليكما',
    'procletic': u'-لِ',
    'tags': u'أداة:توكيد:مجرور',
    'vocalized': u'لِكِلَيْكُمَا',
    'stem': u'كِلَيْكُمَا',
    'type': u'STOPWORD',
    'original': u'كِلَيْكُمَا',
    'encletic': u''
}
STOPWORDS[u'وكليكما'] = {
    'word': u'وكليكما',
    'procletic': u'-وَ',
    'tags': u'أداة:توكيد:معطوف',
    'vocalized': u'وَكِلَيْكُمَا',
    'stem': u'كِلَيْكُمَا',
    'type': u'STOPWORD',
    'original': u'كِلَيْكُمَا',
    'encletic': u''
}
STOPWORDS[u'فكليكما'] = {
    'word': u'فكليكما',
    'procletic': u'-فَ',
    'tags': u'أداة:توكيد:معطوف',
    'vocalized': u'فَكِلَيْكُمَا',
    'stem': u'كِلَيْكُمَا',
    'type': u'STOPWORD',
    'original': u'كِلَيْكُمَا',
    'encletic': u''
}
STOPWORDS[u'وبكليكما'] = {
    'word': u'وبكليكما',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف',
    'vocalized': u'وَبِكِلَيْكُمَا',
    'stem': u'كِلَيْكُمَا',
    'type': u'STOPWORD',
    'original': u'كِلَيْكُمَا',
    'encletic': u''
}
STOPWORDS[u'فبكليكما'] = {
    'word': u'فبكليكما',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف',
    'vocalized': u'فَبِكِلَيْكُمَا',
    'stem': u'كِلَيْكُمَا',
    'type': u'STOPWORD',
    'original': u'كِلَيْكُمَا',
    'encletic': u''
}
STOPWORDS[u'وككليكما'] = {
    'word': u'وككليكما',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف',
    'vocalized': u'وَكَكِلَيْكُمَا',
    'stem': u'كِلَيْكُمَا',
    'type': u'STOPWORD',
    'original': u'كِلَيْكُمَا',
    'encletic': u''
}
STOPWORDS[u'فككليكما'] = {
    'word': u'فككليكما',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف',
    'vocalized': u'فَكَكِلَيْكُمَا',
    'stem': u'كِلَيْكُمَا',
    'type': u'STOPWORD',
    'original': u'كِلَيْكُمَا',
    'encletic': u''
}
STOPWORDS[u'ولكليكما'] = {
    'word': u'ولكليكما',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف',
    'vocalized': u'وَلِكِلَيْكُمَا',
    'stem': u'كِلَيْكُمَا',
    'type': u'STOPWORD',
    'original': u'كِلَيْكُمَا',
    'encletic': u''
}
STOPWORDS[u'فلكليكما'] = {
    'word': u'فلكليكما',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف',
    'vocalized': u'فَلِكِلَيْكُمَا',
    'stem': u'كِلَيْكُمَا',
    'type': u'STOPWORD',
    'original': u'كِلَيْكُمَا',
    'encletic': u''
}
STOPWORDS[u'أكليكما'] = {
    'word': u'أكليكما',
    'procletic': u'-أَ',
    'tags': u'أداة:توكيد:مجرور',
    'vocalized': u'أَكِلَيْكُمَا',
    'stem': u'كِلَيْكُمَا',
    'type': u'STOPWORD',
    'original': u'كِلَيْكُمَا',
    'encletic': u''
}
STOPWORDS[u'أبكليكما'] = {
    'word': u'أبكليكما',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور',
    'vocalized': u'أَبِكِلَيْكُمَا',
    'stem': u'كِلَيْكُمَا',
    'type': u'STOPWORD',
    'original': u'كِلَيْكُمَا',
    'encletic': u''
}
STOPWORDS[u'أككليكما'] = {
    'word': u'أككليكما',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور',
    'vocalized': u'أَكَكِلَيْكُمَا',
    'stem': u'كِلَيْكُمَا',
    'type': u'STOPWORD',
    'original': u'كِلَيْكُمَا',
    'encletic': u''
}
STOPWORDS[u'ألكليكما'] = {
    'word': u'ألكليكما',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور',
    'vocalized': u'أَلِكِلَيْكُمَا',
    'stem': u'كِلَيْكُمَا',
    'type': u'STOPWORD',
    'original': u'كِلَيْكُمَا',
    'encletic': u''
}
STOPWORDS[u'أوكليكما'] = {
    'word': u'أوكليكما',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور',
    'vocalized': u'أَوَكِلَيْكُمَا',
    'stem': u'كِلَيْكُمَا',
    'type': u'STOPWORD',
    'original': u'كِلَيْكُمَا',
    'encletic': u''
}
STOPWORDS[u'أفكليكما'] = {
    'word': u'أفكليكما',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور',
    'vocalized': u'أَفَكِلَيْكُمَا',
    'stem': u'كِلَيْكُمَا',
    'type': u'STOPWORD',
    'original': u'كِلَيْكُمَا',
    'encletic': u''
}
STOPWORDS[u'أوبكليكما'] = {
    'word': u'أوبكليكما',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَبِكِلَيْكُمَا',
    'stem': u'كِلَيْكُمَا',
    'type': u'STOPWORD',
    'original': u'كِلَيْكُمَا',
    'encletic': u''
}
STOPWORDS[u'أفبكليكما'] = {
    'word': u'أفبكليكما',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَبِكِلَيْكُمَا',
    'stem': u'كِلَيْكُمَا',
    'type': u'STOPWORD',
    'original': u'كِلَيْكُمَا',
    'encletic': u''
}
STOPWORDS[u'أوككليكما'] = {
    'word': u'أوككليكما',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَكَكِلَيْكُمَا',
    'stem': u'كِلَيْكُمَا',
    'type': u'STOPWORD',
    'original': u'كِلَيْكُمَا',
    'encletic': u''
}
STOPWORDS[u'أفككليكما'] = {
    'word': u'أفككليكما',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَكَكِلَيْكُمَا',
    'stem': u'كِلَيْكُمَا',
    'type': u'STOPWORD',
    'original': u'كِلَيْكُمَا',
    'encletic': u''
}
STOPWORDS[u'أولكليكما'] = {
    'word': u'أولكليكما',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَلِكِلَيْكُمَا',
    'stem': u'كِلَيْكُمَا',
    'type': u'STOPWORD',
    'original': u'كِلَيْكُمَا',
    'encletic': u''
}
STOPWORDS[u'أفلكليكما'] = {
    'word': u'أفلكليكما',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَلِكِلَيْكُمَا',
    'stem': u'كِلَيْكُمَا',
    'type': u'STOPWORD',
    'original': u'كِلَيْكُمَا',
    'encletic': u''
}
STOPWORDS[u'كليهما'] = {
    'word': u'كِلَيْهِمَا',
    'procletic': u'',
    'tags': u'أداة:توكيد',
    'vocalized': u'كِلَيْهِمَا',
    'stem': u'كِلَيْهِمَا',
    'type': u'STOPWORD',
    'original': u'كِلَيْهِمَا',
    'encletic': u''
}
STOPWORDS[u'بكليهما'] = {
    'word': u'بكليهما',
    'procletic': u'-بِ',
    'tags': u'أداة:توكيد:مجرور',
    'vocalized': u'بِكِلَيْهِمَا',
    'stem': u'كِلَيْهِمَا',
    'type': u'STOPWORD',
    'original': u'كِلَيْهِمَا',
    'encletic': u''
}
STOPWORDS[u'ككليهما'] = {
    'word': u'ككليهما',
    'procletic': u'-كَ',
    'tags': u'أداة:توكيد:مجرور',
    'vocalized': u'كَكِلَيْهِمَا',
    'stem': u'كِلَيْهِمَا',
    'type': u'STOPWORD',
    'original': u'كِلَيْهِمَا',
    'encletic': u''
}
STOPWORDS[u'لكليهما'] = {
    'word': u'لكليهما',
    'procletic': u'-لِ',
    'tags': u'أداة:توكيد:مجرور',
    'vocalized': u'لِكِلَيْهِمَا',
    'stem': u'كِلَيْهِمَا',
    'type': u'STOPWORD',
    'original': u'كِلَيْهِمَا',
    'encletic': u''
}
STOPWORDS[u'وكليهما'] = {
    'word': u'وكليهما',
    'procletic': u'-وَ',
    'tags': u'أداة:توكيد:معطوف',
    'vocalized': u'وَكِلَيْهِمَا',
    'stem': u'كِلَيْهِمَا',
    'type': u'STOPWORD',
    'original': u'كِلَيْهِمَا',
    'encletic': u''
}
STOPWORDS[u'فكليهما'] = {
    'word': u'فكليهما',
    'procletic': u'-فَ',
    'tags': u'أداة:توكيد:معطوف',
    'vocalized': u'فَكِلَيْهِمَا',
    'stem': u'كِلَيْهِمَا',
    'type': u'STOPWORD',
    'original': u'كِلَيْهِمَا',
    'encletic': u''
}
STOPWORDS[u'وبكليهما'] = {
    'word': u'وبكليهما',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف',
    'vocalized': u'وَبِكِلَيْهِمَا',
    'stem': u'كِلَيْهِمَا',
    'type': u'STOPWORD',
    'original': u'كِلَيْهِمَا',
    'encletic': u''
}
STOPWORDS[u'فبكليهما'] = {
    'word': u'فبكليهما',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف',
    'vocalized': u'فَبِكِلَيْهِمَا',
    'stem': u'كِلَيْهِمَا',
    'type': u'STOPWORD',
    'original': u'كِلَيْهِمَا',
    'encletic': u''
}
STOPWORDS[u'وككليهما'] = {
    'word': u'وككليهما',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف',
    'vocalized': u'وَكَكِلَيْهِمَا',
    'stem': u'كِلَيْهِمَا',
    'type': u'STOPWORD',
    'original': u'كِلَيْهِمَا',
    'encletic': u''
}
STOPWORDS[u'فككليهما'] = {
    'word': u'فككليهما',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف',
    'vocalized': u'فَكَكِلَيْهِمَا',
    'stem': u'كِلَيْهِمَا',
    'type': u'STOPWORD',
    'original': u'كِلَيْهِمَا',
    'encletic': u''
}
STOPWORDS[u'ولكليهما'] = {
    'word': u'ولكليهما',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف',
    'vocalized': u'وَلِكِلَيْهِمَا',
    'stem': u'كِلَيْهِمَا',
    'type': u'STOPWORD',
    'original': u'كِلَيْهِمَا',
    'encletic': u''
}
STOPWORDS[u'فلكليهما'] = {
    'word': u'فلكليهما',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف',
    'vocalized': u'فَلِكِلَيْهِمَا',
    'stem': u'كِلَيْهِمَا',
    'type': u'STOPWORD',
    'original': u'كِلَيْهِمَا',
    'encletic': u''
}
STOPWORDS[u'أكليهما'] = {
    'word': u'أكليهما',
    'procletic': u'-أَ',
    'tags': u'أداة:توكيد:مجرور',
    'vocalized': u'أَكِلَيْهِمَا',
    'stem': u'كِلَيْهِمَا',
    'type': u'STOPWORD',
    'original': u'كِلَيْهِمَا',
    'encletic': u''
}
STOPWORDS[u'أبكليهما'] = {
    'word': u'أبكليهما',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور',
    'vocalized': u'أَبِكِلَيْهِمَا',
    'stem': u'كِلَيْهِمَا',
    'type': u'STOPWORD',
    'original': u'كِلَيْهِمَا',
    'encletic': u''
}
STOPWORDS[u'أككليهما'] = {
    'word': u'أككليهما',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور',
    'vocalized': u'أَكَكِلَيْهِمَا',
    'stem': u'كِلَيْهِمَا',
    'type': u'STOPWORD',
    'original': u'كِلَيْهِمَا',
    'encletic': u''
}
STOPWORDS[u'ألكليهما'] = {
    'word': u'ألكليهما',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور',
    'vocalized': u'أَلِكِلَيْهِمَا',
    'stem': u'كِلَيْهِمَا',
    'type': u'STOPWORD',
    'original': u'كِلَيْهِمَا',
    'encletic': u''
}
STOPWORDS[u'أوكليهما'] = {
    'word': u'أوكليهما',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور',
    'vocalized': u'أَوَكِلَيْهِمَا',
    'stem': u'كِلَيْهِمَا',
    'type': u'STOPWORD',
    'original': u'كِلَيْهِمَا',
    'encletic': u''
}
STOPWORDS[u'أفكليهما'] = {
    'word': u'أفكليهما',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور',
    'vocalized': u'أَفَكِلَيْهِمَا',
    'stem': u'كِلَيْهِمَا',
    'type': u'STOPWORD',
    'original': u'كِلَيْهِمَا',
    'encletic': u''
}
STOPWORDS[u'أوبكليهما'] = {
    'word': u'أوبكليهما',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَبِكِلَيْهِمَا',
    'stem': u'كِلَيْهِمَا',
    'type': u'STOPWORD',
    'original': u'كِلَيْهِمَا',
    'encletic': u''
}
STOPWORDS[u'أفبكليهما'] = {
    'word': u'أفبكليهما',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَبِكِلَيْهِمَا',
    'stem': u'كِلَيْهِمَا',
    'type': u'STOPWORD',
    'original': u'كِلَيْهِمَا',
    'encletic': u''
}
STOPWORDS[u'أوككليهما'] = {
    'word': u'أوككليهما',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَكَكِلَيْهِمَا',
    'stem': u'كِلَيْهِمَا',
    'type': u'STOPWORD',
    'original': u'كِلَيْهِمَا',
    'encletic': u''
}
STOPWORDS[u'أفككليهما'] = {
    'word': u'أفككليهما',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَكَكِلَيْهِمَا',
    'stem': u'كِلَيْهِمَا',
    'type': u'STOPWORD',
    'original': u'كِلَيْهِمَا',
    'encletic': u''
}
STOPWORDS[u'أولكليهما'] = {
    'word': u'أولكليهما',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَلِكِلَيْهِمَا',
    'stem': u'كِلَيْهِمَا',
    'type': u'STOPWORD',
    'original': u'كِلَيْهِمَا',
    'encletic': u''
}
STOPWORDS[u'أفلكليهما'] = {
    'word': u'أفلكليهما',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَلِكِلَيْهِمَا',
    'stem': u'كِلَيْهِمَا',
    'type': u'STOPWORD',
    'original': u'كِلَيْهِمَا',
    'encletic': u''
}
STOPWORDS[u'نفس'] = {
    'word': u'نَفْسُ',
    'procletic': u'',
    'tags': u'أداة:توكيد',
    'vocalized': u'نَفْسُ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u''
}
STOPWORDS[u'بنفس'] = {
    'word': u'بنفس',
    'procletic': u'-بِ',
    'tags': u'أداة:توكيد:مجرور',
    'vocalized': u'بِنَفْسُ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u''
}
STOPWORDS[u'كنفس'] = {
    'word': u'كنفس',
    'procletic': u'-كَ',
    'tags': u'أداة:توكيد:مجرور',
    'vocalized': u'كَنَفْسُ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u''
}
STOPWORDS[u'لنفس'] = {
    'word': u'لنفس',
    'procletic': u'-لِ',
    'tags': u'أداة:توكيد:مجرور',
    'vocalized': u'لِنَفْسُ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u''
}
STOPWORDS[u'لنفس'] = {
    'word': u'لنفس',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور',
    'vocalized': u'لَنَفْسُ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u''
}
STOPWORDS[u'ونفس'] = {
    'word': u'ونفس',
    'procletic': u'-وَ',
    'tags': u'أداة:توكيد:معطوف',
    'vocalized': u'وَنَفْسُ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u''
}
STOPWORDS[u'فنفس'] = {
    'word': u'فنفس',
    'procletic': u'-فَ',
    'tags': u'أداة:توكيد:معطوف',
    'vocalized': u'فَنَفْسُ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u''
}
STOPWORDS[u'وبنفس'] = {
    'word': u'وبنفس',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف',
    'vocalized': u'وَبِنَفْسُ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u''
}
STOPWORDS[u'فبنفس'] = {
    'word': u'فبنفس',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف',
    'vocalized': u'فَبِنَفْسُ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u''
}
STOPWORDS[u'وكنفس'] = {
    'word': u'وكنفس',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف',
    'vocalized': u'وَكَنَفْسُ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u''
}
STOPWORDS[u'فكنفس'] = {
    'word': u'فكنفس',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف',
    'vocalized': u'فَكَنَفْسُ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u''
}
STOPWORDS[u'ولنفس'] = {
    'word': u'ولنفس',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف',
    'vocalized': u'وَلِنَفْسُ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u''
}
STOPWORDS[u'فلنفس'] = {
    'word': u'فلنفس',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف',
    'vocalized': u'فَلِنَفْسُ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u''
}
STOPWORDS[u'ولنفس'] = {
    'word': u'ولنفس',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'وَلَنَفْسُ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u''
}
STOPWORDS[u'فلنفس'] = {
    'word': u'فلنفس',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'فَلَنَفْسُ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u''
}
STOPWORDS[u'أنفس'] = {
    'word': u'أنفس',
    'procletic': u'-أَ',
    'tags': u'أداة:توكيد:مجرور',
    'vocalized': u'أَنَفْسُ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u''
}
STOPWORDS[u'أبنفس'] = {
    'word': u'أبنفس',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور',
    'vocalized': u'أَبِنَفْسُ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u''
}
STOPWORDS[u'أكنفس'] = {
    'word': u'أكنفس',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور',
    'vocalized': u'أَكَنَفْسُ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u''
}
STOPWORDS[u'ألنفس'] = {
    'word': u'ألنفس',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور',
    'vocalized': u'أَلِنَفْسُ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u''
}
STOPWORDS[u'ألنفس'] = {
    'word': u'ألنفس',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور',
    'vocalized': u'أَلَنَفْسُ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u''
}
STOPWORDS[u'أونفس'] = {
    'word': u'أونفس',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور',
    'vocalized': u'أَوَنَفْسُ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u''
}
STOPWORDS[u'أفنفس'] = {
    'word': u'أفنفس',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور',
    'vocalized': u'أَفَنَفْسُ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u''
}
STOPWORDS[u'أوبنفس'] = {
    'word': u'أوبنفس',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَبِنَفْسُ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u''
}
STOPWORDS[u'أفبنفس'] = {
    'word': u'أفبنفس',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَبِنَفْسُ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u''
}
STOPWORDS[u'أوكنفس'] = {
    'word': u'أوكنفس',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَكَنَفْسُ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u''
}
STOPWORDS[u'أفكنفس'] = {
    'word': u'أفكنفس',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَكَنَفْسُ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u''
}
STOPWORDS[u'أولنفس'] = {
    'word': u'أولنفس',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَلِنَفْسُ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u''
}
STOPWORDS[u'أفلنفس'] = {
    'word': u'أفلنفس',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَلِنَفْسُ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u''
}
STOPWORDS[u'أولنفس'] = {
    'word': u'أولنفس',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَلَنَفْسُ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u''
}
STOPWORDS[u'أفلنفس'] = {
    'word': u'أفلنفس',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَلَنَفْسُ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u''
}
STOPWORDS[u'نفسي'] = {
    'word': u'نفسي',
    'procletic': u'',
    'tags': u'أداة:توكيد:مضاف',
    'vocalized': u'نَفْسُِي',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'ِي'
}
STOPWORDS[u'نفسك'] = {
    'word': u'نفسك',
    'procletic': u'',
    'tags': u'أداة:توكيد:مضاف',
    'vocalized': u'نَفْسُكَ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كَ'
}
STOPWORDS[u'نفسك'] = {
    'word': u'نفسك',
    'procletic': u'',
    'tags': u'أداة:توكيد:مضاف',
    'vocalized': u'نَفْسُكِ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كِ'
}
STOPWORDS[u'نفسه'] = {
    'word': u'نفسه',
    'procletic': u'',
    'tags': u'أداة:توكيد:مضاف',
    'vocalized': u'نَفْسُه',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'ه'
}
STOPWORDS[u'نفسكم'] = {
    'word': u'نفسكم',
    'procletic': u'',
    'tags': u'أداة:توكيد:مضاف',
    'vocalized': u'نَفْسُكُمْ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'نفسكن'] = {
    'word': u'نفسكن',
    'procletic': u'',
    'tags': u'أداة:توكيد:مضاف',
    'vocalized': u'نَفْسُكُنَّ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'نفسها'] = {
    'word': u'نفسها',
    'procletic': u'',
    'tags': u'أداة:توكيد:مضاف',
    'vocalized': u'نَفْسُهَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هَا'
}
STOPWORDS[u'نفسهم'] = {
    'word': u'نفسهم',
    'procletic': u'',
    'tags': u'أداة:توكيد:مضاف',
    'vocalized': u'نَفْسُهُمْ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'نفسهن'] = {
    'word': u'نفسهن',
    'procletic': u'',
    'tags': u'أداة:توكيد:مضاف',
    'vocalized': u'نَفْسُهُنَّ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'نفسنا'] = {
    'word': u'نفسنا',
    'procletic': u'',
    'tags': u'أداة:توكيد:مضاف',
    'vocalized': u'نَفْسُنَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'نَا'
}
STOPWORDS[u'نفسكما'] = {
    'word': u'نفسكما',
    'procletic': u'',
    'tags': u'أداة:توكيد:مضاف',
    'vocalized': u'نَفْسُكُمَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'نفسهما'] = {
    'word': u'نفسهما',
    'procletic': u'',
    'tags': u'أداة:توكيد:مضاف',
    'vocalized': u'نَفْسُهُمَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'بنفسي'] = {
    'word': u'بنفسي',
    'procletic': u'-بِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'بِنَفْسُِي',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'ِي'
}
STOPWORDS[u'بنفسك'] = {
    'word': u'بنفسك',
    'procletic': u'-بِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'بِنَفْسُكَ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كَ'
}
STOPWORDS[u'بنفسك'] = {
    'word': u'بنفسك',
    'procletic': u'-بِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'بِنَفْسُكِ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كِ'
}
STOPWORDS[u'بنفسه'] = {
    'word': u'بنفسه',
    'procletic': u'-بِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'بِنَفْسُه',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'ه'
}
STOPWORDS[u'بنفسكم'] = {
    'word': u'بنفسكم',
    'procletic': u'-بِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'بِنَفْسُكُمْ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'بنفسكن'] = {
    'word': u'بنفسكن',
    'procletic': u'-بِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'بِنَفْسُكُنَّ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'بنفسها'] = {
    'word': u'بنفسها',
    'procletic': u'-بِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'بِنَفْسُهَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هَا'
}
STOPWORDS[u'بنفسهم'] = {
    'word': u'بنفسهم',
    'procletic': u'-بِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'بِنَفْسُهُمْ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'بنفسهن'] = {
    'word': u'بنفسهن',
    'procletic': u'-بِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'بِنَفْسُهُنَّ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'بنفسنا'] = {
    'word': u'بنفسنا',
    'procletic': u'-بِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'بِنَفْسُنَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'نَا'
}
STOPWORDS[u'بنفسكما'] = {
    'word': u'بنفسكما',
    'procletic': u'-بِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'بِنَفْسُكُمَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'بنفسهما'] = {
    'word': u'بنفسهما',
    'procletic': u'-بِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'بِنَفْسُهُمَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'كنفسي'] = {
    'word': u'كنفسي',
    'procletic': u'-كَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'كَنَفْسُِي',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'ِي'
}
STOPWORDS[u'كنفسك'] = {
    'word': u'كنفسك',
    'procletic': u'-كَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'كَنَفْسُكَ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كَ'
}
STOPWORDS[u'كنفسك'] = {
    'word': u'كنفسك',
    'procletic': u'-كَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'كَنَفْسُكِ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كِ'
}
STOPWORDS[u'كنفسه'] = {
    'word': u'كنفسه',
    'procletic': u'-كَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'كَنَفْسُه',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'ه'
}
STOPWORDS[u'كنفسكم'] = {
    'word': u'كنفسكم',
    'procletic': u'-كَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'كَنَفْسُكُمْ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'كنفسكن'] = {
    'word': u'كنفسكن',
    'procletic': u'-كَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'كَنَفْسُكُنَّ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'كنفسها'] = {
    'word': u'كنفسها',
    'procletic': u'-كَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'كَنَفْسُهَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هَا'
}
STOPWORDS[u'كنفسهم'] = {
    'word': u'كنفسهم',
    'procletic': u'-كَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'كَنَفْسُهُمْ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'كنفسهن'] = {
    'word': u'كنفسهن',
    'procletic': u'-كَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'كَنَفْسُهُنَّ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'كنفسنا'] = {
    'word': u'كنفسنا',
    'procletic': u'-كَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'كَنَفْسُنَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'نَا'
}
STOPWORDS[u'كنفسكما'] = {
    'word': u'كنفسكما',
    'procletic': u'-كَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'كَنَفْسُكُمَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'كنفسهما'] = {
    'word': u'كنفسهما',
    'procletic': u'-كَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'كَنَفْسُهُمَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'لنفسي'] = {
    'word': u'لنفسي',
    'procletic': u'-لِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'لِنَفْسُِي',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'ِي'
}
STOPWORDS[u'لنفسك'] = {
    'word': u'لنفسك',
    'procletic': u'-لِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'لِنَفْسُكَ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كَ'
}
STOPWORDS[u'لنفسك'] = {
    'word': u'لنفسك',
    'procletic': u'-لِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'لِنَفْسُكِ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كِ'
}
STOPWORDS[u'لنفسه'] = {
    'word': u'لنفسه',
    'procletic': u'-لِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'لِنَفْسُه',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'ه'
}
STOPWORDS[u'لنفسكم'] = {
    'word': u'لنفسكم',
    'procletic': u'-لِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'لِنَفْسُكُمْ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'لنفسكن'] = {
    'word': u'لنفسكن',
    'procletic': u'-لِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'لِنَفْسُكُنَّ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'لنفسها'] = {
    'word': u'لنفسها',
    'procletic': u'-لِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'لِنَفْسُهَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هَا'
}
STOPWORDS[u'لنفسهم'] = {
    'word': u'لنفسهم',
    'procletic': u'-لِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'لِنَفْسُهُمْ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'لنفسهن'] = {
    'word': u'لنفسهن',
    'procletic': u'-لِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'لِنَفْسُهُنَّ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'لنفسنا'] = {
    'word': u'لنفسنا',
    'procletic': u'-لِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'لِنَفْسُنَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'نَا'
}
STOPWORDS[u'لنفسكما'] = {
    'word': u'لنفسكما',
    'procletic': u'-لِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'لِنَفْسُكُمَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'لنفسهما'] = {
    'word': u'لنفسهما',
    'procletic': u'-لِ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'لِنَفْسُهُمَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'لنفسي'] = {
    'word': u'لنفسي',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَنَفْسُِي',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'ِي'
}
STOPWORDS[u'لنفسك'] = {
    'word': u'لنفسك',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَنَفْسُكَ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كَ'
}
STOPWORDS[u'لنفسك'] = {
    'word': u'لنفسك',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَنَفْسُكِ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كِ'
}
STOPWORDS[u'لنفسه'] = {
    'word': u'لنفسه',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَنَفْسُه',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'ه'
}
STOPWORDS[u'لنفسكم'] = {
    'word': u'لنفسكم',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَنَفْسُكُمْ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'لنفسكن'] = {
    'word': u'لنفسكن',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَنَفْسُكُنَّ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'لنفسها'] = {
    'word': u'لنفسها',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَنَفْسُهَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هَا'
}
STOPWORDS[u'لنفسهم'] = {
    'word': u'لنفسهم',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَنَفْسُهُمْ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'لنفسهن'] = {
    'word': u'لنفسهن',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَنَفْسُهُنَّ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'لنفسنا'] = {
    'word': u'لنفسنا',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَنَفْسُنَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'نَا'
}
STOPWORDS[u'لنفسكما'] = {
    'word': u'لنفسكما',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَنَفْسُكُمَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'لنفسهما'] = {
    'word': u'لنفسهما',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَنَفْسُهُمَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ونفسي'] = {
    'word': u'ونفسي',
    'procletic': u'-وَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'وَنَفْسُِي',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'ِي'
}
STOPWORDS[u'ونفسك'] = {
    'word': u'ونفسك',
    'procletic': u'-وَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'وَنَفْسُكَ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كَ'
}
STOPWORDS[u'ونفسك'] = {
    'word': u'ونفسك',
    'procletic': u'-وَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'وَنَفْسُكِ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كِ'
}
STOPWORDS[u'ونفسه'] = {
    'word': u'ونفسه',
    'procletic': u'-وَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'وَنَفْسُه',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'ه'
}
STOPWORDS[u'ونفسكم'] = {
    'word': u'ونفسكم',
    'procletic': u'-وَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'وَنَفْسُكُمْ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ونفسكن'] = {
    'word': u'ونفسكن',
    'procletic': u'-وَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'وَنَفْسُكُنَّ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ونفسها'] = {
    'word': u'ونفسها',
    'procletic': u'-وَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'وَنَفْسُهَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هَا'
}
STOPWORDS[u'ونفسهم'] = {
    'word': u'ونفسهم',
    'procletic': u'-وَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'وَنَفْسُهُمْ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ونفسهن'] = {
    'word': u'ونفسهن',
    'procletic': u'-وَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'وَنَفْسُهُنَّ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ونفسنا'] = {
    'word': u'ونفسنا',
    'procletic': u'-وَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'وَنَفْسُنَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'نَا'
}
STOPWORDS[u'ونفسكما'] = {
    'word': u'ونفسكما',
    'procletic': u'-وَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'وَنَفْسُكُمَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ونفسهما'] = {
    'word': u'ونفسهما',
    'procletic': u'-وَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'وَنَفْسُهُمَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فنفسي'] = {
    'word': u'فنفسي',
    'procletic': u'-فَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'فَنَفْسُِي',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'ِي'
}
STOPWORDS[u'فنفسك'] = {
    'word': u'فنفسك',
    'procletic': u'-فَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'فَنَفْسُكَ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كَ'
}
STOPWORDS[u'فنفسك'] = {
    'word': u'فنفسك',
    'procletic': u'-فَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'فَنَفْسُكِ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كِ'
}
STOPWORDS[u'فنفسه'] = {
    'word': u'فنفسه',
    'procletic': u'-فَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'فَنَفْسُه',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'ه'
}
STOPWORDS[u'فنفسكم'] = {
    'word': u'فنفسكم',
    'procletic': u'-فَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'فَنَفْسُكُمْ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فنفسكن'] = {
    'word': u'فنفسكن',
    'procletic': u'-فَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'فَنَفْسُكُنَّ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فنفسها'] = {
    'word': u'فنفسها',
    'procletic': u'-فَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'فَنَفْسُهَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هَا'
}
STOPWORDS[u'فنفسهم'] = {
    'word': u'فنفسهم',
    'procletic': u'-فَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'فَنَفْسُهُمْ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فنفسهن'] = {
    'word': u'فنفسهن',
    'procletic': u'-فَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'فَنَفْسُهُنَّ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فنفسنا'] = {
    'word': u'فنفسنا',
    'procletic': u'-فَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'فَنَفْسُنَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'نَا'
}
STOPWORDS[u'فنفسكما'] = {
    'word': u'فنفسكما',
    'procletic': u'-فَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'فَنَفْسُكُمَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فنفسهما'] = {
    'word': u'فنفسهما',
    'procletic': u'-فَ',
    'tags': u'أداة:توكيد:معطوف:مضاف',
    'vocalized': u'فَنَفْسُهُمَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'وبنفسي'] = {
    'word': u'وبنفسي',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِنَفْسُِي',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'ِي'
}
STOPWORDS[u'وبنفسك'] = {
    'word': u'وبنفسك',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِنَفْسُكَ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كَ'
}
STOPWORDS[u'وبنفسك'] = {
    'word': u'وبنفسك',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِنَفْسُكِ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كِ'
}
STOPWORDS[u'وبنفسه'] = {
    'word': u'وبنفسه',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِنَفْسُه',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'ه'
}
STOPWORDS[u'وبنفسكم'] = {
    'word': u'وبنفسكم',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِنَفْسُكُمْ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'وبنفسكن'] = {
    'word': u'وبنفسكن',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِنَفْسُكُنَّ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'وبنفسها'] = {
    'word': u'وبنفسها',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِنَفْسُهَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هَا'
}
STOPWORDS[u'وبنفسهم'] = {
    'word': u'وبنفسهم',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِنَفْسُهُمْ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'وبنفسهن'] = {
    'word': u'وبنفسهن',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِنَفْسُهُنَّ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'وبنفسنا'] = {
    'word': u'وبنفسنا',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِنَفْسُنَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'نَا'
}
STOPWORDS[u'وبنفسكما'] = {
    'word': u'وبنفسكما',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِنَفْسُكُمَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'وبنفسهما'] = {
    'word': u'وبنفسهما',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَبِنَفْسُهُمَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فبنفسي'] = {
    'word': u'فبنفسي',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِنَفْسُِي',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'ِي'
}
STOPWORDS[u'فبنفسك'] = {
    'word': u'فبنفسك',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِنَفْسُكَ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كَ'
}
STOPWORDS[u'فبنفسك'] = {
    'word': u'فبنفسك',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِنَفْسُكِ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كِ'
}
STOPWORDS[u'فبنفسه'] = {
    'word': u'فبنفسه',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِنَفْسُه',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'ه'
}
STOPWORDS[u'فبنفسكم'] = {
    'word': u'فبنفسكم',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِنَفْسُكُمْ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فبنفسكن'] = {
    'word': u'فبنفسكن',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِنَفْسُكُنَّ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فبنفسها'] = {
    'word': u'فبنفسها',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِنَفْسُهَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هَا'
}
STOPWORDS[u'فبنفسهم'] = {
    'word': u'فبنفسهم',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِنَفْسُهُمْ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فبنفسهن'] = {
    'word': u'فبنفسهن',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِنَفْسُهُنَّ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فبنفسنا'] = {
    'word': u'فبنفسنا',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِنَفْسُنَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'نَا'
}
STOPWORDS[u'فبنفسكما'] = {
    'word': u'فبنفسكما',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِنَفْسُكُمَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فبنفسهما'] = {
    'word': u'فبنفسهما',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَبِنَفْسُهُمَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'وكنفسي'] = {
    'word': u'وكنفسي',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَنَفْسُِي',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'ِي'
}
STOPWORDS[u'وكنفسك'] = {
    'word': u'وكنفسك',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَنَفْسُكَ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كَ'
}
STOPWORDS[u'وكنفسك'] = {
    'word': u'وكنفسك',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَنَفْسُكِ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كِ'
}
STOPWORDS[u'وكنفسه'] = {
    'word': u'وكنفسه',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَنَفْسُه',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'ه'
}
STOPWORDS[u'وكنفسكم'] = {
    'word': u'وكنفسكم',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَنَفْسُكُمْ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'وكنفسكن'] = {
    'word': u'وكنفسكن',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَنَفْسُكُنَّ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'وكنفسها'] = {
    'word': u'وكنفسها',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَنَفْسُهَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هَا'
}
STOPWORDS[u'وكنفسهم'] = {
    'word': u'وكنفسهم',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَنَفْسُهُمْ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'وكنفسهن'] = {
    'word': u'وكنفسهن',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَنَفْسُهُنَّ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'وكنفسنا'] = {
    'word': u'وكنفسنا',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَنَفْسُنَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'نَا'
}
STOPWORDS[u'وكنفسكما'] = {
    'word': u'وكنفسكما',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَنَفْسُكُمَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'وكنفسهما'] = {
    'word': u'وكنفسهما',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَكَنَفْسُهُمَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فكنفسي'] = {
    'word': u'فكنفسي',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَنَفْسُِي',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'ِي'
}
STOPWORDS[u'فكنفسك'] = {
    'word': u'فكنفسك',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَنَفْسُكَ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كَ'
}
STOPWORDS[u'فكنفسك'] = {
    'word': u'فكنفسك',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَنَفْسُكِ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كِ'
}
STOPWORDS[u'فكنفسه'] = {
    'word': u'فكنفسه',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَنَفْسُه',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'ه'
}
STOPWORDS[u'فكنفسكم'] = {
    'word': u'فكنفسكم',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَنَفْسُكُمْ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فكنفسكن'] = {
    'word': u'فكنفسكن',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَنَفْسُكُنَّ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فكنفسها'] = {
    'word': u'فكنفسها',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَنَفْسُهَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هَا'
}
STOPWORDS[u'فكنفسهم'] = {
    'word': u'فكنفسهم',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَنَفْسُهُمْ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فكنفسهن'] = {
    'word': u'فكنفسهن',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَنَفْسُهُنَّ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فكنفسنا'] = {
    'word': u'فكنفسنا',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَنَفْسُنَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'نَا'
}
STOPWORDS[u'فكنفسكما'] = {
    'word': u'فكنفسكما',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَنَفْسُكُمَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فكنفسهما'] = {
    'word': u'فكنفسهما',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَكَنَفْسُهُمَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ولنفسي'] = {
    'word': u'ولنفسي',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِنَفْسُِي',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'ِي'
}
STOPWORDS[u'ولنفسك'] = {
    'word': u'ولنفسك',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِنَفْسُكَ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كَ'
}
STOPWORDS[u'ولنفسك'] = {
    'word': u'ولنفسك',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِنَفْسُكِ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كِ'
}
STOPWORDS[u'ولنفسه'] = {
    'word': u'ولنفسه',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِنَفْسُه',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'ه'
}
STOPWORDS[u'ولنفسكم'] = {
    'word': u'ولنفسكم',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِنَفْسُكُمْ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ولنفسكن'] = {
    'word': u'ولنفسكن',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِنَفْسُكُنَّ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ولنفسها'] = {
    'word': u'ولنفسها',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِنَفْسُهَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هَا'
}
STOPWORDS[u'ولنفسهم'] = {
    'word': u'ولنفسهم',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِنَفْسُهُمْ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ولنفسهن'] = {
    'word': u'ولنفسهن',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِنَفْسُهُنَّ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ولنفسنا'] = {
    'word': u'ولنفسنا',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِنَفْسُنَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'نَا'
}
STOPWORDS[u'ولنفسكما'] = {
    'word': u'ولنفسكما',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِنَفْسُكُمَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ولنفسهما'] = {
    'word': u'ولنفسهما',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'وَلِنَفْسُهُمَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فلنفسي'] = {
    'word': u'فلنفسي',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِنَفْسُِي',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'ِي'
}
STOPWORDS[u'فلنفسك'] = {
    'word': u'فلنفسك',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِنَفْسُكَ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كَ'
}
STOPWORDS[u'فلنفسك'] = {
    'word': u'فلنفسك',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِنَفْسُكِ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كِ'
}
STOPWORDS[u'فلنفسه'] = {
    'word': u'فلنفسه',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِنَفْسُه',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'ه'
}
STOPWORDS[u'فلنفسكم'] = {
    'word': u'فلنفسكم',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِنَفْسُكُمْ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فلنفسكن'] = {
    'word': u'فلنفسكن',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِنَفْسُكُنَّ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فلنفسها'] = {
    'word': u'فلنفسها',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِنَفْسُهَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هَا'
}
STOPWORDS[u'فلنفسهم'] = {
    'word': u'فلنفسهم',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِنَفْسُهُمْ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فلنفسهن'] = {
    'word': u'فلنفسهن',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِنَفْسُهُنَّ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فلنفسنا'] = {
    'word': u'فلنفسنا',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِنَفْسُنَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'نَا'
}
STOPWORDS[u'فلنفسكما'] = {
    'word': u'فلنفسكما',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِنَفْسُكُمَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فلنفسهما'] = {
    'word': u'فلنفسهما',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مضاف',
    'vocalized': u'فَلِنَفْسُهُمَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ولنفسي'] = {
    'word': u'ولنفسي',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَنَفْسُِي',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'ِي'
}
STOPWORDS[u'ولنفسك'] = {
    'word': u'ولنفسك',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَنَفْسُكَ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كَ'
}
STOPWORDS[u'ولنفسك'] = {
    'word': u'ولنفسك',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَنَفْسُكِ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كِ'
}
STOPWORDS[u'ولنفسه'] = {
    'word': u'ولنفسه',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَنَفْسُه',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'ه'
}
STOPWORDS[u'ولنفسكم'] = {
    'word': u'ولنفسكم',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَنَفْسُكُمْ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ولنفسكن'] = {
    'word': u'ولنفسكن',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَنَفْسُكُنَّ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ولنفسها'] = {
    'word': u'ولنفسها',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَنَفْسُهَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هَا'
}
STOPWORDS[u'ولنفسهم'] = {
    'word': u'ولنفسهم',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَنَفْسُهُمْ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ولنفسهن'] = {
    'word': u'ولنفسهن',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَنَفْسُهُنَّ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ولنفسنا'] = {
    'word': u'ولنفسنا',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَنَفْسُنَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'نَا'
}
STOPWORDS[u'ولنفسكما'] = {
    'word': u'ولنفسكما',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَنَفْسُكُمَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ولنفسهما'] = {
    'word': u'ولنفسهما',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَنَفْسُهُمَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فلنفسي'] = {
    'word': u'فلنفسي',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَنَفْسُِي',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'ِي'
}
STOPWORDS[u'فلنفسك'] = {
    'word': u'فلنفسك',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَنَفْسُكَ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كَ'
}
STOPWORDS[u'فلنفسك'] = {
    'word': u'فلنفسك',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَنَفْسُكِ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كِ'
}
STOPWORDS[u'فلنفسه'] = {
    'word': u'فلنفسه',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَنَفْسُه',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'ه'
}
STOPWORDS[u'فلنفسكم'] = {
    'word': u'فلنفسكم',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَنَفْسُكُمْ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فلنفسكن'] = {
    'word': u'فلنفسكن',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَنَفْسُكُنَّ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فلنفسها'] = {
    'word': u'فلنفسها',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَنَفْسُهَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هَا'
}
STOPWORDS[u'فلنفسهم'] = {
    'word': u'فلنفسهم',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَنَفْسُهُمْ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فلنفسهن'] = {
    'word': u'فلنفسهن',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَنَفْسُهُنَّ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فلنفسنا'] = {
    'word': u'فلنفسنا',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَنَفْسُنَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'نَا'
}
STOPWORDS[u'فلنفسكما'] = {
    'word': u'فلنفسكما',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَنَفْسُكُمَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فلنفسهما'] = {
    'word': u'فلنفسهما',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَنَفْسُهُمَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أنفسي'] = {
    'word': u'أنفسي',
    'procletic': u'-أَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'أَنَفْسُِي',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'ِي'
}
STOPWORDS[u'أنفسك'] = {
    'word': u'أنفسك',
    'procletic': u'-أَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'أَنَفْسُكَ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كَ'
}
STOPWORDS[u'أنفسك'] = {
    'word': u'أنفسك',
    'procletic': u'-أَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'أَنَفْسُكِ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كِ'
}
STOPWORDS[u'أنفسه'] = {
    'word': u'أنفسه',
    'procletic': u'-أَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'أَنَفْسُه',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'ه'
}
STOPWORDS[u'أنفسكم'] = {
    'word': u'أنفسكم',
    'procletic': u'-أَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'أَنَفْسُكُمْ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أنفسكن'] = {
    'word': u'أنفسكن',
    'procletic': u'-أَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'أَنَفْسُكُنَّ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أنفسها'] = {
    'word': u'أنفسها',
    'procletic': u'-أَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'أَنَفْسُهَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هَا'
}
STOPWORDS[u'أنفسهم'] = {
    'word': u'أنفسهم',
    'procletic': u'-أَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'أَنَفْسُهُمْ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أنفسهن'] = {
    'word': u'أنفسهن',
    'procletic': u'-أَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'أَنَفْسُهُنَّ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أنفسنا'] = {
    'word': u'أنفسنا',
    'procletic': u'-أَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'أَنَفْسُنَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'نَا'
}
STOPWORDS[u'أنفسكما'] = {
    'word': u'أنفسكما',
    'procletic': u'-أَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'أَنَفْسُكُمَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أنفسهما'] = {
    'word': u'أنفسهما',
    'procletic': u'-أَ',
    'tags': u'أداة:توكيد:مجرور:مضاف',
    'vocalized': u'أَنَفْسُهُمَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أبنفسي'] = {
    'word': u'أبنفسي',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِنَفْسُِي',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'ِي'
}
STOPWORDS[u'أبنفسك'] = {
    'word': u'أبنفسك',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِنَفْسُكَ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كَ'
}
STOPWORDS[u'أبنفسك'] = {
    'word': u'أبنفسك',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِنَفْسُكِ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كِ'
}
STOPWORDS[u'أبنفسه'] = {
    'word': u'أبنفسه',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِنَفْسُه',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'ه'
}
STOPWORDS[u'أبنفسكم'] = {
    'word': u'أبنفسكم',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِنَفْسُكُمْ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أبنفسكن'] = {
    'word': u'أبنفسكن',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِنَفْسُكُنَّ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أبنفسها'] = {
    'word': u'أبنفسها',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِنَفْسُهَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هَا'
}
STOPWORDS[u'أبنفسهم'] = {
    'word': u'أبنفسهم',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِنَفْسُهُمْ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أبنفسهن'] = {
    'word': u'أبنفسهن',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِنَفْسُهُنَّ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أبنفسنا'] = {
    'word': u'أبنفسنا',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِنَفْسُنَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'نَا'
}
STOPWORDS[u'أبنفسكما'] = {
    'word': u'أبنفسكما',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِنَفْسُكُمَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أبنفسهما'] = {
    'word': u'أبنفسهما',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَبِنَفْسُهُمَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أكنفسي'] = {
    'word': u'أكنفسي',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَنَفْسُِي',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'ِي'
}
STOPWORDS[u'أكنفسك'] = {
    'word': u'أكنفسك',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَنَفْسُكَ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كَ'
}
STOPWORDS[u'أكنفسك'] = {
    'word': u'أكنفسك',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَنَفْسُكِ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كِ'
}
STOPWORDS[u'أكنفسه'] = {
    'word': u'أكنفسه',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَنَفْسُه',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'ه'
}
STOPWORDS[u'أكنفسكم'] = {
    'word': u'أكنفسكم',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَنَفْسُكُمْ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أكنفسكن'] = {
    'word': u'أكنفسكن',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَنَفْسُكُنَّ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أكنفسها'] = {
    'word': u'أكنفسها',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَنَفْسُهَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هَا'
}
STOPWORDS[u'أكنفسهم'] = {
    'word': u'أكنفسهم',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَنَفْسُهُمْ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أكنفسهن'] = {
    'word': u'أكنفسهن',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَنَفْسُهُنَّ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أكنفسنا'] = {
    'word': u'أكنفسنا',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَنَفْسُنَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'نَا'
}
STOPWORDS[u'أكنفسكما'] = {
    'word': u'أكنفسكما',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَنَفْسُكُمَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أكنفسهما'] = {
    'word': u'أكنفسهما',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَكَنَفْسُهُمَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ألنفسي'] = {
    'word': u'ألنفسي',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِنَفْسُِي',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'ِي'
}
STOPWORDS[u'ألنفسك'] = {
    'word': u'ألنفسك',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِنَفْسُكَ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كَ'
}
STOPWORDS[u'ألنفسك'] = {
    'word': u'ألنفسك',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِنَفْسُكِ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كِ'
}
STOPWORDS[u'ألنفسه'] = {
    'word': u'ألنفسه',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِنَفْسُه',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'ه'
}
STOPWORDS[u'ألنفسكم'] = {
    'word': u'ألنفسكم',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِنَفْسُكُمْ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ألنفسكن'] = {
    'word': u'ألنفسكن',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِنَفْسُكُنَّ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ألنفسها'] = {
    'word': u'ألنفسها',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِنَفْسُهَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هَا'
}
STOPWORDS[u'ألنفسهم'] = {
    'word': u'ألنفسهم',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِنَفْسُهُمْ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ألنفسهن'] = {
    'word': u'ألنفسهن',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِنَفْسُهُنَّ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ألنفسنا'] = {
    'word': u'ألنفسنا',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِنَفْسُنَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'نَا'
}
STOPWORDS[u'ألنفسكما'] = {
    'word': u'ألنفسكما',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِنَفْسُكُمَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ألنفسهما'] = {
    'word': u'ألنفسهما',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:توكيد:مجرور:مجرور:مضاف',
    'vocalized': u'أَلِنَفْسُهُمَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ألنفسي'] = {
    'word': u'ألنفسي',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَنَفْسُِي',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'ِي'
}
STOPWORDS[u'ألنفسك'] = {
    'word': u'ألنفسك',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَنَفْسُكَ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كَ'
}
STOPWORDS[u'ألنفسك'] = {
    'word': u'ألنفسك',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَنَفْسُكِ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كِ'
}
STOPWORDS[u'ألنفسه'] = {
    'word': u'ألنفسه',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَنَفْسُه',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'ه'
}
STOPWORDS[u'ألنفسكم'] = {
    'word': u'ألنفسكم',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَنَفْسُكُمْ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ألنفسكن'] = {
    'word': u'ألنفسكن',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَنَفْسُكُنَّ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ألنفسها'] = {
    'word': u'ألنفسها',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَنَفْسُهَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هَا'
}
STOPWORDS[u'ألنفسهم'] = {
    'word': u'ألنفسهم',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَنَفْسُهُمْ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ألنفسهن'] = {
    'word': u'ألنفسهن',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَنَفْسُهُنَّ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ألنفسنا'] = {
    'word': u'ألنفسنا',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَنَفْسُنَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'نَا'
}
STOPWORDS[u'ألنفسكما'] = {
    'word': u'ألنفسكما',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَنَفْسُكُمَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ألنفسهما'] = {
    'word': u'ألنفسهما',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَنَفْسُهُمَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أونفسي'] = {
    'word': u'أونفسي',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَنَفْسُِي',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'ِي'
}
STOPWORDS[u'أونفسك'] = {
    'word': u'أونفسك',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَنَفْسُكَ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كَ'
}
STOPWORDS[u'أونفسك'] = {
    'word': u'أونفسك',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَنَفْسُكِ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كِ'
}
STOPWORDS[u'أونفسه'] = {
    'word': u'أونفسه',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَنَفْسُه',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'ه'
}
STOPWORDS[u'أونفسكم'] = {
    'word': u'أونفسكم',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَنَفْسُكُمْ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أونفسكن'] = {
    'word': u'أونفسكن',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَنَفْسُكُنَّ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أونفسها'] = {
    'word': u'أونفسها',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَنَفْسُهَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هَا'
}
STOPWORDS[u'أونفسهم'] = {
    'word': u'أونفسهم',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَنَفْسُهُمْ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أونفسهن'] = {
    'word': u'أونفسهن',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَنَفْسُهُنَّ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أونفسنا'] = {
    'word': u'أونفسنا',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَنَفْسُنَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'نَا'
}
STOPWORDS[u'أونفسكما'] = {
    'word': u'أونفسكما',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَنَفْسُكُمَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أونفسهما'] = {
    'word': u'أونفسهما',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَنَفْسُهُمَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أفنفسي'] = {
    'word': u'أفنفسي',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَنَفْسُِي',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'ِي'
}
STOPWORDS[u'أفنفسك'] = {
    'word': u'أفنفسك',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَنَفْسُكَ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كَ'
}
STOPWORDS[u'أفنفسك'] = {
    'word': u'أفنفسك',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَنَفْسُكِ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كِ'
}
STOPWORDS[u'أفنفسه'] = {
    'word': u'أفنفسه',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَنَفْسُه',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'ه'
}
STOPWORDS[u'أفنفسكم'] = {
    'word': u'أفنفسكم',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَنَفْسُكُمْ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أفنفسكن'] = {
    'word': u'أفنفسكن',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَنَفْسُكُنَّ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أفنفسها'] = {
    'word': u'أفنفسها',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَنَفْسُهَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هَا'
}
STOPWORDS[u'أفنفسهم'] = {
    'word': u'أفنفسهم',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَنَفْسُهُمْ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أفنفسهن'] = {
    'word': u'أفنفسهن',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَنَفْسُهُنَّ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أفنفسنا'] = {
    'word': u'أفنفسنا',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَنَفْسُنَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'نَا'
}
STOPWORDS[u'أفنفسكما'] = {
    'word': u'أفنفسكما',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَنَفْسُكُمَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أفنفسهما'] = {
    'word': u'أفنفسهما',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:توكيد:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَنَفْسُهُمَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أوبنفسي'] = {
    'word': u'أوبنفسي',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِنَفْسُِي',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'ِي'
}
STOPWORDS[u'أوبنفسك'] = {
    'word': u'أوبنفسك',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِنَفْسُكَ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كَ'
}
STOPWORDS[u'أوبنفسك'] = {
    'word': u'أوبنفسك',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِنَفْسُكِ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كِ'
}
STOPWORDS[u'أوبنفسه'] = {
    'word': u'أوبنفسه',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِنَفْسُه',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'ه'
}
STOPWORDS[u'أوبنفسكم'] = {
    'word': u'أوبنفسكم',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِنَفْسُكُمْ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أوبنفسكن'] = {
    'word': u'أوبنفسكن',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِنَفْسُكُنَّ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أوبنفسها'] = {
    'word': u'أوبنفسها',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِنَفْسُهَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هَا'
}
STOPWORDS[u'أوبنفسهم'] = {
    'word': u'أوبنفسهم',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِنَفْسُهُمْ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أوبنفسهن'] = {
    'word': u'أوبنفسهن',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِنَفْسُهُنَّ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أوبنفسنا'] = {
    'word': u'أوبنفسنا',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِنَفْسُنَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'نَا'
}
STOPWORDS[u'أوبنفسكما'] = {
    'word': u'أوبنفسكما',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِنَفْسُكُمَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أوبنفسهما'] = {
    'word': u'أوبنفسهما',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِنَفْسُهُمَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أفبنفسي'] = {
    'word': u'أفبنفسي',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِنَفْسُِي',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'ِي'
}
STOPWORDS[u'أفبنفسك'] = {
    'word': u'أفبنفسك',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِنَفْسُكَ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كَ'
}
STOPWORDS[u'أفبنفسك'] = {
    'word': u'أفبنفسك',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِنَفْسُكِ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كِ'
}
STOPWORDS[u'أفبنفسه'] = {
    'word': u'أفبنفسه',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِنَفْسُه',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'ه'
}
STOPWORDS[u'أفبنفسكم'] = {
    'word': u'أفبنفسكم',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِنَفْسُكُمْ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أفبنفسكن'] = {
    'word': u'أفبنفسكن',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِنَفْسُكُنَّ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أفبنفسها'] = {
    'word': u'أفبنفسها',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِنَفْسُهَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هَا'
}
STOPWORDS[u'أفبنفسهم'] = {
    'word': u'أفبنفسهم',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِنَفْسُهُمْ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أفبنفسهن'] = {
    'word': u'أفبنفسهن',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِنَفْسُهُنَّ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أفبنفسنا'] = {
    'word': u'أفبنفسنا',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِنَفْسُنَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'نَا'
}
STOPWORDS[u'أفبنفسكما'] = {
    'word': u'أفبنفسكما',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِنَفْسُكُمَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أفبنفسهما'] = {
    'word': u'أفبنفسهما',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِنَفْسُهُمَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أوكنفسي'] = {
    'word': u'أوكنفسي',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَنَفْسُِي',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'ِي'
}
STOPWORDS[u'أوكنفسك'] = {
    'word': u'أوكنفسك',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَنَفْسُكَ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كَ'
}
STOPWORDS[u'أوكنفسك'] = {
    'word': u'أوكنفسك',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَنَفْسُكِ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كِ'
}
STOPWORDS[u'أوكنفسه'] = {
    'word': u'أوكنفسه',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَنَفْسُه',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'ه'
}
STOPWORDS[u'أوكنفسكم'] = {
    'word': u'أوكنفسكم',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَنَفْسُكُمْ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أوكنفسكن'] = {
    'word': u'أوكنفسكن',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَنَفْسُكُنَّ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أوكنفسها'] = {
    'word': u'أوكنفسها',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَنَفْسُهَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هَا'
}
STOPWORDS[u'أوكنفسهم'] = {
    'word': u'أوكنفسهم',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَنَفْسُهُمْ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أوكنفسهن'] = {
    'word': u'أوكنفسهن',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَنَفْسُهُنَّ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أوكنفسنا'] = {
    'word': u'أوكنفسنا',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَنَفْسُنَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'نَا'
}
STOPWORDS[u'أوكنفسكما'] = {
    'word': u'أوكنفسكما',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَنَفْسُكُمَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أوكنفسهما'] = {
    'word': u'أوكنفسهما',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَكَنَفْسُهُمَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أفكنفسي'] = {
    'word': u'أفكنفسي',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَنَفْسُِي',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'ِي'
}
STOPWORDS[u'أفكنفسك'] = {
    'word': u'أفكنفسك',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَنَفْسُكَ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كَ'
}
STOPWORDS[u'أفكنفسك'] = {
    'word': u'أفكنفسك',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَنَفْسُكِ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كِ'
}
STOPWORDS[u'أفكنفسه'] = {
    'word': u'أفكنفسه',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَنَفْسُه',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'ه'
}
STOPWORDS[u'أفكنفسكم'] = {
    'word': u'أفكنفسكم',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَنَفْسُكُمْ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أفكنفسكن'] = {
    'word': u'أفكنفسكن',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَنَفْسُكُنَّ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أفكنفسها'] = {
    'word': u'أفكنفسها',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَنَفْسُهَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هَا'
}
STOPWORDS[u'أفكنفسهم'] = {
    'word': u'أفكنفسهم',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَنَفْسُهُمْ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أفكنفسهن'] = {
    'word': u'أفكنفسهن',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَنَفْسُهُنَّ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أفكنفسنا'] = {
    'word': u'أفكنفسنا',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَنَفْسُنَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'نَا'
}
STOPWORDS[u'أفكنفسكما'] = {
    'word': u'أفكنفسكما',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَنَفْسُكُمَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أفكنفسهما'] = {
    'word': u'أفكنفسهما',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَكَنَفْسُهُمَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أولنفسي'] = {
    'word': u'أولنفسي',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِنَفْسُِي',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'ِي'
}
STOPWORDS[u'أولنفسك'] = {
    'word': u'أولنفسك',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِنَفْسُكَ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كَ'
}
STOPWORDS[u'أولنفسك'] = {
    'word': u'أولنفسك',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِنَفْسُكِ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كِ'
}
STOPWORDS[u'أولنفسه'] = {
    'word': u'أولنفسه',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِنَفْسُه',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'ه'
}
STOPWORDS[u'أولنفسكم'] = {
    'word': u'أولنفسكم',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِنَفْسُكُمْ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أولنفسكن'] = {
    'word': u'أولنفسكن',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِنَفْسُكُنَّ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أولنفسها'] = {
    'word': u'أولنفسها',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِنَفْسُهَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هَا'
}
STOPWORDS[u'أولنفسهم'] = {
    'word': u'أولنفسهم',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِنَفْسُهُمْ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أولنفسهن'] = {
    'word': u'أولنفسهن',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِنَفْسُهُنَّ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أولنفسنا'] = {
    'word': u'أولنفسنا',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِنَفْسُنَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'نَا'
}
STOPWORDS[u'أولنفسكما'] = {
    'word': u'أولنفسكما',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِنَفْسُكُمَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أولنفسهما'] = {
    'word': u'أولنفسهما',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلِنَفْسُهُمَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أفلنفسي'] = {
    'word': u'أفلنفسي',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِنَفْسُِي',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'ِي'
}
STOPWORDS[u'أفلنفسك'] = {
    'word': u'أفلنفسك',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِنَفْسُكَ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كَ'
}
STOPWORDS[u'أفلنفسك'] = {
    'word': u'أفلنفسك',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِنَفْسُكِ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كِ'
}
STOPWORDS[u'أفلنفسه'] = {
    'word': u'أفلنفسه',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِنَفْسُه',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'ه'
}
STOPWORDS[u'أفلنفسكم'] = {
    'word': u'أفلنفسكم',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِنَفْسُكُمْ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أفلنفسكن'] = {
    'word': u'أفلنفسكن',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِنَفْسُكُنَّ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أفلنفسها'] = {
    'word': u'أفلنفسها',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِنَفْسُهَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هَا'
}
STOPWORDS[u'أفلنفسهم'] = {
    'word': u'أفلنفسهم',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِنَفْسُهُمْ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أفلنفسهن'] = {
    'word': u'أفلنفسهن',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِنَفْسُهُنَّ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أفلنفسنا'] = {
    'word': u'أفلنفسنا',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِنَفْسُنَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'نَا'
}
STOPWORDS[u'أفلنفسكما'] = {
    'word': u'أفلنفسكما',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِنَفْسُكُمَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أفلنفسهما'] = {
    'word': u'أفلنفسهما',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:توكيد:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلِنَفْسُهُمَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أولنفسي'] = {
    'word': u'أولنفسي',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَنَفْسُِي',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'ِي'
}
STOPWORDS[u'أولنفسك'] = {
    'word': u'أولنفسك',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَنَفْسُكَ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كَ'
}
STOPWORDS[u'أولنفسك'] = {
    'word': u'أولنفسك',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَنَفْسُكِ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كِ'
}
STOPWORDS[u'أولنفسه'] = {
    'word': u'أولنفسه',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَنَفْسُه',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'ه'
}
STOPWORDS[u'أولنفسكم'] = {
    'word': u'أولنفسكم',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَنَفْسُكُمْ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أولنفسكن'] = {
    'word': u'أولنفسكن',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَنَفْسُكُنَّ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أولنفسها'] = {
    'word': u'أولنفسها',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَنَفْسُهَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هَا'
}
STOPWORDS[u'أولنفسهم'] = {
    'word': u'أولنفسهم',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَنَفْسُهُمْ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أولنفسهن'] = {
    'word': u'أولنفسهن',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَنَفْسُهُنَّ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أولنفسنا'] = {
    'word': u'أولنفسنا',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَنَفْسُنَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'نَا'
}
STOPWORDS[u'أولنفسكما'] = {
    'word': u'أولنفسكما',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَنَفْسُكُمَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أولنفسهما'] = {
    'word': u'أولنفسهما',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَنَفْسُهُمَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أفلنفسي'] = {
    'word': u'أفلنفسي',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَنَفْسُِي',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'ِي'
}
STOPWORDS[u'أفلنفسك'] = {
    'word': u'أفلنفسك',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَنَفْسُكَ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كَ'
}
STOPWORDS[u'أفلنفسك'] = {
    'word': u'أفلنفسك',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَنَفْسُكِ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كِ'
}
STOPWORDS[u'أفلنفسه'] = {
    'word': u'أفلنفسه',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَنَفْسُه',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'ه'
}
STOPWORDS[u'أفلنفسكم'] = {
    'word': u'أفلنفسكم',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَنَفْسُكُمْ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أفلنفسكن'] = {
    'word': u'أفلنفسكن',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَنَفْسُكُنَّ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أفلنفسها'] = {
    'word': u'أفلنفسها',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَنَفْسُهَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هَا'
}
STOPWORDS[u'أفلنفسهم'] = {
    'word': u'أفلنفسهم',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَنَفْسُهُمْ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أفلنفسهن'] = {
    'word': u'أفلنفسهن',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَنَفْسُهُنَّ',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أفلنفسنا'] = {
    'word': u'أفلنفسنا',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَنَفْسُنَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'نَا'
}
STOPWORDS[u'أفلنفسكما'] = {
    'word': u'أفلنفسكما',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَنَفْسُكُمَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أفلنفسهما'] = {
    'word': u'أفلنفسهما',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَنَفْسُهُمَا',
    'stem': u'نَفْسُ',
    'type': u'STOPWORD',
    'original': u'نَفْسُ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ء'] = {
    'word': u'ء',
    'procletic': u'',
    'tags': u'أداة:حرف ابجدي',
    'vocalized': u'ء',
    'stem': u'ء',
    'type': u'STOPWORD',
    'original': u'ء',
    'encletic': u''
}
STOPWORDS[u'ؤ'] = {
    'word': u'ؤ',
    'procletic': u'',
    'tags': u'أداة:حرف ابجدي',
    'vocalized': u'ؤ',
    'stem': u'ؤ',
    'type': u'STOPWORD',
    'original': u'ؤ',
    'encletic': u''
}
STOPWORDS[u'ئ'] = {
    'word': u'ئ',
    'procletic': u'',
    'tags': u'أداة:حرف ابجدي',
    'vocalized': u'ئ',
    'stem': u'ئ',
    'type': u'STOPWORD',
    'original': u'ئ',
    'encletic': u''
}
STOPWORDS[u'آ'] = {
    'word': u'آ',
    'procletic': u'',
    'tags': u'أداة:حرف ابجدي',
    'vocalized': u'آ',
    'stem': u'آ',
    'type': u'STOPWORD',
    'original': u'آ',
    'encletic': u''
}
STOPWORDS[u'أ'] = {
    'word': u'أ',
    'procletic': u'',
    'tags': u'أداة:حرف ابجدي',
    'vocalized': u'أ',
    'stem': u'أ',
    'type': u'STOPWORD',
    'original': u'أ',
    'encletic': u''
}
STOPWORDS[u'أ'] = {
    'word': u'أ',
    'procletic': u'',
    'tags': u'أداة:حرف ابجدي',
    'vocalized': u'أ',
    'stem': u'أ',
    'type': u'STOPWORD',
    'original': u'أ',
    'encletic': u''
}
STOPWORDS[u'ب'] = {
    'word': u'ب',
    'procletic': u'',
    'tags': u'أداة:حرف ابجدي',
    'vocalized': u'ب',
    'stem': u'ب',
    'type': u'STOPWORD',
    'original': u'ب',
    'encletic': u''
}
STOPWORDS[u'ت'] = {
    'word': u'ت',
    'procletic': u'',
    'tags': u'أداة:حرف ابجدي',
    'vocalized': u'ت',
    'stem': u'ت',
    'type': u'STOPWORD',
    'original': u'ت',
    'encletic': u''
}
STOPWORDS[u'ة'] = {
    'word': u'ة',
    'procletic': u'',
    'tags': u'أداة:حرف ابجدي',
    'vocalized': u'ة',
    'stem': u'ة',
    'type': u'STOPWORD',
    'original': u'ة',
    'encletic': u''
}
STOPWORDS[u'ث'] = {
    'word': u'ث',
    'procletic': u'',
    'tags': u'أداة:حرف ابجدي',
    'vocalized': u'ث',
    'stem': u'ث',
    'type': u'STOPWORD',
    'original': u'ث',
    'encletic': u''
}
STOPWORDS[u'ج'] = {
    'word': u'ج',
    'procletic': u'',
    'tags': u'أداة:حرف ابجدي',
    'vocalized': u'ج',
    'stem': u'ج',
    'type': u'STOPWORD',
    'original': u'ج',
    'encletic': u''
}
STOPWORDS[u'ح'] = {
    'word': u'ح',
    'procletic': u'',
    'tags': u'أداة:حرف ابجدي',
    'vocalized': u'ح',
    'stem': u'ح',
    'type': u'STOPWORD',
    'original': u'ح',
    'encletic': u''
}
STOPWORDS[u'خ'] = {
    'word': u'خ',
    'procletic': u'',
    'tags': u'أداة:حرف ابجدي',
    'vocalized': u'خ',
    'stem': u'خ',
    'type': u'STOPWORD',
    'original': u'خ',
    'encletic': u''
}
STOPWORDS[u'د'] = {
    'word': u'د',
    'procletic': u'',
    'tags': u'أداة:حرف ابجدي',
    'vocalized': u'د',
    'stem': u'د',
    'type': u'STOPWORD',
    'original': u'د',
    'encletic': u''
}
STOPWORDS[u'ذ'] = {
    'word': u'ذ',
    'procletic': u'',
    'tags': u'أداة:حرف ابجدي',
    'vocalized': u'ذ',
    'stem': u'ذ',
    'type': u'STOPWORD',
    'original': u'ذ',
    'encletic': u''
}
STOPWORDS[u'ر'] = {
    'word': u'ر',
    'procletic': u'',
    'tags': u'أداة:حرف ابجدي',
    'vocalized': u'ر',
    'stem': u'ر',
    'type': u'STOPWORD',
    'original': u'ر',
    'encletic': u''
}
STOPWORDS[u'ز'] = {
    'word': u'ز',
    'procletic': u'',
    'tags': u'أداة:حرف ابجدي',
    'vocalized': u'ز',
    'stem': u'ز',
    'type': u'STOPWORD',
    'original': u'ز',
    'encletic': u''
}
STOPWORDS[u'س'] = {
    'word': u'س',
    'procletic': u'',
    'tags': u'أداة:حرف ابجدي',
    'vocalized': u'س',
    'stem': u'س',
    'type': u'STOPWORD',
    'original': u'س',
    'encletic': u''
}
STOPWORDS[u'ش'] = {
    'word': u'ش',
    'procletic': u'',
    'tags': u'أداة:حرف ابجدي',
    'vocalized': u'ش',
    'stem': u'ش',
    'type': u'STOPWORD',
    'original': u'ش',
    'encletic': u''
}
STOPWORDS[u'ص'] = {
    'word': u'ص',
    'procletic': u'',
    'tags': u'أداة:حرف ابجدي',
    'vocalized': u'ص',
    'stem': u'ص',
    'type': u'STOPWORD',
    'original': u'ص',
    'encletic': u''
}
STOPWORDS[u'ض'] = {
    'word': u'ض',
    'procletic': u'',
    'tags': u'أداة:حرف ابجدي',
    'vocalized': u'ض',
    'stem': u'ض',
    'type': u'STOPWORD',
    'original': u'ض',
    'encletic': u''
}
STOPWORDS[u'ط'] = {
    'word': u'ط',
    'procletic': u'',
    'tags': u'أداة:حرف ابجدي',
    'vocalized': u'ط',
    'stem': u'ط',
    'type': u'STOPWORD',
    'original': u'ط',
    'encletic': u''
}
STOPWORDS[u'ظ'] = {
    'word': u'ظ',
    'procletic': u'',
    'tags': u'أداة:حرف ابجدي',
    'vocalized': u'ظ',
    'stem': u'ظ',
    'type': u'STOPWORD',
    'original': u'ظ',
    'encletic': u''
}
STOPWORDS[u'ع'] = {
    'word': u'ع',
    'procletic': u'',
    'tags': u'أداة:حرف ابجدي',
    'vocalized': u'ع',
    'stem': u'ع',
    'type': u'STOPWORD',
    'original': u'ع',
    'encletic': u''
}
STOPWORDS[u'غ'] = {
    'word': u'غ',
    'procletic': u'',
    'tags': u'أداة:حرف ابجدي',
    'vocalized': u'غ',
    'stem': u'غ',
    'type': u'STOPWORD',
    'original': u'غ',
    'encletic': u''
}
STOPWORDS[u'ف'] = {
    'word': u'ف',
    'procletic': u'',
    'tags': u'أداة:حرف ابجدي',
    'vocalized': u'ف',
    'stem': u'ف',
    'type': u'STOPWORD',
    'original': u'ف',
    'encletic': u''
}
STOPWORDS[u'ق'] = {
    'word': u'ق',
    'procletic': u'',
    'tags': u'أداة:حرف ابجدي',
    'vocalized': u'ق',
    'stem': u'ق',
    'type': u'STOPWORD',
    'original': u'ق',
    'encletic': u''
}
STOPWORDS[u'ك'] = {
    'word': u'ك',
    'procletic': u'',
    'tags': u'أداة:حرف ابجدي',
    'vocalized': u'ك',
    'stem': u'ك',
    'type': u'STOPWORD',
    'original': u'ك',
    'encletic': u''
}
STOPWORDS[u'ل'] = {
    'word': u'ل',
    'procletic': u'',
    'tags': u'أداة:حرف ابجدي',
    'vocalized': u'ل',
    'stem': u'ل',
    'type': u'STOPWORD',
    'original': u'ل',
    'encletic': u''
}
STOPWORDS[u'م'] = {
    'word': u'م',
    'procletic': u'',
    'tags': u'أداة:حرف ابجدي',
    'vocalized': u'م',
    'stem': u'م',
    'type': u'STOPWORD',
    'original': u'م',
    'encletic': u''
}
STOPWORDS[u'ن'] = {
    'word': u'ن',
    'procletic': u'',
    'tags': u'أداة:حرف ابجدي',
    'vocalized': u'ن',
    'stem': u'ن',
    'type': u'STOPWORD',
    'original': u'ن',
    'encletic': u''
}
STOPWORDS[u'ه'] = {
    'word': u'ه',
    'procletic': u'',
    'tags': u'أداة:حرف ابجدي',
    'vocalized': u'ه',
    'stem': u'ه',
    'type': u'STOPWORD',
    'original': u'ه',
    'encletic': u''
}
STOPWORDS[u'و'] = {
    'word': u'و',
    'procletic': u'',
    'tags': u'أداة:حرف ابجدي',
    'vocalized': u'و',
    'stem': u'و',
    'type': u'STOPWORD',
    'original': u'و',
    'encletic': u''
}
STOPWORDS[u'ى'] = {
    'word': u'ى',
    'procletic': u'',
    'tags': u'أداة:حرف ابجدي',
    'vocalized': u'ى',
    'stem': u'ى',
    'type': u'STOPWORD',
    'original': u'ى',
    'encletic': u''
}
STOPWORDS[u'ي'] = {
    'word': u'ي',
    'procletic': u'',
    'tags': u'أداة:حرف ابجدي',
    'vocalized': u'ي',
    'stem': u'ي',
    'type': u'STOPWORD',
    'original': u'ي',
    'encletic': u''
}
STOPWORDS[u'إلا'] = {
    'word': u'إلّا',
    'procletic': u'',
    'tags': u'أداة:حرف استثناء',
    'vocalized': u'إلّا',
    'stem': u'إلّا',
    'type': u'STOPWORD',
    'original': u'إلّا',
    'encletic': u''
}
STOPWORDS[u'وإلا'] = {
    'word': u'وإلا',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف استثناء:معطوف',
    'vocalized': u'وَإلّا',
    'stem': u'إلّا',
    'type': u'STOPWORD',
    'original': u'إلّا',
    'encletic': u''
}
STOPWORDS[u'فإلا'] = {
    'word': u'فإلا',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف استثناء:معطوف',
    'vocalized': u'فَإلّا',
    'stem': u'إلّا',
    'type': u'STOPWORD',
    'original': u'إلّا',
    'encletic': u''
}
STOPWORDS[u'حاشا'] = {
    'word': u'حَاشَا',
    'procletic': u'',
    'tags': u'أداة:حرف استثناء',
    'vocalized': u'حَاشَا',
    'stem': u'حَاشَا',
    'type': u'STOPWORD',
    'original': u'حَاشَا',
    'encletic': u''
}
STOPWORDS[u'وحاشا'] = {
    'word': u'وحاشا',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف استثناء:معطوف',
    'vocalized': u'وَحَاشَا',
    'stem': u'حَاشَا',
    'type': u'STOPWORD',
    'original': u'حَاشَا',
    'encletic': u''
}
STOPWORDS[u'فحاشا'] = {
    'word': u'فحاشا',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف استثناء:معطوف',
    'vocalized': u'فَحَاشَا',
    'stem': u'حَاشَا',
    'type': u'STOPWORD',
    'original': u'حَاشَا',
    'encletic': u''
}
STOPWORDS[u'حاشاي'] = {
    'word': u'حاشاي',
    'procletic': u'',
    'tags': u'أداة:حرف استثناء:مضاف',
    'vocalized': u'حَاشَاِي',
    'stem': u'حَاشَا',
    'type': u'STOPWORD',
    'original': u'حَاشَا',
    'encletic': u'ِي'
}
STOPWORDS[u'حاشاك'] = {
    'word': u'حاشاك',
    'procletic': u'',
    'tags': u'أداة:حرف استثناء:مضاف',
    'vocalized': u'حَاشَاكَ',
    'stem': u'حَاشَا',
    'type': u'STOPWORD',
    'original': u'حَاشَا',
    'encletic': u'كَ'
}
STOPWORDS[u'حاشاك'] = {
    'word': u'حاشاك',
    'procletic': u'',
    'tags': u'أداة:حرف استثناء:مضاف',
    'vocalized': u'حَاشَاكِ',
    'stem': u'حَاشَا',
    'type': u'STOPWORD',
    'original': u'حَاشَا',
    'encletic': u'كِ'
}
STOPWORDS[u'حاشاه'] = {
    'word': u'حاشاه',
    'procletic': u'',
    'tags': u'أداة:حرف استثناء:مضاف',
    'vocalized': u'حَاشَاه',
    'stem': u'حَاشَا',
    'type': u'STOPWORD',
    'original': u'حَاشَا',
    'encletic': u'ه'
}
STOPWORDS[u'حاشاكم'] = {
    'word': u'حاشاكم',
    'procletic': u'',
    'tags': u'أداة:حرف استثناء:مضاف',
    'vocalized': u'حَاشَاكُمْ',
    'stem': u'حَاشَا',
    'type': u'STOPWORD',
    'original': u'حَاشَا',
    'encletic': u'كُمْ'
}
STOPWORDS[u'حاشاكن'] = {
    'word': u'حاشاكن',
    'procletic': u'',
    'tags': u'أداة:حرف استثناء:مضاف',
    'vocalized': u'حَاشَاكُنَّ',
    'stem': u'حَاشَا',
    'type': u'STOPWORD',
    'original': u'حَاشَا',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'حاشاها'] = {
    'word': u'حاشاها',
    'procletic': u'',
    'tags': u'أداة:حرف استثناء:مضاف',
    'vocalized': u'حَاشَاهَا',
    'stem': u'حَاشَا',
    'type': u'STOPWORD',
    'original': u'حَاشَا',
    'encletic': u'هَا'
}
STOPWORDS[u'حاشاهم'] = {
    'word': u'حاشاهم',
    'procletic': u'',
    'tags': u'أداة:حرف استثناء:مضاف',
    'vocalized': u'حَاشَاهُمْ',
    'stem': u'حَاشَا',
    'type': u'STOPWORD',
    'original': u'حَاشَا',
    'encletic': u'هُمْ'
}
STOPWORDS[u'حاشاهن'] = {
    'word': u'حاشاهن',
    'procletic': u'',
    'tags': u'أداة:حرف استثناء:مضاف',
    'vocalized': u'حَاشَاهُنَّ',
    'stem': u'حَاشَا',
    'type': u'STOPWORD',
    'original': u'حَاشَا',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'حاشانا'] = {
    'word': u'حاشانا',
    'procletic': u'',
    'tags': u'أداة:حرف استثناء:مضاف',
    'vocalized': u'حَاشَانَا',
    'stem': u'حَاشَا',
    'type': u'STOPWORD',
    'original': u'حَاشَا',
    'encletic': u'نَا'
}
STOPWORDS[u'حاشاكما'] = {
    'word': u'حاشاكما',
    'procletic': u'',
    'tags': u'أداة:حرف استثناء:مضاف',
    'vocalized': u'حَاشَاكُمَا',
    'stem': u'حَاشَا',
    'type': u'STOPWORD',
    'original': u'حَاشَا',
    'encletic': u'كُمَا'
}
STOPWORDS[u'حاشاهما'] = {
    'word': u'حاشاهما',
    'procletic': u'',
    'tags': u'أداة:حرف استثناء:مضاف',
    'vocalized': u'حَاشَاهُمَا',
    'stem': u'حَاشَا',
    'type': u'STOPWORD',
    'original': u'حَاشَا',
    'encletic': u'هُمَا'
}
STOPWORDS[u'وحاشاي'] = {
    'word': u'وحاشاي',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف استثناء:معطوف:مضاف',
    'vocalized': u'وَحَاشَاِي',
    'stem': u'حَاشَا',
    'type': u'STOPWORD',
    'original': u'حَاشَا',
    'encletic': u'ِي'
}
STOPWORDS[u'وحاشاك'] = {
    'word': u'وحاشاك',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف استثناء:معطوف:مضاف',
    'vocalized': u'وَحَاشَاكَ',
    'stem': u'حَاشَا',
    'type': u'STOPWORD',
    'original': u'حَاشَا',
    'encletic': u'كَ'
}
STOPWORDS[u'وحاشاك'] = {
    'word': u'وحاشاك',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف استثناء:معطوف:مضاف',
    'vocalized': u'وَحَاشَاكِ',
    'stem': u'حَاشَا',
    'type': u'STOPWORD',
    'original': u'حَاشَا',
    'encletic': u'كِ'
}
STOPWORDS[u'وحاشاه'] = {
    'word': u'وحاشاه',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف استثناء:معطوف:مضاف',
    'vocalized': u'وَحَاشَاه',
    'stem': u'حَاشَا',
    'type': u'STOPWORD',
    'original': u'حَاشَا',
    'encletic': u'ه'
}
STOPWORDS[u'وحاشاكم'] = {
    'word': u'وحاشاكم',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف استثناء:معطوف:مضاف',
    'vocalized': u'وَحَاشَاكُمْ',
    'stem': u'حَاشَا',
    'type': u'STOPWORD',
    'original': u'حَاشَا',
    'encletic': u'كُمْ'
}
STOPWORDS[u'وحاشاكن'] = {
    'word': u'وحاشاكن',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف استثناء:معطوف:مضاف',
    'vocalized': u'وَحَاشَاكُنَّ',
    'stem': u'حَاشَا',
    'type': u'STOPWORD',
    'original': u'حَاشَا',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'وحاشاها'] = {
    'word': u'وحاشاها',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف استثناء:معطوف:مضاف',
    'vocalized': u'وَحَاشَاهَا',
    'stem': u'حَاشَا',
    'type': u'STOPWORD',
    'original': u'حَاشَا',
    'encletic': u'هَا'
}
STOPWORDS[u'وحاشاهم'] = {
    'word': u'وحاشاهم',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف استثناء:معطوف:مضاف',
    'vocalized': u'وَحَاشَاهُمْ',
    'stem': u'حَاشَا',
    'type': u'STOPWORD',
    'original': u'حَاشَا',
    'encletic': u'هُمْ'
}
STOPWORDS[u'وحاشاهن'] = {
    'word': u'وحاشاهن',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف استثناء:معطوف:مضاف',
    'vocalized': u'وَحَاشَاهُنَّ',
    'stem': u'حَاشَا',
    'type': u'STOPWORD',
    'original': u'حَاشَا',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'وحاشانا'] = {
    'word': u'وحاشانا',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف استثناء:معطوف:مضاف',
    'vocalized': u'وَحَاشَانَا',
    'stem': u'حَاشَا',
    'type': u'STOPWORD',
    'original': u'حَاشَا',
    'encletic': u'نَا'
}
STOPWORDS[u'وحاشاكما'] = {
    'word': u'وحاشاكما',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف استثناء:معطوف:مضاف',
    'vocalized': u'وَحَاشَاكُمَا',
    'stem': u'حَاشَا',
    'type': u'STOPWORD',
    'original': u'حَاشَا',
    'encletic': u'كُمَا'
}
STOPWORDS[u'وحاشاهما'] = {
    'word': u'وحاشاهما',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف استثناء:معطوف:مضاف',
    'vocalized': u'وَحَاشَاهُمَا',
    'stem': u'حَاشَا',
    'type': u'STOPWORD',
    'original': u'حَاشَا',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فحاشاي'] = {
    'word': u'فحاشاي',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف استثناء:معطوف:مضاف',
    'vocalized': u'فَحَاشَاِي',
    'stem': u'حَاشَا',
    'type': u'STOPWORD',
    'original': u'حَاشَا',
    'encletic': u'ِي'
}
STOPWORDS[u'فحاشاك'] = {
    'word': u'فحاشاك',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف استثناء:معطوف:مضاف',
    'vocalized': u'فَحَاشَاكَ',
    'stem': u'حَاشَا',
    'type': u'STOPWORD',
    'original': u'حَاشَا',
    'encletic': u'كَ'
}
STOPWORDS[u'فحاشاك'] = {
    'word': u'فحاشاك',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف استثناء:معطوف:مضاف',
    'vocalized': u'فَحَاشَاكِ',
    'stem': u'حَاشَا',
    'type': u'STOPWORD',
    'original': u'حَاشَا',
    'encletic': u'كِ'
}
STOPWORDS[u'فحاشاه'] = {
    'word': u'فحاشاه',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف استثناء:معطوف:مضاف',
    'vocalized': u'فَحَاشَاه',
    'stem': u'حَاشَا',
    'type': u'STOPWORD',
    'original': u'حَاشَا',
    'encletic': u'ه'
}
STOPWORDS[u'فحاشاكم'] = {
    'word': u'فحاشاكم',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف استثناء:معطوف:مضاف',
    'vocalized': u'فَحَاشَاكُمْ',
    'stem': u'حَاشَا',
    'type': u'STOPWORD',
    'original': u'حَاشَا',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فحاشاكن'] = {
    'word': u'فحاشاكن',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف استثناء:معطوف:مضاف',
    'vocalized': u'فَحَاشَاكُنَّ',
    'stem': u'حَاشَا',
    'type': u'STOPWORD',
    'original': u'حَاشَا',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فحاشاها'] = {
    'word': u'فحاشاها',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف استثناء:معطوف:مضاف',
    'vocalized': u'فَحَاشَاهَا',
    'stem': u'حَاشَا',
    'type': u'STOPWORD',
    'original': u'حَاشَا',
    'encletic': u'هَا'
}
STOPWORDS[u'فحاشاهم'] = {
    'word': u'فحاشاهم',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف استثناء:معطوف:مضاف',
    'vocalized': u'فَحَاشَاهُمْ',
    'stem': u'حَاشَا',
    'type': u'STOPWORD',
    'original': u'حَاشَا',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فحاشاهن'] = {
    'word': u'فحاشاهن',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف استثناء:معطوف:مضاف',
    'vocalized': u'فَحَاشَاهُنَّ',
    'stem': u'حَاشَا',
    'type': u'STOPWORD',
    'original': u'حَاشَا',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فحاشانا'] = {
    'word': u'فحاشانا',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف استثناء:معطوف:مضاف',
    'vocalized': u'فَحَاشَانَا',
    'stem': u'حَاشَا',
    'type': u'STOPWORD',
    'original': u'حَاشَا',
    'encletic': u'نَا'
}
STOPWORDS[u'فحاشاكما'] = {
    'word': u'فحاشاكما',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف استثناء:معطوف:مضاف',
    'vocalized': u'فَحَاشَاكُمَا',
    'stem': u'حَاشَا',
    'type': u'STOPWORD',
    'original': u'حَاشَا',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فحاشاهما'] = {
    'word': u'فحاشاهما',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف استثناء:معطوف:مضاف',
    'vocalized': u'فَحَاشَاهُمَا',
    'stem': u'حَاشَا',
    'type': u'STOPWORD',
    'original': u'حَاشَا',
    'encletic': u'هُمَا'
}
STOPWORDS[u'خلا'] = {
    'word': u'خَلَا',
    'procletic': u'',
    'tags': u'أداة:حرف استثناء',
    'vocalized': u'خَلَا',
    'stem': u'خَلَا',
    'type': u'STOPWORD',
    'original': u'خَلَا',
    'encletic': u''
}
STOPWORDS[u'وخلا'] = {
    'word': u'وخلا',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف استثناء:معطوف',
    'vocalized': u'وَخَلَا',
    'stem': u'خَلَا',
    'type': u'STOPWORD',
    'original': u'خَلَا',
    'encletic': u''
}
STOPWORDS[u'فخلا'] = {
    'word': u'فخلا',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف استثناء:معطوف',
    'vocalized': u'فَخَلَا',
    'stem': u'خَلَا',
    'type': u'STOPWORD',
    'original': u'خَلَا',
    'encletic': u''
}
STOPWORDS[u'خلاي'] = {
    'word': u'خلاي',
    'procletic': u'',
    'tags': u'أداة:حرف استثناء:مضاف',
    'vocalized': u'خَلَاِي',
    'stem': u'خَلَا',
    'type': u'STOPWORD',
    'original': u'خَلَا',
    'encletic': u'ِي'
}
STOPWORDS[u'خلاك'] = {
    'word': u'خلاك',
    'procletic': u'',
    'tags': u'أداة:حرف استثناء:مضاف',
    'vocalized': u'خَلَاكَ',
    'stem': u'خَلَا',
    'type': u'STOPWORD',
    'original': u'خَلَا',
    'encletic': u'كَ'
}
STOPWORDS[u'خلاك'] = {
    'word': u'خلاك',
    'procletic': u'',
    'tags': u'أداة:حرف استثناء:مضاف',
    'vocalized': u'خَلَاكِ',
    'stem': u'خَلَا',
    'type': u'STOPWORD',
    'original': u'خَلَا',
    'encletic': u'كِ'
}
STOPWORDS[u'خلاه'] = {
    'word': u'خلاه',
    'procletic': u'',
    'tags': u'أداة:حرف استثناء:مضاف',
    'vocalized': u'خَلَاه',
    'stem': u'خَلَا',
    'type': u'STOPWORD',
    'original': u'خَلَا',
    'encletic': u'ه'
}
STOPWORDS[u'خلاكم'] = {
    'word': u'خلاكم',
    'procletic': u'',
    'tags': u'أداة:حرف استثناء:مضاف',
    'vocalized': u'خَلَاكُمْ',
    'stem': u'خَلَا',
    'type': u'STOPWORD',
    'original': u'خَلَا',
    'encletic': u'كُمْ'
}
STOPWORDS[u'خلاكن'] = {
    'word': u'خلاكن',
    'procletic': u'',
    'tags': u'أداة:حرف استثناء:مضاف',
    'vocalized': u'خَلَاكُنَّ',
    'stem': u'خَلَا',
    'type': u'STOPWORD',
    'original': u'خَلَا',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'خلاها'] = {
    'word': u'خلاها',
    'procletic': u'',
    'tags': u'أداة:حرف استثناء:مضاف',
    'vocalized': u'خَلَاهَا',
    'stem': u'خَلَا',
    'type': u'STOPWORD',
    'original': u'خَلَا',
    'encletic': u'هَا'
}
STOPWORDS[u'خلاهم'] = {
    'word': u'خلاهم',
    'procletic': u'',
    'tags': u'أداة:حرف استثناء:مضاف',
    'vocalized': u'خَلَاهُمْ',
    'stem': u'خَلَا',
    'type': u'STOPWORD',
    'original': u'خَلَا',
    'encletic': u'هُمْ'
}
STOPWORDS[u'خلاهن'] = {
    'word': u'خلاهن',
    'procletic': u'',
    'tags': u'أداة:حرف استثناء:مضاف',
    'vocalized': u'خَلَاهُنَّ',
    'stem': u'خَلَا',
    'type': u'STOPWORD',
    'original': u'خَلَا',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'خلانا'] = {
    'word': u'خلانا',
    'procletic': u'',
    'tags': u'أداة:حرف استثناء:مضاف',
    'vocalized': u'خَلَانَا',
    'stem': u'خَلَا',
    'type': u'STOPWORD',
    'original': u'خَلَا',
    'encletic': u'نَا'
}
STOPWORDS[u'خلاكما'] = {
    'word': u'خلاكما',
    'procletic': u'',
    'tags': u'أداة:حرف استثناء:مضاف',
    'vocalized': u'خَلَاكُمَا',
    'stem': u'خَلَا',
    'type': u'STOPWORD',
    'original': u'خَلَا',
    'encletic': u'كُمَا'
}
STOPWORDS[u'خلاهما'] = {
    'word': u'خلاهما',
    'procletic': u'',
    'tags': u'أداة:حرف استثناء:مضاف',
    'vocalized': u'خَلَاهُمَا',
    'stem': u'خَلَا',
    'type': u'STOPWORD',
    'original': u'خَلَا',
    'encletic': u'هُمَا'
}
STOPWORDS[u'وخلاي'] = {
    'word': u'وخلاي',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف استثناء:معطوف:مضاف',
    'vocalized': u'وَخَلَاِي',
    'stem': u'خَلَا',
    'type': u'STOPWORD',
    'original': u'خَلَا',
    'encletic': u'ِي'
}
STOPWORDS[u'وخلاك'] = {
    'word': u'وخلاك',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف استثناء:معطوف:مضاف',
    'vocalized': u'وَخَلَاكَ',
    'stem': u'خَلَا',
    'type': u'STOPWORD',
    'original': u'خَلَا',
    'encletic': u'كَ'
}
STOPWORDS[u'وخلاك'] = {
    'word': u'وخلاك',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف استثناء:معطوف:مضاف',
    'vocalized': u'وَخَلَاكِ',
    'stem': u'خَلَا',
    'type': u'STOPWORD',
    'original': u'خَلَا',
    'encletic': u'كِ'
}
STOPWORDS[u'وخلاه'] = {
    'word': u'وخلاه',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف استثناء:معطوف:مضاف',
    'vocalized': u'وَخَلَاه',
    'stem': u'خَلَا',
    'type': u'STOPWORD',
    'original': u'خَلَا',
    'encletic': u'ه'
}
STOPWORDS[u'وخلاكم'] = {
    'word': u'وخلاكم',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف استثناء:معطوف:مضاف',
    'vocalized': u'وَخَلَاكُمْ',
    'stem': u'خَلَا',
    'type': u'STOPWORD',
    'original': u'خَلَا',
    'encletic': u'كُمْ'
}
STOPWORDS[u'وخلاكن'] = {
    'word': u'وخلاكن',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف استثناء:معطوف:مضاف',
    'vocalized': u'وَخَلَاكُنَّ',
    'stem': u'خَلَا',
    'type': u'STOPWORD',
    'original': u'خَلَا',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'وخلاها'] = {
    'word': u'وخلاها',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف استثناء:معطوف:مضاف',
    'vocalized': u'وَخَلَاهَا',
    'stem': u'خَلَا',
    'type': u'STOPWORD',
    'original': u'خَلَا',
    'encletic': u'هَا'
}
STOPWORDS[u'وخلاهم'] = {
    'word': u'وخلاهم',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف استثناء:معطوف:مضاف',
    'vocalized': u'وَخَلَاهُمْ',
    'stem': u'خَلَا',
    'type': u'STOPWORD',
    'original': u'خَلَا',
    'encletic': u'هُمْ'
}
STOPWORDS[u'وخلاهن'] = {
    'word': u'وخلاهن',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف استثناء:معطوف:مضاف',
    'vocalized': u'وَخَلَاهُنَّ',
    'stem': u'خَلَا',
    'type': u'STOPWORD',
    'original': u'خَلَا',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'وخلانا'] = {
    'word': u'وخلانا',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف استثناء:معطوف:مضاف',
    'vocalized': u'وَخَلَانَا',
    'stem': u'خَلَا',
    'type': u'STOPWORD',
    'original': u'خَلَا',
    'encletic': u'نَا'
}
STOPWORDS[u'وخلاكما'] = {
    'word': u'وخلاكما',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف استثناء:معطوف:مضاف',
    'vocalized': u'وَخَلَاكُمَا',
    'stem': u'خَلَا',
    'type': u'STOPWORD',
    'original': u'خَلَا',
    'encletic': u'كُمَا'
}
STOPWORDS[u'وخلاهما'] = {
    'word': u'وخلاهما',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف استثناء:معطوف:مضاف',
    'vocalized': u'وَخَلَاهُمَا',
    'stem': u'خَلَا',
    'type': u'STOPWORD',
    'original': u'خَلَا',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فخلاي'] = {
    'word': u'فخلاي',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف استثناء:معطوف:مضاف',
    'vocalized': u'فَخَلَاِي',
    'stem': u'خَلَا',
    'type': u'STOPWORD',
    'original': u'خَلَا',
    'encletic': u'ِي'
}
STOPWORDS[u'فخلاك'] = {
    'word': u'فخلاك',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف استثناء:معطوف:مضاف',
    'vocalized': u'فَخَلَاكَ',
    'stem': u'خَلَا',
    'type': u'STOPWORD',
    'original': u'خَلَا',
    'encletic': u'كَ'
}
STOPWORDS[u'فخلاك'] = {
    'word': u'فخلاك',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف استثناء:معطوف:مضاف',
    'vocalized': u'فَخَلَاكِ',
    'stem': u'خَلَا',
    'type': u'STOPWORD',
    'original': u'خَلَا',
    'encletic': u'كِ'
}
STOPWORDS[u'فخلاه'] = {
    'word': u'فخلاه',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف استثناء:معطوف:مضاف',
    'vocalized': u'فَخَلَاه',
    'stem': u'خَلَا',
    'type': u'STOPWORD',
    'original': u'خَلَا',
    'encletic': u'ه'
}
STOPWORDS[u'فخلاكم'] = {
    'word': u'فخلاكم',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف استثناء:معطوف:مضاف',
    'vocalized': u'فَخَلَاكُمْ',
    'stem': u'خَلَا',
    'type': u'STOPWORD',
    'original': u'خَلَا',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فخلاكن'] = {
    'word': u'فخلاكن',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف استثناء:معطوف:مضاف',
    'vocalized': u'فَخَلَاكُنَّ',
    'stem': u'خَلَا',
    'type': u'STOPWORD',
    'original': u'خَلَا',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فخلاها'] = {
    'word': u'فخلاها',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف استثناء:معطوف:مضاف',
    'vocalized': u'فَخَلَاهَا',
    'stem': u'خَلَا',
    'type': u'STOPWORD',
    'original': u'خَلَا',
    'encletic': u'هَا'
}
STOPWORDS[u'فخلاهم'] = {
    'word': u'فخلاهم',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف استثناء:معطوف:مضاف',
    'vocalized': u'فَخَلَاهُمْ',
    'stem': u'خَلَا',
    'type': u'STOPWORD',
    'original': u'خَلَا',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فخلاهن'] = {
    'word': u'فخلاهن',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف استثناء:معطوف:مضاف',
    'vocalized': u'فَخَلَاهُنَّ',
    'stem': u'خَلَا',
    'type': u'STOPWORD',
    'original': u'خَلَا',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فخلانا'] = {
    'word': u'فخلانا',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف استثناء:معطوف:مضاف',
    'vocalized': u'فَخَلَانَا',
    'stem': u'خَلَا',
    'type': u'STOPWORD',
    'original': u'خَلَا',
    'encletic': u'نَا'
}
STOPWORDS[u'فخلاكما'] = {
    'word': u'فخلاكما',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف استثناء:معطوف:مضاف',
    'vocalized': u'فَخَلَاكُمَا',
    'stem': u'خَلَا',
    'type': u'STOPWORD',
    'original': u'خَلَا',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فخلاهما'] = {
    'word': u'فخلاهما',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف استثناء:معطوف:مضاف',
    'vocalized': u'فَخَلَاهُمَا',
    'stem': u'خَلَا',
    'type': u'STOPWORD',
    'original': u'خَلَا',
    'encletic': u'هُمَا'
}
STOPWORDS[u'عدا'] = {
    'word': u'عَدَا',
    'procletic': u'',
    'tags': u'أداة:حرف استثناء',
    'vocalized': u'عَدَا',
    'stem': u'عَدَا',
    'type': u'STOPWORD',
    'original': u'عَدَا',
    'encletic': u''
}
STOPWORDS[u'وعدا'] = {
    'word': u'وعدا',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف استثناء:معطوف',
    'vocalized': u'وَعَدَا',
    'stem': u'عَدَا',
    'type': u'STOPWORD',
    'original': u'عَدَا',
    'encletic': u''
}
STOPWORDS[u'فعدا'] = {
    'word': u'فعدا',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف استثناء:معطوف',
    'vocalized': u'فَعَدَا',
    'stem': u'عَدَا',
    'type': u'STOPWORD',
    'original': u'عَدَا',
    'encletic': u''
}
STOPWORDS[u'عداي'] = {
    'word': u'عداي',
    'procletic': u'',
    'tags': u'أداة:حرف استثناء:مضاف',
    'vocalized': u'عَدَاِي',
    'stem': u'عَدَا',
    'type': u'STOPWORD',
    'original': u'عَدَا',
    'encletic': u'ِي'
}
STOPWORDS[u'عداك'] = {
    'word': u'عداك',
    'procletic': u'',
    'tags': u'أداة:حرف استثناء:مضاف',
    'vocalized': u'عَدَاكَ',
    'stem': u'عَدَا',
    'type': u'STOPWORD',
    'original': u'عَدَا',
    'encletic': u'كَ'
}
STOPWORDS[u'عداك'] = {
    'word': u'عداك',
    'procletic': u'',
    'tags': u'أداة:حرف استثناء:مضاف',
    'vocalized': u'عَدَاكِ',
    'stem': u'عَدَا',
    'type': u'STOPWORD',
    'original': u'عَدَا',
    'encletic': u'كِ'
}
STOPWORDS[u'عداه'] = {
    'word': u'عداه',
    'procletic': u'',
    'tags': u'أداة:حرف استثناء:مضاف',
    'vocalized': u'عَدَاه',
    'stem': u'عَدَا',
    'type': u'STOPWORD',
    'original': u'عَدَا',
    'encletic': u'ه'
}
STOPWORDS[u'عداكم'] = {
    'word': u'عداكم',
    'procletic': u'',
    'tags': u'أداة:حرف استثناء:مضاف',
    'vocalized': u'عَدَاكُمْ',
    'stem': u'عَدَا',
    'type': u'STOPWORD',
    'original': u'عَدَا',
    'encletic': u'كُمْ'
}
STOPWORDS[u'عداكن'] = {
    'word': u'عداكن',
    'procletic': u'',
    'tags': u'أداة:حرف استثناء:مضاف',
    'vocalized': u'عَدَاكُنَّ',
    'stem': u'عَدَا',
    'type': u'STOPWORD',
    'original': u'عَدَا',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'عداها'] = {
    'word': u'عداها',
    'procletic': u'',
    'tags': u'أداة:حرف استثناء:مضاف',
    'vocalized': u'عَدَاهَا',
    'stem': u'عَدَا',
    'type': u'STOPWORD',
    'original': u'عَدَا',
    'encletic': u'هَا'
}
STOPWORDS[u'عداهم'] = {
    'word': u'عداهم',
    'procletic': u'',
    'tags': u'أداة:حرف استثناء:مضاف',
    'vocalized': u'عَدَاهُمْ',
    'stem': u'عَدَا',
    'type': u'STOPWORD',
    'original': u'عَدَا',
    'encletic': u'هُمْ'
}
STOPWORDS[u'عداهن'] = {
    'word': u'عداهن',
    'procletic': u'',
    'tags': u'أداة:حرف استثناء:مضاف',
    'vocalized': u'عَدَاهُنَّ',
    'stem': u'عَدَا',
    'type': u'STOPWORD',
    'original': u'عَدَا',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'عدانا'] = {
    'word': u'عدانا',
    'procletic': u'',
    'tags': u'أداة:حرف استثناء:مضاف',
    'vocalized': u'عَدَانَا',
    'stem': u'عَدَا',
    'type': u'STOPWORD',
    'original': u'عَدَا',
    'encletic': u'نَا'
}
STOPWORDS[u'عداكما'] = {
    'word': u'عداكما',
    'procletic': u'',
    'tags': u'أداة:حرف استثناء:مضاف',
    'vocalized': u'عَدَاكُمَا',
    'stem': u'عَدَا',
    'type': u'STOPWORD',
    'original': u'عَدَا',
    'encletic': u'كُمَا'
}
STOPWORDS[u'عداهما'] = {
    'word': u'عداهما',
    'procletic': u'',
    'tags': u'أداة:حرف استثناء:مضاف',
    'vocalized': u'عَدَاهُمَا',
    'stem': u'عَدَا',
    'type': u'STOPWORD',
    'original': u'عَدَا',
    'encletic': u'هُمَا'
}
STOPWORDS[u'وعداي'] = {
    'word': u'وعداي',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف استثناء:معطوف:مضاف',
    'vocalized': u'وَعَدَاِي',
    'stem': u'عَدَا',
    'type': u'STOPWORD',
    'original': u'عَدَا',
    'encletic': u'ِي'
}
STOPWORDS[u'وعداك'] = {
    'word': u'وعداك',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف استثناء:معطوف:مضاف',
    'vocalized': u'وَعَدَاكَ',
    'stem': u'عَدَا',
    'type': u'STOPWORD',
    'original': u'عَدَا',
    'encletic': u'كَ'
}
STOPWORDS[u'وعداك'] = {
    'word': u'وعداك',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف استثناء:معطوف:مضاف',
    'vocalized': u'وَعَدَاكِ',
    'stem': u'عَدَا',
    'type': u'STOPWORD',
    'original': u'عَدَا',
    'encletic': u'كِ'
}
STOPWORDS[u'وعداه'] = {
    'word': u'وعداه',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف استثناء:معطوف:مضاف',
    'vocalized': u'وَعَدَاه',
    'stem': u'عَدَا',
    'type': u'STOPWORD',
    'original': u'عَدَا',
    'encletic': u'ه'
}
STOPWORDS[u'وعداكم'] = {
    'word': u'وعداكم',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف استثناء:معطوف:مضاف',
    'vocalized': u'وَعَدَاكُمْ',
    'stem': u'عَدَا',
    'type': u'STOPWORD',
    'original': u'عَدَا',
    'encletic': u'كُمْ'
}
STOPWORDS[u'وعداكن'] = {
    'word': u'وعداكن',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف استثناء:معطوف:مضاف',
    'vocalized': u'وَعَدَاكُنَّ',
    'stem': u'عَدَا',
    'type': u'STOPWORD',
    'original': u'عَدَا',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'وعداها'] = {
    'word': u'وعداها',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف استثناء:معطوف:مضاف',
    'vocalized': u'وَعَدَاهَا',
    'stem': u'عَدَا',
    'type': u'STOPWORD',
    'original': u'عَدَا',
    'encletic': u'هَا'
}
STOPWORDS[u'وعداهم'] = {
    'word': u'وعداهم',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف استثناء:معطوف:مضاف',
    'vocalized': u'وَعَدَاهُمْ',
    'stem': u'عَدَا',
    'type': u'STOPWORD',
    'original': u'عَدَا',
    'encletic': u'هُمْ'
}
STOPWORDS[u'وعداهن'] = {
    'word': u'وعداهن',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف استثناء:معطوف:مضاف',
    'vocalized': u'وَعَدَاهُنَّ',
    'stem': u'عَدَا',
    'type': u'STOPWORD',
    'original': u'عَدَا',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'وعدانا'] = {
    'word': u'وعدانا',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف استثناء:معطوف:مضاف',
    'vocalized': u'وَعَدَانَا',
    'stem': u'عَدَا',
    'type': u'STOPWORD',
    'original': u'عَدَا',
    'encletic': u'نَا'
}
STOPWORDS[u'وعداكما'] = {
    'word': u'وعداكما',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف استثناء:معطوف:مضاف',
    'vocalized': u'وَعَدَاكُمَا',
    'stem': u'عَدَا',
    'type': u'STOPWORD',
    'original': u'عَدَا',
    'encletic': u'كُمَا'
}
STOPWORDS[u'وعداهما'] = {
    'word': u'وعداهما',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف استثناء:معطوف:مضاف',
    'vocalized': u'وَعَدَاهُمَا',
    'stem': u'عَدَا',
    'type': u'STOPWORD',
    'original': u'عَدَا',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فعداي'] = {
    'word': u'فعداي',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف استثناء:معطوف:مضاف',
    'vocalized': u'فَعَدَاِي',
    'stem': u'عَدَا',
    'type': u'STOPWORD',
    'original': u'عَدَا',
    'encletic': u'ِي'
}
STOPWORDS[u'فعداك'] = {
    'word': u'فعداك',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف استثناء:معطوف:مضاف',
    'vocalized': u'فَعَدَاكَ',
    'stem': u'عَدَا',
    'type': u'STOPWORD',
    'original': u'عَدَا',
    'encletic': u'كَ'
}
STOPWORDS[u'فعداك'] = {
    'word': u'فعداك',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف استثناء:معطوف:مضاف',
    'vocalized': u'فَعَدَاكِ',
    'stem': u'عَدَا',
    'type': u'STOPWORD',
    'original': u'عَدَا',
    'encletic': u'كِ'
}
STOPWORDS[u'فعداه'] = {
    'word': u'فعداه',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف استثناء:معطوف:مضاف',
    'vocalized': u'فَعَدَاه',
    'stem': u'عَدَا',
    'type': u'STOPWORD',
    'original': u'عَدَا',
    'encletic': u'ه'
}
STOPWORDS[u'فعداكم'] = {
    'word': u'فعداكم',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف استثناء:معطوف:مضاف',
    'vocalized': u'فَعَدَاكُمْ',
    'stem': u'عَدَا',
    'type': u'STOPWORD',
    'original': u'عَدَا',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فعداكن'] = {
    'word': u'فعداكن',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف استثناء:معطوف:مضاف',
    'vocalized': u'فَعَدَاكُنَّ',
    'stem': u'عَدَا',
    'type': u'STOPWORD',
    'original': u'عَدَا',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فعداها'] = {
    'word': u'فعداها',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف استثناء:معطوف:مضاف',
    'vocalized': u'فَعَدَاهَا',
    'stem': u'عَدَا',
    'type': u'STOPWORD',
    'original': u'عَدَا',
    'encletic': u'هَا'
}
STOPWORDS[u'فعداهم'] = {
    'word': u'فعداهم',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف استثناء:معطوف:مضاف',
    'vocalized': u'فَعَدَاهُمْ',
    'stem': u'عَدَا',
    'type': u'STOPWORD',
    'original': u'عَدَا',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فعداهن'] = {
    'word': u'فعداهن',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف استثناء:معطوف:مضاف',
    'vocalized': u'فَعَدَاهُنَّ',
    'stem': u'عَدَا',
    'type': u'STOPWORD',
    'original': u'عَدَا',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فعدانا'] = {
    'word': u'فعدانا',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف استثناء:معطوف:مضاف',
    'vocalized': u'فَعَدَانَا',
    'stem': u'عَدَا',
    'type': u'STOPWORD',
    'original': u'عَدَا',
    'encletic': u'نَا'
}
STOPWORDS[u'فعداكما'] = {
    'word': u'فعداكما',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف استثناء:معطوف:مضاف',
    'vocalized': u'فَعَدَاكُمَا',
    'stem': u'عَدَا',
    'type': u'STOPWORD',
    'original': u'عَدَا',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فعداهما'] = {
    'word': u'فعداهما',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف استثناء:معطوف:مضاف',
    'vocalized': u'فَعَدَاهُمَا',
    'stem': u'عَدَا',
    'type': u'STOPWORD',
    'original': u'عَدَا',
    'encletic': u'هُمَا'
}
STOPWORDS[u'لكن'] = {
    'word': u'لَكِنْ',
    'procletic': u'',
    'tags': u'أداة:حرف استدراك',
    'vocalized': u'لَكِنْ',
    'stem': u'لَكِنْ',
    'type': u'STOPWORD',
    'original': u'لَكِنْ',
    'encletic': u''
}
STOPWORDS[u'ولكن'] = {
    'word': u'ولكن',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف استدراك:معطوف',
    'vocalized': u'وَلَكِنْ',
    'stem': u'لَكِنْ',
    'type': u'STOPWORD',
    'original': u'لَكِنْ',
    'encletic': u''
}
STOPWORDS[u'فلكن'] = {
    'word': u'فلكن',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف استدراك:معطوف',
    'vocalized': u'فَلَكِنْ',
    'stem': u'لَكِنْ',
    'type': u'STOPWORD',
    'original': u'لَكِنْ',
    'encletic': u''
}
STOPWORDS[u'لكني'] = {
    'word': u'لكني',
    'procletic': u'',
    'tags': u'أداة:حرف استدراك:مضاف',
    'vocalized': u'لَكِنِْي',
    'stem': u'لَكِنْ',
    'type': u'STOPWORD',
    'original': u'لَكِنْ',
    'encletic': u'ِي'
}
STOPWORDS[u'لكنك'] = {
    'word': u'لكنك',
    'procletic': u'',
    'tags': u'أداة:حرف استدراك:مضاف',
    'vocalized': u'لَكِنْكَ',
    'stem': u'لَكِنْ',
    'type': u'STOPWORD',
    'original': u'لَكِنْ',
    'encletic': u'كَ'
}
STOPWORDS[u'لكنك'] = {
    'word': u'لكنك',
    'procletic': u'',
    'tags': u'أداة:حرف استدراك:مضاف',
    'vocalized': u'لَكِنْكِ',
    'stem': u'لَكِنْ',
    'type': u'STOPWORD',
    'original': u'لَكِنْ',
    'encletic': u'كِ'
}
STOPWORDS[u'لكنه'] = {
    'word': u'لكنه',
    'procletic': u'',
    'tags': u'أداة:حرف استدراك:مضاف',
    'vocalized': u'لَكِنْه',
    'stem': u'لَكِنْ',
    'type': u'STOPWORD',
    'original': u'لَكِنْ',
    'encletic': u'ه'
}
STOPWORDS[u'لكنكم'] = {
    'word': u'لكنكم',
    'procletic': u'',
    'tags': u'أداة:حرف استدراك:مضاف',
    'vocalized': u'لَكِنْكُمْ',
    'stem': u'لَكِنْ',
    'type': u'STOPWORD',
    'original': u'لَكِنْ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'لكنكن'] = {
    'word': u'لكنكن',
    'procletic': u'',
    'tags': u'أداة:حرف استدراك:مضاف',
    'vocalized': u'لَكِنْكُنَّ',
    'stem': u'لَكِنْ',
    'type': u'STOPWORD',
    'original': u'لَكِنْ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'لكنها'] = {
    'word': u'لكنها',
    'procletic': u'',
    'tags': u'أداة:حرف استدراك:مضاف',
    'vocalized': u'لَكِنْهَا',
    'stem': u'لَكِنْ',
    'type': u'STOPWORD',
    'original': u'لَكِنْ',
    'encletic': u'هَا'
}
STOPWORDS[u'لكنهم'] = {
    'word': u'لكنهم',
    'procletic': u'',
    'tags': u'أداة:حرف استدراك:مضاف',
    'vocalized': u'لَكِنْهُمْ',
    'stem': u'لَكِنْ',
    'type': u'STOPWORD',
    'original': u'لَكِنْ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'لكنهن'] = {
    'word': u'لكنهن',
    'procletic': u'',
    'tags': u'أداة:حرف استدراك:مضاف',
    'vocalized': u'لَكِنْهُنَّ',
    'stem': u'لَكِنْ',
    'type': u'STOPWORD',
    'original': u'لَكِنْ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'لكننا'] = {
    'word': u'لكننا',
    'procletic': u'',
    'tags': u'أداة:حرف استدراك:مضاف',
    'vocalized': u'لَكِنْنَا',
    'stem': u'لَكِنْ',
    'type': u'STOPWORD',
    'original': u'لَكِنْ',
    'encletic': u'نَا'
}
STOPWORDS[u'لكنكما'] = {
    'word': u'لكنكما',
    'procletic': u'',
    'tags': u'أداة:حرف استدراك:مضاف',
    'vocalized': u'لَكِنْكُمَا',
    'stem': u'لَكِنْ',
    'type': u'STOPWORD',
    'original': u'لَكِنْ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'لكنهما'] = {
    'word': u'لكنهما',
    'procletic': u'',
    'tags': u'أداة:حرف استدراك:مضاف',
    'vocalized': u'لَكِنْهُمَا',
    'stem': u'لَكِنْ',
    'type': u'STOPWORD',
    'original': u'لَكِنْ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ولكني'] = {
    'word': u'ولكني',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف استدراك:معطوف:مضاف',
    'vocalized': u'وَلَكِنِْي',
    'stem': u'لَكِنْ',
    'type': u'STOPWORD',
    'original': u'لَكِنْ',
    'encletic': u'ِي'
}
STOPWORDS[u'ولكنك'] = {
    'word': u'ولكنك',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف استدراك:معطوف:مضاف',
    'vocalized': u'وَلَكِنْكَ',
    'stem': u'لَكِنْ',
    'type': u'STOPWORD',
    'original': u'لَكِنْ',
    'encletic': u'كَ'
}
STOPWORDS[u'ولكنك'] = {
    'word': u'ولكنك',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف استدراك:معطوف:مضاف',
    'vocalized': u'وَلَكِنْكِ',
    'stem': u'لَكِنْ',
    'type': u'STOPWORD',
    'original': u'لَكِنْ',
    'encletic': u'كِ'
}
STOPWORDS[u'ولكنه'] = {
    'word': u'ولكنه',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف استدراك:معطوف:مضاف',
    'vocalized': u'وَلَكِنْه',
    'stem': u'لَكِنْ',
    'type': u'STOPWORD',
    'original': u'لَكِنْ',
    'encletic': u'ه'
}
STOPWORDS[u'ولكنكم'] = {
    'word': u'ولكنكم',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف استدراك:معطوف:مضاف',
    'vocalized': u'وَلَكِنْكُمْ',
    'stem': u'لَكِنْ',
    'type': u'STOPWORD',
    'original': u'لَكِنْ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ولكنكن'] = {
    'word': u'ولكنكن',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف استدراك:معطوف:مضاف',
    'vocalized': u'وَلَكِنْكُنَّ',
    'stem': u'لَكِنْ',
    'type': u'STOPWORD',
    'original': u'لَكِنْ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ولكنها'] = {
    'word': u'ولكنها',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف استدراك:معطوف:مضاف',
    'vocalized': u'وَلَكِنْهَا',
    'stem': u'لَكِنْ',
    'type': u'STOPWORD',
    'original': u'لَكِنْ',
    'encletic': u'هَا'
}
STOPWORDS[u'ولكنهم'] = {
    'word': u'ولكنهم',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف استدراك:معطوف:مضاف',
    'vocalized': u'وَلَكِنْهُمْ',
    'stem': u'لَكِنْ',
    'type': u'STOPWORD',
    'original': u'لَكِنْ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ولكنهن'] = {
    'word': u'ولكنهن',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف استدراك:معطوف:مضاف',
    'vocalized': u'وَلَكِنْهُنَّ',
    'stem': u'لَكِنْ',
    'type': u'STOPWORD',
    'original': u'لَكِنْ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ولكننا'] = {
    'word': u'ولكننا',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف استدراك:معطوف:مضاف',
    'vocalized': u'وَلَكِنْنَا',
    'stem': u'لَكِنْ',
    'type': u'STOPWORD',
    'original': u'لَكِنْ',
    'encletic': u'نَا'
}
STOPWORDS[u'ولكنكما'] = {
    'word': u'ولكنكما',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف استدراك:معطوف:مضاف',
    'vocalized': u'وَلَكِنْكُمَا',
    'stem': u'لَكِنْ',
    'type': u'STOPWORD',
    'original': u'لَكِنْ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ولكنهما'] = {
    'word': u'ولكنهما',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف استدراك:معطوف:مضاف',
    'vocalized': u'وَلَكِنْهُمَا',
    'stem': u'لَكِنْ',
    'type': u'STOPWORD',
    'original': u'لَكِنْ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فلكني'] = {
    'word': u'فلكني',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف استدراك:معطوف:مضاف',
    'vocalized': u'فَلَكِنِْي',
    'stem': u'لَكِنْ',
    'type': u'STOPWORD',
    'original': u'لَكِنْ',
    'encletic': u'ِي'
}
STOPWORDS[u'فلكنك'] = {
    'word': u'فلكنك',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف استدراك:معطوف:مضاف',
    'vocalized': u'فَلَكِنْكَ',
    'stem': u'لَكِنْ',
    'type': u'STOPWORD',
    'original': u'لَكِنْ',
    'encletic': u'كَ'
}
STOPWORDS[u'فلكنك'] = {
    'word': u'فلكنك',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف استدراك:معطوف:مضاف',
    'vocalized': u'فَلَكِنْكِ',
    'stem': u'لَكِنْ',
    'type': u'STOPWORD',
    'original': u'لَكِنْ',
    'encletic': u'كِ'
}
STOPWORDS[u'فلكنه'] = {
    'word': u'فلكنه',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف استدراك:معطوف:مضاف',
    'vocalized': u'فَلَكِنْه',
    'stem': u'لَكِنْ',
    'type': u'STOPWORD',
    'original': u'لَكِنْ',
    'encletic': u'ه'
}
STOPWORDS[u'فلكنكم'] = {
    'word': u'فلكنكم',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف استدراك:معطوف:مضاف',
    'vocalized': u'فَلَكِنْكُمْ',
    'stem': u'لَكِنْ',
    'type': u'STOPWORD',
    'original': u'لَكِنْ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فلكنكن'] = {
    'word': u'فلكنكن',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف استدراك:معطوف:مضاف',
    'vocalized': u'فَلَكِنْكُنَّ',
    'stem': u'لَكِنْ',
    'type': u'STOPWORD',
    'original': u'لَكِنْ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فلكنها'] = {
    'word': u'فلكنها',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف استدراك:معطوف:مضاف',
    'vocalized': u'فَلَكِنْهَا',
    'stem': u'لَكِنْ',
    'type': u'STOPWORD',
    'original': u'لَكِنْ',
    'encletic': u'هَا'
}
STOPWORDS[u'فلكنهم'] = {
    'word': u'فلكنهم',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف استدراك:معطوف:مضاف',
    'vocalized': u'فَلَكِنْهُمْ',
    'stem': u'لَكِنْ',
    'type': u'STOPWORD',
    'original': u'لَكِنْ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فلكنهن'] = {
    'word': u'فلكنهن',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف استدراك:معطوف:مضاف',
    'vocalized': u'فَلَكِنْهُنَّ',
    'stem': u'لَكِنْ',
    'type': u'STOPWORD',
    'original': u'لَكِنْ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فلكننا'] = {
    'word': u'فلكننا',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف استدراك:معطوف:مضاف',
    'vocalized': u'فَلَكِنْنَا',
    'stem': u'لَكِنْ',
    'type': u'STOPWORD',
    'original': u'لَكِنْ',
    'encletic': u'نَا'
}
STOPWORDS[u'فلكنكما'] = {
    'word': u'فلكنكما',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف استدراك:معطوف:مضاف',
    'vocalized': u'فَلَكِنْكُمَا',
    'stem': u'لَكِنْ',
    'type': u'STOPWORD',
    'original': u'لَكِنْ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فلكنهما'] = {
    'word': u'فلكنهما',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف استدراك:معطوف:مضاف',
    'vocalized': u'فَلَكِنْهُمَا',
    'stem': u'لَكِنْ',
    'type': u'STOPWORD',
    'original': u'لَكِنْ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فيم'] = {
    'word': u'فِيمَ',
    'procletic': u'',
    'tags': u'أداة:حرف استفهام',
    'vocalized': u'فِيمَ',
    'stem': u'فِيمَ',
    'type': u'STOPWORD',
    'original': u'فِيمَ',
    'encletic': u''
}
STOPWORDS[u'وفيم'] = {
    'word': u'وفيم',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف استفهام:معطوف',
    'vocalized': u'وَفِيمَ',
    'stem': u'فِيمَ',
    'type': u'STOPWORD',
    'original': u'فِيمَ',
    'encletic': u''
}
STOPWORDS[u'ففيم'] = {
    'word': u'ففيم',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف استفهام:معطوف',
    'vocalized': u'فَفِيمَ',
    'stem': u'فِيمَ',
    'type': u'STOPWORD',
    'original': u'فِيمَ',
    'encletic': u''
}
STOPWORDS[u'فيما'] = {
    'word': u'فِيمَا',
    'procletic': u'',
    'tags': u'أداة:حرف استفهام',
    'vocalized': u'فِيمَا',
    'stem': u'فِيمَا',
    'type': u'STOPWORD',
    'original': u'فِيمَا',
    'encletic': u''
}
STOPWORDS[u'وفيما'] = {
    'word': u'وفيما',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف استفهام:معطوف',
    'vocalized': u'وَفِيمَا',
    'stem': u'فِيمَا',
    'type': u'STOPWORD',
    'original': u'فِيمَا',
    'encletic': u''
}
STOPWORDS[u'ففيما'] = {
    'word': u'ففيما',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف استفهام:معطوف',
    'vocalized': u'فَفِيمَا',
    'stem': u'فِيمَا',
    'type': u'STOPWORD',
    'original': u'فِيمَا',
    'encletic': u''
}
STOPWORDS[u'هل'] = {
    'word': u'هَلْ',
    'procletic': u'',
    'tags': u'أداة:حرف استفهام',
    'vocalized': u'هَلْ',
    'stem': u'هَلْ',
    'type': u'STOPWORD',
    'original': u'هَلْ',
    'encletic': u''
}
STOPWORDS[u'وهل'] = {
    'word': u'وهل',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف استفهام:معطوف',
    'vocalized': u'وَهَلْ',
    'stem': u'هَلْ',
    'type': u'STOPWORD',
    'original': u'هَلْ',
    'encletic': u''
}
STOPWORDS[u'فهل'] = {
    'word': u'فهل',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف استفهام:معطوف',
    'vocalized': u'فَهَلْ',
    'stem': u'هَلْ',
    'type': u'STOPWORD',
    'original': u'هَلْ',
    'encletic': u''
}
STOPWORDS[u'سوف'] = {
    'word': u'سَوْفَ',
    'procletic': u'',
    'tags': u'أداة:حرف استقبال',
    'vocalized': u'سَوْفَ',
    'stem': u'سَوْفَ',
    'type': u'STOPWORD',
    'original': u'سَوْفَ',
    'encletic': u''
}
STOPWORDS[u'لسوف'] = {
    'word': u'لسوف',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور',
    'vocalized': u'لَسَوْفَ',
    'stem': u'سَوْفَ',
    'type': u'STOPWORD',
    'original': u'سَوْفَ',
    'encletic': u''
}
STOPWORDS[u'وسوف'] = {
    'word': u'وسوف',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف استقبال:معطوف',
    'vocalized': u'وَسَوْفَ',
    'stem': u'سَوْفَ',
    'type': u'STOPWORD',
    'original': u'سَوْفَ',
    'encletic': u''
}
STOPWORDS[u'فسوف'] = {
    'word': u'فسوف',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف استقبال:معطوف',
    'vocalized': u'فَسَوْفَ',
    'stem': u'سَوْفَ',
    'type': u'STOPWORD',
    'original': u'سَوْفَ',
    'encletic': u''
}
STOPWORDS[u'ولسوف'] = {
    'word': u'ولسوف',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'وَلَسَوْفَ',
    'stem': u'سَوْفَ',
    'type': u'STOPWORD',
    'original': u'سَوْفَ',
    'encletic': u''
}
STOPWORDS[u'فلسوف'] = {
    'word': u'فلسوف',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'فَلَسَوْفَ',
    'stem': u'سَوْفَ',
    'type': u'STOPWORD',
    'original': u'سَوْفَ',
    'encletic': u''
}
STOPWORDS[u'هلا'] = {
    'word': u'هَلّا',
    'procletic': u'',
    'tags': u'أداة:حرف تحضيض',
    'vocalized': u'هَلّا',
    'stem': u'هَلّا',
    'type': u'STOPWORD',
    'original': u'هَلّا',
    'encletic': u''
}
STOPWORDS[u'وهلا'] = {
    'word': u'وهلا',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف تحضيض:معطوف',
    'vocalized': u'وَهَلّا',
    'stem': u'هَلّا',
    'type': u'STOPWORD',
    'original': u'هَلّا',
    'encletic': u''
}
STOPWORDS[u'فهلا'] = {
    'word': u'فهلا',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف تحضيض:معطوف',
    'vocalized': u'فَهَلّا',
    'stem': u'هَلّا',
    'type': u'STOPWORD',
    'original': u'هَلّا',
    'encletic': u''
}
STOPWORDS[u'قد'] = {
    'word': u'قَدْ',
    'procletic': u'',
    'tags': u'أداة:حرف تحقيق/ توقع',
    'vocalized': u'قَدْ',
    'stem': u'قَدْ',
    'type': u'STOPWORD',
    'original': u'قَدْ',
    'encletic': u''
}
STOPWORDS[u'لقد'] = {
    'word': u'لقد',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور',
    'vocalized': u'لَقَدْ',
    'stem': u'قَدْ',
    'type': u'STOPWORD',
    'original': u'قَدْ',
    'encletic': u''
}
STOPWORDS[u'وقد'] = {
    'word': u'وقد',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف تحقيق/ توقع:معطوف',
    'vocalized': u'وَقَدْ',
    'stem': u'قَدْ',
    'type': u'STOPWORD',
    'original': u'قَدْ',
    'encletic': u''
}
STOPWORDS[u'فقد'] = {
    'word': u'فقد',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف تحقيق/ توقع:معطوف',
    'vocalized': u'فَقَدْ',
    'stem': u'قَدْ',
    'type': u'STOPWORD',
    'original': u'قَدْ',
    'encletic': u''
}
STOPWORDS[u'ولقد'] = {
    'word': u'ولقد',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'وَلَقَدْ',
    'stem': u'قَدْ',
    'type': u'STOPWORD',
    'original': u'قَدْ',
    'encletic': u''
}
STOPWORDS[u'فلقد'] = {
    'word': u'فلقد',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'فَلَقَدْ',
    'stem': u'قَدْ',
    'type': u'STOPWORD',
    'original': u'قَدْ',
    'encletic': u''
}
STOPWORDS[u'إما'] = {
    'word': u'إِمَّا',
    'procletic': u'',
    'tags': u'أداة:حرف تخيير وتفصيل',
    'vocalized': u'إِمَّا',
    'stem': u'إِمَّا',
    'type': u'STOPWORD',
    'original': u'إِمَّا',
    'encletic': u''
}
STOPWORDS[u'وإما'] = {
    'word': u'وإما',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف تخيير وتفصيل:معطوف',
    'vocalized': u'وَإِمَّا',
    'stem': u'إِمَّا',
    'type': u'STOPWORD',
    'original': u'إِمَّا',
    'encletic': u''
}
STOPWORDS[u'فإما'] = {
    'word': u'فإما',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف تخيير وتفصيل:معطوف',
    'vocalized': u'فَإِمَّا',
    'stem': u'إِمَّا',
    'type': u'STOPWORD',
    'original': u'إِمَّا',
    'encletic': u''
}
STOPWORDS[u'كأنما'] = {
    'word': u'كَأَنَّمَا',
    'procletic': u'',
    'tags': u'أداة:حرف تشبيه',
    'vocalized': u'كَأَنَّمَا',
    'stem': u'كَأَنَّمَا',
    'type': u'STOPWORD',
    'original': u'كَأَنَّمَا',
    'encletic': u''
}
STOPWORDS[u'لكأنما'] = {
    'word': u'لكأنما',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور',
    'vocalized': u'لَكَأَنَّمَا',
    'stem': u'كَأَنَّمَا',
    'type': u'STOPWORD',
    'original': u'كَأَنَّمَا',
    'encletic': u''
}
STOPWORDS[u'وكأنما'] = {
    'word': u'وكأنما',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف تشبيه:معطوف',
    'vocalized': u'وَكَأَنَّمَا',
    'stem': u'كَأَنَّمَا',
    'type': u'STOPWORD',
    'original': u'كَأَنَّمَا',
    'encletic': u''
}
STOPWORDS[u'فكأنما'] = {
    'word': u'فكأنما',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف تشبيه:معطوف',
    'vocalized': u'فَكَأَنَّمَا',
    'stem': u'كَأَنَّمَا',
    'type': u'STOPWORD',
    'original': u'كَأَنَّمَا',
    'encletic': u''
}
STOPWORDS[u'ولكأنما'] = {
    'word': u'ولكأنما',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'وَلَكَأَنَّمَا',
    'stem': u'كَأَنَّمَا',
    'type': u'STOPWORD',
    'original': u'كَأَنَّمَا',
    'encletic': u''
}
STOPWORDS[u'فلكأنما'] = {
    'word': u'فلكأنما',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'فَلَكَأَنَّمَا',
    'stem': u'كَأَنَّمَا',
    'type': u'STOPWORD',
    'original': u'كَأَنَّمَا',
    'encletic': u''
}
STOPWORDS[u'أكأنما'] = {
    'word': u'أكأنما',
    'procletic': u'-أَ',
    'tags': u'أداة:حرف تشبيه:مجرور',
    'vocalized': u'أَكَأَنَّمَا',
    'stem': u'كَأَنَّمَا',
    'type': u'STOPWORD',
    'original': u'كَأَنَّمَا',
    'encletic': u''
}
STOPWORDS[u'ألكأنما'] = {
    'word': u'ألكأنما',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور',
    'vocalized': u'أَلَكَأَنَّمَا',
    'stem': u'كَأَنَّمَا',
    'type': u'STOPWORD',
    'original': u'كَأَنَّمَا',
    'encletic': u''
}
STOPWORDS[u'أوكأنما'] = {
    'word': u'أوكأنما',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:حرف تشبيه:معطوف:مجرور',
    'vocalized': u'أَوَكَأَنَّمَا',
    'stem': u'كَأَنَّمَا',
    'type': u'STOPWORD',
    'original': u'كَأَنَّمَا',
    'encletic': u''
}
STOPWORDS[u'أفكأنما'] = {
    'word': u'أفكأنما',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:حرف تشبيه:معطوف:مجرور',
    'vocalized': u'أَفَكَأَنَّمَا',
    'stem': u'كَأَنَّمَا',
    'type': u'STOPWORD',
    'original': u'كَأَنَّمَا',
    'encletic': u''
}
STOPWORDS[u'أولكأنما'] = {
    'word': u'أولكأنما',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَلَكَأَنَّمَا',
    'stem': u'كَأَنَّمَا',
    'type': u'STOPWORD',
    'original': u'كَأَنَّمَا',
    'encletic': u''
}
STOPWORDS[u'أفلكأنما'] = {
    'word': u'أفلكأنما',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَلَكَأَنَّمَا',
    'stem': u'كَأَنَّمَا',
    'type': u'STOPWORD',
    'original': u'كَأَنَّمَا',
    'encletic': u''
}
STOPWORDS[u'كما'] = {
    'word': u'كَمَا',
    'procletic': u'',
    'tags': u'أداة:حرف تشبيه',
    'vocalized': u'كَمَا',
    'stem': u'كَمَا',
    'type': u'STOPWORD',
    'original': u'كَمَا',
    'encletic': u''
}
STOPWORDS[u'وكما'] = {
    'word': u'وكما',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف تشبيه:معطوف',
    'vocalized': u'وَكَمَا',
    'stem': u'كَمَا',
    'type': u'STOPWORD',
    'original': u'كَمَا',
    'encletic': u''
}
STOPWORDS[u'فكما'] = {
    'word': u'فكما',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف تشبيه:معطوف',
    'vocalized': u'فَكَمَا',
    'stem': u'كَمَا',
    'type': u'STOPWORD',
    'original': u'كَمَا',
    'encletic': u''
}
STOPWORDS[u'لكي'] = {
    'word': u'لِكَيْ',
    'procletic': u'',
    'tags': u'أداة:حرف تعليل',
    'vocalized': u'لِكَيْ',
    'stem': u'لِكَيْ',
    'type': u'STOPWORD',
    'original': u'لِكَيْ',
    'encletic': u''
}
STOPWORDS[u'ولكي'] = {
    'word': u'ولكي',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف تعليل:معطوف',
    'vocalized': u'وَلِكَيْ',
    'stem': u'لِكَيْ',
    'type': u'STOPWORD',
    'original': u'لِكَيْ',
    'encletic': u''
}
STOPWORDS[u'فلكي'] = {
    'word': u'فلكي',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف تعليل:معطوف',
    'vocalized': u'فَلِكَيْ',
    'stem': u'لِكَيْ',
    'type': u'STOPWORD',
    'original': u'لِكَيْ',
    'encletic': u''
}
STOPWORDS[u'لكيلا'] = {
    'word': u'لِكَيْلَا',
    'procletic': u'',
    'tags': u'أداة:حرف تعليل',
    'vocalized': u'لِكَيْلَا',
    'stem': u'لِكَيْلَا',
    'type': u'STOPWORD',
    'original': u'لِكَيْلَا',
    'encletic': u''
}
STOPWORDS[u'ولكيلا'] = {
    'word': u'ولكيلا',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف تعليل:معطوف',
    'vocalized': u'وَلِكَيْلَا',
    'stem': u'لِكَيْلَا',
    'type': u'STOPWORD',
    'original': u'لِكَيْلَا',
    'encletic': u''
}
STOPWORDS[u'فلكيلا'] = {
    'word': u'فلكيلا',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف تعليل:معطوف',
    'vocalized': u'فَلِكَيْلَا',
    'stem': u'لِكَيْلَا',
    'type': u'STOPWORD',
    'original': u'لِكَيْلَا',
    'encletic': u''
}
STOPWORDS[u'إلى'] = {
    'word': u'إِلَى',
    'procletic': u'',
    'tags': u'أداة:حرف جر',
    'vocalized': u'إِلَى',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u''
}
STOPWORDS[u'لإلى'] = {
    'word': u'لإلى',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور',
    'vocalized': u'لَإِلَى',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u''
}
STOPWORDS[u'وإلى'] = {
    'word': u'وإلى',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف جر:معطوف',
    'vocalized': u'وَإِلَى',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u''
}
STOPWORDS[u'فإلى'] = {
    'word': u'فإلى',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف جر:معطوف',
    'vocalized': u'فَإِلَى',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u''
}
STOPWORDS[u'ولإلى'] = {
    'word': u'ولإلى',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'وَلَإِلَى',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u''
}
STOPWORDS[u'فلإلى'] = {
    'word': u'فلإلى',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'فَلَإِلَى',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u''
}
STOPWORDS[u'أإلى'] = {
    'word': u'أإلى',
    'procletic': u'-أَ',
    'tags': u'أداة:حرف جر:مجرور',
    'vocalized': u'أَإِلَى',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u''
}
STOPWORDS[u'ألإلى'] = {
    'word': u'ألإلى',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور',
    'vocalized': u'أَلَإِلَى',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u''
}
STOPWORDS[u'أوإلى'] = {
    'word': u'أوإلى',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور',
    'vocalized': u'أَوَإِلَى',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u''
}
STOPWORDS[u'أفإلى'] = {
    'word': u'أفإلى',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور',
    'vocalized': u'أَفَإِلَى',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u''
}
STOPWORDS[u'أولإلى'] = {
    'word': u'أولإلى',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَلَإِلَى',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u''
}
STOPWORDS[u'أفلإلى'] = {
    'word': u'أفلإلى',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَلَإِلَى',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u''
}
STOPWORDS[u'إلي'] = {
    'word': u'إلي',
    'procletic': u'',
    'tags': u'أداة:حرف جر:مضاف',
    'vocalized': u'إِلَيِي',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'ِي'
}
STOPWORDS[u'إليك'] = {
    'word': u'إليك',
    'procletic': u'',
    'tags': u'أداة:حرف جر:مضاف',
    'vocalized': u'إِلَيكَ',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'كَ'
}
STOPWORDS[u'إليك'] = {
    'word': u'إليك',
    'procletic': u'',
    'tags': u'أداة:حرف جر:مضاف',
    'vocalized': u'إِلَيكِ',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'كِ'
}
STOPWORDS[u'إليه'] = {
    'word': u'إليه',
    'procletic': u'',
    'tags': u'أداة:حرف جر:مضاف',
    'vocalized': u'إِلَيه',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'ه'
}
STOPWORDS[u'إليكم'] = {
    'word': u'إليكم',
    'procletic': u'',
    'tags': u'أداة:حرف جر:مضاف',
    'vocalized': u'إِلَيكُمْ',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'كُمْ'
}
STOPWORDS[u'إليكن'] = {
    'word': u'إليكن',
    'procletic': u'',
    'tags': u'أداة:حرف جر:مضاف',
    'vocalized': u'إِلَيكُنَّ',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'إليها'] = {
    'word': u'إليها',
    'procletic': u'',
    'tags': u'أداة:حرف جر:مضاف',
    'vocalized': u'إِلَيهَا',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'هَا'
}
STOPWORDS[u'إليهم'] = {
    'word': u'إليهم',
    'procletic': u'',
    'tags': u'أداة:حرف جر:مضاف',
    'vocalized': u'إِلَيهُمْ',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'هُمْ'
}
STOPWORDS[u'إليهن'] = {
    'word': u'إليهن',
    'procletic': u'',
    'tags': u'أداة:حرف جر:مضاف',
    'vocalized': u'إِلَيهُنَّ',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'إلينا'] = {
    'word': u'إلينا',
    'procletic': u'',
    'tags': u'أداة:حرف جر:مضاف',
    'vocalized': u'إِلَينَا',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'نَا'
}
STOPWORDS[u'إليكما'] = {
    'word': u'إليكما',
    'procletic': u'',
    'tags': u'أداة:حرف جر:مضاف',
    'vocalized': u'إِلَيكُمَا',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'كُمَا'
}
STOPWORDS[u'إليهما'] = {
    'word': u'إليهما',
    'procletic': u'',
    'tags': u'أداة:حرف جر:مضاف',
    'vocalized': u'إِلَيهُمَا',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'هُمَا'
}
STOPWORDS[u'لإلي'] = {
    'word': u'لإلي',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَإِلَيِي',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'ِي'
}
STOPWORDS[u'لإليك'] = {
    'word': u'لإليك',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَإِلَيكَ',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'كَ'
}
STOPWORDS[u'لإليك'] = {
    'word': u'لإليك',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَإِلَيكِ',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'كِ'
}
STOPWORDS[u'لإليه'] = {
    'word': u'لإليه',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَإِلَيه',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'ه'
}
STOPWORDS[u'لإليكم'] = {
    'word': u'لإليكم',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَإِلَيكُمْ',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'كُمْ'
}
STOPWORDS[u'لإليكن'] = {
    'word': u'لإليكن',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَإِلَيكُنَّ',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'لإليها'] = {
    'word': u'لإليها',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَإِلَيهَا',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'هَا'
}
STOPWORDS[u'لإليهم'] = {
    'word': u'لإليهم',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَإِلَيهُمْ',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'هُمْ'
}
STOPWORDS[u'لإليهن'] = {
    'word': u'لإليهن',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَإِلَيهُنَّ',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'لإلينا'] = {
    'word': u'لإلينا',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَإِلَينَا',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'نَا'
}
STOPWORDS[u'لإليكما'] = {
    'word': u'لإليكما',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَإِلَيكُمَا',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'كُمَا'
}
STOPWORDS[u'لإليهما'] = {
    'word': u'لإليهما',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَإِلَيهُمَا',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'هُمَا'
}
STOPWORDS[u'وإلي'] = {
    'word': u'وإلي',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'وَإِلَيِي',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'ِي'
}
STOPWORDS[u'وإليك'] = {
    'word': u'وإليك',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'وَإِلَيكَ',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'كَ'
}
STOPWORDS[u'وإليك'] = {
    'word': u'وإليك',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'وَإِلَيكِ',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'كِ'
}
STOPWORDS[u'وإليه'] = {
    'word': u'وإليه',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'وَإِلَيه',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'ه'
}
STOPWORDS[u'وإليكم'] = {
    'word': u'وإليكم',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'وَإِلَيكُمْ',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'كُمْ'
}
STOPWORDS[u'وإليكن'] = {
    'word': u'وإليكن',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'وَإِلَيكُنَّ',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'وإليها'] = {
    'word': u'وإليها',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'وَإِلَيهَا',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'هَا'
}
STOPWORDS[u'وإليهم'] = {
    'word': u'وإليهم',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'وَإِلَيهُمْ',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'هُمْ'
}
STOPWORDS[u'وإليهن'] = {
    'word': u'وإليهن',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'وَإِلَيهُنَّ',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'وإلينا'] = {
    'word': u'وإلينا',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'وَإِلَينَا',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'نَا'
}
STOPWORDS[u'وإليكما'] = {
    'word': u'وإليكما',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'وَإِلَيكُمَا',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'كُمَا'
}
STOPWORDS[u'وإليهما'] = {
    'word': u'وإليهما',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'وَإِلَيهُمَا',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فإلي'] = {
    'word': u'فإلي',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'فَإِلَيِي',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'ِي'
}
STOPWORDS[u'فإليك'] = {
    'word': u'فإليك',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'فَإِلَيكَ',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'كَ'
}
STOPWORDS[u'فإليك'] = {
    'word': u'فإليك',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'فَإِلَيكِ',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'كِ'
}
STOPWORDS[u'فإليه'] = {
    'word': u'فإليه',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'فَإِلَيه',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'ه'
}
STOPWORDS[u'فإليكم'] = {
    'word': u'فإليكم',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'فَإِلَيكُمْ',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فإليكن'] = {
    'word': u'فإليكن',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'فَإِلَيكُنَّ',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فإليها'] = {
    'word': u'فإليها',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'فَإِلَيهَا',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'هَا'
}
STOPWORDS[u'فإليهم'] = {
    'word': u'فإليهم',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'فَإِلَيهُمْ',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فإليهن'] = {
    'word': u'فإليهن',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'فَإِلَيهُنَّ',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فإلينا'] = {
    'word': u'فإلينا',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'فَإِلَينَا',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'نَا'
}
STOPWORDS[u'فإليكما'] = {
    'word': u'فإليكما',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'فَإِلَيكُمَا',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فإليهما'] = {
    'word': u'فإليهما',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'فَإِلَيهُمَا',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ولإلي'] = {
    'word': u'ولإلي',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَإِلَيِي',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'ِي'
}
STOPWORDS[u'ولإليك'] = {
    'word': u'ولإليك',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَإِلَيكَ',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'كَ'
}
STOPWORDS[u'ولإليك'] = {
    'word': u'ولإليك',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَإِلَيكِ',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'كِ'
}
STOPWORDS[u'ولإليه'] = {
    'word': u'ولإليه',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَإِلَيه',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'ه'
}
STOPWORDS[u'ولإليكم'] = {
    'word': u'ولإليكم',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَإِلَيكُمْ',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ولإليكن'] = {
    'word': u'ولإليكن',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَإِلَيكُنَّ',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ولإليها'] = {
    'word': u'ولإليها',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَإِلَيهَا',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'هَا'
}
STOPWORDS[u'ولإليهم'] = {
    'word': u'ولإليهم',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَإِلَيهُمْ',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ولإليهن'] = {
    'word': u'ولإليهن',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَإِلَيهُنَّ',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ولإلينا'] = {
    'word': u'ولإلينا',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَإِلَينَا',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'نَا'
}
STOPWORDS[u'ولإليكما'] = {
    'word': u'ولإليكما',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَإِلَيكُمَا',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ولإليهما'] = {
    'word': u'ولإليهما',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَإِلَيهُمَا',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فلإلي'] = {
    'word': u'فلإلي',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَإِلَيِي',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'ِي'
}
STOPWORDS[u'فلإليك'] = {
    'word': u'فلإليك',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَإِلَيكَ',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'كَ'
}
STOPWORDS[u'فلإليك'] = {
    'word': u'فلإليك',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَإِلَيكِ',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'كِ'
}
STOPWORDS[u'فلإليه'] = {
    'word': u'فلإليه',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَإِلَيه',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'ه'
}
STOPWORDS[u'فلإليكم'] = {
    'word': u'فلإليكم',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَإِلَيكُمْ',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فلإليكن'] = {
    'word': u'فلإليكن',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَإِلَيكُنَّ',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فلإليها'] = {
    'word': u'فلإليها',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَإِلَيهَا',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'هَا'
}
STOPWORDS[u'فلإليهم'] = {
    'word': u'فلإليهم',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَإِلَيهُمْ',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فلإليهن'] = {
    'word': u'فلإليهن',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَإِلَيهُنَّ',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فلإلينا'] = {
    'word': u'فلإلينا',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَإِلَينَا',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'نَا'
}
STOPWORDS[u'فلإليكما'] = {
    'word': u'فلإليكما',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَإِلَيكُمَا',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فلإليهما'] = {
    'word': u'فلإليهما',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَإِلَيهُمَا',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أإلي'] = {
    'word': u'أإلي',
    'procletic': u'-أَ',
    'tags': u'أداة:حرف جر:مجرور:مضاف',
    'vocalized': u'أَإِلَيِي',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'ِي'
}
STOPWORDS[u'أإليك'] = {
    'word': u'أإليك',
    'procletic': u'-أَ',
    'tags': u'أداة:حرف جر:مجرور:مضاف',
    'vocalized': u'أَإِلَيكَ',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'كَ'
}
STOPWORDS[u'أإليك'] = {
    'word': u'أإليك',
    'procletic': u'-أَ',
    'tags': u'أداة:حرف جر:مجرور:مضاف',
    'vocalized': u'أَإِلَيكِ',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'كِ'
}
STOPWORDS[u'أإليه'] = {
    'word': u'أإليه',
    'procletic': u'-أَ',
    'tags': u'أداة:حرف جر:مجرور:مضاف',
    'vocalized': u'أَإِلَيه',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'ه'
}
STOPWORDS[u'أإليكم'] = {
    'word': u'أإليكم',
    'procletic': u'-أَ',
    'tags': u'أداة:حرف جر:مجرور:مضاف',
    'vocalized': u'أَإِلَيكُمْ',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أإليكن'] = {
    'word': u'أإليكن',
    'procletic': u'-أَ',
    'tags': u'أداة:حرف جر:مجرور:مضاف',
    'vocalized': u'أَإِلَيكُنَّ',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أإليها'] = {
    'word': u'أإليها',
    'procletic': u'-أَ',
    'tags': u'أداة:حرف جر:مجرور:مضاف',
    'vocalized': u'أَإِلَيهَا',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'هَا'
}
STOPWORDS[u'أإليهم'] = {
    'word': u'أإليهم',
    'procletic': u'-أَ',
    'tags': u'أداة:حرف جر:مجرور:مضاف',
    'vocalized': u'أَإِلَيهُمْ',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أإليهن'] = {
    'word': u'أإليهن',
    'procletic': u'-أَ',
    'tags': u'أداة:حرف جر:مجرور:مضاف',
    'vocalized': u'أَإِلَيهُنَّ',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أإلينا'] = {
    'word': u'أإلينا',
    'procletic': u'-أَ',
    'tags': u'أداة:حرف جر:مجرور:مضاف',
    'vocalized': u'أَإِلَينَا',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'نَا'
}
STOPWORDS[u'أإليكما'] = {
    'word': u'أإليكما',
    'procletic': u'-أَ',
    'tags': u'أداة:حرف جر:مجرور:مضاف',
    'vocalized': u'أَإِلَيكُمَا',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أإليهما'] = {
    'word': u'أإليهما',
    'procletic': u'-أَ',
    'tags': u'أداة:حرف جر:مجرور:مضاف',
    'vocalized': u'أَإِلَيهُمَا',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ألإلي'] = {
    'word': u'ألإلي',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَإِلَيِي',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'ِي'
}
STOPWORDS[u'ألإليك'] = {
    'word': u'ألإليك',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَإِلَيكَ',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'كَ'
}
STOPWORDS[u'ألإليك'] = {
    'word': u'ألإليك',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَإِلَيكِ',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'كِ'
}
STOPWORDS[u'ألإليه'] = {
    'word': u'ألإليه',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَإِلَيه',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'ه'
}
STOPWORDS[u'ألإليكم'] = {
    'word': u'ألإليكم',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَإِلَيكُمْ',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ألإليكن'] = {
    'word': u'ألإليكن',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَإِلَيكُنَّ',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ألإليها'] = {
    'word': u'ألإليها',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَإِلَيهَا',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'هَا'
}
STOPWORDS[u'ألإليهم'] = {
    'word': u'ألإليهم',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَإِلَيهُمْ',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ألإليهن'] = {
    'word': u'ألإليهن',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَإِلَيهُنَّ',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ألإلينا'] = {
    'word': u'ألإلينا',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَإِلَينَا',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'نَا'
}
STOPWORDS[u'ألإليكما'] = {
    'word': u'ألإليكما',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَإِلَيكُمَا',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ألإليهما'] = {
    'word': u'ألإليهما',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَإِلَيهُمَا',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أوإلي'] = {
    'word': u'أوإلي',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَإِلَيِي',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'ِي'
}
STOPWORDS[u'أوإليك'] = {
    'word': u'أوإليك',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَإِلَيكَ',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'كَ'
}
STOPWORDS[u'أوإليك'] = {
    'word': u'أوإليك',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَإِلَيكِ',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'كِ'
}
STOPWORDS[u'أوإليه'] = {
    'word': u'أوإليه',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَإِلَيه',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'ه'
}
STOPWORDS[u'أوإليكم'] = {
    'word': u'أوإليكم',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَإِلَيكُمْ',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أوإليكن'] = {
    'word': u'أوإليكن',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَإِلَيكُنَّ',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أوإليها'] = {
    'word': u'أوإليها',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَإِلَيهَا',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'هَا'
}
STOPWORDS[u'أوإليهم'] = {
    'word': u'أوإليهم',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَإِلَيهُمْ',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أوإليهن'] = {
    'word': u'أوإليهن',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَإِلَيهُنَّ',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أوإلينا'] = {
    'word': u'أوإلينا',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَإِلَينَا',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'نَا'
}
STOPWORDS[u'أوإليكما'] = {
    'word': u'أوإليكما',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَإِلَيكُمَا',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أوإليهما'] = {
    'word': u'أوإليهما',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَإِلَيهُمَا',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أفإلي'] = {
    'word': u'أفإلي',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَإِلَيِي',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'ِي'
}
STOPWORDS[u'أفإليك'] = {
    'word': u'أفإليك',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَإِلَيكَ',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'كَ'
}
STOPWORDS[u'أفإليك'] = {
    'word': u'أفإليك',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَإِلَيكِ',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'كِ'
}
STOPWORDS[u'أفإليه'] = {
    'word': u'أفإليه',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَإِلَيه',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'ه'
}
STOPWORDS[u'أفإليكم'] = {
    'word': u'أفإليكم',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَإِلَيكُمْ',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أفإليكن'] = {
    'word': u'أفإليكن',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَإِلَيكُنَّ',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أفإليها'] = {
    'word': u'أفإليها',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَإِلَيهَا',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'هَا'
}
STOPWORDS[u'أفإليهم'] = {
    'word': u'أفإليهم',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَإِلَيهُمْ',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أفإليهن'] = {
    'word': u'أفإليهن',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَإِلَيهُنَّ',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أفإلينا'] = {
    'word': u'أفإلينا',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَإِلَينَا',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'نَا'
}
STOPWORDS[u'أفإليكما'] = {
    'word': u'أفإليكما',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَإِلَيكُمَا',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أفإليهما'] = {
    'word': u'أفإليهما',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَإِلَيهُمَا',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أولإلي'] = {
    'word': u'أولإلي',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَإِلَيِي',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'ِي'
}
STOPWORDS[u'أولإليك'] = {
    'word': u'أولإليك',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَإِلَيكَ',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'كَ'
}
STOPWORDS[u'أولإليك'] = {
    'word': u'أولإليك',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَإِلَيكِ',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'كِ'
}
STOPWORDS[u'أولإليه'] = {
    'word': u'أولإليه',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَإِلَيه',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'ه'
}
STOPWORDS[u'أولإليكم'] = {
    'word': u'أولإليكم',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَإِلَيكُمْ',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أولإليكن'] = {
    'word': u'أولإليكن',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَإِلَيكُنَّ',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أولإليها'] = {
    'word': u'أولإليها',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَإِلَيهَا',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'هَا'
}
STOPWORDS[u'أولإليهم'] = {
    'word': u'أولإليهم',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَإِلَيهُمْ',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أولإليهن'] = {
    'word': u'أولإليهن',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَإِلَيهُنَّ',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أولإلينا'] = {
    'word': u'أولإلينا',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَإِلَينَا',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'نَا'
}
STOPWORDS[u'أولإليكما'] = {
    'word': u'أولإليكما',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَإِلَيكُمَا',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أولإليهما'] = {
    'word': u'أولإليهما',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَإِلَيهُمَا',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أفلإلي'] = {
    'word': u'أفلإلي',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَإِلَيِي',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'ِي'
}
STOPWORDS[u'أفلإليك'] = {
    'word': u'أفلإليك',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَإِلَيكَ',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'كَ'
}
STOPWORDS[u'أفلإليك'] = {
    'word': u'أفلإليك',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَإِلَيكِ',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'كِ'
}
STOPWORDS[u'أفلإليه'] = {
    'word': u'أفلإليه',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَإِلَيه',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'ه'
}
STOPWORDS[u'أفلإليكم'] = {
    'word': u'أفلإليكم',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَإِلَيكُمْ',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أفلإليكن'] = {
    'word': u'أفلإليكن',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَإِلَيكُنَّ',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أفلإليها'] = {
    'word': u'أفلإليها',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَإِلَيهَا',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'هَا'
}
STOPWORDS[u'أفلإليهم'] = {
    'word': u'أفلإليهم',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَإِلَيهُمْ',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أفلإليهن'] = {
    'word': u'أفلإليهن',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَإِلَيهُنَّ',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أفلإلينا'] = {
    'word': u'أفلإلينا',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَإِلَينَا',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'نَا'
}
STOPWORDS[u'أفلإليكما'] = {
    'word': u'أفلإليكما',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَإِلَيكُمَا',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أفلإليهما'] = {
    'word': u'أفلإليهما',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَإِلَيهُمَا',
    'stem': u'إِلَى',
    'type': u'STOPWORD',
    'original': u'إِلَى',
    'encletic': u'هُمَا'
}
STOPWORDS[u'رب'] = {
    'word': u'رُبَّ',
    'procletic': u'',
    'tags': u'أداة:حرف جر',
    'vocalized': u'رُبَّ',
    'stem': u'رُبَّ',
    'type': u'STOPWORD',
    'original': u'رُبَّ',
    'encletic': u''
}
STOPWORDS[u'ورب'] = {
    'word': u'ورب',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف جر:معطوف',
    'vocalized': u'وَرُبَّ',
    'stem': u'رُبَّ',
    'type': u'STOPWORD',
    'original': u'رُبَّ',
    'encletic': u''
}
STOPWORDS[u'فرب'] = {
    'word': u'فرب',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف جر:معطوف',
    'vocalized': u'فَرُبَّ',
    'stem': u'رُبَّ',
    'type': u'STOPWORD',
    'original': u'رُبَّ',
    'encletic': u''
}
STOPWORDS[u'على'] = {
    'word': u'عَلَى',
    'procletic': u'',
    'tags': u'أداة:حرف جر',
    'vocalized': u'عَلَى',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u''
}
STOPWORDS[u'لعلى'] = {
    'word': u'لعلى',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور',
    'vocalized': u'لَعَلَى',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u''
}
STOPWORDS[u'وعلى'] = {
    'word': u'وعلى',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف جر:معطوف',
    'vocalized': u'وَعَلَى',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u''
}
STOPWORDS[u'فعلى'] = {
    'word': u'فعلى',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف جر:معطوف',
    'vocalized': u'فَعَلَى',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u''
}
STOPWORDS[u'ولعلى'] = {
    'word': u'ولعلى',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'وَلَعَلَى',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u''
}
STOPWORDS[u'فلعلى'] = {
    'word': u'فلعلى',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'فَلَعَلَى',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u''
}
STOPWORDS[u'أعلى'] = {
    'word': u'أعلى',
    'procletic': u'-أَ',
    'tags': u'أداة:حرف جر:مجرور',
    'vocalized': u'أَعَلَى',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u''
}
STOPWORDS[u'ألعلى'] = {
    'word': u'ألعلى',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور',
    'vocalized': u'أَلَعَلَى',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u''
}
STOPWORDS[u'أوعلى'] = {
    'word': u'أوعلى',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور',
    'vocalized': u'أَوَعَلَى',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u''
}
STOPWORDS[u'أفعلى'] = {
    'word': u'أفعلى',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور',
    'vocalized': u'أَفَعَلَى',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u''
}
STOPWORDS[u'أولعلى'] = {
    'word': u'أولعلى',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَلَعَلَى',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u''
}
STOPWORDS[u'أفلعلى'] = {
    'word': u'أفلعلى',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَلَعَلَى',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u''
}
STOPWORDS[u'علي'] = {
    'word': u'علي',
    'procletic': u'',
    'tags': u'أداة:حرف جر:مضاف',
    'vocalized': u'عَلَيِي',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'ِي'
}
STOPWORDS[u'عليك'] = {
    'word': u'عليك',
    'procletic': u'',
    'tags': u'أداة:حرف جر:مضاف',
    'vocalized': u'عَلَيكَ',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'كَ'
}
STOPWORDS[u'عليك'] = {
    'word': u'عليك',
    'procletic': u'',
    'tags': u'أداة:حرف جر:مضاف',
    'vocalized': u'عَلَيكِ',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'كِ'
}
STOPWORDS[u'عليه'] = {
    'word': u'عليه',
    'procletic': u'',
    'tags': u'أداة:حرف جر:مضاف',
    'vocalized': u'عَلَيه',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'ه'
}
STOPWORDS[u'عليكم'] = {
    'word': u'عليكم',
    'procletic': u'',
    'tags': u'أداة:حرف جر:مضاف',
    'vocalized': u'عَلَيكُمْ',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'كُمْ'
}
STOPWORDS[u'عليكن'] = {
    'word': u'عليكن',
    'procletic': u'',
    'tags': u'أداة:حرف جر:مضاف',
    'vocalized': u'عَلَيكُنَّ',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'عليها'] = {
    'word': u'عليها',
    'procletic': u'',
    'tags': u'أداة:حرف جر:مضاف',
    'vocalized': u'عَلَيهَا',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'هَا'
}
STOPWORDS[u'عليهم'] = {
    'word': u'عليهم',
    'procletic': u'',
    'tags': u'أداة:حرف جر:مضاف',
    'vocalized': u'عَلَيهُمْ',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'هُمْ'
}
STOPWORDS[u'عليهن'] = {
    'word': u'عليهن',
    'procletic': u'',
    'tags': u'أداة:حرف جر:مضاف',
    'vocalized': u'عَلَيهُنَّ',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'علينا'] = {
    'word': u'علينا',
    'procletic': u'',
    'tags': u'أداة:حرف جر:مضاف',
    'vocalized': u'عَلَينَا',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'نَا'
}
STOPWORDS[u'عليكما'] = {
    'word': u'عليكما',
    'procletic': u'',
    'tags': u'أداة:حرف جر:مضاف',
    'vocalized': u'عَلَيكُمَا',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'كُمَا'
}
STOPWORDS[u'عليهما'] = {
    'word': u'عليهما',
    'procletic': u'',
    'tags': u'أداة:حرف جر:مضاف',
    'vocalized': u'عَلَيهُمَا',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'هُمَا'
}
STOPWORDS[u'لعلي'] = {
    'word': u'لعلي',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَعَلَيِي',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'ِي'
}
STOPWORDS[u'لعليك'] = {
    'word': u'لعليك',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَعَلَيكَ',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'كَ'
}
STOPWORDS[u'لعليك'] = {
    'word': u'لعليك',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَعَلَيكِ',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'كِ'
}
STOPWORDS[u'لعليه'] = {
    'word': u'لعليه',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَعَلَيه',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'ه'
}
STOPWORDS[u'لعليكم'] = {
    'word': u'لعليكم',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَعَلَيكُمْ',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'كُمْ'
}
STOPWORDS[u'لعليكن'] = {
    'word': u'لعليكن',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَعَلَيكُنَّ',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'لعليها'] = {
    'word': u'لعليها',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَعَلَيهَا',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'هَا'
}
STOPWORDS[u'لعليهم'] = {
    'word': u'لعليهم',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَعَلَيهُمْ',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'هُمْ'
}
STOPWORDS[u'لعليهن'] = {
    'word': u'لعليهن',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَعَلَيهُنَّ',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'لعلينا'] = {
    'word': u'لعلينا',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَعَلَينَا',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'نَا'
}
STOPWORDS[u'لعليكما'] = {
    'word': u'لعليكما',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَعَلَيكُمَا',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'كُمَا'
}
STOPWORDS[u'لعليهما'] = {
    'word': u'لعليهما',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَعَلَيهُمَا',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'هُمَا'
}
STOPWORDS[u'وعلي'] = {
    'word': u'وعلي',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'وَعَلَيِي',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'ِي'
}
STOPWORDS[u'وعليك'] = {
    'word': u'وعليك',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'وَعَلَيكَ',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'كَ'
}
STOPWORDS[u'وعليك'] = {
    'word': u'وعليك',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'وَعَلَيكِ',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'كِ'
}
STOPWORDS[u'وعليه'] = {
    'word': u'وعليه',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'وَعَلَيه',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'ه'
}
STOPWORDS[u'وعليكم'] = {
    'word': u'وعليكم',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'وَعَلَيكُمْ',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'كُمْ'
}
STOPWORDS[u'وعليكن'] = {
    'word': u'وعليكن',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'وَعَلَيكُنَّ',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'وعليها'] = {
    'word': u'وعليها',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'وَعَلَيهَا',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'هَا'
}
STOPWORDS[u'وعليهم'] = {
    'word': u'وعليهم',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'وَعَلَيهُمْ',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'هُمْ'
}
STOPWORDS[u'وعليهن'] = {
    'word': u'وعليهن',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'وَعَلَيهُنَّ',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'وعلينا'] = {
    'word': u'وعلينا',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'وَعَلَينَا',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'نَا'
}
STOPWORDS[u'وعليكما'] = {
    'word': u'وعليكما',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'وَعَلَيكُمَا',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'كُمَا'
}
STOPWORDS[u'وعليهما'] = {
    'word': u'وعليهما',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'وَعَلَيهُمَا',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فعلي'] = {
    'word': u'فعلي',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'فَعَلَيِي',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'ِي'
}
STOPWORDS[u'فعليك'] = {
    'word': u'فعليك',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'فَعَلَيكَ',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'كَ'
}
STOPWORDS[u'فعليك'] = {
    'word': u'فعليك',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'فَعَلَيكِ',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'كِ'
}
STOPWORDS[u'فعليه'] = {
    'word': u'فعليه',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'فَعَلَيه',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'ه'
}
STOPWORDS[u'فعليكم'] = {
    'word': u'فعليكم',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'فَعَلَيكُمْ',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فعليكن'] = {
    'word': u'فعليكن',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'فَعَلَيكُنَّ',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فعليها'] = {
    'word': u'فعليها',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'فَعَلَيهَا',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'هَا'
}
STOPWORDS[u'فعليهم'] = {
    'word': u'فعليهم',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'فَعَلَيهُمْ',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فعليهن'] = {
    'word': u'فعليهن',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'فَعَلَيهُنَّ',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فعلينا'] = {
    'word': u'فعلينا',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'فَعَلَينَا',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'نَا'
}
STOPWORDS[u'فعليكما'] = {
    'word': u'فعليكما',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'فَعَلَيكُمَا',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فعليهما'] = {
    'word': u'فعليهما',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'فَعَلَيهُمَا',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ولعلي'] = {
    'word': u'ولعلي',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَعَلَيِي',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'ِي'
}
STOPWORDS[u'ولعليك'] = {
    'word': u'ولعليك',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَعَلَيكَ',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'كَ'
}
STOPWORDS[u'ولعليك'] = {
    'word': u'ولعليك',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَعَلَيكِ',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'كِ'
}
STOPWORDS[u'ولعليه'] = {
    'word': u'ولعليه',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَعَلَيه',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'ه'
}
STOPWORDS[u'ولعليكم'] = {
    'word': u'ولعليكم',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَعَلَيكُمْ',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ولعليكن'] = {
    'word': u'ولعليكن',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَعَلَيكُنَّ',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ولعليها'] = {
    'word': u'ولعليها',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَعَلَيهَا',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'هَا'
}
STOPWORDS[u'ولعليهم'] = {
    'word': u'ولعليهم',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَعَلَيهُمْ',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ولعليهن'] = {
    'word': u'ولعليهن',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَعَلَيهُنَّ',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ولعلينا'] = {
    'word': u'ولعلينا',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَعَلَينَا',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'نَا'
}
STOPWORDS[u'ولعليكما'] = {
    'word': u'ولعليكما',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَعَلَيكُمَا',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ولعليهما'] = {
    'word': u'ولعليهما',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَعَلَيهُمَا',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فلعلي'] = {
    'word': u'فلعلي',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَعَلَيِي',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'ِي'
}
STOPWORDS[u'فلعليك'] = {
    'word': u'فلعليك',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَعَلَيكَ',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'كَ'
}
STOPWORDS[u'فلعليك'] = {
    'word': u'فلعليك',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَعَلَيكِ',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'كِ'
}
STOPWORDS[u'فلعليه'] = {
    'word': u'فلعليه',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَعَلَيه',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'ه'
}
STOPWORDS[u'فلعليكم'] = {
    'word': u'فلعليكم',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَعَلَيكُمْ',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فلعليكن'] = {
    'word': u'فلعليكن',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَعَلَيكُنَّ',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فلعليها'] = {
    'word': u'فلعليها',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَعَلَيهَا',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'هَا'
}
STOPWORDS[u'فلعليهم'] = {
    'word': u'فلعليهم',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَعَلَيهُمْ',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فلعليهن'] = {
    'word': u'فلعليهن',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَعَلَيهُنَّ',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فلعلينا'] = {
    'word': u'فلعلينا',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَعَلَينَا',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'نَا'
}
STOPWORDS[u'فلعليكما'] = {
    'word': u'فلعليكما',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَعَلَيكُمَا',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فلعليهما'] = {
    'word': u'فلعليهما',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَعَلَيهُمَا',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أعلي'] = {
    'word': u'أعلي',
    'procletic': u'-أَ',
    'tags': u'أداة:حرف جر:مجرور:مضاف',
    'vocalized': u'أَعَلَيِي',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'ِي'
}
STOPWORDS[u'أعليك'] = {
    'word': u'أعليك',
    'procletic': u'-أَ',
    'tags': u'أداة:حرف جر:مجرور:مضاف',
    'vocalized': u'أَعَلَيكَ',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'كَ'
}
STOPWORDS[u'أعليك'] = {
    'word': u'أعليك',
    'procletic': u'-أَ',
    'tags': u'أداة:حرف جر:مجرور:مضاف',
    'vocalized': u'أَعَلَيكِ',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'كِ'
}
STOPWORDS[u'أعليه'] = {
    'word': u'أعليه',
    'procletic': u'-أَ',
    'tags': u'أداة:حرف جر:مجرور:مضاف',
    'vocalized': u'أَعَلَيه',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'ه'
}
STOPWORDS[u'أعليكم'] = {
    'word': u'أعليكم',
    'procletic': u'-أَ',
    'tags': u'أداة:حرف جر:مجرور:مضاف',
    'vocalized': u'أَعَلَيكُمْ',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أعليكن'] = {
    'word': u'أعليكن',
    'procletic': u'-أَ',
    'tags': u'أداة:حرف جر:مجرور:مضاف',
    'vocalized': u'أَعَلَيكُنَّ',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أعليها'] = {
    'word': u'أعليها',
    'procletic': u'-أَ',
    'tags': u'أداة:حرف جر:مجرور:مضاف',
    'vocalized': u'أَعَلَيهَا',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'هَا'
}
STOPWORDS[u'أعليهم'] = {
    'word': u'أعليهم',
    'procletic': u'-أَ',
    'tags': u'أداة:حرف جر:مجرور:مضاف',
    'vocalized': u'أَعَلَيهُمْ',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أعليهن'] = {
    'word': u'أعليهن',
    'procletic': u'-أَ',
    'tags': u'أداة:حرف جر:مجرور:مضاف',
    'vocalized': u'أَعَلَيهُنَّ',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أعلينا'] = {
    'word': u'أعلينا',
    'procletic': u'-أَ',
    'tags': u'أداة:حرف جر:مجرور:مضاف',
    'vocalized': u'أَعَلَينَا',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'نَا'
}
STOPWORDS[u'أعليكما'] = {
    'word': u'أعليكما',
    'procletic': u'-أَ',
    'tags': u'أداة:حرف جر:مجرور:مضاف',
    'vocalized': u'أَعَلَيكُمَا',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أعليهما'] = {
    'word': u'أعليهما',
    'procletic': u'-أَ',
    'tags': u'أداة:حرف جر:مجرور:مضاف',
    'vocalized': u'أَعَلَيهُمَا',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ألعلي'] = {
    'word': u'ألعلي',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَعَلَيِي',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'ِي'
}
STOPWORDS[u'ألعليك'] = {
    'word': u'ألعليك',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَعَلَيكَ',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'كَ'
}
STOPWORDS[u'ألعليك'] = {
    'word': u'ألعليك',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَعَلَيكِ',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'كِ'
}
STOPWORDS[u'ألعليه'] = {
    'word': u'ألعليه',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَعَلَيه',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'ه'
}
STOPWORDS[u'ألعليكم'] = {
    'word': u'ألعليكم',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَعَلَيكُمْ',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ألعليكن'] = {
    'word': u'ألعليكن',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَعَلَيكُنَّ',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ألعليها'] = {
    'word': u'ألعليها',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَعَلَيهَا',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'هَا'
}
STOPWORDS[u'ألعليهم'] = {
    'word': u'ألعليهم',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَعَلَيهُمْ',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ألعليهن'] = {
    'word': u'ألعليهن',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَعَلَيهُنَّ',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ألعلينا'] = {
    'word': u'ألعلينا',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَعَلَينَا',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'نَا'
}
STOPWORDS[u'ألعليكما'] = {
    'word': u'ألعليكما',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَعَلَيكُمَا',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ألعليهما'] = {
    'word': u'ألعليهما',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَعَلَيهُمَا',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أوعلي'] = {
    'word': u'أوعلي',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَعَلَيِي',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'ِي'
}
STOPWORDS[u'أوعليك'] = {
    'word': u'أوعليك',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَعَلَيكَ',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'كَ'
}
STOPWORDS[u'أوعليك'] = {
    'word': u'أوعليك',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَعَلَيكِ',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'كِ'
}
STOPWORDS[u'أوعليه'] = {
    'word': u'أوعليه',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَعَلَيه',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'ه'
}
STOPWORDS[u'أوعليكم'] = {
    'word': u'أوعليكم',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَعَلَيكُمْ',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أوعليكن'] = {
    'word': u'أوعليكن',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَعَلَيكُنَّ',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أوعليها'] = {
    'word': u'أوعليها',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَعَلَيهَا',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'هَا'
}
STOPWORDS[u'أوعليهم'] = {
    'word': u'أوعليهم',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَعَلَيهُمْ',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أوعليهن'] = {
    'word': u'أوعليهن',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَعَلَيهُنَّ',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أوعلينا'] = {
    'word': u'أوعلينا',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَعَلَينَا',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'نَا'
}
STOPWORDS[u'أوعليكما'] = {
    'word': u'أوعليكما',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَعَلَيكُمَا',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أوعليهما'] = {
    'word': u'أوعليهما',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَعَلَيهُمَا',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أفعلي'] = {
    'word': u'أفعلي',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَعَلَيِي',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'ِي'
}
STOPWORDS[u'أفعليك'] = {
    'word': u'أفعليك',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَعَلَيكَ',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'كَ'
}
STOPWORDS[u'أفعليك'] = {
    'word': u'أفعليك',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَعَلَيكِ',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'كِ'
}
STOPWORDS[u'أفعليه'] = {
    'word': u'أفعليه',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَعَلَيه',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'ه'
}
STOPWORDS[u'أفعليكم'] = {
    'word': u'أفعليكم',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَعَلَيكُمْ',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أفعليكن'] = {
    'word': u'أفعليكن',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَعَلَيكُنَّ',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أفعليها'] = {
    'word': u'أفعليها',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَعَلَيهَا',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'هَا'
}
STOPWORDS[u'أفعليهم'] = {
    'word': u'أفعليهم',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَعَلَيهُمْ',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أفعليهن'] = {
    'word': u'أفعليهن',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَعَلَيهُنَّ',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أفعلينا'] = {
    'word': u'أفعلينا',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَعَلَينَا',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'نَا'
}
STOPWORDS[u'أفعليكما'] = {
    'word': u'أفعليكما',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَعَلَيكُمَا',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أفعليهما'] = {
    'word': u'أفعليهما',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَعَلَيهُمَا',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أولعلي'] = {
    'word': u'أولعلي',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَعَلَيِي',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'ِي'
}
STOPWORDS[u'أولعليك'] = {
    'word': u'أولعليك',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَعَلَيكَ',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'كَ'
}
STOPWORDS[u'أولعليك'] = {
    'word': u'أولعليك',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَعَلَيكِ',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'كِ'
}
STOPWORDS[u'أولعليه'] = {
    'word': u'أولعليه',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَعَلَيه',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'ه'
}
STOPWORDS[u'أولعليكم'] = {
    'word': u'أولعليكم',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَعَلَيكُمْ',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أولعليكن'] = {
    'word': u'أولعليكن',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَعَلَيكُنَّ',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أولعليها'] = {
    'word': u'أولعليها',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَعَلَيهَا',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'هَا'
}
STOPWORDS[u'أولعليهم'] = {
    'word': u'أولعليهم',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَعَلَيهُمْ',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أولعليهن'] = {
    'word': u'أولعليهن',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَعَلَيهُنَّ',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أولعلينا'] = {
    'word': u'أولعلينا',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَعَلَينَا',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'نَا'
}
STOPWORDS[u'أولعليكما'] = {
    'word': u'أولعليكما',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَعَلَيكُمَا',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أولعليهما'] = {
    'word': u'أولعليهما',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَعَلَيهُمَا',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أفلعلي'] = {
    'word': u'أفلعلي',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَعَلَيِي',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'ِي'
}
STOPWORDS[u'أفلعليك'] = {
    'word': u'أفلعليك',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَعَلَيكَ',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'كَ'
}
STOPWORDS[u'أفلعليك'] = {
    'word': u'أفلعليك',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَعَلَيكِ',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'كِ'
}
STOPWORDS[u'أفلعليه'] = {
    'word': u'أفلعليه',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَعَلَيه',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'ه'
}
STOPWORDS[u'أفلعليكم'] = {
    'word': u'أفلعليكم',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَعَلَيكُمْ',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أفلعليكن'] = {
    'word': u'أفلعليكن',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَعَلَيكُنَّ',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أفلعليها'] = {
    'word': u'أفلعليها',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَعَلَيهَا',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'هَا'
}
STOPWORDS[u'أفلعليهم'] = {
    'word': u'أفلعليهم',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَعَلَيهُمْ',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أفلعليهن'] = {
    'word': u'أفلعليهن',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَعَلَيهُنَّ',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أفلعلينا'] = {
    'word': u'أفلعلينا',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَعَلَينَا',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'نَا'
}
STOPWORDS[u'أفلعليكما'] = {
    'word': u'أفلعليكما',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَعَلَيكُمَا',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أفلعليهما'] = {
    'word': u'أفلعليهما',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَعَلَيهُمَا',
    'stem': u'عَلَى',
    'type': u'STOPWORD',
    'original': u'عَلَى',
    'encletic': u'هُمَا'
}
STOPWORDS[u'عن'] = {
    'word': u'عَنْ',
    'procletic': u'',
    'tags': u'أداة:حرف جر',
    'vocalized': u'عَنْ',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u''
}
STOPWORDS[u'لعن'] = {
    'word': u'لعن',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور',
    'vocalized': u'لَعَنْ',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u''
}
STOPWORDS[u'وعن'] = {
    'word': u'وعن',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف جر:معطوف',
    'vocalized': u'وَعَنْ',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u''
}
STOPWORDS[u'فعن'] = {
    'word': u'فعن',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف جر:معطوف',
    'vocalized': u'فَعَنْ',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u''
}
STOPWORDS[u'ولعن'] = {
    'word': u'ولعن',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'وَلَعَنْ',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u''
}
STOPWORDS[u'فلعن'] = {
    'word': u'فلعن',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'فَلَعَنْ',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u''
}
STOPWORDS[u'أعن'] = {
    'word': u'أعن',
    'procletic': u'-أَ',
    'tags': u'أداة:حرف جر:مجرور',
    'vocalized': u'أَعَنْ',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u''
}
STOPWORDS[u'ألعن'] = {
    'word': u'ألعن',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور',
    'vocalized': u'أَلَعَنْ',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u''
}
STOPWORDS[u'أوعن'] = {
    'word': u'أوعن',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور',
    'vocalized': u'أَوَعَنْ',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u''
}
STOPWORDS[u'أفعن'] = {
    'word': u'أفعن',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور',
    'vocalized': u'أَفَعَنْ',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u''
}
STOPWORDS[u'أولعن'] = {
    'word': u'أولعن',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَلَعَنْ',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u''
}
STOPWORDS[u'أفلعن'] = {
    'word': u'أفلعن',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَلَعَنْ',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u''
}
STOPWORDS[u'عني'] = {
    'word': u'عني',
    'procletic': u'',
    'tags': u'أداة:حرف جر:مضاف',
    'vocalized': u'عَنِْي',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'ِي'
}
STOPWORDS[u'عنك'] = {
    'word': u'عنك',
    'procletic': u'',
    'tags': u'أداة:حرف جر:مضاف',
    'vocalized': u'عَنْكَ',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'كَ'
}
STOPWORDS[u'عنك'] = {
    'word': u'عنك',
    'procletic': u'',
    'tags': u'أداة:حرف جر:مضاف',
    'vocalized': u'عَنْكِ',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'كِ'
}
STOPWORDS[u'عنه'] = {
    'word': u'عنه',
    'procletic': u'',
    'tags': u'أداة:حرف جر:مضاف',
    'vocalized': u'عَنْه',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'ه'
}
STOPWORDS[u'عنكم'] = {
    'word': u'عنكم',
    'procletic': u'',
    'tags': u'أداة:حرف جر:مضاف',
    'vocalized': u'عَنْكُمْ',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'عنكن'] = {
    'word': u'عنكن',
    'procletic': u'',
    'tags': u'أداة:حرف جر:مضاف',
    'vocalized': u'عَنْكُنَّ',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'عنها'] = {
    'word': u'عنها',
    'procletic': u'',
    'tags': u'أداة:حرف جر:مضاف',
    'vocalized': u'عَنْهَا',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'هَا'
}
STOPWORDS[u'عنهم'] = {
    'word': u'عنهم',
    'procletic': u'',
    'tags': u'أداة:حرف جر:مضاف',
    'vocalized': u'عَنْهُمْ',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'عنهن'] = {
    'word': u'عنهن',
    'procletic': u'',
    'tags': u'أداة:حرف جر:مضاف',
    'vocalized': u'عَنْهُنَّ',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'عننا'] = {
    'word': u'عننا',
    'procletic': u'',
    'tags': u'أداة:حرف جر:مضاف',
    'vocalized': u'عَنْنَا',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'نَا'
}
STOPWORDS[u'عنكما'] = {
    'word': u'عنكما',
    'procletic': u'',
    'tags': u'أداة:حرف جر:مضاف',
    'vocalized': u'عَنْكُمَا',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'عنهما'] = {
    'word': u'عنهما',
    'procletic': u'',
    'tags': u'أداة:حرف جر:مضاف',
    'vocalized': u'عَنْهُمَا',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'لعني'] = {
    'word': u'لعني',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَعَنِْي',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'ِي'
}
STOPWORDS[u'لعنك'] = {
    'word': u'لعنك',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَعَنْكَ',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'كَ'
}
STOPWORDS[u'لعنك'] = {
    'word': u'لعنك',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَعَنْكِ',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'كِ'
}
STOPWORDS[u'لعنه'] = {
    'word': u'لعنه',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَعَنْه',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'ه'
}
STOPWORDS[u'لعنكم'] = {
    'word': u'لعنكم',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَعَنْكُمْ',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'لعنكن'] = {
    'word': u'لعنكن',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَعَنْكُنَّ',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'لعنها'] = {
    'word': u'لعنها',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَعَنْهَا',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'هَا'
}
STOPWORDS[u'لعنهم'] = {
    'word': u'لعنهم',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَعَنْهُمْ',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'لعنهن'] = {
    'word': u'لعنهن',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَعَنْهُنَّ',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'لعننا'] = {
    'word': u'لعننا',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَعَنْنَا',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'نَا'
}
STOPWORDS[u'لعنكما'] = {
    'word': u'لعنكما',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَعَنْكُمَا',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'لعنهما'] = {
    'word': u'لعنهما',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَعَنْهُمَا',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'وعني'] = {
    'word': u'وعني',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'وَعَنِْي',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'ِي'
}
STOPWORDS[u'وعنك'] = {
    'word': u'وعنك',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'وَعَنْكَ',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'كَ'
}
STOPWORDS[u'وعنك'] = {
    'word': u'وعنك',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'وَعَنْكِ',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'كِ'
}
STOPWORDS[u'وعنه'] = {
    'word': u'وعنه',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'وَعَنْه',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'ه'
}
STOPWORDS[u'وعنكم'] = {
    'word': u'وعنكم',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'وَعَنْكُمْ',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'وعنكن'] = {
    'word': u'وعنكن',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'وَعَنْكُنَّ',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'وعنها'] = {
    'word': u'وعنها',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'وَعَنْهَا',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'هَا'
}
STOPWORDS[u'وعنهم'] = {
    'word': u'وعنهم',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'وَعَنْهُمْ',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'وعنهن'] = {
    'word': u'وعنهن',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'وَعَنْهُنَّ',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'وعننا'] = {
    'word': u'وعننا',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'وَعَنْنَا',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'نَا'
}
STOPWORDS[u'وعنكما'] = {
    'word': u'وعنكما',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'وَعَنْكُمَا',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'وعنهما'] = {
    'word': u'وعنهما',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'وَعَنْهُمَا',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فعني'] = {
    'word': u'فعني',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'فَعَنِْي',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'ِي'
}
STOPWORDS[u'فعنك'] = {
    'word': u'فعنك',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'فَعَنْكَ',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'كَ'
}
STOPWORDS[u'فعنك'] = {
    'word': u'فعنك',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'فَعَنْكِ',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'كِ'
}
STOPWORDS[u'فعنه'] = {
    'word': u'فعنه',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'فَعَنْه',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'ه'
}
STOPWORDS[u'فعنكم'] = {
    'word': u'فعنكم',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'فَعَنْكُمْ',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فعنكن'] = {
    'word': u'فعنكن',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'فَعَنْكُنَّ',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فعنها'] = {
    'word': u'فعنها',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'فَعَنْهَا',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'هَا'
}
STOPWORDS[u'فعنهم'] = {
    'word': u'فعنهم',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'فَعَنْهُمْ',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فعنهن'] = {
    'word': u'فعنهن',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'فَعَنْهُنَّ',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فعننا'] = {
    'word': u'فعننا',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'فَعَنْنَا',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'نَا'
}
STOPWORDS[u'فعنكما'] = {
    'word': u'فعنكما',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'فَعَنْكُمَا',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فعنهما'] = {
    'word': u'فعنهما',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'فَعَنْهُمَا',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ولعني'] = {
    'word': u'ولعني',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَعَنِْي',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'ِي'
}
STOPWORDS[u'ولعنك'] = {
    'word': u'ولعنك',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَعَنْكَ',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'كَ'
}
STOPWORDS[u'ولعنك'] = {
    'word': u'ولعنك',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَعَنْكِ',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'كِ'
}
STOPWORDS[u'ولعنه'] = {
    'word': u'ولعنه',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَعَنْه',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'ه'
}
STOPWORDS[u'ولعنكم'] = {
    'word': u'ولعنكم',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَعَنْكُمْ',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ولعنكن'] = {
    'word': u'ولعنكن',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَعَنْكُنَّ',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ولعنها'] = {
    'word': u'ولعنها',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَعَنْهَا',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'هَا'
}
STOPWORDS[u'ولعنهم'] = {
    'word': u'ولعنهم',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَعَنْهُمْ',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ولعنهن'] = {
    'word': u'ولعنهن',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَعَنْهُنَّ',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ولعننا'] = {
    'word': u'ولعننا',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَعَنْنَا',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'نَا'
}
STOPWORDS[u'ولعنكما'] = {
    'word': u'ولعنكما',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَعَنْكُمَا',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ولعنهما'] = {
    'word': u'ولعنهما',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَعَنْهُمَا',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فلعني'] = {
    'word': u'فلعني',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَعَنِْي',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'ِي'
}
STOPWORDS[u'فلعنك'] = {
    'word': u'فلعنك',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَعَنْكَ',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'كَ'
}
STOPWORDS[u'فلعنك'] = {
    'word': u'فلعنك',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَعَنْكِ',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'كِ'
}
STOPWORDS[u'فلعنه'] = {
    'word': u'فلعنه',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَعَنْه',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'ه'
}
STOPWORDS[u'فلعنكم'] = {
    'word': u'فلعنكم',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَعَنْكُمْ',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فلعنكن'] = {
    'word': u'فلعنكن',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَعَنْكُنَّ',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فلعنها'] = {
    'word': u'فلعنها',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَعَنْهَا',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'هَا'
}
STOPWORDS[u'فلعنهم'] = {
    'word': u'فلعنهم',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَعَنْهُمْ',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فلعنهن'] = {
    'word': u'فلعنهن',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَعَنْهُنَّ',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فلعننا'] = {
    'word': u'فلعننا',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَعَنْنَا',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'نَا'
}
STOPWORDS[u'فلعنكما'] = {
    'word': u'فلعنكما',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَعَنْكُمَا',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فلعنهما'] = {
    'word': u'فلعنهما',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَعَنْهُمَا',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أعني'] = {
    'word': u'أعني',
    'procletic': u'-أَ',
    'tags': u'أداة:حرف جر:مجرور:مضاف',
    'vocalized': u'أَعَنِْي',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'ِي'
}
STOPWORDS[u'أعنك'] = {
    'word': u'أعنك',
    'procletic': u'-أَ',
    'tags': u'أداة:حرف جر:مجرور:مضاف',
    'vocalized': u'أَعَنْكَ',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'كَ'
}
STOPWORDS[u'أعنك'] = {
    'word': u'أعنك',
    'procletic': u'-أَ',
    'tags': u'أداة:حرف جر:مجرور:مضاف',
    'vocalized': u'أَعَنْكِ',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'كِ'
}
STOPWORDS[u'أعنه'] = {
    'word': u'أعنه',
    'procletic': u'-أَ',
    'tags': u'أداة:حرف جر:مجرور:مضاف',
    'vocalized': u'أَعَنْه',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'ه'
}
STOPWORDS[u'أعنكم'] = {
    'word': u'أعنكم',
    'procletic': u'-أَ',
    'tags': u'أداة:حرف جر:مجرور:مضاف',
    'vocalized': u'أَعَنْكُمْ',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أعنكن'] = {
    'word': u'أعنكن',
    'procletic': u'-أَ',
    'tags': u'أداة:حرف جر:مجرور:مضاف',
    'vocalized': u'أَعَنْكُنَّ',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أعنها'] = {
    'word': u'أعنها',
    'procletic': u'-أَ',
    'tags': u'أداة:حرف جر:مجرور:مضاف',
    'vocalized': u'أَعَنْهَا',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'هَا'
}
STOPWORDS[u'أعنهم'] = {
    'word': u'أعنهم',
    'procletic': u'-أَ',
    'tags': u'أداة:حرف جر:مجرور:مضاف',
    'vocalized': u'أَعَنْهُمْ',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أعنهن'] = {
    'word': u'أعنهن',
    'procletic': u'-أَ',
    'tags': u'أداة:حرف جر:مجرور:مضاف',
    'vocalized': u'أَعَنْهُنَّ',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أعننا'] = {
    'word': u'أعننا',
    'procletic': u'-أَ',
    'tags': u'أداة:حرف جر:مجرور:مضاف',
    'vocalized': u'أَعَنْنَا',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'نَا'
}
STOPWORDS[u'أعنكما'] = {
    'word': u'أعنكما',
    'procletic': u'-أَ',
    'tags': u'أداة:حرف جر:مجرور:مضاف',
    'vocalized': u'أَعَنْكُمَا',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أعنهما'] = {
    'word': u'أعنهما',
    'procletic': u'-أَ',
    'tags': u'أداة:حرف جر:مجرور:مضاف',
    'vocalized': u'أَعَنْهُمَا',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ألعني'] = {
    'word': u'ألعني',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَعَنِْي',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'ِي'
}
STOPWORDS[u'ألعنك'] = {
    'word': u'ألعنك',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَعَنْكَ',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'كَ'
}
STOPWORDS[u'ألعنك'] = {
    'word': u'ألعنك',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَعَنْكِ',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'كِ'
}
STOPWORDS[u'ألعنه'] = {
    'word': u'ألعنه',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَعَنْه',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'ه'
}
STOPWORDS[u'ألعنكم'] = {
    'word': u'ألعنكم',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَعَنْكُمْ',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ألعنكن'] = {
    'word': u'ألعنكن',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَعَنْكُنَّ',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ألعنها'] = {
    'word': u'ألعنها',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَعَنْهَا',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'هَا'
}
STOPWORDS[u'ألعنهم'] = {
    'word': u'ألعنهم',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَعَنْهُمْ',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ألعنهن'] = {
    'word': u'ألعنهن',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَعَنْهُنَّ',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ألعننا'] = {
    'word': u'ألعننا',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَعَنْنَا',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'نَا'
}
STOPWORDS[u'ألعنكما'] = {
    'word': u'ألعنكما',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَعَنْكُمَا',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ألعنهما'] = {
    'word': u'ألعنهما',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَعَنْهُمَا',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أوعني'] = {
    'word': u'أوعني',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَعَنِْي',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'ِي'
}
STOPWORDS[u'أوعنك'] = {
    'word': u'أوعنك',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَعَنْكَ',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'كَ'
}
STOPWORDS[u'أوعنك'] = {
    'word': u'أوعنك',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَعَنْكِ',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'كِ'
}
STOPWORDS[u'أوعنه'] = {
    'word': u'أوعنه',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَعَنْه',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'ه'
}
STOPWORDS[u'أوعنكم'] = {
    'word': u'أوعنكم',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَعَنْكُمْ',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أوعنكن'] = {
    'word': u'أوعنكن',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَعَنْكُنَّ',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أوعنها'] = {
    'word': u'أوعنها',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَعَنْهَا',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'هَا'
}
STOPWORDS[u'أوعنهم'] = {
    'word': u'أوعنهم',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَعَنْهُمْ',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أوعنهن'] = {
    'word': u'أوعنهن',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَعَنْهُنَّ',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أوعننا'] = {
    'word': u'أوعننا',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَعَنْنَا',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'نَا'
}
STOPWORDS[u'أوعنكما'] = {
    'word': u'أوعنكما',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَعَنْكُمَا',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أوعنهما'] = {
    'word': u'أوعنهما',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَعَنْهُمَا',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أفعني'] = {
    'word': u'أفعني',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَعَنِْي',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'ِي'
}
STOPWORDS[u'أفعنك'] = {
    'word': u'أفعنك',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَعَنْكَ',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'كَ'
}
STOPWORDS[u'أفعنك'] = {
    'word': u'أفعنك',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَعَنْكِ',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'كِ'
}
STOPWORDS[u'أفعنه'] = {
    'word': u'أفعنه',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَعَنْه',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'ه'
}
STOPWORDS[u'أفعنكم'] = {
    'word': u'أفعنكم',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَعَنْكُمْ',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أفعنكن'] = {
    'word': u'أفعنكن',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَعَنْكُنَّ',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أفعنها'] = {
    'word': u'أفعنها',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَعَنْهَا',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'هَا'
}
STOPWORDS[u'أفعنهم'] = {
    'word': u'أفعنهم',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَعَنْهُمْ',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أفعنهن'] = {
    'word': u'أفعنهن',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَعَنْهُنَّ',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أفعننا'] = {
    'word': u'أفعننا',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَعَنْنَا',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'نَا'
}
STOPWORDS[u'أفعنكما'] = {
    'word': u'أفعنكما',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَعَنْكُمَا',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أفعنهما'] = {
    'word': u'أفعنهما',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَعَنْهُمَا',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أولعني'] = {
    'word': u'أولعني',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَعَنِْي',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'ِي'
}
STOPWORDS[u'أولعنك'] = {
    'word': u'أولعنك',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَعَنْكَ',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'كَ'
}
STOPWORDS[u'أولعنك'] = {
    'word': u'أولعنك',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَعَنْكِ',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'كِ'
}
STOPWORDS[u'أولعنه'] = {
    'word': u'أولعنه',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَعَنْه',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'ه'
}
STOPWORDS[u'أولعنكم'] = {
    'word': u'أولعنكم',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَعَنْكُمْ',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أولعنكن'] = {
    'word': u'أولعنكن',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَعَنْكُنَّ',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أولعنها'] = {
    'word': u'أولعنها',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَعَنْهَا',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'هَا'
}
STOPWORDS[u'أولعنهم'] = {
    'word': u'أولعنهم',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَعَنْهُمْ',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أولعنهن'] = {
    'word': u'أولعنهن',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَعَنْهُنَّ',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أولعننا'] = {
    'word': u'أولعننا',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَعَنْنَا',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'نَا'
}
STOPWORDS[u'أولعنكما'] = {
    'word': u'أولعنكما',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَعَنْكُمَا',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أولعنهما'] = {
    'word': u'أولعنهما',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَعَنْهُمَا',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أفلعني'] = {
    'word': u'أفلعني',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَعَنِْي',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'ِي'
}
STOPWORDS[u'أفلعنك'] = {
    'word': u'أفلعنك',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَعَنْكَ',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'كَ'
}
STOPWORDS[u'أفلعنك'] = {
    'word': u'أفلعنك',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَعَنْكِ',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'كِ'
}
STOPWORDS[u'أفلعنه'] = {
    'word': u'أفلعنه',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَعَنْه',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'ه'
}
STOPWORDS[u'أفلعنكم'] = {
    'word': u'أفلعنكم',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَعَنْكُمْ',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أفلعنكن'] = {
    'word': u'أفلعنكن',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَعَنْكُنَّ',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أفلعنها'] = {
    'word': u'أفلعنها',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَعَنْهَا',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'هَا'
}
STOPWORDS[u'أفلعنهم'] = {
    'word': u'أفلعنهم',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَعَنْهُمْ',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أفلعنهن'] = {
    'word': u'أفلعنهن',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَعَنْهُنَّ',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أفلعننا'] = {
    'word': u'أفلعننا',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَعَنْنَا',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'نَا'
}
STOPWORDS[u'أفلعنكما'] = {
    'word': u'أفلعنكما',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَعَنْكُمَا',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أفلعنهما'] = {
    'word': u'أفلعنهما',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَعَنْهُمَا',
    'stem': u'عَنْ',
    'type': u'STOPWORD',
    'original': u'عَنْ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'في'] = {
    'word': u'فِي',
    'procletic': u'',
    'tags': u'أداة:حرف جر',
    'vocalized': u'فِي',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u''
}
STOPWORDS[u'لفي'] = {
    'word': u'لفي',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور',
    'vocalized': u'لَفِي',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u''
}
STOPWORDS[u'وفي'] = {
    'word': u'وفي',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف جر:معطوف',
    'vocalized': u'وَفِي',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u''
}
STOPWORDS[u'ففي'] = {
    'word': u'ففي',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف جر:معطوف',
    'vocalized': u'فَفِي',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u''
}
STOPWORDS[u'ولفي'] = {
    'word': u'ولفي',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'وَلَفِي',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u''
}
STOPWORDS[u'فلفي'] = {
    'word': u'فلفي',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'فَلَفِي',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u''
}
STOPWORDS[u'أفي'] = {
    'word': u'أفي',
    'procletic': u'-أَ',
    'tags': u'أداة:حرف جر:مجرور',
    'vocalized': u'أَفِي',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u''
}
STOPWORDS[u'ألفي'] = {
    'word': u'ألفي',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور',
    'vocalized': u'أَلَفِي',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u''
}
STOPWORDS[u'أوفي'] = {
    'word': u'أوفي',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور',
    'vocalized': u'أَوَفِي',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u''
}
STOPWORDS[u'أففي'] = {
    'word': u'أففي',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور',
    'vocalized': u'أَفَفِي',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u''
}
STOPWORDS[u'أولفي'] = {
    'word': u'أولفي',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَلَفِي',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u''
}
STOPWORDS[u'أفلفي'] = {
    'word': u'أفلفي',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَلَفِي',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u''
}
STOPWORDS[u'فيي'] = {
    'word': u'فيي',
    'procletic': u'',
    'tags': u'أداة:حرف جر:مضاف',
    'vocalized': u'فِيِي',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'ِي'
}
STOPWORDS[u'فيك'] = {
    'word': u'فيك',
    'procletic': u'',
    'tags': u'أداة:حرف جر:مضاف',
    'vocalized': u'فِيكَ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كَ'
}
STOPWORDS[u'فيك'] = {
    'word': u'فيك',
    'procletic': u'',
    'tags': u'أداة:حرف جر:مضاف',
    'vocalized': u'فِيكِ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كِ'
}
STOPWORDS[u'فيه'] = {
    'word': u'فيه',
    'procletic': u'',
    'tags': u'أداة:حرف جر:مضاف',
    'vocalized': u'فِيه',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'ه'
}
STOPWORDS[u'فيكم'] = {
    'word': u'فيكم',
    'procletic': u'',
    'tags': u'أداة:حرف جر:مضاف',
    'vocalized': u'فِيكُمْ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فيكن'] = {
    'word': u'فيكن',
    'procletic': u'',
    'tags': u'أداة:حرف جر:مضاف',
    'vocalized': u'فِيكُنَّ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فيها'] = {
    'word': u'فيها',
    'procletic': u'',
    'tags': u'أداة:حرف جر:مضاف',
    'vocalized': u'فِيهَا',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'هَا'
}
STOPWORDS[u'فيهم'] = {
    'word': u'فيهم',
    'procletic': u'',
    'tags': u'أداة:حرف جر:مضاف',
    'vocalized': u'فِيهُمْ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فيهن'] = {
    'word': u'فيهن',
    'procletic': u'',
    'tags': u'أداة:حرف جر:مضاف',
    'vocalized': u'فِيهُنَّ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فينا'] = {
    'word': u'فينا',
    'procletic': u'',
    'tags': u'أداة:حرف جر:مضاف',
    'vocalized': u'فِينَا',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'نَا'
}
STOPWORDS[u'فيكما'] = {
    'word': u'فيكما',
    'procletic': u'',
    'tags': u'أداة:حرف جر:مضاف',
    'vocalized': u'فِيكُمَا',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فيهما'] = {
    'word': u'فيهما',
    'procletic': u'',
    'tags': u'أداة:حرف جر:مضاف',
    'vocalized': u'فِيهُمَا',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'هُمَا'
}
STOPWORDS[u'لفيي'] = {
    'word': u'لفيي',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَفِيِي',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'ِي'
}
STOPWORDS[u'لفيك'] = {
    'word': u'لفيك',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَفِيكَ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كَ'
}
STOPWORDS[u'لفيك'] = {
    'word': u'لفيك',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَفِيكِ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كِ'
}
STOPWORDS[u'لفيه'] = {
    'word': u'لفيه',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَفِيه',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'ه'
}
STOPWORDS[u'لفيكم'] = {
    'word': u'لفيكم',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَفِيكُمْ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كُمْ'
}
STOPWORDS[u'لفيكن'] = {
    'word': u'لفيكن',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَفِيكُنَّ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'لفيها'] = {
    'word': u'لفيها',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَفِيهَا',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'هَا'
}
STOPWORDS[u'لفيهم'] = {
    'word': u'لفيهم',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَفِيهُمْ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'هُمْ'
}
STOPWORDS[u'لفيهن'] = {
    'word': u'لفيهن',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَفِيهُنَّ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'لفينا'] = {
    'word': u'لفينا',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَفِينَا',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'نَا'
}
STOPWORDS[u'لفيكما'] = {
    'word': u'لفيكما',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَفِيكُمَا',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كُمَا'
}
STOPWORDS[u'لفيهما'] = {
    'word': u'لفيهما',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَفِيهُمَا',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'هُمَا'
}
STOPWORDS[u'وفيي'] = {
    'word': u'وفيي',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'وَفِيِي',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'ِي'
}
STOPWORDS[u'وفيك'] = {
    'word': u'وفيك',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'وَفِيكَ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كَ'
}
STOPWORDS[u'وفيك'] = {
    'word': u'وفيك',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'وَفِيكِ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كِ'
}
STOPWORDS[u'وفيه'] = {
    'word': u'وفيه',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'وَفِيه',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'ه'
}
STOPWORDS[u'وفيكم'] = {
    'word': u'وفيكم',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'وَفِيكُمْ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كُمْ'
}
STOPWORDS[u'وفيكن'] = {
    'word': u'وفيكن',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'وَفِيكُنَّ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'وفيها'] = {
    'word': u'وفيها',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'وَفِيهَا',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'هَا'
}
STOPWORDS[u'وفيهم'] = {
    'word': u'وفيهم',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'وَفِيهُمْ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'هُمْ'
}
STOPWORDS[u'وفيهن'] = {
    'word': u'وفيهن',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'وَفِيهُنَّ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'وفينا'] = {
    'word': u'وفينا',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'وَفِينَا',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'نَا'
}
STOPWORDS[u'وفيكما'] = {
    'word': u'وفيكما',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'وَفِيكُمَا',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كُمَا'
}
STOPWORDS[u'وفيهما'] = {
    'word': u'وفيهما',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'وَفِيهُمَا',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ففيي'] = {
    'word': u'ففيي',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'فَفِيِي',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'ِي'
}
STOPWORDS[u'ففيك'] = {
    'word': u'ففيك',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'فَفِيكَ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كَ'
}
STOPWORDS[u'ففيك'] = {
    'word': u'ففيك',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'فَفِيكِ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كِ'
}
STOPWORDS[u'ففيه'] = {
    'word': u'ففيه',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'فَفِيه',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'ه'
}
STOPWORDS[u'ففيكم'] = {
    'word': u'ففيكم',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'فَفِيكُمْ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ففيكن'] = {
    'word': u'ففيكن',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'فَفِيكُنَّ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ففيها'] = {
    'word': u'ففيها',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'فَفِيهَا',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'هَا'
}
STOPWORDS[u'ففيهم'] = {
    'word': u'ففيهم',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'فَفِيهُمْ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ففيهن'] = {
    'word': u'ففيهن',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'فَفِيهُنَّ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ففينا'] = {
    'word': u'ففينا',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'فَفِينَا',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'نَا'
}
STOPWORDS[u'ففيكما'] = {
    'word': u'ففيكما',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'فَفِيكُمَا',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ففيهما'] = {
    'word': u'ففيهما',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'فَفِيهُمَا',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ولفيي'] = {
    'word': u'ولفيي',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَفِيِي',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'ِي'
}
STOPWORDS[u'ولفيك'] = {
    'word': u'ولفيك',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَفِيكَ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كَ'
}
STOPWORDS[u'ولفيك'] = {
    'word': u'ولفيك',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَفِيكِ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كِ'
}
STOPWORDS[u'ولفيه'] = {
    'word': u'ولفيه',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَفِيه',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'ه'
}
STOPWORDS[u'ولفيكم'] = {
    'word': u'ولفيكم',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَفِيكُمْ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ولفيكن'] = {
    'word': u'ولفيكن',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَفِيكُنَّ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ولفيها'] = {
    'word': u'ولفيها',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَفِيهَا',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'هَا'
}
STOPWORDS[u'ولفيهم'] = {
    'word': u'ولفيهم',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَفِيهُمْ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ولفيهن'] = {
    'word': u'ولفيهن',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَفِيهُنَّ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ولفينا'] = {
    'word': u'ولفينا',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَفِينَا',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'نَا'
}
STOPWORDS[u'ولفيكما'] = {
    'word': u'ولفيكما',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَفِيكُمَا',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ولفيهما'] = {
    'word': u'ولفيهما',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَفِيهُمَا',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فلفيي'] = {
    'word': u'فلفيي',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَفِيِي',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'ِي'
}
STOPWORDS[u'فلفيك'] = {
    'word': u'فلفيك',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَفِيكَ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كَ'
}
STOPWORDS[u'فلفيك'] = {
    'word': u'فلفيك',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَفِيكِ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كِ'
}
STOPWORDS[u'فلفيه'] = {
    'word': u'فلفيه',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَفِيه',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'ه'
}
STOPWORDS[u'فلفيكم'] = {
    'word': u'فلفيكم',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَفِيكُمْ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فلفيكن'] = {
    'word': u'فلفيكن',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَفِيكُنَّ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فلفيها'] = {
    'word': u'فلفيها',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَفِيهَا',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'هَا'
}
STOPWORDS[u'فلفيهم'] = {
    'word': u'فلفيهم',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَفِيهُمْ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فلفيهن'] = {
    'word': u'فلفيهن',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَفِيهُنَّ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فلفينا'] = {
    'word': u'فلفينا',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَفِينَا',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'نَا'
}
STOPWORDS[u'فلفيكما'] = {
    'word': u'فلفيكما',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَفِيكُمَا',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فلفيهما'] = {
    'word': u'فلفيهما',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَفِيهُمَا',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أفيي'] = {
    'word': u'أفيي',
    'procletic': u'-أَ',
    'tags': u'أداة:حرف جر:مجرور:مضاف',
    'vocalized': u'أَفِيِي',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'ِي'
}
STOPWORDS[u'أفيك'] = {
    'word': u'أفيك',
    'procletic': u'-أَ',
    'tags': u'أداة:حرف جر:مجرور:مضاف',
    'vocalized': u'أَفِيكَ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كَ'
}
STOPWORDS[u'أفيك'] = {
    'word': u'أفيك',
    'procletic': u'-أَ',
    'tags': u'أداة:حرف جر:مجرور:مضاف',
    'vocalized': u'أَفِيكِ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كِ'
}
STOPWORDS[u'أفيه'] = {
    'word': u'أفيه',
    'procletic': u'-أَ',
    'tags': u'أداة:حرف جر:مجرور:مضاف',
    'vocalized': u'أَفِيه',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'ه'
}
STOPWORDS[u'أفيكم'] = {
    'word': u'أفيكم',
    'procletic': u'-أَ',
    'tags': u'أداة:حرف جر:مجرور:مضاف',
    'vocalized': u'أَفِيكُمْ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أفيكن'] = {
    'word': u'أفيكن',
    'procletic': u'-أَ',
    'tags': u'أداة:حرف جر:مجرور:مضاف',
    'vocalized': u'أَفِيكُنَّ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أفيها'] = {
    'word': u'أفيها',
    'procletic': u'-أَ',
    'tags': u'أداة:حرف جر:مجرور:مضاف',
    'vocalized': u'أَفِيهَا',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'هَا'
}
STOPWORDS[u'أفيهم'] = {
    'word': u'أفيهم',
    'procletic': u'-أَ',
    'tags': u'أداة:حرف جر:مجرور:مضاف',
    'vocalized': u'أَفِيهُمْ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أفيهن'] = {
    'word': u'أفيهن',
    'procletic': u'-أَ',
    'tags': u'أداة:حرف جر:مجرور:مضاف',
    'vocalized': u'أَفِيهُنَّ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أفينا'] = {
    'word': u'أفينا',
    'procletic': u'-أَ',
    'tags': u'أداة:حرف جر:مجرور:مضاف',
    'vocalized': u'أَفِينَا',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'نَا'
}
STOPWORDS[u'أفيكما'] = {
    'word': u'أفيكما',
    'procletic': u'-أَ',
    'tags': u'أداة:حرف جر:مجرور:مضاف',
    'vocalized': u'أَفِيكُمَا',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أفيهما'] = {
    'word': u'أفيهما',
    'procletic': u'-أَ',
    'tags': u'أداة:حرف جر:مجرور:مضاف',
    'vocalized': u'أَفِيهُمَا',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ألفيي'] = {
    'word': u'ألفيي',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَفِيِي',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'ِي'
}
STOPWORDS[u'ألفيك'] = {
    'word': u'ألفيك',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَفِيكَ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كَ'
}
STOPWORDS[u'ألفيك'] = {
    'word': u'ألفيك',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَفِيكِ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كِ'
}
STOPWORDS[u'ألفيه'] = {
    'word': u'ألفيه',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَفِيه',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'ه'
}
STOPWORDS[u'ألفيكم'] = {
    'word': u'ألفيكم',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَفِيكُمْ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ألفيكن'] = {
    'word': u'ألفيكن',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَفِيكُنَّ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ألفيها'] = {
    'word': u'ألفيها',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَفِيهَا',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'هَا'
}
STOPWORDS[u'ألفيهم'] = {
    'word': u'ألفيهم',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَفِيهُمْ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ألفيهن'] = {
    'word': u'ألفيهن',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَفِيهُنَّ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ألفينا'] = {
    'word': u'ألفينا',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَفِينَا',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'نَا'
}
STOPWORDS[u'ألفيكما'] = {
    'word': u'ألفيكما',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَفِيكُمَا',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ألفيهما'] = {
    'word': u'ألفيهما',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَفِيهُمَا',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أوفيي'] = {
    'word': u'أوفيي',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَفِيِي',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'ِي'
}
STOPWORDS[u'أوفيك'] = {
    'word': u'أوفيك',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَفِيكَ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كَ'
}
STOPWORDS[u'أوفيك'] = {
    'word': u'أوفيك',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَفِيكِ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كِ'
}
STOPWORDS[u'أوفيه'] = {
    'word': u'أوفيه',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَفِيه',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'ه'
}
STOPWORDS[u'أوفيكم'] = {
    'word': u'أوفيكم',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَفِيكُمْ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أوفيكن'] = {
    'word': u'أوفيكن',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَفِيكُنَّ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أوفيها'] = {
    'word': u'أوفيها',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَفِيهَا',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'هَا'
}
STOPWORDS[u'أوفيهم'] = {
    'word': u'أوفيهم',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَفِيهُمْ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أوفيهن'] = {
    'word': u'أوفيهن',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَفِيهُنَّ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أوفينا'] = {
    'word': u'أوفينا',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَفِينَا',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'نَا'
}
STOPWORDS[u'أوفيكما'] = {
    'word': u'أوفيكما',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَفِيكُمَا',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أوفيهما'] = {
    'word': u'أوفيهما',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَفِيهُمَا',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أففيي'] = {
    'word': u'أففيي',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَفِيِي',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'ِي'
}
STOPWORDS[u'أففيك'] = {
    'word': u'أففيك',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَفِيكَ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كَ'
}
STOPWORDS[u'أففيك'] = {
    'word': u'أففيك',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَفِيكِ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كِ'
}
STOPWORDS[u'أففيه'] = {
    'word': u'أففيه',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَفِيه',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'ه'
}
STOPWORDS[u'أففيكم'] = {
    'word': u'أففيكم',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَفِيكُمْ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أففيكن'] = {
    'word': u'أففيكن',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَفِيكُنَّ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أففيها'] = {
    'word': u'أففيها',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَفِيهَا',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'هَا'
}
STOPWORDS[u'أففيهم'] = {
    'word': u'أففيهم',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَفِيهُمْ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أففيهن'] = {
    'word': u'أففيهن',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَفِيهُنَّ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أففينا'] = {
    'word': u'أففينا',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَفِينَا',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'نَا'
}
STOPWORDS[u'أففيكما'] = {
    'word': u'أففيكما',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَفِيكُمَا',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أففيهما'] = {
    'word': u'أففيهما',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَفِيهُمَا',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أولفيي'] = {
    'word': u'أولفيي',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَفِيِي',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'ِي'
}
STOPWORDS[u'أولفيك'] = {
    'word': u'أولفيك',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَفِيكَ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كَ'
}
STOPWORDS[u'أولفيك'] = {
    'word': u'أولفيك',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَفِيكِ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كِ'
}
STOPWORDS[u'أولفيه'] = {
    'word': u'أولفيه',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَفِيه',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'ه'
}
STOPWORDS[u'أولفيكم'] = {
    'word': u'أولفيكم',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَفِيكُمْ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أولفيكن'] = {
    'word': u'أولفيكن',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَفِيكُنَّ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أولفيها'] = {
    'word': u'أولفيها',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَفِيهَا',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'هَا'
}
STOPWORDS[u'أولفيهم'] = {
    'word': u'أولفيهم',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَفِيهُمْ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أولفيهن'] = {
    'word': u'أولفيهن',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَفِيهُنَّ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أولفينا'] = {
    'word': u'أولفينا',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَفِينَا',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'نَا'
}
STOPWORDS[u'أولفيكما'] = {
    'word': u'أولفيكما',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَفِيكُمَا',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أولفيهما'] = {
    'word': u'أولفيهما',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَفِيهُمَا',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أفلفيي'] = {
    'word': u'أفلفيي',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَفِيِي',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'ِي'
}
STOPWORDS[u'أفلفيك'] = {
    'word': u'أفلفيك',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَفِيكَ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كَ'
}
STOPWORDS[u'أفلفيك'] = {
    'word': u'أفلفيك',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَفِيكِ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كِ'
}
STOPWORDS[u'أفلفيه'] = {
    'word': u'أفلفيه',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَفِيه',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'ه'
}
STOPWORDS[u'أفلفيكم'] = {
    'word': u'أفلفيكم',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَفِيكُمْ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أفلفيكن'] = {
    'word': u'أفلفيكن',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَفِيكُنَّ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أفلفيها'] = {
    'word': u'أفلفيها',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَفِيهَا',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'هَا'
}
STOPWORDS[u'أفلفيهم'] = {
    'word': u'أفلفيهم',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَفِيهُمْ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أفلفيهن'] = {
    'word': u'أفلفيهن',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَفِيهُنَّ',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أفلفينا'] = {
    'word': u'أفلفينا',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَفِينَا',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'نَا'
}
STOPWORDS[u'أفلفيكما'] = {
    'word': u'أفلفيكما',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَفِيكُمَا',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أفلفيهما'] = {
    'word': u'أفلفيهما',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَفِيهُمَا',
    'stem': u'فِي',
    'type': u'STOPWORD',
    'original': u'فِي',
    'encletic': u'هُمَا'
}
STOPWORDS[u'من'] = {
    'word': u'مِنْ',
    'procletic': u'',
    'tags': u'أداة:حرف جر',
    'vocalized': u'مِنْ',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u''
}
STOPWORDS[u'لمن'] = {
    'word': u'لمن',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور',
    'vocalized': u'لَمِنْ',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u''
}
STOPWORDS[u'ومن'] = {
    'word': u'ومن',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف جر:معطوف',
    'vocalized': u'وَمِنْ',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u''
}
STOPWORDS[u'فمن'] = {
    'word': u'فمن',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف جر:معطوف',
    'vocalized': u'فَمِنْ',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u''
}
STOPWORDS[u'ولمن'] = {
    'word': u'ولمن',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'وَلَمِنْ',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u''
}
STOPWORDS[u'فلمن'] = {
    'word': u'فلمن',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'فَلَمِنْ',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u''
}
STOPWORDS[u'أمن'] = {
    'word': u'أمن',
    'procletic': u'-أَ',
    'tags': u'أداة:حرف جر:مجرور',
    'vocalized': u'أَمِنْ',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u''
}
STOPWORDS[u'ألمن'] = {
    'word': u'ألمن',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور',
    'vocalized': u'أَلَمِنْ',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u''
}
STOPWORDS[u'أومن'] = {
    'word': u'أومن',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور',
    'vocalized': u'أَوَمِنْ',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u''
}
STOPWORDS[u'أفمن'] = {
    'word': u'أفمن',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور',
    'vocalized': u'أَفَمِنْ',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u''
}
STOPWORDS[u'أولمن'] = {
    'word': u'أولمن',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَلَمِنْ',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u''
}
STOPWORDS[u'أفلمن'] = {
    'word': u'أفلمن',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَلَمِنْ',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u''
}
STOPWORDS[u'مني'] = {
    'word': u'مني',
    'procletic': u'',
    'tags': u'أداة:حرف جر:مضاف',
    'vocalized': u'مِنِْي',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'ِي'
}
STOPWORDS[u'منك'] = {
    'word': u'منك',
    'procletic': u'',
    'tags': u'أداة:حرف جر:مضاف',
    'vocalized': u'مِنْكَ',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'كَ'
}
STOPWORDS[u'منك'] = {
    'word': u'منك',
    'procletic': u'',
    'tags': u'أداة:حرف جر:مضاف',
    'vocalized': u'مِنْكِ',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'كِ'
}
STOPWORDS[u'منه'] = {
    'word': u'منه',
    'procletic': u'',
    'tags': u'أداة:حرف جر:مضاف',
    'vocalized': u'مِنْهُ',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'ه'
}
STOPWORDS[u'منكم'] = {
    'word': u'منكم',
    'procletic': u'',
    'tags': u'أداة:حرف جر:مضاف',
    'vocalized': u'مِنْكُمْ',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'منكن'] = {
    'word': u'منكن',
    'procletic': u'',
    'tags': u'أداة:حرف جر:مضاف',
    'vocalized': u'مِنْكُنَّ',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'منها'] = {
    'word': u'منها',
    'procletic': u'',
    'tags': u'أداة:حرف جر:مضاف',
    'vocalized': u'مِنْهَا',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'هَا'
}
STOPWORDS[u'منهم'] = {
    'word': u'منهم',
    'procletic': u'',
    'tags': u'أداة:حرف جر:مضاف',
    'vocalized': u'مِنْهُمْ',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'منهن'] = {
    'word': u'منهن',
    'procletic': u'',
    'tags': u'أداة:حرف جر:مضاف',
    'vocalized': u'مِنْهُنَّ',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'مننا'] = {
    'word': u'مننا',
    'procletic': u'',
    'tags': u'أداة:حرف جر:مضاف',
    'vocalized': u'مِنْنَا',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'نَا'
}
STOPWORDS[u'منكما'] = {
    'word': u'منكما',
    'procletic': u'',
    'tags': u'أداة:حرف جر:مضاف',
    'vocalized': u'مِنْكُمَا',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'منهما'] = {
    'word': u'منهما',
    'procletic': u'',
    'tags': u'أداة:حرف جر:مضاف',
    'vocalized': u'مِنْهُمَا',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'لمني'] = {
    'word': u'لمني',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَمِنِْي',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'ِي'
}
STOPWORDS[u'لمنك'] = {
    'word': u'لمنك',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَمِنْكَ',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'كَ'
}
STOPWORDS[u'لمنك'] = {
    'word': u'لمنك',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَمِنْكِ',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'كِ'
}
STOPWORDS[u'لمنه'] = {
    'word': u'لمنه',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَمِنْه',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'ه'
}
STOPWORDS[u'لمنكم'] = {
    'word': u'لمنكم',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَمِنْكُمْ',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'لمنكن'] = {
    'word': u'لمنكن',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَمِنْكُنَّ',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'لمنها'] = {
    'word': u'لمنها',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَمِنْهَا',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'هَا'
}
STOPWORDS[u'لمنهم'] = {
    'word': u'لمنهم',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَمِنْهُمْ',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'لمنهن'] = {
    'word': u'لمنهن',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَمِنْهُنَّ',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'لمننا'] = {
    'word': u'لمننا',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَمِنْنَا',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'نَا'
}
STOPWORDS[u'لمنكما'] = {
    'word': u'لمنكما',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَمِنْكُمَا',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'لمنهما'] = {
    'word': u'لمنهما',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَمِنْهُمَا',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ومني'] = {
    'word': u'ومني',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'وَمِنِْي',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'ِي'
}
STOPWORDS[u'ومنك'] = {
    'word': u'ومنك',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'وَمِنْكَ',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'كَ'
}
STOPWORDS[u'ومنك'] = {
    'word': u'ومنك',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'وَمِنْكِ',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'كِ'
}
STOPWORDS[u'ومنه'] = {
    'word': u'ومنه',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'وَمِنْه',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'ه'
}
STOPWORDS[u'ومنكم'] = {
    'word': u'ومنكم',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'وَمِنْكُمْ',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ومنكن'] = {
    'word': u'ومنكن',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'وَمِنْكُنَّ',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ومنها'] = {
    'word': u'ومنها',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'وَمِنْهَا',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'هَا'
}
STOPWORDS[u'ومنهم'] = {
    'word': u'ومنهم',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'وَمِنْهُمْ',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ومنهن'] = {
    'word': u'ومنهن',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'وَمِنْهُنَّ',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ومننا'] = {
    'word': u'ومننا',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'وَمِنْنَا',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'نَا'
}
STOPWORDS[u'ومنكما'] = {
    'word': u'ومنكما',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'وَمِنْكُمَا',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ومنهما'] = {
    'word': u'ومنهما',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'وَمِنْهُمَا',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فمني'] = {
    'word': u'فمني',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'فَمِنِْي',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'ِي'
}
STOPWORDS[u'فمنك'] = {
    'word': u'فمنك',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'فَمِنْكَ',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'كَ'
}
STOPWORDS[u'فمنك'] = {
    'word': u'فمنك',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'فَمِنْكِ',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'كِ'
}
STOPWORDS[u'فمنه'] = {
    'word': u'فمنه',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'فَمِنْه',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'ه'
}
STOPWORDS[u'فمنكم'] = {
    'word': u'فمنكم',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'فَمِنْكُمْ',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فمنكن'] = {
    'word': u'فمنكن',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'فَمِنْكُنَّ',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فمنها'] = {
    'word': u'فمنها',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'فَمِنْهَا',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'هَا'
}
STOPWORDS[u'فمنهم'] = {
    'word': u'فمنهم',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'فَمِنْهُمْ',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فمنهن'] = {
    'word': u'فمنهن',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'فَمِنْهُنَّ',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فمننا'] = {
    'word': u'فمننا',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'فَمِنْنَا',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'نَا'
}
STOPWORDS[u'فمنكما'] = {
    'word': u'فمنكما',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'فَمِنْكُمَا',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فمنهما'] = {
    'word': u'فمنهما',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف جر:معطوف:مضاف',
    'vocalized': u'فَمِنْهُمَا',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ولمني'] = {
    'word': u'ولمني',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَمِنِْي',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'ِي'
}
STOPWORDS[u'ولمنك'] = {
    'word': u'ولمنك',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَمِنْكَ',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'كَ'
}
STOPWORDS[u'ولمنك'] = {
    'word': u'ولمنك',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَمِنْكِ',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'كِ'
}
STOPWORDS[u'ولمنه'] = {
    'word': u'ولمنه',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَمِنْه',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'ه'
}
STOPWORDS[u'ولمنكم'] = {
    'word': u'ولمنكم',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَمِنْكُمْ',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ولمنكن'] = {
    'word': u'ولمنكن',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَمِنْكُنَّ',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ولمنها'] = {
    'word': u'ولمنها',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَمِنْهَا',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'هَا'
}
STOPWORDS[u'ولمنهم'] = {
    'word': u'ولمنهم',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَمِنْهُمْ',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ولمنهن'] = {
    'word': u'ولمنهن',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَمِنْهُنَّ',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ولمننا'] = {
    'word': u'ولمننا',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَمِنْنَا',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'نَا'
}
STOPWORDS[u'ولمنكما'] = {
    'word': u'ولمنكما',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَمِنْكُمَا',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ولمنهما'] = {
    'word': u'ولمنهما',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَمِنْهُمَا',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فلمني'] = {
    'word': u'فلمني',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَمِنِْي',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'ِي'
}
STOPWORDS[u'فلمنك'] = {
    'word': u'فلمنك',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَمِنْكَ',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'كَ'
}
STOPWORDS[u'فلمنك'] = {
    'word': u'فلمنك',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَمِنْكِ',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'كِ'
}
STOPWORDS[u'فلمنه'] = {
    'word': u'فلمنه',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَمِنْه',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'ه'
}
STOPWORDS[u'فلمنكم'] = {
    'word': u'فلمنكم',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَمِنْكُمْ',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فلمنكن'] = {
    'word': u'فلمنكن',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَمِنْكُنَّ',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فلمنها'] = {
    'word': u'فلمنها',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَمِنْهَا',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'هَا'
}
STOPWORDS[u'فلمنهم'] = {
    'word': u'فلمنهم',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَمِنْهُمْ',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فلمنهن'] = {
    'word': u'فلمنهن',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَمِنْهُنَّ',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فلمننا'] = {
    'word': u'فلمننا',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَمِنْنَا',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'نَا'
}
STOPWORDS[u'فلمنكما'] = {
    'word': u'فلمنكما',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَمِنْكُمَا',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فلمنهما'] = {
    'word': u'فلمنهما',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَمِنْهُمَا',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أمني'] = {
    'word': u'أمني',
    'procletic': u'-أَ',
    'tags': u'أداة:حرف جر:مجرور:مضاف',
    'vocalized': u'أَمِنِْي',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'ِي'
}
STOPWORDS[u'أمنك'] = {
    'word': u'أمنك',
    'procletic': u'-أَ',
    'tags': u'أداة:حرف جر:مجرور:مضاف',
    'vocalized': u'أَمِنْكَ',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'كَ'
}
STOPWORDS[u'أمنك'] = {
    'word': u'أمنك',
    'procletic': u'-أَ',
    'tags': u'أداة:حرف جر:مجرور:مضاف',
    'vocalized': u'أَمِنْكِ',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'كِ'
}
STOPWORDS[u'أمنه'] = {
    'word': u'أمنه',
    'procletic': u'-أَ',
    'tags': u'أداة:حرف جر:مجرور:مضاف',
    'vocalized': u'أَمِنْه',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'ه'
}
STOPWORDS[u'أمنكم'] = {
    'word': u'أمنكم',
    'procletic': u'-أَ',
    'tags': u'أداة:حرف جر:مجرور:مضاف',
    'vocalized': u'أَمِنْكُمْ',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أمنكن'] = {
    'word': u'أمنكن',
    'procletic': u'-أَ',
    'tags': u'أداة:حرف جر:مجرور:مضاف',
    'vocalized': u'أَمِنْكُنَّ',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أمنها'] = {
    'word': u'أمنها',
    'procletic': u'-أَ',
    'tags': u'أداة:حرف جر:مجرور:مضاف',
    'vocalized': u'أَمِنْهَا',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'هَا'
}
STOPWORDS[u'أمنهم'] = {
    'word': u'أمنهم',
    'procletic': u'-أَ',
    'tags': u'أداة:حرف جر:مجرور:مضاف',
    'vocalized': u'أَمِنْهُمْ',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أمنهن'] = {
    'word': u'أمنهن',
    'procletic': u'-أَ',
    'tags': u'أداة:حرف جر:مجرور:مضاف',
    'vocalized': u'أَمِنْهُنَّ',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أمننا'] = {
    'word': u'أمننا',
    'procletic': u'-أَ',
    'tags': u'أداة:حرف جر:مجرور:مضاف',
    'vocalized': u'أَمِنْنَا',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'نَا'
}
STOPWORDS[u'أمنكما'] = {
    'word': u'أمنكما',
    'procletic': u'-أَ',
    'tags': u'أداة:حرف جر:مجرور:مضاف',
    'vocalized': u'أَمِنْكُمَا',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أمنهما'] = {
    'word': u'أمنهما',
    'procletic': u'-أَ',
    'tags': u'أداة:حرف جر:مجرور:مضاف',
    'vocalized': u'أَمِنْهُمَا',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ألمني'] = {
    'word': u'ألمني',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَمِنِْي',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'ِي'
}
STOPWORDS[u'ألمنك'] = {
    'word': u'ألمنك',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَمِنْكَ',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'كَ'
}
STOPWORDS[u'ألمنك'] = {
    'word': u'ألمنك',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَمِنْكِ',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'كِ'
}
STOPWORDS[u'ألمنه'] = {
    'word': u'ألمنه',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَمِنْه',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'ه'
}
STOPWORDS[u'ألمنكم'] = {
    'word': u'ألمنكم',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَمِنْكُمْ',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ألمنكن'] = {
    'word': u'ألمنكن',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَمِنْكُنَّ',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ألمنها'] = {
    'word': u'ألمنها',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَمِنْهَا',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'هَا'
}
STOPWORDS[u'ألمنهم'] = {
    'word': u'ألمنهم',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَمِنْهُمْ',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ألمنهن'] = {
    'word': u'ألمنهن',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَمِنْهُنَّ',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ألمننا'] = {
    'word': u'ألمننا',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَمِنْنَا',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'نَا'
}
STOPWORDS[u'ألمنكما'] = {
    'word': u'ألمنكما',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَمِنْكُمَا',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ألمنهما'] = {
    'word': u'ألمنهما',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَمِنْهُمَا',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أومني'] = {
    'word': u'أومني',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَمِنِْي',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'ِي'
}
STOPWORDS[u'أومنك'] = {
    'word': u'أومنك',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَمِنْكَ',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'كَ'
}
STOPWORDS[u'أومنك'] = {
    'word': u'أومنك',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَمِنْكِ',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'كِ'
}
STOPWORDS[u'أومنه'] = {
    'word': u'أومنه',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَمِنْه',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'ه'
}
STOPWORDS[u'أومنكم'] = {
    'word': u'أومنكم',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَمِنْكُمْ',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أومنكن'] = {
    'word': u'أومنكن',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَمِنْكُنَّ',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أومنها'] = {
    'word': u'أومنها',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَمِنْهَا',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'هَا'
}
STOPWORDS[u'أومنهم'] = {
    'word': u'أومنهم',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَمِنْهُمْ',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أومنهن'] = {
    'word': u'أومنهن',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَمِنْهُنَّ',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أومننا'] = {
    'word': u'أومننا',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَمِنْنَا',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'نَا'
}
STOPWORDS[u'أومنكما'] = {
    'word': u'أومنكما',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَمِنْكُمَا',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أومنهما'] = {
    'word': u'أومنهما',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَمِنْهُمَا',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أفمني'] = {
    'word': u'أفمني',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَمِنِْي',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'ِي'
}
STOPWORDS[u'أفمنك'] = {
    'word': u'أفمنك',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَمِنْكَ',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'كَ'
}
STOPWORDS[u'أفمنك'] = {
    'word': u'أفمنك',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَمِنْكِ',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'كِ'
}
STOPWORDS[u'أفمنه'] = {
    'word': u'أفمنه',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَمِنْه',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'ه'
}
STOPWORDS[u'أفمنكم'] = {
    'word': u'أفمنكم',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَمِنْكُمْ',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أفمنكن'] = {
    'word': u'أفمنكن',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَمِنْكُنَّ',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أفمنها'] = {
    'word': u'أفمنها',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَمِنْهَا',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'هَا'
}
STOPWORDS[u'أفمنهم'] = {
    'word': u'أفمنهم',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَمِنْهُمْ',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أفمنهن'] = {
    'word': u'أفمنهن',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَمِنْهُنَّ',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أفمننا'] = {
    'word': u'أفمننا',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَمِنْنَا',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'نَا'
}
STOPWORDS[u'أفمنكما'] = {
    'word': u'أفمنكما',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَمِنْكُمَا',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أفمنهما'] = {
    'word': u'أفمنهما',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:حرف جر:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَمِنْهُمَا',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أولمني'] = {
    'word': u'أولمني',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَمِنِْي',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'ِي'
}
STOPWORDS[u'أولمنك'] = {
    'word': u'أولمنك',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَمِنْكَ',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'كَ'
}
STOPWORDS[u'أولمنك'] = {
    'word': u'أولمنك',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَمِنْكِ',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'كِ'
}
STOPWORDS[u'أولمنه'] = {
    'word': u'أولمنه',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَمِنْه',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'ه'
}
STOPWORDS[u'أولمنكم'] = {
    'word': u'أولمنكم',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَمِنْكُمْ',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أولمنكن'] = {
    'word': u'أولمنكن',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَمِنْكُنَّ',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أولمنها'] = {
    'word': u'أولمنها',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَمِنْهَا',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'هَا'
}
STOPWORDS[u'أولمنهم'] = {
    'word': u'أولمنهم',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَمِنْهُمْ',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أولمنهن'] = {
    'word': u'أولمنهن',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَمِنْهُنَّ',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أولمننا'] = {
    'word': u'أولمننا',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَمِنْنَا',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'نَا'
}
STOPWORDS[u'أولمنكما'] = {
    'word': u'أولمنكما',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَمِنْكُمَا',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أولمنهما'] = {
    'word': u'أولمنهما',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَمِنْهُمَا',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أفلمني'] = {
    'word': u'أفلمني',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَمِنِْي',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'ِي'
}
STOPWORDS[u'أفلمنك'] = {
    'word': u'أفلمنك',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَمِنْكَ',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'كَ'
}
STOPWORDS[u'أفلمنك'] = {
    'word': u'أفلمنك',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَمِنْكِ',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'كِ'
}
STOPWORDS[u'أفلمنه'] = {
    'word': u'أفلمنه',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَمِنْه',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'ه'
}
STOPWORDS[u'أفلمنكم'] = {
    'word': u'أفلمنكم',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَمِنْكُمْ',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أفلمنكن'] = {
    'word': u'أفلمنكن',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَمِنْكُنَّ',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أفلمنها'] = {
    'word': u'أفلمنها',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَمِنْهَا',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'هَا'
}
STOPWORDS[u'أفلمنهم'] = {
    'word': u'أفلمنهم',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَمِنْهُمْ',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أفلمنهن'] = {
    'word': u'أفلمنهن',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَمِنْهُنَّ',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أفلمننا'] = {
    'word': u'أفلمننا',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَمِنْنَا',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'نَا'
}
STOPWORDS[u'أفلمنكما'] = {
    'word': u'أفلمنكما',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَمِنْكُمَا',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أفلمنهما'] = {
    'word': u'أفلمنهما',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَمِنْهُمَا',
    'stem': u'مِنْ',
    'type': u'STOPWORD',
    'original': u'مِنْ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'عما'] = {
    'word': u'عَمَّا',
    'procletic': u'',
    'tags': u'أداة:حرف جر مكفوف',
    'vocalized': u'عَمَّا',
    'stem': u'عَمَّا',
    'type': u'STOPWORD',
    'original': u'عَمَّا',
    'encletic': u''
}
STOPWORDS[u'وعما'] = {
    'word': u'وعما',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف جر مكفوف:معطوف',
    'vocalized': u'وَعَمَّا',
    'stem': u'عَمَّا',
    'type': u'STOPWORD',
    'original': u'عَمَّا',
    'encletic': u''
}
STOPWORDS[u'فعما'] = {
    'word': u'فعما',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف جر مكفوف:معطوف',
    'vocalized': u'فَعَمَّا',
    'stem': u'عَمَّا',
    'type': u'STOPWORD',
    'original': u'عَمَّا',
    'encletic': u''
}
STOPWORDS[u'حتى'] = {
    'word': u'حَتَّى',
    'procletic': u'',
    'tags': u'أداة:حرف جر/ تعليل',
    'vocalized': u'حَتَّى',
    'stem': u'حَتَّى',
    'type': u'STOPWORD',
    'original': u'حَتَّى',
    'encletic': u''
}
STOPWORDS[u'وحتى'] = {
    'word': u'وحتى',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف جر/ تعليل:معطوف',
    'vocalized': u'وَحَتَّى',
    'stem': u'حَتَّى',
    'type': u'STOPWORD',
    'original': u'حَتَّى',
    'encletic': u''
}
STOPWORDS[u'فحتى'] = {
    'word': u'فحتى',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف جر/ تعليل:معطوف',
    'vocalized': u'فَحَتَّى',
    'stem': u'حَتَّى',
    'type': u'STOPWORD',
    'original': u'حَتَّى',
    'encletic': u''
}
STOPWORDS[u'منذ'] = {
    'word': u'مُنْذُ',
    'procletic': u'',
    'tags': u'أداة:حرف جر/ ظرف زمان',
    'vocalized': u'مُنْذُ',
    'stem': u'مُنْذُ',
    'type': u'STOPWORD',
    'original': u'مُنْذُ',
    'encletic': u''
}
STOPWORDS[u'ومنذ'] = {
    'word': u'ومنذ',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف جر/ ظرف زمان:معطوف',
    'vocalized': u'وَمُنْذُ',
    'stem': u'مُنْذُ',
    'type': u'STOPWORD',
    'original': u'مُنْذُ',
    'encletic': u''
}
STOPWORDS[u'فمنذ'] = {
    'word': u'فمنذ',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف جر/ ظرف زمان:معطوف',
    'vocalized': u'فَمُنْذُ',
    'stem': u'مُنْذُ',
    'type': u'STOPWORD',
    'original': u'مُنْذُ',
    'encletic': u''
}
STOPWORDS[u'مذ'] = {
    'word': u'مُذْ',
    'procletic': u'',
    'tags': u'أداة:حرف جر/ظرف زمان',
    'vocalized': u'مُذْ',
    'stem': u'مُذْ',
    'type': u'STOPWORD',
    'original': u'مُذْ',
    'encletic': u''
}
STOPWORDS[u'ومذ'] = {
    'word': u'ومذ',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف جر/ظرف زمان:معطوف',
    'vocalized': u'وَمُذْ',
    'stem': u'مُذْ',
    'type': u'STOPWORD',
    'original': u'مُذْ',
    'encletic': u''
}
STOPWORDS[u'فمذ'] = {
    'word': u'فمذ',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف جر/ظرف زمان:معطوف',
    'vocalized': u'فَمُذْ',
    'stem': u'مُذْ',
    'type': u'STOPWORD',
    'original': u'مُذْ',
    'encletic': u''
}
STOPWORDS[u'لم'] = {
    'word': u'لَمْ',
    'procletic': u'',
    'tags': u'أداة:حرف جزم',
    'vocalized': u'لَمْ',
    'stem': u'لَمْ',
    'type': u'STOPWORD',
    'original': u'لَمْ',
    'encletic': u''
}
STOPWORDS[u'ولم'] = {
    'word': u'ولم',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف جزم:معطوف',
    'vocalized': u'وَلَمْ',
    'stem': u'لَمْ',
    'type': u'STOPWORD',
    'original': u'لَمْ',
    'encletic': u''
}
STOPWORDS[u'فلم'] = {
    'word': u'فلم',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف جزم:معطوف',
    'vocalized': u'فَلَمْ',
    'stem': u'لَمْ',
    'type': u'STOPWORD',
    'original': u'لَمْ',
    'encletic': u''
}
STOPWORDS[u'ألم'] = {
    'word': u'ألم',
    'procletic': u'-أَ',
    'tags': u'أداة:حرف جزم:مجرور',
    'vocalized': u'أَلَمْ',
    'stem': u'لَمْ',
    'type': u'STOPWORD',
    'original': u'لَمْ',
    'encletic': u''
}
STOPWORDS[u'أولم'] = {
    'word': u'أولم',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:حرف جزم:معطوف:مجرور',
    'vocalized': u'أَوَلَمْ',
    'stem': u'لَمْ',
    'type': u'STOPWORD',
    'original': u'لَمْ',
    'encletic': u''
}
STOPWORDS[u'أفلم'] = {
    'word': u'أفلم',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:حرف جزم:معطوف:مجرور',
    'vocalized': u'أَفَلَمْ',
    'stem': u'لَمْ',
    'type': u'STOPWORD',
    'original': u'لَمْ',
    'encletic': u''
}
STOPWORDS[u'لما'] = {
    'word': u'لَمَّا',
    'procletic': u'',
    'tags': u'أداة:حرف جزم',
    'vocalized': u'لَمَّا',
    'stem': u'لَمَّا',
    'type': u'STOPWORD',
    'original': u'لَمَّا',
    'encletic': u''
}
STOPWORDS[u'ولما'] = {
    'word': u'ولما',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف جزم:معطوف',
    'vocalized': u'وَلَمَّا',
    'stem': u'لَمَّا',
    'type': u'STOPWORD',
    'original': u'لَمَّا',
    'encletic': u''
}
STOPWORDS[u'فلما'] = {
    'word': u'فلما',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف جزم:معطوف',
    'vocalized': u'فَلَمَّا',
    'stem': u'لَمَّا',
    'type': u'STOPWORD',
    'original': u'لَمَّا',
    'encletic': u''
}
STOPWORDS[u'ألما'] = {
    'word': u'ألما',
    'procletic': u'-أَ',
    'tags': u'أداة:حرف جزم:مجرور',
    'vocalized': u'أَلَمَّا',
    'stem': u'لَمَّا',
    'type': u'STOPWORD',
    'original': u'لَمَّا',
    'encletic': u''
}
STOPWORDS[u'أولما'] = {
    'word': u'أولما',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:حرف جزم:معطوف:مجرور',
    'vocalized': u'أَوَلَمَّا',
    'stem': u'لَمَّا',
    'type': u'STOPWORD',
    'original': u'لَمَّا',
    'encletic': u''
}
STOPWORDS[u'أفلما'] = {
    'word': u'أفلما',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:حرف جزم:معطوف:مجرور',
    'vocalized': u'أَفَلَمَّا',
    'stem': u'لَمَّا',
    'type': u'STOPWORD',
    'original': u'لَمَّا',
    'encletic': u''
}
STOPWORDS[u'أجل'] = {
    'word': u'أَجَلْ',
    'procletic': u'',
    'tags': u'أداة:حرف جواب',
    'vocalized': u'أَجَلْ',
    'stem': u'أَجَلْ',
    'type': u'STOPWORD',
    'original': u'أَجَلْ',
    'encletic': u''
}
STOPWORDS[u'إذن'] = {
    'word': u'إِذَنْ',
    'procletic': u'',
    'tags': u'أداة:حرف جواب',
    'vocalized': u'إِذَنْ',
    'stem': u'إِذَنْ',
    'type': u'STOPWORD',
    'original': u'إِذَنْ',
    'encletic': u''
}
STOPWORDS[u'وإذن'] = {
    'word': u'وإذن',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف جواب:معطوف',
    'vocalized': u'وَإِذَنْ',
    'stem': u'إِذَنْ',
    'type': u'STOPWORD',
    'original': u'إِذَنْ',
    'encletic': u''
}
STOPWORDS[u'فإذن'] = {
    'word': u'فإذن',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف جواب:معطوف',
    'vocalized': u'فَإِذَنْ',
    'stem': u'إِذَنْ',
    'type': u'STOPWORD',
    'original': u'إِذَنْ',
    'encletic': u''
}
STOPWORDS[u'إي'] = {
    'word': u'إِي',
    'procletic': u'',
    'tags': u'أداة:حرف جواب',
    'vocalized': u'إِي',
    'stem': u'إِي',
    'type': u'STOPWORD',
    'original': u'إِي',
    'encletic': u''
}
STOPWORDS[u'وإي'] = {
    'word': u'وإي',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف جواب:معطوف',
    'vocalized': u'وَإِي',
    'stem': u'إِي',
    'type': u'STOPWORD',
    'original': u'إِي',
    'encletic': u''
}
STOPWORDS[u'فإي'] = {
    'word': u'فإي',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف جواب:معطوف',
    'vocalized': u'فَإِي',
    'stem': u'إِي',
    'type': u'STOPWORD',
    'original': u'إِي',
    'encletic': u''
}
STOPWORDS[u'بلى'] = {
    'word': u'بَلَى',
    'procletic': u'',
    'tags': u'أداة:حرف جواب',
    'vocalized': u'بَلَى',
    'stem': u'بَلَى',
    'type': u'STOPWORD',
    'original': u'بَلَى',
    'encletic': u''
}
STOPWORDS[u'جلل'] = {
    'word': u'جَلَلْ',
    'procletic': u'',
    'tags': u'أداة:حرف جواب',
    'vocalized': u'جَلَلْ',
    'stem': u'جَلَلْ',
    'type': u'STOPWORD',
    'original': u'جَلَلْ',
    'encletic': u''
}
STOPWORDS[u'جير'] = {
    'word': u'جَيْرِ',
    'procletic': u'',
    'tags': u'أداة:حرف جواب',
    'vocalized': u'جَيْرِ',
    'stem': u'جَيْرِ',
    'type': u'STOPWORD',
    'original': u'جَيْرِ',
    'encletic': u''
}
STOPWORDS[u'وجير'] = {
    'word': u'وجير',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف جواب:معطوف',
    'vocalized': u'وَجَيْرِ',
    'stem': u'جَيْرِ',
    'type': u'STOPWORD',
    'original': u'جَيْرِ',
    'encletic': u''
}
STOPWORDS[u'فجير'] = {
    'word': u'فجير',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف جواب:معطوف',
    'vocalized': u'فَجَيْرِ',
    'stem': u'جَيْرِ',
    'type': u'STOPWORD',
    'original': u'جَيْرِ',
    'encletic': u''
}
STOPWORDS[u'كلا'] = {
    'word': u'كَلَّا',
    'procletic': u'',
    'tags': u'أداة:حرف ردع',
    'vocalized': u'كَلَّا',
    'stem': u'كَلَّا',
    'type': u'STOPWORD',
    'original': u'كَلَّا',
    'encletic': u''
}
STOPWORDS[u'وكلا'] = {
    'word': u'وكلا',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف ردع:معطوف',
    'vocalized': u'وَكَلَّا',
    'stem': u'كَلَّا',
    'type': u'STOPWORD',
    'original': u'كَلَّا',
    'encletic': u''
}
STOPWORDS[u'فكلا'] = {
    'word': u'فكلا',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف ردع:معطوف',
    'vocalized': u'فَكَلَّا',
    'stem': u'كَلَّا',
    'type': u'STOPWORD',
    'original': u'كَلَّا',
    'encletic': u''
}
STOPWORDS[u'إذما'] = {
    'word': u'إَذْمَا',
    'procletic': u'',
    'tags': u'أداة:حرف شرط',
    'vocalized': u'إَذْمَا',
    'stem': u'إَذْمَا',
    'type': u'STOPWORD',
    'original': u'إَذْمَا',
    'encletic': u''
}
STOPWORDS[u'وإذما'] = {
    'word': u'وإذما',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف شرط:معطوف',
    'vocalized': u'وَإَذْمَا',
    'stem': u'إَذْمَا',
    'type': u'STOPWORD',
    'original': u'إَذْمَا',
    'encletic': u''
}
STOPWORDS[u'فإذما'] = {
    'word': u'فإذما',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف شرط:معطوف',
    'vocalized': u'فَإَذْمَا',
    'stem': u'إَذْمَا',
    'type': u'STOPWORD',
    'original': u'إَذْمَا',
    'encletic': u''
}
STOPWORDS[u'لئن'] = {
    'word': u'لَئِنْ',
    'procletic': u'',
    'tags': u'أداة:حرف شرط',
    'vocalized': u'لَئِنْ',
    'stem': u'لَئِنْ',
    'type': u'STOPWORD',
    'original': u'لَئِنْ',
    'encletic': u''
}
STOPWORDS[u'ولئن'] = {
    'word': u'ولئن',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف شرط:معطوف',
    'vocalized': u'وَلَئِنْ',
    'stem': u'لَئِنْ',
    'type': u'STOPWORD',
    'original': u'لَئِنْ',
    'encletic': u''
}
STOPWORDS[u'فلئن'] = {
    'word': u'فلئن',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف شرط:معطوف',
    'vocalized': u'فَلَئِنْ',
    'stem': u'لَئِنْ',
    'type': u'STOPWORD',
    'original': u'لَئِنْ',
    'encletic': u''
}
STOPWORDS[u'أما'] = {
    'word': u'أَمَّا',
    'procletic': u'',
    'tags': u'أداة:حرف شرط وتفصيل وتوكيد',
    'vocalized': u'أَمَّا',
    'stem': u'أَمَّا',
    'type': u'STOPWORD',
    'original': u'أَمَّا',
    'encletic': u''
}
STOPWORDS[u'وأما'] = {
    'word': u'وأما',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف شرط وتفصيل وتوكيد:معطوف',
    'vocalized': u'وَأَمَّا',
    'stem': u'أَمَّا',
    'type': u'STOPWORD',
    'original': u'أَمَّا',
    'encletic': u''
}
STOPWORDS[u'فأما'] = {
    'word': u'فأما',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف شرط وتفصيل وتوكيد:معطوف',
    'vocalized': u'فَأَمَّا',
    'stem': u'أَمَّا',
    'type': u'STOPWORD',
    'original': u'أَمَّا',
    'encletic': u''
}
STOPWORDS[u'ألا'] = {
    'word': u'أَلَا',
    'procletic': u'',
    'tags': u'أداة:حرف عرض',
    'vocalized': u'أَلَا',
    'stem': u'أَلَا',
    'type': u'STOPWORD',
    'original': u'أَلَا',
    'encletic': u''
}
STOPWORDS[u'وألا'] = {
    'word': u'وألا',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف عرض:معطوف',
    'vocalized': u'وَأَلَا',
    'stem': u'أَلَا',
    'type': u'STOPWORD',
    'original': u'أَلَا',
    'encletic': u''
}
STOPWORDS[u'فألا'] = {
    'word': u'فألا',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف عرض:معطوف',
    'vocalized': u'فَأَلَا',
    'stem': u'أَلَا',
    'type': u'STOPWORD',
    'original': u'أَلَا',
    'encletic': u''
}
STOPWORDS[u'أما'] = {
    'word': u'أَمَا',
    'procletic': u'',
    'tags': u'أداة:حرف عرض',
    'vocalized': u'أَمَا',
    'stem': u'أَمَا',
    'type': u'STOPWORD',
    'original': u'أَمَا',
    'encletic': u''
}
STOPWORDS[u'وأما'] = {
    'word': u'وأما',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف عرض:معطوف',
    'vocalized': u'وَأَمَا',
    'stem': u'أَمَا',
    'type': u'STOPWORD',
    'original': u'أَمَا',
    'encletic': u''
}
STOPWORDS[u'فأما'] = {
    'word': u'فأما',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف عرض:معطوف',
    'vocalized': u'فَأَمَا',
    'stem': u'أَمَا',
    'type': u'STOPWORD',
    'original': u'أَمَا',
    'encletic': u''
}
STOPWORDS[u'أم'] = {
    'word': u'أَمْ',
    'procletic': u'',
    'tags': u'أداة:حرف عطف منفصل',
    'vocalized': u'أَمْ',
    'stem': u'أَمْ',
    'type': u'STOPWORD',
    'original': u'أَمْ',
    'encletic': u''
}
STOPWORDS[u'أو'] = {
    'word': u'أَوْ',
    'procletic': u'',
    'tags': u'أداة:حرف عطف منفصل',
    'vocalized': u'أَوْ',
    'stem': u'أَوْ',
    'type': u'STOPWORD',
    'original': u'أَوْ',
    'encletic': u''
}
STOPWORDS[u'بل'] = {
    'word': u'بَلْ',
    'procletic': u'',
    'tags': u'أداة:حرف عطف منفصل',
    'vocalized': u'بَلْ',
    'stem': u'بَلْ',
    'type': u'STOPWORD',
    'original': u'بَلْ',
    'encletic': u''
}
STOPWORDS[u'ثم'] = {
    'word': u'ثُمَّ',
    'procletic': u'',
    'tags': u'أداة:حرف عطف منفصل',
    'vocalized': u'ثُمَّ',
    'stem': u'ثُمَّ',
    'type': u'STOPWORD',
    'original': u'ثُمَّ',
    'encletic': u''
}
STOPWORDS[u'أيا'] = {
    'word': u'أَيَا',
    'procletic': u'',
    'tags': u'أداة:حرف نداء',
    'vocalized': u'أَيَا',
    'stem': u'أَيَا',
    'type': u'STOPWORD',
    'original': u'أَيَا',
    'encletic': u''
}
STOPWORDS[u'وأيا'] = {
    'word': u'وأيا',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف نداء:معطوف',
    'vocalized': u'وَأَيَا',
    'stem': u'أَيَا',
    'type': u'STOPWORD',
    'original': u'أَيَا',
    'encletic': u''
}
STOPWORDS[u'فأيا'] = {
    'word': u'فأيا',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف نداء:معطوف',
    'vocalized': u'فَأَيَا',
    'stem': u'أَيَا',
    'type': u'STOPWORD',
    'original': u'أَيَا',
    'encletic': u''
}
STOPWORDS[u'هيا'] = {
    'word': u'هَيَا',
    'procletic': u'',
    'tags': u'أداة:حرف نداء',
    'vocalized': u'هَيَا',
    'stem': u'هَيَا',
    'type': u'STOPWORD',
    'original': u'هَيَا',
    'encletic': u''
}
STOPWORDS[u'وهيا'] = {
    'word': u'وهيا',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف نداء:معطوف',
    'vocalized': u'وَهَيَا',
    'stem': u'هَيَا',
    'type': u'STOPWORD',
    'original': u'هَيَا',
    'encletic': u''
}
STOPWORDS[u'فهيا'] = {
    'word': u'فهيا',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف نداء:معطوف',
    'vocalized': u'فَهَيَا',
    'stem': u'هَيَا',
    'type': u'STOPWORD',
    'original': u'هَيَا',
    'encletic': u''
}
STOPWORDS[u'أن'] = {
    'word': u'أَنْ',
    'procletic': u'',
    'tags': u'أداة:حرف نصب',
    'vocalized': u'أَنْ',
    'stem': u'أَنْ',
    'type': u'STOPWORD',
    'original': u'أَنْ',
    'encletic': u''
}
STOPWORDS[u'بأن'] = {
    'word': u'بأن',
    'procletic': u'-بِ',
    'tags': u'أداة:حرف نصب:مجرور',
    'vocalized': u'بِأَنْ',
    'stem': u'أَنْ',
    'type': u'STOPWORD',
    'original': u'أَنْ',
    'encletic': u''
}
STOPWORDS[u'كأن'] = {
    'word': u'كأن',
    'procletic': u'-كَ',
    'tags': u'أداة:حرف نصب:مجرور',
    'vocalized': u'كَأَنْ',
    'stem': u'أَنْ',
    'type': u'STOPWORD',
    'original': u'أَنْ',
    'encletic': u''
}
STOPWORDS[u'لأن'] = {
    'word': u'لأن',
    'procletic': u'-لِ',
    'tags': u'أداة:حرف نصب:مجرور',
    'vocalized': u'لِأَنْ',
    'stem': u'أَنْ',
    'type': u'STOPWORD',
    'original': u'أَنْ',
    'encletic': u''
}
STOPWORDS[u'لأن'] = {
    'word': u'لأن',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور',
    'vocalized': u'لَأَنْ',
    'stem': u'أَنْ',
    'type': u'STOPWORD',
    'original': u'أَنْ',
    'encletic': u''
}
STOPWORDS[u'وأن'] = {
    'word': u'وأن',
    'procletic': u'-وَ',
    'tags': u'أداة:حرف نصب:معطوف',
    'vocalized': u'وَأَنْ',
    'stem': u'أَنْ',
    'type': u'STOPWORD',
    'original': u'أَنْ',
    'encletic': u''
}
STOPWORDS[u'فأن'] = {
    'word': u'فأن',
    'procletic': u'-فَ',
    'tags': u'أداة:حرف نصب:معطوف',
    'vocalized': u'فَأَنْ',
    'stem': u'أَنْ',
    'type': u'STOPWORD',
    'original': u'أَنْ',
    'encletic': u''
}
STOPWORDS[u'وبأن'] = {
    'word': u'وبأن',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:حرف نصب:مجرور:معطوف',
    'vocalized': u'وَبِأَنْ',
    'stem': u'أَنْ',
    'type': u'STOPWORD',
    'original': u'أَنْ',
    'encletic': u''
}
STOPWORDS[u'فبأن'] = {
    'word': u'فبأن',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:حرف نصب:مجرور:معطوف',
    'vocalized': u'فَبِأَنْ',
    'stem': u'أَنْ',
    'type': u'STOPWORD',
    'original': u'أَنْ',
    'encletic': u''
}
STOPWORDS[u'وكأن'] = {
    'word': u'وكأن',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:حرف نصب:مجرور:معطوف',
    'vocalized': u'وَكَأَنْ',
    'stem': u'أَنْ',
    'type': u'STOPWORD',
    'original': u'أَنْ',
    'encletic': u''
}
STOPWORDS[u'فكأن'] = {
    'word': u'فكأن',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:حرف نصب:مجرور:معطوف',
    'vocalized': u'فَكَأَنْ',
    'stem': u'أَنْ',
    'type': u'STOPWORD',
    'original': u'أَنْ',
    'encletic': u''
}
STOPWORDS[u'ولأن'] = {
    'word': u'ولأن',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:حرف نصب:مجرور:معطوف',
    'vocalized': u'وَلِأَنْ',
    'stem': u'أَنْ',
    'type': u'STOPWORD',
    'original': u'أَنْ',
    'encletic': u''
}
STOPWORDS[u'فلأن'] = {
    'word': u'فلأن',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:حرف نصب:مجرور:معطوف',
    'vocalized': u'فَلِأَنْ',
    'stem': u'أَنْ',
    'type': u'STOPWORD',
    'original': u'أَنْ',
    'encletic': u''
}
STOPWORDS[u'ولأن'] = {
    'word': u'ولأن',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'وَلَأَنْ',
    'stem': u'أَنْ',
    'type': u'STOPWORD',
    'original': u'أَنْ',
    'encletic': u''
}
STOPWORDS[u'فلأن'] = {
    'word': u'فلأن',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'فَلَأَنْ',
    'stem': u'أَنْ',
    'type': u'STOPWORD',
    'original': u'أَنْ',
    'encletic': u''
}
STOPWORDS[u'بك'] = {
    'word': u'بِكَ',
    'procletic': u'',
    'tags': u'أداة:ضمير متصل مجرور',
    'vocalized': u'بِكَ',
    'stem': u'بِكَ',
    'type': u'STOPWORD',
    'original': u'بِكَ',
    'encletic': u''
}
STOPWORDS[u'وبك'] = {
    'word': u'وبك',
    'procletic': u'-وَ',
    'tags': u'أداة:ضمير متصل مجرور:معطوف',
    'vocalized': u'وَبِكَ',
    'stem': u'بِكَ',
    'type': u'STOPWORD',
    'original': u'بِكَ',
    'encletic': u''
}
STOPWORDS[u'فبك'] = {
    'word': u'فبك',
    'procletic': u'-فَ',
    'tags': u'أداة:ضمير متصل مجرور:معطوف',
    'vocalized': u'فَبِكَ',
    'stem': u'بِكَ',
    'type': u'STOPWORD',
    'original': u'بِكَ',
    'encletic': u''
}
STOPWORDS[u'أبك'] = {
    'word': u'أبك',
    'procletic': u'-أَ',
    'tags': u'أداة:ضمير متصل مجرور:مجرور',
    'vocalized': u'أَبِكَ',
    'stem': u'بِكَ',
    'type': u'STOPWORD',
    'original': u'بِكَ',
    'encletic': u''
}
STOPWORDS[u'أوبك'] = {
    'word': u'أوبك',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ضمير متصل مجرور:معطوف:مجرور',
    'vocalized': u'أَوَبِكَ',
    'stem': u'بِكَ',
    'type': u'STOPWORD',
    'original': u'بِكَ',
    'encletic': u''
}
STOPWORDS[u'أفبك'] = {
    'word': u'أفبك',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ضمير متصل مجرور:معطوف:مجرور',
    'vocalized': u'أَفَبِكَ',
    'stem': u'بِكَ',
    'type': u'STOPWORD',
    'original': u'بِكَ',
    'encletic': u''
}
STOPWORDS[u'بكم'] = {
    'word': u'بِكُمْ',
    'procletic': u'',
    'tags': u'أداة:ضمير متصل مجرور',
    'vocalized': u'بِكُمْ',
    'stem': u'بِكُمْ',
    'type': u'STOPWORD',
    'original': u'بِكُمْ',
    'encletic': u''
}
STOPWORDS[u'وبكم'] = {
    'word': u'وبكم',
    'procletic': u'-وَ',
    'tags': u'أداة:ضمير متصل مجرور:معطوف',
    'vocalized': u'وَبِكُمْ',
    'stem': u'بِكُمْ',
    'type': u'STOPWORD',
    'original': u'بِكُمْ',
    'encletic': u''
}
STOPWORDS[u'فبكم'] = {
    'word': u'فبكم',
    'procletic': u'-فَ',
    'tags': u'أداة:ضمير متصل مجرور:معطوف',
    'vocalized': u'فَبِكُمْ',
    'stem': u'بِكُمْ',
    'type': u'STOPWORD',
    'original': u'بِكُمْ',
    'encletic': u''
}
STOPWORDS[u'أبكم'] = {
    'word': u'أبكم',
    'procletic': u'-أَ',
    'tags': u'أداة:ضمير متصل مجرور:مجرور',
    'vocalized': u'أَبِكُمْ',
    'stem': u'بِكُمْ',
    'type': u'STOPWORD',
    'original': u'بِكُمْ',
    'encletic': u''
}
STOPWORDS[u'أوبكم'] = {
    'word': u'أوبكم',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ضمير متصل مجرور:معطوف:مجرور',
    'vocalized': u'أَوَبِكُمْ',
    'stem': u'بِكُمْ',
    'type': u'STOPWORD',
    'original': u'بِكُمْ',
    'encletic': u''
}
STOPWORDS[u'أفبكم'] = {
    'word': u'أفبكم',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ضمير متصل مجرور:معطوف:مجرور',
    'vocalized': u'أَفَبِكُمْ',
    'stem': u'بِكُمْ',
    'type': u'STOPWORD',
    'original': u'بِكُمْ',
    'encletic': u''
}
STOPWORDS[u'بكما'] = {
    'word': u'بِكُمَا',
    'procletic': u'',
    'tags': u'أداة:ضمير متصل مجرور',
    'vocalized': u'بِكُمَا',
    'stem': u'بِكُمَا',
    'type': u'STOPWORD',
    'original': u'بِكُمَا',
    'encletic': u''
}
STOPWORDS[u'وبكما'] = {
    'word': u'وبكما',
    'procletic': u'-وَ',
    'tags': u'أداة:ضمير متصل مجرور:معطوف',
    'vocalized': u'وَبِكُمَا',
    'stem': u'بِكُمَا',
    'type': u'STOPWORD',
    'original': u'بِكُمَا',
    'encletic': u''
}
STOPWORDS[u'فبكما'] = {
    'word': u'فبكما',
    'procletic': u'-فَ',
    'tags': u'أداة:ضمير متصل مجرور:معطوف',
    'vocalized': u'فَبِكُمَا',
    'stem': u'بِكُمَا',
    'type': u'STOPWORD',
    'original': u'بِكُمَا',
    'encletic': u''
}
STOPWORDS[u'أبكما'] = {
    'word': u'أبكما',
    'procletic': u'-أَ',
    'tags': u'أداة:ضمير متصل مجرور:مجرور',
    'vocalized': u'أَبِكُمَا',
    'stem': u'بِكُمَا',
    'type': u'STOPWORD',
    'original': u'بِكُمَا',
    'encletic': u''
}
STOPWORDS[u'أوبكما'] = {
    'word': u'أوبكما',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ضمير متصل مجرور:معطوف:مجرور',
    'vocalized': u'أَوَبِكُمَا',
    'stem': u'بِكُمَا',
    'type': u'STOPWORD',
    'original': u'بِكُمَا',
    'encletic': u''
}
STOPWORDS[u'أفبكما'] = {
    'word': u'أفبكما',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ضمير متصل مجرور:معطوف:مجرور',
    'vocalized': u'أَفَبِكُمَا',
    'stem': u'بِكُمَا',
    'type': u'STOPWORD',
    'original': u'بِكُمَا',
    'encletic': u''
}
STOPWORDS[u'بكن'] = {
    'word': u'بِكُنَّ',
    'procletic': u'',
    'tags': u'أداة:ضمير متصل مجرور',
    'vocalized': u'بِكُنَّ',
    'stem': u'بِكُنَّ',
    'type': u'STOPWORD',
    'original': u'بِكُنَّ',
    'encletic': u''
}
STOPWORDS[u'وبكن'] = {
    'word': u'وبكن',
    'procletic': u'-وَ',
    'tags': u'أداة:ضمير متصل مجرور:معطوف',
    'vocalized': u'وَبِكُنَّ',
    'stem': u'بِكُنَّ',
    'type': u'STOPWORD',
    'original': u'بِكُنَّ',
    'encletic': u''
}
STOPWORDS[u'فبكن'] = {
    'word': u'فبكن',
    'procletic': u'-فَ',
    'tags': u'أداة:ضمير متصل مجرور:معطوف',
    'vocalized': u'فَبِكُنَّ',
    'stem': u'بِكُنَّ',
    'type': u'STOPWORD',
    'original': u'بِكُنَّ',
    'encletic': u''
}
STOPWORDS[u'أبكن'] = {
    'word': u'أبكن',
    'procletic': u'-أَ',
    'tags': u'أداة:ضمير متصل مجرور:مجرور',
    'vocalized': u'أَبِكُنَّ',
    'stem': u'بِكُنَّ',
    'type': u'STOPWORD',
    'original': u'بِكُنَّ',
    'encletic': u''
}
STOPWORDS[u'أوبكن'] = {
    'word': u'أوبكن',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ضمير متصل مجرور:معطوف:مجرور',
    'vocalized': u'أَوَبِكُنَّ',
    'stem': u'بِكُنَّ',
    'type': u'STOPWORD',
    'original': u'بِكُنَّ',
    'encletic': u''
}
STOPWORDS[u'أفبكن'] = {
    'word': u'أفبكن',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ضمير متصل مجرور:معطوف:مجرور',
    'vocalized': u'أَفَبِكُنَّ',
    'stem': u'بِكُنَّ',
    'type': u'STOPWORD',
    'original': u'بِكُنَّ',
    'encletic': u''
}
STOPWORDS[u'بنا'] = {
    'word': u'بِنَا',
    'procletic': u'',
    'tags': u'أداة:ضمير متصل مجرور',
    'vocalized': u'بِنَا',
    'stem': u'بِنَا',
    'type': u'STOPWORD',
    'original': u'بِنَا',
    'encletic': u''
}
STOPWORDS[u'وبنا'] = {
    'word': u'وبنا',
    'procletic': u'-وَ',
    'tags': u'أداة:ضمير متصل مجرور:معطوف',
    'vocalized': u'وَبِنَا',
    'stem': u'بِنَا',
    'type': u'STOPWORD',
    'original': u'بِنَا',
    'encletic': u''
}
STOPWORDS[u'فبنا'] = {
    'word': u'فبنا',
    'procletic': u'-فَ',
    'tags': u'أداة:ضمير متصل مجرور:معطوف',
    'vocalized': u'فَبِنَا',
    'stem': u'بِنَا',
    'type': u'STOPWORD',
    'original': u'بِنَا',
    'encletic': u''
}
STOPWORDS[u'أبنا'] = {
    'word': u'أبنا',
    'procletic': u'-أَ',
    'tags': u'أداة:ضمير متصل مجرور:مجرور',
    'vocalized': u'أَبِنَا',
    'stem': u'بِنَا',
    'type': u'STOPWORD',
    'original': u'بِنَا',
    'encletic': u''
}
STOPWORDS[u'أوبنا'] = {
    'word': u'أوبنا',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ضمير متصل مجرور:معطوف:مجرور',
    'vocalized': u'أَوَبِنَا',
    'stem': u'بِنَا',
    'type': u'STOPWORD',
    'original': u'بِنَا',
    'encletic': u''
}
STOPWORDS[u'أفبنا'] = {
    'word': u'أفبنا',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ضمير متصل مجرور:معطوف:مجرور',
    'vocalized': u'أَفَبِنَا',
    'stem': u'بِنَا',
    'type': u'STOPWORD',
    'original': u'بِنَا',
    'encletic': u''
}
STOPWORDS[u'به'] = {
    'word': u'بِهِ',
    'procletic': u'',
    'tags': u'أداة:ضمير متصل مجرور',
    'vocalized': u'بِهِ',
    'stem': u'بِهِ',
    'type': u'STOPWORD',
    'original': u'بِهِ',
    'encletic': u''
}
STOPWORDS[u'وبه'] = {
    'word': u'وبه',
    'procletic': u'-وَ',
    'tags': u'أداة:ضمير متصل مجرور:معطوف',
    'vocalized': u'وَبِهِ',
    'stem': u'بِهِ',
    'type': u'STOPWORD',
    'original': u'بِهِ',
    'encletic': u''
}
STOPWORDS[u'فبه'] = {
    'word': u'فبه',
    'procletic': u'-فَ',
    'tags': u'أداة:ضمير متصل مجرور:معطوف',
    'vocalized': u'فَبِهِ',
    'stem': u'بِهِ',
    'type': u'STOPWORD',
    'original': u'بِهِ',
    'encletic': u''
}
STOPWORDS[u'أبه'] = {
    'word': u'أبه',
    'procletic': u'-أَ',
    'tags': u'أداة:ضمير متصل مجرور:مجرور',
    'vocalized': u'أَبِهِ',
    'stem': u'بِهِ',
    'type': u'STOPWORD',
    'original': u'بِهِ',
    'encletic': u''
}
STOPWORDS[u'أوبه'] = {
    'word': u'أوبه',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ضمير متصل مجرور:معطوف:مجرور',
    'vocalized': u'أَوَبِهِ',
    'stem': u'بِهِ',
    'type': u'STOPWORD',
    'original': u'بِهِ',
    'encletic': u''
}
STOPWORDS[u'أفبه'] = {
    'word': u'أفبه',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ضمير متصل مجرور:معطوف:مجرور',
    'vocalized': u'أَفَبِهِ',
    'stem': u'بِهِ',
    'type': u'STOPWORD',
    'original': u'بِهِ',
    'encletic': u''
}
STOPWORDS[u'بها'] = {
    'word': u'بِهَا',
    'procletic': u'',
    'tags': u'أداة:ضمير متصل مجرور',
    'vocalized': u'بِهَا',
    'stem': u'بِهَا',
    'type': u'STOPWORD',
    'original': u'بِهَا',
    'encletic': u''
}
STOPWORDS[u'وبها'] = {
    'word': u'وبها',
    'procletic': u'-وَ',
    'tags': u'أداة:ضمير متصل مجرور:معطوف',
    'vocalized': u'وَبِهَا',
    'stem': u'بِهَا',
    'type': u'STOPWORD',
    'original': u'بِهَا',
    'encletic': u''
}
STOPWORDS[u'فبها'] = {
    'word': u'فبها',
    'procletic': u'-فَ',
    'tags': u'أداة:ضمير متصل مجرور:معطوف',
    'vocalized': u'فَبِهَا',
    'stem': u'بِهَا',
    'type': u'STOPWORD',
    'original': u'بِهَا',
    'encletic': u''
}
STOPWORDS[u'أبها'] = {
    'word': u'أبها',
    'procletic': u'-أَ',
    'tags': u'أداة:ضمير متصل مجرور:مجرور',
    'vocalized': u'أَبِهَا',
    'stem': u'بِهَا',
    'type': u'STOPWORD',
    'original': u'بِهَا',
    'encletic': u''
}
STOPWORDS[u'أوبها'] = {
    'word': u'أوبها',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ضمير متصل مجرور:معطوف:مجرور',
    'vocalized': u'أَوَبِهَا',
    'stem': u'بِهَا',
    'type': u'STOPWORD',
    'original': u'بِهَا',
    'encletic': u''
}
STOPWORDS[u'أفبها'] = {
    'word': u'أفبها',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ضمير متصل مجرور:معطوف:مجرور',
    'vocalized': u'أَفَبِهَا',
    'stem': u'بِهَا',
    'type': u'STOPWORD',
    'original': u'بِهَا',
    'encletic': u''
}
STOPWORDS[u'بي'] = {
    'word': u'بِي',
    'procletic': u'',
    'tags': u'أداة:ضمير متصل مجرور',
    'vocalized': u'بِي',
    'stem': u'بِي',
    'type': u'STOPWORD',
    'original': u'بِي',
    'encletic': u''
}
STOPWORDS[u'وبي'] = {
    'word': u'وبي',
    'procletic': u'-وَ',
    'tags': u'أداة:ضمير متصل مجرور:معطوف',
    'vocalized': u'وَبِي',
    'stem': u'بِي',
    'type': u'STOPWORD',
    'original': u'بِي',
    'encletic': u''
}
STOPWORDS[u'فبي'] = {
    'word': u'فبي',
    'procletic': u'-فَ',
    'tags': u'أداة:ضمير متصل مجرور:معطوف',
    'vocalized': u'فَبِي',
    'stem': u'بِي',
    'type': u'STOPWORD',
    'original': u'بِي',
    'encletic': u''
}
STOPWORDS[u'أبي'] = {
    'word': u'أبي',
    'procletic': u'-أَ',
    'tags': u'أداة:ضمير متصل مجرور:مجرور',
    'vocalized': u'أَبِي',
    'stem': u'بِي',
    'type': u'STOPWORD',
    'original': u'بِي',
    'encletic': u''
}
STOPWORDS[u'أوبي'] = {
    'word': u'أوبي',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ضمير متصل مجرور:معطوف:مجرور',
    'vocalized': u'أَوَبِي',
    'stem': u'بِي',
    'type': u'STOPWORD',
    'original': u'بِي',
    'encletic': u''
}
STOPWORDS[u'أفبي'] = {
    'word': u'أفبي',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ضمير متصل مجرور:معطوف:مجرور',
    'vocalized': u'أَفَبِي',
    'stem': u'بِي',
    'type': u'STOPWORD',
    'original': u'بِي',
    'encletic': u''
}
STOPWORDS[u'لك'] = {
    'word': u'لَكَ',
    'procletic': u'',
    'tags': u'أداة:ضمير متصل مجرور',
    'vocalized': u'لَكَ',
    'stem': u'لَكَ',
    'type': u'STOPWORD',
    'original': u'لَكَ',
    'encletic': u''
}
STOPWORDS[u'ولك'] = {
    'word': u'ولك',
    'procletic': u'-وَ',
    'tags': u'أداة:ضمير متصل مجرور:معطوف',
    'vocalized': u'وَلَكَ',
    'stem': u'لَكَ',
    'type': u'STOPWORD',
    'original': u'لَكَ',
    'encletic': u''
}
STOPWORDS[u'فلك'] = {
    'word': u'فلك',
    'procletic': u'-فَ',
    'tags': u'أداة:ضمير متصل مجرور:معطوف',
    'vocalized': u'فَلَكَ',
    'stem': u'لَكَ',
    'type': u'STOPWORD',
    'original': u'لَكَ',
    'encletic': u''
}
STOPWORDS[u'ألك'] = {
    'word': u'ألك',
    'procletic': u'-أَ',
    'tags': u'أداة:ضمير متصل مجرور:مجرور',
    'vocalized': u'أَلَكَ',
    'stem': u'لَكَ',
    'type': u'STOPWORD',
    'original': u'لَكَ',
    'encletic': u''
}
STOPWORDS[u'أولك'] = {
    'word': u'أولك',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ضمير متصل مجرور:معطوف:مجرور',
    'vocalized': u'أَوَلَكَ',
    'stem': u'لَكَ',
    'type': u'STOPWORD',
    'original': u'لَكَ',
    'encletic': u''
}
STOPWORDS[u'أفلك'] = {
    'word': u'أفلك',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ضمير متصل مجرور:معطوف:مجرور',
    'vocalized': u'أَفَلَكَ',
    'stem': u'لَكَ',
    'type': u'STOPWORD',
    'original': u'لَكَ',
    'encletic': u''
}
STOPWORDS[u'لكم'] = {
    'word': u'لَكُمْ',
    'procletic': u'',
    'tags': u'أداة:ضمير متصل مجرور',
    'vocalized': u'لَكُمْ',
    'stem': u'لَكُمْ',
    'type': u'STOPWORD',
    'original': u'لَكُمْ',
    'encletic': u''
}
STOPWORDS[u'ولكم'] = {
    'word': u'ولكم',
    'procletic': u'-وَ',
    'tags': u'أداة:ضمير متصل مجرور:معطوف',
    'vocalized': u'وَلَكُمْ',
    'stem': u'لَكُمْ',
    'type': u'STOPWORD',
    'original': u'لَكُمْ',
    'encletic': u''
}
STOPWORDS[u'فلكم'] = {
    'word': u'فلكم',
    'procletic': u'-فَ',
    'tags': u'أداة:ضمير متصل مجرور:معطوف',
    'vocalized': u'فَلَكُمْ',
    'stem': u'لَكُمْ',
    'type': u'STOPWORD',
    'original': u'لَكُمْ',
    'encletic': u''
}
STOPWORDS[u'ألكم'] = {
    'word': u'ألكم',
    'procletic': u'-أَ',
    'tags': u'أداة:ضمير متصل مجرور:مجرور',
    'vocalized': u'أَلَكُمْ',
    'stem': u'لَكُمْ',
    'type': u'STOPWORD',
    'original': u'لَكُمْ',
    'encletic': u''
}
STOPWORDS[u'أولكم'] = {
    'word': u'أولكم',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ضمير متصل مجرور:معطوف:مجرور',
    'vocalized': u'أَوَلَكُمْ',
    'stem': u'لَكُمْ',
    'type': u'STOPWORD',
    'original': u'لَكُمْ',
    'encletic': u''
}
STOPWORDS[u'أفلكم'] = {
    'word': u'أفلكم',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ضمير متصل مجرور:معطوف:مجرور',
    'vocalized': u'أَفَلَكُمْ',
    'stem': u'لَكُمْ',
    'type': u'STOPWORD',
    'original': u'لَكُمْ',
    'encletic': u''
}
STOPWORDS[u'لكما'] = {
    'word': u'لَكُمَا',
    'procletic': u'',
    'tags': u'أداة:ضمير متصل مجرور',
    'vocalized': u'لَكُمَا',
    'stem': u'لَكُمَا',
    'type': u'STOPWORD',
    'original': u'لَكُمَا',
    'encletic': u''
}
STOPWORDS[u'ولكما'] = {
    'word': u'ولكما',
    'procletic': u'-وَ',
    'tags': u'أداة:ضمير متصل مجرور:معطوف',
    'vocalized': u'وَلَكُمَا',
    'stem': u'لَكُمَا',
    'type': u'STOPWORD',
    'original': u'لَكُمَا',
    'encletic': u''
}
STOPWORDS[u'فلكما'] = {
    'word': u'فلكما',
    'procletic': u'-فَ',
    'tags': u'أداة:ضمير متصل مجرور:معطوف',
    'vocalized': u'فَلَكُمَا',
    'stem': u'لَكُمَا',
    'type': u'STOPWORD',
    'original': u'لَكُمَا',
    'encletic': u''
}
STOPWORDS[u'ألكما'] = {
    'word': u'ألكما',
    'procletic': u'-أَ',
    'tags': u'أداة:ضمير متصل مجرور:مجرور',
    'vocalized': u'أَلَكُمَا',
    'stem': u'لَكُمَا',
    'type': u'STOPWORD',
    'original': u'لَكُمَا',
    'encletic': u''
}
STOPWORDS[u'أولكما'] = {
    'word': u'أولكما',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ضمير متصل مجرور:معطوف:مجرور',
    'vocalized': u'أَوَلَكُمَا',
    'stem': u'لَكُمَا',
    'type': u'STOPWORD',
    'original': u'لَكُمَا',
    'encletic': u''
}
STOPWORDS[u'أفلكما'] = {
    'word': u'أفلكما',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ضمير متصل مجرور:معطوف:مجرور',
    'vocalized': u'أَفَلَكُمَا',
    'stem': u'لَكُمَا',
    'type': u'STOPWORD',
    'original': u'لَكُمَا',
    'encletic': u''
}
STOPWORDS[u'لكن'] = {
    'word': u'لَكُنَّ',
    'procletic': u'',
    'tags': u'أداة:ضمير متصل مجرور',
    'vocalized': u'لَكُنَّ',
    'stem': u'لَكُنَّ',
    'type': u'STOPWORD',
    'original': u'لَكُنَّ',
    'encletic': u''
}
STOPWORDS[u'ولكن'] = {
    'word': u'ولكن',
    'procletic': u'-وَ',
    'tags': u'أداة:ضمير متصل مجرور:معطوف',
    'vocalized': u'وَلَكُنَّ',
    'stem': u'لَكُنَّ',
    'type': u'STOPWORD',
    'original': u'لَكُنَّ',
    'encletic': u''
}
STOPWORDS[u'فلكن'] = {
    'word': u'فلكن',
    'procletic': u'-فَ',
    'tags': u'أداة:ضمير متصل مجرور:معطوف',
    'vocalized': u'فَلَكُنَّ',
    'stem': u'لَكُنَّ',
    'type': u'STOPWORD',
    'original': u'لَكُنَّ',
    'encletic': u''
}
STOPWORDS[u'ألكن'] = {
    'word': u'ألكن',
    'procletic': u'-أَ',
    'tags': u'أداة:ضمير متصل مجرور:مجرور',
    'vocalized': u'أَلَكُنَّ',
    'stem': u'لَكُنَّ',
    'type': u'STOPWORD',
    'original': u'لَكُنَّ',
    'encletic': u''
}
STOPWORDS[u'أولكن'] = {
    'word': u'أولكن',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ضمير متصل مجرور:معطوف:مجرور',
    'vocalized': u'أَوَلَكُنَّ',
    'stem': u'لَكُنَّ',
    'type': u'STOPWORD',
    'original': u'لَكُنَّ',
    'encletic': u''
}
STOPWORDS[u'أفلكن'] = {
    'word': u'أفلكن',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ضمير متصل مجرور:معطوف:مجرور',
    'vocalized': u'أَفَلَكُنَّ',
    'stem': u'لَكُنَّ',
    'type': u'STOPWORD',
    'original': u'لَكُنَّ',
    'encletic': u''
}
STOPWORDS[u'لنا'] = {
    'word': u'لَنَا',
    'procletic': u'',
    'tags': u'أداة:ضمير متصل مجرور',
    'vocalized': u'لَنَا',
    'stem': u'لَنَا',
    'type': u'STOPWORD',
    'original': u'لَنَا',
    'encletic': u''
}
STOPWORDS[u'ولنا'] = {
    'word': u'ولنا',
    'procletic': u'-وَ',
    'tags': u'أداة:ضمير متصل مجرور:معطوف',
    'vocalized': u'وَلَنَا',
    'stem': u'لَنَا',
    'type': u'STOPWORD',
    'original': u'لَنَا',
    'encletic': u''
}
STOPWORDS[u'فلنا'] = {
    'word': u'فلنا',
    'procletic': u'-فَ',
    'tags': u'أداة:ضمير متصل مجرور:معطوف',
    'vocalized': u'فَلَنَا',
    'stem': u'لَنَا',
    'type': u'STOPWORD',
    'original': u'لَنَا',
    'encletic': u''
}
STOPWORDS[u'ألنا'] = {
    'word': u'ألنا',
    'procletic': u'-أَ',
    'tags': u'أداة:ضمير متصل مجرور:مجرور',
    'vocalized': u'أَلَنَا',
    'stem': u'لَنَا',
    'type': u'STOPWORD',
    'original': u'لَنَا',
    'encletic': u''
}
STOPWORDS[u'أولنا'] = {
    'word': u'أولنا',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ضمير متصل مجرور:معطوف:مجرور',
    'vocalized': u'أَوَلَنَا',
    'stem': u'لَنَا',
    'type': u'STOPWORD',
    'original': u'لَنَا',
    'encletic': u''
}
STOPWORDS[u'أفلنا'] = {
    'word': u'أفلنا',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ضمير متصل مجرور:معطوف:مجرور',
    'vocalized': u'أَفَلَنَا',
    'stem': u'لَنَا',
    'type': u'STOPWORD',
    'original': u'لَنَا',
    'encletic': u''
}
STOPWORDS[u'له'] = {
    'word': u'لَهُ',
    'procletic': u'',
    'tags': u'أداة:ضمير متصل مجرور',
    'vocalized': u'لَهُ',
    'stem': u'لَهُ',
    'type': u'STOPWORD',
    'original': u'لَهُ',
    'encletic': u''
}
STOPWORDS[u'وله'] = {
    'word': u'وله',
    'procletic': u'-وَ',
    'tags': u'أداة:ضمير متصل مجرور:معطوف',
    'vocalized': u'وَلَهُ',
    'stem': u'لَهُ',
    'type': u'STOPWORD',
    'original': u'لَهُ',
    'encletic': u''
}
STOPWORDS[u'فله'] = {
    'word': u'فله',
    'procletic': u'-فَ',
    'tags': u'أداة:ضمير متصل مجرور:معطوف',
    'vocalized': u'فَلَهُ',
    'stem': u'لَهُ',
    'type': u'STOPWORD',
    'original': u'لَهُ',
    'encletic': u''
}
STOPWORDS[u'أله'] = {
    'word': u'أله',
    'procletic': u'-أَ',
    'tags': u'أداة:ضمير متصل مجرور:مجرور',
    'vocalized': u'أَلَهُ',
    'stem': u'لَهُ',
    'type': u'STOPWORD',
    'original': u'لَهُ',
    'encletic': u''
}
STOPWORDS[u'أوله'] = {
    'word': u'أوله',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ضمير متصل مجرور:معطوف:مجرور',
    'vocalized': u'أَوَلَهُ',
    'stem': u'لَهُ',
    'type': u'STOPWORD',
    'original': u'لَهُ',
    'encletic': u''
}
STOPWORDS[u'أفله'] = {
    'word': u'أفله',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ضمير متصل مجرور:معطوف:مجرور',
    'vocalized': u'أَفَلَهُ',
    'stem': u'لَهُ',
    'type': u'STOPWORD',
    'original': u'لَهُ',
    'encletic': u''
}
STOPWORDS[u'لها'] = {
    'word': u'لَهَا',
    'procletic': u'',
    'tags': u'أداة:ضمير متصل مجرور',
    'vocalized': u'لَهَا',
    'stem': u'لَهَا',
    'type': u'STOPWORD',
    'original': u'لَهَا',
    'encletic': u''
}
STOPWORDS[u'ولها'] = {
    'word': u'ولها',
    'procletic': u'-وَ',
    'tags': u'أداة:ضمير متصل مجرور:معطوف',
    'vocalized': u'وَلَهَا',
    'stem': u'لَهَا',
    'type': u'STOPWORD',
    'original': u'لَهَا',
    'encletic': u''
}
STOPWORDS[u'فلها'] = {
    'word': u'فلها',
    'procletic': u'-فَ',
    'tags': u'أداة:ضمير متصل مجرور:معطوف',
    'vocalized': u'فَلَهَا',
    'stem': u'لَهَا',
    'type': u'STOPWORD',
    'original': u'لَهَا',
    'encletic': u''
}
STOPWORDS[u'ألها'] = {
    'word': u'ألها',
    'procletic': u'-أَ',
    'tags': u'أداة:ضمير متصل مجرور:مجرور',
    'vocalized': u'أَلَهَا',
    'stem': u'لَهَا',
    'type': u'STOPWORD',
    'original': u'لَهَا',
    'encletic': u''
}
STOPWORDS[u'أولها'] = {
    'word': u'أولها',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ضمير متصل مجرور:معطوف:مجرور',
    'vocalized': u'أَوَلَهَا',
    'stem': u'لَهَا',
    'type': u'STOPWORD',
    'original': u'لَهَا',
    'encletic': u''
}
STOPWORDS[u'أفلها'] = {
    'word': u'أفلها',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ضمير متصل مجرور:معطوف:مجرور',
    'vocalized': u'أَفَلَهَا',
    'stem': u'لَهَا',
    'type': u'STOPWORD',
    'original': u'لَهَا',
    'encletic': u''
}
STOPWORDS[u'لي'] = {
    'word': u'لِي',
    'procletic': u'',
    'tags': u'أداة:ضمير متصل مجرور',
    'vocalized': u'لِي',
    'stem': u'لِي',
    'type': u'STOPWORD',
    'original': u'لِي',
    'encletic': u''
}
STOPWORDS[u'ولي'] = {
    'word': u'ولي',
    'procletic': u'-وَ',
    'tags': u'أداة:ضمير متصل مجرور:معطوف',
    'vocalized': u'وَلِي',
    'stem': u'لِي',
    'type': u'STOPWORD',
    'original': u'لِي',
    'encletic': u''
}
STOPWORDS[u'فلي'] = {
    'word': u'فلي',
    'procletic': u'-فَ',
    'tags': u'أداة:ضمير متصل مجرور:معطوف',
    'vocalized': u'فَلِي',
    'stem': u'لِي',
    'type': u'STOPWORD',
    'original': u'لِي',
    'encletic': u''
}
STOPWORDS[u'ألي'] = {
    'word': u'ألي',
    'procletic': u'-أَ',
    'tags': u'أداة:ضمير متصل مجرور:مجرور',
    'vocalized': u'أَلِي',
    'stem': u'لِي',
    'type': u'STOPWORD',
    'original': u'لِي',
    'encletic': u''
}
STOPWORDS[u'أولي'] = {
    'word': u'أولي',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ضمير متصل مجرور:معطوف:مجرور',
    'vocalized': u'أَوَلِي',
    'stem': u'لِي',
    'type': u'STOPWORD',
    'original': u'لِي',
    'encletic': u''
}
STOPWORDS[u'أفلي'] = {
    'word': u'أفلي',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ضمير متصل مجرور:معطوف:مجرور',
    'vocalized': u'أَفَلِي',
    'stem': u'لِي',
    'type': u'STOPWORD',
    'original': u'لِي',
    'encletic': u''
}
STOPWORDS[u'أنا'] = {
    'word': u'أَنَا',
    'procletic': u'',
    'tags': u'أداة:ضمير منفصل',
    'vocalized': u'أَنَا',
    'stem': u'أَنَا',
    'type': u'STOPWORD',
    'original': u'أَنَا',
    'encletic': u''
}
STOPWORDS[u'لأنا'] = {
    'word': u'لأنا',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور',
    'vocalized': u'لَأَنَا',
    'stem': u'أَنَا',
    'type': u'STOPWORD',
    'original': u'أَنَا',
    'encletic': u''
}
STOPWORDS[u'وأنا'] = {
    'word': u'وأنا',
    'procletic': u'-وَ',
    'tags': u'أداة:ضمير منفصل:معطوف',
    'vocalized': u'وَأَنَا',
    'stem': u'أَنَا',
    'type': u'STOPWORD',
    'original': u'أَنَا',
    'encletic': u''
}
STOPWORDS[u'فأنا'] = {
    'word': u'فأنا',
    'procletic': u'-فَ',
    'tags': u'أداة:ضمير منفصل:معطوف',
    'vocalized': u'فَأَنَا',
    'stem': u'أَنَا',
    'type': u'STOPWORD',
    'original': u'أَنَا',
    'encletic': u''
}
STOPWORDS[u'ولأنا'] = {
    'word': u'ولأنا',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'وَلَأَنَا',
    'stem': u'أَنَا',
    'type': u'STOPWORD',
    'original': u'أَنَا',
    'encletic': u''
}
STOPWORDS[u'فلأنا'] = {
    'word': u'فلأنا',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'فَلَأَنَا',
    'stem': u'أَنَا',
    'type': u'STOPWORD',
    'original': u'أَنَا',
    'encletic': u''
}
STOPWORDS[u'أأنا'] = {
    'word': u'أأنا',
    'procletic': u'-أَ',
    'tags': u'أداة:ضمير منفصل:مجرور',
    'vocalized': u'أَأَنَا',
    'stem': u'أَنَا',
    'type': u'STOPWORD',
    'original': u'أَنَا',
    'encletic': u''
}
STOPWORDS[u'ألأنا'] = {
    'word': u'ألأنا',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور',
    'vocalized': u'أَلَأَنَا',
    'stem': u'أَنَا',
    'type': u'STOPWORD',
    'original': u'أَنَا',
    'encletic': u''
}
STOPWORDS[u'أوأنا'] = {
    'word': u'أوأنا',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ضمير منفصل:معطوف:مجرور',
    'vocalized': u'أَوَأَنَا',
    'stem': u'أَنَا',
    'type': u'STOPWORD',
    'original': u'أَنَا',
    'encletic': u''
}
STOPWORDS[u'أفأنا'] = {
    'word': u'أفأنا',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ضمير منفصل:معطوف:مجرور',
    'vocalized': u'أَفَأَنَا',
    'stem': u'أَنَا',
    'type': u'STOPWORD',
    'original': u'أَنَا',
    'encletic': u''
}
STOPWORDS[u'أولأنا'] = {
    'word': u'أولأنا',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَلَأَنَا',
    'stem': u'أَنَا',
    'type': u'STOPWORD',
    'original': u'أَنَا',
    'encletic': u''
}
STOPWORDS[u'أفلأنا'] = {
    'word': u'أفلأنا',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَلَأَنَا',
    'stem': u'أَنَا',
    'type': u'STOPWORD',
    'original': u'أَنَا',
    'encletic': u''
}
STOPWORDS[u'أنت'] = {
    'word': u'أَنْتَ',
    'procletic': u'',
    'tags': u'أداة:ضمير منفصل',
    'vocalized': u'أَنْتَ',
    'stem': u'أَنْتَ',
    'type': u'STOPWORD',
    'original': u'أَنْتَ',
    'encletic': u''
}
STOPWORDS[u'لأنت'] = {
    'word': u'لأنت',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور',
    'vocalized': u'لَأَنْتَ',
    'stem': u'أَنْتَ',
    'type': u'STOPWORD',
    'original': u'أَنْتَ',
    'encletic': u''
}
STOPWORDS[u'وأنت'] = {
    'word': u'وأنت',
    'procletic': u'-وَ',
    'tags': u'أداة:ضمير منفصل:معطوف',
    'vocalized': u'وَأَنْتَ',
    'stem': u'أَنْتَ',
    'type': u'STOPWORD',
    'original': u'أَنْتَ',
    'encletic': u''
}
STOPWORDS[u'فأنت'] = {
    'word': u'فأنت',
    'procletic': u'-فَ',
    'tags': u'أداة:ضمير منفصل:معطوف',
    'vocalized': u'فَأَنْتَ',
    'stem': u'أَنْتَ',
    'type': u'STOPWORD',
    'original': u'أَنْتَ',
    'encletic': u''
}
STOPWORDS[u'ولأنت'] = {
    'word': u'ولأنت',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'وَلَأَنْتَ',
    'stem': u'أَنْتَ',
    'type': u'STOPWORD',
    'original': u'أَنْتَ',
    'encletic': u''
}
STOPWORDS[u'فلأنت'] = {
    'word': u'فلأنت',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'فَلَأَنْتَ',
    'stem': u'أَنْتَ',
    'type': u'STOPWORD',
    'original': u'أَنْتَ',
    'encletic': u''
}
STOPWORDS[u'أأنت'] = {
    'word': u'أأنت',
    'procletic': u'-أَ',
    'tags': u'أداة:ضمير منفصل:مجرور',
    'vocalized': u'أَأَنْتَ',
    'stem': u'أَنْتَ',
    'type': u'STOPWORD',
    'original': u'أَنْتَ',
    'encletic': u''
}
STOPWORDS[u'ألأنت'] = {
    'word': u'ألأنت',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور',
    'vocalized': u'أَلَأَنْتَ',
    'stem': u'أَنْتَ',
    'type': u'STOPWORD',
    'original': u'أَنْتَ',
    'encletic': u''
}
STOPWORDS[u'أوأنت'] = {
    'word': u'أوأنت',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ضمير منفصل:معطوف:مجرور',
    'vocalized': u'أَوَأَنْتَ',
    'stem': u'أَنْتَ',
    'type': u'STOPWORD',
    'original': u'أَنْتَ',
    'encletic': u''
}
STOPWORDS[u'أفأنت'] = {
    'word': u'أفأنت',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ضمير منفصل:معطوف:مجرور',
    'vocalized': u'أَفَأَنْتَ',
    'stem': u'أَنْتَ',
    'type': u'STOPWORD',
    'original': u'أَنْتَ',
    'encletic': u''
}
STOPWORDS[u'أولأنت'] = {
    'word': u'أولأنت',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَلَأَنْتَ',
    'stem': u'أَنْتَ',
    'type': u'STOPWORD',
    'original': u'أَنْتَ',
    'encletic': u''
}
STOPWORDS[u'أفلأنت'] = {
    'word': u'أفلأنت',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَلَأَنْتَ',
    'stem': u'أَنْتَ',
    'type': u'STOPWORD',
    'original': u'أَنْتَ',
    'encletic': u''
}
STOPWORDS[u'أنت'] = {
    'word': u'أَنْتِ',
    'procletic': u'',
    'tags': u'أداة:ضمير منفصل',
    'vocalized': u'أَنْتِ',
    'stem': u'أَنْتِ',
    'type': u'STOPWORD',
    'original': u'أَنْتِ',
    'encletic': u''
}
STOPWORDS[u'لأنت'] = {
    'word': u'لأنت',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور',
    'vocalized': u'لَأَنْتِ',
    'stem': u'أَنْتِ',
    'type': u'STOPWORD',
    'original': u'أَنْتِ',
    'encletic': u''
}
STOPWORDS[u'وأنت'] = {
    'word': u'وأنت',
    'procletic': u'-وَ',
    'tags': u'أداة:ضمير منفصل:معطوف',
    'vocalized': u'وَأَنْتِ',
    'stem': u'أَنْتِ',
    'type': u'STOPWORD',
    'original': u'أَنْتِ',
    'encletic': u''
}
STOPWORDS[u'فأنت'] = {
    'word': u'فأنت',
    'procletic': u'-فَ',
    'tags': u'أداة:ضمير منفصل:معطوف',
    'vocalized': u'فَأَنْتِ',
    'stem': u'أَنْتِ',
    'type': u'STOPWORD',
    'original': u'أَنْتِ',
    'encletic': u''
}
STOPWORDS[u'ولأنت'] = {
    'word': u'ولأنت',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'وَلَأَنْتِ',
    'stem': u'أَنْتِ',
    'type': u'STOPWORD',
    'original': u'أَنْتِ',
    'encletic': u''
}
STOPWORDS[u'فلأنت'] = {
    'word': u'فلأنت',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'فَلَأَنْتِ',
    'stem': u'أَنْتِ',
    'type': u'STOPWORD',
    'original': u'أَنْتِ',
    'encletic': u''
}
STOPWORDS[u'أأنت'] = {
    'word': u'أأنت',
    'procletic': u'-أَ',
    'tags': u'أداة:ضمير منفصل:مجرور',
    'vocalized': u'أَأَنْتِ',
    'stem': u'أَنْتِ',
    'type': u'STOPWORD',
    'original': u'أَنْتِ',
    'encletic': u''
}
STOPWORDS[u'ألأنت'] = {
    'word': u'ألأنت',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور',
    'vocalized': u'أَلَأَنْتِ',
    'stem': u'أَنْتِ',
    'type': u'STOPWORD',
    'original': u'أَنْتِ',
    'encletic': u''
}
STOPWORDS[u'أوأنت'] = {
    'word': u'أوأنت',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ضمير منفصل:معطوف:مجرور',
    'vocalized': u'أَوَأَنْتِ',
    'stem': u'أَنْتِ',
    'type': u'STOPWORD',
    'original': u'أَنْتِ',
    'encletic': u''
}
STOPWORDS[u'أفأنت'] = {
    'word': u'أفأنت',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ضمير منفصل:معطوف:مجرور',
    'vocalized': u'أَفَأَنْتِ',
    'stem': u'أَنْتِ',
    'type': u'STOPWORD',
    'original': u'أَنْتِ',
    'encletic': u''
}
STOPWORDS[u'أولأنت'] = {
    'word': u'أولأنت',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَلَأَنْتِ',
    'stem': u'أَنْتِ',
    'type': u'STOPWORD',
    'original': u'أَنْتِ',
    'encletic': u''
}
STOPWORDS[u'أفلأنت'] = {
    'word': u'أفلأنت',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَلَأَنْتِ',
    'stem': u'أَنْتِ',
    'type': u'STOPWORD',
    'original': u'أَنْتِ',
    'encletic': u''
}
STOPWORDS[u'أنتم'] = {
    'word': u'أَنْتُمْ',
    'procletic': u'',
    'tags': u'أداة:ضمير منفصل',
    'vocalized': u'أَنْتُمْ',
    'stem': u'أَنْتُمْ',
    'type': u'STOPWORD',
    'original': u'أَنْتُمْ',
    'encletic': u''
}
STOPWORDS[u'لأنتم'] = {
    'word': u'لأنتم',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور',
    'vocalized': u'لَأَنْتُمْ',
    'stem': u'أَنْتُمْ',
    'type': u'STOPWORD',
    'original': u'أَنْتُمْ',
    'encletic': u''
}
STOPWORDS[u'وأنتم'] = {
    'word': u'وأنتم',
    'procletic': u'-وَ',
    'tags': u'أداة:ضمير منفصل:معطوف',
    'vocalized': u'وَأَنْتُمْ',
    'stem': u'أَنْتُمْ',
    'type': u'STOPWORD',
    'original': u'أَنْتُمْ',
    'encletic': u''
}
STOPWORDS[u'فأنتم'] = {
    'word': u'فأنتم',
    'procletic': u'-فَ',
    'tags': u'أداة:ضمير منفصل:معطوف',
    'vocalized': u'فَأَنْتُمْ',
    'stem': u'أَنْتُمْ',
    'type': u'STOPWORD',
    'original': u'أَنْتُمْ',
    'encletic': u''
}
STOPWORDS[u'ولأنتم'] = {
    'word': u'ولأنتم',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'وَلَأَنْتُمْ',
    'stem': u'أَنْتُمْ',
    'type': u'STOPWORD',
    'original': u'أَنْتُمْ',
    'encletic': u''
}
STOPWORDS[u'فلأنتم'] = {
    'word': u'فلأنتم',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'فَلَأَنْتُمْ',
    'stem': u'أَنْتُمْ',
    'type': u'STOPWORD',
    'original': u'أَنْتُمْ',
    'encletic': u''
}
STOPWORDS[u'أأنتم'] = {
    'word': u'أأنتم',
    'procletic': u'-أَ',
    'tags': u'أداة:ضمير منفصل:مجرور',
    'vocalized': u'أَأَنْتُمْ',
    'stem': u'أَنْتُمْ',
    'type': u'STOPWORD',
    'original': u'أَنْتُمْ',
    'encletic': u''
}
STOPWORDS[u'ألأنتم'] = {
    'word': u'ألأنتم',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور',
    'vocalized': u'أَلَأَنْتُمْ',
    'stem': u'أَنْتُمْ',
    'type': u'STOPWORD',
    'original': u'أَنْتُمْ',
    'encletic': u''
}
STOPWORDS[u'أوأنتم'] = {
    'word': u'أوأنتم',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ضمير منفصل:معطوف:مجرور',
    'vocalized': u'أَوَأَنْتُمْ',
    'stem': u'أَنْتُمْ',
    'type': u'STOPWORD',
    'original': u'أَنْتُمْ',
    'encletic': u''
}
STOPWORDS[u'أفأنتم'] = {
    'word': u'أفأنتم',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ضمير منفصل:معطوف:مجرور',
    'vocalized': u'أَفَأَنْتُمْ',
    'stem': u'أَنْتُمْ',
    'type': u'STOPWORD',
    'original': u'أَنْتُمْ',
    'encletic': u''
}
STOPWORDS[u'أولأنتم'] = {
    'word': u'أولأنتم',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَلَأَنْتُمْ',
    'stem': u'أَنْتُمْ',
    'type': u'STOPWORD',
    'original': u'أَنْتُمْ',
    'encletic': u''
}
STOPWORDS[u'أفلأنتم'] = {
    'word': u'أفلأنتم',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَلَأَنْتُمْ',
    'stem': u'أَنْتُمْ',
    'type': u'STOPWORD',
    'original': u'أَنْتُمْ',
    'encletic': u''
}
STOPWORDS[u'أنتما'] = {
    'word': u'أَنْتُمَا',
    'procletic': u'',
    'tags': u'أداة:ضمير منفصل',
    'vocalized': u'أَنْتُمَا',
    'stem': u'أَنْتُمَا',
    'type': u'STOPWORD',
    'original': u'أَنْتُمَا',
    'encletic': u''
}
STOPWORDS[u'لأنتما'] = {
    'word': u'لأنتما',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور',
    'vocalized': u'لَأَنْتُمَا',
    'stem': u'أَنْتُمَا',
    'type': u'STOPWORD',
    'original': u'أَنْتُمَا',
    'encletic': u''
}
STOPWORDS[u'وأنتما'] = {
    'word': u'وأنتما',
    'procletic': u'-وَ',
    'tags': u'أداة:ضمير منفصل:معطوف',
    'vocalized': u'وَأَنْتُمَا',
    'stem': u'أَنْتُمَا',
    'type': u'STOPWORD',
    'original': u'أَنْتُمَا',
    'encletic': u''
}
STOPWORDS[u'فأنتما'] = {
    'word': u'فأنتما',
    'procletic': u'-فَ',
    'tags': u'أداة:ضمير منفصل:معطوف',
    'vocalized': u'فَأَنْتُمَا',
    'stem': u'أَنْتُمَا',
    'type': u'STOPWORD',
    'original': u'أَنْتُمَا',
    'encletic': u''
}
STOPWORDS[u'ولأنتما'] = {
    'word': u'ولأنتما',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'وَلَأَنْتُمَا',
    'stem': u'أَنْتُمَا',
    'type': u'STOPWORD',
    'original': u'أَنْتُمَا',
    'encletic': u''
}
STOPWORDS[u'فلأنتما'] = {
    'word': u'فلأنتما',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'فَلَأَنْتُمَا',
    'stem': u'أَنْتُمَا',
    'type': u'STOPWORD',
    'original': u'أَنْتُمَا',
    'encletic': u''
}
STOPWORDS[u'أأنتما'] = {
    'word': u'أأنتما',
    'procletic': u'-أَ',
    'tags': u'أداة:ضمير منفصل:مجرور',
    'vocalized': u'أَأَنْتُمَا',
    'stem': u'أَنْتُمَا',
    'type': u'STOPWORD',
    'original': u'أَنْتُمَا',
    'encletic': u''
}
STOPWORDS[u'ألأنتما'] = {
    'word': u'ألأنتما',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور',
    'vocalized': u'أَلَأَنْتُمَا',
    'stem': u'أَنْتُمَا',
    'type': u'STOPWORD',
    'original': u'أَنْتُمَا',
    'encletic': u''
}
STOPWORDS[u'أوأنتما'] = {
    'word': u'أوأنتما',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ضمير منفصل:معطوف:مجرور',
    'vocalized': u'أَوَأَنْتُمَا',
    'stem': u'أَنْتُمَا',
    'type': u'STOPWORD',
    'original': u'أَنْتُمَا',
    'encletic': u''
}
STOPWORDS[u'أفأنتما'] = {
    'word': u'أفأنتما',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ضمير منفصل:معطوف:مجرور',
    'vocalized': u'أَفَأَنْتُمَا',
    'stem': u'أَنْتُمَا',
    'type': u'STOPWORD',
    'original': u'أَنْتُمَا',
    'encletic': u''
}
STOPWORDS[u'أولأنتما'] = {
    'word': u'أولأنتما',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَلَأَنْتُمَا',
    'stem': u'أَنْتُمَا',
    'type': u'STOPWORD',
    'original': u'أَنْتُمَا',
    'encletic': u''
}
STOPWORDS[u'أفلأنتما'] = {
    'word': u'أفلأنتما',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَلَأَنْتُمَا',
    'stem': u'أَنْتُمَا',
    'type': u'STOPWORD',
    'original': u'أَنْتُمَا',
    'encletic': u''
}
STOPWORDS[u'أنتن'] = {
    'word': u'أَنْتُنَّ',
    'procletic': u'',
    'tags': u'أداة:ضمير منفصل',
    'vocalized': u'أَنْتُنَّ',
    'stem': u'أَنْتُنَّ',
    'type': u'STOPWORD',
    'original': u'أَنْتُنَّ',
    'encletic': u''
}
STOPWORDS[u'لأنتن'] = {
    'word': u'لأنتن',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور',
    'vocalized': u'لَأَنْتُنَّ',
    'stem': u'أَنْتُنَّ',
    'type': u'STOPWORD',
    'original': u'أَنْتُنَّ',
    'encletic': u''
}
STOPWORDS[u'وأنتن'] = {
    'word': u'وأنتن',
    'procletic': u'-وَ',
    'tags': u'أداة:ضمير منفصل:معطوف',
    'vocalized': u'وَأَنْتُنَّ',
    'stem': u'أَنْتُنَّ',
    'type': u'STOPWORD',
    'original': u'أَنْتُنَّ',
    'encletic': u''
}
STOPWORDS[u'فأنتن'] = {
    'word': u'فأنتن',
    'procletic': u'-فَ',
    'tags': u'أداة:ضمير منفصل:معطوف',
    'vocalized': u'فَأَنْتُنَّ',
    'stem': u'أَنْتُنَّ',
    'type': u'STOPWORD',
    'original': u'أَنْتُنَّ',
    'encletic': u''
}
STOPWORDS[u'ولأنتن'] = {
    'word': u'ولأنتن',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'وَلَأَنْتُنَّ',
    'stem': u'أَنْتُنَّ',
    'type': u'STOPWORD',
    'original': u'أَنْتُنَّ',
    'encletic': u''
}
STOPWORDS[u'فلأنتن'] = {
    'word': u'فلأنتن',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'فَلَأَنْتُنَّ',
    'stem': u'أَنْتُنَّ',
    'type': u'STOPWORD',
    'original': u'أَنْتُنَّ',
    'encletic': u''
}
STOPWORDS[u'أأنتن'] = {
    'word': u'أأنتن',
    'procletic': u'-أَ',
    'tags': u'أداة:ضمير منفصل:مجرور',
    'vocalized': u'أَأَنْتُنَّ',
    'stem': u'أَنْتُنَّ',
    'type': u'STOPWORD',
    'original': u'أَنْتُنَّ',
    'encletic': u''
}
STOPWORDS[u'ألأنتن'] = {
    'word': u'ألأنتن',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور',
    'vocalized': u'أَلَأَنْتُنَّ',
    'stem': u'أَنْتُنَّ',
    'type': u'STOPWORD',
    'original': u'أَنْتُنَّ',
    'encletic': u''
}
STOPWORDS[u'أوأنتن'] = {
    'word': u'أوأنتن',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ضمير منفصل:معطوف:مجرور',
    'vocalized': u'أَوَأَنْتُنَّ',
    'stem': u'أَنْتُنَّ',
    'type': u'STOPWORD',
    'original': u'أَنْتُنَّ',
    'encletic': u''
}
STOPWORDS[u'أفأنتن'] = {
    'word': u'أفأنتن',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ضمير منفصل:معطوف:مجرور',
    'vocalized': u'أَفَأَنْتُنَّ',
    'stem': u'أَنْتُنَّ',
    'type': u'STOPWORD',
    'original': u'أَنْتُنَّ',
    'encletic': u''
}
STOPWORDS[u'أولأنتن'] = {
    'word': u'أولأنتن',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَلَأَنْتُنَّ',
    'stem': u'أَنْتُنَّ',
    'type': u'STOPWORD',
    'original': u'أَنْتُنَّ',
    'encletic': u''
}
STOPWORDS[u'أفلأنتن'] = {
    'word': u'أفلأنتن',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَلَأَنْتُنَّ',
    'stem': u'أَنْتُنَّ',
    'type': u'STOPWORD',
    'original': u'أَنْتُنَّ',
    'encletic': u''
}
STOPWORDS[u'نحن'] = {
    'word': u'نَحْنُ',
    'procletic': u'',
    'tags': u'أداة:ضمير منفصل',
    'vocalized': u'نَحْنُ',
    'stem': u'نَحْنُ',
    'type': u'STOPWORD',
    'original': u'نَحْنُ',
    'encletic': u''
}
STOPWORDS[u'لنحن'] = {
    'word': u'لنحن',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور',
    'vocalized': u'لَنَحْنُ',
    'stem': u'نَحْنُ',
    'type': u'STOPWORD',
    'original': u'نَحْنُ',
    'encletic': u''
}
STOPWORDS[u'ونحن'] = {
    'word': u'ونحن',
    'procletic': u'-وَ',
    'tags': u'أداة:ضمير منفصل:معطوف',
    'vocalized': u'وَنَحْنُ',
    'stem': u'نَحْنُ',
    'type': u'STOPWORD',
    'original': u'نَحْنُ',
    'encletic': u''
}
STOPWORDS[u'فنحن'] = {
    'word': u'فنحن',
    'procletic': u'-فَ',
    'tags': u'أداة:ضمير منفصل:معطوف',
    'vocalized': u'فَنَحْنُ',
    'stem': u'نَحْنُ',
    'type': u'STOPWORD',
    'original': u'نَحْنُ',
    'encletic': u''
}
STOPWORDS[u'ولنحن'] = {
    'word': u'ولنحن',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'وَلَنَحْنُ',
    'stem': u'نَحْنُ',
    'type': u'STOPWORD',
    'original': u'نَحْنُ',
    'encletic': u''
}
STOPWORDS[u'فلنحن'] = {
    'word': u'فلنحن',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'فَلَنَحْنُ',
    'stem': u'نَحْنُ',
    'type': u'STOPWORD',
    'original': u'نَحْنُ',
    'encletic': u''
}
STOPWORDS[u'أنحن'] = {
    'word': u'أنحن',
    'procletic': u'-أَ',
    'tags': u'أداة:ضمير منفصل:مجرور',
    'vocalized': u'أَنَحْنُ',
    'stem': u'نَحْنُ',
    'type': u'STOPWORD',
    'original': u'نَحْنُ',
    'encletic': u''
}
STOPWORDS[u'ألنحن'] = {
    'word': u'ألنحن',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور',
    'vocalized': u'أَلَنَحْنُ',
    'stem': u'نَحْنُ',
    'type': u'STOPWORD',
    'original': u'نَحْنُ',
    'encletic': u''
}
STOPWORDS[u'أونحن'] = {
    'word': u'أونحن',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ضمير منفصل:معطوف:مجرور',
    'vocalized': u'أَوَنَحْنُ',
    'stem': u'نَحْنُ',
    'type': u'STOPWORD',
    'original': u'نَحْنُ',
    'encletic': u''
}
STOPWORDS[u'أفنحن'] = {
    'word': u'أفنحن',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ضمير منفصل:معطوف:مجرور',
    'vocalized': u'أَفَنَحْنُ',
    'stem': u'نَحْنُ',
    'type': u'STOPWORD',
    'original': u'نَحْنُ',
    'encletic': u''
}
STOPWORDS[u'أولنحن'] = {
    'word': u'أولنحن',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَلَنَحْنُ',
    'stem': u'نَحْنُ',
    'type': u'STOPWORD',
    'original': u'نَحْنُ',
    'encletic': u''
}
STOPWORDS[u'أفلنحن'] = {
    'word': u'أفلنحن',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَلَنَحْنُ',
    'stem': u'نَحْنُ',
    'type': u'STOPWORD',
    'original': u'نَحْنُ',
    'encletic': u''
}
STOPWORDS[u'هم'] = {
    'word': u'هُمْ',
    'procletic': u'',
    'tags': u'أداة:ضمير منفصل',
    'vocalized': u'هُمْ',
    'stem': u'هُمْ',
    'type': u'STOPWORD',
    'original': u'هُمْ',
    'encletic': u''
}
STOPWORDS[u'بهم'] = {
    'word': u'بهم',
    'procletic': u'-بِ',
    'tags': u'أداة:ضمير منفصل:مجرور',
    'vocalized': u'بِهُمْ',
    'stem': u'هُمْ',
    'type': u'STOPWORD',
    'original': u'هُمْ',
    'encletic': u''
}
STOPWORDS[u'كهم'] = {
    'word': u'كهم',
    'procletic': u'-كَ',
    'tags': u'أداة:ضمير منفصل:مجرور',
    'vocalized': u'كَهُمْ',
    'stem': u'هُمْ',
    'type': u'STOPWORD',
    'original': u'هُمْ',
    'encletic': u''
}
STOPWORDS[u'لهم'] = {
    'word': u'لهم',
    'procletic': u'-لِ',
    'tags': u'أداة:ضمير منفصل:مجرور',
    'vocalized': u'لِهُمْ',
    'stem': u'هُمْ',
    'type': u'STOPWORD',
    'original': u'هُمْ',
    'encletic': u''
}
STOPWORDS[u'لهم'] = {
    'word': u'لهم',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور',
    'vocalized': u'لَهُمْ',
    'stem': u'هُمْ',
    'type': u'STOPWORD',
    'original': u'هُمْ',
    'encletic': u''
}
STOPWORDS[u'وهم'] = {
    'word': u'وهم',
    'procletic': u'-وَ',
    'tags': u'أداة:ضمير منفصل:معطوف',
    'vocalized': u'وَهُمْ',
    'stem': u'هُمْ',
    'type': u'STOPWORD',
    'original': u'هُمْ',
    'encletic': u''
}
STOPWORDS[u'فهم'] = {
    'word': u'فهم',
    'procletic': u'-فَ',
    'tags': u'أداة:ضمير منفصل:معطوف',
    'vocalized': u'فَهُمْ',
    'stem': u'هُمْ',
    'type': u'STOPWORD',
    'original': u'هُمْ',
    'encletic': u''
}
STOPWORDS[u'وبهم'] = {
    'word': u'وبهم',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:ضمير منفصل:مجرور:معطوف',
    'vocalized': u'وَبِهُمْ',
    'stem': u'هُمْ',
    'type': u'STOPWORD',
    'original': u'هُمْ',
    'encletic': u''
}
STOPWORDS[u'فبهم'] = {
    'word': u'فبهم',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:ضمير منفصل:مجرور:معطوف',
    'vocalized': u'فَبِهُمْ',
    'stem': u'هُمْ',
    'type': u'STOPWORD',
    'original': u'هُمْ',
    'encletic': u''
}
STOPWORDS[u'وكهم'] = {
    'word': u'وكهم',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:ضمير منفصل:مجرور:معطوف',
    'vocalized': u'وَكَهُمْ',
    'stem': u'هُمْ',
    'type': u'STOPWORD',
    'original': u'هُمْ',
    'encletic': u''
}
STOPWORDS[u'فكهم'] = {
    'word': u'فكهم',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:ضمير منفصل:مجرور:معطوف',
    'vocalized': u'فَكَهُمْ',
    'stem': u'هُمْ',
    'type': u'STOPWORD',
    'original': u'هُمْ',
    'encletic': u''
}
STOPWORDS[u'ولهم'] = {
    'word': u'ولهم',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:ضمير منفصل:مجرور:معطوف',
    'vocalized': u'وَلِهُمْ',
    'stem': u'هُمْ',
    'type': u'STOPWORD',
    'original': u'هُمْ',
    'encletic': u''
}
STOPWORDS[u'فلهم'] = {
    'word': u'فلهم',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:ضمير منفصل:مجرور:معطوف',
    'vocalized': u'فَلِهُمْ',
    'stem': u'هُمْ',
    'type': u'STOPWORD',
    'original': u'هُمْ',
    'encletic': u''
}
STOPWORDS[u'ولهم'] = {
    'word': u'ولهم',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'وَلَهُمْ',
    'stem': u'هُمْ',
    'type': u'STOPWORD',
    'original': u'هُمْ',
    'encletic': u''
}
STOPWORDS[u'فلهم'] = {
    'word': u'فلهم',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'فَلَهُمْ',
    'stem': u'هُمْ',
    'type': u'STOPWORD',
    'original': u'هُمْ',
    'encletic': u''
}
STOPWORDS[u'أهم'] = {
    'word': u'أهم',
    'procletic': u'-أَ',
    'tags': u'أداة:ضمير منفصل:مجرور',
    'vocalized': u'أَهُمْ',
    'stem': u'هُمْ',
    'type': u'STOPWORD',
    'original': u'هُمْ',
    'encletic': u''
}
STOPWORDS[u'أبهم'] = {
    'word': u'أبهم',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:ضمير منفصل:مجرور:مجرور',
    'vocalized': u'أَبِهُمْ',
    'stem': u'هُمْ',
    'type': u'STOPWORD',
    'original': u'هُمْ',
    'encletic': u''
}
STOPWORDS[u'أكهم'] = {
    'word': u'أكهم',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:ضمير منفصل:مجرور:مجرور',
    'vocalized': u'أَكَهُمْ',
    'stem': u'هُمْ',
    'type': u'STOPWORD',
    'original': u'هُمْ',
    'encletic': u''
}
STOPWORDS[u'ألهم'] = {
    'word': u'ألهم',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:ضمير منفصل:مجرور:مجرور',
    'vocalized': u'أَلِهُمْ',
    'stem': u'هُمْ',
    'type': u'STOPWORD',
    'original': u'هُمْ',
    'encletic': u''
}
STOPWORDS[u'ألهم'] = {
    'word': u'ألهم',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور',
    'vocalized': u'أَلَهُمْ',
    'stem': u'هُمْ',
    'type': u'STOPWORD',
    'original': u'هُمْ',
    'encletic': u''
}
STOPWORDS[u'أوهم'] = {
    'word': u'أوهم',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ضمير منفصل:معطوف:مجرور',
    'vocalized': u'أَوَهُمْ',
    'stem': u'هُمْ',
    'type': u'STOPWORD',
    'original': u'هُمْ',
    'encletic': u''
}
STOPWORDS[u'أفهم'] = {
    'word': u'أفهم',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ضمير منفصل:معطوف:مجرور',
    'vocalized': u'أَفَهُمْ',
    'stem': u'هُمْ',
    'type': u'STOPWORD',
    'original': u'هُمْ',
    'encletic': u''
}
STOPWORDS[u'أوبهم'] = {
    'word': u'أوبهم',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:ضمير منفصل:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَبِهُمْ',
    'stem': u'هُمْ',
    'type': u'STOPWORD',
    'original': u'هُمْ',
    'encletic': u''
}
STOPWORDS[u'أفبهم'] = {
    'word': u'أفبهم',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:ضمير منفصل:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَبِهُمْ',
    'stem': u'هُمْ',
    'type': u'STOPWORD',
    'original': u'هُمْ',
    'encletic': u''
}
STOPWORDS[u'أوكهم'] = {
    'word': u'أوكهم',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:ضمير منفصل:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَكَهُمْ',
    'stem': u'هُمْ',
    'type': u'STOPWORD',
    'original': u'هُمْ',
    'encletic': u''
}
STOPWORDS[u'أفكهم'] = {
    'word': u'أفكهم',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:ضمير منفصل:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَكَهُمْ',
    'stem': u'هُمْ',
    'type': u'STOPWORD',
    'original': u'هُمْ',
    'encletic': u''
}
STOPWORDS[u'أولهم'] = {
    'word': u'أولهم',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:ضمير منفصل:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَلِهُمْ',
    'stem': u'هُمْ',
    'type': u'STOPWORD',
    'original': u'هُمْ',
    'encletic': u''
}
STOPWORDS[u'أفلهم'] = {
    'word': u'أفلهم',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:ضمير منفصل:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَلِهُمْ',
    'stem': u'هُمْ',
    'type': u'STOPWORD',
    'original': u'هُمْ',
    'encletic': u''
}
STOPWORDS[u'أولهم'] = {
    'word': u'أولهم',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَلَهُمْ',
    'stem': u'هُمْ',
    'type': u'STOPWORD',
    'original': u'هُمْ',
    'encletic': u''
}
STOPWORDS[u'أفلهم'] = {
    'word': u'أفلهم',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَلَهُمْ',
    'stem': u'هُمْ',
    'type': u'STOPWORD',
    'original': u'هُمْ',
    'encletic': u''
}
STOPWORDS[u'هما'] = {
    'word': u'هُمَا',
    'procletic': u'',
    'tags': u'أداة:ضمير منفصل',
    'vocalized': u'هُمَا',
    'stem': u'هُمَا',
    'type': u'STOPWORD',
    'original': u'هُمَا',
    'encletic': u''
}
STOPWORDS[u'بهما'] = {
    'word': u'بهما',
    'procletic': u'-بِ',
    'tags': u'أداة:ضمير منفصل:مجرور',
    'vocalized': u'بِهُمَا',
    'stem': u'هُمَا',
    'type': u'STOPWORD',
    'original': u'هُمَا',
    'encletic': u''
}
STOPWORDS[u'كهما'] = {
    'word': u'كهما',
    'procletic': u'-كَ',
    'tags': u'أداة:ضمير منفصل:مجرور',
    'vocalized': u'كَهُمَا',
    'stem': u'هُمَا',
    'type': u'STOPWORD',
    'original': u'هُمَا',
    'encletic': u''
}
STOPWORDS[u'لهما'] = {
    'word': u'لهما',
    'procletic': u'-لِ',
    'tags': u'أداة:ضمير منفصل:مجرور',
    'vocalized': u'لِهُمَا',
    'stem': u'هُمَا',
    'type': u'STOPWORD',
    'original': u'هُمَا',
    'encletic': u''
}
STOPWORDS[u'لهما'] = {
    'word': u'لهما',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور',
    'vocalized': u'لَهُمَا',
    'stem': u'هُمَا',
    'type': u'STOPWORD',
    'original': u'هُمَا',
    'encletic': u''
}
STOPWORDS[u'وهما'] = {
    'word': u'وهما',
    'procletic': u'-وَ',
    'tags': u'أداة:ضمير منفصل:معطوف',
    'vocalized': u'وَهُمَا',
    'stem': u'هُمَا',
    'type': u'STOPWORD',
    'original': u'هُمَا',
    'encletic': u''
}
STOPWORDS[u'فهما'] = {
    'word': u'فهما',
    'procletic': u'-فَ',
    'tags': u'أداة:ضمير منفصل:معطوف',
    'vocalized': u'فَهُمَا',
    'stem': u'هُمَا',
    'type': u'STOPWORD',
    'original': u'هُمَا',
    'encletic': u''
}
STOPWORDS[u'وبهما'] = {
    'word': u'وبهما',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:ضمير منفصل:مجرور:معطوف',
    'vocalized': u'وَبِهُمَا',
    'stem': u'هُمَا',
    'type': u'STOPWORD',
    'original': u'هُمَا',
    'encletic': u''
}
STOPWORDS[u'فبهما'] = {
    'word': u'فبهما',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:ضمير منفصل:مجرور:معطوف',
    'vocalized': u'فَبِهُمَا',
    'stem': u'هُمَا',
    'type': u'STOPWORD',
    'original': u'هُمَا',
    'encletic': u''
}
STOPWORDS[u'وكهما'] = {
    'word': u'وكهما',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:ضمير منفصل:مجرور:معطوف',
    'vocalized': u'وَكَهُمَا',
    'stem': u'هُمَا',
    'type': u'STOPWORD',
    'original': u'هُمَا',
    'encletic': u''
}
STOPWORDS[u'فكهما'] = {
    'word': u'فكهما',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:ضمير منفصل:مجرور:معطوف',
    'vocalized': u'فَكَهُمَا',
    'stem': u'هُمَا',
    'type': u'STOPWORD',
    'original': u'هُمَا',
    'encletic': u''
}
STOPWORDS[u'ولهما'] = {
    'word': u'ولهما',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:ضمير منفصل:مجرور:معطوف',
    'vocalized': u'وَلِهُمَا',
    'stem': u'هُمَا',
    'type': u'STOPWORD',
    'original': u'هُمَا',
    'encletic': u''
}
STOPWORDS[u'فلهما'] = {
    'word': u'فلهما',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:ضمير منفصل:مجرور:معطوف',
    'vocalized': u'فَلِهُمَا',
    'stem': u'هُمَا',
    'type': u'STOPWORD',
    'original': u'هُمَا',
    'encletic': u''
}
STOPWORDS[u'ولهما'] = {
    'word': u'ولهما',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'وَلَهُمَا',
    'stem': u'هُمَا',
    'type': u'STOPWORD',
    'original': u'هُمَا',
    'encletic': u''
}
STOPWORDS[u'فلهما'] = {
    'word': u'فلهما',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'فَلَهُمَا',
    'stem': u'هُمَا',
    'type': u'STOPWORD',
    'original': u'هُمَا',
    'encletic': u''
}
STOPWORDS[u'أهما'] = {
    'word': u'أهما',
    'procletic': u'-أَ',
    'tags': u'أداة:ضمير منفصل:مجرور',
    'vocalized': u'أَهُمَا',
    'stem': u'هُمَا',
    'type': u'STOPWORD',
    'original': u'هُمَا',
    'encletic': u''
}
STOPWORDS[u'أبهما'] = {
    'word': u'أبهما',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:ضمير منفصل:مجرور:مجرور',
    'vocalized': u'أَبِهُمَا',
    'stem': u'هُمَا',
    'type': u'STOPWORD',
    'original': u'هُمَا',
    'encletic': u''
}
STOPWORDS[u'أكهما'] = {
    'word': u'أكهما',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:ضمير منفصل:مجرور:مجرور',
    'vocalized': u'أَكَهُمَا',
    'stem': u'هُمَا',
    'type': u'STOPWORD',
    'original': u'هُمَا',
    'encletic': u''
}
STOPWORDS[u'ألهما'] = {
    'word': u'ألهما',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:ضمير منفصل:مجرور:مجرور',
    'vocalized': u'أَلِهُمَا',
    'stem': u'هُمَا',
    'type': u'STOPWORD',
    'original': u'هُمَا',
    'encletic': u''
}
STOPWORDS[u'ألهما'] = {
    'word': u'ألهما',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور',
    'vocalized': u'أَلَهُمَا',
    'stem': u'هُمَا',
    'type': u'STOPWORD',
    'original': u'هُمَا',
    'encletic': u''
}
STOPWORDS[u'أوهما'] = {
    'word': u'أوهما',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ضمير منفصل:معطوف:مجرور',
    'vocalized': u'أَوَهُمَا',
    'stem': u'هُمَا',
    'type': u'STOPWORD',
    'original': u'هُمَا',
    'encletic': u''
}
STOPWORDS[u'أفهما'] = {
    'word': u'أفهما',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ضمير منفصل:معطوف:مجرور',
    'vocalized': u'أَفَهُمَا',
    'stem': u'هُمَا',
    'type': u'STOPWORD',
    'original': u'هُمَا',
    'encletic': u''
}
STOPWORDS[u'أوبهما'] = {
    'word': u'أوبهما',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:ضمير منفصل:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَبِهُمَا',
    'stem': u'هُمَا',
    'type': u'STOPWORD',
    'original': u'هُمَا',
    'encletic': u''
}
STOPWORDS[u'أفبهما'] = {
    'word': u'أفبهما',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:ضمير منفصل:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَبِهُمَا',
    'stem': u'هُمَا',
    'type': u'STOPWORD',
    'original': u'هُمَا',
    'encletic': u''
}
STOPWORDS[u'أوكهما'] = {
    'word': u'أوكهما',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:ضمير منفصل:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَكَهُمَا',
    'stem': u'هُمَا',
    'type': u'STOPWORD',
    'original': u'هُمَا',
    'encletic': u''
}
STOPWORDS[u'أفكهما'] = {
    'word': u'أفكهما',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:ضمير منفصل:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَكَهُمَا',
    'stem': u'هُمَا',
    'type': u'STOPWORD',
    'original': u'هُمَا',
    'encletic': u''
}
STOPWORDS[u'أولهما'] = {
    'word': u'أولهما',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:ضمير منفصل:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَلِهُمَا',
    'stem': u'هُمَا',
    'type': u'STOPWORD',
    'original': u'هُمَا',
    'encletic': u''
}
STOPWORDS[u'أفلهما'] = {
    'word': u'أفلهما',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:ضمير منفصل:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَلِهُمَا',
    'stem': u'هُمَا',
    'type': u'STOPWORD',
    'original': u'هُمَا',
    'encletic': u''
}
STOPWORDS[u'أولهما'] = {
    'word': u'أولهما',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَلَهُمَا',
    'stem': u'هُمَا',
    'type': u'STOPWORD',
    'original': u'هُمَا',
    'encletic': u''
}
STOPWORDS[u'أفلهما'] = {
    'word': u'أفلهما',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَلَهُمَا',
    'stem': u'هُمَا',
    'type': u'STOPWORD',
    'original': u'هُمَا',
    'encletic': u''
}
STOPWORDS[u'هن'] = {
    'word': u'هُنَّ',
    'procletic': u'',
    'tags': u'أداة:ضمير منفصل',
    'vocalized': u'هُنَّ',
    'stem': u'هُنَّ',
    'type': u'STOPWORD',
    'original': u'هُنَّ',
    'encletic': u''
}
STOPWORDS[u'بهن'] = {
    'word': u'بهن',
    'procletic': u'-بِ',
    'tags': u'أداة:ضمير منفصل:مجرور',
    'vocalized': u'بِهُنَّ',
    'stem': u'هُنَّ',
    'type': u'STOPWORD',
    'original': u'هُنَّ',
    'encletic': u''
}
STOPWORDS[u'كهن'] = {
    'word': u'كهن',
    'procletic': u'-كَ',
    'tags': u'أداة:ضمير منفصل:مجرور',
    'vocalized': u'كَهُنَّ',
    'stem': u'هُنَّ',
    'type': u'STOPWORD',
    'original': u'هُنَّ',
    'encletic': u''
}
STOPWORDS[u'لهن'] = {
    'word': u'لهن',
    'procletic': u'-لِ',
    'tags': u'أداة:ضمير منفصل:مجرور',
    'vocalized': u'لِهُنَّ',
    'stem': u'هُنَّ',
    'type': u'STOPWORD',
    'original': u'هُنَّ',
    'encletic': u''
}
STOPWORDS[u'لهن'] = {
    'word': u'لهن',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور',
    'vocalized': u'لَهُنَّ',
    'stem': u'هُنَّ',
    'type': u'STOPWORD',
    'original': u'هُنَّ',
    'encletic': u''
}
STOPWORDS[u'وهن'] = {
    'word': u'وهن',
    'procletic': u'-وَ',
    'tags': u'أداة:ضمير منفصل:معطوف',
    'vocalized': u'وَهُنَّ',
    'stem': u'هُنَّ',
    'type': u'STOPWORD',
    'original': u'هُنَّ',
    'encletic': u''
}
STOPWORDS[u'فهن'] = {
    'word': u'فهن',
    'procletic': u'-فَ',
    'tags': u'أداة:ضمير منفصل:معطوف',
    'vocalized': u'فَهُنَّ',
    'stem': u'هُنَّ',
    'type': u'STOPWORD',
    'original': u'هُنَّ',
    'encletic': u''
}
STOPWORDS[u'وبهن'] = {
    'word': u'وبهن',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:ضمير منفصل:مجرور:معطوف',
    'vocalized': u'وَبِهُنَّ',
    'stem': u'هُنَّ',
    'type': u'STOPWORD',
    'original': u'هُنَّ',
    'encletic': u''
}
STOPWORDS[u'فبهن'] = {
    'word': u'فبهن',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:ضمير منفصل:مجرور:معطوف',
    'vocalized': u'فَبِهُنَّ',
    'stem': u'هُنَّ',
    'type': u'STOPWORD',
    'original': u'هُنَّ',
    'encletic': u''
}
STOPWORDS[u'وكهن'] = {
    'word': u'وكهن',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:ضمير منفصل:مجرور:معطوف',
    'vocalized': u'وَكَهُنَّ',
    'stem': u'هُنَّ',
    'type': u'STOPWORD',
    'original': u'هُنَّ',
    'encletic': u''
}
STOPWORDS[u'فكهن'] = {
    'word': u'فكهن',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:ضمير منفصل:مجرور:معطوف',
    'vocalized': u'فَكَهُنَّ',
    'stem': u'هُنَّ',
    'type': u'STOPWORD',
    'original': u'هُنَّ',
    'encletic': u''
}
STOPWORDS[u'ولهن'] = {
    'word': u'ولهن',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:ضمير منفصل:مجرور:معطوف',
    'vocalized': u'وَلِهُنَّ',
    'stem': u'هُنَّ',
    'type': u'STOPWORD',
    'original': u'هُنَّ',
    'encletic': u''
}
STOPWORDS[u'فلهن'] = {
    'word': u'فلهن',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:ضمير منفصل:مجرور:معطوف',
    'vocalized': u'فَلِهُنَّ',
    'stem': u'هُنَّ',
    'type': u'STOPWORD',
    'original': u'هُنَّ',
    'encletic': u''
}
STOPWORDS[u'ولهن'] = {
    'word': u'ولهن',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'وَلَهُنَّ',
    'stem': u'هُنَّ',
    'type': u'STOPWORD',
    'original': u'هُنَّ',
    'encletic': u''
}
STOPWORDS[u'فلهن'] = {
    'word': u'فلهن',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'فَلَهُنَّ',
    'stem': u'هُنَّ',
    'type': u'STOPWORD',
    'original': u'هُنَّ',
    'encletic': u''
}
STOPWORDS[u'أهن'] = {
    'word': u'أهن',
    'procletic': u'-أَ',
    'tags': u'أداة:ضمير منفصل:مجرور',
    'vocalized': u'أَهُنَّ',
    'stem': u'هُنَّ',
    'type': u'STOPWORD',
    'original': u'هُنَّ',
    'encletic': u''
}
STOPWORDS[u'أبهن'] = {
    'word': u'أبهن',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:ضمير منفصل:مجرور:مجرور',
    'vocalized': u'أَبِهُنَّ',
    'stem': u'هُنَّ',
    'type': u'STOPWORD',
    'original': u'هُنَّ',
    'encletic': u''
}
STOPWORDS[u'أكهن'] = {
    'word': u'أكهن',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:ضمير منفصل:مجرور:مجرور',
    'vocalized': u'أَكَهُنَّ',
    'stem': u'هُنَّ',
    'type': u'STOPWORD',
    'original': u'هُنَّ',
    'encletic': u''
}
STOPWORDS[u'ألهن'] = {
    'word': u'ألهن',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:ضمير منفصل:مجرور:مجرور',
    'vocalized': u'أَلِهُنَّ',
    'stem': u'هُنَّ',
    'type': u'STOPWORD',
    'original': u'هُنَّ',
    'encletic': u''
}
STOPWORDS[u'ألهن'] = {
    'word': u'ألهن',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور',
    'vocalized': u'أَلَهُنَّ',
    'stem': u'هُنَّ',
    'type': u'STOPWORD',
    'original': u'هُنَّ',
    'encletic': u''
}
STOPWORDS[u'أوهن'] = {
    'word': u'أوهن',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ضمير منفصل:معطوف:مجرور',
    'vocalized': u'أَوَهُنَّ',
    'stem': u'هُنَّ',
    'type': u'STOPWORD',
    'original': u'هُنَّ',
    'encletic': u''
}
STOPWORDS[u'أفهن'] = {
    'word': u'أفهن',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ضمير منفصل:معطوف:مجرور',
    'vocalized': u'أَفَهُنَّ',
    'stem': u'هُنَّ',
    'type': u'STOPWORD',
    'original': u'هُنَّ',
    'encletic': u''
}
STOPWORDS[u'أوبهن'] = {
    'word': u'أوبهن',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:ضمير منفصل:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَبِهُنَّ',
    'stem': u'هُنَّ',
    'type': u'STOPWORD',
    'original': u'هُنَّ',
    'encletic': u''
}
STOPWORDS[u'أفبهن'] = {
    'word': u'أفبهن',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:ضمير منفصل:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَبِهُنَّ',
    'stem': u'هُنَّ',
    'type': u'STOPWORD',
    'original': u'هُنَّ',
    'encletic': u''
}
STOPWORDS[u'أوكهن'] = {
    'word': u'أوكهن',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:ضمير منفصل:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَكَهُنَّ',
    'stem': u'هُنَّ',
    'type': u'STOPWORD',
    'original': u'هُنَّ',
    'encletic': u''
}
STOPWORDS[u'أفكهن'] = {
    'word': u'أفكهن',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:ضمير منفصل:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَكَهُنَّ',
    'stem': u'هُنَّ',
    'type': u'STOPWORD',
    'original': u'هُنَّ',
    'encletic': u''
}
STOPWORDS[u'أولهن'] = {
    'word': u'أولهن',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:ضمير منفصل:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَلِهُنَّ',
    'stem': u'هُنَّ',
    'type': u'STOPWORD',
    'original': u'هُنَّ',
    'encletic': u''
}
STOPWORDS[u'أفلهن'] = {
    'word': u'أفلهن',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:ضمير منفصل:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَلِهُنَّ',
    'stem': u'هُنَّ',
    'type': u'STOPWORD',
    'original': u'هُنَّ',
    'encletic': u''
}
STOPWORDS[u'أولهن'] = {
    'word': u'أولهن',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَلَهُنَّ',
    'stem': u'هُنَّ',
    'type': u'STOPWORD',
    'original': u'هُنَّ',
    'encletic': u''
}
STOPWORDS[u'أفلهن'] = {
    'word': u'أفلهن',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَلَهُنَّ',
    'stem': u'هُنَّ',
    'type': u'STOPWORD',
    'original': u'هُنَّ',
    'encletic': u''
}
STOPWORDS[u'هو'] = {
    'word': u'هُوَ',
    'procletic': u'',
    'tags': u'أداة:ضمير منفصل',
    'vocalized': u'هُوَ',
    'stem': u'هُوَ',
    'type': u'STOPWORD',
    'original': u'هُوَ',
    'encletic': u''
}
STOPWORDS[u'بهو'] = {
    'word': u'بهو',
    'procletic': u'-بِ',
    'tags': u'أداة:ضمير منفصل:مجرور',
    'vocalized': u'بِهُوَ',
    'stem': u'هُوَ',
    'type': u'STOPWORD',
    'original': u'هُوَ',
    'encletic': u''
}
STOPWORDS[u'كهو'] = {
    'word': u'كهو',
    'procletic': u'-كَ',
    'tags': u'أداة:ضمير منفصل:مجرور',
    'vocalized': u'كَهُوَ',
    'stem': u'هُوَ',
    'type': u'STOPWORD',
    'original': u'هُوَ',
    'encletic': u''
}
STOPWORDS[u'لهو'] = {
    'word': u'لهو',
    'procletic': u'-لِ',
    'tags': u'أداة:ضمير منفصل:مجرور',
    'vocalized': u'لِهُوَ',
    'stem': u'هُوَ',
    'type': u'STOPWORD',
    'original': u'هُوَ',
    'encletic': u''
}
STOPWORDS[u'لهو'] = {
    'word': u'لهو',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور',
    'vocalized': u'لَهُوَ',
    'stem': u'هُوَ',
    'type': u'STOPWORD',
    'original': u'هُوَ',
    'encletic': u''
}
STOPWORDS[u'وهو'] = {
    'word': u'وهو',
    'procletic': u'-وَ',
    'tags': u'أداة:ضمير منفصل:معطوف',
    'vocalized': u'وَهُوَ',
    'stem': u'هُوَ',
    'type': u'STOPWORD',
    'original': u'هُوَ',
    'encletic': u''
}
STOPWORDS[u'فهو'] = {
    'word': u'فهو',
    'procletic': u'-فَ',
    'tags': u'أداة:ضمير منفصل:معطوف',
    'vocalized': u'فَهُوَ',
    'stem': u'هُوَ',
    'type': u'STOPWORD',
    'original': u'هُوَ',
    'encletic': u''
}
STOPWORDS[u'وبهو'] = {
    'word': u'وبهو',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:ضمير منفصل:مجرور:معطوف',
    'vocalized': u'وَبِهُوَ',
    'stem': u'هُوَ',
    'type': u'STOPWORD',
    'original': u'هُوَ',
    'encletic': u''
}
STOPWORDS[u'فبهو'] = {
    'word': u'فبهو',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:ضمير منفصل:مجرور:معطوف',
    'vocalized': u'فَبِهُوَ',
    'stem': u'هُوَ',
    'type': u'STOPWORD',
    'original': u'هُوَ',
    'encletic': u''
}
STOPWORDS[u'وكهو'] = {
    'word': u'وكهو',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:ضمير منفصل:مجرور:معطوف',
    'vocalized': u'وَكَهُوَ',
    'stem': u'هُوَ',
    'type': u'STOPWORD',
    'original': u'هُوَ',
    'encletic': u''
}
STOPWORDS[u'فكهو'] = {
    'word': u'فكهو',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:ضمير منفصل:مجرور:معطوف',
    'vocalized': u'فَكَهُوَ',
    'stem': u'هُوَ',
    'type': u'STOPWORD',
    'original': u'هُوَ',
    'encletic': u''
}
STOPWORDS[u'ولهو'] = {
    'word': u'ولهو',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:ضمير منفصل:مجرور:معطوف',
    'vocalized': u'وَلِهُوَ',
    'stem': u'هُوَ',
    'type': u'STOPWORD',
    'original': u'هُوَ',
    'encletic': u''
}
STOPWORDS[u'فلهو'] = {
    'word': u'فلهو',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:ضمير منفصل:مجرور:معطوف',
    'vocalized': u'فَلِهُوَ',
    'stem': u'هُوَ',
    'type': u'STOPWORD',
    'original': u'هُوَ',
    'encletic': u''
}
STOPWORDS[u'ولهو'] = {
    'word': u'ولهو',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'وَلَهُوَ',
    'stem': u'هُوَ',
    'type': u'STOPWORD',
    'original': u'هُوَ',
    'encletic': u''
}
STOPWORDS[u'فلهو'] = {
    'word': u'فلهو',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'فَلَهُوَ',
    'stem': u'هُوَ',
    'type': u'STOPWORD',
    'original': u'هُوَ',
    'encletic': u''
}
STOPWORDS[u'أهو'] = {
    'word': u'أهو',
    'procletic': u'-أَ',
    'tags': u'أداة:ضمير منفصل:مجرور',
    'vocalized': u'أَهُوَ',
    'stem': u'هُوَ',
    'type': u'STOPWORD',
    'original': u'هُوَ',
    'encletic': u''
}
STOPWORDS[u'أبهو'] = {
    'word': u'أبهو',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:ضمير منفصل:مجرور:مجرور',
    'vocalized': u'أَبِهُوَ',
    'stem': u'هُوَ',
    'type': u'STOPWORD',
    'original': u'هُوَ',
    'encletic': u''
}
STOPWORDS[u'أكهو'] = {
    'word': u'أكهو',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:ضمير منفصل:مجرور:مجرور',
    'vocalized': u'أَكَهُوَ',
    'stem': u'هُوَ',
    'type': u'STOPWORD',
    'original': u'هُوَ',
    'encletic': u''
}
STOPWORDS[u'ألهو'] = {
    'word': u'ألهو',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:ضمير منفصل:مجرور:مجرور',
    'vocalized': u'أَلِهُوَ',
    'stem': u'هُوَ',
    'type': u'STOPWORD',
    'original': u'هُوَ',
    'encletic': u''
}
STOPWORDS[u'ألهو'] = {
    'word': u'ألهو',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور',
    'vocalized': u'أَلَهُوَ',
    'stem': u'هُوَ',
    'type': u'STOPWORD',
    'original': u'هُوَ',
    'encletic': u''
}
STOPWORDS[u'أوهو'] = {
    'word': u'أوهو',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ضمير منفصل:معطوف:مجرور',
    'vocalized': u'أَوَهُوَ',
    'stem': u'هُوَ',
    'type': u'STOPWORD',
    'original': u'هُوَ',
    'encletic': u''
}
STOPWORDS[u'أفهو'] = {
    'word': u'أفهو',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ضمير منفصل:معطوف:مجرور',
    'vocalized': u'أَفَهُوَ',
    'stem': u'هُوَ',
    'type': u'STOPWORD',
    'original': u'هُوَ',
    'encletic': u''
}
STOPWORDS[u'أوبهو'] = {
    'word': u'أوبهو',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:ضمير منفصل:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَبِهُوَ',
    'stem': u'هُوَ',
    'type': u'STOPWORD',
    'original': u'هُوَ',
    'encletic': u''
}
STOPWORDS[u'أفبهو'] = {
    'word': u'أفبهو',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:ضمير منفصل:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَبِهُوَ',
    'stem': u'هُوَ',
    'type': u'STOPWORD',
    'original': u'هُوَ',
    'encletic': u''
}
STOPWORDS[u'أوكهو'] = {
    'word': u'أوكهو',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:ضمير منفصل:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَكَهُوَ',
    'stem': u'هُوَ',
    'type': u'STOPWORD',
    'original': u'هُوَ',
    'encletic': u''
}
STOPWORDS[u'أفكهو'] = {
    'word': u'أفكهو',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:ضمير منفصل:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَكَهُوَ',
    'stem': u'هُوَ',
    'type': u'STOPWORD',
    'original': u'هُوَ',
    'encletic': u''
}
STOPWORDS[u'أولهو'] = {
    'word': u'أولهو',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:ضمير منفصل:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَلِهُوَ',
    'stem': u'هُوَ',
    'type': u'STOPWORD',
    'original': u'هُوَ',
    'encletic': u''
}
STOPWORDS[u'أفلهو'] = {
    'word': u'أفلهو',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:ضمير منفصل:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَلِهُوَ',
    'stem': u'هُوَ',
    'type': u'STOPWORD',
    'original': u'هُوَ',
    'encletic': u''
}
STOPWORDS[u'أولهو'] = {
    'word': u'أولهو',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَلَهُوَ',
    'stem': u'هُوَ',
    'type': u'STOPWORD',
    'original': u'هُوَ',
    'encletic': u''
}
STOPWORDS[u'أفلهو'] = {
    'word': u'أفلهو',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَلَهُوَ',
    'stem': u'هُوَ',
    'type': u'STOPWORD',
    'original': u'هُوَ',
    'encletic': u''
}
STOPWORDS[u'هي'] = {
    'word': u'هِي',
    'procletic': u'',
    'tags': u'أداة:ضمير منفصل',
    'vocalized': u'هِي',
    'stem': u'هِي',
    'type': u'STOPWORD',
    'original': u'هِي',
    'encletic': u''
}
STOPWORDS[u'بهي'] = {
    'word': u'بهي',
    'procletic': u'-بِ',
    'tags': u'أداة:ضمير منفصل:مجرور',
    'vocalized': u'بِهِي',
    'stem': u'هِي',
    'type': u'STOPWORD',
    'original': u'هِي',
    'encletic': u''
}
STOPWORDS[u'كهي'] = {
    'word': u'كهي',
    'procletic': u'-كَ',
    'tags': u'أداة:ضمير منفصل:مجرور',
    'vocalized': u'كَهِي',
    'stem': u'هِي',
    'type': u'STOPWORD',
    'original': u'هِي',
    'encletic': u''
}
STOPWORDS[u'لهي'] = {
    'word': u'لهي',
    'procletic': u'-لِ',
    'tags': u'أداة:ضمير منفصل:مجرور',
    'vocalized': u'لِهِي',
    'stem': u'هِي',
    'type': u'STOPWORD',
    'original': u'هِي',
    'encletic': u''
}
STOPWORDS[u'لهي'] = {
    'word': u'لهي',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور',
    'vocalized': u'لَهِي',
    'stem': u'هِي',
    'type': u'STOPWORD',
    'original': u'هِي',
    'encletic': u''
}
STOPWORDS[u'وهي'] = {
    'word': u'وهي',
    'procletic': u'-وَ',
    'tags': u'أداة:ضمير منفصل:معطوف',
    'vocalized': u'وَهِي',
    'stem': u'هِي',
    'type': u'STOPWORD',
    'original': u'هِي',
    'encletic': u''
}
STOPWORDS[u'فهي'] = {
    'word': u'فهي',
    'procletic': u'-فَ',
    'tags': u'أداة:ضمير منفصل:معطوف',
    'vocalized': u'فَهِي',
    'stem': u'هِي',
    'type': u'STOPWORD',
    'original': u'هِي',
    'encletic': u''
}
STOPWORDS[u'وبهي'] = {
    'word': u'وبهي',
    'procletic': u'-بِ-وَ',
    'tags': u'أداة:ضمير منفصل:مجرور:معطوف',
    'vocalized': u'وَبِهِي',
    'stem': u'هِي',
    'type': u'STOPWORD',
    'original': u'هِي',
    'encletic': u''
}
STOPWORDS[u'فبهي'] = {
    'word': u'فبهي',
    'procletic': u'-بِ-فَ',
    'tags': u'أداة:ضمير منفصل:مجرور:معطوف',
    'vocalized': u'فَبِهِي',
    'stem': u'هِي',
    'type': u'STOPWORD',
    'original': u'هِي',
    'encletic': u''
}
STOPWORDS[u'وكهي'] = {
    'word': u'وكهي',
    'procletic': u'-كَ-وَ',
    'tags': u'أداة:ضمير منفصل:مجرور:معطوف',
    'vocalized': u'وَكَهِي',
    'stem': u'هِي',
    'type': u'STOPWORD',
    'original': u'هِي',
    'encletic': u''
}
STOPWORDS[u'فكهي'] = {
    'word': u'فكهي',
    'procletic': u'-كَ-فَ',
    'tags': u'أداة:ضمير منفصل:مجرور:معطوف',
    'vocalized': u'فَكَهِي',
    'stem': u'هِي',
    'type': u'STOPWORD',
    'original': u'هِي',
    'encletic': u''
}
STOPWORDS[u'ولهي'] = {
    'word': u'ولهي',
    'procletic': u'-لِ-وَ',
    'tags': u'أداة:ضمير منفصل:مجرور:معطوف',
    'vocalized': u'وَلِهِي',
    'stem': u'هِي',
    'type': u'STOPWORD',
    'original': u'هِي',
    'encletic': u''
}
STOPWORDS[u'فلهي'] = {
    'word': u'فلهي',
    'procletic': u'-لِ-فَ',
    'tags': u'أداة:ضمير منفصل:مجرور:معطوف',
    'vocalized': u'فَلِهِي',
    'stem': u'هِي',
    'type': u'STOPWORD',
    'original': u'هِي',
    'encletic': u''
}
STOPWORDS[u'ولهي'] = {
    'word': u'ولهي',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'وَلَهِي',
    'stem': u'هِي',
    'type': u'STOPWORD',
    'original': u'هِي',
    'encletic': u''
}
STOPWORDS[u'فلهي'] = {
    'word': u'فلهي',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'فَلَهِي',
    'stem': u'هِي',
    'type': u'STOPWORD',
    'original': u'هِي',
    'encletic': u''
}
STOPWORDS[u'أهي'] = {
    'word': u'أهي',
    'procletic': u'-أَ',
    'tags': u'أداة:ضمير منفصل:مجرور',
    'vocalized': u'أَهِي',
    'stem': u'هِي',
    'type': u'STOPWORD',
    'original': u'هِي',
    'encletic': u''
}
STOPWORDS[u'أبهي'] = {
    'word': u'أبهي',
    'procletic': u'-بِ-أَ',
    'tags': u'أداة:ضمير منفصل:مجرور:مجرور',
    'vocalized': u'أَبِهِي',
    'stem': u'هِي',
    'type': u'STOPWORD',
    'original': u'هِي',
    'encletic': u''
}
STOPWORDS[u'أكهي'] = {
    'word': u'أكهي',
    'procletic': u'-كَ-أَ',
    'tags': u'أداة:ضمير منفصل:مجرور:مجرور',
    'vocalized': u'أَكَهِي',
    'stem': u'هِي',
    'type': u'STOPWORD',
    'original': u'هِي',
    'encletic': u''
}
STOPWORDS[u'ألهي'] = {
    'word': u'ألهي',
    'procletic': u'-لِ-أَ',
    'tags': u'أداة:ضمير منفصل:مجرور:مجرور',
    'vocalized': u'أَلِهِي',
    'stem': u'هِي',
    'type': u'STOPWORD',
    'original': u'هِي',
    'encletic': u''
}
STOPWORDS[u'ألهي'] = {
    'word': u'ألهي',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور',
    'vocalized': u'أَلَهِي',
    'stem': u'هِي',
    'type': u'STOPWORD',
    'original': u'هِي',
    'encletic': u''
}
STOPWORDS[u'أوهي'] = {
    'word': u'أوهي',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ضمير منفصل:معطوف:مجرور',
    'vocalized': u'أَوَهِي',
    'stem': u'هِي',
    'type': u'STOPWORD',
    'original': u'هِي',
    'encletic': u''
}
STOPWORDS[u'أفهي'] = {
    'word': u'أفهي',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ضمير منفصل:معطوف:مجرور',
    'vocalized': u'أَفَهِي',
    'stem': u'هِي',
    'type': u'STOPWORD',
    'original': u'هِي',
    'encletic': u''
}
STOPWORDS[u'أوبهي'] = {
    'word': u'أوبهي',
    'procletic': u'-بِ-وَ-أَ',
    'tags': u'أداة:ضمير منفصل:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَبِهِي',
    'stem': u'هِي',
    'type': u'STOPWORD',
    'original': u'هِي',
    'encletic': u''
}
STOPWORDS[u'أفبهي'] = {
    'word': u'أفبهي',
    'procletic': u'-بِ-فَ-أَ',
    'tags': u'أداة:ضمير منفصل:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَبِهِي',
    'stem': u'هِي',
    'type': u'STOPWORD',
    'original': u'هِي',
    'encletic': u''
}
STOPWORDS[u'أوكهي'] = {
    'word': u'أوكهي',
    'procletic': u'-كَ-وَ-أَ',
    'tags': u'أداة:ضمير منفصل:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَكَهِي',
    'stem': u'هِي',
    'type': u'STOPWORD',
    'original': u'هِي',
    'encletic': u''
}
STOPWORDS[u'أفكهي'] = {
    'word': u'أفكهي',
    'procletic': u'-كَ-فَ-أَ',
    'tags': u'أداة:ضمير منفصل:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَكَهِي',
    'stem': u'هِي',
    'type': u'STOPWORD',
    'original': u'هِي',
    'encletic': u''
}
STOPWORDS[u'أولهي'] = {
    'word': u'أولهي',
    'procletic': u'-لِ-وَ-أَ',
    'tags': u'أداة:ضمير منفصل:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَلِهِي',
    'stem': u'هِي',
    'type': u'STOPWORD',
    'original': u'هِي',
    'encletic': u''
}
STOPWORDS[u'أفلهي'] = {
    'word': u'أفلهي',
    'procletic': u'-لِ-فَ-أَ',
    'tags': u'أداة:ضمير منفصل:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَلِهِي',
    'stem': u'هِي',
    'type': u'STOPWORD',
    'original': u'هِي',
    'encletic': u''
}
STOPWORDS[u'أولهي'] = {
    'word': u'أولهي',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَلَهِي',
    'stem': u'هِي',
    'type': u'STOPWORD',
    'original': u'هِي',
    'encletic': u''
}
STOPWORDS[u'أفلهي'] = {
    'word': u'أفلهي',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَلَهِي',
    'stem': u'هِي',
    'type': u'STOPWORD',
    'original': u'هِي',
    'encletic': u''
}
STOPWORDS[u'إياك'] = {
    'word': u'إِيَّاكَ',
    'procletic': u'',
    'tags': u'أداة:ضمير منفصل منصوب',
    'vocalized': u'إِيَّاكَ',
    'stem': u'إِيَّاكَ',
    'type': u'STOPWORD',
    'original': u'إِيَّاكَ',
    'encletic': u''
}
STOPWORDS[u'وإياك'] = {
    'word': u'وإياك',
    'procletic': u'-وَ',
    'tags': u'أداة:ضمير منفصل منصوب:معطوف',
    'vocalized': u'وَإِيَّاكَ',
    'stem': u'إِيَّاكَ',
    'type': u'STOPWORD',
    'original': u'إِيَّاكَ',
    'encletic': u''
}
STOPWORDS[u'فإياك'] = {
    'word': u'فإياك',
    'procletic': u'-فَ',
    'tags': u'أداة:ضمير منفصل منصوب:معطوف',
    'vocalized': u'فَإِيَّاكَ',
    'stem': u'إِيَّاكَ',
    'type': u'STOPWORD',
    'original': u'إِيَّاكَ',
    'encletic': u''
}
STOPWORDS[u'إياك'] = {
    'word': u'إِيَّاكِ',
    'procletic': u'',
    'tags': u'أداة:ضمير منفصل منصوب',
    'vocalized': u'إِيَّاكِ',
    'stem': u'إِيَّاكِ',
    'type': u'STOPWORD',
    'original': u'إِيَّاكِ',
    'encletic': u''
}
STOPWORDS[u'وإياك'] = {
    'word': u'وإياك',
    'procletic': u'-وَ',
    'tags': u'أداة:ضمير منفصل منصوب:معطوف',
    'vocalized': u'وَإِيَّاكِ',
    'stem': u'إِيَّاكِ',
    'type': u'STOPWORD',
    'original': u'إِيَّاكِ',
    'encletic': u''
}
STOPWORDS[u'فإياك'] = {
    'word': u'فإياك',
    'procletic': u'-فَ',
    'tags': u'أداة:ضمير منفصل منصوب:معطوف',
    'vocalized': u'فَإِيَّاكِ',
    'stem': u'إِيَّاكِ',
    'type': u'STOPWORD',
    'original': u'إِيَّاكِ',
    'encletic': u''
}
STOPWORDS[u'إياكم'] = {
    'word': u'إِيَّاكُمْ',
    'procletic': u'',
    'tags': u'أداة:ضمير منفصل منصوب',
    'vocalized': u'إِيَّاكُمْ',
    'stem': u'إِيَّاكُمْ',
    'type': u'STOPWORD',
    'original': u'إِيَّاكُمْ',
    'encletic': u''
}
STOPWORDS[u'وإياكم'] = {
    'word': u'وإياكم',
    'procletic': u'-وَ',
    'tags': u'أداة:ضمير منفصل منصوب:معطوف',
    'vocalized': u'وَإِيَّاكُمْ',
    'stem': u'إِيَّاكُمْ',
    'type': u'STOPWORD',
    'original': u'إِيَّاكُمْ',
    'encletic': u''
}
STOPWORDS[u'فإياكم'] = {
    'word': u'فإياكم',
    'procletic': u'-فَ',
    'tags': u'أداة:ضمير منفصل منصوب:معطوف',
    'vocalized': u'فَإِيَّاكُمْ',
    'stem': u'إِيَّاكُمْ',
    'type': u'STOPWORD',
    'original': u'إِيَّاكُمْ',
    'encletic': u''
}
STOPWORDS[u'إياكما'] = {
    'word': u'إِيَّاكُمَا',
    'procletic': u'',
    'tags': u'أداة:ضمير منفصل منصوب',
    'vocalized': u'إِيَّاكُمَا',
    'stem': u'إِيَّاكُمَا',
    'type': u'STOPWORD',
    'original': u'إِيَّاكُمَا',
    'encletic': u''
}
STOPWORDS[u'وإياكما'] = {
    'word': u'وإياكما',
    'procletic': u'-وَ',
    'tags': u'أداة:ضمير منفصل منصوب:معطوف',
    'vocalized': u'وَإِيَّاكُمَا',
    'stem': u'إِيَّاكُمَا',
    'type': u'STOPWORD',
    'original': u'إِيَّاكُمَا',
    'encletic': u''
}
STOPWORDS[u'فإياكما'] = {
    'word': u'فإياكما',
    'procletic': u'-فَ',
    'tags': u'أداة:ضمير منفصل منصوب:معطوف',
    'vocalized': u'فَإِيَّاكُمَا',
    'stem': u'إِيَّاكُمَا',
    'type': u'STOPWORD',
    'original': u'إِيَّاكُمَا',
    'encletic': u''
}
STOPWORDS[u'إياكن'] = {
    'word': u'إِيَّاكُنَّ',
    'procletic': u'',
    'tags': u'أداة:ضمير منفصل منصوب',
    'vocalized': u'إِيَّاكُنَّ',
    'stem': u'إِيَّاكُنَّ',
    'type': u'STOPWORD',
    'original': u'إِيَّاكُنَّ',
    'encletic': u''
}
STOPWORDS[u'وإياكن'] = {
    'word': u'وإياكن',
    'procletic': u'-وَ',
    'tags': u'أداة:ضمير منفصل منصوب:معطوف',
    'vocalized': u'وَإِيَّاكُنَّ',
    'stem': u'إِيَّاكُنَّ',
    'type': u'STOPWORD',
    'original': u'إِيَّاكُنَّ',
    'encletic': u''
}
STOPWORDS[u'فإياكن'] = {
    'word': u'فإياكن',
    'procletic': u'-فَ',
    'tags': u'أداة:ضمير منفصل منصوب:معطوف',
    'vocalized': u'فَإِيَّاكُنَّ',
    'stem': u'إِيَّاكُنَّ',
    'type': u'STOPWORD',
    'original': u'إِيَّاكُنَّ',
    'encletic': u''
}
STOPWORDS[u'إيانا'] = {
    'word': u'إِيَّانَا',
    'procletic': u'',
    'tags': u'أداة:ضمير منفصل منصوب',
    'vocalized': u'إِيَّانَا',
    'stem': u'إِيَّانَا',
    'type': u'STOPWORD',
    'original': u'إِيَّانَا',
    'encletic': u''
}
STOPWORDS[u'وإيانا'] = {
    'word': u'وإيانا',
    'procletic': u'-وَ',
    'tags': u'أداة:ضمير منفصل منصوب:معطوف',
    'vocalized': u'وَإِيَّانَا',
    'stem': u'إِيَّانَا',
    'type': u'STOPWORD',
    'original': u'إِيَّانَا',
    'encletic': u''
}
STOPWORDS[u'فإيانا'] = {
    'word': u'فإيانا',
    'procletic': u'-فَ',
    'tags': u'أداة:ضمير منفصل منصوب:معطوف',
    'vocalized': u'فَإِيَّانَا',
    'stem': u'إِيَّانَا',
    'type': u'STOPWORD',
    'original': u'إِيَّانَا',
    'encletic': u''
}
STOPWORDS[u'إياه'] = {
    'word': u'إِيَّاهُ',
    'procletic': u'',
    'tags': u'أداة:ضمير منفصل منصوب',
    'vocalized': u'إِيَّاهُ',
    'stem': u'إِيَّاهُ',
    'type': u'STOPWORD',
    'original': u'إِيَّاهُ',
    'encletic': u''
}
STOPWORDS[u'وإياه'] = {
    'word': u'وإياه',
    'procletic': u'-وَ',
    'tags': u'أداة:ضمير منفصل منصوب:معطوف',
    'vocalized': u'وَإِيَّاهُ',
    'stem': u'إِيَّاهُ',
    'type': u'STOPWORD',
    'original': u'إِيَّاهُ',
    'encletic': u''
}
STOPWORDS[u'فإياه'] = {
    'word': u'فإياه',
    'procletic': u'-فَ',
    'tags': u'أداة:ضمير منفصل منصوب:معطوف',
    'vocalized': u'فَإِيَّاهُ',
    'stem': u'إِيَّاهُ',
    'type': u'STOPWORD',
    'original': u'إِيَّاهُ',
    'encletic': u''
}
STOPWORDS[u'إياها'] = {
    'word': u'إِيَّاهَا',
    'procletic': u'',
    'tags': u'أداة:ضمير منفصل منصوب',
    'vocalized': u'إِيَّاهَا',
    'stem': u'إِيَّاهَا',
    'type': u'STOPWORD',
    'original': u'إِيَّاهَا',
    'encletic': u''
}
STOPWORDS[u'وإياها'] = {
    'word': u'وإياها',
    'procletic': u'-وَ',
    'tags': u'أداة:ضمير منفصل منصوب:معطوف',
    'vocalized': u'وَإِيَّاهَا',
    'stem': u'إِيَّاهَا',
    'type': u'STOPWORD',
    'original': u'إِيَّاهَا',
    'encletic': u''
}
STOPWORDS[u'فإياها'] = {
    'word': u'فإياها',
    'procletic': u'-فَ',
    'tags': u'أداة:ضمير منفصل منصوب:معطوف',
    'vocalized': u'فَإِيَّاهَا',
    'stem': u'إِيَّاهَا',
    'type': u'STOPWORD',
    'original': u'إِيَّاهَا',
    'encletic': u''
}
STOPWORDS[u'إياهم'] = {
    'word': u'إِيَّاهُمْ',
    'procletic': u'',
    'tags': u'أداة:ضمير منفصل منصوب',
    'vocalized': u'إِيَّاهُمْ',
    'stem': u'إِيَّاهُمْ',
    'type': u'STOPWORD',
    'original': u'إِيَّاهُمْ',
    'encletic': u''
}
STOPWORDS[u'وإياهم'] = {
    'word': u'وإياهم',
    'procletic': u'-وَ',
    'tags': u'أداة:ضمير منفصل منصوب:معطوف',
    'vocalized': u'وَإِيَّاهُمْ',
    'stem': u'إِيَّاهُمْ',
    'type': u'STOPWORD',
    'original': u'إِيَّاهُمْ',
    'encletic': u''
}
STOPWORDS[u'فإياهم'] = {
    'word': u'فإياهم',
    'procletic': u'-فَ',
    'tags': u'أداة:ضمير منفصل منصوب:معطوف',
    'vocalized': u'فَإِيَّاهُمْ',
    'stem': u'إِيَّاهُمْ',
    'type': u'STOPWORD',
    'original': u'إِيَّاهُمْ',
    'encletic': u''
}
STOPWORDS[u'إياهما'] = {
    'word': u'إِيَّاهُمَا',
    'procletic': u'',
    'tags': u'أداة:ضمير منفصل منصوب',
    'vocalized': u'إِيَّاهُمَا',
    'stem': u'إِيَّاهُمَا',
    'type': u'STOPWORD',
    'original': u'إِيَّاهُمَا',
    'encletic': u''
}
STOPWORDS[u'وإياهما'] = {
    'word': u'وإياهما',
    'procletic': u'-وَ',
    'tags': u'أداة:ضمير منفصل منصوب:معطوف',
    'vocalized': u'وَإِيَّاهُمَا',
    'stem': u'إِيَّاهُمَا',
    'type': u'STOPWORD',
    'original': u'إِيَّاهُمَا',
    'encletic': u''
}
STOPWORDS[u'فإياهما'] = {
    'word': u'فإياهما',
    'procletic': u'-فَ',
    'tags': u'أداة:ضمير منفصل منصوب:معطوف',
    'vocalized': u'فَإِيَّاهُمَا',
    'stem': u'إِيَّاهُمَا',
    'type': u'STOPWORD',
    'original': u'إِيَّاهُمَا',
    'encletic': u''
}
STOPWORDS[u'إياهن'] = {
    'word': u'إِيَّاهُنَّ',
    'procletic': u'',
    'tags': u'أداة:ضمير منفصل منصوب',
    'vocalized': u'إِيَّاهُنَّ',
    'stem': u'إِيَّاهُنَّ',
    'type': u'STOPWORD',
    'original': u'إِيَّاهُنَّ',
    'encletic': u''
}
STOPWORDS[u'وإياهن'] = {
    'word': u'وإياهن',
    'procletic': u'-وَ',
    'tags': u'أداة:ضمير منفصل منصوب:معطوف',
    'vocalized': u'وَإِيَّاهُنَّ',
    'stem': u'إِيَّاهُنَّ',
    'type': u'STOPWORD',
    'original': u'إِيَّاهُنَّ',
    'encletic': u''
}
STOPWORDS[u'فإياهن'] = {
    'word': u'فإياهن',
    'procletic': u'-فَ',
    'tags': u'أداة:ضمير منفصل منصوب:معطوف',
    'vocalized': u'فَإِيَّاهُنَّ',
    'stem': u'إِيَّاهُنَّ',
    'type': u'STOPWORD',
    'original': u'إِيَّاهُنَّ',
    'encletic': u''
}
STOPWORDS[u'إياي'] = {
    'word': u'إِيَّايَ',
    'procletic': u'',
    'tags': u'أداة:ضمير منفصل منصوب',
    'vocalized': u'إِيَّايَ',
    'stem': u'إِيَّايَ',
    'type': u'STOPWORD',
    'original': u'إِيَّايَ',
    'encletic': u''
}
STOPWORDS[u'وإياي'] = {
    'word': u'وإياي',
    'procletic': u'-وَ',
    'tags': u'أداة:ضمير منفصل منصوب:معطوف',
    'vocalized': u'وَإِيَّايَ',
    'stem': u'إِيَّايَ',
    'type': u'STOPWORD',
    'original': u'إِيَّايَ',
    'encletic': u''
}
STOPWORDS[u'فإياي'] = {
    'word': u'فإياي',
    'procletic': u'-فَ',
    'tags': u'أداة:ضمير منفصل منصوب:معطوف',
    'vocalized': u'فَإِيَّايَ',
    'stem': u'إِيَّايَ',
    'type': u'STOPWORD',
    'original': u'إِيَّايَ',
    'encletic': u''
}
STOPWORDS[u'دون'] = {
    'word': u'دُونَ',
    'procletic': u'',
    'tags': u'أداة:ظرف',
    'vocalized': u'دُونَ',
    'stem': u'دُونَ',
    'type': u'STOPWORD',
    'original': u'دُونَ',
    'encletic': u''
}
STOPWORDS[u'ودون'] = {
    'word': u'ودون',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف:معطوف',
    'vocalized': u'وَدُونَ',
    'stem': u'دُونَ',
    'type': u'STOPWORD',
    'original': u'دُونَ',
    'encletic': u''
}
STOPWORDS[u'فدون'] = {
    'word': u'فدون',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف:معطوف',
    'vocalized': u'فَدُونَ',
    'stem': u'دُونَ',
    'type': u'STOPWORD',
    'original': u'دُونَ',
    'encletic': u''
}
STOPWORDS[u'ريث'] = {
    'word': u'رَيْثَ',
    'procletic': u'',
    'tags': u'أداة:ظرف',
    'vocalized': u'رَيْثَ',
    'stem': u'رَيْثَ',
    'type': u'STOPWORD',
    'original': u'رَيْثَ',
    'encletic': u''
}
STOPWORDS[u'وريث'] = {
    'word': u'وريث',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف:معطوف',
    'vocalized': u'وَرَيْثَ',
    'stem': u'رَيْثَ',
    'type': u'STOPWORD',
    'original': u'رَيْثَ',
    'encletic': u''
}
STOPWORDS[u'فريث'] = {
    'word': u'فريث',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف:معطوف',
    'vocalized': u'فَرَيْثَ',
    'stem': u'رَيْثَ',
    'type': u'STOPWORD',
    'original': u'رَيْثَ',
    'encletic': u''
}
STOPWORDS[u'عند'] = {
    'word': u'عِنْدَ',
    'procletic': u'',
    'tags': u'أداة:ظرف',
    'vocalized': u'عِنْدَ',
    'stem': u'عِنْدَ',
    'type': u'STOPWORD',
    'original': u'عِنْدَ',
    'encletic': u''
}
STOPWORDS[u'وعند'] = {
    'word': u'وعند',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف:معطوف',
    'vocalized': u'وَعِنْدَ',
    'stem': u'عِنْدَ',
    'type': u'STOPWORD',
    'original': u'عِنْدَ',
    'encletic': u''
}
STOPWORDS[u'فعند'] = {
    'word': u'فعند',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف:معطوف',
    'vocalized': u'فَعِنْدَ',
    'stem': u'عِنْدَ',
    'type': u'STOPWORD',
    'original': u'عِنْدَ',
    'encletic': u''
}
STOPWORDS[u'عندي'] = {
    'word': u'عندي',
    'procletic': u'',
    'tags': u'أداة:ظرف:مضاف',
    'vocalized': u'عِنْدَِي',
    'stem': u'عِنْدَ',
    'type': u'STOPWORD',
    'original': u'عِنْدَ',
    'encletic': u'ِي'
}
STOPWORDS[u'عندك'] = {
    'word': u'عندك',
    'procletic': u'',
    'tags': u'أداة:ظرف:مضاف',
    'vocalized': u'عِنْدَكَ',
    'stem': u'عِنْدَ',
    'type': u'STOPWORD',
    'original': u'عِنْدَ',
    'encletic': u'كَ'
}
STOPWORDS[u'عندك'] = {
    'word': u'عندك',
    'procletic': u'',
    'tags': u'أداة:ظرف:مضاف',
    'vocalized': u'عِنْدَكِ',
    'stem': u'عِنْدَ',
    'type': u'STOPWORD',
    'original': u'عِنْدَ',
    'encletic': u'كِ'
}
STOPWORDS[u'عنده'] = {
    'word': u'عنده',
    'procletic': u'',
    'tags': u'أداة:ظرف:مضاف',
    'vocalized': u'عِنْدَه',
    'stem': u'عِنْدَ',
    'type': u'STOPWORD',
    'original': u'عِنْدَ',
    'encletic': u'ه'
}
STOPWORDS[u'عندكم'] = {
    'word': u'عندكم',
    'procletic': u'',
    'tags': u'أداة:ظرف:مضاف',
    'vocalized': u'عِنْدَكُمْ',
    'stem': u'عِنْدَ',
    'type': u'STOPWORD',
    'original': u'عِنْدَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'عندكن'] = {
    'word': u'عندكن',
    'procletic': u'',
    'tags': u'أداة:ظرف:مضاف',
    'vocalized': u'عِنْدَكُنَّ',
    'stem': u'عِنْدَ',
    'type': u'STOPWORD',
    'original': u'عِنْدَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'عندها'] = {
    'word': u'عندها',
    'procletic': u'',
    'tags': u'أداة:ظرف:مضاف',
    'vocalized': u'عِنْدَهَا',
    'stem': u'عِنْدَ',
    'type': u'STOPWORD',
    'original': u'عِنْدَ',
    'encletic': u'هَا'
}
STOPWORDS[u'عندهم'] = {
    'word': u'عندهم',
    'procletic': u'',
    'tags': u'أداة:ظرف:مضاف',
    'vocalized': u'عِنْدَهُمْ',
    'stem': u'عِنْدَ',
    'type': u'STOPWORD',
    'original': u'عِنْدَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'عندهن'] = {
    'word': u'عندهن',
    'procletic': u'',
    'tags': u'أداة:ظرف:مضاف',
    'vocalized': u'عِنْدَهُنَّ',
    'stem': u'عِنْدَ',
    'type': u'STOPWORD',
    'original': u'عِنْدَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'عندنا'] = {
    'word': u'عندنا',
    'procletic': u'',
    'tags': u'أداة:ظرف:مضاف',
    'vocalized': u'عِنْدَنَا',
    'stem': u'عِنْدَ',
    'type': u'STOPWORD',
    'original': u'عِنْدَ',
    'encletic': u'نَا'
}
STOPWORDS[u'عندكما'] = {
    'word': u'عندكما',
    'procletic': u'',
    'tags': u'أداة:ظرف:مضاف',
    'vocalized': u'عِنْدَكُمَا',
    'stem': u'عِنْدَ',
    'type': u'STOPWORD',
    'original': u'عِنْدَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'عندهما'] = {
    'word': u'عندهما',
    'procletic': u'',
    'tags': u'أداة:ظرف:مضاف',
    'vocalized': u'عِنْدَهُمَا',
    'stem': u'عِنْدَ',
    'type': u'STOPWORD',
    'original': u'عِنْدَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'وعندي'] = {
    'word': u'وعندي',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف:معطوف:مضاف',
    'vocalized': u'وَعِنْدَِي',
    'stem': u'عِنْدَ',
    'type': u'STOPWORD',
    'original': u'عِنْدَ',
    'encletic': u'ِي'
}
STOPWORDS[u'وعندك'] = {
    'word': u'وعندك',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف:معطوف:مضاف',
    'vocalized': u'وَعِنْدَكَ',
    'stem': u'عِنْدَ',
    'type': u'STOPWORD',
    'original': u'عِنْدَ',
    'encletic': u'كَ'
}
STOPWORDS[u'وعندك'] = {
    'word': u'وعندك',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف:معطوف:مضاف',
    'vocalized': u'وَعِنْدَكِ',
    'stem': u'عِنْدَ',
    'type': u'STOPWORD',
    'original': u'عِنْدَ',
    'encletic': u'كِ'
}
STOPWORDS[u'وعنده'] = {
    'word': u'وعنده',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف:معطوف:مضاف',
    'vocalized': u'وَعِنْدَه',
    'stem': u'عِنْدَ',
    'type': u'STOPWORD',
    'original': u'عِنْدَ',
    'encletic': u'ه'
}
STOPWORDS[u'وعندكم'] = {
    'word': u'وعندكم',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف:معطوف:مضاف',
    'vocalized': u'وَعِنْدَكُمْ',
    'stem': u'عِنْدَ',
    'type': u'STOPWORD',
    'original': u'عِنْدَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'وعندكن'] = {
    'word': u'وعندكن',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف:معطوف:مضاف',
    'vocalized': u'وَعِنْدَكُنَّ',
    'stem': u'عِنْدَ',
    'type': u'STOPWORD',
    'original': u'عِنْدَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'وعندها'] = {
    'word': u'وعندها',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف:معطوف:مضاف',
    'vocalized': u'وَعِنْدَهَا',
    'stem': u'عِنْدَ',
    'type': u'STOPWORD',
    'original': u'عِنْدَ',
    'encletic': u'هَا'
}
STOPWORDS[u'وعندهم'] = {
    'word': u'وعندهم',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف:معطوف:مضاف',
    'vocalized': u'وَعِنْدَهُمْ',
    'stem': u'عِنْدَ',
    'type': u'STOPWORD',
    'original': u'عِنْدَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'وعندهن'] = {
    'word': u'وعندهن',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف:معطوف:مضاف',
    'vocalized': u'وَعِنْدَهُنَّ',
    'stem': u'عِنْدَ',
    'type': u'STOPWORD',
    'original': u'عِنْدَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'وعندنا'] = {
    'word': u'وعندنا',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف:معطوف:مضاف',
    'vocalized': u'وَعِنْدَنَا',
    'stem': u'عِنْدَ',
    'type': u'STOPWORD',
    'original': u'عِنْدَ',
    'encletic': u'نَا'
}
STOPWORDS[u'وعندكما'] = {
    'word': u'وعندكما',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف:معطوف:مضاف',
    'vocalized': u'وَعِنْدَكُمَا',
    'stem': u'عِنْدَ',
    'type': u'STOPWORD',
    'original': u'عِنْدَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'وعندهما'] = {
    'word': u'وعندهما',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف:معطوف:مضاف',
    'vocalized': u'وَعِنْدَهُمَا',
    'stem': u'عِنْدَ',
    'type': u'STOPWORD',
    'original': u'عِنْدَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فعندي'] = {
    'word': u'فعندي',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف:معطوف:مضاف',
    'vocalized': u'فَعِنْدَِي',
    'stem': u'عِنْدَ',
    'type': u'STOPWORD',
    'original': u'عِنْدَ',
    'encletic': u'ِي'
}
STOPWORDS[u'فعندك'] = {
    'word': u'فعندك',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف:معطوف:مضاف',
    'vocalized': u'فَعِنْدَكَ',
    'stem': u'عِنْدَ',
    'type': u'STOPWORD',
    'original': u'عِنْدَ',
    'encletic': u'كَ'
}
STOPWORDS[u'فعندك'] = {
    'word': u'فعندك',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف:معطوف:مضاف',
    'vocalized': u'فَعِنْدَكِ',
    'stem': u'عِنْدَ',
    'type': u'STOPWORD',
    'original': u'عِنْدَ',
    'encletic': u'كِ'
}
STOPWORDS[u'فعنده'] = {
    'word': u'فعنده',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف:معطوف:مضاف',
    'vocalized': u'فَعِنْدَه',
    'stem': u'عِنْدَ',
    'type': u'STOPWORD',
    'original': u'عِنْدَ',
    'encletic': u'ه'
}
STOPWORDS[u'فعندكم'] = {
    'word': u'فعندكم',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف:معطوف:مضاف',
    'vocalized': u'فَعِنْدَكُمْ',
    'stem': u'عِنْدَ',
    'type': u'STOPWORD',
    'original': u'عِنْدَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فعندكن'] = {
    'word': u'فعندكن',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف:معطوف:مضاف',
    'vocalized': u'فَعِنْدَكُنَّ',
    'stem': u'عِنْدَ',
    'type': u'STOPWORD',
    'original': u'عِنْدَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فعندها'] = {
    'word': u'فعندها',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف:معطوف:مضاف',
    'vocalized': u'فَعِنْدَهَا',
    'stem': u'عِنْدَ',
    'type': u'STOPWORD',
    'original': u'عِنْدَ',
    'encletic': u'هَا'
}
STOPWORDS[u'فعندهم'] = {
    'word': u'فعندهم',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف:معطوف:مضاف',
    'vocalized': u'فَعِنْدَهُمْ',
    'stem': u'عِنْدَ',
    'type': u'STOPWORD',
    'original': u'عِنْدَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فعندهن'] = {
    'word': u'فعندهن',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف:معطوف:مضاف',
    'vocalized': u'فَعِنْدَهُنَّ',
    'stem': u'عِنْدَ',
    'type': u'STOPWORD',
    'original': u'عِنْدَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فعندنا'] = {
    'word': u'فعندنا',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف:معطوف:مضاف',
    'vocalized': u'فَعِنْدَنَا',
    'stem': u'عِنْدَ',
    'type': u'STOPWORD',
    'original': u'عِنْدَ',
    'encletic': u'نَا'
}
STOPWORDS[u'فعندكما'] = {
    'word': u'فعندكما',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف:معطوف:مضاف',
    'vocalized': u'فَعِنْدَكُمَا',
    'stem': u'عِنْدَ',
    'type': u'STOPWORD',
    'original': u'عِنْدَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فعندهما'] = {
    'word': u'فعندهما',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف:معطوف:مضاف',
    'vocalized': u'فَعِنْدَهُمَا',
    'stem': u'عِنْدَ',
    'type': u'STOPWORD',
    'original': u'عِنْدَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'عوض'] = {
    'word': u'عِوَضَ',
    'procletic': u'',
    'tags': u'أداة:ظرف',
    'vocalized': u'عِوَضَ',
    'stem': u'عِوَضَ',
    'type': u'STOPWORD',
    'original': u'عِوَضَ',
    'encletic': u''
}
STOPWORDS[u'وعوض'] = {
    'word': u'وعوض',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف:معطوف',
    'vocalized': u'وَعِوَضَ',
    'stem': u'عِوَضَ',
    'type': u'STOPWORD',
    'original': u'عِوَضَ',
    'encletic': u''
}
STOPWORDS[u'فعوض'] = {
    'word': u'فعوض',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف:معطوف',
    'vocalized': u'فَعِوَضَ',
    'stem': u'عِوَضَ',
    'type': u'STOPWORD',
    'original': u'عِوَضَ',
    'encletic': u''
}
STOPWORDS[u'قبل'] = {
    'word': u'قَبْلَ',
    'procletic': u'',
    'tags': u'أداة:ظرف',
    'vocalized': u'قَبْلَ',
    'stem': u'قَبْلَ',
    'type': u'STOPWORD',
    'original': u'قَبْلَ',
    'encletic': u''
}
STOPWORDS[u'وقبل'] = {
    'word': u'وقبل',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف:معطوف',
    'vocalized': u'وَقَبْلَ',
    'stem': u'قَبْلَ',
    'type': u'STOPWORD',
    'original': u'قَبْلَ',
    'encletic': u''
}
STOPWORDS[u'فقبل'] = {
    'word': u'فقبل',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف:معطوف',
    'vocalized': u'فَقَبْلَ',
    'stem': u'قَبْلَ',
    'type': u'STOPWORD',
    'original': u'قَبْلَ',
    'encletic': u''
}
STOPWORDS[u'قط'] = {
    'word': u'قَطُّ',
    'procletic': u'',
    'tags': u'أداة:ظرف',
    'vocalized': u'قَطُّ',
    'stem': u'قَطُّ',
    'type': u'STOPWORD',
    'original': u'قَطُّ',
    'encletic': u''
}
STOPWORDS[u'وقط'] = {
    'word': u'وقط',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف:معطوف',
    'vocalized': u'وَقَطُّ',
    'stem': u'قَطُّ',
    'type': u'STOPWORD',
    'original': u'قَطُّ',
    'encletic': u''
}
STOPWORDS[u'فقط'] = {
    'word': u'فقط',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف:معطوف',
    'vocalized': u'فَقَطُّ',
    'stem': u'قَطُّ',
    'type': u'STOPWORD',
    'original': u'قَطُّ',
    'encletic': u''
}
STOPWORDS[u'كلما'] = {
    'word': u'كُلَّمَا',
    'procletic': u'',
    'tags': u'أداة:ظرف',
    'vocalized': u'كُلَّمَا',
    'stem': u'كُلَّمَا',
    'type': u'STOPWORD',
    'original': u'كُلَّمَا',
    'encletic': u''
}
STOPWORDS[u'وكلما'] = {
    'word': u'وكلما',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف:معطوف',
    'vocalized': u'وَكُلَّمَا',
    'stem': u'كُلَّمَا',
    'type': u'STOPWORD',
    'original': u'كُلَّمَا',
    'encletic': u''
}
STOPWORDS[u'فكلما'] = {
    'word': u'فكلما',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف:معطوف',
    'vocalized': u'فَكُلَّمَا',
    'stem': u'كُلَّمَا',
    'type': u'STOPWORD',
    'original': u'كُلَّمَا',
    'encletic': u''
}
STOPWORDS[u'أكلما'] = {
    'word': u'أكلما',
    'procletic': u'-أَ',
    'tags': u'أداة:ظرف:مجرور',
    'vocalized': u'أَكُلَّمَا',
    'stem': u'كُلَّمَا',
    'type': u'STOPWORD',
    'original': u'كُلَّمَا',
    'encletic': u''
}
STOPWORDS[u'أوكلما'] = {
    'word': u'أوكلما',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ظرف:معطوف:مجرور',
    'vocalized': u'أَوَكُلَّمَا',
    'stem': u'كُلَّمَا',
    'type': u'STOPWORD',
    'original': u'كُلَّمَا',
    'encletic': u''
}
STOPWORDS[u'أفكلما'] = {
    'word': u'أفكلما',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ظرف:معطوف:مجرور',
    'vocalized': u'أَفَكُلَّمَا',
    'stem': u'كُلَّمَا',
    'type': u'STOPWORD',
    'original': u'كُلَّمَا',
    'encletic': u''
}
STOPWORDS[u'لدن'] = {
    'word': u'لَدُنْ',
    'procletic': u'',
    'tags': u'أداة:ظرف',
    'vocalized': u'لَدُنْ',
    'stem': u'لَدُنْ',
    'type': u'STOPWORD',
    'original': u'لَدُنْ',
    'encletic': u''
}
STOPWORDS[u'ولدن'] = {
    'word': u'ولدن',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف:معطوف',
    'vocalized': u'وَلَدُنْ',
    'stem': u'لَدُنْ',
    'type': u'STOPWORD',
    'original': u'لَدُنْ',
    'encletic': u''
}
STOPWORDS[u'فلدن'] = {
    'word': u'فلدن',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف:معطوف',
    'vocalized': u'فَلَدُنْ',
    'stem': u'لَدُنْ',
    'type': u'STOPWORD',
    'original': u'لَدُنْ',
    'encletic': u''
}
STOPWORDS[u'لدني'] = {
    'word': u'لدني',
    'procletic': u'',
    'tags': u'أداة:ظرف:مضاف',
    'vocalized': u'لَدُنِْي',
    'stem': u'لَدُنْ',
    'type': u'STOPWORD',
    'original': u'لَدُنْ',
    'encletic': u'ِي'
}
STOPWORDS[u'لدنك'] = {
    'word': u'لدنك',
    'procletic': u'',
    'tags': u'أداة:ظرف:مضاف',
    'vocalized': u'لَدُنْكَ',
    'stem': u'لَدُنْ',
    'type': u'STOPWORD',
    'original': u'لَدُنْ',
    'encletic': u'كَ'
}
STOPWORDS[u'لدنك'] = {
    'word': u'لدنك',
    'procletic': u'',
    'tags': u'أداة:ظرف:مضاف',
    'vocalized': u'لَدُنْكِ',
    'stem': u'لَدُنْ',
    'type': u'STOPWORD',
    'original': u'لَدُنْ',
    'encletic': u'كِ'
}
STOPWORDS[u'لدنه'] = {
    'word': u'لدنه',
    'procletic': u'',
    'tags': u'أداة:ظرف:مضاف',
    'vocalized': u'لَدُنْه',
    'stem': u'لَدُنْ',
    'type': u'STOPWORD',
    'original': u'لَدُنْ',
    'encletic': u'ه'
}
STOPWORDS[u'لدنكم'] = {
    'word': u'لدنكم',
    'procletic': u'',
    'tags': u'أداة:ظرف:مضاف',
    'vocalized': u'لَدُنْكُمْ',
    'stem': u'لَدُنْ',
    'type': u'STOPWORD',
    'original': u'لَدُنْ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'لدنكن'] = {
    'word': u'لدنكن',
    'procletic': u'',
    'tags': u'أداة:ظرف:مضاف',
    'vocalized': u'لَدُنْكُنَّ',
    'stem': u'لَدُنْ',
    'type': u'STOPWORD',
    'original': u'لَدُنْ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'لدنها'] = {
    'word': u'لدنها',
    'procletic': u'',
    'tags': u'أداة:ظرف:مضاف',
    'vocalized': u'لَدُنْهَا',
    'stem': u'لَدُنْ',
    'type': u'STOPWORD',
    'original': u'لَدُنْ',
    'encletic': u'هَا'
}
STOPWORDS[u'لدنهم'] = {
    'word': u'لدنهم',
    'procletic': u'',
    'tags': u'أداة:ظرف:مضاف',
    'vocalized': u'لَدُنْهُمْ',
    'stem': u'لَدُنْ',
    'type': u'STOPWORD',
    'original': u'لَدُنْ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'لدنهن'] = {
    'word': u'لدنهن',
    'procletic': u'',
    'tags': u'أداة:ظرف:مضاف',
    'vocalized': u'لَدُنْهُنَّ',
    'stem': u'لَدُنْ',
    'type': u'STOPWORD',
    'original': u'لَدُنْ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'لدننا'] = {
    'word': u'لدننا',
    'procletic': u'',
    'tags': u'أداة:ظرف:مضاف',
    'vocalized': u'لَدُنْنَا',
    'stem': u'لَدُنْ',
    'type': u'STOPWORD',
    'original': u'لَدُنْ',
    'encletic': u'نَا'
}
STOPWORDS[u'لدنكما'] = {
    'word': u'لدنكما',
    'procletic': u'',
    'tags': u'أداة:ظرف:مضاف',
    'vocalized': u'لَدُنْكُمَا',
    'stem': u'لَدُنْ',
    'type': u'STOPWORD',
    'original': u'لَدُنْ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'لدنهما'] = {
    'word': u'لدنهما',
    'procletic': u'',
    'tags': u'أداة:ظرف:مضاف',
    'vocalized': u'لَدُنْهُمَا',
    'stem': u'لَدُنْ',
    'type': u'STOPWORD',
    'original': u'لَدُنْ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ولدني'] = {
    'word': u'ولدني',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف:معطوف:مضاف',
    'vocalized': u'وَلَدُنِْي',
    'stem': u'لَدُنْ',
    'type': u'STOPWORD',
    'original': u'لَدُنْ',
    'encletic': u'ِي'
}
STOPWORDS[u'ولدنك'] = {
    'word': u'ولدنك',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف:معطوف:مضاف',
    'vocalized': u'وَلَدُنْكَ',
    'stem': u'لَدُنْ',
    'type': u'STOPWORD',
    'original': u'لَدُنْ',
    'encletic': u'كَ'
}
STOPWORDS[u'ولدنك'] = {
    'word': u'ولدنك',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف:معطوف:مضاف',
    'vocalized': u'وَلَدُنْكِ',
    'stem': u'لَدُنْ',
    'type': u'STOPWORD',
    'original': u'لَدُنْ',
    'encletic': u'كِ'
}
STOPWORDS[u'ولدنه'] = {
    'word': u'ولدنه',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف:معطوف:مضاف',
    'vocalized': u'وَلَدُنْه',
    'stem': u'لَدُنْ',
    'type': u'STOPWORD',
    'original': u'لَدُنْ',
    'encletic': u'ه'
}
STOPWORDS[u'ولدنكم'] = {
    'word': u'ولدنكم',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف:معطوف:مضاف',
    'vocalized': u'وَلَدُنْكُمْ',
    'stem': u'لَدُنْ',
    'type': u'STOPWORD',
    'original': u'لَدُنْ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ولدنكن'] = {
    'word': u'ولدنكن',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف:معطوف:مضاف',
    'vocalized': u'وَلَدُنْكُنَّ',
    'stem': u'لَدُنْ',
    'type': u'STOPWORD',
    'original': u'لَدُنْ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ولدنها'] = {
    'word': u'ولدنها',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف:معطوف:مضاف',
    'vocalized': u'وَلَدُنْهَا',
    'stem': u'لَدُنْ',
    'type': u'STOPWORD',
    'original': u'لَدُنْ',
    'encletic': u'هَا'
}
STOPWORDS[u'ولدنهم'] = {
    'word': u'ولدنهم',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف:معطوف:مضاف',
    'vocalized': u'وَلَدُنْهُمْ',
    'stem': u'لَدُنْ',
    'type': u'STOPWORD',
    'original': u'لَدُنْ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ولدنهن'] = {
    'word': u'ولدنهن',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف:معطوف:مضاف',
    'vocalized': u'وَلَدُنْهُنَّ',
    'stem': u'لَدُنْ',
    'type': u'STOPWORD',
    'original': u'لَدُنْ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ولدننا'] = {
    'word': u'ولدننا',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف:معطوف:مضاف',
    'vocalized': u'وَلَدُنْنَا',
    'stem': u'لَدُنْ',
    'type': u'STOPWORD',
    'original': u'لَدُنْ',
    'encletic': u'نَا'
}
STOPWORDS[u'ولدنكما'] = {
    'word': u'ولدنكما',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف:معطوف:مضاف',
    'vocalized': u'وَلَدُنْكُمَا',
    'stem': u'لَدُنْ',
    'type': u'STOPWORD',
    'original': u'لَدُنْ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ولدنهما'] = {
    'word': u'ولدنهما',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف:معطوف:مضاف',
    'vocalized': u'وَلَدُنْهُمَا',
    'stem': u'لَدُنْ',
    'type': u'STOPWORD',
    'original': u'لَدُنْ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فلدني'] = {
    'word': u'فلدني',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف:معطوف:مضاف',
    'vocalized': u'فَلَدُنِْي',
    'stem': u'لَدُنْ',
    'type': u'STOPWORD',
    'original': u'لَدُنْ',
    'encletic': u'ِي'
}
STOPWORDS[u'فلدنك'] = {
    'word': u'فلدنك',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف:معطوف:مضاف',
    'vocalized': u'فَلَدُنْكَ',
    'stem': u'لَدُنْ',
    'type': u'STOPWORD',
    'original': u'لَدُنْ',
    'encletic': u'كَ'
}
STOPWORDS[u'فلدنك'] = {
    'word': u'فلدنك',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف:معطوف:مضاف',
    'vocalized': u'فَلَدُنْكِ',
    'stem': u'لَدُنْ',
    'type': u'STOPWORD',
    'original': u'لَدُنْ',
    'encletic': u'كِ'
}
STOPWORDS[u'فلدنه'] = {
    'word': u'فلدنه',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف:معطوف:مضاف',
    'vocalized': u'فَلَدُنْه',
    'stem': u'لَدُنْ',
    'type': u'STOPWORD',
    'original': u'لَدُنْ',
    'encletic': u'ه'
}
STOPWORDS[u'فلدنكم'] = {
    'word': u'فلدنكم',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف:معطوف:مضاف',
    'vocalized': u'فَلَدُنْكُمْ',
    'stem': u'لَدُنْ',
    'type': u'STOPWORD',
    'original': u'لَدُنْ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فلدنكن'] = {
    'word': u'فلدنكن',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف:معطوف:مضاف',
    'vocalized': u'فَلَدُنْكُنَّ',
    'stem': u'لَدُنْ',
    'type': u'STOPWORD',
    'original': u'لَدُنْ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فلدنها'] = {
    'word': u'فلدنها',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف:معطوف:مضاف',
    'vocalized': u'فَلَدُنْهَا',
    'stem': u'لَدُنْ',
    'type': u'STOPWORD',
    'original': u'لَدُنْ',
    'encletic': u'هَا'
}
STOPWORDS[u'فلدنهم'] = {
    'word': u'فلدنهم',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف:معطوف:مضاف',
    'vocalized': u'فَلَدُنْهُمْ',
    'stem': u'لَدُنْ',
    'type': u'STOPWORD',
    'original': u'لَدُنْ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فلدنهن'] = {
    'word': u'فلدنهن',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف:معطوف:مضاف',
    'vocalized': u'فَلَدُنْهُنَّ',
    'stem': u'لَدُنْ',
    'type': u'STOPWORD',
    'original': u'لَدُنْ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فلدننا'] = {
    'word': u'فلدننا',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف:معطوف:مضاف',
    'vocalized': u'فَلَدُنْنَا',
    'stem': u'لَدُنْ',
    'type': u'STOPWORD',
    'original': u'لَدُنْ',
    'encletic': u'نَا'
}
STOPWORDS[u'فلدنكما'] = {
    'word': u'فلدنكما',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف:معطوف:مضاف',
    'vocalized': u'فَلَدُنْكُمَا',
    'stem': u'لَدُنْ',
    'type': u'STOPWORD',
    'original': u'لَدُنْ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فلدنهما'] = {
    'word': u'فلدنهما',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف:معطوف:مضاف',
    'vocalized': u'فَلَدُنْهُمَا',
    'stem': u'لَدُنْ',
    'type': u'STOPWORD',
    'original': u'لَدُنْ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'لدى'] = {
    'word': u'لَدَى',
    'procletic': u'',
    'tags': u'أداة:ظرف',
    'vocalized': u'لَدَى',
    'stem': u'لَدَى',
    'type': u'STOPWORD',
    'original': u'لَدَى',
    'encletic': u''
}
STOPWORDS[u'ولدى'] = {
    'word': u'ولدى',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف:معطوف',
    'vocalized': u'وَلَدَى',
    'stem': u'لَدَى',
    'type': u'STOPWORD',
    'original': u'لَدَى',
    'encletic': u''
}
STOPWORDS[u'فلدى'] = {
    'word': u'فلدى',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف:معطوف',
    'vocalized': u'فَلَدَى',
    'stem': u'لَدَى',
    'type': u'STOPWORD',
    'original': u'لَدَى',
    'encletic': u''
}
STOPWORDS[u'ألدى'] = {
    'word': u'ألدى',
    'procletic': u'-أَ',
    'tags': u'أداة:ظرف:مجرور',
    'vocalized': u'أَلَدَى',
    'stem': u'لَدَى',
    'type': u'STOPWORD',
    'original': u'لَدَى',
    'encletic': u''
}
STOPWORDS[u'أولدى'] = {
    'word': u'أولدى',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ظرف:معطوف:مجرور',
    'vocalized': u'أَوَلَدَى',
    'stem': u'لَدَى',
    'type': u'STOPWORD',
    'original': u'لَدَى',
    'encletic': u''
}
STOPWORDS[u'أفلدى'] = {
    'word': u'أفلدى',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ظرف:معطوف:مجرور',
    'vocalized': u'أَفَلَدَى',
    'stem': u'لَدَى',
    'type': u'STOPWORD',
    'original': u'لَدَى',
    'encletic': u''
}
STOPWORDS[u'لدي'] = {
    'word': u'لدي',
    'procletic': u'',
    'tags': u'أداة:ظرف:مضاف',
    'vocalized': u'لَدَيِي',
    'stem': u'لَدَى',
    'type': u'STOPWORD',
    'original': u'لَدَى',
    'encletic': u'ِي'
}
STOPWORDS[u'لديك'] = {
    'word': u'لديك',
    'procletic': u'',
    'tags': u'أداة:ظرف:مضاف',
    'vocalized': u'لَدَيكَ',
    'stem': u'لَدَى',
    'type': u'STOPWORD',
    'original': u'لَدَى',
    'encletic': u'كَ'
}
STOPWORDS[u'لديك'] = {
    'word': u'لديك',
    'procletic': u'',
    'tags': u'أداة:ظرف:مضاف',
    'vocalized': u'لَدَيكِ',
    'stem': u'لَدَى',
    'type': u'STOPWORD',
    'original': u'لَدَى',
    'encletic': u'كِ'
}
STOPWORDS[u'لديه'] = {
    'word': u'لديه',
    'procletic': u'',
    'tags': u'أداة:ظرف:مضاف',
    'vocalized': u'لَدَيه',
    'stem': u'لَدَى',
    'type': u'STOPWORD',
    'original': u'لَدَى',
    'encletic': u'ه'
}
STOPWORDS[u'لديكم'] = {
    'word': u'لديكم',
    'procletic': u'',
    'tags': u'أداة:ظرف:مضاف',
    'vocalized': u'لَدَيكُمْ',
    'stem': u'لَدَى',
    'type': u'STOPWORD',
    'original': u'لَدَى',
    'encletic': u'كُمْ'
}
STOPWORDS[u'لديكن'] = {
    'word': u'لديكن',
    'procletic': u'',
    'tags': u'أداة:ظرف:مضاف',
    'vocalized': u'لَدَيكُنَّ',
    'stem': u'لَدَى',
    'type': u'STOPWORD',
    'original': u'لَدَى',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'لديها'] = {
    'word': u'لديها',
    'procletic': u'',
    'tags': u'أداة:ظرف:مضاف',
    'vocalized': u'لَدَيهَا',
    'stem': u'لَدَى',
    'type': u'STOPWORD',
    'original': u'لَدَى',
    'encletic': u'هَا'
}
STOPWORDS[u'لديهم'] = {
    'word': u'لديهم',
    'procletic': u'',
    'tags': u'أداة:ظرف:مضاف',
    'vocalized': u'لَدَيهُمْ',
    'stem': u'لَدَى',
    'type': u'STOPWORD',
    'original': u'لَدَى',
    'encletic': u'هُمْ'
}
STOPWORDS[u'لديهن'] = {
    'word': u'لديهن',
    'procletic': u'',
    'tags': u'أداة:ظرف:مضاف',
    'vocalized': u'لَدَيهُنَّ',
    'stem': u'لَدَى',
    'type': u'STOPWORD',
    'original': u'لَدَى',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'لدينا'] = {
    'word': u'لدينا',
    'procletic': u'',
    'tags': u'أداة:ظرف:مضاف',
    'vocalized': u'لَدَينَا',
    'stem': u'لَدَى',
    'type': u'STOPWORD',
    'original': u'لَدَى',
    'encletic': u'نَا'
}
STOPWORDS[u'لديكما'] = {
    'word': u'لديكما',
    'procletic': u'',
    'tags': u'أداة:ظرف:مضاف',
    'vocalized': u'لَدَيكُمَا',
    'stem': u'لَدَى',
    'type': u'STOPWORD',
    'original': u'لَدَى',
    'encletic': u'كُمَا'
}
STOPWORDS[u'لديهما'] = {
    'word': u'لديهما',
    'procletic': u'',
    'tags': u'أداة:ظرف:مضاف',
    'vocalized': u'لَدَيهُمَا',
    'stem': u'لَدَى',
    'type': u'STOPWORD',
    'original': u'لَدَى',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ولدي'] = {
    'word': u'ولدي',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف:معطوف:مضاف',
    'vocalized': u'وَلَدَيِي',
    'stem': u'لَدَى',
    'type': u'STOPWORD',
    'original': u'لَدَى',
    'encletic': u'ِي'
}
STOPWORDS[u'ولديك'] = {
    'word': u'ولديك',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف:معطوف:مضاف',
    'vocalized': u'وَلَدَيكَ',
    'stem': u'لَدَى',
    'type': u'STOPWORD',
    'original': u'لَدَى',
    'encletic': u'كَ'
}
STOPWORDS[u'ولديك'] = {
    'word': u'ولديك',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف:معطوف:مضاف',
    'vocalized': u'وَلَدَيكِ',
    'stem': u'لَدَى',
    'type': u'STOPWORD',
    'original': u'لَدَى',
    'encletic': u'كِ'
}
STOPWORDS[u'ولديه'] = {
    'word': u'ولديه',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف:معطوف:مضاف',
    'vocalized': u'وَلَدَيه',
    'stem': u'لَدَى',
    'type': u'STOPWORD',
    'original': u'لَدَى',
    'encletic': u'ه'
}
STOPWORDS[u'ولديكم'] = {
    'word': u'ولديكم',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف:معطوف:مضاف',
    'vocalized': u'وَلَدَيكُمْ',
    'stem': u'لَدَى',
    'type': u'STOPWORD',
    'original': u'لَدَى',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ولديكن'] = {
    'word': u'ولديكن',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف:معطوف:مضاف',
    'vocalized': u'وَلَدَيكُنَّ',
    'stem': u'لَدَى',
    'type': u'STOPWORD',
    'original': u'لَدَى',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ولديها'] = {
    'word': u'ولديها',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف:معطوف:مضاف',
    'vocalized': u'وَلَدَيهَا',
    'stem': u'لَدَى',
    'type': u'STOPWORD',
    'original': u'لَدَى',
    'encletic': u'هَا'
}
STOPWORDS[u'ولديهم'] = {
    'word': u'ولديهم',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف:معطوف:مضاف',
    'vocalized': u'وَلَدَيهُمْ',
    'stem': u'لَدَى',
    'type': u'STOPWORD',
    'original': u'لَدَى',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ولديهن'] = {
    'word': u'ولديهن',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف:معطوف:مضاف',
    'vocalized': u'وَلَدَيهُنَّ',
    'stem': u'لَدَى',
    'type': u'STOPWORD',
    'original': u'لَدَى',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ولدينا'] = {
    'word': u'ولدينا',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف:معطوف:مضاف',
    'vocalized': u'وَلَدَينَا',
    'stem': u'لَدَى',
    'type': u'STOPWORD',
    'original': u'لَدَى',
    'encletic': u'نَا'
}
STOPWORDS[u'ولديكما'] = {
    'word': u'ولديكما',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف:معطوف:مضاف',
    'vocalized': u'وَلَدَيكُمَا',
    'stem': u'لَدَى',
    'type': u'STOPWORD',
    'original': u'لَدَى',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ولديهما'] = {
    'word': u'ولديهما',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف:معطوف:مضاف',
    'vocalized': u'وَلَدَيهُمَا',
    'stem': u'لَدَى',
    'type': u'STOPWORD',
    'original': u'لَدَى',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فلدي'] = {
    'word': u'فلدي',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف:معطوف:مضاف',
    'vocalized': u'فَلَدَيِي',
    'stem': u'لَدَى',
    'type': u'STOPWORD',
    'original': u'لَدَى',
    'encletic': u'ِي'
}
STOPWORDS[u'فلديك'] = {
    'word': u'فلديك',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف:معطوف:مضاف',
    'vocalized': u'فَلَدَيكَ',
    'stem': u'لَدَى',
    'type': u'STOPWORD',
    'original': u'لَدَى',
    'encletic': u'كَ'
}
STOPWORDS[u'فلديك'] = {
    'word': u'فلديك',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف:معطوف:مضاف',
    'vocalized': u'فَلَدَيكِ',
    'stem': u'لَدَى',
    'type': u'STOPWORD',
    'original': u'لَدَى',
    'encletic': u'كِ'
}
STOPWORDS[u'فلديه'] = {
    'word': u'فلديه',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف:معطوف:مضاف',
    'vocalized': u'فَلَدَيه',
    'stem': u'لَدَى',
    'type': u'STOPWORD',
    'original': u'لَدَى',
    'encletic': u'ه'
}
STOPWORDS[u'فلديكم'] = {
    'word': u'فلديكم',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف:معطوف:مضاف',
    'vocalized': u'فَلَدَيكُمْ',
    'stem': u'لَدَى',
    'type': u'STOPWORD',
    'original': u'لَدَى',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فلديكن'] = {
    'word': u'فلديكن',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف:معطوف:مضاف',
    'vocalized': u'فَلَدَيكُنَّ',
    'stem': u'لَدَى',
    'type': u'STOPWORD',
    'original': u'لَدَى',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فلديها'] = {
    'word': u'فلديها',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف:معطوف:مضاف',
    'vocalized': u'فَلَدَيهَا',
    'stem': u'لَدَى',
    'type': u'STOPWORD',
    'original': u'لَدَى',
    'encletic': u'هَا'
}
STOPWORDS[u'فلديهم'] = {
    'word': u'فلديهم',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف:معطوف:مضاف',
    'vocalized': u'فَلَدَيهُمْ',
    'stem': u'لَدَى',
    'type': u'STOPWORD',
    'original': u'لَدَى',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فلديهن'] = {
    'word': u'فلديهن',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف:معطوف:مضاف',
    'vocalized': u'فَلَدَيهُنَّ',
    'stem': u'لَدَى',
    'type': u'STOPWORD',
    'original': u'لَدَى',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فلدينا'] = {
    'word': u'فلدينا',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف:معطوف:مضاف',
    'vocalized': u'فَلَدَينَا',
    'stem': u'لَدَى',
    'type': u'STOPWORD',
    'original': u'لَدَى',
    'encletic': u'نَا'
}
STOPWORDS[u'فلديكما'] = {
    'word': u'فلديكما',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف:معطوف:مضاف',
    'vocalized': u'فَلَدَيكُمَا',
    'stem': u'لَدَى',
    'type': u'STOPWORD',
    'original': u'لَدَى',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فلديهما'] = {
    'word': u'فلديهما',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف:معطوف:مضاف',
    'vocalized': u'فَلَدَيهُمَا',
    'stem': u'لَدَى',
    'type': u'STOPWORD',
    'original': u'لَدَى',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ألدي'] = {
    'word': u'ألدي',
    'procletic': u'-أَ',
    'tags': u'أداة:ظرف:مجرور:مضاف',
    'vocalized': u'أَلَدَيِي',
    'stem': u'لَدَى',
    'type': u'STOPWORD',
    'original': u'لَدَى',
    'encletic': u'ِي'
}
STOPWORDS[u'ألديك'] = {
    'word': u'ألديك',
    'procletic': u'-أَ',
    'tags': u'أداة:ظرف:مجرور:مضاف',
    'vocalized': u'أَلَدَيكَ',
    'stem': u'لَدَى',
    'type': u'STOPWORD',
    'original': u'لَدَى',
    'encletic': u'كَ'
}
STOPWORDS[u'ألديك'] = {
    'word': u'ألديك',
    'procletic': u'-أَ',
    'tags': u'أداة:ظرف:مجرور:مضاف',
    'vocalized': u'أَلَدَيكِ',
    'stem': u'لَدَى',
    'type': u'STOPWORD',
    'original': u'لَدَى',
    'encletic': u'كِ'
}
STOPWORDS[u'ألديه'] = {
    'word': u'ألديه',
    'procletic': u'-أَ',
    'tags': u'أداة:ظرف:مجرور:مضاف',
    'vocalized': u'أَلَدَيه',
    'stem': u'لَدَى',
    'type': u'STOPWORD',
    'original': u'لَدَى',
    'encletic': u'ه'
}
STOPWORDS[u'ألديكم'] = {
    'word': u'ألديكم',
    'procletic': u'-أَ',
    'tags': u'أداة:ظرف:مجرور:مضاف',
    'vocalized': u'أَلَدَيكُمْ',
    'stem': u'لَدَى',
    'type': u'STOPWORD',
    'original': u'لَدَى',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ألديكن'] = {
    'word': u'ألديكن',
    'procletic': u'-أَ',
    'tags': u'أداة:ظرف:مجرور:مضاف',
    'vocalized': u'أَلَدَيكُنَّ',
    'stem': u'لَدَى',
    'type': u'STOPWORD',
    'original': u'لَدَى',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ألديها'] = {
    'word': u'ألديها',
    'procletic': u'-أَ',
    'tags': u'أداة:ظرف:مجرور:مضاف',
    'vocalized': u'أَلَدَيهَا',
    'stem': u'لَدَى',
    'type': u'STOPWORD',
    'original': u'لَدَى',
    'encletic': u'هَا'
}
STOPWORDS[u'ألديهم'] = {
    'word': u'ألديهم',
    'procletic': u'-أَ',
    'tags': u'أداة:ظرف:مجرور:مضاف',
    'vocalized': u'أَلَدَيهُمْ',
    'stem': u'لَدَى',
    'type': u'STOPWORD',
    'original': u'لَدَى',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ألديهن'] = {
    'word': u'ألديهن',
    'procletic': u'-أَ',
    'tags': u'أداة:ظرف:مجرور:مضاف',
    'vocalized': u'أَلَدَيهُنَّ',
    'stem': u'لَدَى',
    'type': u'STOPWORD',
    'original': u'لَدَى',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ألدينا'] = {
    'word': u'ألدينا',
    'procletic': u'-أَ',
    'tags': u'أداة:ظرف:مجرور:مضاف',
    'vocalized': u'أَلَدَينَا',
    'stem': u'لَدَى',
    'type': u'STOPWORD',
    'original': u'لَدَى',
    'encletic': u'نَا'
}
STOPWORDS[u'ألديكما'] = {
    'word': u'ألديكما',
    'procletic': u'-أَ',
    'tags': u'أداة:ظرف:مجرور:مضاف',
    'vocalized': u'أَلَدَيكُمَا',
    'stem': u'لَدَى',
    'type': u'STOPWORD',
    'original': u'لَدَى',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ألديهما'] = {
    'word': u'ألديهما',
    'procletic': u'-أَ',
    'tags': u'أداة:ظرف:مجرور:مضاف',
    'vocalized': u'أَلَدَيهُمَا',
    'stem': u'لَدَى',
    'type': u'STOPWORD',
    'original': u'لَدَى',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أولدي'] = {
    'word': u'أولدي',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ظرف:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَدَيِي',
    'stem': u'لَدَى',
    'type': u'STOPWORD',
    'original': u'لَدَى',
    'encletic': u'ِي'
}
STOPWORDS[u'أولديك'] = {
    'word': u'أولديك',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ظرف:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَدَيكَ',
    'stem': u'لَدَى',
    'type': u'STOPWORD',
    'original': u'لَدَى',
    'encletic': u'كَ'
}
STOPWORDS[u'أولديك'] = {
    'word': u'أولديك',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ظرف:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَدَيكِ',
    'stem': u'لَدَى',
    'type': u'STOPWORD',
    'original': u'لَدَى',
    'encletic': u'كِ'
}
STOPWORDS[u'أولديه'] = {
    'word': u'أولديه',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ظرف:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَدَيه',
    'stem': u'لَدَى',
    'type': u'STOPWORD',
    'original': u'لَدَى',
    'encletic': u'ه'
}
STOPWORDS[u'أولديكم'] = {
    'word': u'أولديكم',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ظرف:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَدَيكُمْ',
    'stem': u'لَدَى',
    'type': u'STOPWORD',
    'original': u'لَدَى',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أولديكن'] = {
    'word': u'أولديكن',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ظرف:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَدَيكُنَّ',
    'stem': u'لَدَى',
    'type': u'STOPWORD',
    'original': u'لَدَى',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أولديها'] = {
    'word': u'أولديها',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ظرف:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَدَيهَا',
    'stem': u'لَدَى',
    'type': u'STOPWORD',
    'original': u'لَدَى',
    'encletic': u'هَا'
}
STOPWORDS[u'أولديهم'] = {
    'word': u'أولديهم',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ظرف:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَدَيهُمْ',
    'stem': u'لَدَى',
    'type': u'STOPWORD',
    'original': u'لَدَى',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أولديهن'] = {
    'word': u'أولديهن',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ظرف:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَدَيهُنَّ',
    'stem': u'لَدَى',
    'type': u'STOPWORD',
    'original': u'لَدَى',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أولدينا'] = {
    'word': u'أولدينا',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ظرف:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَدَينَا',
    'stem': u'لَدَى',
    'type': u'STOPWORD',
    'original': u'لَدَى',
    'encletic': u'نَا'
}
STOPWORDS[u'أولديكما'] = {
    'word': u'أولديكما',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ظرف:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَدَيكُمَا',
    'stem': u'لَدَى',
    'type': u'STOPWORD',
    'original': u'لَدَى',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أولديهما'] = {
    'word': u'أولديهما',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ظرف:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَدَيهُمَا',
    'stem': u'لَدَى',
    'type': u'STOPWORD',
    'original': u'لَدَى',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أفلدي'] = {
    'word': u'أفلدي',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ظرف:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَدَيِي',
    'stem': u'لَدَى',
    'type': u'STOPWORD',
    'original': u'لَدَى',
    'encletic': u'ِي'
}
STOPWORDS[u'أفلديك'] = {
    'word': u'أفلديك',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ظرف:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَدَيكَ',
    'stem': u'لَدَى',
    'type': u'STOPWORD',
    'original': u'لَدَى',
    'encletic': u'كَ'
}
STOPWORDS[u'أفلديك'] = {
    'word': u'أفلديك',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ظرف:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَدَيكِ',
    'stem': u'لَدَى',
    'type': u'STOPWORD',
    'original': u'لَدَى',
    'encletic': u'كِ'
}
STOPWORDS[u'أفلديه'] = {
    'word': u'أفلديه',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ظرف:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَدَيه',
    'stem': u'لَدَى',
    'type': u'STOPWORD',
    'original': u'لَدَى',
    'encletic': u'ه'
}
STOPWORDS[u'أفلديكم'] = {
    'word': u'أفلديكم',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ظرف:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَدَيكُمْ',
    'stem': u'لَدَى',
    'type': u'STOPWORD',
    'original': u'لَدَى',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أفلديكن'] = {
    'word': u'أفلديكن',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ظرف:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَدَيكُنَّ',
    'stem': u'لَدَى',
    'type': u'STOPWORD',
    'original': u'لَدَى',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أفلديها'] = {
    'word': u'أفلديها',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ظرف:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَدَيهَا',
    'stem': u'لَدَى',
    'type': u'STOPWORD',
    'original': u'لَدَى',
    'encletic': u'هَا'
}
STOPWORDS[u'أفلديهم'] = {
    'word': u'أفلديهم',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ظرف:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَدَيهُمْ',
    'stem': u'لَدَى',
    'type': u'STOPWORD',
    'original': u'لَدَى',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أفلديهن'] = {
    'word': u'أفلديهن',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ظرف:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَدَيهُنَّ',
    'stem': u'لَدَى',
    'type': u'STOPWORD',
    'original': u'لَدَى',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أفلدينا'] = {
    'word': u'أفلدينا',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ظرف:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَدَينَا',
    'stem': u'لَدَى',
    'type': u'STOPWORD',
    'original': u'لَدَى',
    'encletic': u'نَا'
}
STOPWORDS[u'أفلديكما'] = {
    'word': u'أفلديكما',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ظرف:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَدَيكُمَا',
    'stem': u'لَدَى',
    'type': u'STOPWORD',
    'original': u'لَدَى',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أفلديهما'] = {
    'word': u'أفلديهما',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ظرف:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَدَيهُمَا',
    'stem': u'لَدَى',
    'type': u'STOPWORD',
    'original': u'لَدَى',
    'encletic': u'هُمَا'
}
STOPWORDS[u'لما'] = {
    'word': u'لَمَّا',
    'procletic': u'',
    'tags': u'أداة:ظرف',
    'vocalized': u'لَمَّا',
    'stem': u'لَمَّا',
    'type': u'STOPWORD',
    'original': u'لَمَّا',
    'encletic': u''
}
STOPWORDS[u'ولما'] = {
    'word': u'ولما',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف:معطوف',
    'vocalized': u'وَلَمَّا',
    'stem': u'لَمَّا',
    'type': u'STOPWORD',
    'original': u'لَمَّا',
    'encletic': u''
}
STOPWORDS[u'فلما'] = {
    'word': u'فلما',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف:معطوف',
    'vocalized': u'فَلَمَّا',
    'stem': u'لَمَّا',
    'type': u'STOPWORD',
    'original': u'لَمَّا',
    'encletic': u''
}
STOPWORDS[u'ألما'] = {
    'word': u'ألما',
    'procletic': u'-أَ',
    'tags': u'أداة:ظرف:مجرور',
    'vocalized': u'أَلَمَّا',
    'stem': u'لَمَّا',
    'type': u'STOPWORD',
    'original': u'لَمَّا',
    'encletic': u''
}
STOPWORDS[u'أولما'] = {
    'word': u'أولما',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ظرف:معطوف:مجرور',
    'vocalized': u'أَوَلَمَّا',
    'stem': u'لَمَّا',
    'type': u'STOPWORD',
    'original': u'لَمَّا',
    'encletic': u''
}
STOPWORDS[u'أفلما'] = {
    'word': u'أفلما',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ظرف:معطوف:مجرور',
    'vocalized': u'أَفَلَمَّا',
    'stem': u'لَمَّا',
    'type': u'STOPWORD',
    'original': u'لَمَّا',
    'encletic': u''
}
STOPWORDS[u'الآن'] = {
    'word': u'الْآنَ',
    'procletic': u'',
    'tags': u'أداة:ظرف زمان',
    'vocalized': u'الْآنَ',
    'stem': u'الْآنَ',
    'type': u'STOPWORD',
    'original': u'الْآنَ',
    'encletic': u''
}
STOPWORDS[u'والآن'] = {
    'word': u'والآن',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف زمان:معطوف',
    'vocalized': u'وَالْآنَ',
    'stem': u'الْآنَ',
    'type': u'STOPWORD',
    'original': u'الْآنَ',
    'encletic': u''
}
STOPWORDS[u'فالآن'] = {
    'word': u'فالآن',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف زمان:معطوف',
    'vocalized': u'فَالْآنَ',
    'stem': u'الْآنَ',
    'type': u'STOPWORD',
    'original': u'الْآنَ',
    'encletic': u''
}
STOPWORDS[u'آناء'] = {
    'word': u'آنَاءَ',
    'procletic': u'',
    'tags': u'أداة:ظرف زمان',
    'vocalized': u'آنَاءَ',
    'stem': u'آنَاءَ',
    'type': u'STOPWORD',
    'original': u'آنَاءَ',
    'encletic': u''
}
STOPWORDS[u'وآناء'] = {
    'word': u'وآناء',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف زمان:معطوف',
    'vocalized': u'وَآنَاءَ',
    'stem': u'آنَاءَ',
    'type': u'STOPWORD',
    'original': u'آنَاءَ',
    'encletic': u''
}
STOPWORDS[u'فآناء'] = {
    'word': u'فآناء',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف زمان:معطوف',
    'vocalized': u'فَآنَاءَ',
    'stem': u'آنَاءَ',
    'type': u'STOPWORD',
    'original': u'آنَاءَ',
    'encletic': u''
}
STOPWORDS[u'آنفا'] = {
    'word': u'آنِفًا',
    'procletic': u'',
    'tags': u'أداة:ظرف زمان',
    'vocalized': u'آنِفًا',
    'stem': u'آنِفًا',
    'type': u'STOPWORD',
    'original': u'آنِفًا',
    'encletic': u''
}
STOPWORDS[u'وآنفا'] = {
    'word': u'وآنفا',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف زمان:معطوف',
    'vocalized': u'وَآنِفًا',
    'stem': u'آنِفًا',
    'type': u'STOPWORD',
    'original': u'آنِفًا',
    'encletic': u''
}
STOPWORDS[u'فآنفا'] = {
    'word': u'فآنفا',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف زمان:معطوف',
    'vocalized': u'فَآنِفًا',
    'stem': u'آنِفًا',
    'type': u'STOPWORD',
    'original': u'آنِفًا',
    'encletic': u''
}
STOPWORDS[u'أبدا'] = {
    'word': u'أَبَدًا',
    'procletic': u'',
    'tags': u'أداة:ظرف زمان',
    'vocalized': u'أَبَدًا',
    'stem': u'أَبَدًا',
    'type': u'STOPWORD',
    'original': u'أَبَدًا',
    'encletic': u''
}
STOPWORDS[u'وأبدا'] = {
    'word': u'وأبدا',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف زمان:معطوف',
    'vocalized': u'وَأَبَدًا',
    'stem': u'أَبَدًا',
    'type': u'STOPWORD',
    'original': u'أَبَدًا',
    'encletic': u''
}
STOPWORDS[u'فأبدا'] = {
    'word': u'فأبدا',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف زمان:معطوف',
    'vocalized': u'فَأَبَدًا',
    'stem': u'أَبَدًا',
    'type': u'STOPWORD',
    'original': u'أَبَدًا',
    'encletic': u''
}
STOPWORDS[u'أصلا'] = {
    'word': u'أَصْلًا',
    'procletic': u'',
    'tags': u'أداة:ظرف زمان',
    'vocalized': u'أَصْلًا',
    'stem': u'أَصْلًا',
    'type': u'STOPWORD',
    'original': u'أَصْلًا',
    'encletic': u''
}
STOPWORDS[u'وأصلا'] = {
    'word': u'وأصلا',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف زمان:معطوف',
    'vocalized': u'وَأَصْلًا',
    'stem': u'أَصْلًا',
    'type': u'STOPWORD',
    'original': u'أَصْلًا',
    'encletic': u''
}
STOPWORDS[u'فأصلا'] = {
    'word': u'فأصلا',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف زمان:معطوف',
    'vocalized': u'فَأَصْلًا',
    'stem': u'أَصْلًا',
    'type': u'STOPWORD',
    'original': u'أَصْلًا',
    'encletic': u''
}
STOPWORDS[u'أمد'] = {
    'word': u'أَمَدَ',
    'procletic': u'',
    'tags': u'أداة:ظرف زمان',
    'vocalized': u'أَمَدَ',
    'stem': u'أَمَدَ',
    'type': u'STOPWORD',
    'original': u'أَمَدَ',
    'encletic': u''
}
STOPWORDS[u'وأمد'] = {
    'word': u'وأمد',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف زمان:معطوف',
    'vocalized': u'وَأَمَدَ',
    'stem': u'أَمَدَ',
    'type': u'STOPWORD',
    'original': u'أَمَدَ',
    'encletic': u''
}
STOPWORDS[u'فأمد'] = {
    'word': u'فأمد',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف زمان:معطوف',
    'vocalized': u'فَأَمَدَ',
    'stem': u'أَمَدَ',
    'type': u'STOPWORD',
    'original': u'أَمَدَ',
    'encletic': u''
}
STOPWORDS[u'أمس'] = {
    'word': u'أَمْسِ',
    'procletic': u'',
    'tags': u'أداة:ظرف زمان',
    'vocalized': u'أَمْسِ',
    'stem': u'أَمْسِ',
    'type': u'STOPWORD',
    'original': u'أَمْسِ',
    'encletic': u''
}
STOPWORDS[u'وأمس'] = {
    'word': u'وأمس',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف زمان:معطوف',
    'vocalized': u'وَأَمْسِ',
    'stem': u'أَمْسِ',
    'type': u'STOPWORD',
    'original': u'أَمْسِ',
    'encletic': u''
}
STOPWORDS[u'فأمس'] = {
    'word': u'فأمس',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف زمان:معطوف',
    'vocalized': u'فَأَمْسِ',
    'stem': u'أَمْسِ',
    'type': u'STOPWORD',
    'original': u'أَمْسِ',
    'encletic': u''
}
STOPWORDS[u'أول'] = {
    'word': u'أَوَّلَ',
    'procletic': u'',
    'tags': u'أداة:ظرف زمان',
    'vocalized': u'أَوَّلَ',
    'stem': u'أَوَّلَ',
    'type': u'STOPWORD',
    'original': u'أَوَّلَ',
    'encletic': u''
}
STOPWORDS[u'وأول'] = {
    'word': u'وأول',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف زمان:معطوف',
    'vocalized': u'وَأَوَّلَ',
    'stem': u'أَوَّلَ',
    'type': u'STOPWORD',
    'original': u'أَوَّلَ',
    'encletic': u''
}
STOPWORDS[u'فأول'] = {
    'word': u'فأول',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف زمان:معطوف',
    'vocalized': u'فَأَوَّلَ',
    'stem': u'أَوَّلَ',
    'type': u'STOPWORD',
    'original': u'أَوَّلَ',
    'encletic': u''
}
STOPWORDS[u'أيان'] = {
    'word': u'أَيَّانَ',
    'procletic': u'',
    'tags': u'أداة:ظرف زمان',
    'vocalized': u'أَيَّانَ',
    'stem': u'أَيَّانَ',
    'type': u'STOPWORD',
    'original': u'أَيَّانَ',
    'encletic': u''
}
STOPWORDS[u'وأيان'] = {
    'word': u'وأيان',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف زمان:معطوف',
    'vocalized': u'وَأَيَّانَ',
    'stem': u'أَيَّانَ',
    'type': u'STOPWORD',
    'original': u'أَيَّانَ',
    'encletic': u''
}
STOPWORDS[u'فأيان'] = {
    'word': u'فأيان',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف زمان:معطوف',
    'vocalized': u'فَأَيَّانَ',
    'stem': u'أَيَّانَ',
    'type': u'STOPWORD',
    'original': u'أَيَّانَ',
    'encletic': u''
}
STOPWORDS[u'إذ'] = {
    'word': u'إِذْ',
    'procletic': u'',
    'tags': u'أداة:ظرف زمان',
    'vocalized': u'إِذْ',
    'stem': u'إِذْ',
    'type': u'STOPWORD',
    'original': u'إِذْ',
    'encletic': u''
}
STOPWORDS[u'وإذ'] = {
    'word': u'وإذ',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف زمان:معطوف',
    'vocalized': u'وَإِذْ',
    'stem': u'إِذْ',
    'type': u'STOPWORD',
    'original': u'إِذْ',
    'encletic': u''
}
STOPWORDS[u'فإذ'] = {
    'word': u'فإذ',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف زمان:معطوف',
    'vocalized': u'فَإِذْ',
    'stem': u'إِذْ',
    'type': u'STOPWORD',
    'original': u'إِذْ',
    'encletic': u''
}
STOPWORDS[u'بعد'] = {
    'word': u'بَعْدَ',
    'procletic': u'',
    'tags': u'أداة:ظرف زمان',
    'vocalized': u'بَعْدَ',
    'stem': u'بَعْدَ',
    'type': u'STOPWORD',
    'original': u'بَعْدَ',
    'encletic': u''
}
STOPWORDS[u'وبعد'] = {
    'word': u'وبعد',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف زمان:معطوف',
    'vocalized': u'وَبَعْدَ',
    'stem': u'بَعْدَ',
    'type': u'STOPWORD',
    'original': u'بَعْدَ',
    'encletic': u''
}
STOPWORDS[u'فبعد'] = {
    'word': u'فبعد',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف زمان:معطوف',
    'vocalized': u'فَبَعْدَ',
    'stem': u'بَعْدَ',
    'type': u'STOPWORD',
    'original': u'بَعْدَ',
    'encletic': u''
}
STOPWORDS[u'بعدي'] = {
    'word': u'بعدي',
    'procletic': u'',
    'tags': u'أداة:ظرف زمان:مضاف',
    'vocalized': u'بَعْدَِي',
    'stem': u'بَعْدَ',
    'type': u'STOPWORD',
    'original': u'بَعْدَ',
    'encletic': u'ِي'
}
STOPWORDS[u'بعدك'] = {
    'word': u'بعدك',
    'procletic': u'',
    'tags': u'أداة:ظرف زمان:مضاف',
    'vocalized': u'بَعْدَكَ',
    'stem': u'بَعْدَ',
    'type': u'STOPWORD',
    'original': u'بَعْدَ',
    'encletic': u'كَ'
}
STOPWORDS[u'بعدك'] = {
    'word': u'بعدك',
    'procletic': u'',
    'tags': u'أداة:ظرف زمان:مضاف',
    'vocalized': u'بَعْدَكِ',
    'stem': u'بَعْدَ',
    'type': u'STOPWORD',
    'original': u'بَعْدَ',
    'encletic': u'كِ'
}
STOPWORDS[u'بعده'] = {
    'word': u'بعده',
    'procletic': u'',
    'tags': u'أداة:ظرف زمان:مضاف',
    'vocalized': u'بَعْدَه',
    'stem': u'بَعْدَ',
    'type': u'STOPWORD',
    'original': u'بَعْدَ',
    'encletic': u'ه'
}
STOPWORDS[u'بعدكم'] = {
    'word': u'بعدكم',
    'procletic': u'',
    'tags': u'أداة:ظرف زمان:مضاف',
    'vocalized': u'بَعْدَكُمْ',
    'stem': u'بَعْدَ',
    'type': u'STOPWORD',
    'original': u'بَعْدَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'بعدكن'] = {
    'word': u'بعدكن',
    'procletic': u'',
    'tags': u'أداة:ظرف زمان:مضاف',
    'vocalized': u'بَعْدَكُنَّ',
    'stem': u'بَعْدَ',
    'type': u'STOPWORD',
    'original': u'بَعْدَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'بعدها'] = {
    'word': u'بعدها',
    'procletic': u'',
    'tags': u'أداة:ظرف زمان:مضاف',
    'vocalized': u'بَعْدَهَا',
    'stem': u'بَعْدَ',
    'type': u'STOPWORD',
    'original': u'بَعْدَ',
    'encletic': u'هَا'
}
STOPWORDS[u'بعدهم'] = {
    'word': u'بعدهم',
    'procletic': u'',
    'tags': u'أداة:ظرف زمان:مضاف',
    'vocalized': u'بَعْدَهُمْ',
    'stem': u'بَعْدَ',
    'type': u'STOPWORD',
    'original': u'بَعْدَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'بعدهن'] = {
    'word': u'بعدهن',
    'procletic': u'',
    'tags': u'أداة:ظرف زمان:مضاف',
    'vocalized': u'بَعْدَهُنَّ',
    'stem': u'بَعْدَ',
    'type': u'STOPWORD',
    'original': u'بَعْدَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'بعدنا'] = {
    'word': u'بعدنا',
    'procletic': u'',
    'tags': u'أداة:ظرف زمان:مضاف',
    'vocalized': u'بَعْدَنَا',
    'stem': u'بَعْدَ',
    'type': u'STOPWORD',
    'original': u'بَعْدَ',
    'encletic': u'نَا'
}
STOPWORDS[u'بعدكما'] = {
    'word': u'بعدكما',
    'procletic': u'',
    'tags': u'أداة:ظرف زمان:مضاف',
    'vocalized': u'بَعْدَكُمَا',
    'stem': u'بَعْدَ',
    'type': u'STOPWORD',
    'original': u'بَعْدَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'بعدهما'] = {
    'word': u'بعدهما',
    'procletic': u'',
    'tags': u'أداة:ظرف زمان:مضاف',
    'vocalized': u'بَعْدَهُمَا',
    'stem': u'بَعْدَ',
    'type': u'STOPWORD',
    'original': u'بَعْدَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'وبعدي'] = {
    'word': u'وبعدي',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف زمان:معطوف:مضاف',
    'vocalized': u'وَبَعْدَِي',
    'stem': u'بَعْدَ',
    'type': u'STOPWORD',
    'original': u'بَعْدَ',
    'encletic': u'ِي'
}
STOPWORDS[u'وبعدك'] = {
    'word': u'وبعدك',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف زمان:معطوف:مضاف',
    'vocalized': u'وَبَعْدَكَ',
    'stem': u'بَعْدَ',
    'type': u'STOPWORD',
    'original': u'بَعْدَ',
    'encletic': u'كَ'
}
STOPWORDS[u'وبعدك'] = {
    'word': u'وبعدك',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف زمان:معطوف:مضاف',
    'vocalized': u'وَبَعْدَكِ',
    'stem': u'بَعْدَ',
    'type': u'STOPWORD',
    'original': u'بَعْدَ',
    'encletic': u'كِ'
}
STOPWORDS[u'وبعده'] = {
    'word': u'وبعده',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف زمان:معطوف:مضاف',
    'vocalized': u'وَبَعْدَه',
    'stem': u'بَعْدَ',
    'type': u'STOPWORD',
    'original': u'بَعْدَ',
    'encletic': u'ه'
}
STOPWORDS[u'وبعدكم'] = {
    'word': u'وبعدكم',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف زمان:معطوف:مضاف',
    'vocalized': u'وَبَعْدَكُمْ',
    'stem': u'بَعْدَ',
    'type': u'STOPWORD',
    'original': u'بَعْدَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'وبعدكن'] = {
    'word': u'وبعدكن',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف زمان:معطوف:مضاف',
    'vocalized': u'وَبَعْدَكُنَّ',
    'stem': u'بَعْدَ',
    'type': u'STOPWORD',
    'original': u'بَعْدَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'وبعدها'] = {
    'word': u'وبعدها',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف زمان:معطوف:مضاف',
    'vocalized': u'وَبَعْدَهَا',
    'stem': u'بَعْدَ',
    'type': u'STOPWORD',
    'original': u'بَعْدَ',
    'encletic': u'هَا'
}
STOPWORDS[u'وبعدهم'] = {
    'word': u'وبعدهم',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف زمان:معطوف:مضاف',
    'vocalized': u'وَبَعْدَهُمْ',
    'stem': u'بَعْدَ',
    'type': u'STOPWORD',
    'original': u'بَعْدَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'وبعدهن'] = {
    'word': u'وبعدهن',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف زمان:معطوف:مضاف',
    'vocalized': u'وَبَعْدَهُنَّ',
    'stem': u'بَعْدَ',
    'type': u'STOPWORD',
    'original': u'بَعْدَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'وبعدنا'] = {
    'word': u'وبعدنا',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف زمان:معطوف:مضاف',
    'vocalized': u'وَبَعْدَنَا',
    'stem': u'بَعْدَ',
    'type': u'STOPWORD',
    'original': u'بَعْدَ',
    'encletic': u'نَا'
}
STOPWORDS[u'وبعدكما'] = {
    'word': u'وبعدكما',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف زمان:معطوف:مضاف',
    'vocalized': u'وَبَعْدَكُمَا',
    'stem': u'بَعْدَ',
    'type': u'STOPWORD',
    'original': u'بَعْدَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'وبعدهما'] = {
    'word': u'وبعدهما',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف زمان:معطوف:مضاف',
    'vocalized': u'وَبَعْدَهُمَا',
    'stem': u'بَعْدَ',
    'type': u'STOPWORD',
    'original': u'بَعْدَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فبعدي'] = {
    'word': u'فبعدي',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف زمان:معطوف:مضاف',
    'vocalized': u'فَبَعْدَِي',
    'stem': u'بَعْدَ',
    'type': u'STOPWORD',
    'original': u'بَعْدَ',
    'encletic': u'ِي'
}
STOPWORDS[u'فبعدك'] = {
    'word': u'فبعدك',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف زمان:معطوف:مضاف',
    'vocalized': u'فَبَعْدَكَ',
    'stem': u'بَعْدَ',
    'type': u'STOPWORD',
    'original': u'بَعْدَ',
    'encletic': u'كَ'
}
STOPWORDS[u'فبعدك'] = {
    'word': u'فبعدك',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف زمان:معطوف:مضاف',
    'vocalized': u'فَبَعْدَكِ',
    'stem': u'بَعْدَ',
    'type': u'STOPWORD',
    'original': u'بَعْدَ',
    'encletic': u'كِ'
}
STOPWORDS[u'فبعده'] = {
    'word': u'فبعده',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف زمان:معطوف:مضاف',
    'vocalized': u'فَبَعْدَه',
    'stem': u'بَعْدَ',
    'type': u'STOPWORD',
    'original': u'بَعْدَ',
    'encletic': u'ه'
}
STOPWORDS[u'فبعدكم'] = {
    'word': u'فبعدكم',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف زمان:معطوف:مضاف',
    'vocalized': u'فَبَعْدَكُمْ',
    'stem': u'بَعْدَ',
    'type': u'STOPWORD',
    'original': u'بَعْدَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فبعدكن'] = {
    'word': u'فبعدكن',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف زمان:معطوف:مضاف',
    'vocalized': u'فَبَعْدَكُنَّ',
    'stem': u'بَعْدَ',
    'type': u'STOPWORD',
    'original': u'بَعْدَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فبعدها'] = {
    'word': u'فبعدها',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف زمان:معطوف:مضاف',
    'vocalized': u'فَبَعْدَهَا',
    'stem': u'بَعْدَ',
    'type': u'STOPWORD',
    'original': u'بَعْدَ',
    'encletic': u'هَا'
}
STOPWORDS[u'فبعدهم'] = {
    'word': u'فبعدهم',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف زمان:معطوف:مضاف',
    'vocalized': u'فَبَعْدَهُمْ',
    'stem': u'بَعْدَ',
    'type': u'STOPWORD',
    'original': u'بَعْدَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فبعدهن'] = {
    'word': u'فبعدهن',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف زمان:معطوف:مضاف',
    'vocalized': u'فَبَعْدَهُنَّ',
    'stem': u'بَعْدَ',
    'type': u'STOPWORD',
    'original': u'بَعْدَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فبعدنا'] = {
    'word': u'فبعدنا',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف زمان:معطوف:مضاف',
    'vocalized': u'فَبَعْدَنَا',
    'stem': u'بَعْدَ',
    'type': u'STOPWORD',
    'original': u'بَعْدَ',
    'encletic': u'نَا'
}
STOPWORDS[u'فبعدكما'] = {
    'word': u'فبعدكما',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف زمان:معطوف:مضاف',
    'vocalized': u'فَبَعْدَكُمَا',
    'stem': u'بَعْدَ',
    'type': u'STOPWORD',
    'original': u'بَعْدَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فبعدهما'] = {
    'word': u'فبعدهما',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف زمان:معطوف:مضاف',
    'vocalized': u'فَبَعْدَهُمَا',
    'stem': u'بَعْدَ',
    'type': u'STOPWORD',
    'original': u'بَعْدَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'تارة'] = {
    'word': u'تَارَةً',
    'procletic': u'',
    'tags': u'أداة:ظرف زمان',
    'vocalized': u'تَارَةً',
    'stem': u'تَارَةً',
    'type': u'STOPWORD',
    'original': u'تَارَةً',
    'encletic': u''
}
STOPWORDS[u'وتارة'] = {
    'word': u'وتارة',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف زمان:معطوف',
    'vocalized': u'وَتَارَةً',
    'stem': u'تَارَةً',
    'type': u'STOPWORD',
    'original': u'تَارَةً',
    'encletic': u''
}
STOPWORDS[u'فتارة'] = {
    'word': u'فتارة',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف زمان:معطوف',
    'vocalized': u'فَتَارَةً',
    'stem': u'تَارَةً',
    'type': u'STOPWORD',
    'original': u'تَارَةً',
    'encletic': u''
}
STOPWORDS[u'حين'] = {
    'word': u'حِينَ',
    'procletic': u'',
    'tags': u'أداة:ظرف زمان',
    'vocalized': u'حِينَ',
    'stem': u'حِينَ',
    'type': u'STOPWORD',
    'original': u'حِينَ',
    'encletic': u''
}
STOPWORDS[u'وحين'] = {
    'word': u'وحين',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف زمان:معطوف',
    'vocalized': u'وَحِينَ',
    'stem': u'حِينَ',
    'type': u'STOPWORD',
    'original': u'حِينَ',
    'encletic': u''
}
STOPWORDS[u'فحين'] = {
    'word': u'فحين',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف زمان:معطوف',
    'vocalized': u'فَحِينَ',
    'stem': u'حِينَ',
    'type': u'STOPWORD',
    'original': u'حِينَ',
    'encletic': u''
}
STOPWORDS[u'حيني'] = {
    'word': u'حيني',
    'procletic': u'',
    'tags': u'أداة:ظرف زمان:مضاف',
    'vocalized': u'حِينَِي',
    'stem': u'حِينَ',
    'type': u'STOPWORD',
    'original': u'حِينَ',
    'encletic': u'ِي'
}
STOPWORDS[u'حينك'] = {
    'word': u'حينك',
    'procletic': u'',
    'tags': u'أداة:ظرف زمان:مضاف',
    'vocalized': u'حِينَكَ',
    'stem': u'حِينَ',
    'type': u'STOPWORD',
    'original': u'حِينَ',
    'encletic': u'كَ'
}
STOPWORDS[u'حينك'] = {
    'word': u'حينك',
    'procletic': u'',
    'tags': u'أداة:ظرف زمان:مضاف',
    'vocalized': u'حِينَكِ',
    'stem': u'حِينَ',
    'type': u'STOPWORD',
    'original': u'حِينَ',
    'encletic': u'كِ'
}
STOPWORDS[u'حينه'] = {
    'word': u'حينه',
    'procletic': u'',
    'tags': u'أداة:ظرف زمان:مضاف',
    'vocalized': u'حِينَه',
    'stem': u'حِينَ',
    'type': u'STOPWORD',
    'original': u'حِينَ',
    'encletic': u'ه'
}
STOPWORDS[u'حينكم'] = {
    'word': u'حينكم',
    'procletic': u'',
    'tags': u'أداة:ظرف زمان:مضاف',
    'vocalized': u'حِينَكُمْ',
    'stem': u'حِينَ',
    'type': u'STOPWORD',
    'original': u'حِينَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'حينكن'] = {
    'word': u'حينكن',
    'procletic': u'',
    'tags': u'أداة:ظرف زمان:مضاف',
    'vocalized': u'حِينَكُنَّ',
    'stem': u'حِينَ',
    'type': u'STOPWORD',
    'original': u'حِينَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'حينها'] = {
    'word': u'حينها',
    'procletic': u'',
    'tags': u'أداة:ظرف زمان:مضاف',
    'vocalized': u'حِينَهَا',
    'stem': u'حِينَ',
    'type': u'STOPWORD',
    'original': u'حِينَ',
    'encletic': u'هَا'
}
STOPWORDS[u'حينهم'] = {
    'word': u'حينهم',
    'procletic': u'',
    'tags': u'أداة:ظرف زمان:مضاف',
    'vocalized': u'حِينَهُمْ',
    'stem': u'حِينَ',
    'type': u'STOPWORD',
    'original': u'حِينَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'حينهن'] = {
    'word': u'حينهن',
    'procletic': u'',
    'tags': u'أداة:ظرف زمان:مضاف',
    'vocalized': u'حِينَهُنَّ',
    'stem': u'حِينَ',
    'type': u'STOPWORD',
    'original': u'حِينَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'حيننا'] = {
    'word': u'حيننا',
    'procletic': u'',
    'tags': u'أداة:ظرف زمان:مضاف',
    'vocalized': u'حِينَنَا',
    'stem': u'حِينَ',
    'type': u'STOPWORD',
    'original': u'حِينَ',
    'encletic': u'نَا'
}
STOPWORDS[u'حينكما'] = {
    'word': u'حينكما',
    'procletic': u'',
    'tags': u'أداة:ظرف زمان:مضاف',
    'vocalized': u'حِينَكُمَا',
    'stem': u'حِينَ',
    'type': u'STOPWORD',
    'original': u'حِينَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'حينهما'] = {
    'word': u'حينهما',
    'procletic': u'',
    'tags': u'أداة:ظرف زمان:مضاف',
    'vocalized': u'حِينَهُمَا',
    'stem': u'حِينَ',
    'type': u'STOPWORD',
    'original': u'حِينَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'وحيني'] = {
    'word': u'وحيني',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف زمان:معطوف:مضاف',
    'vocalized': u'وَحِينَِي',
    'stem': u'حِينَ',
    'type': u'STOPWORD',
    'original': u'حِينَ',
    'encletic': u'ِي'
}
STOPWORDS[u'وحينك'] = {
    'word': u'وحينك',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف زمان:معطوف:مضاف',
    'vocalized': u'وَحِينَكَ',
    'stem': u'حِينَ',
    'type': u'STOPWORD',
    'original': u'حِينَ',
    'encletic': u'كَ'
}
STOPWORDS[u'وحينك'] = {
    'word': u'وحينك',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف زمان:معطوف:مضاف',
    'vocalized': u'وَحِينَكِ',
    'stem': u'حِينَ',
    'type': u'STOPWORD',
    'original': u'حِينَ',
    'encletic': u'كِ'
}
STOPWORDS[u'وحينه'] = {
    'word': u'وحينه',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف زمان:معطوف:مضاف',
    'vocalized': u'وَحِينَه',
    'stem': u'حِينَ',
    'type': u'STOPWORD',
    'original': u'حِينَ',
    'encletic': u'ه'
}
STOPWORDS[u'وحينكم'] = {
    'word': u'وحينكم',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف زمان:معطوف:مضاف',
    'vocalized': u'وَحِينَكُمْ',
    'stem': u'حِينَ',
    'type': u'STOPWORD',
    'original': u'حِينَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'وحينكن'] = {
    'word': u'وحينكن',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف زمان:معطوف:مضاف',
    'vocalized': u'وَحِينَكُنَّ',
    'stem': u'حِينَ',
    'type': u'STOPWORD',
    'original': u'حِينَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'وحينها'] = {
    'word': u'وحينها',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف زمان:معطوف:مضاف',
    'vocalized': u'وَحِينَهَا',
    'stem': u'حِينَ',
    'type': u'STOPWORD',
    'original': u'حِينَ',
    'encletic': u'هَا'
}
STOPWORDS[u'وحينهم'] = {
    'word': u'وحينهم',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف زمان:معطوف:مضاف',
    'vocalized': u'وَحِينَهُمْ',
    'stem': u'حِينَ',
    'type': u'STOPWORD',
    'original': u'حِينَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'وحينهن'] = {
    'word': u'وحينهن',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف زمان:معطوف:مضاف',
    'vocalized': u'وَحِينَهُنَّ',
    'stem': u'حِينَ',
    'type': u'STOPWORD',
    'original': u'حِينَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'وحيننا'] = {
    'word': u'وحيننا',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف زمان:معطوف:مضاف',
    'vocalized': u'وَحِينَنَا',
    'stem': u'حِينَ',
    'type': u'STOPWORD',
    'original': u'حِينَ',
    'encletic': u'نَا'
}
STOPWORDS[u'وحينكما'] = {
    'word': u'وحينكما',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف زمان:معطوف:مضاف',
    'vocalized': u'وَحِينَكُمَا',
    'stem': u'حِينَ',
    'type': u'STOPWORD',
    'original': u'حِينَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'وحينهما'] = {
    'word': u'وحينهما',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف زمان:معطوف:مضاف',
    'vocalized': u'وَحِينَهُمَا',
    'stem': u'حِينَ',
    'type': u'STOPWORD',
    'original': u'حِينَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فحيني'] = {
    'word': u'فحيني',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف زمان:معطوف:مضاف',
    'vocalized': u'فَحِينَِي',
    'stem': u'حِينَ',
    'type': u'STOPWORD',
    'original': u'حِينَ',
    'encletic': u'ِي'
}
STOPWORDS[u'فحينك'] = {
    'word': u'فحينك',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف زمان:معطوف:مضاف',
    'vocalized': u'فَحِينَكَ',
    'stem': u'حِينَ',
    'type': u'STOPWORD',
    'original': u'حِينَ',
    'encletic': u'كَ'
}
STOPWORDS[u'فحينك'] = {
    'word': u'فحينك',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف زمان:معطوف:مضاف',
    'vocalized': u'فَحِينَكِ',
    'stem': u'حِينَ',
    'type': u'STOPWORD',
    'original': u'حِينَ',
    'encletic': u'كِ'
}
STOPWORDS[u'فحينه'] = {
    'word': u'فحينه',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف زمان:معطوف:مضاف',
    'vocalized': u'فَحِينَه',
    'stem': u'حِينَ',
    'type': u'STOPWORD',
    'original': u'حِينَ',
    'encletic': u'ه'
}
STOPWORDS[u'فحينكم'] = {
    'word': u'فحينكم',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف زمان:معطوف:مضاف',
    'vocalized': u'فَحِينَكُمْ',
    'stem': u'حِينَ',
    'type': u'STOPWORD',
    'original': u'حِينَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فحينكن'] = {
    'word': u'فحينكن',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف زمان:معطوف:مضاف',
    'vocalized': u'فَحِينَكُنَّ',
    'stem': u'حِينَ',
    'type': u'STOPWORD',
    'original': u'حِينَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فحينها'] = {
    'word': u'فحينها',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف زمان:معطوف:مضاف',
    'vocalized': u'فَحِينَهَا',
    'stem': u'حِينَ',
    'type': u'STOPWORD',
    'original': u'حِينَ',
    'encletic': u'هَا'
}
STOPWORDS[u'فحينهم'] = {
    'word': u'فحينهم',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف زمان:معطوف:مضاف',
    'vocalized': u'فَحِينَهُمْ',
    'stem': u'حِينَ',
    'type': u'STOPWORD',
    'original': u'حِينَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فحينهن'] = {
    'word': u'فحينهن',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف زمان:معطوف:مضاف',
    'vocalized': u'فَحِينَهُنَّ',
    'stem': u'حِينَ',
    'type': u'STOPWORD',
    'original': u'حِينَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فحيننا'] = {
    'word': u'فحيننا',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف زمان:معطوف:مضاف',
    'vocalized': u'فَحِينَنَا',
    'stem': u'حِينَ',
    'type': u'STOPWORD',
    'original': u'حِينَ',
    'encletic': u'نَا'
}
STOPWORDS[u'فحينكما'] = {
    'word': u'فحينكما',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف زمان:معطوف:مضاف',
    'vocalized': u'فَحِينَكُمَا',
    'stem': u'حِينَ',
    'type': u'STOPWORD',
    'original': u'حِينَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فحينهما'] = {
    'word': u'فحينهما',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف زمان:معطوف:مضاف',
    'vocalized': u'فَحِينَهُمَا',
    'stem': u'حِينَ',
    'type': u'STOPWORD',
    'original': u'حِينَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'صباح'] = {
    'word': u'صَبَاحَ',
    'procletic': u'',
    'tags': u'أداة:ظرف زمان',
    'vocalized': u'صَبَاحَ',
    'stem': u'صَبَاحَ',
    'type': u'STOPWORD',
    'original': u'صَبَاحَ',
    'encletic': u''
}
STOPWORDS[u'وصباح'] = {
    'word': u'وصباح',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف زمان:معطوف',
    'vocalized': u'وَصَبَاحَ',
    'stem': u'صَبَاحَ',
    'type': u'STOPWORD',
    'original': u'صَبَاحَ',
    'encletic': u''
}
STOPWORDS[u'فصباح'] = {
    'word': u'فصباح',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف زمان:معطوف',
    'vocalized': u'فَصَبَاحَ',
    'stem': u'صَبَاحَ',
    'type': u'STOPWORD',
    'original': u'صَبَاحَ',
    'encletic': u''
}
STOPWORDS[u'ضحوة'] = {
    'word': u'ضَحْوَةً',
    'procletic': u'',
    'tags': u'أداة:ظرف زمان',
    'vocalized': u'ضَحْوَةً',
    'stem': u'ضَحْوَةً',
    'type': u'STOPWORD',
    'original': u'ضَحْوَةً',
    'encletic': u''
}
STOPWORDS[u'وضحوة'] = {
    'word': u'وضحوة',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف زمان:معطوف',
    'vocalized': u'وَضَحْوَةً',
    'stem': u'ضَحْوَةً',
    'type': u'STOPWORD',
    'original': u'ضَحْوَةً',
    'encletic': u''
}
STOPWORDS[u'فضحوة'] = {
    'word': u'فضحوة',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف زمان:معطوف',
    'vocalized': u'فَضَحْوَةً',
    'stem': u'ضَحْوَةً',
    'type': u'STOPWORD',
    'original': u'ضَحْوَةً',
    'encletic': u''
}
STOPWORDS[u'غدا'] = {
    'word': u'غَدًا',
    'procletic': u'',
    'tags': u'أداة:ظرف زمان',
    'vocalized': u'غَدًا',
    'stem': u'غَدًا',
    'type': u'STOPWORD',
    'original': u'غَدًا',
    'encletic': u''
}
STOPWORDS[u'وغدا'] = {
    'word': u'وغدا',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف زمان:معطوف',
    'vocalized': u'وَغَدًا',
    'stem': u'غَدًا',
    'type': u'STOPWORD',
    'original': u'غَدًا',
    'encletic': u''
}
STOPWORDS[u'فغدا'] = {
    'word': u'فغدا',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف زمان:معطوف',
    'vocalized': u'فَغَدًا',
    'stem': u'غَدًا',
    'type': u'STOPWORD',
    'original': u'غَدًا',
    'encletic': u''
}
STOPWORDS[u'غداة'] = {
    'word': u'غَدَاةَ',
    'procletic': u'',
    'tags': u'أداة:ظرف زمان',
    'vocalized': u'غَدَاةَ',
    'stem': u'غَدَاةَ',
    'type': u'STOPWORD',
    'original': u'غَدَاةَ',
    'encletic': u''
}
STOPWORDS[u'وغداة'] = {
    'word': u'وغداة',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف زمان:معطوف',
    'vocalized': u'وَغَدَاةَ',
    'stem': u'غَدَاةَ',
    'type': u'STOPWORD',
    'original': u'غَدَاةَ',
    'encletic': u''
}
STOPWORDS[u'فغداة'] = {
    'word': u'فغداة',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف زمان:معطوف',
    'vocalized': u'فَغَدَاةَ',
    'stem': u'غَدَاةَ',
    'type': u'STOPWORD',
    'original': u'غَدَاةَ',
    'encletic': u''
}
STOPWORDS[u'مرة'] = {
    'word': u'مَرَّةً',
    'procletic': u'',
    'tags': u'أداة:ظرف زمان',
    'vocalized': u'مَرَّةً',
    'stem': u'مَرَّةً',
    'type': u'STOPWORD',
    'original': u'مَرَّةً',
    'encletic': u''
}
STOPWORDS[u'ومرة'] = {
    'word': u'ومرة',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف زمان:معطوف',
    'vocalized': u'وَمَرَّةً',
    'stem': u'مَرَّةً',
    'type': u'STOPWORD',
    'original': u'مَرَّةً',
    'encletic': u''
}
STOPWORDS[u'فمرة'] = {
    'word': u'فمرة',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف زمان:معطوف',
    'vocalized': u'فَمَرَّةً',
    'stem': u'مَرَّةً',
    'type': u'STOPWORD',
    'original': u'مَرَّةً',
    'encletic': u''
}
STOPWORDS[u'مساء'] = {
    'word': u'مَسَاءَ',
    'procletic': u'',
    'tags': u'أداة:ظرف زمان',
    'vocalized': u'مَسَاءَ',
    'stem': u'مَسَاءَ',
    'type': u'STOPWORD',
    'original': u'مَسَاءَ',
    'encletic': u''
}
STOPWORDS[u'ومساء'] = {
    'word': u'ومساء',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف زمان:معطوف',
    'vocalized': u'وَمَسَاءَ',
    'stem': u'مَسَاءَ',
    'type': u'STOPWORD',
    'original': u'مَسَاءَ',
    'encletic': u''
}
STOPWORDS[u'فمساء'] = {
    'word': u'فمساء',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف زمان:معطوف',
    'vocalized': u'فَمَسَاءَ',
    'stem': u'مَسَاءَ',
    'type': u'STOPWORD',
    'original': u'مَسَاءَ',
    'encletic': u''
}
STOPWORDS[u'يومئذ'] = {
    'word': u'يَوْمَئِذٍ',
    'procletic': u'',
    'tags': u'أداة:ظرف زمان',
    'vocalized': u'يَوْمَئِذٍ',
    'stem': u'يَوْمَئِذٍ',
    'type': u'STOPWORD',
    'original': u'يَوْمَئِذٍ',
    'encletic': u''
}
STOPWORDS[u'ويومئذ'] = {
    'word': u'ويومئذ',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف زمان:معطوف',
    'vocalized': u'وَيَوْمَئِذٍ',
    'stem': u'يَوْمَئِذٍ',
    'type': u'STOPWORD',
    'original': u'يَوْمَئِذٍ',
    'encletic': u''
}
STOPWORDS[u'فيومئذ'] = {
    'word': u'فيومئذ',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف زمان:معطوف',
    'vocalized': u'فَيَوْمَئِذٍ',
    'stem': u'يَوْمَئِذٍ',
    'type': u'STOPWORD',
    'original': u'يَوْمَئِذٍ',
    'encletic': u''
}
STOPWORDS[u'خلال'] = {
    'word': u'خِلَالَ',
    'procletic': u'',
    'tags': u'أداة:ظرف زمان/مكان',
    'vocalized': u'خِلَالَ',
    'stem': u'خِلَالَ',
    'type': u'STOPWORD',
    'original': u'خِلَالَ',
    'encletic': u''
}
STOPWORDS[u'وخلال'] = {
    'word': u'وخلال',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف زمان/مكان:معطوف',
    'vocalized': u'وَخِلَالَ',
    'stem': u'خِلَالَ',
    'type': u'STOPWORD',
    'original': u'خِلَالَ',
    'encletic': u''
}
STOPWORDS[u'فخلال'] = {
    'word': u'فخلال',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف زمان/مكان:معطوف',
    'vocalized': u'فَخِلَالَ',
    'stem': u'خِلَالَ',
    'type': u'STOPWORD',
    'original': u'خِلَالَ',
    'encletic': u''
}
STOPWORDS[u'خلالي'] = {
    'word': u'خلالي',
    'procletic': u'',
    'tags': u'أداة:ظرف زمان/مكان:مضاف',
    'vocalized': u'خِلَالَِي',
    'stem': u'خِلَالَ',
    'type': u'STOPWORD',
    'original': u'خِلَالَ',
    'encletic': u'ِي'
}
STOPWORDS[u'خلالك'] = {
    'word': u'خلالك',
    'procletic': u'',
    'tags': u'أداة:ظرف زمان/مكان:مضاف',
    'vocalized': u'خِلَالَكَ',
    'stem': u'خِلَالَ',
    'type': u'STOPWORD',
    'original': u'خِلَالَ',
    'encletic': u'كَ'
}
STOPWORDS[u'خلالك'] = {
    'word': u'خلالك',
    'procletic': u'',
    'tags': u'أداة:ظرف زمان/مكان:مضاف',
    'vocalized': u'خِلَالَكِ',
    'stem': u'خِلَالَ',
    'type': u'STOPWORD',
    'original': u'خِلَالَ',
    'encletic': u'كِ'
}
STOPWORDS[u'خلاله'] = {
    'word': u'خلاله',
    'procletic': u'',
    'tags': u'أداة:ظرف زمان/مكان:مضاف',
    'vocalized': u'خِلَالَه',
    'stem': u'خِلَالَ',
    'type': u'STOPWORD',
    'original': u'خِلَالَ',
    'encletic': u'ه'
}
STOPWORDS[u'خلالكم'] = {
    'word': u'خلالكم',
    'procletic': u'',
    'tags': u'أداة:ظرف زمان/مكان:مضاف',
    'vocalized': u'خِلَالَكُمْ',
    'stem': u'خِلَالَ',
    'type': u'STOPWORD',
    'original': u'خِلَالَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'خلالكن'] = {
    'word': u'خلالكن',
    'procletic': u'',
    'tags': u'أداة:ظرف زمان/مكان:مضاف',
    'vocalized': u'خِلَالَكُنَّ',
    'stem': u'خِلَالَ',
    'type': u'STOPWORD',
    'original': u'خِلَالَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'خلالها'] = {
    'word': u'خلالها',
    'procletic': u'',
    'tags': u'أداة:ظرف زمان/مكان:مضاف',
    'vocalized': u'خِلَالَهَا',
    'stem': u'خِلَالَ',
    'type': u'STOPWORD',
    'original': u'خِلَالَ',
    'encletic': u'هَا'
}
STOPWORDS[u'خلالهم'] = {
    'word': u'خلالهم',
    'procletic': u'',
    'tags': u'أداة:ظرف زمان/مكان:مضاف',
    'vocalized': u'خِلَالَهُمْ',
    'stem': u'خِلَالَ',
    'type': u'STOPWORD',
    'original': u'خِلَالَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'خلالهن'] = {
    'word': u'خلالهن',
    'procletic': u'',
    'tags': u'أداة:ظرف زمان/مكان:مضاف',
    'vocalized': u'خِلَالَهُنَّ',
    'stem': u'خِلَالَ',
    'type': u'STOPWORD',
    'original': u'خِلَالَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'خلالنا'] = {
    'word': u'خلالنا',
    'procletic': u'',
    'tags': u'أداة:ظرف زمان/مكان:مضاف',
    'vocalized': u'خِلَالَنَا',
    'stem': u'خِلَالَ',
    'type': u'STOPWORD',
    'original': u'خِلَالَ',
    'encletic': u'نَا'
}
STOPWORDS[u'خلالكما'] = {
    'word': u'خلالكما',
    'procletic': u'',
    'tags': u'أداة:ظرف زمان/مكان:مضاف',
    'vocalized': u'خِلَالَكُمَا',
    'stem': u'خِلَالَ',
    'type': u'STOPWORD',
    'original': u'خِلَالَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'خلالهما'] = {
    'word': u'خلالهما',
    'procletic': u'',
    'tags': u'أداة:ظرف زمان/مكان:مضاف',
    'vocalized': u'خِلَالَهُمَا',
    'stem': u'خِلَالَ',
    'type': u'STOPWORD',
    'original': u'خِلَالَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'وخلالي'] = {
    'word': u'وخلالي',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف زمان/مكان:معطوف:مضاف',
    'vocalized': u'وَخِلَالَِي',
    'stem': u'خِلَالَ',
    'type': u'STOPWORD',
    'original': u'خِلَالَ',
    'encletic': u'ِي'
}
STOPWORDS[u'وخلالك'] = {
    'word': u'وخلالك',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف زمان/مكان:معطوف:مضاف',
    'vocalized': u'وَخِلَالَكَ',
    'stem': u'خِلَالَ',
    'type': u'STOPWORD',
    'original': u'خِلَالَ',
    'encletic': u'كَ'
}
STOPWORDS[u'وخلالك'] = {
    'word': u'وخلالك',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف زمان/مكان:معطوف:مضاف',
    'vocalized': u'وَخِلَالَكِ',
    'stem': u'خِلَالَ',
    'type': u'STOPWORD',
    'original': u'خِلَالَ',
    'encletic': u'كِ'
}
STOPWORDS[u'وخلاله'] = {
    'word': u'وخلاله',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف زمان/مكان:معطوف:مضاف',
    'vocalized': u'وَخِلَالَه',
    'stem': u'خِلَالَ',
    'type': u'STOPWORD',
    'original': u'خِلَالَ',
    'encletic': u'ه'
}
STOPWORDS[u'وخلالكم'] = {
    'word': u'وخلالكم',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف زمان/مكان:معطوف:مضاف',
    'vocalized': u'وَخِلَالَكُمْ',
    'stem': u'خِلَالَ',
    'type': u'STOPWORD',
    'original': u'خِلَالَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'وخلالكن'] = {
    'word': u'وخلالكن',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف زمان/مكان:معطوف:مضاف',
    'vocalized': u'وَخِلَالَكُنَّ',
    'stem': u'خِلَالَ',
    'type': u'STOPWORD',
    'original': u'خِلَالَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'وخلالها'] = {
    'word': u'وخلالها',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف زمان/مكان:معطوف:مضاف',
    'vocalized': u'وَخِلَالَهَا',
    'stem': u'خِلَالَ',
    'type': u'STOPWORD',
    'original': u'خِلَالَ',
    'encletic': u'هَا'
}
STOPWORDS[u'وخلالهم'] = {
    'word': u'وخلالهم',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف زمان/مكان:معطوف:مضاف',
    'vocalized': u'وَخِلَالَهُمْ',
    'stem': u'خِلَالَ',
    'type': u'STOPWORD',
    'original': u'خِلَالَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'وخلالهن'] = {
    'word': u'وخلالهن',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف زمان/مكان:معطوف:مضاف',
    'vocalized': u'وَخِلَالَهُنَّ',
    'stem': u'خِلَالَ',
    'type': u'STOPWORD',
    'original': u'خِلَالَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'وخلالنا'] = {
    'word': u'وخلالنا',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف زمان/مكان:معطوف:مضاف',
    'vocalized': u'وَخِلَالَنَا',
    'stem': u'خِلَالَ',
    'type': u'STOPWORD',
    'original': u'خِلَالَ',
    'encletic': u'نَا'
}
STOPWORDS[u'وخلالكما'] = {
    'word': u'وخلالكما',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف زمان/مكان:معطوف:مضاف',
    'vocalized': u'وَخِلَالَكُمَا',
    'stem': u'خِلَالَ',
    'type': u'STOPWORD',
    'original': u'خِلَالَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'وخلالهما'] = {
    'word': u'وخلالهما',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف زمان/مكان:معطوف:مضاف',
    'vocalized': u'وَخِلَالَهُمَا',
    'stem': u'خِلَالَ',
    'type': u'STOPWORD',
    'original': u'خِلَالَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فخلالي'] = {
    'word': u'فخلالي',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف زمان/مكان:معطوف:مضاف',
    'vocalized': u'فَخِلَالَِي',
    'stem': u'خِلَالَ',
    'type': u'STOPWORD',
    'original': u'خِلَالَ',
    'encletic': u'ِي'
}
STOPWORDS[u'فخلالك'] = {
    'word': u'فخلالك',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف زمان/مكان:معطوف:مضاف',
    'vocalized': u'فَخِلَالَكَ',
    'stem': u'خِلَالَ',
    'type': u'STOPWORD',
    'original': u'خِلَالَ',
    'encletic': u'كَ'
}
STOPWORDS[u'فخلالك'] = {
    'word': u'فخلالك',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف زمان/مكان:معطوف:مضاف',
    'vocalized': u'فَخِلَالَكِ',
    'stem': u'خِلَالَ',
    'type': u'STOPWORD',
    'original': u'خِلَالَ',
    'encletic': u'كِ'
}
STOPWORDS[u'فخلاله'] = {
    'word': u'فخلاله',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف زمان/مكان:معطوف:مضاف',
    'vocalized': u'فَخِلَالَه',
    'stem': u'خِلَالَ',
    'type': u'STOPWORD',
    'original': u'خِلَالَ',
    'encletic': u'ه'
}
STOPWORDS[u'فخلالكم'] = {
    'word': u'فخلالكم',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف زمان/مكان:معطوف:مضاف',
    'vocalized': u'فَخِلَالَكُمْ',
    'stem': u'خِلَالَ',
    'type': u'STOPWORD',
    'original': u'خِلَالَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فخلالكن'] = {
    'word': u'فخلالكن',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف زمان/مكان:معطوف:مضاف',
    'vocalized': u'فَخِلَالَكُنَّ',
    'stem': u'خِلَالَ',
    'type': u'STOPWORD',
    'original': u'خِلَالَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فخلالها'] = {
    'word': u'فخلالها',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف زمان/مكان:معطوف:مضاف',
    'vocalized': u'فَخِلَالَهَا',
    'stem': u'خِلَالَ',
    'type': u'STOPWORD',
    'original': u'خِلَالَ',
    'encletic': u'هَا'
}
STOPWORDS[u'فخلالهم'] = {
    'word': u'فخلالهم',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف زمان/مكان:معطوف:مضاف',
    'vocalized': u'فَخِلَالَهُمْ',
    'stem': u'خِلَالَ',
    'type': u'STOPWORD',
    'original': u'خِلَالَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فخلالهن'] = {
    'word': u'فخلالهن',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف زمان/مكان:معطوف:مضاف',
    'vocalized': u'فَخِلَالَهُنَّ',
    'stem': u'خِلَالَ',
    'type': u'STOPWORD',
    'original': u'خِلَالَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فخلالنا'] = {
    'word': u'فخلالنا',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف زمان/مكان:معطوف:مضاف',
    'vocalized': u'فَخِلَالَنَا',
    'stem': u'خِلَالَ',
    'type': u'STOPWORD',
    'original': u'خِلَالَ',
    'encletic': u'نَا'
}
STOPWORDS[u'فخلالكما'] = {
    'word': u'فخلالكما',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف زمان/مكان:معطوف:مضاف',
    'vocalized': u'فَخِلَالَكُمَا',
    'stem': u'خِلَالَ',
    'type': u'STOPWORD',
    'original': u'خِلَالَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فخلالهما'] = {
    'word': u'فخلالهما',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف زمان/مكان:معطوف:مضاف',
    'vocalized': u'فَخِلَالَهُمَا',
    'stem': u'خِلَالَ',
    'type': u'STOPWORD',
    'original': u'خِلَالَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أمام'] = {
    'word': u'أَمَامَ',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان',
    'vocalized': u'أَمَامَ',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u''
}
STOPWORDS[u'لأمام'] = {
    'word': u'لأمام',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور',
    'vocalized': u'لَأَمَامَ',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u''
}
STOPWORDS[u'وأمام'] = {
    'word': u'وأمام',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف',
    'vocalized': u'وَأَمَامَ',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u''
}
STOPWORDS[u'فأمام'] = {
    'word': u'فأمام',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف',
    'vocalized': u'فَأَمَامَ',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u''
}
STOPWORDS[u'ولأمام'] = {
    'word': u'ولأمام',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'وَلَأَمَامَ',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u''
}
STOPWORDS[u'فلأمام'] = {
    'word': u'فلأمام',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'فَلَأَمَامَ',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u''
}
STOPWORDS[u'أأمام'] = {
    'word': u'أأمام',
    'procletic': u'-أَ',
    'tags': u'أداة:ظرف مكان:مجرور',
    'vocalized': u'أَأَمَامَ',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u''
}
STOPWORDS[u'ألأمام'] = {
    'word': u'ألأمام',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور',
    'vocalized': u'أَلَأَمَامَ',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u''
}
STOPWORDS[u'أوأمام'] = {
    'word': u'أوأمام',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور',
    'vocalized': u'أَوَأَمَامَ',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u''
}
STOPWORDS[u'أفأمام'] = {
    'word': u'أفأمام',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور',
    'vocalized': u'أَفَأَمَامَ',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u''
}
STOPWORDS[u'أولأمام'] = {
    'word': u'أولأمام',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَلَأَمَامَ',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u''
}
STOPWORDS[u'أفلأمام'] = {
    'word': u'أفلأمام',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَلَأَمَامَ',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u''
}
STOPWORDS[u'أمامي'] = {
    'word': u'أمامي',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'أَمَامَِي',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أمامك'] = {
    'word': u'أمامك',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'أَمَامَكَ',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أمامك'] = {
    'word': u'أمامك',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'أَمَامَكِ',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أمامه'] = {
    'word': u'أمامه',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'أَمَامَه',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'ه'
}
STOPWORDS[u'أمامكم'] = {
    'word': u'أمامكم',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'أَمَامَكُمْ',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أمامكن'] = {
    'word': u'أمامكن',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'أَمَامَكُنَّ',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أمامها'] = {
    'word': u'أمامها',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'أَمَامَهَا',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أمامهم'] = {
    'word': u'أمامهم',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'أَمَامَهُمْ',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أمامهن'] = {
    'word': u'أمامهن',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'أَمَامَهُنَّ',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أمامنا'] = {
    'word': u'أمامنا',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'أَمَامَنَا',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أمامكما'] = {
    'word': u'أمامكما',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'أَمَامَكُمَا',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أمامهما'] = {
    'word': u'أمامهما',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'أَمَامَهُمَا',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'لأمامي'] = {
    'word': u'لأمامي',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَأَمَامَِي',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'ِي'
}
STOPWORDS[u'لأمامك'] = {
    'word': u'لأمامك',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَأَمَامَكَ',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'كَ'
}
STOPWORDS[u'لأمامك'] = {
    'word': u'لأمامك',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَأَمَامَكِ',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'كِ'
}
STOPWORDS[u'لأمامه'] = {
    'word': u'لأمامه',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَأَمَامَه',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'ه'
}
STOPWORDS[u'لأمامكم'] = {
    'word': u'لأمامكم',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَأَمَامَكُمْ',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'لأمامكن'] = {
    'word': u'لأمامكن',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَأَمَامَكُنَّ',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'لأمامها'] = {
    'word': u'لأمامها',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَأَمَامَهَا',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'هَا'
}
STOPWORDS[u'لأمامهم'] = {
    'word': u'لأمامهم',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَأَمَامَهُمْ',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'لأمامهن'] = {
    'word': u'لأمامهن',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَأَمَامَهُنَّ',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'لأمامنا'] = {
    'word': u'لأمامنا',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَأَمَامَنَا',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'نَا'
}
STOPWORDS[u'لأمامكما'] = {
    'word': u'لأمامكما',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَأَمَامَكُمَا',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'لأمامهما'] = {
    'word': u'لأمامهما',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَأَمَامَهُمَا',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'وأمامي'] = {
    'word': u'وأمامي',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَأَمَامَِي',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'ِي'
}
STOPWORDS[u'وأمامك'] = {
    'word': u'وأمامك',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَأَمَامَكَ',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'كَ'
}
STOPWORDS[u'وأمامك'] = {
    'word': u'وأمامك',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَأَمَامَكِ',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'كِ'
}
STOPWORDS[u'وأمامه'] = {
    'word': u'وأمامه',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَأَمَامَه',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'ه'
}
STOPWORDS[u'وأمامكم'] = {
    'word': u'وأمامكم',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَأَمَامَكُمْ',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'وأمامكن'] = {
    'word': u'وأمامكن',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَأَمَامَكُنَّ',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'وأمامها'] = {
    'word': u'وأمامها',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَأَمَامَهَا',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'هَا'
}
STOPWORDS[u'وأمامهم'] = {
    'word': u'وأمامهم',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَأَمَامَهُمْ',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'وأمامهن'] = {
    'word': u'وأمامهن',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَأَمَامَهُنَّ',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'وأمامنا'] = {
    'word': u'وأمامنا',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَأَمَامَنَا',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'نَا'
}
STOPWORDS[u'وأمامكما'] = {
    'word': u'وأمامكما',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَأَمَامَكُمَا',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'وأمامهما'] = {
    'word': u'وأمامهما',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَأَمَامَهُمَا',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فأمامي'] = {
    'word': u'فأمامي',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَأَمَامَِي',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'ِي'
}
STOPWORDS[u'فأمامك'] = {
    'word': u'فأمامك',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَأَمَامَكَ',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'كَ'
}
STOPWORDS[u'فأمامك'] = {
    'word': u'فأمامك',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَأَمَامَكِ',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'كِ'
}
STOPWORDS[u'فأمامه'] = {
    'word': u'فأمامه',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَأَمَامَه',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'ه'
}
STOPWORDS[u'فأمامكم'] = {
    'word': u'فأمامكم',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَأَمَامَكُمْ',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فأمامكن'] = {
    'word': u'فأمامكن',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَأَمَامَكُنَّ',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فأمامها'] = {
    'word': u'فأمامها',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَأَمَامَهَا',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'هَا'
}
STOPWORDS[u'فأمامهم'] = {
    'word': u'فأمامهم',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَأَمَامَهُمْ',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فأمامهن'] = {
    'word': u'فأمامهن',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَأَمَامَهُنَّ',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فأمامنا'] = {
    'word': u'فأمامنا',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَأَمَامَنَا',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'نَا'
}
STOPWORDS[u'فأمامكما'] = {
    'word': u'فأمامكما',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَأَمَامَكُمَا',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فأمامهما'] = {
    'word': u'فأمامهما',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَأَمَامَهُمَا',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ولأمامي'] = {
    'word': u'ولأمامي',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَأَمَامَِي',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'ِي'
}
STOPWORDS[u'ولأمامك'] = {
    'word': u'ولأمامك',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَأَمَامَكَ',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'كَ'
}
STOPWORDS[u'ولأمامك'] = {
    'word': u'ولأمامك',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَأَمَامَكِ',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'كِ'
}
STOPWORDS[u'ولأمامه'] = {
    'word': u'ولأمامه',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَأَمَامَه',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'ه'
}
STOPWORDS[u'ولأمامكم'] = {
    'word': u'ولأمامكم',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَأَمَامَكُمْ',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ولأمامكن'] = {
    'word': u'ولأمامكن',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَأَمَامَكُنَّ',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ولأمامها'] = {
    'word': u'ولأمامها',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَأَمَامَهَا',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'هَا'
}
STOPWORDS[u'ولأمامهم'] = {
    'word': u'ولأمامهم',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَأَمَامَهُمْ',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ولأمامهن'] = {
    'word': u'ولأمامهن',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَأَمَامَهُنَّ',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ولأمامنا'] = {
    'word': u'ولأمامنا',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَأَمَامَنَا',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'نَا'
}
STOPWORDS[u'ولأمامكما'] = {
    'word': u'ولأمامكما',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَأَمَامَكُمَا',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ولأمامهما'] = {
    'word': u'ولأمامهما',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَأَمَامَهُمَا',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فلأمامي'] = {
    'word': u'فلأمامي',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَأَمَامَِي',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'ِي'
}
STOPWORDS[u'فلأمامك'] = {
    'word': u'فلأمامك',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَأَمَامَكَ',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'كَ'
}
STOPWORDS[u'فلأمامك'] = {
    'word': u'فلأمامك',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَأَمَامَكِ',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'كِ'
}
STOPWORDS[u'فلأمامه'] = {
    'word': u'فلأمامه',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَأَمَامَه',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'ه'
}
STOPWORDS[u'فلأمامكم'] = {
    'word': u'فلأمامكم',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَأَمَامَكُمْ',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فلأمامكن'] = {
    'word': u'فلأمامكن',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَأَمَامَكُنَّ',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فلأمامها'] = {
    'word': u'فلأمامها',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَأَمَامَهَا',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'هَا'
}
STOPWORDS[u'فلأمامهم'] = {
    'word': u'فلأمامهم',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَأَمَامَهُمْ',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فلأمامهن'] = {
    'word': u'فلأمامهن',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَأَمَامَهُنَّ',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فلأمامنا'] = {
    'word': u'فلأمامنا',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَأَمَامَنَا',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'نَا'
}
STOPWORDS[u'فلأمامكما'] = {
    'word': u'فلأمامكما',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَأَمَامَكُمَا',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فلأمامهما'] = {
    'word': u'فلأمامهما',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَأَمَامَهُمَا',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أأمامي'] = {
    'word': u'أأمامي',
    'procletic': u'-أَ',
    'tags': u'أداة:ظرف مكان:مجرور:مضاف',
    'vocalized': u'أَأَمَامَِي',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أأمامك'] = {
    'word': u'أأمامك',
    'procletic': u'-أَ',
    'tags': u'أداة:ظرف مكان:مجرور:مضاف',
    'vocalized': u'أَأَمَامَكَ',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أأمامك'] = {
    'word': u'أأمامك',
    'procletic': u'-أَ',
    'tags': u'أداة:ظرف مكان:مجرور:مضاف',
    'vocalized': u'أَأَمَامَكِ',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أأمامه'] = {
    'word': u'أأمامه',
    'procletic': u'-أَ',
    'tags': u'أداة:ظرف مكان:مجرور:مضاف',
    'vocalized': u'أَأَمَامَه',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'ه'
}
STOPWORDS[u'أأمامكم'] = {
    'word': u'أأمامكم',
    'procletic': u'-أَ',
    'tags': u'أداة:ظرف مكان:مجرور:مضاف',
    'vocalized': u'أَأَمَامَكُمْ',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أأمامكن'] = {
    'word': u'أأمامكن',
    'procletic': u'-أَ',
    'tags': u'أداة:ظرف مكان:مجرور:مضاف',
    'vocalized': u'أَأَمَامَكُنَّ',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أأمامها'] = {
    'word': u'أأمامها',
    'procletic': u'-أَ',
    'tags': u'أداة:ظرف مكان:مجرور:مضاف',
    'vocalized': u'أَأَمَامَهَا',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أأمامهم'] = {
    'word': u'أأمامهم',
    'procletic': u'-أَ',
    'tags': u'أداة:ظرف مكان:مجرور:مضاف',
    'vocalized': u'أَأَمَامَهُمْ',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أأمامهن'] = {
    'word': u'أأمامهن',
    'procletic': u'-أَ',
    'tags': u'أداة:ظرف مكان:مجرور:مضاف',
    'vocalized': u'أَأَمَامَهُنَّ',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أأمامنا'] = {
    'word': u'أأمامنا',
    'procletic': u'-أَ',
    'tags': u'أداة:ظرف مكان:مجرور:مضاف',
    'vocalized': u'أَأَمَامَنَا',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أأمامكما'] = {
    'word': u'أأمامكما',
    'procletic': u'-أَ',
    'tags': u'أداة:ظرف مكان:مجرور:مضاف',
    'vocalized': u'أَأَمَامَكُمَا',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أأمامهما'] = {
    'word': u'أأمامهما',
    'procletic': u'-أَ',
    'tags': u'أداة:ظرف مكان:مجرور:مضاف',
    'vocalized': u'أَأَمَامَهُمَا',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ألأمامي'] = {
    'word': u'ألأمامي',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَأَمَامَِي',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'ِي'
}
STOPWORDS[u'ألأمامك'] = {
    'word': u'ألأمامك',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَأَمَامَكَ',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'كَ'
}
STOPWORDS[u'ألأمامك'] = {
    'word': u'ألأمامك',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَأَمَامَكِ',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'كِ'
}
STOPWORDS[u'ألأمامه'] = {
    'word': u'ألأمامه',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَأَمَامَه',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'ه'
}
STOPWORDS[u'ألأمامكم'] = {
    'word': u'ألأمامكم',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَأَمَامَكُمْ',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ألأمامكن'] = {
    'word': u'ألأمامكن',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَأَمَامَكُنَّ',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ألأمامها'] = {
    'word': u'ألأمامها',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَأَمَامَهَا',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'هَا'
}
STOPWORDS[u'ألأمامهم'] = {
    'word': u'ألأمامهم',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَأَمَامَهُمْ',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ألأمامهن'] = {
    'word': u'ألأمامهن',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَأَمَامَهُنَّ',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ألأمامنا'] = {
    'word': u'ألأمامنا',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَأَمَامَنَا',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'نَا'
}
STOPWORDS[u'ألأمامكما'] = {
    'word': u'ألأمامكما',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَأَمَامَكُمَا',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ألأمامهما'] = {
    'word': u'ألأمامهما',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَأَمَامَهُمَا',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أوأمامي'] = {
    'word': u'أوأمامي',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَأَمَامَِي',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أوأمامك'] = {
    'word': u'أوأمامك',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَأَمَامَكَ',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أوأمامك'] = {
    'word': u'أوأمامك',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَأَمَامَكِ',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أوأمامه'] = {
    'word': u'أوأمامه',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَأَمَامَه',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'ه'
}
STOPWORDS[u'أوأمامكم'] = {
    'word': u'أوأمامكم',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَأَمَامَكُمْ',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أوأمامكن'] = {
    'word': u'أوأمامكن',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَأَمَامَكُنَّ',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أوأمامها'] = {
    'word': u'أوأمامها',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَأَمَامَهَا',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أوأمامهم'] = {
    'word': u'أوأمامهم',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَأَمَامَهُمْ',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أوأمامهن'] = {
    'word': u'أوأمامهن',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَأَمَامَهُنَّ',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أوأمامنا'] = {
    'word': u'أوأمامنا',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَأَمَامَنَا',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أوأمامكما'] = {
    'word': u'أوأمامكما',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَأَمَامَكُمَا',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أوأمامهما'] = {
    'word': u'أوأمامهما',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَأَمَامَهُمَا',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أفأمامي'] = {
    'word': u'أفأمامي',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَأَمَامَِي',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أفأمامك'] = {
    'word': u'أفأمامك',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَأَمَامَكَ',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أفأمامك'] = {
    'word': u'أفأمامك',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَأَمَامَكِ',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أفأمامه'] = {
    'word': u'أفأمامه',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَأَمَامَه',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'ه'
}
STOPWORDS[u'أفأمامكم'] = {
    'word': u'أفأمامكم',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَأَمَامَكُمْ',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أفأمامكن'] = {
    'word': u'أفأمامكن',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَأَمَامَكُنَّ',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أفأمامها'] = {
    'word': u'أفأمامها',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَأَمَامَهَا',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أفأمامهم'] = {
    'word': u'أفأمامهم',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَأَمَامَهُمْ',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أفأمامهن'] = {
    'word': u'أفأمامهن',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَأَمَامَهُنَّ',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أفأمامنا'] = {
    'word': u'أفأمامنا',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَأَمَامَنَا',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أفأمامكما'] = {
    'word': u'أفأمامكما',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَأَمَامَكُمَا',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أفأمامهما'] = {
    'word': u'أفأمامهما',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَأَمَامَهُمَا',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أولأمامي'] = {
    'word': u'أولأمامي',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَأَمَامَِي',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أولأمامك'] = {
    'word': u'أولأمامك',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَأَمَامَكَ',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أولأمامك'] = {
    'word': u'أولأمامك',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَأَمَامَكِ',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أولأمامه'] = {
    'word': u'أولأمامه',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَأَمَامَه',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'ه'
}
STOPWORDS[u'أولأمامكم'] = {
    'word': u'أولأمامكم',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَأَمَامَكُمْ',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أولأمامكن'] = {
    'word': u'أولأمامكن',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَأَمَامَكُنَّ',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أولأمامها'] = {
    'word': u'أولأمامها',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَأَمَامَهَا',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أولأمامهم'] = {
    'word': u'أولأمامهم',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَأَمَامَهُمْ',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أولأمامهن'] = {
    'word': u'أولأمامهن',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَأَمَامَهُنَّ',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أولأمامنا'] = {
    'word': u'أولأمامنا',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَأَمَامَنَا',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أولأمامكما'] = {
    'word': u'أولأمامكما',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَأَمَامَكُمَا',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أولأمامهما'] = {
    'word': u'أولأمامهما',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَأَمَامَهُمَا',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أفلأمامي'] = {
    'word': u'أفلأمامي',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَأَمَامَِي',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أفلأمامك'] = {
    'word': u'أفلأمامك',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَأَمَامَكَ',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أفلأمامك'] = {
    'word': u'أفلأمامك',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَأَمَامَكِ',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أفلأمامه'] = {
    'word': u'أفلأمامه',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَأَمَامَه',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'ه'
}
STOPWORDS[u'أفلأمامكم'] = {
    'word': u'أفلأمامكم',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَأَمَامَكُمْ',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أفلأمامكن'] = {
    'word': u'أفلأمامكن',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَأَمَامَكُنَّ',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أفلأمامها'] = {
    'word': u'أفلأمامها',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَأَمَامَهَا',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أفلأمامهم'] = {
    'word': u'أفلأمامهم',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَأَمَامَهُمْ',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أفلأمامهن'] = {
    'word': u'أفلأمامهن',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَأَمَامَهُنَّ',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أفلأمامنا'] = {
    'word': u'أفلأمامنا',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَأَمَامَنَا',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أفلأمامكما'] = {
    'word': u'أفلأمامكما',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَأَمَامَكُمَا',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أفلأمامهما'] = {
    'word': u'أفلأمامهما',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَأَمَامَهُمَا',
    'stem': u'أَمَامَ',
    'type': u'STOPWORD',
    'original': u'أَمَامَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أين'] = {
    'word': u'أَيْنَ',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان',
    'vocalized': u'أَيْنَ',
    'stem': u'أَيْنَ',
    'type': u'STOPWORD',
    'original': u'أَيْنَ',
    'encletic': u''
}
STOPWORDS[u'وأين'] = {
    'word': u'وأين',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف',
    'vocalized': u'وَأَيْنَ',
    'stem': u'أَيْنَ',
    'type': u'STOPWORD',
    'original': u'أَيْنَ',
    'encletic': u''
}
STOPWORDS[u'فأين'] = {
    'word': u'فأين',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف',
    'vocalized': u'فَأَيْنَ',
    'stem': u'أَيْنَ',
    'type': u'STOPWORD',
    'original': u'أَيْنَ',
    'encletic': u''
}
STOPWORDS[u'إزاء'] = {
    'word': u'إِزَاءَ',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان',
    'vocalized': u'إِزَاءَ',
    'stem': u'إِزَاءَ',
    'type': u'STOPWORD',
    'original': u'إِزَاءَ',
    'encletic': u''
}
STOPWORDS[u'وإزاء'] = {
    'word': u'وإزاء',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف',
    'vocalized': u'وَإِزَاءَ',
    'stem': u'إِزَاءَ',
    'type': u'STOPWORD',
    'original': u'إِزَاءَ',
    'encletic': u''
}
STOPWORDS[u'فإزاء'] = {
    'word': u'فإزاء',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف',
    'vocalized': u'فَإِزَاءَ',
    'stem': u'إِزَاءَ',
    'type': u'STOPWORD',
    'original': u'إِزَاءَ',
    'encletic': u''
}
STOPWORDS[u'بين'] = {
    'word': u'بَيْنَ',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان',
    'vocalized': u'بَيْنَ',
    'stem': u'بَيْنَ',
    'type': u'STOPWORD',
    'original': u'بَيْنَ',
    'encletic': u''
}
STOPWORDS[u'وبين'] = {
    'word': u'وبين',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف',
    'vocalized': u'وَبَيْنَ',
    'stem': u'بَيْنَ',
    'type': u'STOPWORD',
    'original': u'بَيْنَ',
    'encletic': u''
}
STOPWORDS[u'فبين'] = {
    'word': u'فبين',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف',
    'vocalized': u'فَبَيْنَ',
    'stem': u'بَيْنَ',
    'type': u'STOPWORD',
    'original': u'بَيْنَ',
    'encletic': u''
}
STOPWORDS[u'بيني'] = {
    'word': u'بيني',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'بَيْنَِي',
    'stem': u'بَيْنَ',
    'type': u'STOPWORD',
    'original': u'بَيْنَ',
    'encletic': u'ِي'
}
STOPWORDS[u'بينك'] = {
    'word': u'بينك',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'بَيْنَكَ',
    'stem': u'بَيْنَ',
    'type': u'STOPWORD',
    'original': u'بَيْنَ',
    'encletic': u'كَ'
}
STOPWORDS[u'بينك'] = {
    'word': u'بينك',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'بَيْنَكِ',
    'stem': u'بَيْنَ',
    'type': u'STOPWORD',
    'original': u'بَيْنَ',
    'encletic': u'كِ'
}
STOPWORDS[u'بينه'] = {
    'word': u'بينه',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'بَيْنَه',
    'stem': u'بَيْنَ',
    'type': u'STOPWORD',
    'original': u'بَيْنَ',
    'encletic': u'ه'
}
STOPWORDS[u'بينكم'] = {
    'word': u'بينكم',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'بَيْنَكُمْ',
    'stem': u'بَيْنَ',
    'type': u'STOPWORD',
    'original': u'بَيْنَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'بينكن'] = {
    'word': u'بينكن',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'بَيْنَكُنَّ',
    'stem': u'بَيْنَ',
    'type': u'STOPWORD',
    'original': u'بَيْنَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'بينها'] = {
    'word': u'بينها',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'بَيْنَهَا',
    'stem': u'بَيْنَ',
    'type': u'STOPWORD',
    'original': u'بَيْنَ',
    'encletic': u'هَا'
}
STOPWORDS[u'بينهم'] = {
    'word': u'بينهم',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'بَيْنَهُمْ',
    'stem': u'بَيْنَ',
    'type': u'STOPWORD',
    'original': u'بَيْنَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'بينهن'] = {
    'word': u'بينهن',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'بَيْنَهُنَّ',
    'stem': u'بَيْنَ',
    'type': u'STOPWORD',
    'original': u'بَيْنَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'بيننا'] = {
    'word': u'بيننا',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'بَيْنَنَا',
    'stem': u'بَيْنَ',
    'type': u'STOPWORD',
    'original': u'بَيْنَ',
    'encletic': u'نَا'
}
STOPWORDS[u'بينكما'] = {
    'word': u'بينكما',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'بَيْنَكُمَا',
    'stem': u'بَيْنَ',
    'type': u'STOPWORD',
    'original': u'بَيْنَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'بينهما'] = {
    'word': u'بينهما',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'بَيْنَهُمَا',
    'stem': u'بَيْنَ',
    'type': u'STOPWORD',
    'original': u'بَيْنَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'وبيني'] = {
    'word': u'وبيني',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَبَيْنَِي',
    'stem': u'بَيْنَ',
    'type': u'STOPWORD',
    'original': u'بَيْنَ',
    'encletic': u'ِي'
}
STOPWORDS[u'وبينك'] = {
    'word': u'وبينك',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَبَيْنَكَ',
    'stem': u'بَيْنَ',
    'type': u'STOPWORD',
    'original': u'بَيْنَ',
    'encletic': u'كَ'
}
STOPWORDS[u'وبينك'] = {
    'word': u'وبينك',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَبَيْنَكِ',
    'stem': u'بَيْنَ',
    'type': u'STOPWORD',
    'original': u'بَيْنَ',
    'encletic': u'كِ'
}
STOPWORDS[u'وبينه'] = {
    'word': u'وبينه',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَبَيْنَه',
    'stem': u'بَيْنَ',
    'type': u'STOPWORD',
    'original': u'بَيْنَ',
    'encletic': u'ه'
}
STOPWORDS[u'وبينكم'] = {
    'word': u'وبينكم',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَبَيْنَكُمْ',
    'stem': u'بَيْنَ',
    'type': u'STOPWORD',
    'original': u'بَيْنَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'وبينكن'] = {
    'word': u'وبينكن',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَبَيْنَكُنَّ',
    'stem': u'بَيْنَ',
    'type': u'STOPWORD',
    'original': u'بَيْنَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'وبينها'] = {
    'word': u'وبينها',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَبَيْنَهَا',
    'stem': u'بَيْنَ',
    'type': u'STOPWORD',
    'original': u'بَيْنَ',
    'encletic': u'هَا'
}
STOPWORDS[u'وبينهم'] = {
    'word': u'وبينهم',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَبَيْنَهُمْ',
    'stem': u'بَيْنَ',
    'type': u'STOPWORD',
    'original': u'بَيْنَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'وبينهن'] = {
    'word': u'وبينهن',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَبَيْنَهُنَّ',
    'stem': u'بَيْنَ',
    'type': u'STOPWORD',
    'original': u'بَيْنَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'وبيننا'] = {
    'word': u'وبيننا',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَبَيْنَنَا',
    'stem': u'بَيْنَ',
    'type': u'STOPWORD',
    'original': u'بَيْنَ',
    'encletic': u'نَا'
}
STOPWORDS[u'وبينكما'] = {
    'word': u'وبينكما',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَبَيْنَكُمَا',
    'stem': u'بَيْنَ',
    'type': u'STOPWORD',
    'original': u'بَيْنَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'وبينهما'] = {
    'word': u'وبينهما',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَبَيْنَهُمَا',
    'stem': u'بَيْنَ',
    'type': u'STOPWORD',
    'original': u'بَيْنَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فبيني'] = {
    'word': u'فبيني',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَبَيْنَِي',
    'stem': u'بَيْنَ',
    'type': u'STOPWORD',
    'original': u'بَيْنَ',
    'encletic': u'ِي'
}
STOPWORDS[u'فبينك'] = {
    'word': u'فبينك',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَبَيْنَكَ',
    'stem': u'بَيْنَ',
    'type': u'STOPWORD',
    'original': u'بَيْنَ',
    'encletic': u'كَ'
}
STOPWORDS[u'فبينك'] = {
    'word': u'فبينك',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَبَيْنَكِ',
    'stem': u'بَيْنَ',
    'type': u'STOPWORD',
    'original': u'بَيْنَ',
    'encletic': u'كِ'
}
STOPWORDS[u'فبينه'] = {
    'word': u'فبينه',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَبَيْنَه',
    'stem': u'بَيْنَ',
    'type': u'STOPWORD',
    'original': u'بَيْنَ',
    'encletic': u'ه'
}
STOPWORDS[u'فبينكم'] = {
    'word': u'فبينكم',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَبَيْنَكُمْ',
    'stem': u'بَيْنَ',
    'type': u'STOPWORD',
    'original': u'بَيْنَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فبينكن'] = {
    'word': u'فبينكن',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَبَيْنَكُنَّ',
    'stem': u'بَيْنَ',
    'type': u'STOPWORD',
    'original': u'بَيْنَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فبينها'] = {
    'word': u'فبينها',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَبَيْنَهَا',
    'stem': u'بَيْنَ',
    'type': u'STOPWORD',
    'original': u'بَيْنَ',
    'encletic': u'هَا'
}
STOPWORDS[u'فبينهم'] = {
    'word': u'فبينهم',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَبَيْنَهُمْ',
    'stem': u'بَيْنَ',
    'type': u'STOPWORD',
    'original': u'بَيْنَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فبينهن'] = {
    'word': u'فبينهن',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَبَيْنَهُنَّ',
    'stem': u'بَيْنَ',
    'type': u'STOPWORD',
    'original': u'بَيْنَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فبيننا'] = {
    'word': u'فبيننا',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَبَيْنَنَا',
    'stem': u'بَيْنَ',
    'type': u'STOPWORD',
    'original': u'بَيْنَ',
    'encletic': u'نَا'
}
STOPWORDS[u'فبينكما'] = {
    'word': u'فبينكما',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَبَيْنَكُمَا',
    'stem': u'بَيْنَ',
    'type': u'STOPWORD',
    'original': u'بَيْنَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فبينهما'] = {
    'word': u'فبينهما',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَبَيْنَهُمَا',
    'stem': u'بَيْنَ',
    'type': u'STOPWORD',
    'original': u'بَيْنَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'تحت'] = {
    'word': u'تَحْتَ',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان',
    'vocalized': u'تَحْتَ',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u''
}
STOPWORDS[u'لتحت'] = {
    'word': u'لتحت',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور',
    'vocalized': u'لَتَحْتَ',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u''
}
STOPWORDS[u'وتحت'] = {
    'word': u'وتحت',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف',
    'vocalized': u'وَتَحْتَ',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u''
}
STOPWORDS[u'فتحت'] = {
    'word': u'فتحت',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف',
    'vocalized': u'فَتَحْتَ',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u''
}
STOPWORDS[u'ولتحت'] = {
    'word': u'ولتحت',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'وَلَتَحْتَ',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u''
}
STOPWORDS[u'فلتحت'] = {
    'word': u'فلتحت',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'فَلَتَحْتَ',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u''
}
STOPWORDS[u'أتحت'] = {
    'word': u'أتحت',
    'procletic': u'-أَ',
    'tags': u'أداة:ظرف مكان:مجرور',
    'vocalized': u'أَتَحْتَ',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u''
}
STOPWORDS[u'ألتحت'] = {
    'word': u'ألتحت',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور',
    'vocalized': u'أَلَتَحْتَ',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u''
}
STOPWORDS[u'أوتحت'] = {
    'word': u'أوتحت',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور',
    'vocalized': u'أَوَتَحْتَ',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u''
}
STOPWORDS[u'أفتحت'] = {
    'word': u'أفتحت',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور',
    'vocalized': u'أَفَتَحْتَ',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u''
}
STOPWORDS[u'أولتحت'] = {
    'word': u'أولتحت',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَلَتَحْتَ',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u''
}
STOPWORDS[u'أفلتحت'] = {
    'word': u'أفلتحت',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَلَتَحْتَ',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u''
}
STOPWORDS[u'تحتي'] = {
    'word': u'تحتي',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'تَحْتَِي',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'ِي'
}
STOPWORDS[u'تحتك'] = {
    'word': u'تحتك',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'تَحْتَكَ',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'كَ'
}
STOPWORDS[u'تحتك'] = {
    'word': u'تحتك',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'تَحْتَكِ',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'كِ'
}
STOPWORDS[u'تحته'] = {
    'word': u'تحته',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'تَحْتَه',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'ه'
}
STOPWORDS[u'تحتكم'] = {
    'word': u'تحتكم',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'تَحْتَكُمْ',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'تحتكن'] = {
    'word': u'تحتكن',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'تَحْتَكُنَّ',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'تحتها'] = {
    'word': u'تحتها',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'تَحْتَهَا',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'هَا'
}
STOPWORDS[u'تحتهم'] = {
    'word': u'تحتهم',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'تَحْتَهُمْ',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'تحتهن'] = {
    'word': u'تحتهن',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'تَحْتَهُنَّ',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'تحتنا'] = {
    'word': u'تحتنا',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'تَحْتَنَا',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'نَا'
}
STOPWORDS[u'تحتكما'] = {
    'word': u'تحتكما',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'تَحْتَكُمَا',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'تحتهما'] = {
    'word': u'تحتهما',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'تَحْتَهُمَا',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'لتحتي'] = {
    'word': u'لتحتي',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَتَحْتَِي',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'ِي'
}
STOPWORDS[u'لتحتك'] = {
    'word': u'لتحتك',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَتَحْتَكَ',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'كَ'
}
STOPWORDS[u'لتحتك'] = {
    'word': u'لتحتك',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَتَحْتَكِ',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'كِ'
}
STOPWORDS[u'لتحته'] = {
    'word': u'لتحته',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَتَحْتَه',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'ه'
}
STOPWORDS[u'لتحتكم'] = {
    'word': u'لتحتكم',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَتَحْتَكُمْ',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'لتحتكن'] = {
    'word': u'لتحتكن',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَتَحْتَكُنَّ',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'لتحتها'] = {
    'word': u'لتحتها',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَتَحْتَهَا',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'هَا'
}
STOPWORDS[u'لتحتهم'] = {
    'word': u'لتحتهم',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَتَحْتَهُمْ',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'لتحتهن'] = {
    'word': u'لتحتهن',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَتَحْتَهُنَّ',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'لتحتنا'] = {
    'word': u'لتحتنا',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَتَحْتَنَا',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'نَا'
}
STOPWORDS[u'لتحتكما'] = {
    'word': u'لتحتكما',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَتَحْتَكُمَا',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'لتحتهما'] = {
    'word': u'لتحتهما',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَتَحْتَهُمَا',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'وتحتي'] = {
    'word': u'وتحتي',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَتَحْتَِي',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'ِي'
}
STOPWORDS[u'وتحتك'] = {
    'word': u'وتحتك',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَتَحْتَكَ',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'كَ'
}
STOPWORDS[u'وتحتك'] = {
    'word': u'وتحتك',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَتَحْتَكِ',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'كِ'
}
STOPWORDS[u'وتحته'] = {
    'word': u'وتحته',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَتَحْتَه',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'ه'
}
STOPWORDS[u'وتحتكم'] = {
    'word': u'وتحتكم',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَتَحْتَكُمْ',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'وتحتكن'] = {
    'word': u'وتحتكن',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَتَحْتَكُنَّ',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'وتحتها'] = {
    'word': u'وتحتها',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَتَحْتَهَا',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'هَا'
}
STOPWORDS[u'وتحتهم'] = {
    'word': u'وتحتهم',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَتَحْتَهُمْ',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'وتحتهن'] = {
    'word': u'وتحتهن',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَتَحْتَهُنَّ',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'وتحتنا'] = {
    'word': u'وتحتنا',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَتَحْتَنَا',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'نَا'
}
STOPWORDS[u'وتحتكما'] = {
    'word': u'وتحتكما',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَتَحْتَكُمَا',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'وتحتهما'] = {
    'word': u'وتحتهما',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَتَحْتَهُمَا',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فتحتي'] = {
    'word': u'فتحتي',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَتَحْتَِي',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'ِي'
}
STOPWORDS[u'فتحتك'] = {
    'word': u'فتحتك',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَتَحْتَكَ',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'كَ'
}
STOPWORDS[u'فتحتك'] = {
    'word': u'فتحتك',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَتَحْتَكِ',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'كِ'
}
STOPWORDS[u'فتحته'] = {
    'word': u'فتحته',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَتَحْتَه',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'ه'
}
STOPWORDS[u'فتحتكم'] = {
    'word': u'فتحتكم',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَتَحْتَكُمْ',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فتحتكن'] = {
    'word': u'فتحتكن',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَتَحْتَكُنَّ',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فتحتها'] = {
    'word': u'فتحتها',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَتَحْتَهَا',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'هَا'
}
STOPWORDS[u'فتحتهم'] = {
    'word': u'فتحتهم',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَتَحْتَهُمْ',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فتحتهن'] = {
    'word': u'فتحتهن',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَتَحْتَهُنَّ',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فتحتنا'] = {
    'word': u'فتحتنا',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَتَحْتَنَا',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'نَا'
}
STOPWORDS[u'فتحتكما'] = {
    'word': u'فتحتكما',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَتَحْتَكُمَا',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فتحتهما'] = {
    'word': u'فتحتهما',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَتَحْتَهُمَا',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ولتحتي'] = {
    'word': u'ولتحتي',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَتَحْتَِي',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'ِي'
}
STOPWORDS[u'ولتحتك'] = {
    'word': u'ولتحتك',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَتَحْتَكَ',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'كَ'
}
STOPWORDS[u'ولتحتك'] = {
    'word': u'ولتحتك',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَتَحْتَكِ',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'كِ'
}
STOPWORDS[u'ولتحته'] = {
    'word': u'ولتحته',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَتَحْتَه',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'ه'
}
STOPWORDS[u'ولتحتكم'] = {
    'word': u'ولتحتكم',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَتَحْتَكُمْ',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ولتحتكن'] = {
    'word': u'ولتحتكن',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَتَحْتَكُنَّ',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ولتحتها'] = {
    'word': u'ولتحتها',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَتَحْتَهَا',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'هَا'
}
STOPWORDS[u'ولتحتهم'] = {
    'word': u'ولتحتهم',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَتَحْتَهُمْ',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ولتحتهن'] = {
    'word': u'ولتحتهن',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَتَحْتَهُنَّ',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ولتحتنا'] = {
    'word': u'ولتحتنا',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَتَحْتَنَا',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'نَا'
}
STOPWORDS[u'ولتحتكما'] = {
    'word': u'ولتحتكما',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَتَحْتَكُمَا',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ولتحتهما'] = {
    'word': u'ولتحتهما',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَتَحْتَهُمَا',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فلتحتي'] = {
    'word': u'فلتحتي',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَتَحْتَِي',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'ِي'
}
STOPWORDS[u'فلتحتك'] = {
    'word': u'فلتحتك',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَتَحْتَكَ',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'كَ'
}
STOPWORDS[u'فلتحتك'] = {
    'word': u'فلتحتك',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَتَحْتَكِ',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'كِ'
}
STOPWORDS[u'فلتحته'] = {
    'word': u'فلتحته',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَتَحْتَه',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'ه'
}
STOPWORDS[u'فلتحتكم'] = {
    'word': u'فلتحتكم',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَتَحْتَكُمْ',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فلتحتكن'] = {
    'word': u'فلتحتكن',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَتَحْتَكُنَّ',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فلتحتها'] = {
    'word': u'فلتحتها',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَتَحْتَهَا',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'هَا'
}
STOPWORDS[u'فلتحتهم'] = {
    'word': u'فلتحتهم',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَتَحْتَهُمْ',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فلتحتهن'] = {
    'word': u'فلتحتهن',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَتَحْتَهُنَّ',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فلتحتنا'] = {
    'word': u'فلتحتنا',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَتَحْتَنَا',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'نَا'
}
STOPWORDS[u'فلتحتكما'] = {
    'word': u'فلتحتكما',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَتَحْتَكُمَا',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فلتحتهما'] = {
    'word': u'فلتحتهما',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَتَحْتَهُمَا',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أتحتي'] = {
    'word': u'أتحتي',
    'procletic': u'-أَ',
    'tags': u'أداة:ظرف مكان:مجرور:مضاف',
    'vocalized': u'أَتَحْتَِي',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أتحتك'] = {
    'word': u'أتحتك',
    'procletic': u'-أَ',
    'tags': u'أداة:ظرف مكان:مجرور:مضاف',
    'vocalized': u'أَتَحْتَكَ',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أتحتك'] = {
    'word': u'أتحتك',
    'procletic': u'-أَ',
    'tags': u'أداة:ظرف مكان:مجرور:مضاف',
    'vocalized': u'أَتَحْتَكِ',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أتحته'] = {
    'word': u'أتحته',
    'procletic': u'-أَ',
    'tags': u'أداة:ظرف مكان:مجرور:مضاف',
    'vocalized': u'أَتَحْتَه',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'ه'
}
STOPWORDS[u'أتحتكم'] = {
    'word': u'أتحتكم',
    'procletic': u'-أَ',
    'tags': u'أداة:ظرف مكان:مجرور:مضاف',
    'vocalized': u'أَتَحْتَكُمْ',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أتحتكن'] = {
    'word': u'أتحتكن',
    'procletic': u'-أَ',
    'tags': u'أداة:ظرف مكان:مجرور:مضاف',
    'vocalized': u'أَتَحْتَكُنَّ',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أتحتها'] = {
    'word': u'أتحتها',
    'procletic': u'-أَ',
    'tags': u'أداة:ظرف مكان:مجرور:مضاف',
    'vocalized': u'أَتَحْتَهَا',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أتحتهم'] = {
    'word': u'أتحتهم',
    'procletic': u'-أَ',
    'tags': u'أداة:ظرف مكان:مجرور:مضاف',
    'vocalized': u'أَتَحْتَهُمْ',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أتحتهن'] = {
    'word': u'أتحتهن',
    'procletic': u'-أَ',
    'tags': u'أداة:ظرف مكان:مجرور:مضاف',
    'vocalized': u'أَتَحْتَهُنَّ',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أتحتنا'] = {
    'word': u'أتحتنا',
    'procletic': u'-أَ',
    'tags': u'أداة:ظرف مكان:مجرور:مضاف',
    'vocalized': u'أَتَحْتَنَا',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أتحتكما'] = {
    'word': u'أتحتكما',
    'procletic': u'-أَ',
    'tags': u'أداة:ظرف مكان:مجرور:مضاف',
    'vocalized': u'أَتَحْتَكُمَا',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أتحتهما'] = {
    'word': u'أتحتهما',
    'procletic': u'-أَ',
    'tags': u'أداة:ظرف مكان:مجرور:مضاف',
    'vocalized': u'أَتَحْتَهُمَا',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ألتحتي'] = {
    'word': u'ألتحتي',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَتَحْتَِي',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'ِي'
}
STOPWORDS[u'ألتحتك'] = {
    'word': u'ألتحتك',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَتَحْتَكَ',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'كَ'
}
STOPWORDS[u'ألتحتك'] = {
    'word': u'ألتحتك',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَتَحْتَكِ',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'كِ'
}
STOPWORDS[u'ألتحته'] = {
    'word': u'ألتحته',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَتَحْتَه',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'ه'
}
STOPWORDS[u'ألتحتكم'] = {
    'word': u'ألتحتكم',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَتَحْتَكُمْ',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ألتحتكن'] = {
    'word': u'ألتحتكن',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَتَحْتَكُنَّ',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ألتحتها'] = {
    'word': u'ألتحتها',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَتَحْتَهَا',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'هَا'
}
STOPWORDS[u'ألتحتهم'] = {
    'word': u'ألتحتهم',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَتَحْتَهُمْ',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ألتحتهن'] = {
    'word': u'ألتحتهن',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَتَحْتَهُنَّ',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ألتحتنا'] = {
    'word': u'ألتحتنا',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَتَحْتَنَا',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'نَا'
}
STOPWORDS[u'ألتحتكما'] = {
    'word': u'ألتحتكما',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَتَحْتَكُمَا',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ألتحتهما'] = {
    'word': u'ألتحتهما',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَتَحْتَهُمَا',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أوتحتي'] = {
    'word': u'أوتحتي',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَتَحْتَِي',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أوتحتك'] = {
    'word': u'أوتحتك',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَتَحْتَكَ',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أوتحتك'] = {
    'word': u'أوتحتك',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَتَحْتَكِ',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أوتحته'] = {
    'word': u'أوتحته',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَتَحْتَه',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'ه'
}
STOPWORDS[u'أوتحتكم'] = {
    'word': u'أوتحتكم',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَتَحْتَكُمْ',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أوتحتكن'] = {
    'word': u'أوتحتكن',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَتَحْتَكُنَّ',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أوتحتها'] = {
    'word': u'أوتحتها',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَتَحْتَهَا',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أوتحتهم'] = {
    'word': u'أوتحتهم',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَتَحْتَهُمْ',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أوتحتهن'] = {
    'word': u'أوتحتهن',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَتَحْتَهُنَّ',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أوتحتنا'] = {
    'word': u'أوتحتنا',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَتَحْتَنَا',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أوتحتكما'] = {
    'word': u'أوتحتكما',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَتَحْتَكُمَا',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أوتحتهما'] = {
    'word': u'أوتحتهما',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَتَحْتَهُمَا',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أفتحتي'] = {
    'word': u'أفتحتي',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَتَحْتَِي',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أفتحتك'] = {
    'word': u'أفتحتك',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَتَحْتَكَ',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أفتحتك'] = {
    'word': u'أفتحتك',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَتَحْتَكِ',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أفتحته'] = {
    'word': u'أفتحته',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَتَحْتَه',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'ه'
}
STOPWORDS[u'أفتحتكم'] = {
    'word': u'أفتحتكم',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَتَحْتَكُمْ',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أفتحتكن'] = {
    'word': u'أفتحتكن',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَتَحْتَكُنَّ',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أفتحتها'] = {
    'word': u'أفتحتها',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَتَحْتَهَا',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أفتحتهم'] = {
    'word': u'أفتحتهم',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَتَحْتَهُمْ',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أفتحتهن'] = {
    'word': u'أفتحتهن',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَتَحْتَهُنَّ',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أفتحتنا'] = {
    'word': u'أفتحتنا',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَتَحْتَنَا',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أفتحتكما'] = {
    'word': u'أفتحتكما',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَتَحْتَكُمَا',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أفتحتهما'] = {
    'word': u'أفتحتهما',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَتَحْتَهُمَا',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أولتحتي'] = {
    'word': u'أولتحتي',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَتَحْتَِي',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أولتحتك'] = {
    'word': u'أولتحتك',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَتَحْتَكَ',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أولتحتك'] = {
    'word': u'أولتحتك',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَتَحْتَكِ',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أولتحته'] = {
    'word': u'أولتحته',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَتَحْتَه',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'ه'
}
STOPWORDS[u'أولتحتكم'] = {
    'word': u'أولتحتكم',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَتَحْتَكُمْ',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أولتحتكن'] = {
    'word': u'أولتحتكن',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَتَحْتَكُنَّ',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أولتحتها'] = {
    'word': u'أولتحتها',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَتَحْتَهَا',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أولتحتهم'] = {
    'word': u'أولتحتهم',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَتَحْتَهُمْ',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أولتحتهن'] = {
    'word': u'أولتحتهن',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَتَحْتَهُنَّ',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أولتحتنا'] = {
    'word': u'أولتحتنا',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَتَحْتَنَا',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أولتحتكما'] = {
    'word': u'أولتحتكما',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَتَحْتَكُمَا',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أولتحتهما'] = {
    'word': u'أولتحتهما',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَتَحْتَهُمَا',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أفلتحتي'] = {
    'word': u'أفلتحتي',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَتَحْتَِي',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أفلتحتك'] = {
    'word': u'أفلتحتك',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَتَحْتَكَ',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أفلتحتك'] = {
    'word': u'أفلتحتك',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَتَحْتَكِ',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أفلتحته'] = {
    'word': u'أفلتحته',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَتَحْتَه',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'ه'
}
STOPWORDS[u'أفلتحتكم'] = {
    'word': u'أفلتحتكم',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَتَحْتَكُمْ',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أفلتحتكن'] = {
    'word': u'أفلتحتكن',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَتَحْتَكُنَّ',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أفلتحتها'] = {
    'word': u'أفلتحتها',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَتَحْتَهَا',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أفلتحتهم'] = {
    'word': u'أفلتحتهم',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَتَحْتَهُمْ',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أفلتحتهن'] = {
    'word': u'أفلتحتهن',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَتَحْتَهُنَّ',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أفلتحتنا'] = {
    'word': u'أفلتحتنا',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَتَحْتَنَا',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أفلتحتكما'] = {
    'word': u'أفلتحتكما',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَتَحْتَكُمَا',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أفلتحتهما'] = {
    'word': u'أفلتحتهما',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَتَحْتَهُمَا',
    'stem': u'تَحْتَ',
    'type': u'STOPWORD',
    'original': u'تَحْتَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ثم'] = {
    'word': u'ثَمَّ',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان',
    'vocalized': u'ثَمَّ',
    'stem': u'ثَمَّ',
    'type': u'STOPWORD',
    'original': u'ثَمَّ',
    'encletic': u''
}
STOPWORDS[u'وثم'] = {
    'word': u'وثم',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف',
    'vocalized': u'وَثَمَّ',
    'stem': u'ثَمَّ',
    'type': u'STOPWORD',
    'original': u'ثَمَّ',
    'encletic': u''
}
STOPWORDS[u'فثم'] = {
    'word': u'فثم',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف',
    'vocalized': u'فَثَمَّ',
    'stem': u'ثَمَّ',
    'type': u'STOPWORD',
    'original': u'ثَمَّ',
    'encletic': u''
}
STOPWORDS[u'خلف'] = {
    'word': u'خَلْفَ',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان',
    'vocalized': u'خَلْفَ',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u''
}
STOPWORDS[u'لخلف'] = {
    'word': u'لخلف',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور',
    'vocalized': u'لَخَلْفَ',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u''
}
STOPWORDS[u'وخلف'] = {
    'word': u'وخلف',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف',
    'vocalized': u'وَخَلْفَ',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u''
}
STOPWORDS[u'فخلف'] = {
    'word': u'فخلف',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف',
    'vocalized': u'فَخَلْفَ',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u''
}
STOPWORDS[u'ولخلف'] = {
    'word': u'ولخلف',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'وَلَخَلْفَ',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u''
}
STOPWORDS[u'فلخلف'] = {
    'word': u'فلخلف',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'فَلَخَلْفَ',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u''
}
STOPWORDS[u'أخلف'] = {
    'word': u'أخلف',
    'procletic': u'-أَ',
    'tags': u'أداة:ظرف مكان:مجرور',
    'vocalized': u'أَخَلْفَ',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u''
}
STOPWORDS[u'ألخلف'] = {
    'word': u'ألخلف',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور',
    'vocalized': u'أَلَخَلْفَ',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u''
}
STOPWORDS[u'أوخلف'] = {
    'word': u'أوخلف',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور',
    'vocalized': u'أَوَخَلْفَ',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u''
}
STOPWORDS[u'أفخلف'] = {
    'word': u'أفخلف',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور',
    'vocalized': u'أَفَخَلْفَ',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u''
}
STOPWORDS[u'أولخلف'] = {
    'word': u'أولخلف',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَلَخَلْفَ',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u''
}
STOPWORDS[u'أفلخلف'] = {
    'word': u'أفلخلف',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَلَخَلْفَ',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u''
}
STOPWORDS[u'خلفي'] = {
    'word': u'خلفي',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'خَلْفَِي',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'ِي'
}
STOPWORDS[u'خلفك'] = {
    'word': u'خلفك',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'خَلْفَكَ',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'كَ'
}
STOPWORDS[u'خلفك'] = {
    'word': u'خلفك',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'خَلْفَكِ',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'كِ'
}
STOPWORDS[u'خلفه'] = {
    'word': u'خلفه',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'خَلْفَه',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'ه'
}
STOPWORDS[u'خلفكم'] = {
    'word': u'خلفكم',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'خَلْفَكُمْ',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'خلفكن'] = {
    'word': u'خلفكن',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'خَلْفَكُنَّ',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'خلفها'] = {
    'word': u'خلفها',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'خَلْفَهَا',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'هَا'
}
STOPWORDS[u'خلفهم'] = {
    'word': u'خلفهم',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'خَلْفَهُمْ',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'خلفهن'] = {
    'word': u'خلفهن',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'خَلْفَهُنَّ',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'خلفنا'] = {
    'word': u'خلفنا',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'خَلْفَنَا',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'نَا'
}
STOPWORDS[u'خلفكما'] = {
    'word': u'خلفكما',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'خَلْفَكُمَا',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'خلفهما'] = {
    'word': u'خلفهما',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'خَلْفَهُمَا',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'لخلفي'] = {
    'word': u'لخلفي',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَخَلْفَِي',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'ِي'
}
STOPWORDS[u'لخلفك'] = {
    'word': u'لخلفك',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَخَلْفَكَ',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'كَ'
}
STOPWORDS[u'لخلفك'] = {
    'word': u'لخلفك',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَخَلْفَكِ',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'كِ'
}
STOPWORDS[u'لخلفه'] = {
    'word': u'لخلفه',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَخَلْفَه',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'ه'
}
STOPWORDS[u'لخلفكم'] = {
    'word': u'لخلفكم',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَخَلْفَكُمْ',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'لخلفكن'] = {
    'word': u'لخلفكن',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَخَلْفَكُنَّ',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'لخلفها'] = {
    'word': u'لخلفها',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَخَلْفَهَا',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'هَا'
}
STOPWORDS[u'لخلفهم'] = {
    'word': u'لخلفهم',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَخَلْفَهُمْ',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'لخلفهن'] = {
    'word': u'لخلفهن',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَخَلْفَهُنَّ',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'لخلفنا'] = {
    'word': u'لخلفنا',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَخَلْفَنَا',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'نَا'
}
STOPWORDS[u'لخلفكما'] = {
    'word': u'لخلفكما',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَخَلْفَكُمَا',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'لخلفهما'] = {
    'word': u'لخلفهما',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَخَلْفَهُمَا',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'وخلفي'] = {
    'word': u'وخلفي',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَخَلْفَِي',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'ِي'
}
STOPWORDS[u'وخلفك'] = {
    'word': u'وخلفك',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَخَلْفَكَ',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'كَ'
}
STOPWORDS[u'وخلفك'] = {
    'word': u'وخلفك',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَخَلْفَكِ',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'كِ'
}
STOPWORDS[u'وخلفه'] = {
    'word': u'وخلفه',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَخَلْفَه',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'ه'
}
STOPWORDS[u'وخلفكم'] = {
    'word': u'وخلفكم',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَخَلْفَكُمْ',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'وخلفكن'] = {
    'word': u'وخلفكن',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَخَلْفَكُنَّ',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'وخلفها'] = {
    'word': u'وخلفها',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَخَلْفَهَا',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'هَا'
}
STOPWORDS[u'وخلفهم'] = {
    'word': u'وخلفهم',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَخَلْفَهُمْ',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'وخلفهن'] = {
    'word': u'وخلفهن',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَخَلْفَهُنَّ',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'وخلفنا'] = {
    'word': u'وخلفنا',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَخَلْفَنَا',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'نَا'
}
STOPWORDS[u'وخلفكما'] = {
    'word': u'وخلفكما',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَخَلْفَكُمَا',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'وخلفهما'] = {
    'word': u'وخلفهما',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَخَلْفَهُمَا',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فخلفي'] = {
    'word': u'فخلفي',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَخَلْفَِي',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'ِي'
}
STOPWORDS[u'فخلفك'] = {
    'word': u'فخلفك',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَخَلْفَكَ',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'كَ'
}
STOPWORDS[u'فخلفك'] = {
    'word': u'فخلفك',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَخَلْفَكِ',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'كِ'
}
STOPWORDS[u'فخلفه'] = {
    'word': u'فخلفه',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَخَلْفَه',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'ه'
}
STOPWORDS[u'فخلفكم'] = {
    'word': u'فخلفكم',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَخَلْفَكُمْ',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فخلفكن'] = {
    'word': u'فخلفكن',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَخَلْفَكُنَّ',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فخلفها'] = {
    'word': u'فخلفها',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَخَلْفَهَا',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'هَا'
}
STOPWORDS[u'فخلفهم'] = {
    'word': u'فخلفهم',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَخَلْفَهُمْ',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فخلفهن'] = {
    'word': u'فخلفهن',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَخَلْفَهُنَّ',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فخلفنا'] = {
    'word': u'فخلفنا',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَخَلْفَنَا',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'نَا'
}
STOPWORDS[u'فخلفكما'] = {
    'word': u'فخلفكما',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَخَلْفَكُمَا',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فخلفهما'] = {
    'word': u'فخلفهما',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَخَلْفَهُمَا',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ولخلفي'] = {
    'word': u'ولخلفي',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَخَلْفَِي',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'ِي'
}
STOPWORDS[u'ولخلفك'] = {
    'word': u'ولخلفك',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَخَلْفَكَ',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'كَ'
}
STOPWORDS[u'ولخلفك'] = {
    'word': u'ولخلفك',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَخَلْفَكِ',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'كِ'
}
STOPWORDS[u'ولخلفه'] = {
    'word': u'ولخلفه',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَخَلْفَه',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'ه'
}
STOPWORDS[u'ولخلفكم'] = {
    'word': u'ولخلفكم',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَخَلْفَكُمْ',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ولخلفكن'] = {
    'word': u'ولخلفكن',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَخَلْفَكُنَّ',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ولخلفها'] = {
    'word': u'ولخلفها',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَخَلْفَهَا',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'هَا'
}
STOPWORDS[u'ولخلفهم'] = {
    'word': u'ولخلفهم',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَخَلْفَهُمْ',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ولخلفهن'] = {
    'word': u'ولخلفهن',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَخَلْفَهُنَّ',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ولخلفنا'] = {
    'word': u'ولخلفنا',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَخَلْفَنَا',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'نَا'
}
STOPWORDS[u'ولخلفكما'] = {
    'word': u'ولخلفكما',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَخَلْفَكُمَا',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ولخلفهما'] = {
    'word': u'ولخلفهما',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَخَلْفَهُمَا',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فلخلفي'] = {
    'word': u'فلخلفي',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَخَلْفَِي',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'ِي'
}
STOPWORDS[u'فلخلفك'] = {
    'word': u'فلخلفك',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَخَلْفَكَ',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'كَ'
}
STOPWORDS[u'فلخلفك'] = {
    'word': u'فلخلفك',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَخَلْفَكِ',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'كِ'
}
STOPWORDS[u'فلخلفه'] = {
    'word': u'فلخلفه',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَخَلْفَه',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'ه'
}
STOPWORDS[u'فلخلفكم'] = {
    'word': u'فلخلفكم',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَخَلْفَكُمْ',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فلخلفكن'] = {
    'word': u'فلخلفكن',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَخَلْفَكُنَّ',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فلخلفها'] = {
    'word': u'فلخلفها',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَخَلْفَهَا',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'هَا'
}
STOPWORDS[u'فلخلفهم'] = {
    'word': u'فلخلفهم',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَخَلْفَهُمْ',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فلخلفهن'] = {
    'word': u'فلخلفهن',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَخَلْفَهُنَّ',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فلخلفنا'] = {
    'word': u'فلخلفنا',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَخَلْفَنَا',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'نَا'
}
STOPWORDS[u'فلخلفكما'] = {
    'word': u'فلخلفكما',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَخَلْفَكُمَا',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فلخلفهما'] = {
    'word': u'فلخلفهما',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَخَلْفَهُمَا',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أخلفي'] = {
    'word': u'أخلفي',
    'procletic': u'-أَ',
    'tags': u'أداة:ظرف مكان:مجرور:مضاف',
    'vocalized': u'أَخَلْفَِي',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أخلفك'] = {
    'word': u'أخلفك',
    'procletic': u'-أَ',
    'tags': u'أداة:ظرف مكان:مجرور:مضاف',
    'vocalized': u'أَخَلْفَكَ',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أخلفك'] = {
    'word': u'أخلفك',
    'procletic': u'-أَ',
    'tags': u'أداة:ظرف مكان:مجرور:مضاف',
    'vocalized': u'أَخَلْفَكِ',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أخلفه'] = {
    'word': u'أخلفه',
    'procletic': u'-أَ',
    'tags': u'أداة:ظرف مكان:مجرور:مضاف',
    'vocalized': u'أَخَلْفَه',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'ه'
}
STOPWORDS[u'أخلفكم'] = {
    'word': u'أخلفكم',
    'procletic': u'-أَ',
    'tags': u'أداة:ظرف مكان:مجرور:مضاف',
    'vocalized': u'أَخَلْفَكُمْ',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أخلفكن'] = {
    'word': u'أخلفكن',
    'procletic': u'-أَ',
    'tags': u'أداة:ظرف مكان:مجرور:مضاف',
    'vocalized': u'أَخَلْفَكُنَّ',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أخلفها'] = {
    'word': u'أخلفها',
    'procletic': u'-أَ',
    'tags': u'أداة:ظرف مكان:مجرور:مضاف',
    'vocalized': u'أَخَلْفَهَا',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أخلفهم'] = {
    'word': u'أخلفهم',
    'procletic': u'-أَ',
    'tags': u'أداة:ظرف مكان:مجرور:مضاف',
    'vocalized': u'أَخَلْفَهُمْ',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أخلفهن'] = {
    'word': u'أخلفهن',
    'procletic': u'-أَ',
    'tags': u'أداة:ظرف مكان:مجرور:مضاف',
    'vocalized': u'أَخَلْفَهُنَّ',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أخلفنا'] = {
    'word': u'أخلفنا',
    'procletic': u'-أَ',
    'tags': u'أداة:ظرف مكان:مجرور:مضاف',
    'vocalized': u'أَخَلْفَنَا',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أخلفكما'] = {
    'word': u'أخلفكما',
    'procletic': u'-أَ',
    'tags': u'أداة:ظرف مكان:مجرور:مضاف',
    'vocalized': u'أَخَلْفَكُمَا',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أخلفهما'] = {
    'word': u'أخلفهما',
    'procletic': u'-أَ',
    'tags': u'أداة:ظرف مكان:مجرور:مضاف',
    'vocalized': u'أَخَلْفَهُمَا',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ألخلفي'] = {
    'word': u'ألخلفي',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَخَلْفَِي',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'ِي'
}
STOPWORDS[u'ألخلفك'] = {
    'word': u'ألخلفك',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَخَلْفَكَ',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'كَ'
}
STOPWORDS[u'ألخلفك'] = {
    'word': u'ألخلفك',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَخَلْفَكِ',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'كِ'
}
STOPWORDS[u'ألخلفه'] = {
    'word': u'ألخلفه',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَخَلْفَه',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'ه'
}
STOPWORDS[u'ألخلفكم'] = {
    'word': u'ألخلفكم',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَخَلْفَكُمْ',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ألخلفكن'] = {
    'word': u'ألخلفكن',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَخَلْفَكُنَّ',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ألخلفها'] = {
    'word': u'ألخلفها',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَخَلْفَهَا',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'هَا'
}
STOPWORDS[u'ألخلفهم'] = {
    'word': u'ألخلفهم',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَخَلْفَهُمْ',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ألخلفهن'] = {
    'word': u'ألخلفهن',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَخَلْفَهُنَّ',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ألخلفنا'] = {
    'word': u'ألخلفنا',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَخَلْفَنَا',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'نَا'
}
STOPWORDS[u'ألخلفكما'] = {
    'word': u'ألخلفكما',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَخَلْفَكُمَا',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ألخلفهما'] = {
    'word': u'ألخلفهما',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَخَلْفَهُمَا',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أوخلفي'] = {
    'word': u'أوخلفي',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَخَلْفَِي',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أوخلفك'] = {
    'word': u'أوخلفك',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَخَلْفَكَ',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أوخلفك'] = {
    'word': u'أوخلفك',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَخَلْفَكِ',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أوخلفه'] = {
    'word': u'أوخلفه',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَخَلْفَه',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'ه'
}
STOPWORDS[u'أوخلفكم'] = {
    'word': u'أوخلفكم',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَخَلْفَكُمْ',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أوخلفكن'] = {
    'word': u'أوخلفكن',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَخَلْفَكُنَّ',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أوخلفها'] = {
    'word': u'أوخلفها',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَخَلْفَهَا',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أوخلفهم'] = {
    'word': u'أوخلفهم',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَخَلْفَهُمْ',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أوخلفهن'] = {
    'word': u'أوخلفهن',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَخَلْفَهُنَّ',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أوخلفنا'] = {
    'word': u'أوخلفنا',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَخَلْفَنَا',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أوخلفكما'] = {
    'word': u'أوخلفكما',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَخَلْفَكُمَا',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أوخلفهما'] = {
    'word': u'أوخلفهما',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَخَلْفَهُمَا',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أفخلفي'] = {
    'word': u'أفخلفي',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَخَلْفَِي',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أفخلفك'] = {
    'word': u'أفخلفك',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَخَلْفَكَ',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أفخلفك'] = {
    'word': u'أفخلفك',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَخَلْفَكِ',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أفخلفه'] = {
    'word': u'أفخلفه',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَخَلْفَه',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'ه'
}
STOPWORDS[u'أفخلفكم'] = {
    'word': u'أفخلفكم',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَخَلْفَكُمْ',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أفخلفكن'] = {
    'word': u'أفخلفكن',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَخَلْفَكُنَّ',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أفخلفها'] = {
    'word': u'أفخلفها',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَخَلْفَهَا',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أفخلفهم'] = {
    'word': u'أفخلفهم',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَخَلْفَهُمْ',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أفخلفهن'] = {
    'word': u'أفخلفهن',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَخَلْفَهُنَّ',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أفخلفنا'] = {
    'word': u'أفخلفنا',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَخَلْفَنَا',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أفخلفكما'] = {
    'word': u'أفخلفكما',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَخَلْفَكُمَا',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أفخلفهما'] = {
    'word': u'أفخلفهما',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَخَلْفَهُمَا',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أولخلفي'] = {
    'word': u'أولخلفي',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَخَلْفَِي',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أولخلفك'] = {
    'word': u'أولخلفك',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَخَلْفَكَ',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أولخلفك'] = {
    'word': u'أولخلفك',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَخَلْفَكِ',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أولخلفه'] = {
    'word': u'أولخلفه',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَخَلْفَه',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'ه'
}
STOPWORDS[u'أولخلفكم'] = {
    'word': u'أولخلفكم',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَخَلْفَكُمْ',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أولخلفكن'] = {
    'word': u'أولخلفكن',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَخَلْفَكُنَّ',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أولخلفها'] = {
    'word': u'أولخلفها',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَخَلْفَهَا',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أولخلفهم'] = {
    'word': u'أولخلفهم',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَخَلْفَهُمْ',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أولخلفهن'] = {
    'word': u'أولخلفهن',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَخَلْفَهُنَّ',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أولخلفنا'] = {
    'word': u'أولخلفنا',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَخَلْفَنَا',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أولخلفكما'] = {
    'word': u'أولخلفكما',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَخَلْفَكُمَا',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أولخلفهما'] = {
    'word': u'أولخلفهما',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَخَلْفَهُمَا',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أفلخلفي'] = {
    'word': u'أفلخلفي',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَخَلْفَِي',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أفلخلفك'] = {
    'word': u'أفلخلفك',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَخَلْفَكَ',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أفلخلفك'] = {
    'word': u'أفلخلفك',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَخَلْفَكِ',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أفلخلفه'] = {
    'word': u'أفلخلفه',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَخَلْفَه',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'ه'
}
STOPWORDS[u'أفلخلفكم'] = {
    'word': u'أفلخلفكم',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَخَلْفَكُمْ',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أفلخلفكن'] = {
    'word': u'أفلخلفكن',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَخَلْفَكُنَّ',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أفلخلفها'] = {
    'word': u'أفلخلفها',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَخَلْفَهَا',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أفلخلفهم'] = {
    'word': u'أفلخلفهم',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَخَلْفَهُمْ',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أفلخلفهن'] = {
    'word': u'أفلخلفهن',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَخَلْفَهُنَّ',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أفلخلفنا'] = {
    'word': u'أفلخلفنا',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَخَلْفَنَا',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أفلخلفكما'] = {
    'word': u'أفلخلفكما',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَخَلْفَكُمَا',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أفلخلفهما'] = {
    'word': u'أفلخلفهما',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَخَلْفَهُمَا',
    'stem': u'خَلْفَ',
    'type': u'STOPWORD',
    'original': u'خَلْفَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'شمال'] = {
    'word': u'شَمَالَ',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان',
    'vocalized': u'شَمَالَ',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u''
}
STOPWORDS[u'لشمال'] = {
    'word': u'لشمال',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور',
    'vocalized': u'لَشَمَالَ',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u''
}
STOPWORDS[u'وشمال'] = {
    'word': u'وشمال',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف',
    'vocalized': u'وَشَمَالَ',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u''
}
STOPWORDS[u'فشمال'] = {
    'word': u'فشمال',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف',
    'vocalized': u'فَشَمَالَ',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u''
}
STOPWORDS[u'ولشمال'] = {
    'word': u'ولشمال',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'وَلَشَمَالَ',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u''
}
STOPWORDS[u'فلشمال'] = {
    'word': u'فلشمال',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'فَلَشَمَالَ',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u''
}
STOPWORDS[u'أشمال'] = {
    'word': u'أشمال',
    'procletic': u'-أَ',
    'tags': u'أداة:ظرف مكان:مجرور',
    'vocalized': u'أَشَمَالَ',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u''
}
STOPWORDS[u'ألشمال'] = {
    'word': u'ألشمال',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور',
    'vocalized': u'أَلَشَمَالَ',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u''
}
STOPWORDS[u'أوشمال'] = {
    'word': u'أوشمال',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور',
    'vocalized': u'أَوَشَمَالَ',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u''
}
STOPWORDS[u'أفشمال'] = {
    'word': u'أفشمال',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور',
    'vocalized': u'أَفَشَمَالَ',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u''
}
STOPWORDS[u'أولشمال'] = {
    'word': u'أولشمال',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَلَشَمَالَ',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u''
}
STOPWORDS[u'أفلشمال'] = {
    'word': u'أفلشمال',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَلَشَمَالَ',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u''
}
STOPWORDS[u'شمالي'] = {
    'word': u'شمالي',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'شَمَالَِي',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'ِي'
}
STOPWORDS[u'شمالك'] = {
    'word': u'شمالك',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'شَمَالَكَ',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'كَ'
}
STOPWORDS[u'شمالك'] = {
    'word': u'شمالك',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'شَمَالَكِ',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'كِ'
}
STOPWORDS[u'شماله'] = {
    'word': u'شماله',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'شَمَالَه',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'ه'
}
STOPWORDS[u'شمالكم'] = {
    'word': u'شمالكم',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'شَمَالَكُمْ',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'شمالكن'] = {
    'word': u'شمالكن',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'شَمَالَكُنَّ',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'شمالها'] = {
    'word': u'شمالها',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'شَمَالَهَا',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'هَا'
}
STOPWORDS[u'شمالهم'] = {
    'word': u'شمالهم',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'شَمَالَهُمْ',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'شمالهن'] = {
    'word': u'شمالهن',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'شَمَالَهُنَّ',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'شمالنا'] = {
    'word': u'شمالنا',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'شَمَالَنَا',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'نَا'
}
STOPWORDS[u'شمالكما'] = {
    'word': u'شمالكما',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'شَمَالَكُمَا',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'شمالهما'] = {
    'word': u'شمالهما',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'شَمَالَهُمَا',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'لشمالي'] = {
    'word': u'لشمالي',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَشَمَالَِي',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'ِي'
}
STOPWORDS[u'لشمالك'] = {
    'word': u'لشمالك',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَشَمَالَكَ',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'كَ'
}
STOPWORDS[u'لشمالك'] = {
    'word': u'لشمالك',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَشَمَالَكِ',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'كِ'
}
STOPWORDS[u'لشماله'] = {
    'word': u'لشماله',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَشَمَالَه',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'ه'
}
STOPWORDS[u'لشمالكم'] = {
    'word': u'لشمالكم',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَشَمَالَكُمْ',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'لشمالكن'] = {
    'word': u'لشمالكن',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَشَمَالَكُنَّ',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'لشمالها'] = {
    'word': u'لشمالها',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَشَمَالَهَا',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'هَا'
}
STOPWORDS[u'لشمالهم'] = {
    'word': u'لشمالهم',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَشَمَالَهُمْ',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'لشمالهن'] = {
    'word': u'لشمالهن',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَشَمَالَهُنَّ',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'لشمالنا'] = {
    'word': u'لشمالنا',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَشَمَالَنَا',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'نَا'
}
STOPWORDS[u'لشمالكما'] = {
    'word': u'لشمالكما',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَشَمَالَكُمَا',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'لشمالهما'] = {
    'word': u'لشمالهما',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَشَمَالَهُمَا',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'وشمالي'] = {
    'word': u'وشمالي',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَشَمَالَِي',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'ِي'
}
STOPWORDS[u'وشمالك'] = {
    'word': u'وشمالك',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَشَمَالَكَ',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'كَ'
}
STOPWORDS[u'وشمالك'] = {
    'word': u'وشمالك',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَشَمَالَكِ',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'كِ'
}
STOPWORDS[u'وشماله'] = {
    'word': u'وشماله',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَشَمَالَه',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'ه'
}
STOPWORDS[u'وشمالكم'] = {
    'word': u'وشمالكم',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَشَمَالَكُمْ',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'وشمالكن'] = {
    'word': u'وشمالكن',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَشَمَالَكُنَّ',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'وشمالها'] = {
    'word': u'وشمالها',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَشَمَالَهَا',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'هَا'
}
STOPWORDS[u'وشمالهم'] = {
    'word': u'وشمالهم',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَشَمَالَهُمْ',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'وشمالهن'] = {
    'word': u'وشمالهن',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَشَمَالَهُنَّ',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'وشمالنا'] = {
    'word': u'وشمالنا',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَشَمَالَنَا',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'نَا'
}
STOPWORDS[u'وشمالكما'] = {
    'word': u'وشمالكما',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَشَمَالَكُمَا',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'وشمالهما'] = {
    'word': u'وشمالهما',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَشَمَالَهُمَا',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فشمالي'] = {
    'word': u'فشمالي',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَشَمَالَِي',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'ِي'
}
STOPWORDS[u'فشمالك'] = {
    'word': u'فشمالك',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَشَمَالَكَ',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'كَ'
}
STOPWORDS[u'فشمالك'] = {
    'word': u'فشمالك',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَشَمَالَكِ',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'كِ'
}
STOPWORDS[u'فشماله'] = {
    'word': u'فشماله',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَشَمَالَه',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'ه'
}
STOPWORDS[u'فشمالكم'] = {
    'word': u'فشمالكم',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَشَمَالَكُمْ',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فشمالكن'] = {
    'word': u'فشمالكن',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَشَمَالَكُنَّ',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فشمالها'] = {
    'word': u'فشمالها',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَشَمَالَهَا',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'هَا'
}
STOPWORDS[u'فشمالهم'] = {
    'word': u'فشمالهم',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَشَمَالَهُمْ',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فشمالهن'] = {
    'word': u'فشمالهن',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَشَمَالَهُنَّ',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فشمالنا'] = {
    'word': u'فشمالنا',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَشَمَالَنَا',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'نَا'
}
STOPWORDS[u'فشمالكما'] = {
    'word': u'فشمالكما',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَشَمَالَكُمَا',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فشمالهما'] = {
    'word': u'فشمالهما',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَشَمَالَهُمَا',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ولشمالي'] = {
    'word': u'ولشمالي',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَشَمَالَِي',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'ِي'
}
STOPWORDS[u'ولشمالك'] = {
    'word': u'ولشمالك',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَشَمَالَكَ',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'كَ'
}
STOPWORDS[u'ولشمالك'] = {
    'word': u'ولشمالك',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَشَمَالَكِ',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'كِ'
}
STOPWORDS[u'ولشماله'] = {
    'word': u'ولشماله',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَشَمَالَه',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'ه'
}
STOPWORDS[u'ولشمالكم'] = {
    'word': u'ولشمالكم',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَشَمَالَكُمْ',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ولشمالكن'] = {
    'word': u'ولشمالكن',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَشَمَالَكُنَّ',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ولشمالها'] = {
    'word': u'ولشمالها',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَشَمَالَهَا',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'هَا'
}
STOPWORDS[u'ولشمالهم'] = {
    'word': u'ولشمالهم',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَشَمَالَهُمْ',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ولشمالهن'] = {
    'word': u'ولشمالهن',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَشَمَالَهُنَّ',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ولشمالنا'] = {
    'word': u'ولشمالنا',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَشَمَالَنَا',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'نَا'
}
STOPWORDS[u'ولشمالكما'] = {
    'word': u'ولشمالكما',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَشَمَالَكُمَا',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ولشمالهما'] = {
    'word': u'ولشمالهما',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَشَمَالَهُمَا',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فلشمالي'] = {
    'word': u'فلشمالي',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَشَمَالَِي',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'ِي'
}
STOPWORDS[u'فلشمالك'] = {
    'word': u'فلشمالك',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَشَمَالَكَ',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'كَ'
}
STOPWORDS[u'فلشمالك'] = {
    'word': u'فلشمالك',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَشَمَالَكِ',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'كِ'
}
STOPWORDS[u'فلشماله'] = {
    'word': u'فلشماله',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَشَمَالَه',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'ه'
}
STOPWORDS[u'فلشمالكم'] = {
    'word': u'فلشمالكم',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَشَمَالَكُمْ',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فلشمالكن'] = {
    'word': u'فلشمالكن',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَشَمَالَكُنَّ',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فلشمالها'] = {
    'word': u'فلشمالها',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَشَمَالَهَا',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'هَا'
}
STOPWORDS[u'فلشمالهم'] = {
    'word': u'فلشمالهم',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَشَمَالَهُمْ',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فلشمالهن'] = {
    'word': u'فلشمالهن',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَشَمَالَهُنَّ',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فلشمالنا'] = {
    'word': u'فلشمالنا',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَشَمَالَنَا',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'نَا'
}
STOPWORDS[u'فلشمالكما'] = {
    'word': u'فلشمالكما',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَشَمَالَكُمَا',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فلشمالهما'] = {
    'word': u'فلشمالهما',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَشَمَالَهُمَا',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أشمالي'] = {
    'word': u'أشمالي',
    'procletic': u'-أَ',
    'tags': u'أداة:ظرف مكان:مجرور:مضاف',
    'vocalized': u'أَشَمَالَِي',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أشمالك'] = {
    'word': u'أشمالك',
    'procletic': u'-أَ',
    'tags': u'أداة:ظرف مكان:مجرور:مضاف',
    'vocalized': u'أَشَمَالَكَ',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أشمالك'] = {
    'word': u'أشمالك',
    'procletic': u'-أَ',
    'tags': u'أداة:ظرف مكان:مجرور:مضاف',
    'vocalized': u'أَشَمَالَكِ',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أشماله'] = {
    'word': u'أشماله',
    'procletic': u'-أَ',
    'tags': u'أداة:ظرف مكان:مجرور:مضاف',
    'vocalized': u'أَشَمَالَه',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'ه'
}
STOPWORDS[u'أشمالكم'] = {
    'word': u'أشمالكم',
    'procletic': u'-أَ',
    'tags': u'أداة:ظرف مكان:مجرور:مضاف',
    'vocalized': u'أَشَمَالَكُمْ',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أشمالكن'] = {
    'word': u'أشمالكن',
    'procletic': u'-أَ',
    'tags': u'أداة:ظرف مكان:مجرور:مضاف',
    'vocalized': u'أَشَمَالَكُنَّ',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أشمالها'] = {
    'word': u'أشمالها',
    'procletic': u'-أَ',
    'tags': u'أداة:ظرف مكان:مجرور:مضاف',
    'vocalized': u'أَشَمَالَهَا',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أشمالهم'] = {
    'word': u'أشمالهم',
    'procletic': u'-أَ',
    'tags': u'أداة:ظرف مكان:مجرور:مضاف',
    'vocalized': u'أَشَمَالَهُمْ',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أشمالهن'] = {
    'word': u'أشمالهن',
    'procletic': u'-أَ',
    'tags': u'أداة:ظرف مكان:مجرور:مضاف',
    'vocalized': u'أَشَمَالَهُنَّ',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أشمالنا'] = {
    'word': u'أشمالنا',
    'procletic': u'-أَ',
    'tags': u'أداة:ظرف مكان:مجرور:مضاف',
    'vocalized': u'أَشَمَالَنَا',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أشمالكما'] = {
    'word': u'أشمالكما',
    'procletic': u'-أَ',
    'tags': u'أداة:ظرف مكان:مجرور:مضاف',
    'vocalized': u'أَشَمَالَكُمَا',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أشمالهما'] = {
    'word': u'أشمالهما',
    'procletic': u'-أَ',
    'tags': u'أداة:ظرف مكان:مجرور:مضاف',
    'vocalized': u'أَشَمَالَهُمَا',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ألشمالي'] = {
    'word': u'ألشمالي',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَشَمَالَِي',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'ِي'
}
STOPWORDS[u'ألشمالك'] = {
    'word': u'ألشمالك',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَشَمَالَكَ',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'كَ'
}
STOPWORDS[u'ألشمالك'] = {
    'word': u'ألشمالك',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَشَمَالَكِ',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'كِ'
}
STOPWORDS[u'ألشماله'] = {
    'word': u'ألشماله',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَشَمَالَه',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'ه'
}
STOPWORDS[u'ألشمالكم'] = {
    'word': u'ألشمالكم',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَشَمَالَكُمْ',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ألشمالكن'] = {
    'word': u'ألشمالكن',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَشَمَالَكُنَّ',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ألشمالها'] = {
    'word': u'ألشمالها',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَشَمَالَهَا',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'هَا'
}
STOPWORDS[u'ألشمالهم'] = {
    'word': u'ألشمالهم',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَشَمَالَهُمْ',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ألشمالهن'] = {
    'word': u'ألشمالهن',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَشَمَالَهُنَّ',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ألشمالنا'] = {
    'word': u'ألشمالنا',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَشَمَالَنَا',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'نَا'
}
STOPWORDS[u'ألشمالكما'] = {
    'word': u'ألشمالكما',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَشَمَالَكُمَا',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ألشمالهما'] = {
    'word': u'ألشمالهما',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَشَمَالَهُمَا',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أوشمالي'] = {
    'word': u'أوشمالي',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَشَمَالَِي',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أوشمالك'] = {
    'word': u'أوشمالك',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَشَمَالَكَ',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أوشمالك'] = {
    'word': u'أوشمالك',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَشَمَالَكِ',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أوشماله'] = {
    'word': u'أوشماله',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَشَمَالَه',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'ه'
}
STOPWORDS[u'أوشمالكم'] = {
    'word': u'أوشمالكم',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَشَمَالَكُمْ',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أوشمالكن'] = {
    'word': u'أوشمالكن',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَشَمَالَكُنَّ',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أوشمالها'] = {
    'word': u'أوشمالها',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَشَمَالَهَا',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أوشمالهم'] = {
    'word': u'أوشمالهم',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَشَمَالَهُمْ',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أوشمالهن'] = {
    'word': u'أوشمالهن',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَشَمَالَهُنَّ',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أوشمالنا'] = {
    'word': u'أوشمالنا',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَشَمَالَنَا',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أوشمالكما'] = {
    'word': u'أوشمالكما',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَشَمَالَكُمَا',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أوشمالهما'] = {
    'word': u'أوشمالهما',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَشَمَالَهُمَا',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أفشمالي'] = {
    'word': u'أفشمالي',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَشَمَالَِي',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أفشمالك'] = {
    'word': u'أفشمالك',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَشَمَالَكَ',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أفشمالك'] = {
    'word': u'أفشمالك',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَشَمَالَكِ',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أفشماله'] = {
    'word': u'أفشماله',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَشَمَالَه',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'ه'
}
STOPWORDS[u'أفشمالكم'] = {
    'word': u'أفشمالكم',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَشَمَالَكُمْ',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أفشمالكن'] = {
    'word': u'أفشمالكن',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَشَمَالَكُنَّ',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أفشمالها'] = {
    'word': u'أفشمالها',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَشَمَالَهَا',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أفشمالهم'] = {
    'word': u'أفشمالهم',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَشَمَالَهُمْ',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أفشمالهن'] = {
    'word': u'أفشمالهن',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَشَمَالَهُنَّ',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أفشمالنا'] = {
    'word': u'أفشمالنا',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَشَمَالَنَا',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أفشمالكما'] = {
    'word': u'أفشمالكما',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَشَمَالَكُمَا',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أفشمالهما'] = {
    'word': u'أفشمالهما',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَشَمَالَهُمَا',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أولشمالي'] = {
    'word': u'أولشمالي',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَشَمَالَِي',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أولشمالك'] = {
    'word': u'أولشمالك',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَشَمَالَكَ',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أولشمالك'] = {
    'word': u'أولشمالك',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَشَمَالَكِ',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أولشماله'] = {
    'word': u'أولشماله',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَشَمَالَه',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'ه'
}
STOPWORDS[u'أولشمالكم'] = {
    'word': u'أولشمالكم',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَشَمَالَكُمْ',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أولشمالكن'] = {
    'word': u'أولشمالكن',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَشَمَالَكُنَّ',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أولشمالها'] = {
    'word': u'أولشمالها',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَشَمَالَهَا',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أولشمالهم'] = {
    'word': u'أولشمالهم',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَشَمَالَهُمْ',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أولشمالهن'] = {
    'word': u'أولشمالهن',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَشَمَالَهُنَّ',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أولشمالنا'] = {
    'word': u'أولشمالنا',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَشَمَالَنَا',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أولشمالكما'] = {
    'word': u'أولشمالكما',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَشَمَالَكُمَا',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أولشمالهما'] = {
    'word': u'أولشمالهما',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَشَمَالَهُمَا',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أفلشمالي'] = {
    'word': u'أفلشمالي',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَشَمَالَِي',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أفلشمالك'] = {
    'word': u'أفلشمالك',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَشَمَالَكَ',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أفلشمالك'] = {
    'word': u'أفلشمالك',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَشَمَالَكِ',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أفلشماله'] = {
    'word': u'أفلشماله',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَشَمَالَه',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'ه'
}
STOPWORDS[u'أفلشمالكم'] = {
    'word': u'أفلشمالكم',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَشَمَالَكُمْ',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أفلشمالكن'] = {
    'word': u'أفلشمالكن',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَشَمَالَكُنَّ',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أفلشمالها'] = {
    'word': u'أفلشمالها',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَشَمَالَهَا',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أفلشمالهم'] = {
    'word': u'أفلشمالهم',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَشَمَالَهُمْ',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أفلشمالهن'] = {
    'word': u'أفلشمالهن',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَشَمَالَهُنَّ',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أفلشمالنا'] = {
    'word': u'أفلشمالنا',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَشَمَالَنَا',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أفلشمالكما'] = {
    'word': u'أفلشمالكما',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَشَمَالَكُمَا',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أفلشمالهما'] = {
    'word': u'أفلشمالهما',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَشَمَالَهُمَا',
    'stem': u'شَمَالَ',
    'type': u'STOPWORD',
    'original': u'شَمَالَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ضمن'] = {
    'word': u'ضِمْنَ',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان',
    'vocalized': u'ضِمْنَ',
    'stem': u'ضِمْنَ',
    'type': u'STOPWORD',
    'original': u'ضِمْنَ',
    'encletic': u''
}
STOPWORDS[u'وضمن'] = {
    'word': u'وضمن',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف',
    'vocalized': u'وَضِمْنَ',
    'stem': u'ضِمْنَ',
    'type': u'STOPWORD',
    'original': u'ضِمْنَ',
    'encletic': u''
}
STOPWORDS[u'فضمن'] = {
    'word': u'فضمن',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف',
    'vocalized': u'فَضِمْنَ',
    'stem': u'ضِمْنَ',
    'type': u'STOPWORD',
    'original': u'ضِمْنَ',
    'encletic': u''
}
STOPWORDS[u'ضمني'] = {
    'word': u'ضمني',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'ضِمْنَِي',
    'stem': u'ضِمْنَ',
    'type': u'STOPWORD',
    'original': u'ضِمْنَ',
    'encletic': u'ِي'
}
STOPWORDS[u'ضمنك'] = {
    'word': u'ضمنك',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'ضِمْنَكَ',
    'stem': u'ضِمْنَ',
    'type': u'STOPWORD',
    'original': u'ضِمْنَ',
    'encletic': u'كَ'
}
STOPWORDS[u'ضمنك'] = {
    'word': u'ضمنك',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'ضِمْنَكِ',
    'stem': u'ضِمْنَ',
    'type': u'STOPWORD',
    'original': u'ضِمْنَ',
    'encletic': u'كِ'
}
STOPWORDS[u'ضمنه'] = {
    'word': u'ضمنه',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'ضِمْنَه',
    'stem': u'ضِمْنَ',
    'type': u'STOPWORD',
    'original': u'ضِمْنَ',
    'encletic': u'ه'
}
STOPWORDS[u'ضمنكم'] = {
    'word': u'ضمنكم',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'ضِمْنَكُمْ',
    'stem': u'ضِمْنَ',
    'type': u'STOPWORD',
    'original': u'ضِمْنَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ضمنكن'] = {
    'word': u'ضمنكن',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'ضِمْنَكُنَّ',
    'stem': u'ضِمْنَ',
    'type': u'STOPWORD',
    'original': u'ضِمْنَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ضمنها'] = {
    'word': u'ضمنها',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'ضِمْنَهَا',
    'stem': u'ضِمْنَ',
    'type': u'STOPWORD',
    'original': u'ضِمْنَ',
    'encletic': u'هَا'
}
STOPWORDS[u'ضمنهم'] = {
    'word': u'ضمنهم',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'ضِمْنَهُمْ',
    'stem': u'ضِمْنَ',
    'type': u'STOPWORD',
    'original': u'ضِمْنَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ضمنهن'] = {
    'word': u'ضمنهن',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'ضِمْنَهُنَّ',
    'stem': u'ضِمْنَ',
    'type': u'STOPWORD',
    'original': u'ضِمْنَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ضمننا'] = {
    'word': u'ضمننا',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'ضِمْنَنَا',
    'stem': u'ضِمْنَ',
    'type': u'STOPWORD',
    'original': u'ضِمْنَ',
    'encletic': u'نَا'
}
STOPWORDS[u'ضمنكما'] = {
    'word': u'ضمنكما',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'ضِمْنَكُمَا',
    'stem': u'ضِمْنَ',
    'type': u'STOPWORD',
    'original': u'ضِمْنَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ضمنهما'] = {
    'word': u'ضمنهما',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'ضِمْنَهُمَا',
    'stem': u'ضِمْنَ',
    'type': u'STOPWORD',
    'original': u'ضِمْنَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'وضمني'] = {
    'word': u'وضمني',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَضِمْنَِي',
    'stem': u'ضِمْنَ',
    'type': u'STOPWORD',
    'original': u'ضِمْنَ',
    'encletic': u'ِي'
}
STOPWORDS[u'وضمنك'] = {
    'word': u'وضمنك',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَضِمْنَكَ',
    'stem': u'ضِمْنَ',
    'type': u'STOPWORD',
    'original': u'ضِمْنَ',
    'encletic': u'كَ'
}
STOPWORDS[u'وضمنك'] = {
    'word': u'وضمنك',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَضِمْنَكِ',
    'stem': u'ضِمْنَ',
    'type': u'STOPWORD',
    'original': u'ضِمْنَ',
    'encletic': u'كِ'
}
STOPWORDS[u'وضمنه'] = {
    'word': u'وضمنه',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَضِمْنَه',
    'stem': u'ضِمْنَ',
    'type': u'STOPWORD',
    'original': u'ضِمْنَ',
    'encletic': u'ه'
}
STOPWORDS[u'وضمنكم'] = {
    'word': u'وضمنكم',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَضِمْنَكُمْ',
    'stem': u'ضِمْنَ',
    'type': u'STOPWORD',
    'original': u'ضِمْنَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'وضمنكن'] = {
    'word': u'وضمنكن',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَضِمْنَكُنَّ',
    'stem': u'ضِمْنَ',
    'type': u'STOPWORD',
    'original': u'ضِمْنَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'وضمنها'] = {
    'word': u'وضمنها',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَضِمْنَهَا',
    'stem': u'ضِمْنَ',
    'type': u'STOPWORD',
    'original': u'ضِمْنَ',
    'encletic': u'هَا'
}
STOPWORDS[u'وضمنهم'] = {
    'word': u'وضمنهم',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَضِمْنَهُمْ',
    'stem': u'ضِمْنَ',
    'type': u'STOPWORD',
    'original': u'ضِمْنَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'وضمنهن'] = {
    'word': u'وضمنهن',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَضِمْنَهُنَّ',
    'stem': u'ضِمْنَ',
    'type': u'STOPWORD',
    'original': u'ضِمْنَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'وضمننا'] = {
    'word': u'وضمننا',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَضِمْنَنَا',
    'stem': u'ضِمْنَ',
    'type': u'STOPWORD',
    'original': u'ضِمْنَ',
    'encletic': u'نَا'
}
STOPWORDS[u'وضمنكما'] = {
    'word': u'وضمنكما',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَضِمْنَكُمَا',
    'stem': u'ضِمْنَ',
    'type': u'STOPWORD',
    'original': u'ضِمْنَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'وضمنهما'] = {
    'word': u'وضمنهما',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَضِمْنَهُمَا',
    'stem': u'ضِمْنَ',
    'type': u'STOPWORD',
    'original': u'ضِمْنَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فضمني'] = {
    'word': u'فضمني',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَضِمْنَِي',
    'stem': u'ضِمْنَ',
    'type': u'STOPWORD',
    'original': u'ضِمْنَ',
    'encletic': u'ِي'
}
STOPWORDS[u'فضمنك'] = {
    'word': u'فضمنك',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَضِمْنَكَ',
    'stem': u'ضِمْنَ',
    'type': u'STOPWORD',
    'original': u'ضِمْنَ',
    'encletic': u'كَ'
}
STOPWORDS[u'فضمنك'] = {
    'word': u'فضمنك',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَضِمْنَكِ',
    'stem': u'ضِمْنَ',
    'type': u'STOPWORD',
    'original': u'ضِمْنَ',
    'encletic': u'كِ'
}
STOPWORDS[u'فضمنه'] = {
    'word': u'فضمنه',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَضِمْنَه',
    'stem': u'ضِمْنَ',
    'type': u'STOPWORD',
    'original': u'ضِمْنَ',
    'encletic': u'ه'
}
STOPWORDS[u'فضمنكم'] = {
    'word': u'فضمنكم',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَضِمْنَكُمْ',
    'stem': u'ضِمْنَ',
    'type': u'STOPWORD',
    'original': u'ضِمْنَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فضمنكن'] = {
    'word': u'فضمنكن',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَضِمْنَكُنَّ',
    'stem': u'ضِمْنَ',
    'type': u'STOPWORD',
    'original': u'ضِمْنَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فضمنها'] = {
    'word': u'فضمنها',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَضِمْنَهَا',
    'stem': u'ضِمْنَ',
    'type': u'STOPWORD',
    'original': u'ضِمْنَ',
    'encletic': u'هَا'
}
STOPWORDS[u'فضمنهم'] = {
    'word': u'فضمنهم',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَضِمْنَهُمْ',
    'stem': u'ضِمْنَ',
    'type': u'STOPWORD',
    'original': u'ضِمْنَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فضمنهن'] = {
    'word': u'فضمنهن',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَضِمْنَهُنَّ',
    'stem': u'ضِمْنَ',
    'type': u'STOPWORD',
    'original': u'ضِمْنَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فضمننا'] = {
    'word': u'فضمننا',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَضِمْنَنَا',
    'stem': u'ضِمْنَ',
    'type': u'STOPWORD',
    'original': u'ضِمْنَ',
    'encletic': u'نَا'
}
STOPWORDS[u'فضمنكما'] = {
    'word': u'فضمنكما',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَضِمْنَكُمَا',
    'stem': u'ضِمْنَ',
    'type': u'STOPWORD',
    'original': u'ضِمْنَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فضمنهما'] = {
    'word': u'فضمنهما',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَضِمْنَهُمَا',
    'stem': u'ضِمْنَ',
    'type': u'STOPWORD',
    'original': u'ضِمْنَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فوق'] = {
    'word': u'فَوْقَ',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان',
    'vocalized': u'فَوْقَ',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u''
}
STOPWORDS[u'لفوق'] = {
    'word': u'لفوق',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور',
    'vocalized': u'لَفَوْقَ',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u''
}
STOPWORDS[u'وفوق'] = {
    'word': u'وفوق',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف',
    'vocalized': u'وَفَوْقَ',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u''
}
STOPWORDS[u'ففوق'] = {
    'word': u'ففوق',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف',
    'vocalized': u'فَفَوْقَ',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u''
}
STOPWORDS[u'ولفوق'] = {
    'word': u'ولفوق',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'وَلَفَوْقَ',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u''
}
STOPWORDS[u'فلفوق'] = {
    'word': u'فلفوق',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'فَلَفَوْقَ',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u''
}
STOPWORDS[u'أفوق'] = {
    'word': u'أفوق',
    'procletic': u'-أَ',
    'tags': u'أداة:ظرف مكان:مجرور',
    'vocalized': u'أَفَوْقَ',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u''
}
STOPWORDS[u'ألفوق'] = {
    'word': u'ألفوق',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور',
    'vocalized': u'أَلَفَوْقَ',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u''
}
STOPWORDS[u'أوفوق'] = {
    'word': u'أوفوق',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور',
    'vocalized': u'أَوَفَوْقَ',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u''
}
STOPWORDS[u'أففوق'] = {
    'word': u'أففوق',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور',
    'vocalized': u'أَفَفَوْقَ',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u''
}
STOPWORDS[u'أولفوق'] = {
    'word': u'أولفوق',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَلَفَوْقَ',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u''
}
STOPWORDS[u'أفلفوق'] = {
    'word': u'أفلفوق',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَلَفَوْقَ',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u''
}
STOPWORDS[u'فوقي'] = {
    'word': u'فوقي',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'فَوْقَِي',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'ِي'
}
STOPWORDS[u'فوقك'] = {
    'word': u'فوقك',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'فَوْقَكَ',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'كَ'
}
STOPWORDS[u'فوقك'] = {
    'word': u'فوقك',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'فَوْقَكِ',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'كِ'
}
STOPWORDS[u'فوقه'] = {
    'word': u'فوقه',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'فَوْقَه',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'ه'
}
STOPWORDS[u'فوقكم'] = {
    'word': u'فوقكم',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'فَوْقَكُمْ',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فوقكن'] = {
    'word': u'فوقكن',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'فَوْقَكُنَّ',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فوقها'] = {
    'word': u'فوقها',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'فَوْقَهَا',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'هَا'
}
STOPWORDS[u'فوقهم'] = {
    'word': u'فوقهم',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'فَوْقَهُمْ',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فوقهن'] = {
    'word': u'فوقهن',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'فَوْقَهُنَّ',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فوقنا'] = {
    'word': u'فوقنا',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'فَوْقَنَا',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'نَا'
}
STOPWORDS[u'فوقكما'] = {
    'word': u'فوقكما',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'فَوْقَكُمَا',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فوقهما'] = {
    'word': u'فوقهما',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'فَوْقَهُمَا',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'لفوقي'] = {
    'word': u'لفوقي',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَفَوْقَِي',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'ِي'
}
STOPWORDS[u'لفوقك'] = {
    'word': u'لفوقك',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَفَوْقَكَ',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'كَ'
}
STOPWORDS[u'لفوقك'] = {
    'word': u'لفوقك',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَفَوْقَكِ',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'كِ'
}
STOPWORDS[u'لفوقه'] = {
    'word': u'لفوقه',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَفَوْقَه',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'ه'
}
STOPWORDS[u'لفوقكم'] = {
    'word': u'لفوقكم',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَفَوْقَكُمْ',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'لفوقكن'] = {
    'word': u'لفوقكن',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَفَوْقَكُنَّ',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'لفوقها'] = {
    'word': u'لفوقها',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَفَوْقَهَا',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'هَا'
}
STOPWORDS[u'لفوقهم'] = {
    'word': u'لفوقهم',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَفَوْقَهُمْ',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'لفوقهن'] = {
    'word': u'لفوقهن',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَفَوْقَهُنَّ',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'لفوقنا'] = {
    'word': u'لفوقنا',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَفَوْقَنَا',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'نَا'
}
STOPWORDS[u'لفوقكما'] = {
    'word': u'لفوقكما',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَفَوْقَكُمَا',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'لفوقهما'] = {
    'word': u'لفوقهما',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَفَوْقَهُمَا',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'وفوقي'] = {
    'word': u'وفوقي',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَفَوْقَِي',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'ِي'
}
STOPWORDS[u'وفوقك'] = {
    'word': u'وفوقك',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَفَوْقَكَ',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'كَ'
}
STOPWORDS[u'وفوقك'] = {
    'word': u'وفوقك',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَفَوْقَكِ',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'كِ'
}
STOPWORDS[u'وفوقه'] = {
    'word': u'وفوقه',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَفَوْقَه',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'ه'
}
STOPWORDS[u'وفوقكم'] = {
    'word': u'وفوقكم',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَفَوْقَكُمْ',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'وفوقكن'] = {
    'word': u'وفوقكن',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَفَوْقَكُنَّ',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'وفوقها'] = {
    'word': u'وفوقها',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَفَوْقَهَا',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'هَا'
}
STOPWORDS[u'وفوقهم'] = {
    'word': u'وفوقهم',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَفَوْقَهُمْ',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'وفوقهن'] = {
    'word': u'وفوقهن',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَفَوْقَهُنَّ',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'وفوقنا'] = {
    'word': u'وفوقنا',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَفَوْقَنَا',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'نَا'
}
STOPWORDS[u'وفوقكما'] = {
    'word': u'وفوقكما',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَفَوْقَكُمَا',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'وفوقهما'] = {
    'word': u'وفوقهما',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَفَوْقَهُمَا',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ففوقي'] = {
    'word': u'ففوقي',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَفَوْقَِي',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'ِي'
}
STOPWORDS[u'ففوقك'] = {
    'word': u'ففوقك',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَفَوْقَكَ',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'كَ'
}
STOPWORDS[u'ففوقك'] = {
    'word': u'ففوقك',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَفَوْقَكِ',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'كِ'
}
STOPWORDS[u'ففوقه'] = {
    'word': u'ففوقه',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَفَوْقَه',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'ه'
}
STOPWORDS[u'ففوقكم'] = {
    'word': u'ففوقكم',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَفَوْقَكُمْ',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ففوقكن'] = {
    'word': u'ففوقكن',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَفَوْقَكُنَّ',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ففوقها'] = {
    'word': u'ففوقها',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَفَوْقَهَا',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'هَا'
}
STOPWORDS[u'ففوقهم'] = {
    'word': u'ففوقهم',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَفَوْقَهُمْ',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ففوقهن'] = {
    'word': u'ففوقهن',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَفَوْقَهُنَّ',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ففوقنا'] = {
    'word': u'ففوقنا',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَفَوْقَنَا',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'نَا'
}
STOPWORDS[u'ففوقكما'] = {
    'word': u'ففوقكما',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَفَوْقَكُمَا',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ففوقهما'] = {
    'word': u'ففوقهما',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَفَوْقَهُمَا',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ولفوقي'] = {
    'word': u'ولفوقي',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَفَوْقَِي',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'ِي'
}
STOPWORDS[u'ولفوقك'] = {
    'word': u'ولفوقك',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَفَوْقَكَ',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'كَ'
}
STOPWORDS[u'ولفوقك'] = {
    'word': u'ولفوقك',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَفَوْقَكِ',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'كِ'
}
STOPWORDS[u'ولفوقه'] = {
    'word': u'ولفوقه',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَفَوْقَه',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'ه'
}
STOPWORDS[u'ولفوقكم'] = {
    'word': u'ولفوقكم',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَفَوْقَكُمْ',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ولفوقكن'] = {
    'word': u'ولفوقكن',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَفَوْقَكُنَّ',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ولفوقها'] = {
    'word': u'ولفوقها',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَفَوْقَهَا',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'هَا'
}
STOPWORDS[u'ولفوقهم'] = {
    'word': u'ولفوقهم',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَفَوْقَهُمْ',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ولفوقهن'] = {
    'word': u'ولفوقهن',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَفَوْقَهُنَّ',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ولفوقنا'] = {
    'word': u'ولفوقنا',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَفَوْقَنَا',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'نَا'
}
STOPWORDS[u'ولفوقكما'] = {
    'word': u'ولفوقكما',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَفَوْقَكُمَا',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ولفوقهما'] = {
    'word': u'ولفوقهما',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَفَوْقَهُمَا',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فلفوقي'] = {
    'word': u'فلفوقي',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَفَوْقَِي',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'ِي'
}
STOPWORDS[u'فلفوقك'] = {
    'word': u'فلفوقك',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَفَوْقَكَ',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'كَ'
}
STOPWORDS[u'فلفوقك'] = {
    'word': u'فلفوقك',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَفَوْقَكِ',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'كِ'
}
STOPWORDS[u'فلفوقه'] = {
    'word': u'فلفوقه',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَفَوْقَه',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'ه'
}
STOPWORDS[u'فلفوقكم'] = {
    'word': u'فلفوقكم',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَفَوْقَكُمْ',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فلفوقكن'] = {
    'word': u'فلفوقكن',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَفَوْقَكُنَّ',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فلفوقها'] = {
    'word': u'فلفوقها',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَفَوْقَهَا',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'هَا'
}
STOPWORDS[u'فلفوقهم'] = {
    'word': u'فلفوقهم',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَفَوْقَهُمْ',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فلفوقهن'] = {
    'word': u'فلفوقهن',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَفَوْقَهُنَّ',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فلفوقنا'] = {
    'word': u'فلفوقنا',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَفَوْقَنَا',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'نَا'
}
STOPWORDS[u'فلفوقكما'] = {
    'word': u'فلفوقكما',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَفَوْقَكُمَا',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فلفوقهما'] = {
    'word': u'فلفوقهما',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَفَوْقَهُمَا',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أفوقي'] = {
    'word': u'أفوقي',
    'procletic': u'-أَ',
    'tags': u'أداة:ظرف مكان:مجرور:مضاف',
    'vocalized': u'أَفَوْقَِي',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أفوقك'] = {
    'word': u'أفوقك',
    'procletic': u'-أَ',
    'tags': u'أداة:ظرف مكان:مجرور:مضاف',
    'vocalized': u'أَفَوْقَكَ',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أفوقك'] = {
    'word': u'أفوقك',
    'procletic': u'-أَ',
    'tags': u'أداة:ظرف مكان:مجرور:مضاف',
    'vocalized': u'أَفَوْقَكِ',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أفوقه'] = {
    'word': u'أفوقه',
    'procletic': u'-أَ',
    'tags': u'أداة:ظرف مكان:مجرور:مضاف',
    'vocalized': u'أَفَوْقَه',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'ه'
}
STOPWORDS[u'أفوقكم'] = {
    'word': u'أفوقكم',
    'procletic': u'-أَ',
    'tags': u'أداة:ظرف مكان:مجرور:مضاف',
    'vocalized': u'أَفَوْقَكُمْ',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أفوقكن'] = {
    'word': u'أفوقكن',
    'procletic': u'-أَ',
    'tags': u'أداة:ظرف مكان:مجرور:مضاف',
    'vocalized': u'أَفَوْقَكُنَّ',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أفوقها'] = {
    'word': u'أفوقها',
    'procletic': u'-أَ',
    'tags': u'أداة:ظرف مكان:مجرور:مضاف',
    'vocalized': u'أَفَوْقَهَا',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أفوقهم'] = {
    'word': u'أفوقهم',
    'procletic': u'-أَ',
    'tags': u'أداة:ظرف مكان:مجرور:مضاف',
    'vocalized': u'أَفَوْقَهُمْ',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أفوقهن'] = {
    'word': u'أفوقهن',
    'procletic': u'-أَ',
    'tags': u'أداة:ظرف مكان:مجرور:مضاف',
    'vocalized': u'أَفَوْقَهُنَّ',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أفوقنا'] = {
    'word': u'أفوقنا',
    'procletic': u'-أَ',
    'tags': u'أداة:ظرف مكان:مجرور:مضاف',
    'vocalized': u'أَفَوْقَنَا',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أفوقكما'] = {
    'word': u'أفوقكما',
    'procletic': u'-أَ',
    'tags': u'أداة:ظرف مكان:مجرور:مضاف',
    'vocalized': u'أَفَوْقَكُمَا',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أفوقهما'] = {
    'word': u'أفوقهما',
    'procletic': u'-أَ',
    'tags': u'أداة:ظرف مكان:مجرور:مضاف',
    'vocalized': u'أَفَوْقَهُمَا',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ألفوقي'] = {
    'word': u'ألفوقي',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَفَوْقَِي',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'ِي'
}
STOPWORDS[u'ألفوقك'] = {
    'word': u'ألفوقك',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَفَوْقَكَ',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'كَ'
}
STOPWORDS[u'ألفوقك'] = {
    'word': u'ألفوقك',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَفَوْقَكِ',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'كِ'
}
STOPWORDS[u'ألفوقه'] = {
    'word': u'ألفوقه',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَفَوْقَه',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'ه'
}
STOPWORDS[u'ألفوقكم'] = {
    'word': u'ألفوقكم',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَفَوْقَكُمْ',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ألفوقكن'] = {
    'word': u'ألفوقكن',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَفَوْقَكُنَّ',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ألفوقها'] = {
    'word': u'ألفوقها',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَفَوْقَهَا',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'هَا'
}
STOPWORDS[u'ألفوقهم'] = {
    'word': u'ألفوقهم',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَفَوْقَهُمْ',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ألفوقهن'] = {
    'word': u'ألفوقهن',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَفَوْقَهُنَّ',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ألفوقنا'] = {
    'word': u'ألفوقنا',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَفَوْقَنَا',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'نَا'
}
STOPWORDS[u'ألفوقكما'] = {
    'word': u'ألفوقكما',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَفَوْقَكُمَا',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ألفوقهما'] = {
    'word': u'ألفوقهما',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَفَوْقَهُمَا',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أوفوقي'] = {
    'word': u'أوفوقي',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَفَوْقَِي',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أوفوقك'] = {
    'word': u'أوفوقك',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَفَوْقَكَ',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أوفوقك'] = {
    'word': u'أوفوقك',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَفَوْقَكِ',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أوفوقه'] = {
    'word': u'أوفوقه',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَفَوْقَه',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'ه'
}
STOPWORDS[u'أوفوقكم'] = {
    'word': u'أوفوقكم',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَفَوْقَكُمْ',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أوفوقكن'] = {
    'word': u'أوفوقكن',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَفَوْقَكُنَّ',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أوفوقها'] = {
    'word': u'أوفوقها',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَفَوْقَهَا',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أوفوقهم'] = {
    'word': u'أوفوقهم',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَفَوْقَهُمْ',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أوفوقهن'] = {
    'word': u'أوفوقهن',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَفَوْقَهُنَّ',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أوفوقنا'] = {
    'word': u'أوفوقنا',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَفَوْقَنَا',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أوفوقكما'] = {
    'word': u'أوفوقكما',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَفَوْقَكُمَا',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أوفوقهما'] = {
    'word': u'أوفوقهما',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَفَوْقَهُمَا',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أففوقي'] = {
    'word': u'أففوقي',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَفَوْقَِي',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أففوقك'] = {
    'word': u'أففوقك',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَفَوْقَكَ',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أففوقك'] = {
    'word': u'أففوقك',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَفَوْقَكِ',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أففوقه'] = {
    'word': u'أففوقه',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَفَوْقَه',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'ه'
}
STOPWORDS[u'أففوقكم'] = {
    'word': u'أففوقكم',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَفَوْقَكُمْ',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أففوقكن'] = {
    'word': u'أففوقكن',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَفَوْقَكُنَّ',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أففوقها'] = {
    'word': u'أففوقها',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَفَوْقَهَا',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أففوقهم'] = {
    'word': u'أففوقهم',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَفَوْقَهُمْ',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أففوقهن'] = {
    'word': u'أففوقهن',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَفَوْقَهُنَّ',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أففوقنا'] = {
    'word': u'أففوقنا',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَفَوْقَنَا',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أففوقكما'] = {
    'word': u'أففوقكما',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَفَوْقَكُمَا',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أففوقهما'] = {
    'word': u'أففوقهما',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَفَوْقَهُمَا',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أولفوقي'] = {
    'word': u'أولفوقي',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَفَوْقَِي',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أولفوقك'] = {
    'word': u'أولفوقك',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَفَوْقَكَ',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أولفوقك'] = {
    'word': u'أولفوقك',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَفَوْقَكِ',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أولفوقه'] = {
    'word': u'أولفوقه',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَفَوْقَه',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'ه'
}
STOPWORDS[u'أولفوقكم'] = {
    'word': u'أولفوقكم',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَفَوْقَكُمْ',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أولفوقكن'] = {
    'word': u'أولفوقكن',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَفَوْقَكُنَّ',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أولفوقها'] = {
    'word': u'أولفوقها',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَفَوْقَهَا',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أولفوقهم'] = {
    'word': u'أولفوقهم',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَفَوْقَهُمْ',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أولفوقهن'] = {
    'word': u'أولفوقهن',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَفَوْقَهُنَّ',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أولفوقنا'] = {
    'word': u'أولفوقنا',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَفَوْقَنَا',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أولفوقكما'] = {
    'word': u'أولفوقكما',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَفَوْقَكُمَا',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أولفوقهما'] = {
    'word': u'أولفوقهما',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَفَوْقَهُمَا',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أفلفوقي'] = {
    'word': u'أفلفوقي',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَفَوْقَِي',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أفلفوقك'] = {
    'word': u'أفلفوقك',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَفَوْقَكَ',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أفلفوقك'] = {
    'word': u'أفلفوقك',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَفَوْقَكِ',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أفلفوقه'] = {
    'word': u'أفلفوقه',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَفَوْقَه',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'ه'
}
STOPWORDS[u'أفلفوقكم'] = {
    'word': u'أفلفوقكم',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَفَوْقَكُمْ',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أفلفوقكن'] = {
    'word': u'أفلفوقكن',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَفَوْقَكُنَّ',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أفلفوقها'] = {
    'word': u'أفلفوقها',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَفَوْقَهَا',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أفلفوقهم'] = {
    'word': u'أفلفوقهم',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَفَوْقَهُمْ',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أفلفوقهن'] = {
    'word': u'أفلفوقهن',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَفَوْقَهُنَّ',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أفلفوقنا'] = {
    'word': u'أفلفوقنا',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَفَوْقَنَا',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أفلفوقكما'] = {
    'word': u'أفلفوقكما',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَفَوْقَكُمَا',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أفلفوقهما'] = {
    'word': u'أفلفوقهما',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَفَوْقَهُمَا',
    'stem': u'فَوْقَ',
    'type': u'STOPWORD',
    'original': u'فَوْقَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'يمين'] = {
    'word': u'يَمِينَ',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان',
    'vocalized': u'يَمِينَ',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u''
}
STOPWORDS[u'ليمين'] = {
    'word': u'ليمين',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور',
    'vocalized': u'لَيَمِينَ',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u''
}
STOPWORDS[u'ويمين'] = {
    'word': u'ويمين',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف',
    'vocalized': u'وَيَمِينَ',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u''
}
STOPWORDS[u'فيمين'] = {
    'word': u'فيمين',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف',
    'vocalized': u'فَيَمِينَ',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u''
}
STOPWORDS[u'وليمين'] = {
    'word': u'وليمين',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'وَلَيَمِينَ',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u''
}
STOPWORDS[u'فليمين'] = {
    'word': u'فليمين',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'فَلَيَمِينَ',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u''
}
STOPWORDS[u'أيمين'] = {
    'word': u'أيمين',
    'procletic': u'-أَ',
    'tags': u'أداة:ظرف مكان:مجرور',
    'vocalized': u'أَيَمِينَ',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u''
}
STOPWORDS[u'أليمين'] = {
    'word': u'أليمين',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور',
    'vocalized': u'أَلَيَمِينَ',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u''
}
STOPWORDS[u'أويمين'] = {
    'word': u'أويمين',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور',
    'vocalized': u'أَوَيَمِينَ',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u''
}
STOPWORDS[u'أفيمين'] = {
    'word': u'أفيمين',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور',
    'vocalized': u'أَفَيَمِينَ',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u''
}
STOPWORDS[u'أوليمين'] = {
    'word': u'أوليمين',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَلَيَمِينَ',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u''
}
STOPWORDS[u'أفليمين'] = {
    'word': u'أفليمين',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَلَيَمِينَ',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u''
}
STOPWORDS[u'يميني'] = {
    'word': u'يميني',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'يَمِينَِي',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'ِي'
}
STOPWORDS[u'يمينك'] = {
    'word': u'يمينك',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'يَمِينَكَ',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'كَ'
}
STOPWORDS[u'يمينك'] = {
    'word': u'يمينك',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'يَمِينَكِ',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'كِ'
}
STOPWORDS[u'يمينه'] = {
    'word': u'يمينه',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'يَمِينَه',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'ه'
}
STOPWORDS[u'يمينكم'] = {
    'word': u'يمينكم',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'يَمِينَكُمْ',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'يمينكن'] = {
    'word': u'يمينكن',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'يَمِينَكُنَّ',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'يمينها'] = {
    'word': u'يمينها',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'يَمِينَهَا',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'هَا'
}
STOPWORDS[u'يمينهم'] = {
    'word': u'يمينهم',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'يَمِينَهُمْ',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'يمينهن'] = {
    'word': u'يمينهن',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'يَمِينَهُنَّ',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'يميننا'] = {
    'word': u'يميننا',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'يَمِينَنَا',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'نَا'
}
STOPWORDS[u'يمينكما'] = {
    'word': u'يمينكما',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'يَمِينَكُمَا',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'يمينهما'] = {
    'word': u'يمينهما',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'يَمِينَهُمَا',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ليميني'] = {
    'word': u'ليميني',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَيَمِينَِي',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'ِي'
}
STOPWORDS[u'ليمينك'] = {
    'word': u'ليمينك',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَيَمِينَكَ',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'كَ'
}
STOPWORDS[u'ليمينك'] = {
    'word': u'ليمينك',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَيَمِينَكِ',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'كِ'
}
STOPWORDS[u'ليمينه'] = {
    'word': u'ليمينه',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَيَمِينَه',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'ه'
}
STOPWORDS[u'ليمينكم'] = {
    'word': u'ليمينكم',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَيَمِينَكُمْ',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ليمينكن'] = {
    'word': u'ليمينكن',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَيَمِينَكُنَّ',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ليمينها'] = {
    'word': u'ليمينها',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَيَمِينَهَا',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'هَا'
}
STOPWORDS[u'ليمينهم'] = {
    'word': u'ليمينهم',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَيَمِينَهُمْ',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ليمينهن'] = {
    'word': u'ليمينهن',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَيَمِينَهُنَّ',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ليميننا'] = {
    'word': u'ليميننا',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَيَمِينَنَا',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'نَا'
}
STOPWORDS[u'ليمينكما'] = {
    'word': u'ليمينكما',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَيَمِينَكُمَا',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ليمينهما'] = {
    'word': u'ليمينهما',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور:مضاف',
    'vocalized': u'لَيَمِينَهُمَا',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ويميني'] = {
    'word': u'ويميني',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَيَمِينَِي',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'ِي'
}
STOPWORDS[u'ويمينك'] = {
    'word': u'ويمينك',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَيَمِينَكَ',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'كَ'
}
STOPWORDS[u'ويمينك'] = {
    'word': u'ويمينك',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَيَمِينَكِ',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'كِ'
}
STOPWORDS[u'ويمينه'] = {
    'word': u'ويمينه',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَيَمِينَه',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'ه'
}
STOPWORDS[u'ويمينكم'] = {
    'word': u'ويمينكم',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَيَمِينَكُمْ',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'ويمينكن'] = {
    'word': u'ويمينكن',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَيَمِينَكُنَّ',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'ويمينها'] = {
    'word': u'ويمينها',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَيَمِينَهَا',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'هَا'
}
STOPWORDS[u'ويمينهم'] = {
    'word': u'ويمينهم',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَيَمِينَهُمْ',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'ويمينهن'] = {
    'word': u'ويمينهن',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَيَمِينَهُنَّ',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'ويميننا'] = {
    'word': u'ويميننا',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَيَمِينَنَا',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'نَا'
}
STOPWORDS[u'ويمينكما'] = {
    'word': u'ويمينكما',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَيَمِينَكُمَا',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'ويمينهما'] = {
    'word': u'ويمينهما',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَيَمِينَهُمَا',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فيميني'] = {
    'word': u'فيميني',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَيَمِينَِي',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'ِي'
}
STOPWORDS[u'فيمينك'] = {
    'word': u'فيمينك',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَيَمِينَكَ',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'كَ'
}
STOPWORDS[u'فيمينك'] = {
    'word': u'فيمينك',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَيَمِينَكِ',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'كِ'
}
STOPWORDS[u'فيمينه'] = {
    'word': u'فيمينه',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَيَمِينَه',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'ه'
}
STOPWORDS[u'فيمينكم'] = {
    'word': u'فيمينكم',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَيَمِينَكُمْ',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فيمينكن'] = {
    'word': u'فيمينكن',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَيَمِينَكُنَّ',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فيمينها'] = {
    'word': u'فيمينها',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَيَمِينَهَا',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'هَا'
}
STOPWORDS[u'فيمينهم'] = {
    'word': u'فيمينهم',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَيَمِينَهُمْ',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فيمينهن'] = {
    'word': u'فيمينهن',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَيَمِينَهُنَّ',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فيميننا'] = {
    'word': u'فيميننا',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَيَمِينَنَا',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'نَا'
}
STOPWORDS[u'فيمينكما'] = {
    'word': u'فيمينكما',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَيَمِينَكُمَا',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فيمينهما'] = {
    'word': u'فيمينهما',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَيَمِينَهُمَا',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'وليميني'] = {
    'word': u'وليميني',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَيَمِينَِي',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'ِي'
}
STOPWORDS[u'وليمينك'] = {
    'word': u'وليمينك',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَيَمِينَكَ',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'كَ'
}
STOPWORDS[u'وليمينك'] = {
    'word': u'وليمينك',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَيَمِينَكِ',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'كِ'
}
STOPWORDS[u'وليمينه'] = {
    'word': u'وليمينه',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَيَمِينَه',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'ه'
}
STOPWORDS[u'وليمينكم'] = {
    'word': u'وليمينكم',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَيَمِينَكُمْ',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'وليمينكن'] = {
    'word': u'وليمينكن',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَيَمِينَكُنَّ',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'وليمينها'] = {
    'word': u'وليمينها',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَيَمِينَهَا',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'هَا'
}
STOPWORDS[u'وليمينهم'] = {
    'word': u'وليمينهم',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَيَمِينَهُمْ',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'وليمينهن'] = {
    'word': u'وليمينهن',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَيَمِينَهُنَّ',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'وليميننا'] = {
    'word': u'وليميننا',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَيَمِينَنَا',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'نَا'
}
STOPWORDS[u'وليمينكما'] = {
    'word': u'وليمينكما',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَيَمِينَكُمَا',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'وليمينهما'] = {
    'word': u'وليمينهما',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'وَلَيَمِينَهُمَا',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فليميني'] = {
    'word': u'فليميني',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَيَمِينَِي',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'ِي'
}
STOPWORDS[u'فليمينك'] = {
    'word': u'فليمينك',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَيَمِينَكَ',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'كَ'
}
STOPWORDS[u'فليمينك'] = {
    'word': u'فليمينك',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَيَمِينَكِ',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'كِ'
}
STOPWORDS[u'فليمينه'] = {
    'word': u'فليمينه',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَيَمِينَه',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'ه'
}
STOPWORDS[u'فليمينكم'] = {
    'word': u'فليمينكم',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَيَمِينَكُمْ',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فليمينكن'] = {
    'word': u'فليمينكن',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَيَمِينَكُنَّ',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فليمينها'] = {
    'word': u'فليمينها',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَيَمِينَهَا',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'هَا'
}
STOPWORDS[u'فليمينهم'] = {
    'word': u'فليمينهم',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَيَمِينَهُمْ',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فليمينهن'] = {
    'word': u'فليمينهن',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَيَمِينَهُنَّ',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فليميننا'] = {
    'word': u'فليميننا',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَيَمِينَنَا',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'نَا'
}
STOPWORDS[u'فليمينكما'] = {
    'word': u'فليمينكما',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَيَمِينَكُمَا',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فليمينهما'] = {
    'word': u'فليمينهما',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف:مضاف',
    'vocalized': u'فَلَيَمِينَهُمَا',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أيميني'] = {
    'word': u'أيميني',
    'procletic': u'-أَ',
    'tags': u'أداة:ظرف مكان:مجرور:مضاف',
    'vocalized': u'أَيَمِينَِي',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أيمينك'] = {
    'word': u'أيمينك',
    'procletic': u'-أَ',
    'tags': u'أداة:ظرف مكان:مجرور:مضاف',
    'vocalized': u'أَيَمِينَكَ',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أيمينك'] = {
    'word': u'أيمينك',
    'procletic': u'-أَ',
    'tags': u'أداة:ظرف مكان:مجرور:مضاف',
    'vocalized': u'أَيَمِينَكِ',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أيمينه'] = {
    'word': u'أيمينه',
    'procletic': u'-أَ',
    'tags': u'أداة:ظرف مكان:مجرور:مضاف',
    'vocalized': u'أَيَمِينَه',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'ه'
}
STOPWORDS[u'أيمينكم'] = {
    'word': u'أيمينكم',
    'procletic': u'-أَ',
    'tags': u'أداة:ظرف مكان:مجرور:مضاف',
    'vocalized': u'أَيَمِينَكُمْ',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أيمينكن'] = {
    'word': u'أيمينكن',
    'procletic': u'-أَ',
    'tags': u'أداة:ظرف مكان:مجرور:مضاف',
    'vocalized': u'أَيَمِينَكُنَّ',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أيمينها'] = {
    'word': u'أيمينها',
    'procletic': u'-أَ',
    'tags': u'أداة:ظرف مكان:مجرور:مضاف',
    'vocalized': u'أَيَمِينَهَا',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أيمينهم'] = {
    'word': u'أيمينهم',
    'procletic': u'-أَ',
    'tags': u'أداة:ظرف مكان:مجرور:مضاف',
    'vocalized': u'أَيَمِينَهُمْ',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أيمينهن'] = {
    'word': u'أيمينهن',
    'procletic': u'-أَ',
    'tags': u'أداة:ظرف مكان:مجرور:مضاف',
    'vocalized': u'أَيَمِينَهُنَّ',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أيميننا'] = {
    'word': u'أيميننا',
    'procletic': u'-أَ',
    'tags': u'أداة:ظرف مكان:مجرور:مضاف',
    'vocalized': u'أَيَمِينَنَا',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أيمينكما'] = {
    'word': u'أيمينكما',
    'procletic': u'-أَ',
    'tags': u'أداة:ظرف مكان:مجرور:مضاف',
    'vocalized': u'أَيَمِينَكُمَا',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أيمينهما'] = {
    'word': u'أيمينهما',
    'procletic': u'-أَ',
    'tags': u'أداة:ظرف مكان:مجرور:مضاف',
    'vocalized': u'أَيَمِينَهُمَا',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أليميني'] = {
    'word': u'أليميني',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَيَمِينَِي',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أليمينك'] = {
    'word': u'أليمينك',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَيَمِينَكَ',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أليمينك'] = {
    'word': u'أليمينك',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَيَمِينَكِ',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أليمينه'] = {
    'word': u'أليمينه',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَيَمِينَه',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'ه'
}
STOPWORDS[u'أليمينكم'] = {
    'word': u'أليمينكم',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَيَمِينَكُمْ',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أليمينكن'] = {
    'word': u'أليمينكن',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَيَمِينَكُنَّ',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أليمينها'] = {
    'word': u'أليمينها',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَيَمِينَهَا',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أليمينهم'] = {
    'word': u'أليمينهم',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَيَمِينَهُمْ',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أليمينهن'] = {
    'word': u'أليمينهن',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَيَمِينَهُنَّ',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أليميننا'] = {
    'word': u'أليميننا',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَيَمِينَنَا',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أليمينكما'] = {
    'word': u'أليمينكما',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَيَمِينَكُمَا',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أليمينهما'] = {
    'word': u'أليمينهما',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور:مضاف',
    'vocalized': u'أَلَيَمِينَهُمَا',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أويميني'] = {
    'word': u'أويميني',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَيَمِينَِي',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أويمينك'] = {
    'word': u'أويمينك',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَيَمِينَكَ',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أويمينك'] = {
    'word': u'أويمينك',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَيَمِينَكِ',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أويمينه'] = {
    'word': u'أويمينه',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَيَمِينَه',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'ه'
}
STOPWORDS[u'أويمينكم'] = {
    'word': u'أويمينكم',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَيَمِينَكُمْ',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أويمينكن'] = {
    'word': u'أويمينكن',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَيَمِينَكُنَّ',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أويمينها'] = {
    'word': u'أويمينها',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَيَمِينَهَا',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أويمينهم'] = {
    'word': u'أويمينهم',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَيَمِينَهُمْ',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أويمينهن'] = {
    'word': u'أويمينهن',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَيَمِينَهُنَّ',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أويميننا'] = {
    'word': u'أويميننا',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَيَمِينَنَا',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أويمينكما'] = {
    'word': u'أويمينكما',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَيَمِينَكُمَا',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أويمينهما'] = {
    'word': u'أويمينهما',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَيَمِينَهُمَا',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أفيميني'] = {
    'word': u'أفيميني',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَيَمِينَِي',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أفيمينك'] = {
    'word': u'أفيمينك',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَيَمِينَكَ',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أفيمينك'] = {
    'word': u'أفيمينك',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَيَمِينَكِ',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أفيمينه'] = {
    'word': u'أفيمينه',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَيَمِينَه',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'ه'
}
STOPWORDS[u'أفيمينكم'] = {
    'word': u'أفيمينكم',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَيَمِينَكُمْ',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أفيمينكن'] = {
    'word': u'أفيمينكن',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَيَمِينَكُنَّ',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أفيمينها'] = {
    'word': u'أفيمينها',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَيَمِينَهَا',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أفيمينهم'] = {
    'word': u'أفيمينهم',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَيَمِينَهُمْ',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أفيمينهن'] = {
    'word': u'أفيمينهن',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَيَمِينَهُنَّ',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أفيميننا'] = {
    'word': u'أفيميننا',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَيَمِينَنَا',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أفيمينكما'] = {
    'word': u'أفيمينكما',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَيَمِينَكُمَا',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أفيمينهما'] = {
    'word': u'أفيمينهما',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:ظرف مكان:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَيَمِينَهُمَا',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أوليميني'] = {
    'word': u'أوليميني',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَيَمِينَِي',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أوليمينك'] = {
    'word': u'أوليمينك',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَيَمِينَكَ',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أوليمينك'] = {
    'word': u'أوليمينك',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَيَمِينَكِ',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أوليمينه'] = {
    'word': u'أوليمينه',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَيَمِينَه',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'ه'
}
STOPWORDS[u'أوليمينكم'] = {
    'word': u'أوليمينكم',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَيَمِينَكُمْ',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أوليمينكن'] = {
    'word': u'أوليمينكن',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَيَمِينَكُنَّ',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أوليمينها'] = {
    'word': u'أوليمينها',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَيَمِينَهَا',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أوليمينهم'] = {
    'word': u'أوليمينهم',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَيَمِينَهُمْ',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أوليمينهن'] = {
    'word': u'أوليمينهن',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَيَمِينَهُنَّ',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أوليميننا'] = {
    'word': u'أوليميننا',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَيَمِينَنَا',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أوليمينكما'] = {
    'word': u'أوليمينكما',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَيَمِينَكُمَا',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أوليمينهما'] = {
    'word': u'أوليمينهما',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَلَيَمِينَهُمَا',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أفليميني'] = {
    'word': u'أفليميني',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَيَمِينَِي',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أفليمينك'] = {
    'word': u'أفليمينك',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَيَمِينَكَ',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أفليمينك'] = {
    'word': u'أفليمينك',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَيَمِينَكِ',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أفليمينه'] = {
    'word': u'أفليمينه',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَيَمِينَه',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'ه'
}
STOPWORDS[u'أفليمينكم'] = {
    'word': u'أفليمينكم',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَيَمِينَكُمْ',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أفليمينكن'] = {
    'word': u'أفليمينكن',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَيَمِينَكُنَّ',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أفليمينها'] = {
    'word': u'أفليمينها',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَيَمِينَهَا',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أفليمينهم'] = {
    'word': u'أفليمينهم',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَيَمِينَهُمْ',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أفليمينهن'] = {
    'word': u'أفليمينهن',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَيَمِينَهُنَّ',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أفليميننا'] = {
    'word': u'أفليميننا',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَيَمِينَنَا',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أفليمينكما'] = {
    'word': u'أفليمينكما',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَيَمِينَكُمَا',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أفليمينهما'] = {
    'word': u'أفليمينهما',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَلَيَمِينَهُمَا',
    'stem': u'يَمِينَ',
    'type': u'STOPWORD',
    'original': u'يَمِينَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'حوالى'] = {
    'word': u'حَوَالَى',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان',
    'vocalized': u'حَوَالَى',
    'stem': u'حَوَالَى',
    'type': u'STOPWORD',
    'original': u'حَوَالَى',
    'encletic': u''
}
STOPWORDS[u'وحوالى'] = {
    'word': u'وحوالى',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف',
    'vocalized': u'وَحَوَالَى',
    'stem': u'حَوَالَى',
    'type': u'STOPWORD',
    'original': u'حَوَالَى',
    'encletic': u''
}
STOPWORDS[u'فحوالى'] = {
    'word': u'فحوالى',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف',
    'vocalized': u'فَحَوَالَى',
    'stem': u'حَوَالَى',
    'type': u'STOPWORD',
    'original': u'حَوَالَى',
    'encletic': u''
}
STOPWORDS[u'حوالي'] = {
    'word': u'حوالي',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'حَوَالَيِي',
    'stem': u'حَوَالَى',
    'type': u'STOPWORD',
    'original': u'حَوَالَى',
    'encletic': u'ِي'
}
STOPWORDS[u'حواليك'] = {
    'word': u'حواليك',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'حَوَالَيكَ',
    'stem': u'حَوَالَى',
    'type': u'STOPWORD',
    'original': u'حَوَالَى',
    'encletic': u'كَ'
}
STOPWORDS[u'حواليك'] = {
    'word': u'حواليك',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'حَوَالَيكِ',
    'stem': u'حَوَالَى',
    'type': u'STOPWORD',
    'original': u'حَوَالَى',
    'encletic': u'كِ'
}
STOPWORDS[u'حواليه'] = {
    'word': u'حواليه',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'حَوَالَيه',
    'stem': u'حَوَالَى',
    'type': u'STOPWORD',
    'original': u'حَوَالَى',
    'encletic': u'ه'
}
STOPWORDS[u'حواليكم'] = {
    'word': u'حواليكم',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'حَوَالَيكُمْ',
    'stem': u'حَوَالَى',
    'type': u'STOPWORD',
    'original': u'حَوَالَى',
    'encletic': u'كُمْ'
}
STOPWORDS[u'حواليكن'] = {
    'word': u'حواليكن',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'حَوَالَيكُنَّ',
    'stem': u'حَوَالَى',
    'type': u'STOPWORD',
    'original': u'حَوَالَى',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'حواليها'] = {
    'word': u'حواليها',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'حَوَالَيهَا',
    'stem': u'حَوَالَى',
    'type': u'STOPWORD',
    'original': u'حَوَالَى',
    'encletic': u'هَا'
}
STOPWORDS[u'حواليهم'] = {
    'word': u'حواليهم',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'حَوَالَيهُمْ',
    'stem': u'حَوَالَى',
    'type': u'STOPWORD',
    'original': u'حَوَالَى',
    'encletic': u'هُمْ'
}
STOPWORDS[u'حواليهن'] = {
    'word': u'حواليهن',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'حَوَالَيهُنَّ',
    'stem': u'حَوَالَى',
    'type': u'STOPWORD',
    'original': u'حَوَالَى',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'حوالينا'] = {
    'word': u'حوالينا',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'حَوَالَينَا',
    'stem': u'حَوَالَى',
    'type': u'STOPWORD',
    'original': u'حَوَالَى',
    'encletic': u'نَا'
}
STOPWORDS[u'حواليكما'] = {
    'word': u'حواليكما',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'حَوَالَيكُمَا',
    'stem': u'حَوَالَى',
    'type': u'STOPWORD',
    'original': u'حَوَالَى',
    'encletic': u'كُمَا'
}
STOPWORDS[u'حواليهما'] = {
    'word': u'حواليهما',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'حَوَالَيهُمَا',
    'stem': u'حَوَالَى',
    'type': u'STOPWORD',
    'original': u'حَوَالَى',
    'encletic': u'هُمَا'
}
STOPWORDS[u'وحوالي'] = {
    'word': u'وحوالي',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَحَوَالَيِي',
    'stem': u'حَوَالَى',
    'type': u'STOPWORD',
    'original': u'حَوَالَى',
    'encletic': u'ِي'
}
STOPWORDS[u'وحواليك'] = {
    'word': u'وحواليك',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَحَوَالَيكَ',
    'stem': u'حَوَالَى',
    'type': u'STOPWORD',
    'original': u'حَوَالَى',
    'encletic': u'كَ'
}
STOPWORDS[u'وحواليك'] = {
    'word': u'وحواليك',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَحَوَالَيكِ',
    'stem': u'حَوَالَى',
    'type': u'STOPWORD',
    'original': u'حَوَالَى',
    'encletic': u'كِ'
}
STOPWORDS[u'وحواليه'] = {
    'word': u'وحواليه',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَحَوَالَيه',
    'stem': u'حَوَالَى',
    'type': u'STOPWORD',
    'original': u'حَوَالَى',
    'encletic': u'ه'
}
STOPWORDS[u'وحواليكم'] = {
    'word': u'وحواليكم',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَحَوَالَيكُمْ',
    'stem': u'حَوَالَى',
    'type': u'STOPWORD',
    'original': u'حَوَالَى',
    'encletic': u'كُمْ'
}
STOPWORDS[u'وحواليكن'] = {
    'word': u'وحواليكن',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَحَوَالَيكُنَّ',
    'stem': u'حَوَالَى',
    'type': u'STOPWORD',
    'original': u'حَوَالَى',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'وحواليها'] = {
    'word': u'وحواليها',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَحَوَالَيهَا',
    'stem': u'حَوَالَى',
    'type': u'STOPWORD',
    'original': u'حَوَالَى',
    'encletic': u'هَا'
}
STOPWORDS[u'وحواليهم'] = {
    'word': u'وحواليهم',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَحَوَالَيهُمْ',
    'stem': u'حَوَالَى',
    'type': u'STOPWORD',
    'original': u'حَوَالَى',
    'encletic': u'هُمْ'
}
STOPWORDS[u'وحواليهن'] = {
    'word': u'وحواليهن',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَحَوَالَيهُنَّ',
    'stem': u'حَوَالَى',
    'type': u'STOPWORD',
    'original': u'حَوَالَى',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'وحوالينا'] = {
    'word': u'وحوالينا',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَحَوَالَينَا',
    'stem': u'حَوَالَى',
    'type': u'STOPWORD',
    'original': u'حَوَالَى',
    'encletic': u'نَا'
}
STOPWORDS[u'وحواليكما'] = {
    'word': u'وحواليكما',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَحَوَالَيكُمَا',
    'stem': u'حَوَالَى',
    'type': u'STOPWORD',
    'original': u'حَوَالَى',
    'encletic': u'كُمَا'
}
STOPWORDS[u'وحواليهما'] = {
    'word': u'وحواليهما',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَحَوَالَيهُمَا',
    'stem': u'حَوَالَى',
    'type': u'STOPWORD',
    'original': u'حَوَالَى',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فحوالي'] = {
    'word': u'فحوالي',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَحَوَالَيِي',
    'stem': u'حَوَالَى',
    'type': u'STOPWORD',
    'original': u'حَوَالَى',
    'encletic': u'ِي'
}
STOPWORDS[u'فحواليك'] = {
    'word': u'فحواليك',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَحَوَالَيكَ',
    'stem': u'حَوَالَى',
    'type': u'STOPWORD',
    'original': u'حَوَالَى',
    'encletic': u'كَ'
}
STOPWORDS[u'فحواليك'] = {
    'word': u'فحواليك',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَحَوَالَيكِ',
    'stem': u'حَوَالَى',
    'type': u'STOPWORD',
    'original': u'حَوَالَى',
    'encletic': u'كِ'
}
STOPWORDS[u'فحواليه'] = {
    'word': u'فحواليه',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَحَوَالَيه',
    'stem': u'حَوَالَى',
    'type': u'STOPWORD',
    'original': u'حَوَالَى',
    'encletic': u'ه'
}
STOPWORDS[u'فحواليكم'] = {
    'word': u'فحواليكم',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَحَوَالَيكُمْ',
    'stem': u'حَوَالَى',
    'type': u'STOPWORD',
    'original': u'حَوَالَى',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فحواليكن'] = {
    'word': u'فحواليكن',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَحَوَالَيكُنَّ',
    'stem': u'حَوَالَى',
    'type': u'STOPWORD',
    'original': u'حَوَالَى',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فحواليها'] = {
    'word': u'فحواليها',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَحَوَالَيهَا',
    'stem': u'حَوَالَى',
    'type': u'STOPWORD',
    'original': u'حَوَالَى',
    'encletic': u'هَا'
}
STOPWORDS[u'فحواليهم'] = {
    'word': u'فحواليهم',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَحَوَالَيهُمْ',
    'stem': u'حَوَالَى',
    'type': u'STOPWORD',
    'original': u'حَوَالَى',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فحواليهن'] = {
    'word': u'فحواليهن',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَحَوَالَيهُنَّ',
    'stem': u'حَوَالَى',
    'type': u'STOPWORD',
    'original': u'حَوَالَى',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فحوالينا'] = {
    'word': u'فحوالينا',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَحَوَالَينَا',
    'stem': u'حَوَالَى',
    'type': u'STOPWORD',
    'original': u'حَوَالَى',
    'encletic': u'نَا'
}
STOPWORDS[u'فحواليكما'] = {
    'word': u'فحواليكما',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَحَوَالَيكُمَا',
    'stem': u'حَوَالَى',
    'type': u'STOPWORD',
    'original': u'حَوَالَى',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فحواليهما'] = {
    'word': u'فحواليهما',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَحَوَالَيهُمَا',
    'stem': u'حَوَالَى',
    'type': u'STOPWORD',
    'original': u'حَوَالَى',
    'encletic': u'هُمَا'
}
STOPWORDS[u'حول'] = {
    'word': u'حَوْلَ',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان',
    'vocalized': u'حَوْلَ',
    'stem': u'حَوْلَ',
    'type': u'STOPWORD',
    'original': u'حَوْلَ',
    'encletic': u''
}
STOPWORDS[u'وحول'] = {
    'word': u'وحول',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف',
    'vocalized': u'وَحَوْلَ',
    'stem': u'حَوْلَ',
    'type': u'STOPWORD',
    'original': u'حَوْلَ',
    'encletic': u''
}
STOPWORDS[u'فحول'] = {
    'word': u'فحول',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف',
    'vocalized': u'فَحَوْلَ',
    'stem': u'حَوْلَ',
    'type': u'STOPWORD',
    'original': u'حَوْلَ',
    'encletic': u''
}
STOPWORDS[u'حولي'] = {
    'word': u'حولي',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'حَوْلَِي',
    'stem': u'حَوْلَ',
    'type': u'STOPWORD',
    'original': u'حَوْلَ',
    'encletic': u'ِي'
}
STOPWORDS[u'حولك'] = {
    'word': u'حولك',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'حَوْلَكَ',
    'stem': u'حَوْلَ',
    'type': u'STOPWORD',
    'original': u'حَوْلَ',
    'encletic': u'كَ'
}
STOPWORDS[u'حولك'] = {
    'word': u'حولك',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'حَوْلَكِ',
    'stem': u'حَوْلَ',
    'type': u'STOPWORD',
    'original': u'حَوْلَ',
    'encletic': u'كِ'
}
STOPWORDS[u'حوله'] = {
    'word': u'حوله',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'حَوْلَه',
    'stem': u'حَوْلَ',
    'type': u'STOPWORD',
    'original': u'حَوْلَ',
    'encletic': u'ه'
}
STOPWORDS[u'حولكم'] = {
    'word': u'حولكم',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'حَوْلَكُمْ',
    'stem': u'حَوْلَ',
    'type': u'STOPWORD',
    'original': u'حَوْلَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'حولكن'] = {
    'word': u'حولكن',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'حَوْلَكُنَّ',
    'stem': u'حَوْلَ',
    'type': u'STOPWORD',
    'original': u'حَوْلَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'حولها'] = {
    'word': u'حولها',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'حَوْلَهَا',
    'stem': u'حَوْلَ',
    'type': u'STOPWORD',
    'original': u'حَوْلَ',
    'encletic': u'هَا'
}
STOPWORDS[u'حولهم'] = {
    'word': u'حولهم',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'حَوْلَهُمْ',
    'stem': u'حَوْلَ',
    'type': u'STOPWORD',
    'original': u'حَوْلَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'حولهن'] = {
    'word': u'حولهن',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'حَوْلَهُنَّ',
    'stem': u'حَوْلَ',
    'type': u'STOPWORD',
    'original': u'حَوْلَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'حولنا'] = {
    'word': u'حولنا',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'حَوْلَنَا',
    'stem': u'حَوْلَ',
    'type': u'STOPWORD',
    'original': u'حَوْلَ',
    'encletic': u'نَا'
}
STOPWORDS[u'حولكما'] = {
    'word': u'حولكما',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'حَوْلَكُمَا',
    'stem': u'حَوْلَ',
    'type': u'STOPWORD',
    'original': u'حَوْلَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'حولهما'] = {
    'word': u'حولهما',
    'procletic': u'',
    'tags': u'أداة:ظرف مكان:مضاف',
    'vocalized': u'حَوْلَهُمَا',
    'stem': u'حَوْلَ',
    'type': u'STOPWORD',
    'original': u'حَوْلَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'وحولي'] = {
    'word': u'وحولي',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَحَوْلَِي',
    'stem': u'حَوْلَ',
    'type': u'STOPWORD',
    'original': u'حَوْلَ',
    'encletic': u'ِي'
}
STOPWORDS[u'وحولك'] = {
    'word': u'وحولك',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَحَوْلَكَ',
    'stem': u'حَوْلَ',
    'type': u'STOPWORD',
    'original': u'حَوْلَ',
    'encletic': u'كَ'
}
STOPWORDS[u'وحولك'] = {
    'word': u'وحولك',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَحَوْلَكِ',
    'stem': u'حَوْلَ',
    'type': u'STOPWORD',
    'original': u'حَوْلَ',
    'encletic': u'كِ'
}
STOPWORDS[u'وحوله'] = {
    'word': u'وحوله',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَحَوْلَه',
    'stem': u'حَوْلَ',
    'type': u'STOPWORD',
    'original': u'حَوْلَ',
    'encletic': u'ه'
}
STOPWORDS[u'وحولكم'] = {
    'word': u'وحولكم',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَحَوْلَكُمْ',
    'stem': u'حَوْلَ',
    'type': u'STOPWORD',
    'original': u'حَوْلَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'وحولكن'] = {
    'word': u'وحولكن',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَحَوْلَكُنَّ',
    'stem': u'حَوْلَ',
    'type': u'STOPWORD',
    'original': u'حَوْلَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'وحولها'] = {
    'word': u'وحولها',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَحَوْلَهَا',
    'stem': u'حَوْلَ',
    'type': u'STOPWORD',
    'original': u'حَوْلَ',
    'encletic': u'هَا'
}
STOPWORDS[u'وحولهم'] = {
    'word': u'وحولهم',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَحَوْلَهُمْ',
    'stem': u'حَوْلَ',
    'type': u'STOPWORD',
    'original': u'حَوْلَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'وحولهن'] = {
    'word': u'وحولهن',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَحَوْلَهُنَّ',
    'stem': u'حَوْلَ',
    'type': u'STOPWORD',
    'original': u'حَوْلَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'وحولنا'] = {
    'word': u'وحولنا',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَحَوْلَنَا',
    'stem': u'حَوْلَ',
    'type': u'STOPWORD',
    'original': u'حَوْلَ',
    'encletic': u'نَا'
}
STOPWORDS[u'وحولكما'] = {
    'word': u'وحولكما',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَحَوْلَكُمَا',
    'stem': u'حَوْلَ',
    'type': u'STOPWORD',
    'original': u'حَوْلَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'وحولهما'] = {
    'word': u'وحولهما',
    'procletic': u'-وَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'وَحَوْلَهُمَا',
    'stem': u'حَوْلَ',
    'type': u'STOPWORD',
    'original': u'حَوْلَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فحولي'] = {
    'word': u'فحولي',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَحَوْلَِي',
    'stem': u'حَوْلَ',
    'type': u'STOPWORD',
    'original': u'حَوْلَ',
    'encletic': u'ِي'
}
STOPWORDS[u'فحولك'] = {
    'word': u'فحولك',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَحَوْلَكَ',
    'stem': u'حَوْلَ',
    'type': u'STOPWORD',
    'original': u'حَوْلَ',
    'encletic': u'كَ'
}
STOPWORDS[u'فحولك'] = {
    'word': u'فحولك',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَحَوْلَكِ',
    'stem': u'حَوْلَ',
    'type': u'STOPWORD',
    'original': u'حَوْلَ',
    'encletic': u'كِ'
}
STOPWORDS[u'فحوله'] = {
    'word': u'فحوله',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَحَوْلَه',
    'stem': u'حَوْلَ',
    'type': u'STOPWORD',
    'original': u'حَوْلَ',
    'encletic': u'ه'
}
STOPWORDS[u'فحولكم'] = {
    'word': u'فحولكم',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَحَوْلَكُمْ',
    'stem': u'حَوْلَ',
    'type': u'STOPWORD',
    'original': u'حَوْلَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فحولكن'] = {
    'word': u'فحولكن',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَحَوْلَكُنَّ',
    'stem': u'حَوْلَ',
    'type': u'STOPWORD',
    'original': u'حَوْلَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فحولها'] = {
    'word': u'فحولها',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَحَوْلَهَا',
    'stem': u'حَوْلَ',
    'type': u'STOPWORD',
    'original': u'حَوْلَ',
    'encletic': u'هَا'
}
STOPWORDS[u'فحولهم'] = {
    'word': u'فحولهم',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَحَوْلَهُمْ',
    'stem': u'حَوْلَ',
    'type': u'STOPWORD',
    'original': u'حَوْلَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فحولهن'] = {
    'word': u'فحولهن',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَحَوْلَهُنَّ',
    'stem': u'حَوْلَ',
    'type': u'STOPWORD',
    'original': u'حَوْلَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فحولنا'] = {
    'word': u'فحولنا',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَحَوْلَنَا',
    'stem': u'حَوْلَ',
    'type': u'STOPWORD',
    'original': u'حَوْلَ',
    'encletic': u'نَا'
}
STOPWORDS[u'فحولكما'] = {
    'word': u'فحولكما',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَحَوْلَكُمَا',
    'stem': u'حَوْلَ',
    'type': u'STOPWORD',
    'original': u'حَوْلَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فحولهما'] = {
    'word': u'فحولهما',
    'procletic': u'-فَ',
    'tags': u'أداة:ظرف مكان:معطوف:مضاف',
    'vocalized': u'فَحَوْلَهُمَا',
    'stem': u'حَوْلَ',
    'type': u'STOPWORD',
    'original': u'حَوْلَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'طالما'] = {
    'word': u'طَالَمَا',
    'procletic': u'',
    'tags': u'أداة:فعل جامد',
    'vocalized': u'طَالَمَا',
    'stem': u'طَالَمَا',
    'type': u'STOPWORD',
    'original': u'طَالَمَا',
    'encletic': u''
}
STOPWORDS[u'لطالما'] = {
    'word': u'لطالما',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور',
    'vocalized': u'لَطَالَمَا',
    'stem': u'طَالَمَا',
    'type': u'STOPWORD',
    'original': u'طَالَمَا',
    'encletic': u''
}
STOPWORDS[u'وطالما'] = {
    'word': u'وطالما',
    'procletic': u'-وَ',
    'tags': u'أداة:فعل جامد:معطوف',
    'vocalized': u'وَطَالَمَا',
    'stem': u'طَالَمَا',
    'type': u'STOPWORD',
    'original': u'طَالَمَا',
    'encletic': u''
}
STOPWORDS[u'فطالما'] = {
    'word': u'فطالما',
    'procletic': u'-فَ',
    'tags': u'أداة:فعل جامد:معطوف',
    'vocalized': u'فَطَالَمَا',
    'stem': u'طَالَمَا',
    'type': u'STOPWORD',
    'original': u'طَالَمَا',
    'encletic': u''
}
STOPWORDS[u'ولطالما'] = {
    'word': u'ولطالما',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'وَلَطَالَمَا',
    'stem': u'طَالَمَا',
    'type': u'STOPWORD',
    'original': u'طَالَمَا',
    'encletic': u''
}
STOPWORDS[u'فلطالما'] = {
    'word': u'فلطالما',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'فَلَطَالَمَا',
    'stem': u'طَالَمَا',
    'type': u'STOPWORD',
    'original': u'طَالَمَا',
    'encletic': u''
}
STOPWORDS[u'أطالما'] = {
    'word': u'أطالما',
    'procletic': u'-أَ',
    'tags': u'أداة:فعل جامد:مجرور',
    'vocalized': u'أَطَالَمَا',
    'stem': u'طَالَمَا',
    'type': u'STOPWORD',
    'original': u'طَالَمَا',
    'encletic': u''
}
STOPWORDS[u'ألطالما'] = {
    'word': u'ألطالما',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور',
    'vocalized': u'أَلَطَالَمَا',
    'stem': u'طَالَمَا',
    'type': u'STOPWORD',
    'original': u'طَالَمَا',
    'encletic': u''
}
STOPWORDS[u'أوطالما'] = {
    'word': u'أوطالما',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:فعل جامد:معطوف:مجرور',
    'vocalized': u'أَوَطَالَمَا',
    'stem': u'طَالَمَا',
    'type': u'STOPWORD',
    'original': u'طَالَمَا',
    'encletic': u''
}
STOPWORDS[u'أفطالما'] = {
    'word': u'أفطالما',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:فعل جامد:معطوف:مجرور',
    'vocalized': u'أَفَطَالَمَا',
    'stem': u'طَالَمَا',
    'type': u'STOPWORD',
    'original': u'طَالَمَا',
    'encletic': u''
}
STOPWORDS[u'أولطالما'] = {
    'word': u'أولطالما',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَلَطَالَمَا',
    'stem': u'طَالَمَا',
    'type': u'STOPWORD',
    'original': u'طَالَمَا',
    'encletic': u''
}
STOPWORDS[u'أفلطالما'] = {
    'word': u'أفلطالما',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَلَطَالَمَا',
    'stem': u'طَالَمَا',
    'type': u'STOPWORD',
    'original': u'طَالَمَا',
    'encletic': u''
}
STOPWORDS[u'قلما'] = {
    'word': u'قَلًَّمَا',
    'procletic': u'',
    'tags': u'أداة:فعل جامد',
    'vocalized': u'قَلًَّمَا',
    'stem': u'قَلًَّمَا',
    'type': u'STOPWORD',
    'original': u'قَلًَّمَا',
    'encletic': u''
}
STOPWORDS[u'لقلما'] = {
    'word': u'لقلما',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور',
    'vocalized': u'لَقَلًَّمَا',
    'stem': u'قَلًَّمَا',
    'type': u'STOPWORD',
    'original': u'قَلًَّمَا',
    'encletic': u''
}
STOPWORDS[u'وقلما'] = {
    'word': u'وقلما',
    'procletic': u'-وَ',
    'tags': u'أداة:فعل جامد:معطوف',
    'vocalized': u'وَقَلًَّمَا',
    'stem': u'قَلًَّمَا',
    'type': u'STOPWORD',
    'original': u'قَلًَّمَا',
    'encletic': u''
}
STOPWORDS[u'فقلما'] = {
    'word': u'فقلما',
    'procletic': u'-فَ',
    'tags': u'أداة:فعل جامد:معطوف',
    'vocalized': u'فَقَلًَّمَا',
    'stem': u'قَلًَّمَا',
    'type': u'STOPWORD',
    'original': u'قَلًَّمَا',
    'encletic': u''
}
STOPWORDS[u'ولقلما'] = {
    'word': u'ولقلما',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'وَلَقَلًَّمَا',
    'stem': u'قَلًَّمَا',
    'type': u'STOPWORD',
    'original': u'قَلًَّمَا',
    'encletic': u''
}
STOPWORDS[u'فلقلما'] = {
    'word': u'فلقلما',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'فَلَقَلًَّمَا',
    'stem': u'قَلًَّمَا',
    'type': u'STOPWORD',
    'original': u'قَلًَّمَا',
    'encletic': u''
}
STOPWORDS[u'أقلما'] = {
    'word': u'أقلما',
    'procletic': u'-أَ',
    'tags': u'أداة:فعل جامد:مجرور',
    'vocalized': u'أَقَلًَّمَا',
    'stem': u'قَلًَّمَا',
    'type': u'STOPWORD',
    'original': u'قَلًَّمَا',
    'encletic': u''
}
STOPWORDS[u'ألقلما'] = {
    'word': u'ألقلما',
    'procletic': u'لَ-أَ',
    'tags': u'أداة:مجرور:مجرور',
    'vocalized': u'أَلَقَلًَّمَا',
    'stem': u'قَلًَّمَا',
    'type': u'STOPWORD',
    'original': u'قَلًَّمَا',
    'encletic': u''
}
STOPWORDS[u'أوقلما'] = {
    'word': u'أوقلما',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:فعل جامد:معطوف:مجرور',
    'vocalized': u'أَوَقَلًَّمَا',
    'stem': u'قَلًَّمَا',
    'type': u'STOPWORD',
    'original': u'قَلًَّمَا',
    'encletic': u''
}
STOPWORDS[u'أفقلما'] = {
    'word': u'أفقلما',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:فعل جامد:معطوف:مجرور',
    'vocalized': u'أَفَقَلًَّمَا',
    'stem': u'قَلًَّمَا',
    'type': u'STOPWORD',
    'original': u'قَلًَّمَا',
    'encletic': u''
}
STOPWORDS[u'أولقلما'] = {
    'word': u'أولقلما',
    'procletic': u'لَ-وَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور',
    'vocalized': u'أَوَلَقَلًَّمَا',
    'stem': u'قَلًَّمَا',
    'type': u'STOPWORD',
    'original': u'قَلًَّمَا',
    'encletic': u''
}
STOPWORDS[u'أفلقلما'] = {
    'word': u'أفلقلما',
    'procletic': u'لَ-فَ-أَ',
    'tags': u'أداة:مجرور:معطوف:مجرور',
    'vocalized': u'أَفَلَقَلًَّمَا',
    'stem': u'قَلًَّمَا',
    'type': u'STOPWORD',
    'original': u'قَلًَّمَا',
    'encletic': u''
}
STOPWORDS[u'ابتدأ'] = {
    'word': u'ابْتَدَأَ',
    'procletic': u'',
    'tags': u'أداة:كاد و اخواتها',
    'vocalized': u'ابْتَدَأَ',
    'stem': u'ابْتَدَأَ',
    'type': u'STOPWORD',
    'original': u'ابْتَدَأَ',
    'encletic': u''
}
STOPWORDS[u'لابتدأ'] = {
    'word': u'لابتدأ',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور',
    'vocalized': u'لَابْتَدَأَ',
    'stem': u'ابْتَدَأَ',
    'type': u'STOPWORD',
    'original': u'ابْتَدَأَ',
    'encletic': u''
}
STOPWORDS[u'وابتدأ'] = {
    'word': u'وابتدأ',
    'procletic': u'-وَ',
    'tags': u'أداة:كاد و اخواتها:معطوف',
    'vocalized': u'وَابْتَدَأَ',
    'stem': u'ابْتَدَأَ',
    'type': u'STOPWORD',
    'original': u'ابْتَدَأَ',
    'encletic': u''
}
STOPWORDS[u'فابتدأ'] = {
    'word': u'فابتدأ',
    'procletic': u'-فَ',
    'tags': u'أداة:كاد و اخواتها:معطوف',
    'vocalized': u'فَابْتَدَأَ',
    'stem': u'ابْتَدَأَ',
    'type': u'STOPWORD',
    'original': u'ابْتَدَأَ',
    'encletic': u''
}
STOPWORDS[u'ولابتدأ'] = {
    'word': u'ولابتدأ',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'وَلَابْتَدَأَ',
    'stem': u'ابْتَدَأَ',
    'type': u'STOPWORD',
    'original': u'ابْتَدَأَ',
    'encletic': u''
}
STOPWORDS[u'فلابتدأ'] = {
    'word': u'فلابتدأ',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'فَلَابْتَدَأَ',
    'stem': u'ابْتَدَأَ',
    'type': u'STOPWORD',
    'original': u'ابْتَدَأَ',
    'encletic': u''
}
STOPWORDS[u'اخلولق'] = {
    'word': u'اِخْلَوْلَقَ',
    'procletic': u'',
    'tags': u'أداة:كاد و اخواتها',
    'vocalized': u'اِخْلَوْلَقَ',
    'stem': u'اِخْلَوْلَقَ',
    'type': u'STOPWORD',
    'original': u'اِخْلَوْلَقَ',
    'encletic': u''
}
STOPWORDS[u'لاخلولق'] = {
    'word': u'لاخلولق',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور',
    'vocalized': u'لَاِخْلَوْلَقَ',
    'stem': u'اِخْلَوْلَقَ',
    'type': u'STOPWORD',
    'original': u'اِخْلَوْلَقَ',
    'encletic': u''
}
STOPWORDS[u'واخلولق'] = {
    'word': u'واخلولق',
    'procletic': u'-وَ',
    'tags': u'أداة:كاد و اخواتها:معطوف',
    'vocalized': u'وَاِخْلَوْلَقَ',
    'stem': u'اِخْلَوْلَقَ',
    'type': u'STOPWORD',
    'original': u'اِخْلَوْلَقَ',
    'encletic': u''
}
STOPWORDS[u'فاخلولق'] = {
    'word': u'فاخلولق',
    'procletic': u'-فَ',
    'tags': u'أداة:كاد و اخواتها:معطوف',
    'vocalized': u'فَاِخْلَوْلَقَ',
    'stem': u'اِخْلَوْلَقَ',
    'type': u'STOPWORD',
    'original': u'اِخْلَوْلَقَ',
    'encletic': u''
}
STOPWORDS[u'ولاخلولق'] = {
    'word': u'ولاخلولق',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'وَلَاِخْلَوْلَقَ',
    'stem': u'اِخْلَوْلَقَ',
    'type': u'STOPWORD',
    'original': u'اِخْلَوْلَقَ',
    'encletic': u''
}
STOPWORDS[u'فلاخلولق'] = {
    'word': u'فلاخلولق',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'فَلَاِخْلَوْلَقَ',
    'stem': u'اِخْلَوْلَقَ',
    'type': u'STOPWORD',
    'original': u'اِخْلَوْلَقَ',
    'encletic': u''
}
STOPWORDS[u'انبرى'] = {
    'word': u'اِنْبَرَى',
    'procletic': u'',
    'tags': u'أداة:كاد و اخواتها',
    'vocalized': u'اِنْبَرَى',
    'stem': u'اِنْبَرَى',
    'type': u'STOPWORD',
    'original': u'اِنْبَرَى',
    'encletic': u''
}
STOPWORDS[u'لانبرى'] = {
    'word': u'لانبرى',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور',
    'vocalized': u'لَاِنْبَرَى',
    'stem': u'اِنْبَرَى',
    'type': u'STOPWORD',
    'original': u'اِنْبَرَى',
    'encletic': u''
}
STOPWORDS[u'وانبرى'] = {
    'word': u'وانبرى',
    'procletic': u'-وَ',
    'tags': u'أداة:كاد و اخواتها:معطوف',
    'vocalized': u'وَاِنْبَرَى',
    'stem': u'اِنْبَرَى',
    'type': u'STOPWORD',
    'original': u'اِنْبَرَى',
    'encletic': u''
}
STOPWORDS[u'فانبرى'] = {
    'word': u'فانبرى',
    'procletic': u'-فَ',
    'tags': u'أداة:كاد و اخواتها:معطوف',
    'vocalized': u'فَاِنْبَرَى',
    'stem': u'اِنْبَرَى',
    'type': u'STOPWORD',
    'original': u'اِنْبَرَى',
    'encletic': u''
}
STOPWORDS[u'ولانبرى'] = {
    'word': u'ولانبرى',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'وَلَاِنْبَرَى',
    'stem': u'اِنْبَرَى',
    'type': u'STOPWORD',
    'original': u'اِنْبَرَى',
    'encletic': u''
}
STOPWORDS[u'فلانبرى'] = {
    'word': u'فلانبرى',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'فَلَاِنْبَرَى',
    'stem': u'اِنْبَرَى',
    'type': u'STOPWORD',
    'original': u'اِنْبَرَى',
    'encletic': u''
}
STOPWORDS[u'أخذ'] = {
    'word': u'أَخَذَ',
    'procletic': u'',
    'tags': u'أداة:كاد و اخواتها',
    'vocalized': u'أَخَذَ',
    'stem': u'أَخَذَ',
    'type': u'STOPWORD',
    'original': u'أَخَذَ',
    'encletic': u''
}
STOPWORDS[u'لأخذ'] = {
    'word': u'لأخذ',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور',
    'vocalized': u'لَأَخَذَ',
    'stem': u'أَخَذَ',
    'type': u'STOPWORD',
    'original': u'أَخَذَ',
    'encletic': u''
}
STOPWORDS[u'وأخذ'] = {
    'word': u'وأخذ',
    'procletic': u'-وَ',
    'tags': u'أداة:كاد و اخواتها:معطوف',
    'vocalized': u'وَأَخَذَ',
    'stem': u'أَخَذَ',
    'type': u'STOPWORD',
    'original': u'أَخَذَ',
    'encletic': u''
}
STOPWORDS[u'فأخذ'] = {
    'word': u'فأخذ',
    'procletic': u'-فَ',
    'tags': u'أداة:كاد و اخواتها:معطوف',
    'vocalized': u'فَأَخَذَ',
    'stem': u'أَخَذَ',
    'type': u'STOPWORD',
    'original': u'أَخَذَ',
    'encletic': u''
}
STOPWORDS[u'ولأخذ'] = {
    'word': u'ولأخذ',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'وَلَأَخَذَ',
    'stem': u'أَخَذَ',
    'type': u'STOPWORD',
    'original': u'أَخَذَ',
    'encletic': u''
}
STOPWORDS[u'فلأخذ'] = {
    'word': u'فلأخذ',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'فَلَأَخَذَ',
    'stem': u'أَخَذَ',
    'type': u'STOPWORD',
    'original': u'أَخَذَ',
    'encletic': u''
}
STOPWORDS[u'أقبل'] = {
    'word': u'أَقْبَلَ',
    'procletic': u'',
    'tags': u'أداة:كاد و اخواتها',
    'vocalized': u'أَقْبَلَ',
    'stem': u'أَقْبَلَ',
    'type': u'STOPWORD',
    'original': u'أَقْبَلَ',
    'encletic': u''
}
STOPWORDS[u'لأقبل'] = {
    'word': u'لأقبل',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور',
    'vocalized': u'لَأَقْبَلَ',
    'stem': u'أَقْبَلَ',
    'type': u'STOPWORD',
    'original': u'أَقْبَلَ',
    'encletic': u''
}
STOPWORDS[u'وأقبل'] = {
    'word': u'وأقبل',
    'procletic': u'-وَ',
    'tags': u'أداة:كاد و اخواتها:معطوف',
    'vocalized': u'وَأَقْبَلَ',
    'stem': u'أَقْبَلَ',
    'type': u'STOPWORD',
    'original': u'أَقْبَلَ',
    'encletic': u''
}
STOPWORDS[u'فأقبل'] = {
    'word': u'فأقبل',
    'procletic': u'-فَ',
    'tags': u'أداة:كاد و اخواتها:معطوف',
    'vocalized': u'فَأَقْبَلَ',
    'stem': u'أَقْبَلَ',
    'type': u'STOPWORD',
    'original': u'أَقْبَلَ',
    'encletic': u''
}
STOPWORDS[u'ولأقبل'] = {
    'word': u'ولأقبل',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'وَلَأَقْبَلَ',
    'stem': u'أَقْبَلَ',
    'type': u'STOPWORD',
    'original': u'أَقْبَلَ',
    'encletic': u''
}
STOPWORDS[u'فلأقبل'] = {
    'word': u'فلأقبل',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'فَلَأَقْبَلَ',
    'stem': u'أَقْبَلَ',
    'type': u'STOPWORD',
    'original': u'أَقْبَلَ',
    'encletic': u''
}
STOPWORDS[u'أنشأ'] = {
    'word': u'أَنْشَأَ',
    'procletic': u'',
    'tags': u'أداة:كاد و اخواتها',
    'vocalized': u'أَنْشَأَ',
    'stem': u'أَنْشَأَ',
    'type': u'STOPWORD',
    'original': u'أَنْشَأَ',
    'encletic': u''
}
STOPWORDS[u'لأنشأ'] = {
    'word': u'لأنشأ',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور',
    'vocalized': u'لَأَنْشَأَ',
    'stem': u'أَنْشَأَ',
    'type': u'STOPWORD',
    'original': u'أَنْشَأَ',
    'encletic': u''
}
STOPWORDS[u'وأنشأ'] = {
    'word': u'وأنشأ',
    'procletic': u'-وَ',
    'tags': u'أداة:كاد و اخواتها:معطوف',
    'vocalized': u'وَأَنْشَأَ',
    'stem': u'أَنْشَأَ',
    'type': u'STOPWORD',
    'original': u'أَنْشَأَ',
    'encletic': u''
}
STOPWORDS[u'فأنشأ'] = {
    'word': u'فأنشأ',
    'procletic': u'-فَ',
    'tags': u'أداة:كاد و اخواتها:معطوف',
    'vocalized': u'فَأَنْشَأَ',
    'stem': u'أَنْشَأَ',
    'type': u'STOPWORD',
    'original': u'أَنْشَأَ',
    'encletic': u''
}
STOPWORDS[u'ولأنشأ'] = {
    'word': u'ولأنشأ',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'وَلَأَنْشَأَ',
    'stem': u'أَنْشَأَ',
    'type': u'STOPWORD',
    'original': u'أَنْشَأَ',
    'encletic': u''
}
STOPWORDS[u'فلأنشأ'] = {
    'word': u'فلأنشأ',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'فَلَأَنْشَأَ',
    'stem': u'أَنْشَأَ',
    'type': u'STOPWORD',
    'original': u'أَنْشَأَ',
    'encletic': u''
}
STOPWORDS[u'أوشك'] = {
    'word': u'أَوْشَكَ',
    'procletic': u'',
    'tags': u'أداة:كاد و اخواتها',
    'vocalized': u'أَوْشَكَ',
    'stem': u'أَوْشَكَ',
    'type': u'STOPWORD',
    'original': u'أَوْشَكَ',
    'encletic': u''
}
STOPWORDS[u'لأوشك'] = {
    'word': u'لأوشك',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور',
    'vocalized': u'لَأَوْشَكَ',
    'stem': u'أَوْشَكَ',
    'type': u'STOPWORD',
    'original': u'أَوْشَكَ',
    'encletic': u''
}
STOPWORDS[u'وأوشك'] = {
    'word': u'وأوشك',
    'procletic': u'-وَ',
    'tags': u'أداة:كاد و اخواتها:معطوف',
    'vocalized': u'وَأَوْشَكَ',
    'stem': u'أَوْشَكَ',
    'type': u'STOPWORD',
    'original': u'أَوْشَكَ',
    'encletic': u''
}
STOPWORDS[u'فأوشك'] = {
    'word': u'فأوشك',
    'procletic': u'-فَ',
    'tags': u'أداة:كاد و اخواتها:معطوف',
    'vocalized': u'فَأَوْشَكَ',
    'stem': u'أَوْشَكَ',
    'type': u'STOPWORD',
    'original': u'أَوْشَكَ',
    'encletic': u''
}
STOPWORDS[u'ولأوشك'] = {
    'word': u'ولأوشك',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'وَلَأَوْشَكَ',
    'stem': u'أَوْشَكَ',
    'type': u'STOPWORD',
    'original': u'أَوْشَكَ',
    'encletic': u''
}
STOPWORDS[u'فلأوشك'] = {
    'word': u'فلأوشك',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'فَلَأَوْشَكَ',
    'stem': u'أَوْشَكَ',
    'type': u'STOPWORD',
    'original': u'أَوْشَكَ',
    'encletic': u''
}
STOPWORDS[u'جعل'] = {
    'word': u'جَعَلَ',
    'procletic': u'',
    'tags': u'أداة:كاد و اخواتها',
    'vocalized': u'جَعَلَ',
    'stem': u'جَعَلَ',
    'type': u'STOPWORD',
    'original': u'جَعَلَ',
    'encletic': u''
}
STOPWORDS[u'لجعل'] = {
    'word': u'لجعل',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور',
    'vocalized': u'لَجَعَلَ',
    'stem': u'جَعَلَ',
    'type': u'STOPWORD',
    'original': u'جَعَلَ',
    'encletic': u''
}
STOPWORDS[u'وجعل'] = {
    'word': u'وجعل',
    'procletic': u'-وَ',
    'tags': u'أداة:كاد و اخواتها:معطوف',
    'vocalized': u'وَجَعَلَ',
    'stem': u'جَعَلَ',
    'type': u'STOPWORD',
    'original': u'جَعَلَ',
    'encletic': u''
}
STOPWORDS[u'فجعل'] = {
    'word': u'فجعل',
    'procletic': u'-فَ',
    'tags': u'أداة:كاد و اخواتها:معطوف',
    'vocalized': u'فَجَعَلَ',
    'stem': u'جَعَلَ',
    'type': u'STOPWORD',
    'original': u'جَعَلَ',
    'encletic': u''
}
STOPWORDS[u'ولجعل'] = {
    'word': u'ولجعل',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'وَلَجَعَلَ',
    'stem': u'جَعَلَ',
    'type': u'STOPWORD',
    'original': u'جَعَلَ',
    'encletic': u''
}
STOPWORDS[u'فلجعل'] = {
    'word': u'فلجعل',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'فَلَجَعَلَ',
    'stem': u'جَعَلَ',
    'type': u'STOPWORD',
    'original': u'جَعَلَ',
    'encletic': u''
}
STOPWORDS[u'حرى'] = {
    'word': u'حَرَى',
    'procletic': u'',
    'tags': u'أداة:كاد و اخواتها',
    'vocalized': u'حَرَى',
    'stem': u'حَرَى',
    'type': u'STOPWORD',
    'original': u'حَرَى',
    'encletic': u''
}
STOPWORDS[u'لحرى'] = {
    'word': u'لحرى',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور',
    'vocalized': u'لَحَرَى',
    'stem': u'حَرَى',
    'type': u'STOPWORD',
    'original': u'حَرَى',
    'encletic': u''
}
STOPWORDS[u'وحرى'] = {
    'word': u'وحرى',
    'procletic': u'-وَ',
    'tags': u'أداة:كاد و اخواتها:معطوف',
    'vocalized': u'وَحَرَى',
    'stem': u'حَرَى',
    'type': u'STOPWORD',
    'original': u'حَرَى',
    'encletic': u''
}
STOPWORDS[u'فحرى'] = {
    'word': u'فحرى',
    'procletic': u'-فَ',
    'tags': u'أداة:كاد و اخواتها:معطوف',
    'vocalized': u'فَحَرَى',
    'stem': u'حَرَى',
    'type': u'STOPWORD',
    'original': u'حَرَى',
    'encletic': u''
}
STOPWORDS[u'ولحرى'] = {
    'word': u'ولحرى',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'وَلَحَرَى',
    'stem': u'حَرَى',
    'type': u'STOPWORD',
    'original': u'حَرَى',
    'encletic': u''
}
STOPWORDS[u'فلحرى'] = {
    'word': u'فلحرى',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'فَلَحَرَى',
    'stem': u'حَرَى',
    'type': u'STOPWORD',
    'original': u'حَرَى',
    'encletic': u''
}
STOPWORDS[u'شرع'] = {
    'word': u'شَرَعَ',
    'procletic': u'',
    'tags': u'أداة:كاد و اخواتها',
    'vocalized': u'شَرَعَ',
    'stem': u'شَرَعَ',
    'type': u'STOPWORD',
    'original': u'شَرَعَ',
    'encletic': u''
}
STOPWORDS[u'لشرع'] = {
    'word': u'لشرع',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور',
    'vocalized': u'لَشَرَعَ',
    'stem': u'شَرَعَ',
    'type': u'STOPWORD',
    'original': u'شَرَعَ',
    'encletic': u''
}
STOPWORDS[u'وشرع'] = {
    'word': u'وشرع',
    'procletic': u'-وَ',
    'tags': u'أداة:كاد و اخواتها:معطوف',
    'vocalized': u'وَشَرَعَ',
    'stem': u'شَرَعَ',
    'type': u'STOPWORD',
    'original': u'شَرَعَ',
    'encletic': u''
}
STOPWORDS[u'فشرع'] = {
    'word': u'فشرع',
    'procletic': u'-فَ',
    'tags': u'أداة:كاد و اخواتها:معطوف',
    'vocalized': u'فَشَرَعَ',
    'stem': u'شَرَعَ',
    'type': u'STOPWORD',
    'original': u'شَرَعَ',
    'encletic': u''
}
STOPWORDS[u'ولشرع'] = {
    'word': u'ولشرع',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'وَلَشَرَعَ',
    'stem': u'شَرَعَ',
    'type': u'STOPWORD',
    'original': u'شَرَعَ',
    'encletic': u''
}
STOPWORDS[u'فلشرع'] = {
    'word': u'فلشرع',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'فَلَشَرَعَ',
    'stem': u'شَرَعَ',
    'type': u'STOPWORD',
    'original': u'شَرَعَ',
    'encletic': u''
}
STOPWORDS[u'طفق'] = {
    'word': u'طَفِقَ',
    'procletic': u'',
    'tags': u'أداة:كاد و اخواتها',
    'vocalized': u'طَفِقَ',
    'stem': u'طَفِقَ',
    'type': u'STOPWORD',
    'original': u'طَفِقَ',
    'encletic': u''
}
STOPWORDS[u'لطفق'] = {
    'word': u'لطفق',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور',
    'vocalized': u'لَطَفِقَ',
    'stem': u'طَفِقَ',
    'type': u'STOPWORD',
    'original': u'طَفِقَ',
    'encletic': u''
}
STOPWORDS[u'وطفق'] = {
    'word': u'وطفق',
    'procletic': u'-وَ',
    'tags': u'أداة:كاد و اخواتها:معطوف',
    'vocalized': u'وَطَفِقَ',
    'stem': u'طَفِقَ',
    'type': u'STOPWORD',
    'original': u'طَفِقَ',
    'encletic': u''
}
STOPWORDS[u'فطفق'] = {
    'word': u'فطفق',
    'procletic': u'-فَ',
    'tags': u'أداة:كاد و اخواتها:معطوف',
    'vocalized': u'فَطَفِقَ',
    'stem': u'طَفِقَ',
    'type': u'STOPWORD',
    'original': u'طَفِقَ',
    'encletic': u''
}
STOPWORDS[u'ولطفق'] = {
    'word': u'ولطفق',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'وَلَطَفِقَ',
    'stem': u'طَفِقَ',
    'type': u'STOPWORD',
    'original': u'طَفِقَ',
    'encletic': u''
}
STOPWORDS[u'فلطفق'] = {
    'word': u'فلطفق',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'فَلَطَفِقَ',
    'stem': u'طَفِقَ',
    'type': u'STOPWORD',
    'original': u'طَفِقَ',
    'encletic': u''
}
STOPWORDS[u'عسى'] = {
    'word': u'عَسَى',
    'procletic': u'',
    'tags': u'أداة:كاد و اخواتها',
    'vocalized': u'عَسَى',
    'stem': u'عَسَى',
    'type': u'STOPWORD',
    'original': u'عَسَى',
    'encletic': u''
}
STOPWORDS[u'لعسى'] = {
    'word': u'لعسى',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور',
    'vocalized': u'لَعَسَى',
    'stem': u'عَسَى',
    'type': u'STOPWORD',
    'original': u'عَسَى',
    'encletic': u''
}
STOPWORDS[u'وعسى'] = {
    'word': u'وعسى',
    'procletic': u'-وَ',
    'tags': u'أداة:كاد و اخواتها:معطوف',
    'vocalized': u'وَعَسَى',
    'stem': u'عَسَى',
    'type': u'STOPWORD',
    'original': u'عَسَى',
    'encletic': u''
}
STOPWORDS[u'فعسى'] = {
    'word': u'فعسى',
    'procletic': u'-فَ',
    'tags': u'أداة:كاد و اخواتها:معطوف',
    'vocalized': u'فَعَسَى',
    'stem': u'عَسَى',
    'type': u'STOPWORD',
    'original': u'عَسَى',
    'encletic': u''
}
STOPWORDS[u'ولعسى'] = {
    'word': u'ولعسى',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'وَلَعَسَى',
    'stem': u'عَسَى',
    'type': u'STOPWORD',
    'original': u'عَسَى',
    'encletic': u''
}
STOPWORDS[u'فلعسى'] = {
    'word': u'فلعسى',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'فَلَعَسَى',
    'stem': u'عَسَى',
    'type': u'STOPWORD',
    'original': u'عَسَى',
    'encletic': u''
}
STOPWORDS[u'علق'] = {
    'word': u'عَلِقَ',
    'procletic': u'',
    'tags': u'أداة:كاد و اخواتها',
    'vocalized': u'عَلِقَ',
    'stem': u'عَلِقَ',
    'type': u'STOPWORD',
    'original': u'عَلِقَ',
    'encletic': u''
}
STOPWORDS[u'لعلق'] = {
    'word': u'لعلق',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور',
    'vocalized': u'لَعَلِقَ',
    'stem': u'عَلِقَ',
    'type': u'STOPWORD',
    'original': u'عَلِقَ',
    'encletic': u''
}
STOPWORDS[u'وعلق'] = {
    'word': u'وعلق',
    'procletic': u'-وَ',
    'tags': u'أداة:كاد و اخواتها:معطوف',
    'vocalized': u'وَعَلِقَ',
    'stem': u'عَلِقَ',
    'type': u'STOPWORD',
    'original': u'عَلِقَ',
    'encletic': u''
}
STOPWORDS[u'فعلق'] = {
    'word': u'فعلق',
    'procletic': u'-فَ',
    'tags': u'أداة:كاد و اخواتها:معطوف',
    'vocalized': u'فَعَلِقَ',
    'stem': u'عَلِقَ',
    'type': u'STOPWORD',
    'original': u'عَلِقَ',
    'encletic': u''
}
STOPWORDS[u'ولعلق'] = {
    'word': u'ولعلق',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'وَلَعَلِقَ',
    'stem': u'عَلِقَ',
    'type': u'STOPWORD',
    'original': u'عَلِقَ',
    'encletic': u''
}
STOPWORDS[u'فلعلق'] = {
    'word': u'فلعلق',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'فَلَعَلِقَ',
    'stem': u'عَلِقَ',
    'type': u'STOPWORD',
    'original': u'عَلِقَ',
    'encletic': u''
}
STOPWORDS[u'قام'] = {
    'word': u'قَامَ',
    'procletic': u'',
    'tags': u'أداة:كاد و اخواتها',
    'vocalized': u'قَامَ',
    'stem': u'قَامَ',
    'type': u'STOPWORD',
    'original': u'قَامَ',
    'encletic': u''
}
STOPWORDS[u'لقام'] = {
    'word': u'لقام',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور',
    'vocalized': u'لَقَامَ',
    'stem': u'قَامَ',
    'type': u'STOPWORD',
    'original': u'قَامَ',
    'encletic': u''
}
STOPWORDS[u'وقام'] = {
    'word': u'وقام',
    'procletic': u'-وَ',
    'tags': u'أداة:كاد و اخواتها:معطوف',
    'vocalized': u'وَقَامَ',
    'stem': u'قَامَ',
    'type': u'STOPWORD',
    'original': u'قَامَ',
    'encletic': u''
}
STOPWORDS[u'فقام'] = {
    'word': u'فقام',
    'procletic': u'-فَ',
    'tags': u'أداة:كاد و اخواتها:معطوف',
    'vocalized': u'فَقَامَ',
    'stem': u'قَامَ',
    'type': u'STOPWORD',
    'original': u'قَامَ',
    'encletic': u''
}
STOPWORDS[u'ولقام'] = {
    'word': u'ولقام',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'وَلَقَامَ',
    'stem': u'قَامَ',
    'type': u'STOPWORD',
    'original': u'قَامَ',
    'encletic': u''
}
STOPWORDS[u'فلقام'] = {
    'word': u'فلقام',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'فَلَقَامَ',
    'stem': u'قَامَ',
    'type': u'STOPWORD',
    'original': u'قَامَ',
    'encletic': u''
}
STOPWORDS[u'كاد'] = {
    'word': u'كَادَ',
    'procletic': u'',
    'tags': u'أداة:كاد و اخواتها',
    'vocalized': u'كَادَ',
    'stem': u'كَادَ',
    'type': u'STOPWORD',
    'original': u'كَادَ',
    'encletic': u''
}
STOPWORDS[u'لكاد'] = {
    'word': u'لكاد',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور',
    'vocalized': u'لَكَادَ',
    'stem': u'كَادَ',
    'type': u'STOPWORD',
    'original': u'كَادَ',
    'encletic': u''
}
STOPWORDS[u'وكاد'] = {
    'word': u'وكاد',
    'procletic': u'-وَ',
    'tags': u'أداة:كاد و اخواتها:معطوف',
    'vocalized': u'وَكَادَ',
    'stem': u'كَادَ',
    'type': u'STOPWORD',
    'original': u'كَادَ',
    'encletic': u''
}
STOPWORDS[u'فكاد'] = {
    'word': u'فكاد',
    'procletic': u'-فَ',
    'tags': u'أداة:كاد و اخواتها:معطوف',
    'vocalized': u'فَكَادَ',
    'stem': u'كَادَ',
    'type': u'STOPWORD',
    'original': u'كَادَ',
    'encletic': u''
}
STOPWORDS[u'ولكاد'] = {
    'word': u'ولكاد',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'وَلَكَادَ',
    'stem': u'كَادَ',
    'type': u'STOPWORD',
    'original': u'كَادَ',
    'encletic': u''
}
STOPWORDS[u'فلكاد'] = {
    'word': u'فلكاد',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'فَلَكَادَ',
    'stem': u'كَادَ',
    'type': u'STOPWORD',
    'original': u'كَادَ',
    'encletic': u''
}
STOPWORDS[u'كرب'] = {
    'word': u'كَرِبَ',
    'procletic': u'',
    'tags': u'أداة:كاد و اخواتها',
    'vocalized': u'كَرِبَ',
    'stem': u'كَرِبَ',
    'type': u'STOPWORD',
    'original': u'كَرِبَ',
    'encletic': u''
}
STOPWORDS[u'لكرب'] = {
    'word': u'لكرب',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور',
    'vocalized': u'لَكَرِبَ',
    'stem': u'كَرِبَ',
    'type': u'STOPWORD',
    'original': u'كَرِبَ',
    'encletic': u''
}
STOPWORDS[u'وكرب'] = {
    'word': u'وكرب',
    'procletic': u'-وَ',
    'tags': u'أداة:كاد و اخواتها:معطوف',
    'vocalized': u'وَكَرِبَ',
    'stem': u'كَرِبَ',
    'type': u'STOPWORD',
    'original': u'كَرِبَ',
    'encletic': u''
}
STOPWORDS[u'فكرب'] = {
    'word': u'فكرب',
    'procletic': u'-فَ',
    'tags': u'أداة:كاد و اخواتها:معطوف',
    'vocalized': u'فَكَرِبَ',
    'stem': u'كَرِبَ',
    'type': u'STOPWORD',
    'original': u'كَرِبَ',
    'encletic': u''
}
STOPWORDS[u'ولكرب'] = {
    'word': u'ولكرب',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'وَلَكَرِبَ',
    'stem': u'كَرِبَ',
    'type': u'STOPWORD',
    'original': u'كَرِبَ',
    'encletic': u''
}
STOPWORDS[u'فلكرب'] = {
    'word': u'فلكرب',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'فَلَكَرِبَ',
    'stem': u'كَرِبَ',
    'type': u'STOPWORD',
    'original': u'كَرِبَ',
    'encletic': u''
}
STOPWORDS[u'هب'] = {
    'word': u'هَبَّ',
    'procletic': u'',
    'tags': u'أداة:كاد و اخواتها',
    'vocalized': u'هَبَّ',
    'stem': u'هَبَّ',
    'type': u'STOPWORD',
    'original': u'هَبَّ',
    'encletic': u''
}
STOPWORDS[u'لهب'] = {
    'word': u'لهب',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور',
    'vocalized': u'لَهَبَّ',
    'stem': u'هَبَّ',
    'type': u'STOPWORD',
    'original': u'هَبَّ',
    'encletic': u''
}
STOPWORDS[u'وهب'] = {
    'word': u'وهب',
    'procletic': u'-وَ',
    'tags': u'أداة:كاد و اخواتها:معطوف',
    'vocalized': u'وَهَبَّ',
    'stem': u'هَبَّ',
    'type': u'STOPWORD',
    'original': u'هَبَّ',
    'encletic': u''
}
STOPWORDS[u'فهب'] = {
    'word': u'فهب',
    'procletic': u'-فَ',
    'tags': u'أداة:كاد و اخواتها:معطوف',
    'vocalized': u'فَهَبَّ',
    'stem': u'هَبَّ',
    'type': u'STOPWORD',
    'original': u'هَبَّ',
    'encletic': u''
}
STOPWORDS[u'ولهب'] = {
    'word': u'ولهب',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'وَلَهَبَّ',
    'stem': u'هَبَّ',
    'type': u'STOPWORD',
    'original': u'هَبَّ',
    'encletic': u''
}
STOPWORDS[u'فلهب'] = {
    'word': u'فلهب',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'فَلَهَبَّ',
    'stem': u'هَبَّ',
    'type': u'STOPWORD',
    'original': u'هَبَّ',
    'encletic': u''
}
STOPWORDS[u'إنما'] = {
    'word': u'إِنَّمَا',
    'procletic': u'',
    'tags': u'أداة:كافة ومكفوفة',
    'vocalized': u'إِنَّمَا',
    'stem': u'إِنَّمَا',
    'type': u'STOPWORD',
    'original': u'إِنَّمَا',
    'encletic': u''
}
STOPWORDS[u'وإنما'] = {
    'word': u'وإنما',
    'procletic': u'-وَ',
    'tags': u'أداة:كافة ومكفوفة:معطوف',
    'vocalized': u'وَإِنَّمَا',
    'stem': u'إِنَّمَا',
    'type': u'STOPWORD',
    'original': u'إِنَّمَا',
    'encletic': u''
}
STOPWORDS[u'فإنما'] = {
    'word': u'فإنما',
    'procletic': u'-فَ',
    'tags': u'أداة:كافة ومكفوفة:معطوف',
    'vocalized': u'فَإِنَّمَا',
    'stem': u'إِنَّمَا',
    'type': u'STOPWORD',
    'original': u'إِنَّمَا',
    'encletic': u''
}
STOPWORDS[u'لكنما'] = {
    'word': u'لَكِنَّمَا',
    'procletic': u'',
    'tags': u'أداة:كافة ومكفوفة',
    'vocalized': u'لَكِنَّمَا',
    'stem': u'لَكِنَّمَا',
    'type': u'STOPWORD',
    'original': u'لَكِنَّمَا',
    'encletic': u''
}
STOPWORDS[u'ولكنما'] = {
    'word': u'ولكنما',
    'procletic': u'-وَ',
    'tags': u'أداة:كافة ومكفوفة:معطوف',
    'vocalized': u'وَلَكِنَّمَا',
    'stem': u'لَكِنَّمَا',
    'type': u'STOPWORD',
    'original': u'لَكِنَّمَا',
    'encletic': u''
}
STOPWORDS[u'فلكنما'] = {
    'word': u'فلكنما',
    'procletic': u'-فَ',
    'tags': u'أداة:كافة ومكفوفة:معطوف',
    'vocalized': u'فَلَكِنَّمَا',
    'stem': u'لَكِنَّمَا',
    'type': u'STOPWORD',
    'original': u'لَكِنَّمَا',
    'encletic': u''
}
STOPWORDS[u'ارتد'] = {
    'word': u'اِرْتَدَّ',
    'procletic': u'',
    'tags': u'أداة:كان و أخواتها',
    'vocalized': u'اِرْتَدَّ',
    'stem': u'اِرْتَدَّ',
    'type': u'STOPWORD',
    'original': u'اِرْتَدَّ',
    'encletic': u''
}
STOPWORDS[u'لارتد'] = {
    'word': u'لارتد',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور',
    'vocalized': u'لَاِرْتَدَّ',
    'stem': u'اِرْتَدَّ',
    'type': u'STOPWORD',
    'original': u'اِرْتَدَّ',
    'encletic': u''
}
STOPWORDS[u'وارتد'] = {
    'word': u'وارتد',
    'procletic': u'-وَ',
    'tags': u'أداة:كان و أخواتها:معطوف',
    'vocalized': u'وَاِرْتَدَّ',
    'stem': u'اِرْتَدَّ',
    'type': u'STOPWORD',
    'original': u'اِرْتَدَّ',
    'encletic': u''
}
STOPWORDS[u'فارتد'] = {
    'word': u'فارتد',
    'procletic': u'-فَ',
    'tags': u'أداة:كان و أخواتها:معطوف',
    'vocalized': u'فَاِرْتَدَّ',
    'stem': u'اِرْتَدَّ',
    'type': u'STOPWORD',
    'original': u'اِرْتَدَّ',
    'encletic': u''
}
STOPWORDS[u'ولارتد'] = {
    'word': u'ولارتد',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'وَلَاِرْتَدَّ',
    'stem': u'اِرْتَدَّ',
    'type': u'STOPWORD',
    'original': u'اِرْتَدَّ',
    'encletic': u''
}
STOPWORDS[u'فلارتد'] = {
    'word': u'فلارتد',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'فَلَاِرْتَدَّ',
    'stem': u'اِرْتَدَّ',
    'type': u'STOPWORD',
    'original': u'اِرْتَدَّ',
    'encletic': u''
}
STOPWORDS[u'استحال'] = {
    'word': u'اِسْتَحَالَ',
    'procletic': u'',
    'tags': u'أداة:كان و أخواتها',
    'vocalized': u'اِسْتَحَالَ',
    'stem': u'اِسْتَحَالَ',
    'type': u'STOPWORD',
    'original': u'اِسْتَحَالَ',
    'encletic': u''
}
STOPWORDS[u'لاستحال'] = {
    'word': u'لاستحال',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور',
    'vocalized': u'لَاِسْتَحَالَ',
    'stem': u'اِسْتَحَالَ',
    'type': u'STOPWORD',
    'original': u'اِسْتَحَالَ',
    'encletic': u''
}
STOPWORDS[u'واستحال'] = {
    'word': u'واستحال',
    'procletic': u'-وَ',
    'tags': u'أداة:كان و أخواتها:معطوف',
    'vocalized': u'وَاِسْتَحَالَ',
    'stem': u'اِسْتَحَالَ',
    'type': u'STOPWORD',
    'original': u'اِسْتَحَالَ',
    'encletic': u''
}
STOPWORDS[u'فاستحال'] = {
    'word': u'فاستحال',
    'procletic': u'-فَ',
    'tags': u'أداة:كان و أخواتها:معطوف',
    'vocalized': u'فَاِسْتَحَالَ',
    'stem': u'اِسْتَحَالَ',
    'type': u'STOPWORD',
    'original': u'اِسْتَحَالَ',
    'encletic': u''
}
STOPWORDS[u'ولاستحال'] = {
    'word': u'ولاستحال',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'وَلَاِسْتَحَالَ',
    'stem': u'اِسْتَحَالَ',
    'type': u'STOPWORD',
    'original': u'اِسْتَحَالَ',
    'encletic': u''
}
STOPWORDS[u'فلاستحال'] = {
    'word': u'فلاستحال',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'فَلَاِسْتَحَالَ',
    'stem': u'اِسْتَحَالَ',
    'type': u'STOPWORD',
    'original': u'اِسْتَحَالَ',
    'encletic': u''
}
STOPWORDS[u'انقلب'] = {
    'word': u'اِنْقَلَبَ',
    'procletic': u'',
    'tags': u'أداة:كان و أخواتها',
    'vocalized': u'اِنْقَلَبَ',
    'stem': u'اِنْقَلَبَ',
    'type': u'STOPWORD',
    'original': u'اِنْقَلَبَ',
    'encletic': u''
}
STOPWORDS[u'لانقلب'] = {
    'word': u'لانقلب',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور',
    'vocalized': u'لَاِنْقَلَبَ',
    'stem': u'اِنْقَلَبَ',
    'type': u'STOPWORD',
    'original': u'اِنْقَلَبَ',
    'encletic': u''
}
STOPWORDS[u'وانقلب'] = {
    'word': u'وانقلب',
    'procletic': u'-وَ',
    'tags': u'أداة:كان و أخواتها:معطوف',
    'vocalized': u'وَاِنْقَلَبَ',
    'stem': u'اِنْقَلَبَ',
    'type': u'STOPWORD',
    'original': u'اِنْقَلَبَ',
    'encletic': u''
}
STOPWORDS[u'فانقلب'] = {
    'word': u'فانقلب',
    'procletic': u'-فَ',
    'tags': u'أداة:كان و أخواتها:معطوف',
    'vocalized': u'فَاِنْقَلَبَ',
    'stem': u'اِنْقَلَبَ',
    'type': u'STOPWORD',
    'original': u'اِنْقَلَبَ',
    'encletic': u''
}
STOPWORDS[u'ولانقلب'] = {
    'word': u'ولانقلب',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'وَلَاِنْقَلَبَ',
    'stem': u'اِنْقَلَبَ',
    'type': u'STOPWORD',
    'original': u'اِنْقَلَبَ',
    'encletic': u''
}
STOPWORDS[u'فلانقلب'] = {
    'word': u'فلانقلب',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'فَلَاِنْقَلَبَ',
    'stem': u'اِنْقَلَبَ',
    'type': u'STOPWORD',
    'original': u'اِنْقَلَبَ',
    'encletic': u''
}
STOPWORDS[u'آض'] = {
    'word': u'آضَ',
    'procletic': u'',
    'tags': u'أداة:كان و أخواتها',
    'vocalized': u'آضَ',
    'stem': u'آضَ',
    'type': u'STOPWORD',
    'original': u'آضَ',
    'encletic': u''
}
STOPWORDS[u'لآض'] = {
    'word': u'لآض',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور',
    'vocalized': u'لَآضَ',
    'stem': u'آضَ',
    'type': u'STOPWORD',
    'original': u'آضَ',
    'encletic': u''
}
STOPWORDS[u'وآض'] = {
    'word': u'وآض',
    'procletic': u'-وَ',
    'tags': u'أداة:كان و أخواتها:معطوف',
    'vocalized': u'وَآضَ',
    'stem': u'آضَ',
    'type': u'STOPWORD',
    'original': u'آضَ',
    'encletic': u''
}
STOPWORDS[u'فآض'] = {
    'word': u'فآض',
    'procletic': u'-فَ',
    'tags': u'أداة:كان و أخواتها:معطوف',
    'vocalized': u'فَآضَ',
    'stem': u'آضَ',
    'type': u'STOPWORD',
    'original': u'آضَ',
    'encletic': u''
}
STOPWORDS[u'ولآض'] = {
    'word': u'ولآض',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'وَلَآضَ',
    'stem': u'آضَ',
    'type': u'STOPWORD',
    'original': u'آضَ',
    'encletic': u''
}
STOPWORDS[u'فلآض'] = {
    'word': u'فلآض',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'فَلَآضَ',
    'stem': u'آضَ',
    'type': u'STOPWORD',
    'original': u'آضَ',
    'encletic': u''
}
STOPWORDS[u'أصبح'] = {
    'word': u'أَصْبَحَ',
    'procletic': u'',
    'tags': u'أداة:كان و أخواتها',
    'vocalized': u'أَصْبَحَ',
    'stem': u'أَصْبَحَ',
    'type': u'STOPWORD',
    'original': u'أَصْبَحَ',
    'encletic': u''
}
STOPWORDS[u'لأصبح'] = {
    'word': u'لأصبح',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور',
    'vocalized': u'لَأَصْبَحَ',
    'stem': u'أَصْبَحَ',
    'type': u'STOPWORD',
    'original': u'أَصْبَحَ',
    'encletic': u''
}
STOPWORDS[u'وأصبح'] = {
    'word': u'وأصبح',
    'procletic': u'-وَ',
    'tags': u'أداة:كان و أخواتها:معطوف',
    'vocalized': u'وَأَصْبَحَ',
    'stem': u'أَصْبَحَ',
    'type': u'STOPWORD',
    'original': u'أَصْبَحَ',
    'encletic': u''
}
STOPWORDS[u'فأصبح'] = {
    'word': u'فأصبح',
    'procletic': u'-فَ',
    'tags': u'أداة:كان و أخواتها:معطوف',
    'vocalized': u'فَأَصْبَحَ',
    'stem': u'أَصْبَحَ',
    'type': u'STOPWORD',
    'original': u'أَصْبَحَ',
    'encletic': u''
}
STOPWORDS[u'ولأصبح'] = {
    'word': u'ولأصبح',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'وَلَأَصْبَحَ',
    'stem': u'أَصْبَحَ',
    'type': u'STOPWORD',
    'original': u'أَصْبَحَ',
    'encletic': u''
}
STOPWORDS[u'فلأصبح'] = {
    'word': u'فلأصبح',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'فَلَأَصْبَحَ',
    'stem': u'أَصْبَحَ',
    'type': u'STOPWORD',
    'original': u'أَصْبَحَ',
    'encletic': u''
}
STOPWORDS[u'أضحى'] = {
    'word': u'أَضْحَى',
    'procletic': u'',
    'tags': u'أداة:كان و أخواتها',
    'vocalized': u'أَضْحَى',
    'stem': u'أَضْحَى',
    'type': u'STOPWORD',
    'original': u'أَضْحَى',
    'encletic': u''
}
STOPWORDS[u'لأضحى'] = {
    'word': u'لأضحى',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور',
    'vocalized': u'لَأَضْحَى',
    'stem': u'أَضْحَى',
    'type': u'STOPWORD',
    'original': u'أَضْحَى',
    'encletic': u''
}
STOPWORDS[u'وأضحى'] = {
    'word': u'وأضحى',
    'procletic': u'-وَ',
    'tags': u'أداة:كان و أخواتها:معطوف',
    'vocalized': u'وَأَضْحَى',
    'stem': u'أَضْحَى',
    'type': u'STOPWORD',
    'original': u'أَضْحَى',
    'encletic': u''
}
STOPWORDS[u'فأضحى'] = {
    'word': u'فأضحى',
    'procletic': u'-فَ',
    'tags': u'أداة:كان و أخواتها:معطوف',
    'vocalized': u'فَأَضْحَى',
    'stem': u'أَضْحَى',
    'type': u'STOPWORD',
    'original': u'أَضْحَى',
    'encletic': u''
}
STOPWORDS[u'ولأضحى'] = {
    'word': u'ولأضحى',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'وَلَأَضْحَى',
    'stem': u'أَضْحَى',
    'type': u'STOPWORD',
    'original': u'أَضْحَى',
    'encletic': u''
}
STOPWORDS[u'فلأضحى'] = {
    'word': u'فلأضحى',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'فَلَأَضْحَى',
    'stem': u'أَضْحَى',
    'type': u'STOPWORD',
    'original': u'أَضْحَى',
    'encletic': u''
}
STOPWORDS[u'أمسى'] = {
    'word': u'أَمْسَى',
    'procletic': u'',
    'tags': u'أداة:كان و أخواتها',
    'vocalized': u'أَمْسَى',
    'stem': u'أَمْسَى',
    'type': u'STOPWORD',
    'original': u'أَمْسَى',
    'encletic': u''
}
STOPWORDS[u'لأمسى'] = {
    'word': u'لأمسى',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور',
    'vocalized': u'لَأَمْسَى',
    'stem': u'أَمْسَى',
    'type': u'STOPWORD',
    'original': u'أَمْسَى',
    'encletic': u''
}
STOPWORDS[u'وأمسى'] = {
    'word': u'وأمسى',
    'procletic': u'-وَ',
    'tags': u'أداة:كان و أخواتها:معطوف',
    'vocalized': u'وَأَمْسَى',
    'stem': u'أَمْسَى',
    'type': u'STOPWORD',
    'original': u'أَمْسَى',
    'encletic': u''
}
STOPWORDS[u'فأمسى'] = {
    'word': u'فأمسى',
    'procletic': u'-فَ',
    'tags': u'أداة:كان و أخواتها:معطوف',
    'vocalized': u'فَأَمْسَى',
    'stem': u'أَمْسَى',
    'type': u'STOPWORD',
    'original': u'أَمْسَى',
    'encletic': u''
}
STOPWORDS[u'ولأمسى'] = {
    'word': u'ولأمسى',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'وَلَأَمْسَى',
    'stem': u'أَمْسَى',
    'type': u'STOPWORD',
    'original': u'أَمْسَى',
    'encletic': u''
}
STOPWORDS[u'فلأمسى'] = {
    'word': u'فلأمسى',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'فَلَأَمْسَى',
    'stem': u'أَمْسَى',
    'type': u'STOPWORD',
    'original': u'أَمْسَى',
    'encletic': u''
}
STOPWORDS[u'بات'] = {
    'word': u'بَاتَ',
    'procletic': u'',
    'tags': u'أداة:كان و أخواتها',
    'vocalized': u'بَاتَ',
    'stem': u'بَاتَ',
    'type': u'STOPWORD',
    'original': u'بَاتَ',
    'encletic': u''
}
STOPWORDS[u'لبات'] = {
    'word': u'لبات',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور',
    'vocalized': u'لَبَاتَ',
    'stem': u'بَاتَ',
    'type': u'STOPWORD',
    'original': u'بَاتَ',
    'encletic': u''
}
STOPWORDS[u'وبات'] = {
    'word': u'وبات',
    'procletic': u'-وَ',
    'tags': u'أداة:كان و أخواتها:معطوف',
    'vocalized': u'وَبَاتَ',
    'stem': u'بَاتَ',
    'type': u'STOPWORD',
    'original': u'بَاتَ',
    'encletic': u''
}
STOPWORDS[u'فبات'] = {
    'word': u'فبات',
    'procletic': u'-فَ',
    'tags': u'أداة:كان و أخواتها:معطوف',
    'vocalized': u'فَبَاتَ',
    'stem': u'بَاتَ',
    'type': u'STOPWORD',
    'original': u'بَاتَ',
    'encletic': u''
}
STOPWORDS[u'ولبات'] = {
    'word': u'ولبات',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'وَلَبَاتَ',
    'stem': u'بَاتَ',
    'type': u'STOPWORD',
    'original': u'بَاتَ',
    'encletic': u''
}
STOPWORDS[u'فلبات'] = {
    'word': u'فلبات',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'فَلَبَاتَ',
    'stem': u'بَاتَ',
    'type': u'STOPWORD',
    'original': u'بَاتَ',
    'encletic': u''
}
STOPWORDS[u'تبدل'] = {
    'word': u'تَبَدَّلَ',
    'procletic': u'',
    'tags': u'أداة:كان و أخواتها',
    'vocalized': u'تَبَدَّلَ',
    'stem': u'تَبَدَّلَ',
    'type': u'STOPWORD',
    'original': u'تَبَدَّلَ',
    'encletic': u''
}
STOPWORDS[u'لتبدل'] = {
    'word': u'لتبدل',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور',
    'vocalized': u'لَتَبَدَّلَ',
    'stem': u'تَبَدَّلَ',
    'type': u'STOPWORD',
    'original': u'تَبَدَّلَ',
    'encletic': u''
}
STOPWORDS[u'وتبدل'] = {
    'word': u'وتبدل',
    'procletic': u'-وَ',
    'tags': u'أداة:كان و أخواتها:معطوف',
    'vocalized': u'وَتَبَدَّلَ',
    'stem': u'تَبَدَّلَ',
    'type': u'STOPWORD',
    'original': u'تَبَدَّلَ',
    'encletic': u''
}
STOPWORDS[u'فتبدل'] = {
    'word': u'فتبدل',
    'procletic': u'-فَ',
    'tags': u'أداة:كان و أخواتها:معطوف',
    'vocalized': u'فَتَبَدَّلَ',
    'stem': u'تَبَدَّلَ',
    'type': u'STOPWORD',
    'original': u'تَبَدَّلَ',
    'encletic': u''
}
STOPWORDS[u'ولتبدل'] = {
    'word': u'ولتبدل',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'وَلَتَبَدَّلَ',
    'stem': u'تَبَدَّلَ',
    'type': u'STOPWORD',
    'original': u'تَبَدَّلَ',
    'encletic': u''
}
STOPWORDS[u'فلتبدل'] = {
    'word': u'فلتبدل',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'فَلَتَبَدَّلَ',
    'stem': u'تَبَدَّلَ',
    'type': u'STOPWORD',
    'original': u'تَبَدَّلَ',
    'encletic': u''
}
STOPWORDS[u'تحول'] = {
    'word': u'تَحَوَّلَ',
    'procletic': u'',
    'tags': u'أداة:كان و أخواتها',
    'vocalized': u'تَحَوَّلَ',
    'stem': u'تَحَوَّلَ',
    'type': u'STOPWORD',
    'original': u'تَحَوَّلَ',
    'encletic': u''
}
STOPWORDS[u'لتحول'] = {
    'word': u'لتحول',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور',
    'vocalized': u'لَتَحَوَّلَ',
    'stem': u'تَحَوَّلَ',
    'type': u'STOPWORD',
    'original': u'تَحَوَّلَ',
    'encletic': u''
}
STOPWORDS[u'وتحول'] = {
    'word': u'وتحول',
    'procletic': u'-وَ',
    'tags': u'أداة:كان و أخواتها:معطوف',
    'vocalized': u'وَتَحَوَّلَ',
    'stem': u'تَحَوَّلَ',
    'type': u'STOPWORD',
    'original': u'تَحَوَّلَ',
    'encletic': u''
}
STOPWORDS[u'فتحول'] = {
    'word': u'فتحول',
    'procletic': u'-فَ',
    'tags': u'أداة:كان و أخواتها:معطوف',
    'vocalized': u'فَتَحَوَّلَ',
    'stem': u'تَحَوَّلَ',
    'type': u'STOPWORD',
    'original': u'تَحَوَّلَ',
    'encletic': u''
}
STOPWORDS[u'ولتحول'] = {
    'word': u'ولتحول',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'وَلَتَحَوَّلَ',
    'stem': u'تَحَوَّلَ',
    'type': u'STOPWORD',
    'original': u'تَحَوَّلَ',
    'encletic': u''
}
STOPWORDS[u'فلتحول'] = {
    'word': u'فلتحول',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'فَلَتَحَوَّلَ',
    'stem': u'تَحَوَّلَ',
    'type': u'STOPWORD',
    'original': u'تَحَوَّلَ',
    'encletic': u''
}
STOPWORDS[u'حار'] = {
    'word': u'حَارَ',
    'procletic': u'',
    'tags': u'أداة:كان و أخواتها',
    'vocalized': u'حَارَ',
    'stem': u'حَارَ',
    'type': u'STOPWORD',
    'original': u'حَارَ',
    'encletic': u''
}
STOPWORDS[u'لحار'] = {
    'word': u'لحار',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور',
    'vocalized': u'لَحَارَ',
    'stem': u'حَارَ',
    'type': u'STOPWORD',
    'original': u'حَارَ',
    'encletic': u''
}
STOPWORDS[u'وحار'] = {
    'word': u'وحار',
    'procletic': u'-وَ',
    'tags': u'أداة:كان و أخواتها:معطوف',
    'vocalized': u'وَحَارَ',
    'stem': u'حَارَ',
    'type': u'STOPWORD',
    'original': u'حَارَ',
    'encletic': u''
}
STOPWORDS[u'فحار'] = {
    'word': u'فحار',
    'procletic': u'-فَ',
    'tags': u'أداة:كان و أخواتها:معطوف',
    'vocalized': u'فَحَارَ',
    'stem': u'حَارَ',
    'type': u'STOPWORD',
    'original': u'حَارَ',
    'encletic': u''
}
STOPWORDS[u'ولحار'] = {
    'word': u'ولحار',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'وَلَحَارَ',
    'stem': u'حَارَ',
    'type': u'STOPWORD',
    'original': u'حَارَ',
    'encletic': u''
}
STOPWORDS[u'فلحار'] = {
    'word': u'فلحار',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'فَلَحَارَ',
    'stem': u'حَارَ',
    'type': u'STOPWORD',
    'original': u'حَارَ',
    'encletic': u''
}
STOPWORDS[u'راح'] = {
    'word': u'رَاحَ',
    'procletic': u'',
    'tags': u'أداة:كان و أخواتها',
    'vocalized': u'رَاحَ',
    'stem': u'رَاحَ',
    'type': u'STOPWORD',
    'original': u'رَاحَ',
    'encletic': u''
}
STOPWORDS[u'لراح'] = {
    'word': u'لراح',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور',
    'vocalized': u'لَرَاحَ',
    'stem': u'رَاحَ',
    'type': u'STOPWORD',
    'original': u'رَاحَ',
    'encletic': u''
}
STOPWORDS[u'وراح'] = {
    'word': u'وراح',
    'procletic': u'-وَ',
    'tags': u'أداة:كان و أخواتها:معطوف',
    'vocalized': u'وَرَاحَ',
    'stem': u'رَاحَ',
    'type': u'STOPWORD',
    'original': u'رَاحَ',
    'encletic': u''
}
STOPWORDS[u'فراح'] = {
    'word': u'فراح',
    'procletic': u'-فَ',
    'tags': u'أداة:كان و أخواتها:معطوف',
    'vocalized': u'فَرَاحَ',
    'stem': u'رَاحَ',
    'type': u'STOPWORD',
    'original': u'رَاحَ',
    'encletic': u''
}
STOPWORDS[u'ولراح'] = {
    'word': u'ولراح',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'وَلَرَاحَ',
    'stem': u'رَاحَ',
    'type': u'STOPWORD',
    'original': u'رَاحَ',
    'encletic': u''
}
STOPWORDS[u'فلراح'] = {
    'word': u'فلراح',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'فَلَرَاحَ',
    'stem': u'رَاحَ',
    'type': u'STOPWORD',
    'original': u'رَاحَ',
    'encletic': u''
}
STOPWORDS[u'رجع'] = {
    'word': u'رَجَعَ',
    'procletic': u'',
    'tags': u'أداة:كان و أخواتها',
    'vocalized': u'رَجَعَ',
    'stem': u'رَجَعَ',
    'type': u'STOPWORD',
    'original': u'رَجَعَ',
    'encletic': u''
}
STOPWORDS[u'لرجع'] = {
    'word': u'لرجع',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور',
    'vocalized': u'لَرَجَعَ',
    'stem': u'رَجَعَ',
    'type': u'STOPWORD',
    'original': u'رَجَعَ',
    'encletic': u''
}
STOPWORDS[u'ورجع'] = {
    'word': u'ورجع',
    'procletic': u'-وَ',
    'tags': u'أداة:كان و أخواتها:معطوف',
    'vocalized': u'وَرَجَعَ',
    'stem': u'رَجَعَ',
    'type': u'STOPWORD',
    'original': u'رَجَعَ',
    'encletic': u''
}
STOPWORDS[u'فرجع'] = {
    'word': u'فرجع',
    'procletic': u'-فَ',
    'tags': u'أداة:كان و أخواتها:معطوف',
    'vocalized': u'فَرَجَعَ',
    'stem': u'رَجَعَ',
    'type': u'STOPWORD',
    'original': u'رَجَعَ',
    'encletic': u''
}
STOPWORDS[u'ولرجع'] = {
    'word': u'ولرجع',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'وَلَرَجَعَ',
    'stem': u'رَجَعَ',
    'type': u'STOPWORD',
    'original': u'رَجَعَ',
    'encletic': u''
}
STOPWORDS[u'فلرجع'] = {
    'word': u'فلرجع',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'فَلَرَجَعَ',
    'stem': u'رَجَعَ',
    'type': u'STOPWORD',
    'original': u'رَجَعَ',
    'encletic': u''
}
STOPWORDS[u'صار'] = {
    'word': u'صَارَ',
    'procletic': u'',
    'tags': u'أداة:كان و أخواتها',
    'vocalized': u'صَارَ',
    'stem': u'صَارَ',
    'type': u'STOPWORD',
    'original': u'صَارَ',
    'encletic': u''
}
STOPWORDS[u'لصار'] = {
    'word': u'لصار',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور',
    'vocalized': u'لَصَارَ',
    'stem': u'صَارَ',
    'type': u'STOPWORD',
    'original': u'صَارَ',
    'encletic': u''
}
STOPWORDS[u'وصار'] = {
    'word': u'وصار',
    'procletic': u'-وَ',
    'tags': u'أداة:كان و أخواتها:معطوف',
    'vocalized': u'وَصَارَ',
    'stem': u'صَارَ',
    'type': u'STOPWORD',
    'original': u'صَارَ',
    'encletic': u''
}
STOPWORDS[u'فصار'] = {
    'word': u'فصار',
    'procletic': u'-فَ',
    'tags': u'أداة:كان و أخواتها:معطوف',
    'vocalized': u'فَصَارَ',
    'stem': u'صَارَ',
    'type': u'STOPWORD',
    'original': u'صَارَ',
    'encletic': u''
}
STOPWORDS[u'ولصار'] = {
    'word': u'ولصار',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'وَلَصَارَ',
    'stem': u'صَارَ',
    'type': u'STOPWORD',
    'original': u'صَارَ',
    'encletic': u''
}
STOPWORDS[u'فلصار'] = {
    'word': u'فلصار',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'فَلَصَارَ',
    'stem': u'صَارَ',
    'type': u'STOPWORD',
    'original': u'صَارَ',
    'encletic': u''
}
STOPWORDS[u'ظل'] = {
    'word': u'ظَلَّ',
    'procletic': u'',
    'tags': u'أداة:كان و أخواتها',
    'vocalized': u'ظَلَّ',
    'stem': u'ظَلَّ',
    'type': u'STOPWORD',
    'original': u'ظَلَّ',
    'encletic': u''
}
STOPWORDS[u'لظل'] = {
    'word': u'لظل',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور',
    'vocalized': u'لَظَلَّ',
    'stem': u'ظَلَّ',
    'type': u'STOPWORD',
    'original': u'ظَلَّ',
    'encletic': u''
}
STOPWORDS[u'وظل'] = {
    'word': u'وظل',
    'procletic': u'-وَ',
    'tags': u'أداة:كان و أخواتها:معطوف',
    'vocalized': u'وَظَلَّ',
    'stem': u'ظَلَّ',
    'type': u'STOPWORD',
    'original': u'ظَلَّ',
    'encletic': u''
}
STOPWORDS[u'فظل'] = {
    'word': u'فظل',
    'procletic': u'-فَ',
    'tags': u'أداة:كان و أخواتها:معطوف',
    'vocalized': u'فَظَلَّ',
    'stem': u'ظَلَّ',
    'type': u'STOPWORD',
    'original': u'ظَلَّ',
    'encletic': u''
}
STOPWORDS[u'ولظل'] = {
    'word': u'ولظل',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'وَلَظَلَّ',
    'stem': u'ظَلَّ',
    'type': u'STOPWORD',
    'original': u'ظَلَّ',
    'encletic': u''
}
STOPWORDS[u'فلظل'] = {
    'word': u'فلظل',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'فَلَظَلَّ',
    'stem': u'ظَلَّ',
    'type': u'STOPWORD',
    'original': u'ظَلَّ',
    'encletic': u''
}
STOPWORDS[u'عاد'] = {
    'word': u'عَادَ',
    'procletic': u'',
    'tags': u'أداة:كان و أخواتها',
    'vocalized': u'عَادَ',
    'stem': u'عَادَ',
    'type': u'STOPWORD',
    'original': u'عَادَ',
    'encletic': u''
}
STOPWORDS[u'لعاد'] = {
    'word': u'لعاد',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور',
    'vocalized': u'لَعَادَ',
    'stem': u'عَادَ',
    'type': u'STOPWORD',
    'original': u'عَادَ',
    'encletic': u''
}
STOPWORDS[u'وعاد'] = {
    'word': u'وعاد',
    'procletic': u'-وَ',
    'tags': u'أداة:كان و أخواتها:معطوف',
    'vocalized': u'وَعَادَ',
    'stem': u'عَادَ',
    'type': u'STOPWORD',
    'original': u'عَادَ',
    'encletic': u''
}
STOPWORDS[u'فعاد'] = {
    'word': u'فعاد',
    'procletic': u'-فَ',
    'tags': u'أداة:كان و أخواتها:معطوف',
    'vocalized': u'فَعَادَ',
    'stem': u'عَادَ',
    'type': u'STOPWORD',
    'original': u'عَادَ',
    'encletic': u''
}
STOPWORDS[u'ولعاد'] = {
    'word': u'ولعاد',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'وَلَعَادَ',
    'stem': u'عَادَ',
    'type': u'STOPWORD',
    'original': u'عَادَ',
    'encletic': u''
}
STOPWORDS[u'فلعاد'] = {
    'word': u'فلعاد',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'فَلَعَادَ',
    'stem': u'عَادَ',
    'type': u'STOPWORD',
    'original': u'عَادَ',
    'encletic': u''
}
STOPWORDS[u'غدا'] = {
    'word': u'غَدَا',
    'procletic': u'',
    'tags': u'أداة:كان و أخواتها',
    'vocalized': u'غَدَا',
    'stem': u'غَدَا',
    'type': u'STOPWORD',
    'original': u'غَدَا',
    'encletic': u''
}
STOPWORDS[u'لغدا'] = {
    'word': u'لغدا',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور',
    'vocalized': u'لَغَدَا',
    'stem': u'غَدَا',
    'type': u'STOPWORD',
    'original': u'غَدَا',
    'encletic': u''
}
STOPWORDS[u'وغدا'] = {
    'word': u'وغدا',
    'procletic': u'-وَ',
    'tags': u'أداة:كان و أخواتها:معطوف',
    'vocalized': u'وَغَدَا',
    'stem': u'غَدَا',
    'type': u'STOPWORD',
    'original': u'غَدَا',
    'encletic': u''
}
STOPWORDS[u'فغدا'] = {
    'word': u'فغدا',
    'procletic': u'-فَ',
    'tags': u'أداة:كان و أخواتها:معطوف',
    'vocalized': u'فَغَدَا',
    'stem': u'غَدَا',
    'type': u'STOPWORD',
    'original': u'غَدَا',
    'encletic': u''
}
STOPWORDS[u'ولغدا'] = {
    'word': u'ولغدا',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'وَلَغَدَا',
    'stem': u'غَدَا',
    'type': u'STOPWORD',
    'original': u'غَدَا',
    'encletic': u''
}
STOPWORDS[u'فلغدا'] = {
    'word': u'فلغدا',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'فَلَغَدَا',
    'stem': u'غَدَا',
    'type': u'STOPWORD',
    'original': u'غَدَا',
    'encletic': u''
}
STOPWORDS[u'كان'] = {
    'word': u'كَانَ',
    'procletic': u'',
    'tags': u'أداة:كان و أخواتها',
    'vocalized': u'كَانَ',
    'stem': u'كَانَ',
    'type': u'STOPWORD',
    'original': u'كَانَ',
    'encletic': u''
}
STOPWORDS[u'لكان'] = {
    'word': u'لكان',
    'procletic': u'لَ',
    'tags': u'أداة:مجرور',
    'vocalized': u'لَكَانَ',
    'stem': u'كَانَ',
    'type': u'STOPWORD',
    'original': u'كَانَ',
    'encletic': u''
}
STOPWORDS[u'وكان'] = {
    'word': u'وكان',
    'procletic': u'-وَ',
    'tags': u'أداة:كان و أخواتها:معطوف',
    'vocalized': u'وَكَانَ',
    'stem': u'كَانَ',
    'type': u'STOPWORD',
    'original': u'كَانَ',
    'encletic': u''
}
STOPWORDS[u'فكان'] = {
    'word': u'فكان',
    'procletic': u'-فَ',
    'tags': u'أداة:كان و أخواتها:معطوف',
    'vocalized': u'فَكَانَ',
    'stem': u'كَانَ',
    'type': u'STOPWORD',
    'original': u'كَانَ',
    'encletic': u''
}
STOPWORDS[u'ولكان'] = {
    'word': u'ولكان',
    'procletic': u'لَ-وَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'وَلَكَانَ',
    'stem': u'كَانَ',
    'type': u'STOPWORD',
    'original': u'كَانَ',
    'encletic': u''
}
STOPWORDS[u'فلكان'] = {
    'word': u'فلكان',
    'procletic': u'لَ-فَ',
    'tags': u'أداة:مجرور:معطوف',
    'vocalized': u'فَلَكَانَ',
    'stem': u'كَانَ',
    'type': u'STOPWORD',
    'original': u'كَانَ',
    'encletic': u''
}
STOPWORDS[u'ماانفك'] = {
    'word': u'مَااِنْفَكَّ',
    'procletic': u'',
    'tags': u'أداة:كان و أخواتها',
    'vocalized': u'مَااِنْفَكَّ',
    'stem': u'مَااِنْفَكَّ',
    'type': u'STOPWORD',
    'original': u'مَااِنْفَكَّ',
    'encletic': u''
}
STOPWORDS[u'وماانفك'] = {
    'word': u'وماانفك',
    'procletic': u'-وَ',
    'tags': u'أداة:كان و أخواتها:معطوف',
    'vocalized': u'وَمَااِنْفَكَّ',
    'stem': u'مَااِنْفَكَّ',
    'type': u'STOPWORD',
    'original': u'مَااِنْفَكَّ',
    'encletic': u''
}
STOPWORDS[u'فماانفك'] = {
    'word': u'فماانفك',
    'procletic': u'-فَ',
    'tags': u'أداة:كان و أخواتها:معطوف',
    'vocalized': u'فَمَااِنْفَكَّ',
    'stem': u'مَااِنْفَكَّ',
    'type': u'STOPWORD',
    'original': u'مَااِنْفَكَّ',
    'encletic': u''
}
STOPWORDS[u'مابرح'] = {
    'word': u'مَابَرِحَ',
    'procletic': u'',
    'tags': u'أداة:كان و أخواتها',
    'vocalized': u'مَابَرِحَ',
    'stem': u'مَابَرِحَ',
    'type': u'STOPWORD',
    'original': u'مَابَرِحَ',
    'encletic': u''
}
STOPWORDS[u'ومابرح'] = {
    'word': u'ومابرح',
    'procletic': u'-وَ',
    'tags': u'أداة:كان و أخواتها:معطوف',
    'vocalized': u'وَمَابَرِحَ',
    'stem': u'مَابَرِحَ',
    'type': u'STOPWORD',
    'original': u'مَابَرِحَ',
    'encletic': u''
}
STOPWORDS[u'فمابرح'] = {
    'word': u'فمابرح',
    'procletic': u'-فَ',
    'tags': u'أداة:كان و أخواتها:معطوف',
    'vocalized': u'فَمَابَرِحَ',
    'stem': u'مَابَرِحَ',
    'type': u'STOPWORD',
    'original': u'مَابَرِحَ',
    'encletic': u''
}
STOPWORDS[u'مادام'] = {
    'word': u'مَادَامَ',
    'procletic': u'',
    'tags': u'أداة:كان و أخواتها',
    'vocalized': u'مَادَامَ',
    'stem': u'مَادَامَ',
    'type': u'STOPWORD',
    'original': u'مَادَامَ',
    'encletic': u''
}
STOPWORDS[u'ومادام'] = {
    'word': u'ومادام',
    'procletic': u'-وَ',
    'tags': u'أداة:كان و أخواتها:معطوف',
    'vocalized': u'وَمَادَامَ',
    'stem': u'مَادَامَ',
    'type': u'STOPWORD',
    'original': u'مَادَامَ',
    'encletic': u''
}
STOPWORDS[u'فمادام'] = {
    'word': u'فمادام',
    'procletic': u'-فَ',
    'tags': u'أداة:كان و أخواتها:معطوف',
    'vocalized': u'فَمَادَامَ',
    'stem': u'مَادَامَ',
    'type': u'STOPWORD',
    'original': u'مَادَامَ',
    'encletic': u''
}
STOPWORDS[u'مازال'] = {
    'word': u'مَازَالَ',
    'procletic': u'',
    'tags': u'أداة:كان و أخواتها',
    'vocalized': u'مَازَالَ',
    'stem': u'مَازَالَ',
    'type': u'STOPWORD',
    'original': u'مَازَالَ',
    'encletic': u''
}
STOPWORDS[u'ومازال'] = {
    'word': u'ومازال',
    'procletic': u'-وَ',
    'tags': u'أداة:كان و أخواتها:معطوف',
    'vocalized': u'وَمَازَالَ',
    'stem': u'مَازَالَ',
    'type': u'STOPWORD',
    'original': u'مَازَالَ',
    'encletic': u''
}
STOPWORDS[u'فمازال'] = {
    'word': u'فمازال',
    'procletic': u'-فَ',
    'tags': u'أداة:كان و أخواتها:معطوف',
    'vocalized': u'فَمَازَالَ',
    'stem': u'مَازَالَ',
    'type': u'STOPWORD',
    'original': u'مَازَالَ',
    'encletic': u''
}
STOPWORDS[u'مافتئ'] = {
    'word': u'مَافَتِئَ',
    'procletic': u'',
    'tags': u'أداة:كان و أخواتها',
    'vocalized': u'مَافَتِئَ',
    'stem': u'مَافَتِئَ',
    'type': u'STOPWORD',
    'original': u'مَافَتِئَ',
    'encletic': u''
}
STOPWORDS[u'ومافتئ'] = {
    'word': u'ومافتئ',
    'procletic': u'-وَ',
    'tags': u'أداة:كان و أخواتها:معطوف',
    'vocalized': u'وَمَافَتِئَ',
    'stem': u'مَافَتِئَ',
    'type': u'STOPWORD',
    'original': u'مَافَتِئَ',
    'encletic': u''
}
STOPWORDS[u'فمافتئ'] = {
    'word': u'فمافتئ',
    'procletic': u'-فَ',
    'tags': u'أداة:كان و أخواتها:معطوف',
    'vocalized': u'فَمَافَتِئَ',
    'stem': u'مَافَتِئَ',
    'type': u'STOPWORD',
    'original': u'مَافَتِئَ',
    'encletic': u''
}
STOPWORDS[u'ليس'] = {
    'word': u'لَيْسَ',
    'procletic': u'',
    'tags': u'أداة:كان و أخواتها',
    'vocalized': u'لَيْسَ',
    'stem': u'لَيْسَ',
    'type': u'STOPWORD',
    'original': u'لَيْسَ',
    'encletic': u''
}
STOPWORDS[u'وليس'] = {
    'word': u'وليس',
    'procletic': u'-وَ',
    'tags': u'أداة:كان و أخواتها:معطوف',
    'vocalized': u'وَلَيْسَ',
    'stem': u'لَيْسَ',
    'type': u'STOPWORD',
    'original': u'لَيْسَ',
    'encletic': u''
}
STOPWORDS[u'فليس'] = {
    'word': u'فليس',
    'procletic': u'-فَ',
    'tags': u'أداة:كان و أخواتها:معطوف',
    'vocalized': u'فَلَيْسَ',
    'stem': u'لَيْسَ',
    'type': u'STOPWORD',
    'original': u'لَيْسَ',
    'encletic': u''
}
STOPWORDS[u'أليس'] = {
    'word': u'أليس',
    'procletic': u'-أَ',
    'tags': u'أداة:كان و أخواتها:مجرور',
    'vocalized': u'أَلَيْسَ',
    'stem': u'لَيْسَ',
    'type': u'STOPWORD',
    'original': u'لَيْسَ',
    'encletic': u''
}
STOPWORDS[u'أوليس'] = {
    'word': u'أوليس',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:كان و أخواتها:معطوف:مجرور',
    'vocalized': u'أَوَلَيْسَ',
    'stem': u'لَيْسَ',
    'type': u'STOPWORD',
    'original': u'لَيْسَ',
    'encletic': u''
}
STOPWORDS[u'أفليس'] = {
    'word': u'أفليس',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:كان و أخواتها:معطوف:مجرور',
    'vocalized': u'أَفَلَيْسَ',
    'stem': u'لَيْسَ',
    'type': u'STOPWORD',
    'original': u'لَيْسَ',
    'encletic': u''
}
STOPWORDS[u'لست'] = {
    'word': u'لَسْتُ',
    'procletic': u'',
    'tags': u'أداة:كان و أخواتها',
    'vocalized': u'لَسْتُ',
    'stem': u'لَسْتُ',
    'type': u'STOPWORD',
    'original': u'لَسْتُ',
    'encletic': u''
}
STOPWORDS[u'ولست'] = {
    'word': u'ولست',
    'procletic': u'-وَ',
    'tags': u'أداة:كان و أخواتها:معطوف',
    'vocalized': u'وَلَسْتُ',
    'stem': u'لَسْتُ',
    'type': u'STOPWORD',
    'original': u'لَسْتُ',
    'encletic': u''
}
STOPWORDS[u'فلست'] = {
    'word': u'فلست',
    'procletic': u'-فَ',
    'tags': u'أداة:كان و أخواتها:معطوف',
    'vocalized': u'فَلَسْتُ',
    'stem': u'لَسْتُ',
    'type': u'STOPWORD',
    'original': u'لَسْتُ',
    'encletic': u''
}
STOPWORDS[u'ألست'] = {
    'word': u'ألست',
    'procletic': u'-أَ',
    'tags': u'أداة:كان و أخواتها:مجرور',
    'vocalized': u'أَلَسْتُ',
    'stem': u'لَسْتُ',
    'type': u'STOPWORD',
    'original': u'لَسْتُ',
    'encletic': u''
}
STOPWORDS[u'أولست'] = {
    'word': u'أولست',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:كان و أخواتها:معطوف:مجرور',
    'vocalized': u'أَوَلَسْتُ',
    'stem': u'لَسْتُ',
    'type': u'STOPWORD',
    'original': u'لَسْتُ',
    'encletic': u''
}
STOPWORDS[u'أفلست'] = {
    'word': u'أفلست',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:كان و أخواتها:معطوف:مجرور',
    'vocalized': u'أَفَلَسْتُ',
    'stem': u'لَسْتُ',
    'type': u'STOPWORD',
    'original': u'لَسْتُ',
    'encletic': u''
}
STOPWORDS[u'لسنا'] = {
    'word': u'لَسْنَا',
    'procletic': u'',
    'tags': u'أداة:كان و أخواتها',
    'vocalized': u'لَسْنَا',
    'stem': u'لَسْنَا',
    'type': u'STOPWORD',
    'original': u'لَسْنَا',
    'encletic': u''
}
STOPWORDS[u'ولسنا'] = {
    'word': u'ولسنا',
    'procletic': u'-وَ',
    'tags': u'أداة:كان و أخواتها:معطوف',
    'vocalized': u'وَلَسْنَا',
    'stem': u'لَسْنَا',
    'type': u'STOPWORD',
    'original': u'لَسْنَا',
    'encletic': u''
}
STOPWORDS[u'فلسنا'] = {
    'word': u'فلسنا',
    'procletic': u'-فَ',
    'tags': u'أداة:كان و أخواتها:معطوف',
    'vocalized': u'فَلَسْنَا',
    'stem': u'لَسْنَا',
    'type': u'STOPWORD',
    'original': u'لَسْنَا',
    'encletic': u''
}
STOPWORDS[u'ألسنا'] = {
    'word': u'ألسنا',
    'procletic': u'-أَ',
    'tags': u'أداة:كان و أخواتها:مجرور',
    'vocalized': u'أَلَسْنَا',
    'stem': u'لَسْنَا',
    'type': u'STOPWORD',
    'original': u'لَسْنَا',
    'encletic': u''
}
STOPWORDS[u'أولسنا'] = {
    'word': u'أولسنا',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:كان و أخواتها:معطوف:مجرور',
    'vocalized': u'أَوَلَسْنَا',
    'stem': u'لَسْنَا',
    'type': u'STOPWORD',
    'original': u'لَسْنَا',
    'encletic': u''
}
STOPWORDS[u'أفلسنا'] = {
    'word': u'أفلسنا',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:كان و أخواتها:معطوف:مجرور',
    'vocalized': u'أَفَلَسْنَا',
    'stem': u'لَسْنَا',
    'type': u'STOPWORD',
    'original': u'لَسْنَا',
    'encletic': u''
}
STOPWORDS[u'لست'] = {
    'word': u'لَسْتَ',
    'procletic': u'',
    'tags': u'أداة:كان و أخواتها',
    'vocalized': u'لَسْتَ',
    'stem': u'لَسْتَ',
    'type': u'STOPWORD',
    'original': u'لَسْتَ',
    'encletic': u''
}
STOPWORDS[u'ولست'] = {
    'word': u'ولست',
    'procletic': u'-وَ',
    'tags': u'أداة:كان و أخواتها:معطوف',
    'vocalized': u'وَلَسْتَ',
    'stem': u'لَسْتَ',
    'type': u'STOPWORD',
    'original': u'لَسْتَ',
    'encletic': u''
}
STOPWORDS[u'فلست'] = {
    'word': u'فلست',
    'procletic': u'-فَ',
    'tags': u'أداة:كان و أخواتها:معطوف',
    'vocalized': u'فَلَسْتَ',
    'stem': u'لَسْتَ',
    'type': u'STOPWORD',
    'original': u'لَسْتَ',
    'encletic': u''
}
STOPWORDS[u'ألست'] = {
    'word': u'ألست',
    'procletic': u'-أَ',
    'tags': u'أداة:كان و أخواتها:مجرور',
    'vocalized': u'أَلَسْتَ',
    'stem': u'لَسْتَ',
    'type': u'STOPWORD',
    'original': u'لَسْتَ',
    'encletic': u''
}
STOPWORDS[u'أولست'] = {
    'word': u'أولست',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:كان و أخواتها:معطوف:مجرور',
    'vocalized': u'أَوَلَسْتَ',
    'stem': u'لَسْتَ',
    'type': u'STOPWORD',
    'original': u'لَسْتَ',
    'encletic': u''
}
STOPWORDS[u'أفلست'] = {
    'word': u'أفلست',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:كان و أخواتها:معطوف:مجرور',
    'vocalized': u'أَفَلَسْتَ',
    'stem': u'لَسْتَ',
    'type': u'STOPWORD',
    'original': u'لَسْتَ',
    'encletic': u''
}
STOPWORDS[u'لست'] = {
    'word': u'لَسْتِ',
    'procletic': u'',
    'tags': u'أداة:كان و أخواتها',
    'vocalized': u'لَسْتِ',
    'stem': u'لَسْتِ',
    'type': u'STOPWORD',
    'original': u'لَسْتِ',
    'encletic': u''
}
STOPWORDS[u'ولست'] = {
    'word': u'ولست',
    'procletic': u'-وَ',
    'tags': u'أداة:كان و أخواتها:معطوف',
    'vocalized': u'وَلَسْتِ',
    'stem': u'لَسْتِ',
    'type': u'STOPWORD',
    'original': u'لَسْتِ',
    'encletic': u''
}
STOPWORDS[u'فلست'] = {
    'word': u'فلست',
    'procletic': u'-فَ',
    'tags': u'أداة:كان و أخواتها:معطوف',
    'vocalized': u'فَلَسْتِ',
    'stem': u'لَسْتِ',
    'type': u'STOPWORD',
    'original': u'لَسْتِ',
    'encletic': u''
}
STOPWORDS[u'ألست'] = {
    'word': u'ألست',
    'procletic': u'-أَ',
    'tags': u'أداة:كان و أخواتها:مجرور',
    'vocalized': u'أَلَسْتِ',
    'stem': u'لَسْتِ',
    'type': u'STOPWORD',
    'original': u'لَسْتِ',
    'encletic': u''
}
STOPWORDS[u'أولست'] = {
    'word': u'أولست',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:كان و أخواتها:معطوف:مجرور',
    'vocalized': u'أَوَلَسْتِ',
    'stem': u'لَسْتِ',
    'type': u'STOPWORD',
    'original': u'لَسْتِ',
    'encletic': u''
}
STOPWORDS[u'أفلست'] = {
    'word': u'أفلست',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:كان و أخواتها:معطوف:مجرور',
    'vocalized': u'أَفَلَسْتِ',
    'stem': u'لَسْتِ',
    'type': u'STOPWORD',
    'original': u'لَسْتِ',
    'encletic': u''
}
STOPWORDS[u'لستما'] = {
    'word': u'لَسْتُمَا',
    'procletic': u'',
    'tags': u'أداة:كان و أخواتها',
    'vocalized': u'لَسْتُمَا',
    'stem': u'لَسْتُمَا',
    'type': u'STOPWORD',
    'original': u'لَسْتُمَا',
    'encletic': u''
}
STOPWORDS[u'ولستما'] = {
    'word': u'ولستما',
    'procletic': u'-وَ',
    'tags': u'أداة:كان و أخواتها:معطوف',
    'vocalized': u'وَلَسْتُمَا',
    'stem': u'لَسْتُمَا',
    'type': u'STOPWORD',
    'original': u'لَسْتُمَا',
    'encletic': u''
}
STOPWORDS[u'فلستما'] = {
    'word': u'فلستما',
    'procletic': u'-فَ',
    'tags': u'أداة:كان و أخواتها:معطوف',
    'vocalized': u'فَلَسْتُمَا',
    'stem': u'لَسْتُمَا',
    'type': u'STOPWORD',
    'original': u'لَسْتُمَا',
    'encletic': u''
}
STOPWORDS[u'ألستما'] = {
    'word': u'ألستما',
    'procletic': u'-أَ',
    'tags': u'أداة:كان و أخواتها:مجرور',
    'vocalized': u'أَلَسْتُمَا',
    'stem': u'لَسْتُمَا',
    'type': u'STOPWORD',
    'original': u'لَسْتُمَا',
    'encletic': u''
}
STOPWORDS[u'أولستما'] = {
    'word': u'أولستما',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:كان و أخواتها:معطوف:مجرور',
    'vocalized': u'أَوَلَسْتُمَا',
    'stem': u'لَسْتُمَا',
    'type': u'STOPWORD',
    'original': u'لَسْتُمَا',
    'encletic': u''
}
STOPWORDS[u'أفلستما'] = {
    'word': u'أفلستما',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:كان و أخواتها:معطوف:مجرور',
    'vocalized': u'أَفَلَسْتُمَا',
    'stem': u'لَسْتُمَا',
    'type': u'STOPWORD',
    'original': u'لَسْتُمَا',
    'encletic': u''
}
STOPWORDS[u'لستم'] = {
    'word': u'لَسْتُم',
    'procletic': u'',
    'tags': u'أداة:كان و أخواتها',
    'vocalized': u'لَسْتُم',
    'stem': u'لَسْتُم',
    'type': u'STOPWORD',
    'original': u'لَسْتُم',
    'encletic': u''
}
STOPWORDS[u'ولستم'] = {
    'word': u'ولستم',
    'procletic': u'-وَ',
    'tags': u'أداة:كان و أخواتها:معطوف',
    'vocalized': u'وَلَسْتُم',
    'stem': u'لَسْتُم',
    'type': u'STOPWORD',
    'original': u'لَسْتُم',
    'encletic': u''
}
STOPWORDS[u'فلستم'] = {
    'word': u'فلستم',
    'procletic': u'-فَ',
    'tags': u'أداة:كان و أخواتها:معطوف',
    'vocalized': u'فَلَسْتُم',
    'stem': u'لَسْتُم',
    'type': u'STOPWORD',
    'original': u'لَسْتُم',
    'encletic': u''
}
STOPWORDS[u'ألستم'] = {
    'word': u'ألستم',
    'procletic': u'-أَ',
    'tags': u'أداة:كان و أخواتها:مجرور',
    'vocalized': u'أَلَسْتُم',
    'stem': u'لَسْتُم',
    'type': u'STOPWORD',
    'original': u'لَسْتُم',
    'encletic': u''
}
STOPWORDS[u'أولستم'] = {
    'word': u'أولستم',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:كان و أخواتها:معطوف:مجرور',
    'vocalized': u'أَوَلَسْتُم',
    'stem': u'لَسْتُم',
    'type': u'STOPWORD',
    'original': u'لَسْتُم',
    'encletic': u''
}
STOPWORDS[u'أفلستم'] = {
    'word': u'أفلستم',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:كان و أخواتها:معطوف:مجرور',
    'vocalized': u'أَفَلَسْتُم',
    'stem': u'لَسْتُم',
    'type': u'STOPWORD',
    'original': u'لَسْتُم',
    'encletic': u''
}
STOPWORDS[u'لستن'] = {
    'word': u'لَسْتُنَّ',
    'procletic': u'',
    'tags': u'أداة:كان و أخواتها',
    'vocalized': u'لَسْتُنَّ',
    'stem': u'لَسْتُنَّ',
    'type': u'STOPWORD',
    'original': u'لَسْتُنَّ',
    'encletic': u''
}
STOPWORDS[u'ولستن'] = {
    'word': u'ولستن',
    'procletic': u'-وَ',
    'tags': u'أداة:كان و أخواتها:معطوف',
    'vocalized': u'وَلَسْتُنَّ',
    'stem': u'لَسْتُنَّ',
    'type': u'STOPWORD',
    'original': u'لَسْتُنَّ',
    'encletic': u''
}
STOPWORDS[u'فلستن'] = {
    'word': u'فلستن',
    'procletic': u'-فَ',
    'tags': u'أداة:كان و أخواتها:معطوف',
    'vocalized': u'فَلَسْتُنَّ',
    'stem': u'لَسْتُنَّ',
    'type': u'STOPWORD',
    'original': u'لَسْتُنَّ',
    'encletic': u''
}
STOPWORDS[u'ألستن'] = {
    'word': u'ألستن',
    'procletic': u'-أَ',
    'tags': u'أداة:كان و أخواتها:مجرور',
    'vocalized': u'أَلَسْتُنَّ',
    'stem': u'لَسْتُنَّ',
    'type': u'STOPWORD',
    'original': u'لَسْتُنَّ',
    'encletic': u''
}
STOPWORDS[u'أولستن'] = {
    'word': u'أولستن',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:كان و أخواتها:معطوف:مجرور',
    'vocalized': u'أَوَلَسْتُنَّ',
    'stem': u'لَسْتُنَّ',
    'type': u'STOPWORD',
    'original': u'لَسْتُنَّ',
    'encletic': u''
}
STOPWORDS[u'أفلستن'] = {
    'word': u'أفلستن',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:كان و أخواتها:معطوف:مجرور',
    'vocalized': u'أَفَلَسْتُنَّ',
    'stem': u'لَسْتُنَّ',
    'type': u'STOPWORD',
    'original': u'لَسْتُنَّ',
    'encletic': u''
}
STOPWORDS[u'ليست'] = {
    'word': u'لَيْسَتْ',
    'procletic': u'',
    'tags': u'أداة:كان و أخواتها',
    'vocalized': u'لَيْسَتْ',
    'stem': u'لَيْسَتْ',
    'type': u'STOPWORD',
    'original': u'لَيْسَتْ',
    'encletic': u''
}
STOPWORDS[u'وليست'] = {
    'word': u'وليست',
    'procletic': u'-وَ',
    'tags': u'أداة:كان و أخواتها:معطوف',
    'vocalized': u'وَلَيْسَتْ',
    'stem': u'لَيْسَتْ',
    'type': u'STOPWORD',
    'original': u'لَيْسَتْ',
    'encletic': u''
}
STOPWORDS[u'فليست'] = {
    'word': u'فليست',
    'procletic': u'-فَ',
    'tags': u'أداة:كان و أخواتها:معطوف',
    'vocalized': u'فَلَيْسَتْ',
    'stem': u'لَيْسَتْ',
    'type': u'STOPWORD',
    'original': u'لَيْسَتْ',
    'encletic': u''
}
STOPWORDS[u'أليست'] = {
    'word': u'أليست',
    'procletic': u'-أَ',
    'tags': u'أداة:كان و أخواتها:مجرور',
    'vocalized': u'أَلَيْسَتْ',
    'stem': u'لَيْسَتْ',
    'type': u'STOPWORD',
    'original': u'لَيْسَتْ',
    'encletic': u''
}
STOPWORDS[u'أوليست'] = {
    'word': u'أوليست',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:كان و أخواتها:معطوف:مجرور',
    'vocalized': u'أَوَلَيْسَتْ',
    'stem': u'لَيْسَتْ',
    'type': u'STOPWORD',
    'original': u'لَيْسَتْ',
    'encletic': u''
}
STOPWORDS[u'أفليست'] = {
    'word': u'أفليست',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:كان و أخواتها:معطوف:مجرور',
    'vocalized': u'أَفَلَيْسَتْ',
    'stem': u'لَيْسَتْ',
    'type': u'STOPWORD',
    'original': u'لَيْسَتْ',
    'encletic': u''
}
STOPWORDS[u'ليسا'] = {
    'word': u'لَيْسَا',
    'procletic': u'',
    'tags': u'أداة:كان و أخواتها',
    'vocalized': u'لَيْسَا',
    'stem': u'لَيْسَا',
    'type': u'STOPWORD',
    'original': u'لَيْسَا',
    'encletic': u''
}
STOPWORDS[u'وليسا'] = {
    'word': u'وليسا',
    'procletic': u'-وَ',
    'tags': u'أداة:كان و أخواتها:معطوف',
    'vocalized': u'وَلَيْسَا',
    'stem': u'لَيْسَا',
    'type': u'STOPWORD',
    'original': u'لَيْسَا',
    'encletic': u''
}
STOPWORDS[u'فليسا'] = {
    'word': u'فليسا',
    'procletic': u'-فَ',
    'tags': u'أداة:كان و أخواتها:معطوف',
    'vocalized': u'فَلَيْسَا',
    'stem': u'لَيْسَا',
    'type': u'STOPWORD',
    'original': u'لَيْسَا',
    'encletic': u''
}
STOPWORDS[u'أليسا'] = {
    'word': u'أليسا',
    'procletic': u'-أَ',
    'tags': u'أداة:كان و أخواتها:مجرور',
    'vocalized': u'أَلَيْسَا',
    'stem': u'لَيْسَا',
    'type': u'STOPWORD',
    'original': u'لَيْسَا',
    'encletic': u''
}
STOPWORDS[u'أوليسا'] = {
    'word': u'أوليسا',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:كان و أخواتها:معطوف:مجرور',
    'vocalized': u'أَوَلَيْسَا',
    'stem': u'لَيْسَا',
    'type': u'STOPWORD',
    'original': u'لَيْسَا',
    'encletic': u''
}
STOPWORDS[u'أفليسا'] = {
    'word': u'أفليسا',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:كان و أخواتها:معطوف:مجرور',
    'vocalized': u'أَفَلَيْسَا',
    'stem': u'لَيْسَا',
    'type': u'STOPWORD',
    'original': u'لَيْسَا',
    'encletic': u''
}
STOPWORDS[u'ليستا'] = {
    'word': u'لَيْسَتَا',
    'procletic': u'',
    'tags': u'أداة:كان و أخواتها',
    'vocalized': u'لَيْسَتَا',
    'stem': u'لَيْسَتَا',
    'type': u'STOPWORD',
    'original': u'لَيْسَتَا',
    'encletic': u''
}
STOPWORDS[u'وليستا'] = {
    'word': u'وليستا',
    'procletic': u'-وَ',
    'tags': u'أداة:كان و أخواتها:معطوف',
    'vocalized': u'وَلَيْسَتَا',
    'stem': u'لَيْسَتَا',
    'type': u'STOPWORD',
    'original': u'لَيْسَتَا',
    'encletic': u''
}
STOPWORDS[u'فليستا'] = {
    'word': u'فليستا',
    'procletic': u'-فَ',
    'tags': u'أداة:كان و أخواتها:معطوف',
    'vocalized': u'فَلَيْسَتَا',
    'stem': u'لَيْسَتَا',
    'type': u'STOPWORD',
    'original': u'لَيْسَتَا',
    'encletic': u''
}
STOPWORDS[u'أليستا'] = {
    'word': u'أليستا',
    'procletic': u'-أَ',
    'tags': u'أداة:كان و أخواتها:مجرور',
    'vocalized': u'أَلَيْسَتَا',
    'stem': u'لَيْسَتَا',
    'type': u'STOPWORD',
    'original': u'لَيْسَتَا',
    'encletic': u''
}
STOPWORDS[u'أوليستا'] = {
    'word': u'أوليستا',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:كان و أخواتها:معطوف:مجرور',
    'vocalized': u'أَوَلَيْسَتَا',
    'stem': u'لَيْسَتَا',
    'type': u'STOPWORD',
    'original': u'لَيْسَتَا',
    'encletic': u''
}
STOPWORDS[u'أفليستا'] = {
    'word': u'أفليستا',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:كان و أخواتها:معطوف:مجرور',
    'vocalized': u'أَفَلَيْسَتَا',
    'stem': u'لَيْسَتَا',
    'type': u'STOPWORD',
    'original': u'لَيْسَتَا',
    'encletic': u''
}
STOPWORDS[u'ليسوا'] = {
    'word': u'لَيْسُوا',
    'procletic': u'',
    'tags': u'أداة:كان و أخواتها',
    'vocalized': u'لَيْسُوا',
    'stem': u'لَيْسُوا',
    'type': u'STOPWORD',
    'original': u'لَيْسُوا',
    'encletic': u''
}
STOPWORDS[u'وليسوا'] = {
    'word': u'وليسوا',
    'procletic': u'-وَ',
    'tags': u'أداة:كان و أخواتها:معطوف',
    'vocalized': u'وَلَيْسُوا',
    'stem': u'لَيْسُوا',
    'type': u'STOPWORD',
    'original': u'لَيْسُوا',
    'encletic': u''
}
STOPWORDS[u'فليسوا'] = {
    'word': u'فليسوا',
    'procletic': u'-فَ',
    'tags': u'أداة:كان و أخواتها:معطوف',
    'vocalized': u'فَلَيْسُوا',
    'stem': u'لَيْسُوا',
    'type': u'STOPWORD',
    'original': u'لَيْسُوا',
    'encletic': u''
}
STOPWORDS[u'أليسوا'] = {
    'word': u'أليسوا',
    'procletic': u'-أَ',
    'tags': u'أداة:كان و أخواتها:مجرور',
    'vocalized': u'أَلَيْسُوا',
    'stem': u'لَيْسُوا',
    'type': u'STOPWORD',
    'original': u'لَيْسُوا',
    'encletic': u''
}
STOPWORDS[u'أوليسوا'] = {
    'word': u'أوليسوا',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:كان و أخواتها:معطوف:مجرور',
    'vocalized': u'أَوَلَيْسُوا',
    'stem': u'لَيْسُوا',
    'type': u'STOPWORD',
    'original': u'لَيْسُوا',
    'encletic': u''
}
STOPWORDS[u'أفليسوا'] = {
    'word': u'أفليسوا',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:كان و أخواتها:معطوف:مجرور',
    'vocalized': u'أَفَلَيْسُوا',
    'stem': u'لَيْسُوا',
    'type': u'STOPWORD',
    'original': u'لَيْسُوا',
    'encletic': u''
}
STOPWORDS[u'لسن'] = {
    'word': u'لَسْنَ',
    'procletic': u'',
    'tags': u'أداة:كان و أخواتها',
    'vocalized': u'لَسْنَ',
    'stem': u'لَسْنَ',
    'type': u'STOPWORD',
    'original': u'لَسْنَ',
    'encletic': u''
}
STOPWORDS[u'ولسن'] = {
    'word': u'ولسن',
    'procletic': u'-وَ',
    'tags': u'أداة:كان و أخواتها:معطوف',
    'vocalized': u'وَلَسْنَ',
    'stem': u'لَسْنَ',
    'type': u'STOPWORD',
    'original': u'لَسْنَ',
    'encletic': u''
}
STOPWORDS[u'فلسن'] = {
    'word': u'فلسن',
    'procletic': u'-فَ',
    'tags': u'أداة:كان و أخواتها:معطوف',
    'vocalized': u'فَلَسْنَ',
    'stem': u'لَسْنَ',
    'type': u'STOPWORD',
    'original': u'لَسْنَ',
    'encletic': u''
}
STOPWORDS[u'ألسن'] = {
    'word': u'ألسن',
    'procletic': u'-أَ',
    'tags': u'أداة:كان و أخواتها:مجرور',
    'vocalized': u'أَلَسْنَ',
    'stem': u'لَسْنَ',
    'type': u'STOPWORD',
    'original': u'لَسْنَ',
    'encletic': u''
}
STOPWORDS[u'أولسن'] = {
    'word': u'أولسن',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:كان و أخواتها:معطوف:مجرور',
    'vocalized': u'أَوَلَسْنَ',
    'stem': u'لَسْنَ',
    'type': u'STOPWORD',
    'original': u'لَسْنَ',
    'encletic': u''
}
STOPWORDS[u'أفلسن'] = {
    'word': u'أفلسن',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:كان و أخواتها:معطوف:مجرور',
    'vocalized': u'أَفَلَسْنَ',
    'stem': u'لَسْنَ',
    'type': u'STOPWORD',
    'original': u'لَسْنَ',
    'encletic': u''
}
STOPWORDS[u'بضع'] = {
    'word': u'بِضْعَ',
    'procletic': u'',
    'tags': u'أداة:كناية',
    'vocalized': u'بِضْعَ',
    'stem': u'بِضْعَ',
    'type': u'STOPWORD',
    'original': u'بِضْعَ',
    'encletic': u''
}
STOPWORDS[u'وبضع'] = {
    'word': u'وبضع',
    'procletic': u'-وَ',
    'tags': u'أداة:كناية:معطوف',
    'vocalized': u'وَبِضْعَ',
    'stem': u'بِضْعَ',
    'type': u'STOPWORD',
    'original': u'بِضْعَ',
    'encletic': u''
}
STOPWORDS[u'فبضع'] = {
    'word': u'فبضع',
    'procletic': u'-فَ',
    'tags': u'أداة:كناية:معطوف',
    'vocalized': u'فَبِضْعَ',
    'stem': u'بِضْعَ',
    'type': u'STOPWORD',
    'original': u'بِضْعَ',
    'encletic': u''
}
STOPWORDS[u'أبضع'] = {
    'word': u'أبضع',
    'procletic': u'-أَ',
    'tags': u'أداة:كناية:مجرور',
    'vocalized': u'أَبِضْعَ',
    'stem': u'بِضْعَ',
    'type': u'STOPWORD',
    'original': u'بِضْعَ',
    'encletic': u''
}
STOPWORDS[u'أوبضع'] = {
    'word': u'أوبضع',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:كناية:معطوف:مجرور',
    'vocalized': u'أَوَبِضْعَ',
    'stem': u'بِضْعَ',
    'type': u'STOPWORD',
    'original': u'بِضْعَ',
    'encletic': u''
}
STOPWORDS[u'أفبضع'] = {
    'word': u'أفبضع',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:كناية:معطوف:مجرور',
    'vocalized': u'أَفَبِضْعَ',
    'stem': u'بِضْعَ',
    'type': u'STOPWORD',
    'original': u'بِضْعَ',
    'encletic': u''
}
STOPWORDS[u'بضعي'] = {
    'word': u'بضعي',
    'procletic': u'',
    'tags': u'أداة:كناية:مضاف',
    'vocalized': u'بِضْعَِي',
    'stem': u'بِضْعَ',
    'type': u'STOPWORD',
    'original': u'بِضْعَ',
    'encletic': u'ِي'
}
STOPWORDS[u'بضعك'] = {
    'word': u'بضعك',
    'procletic': u'',
    'tags': u'أداة:كناية:مضاف',
    'vocalized': u'بِضْعَكَ',
    'stem': u'بِضْعَ',
    'type': u'STOPWORD',
    'original': u'بِضْعَ',
    'encletic': u'كَ'
}
STOPWORDS[u'بضعك'] = {
    'word': u'بضعك',
    'procletic': u'',
    'tags': u'أداة:كناية:مضاف',
    'vocalized': u'بِضْعَكِ',
    'stem': u'بِضْعَ',
    'type': u'STOPWORD',
    'original': u'بِضْعَ',
    'encletic': u'كِ'
}
STOPWORDS[u'بضعه'] = {
    'word': u'بضعه',
    'procletic': u'',
    'tags': u'أداة:كناية:مضاف',
    'vocalized': u'بِضْعَه',
    'stem': u'بِضْعَ',
    'type': u'STOPWORD',
    'original': u'بِضْعَ',
    'encletic': u'ه'
}
STOPWORDS[u'بضعكم'] = {
    'word': u'بضعكم',
    'procletic': u'',
    'tags': u'أداة:كناية:مضاف',
    'vocalized': u'بِضْعَكُمْ',
    'stem': u'بِضْعَ',
    'type': u'STOPWORD',
    'original': u'بِضْعَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'بضعكن'] = {
    'word': u'بضعكن',
    'procletic': u'',
    'tags': u'أداة:كناية:مضاف',
    'vocalized': u'بِضْعَكُنَّ',
    'stem': u'بِضْعَ',
    'type': u'STOPWORD',
    'original': u'بِضْعَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'بضعها'] = {
    'word': u'بضعها',
    'procletic': u'',
    'tags': u'أداة:كناية:مضاف',
    'vocalized': u'بِضْعَهَا',
    'stem': u'بِضْعَ',
    'type': u'STOPWORD',
    'original': u'بِضْعَ',
    'encletic': u'هَا'
}
STOPWORDS[u'بضعهم'] = {
    'word': u'بضعهم',
    'procletic': u'',
    'tags': u'أداة:كناية:مضاف',
    'vocalized': u'بِضْعَهُمْ',
    'stem': u'بِضْعَ',
    'type': u'STOPWORD',
    'original': u'بِضْعَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'بضعهن'] = {
    'word': u'بضعهن',
    'procletic': u'',
    'tags': u'أداة:كناية:مضاف',
    'vocalized': u'بِضْعَهُنَّ',
    'stem': u'بِضْعَ',
    'type': u'STOPWORD',
    'original': u'بِضْعَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'بضعنا'] = {
    'word': u'بضعنا',
    'procletic': u'',
    'tags': u'أداة:كناية:مضاف',
    'vocalized': u'بِضْعَنَا',
    'stem': u'بِضْعَ',
    'type': u'STOPWORD',
    'original': u'بِضْعَ',
    'encletic': u'نَا'
}
STOPWORDS[u'بضعكما'] = {
    'word': u'بضعكما',
    'procletic': u'',
    'tags': u'أداة:كناية:مضاف',
    'vocalized': u'بِضْعَكُمَا',
    'stem': u'بِضْعَ',
    'type': u'STOPWORD',
    'original': u'بِضْعَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'بضعهما'] = {
    'word': u'بضعهما',
    'procletic': u'',
    'tags': u'أداة:كناية:مضاف',
    'vocalized': u'بِضْعَهُمَا',
    'stem': u'بِضْعَ',
    'type': u'STOPWORD',
    'original': u'بِضْعَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'وبضعي'] = {
    'word': u'وبضعي',
    'procletic': u'-وَ',
    'tags': u'أداة:كناية:معطوف:مضاف',
    'vocalized': u'وَبِضْعَِي',
    'stem': u'بِضْعَ',
    'type': u'STOPWORD',
    'original': u'بِضْعَ',
    'encletic': u'ِي'
}
STOPWORDS[u'وبضعك'] = {
    'word': u'وبضعك',
    'procletic': u'-وَ',
    'tags': u'أداة:كناية:معطوف:مضاف',
    'vocalized': u'وَبِضْعَكَ',
    'stem': u'بِضْعَ',
    'type': u'STOPWORD',
    'original': u'بِضْعَ',
    'encletic': u'كَ'
}
STOPWORDS[u'وبضعك'] = {
    'word': u'وبضعك',
    'procletic': u'-وَ',
    'tags': u'أداة:كناية:معطوف:مضاف',
    'vocalized': u'وَبِضْعَكِ',
    'stem': u'بِضْعَ',
    'type': u'STOPWORD',
    'original': u'بِضْعَ',
    'encletic': u'كِ'
}
STOPWORDS[u'وبضعه'] = {
    'word': u'وبضعه',
    'procletic': u'-وَ',
    'tags': u'أداة:كناية:معطوف:مضاف',
    'vocalized': u'وَبِضْعَه',
    'stem': u'بِضْعَ',
    'type': u'STOPWORD',
    'original': u'بِضْعَ',
    'encletic': u'ه'
}
STOPWORDS[u'وبضعكم'] = {
    'word': u'وبضعكم',
    'procletic': u'-وَ',
    'tags': u'أداة:كناية:معطوف:مضاف',
    'vocalized': u'وَبِضْعَكُمْ',
    'stem': u'بِضْعَ',
    'type': u'STOPWORD',
    'original': u'بِضْعَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'وبضعكن'] = {
    'word': u'وبضعكن',
    'procletic': u'-وَ',
    'tags': u'أداة:كناية:معطوف:مضاف',
    'vocalized': u'وَبِضْعَكُنَّ',
    'stem': u'بِضْعَ',
    'type': u'STOPWORD',
    'original': u'بِضْعَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'وبضعها'] = {
    'word': u'وبضعها',
    'procletic': u'-وَ',
    'tags': u'أداة:كناية:معطوف:مضاف',
    'vocalized': u'وَبِضْعَهَا',
    'stem': u'بِضْعَ',
    'type': u'STOPWORD',
    'original': u'بِضْعَ',
    'encletic': u'هَا'
}
STOPWORDS[u'وبضعهم'] = {
    'word': u'وبضعهم',
    'procletic': u'-وَ',
    'tags': u'أداة:كناية:معطوف:مضاف',
    'vocalized': u'وَبِضْعَهُمْ',
    'stem': u'بِضْعَ',
    'type': u'STOPWORD',
    'original': u'بِضْعَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'وبضعهن'] = {
    'word': u'وبضعهن',
    'procletic': u'-وَ',
    'tags': u'أداة:كناية:معطوف:مضاف',
    'vocalized': u'وَبِضْعَهُنَّ',
    'stem': u'بِضْعَ',
    'type': u'STOPWORD',
    'original': u'بِضْعَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'وبضعنا'] = {
    'word': u'وبضعنا',
    'procletic': u'-وَ',
    'tags': u'أداة:كناية:معطوف:مضاف',
    'vocalized': u'وَبِضْعَنَا',
    'stem': u'بِضْعَ',
    'type': u'STOPWORD',
    'original': u'بِضْعَ',
    'encletic': u'نَا'
}
STOPWORDS[u'وبضعكما'] = {
    'word': u'وبضعكما',
    'procletic': u'-وَ',
    'tags': u'أداة:كناية:معطوف:مضاف',
    'vocalized': u'وَبِضْعَكُمَا',
    'stem': u'بِضْعَ',
    'type': u'STOPWORD',
    'original': u'بِضْعَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'وبضعهما'] = {
    'word': u'وبضعهما',
    'procletic': u'-وَ',
    'tags': u'أداة:كناية:معطوف:مضاف',
    'vocalized': u'وَبِضْعَهُمَا',
    'stem': u'بِضْعَ',
    'type': u'STOPWORD',
    'original': u'بِضْعَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'فبضعي'] = {
    'word': u'فبضعي',
    'procletic': u'-فَ',
    'tags': u'أداة:كناية:معطوف:مضاف',
    'vocalized': u'فَبِضْعَِي',
    'stem': u'بِضْعَ',
    'type': u'STOPWORD',
    'original': u'بِضْعَ',
    'encletic': u'ِي'
}
STOPWORDS[u'فبضعك'] = {
    'word': u'فبضعك',
    'procletic': u'-فَ',
    'tags': u'أداة:كناية:معطوف:مضاف',
    'vocalized': u'فَبِضْعَكَ',
    'stem': u'بِضْعَ',
    'type': u'STOPWORD',
    'original': u'بِضْعَ',
    'encletic': u'كَ'
}
STOPWORDS[u'فبضعك'] = {
    'word': u'فبضعك',
    'procletic': u'-فَ',
    'tags': u'أداة:كناية:معطوف:مضاف',
    'vocalized': u'فَبِضْعَكِ',
    'stem': u'بِضْعَ',
    'type': u'STOPWORD',
    'original': u'بِضْعَ',
    'encletic': u'كِ'
}
STOPWORDS[u'فبضعه'] = {
    'word': u'فبضعه',
    'procletic': u'-فَ',
    'tags': u'أداة:كناية:معطوف:مضاف',
    'vocalized': u'فَبِضْعَه',
    'stem': u'بِضْعَ',
    'type': u'STOPWORD',
    'original': u'بِضْعَ',
    'encletic': u'ه'
}
STOPWORDS[u'فبضعكم'] = {
    'word': u'فبضعكم',
    'procletic': u'-فَ',
    'tags': u'أداة:كناية:معطوف:مضاف',
    'vocalized': u'فَبِضْعَكُمْ',
    'stem': u'بِضْعَ',
    'type': u'STOPWORD',
    'original': u'بِضْعَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'فبضعكن'] = {
    'word': u'فبضعكن',
    'procletic': u'-فَ',
    'tags': u'أداة:كناية:معطوف:مضاف',
    'vocalized': u'فَبِضْعَكُنَّ',
    'stem': u'بِضْعَ',
    'type': u'STOPWORD',
    'original': u'بِضْعَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'فبضعها'] = {
    'word': u'فبضعها',
    'procletic': u'-فَ',
    'tags': u'أداة:كناية:معطوف:مضاف',
    'vocalized': u'فَبِضْعَهَا',
    'stem': u'بِضْعَ',
    'type': u'STOPWORD',
    'original': u'بِضْعَ',
    'encletic': u'هَا'
}
STOPWORDS[u'فبضعهم'] = {
    'word': u'فبضعهم',
    'procletic': u'-فَ',
    'tags': u'أداة:كناية:معطوف:مضاف',
    'vocalized': u'فَبِضْعَهُمْ',
    'stem': u'بِضْعَ',
    'type': u'STOPWORD',
    'original': u'بِضْعَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'فبضعهن'] = {
    'word': u'فبضعهن',
    'procletic': u'-فَ',
    'tags': u'أداة:كناية:معطوف:مضاف',
    'vocalized': u'فَبِضْعَهُنَّ',
    'stem': u'بِضْعَ',
    'type': u'STOPWORD',
    'original': u'بِضْعَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'فبضعنا'] = {
    'word': u'فبضعنا',
    'procletic': u'-فَ',
    'tags': u'أداة:كناية:معطوف:مضاف',
    'vocalized': u'فَبِضْعَنَا',
    'stem': u'بِضْعَ',
    'type': u'STOPWORD',
    'original': u'بِضْعَ',
    'encletic': u'نَا'
}
STOPWORDS[u'فبضعكما'] = {
    'word': u'فبضعكما',
    'procletic': u'-فَ',
    'tags': u'أداة:كناية:معطوف:مضاف',
    'vocalized': u'فَبِضْعَكُمَا',
    'stem': u'بِضْعَ',
    'type': u'STOPWORD',
    'original': u'بِضْعَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'فبضعهما'] = {
    'word': u'فبضعهما',
    'procletic': u'-فَ',
    'tags': u'أداة:كناية:معطوف:مضاف',
    'vocalized': u'فَبِضْعَهُمَا',
    'stem': u'بِضْعَ',
    'type': u'STOPWORD',
    'original': u'بِضْعَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أبضعي'] = {
    'word': u'أبضعي',
    'procletic': u'-أَ',
    'tags': u'أداة:كناية:مجرور:مضاف',
    'vocalized': u'أَبِضْعَِي',
    'stem': u'بِضْعَ',
    'type': u'STOPWORD',
    'original': u'بِضْعَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أبضعك'] = {
    'word': u'أبضعك',
    'procletic': u'-أَ',
    'tags': u'أداة:كناية:مجرور:مضاف',
    'vocalized': u'أَبِضْعَكَ',
    'stem': u'بِضْعَ',
    'type': u'STOPWORD',
    'original': u'بِضْعَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أبضعك'] = {
    'word': u'أبضعك',
    'procletic': u'-أَ',
    'tags': u'أداة:كناية:مجرور:مضاف',
    'vocalized': u'أَبِضْعَكِ',
    'stem': u'بِضْعَ',
    'type': u'STOPWORD',
    'original': u'بِضْعَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أبضعه'] = {
    'word': u'أبضعه',
    'procletic': u'-أَ',
    'tags': u'أداة:كناية:مجرور:مضاف',
    'vocalized': u'أَبِضْعَه',
    'stem': u'بِضْعَ',
    'type': u'STOPWORD',
    'original': u'بِضْعَ',
    'encletic': u'ه'
}
STOPWORDS[u'أبضعكم'] = {
    'word': u'أبضعكم',
    'procletic': u'-أَ',
    'tags': u'أداة:كناية:مجرور:مضاف',
    'vocalized': u'أَبِضْعَكُمْ',
    'stem': u'بِضْعَ',
    'type': u'STOPWORD',
    'original': u'بِضْعَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أبضعكن'] = {
    'word': u'أبضعكن',
    'procletic': u'-أَ',
    'tags': u'أداة:كناية:مجرور:مضاف',
    'vocalized': u'أَبِضْعَكُنَّ',
    'stem': u'بِضْعَ',
    'type': u'STOPWORD',
    'original': u'بِضْعَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أبضعها'] = {
    'word': u'أبضعها',
    'procletic': u'-أَ',
    'tags': u'أداة:كناية:مجرور:مضاف',
    'vocalized': u'أَبِضْعَهَا',
    'stem': u'بِضْعَ',
    'type': u'STOPWORD',
    'original': u'بِضْعَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أبضعهم'] = {
    'word': u'أبضعهم',
    'procletic': u'-أَ',
    'tags': u'أداة:كناية:مجرور:مضاف',
    'vocalized': u'أَبِضْعَهُمْ',
    'stem': u'بِضْعَ',
    'type': u'STOPWORD',
    'original': u'بِضْعَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أبضعهن'] = {
    'word': u'أبضعهن',
    'procletic': u'-أَ',
    'tags': u'أداة:كناية:مجرور:مضاف',
    'vocalized': u'أَبِضْعَهُنَّ',
    'stem': u'بِضْعَ',
    'type': u'STOPWORD',
    'original': u'بِضْعَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أبضعنا'] = {
    'word': u'أبضعنا',
    'procletic': u'-أَ',
    'tags': u'أداة:كناية:مجرور:مضاف',
    'vocalized': u'أَبِضْعَنَا',
    'stem': u'بِضْعَ',
    'type': u'STOPWORD',
    'original': u'بِضْعَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أبضعكما'] = {
    'word': u'أبضعكما',
    'procletic': u'-أَ',
    'tags': u'أداة:كناية:مجرور:مضاف',
    'vocalized': u'أَبِضْعَكُمَا',
    'stem': u'بِضْعَ',
    'type': u'STOPWORD',
    'original': u'بِضْعَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أبضعهما'] = {
    'word': u'أبضعهما',
    'procletic': u'-أَ',
    'tags': u'أداة:كناية:مجرور:مضاف',
    'vocalized': u'أَبِضْعَهُمَا',
    'stem': u'بِضْعَ',
    'type': u'STOPWORD',
    'original': u'بِضْعَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أوبضعي'] = {
    'word': u'أوبضعي',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:كناية:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِضْعَِي',
    'stem': u'بِضْعَ',
    'type': u'STOPWORD',
    'original': u'بِضْعَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أوبضعك'] = {
    'word': u'أوبضعك',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:كناية:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِضْعَكَ',
    'stem': u'بِضْعَ',
    'type': u'STOPWORD',
    'original': u'بِضْعَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أوبضعك'] = {
    'word': u'أوبضعك',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:كناية:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِضْعَكِ',
    'stem': u'بِضْعَ',
    'type': u'STOPWORD',
    'original': u'بِضْعَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أوبضعه'] = {
    'word': u'أوبضعه',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:كناية:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِضْعَه',
    'stem': u'بِضْعَ',
    'type': u'STOPWORD',
    'original': u'بِضْعَ',
    'encletic': u'ه'
}
STOPWORDS[u'أوبضعكم'] = {
    'word': u'أوبضعكم',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:كناية:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِضْعَكُمْ',
    'stem': u'بِضْعَ',
    'type': u'STOPWORD',
    'original': u'بِضْعَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أوبضعكن'] = {
    'word': u'أوبضعكن',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:كناية:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِضْعَكُنَّ',
    'stem': u'بِضْعَ',
    'type': u'STOPWORD',
    'original': u'بِضْعَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أوبضعها'] = {
    'word': u'أوبضعها',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:كناية:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِضْعَهَا',
    'stem': u'بِضْعَ',
    'type': u'STOPWORD',
    'original': u'بِضْعَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أوبضعهم'] = {
    'word': u'أوبضعهم',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:كناية:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِضْعَهُمْ',
    'stem': u'بِضْعَ',
    'type': u'STOPWORD',
    'original': u'بِضْعَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أوبضعهن'] = {
    'word': u'أوبضعهن',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:كناية:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِضْعَهُنَّ',
    'stem': u'بِضْعَ',
    'type': u'STOPWORD',
    'original': u'بِضْعَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أوبضعنا'] = {
    'word': u'أوبضعنا',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:كناية:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِضْعَنَا',
    'stem': u'بِضْعَ',
    'type': u'STOPWORD',
    'original': u'بِضْعَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أوبضعكما'] = {
    'word': u'أوبضعكما',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:كناية:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِضْعَكُمَا',
    'stem': u'بِضْعَ',
    'type': u'STOPWORD',
    'original': u'بِضْعَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أوبضعهما'] = {
    'word': u'أوبضعهما',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:كناية:معطوف:مجرور:مضاف',
    'vocalized': u'أَوَبِضْعَهُمَا',
    'stem': u'بِضْعَ',
    'type': u'STOPWORD',
    'original': u'بِضْعَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'أفبضعي'] = {
    'word': u'أفبضعي',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:كناية:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِضْعَِي',
    'stem': u'بِضْعَ',
    'type': u'STOPWORD',
    'original': u'بِضْعَ',
    'encletic': u'ِي'
}
STOPWORDS[u'أفبضعك'] = {
    'word': u'أفبضعك',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:كناية:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِضْعَكَ',
    'stem': u'بِضْعَ',
    'type': u'STOPWORD',
    'original': u'بِضْعَ',
    'encletic': u'كَ'
}
STOPWORDS[u'أفبضعك'] = {
    'word': u'أفبضعك',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:كناية:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِضْعَكِ',
    'stem': u'بِضْعَ',
    'type': u'STOPWORD',
    'original': u'بِضْعَ',
    'encletic': u'كِ'
}
STOPWORDS[u'أفبضعه'] = {
    'word': u'أفبضعه',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:كناية:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِضْعَه',
    'stem': u'بِضْعَ',
    'type': u'STOPWORD',
    'original': u'بِضْعَ',
    'encletic': u'ه'
}
STOPWORDS[u'أفبضعكم'] = {
    'word': u'أفبضعكم',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:كناية:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِضْعَكُمْ',
    'stem': u'بِضْعَ',
    'type': u'STOPWORD',
    'original': u'بِضْعَ',
    'encletic': u'كُمْ'
}
STOPWORDS[u'أفبضعكن'] = {
    'word': u'أفبضعكن',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:كناية:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِضْعَكُنَّ',
    'stem': u'بِضْعَ',
    'type': u'STOPWORD',
    'original': u'بِضْعَ',
    'encletic': u'كُنَّ'
}
STOPWORDS[u'أفبضعها'] = {
    'word': u'أفبضعها',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:كناية:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِضْعَهَا',
    'stem': u'بِضْعَ',
    'type': u'STOPWORD',
    'original': u'بِضْعَ',
    'encletic': u'هَا'
}
STOPWORDS[u'أفبضعهم'] = {
    'word': u'أفبضعهم',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:كناية:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِضْعَهُمْ',
    'stem': u'بِضْعَ',
    'type': u'STOPWORD',
    'original': u'بِضْعَ',
    'encletic': u'هُمْ'
}
STOPWORDS[u'أفبضعهن'] = {
    'word': u'أفبضعهن',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:كناية:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِضْعَهُنَّ',
    'stem': u'بِضْعَ',
    'type': u'STOPWORD',
    'original': u'بِضْعَ',
    'encletic': u'هُنَّ'
}
STOPWORDS[u'أفبضعنا'] = {
    'word': u'أفبضعنا',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:كناية:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِضْعَنَا',
    'stem': u'بِضْعَ',
    'type': u'STOPWORD',
    'original': u'بِضْعَ',
    'encletic': u'نَا'
}
STOPWORDS[u'أفبضعكما'] = {
    'word': u'أفبضعكما',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:كناية:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِضْعَكُمَا',
    'stem': u'بِضْعَ',
    'type': u'STOPWORD',
    'original': u'بِضْعَ',
    'encletic': u'كُمَا'
}
STOPWORDS[u'أفبضعهما'] = {
    'word': u'أفبضعهما',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:كناية:معطوف:مجرور:مضاف',
    'vocalized': u'أَفَبِضْعَهُمَا',
    'stem': u'بِضْعَ',
    'type': u'STOPWORD',
    'original': u'بِضْعَ',
    'encletic': u'هُمَا'
}
STOPWORDS[u'ذيت'] = {
    'word': u'ذَيْتَ',
    'procletic': u'',
    'tags': u'أداة:كناية',
    'vocalized': u'ذَيْتَ',
    'stem': u'ذَيْتَ',
    'type': u'STOPWORD',
    'original': u'ذَيْتَ',
    'encletic': u''
}
STOPWORDS[u'وذيت'] = {
    'word': u'وذيت',
    'procletic': u'-وَ',
    'tags': u'أداة:كناية:معطوف',
    'vocalized': u'وَذَيْتَ',
    'stem': u'ذَيْتَ',
    'type': u'STOPWORD',
    'original': u'ذَيْتَ',
    'encletic': u''
}
STOPWORDS[u'فذيت'] = {
    'word': u'فذيت',
    'procletic': u'-فَ',
    'tags': u'أداة:كناية:معطوف',
    'vocalized': u'فَذَيْتَ',
    'stem': u'ذَيْتَ',
    'type': u'STOPWORD',
    'original': u'ذَيْتَ',
    'encletic': u''
}
STOPWORDS[u'فلان'] = {
    'word': u'فُلَانٌ',
    'procletic': u'',
    'tags': u'أداة:كناية',
    'vocalized': u'فُلَانٌ',
    'stem': u'فُلَانٌ',
    'type': u'STOPWORD',
    'original': u'فُلَانٌ',
    'encletic': u''
}
STOPWORDS[u'وفلان'] = {
    'word': u'وفلان',
    'procletic': u'-وَ',
    'tags': u'أداة:كناية:معطوف',
    'vocalized': u'وَفُلَانٌ',
    'stem': u'فُلَانٌ',
    'type': u'STOPWORD',
    'original': u'فُلَانٌ',
    'encletic': u''
}
STOPWORDS[u'ففلان'] = {
    'word': u'ففلان',
    'procletic': u'-فَ',
    'tags': u'أداة:كناية:معطوف',
    'vocalized': u'فَفُلَانٌ',
    'stem': u'فُلَانٌ',
    'type': u'STOPWORD',
    'original': u'فُلَانٌ',
    'encletic': u''
}
STOPWORDS[u'أفلان'] = {
    'word': u'أفلان',
    'procletic': u'-أَ',
    'tags': u'أداة:كناية:مجرور',
    'vocalized': u'أَفُلَانٌ',
    'stem': u'فُلَانٌ',
    'type': u'STOPWORD',
    'original': u'فُلَانٌ',
    'encletic': u''
}
STOPWORDS[u'أوفلان'] = {
    'word': u'أوفلان',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:كناية:معطوف:مجرور',
    'vocalized': u'أَوَفُلَانٌ',
    'stem': u'فُلَانٌ',
    'type': u'STOPWORD',
    'original': u'فُلَانٌ',
    'encletic': u''
}
STOPWORDS[u'أففلان'] = {
    'word': u'أففلان',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:كناية:معطوف:مجرور',
    'vocalized': u'أَفَفُلَانٌ',
    'stem': u'فُلَانٌ',
    'type': u'STOPWORD',
    'original': u'فُلَانٌ',
    'encletic': u''
}
STOPWORDS[u'كأي'] = {
    'word': u'كَأَيٍّ',
    'procletic': u'',
    'tags': u'أداة:كناية',
    'vocalized': u'كَأَيٍّ',
    'stem': u'كَأَيٍّ',
    'type': u'STOPWORD',
    'original': u'كَأَيٍّ',
    'encletic': u''
}
STOPWORDS[u'وكأي'] = {
    'word': u'وكأي',
    'procletic': u'-وَ',
    'tags': u'أداة:كناية:معطوف',
    'vocalized': u'وَكَأَيٍّ',
    'stem': u'كَأَيٍّ',
    'type': u'STOPWORD',
    'original': u'كَأَيٍّ',
    'encletic': u''
}
STOPWORDS[u'فكأي'] = {
    'word': u'فكأي',
    'procletic': u'-فَ',
    'tags': u'أداة:كناية:معطوف',
    'vocalized': u'فَكَأَيٍّ',
    'stem': u'كَأَيٍّ',
    'type': u'STOPWORD',
    'original': u'كَأَيٍّ',
    'encletic': u''
}
STOPWORDS[u'كأين'] = {
    'word': u'كَأَيْنَ',
    'procletic': u'',
    'tags': u'أداة:كناية',
    'vocalized': u'كَأَيْنَ',
    'stem': u'كَأَيْنَ',
    'type': u'STOPWORD',
    'original': u'كَأَيْنَ',
    'encletic': u''
}
STOPWORDS[u'وكأين'] = {
    'word': u'وكأين',
    'procletic': u'-وَ',
    'tags': u'أداة:كناية:معطوف',
    'vocalized': u'وَكَأَيْنَ',
    'stem': u'كَأَيْنَ',
    'type': u'STOPWORD',
    'original': u'كَأَيْنَ',
    'encletic': u''
}
STOPWORDS[u'فكأين'] = {
    'word': u'فكأين',
    'procletic': u'-فَ',
    'tags': u'أداة:كناية:معطوف',
    'vocalized': u'فَكَأَيْنَ',
    'stem': u'كَأَيْنَ',
    'type': u'STOPWORD',
    'original': u'كَأَيْنَ',
    'encletic': u''
}
STOPWORDS[u'كأين'] = {
    'word': u'كَأَيَّنَ',
    'procletic': u'',
    'tags': u'أداة:كناية',
    'vocalized': u'كَأَيَّنَ',
    'stem': u'كَأَيَّنَ',
    'type': u'STOPWORD',
    'original': u'كَأَيَّنَ',
    'encletic': u''
}
STOPWORDS[u'وكأين'] = {
    'word': u'وكأين',
    'procletic': u'-وَ',
    'tags': u'أداة:كناية:معطوف',
    'vocalized': u'وَكَأَيَّنَ',
    'stem': u'كَأَيَّنَ',
    'type': u'STOPWORD',
    'original': u'كَأَيَّنَ',
    'encletic': u''
}
STOPWORDS[u'فكأين'] = {
    'word': u'فكأين',
    'procletic': u'-فَ',
    'tags': u'أداة:كناية:معطوف',
    'vocalized': u'فَكَأَيَّنَ',
    'stem': u'كَأَيَّنَ',
    'type': u'STOPWORD',
    'original': u'كَأَيَّنَ',
    'encletic': u''
}
STOPWORDS[u'كذا'] = {
    'word': u'كَذَا',
    'procletic': u'',
    'tags': u'أداة:كناية',
    'vocalized': u'كَذَا',
    'stem': u'كَذَا',
    'type': u'STOPWORD',
    'original': u'كَذَا',
    'encletic': u''
}
STOPWORDS[u'وكذا'] = {
    'word': u'وكذا',
    'procletic': u'-وَ',
    'tags': u'أداة:كناية:معطوف',
    'vocalized': u'وَكَذَا',
    'stem': u'كَذَا',
    'type': u'STOPWORD',
    'original': u'كَذَا',
    'encletic': u''
}
STOPWORDS[u'فكذا'] = {
    'word': u'فكذا',
    'procletic': u'-فَ',
    'tags': u'أداة:كناية:معطوف',
    'vocalized': u'فَكَذَا',
    'stem': u'كَذَا',
    'type': u'STOPWORD',
    'original': u'كَذَا',
    'encletic': u''
}
STOPWORDS[u'أكذا'] = {
    'word': u'أكذا',
    'procletic': u'-أَ',
    'tags': u'أداة:كناية:مجرور',
    'vocalized': u'أَكَذَا',
    'stem': u'كَذَا',
    'type': u'STOPWORD',
    'original': u'كَذَا',
    'encletic': u''
}
STOPWORDS[u'أوكذا'] = {
    'word': u'أوكذا',
    'procletic': u'-وَ-أَ',
    'tags': u'أداة:كناية:معطوف:مجرور',
    'vocalized': u'أَوَكَذَا',
    'stem': u'كَذَا',
    'type': u'STOPWORD',
    'original': u'كَذَا',
    'encletic': u''
}
STOPWORDS[u'أفكذا'] = {
    'word': u'أفكذا',
    'procletic': u'-فَ-أَ',
    'tags': u'أداة:كناية:معطوف:مجرور',
    'vocalized': u'أَفَكَذَا',
    'stem': u'كَذَا',
    'type': u'STOPWORD',
    'original': u'كَذَا',
    'encletic': u''
}
STOPWORDS[u'كم'] = {
    'word': u'كَمْ',
    'procletic': u'',
    'tags': u'أداة:كناية',
    'vocalized': u'كَمْ',
    'stem': u'كَمْ',
    'type': u'STOPWORD',
    'original': u'كَمْ',
    'encletic': u''
}
STOPWORDS[u'وكم'] = {
    'word': u'وكم',
    'procletic': u'-وَ',
    'tags': u'أداة:كناية:معطوف',
    'vocalized': u'وَكَمْ',
    'stem': u'كَمْ',
    'type': u'STOPWORD',
    'original': u'كَمْ',
    'encletic': u''
}
STOPWORDS[u'فكم'] = {
    'word': u'فكم',
    'procletic': u'-فَ',
    'tags': u'أداة:كناية:معطوف',
    'vocalized': u'فَكَمْ',
    'stem': u'كَمْ',
    'type': u'STOPWORD',
    'original': u'كَمْ',
    'encletic': u''
}
STOPWORDS[u'كيت'] = {
    'word': u'كَيْتَ',
    'procletic': u'',
    'tags': u'أداة:كناية',
    'vocalized': u'كَيْتَ',
    'stem': u'كَيْتَ',
    'type': u'STOPWORD',
    'original': u'كَيْتَ',
    'encletic': u''
}
STOPWORDS[u'وكيت'] = {
    'word': u'وكيت',
    'procletic': u'-وَ',
    'tags': u'أداة:كناية:معطوف',
    'vocalized': u'وَكَيْتَ',
    'stem': u'كَيْتَ',
    'type': u'STOPWORD',
    'original': u'كَيْتَ',
    'encletic': u''
}
STOPWORDS[u'فكيت'] = {
    'word': u'فكيت',
    'procletic': u'-فَ',
    'tags': u'أداة:كناية:معطوف',
    'vocalized': u'فَكَيْتَ',
    'stem': u'كَيْتَ',
    'type': u'STOPWORD',
    'original': u'كَيْتَ',
    'encletic': u''
}
if __name__ == "__main__":
    print(len(STOPWORDS) - 13021)
